# -*- coding: iso8859_2 -*-

#-------------------------------------------------------------------------------
# Name:        fs_artykuly_pom.py
# Purpose:     podaje dane artykuu raze mz informacj o przelicznikach, vacie czy cechach
# Author:      Micha Raburski
# Created:     2014/07/07
# RCS-ID:      $Id: fs_artykuly_pom.py $
# Copyright:   (c) 2014 RecordSystem
# Licence:     GNU GPL
#-------------------------------------------------------------------------------

# Funkcja zwraca dane podstawowe artykuu, w tym informacje o cechach
# podanie daty pozwala na pobranie vat
# jeli podany jest kontrahent i dane cennik itp to moe pobra dla niego ceny
# jeli podana jest faktura/korekta to pobiera z niej kontrahenta,wersj, walut, kurs i dla niego sprawdza ceny
# tabela to informacja czy to rs_faktury, rs_korekty,rs_zamowienia_odbiorcy itp

# 2020-03-19 - Mikoaj Kozowski  (11906) - pobieranie kodu gtu
# 2021-02-09 - Aleksander Bargie (12321) - [SP] uwzgldnia oddzia przy pobieraniu cen do faktur


# Wraca stat,LDane

#            0        1                   2            3    4                 5                  6
# LDane = [nazwa, przelicznik,dynamiczny przelicznik, jm1, jm2, wiodca jednostka miary (1,2),stawka vat,
#                     7                        8            9           10                  11
#          Informacje o cechach (LCechy), faktura_w_jm2, czy zom, nie podlega vat, ceny dla kontrahenta (LCeny),
#                  12           13                 14                15              16                  17
#          stawka vat import, co_p, kraj pochodzenia artykuu, typ artykulu, czy cakowita jm1, czy cakowita jm2,
#                  18                 19                 20                   21
#          dni przydatnosci, termin przydatnosci, ledzenie dostaw, wydanie proporcjonalne,
#                  22
#          wydanie proporcjonalne z uwzgldnieniem jm kalk]

#              0                1              2             3
# LCechy = [cecha nazwa, cecha opis, liczba wartoci, warto domylna]

#            0      1      2      3      4       5       6
# LCeny = [Upust, CenaN1,CenaN2,CenaB1,CenaB2,CenaWN1,CenaWN2,
#            7        8       9       10      11       12     13      14       15        16
#          CenaWB1,CenaWB2,CenaN1U,CenaN2U,CenaB1U,CenaB2U,CenaWN1U,CenaWN2U,CenaWB1U,CenaWB2U]

def PodajDaneArtykuluIFaktury(self,Art,Wyzn,Data,Numer,Kontrahent,Wersja,Waluta,Kurs,Tabela,BrakArtykuluOK = True):
    import rsglobal
    from funkcje import fs_artykuly_cechy_pom 
    from funkcje import fs_global_ceny_kontrahenta
    
    LDane =[]
    if Data == '':
        Data = rsglobal.PodajDzis()
    
    SQL="SELECT a.nazwa,a.przelicznik,COALESCE(a.dynamiczny_przelicznik,FALSE),a.jm1,a.jm2, a.jm_kalk,"                # 0-5
    SQL+=" rsf_podaj_vat_artykulu(a.firma_id,'"+Art+"','"+Data+"'), a.faktura_w_jm2,"                                  # 6-7
    SQL+=" rsf_podaj_vat_import_artykulu(a.firma_id,'"+Art+"','"+Data+"'),a.clo_p,a.typ_artykulu, "                    #8-10
    SQL+=" COALESCE(jm1.calkowita,FALSE), COALESCE(jm2.calkowita,FALSE),"                                              #11-12
    SQL+=" COALESCE(a.dni_przydatnosci,0),'"+Data+"'::DATE + a.dni_przydatnosci::INTEGER,"                             #13-14
    SQL+=" COALESCE(a.sledzenie_dostaw,FALSE), COALESCE(a.wydanie_proporcjonalne,FALSE),"                              #15-16
    SQL+=" CASE WHEN NOT COALESCE(a.wydanie_proporcjonalne,FALSE) THEN 0 ELSE a.jm_kalk::INTEGER END, a.kod_gtu"       #17-18  #MK 2020-03-19
    SQL+=" FROM rs_artykuly a "
    SQL+=" LEFT OUTER JOIN rs_jednostki_miar jm1 ON (jm1.firma_id= a.firma_id AND jm1.kod = a.jm1)"
    SQL+=" LEFT OUTER JOIN rs_jednostki_miar jm2 ON (jm2.firma_id= a.firma_id AND jm2.kod = a.jm2)"
    SQL+=" WHERE a.firma_id='"+self.firma_id+"' AND a.artykul_id='"+Art+"'"
    stat,DArt,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat <> 'OK':
        return 'Bd pobierania danych artykuu 7',[]
    if len(DArt) == 0:
        if BrakArtykuluOK:
            return 'OK',[]
        else:
            return 'Brak podanego artykuu : '+Art,[]
    
    LDane.append(DArt[0][0]) # nazwa (0)
    LDane.append(DArt[0][1]) # przelicznik (1)
    LDane.append(DArt[0][2]) # dynamiczny przelicznik (2)
    LDane.append(DArt[0][3]) # jm1 (3)
    LDane.append(DArt[0][4]) # jm2 (4)
    LDane.append(DArt[0][5]) # jm_kalk (5)
    LDane.append(DArt[0][6]) # stawka vat na dzi (6)
    
    stat,DCech = fs_artykuly_cechy_pom.PodajDaneCechyArtykulu(self,Art)
    if stat <> 'OK':
        return 'Bd pobierania danych cechy artykuu :'+stat,[]
    
    LDane.append(DCech)      # informacje o cechach (7)
    
    LDane.append(DArt[0][7]) # czy faktura w jm2 (8)
    
    # cz pobierajca ceny stworzona w oparciu o fs_artykuly_podaj_dane
    # pobierana jeli jest podany dokument lub parametry cennikowe
    CzyZlom = False
    NiePodlegaVAT = False
    LCeny= [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]
    Oddzial = ''  #AB 09.02.21r
    
    Upust = -1
    
    if Wyzn <> '' or (Kontrahent <> '' and Wersja <> '' and Waluta <> '' and Kurs <> '' and Tabela <> ''):
        if Tabela == 'rs_zamowienia_odbiorcy':
            PolePodlVAT = "FALSE"
        else:
            PolePodlVAT = "t.nie_podlega_vat"
            
        if Wyzn <> '':
            SQL =" SELECT COALESCE(t.wersja,'BRAK'),t.waluta,t.kurs,t.kontrahent_id,w.typ_faktury,"+PolePodlVAT+", oddzial_id"  #AB 09.02.21r
            SQL+=" FROM "+Tabela+" t  INNER JOIN rs_wyznaczniki w USING (firma_id,wyznacznik_id) "
            SQL+=" WHERE t.firma_id='"+self.firma_id+"' AND t.wyznacznik_id='"+Wyzn+"'"
            SQL+=" AND t.data='"+Data+"' AND t.numer="+str(Numer)+";"
            stat,DDokum,dump,dump = self.PolSQL.Execute(SQL,3)
            if stat <> 'OK':
                return 'Bd pobierania danych dokumentu',[]
            
            if len(DDokum) ==0:
                return 'Brak podanego dokumentu',[]
        
            Wersja        = DDokum[0][0]
            Waluta        = DDokum[0][1]
            Kurs          = DDokum[0][2]
            Kontrahent    = DDokum[0][3]
            CzyZlom       = (DDokum[0][4] == 'ZL')
            NiePodlegaVAT = DDokum[0][5]
            Oddzial       = DDokum[0][6]  #AB 09.02.21r

    
    ceny=fs_global_ceny_kontrahenta.PodajCenyDlaKontrahenta(self,self.firma_id,Kontrahent,Waluta,Data.strip("'"),Art,Wersja,Upust, Oddzial)  #AB 09.02.21r
    
    if ceny<>None:
        Upust=ceny[8]
        CenaN1=rsglobal.RSRound(ceny[0]*Kurs,2)
        CenaN2=rsglobal.RSRound(ceny[1]*Kurs,2)
        CenaB1=rsglobal.RSRound(ceny[2]*Kurs,2)        
        CenaB2=rsglobal.RSRound(ceny[3]*Kurs,2)
        CenaWN1=rsglobal.RSRound(ceny[0],2)
        CenaWN2=rsglobal.RSRound(ceny[1],2)
        CenaWB1=rsglobal.RSRound(ceny[2],2)
        CenaWB2=rsglobal.RSRound(ceny[3],2)
            
        CenaN1U=rsglobal.RSRound(CenaN1*((100-Upust)*0.01),2)
        CenaN2U=rsglobal.RSRound(CenaN2*((100-Upust)*0.01),2)
        CenaB1U=rsglobal.RSRound(CenaB1*((100-Upust)*0.01),2)
        CenaB2U=rsglobal.RSRound(CenaB2*((100-Upust)*0.01),2)
        CenaWN1U=rsglobal.RSRound(CenaWN1*((100-Upust)*0.01),2)
        CenaWN2U=rsglobal.RSRound(CenaWN2*((100-Upust)*0.01),2)
        CenaWB1U=rsglobal.RSRound(CenaWB1*((100-Upust)*0.01),2)
        CenaWB2U=rsglobal.RSRound(CenaWB2*((100-Upust)*0.01),2)
        
        LCeny =[Upust,CenaN1,CenaN2,CenaB1,CenaB2,CenaWN1,CenaWN2, 
                CenaWB1,CenaWB2,CenaN1U,CenaN2U,CenaB1U,CenaB2U,CenaWN1U,CenaWN2U,CenaWB1U,CenaWB2U]
                        
    LDane.append(CzyZlom) # czy faktura zom (9)
    LDane.append(NiePodlegaVAT) # czy faktura nie podlega VAT (10)
    LDane.append(LCeny) # ceny artykuu (11) 
    if not DArt[0][8] in ['','None',None]:
        LDane.append(DArt[0][8]) # stawka vat import (12)
    else:
        LDane.append('')
    LDane.append(DArt[0][9]) # clo_p (13)
    
    if stat=='OK':
        SQL = "SELECT COALESCE(k.kraj,'') FROM rs_artykuly a "
        SQL+= "LEFT OUTER JOIN rs_kontrahenci k on (a.firma_id=k.firma_id AND a.producent_id=k.kontrahent_id) "
        SQL+= "WHERE a.artykul_id='"+Art+"' AND a.firma_id = '"+self.firma_id+"';"
        stat,DaneKraj,dump,dump=self.PolSQL.Execute(SQL,3)
        if stat<>'OK':
            return 'Nie mona pobra kraju pochodzenia artykulu',[]
    LDane.append(DaneKraj[0][0]) # kraj pochodzenia artykuu (14)
    LDane.append(DArt[0][10]) # typ artykuu (15)
    LDane.append(DArt[0][11]) # cakowito jm1 (16)
    LDane.append(DArt[0][12]) # cakowito jm2 (17)
    LDane.append(DArt[0][13]) # dni przydatnoci (18)
    LDane.append(DArt[0][14]) # termin przydatnoci (19)
    LDane.append(DArt[0][15]) # sledzenie dostaw (20)
    LDane.append(DArt[0][16]) # wydanie proporcjonalne (21)
    LDane.append(DArt[0][17]) # wydanie proporcjonalne z uwzgldnieniem jm_kalk (22)
    LDane.append(DArt[0][18])  # kod gtu (23)   #MK 2020-03-19
    
    return 'OK',LDane

#-------------------------------------------------------------------------------
#- FUNKCJA ANALIZUJCA OPAKOWANIA DLA ARTYKUU
#-------------------------------------------------------------------------------

def PodajOpakowaniaDlaArtykulu(self,Art,Ilosc,Kontr='',Cecha='',DzienTyg='',TypKomunikatu=0):
    # TypKomunikatu: 0-puszcza bez komunikatu, 1-bdzie komunikat, 2-komunikat z dniami tygodnia
    # 3- sumy zamowien klucz, brak kontrahenta i cechy, 4-wydruki zestawienie zbiorowe
    # sprawdzenie czy artyku ma opakowania
    import rsglobal
    Ilosc = abs(float(Ilosc))
    SQL ="SELECT artykul_id FROM rs_artykuly "
    SQL+="WHERE firma_id='"+self.firma_id+"' AND artykul_id='"+Art+"' "
    SQL+="AND tylko_w_opakowaniach"
    stat,DArt,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat <> 'OK':
        return 'Bd przy sprawdzaniu czy tylko w opakowaniach:'+stat,{}
    
    if len(DArt) == 0:
        return 'OK',{}
    
    # skadamy zapytanie pobierajce opakowania
    SQLOpak= "SELECT opakowanie_id,ilosc1,ilosc2 FROM rs_artykuly_opakowania_zbiorcze "
    SQLOpak+=" WHERE firma_id='"+self.firma_id+"' AND artykul_id='"+Art+"'"
    
    DOpak = []
    # jeli podana cecha i kontrahent to sprawdzamy czy taka para istnieje
    if Kontr <> '' and Cecha <> '':
        SQL = SQLOpak + " AND kontrahent_id='"+Kontr+"' AND cecha1_wartosc = '"+Cecha+"' ORDER BY ilosc1 DESC"
        stat,DOpak,dump,dump = self.PolSQL.Execute(SQL,3)
        if stat <> 'OK':
            return 'Bd przy pobieraniu danych opakowa z kontrahentem i cech:'+stat,{}
        
    # jeli nie ma opakowa i kontrahent podany to sprawdzamy, czy s dla niego dedykowane opakowania
    if DOpak == [] and Kontr <> '':
        SQL = SQLOpak + " AND kontrahent_id='"+Kontr+"' AND COALESCE(TRIM(cecha1_wartosc),'')= '' ORDER BY ilosc1 DESC"
        stat,DOpak,dump,dump = self.PolSQL.Execute(SQL,3)
        if stat <> 'OK':
            return 'Bd przy pobieraniu danych opakowa z kontrahentem:'+stat,{}
        
    if DOpak == [] and Cecha <> '':
        SQL = SQLOpak + " AND kontrahent_id IS NULL AND cecha1_wartosc = '"+Cecha+"' ORDER BY ilosc1 DESC"
        stat,DOpak,dump,dump = self.PolSQL.Execute(SQL,3)
        if stat <> 'OK':
            return 'Bd przy pobieraniu danych opakowa z kontrahentem:'+stat,{}
        
    # jeli nie ma opakowa to pobieramy opakowania uniwersalne
    if DOpak == []:
        SQL = SQLOpak + " AND kontrahent_id IS NULL AND COALESCE(TRIM(cecha1_wartosc),'')= '' ORDER BY ilosc1 DESC"
        stat,DOpak,dump,dump = self.PolSQL.Execute(SQL,3)
        if stat <> 'OK':
            return 'Bd przy pobieraniu danych opakowa:'+stat,{}
    
    if len(DOpak) == 0:
        if TypKomunikatu == 3:
            return "Artyku "+Art.strip()+" w swoich danych podstawowych ma zaznaczone pole 'tylko w opakowaniach'. Brak definicji opakowa zbiorczych, niezawierajcych zdefiniowanego kontrahenta i wartoci cechy.",{}
        else:
            return "Artyku "+Art.strip()+" w swoich danych podstawowych ma zaznaczone pole 'tylko w opakowaniach'. Brak definicji opakowa zbiorczych dla parametrw wybranego artykuu.",{}
    
    # podzia iloci "prymitywny" od najwikszych
    SOpak = {}
    MinLUzup = 1000
    IlPozost = Ilosc
    for poz in DOpak:
        Opak = poz[0]
        SOpak[Opak] = 0
        IlPodz = poz[1]
        if IlPodz == 0:
            return "Artyku "+Art.strip()+" ma w definicji opakowa opakowanie o iloci zerowej.",{}
        while IlPozost >= IlPodz:
            SOpak[Opak] += 1
            IlPozost -= IlPodz
        UzupLiczba = abs(IlPozost - IlPodz)
        if MinLUzup>UzupLiczba:
            MinLUzup=UzupLiczba
    
    # jeli zostaa reszta to albo j wrzucamy do najmniejszego opakowania
    # albo zgaszamy komunikat
    if IlPozost > 0:
        min=Ilosc-IlPozost
        max=Ilosc+MinLUzup
        
        # opcja dopuszczajca niepene opakowania
        if TypKomunikatu==0:
            SOpak[Opak] += 1
            
        # opcja sprawdzajca podan ilo
        elif TypKomunikatu==1:
            return "Podana liczba sztuk artykuu "+Art.strip()+" jest nieprawidowa. Istnieje moliwo zamwienia "+str(min)+" lub "+str(max)+" sztuk artykuu.",{}
        
        # opcja sprawdza odejmowanie w sumach zamwie 
        elif TypKomunikatu==3:
            return "Podana liczba sztuk artykuu "+Art.strip()+" jest nieprawidowa. Istnieje moliwo wycofania "+str(min)+" lub "+str(max)+" sztuk artykuu.",{}

        # opcja sprawdzajca podan ilo z uwzgldnieniem w komunikacie dnia tygodnia
        elif TypKomunikatu==2:
            return "Podana liczba sztuk artykuu "+Art.strip()+" jest nieprawidowa dla "+DzienTyg+". Istnieje moliwo zamwienia "+str(min)+" lub "+str(max)+" sztuk artykuu.",{}
        
        # opcja do policzenia opakowa dla wydrwkw zestawie zbiorowych
        if TypKomunikatu==4:
            SOpak[Opak] += 1
            
    return 'OK',SOpak

#-------------------------------------------------------------------------------
#- FUNKCJA PODAJCA POJEMNO OPAKOWANIA DLA PODANYCH PARAMETRW
#-------------------------------------------------------------------------------

def PodajPojemnoscOpakowania(self,Art,Opak,Kontr='',Cecha=''):
    import rsglobal
    SQL ="SELECT artykul_id FROM rs_artykuly "
    SQL+="WHERE firma_id='"+self.firma_id+"' AND artykul_id='"+Art+"' "
    SQL+="AND tylko_w_opakowaniach"
    stat,DArt,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat <> 'OK':
        return 'Bd przy sprawdzaniu czy tylko w opakowaniach:'+stat,0
    
    if len(DArt) == 0:
        return 'OK',1

    # skadamy zapytanie pobierajce opakowania
    SQLOpak= "SELECT ilosc1,ilosc2 FROM rs_artykuly_opakowania_zbiorcze "
    SQLOpak+=" WHERE firma_id='"+self.firma_id+"' AND artykul_id='"+Art+"' AND opakowanie_id='"+Opak+"'"
    
    DOpak = []
    # jeli podana cecha i kontrahent to sprawdzamy czy taka para istnieje
    if Kontr <> '' and Cecha <> '':
        SQL = SQLOpak + " AND kontrahent_id='"+Kontr+"' AND cecha1_wartosc = '"+Cecha+"' ORDER BY ilosc1 DESC"
        stat,DOpak,dump,dump = self.PolSQL.Execute(SQL,3)
        if stat <> 'OK':
            return 'Bd przy pobieraniu danych opakowa z kontrahentem i cech:'+stat,0
        
    # jeli nie ma opakowa i kontrahent podany to sprawdzamy, czy s dla niego dedykowane opakowania
    if DOpak == [] and Kontr <> '':
        SQL = SQLOpak + " AND kontrahent_id='"+Kontr+"' AND COALESCE(TRIM(cecha1_wartosc),'')= '' ORDER BY ilosc1 DESC"
        stat,DOpak,dump,dump = self.PolSQL.Execute(SQL,3)
        if stat <> 'OK':
            return 'Bd przy pobieraniu danych opakowa z kontrahentem:'+stat,0
        
    if DOpak == [] and Cecha <> '':
        SQL = SQLOpak + " AND kontrahent_id IS NULL AND cecha1_wartosc = '"+Cecha+"' ORDER BY ilosc1 DESC"
        stat,DOpak,dump,dump = self.PolSQL.Execute(SQL,3)
        if stat <> 'OK':
            return 'Bd przy pobieraniu danych opakowa z kontrahentem:'+stat,0
        
    # jeli nie ma opakowa to pobieramy opakowania uniwersalne
    if DOpak == []:
        SQL = SQLOpak + " AND kontrahent_id IS NULL AND COALESCE(TRIM(cecha1_wartosc),'')= '' ORDER BY ilosc1 DESC"
        stat,DOpak,dump,dump = self.PolSQL.Execute(SQL,3)
        if stat <> 'OK':
            return 'Bd przy pobieraniu danych opakowa:'+stat,0
    
    if len(DOpak) == 0:
        return "Artyku "+Art.strip()+" w swoich danych podstawowych ma zaznaczone pole 'tylko w opakowaniach'. Brak definicji opakowa zbiorczych dla parametrw wybranego artykuu i opakowania.",0
        
    return 'OK',DOpak[0][0]

#-------------------------------------------------------------------------------

def PodajZamiennikiArtykulu(PolSQL,Firma,Art):
    import rsglobal
    SQL = "SELECT TRIM(artykul_id) FROM rs_artykuly WHERE firma_id='"+Firma+"' " \
          " AND (SELECT COALESCE(zamiennik_id,artykul_id) FROM rs_artykuly WHERE firma_id='"+Firma+"'" \
          " AND artykul_id='"+Art+"') IN (artykul_id,zamiennik_id)"

    stat,DZam,dump,dump = PolSQL.Execute(SQL,3)
    if stat <> 'OK':
        return 'Bd przy pobieraniu danych zamiennikw:'+stat,''

    LZam = ''
    for poz in DZam:
        LZam+=rsglobal.Dekoduj(poz[0])+','

    LZam = LZam.strip(',')
    return 'OK',LZam

