#!/usr/bin/python
# -*- coding: iso8859_2 -*-
#-----------------------------------------------------------------------------
# Name:             fs_blokada_kontr_pom.py
# Purpose:          funkcje pomocnicze do zakadania/zdejmowania blokady kontrahentw w oparciu o rozrachunki
# Author:           Micha Raburski
# Created:          2013/01/31
# RCS-ID:           $Id: fs_blokada_kontr_pom.py $
# Copyright:        (c) 2013 RecordSystem
# Licence:          GNU GPL
#-----------------------------------------------------------------------------
# 2017-10-31 Rafa Woniak      (10734) zmiana zapisu do logw z 1 na 5
# 2022-10-19 Jan Raburski       (13214) zmiana w algorytmie wybierania do blokady
# 2022-11-12 Jan Raburski       (13246) maile do kontrahentow o blokadzie
# 2023-01-02 Aleksander Bargie (13287) Wysyanie smsw o blokadzie dostaw
# 2023-03-30 Aleksander Bargie (13376) - Wysyka SMS za porednictwem modemu UART
# 2024-01-16 Wojciech Skowron   (13665) Przypadek niezdefiniowanej zmiennej DFDniPoTerminie

#-------------------------------------------------------------------------------
# FUNKCJA PODAJCA LIST KONTRAHENTW DO ZABLOKOWANIA/ODBLOKOWANIA
#-------------------------------------------------------------------------------

import  rsglobal

def PodajListeKontrahentowDoBlokady(PolSQL,Firma,Data,Blokowanie,DniKarencji,Kontrahent,DniOdpuszczone=0,UwzgledniacSposZapl=False):
    # parametry
    # PolSQL - poczenie do bazy pozwalajce na wykonywanie zapyta
    # Firma - symbol firmy dla ktrej przeprowadzamy masowe blokowanie/odblokowanie
    # Data - dzie na ktry wykonujemy analiz. Jeli pusty to na dzi
    # Blokowanie - jeli True to patrzymy co zablokowa, jeli False to patrzymy co odblokowa
    # DniKarencji - o ile dni naprzd zrobi analiz
    # Kontrahent - jeli analiza dotyczy jednego kontrahenta to podajemy. Jeli pusty to dotyczy wszystkich
    # DniOdpuszczone - dni dyspensy o ktre mona zwleka jeli kontrahent rcznie odblokowany
    # UwzgledniacSposZaplaty - czy dzielic kontrahentow wg sposobow zaplaty(G,P,Inne) z danych do fakturowania
    # Wynik:
    # stat ('OK'-poprawnie)
    # lista kontrahentow do blokady lub informacji o zablokowaniu
    
    import rsglobal
    import rs_anakonda_daty
    from funkcje import fs_rozrachunki_pom
    SKontr = {}
    DniKarencji = int(DniKarencji)
    
    # na podstawie daty oraz DniKarencji tworzymy zakres dat, ktry bdziemy analizowa
    if Data != '':
        if not rsglobal.SprawdzCzyToData(Data):
            return 'Podano nieprawidow dat :'+Data,[]
        
    if Data == '':
        Data = rsglobal.PodajDzis()

    Zakres = [Data]
    if DniKarencji > 0:
        for i in xrange(DniKarencji):
            Zakres.append(rs_anakonda_daty.DodajKilkaDni(Data, i))
    #        SQL = "SELECT '"+Data+"'::DATE+"+str(i+1)+";"
    #        stat, DDaty, dump, dump = PolSQL.Execute(SQL,3)
    #        if stat!='OK':
    #            return 'Bdy w obliczaniu dat zakresu do analizy'+stat,[]
    #        Zakres.append(rsglobal.Dekoduj(DDaty[0][0]))
            
    # pobieramy list patnikw do analizy
    SQL = "SELECT DISTINCT COALESCE(platnik_id,kontrahent_id),konto_odbiorca,"
    SQL += "COALESCE(odblokowany_recznie_dnia,'1990-01-01'::DATE)+"+str(DniOdpuszczone)  # data do kiedy odpuszczone
    SQL += " FROM rs_kontrahenci WHERE firma_id='"+Firma+"' AND konto_odbiorca!=''"
    SQL += " AND NOT kontrahent_id LIKE '*%' AND TRIM(COALESCE(platnik_id,kontrahent_id))!='' AND aktywny"
    if Blokowanie:
        SQL += " AND NOT COALESCE(zablokowany,False) "
    else:
        SQL += " AND zablokowany"
    if Kontrahent != '':
        SQL += " AND kontrahent_id='"+Kontrahent+"'"
    rsglobal.ZapiszDoLogow('SQL PLAT'+SQL, 5)
    stat, DKontr, dump, dump = PolSQL.Execute(SQL, 3)
    rsglobal.ZapiszDoLogow('DKontr '+str(DKontr), 5)
    if stat != 'OK':
        return 'Bdy w wybieraniu kontrahentw do analizy'+stat, []
    
    # dla kadego z nich pobieramy dla kadego dnia z zakresu informacj czy powinni zosta zablokowani
    # pierwsze stwierdzenie e to dzi przerywa ptl
    for kontr in DKontr:
        Kontr = rsglobal.Dekoduj(kontr[0])
        Konto = rsglobal.Dekoduj(kontr[1]).strip('-')+'-'+Kontr.strip()
        #DataDyspensy = '1990-01-01'
        #if DniOdpuszczone > 0:
        #    DataDyspensy = kontr[2]
        DataDyspensy = kontr[2]

        Blokowac = False
        for data in Zakres:
            # jeli ju uzupeniono to dane s gotowe lub nie sprawdzamy,  bo tego dnia kontrahent ma dyspens
            if not Blokowac and data > DataDyspensy :
                # pobranie okresw, ktrych dotyczy analiza 
                stat, LOkr = fs_rozrachunki_pom.PodajOkresyDlaRozrachunkowDlaDaty(PolSQL, Firma, data)
                if stat != 'OK':
                    return 'Bdy pobierania okresw :' + stat
                if len(LOkr) == 0:
                    return 'Dla dnia ' + data + ' nie ma okresu obrachunkowego'
                
                # sprawdzamy informacje o blokadzie na wybrany dzie
                SQL = "SELECT rsf_podaj_pole_z_danych_do_fakturowania('"+Firma+"','"+Kontr+"','"+data+"','dni_po_terminie'),"
                SQL += "rsf_podaj_pole_z_danych_do_fakturowania('"+Firma+"','"+Kontr+"','"+data+"','kwota_blokady'),"
                SQL += "rsf_podaj_pole_z_danych_do_fakturowania('"+Firma+"','"+Kontr+"','"+data+"','od_kiedy_sprawdzac');"
                stat, DDaneF, dump, dump = PolSQL.Execute(SQL,3)
                if stat != 'OK':
                    return 'Bdy w pobieraniu danych z danych do fakturowania:'+stat,[]

                DFDniPoTerminie = 0  #WS 2024-01-16
                if DDaneF[0][0] != '':
                    DFDniPoTerminie = int(DDaneF[0][0])
                DFDniPoTerminie = max(DFDniPoTerminie,0)
                DFKwotaBlokady = -1
                if DDaneF[0][1] != '':
                    DFKwotaBlokady = float(DDaneF[0][1])
                DFOdKiedy = DDaneF[0][2]
                
                # sprawdzamy czy data od kiedy sprawdza nie jest wiksza ni data z zakresu
                if DFOdKiedy == '' or DFOdKiedy <= data:
                    # sprawdzamy kwot nalenoci
                    if not Blokowac and DFKwotaBlokady>0:
                        SQL = "SELECT SUM(saldo_wn-saldo_ma) FROM rs_rozrachunki "
                        SQL += " WHERE firma_id='"+Firma+"' AND okres_id IN ("+LOkr+")"
                        SQL += " AND konto='"+Konto+"' AND data_rozlicz IS NULL"
                        SQL += " AND CASE WHEN saldo_wn!=0 THEN COALESCE(termin_plat,'2200-12-31')+"+str(DFDniPoTerminie)+"<='"+data+"' ELSE TRUE END"
                        if DFOdKiedy != '':
                            SQL += " AND data>='"+DFOdKiedy+"'"
                        stat, DTest, dump, dump = PolSQL.Execute(SQL,3)
                        if stat != 'OK':
                            return 'Bdy w pobieraniu danych z rozrachunkw dla kwoty blokady:'+stat, []
                        if DTest[0][0] > DFKwotaBlokady:
                            KwotaZaleglosci = DTest[0][0]
                            Blokowac = True
                            # kwota blokady przekroczona, sprawdzamy czy termin tez
                            #if DFDniPoTerminie == -1:
                            #    Blokowac = True
                            #else:
                            #    SQL = "SELECT * FROM rs_rozrachunki "
                            #    SQL += " WHERE firma_id='"+Firma+"' AND okres_id IN ("+LOkr+")"
                            #    SQL += " AND konto='"+Konto+"' AND data_rozlicz IS NULL AND NOT termin_plat IS NULL"
                            #    if DFOdKiedy != '':
                            #        SQL += " AND data>='"+DFOdKiedy+"'"
                            #    SQL += " AND termin_plat+"+str(DFDniPoTerminie)+"<='"+data+"'"
                            #    SQL += " AND kwota_wn>0 LIMIT 1"
                            #    stat,DTest,dump,dump=PolSQL.Execute(SQL,3)
                            #    if stat!='OK':
                            #        return 'Bdy w pobieraniu danych z rozrachunkw dla dni po terminie:'+stat,[]
                            #    elif len(DTest)>0:
                            #        Blokowac=True
                            if Blokowanie and Blokowac:
                                SKontr[Kontr]=[str(KwotaZaleglosci),data,Konto]
                                break

                ## sprawdzamy blokad dotyczc terminu
                    #if DFDniPoTerminie>-1:
                    #    SQL="SELECT * FROM rs_rozrachunki "
                    #    SQL+=" WHERE firma_id='"+Firma+"' AND okres_id IN ("+LOkr+")"
                    #    SQL+=" AND konto='"+Konto+"' AND data_rozlicz IS NULL AND NOT termin_plat IS NULL"
                    #    if DFOdKiedy!='':
                    #        SQL+=" AND data>='"+DFOdKiedy+"'"
                    #    SQL+=" AND termin_plat+"+str(DFDniPoTerminie)+"<='"+data+"'"
                    #    SQL+=" AND kwota_wn>0 LIMIT 1"
                    #    stat,DTest,dump,dump=PolSQL.Execute(SQL,3)
                    #    if stat!='OK':
                    #        return 'Bdy w pobieraniu danych z rozrachunkw dla dni po terminie:'+stat,[]

                    #    if len(DTest)>0:
                    #        Blokowac=True
                    #    if Blokowanie and Blokowac:
                    #        SKontr[Kontr]=[data,data,Konto]
                            
                    ## sprawdzamy blokad dotyczc kwoty
                    #if not Blokowac and DFKwotaBlokady>0:
                    #    SQL="SELECT SUM(saldo_wn) FROM rs_rozrachunki "
                    #    SQL+=" WHERE firma_id='"+Firma+"' AND okres_id IN ("+LOkr+")"
                    #    SQL+=" AND konto='"+Konto+"' AND data_rozlicz IS NULL"
                    #    if DFOdKiedy!='':
                    #        SQL+=" AND data>='"+DFOdKiedy+"'"
                    #    stat,DTest,dump,dump=PolSQL.Execute(SQL,3)
                    #    if stat!='OK':
                    #        return 'Bdy w pobieraniu danych z rozrachunkw dla kwoty blokady:'+stat,[]
                    #    if len(DTest)>0:
                    #        if DTest[0][0]>DFKwotaBlokady:
                    #            Blokowac=True
                    #        if Blokowanie and Blokowac:
                    #            SKontr[Kontr]=[str(DTest[0][0]),data,Konto]
    
        if not Blokowac and not Blokowanie:
            #SKontr[Kontr] = [True, data, Konto]
            SKontr[Kontr] = [True, Zakres[0], Konto]
    rsglobal.ZapiszDoLogow('SKontr '+str(SKontr),5)
    # mamy ju informacje  ktrych patnikw bdziemy odblokowywac/blokowa
    # trzeba jeszcze stworzy ich list kontrahentw dla ktrych s patnikami
    # ostatecznie zwracamy list by mc poda patnika i jego kontrahentw w odpowiedniej kolejnoci
    # Robimy trzy listy (LKontrG, LKontrP i LKontrI) jako gotwkowe, przelewowe i  inne, a na koniec je sklejamy
    LKontr=[]
    LKontrG=[]
    LKontrP=[]
    LKontrI=[]
    KluczeSKontr=SKontr.keys()
    KluczeSKontr.sort()
    
    for kontr in KluczeSKontr:
        wart=SKontr[kontr]
        # pobranie listy oddziaw, ktrych dotycz zmiany dla patnika
        
        # pobranie okresw, ktrych dotyczy analiza 
        stat, LOkr = fs_rozrachunki_pom.PodajOkresyDlaRozrachunkowDlaDaty(PolSQL, Firma, wart[1])
        if stat != 'OK':
            return 'Bdy pobierania okresw :'+stat
        if len(LOkr) == 0:
            return 'Dla dnia ' + wart[1] + ' nie ma okresu obrachunkowego'
        
        # pobieramy wyznaczniki
        SQL = "SELECT DISTINCT f_wyznacznik_id FROM rs_rozrachunki"
        SQL += " WHERE firma_id='"+Firma+"' AND okres_id IN ("+LOkr+")"
        SQL += " AND konto='"+wart[2]+"' AND NOT f_wyznacznik_id IS NULL;"
        stat, DWyzn, dump, dump = PolSQL.Execute(SQL, 3)
        if stat != 'OK':
            return 'Bdy w pobieraniu wyznacznikw dla patnika:'+stat, []
        
        LWyzn = []
        for poz in DWyzn:
            LWyzn.append(rsglobal.Dekoduj(poz[0]))
        
        DOddz=[]
        if LWyzn != []:
            SQL = "SELECT DISTINCT oddzial_id FROM rs_faktury"
            SQL += " WHERE firma_id='"+Firma+"' AND wyznacznik_id IN ("+str(LWyzn)[1:-1]+")"
            stat, DOddz, dump, dump = PolSQL.Execute(SQL, 3)
            if stat != 'OK':
                return 'Bdy w pobieraniu oddziaw dla patnika:'+stat,[]
        
        LOddz=[]
        for odd in DOddz:
            LOddz.append(odd[0])
        
        # pobranie listy kontrahentw dla patnika
        SQL = "SELECT kontrahent_id FROM rs_kontrahenci WHERE firma_id='"+Firma+"'"
        SQL += " AND platnik_id='"+kontr+"' AND NOT kontrahent_id='"+kontr+"' AND aktywny ORDER BY kontrahent_id;"
        stat, DKontr, dump, dump = PolSQL.Execute(SQL,3)
        if stat != 'OK':
            return 'Bdy w pobieraniu kontrahentw dla patnika:'+stat, []
        
        # jeli uwzgldniane to pobranie sposobu zapaty dla patnika
        if UwzgledniacSposZapl:
            SQL = "SELECT COALESCE(spos_zaplaty,'') FROM rs_kontrahenci WHERE firma_id='"+Firma+"'"
            SQL += " AND kontrahent_id='"+kontr+"';"
            stat, DSposZ, dump, dump = PolSQL.Execute(SQL, 3)
            if stat != 'OK':
                return 'Bdy w pobieraniu sposobu zaplaty patnika:'+stat, []
            
            SposZapl = DSposZ[0][0].strip()
         
            if SposZapl == 'G':
                LKontrG.append([kontr, 'P', [wart[0], wart[1]], LOddz, 'G'])
                for poz in DKontr:
                    LKontrG.append([poz[0], 'K', [wart[0], wart[1]], LOddz, 'G'])

            elif SposZapl == 'P':
                LKontrP.append([kontr, 'P', [wart[0], wart[1]], LOddz, 'P'])
                for poz in DKontr:
                    LKontrP.append([poz[0], 'K', [wart[0], wart[1]], LOddz, 'P'])
            else:
                LKontrI.append([kontr, 'P', [wart[0], wart[1]], LOddz, 'I'])
                for poz in DKontr:
                    LKontrI.append([poz[0], 'K', [wart[0], wart[1]], LOddz, 'I'])
      
        else:
            LKontr.append([kontr, 'P', [wart[0], wart[1]], LOddz, ''])
            for poz in DKontr:
                LKontr.append([poz[0], 'K', [wart[0], wart[1]], LOddz, 'I'])
            
        
    if UwzgledniacSposZapl:
        LKontr = LKontrG+LKontrP+LKontrI
    rsglobal.ZapiszDoLogow('LKontr '+str(LKontr), 5)
    return 'OK', LKontr

#-------------------------------------------------------------------------------

def ZalozBlokadeNaKontrahenta(PolSQL, Firma, Kontr, Data):
    # zmiana kontrahenta na zablokowanego
    import rsglobal
    
    if Data=='':
        Data=rsglobal.PodajDzis()
        
    SQL="UPDATE rs_kontrahenci SET zablokowany=TRUE WHERE firma_id='"+Firma+"' AND kontrahent_id='"+Kontr+"';"
    stat,dump,dump,dump=PolSQL.Execute(SQL,1)
    if stat != 'OK':    
        return "Bd blokowania kontrahenta "+Kontr+" : "+stat
    
    # zmiana zamwie kontrahenta
    # najpierw pobieramy klucze zamwie do zmiany a potem je zmieniamy
    SQLZam="SELECT wyznacznik_id,data,numer FROM rs_zamowienia_odbiorcy WHERE firma_id='"+Firma+"'"
    SQLZam+=" AND (dostawy_do_dnia IS NULL OR dostawy_do_dnia>'"+Data+"'::DATE - 1) AND kontrahent_id='"+Kontr+"';"
    stat,DZam,dump,dump=PolSQL.Execute(rsglobal.Dekoduj(SQLZam),3)
    if stat != 'OK':    
        return "Bd pobierania danych zamwie kontrahenta "+Kontr+" : "+stat
    
    for zam in DZam:
        SQL="UPDATE rs_zamowienia_odbiorcy SET dostawy_do_dnia='"+Data+"'::DATE - 1 WHERE firma_id='"+Firma+"'"
        SQL+=" AND wyznacznik_id='"+rsglobal.Dekoduj(zam[0])+"' AND data='"+rsglobal.Dekoduj(zam[1])+"' AND numer="+str(zam[2])
        SQL+=" AND (dostawy_do_dnia IS NULL OR dostawy_do_dnia>'"+Data+"'::DATE - 1);"
        stat,dump,dump,dump=PolSQL.Execute(SQL,1)
        if stat != 'OK':    
            return "Bd blokowania zamwie kontrahenta "+Kontr+" : "+stat
        
    return "OK"

#-------------------------------------------------------------------------------

def WyslijMailSmsOPrzewidywanejBlokadzie(PolSQL, Firma, Kontr, Data, Kwota, Operator, Blokada=False):
    # Blokada == True - info o blokadzie, False - informacja o przewidywanej blokadzie
    if Operator < 0:
        return 'OK'
    SQL = "SELECT rsf_podaj_pole_z_danych_do_fakturowania('" + Firma + "','" + Kontr + "','" + Data + "','blokada_info'),"
    SQL += "rsf_podaj_pole_z_danych_do_fakturowania('" + Firma + "','" + Kontr + "','" + Data + "','blokada_tel'),"
    SQL += "rsf_podaj_pole_z_danych_do_fakturowania('" + Firma + "','" + Kontr + "','" + Data + "','blokada_email');"
    stat, DDaneF, dump, dump = PolSQL.Execute(SQL, 3)
    if stat != 'OK':
        return 'Bdy w pobieraniu danych do wyslania maila do platnika:' + stat
    if len(DDaneF) == 0:
        return 'OK'
    if not DDaneF[0][0]:  # nie wysylac blokady
        return 'OK'
    if DDaneF[0][2] == '':  # nie ma adresu mailowego
        return 'OK'
    # pobranie imienia i nazwiska operatora
    SQL = "SELECT imie, nazwisko FROM rs_operatorzy WHERE operator_id=" + str(Operator)
    stat, DDaneO, dump, dump = PolSQL.Execute(SQL, 3)
    if stat != 'OK':
        return 'Bdy w pobieraniu imienia i nazwiska operatora:' + stat
    if len(DDaneO) == 0:
        return 'Brak operatora o id ' + str(Operator)
    Tresc = "Szanowni Pastwo,\n\n"
    if not Blokada:
        Temat = 'Ostrzeenie o przewidywanej blokadzie dostaw.'
        Tresc += "informujemy, e na Pastwa koncie s przeterminowane patnoci na czn kwot "+Kwota+" z. "
        Tresc += "Brak wpaty spowoduje blokad dostaw od dnia "+str(Data)+".\n\n"
    else:
        Temat = 'Informacja o blokadzie dostaw.'
        Tresc += "informujemy, e z powodu przeterminowanych patnoci na czn kwot "+Kwota+" z "
        Tresc += "zostay zablokowane dostawy.\n\n"
    Tresc += "Z powaaniem\n"
    Tresc += DDaneO[0][0]+' '+DDaneO[0][1]

    maile = DDaneF[0][2].split(';')
    for mail in maile:
        stat = rsglobal.RSWyslijEmail(mail, Temat, Tresc, [], Operator, PolSQL)
        if stat != 'OK':
            return stat

    #AB 02.01.23r
    rsglobal.ZapiszDoLogow("Pobrano tel: {}".format(DDaneF[0][1]))
    tel = filter(str.isdigit, str(DDaneF[0][1]))  # zostawia same cyfry, zamiast usuwa ( )#- korzystamy z biaej listy znakw
    if tel:
        stat = rsglobal.WyslijSms(tel, Tresc, Firma, PolSQL, "WyslijMailSmsOPrzewidywanejBlokadzie", kontrahent_id=Kontr) #AB 23.03.23r
        if stat != 'OK':
            return stat

    return 'OK'

#-------------------------------------------------------------------------------

def SprawdzBlokadeKuke(Dzien, Kontrahent,KwotaDokumentu):
    pass