# -*- coding: iso8859_2 -*-
#-----------------------------------------------------------------------------
# Name:        fs_dekretacja_automatyczna_pom.py
# Purpose:     funkcje pomocnicze dla dekretacji automatycznej
# Author:      Micha Raburski
# Created:     2013/10/28
# RCS-ID:      $Id: fs_dekretacja_automatyczna_pom.py $
# Copyright:   (c) 2013 Record System
# Licence:     GNU GPL
#-----------------------------------------------------------------------------

# 2015-02-10 Micha Raburski - (8742) zmiana pl wzorw dekretacji rnic kursowych
# 2018-03-29 Aleksander Bargie - (10987) Dekretacja automatyczna rnic kursowych - ograniczy sprawdzanie rozrachunkw do wybranego okresu
# 2018-10-04 Aleksander Bargie - (11247) Zmiana w dacie ksigowania kosztw podczas dekretacji dostaw
# 2019-05-22 Wojciech Skowron   - (11490) Dekretacja pac
# 2019-06-18 Wojciech Skowron   - (11515) Dekretacja pac - warunek na szablon listy
# 2020-04-03 Wojciech Skowron   - (11912) Uwzgldnienie kodw gtu w dekretacji faktur
# 2020-04-05 Wojciech Skowron   - (11920) Przepisywanie oznacze dla JPK_V7 z nagwka dostawy przy dekretacji dostaw
# 2020-04-15 Wojciech Skowron   - (11918) Przepisywanie oznacze dla JPK_V7 z nag. faktur/korekt przy dekretacji faktur
# 2020-06-23 Wojciech Skowron   - (12020) Przepisanie czy_mpp z danych kontrahenta przy dekretacji dostaw
# 2020-06-23 Wojciech Skowron   - (12023) MPP w dekretacji faktur i korekt
# 2021-11-09 Wojciech Skowron   - (12701) Zmiana w obsudze warunku na szablon listy przy dekretacji pac
# 2021-12-09 Aleksander Bargie - (12767) co bylo zmieniane?(cofniecie rseval)
# 2022-02-18 Aleksander Bargie - (12885) Brak konta WN dla stawki Vat 00
# 2023-01-25 Jan Raburski       - (11522) Zastapienie eval przez evalWarunek dla warunkow dekretacji
# 2024-02-23 Wojciech Skowron   - (13700) Przepisywanie sprzedaz_uslug z nag. faktur/korekt przy dekretacji faktur


#-##############################################################################
#-- DEKRETACJA FAKTUR I KOREKT
#-##############################################################################

#dokument= [wyznacznik_id,data,numer,termin_zaplaty,platnik_id,waluta,kurs,do_zaplaty,do_zaplaty_wal,projekt_id]
import traceback

from funkcje import fs_dekretacja_automatyczna_projekty
import rsglobal

def evalWarunek(Warunek, Wartosci):  #JR (11522)
    """ Warunek - formula, Wartosci - slownik z wartosciami do zastapienia w Warunku
        Wraca z True lub False, jesli Warunek niepoprawny syntaktycznie - wyjtek"""
    Warunek = Warunek.strip()
    for w in Wartosci.keys():
        Warunek = Warunek.replace(w, "'" + Wartosci[w] + "'")
    Warunek = Warunek.replace('<>','!=').replace('"',"'")  #
    rsglobal.ZapiszDoLogow("Warunek dekretacji: " + Warunek)

    # dla pewnosci
    lnawiasow = 0
    lapostrofow = 0
    for c in Warunek:
        if c == "'":
            lapostrofow = 1 - lapostrofow
        if c == "(":
            lnawiasow += 1
            if lapostrofow > 0:
                raise Exception('Niepoprawna skadnia.')
        if c == ")":
            lnawiasow -= 1
            if lapostrofow > 0:
                raise Exception('Niepoprawna skadnia.')
    if lnawiasow > 0 or lapostrofow > 0 or Warunek[0] != "'":
        raise Exception('Niepoprawna liczba apostrofw albo nawiasw.')

    rowne = Warunek.find('==')
    rozne = Warunek.find('!=')
    nalezy = Warunek.find(' in')
    nie = Warunek.find(' not ')
    if rowne < 0 and rozne < 0 and nalezy < 0:  # brak porownan
        raise Exception('Brak ==, <>, != lub in')
    if nie > -1:
        if nalezy != nie + 4:
            raise Exception('Niepoprawna skadnia not i in.')

    if nalezy > 0:  # obsluga in
        if rowne > -1 or rozne > -1:  # tylko jedno porownanie
            raise Exception('Niepoprawna skadnia - wystpuje kilka porwna.')
        t = Warunek[nalezy+3:].strip()
        if t[0] != '(' or t[-1:] != ')':  # na poczatku i koncu musza byc nawiasy
            raise Exception('Niepoprawna skadnia - brak ( lub ).')
        t = t[1:-1].strip()  # pozbywamy sie nawiasow, moga zostac teksty oddzielone przecinkami
        m = 0
        while m < len(t):
            if t[0] != "'":
                raise Exception('Niepoprawna skadnia w nawiasach.')
            m = t.find(",")
            if m > -1:  # sa przecinki
                s = t[1:m].strip()
                if s[-1] != "'":
                    raise Exception('Niepoprawna skadnia w nawiasach.')
            else:
                if t[-1] != "'":
                    raise Exception('Niepoprawna skadnia w nawiasach.')
                break
            t = t[m+1:].strip()
            m = 0
        return eval(Warunek)

    if rowne > -1:  # obsluga ==
        if rozne > -1 or nalezy > -1:  # tylko jedno porownanie
            raise Exception('Niepoprawna skadnia - wystpuje kilka porwna.')
        m = rowne
    elif rozne > -1:  # obsluga !=
        if nalezy > -1:  # tylko jedno porownanie
            raise Exception('Niepoprawna skadnia - wystpuje kilka porwna.')
        m = rozne
    t = Warunek[m+2:].strip()
    if t[0] != "'" or t[-1] != "'":
        raise Exception('Niepoprawna skadnia - brak tekstu po == lub !=.')

    return eval(Warunek)

#-----------------------------------------------

def ZapiszFaktureKorekteDoPozycjiDowoduKsiegowego(self,klucze,dok,dane,tabela,tresc,rejestr):
    Dowod= klucze[0]
    Nazwa= klucze[1]
    OkresDekr=self.okres_id
    if len(klucze)==5:
        OkresDekr=klucze[4]
    stat='OK'

    buf=rsglobal.Dekoduj(dok[0])+'  '
    Wyzn=buf[0:2]
    Data=rsglobal.Dekoduj(dok[1])
    Nr=rsglobal.Dekoduj(str(dok[2]))

    if tabela=='rs_korekty':
        TypD='K'
    else:
        TypD='F'

    stat,DaneFakt,dump,dump=self.PolSQL.Execute("SELECT rsf_podaj_numer_dokumentu_na_wydruk('"+self.firma_id+"','[\\\'"+Wyzn+"\\\',\\\'"+Data+"\\\',"+Nr+"]','"+TypD+"');",3)
    NrFakt=rsglobal.Dekoduj(DaneFakt[0][0])

    # pobranie stanu dokumentu w celu zablokowania wielokrotnej dekretacji
    SQL="SELECT stan FROM "+tabela+" WHERE firma_id='"+self.firma_id+"' AND wyznacznik_id='"+Wyzn+"'"
    SQL+=" AND data='"+Data+"' AND numer="+Nr+" FOR UPDATE NOWAIT;"
    stat,DStan,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        dane.append('BLOK STAN$'+NrFakt)
        return dane
    if DStan[0][0] !='Z':
        dane.append('BL STAN$'+NrFakt)
        return dane

    #Pobranie uprawnien do dokumentw ksigowych dla operatora
    UprTotal=True
    if self.operator_id == '0':
        UprDodawanie=True
        LDod=[]
    else:
        Sel="SELECT dowod_id,nazwa_id,wprowadzanie,edycja,usuwanie,przegladanie,zamykanie,otwieranie "
        Sel+="FROM rs_wzory_dekretacji_upr WHERE firma_id='"+self.firma_id+"' AND operator_id in "
        Sel+=""+str(self.operatorzy).replace("[","(").replace("]",")")+" AND okres_id='"+OkresDekr+"'; "
        stat,daneupr,dump,dump = self.PolSQL.Execute(Sel,3)
        if stat!='OK':
            stat = 'BL SPR UPR'
            dane.append(stat)
        if stat=='OK':
            if len(daneupr)>0:
                UprTotal=False
            else:
                UprTotal=True
            LDod=[]

            UprDodawanie=(self.oddzial_id,'075013') in self.uprawnienia
            if UprDodawanie:
                for d in daneupr:
                    if d[2]:
                        LDod.append((rsglobal.Dekoduj(d[0]).strip(),rsglobal.Dekoduj(d[1]).strip()))

    if not UprDodawanie:
        stat='UPR DOD'
        dane.append(stat)
    if stat=='OK' and not UprTotal:
        if (not(str(Dowod),Nazwa) in LDod)  and self.operator_id!='0':
            stat='UPR DOD'
            dane.append(stat)

    if stat != 'OK':
        return dane

    if stat=='OK':
        try:
            buf=rsglobal.Dekoduj(dok[0])+'  '
            Wyzn=buf[0:2]
            Data=rsglobal.Dekoduj(dok[1])
            Nr=rsglobal.Dekoduj(str(dok[2]))
            TerminPlat=rsglobal.Dekoduj(dok[3])
            if TerminPlat in ('None',None,''):
                TerminPlat='NULL'
            else:
                TerminPlat="'"+TerminPlat+"'"
            Platnik=rsglobal.Dekoduj(dok[4])
            #JR WarPlatnik=rsglobal.Dekoduj(dok[4]).strip()
            #JR WarWaluta=rsglobal.Dekoduj(dok[5]).strip()
            Projekt=rsglobal.Dekoduj(dok[9]).strip()
            #JR WarProjektId=rsglobal.Dekoduj(dok[9]).strip()
            WarKontrahent=rsglobal.Dekoduj(dok[11]).strip()  # zostaje bo uzywany gdzie indziej
            #JR WarZamawiajacy=rsglobal.Dekoduj(dok[12]).strip()
            #JR WarSposobZaplaty=rsglobal.Dekoduj(dok[13]).strip()
            #JR WarKierowca=rsglobal.Dekoduj(dok[14]).strip()
            #JR WarHandlowiec=rsglobal.Dekoduj(dok[15]).strip()
            #JR WarWersja=rsglobal.Dekoduj(dok[16]).strip()
            DataPrzych=rsglobal.Dekoduj(dok[17])
            #WS 2020-04-15
            OznTrans = rsglobal.Dekoduj(dok[18])
            OznSprzedaz = rsglobal.Dekoduj(dok[19])
            #WS 2020-06-23
            CzyMPP = dok[20]
            #WS 2024-02-23
            SprzedazUslug = dok[21]
            #JR 2024-01-25 (1522)
            Wartosci = {}
            Wartosci['WarPlatnik'] = rsglobal.Dekoduj(dok[4]).strip()
            Wartosci['WarWaluta'] = rsglobal.Dekoduj(dok[5]).strip()
            Wartosci['WarProjektId'] = rsglobal.Dekoduj(dok[9]).strip()
            Wartosci['WarKontrahent'] = rsglobal.Dekoduj(dok[11]).strip()
            Wartosci['WarZamawiajacy'] = rsglobal.Dekoduj(dok[12]).strip()
            Wartosci['WarSposobZaplaty'] =  rsglobal.Dekoduj(dok[13]).strip()
            Wartosci['WarKierowca'] = rsglobal.Dekoduj(dok[14]).strip()
            Wartosci['WarHandlowiec'] = rsglobal.Dekoduj(dok[15]).strip()
            Wartosci['WarWersja'] = rsglobal.Dekoduj(dok[16]).strip()

            # poniewa w arcusie wszystkie korekty maj wyznacznik 'K' to musimy bra wyznacznik z oryginalnej faktury do pobierania pozycji wzoru dekretacji
            if tabela=='rs_korekty':
                TypD='K'
                WyznO=rsglobal.Dekoduj(dok[10])
            else:
                TypD='F'
                WyznO=Wyzn

            if Projekt=='None':
                Projekt='NULL'
            else:
                Projekt="'"+Projekt+"'"

            Rej=rsglobal.Dekoduj(rejestr)
            ZmianaStawkiVAT=False
            # jeli jest to korekta to sprawdzamy, czy przypadkiem nie ma do zapaty 0. Jeli tak to
            # nie dekretujemy tylko zmieniamy na K
            if tabela=='rs_korekty':
                SQL="SELECT ROUND(do_zaplaty-do_zaplaty_p,3) FROM rs_korekty WHERE firma_id='"+self.firma_id+"'"
                SQL+=" AND wyznacznik_id='"+Wyzn+"' AND data='"+Data+"' AND numer="+Nr[0:Nr.find('.')]+";"
                stat,PozKor,dump,dump = self.PolSQL.Execute(SQL,3)
                if stat=='OK':
                    if PozKor[0][0]==0:
                        Zam="UPDATE "+tabela+" SET stan='K' WHERE firma_id='"+self.firma_id+"'"
                        Zam+=" AND wyznacznik_id='"+Wyzn+"' AND data='"+Data+"' AND numer="+Nr[0:Nr.find('.')]+";"
                        #rsglobal.ZapiszDoLogow('2 '+Zam,1)
                        stat,dump,dump,dump = self.PolSQL.Execute(Zam,1)
                        if stat!= 'OK':
                            self.PolSQL.Rollback()
                            dane.append(stat)
                        else:
                            self.PolSQL.Commit()
                        return dane

                #sprawdzenie czy na korekcie nastpuje jakakolwiek zmiana stawki VAT
                SQL="SELECT stawka_vat FROM rs_korekty_pozycje WHERE firma_id='"+self.firma_id+"'"
                SQL+=" AND wyznacznik_id='"+Wyzn+"' AND data='"+Data+"' AND numer="+Nr[0:Nr.find('.')]
                SQL+=" AND stawka_vat!= stawka_vat_p;"
                stat,DStawek,dump,dump = self.PolSQL.Execute(SQL,3)
                if stat=='OK':
                    ZmianaStawkiVAT=len(DStawek)>0

            #pobranie potencjalnych stawek vat ze wzoru dekretacji - sprawdzenie czy nie ma by po prostu zalepkowa (bez dekretacji)
            if stat == 'OK':
                SQL = "SELECT nr_poz, konto_wn, czy_kontrahent, konto_ma, konto_vat_ma, stawka_vat_ma, warunek,"
                SQL += " konto_vat_ma_kor, dekretowac, konto_wn_z_kontrahenta FROM rs_wzory_dekretacji_faktur "
                SQL += " WHERE firma_id='"+self.firma_id+"' AND okres_id='"+OkresDekr+"'"
                SQL += " AND dowod_id='"+Dowod+"' AND nazwa_id='"+Nazwa+"' AND wyznacznik_id='"+WyznO+"';"
                stat, PozWz, dump, dump = self.PolSQL.Execute(SQL,3)
            if stat == 'OK':
                if not PozWz[0][8]:
                    Zam = "UPDATE "+tabela+" SET stan='K' WHERE firma_id='"+self.firma_id+"' AND wyznacznik_id='"+Wyzn+"' AND data='"+Data+"' AND numer="+Nr[0:Nr.find('.')]+";"
                    stat, dump, dump, dump = self.PolSQL.Execute(Zam,1)
                    if stat != 'OK':
                        self.PolSQL.Rollback()
                        dane.append(stat)
                    else:
                        self.PolSQL.Commit()
                    return dane

            #wygenerowanie i wstawienie nagwka dowodu ksigowego
            if stat == 'OK':
                stat, Test, dump, dump = self.PolSQL.Execute("SELECT rsf_sprawdz_czy_mozna_dodac_dokument('"+self.firma_id+"','[\\\'"+OkresDekr+"\\\',\\\'"+Dowod+"\\\',\\\'"+Nazwa+"\\\',\\\'"+Data+"\\\']','D');",3)
            if stat == 'OK':
                if Test[0][0] == 'OK':
                    stat, DNr, dump, dump = self.PolSQL.Execute("SELECT rsf_podaj_nowy_numer_dokumentu('"+self.firma_id+"','[\\\'"+OkresDekr+"\\\',\\\'"+Dowod+"\\\',\\\'"+Nazwa+"\\\',\\\'"+Data+"\\\']','D');",3)
                    if DNr[0][0] in (None,'None',''):
                        dane.append("PO DOK")
                        return dane
                    else:
                        Numer=rsglobal.Dekoduj(DNr[0][0])
                else:
                    dane.append("PO DOK")
                    return dane

            stat,DaneFakt,dump,dump=self.PolSQL.Execute("SELECT rsf_podaj_numer_dokumentu_na_wydruk('"+self.firma_id+"','[\\\'"+Wyzn+"\\\',\\\'"+Data+"\\\',"+Nr+"]','"+TypD+"');",3)
            NrFakt=rsglobal.Dekoduj(DaneFakt[0][0])

            InsN= "INSERT INTO rs_dowody_ksiegowe "
            InsN+="(firma_id,okres_id,dowod_id,nazwa_id,data,numer,stan,kwota_wn,kwota_ma,"
            InsN+="symbol_dok,data_wyst,data_wplywu,termin_plat,data_zapl,kontrahent_id,rejestr,"
            InsN+="data_rejestru,nr_faktury,projekt_id,f_wyznacznik_id,f_data,f_numer,f_typ,"
            InsN+="ozn_trans,ozn_sprzedaz,czy_mpp,sprzedaz_uslug)"  #WS 2020-04-15  #WS 2020-06-23  #WS 2024-02-23
            InsN+=" VALUES "
            InsN+="('"+self.firma_id+"','"+OkresDekr+"','"+Dowod+"','"+Nazwa+"','"+Data+"',"+str(Numer)+",'W',"+rsglobal.Dekoduj(str(dok[7]))+","+rsglobal.Dekoduj(str(dok[7]))+","
            InsN+="'"+NrFakt+"','"+Data+"',NULL,"+TerminPlat+",NULL,'"+Platnik+"',"+Rej+","
            InsN+="'"+DataPrzych+"','"+NrFakt+"',"+Projekt+",'"+Wyzn+"','"+Data+"',"+Nr+",'"+TypD+"',"
            InsN+=rsglobal.ZmienNaNULL(OznTrans)+","+rsglobal.ZmienNaNULL(OznSprzedaz)+","+str(CzyMPP)+","+str(SprzedazUslug)+");"  #WS 2020-04-15  #WS 2020-06-23  #WS 2024-02-23

            #sprawdzenie czy bd * i podzia na typy artykuw czy nie.
            CzyRozbicieNaTypyArt= (PozWz[0][3][0]=='*')
            for poz in PozWz:
                if CzyRozbicieNaTypyArt:
                    if poz[3][0] != '*':
                        dane.append("BR GW$"+NrFakt)
                        return dane

            #sprawdzenie warunku i odrzucenie tych pozycji wzoru dekretacji, ktre nie speniaj
            LPozWzPoWar = []
            for poz in PozWz:
                War = poz[6]
                if War not in ['',None,'None']:
                    try:
                        if evalWarunek(War, Wartosci):  #JR 2024-01-25 (11522)
                            #JR LPozWzPoWar.append(eval(str(poz)))
                            LPozWzPoWar.append(poz)
                    except Exception,e:
                        dane.append("BL WAR$"+NrFakt+u' WARUNEK: '+War+u' BD: '+str(e))
                        return dane
                else:
                    #JR LPozWzPoWar.append(eval(str(poz)))
                    LPozWzPoWar.append(poz)

            if len(LPozWzPoWar)==0:
                dane.append("BR POZ WZ$"+NrFakt)
                return dane

            PozWz=LPozWzPoWar

            #WS 2020-04-03 - sprawdzenie, czy na jakiejkolwiek pozycji faktury/korekty wystpuje kod_gtu
            SQL += "SELECT BOOL_OR(jest_kod) FROM ("
            SQL += "SELECT kod_gtu IS NOT NULL AS jest_kod FROM "+tabela+"_pozycje"
            SQL += " WHERE firma_id='"+self.firma_id+"' AND wyznacznik_id='"+Wyzn+"' AND data='"+Data+"' and numer="+Nr
            SQL += " UNION "
            SQL += "SELECT kod_gtu IS NOT NULL AS jest_kod FROM "+tabela+"_pozycje_opisowe"
            SQL += " WHERE firma_id='"+self.firma_id+"' AND wyznacznik_id='"+Wyzn+"' AND data='"+Data+"' and numer="+Nr
            SQL += ") AS spr;"
            stat, DanePozGTU, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
            if stat != 'OK':
                dane.append("Bd pobrania kodw GTU faktury/korekty: "+stat)
                return dane
            else:
                CzyWystepujaKodyGTU = DanePozGTU[0][0]

            #pobranie stawek vat z faktury/korekty
            if tabela=='rs_korekty':
                # pola 0-4
                SQL="SELECT stawka_vat,wartosc_netto-wartosc_netto_p,wartosc_vat-wartosc_vat_p,"
                # pola 5-9
                SQL+=" wartosc_wal_netto-wartosc_wal_netto_p,wartosc_wal_vat-wartosc_wal_vat_p"
                SQL+=" FROM rs_korekty_vat WHERE firma_id='"+self.firma_id+"' AND wyznacznik_id='"+Wyzn+"'"
                SQL+=" AND data='"+Data+"' AND numer="+Nr+" AND (ROUND(wartosc_netto-wartosc_netto_p,2) != 0 "
                SQL+=" OR ROUND(wartosc_vat-wartosc_vat_p,2) != 0);"
            else:
                # pola 0-4
                SQL="SELECT stawka_vat,wartosc_netto,wartosc_vat,wartosc_wal_netto,wartosc_wal_vat"
                SQL+=" FROM rs_faktury_vat WHERE firma_id='"+self.firma_id+"' AND wyznacznik_id='"+Wyzn+"'"
                SQL+=" AND data='"+Data+"' AND numer="+Nr+";"

            stat,StVat,dump,dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
            #WS 2020-04-03
            if stat != 'OK':
                dane.append("Bd pobrania stawek VAT faktury/korekty: "+stat)
                return dane

            #-------------------------------------
            #--- ZWYKA DEKRETACJA, BEZ KODW GTU
            #-------------------------------------
            #WS 2020-04-03
            if not CzyRozbicieNaTypyArt and not CzyWystepujaKodyGTU:
            # if not CzyRozbicie:
                #insert pierwszej pozycji (ze stawk domyln lub bez)
                Insert=''
                KontoWn = ''

                for poz in PozWz:
                    if Insert=='' and stat =='OK': #jeszcze nie znaleziono
                        if poz[1] not in ['','None'] or poz[9]: # jeli uzupenione konto wn lub chb z kontem wn z kontrahenta
                            if poz[9]:
                                SQL ="SELECT COALESCE(konto_odbiorca,'') FROM rs_kontrahenci WHERE firma_id='"+self.firma_id+"'"
                                SQL+=" AND kontrahent_id='"+Platnik+"'"
                                stat,DKontoWn,dump,dump = self.PolSQL.Execute(SQL,3)
                                if stat =='OK':
                                    KontoWn=rsglobal.Dekoduj(DKontoWn[0][0])
                            else:
                                KontoWn=rsglobal.Dekoduj(poz[1])

                            if stat=='OK' and KontoWn != '':
                                if poz[2]:
                                    KontoWn+='-'+Platnik

                                Waluta=rsglobal.Dekoduj(dok[5])
                                Kurs=rsglobal.Dekoduj(str(dok[6]))

                                Insert+="INSERT INTO rs_dowody_ksiegowe_pozycje (firma_id,okres_id,dowod_id,nazwa_id,data,numer,nr_poz,nr_ppoz,nr_pozw,"
                                Insert+="symbol_dok,data_wyst,data_wplywu,termin_plat,data_zapl,kontrahent_id,"
                                Insert+="konto_wn,kwota_wn,konto_vat_wn,kwota_vat_wn,stawka_vat_wn,tresc_wn,sposob_rozl,waluta_wn,kurs_wn,kwota_wal_wn,"
                                Insert+="nkup,konto_ma,kwota_ma,konto_vat_ma,kwota_vat_ma,stawka_vat_ma,tresc_ma,waluta_ma,kurs_ma,kwota_wal_ma,pnop,data_ksieg_kosztow)"
                                Insert+=" VALUES "
                                Insert+="('"+self.firma_id+"','"+OkresDekr+"','"+Dowod+"','"+Nazwa+"','"+Data+"',"+str(Numer)+",1,0,"+rsglobal.Dekoduj(str(poz[0]))+","
                                Insert+="'"+tresc+" "+NrFakt+"','"+Data+"',NULL,"+TerminPlat+",NULL,'"+Platnik+"',"
                                Insert+="'"+KontoWn+"',"+rsglobal.Dekoduj(str(dok[7]))+",'',0,NULL,'"+NrFakt+"','','"+Waluta+"',"+Kurs+","+rsglobal.Dekoduj(str(dok[8]))+","

                                #sprawdzenie czy na pozycji z nagwkiem wn jest tez cz ma. Jeli jest to uzupeniana, jak nie to nie.

                                StawkaP1=''
                                Jest=False
                                if poz[4] not in ['','None']:
                                    StawkaP1=rsglobal.Dekoduj(poz[5])
                                    for pst in StVat:
                                        if pst[0]==StawkaP1:
                                            Jest=True
                                            KontoMa=rsglobal.Dekoduj(poz[3])
                                            KontoVatMa=rsglobal.Dekoduj(poz[4])
                                            if KontoVatMa =='None':
                                                KontoVatMa=''
                                            if tabela=='rs_korekty':
                                                if str(poz[7]).strip() not in ('None',''):
                                                    KontoVatMa=rsglobal.Dekoduj(str(poz[7]))

                                            KwotaMa=rsglobal.Dekoduj(str(pst[1]))
                                            KwotaVatMa=rsglobal.Dekoduj(str(pst[2]))
                                            KwotaWalMa=rsglobal.Dekoduj(str(pst[3]))


                                            Insert+="FALSE,'"+KontoMa+"',"+KwotaMa+",'"+KontoVatMa+"',"+KwotaVatMa+",'"+StawkaP1+"','"+NrFakt+"','"+Waluta+"',"+Kurs+","+KwotaWalMa+",FALSE"

                                if not Jest:
                                    Insert+="FALSE,'',0,'',0,NULL,'','"+Waluta+"',"+Kurs+",0,FALSE"

                                Insert+=",'"+DataPrzych+"');"

                if stat=='OK':
                    #Nie byo konta wn na wzorze
                    if Insert=='':
                        dane.append("BR KWN$"+tresc+" "+NrFakt)
                        return dane

                    #Sprawdzenie pozostaych stawek vat z faktury i wpisanie ich na ma
                    NrPozDow=2

                    for pst in StVat:
                        if pst[0] != StawkaP1:
                            Stawka=rsglobal.Dekoduj(pst[0])
                            Jest=False
                            #ptla po pozycjach wzoru dekretacji
                            for poz in PozWz:
                                if not Jest:
                                    if poz[3] not in ['','None']:
                                        if Stawka==poz[5]:
                                            Jest=True
                                            KontoMa=rsglobal.Dekoduj(poz[3])
                                            KontoVatMa=rsglobal.Dekoduj(poz[4])
                                            if KontoVatMa =='None':
                                                KontoVatMa=''
                                            if tabela=='rs_korekty':
                                                if str(poz[7]).strip() not in ('None',''):
                                                    KontoVatMa=rsglobal.Dekoduj(str(poz[7]))

                                            KwotaMa=rsglobal.Dekoduj(str(pst[1]))
                                            if KwotaMa !=0:
                                                KwotaVatMa=rsglobal.Dekoduj(str(pst[2]))
                                                KwotaWalMa=rsglobal.Dekoduj(str(pst[3]))

                                                Insert+="INSERT INTO rs_dowody_ksiegowe_pozycje (firma_id,okres_id,dowod_id,nazwa_id,data,numer,nr_poz,nr_ppoz,nr_pozw,"
                                                Insert+="symbol_dok,data_wyst,data_wplywu,termin_plat,data_zapl,kontrahent_id,"
                                                Insert+="konto_wn,kwota_wn,konto_vat_wn,kwota_vat_wn,stawka_vat_wn,tresc_wn,sposob_rozl,waluta_wn,kurs_wn,kwota_wal_wn,"
                                                Insert+="nkup,konto_ma,kwota_ma,konto_vat_ma,kwota_vat_ma,stawka_vat_ma,tresc_ma,waluta_ma,kurs_ma,kwota_wal_ma,pnop,data_ksieg_kosztow)"
                                                Insert+=" VALUES "
                                                Insert+="('"+self.firma_id+"','"+OkresDekr+"','"+Dowod+"','"+Nazwa+"','"+Data+"',"+str(Numer)+","+str(NrPozDow)+",0,"+rsglobal.Dekoduj(str(poz[0]))+","
                                                Insert+="'"+tresc+" "+NrFakt+"','"+Data+"',NULL,"+TerminPlat+",NULL,'"+Platnik+"',"
                                                Insert+="'',0,'',0,NULL,'','','"+Waluta+"',"+Kurs+",0,"
                                                Insert+="FALSE,'"+KontoMa+"',"+KwotaMa+",'"+KontoVatMa+"',"+KwotaVatMa+",'"+Stawka+"','"+NrFakt+"','"+Waluta+"',"+Kurs+","+KwotaWalMa+",FALSE,'"+DataPrzych+"');"

                                                NrPozDow+=1

                            #Nie byo konta ma na wzorze
                            if not Jest:
                                dane.append("BR KMA$"+tresc+" "+NrFakt+" dla stawki "+Stawka)
                                return dane

            #WS 2020-04-03
            elif not CzyRozbicieNaTypyArt and CzyWystepujaKodyGTU:
                #------------------------------------------------------
                #--- DEKRETACJA Z PODZIAEM NA KODY GTU, BEZ TYPW ART
                #------------------------------------------------------

                # --- pobranie konta do zaokrgle
                SQL = "SELECT konto_zaokr FROM rs_wzory_dekretacji_typy " \
                      "WHERE firma_id='"+self.firma_id+"' AND okres_id='"+OkresDekr+"';"
                stat, DaneKontoZaokr, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
                if stat != 'OK':
                    dane.append("Bd pobrania konta do zaokrgle: "+stat)
                    return dane
                elif len(DaneKontoZaokr) == 0 or DaneKontoZaokr[0][0].strip() in ('', 'None'):
                    dane.append("Brak zdefiniowanego konta do zaokrgle")
                    return dane
                else:
                    KontoZaokr = rsglobal.Dekoduj(DaneKontoZaokr[0][0])

                KwZaokr=0

                # --- pobranie sum pozycji w podziale na kody gtu
                if tabela == 'rs_korekty':
                    Sumy = "SUM(fp.wartosc_netto-fp.wartosc_netto_p)"
                    Sumy += ", SUM(fp.wartosc_vat-fp.wartosc_vat_p)"
                    Sumy += ", SUM(fp.wartosc_wal_netto-fp.wartosc_wal_netto_p)"
                    # pola dodane dla przypadku gdy mamy rozbicie zwizane z rnymi stawkami vat na jednej pozycji
                    Sumy += ",fp.stawka_vat_p,SUM(fp.wartosc_netto_p)"
                    Sumy += ",SUM(fp.wartosc_vat_p)"
                    Sumy += ",SUM(fp.wartosc_wal_netto_p)"
                    Sumy += ",SUM(fp.wartosc_netto)"
                    Sumy += ",SUM(fp.wartosc_vat)"
                    Sumy += ",SUM(fp.wartosc_wal_netto)"
                    PolaGroupBy = ",fp.stawka_vat_p"
                    FiltrNaVAT = " HAVING SUM(fp.wartosc_netto-fp.wartosc_netto_p) != 0 OR " \
                                 " (fp.stawka_vat != fp.stawka_vat_p" \
                                 "  AND (SUM(fp.wartosc_netto) != 0 AND SUM(fp.wartosc_netto_p) != 0))"
                    Pola = 'fpo.wartosc_netto-fpo.wartosc_netto_p,fpo.wartosc_vat-fpo.wartosc_vat_p,'
                    Pola += 'fpo.wartosc_wal_netto-fpo.wartosc_wal_netto_p'
                    FiltrNaSume = ''
                    WherePom = '(wartosc_netto-wartosc_netto_p != 0 or stawka_vat != stawka_vat_p)'
                else:
                    Sumy = "SUM(fp.wartosc_netto), "
                    Sumy += "ROUND(SUM(fp.wartosc_netto)*" \
                            "CASE WHEN fp.stawka_vat IN ("+str(self.ListaStawekZn)[1:-1]+") " \
                            "THEN 0 ELSE CAST (CAST(fp.stawka_vat AS TEXT) AS NUMERIC)*0.01 END,2)"
                    Sumy += ", SUM(fp.wartosc_wal_netto) "
                    # pomocniczo, dla uoglnienia algorytmu pod stawke_p podstawiamy w fakturze stawke
                    Sumy += ", fp.stawka_vat "
                    FiltrNaVAT = ''
                    PolaGroupBy = ''
                    Pola = 'fpo.wartosc_netto,fpo.wartosc_vat,fpo.wartosc_wal_netto'
                    FiltrNaSume = ' HAVING SUM(fp.wartosc_netto) != 0 '
                    WherePom = 'wartosc_netto != 0'

                Sel = "SELECT fp.stawka_vat,COALESCE(fp.kod_gtu,'') as gtu," + Sumy + " FROM " + tabela + "_pozycje fp"
                Sel += " WHERE firma_id='" + self.firma_id + "' AND wyznacznik_id='" + Wyzn + "'"
                Sel += " AND data='" + Data + "' and numer=" + Nr
                Sel += " GROUP BY fp.stawka_vat,gtu"
                Sel += PolaGroupBy + ' ' + FiltrNaVAT + FiltrNaSume
                Sel += " ORDER BY stawka_vat,gtu;"
                stat, DanePoz, dump, dump = self.PolSQL.Execute(Sel, 3, CzyDodawacBlad=True)
                if stat != 'OK':
                    dane.append("Bd pobrania danych pozycji faktury/korekty: "+stat)
                    return dane

                # --- pobranie pozycji opisowych
                Sel = "SELECT fpo.stawka_vat,"+Pola+",COALESCE(fpo.kod_gtu,'') as gtu FROM "+tabela+"_pozycje_opisowe fpo "
                Sel += " WHERE "+WherePom+" and wyznacznik_id='"+Wyzn+"' AND data='"+Data+"' and numer="+Nr
                Sel += " ORDER BY nr_poz;"
                # jr dodanie wartosc_netto != 0 - przy korektach moze byc zle
                # zmienione - mr
                stat, DanePozOpis, dump, dump = self.PolSQL.Execute(Sel, 3, CzyDodawacBlad=True)
                if stat != 'OK':
                    dane.append("Bd pobrania danych pozycji opisowych faktury/korekty: "+stat)
                    return dane

                Insert=''
                StawkaP1=''
                KodGTUP1=''
                NrPozDow=2
                self.SStawkiVAT={}

                for poz in PozWz:
                    if Insert=='': #jeszcze nie znaleziono
                        if poz[1] not in ['','None'] or poz[9]: # jeli uzupenione konto wn lub chb z kontem wn z kontrahenta
                            if poz[9]:
                                SQL ="SELECT COALESCE(konto_odbiorca,'') FROM rs_kontrahenci WHERE firma_id='"+self.firma_id+"'"
                                SQL+=" AND kontrahent_id='"+Platnik+"'"
                                stat,DKontoWn,dump,dump = self.PolSQL.Execute(SQL,3)
                                if stat =='OK':
                                    KontoWn=rsglobal.Dekoduj(DKontoWn[0][0])
                            else:
                                KontoWn=rsglobal.Dekoduj(poz[1])
                            if stat =='OK':
                                if poz[2]:
                                    KontoWn+='-'+Platnik

                                Waluta=rsglobal.Dekoduj(dok[5])
                                Kurs=rsglobal.Dekoduj(str(dok[6]))

                                Insert="INSERT INTO rs_dowody_ksiegowe_pozycje (firma_id,okres_id,dowod_id,nazwa_id,data,numer,nr_poz,nr_ppoz,nr_pozw,"
                                Insert+="symbol_dok,data_wyst,data_wplywu,termin_plat,data_zapl,kontrahent_id,"
                                Insert+="konto_wn,kwota_wn,konto_vat_wn,kwota_vat_wn,stawka_vat_wn,tresc_wn,sposob_rozl,waluta_wn,kurs_wn,kwota_wal_wn,"
                                Insert+="nkup,konto_ma,kwota_ma,konto_vat_ma,kwota_vat_ma,stawka_vat_ma,tresc_ma,waluta_ma,kurs_ma,kwota_wal_ma,pnop,kod_gtu,data_ksieg_kosztow)"
                                Insert+=" VALUES "
                                Insert+="('"+self.firma_id+"','"+OkresDekr+"','"+Dowod+"','"+Nazwa+"','"+Data+"',"+str(Numer)+",1,0,"+rsglobal.Dekoduj(str(poz[0]))+","
                                Insert+="'"+tresc+" "+NrFakt+"','"+Data+"',NULL,"+TerminPlat+",NULL,'"+Platnik+"',"
                                Insert+="'"+KontoWn+"',"+rsglobal.Dekoduj(str(dok[7]))+",'',0,NULL,'"+NrFakt+"','','"+Waluta+"',"+Kurs+","+rsglobal.Dekoduj(str(dok[8]))+","

                                #sprawdzenie czy na pozycji z nagwkiem wn jest tez cz ma. Jeli jest to uzupeniana, jak nie to nie.

                                Jest=False
                                if poz[4] not in ['','None']:
                                    StawkaP1=rsglobal.Dekoduj(poz[5])
                                    for psttyp in DanePoz:
                                        if psttyp[0]==StawkaP1 and not Jest:
                                            StawkaPoprzP1=rsglobal.Dekoduj(psttyp[5])
                                            KodGTUP1 = psttyp[1]
                                            Jest=True
                                            KontoMa=rsglobal.Dekoduj(poz[3])
                                            KontoVatMa=rsglobal.Dekoduj(poz[4])
                                            if KontoVatMa =='None':
                                                KontoVatMa=''
                                            if tabela=='rs_korekty':
                                                if str(poz[7]).strip() not in ('None',''):
                                                    KontoVatMa=rsglobal.Dekoduj(str(poz[7]))
                                            # jeli stawka i stawka p s takie same to robimy normalnie bo to faktura lub korekta bez zmiany stawki vat
                                            if psttyp[0]==psttyp[5]:
                                                KwotaMa=rsglobal.Dekoduj(str(psttyp[2]))
                                                KwotaVatMa=rsglobal.Dekoduj(str(psttyp[3]))
                                                KwotaWalMa=rsglobal.Dekoduj(str(psttyp[4]))
                                            else:
                                                # to jest sprytny zabieg. Odejmujemy wartoci_p a dodajemy (w drugiej pozycji) wartosci aktualne.
                                                # do domknicia wykorzystujemy cz wspln obu algorytmw (Insert+"FALSE,'"+KontoMa'" itd)

                                                NrPozDow=3 # wiemy e bd na pocztku dwie pozycje

                                                # zapis kocwki pozycji 1 z danymi wartociami_p
                                                KwotaMa=rsglobal.Dekoduj(str(psttyp[6]*(-1)))
                                                KwotaVatMa=rsglobal.Dekoduj(str(psttyp[7]*(-1)))
                                                KwotaWalMa=rsglobal.Dekoduj(str(psttyp[8]*(-1)))
                                                Insert+="FALSE,'"+KontoMa+"',"+KwotaMa+",'"+KontoVatMa+"',"+KwotaVatMa+",'"+StawkaPoprzP1+"','"+NrFakt+"','"+Waluta+"',"+Kurs+","+KwotaWalMa+",FALSE,"+rsglobal.ZmienNaNULL(KodGTUP1)+",'"+DataPrzych+"');"
                                                self.SStawkiVAT[StawkaPoprzP1]=psttyp[7]*(-1)

                                                # pocztek drugiej pozycji z danymi wartociami aktualnymi (strona wn jest tu ju pusta)
                                                Insert+="INSERT INTO rs_dowody_ksiegowe_pozycje (firma_id,okres_id,dowod_id,nazwa_id,data,numer,nr_poz,nr_ppoz,nr_pozw,"
                                                Insert+="symbol_dok,data_wyst,data_wplywu,termin_plat,data_zapl,kontrahent_id,"
                                                Insert+="konto_wn,kwota_wn,konto_vat_wn,kwota_vat_wn,stawka_vat_wn,tresc_wn,sposob_rozl,waluta_wn,kurs_wn,kwota_wal_wn,"
                                                Insert+="nkup,konto_ma,kwota_ma,konto_vat_ma,kwota_vat_ma,stawka_vat_ma,tresc_ma,waluta_ma,kurs_ma,kwota_wal_ma,pnop,kod_gtu,data_ksieg_kosztow)"
                                                Insert+=" VALUES "
                                                Insert+="('"+self.firma_id+"','"+OkresDekr+"','"+Dowod+"','"+Nazwa+"','"+Data+"',"+str(Numer)+",2,0,"+rsglobal.Dekoduj(str(poz[0]))+","
                                                Insert+="'"+tresc+" "+NrFakt+"','"+Data+"',NULL,"+TerminPlat+",NULL,'"+Platnik+"',"
                                                Insert+="'',0,'',0,NULL,'','','"+Waluta+"',"+Kurs+",0,"

                                                KwotaMa=rsglobal.Dekoduj(str(psttyp[9]))
                                                KwotaVatMa=rsglobal.Dekoduj(str(psttyp[10]))
                                                KwotaWalMa=rsglobal.Dekoduj(str(psttyp[11]))
                                                self.SStawkiVAT[StawkaP1]=psttyp[10]

                                            Insert+="FALSE,'"+KontoMa+"',"+KwotaMa+",'"+KontoVatMa+"',"+KwotaVatMa+",'"+StawkaP1+"','"+NrFakt+"','"+Waluta+"',"+Kurs+","+KwotaWalMa+",FALSE,"+rsglobal.ZmienNaNULL(psttyp[1])
                                            if not StawkaP1 in self.SStawkiVAT:
                                                self.SStawkiVAT[StawkaP1]=psttyp[3]
                                if not Jest:
                                    Insert+="FALSE,'',0,'',0,NULL,'','"+Waluta+"',"+Kurs+",0,FALSE,NULL"

                                Insert+=",'"+DataPrzych+"');"

                #Nie byo konta wn na wzorze
                if Insert=='':
                    dane.append("BR KWN$"+tresc+" "+NrFakt)
                    return dane

                #Sprawdzenie pozostaych stawek vat z faktury i wpisanie ich na ma

                #zmienna ta suy do tego by ptla przebiega 2 razy. Za pierwszym razem uzupeniana jest stawka, ktra zostaa
                #ju pobrana w nagwkowej czsci zostaje ew. uzupeniona, potem przebieg drugi uzupeniajcy reszt stawek
                #rsglobal.ZapiszDoLogow('Stawki 1 '+str(self.SStawkiVAT),1)
                PierwszyPrzebieg=True
                Robic=True
                while Robic:
                    for psttyp in DanePoz:
                        Stawka=rsglobal.Dekoduj(psttyp[0])
                        StawkaP=rsglobal.Dekoduj(psttyp[5])
                        KodGTU=rsglobal.Dekoduj(psttyp[1])
                        if (PierwszyPrzebieg and Stawka==StawkaP1 and (StawkaP != StawkaPoprzP1 or KodGTU != KodGTUP1)) or (not PierwszyPrzebieg and Stawka != StawkaP1):
                            Jest=False
                            #ptla po pozycjach wzoru dekretacji
                            for poz in PozWz:
                                if not Jest:
                                    if poz[3] not in ['','None']:
                                        if Stawka==poz[5]:
                                            Jest=True
                                            KontoMa=rsglobal.Dekoduj(poz[3])
                                            KontoVatMa=rsglobal.Dekoduj(poz[4])
                                            if KontoVatMa =='None':
                                                KontoVatMa=''
                                            if tabela=='rs_korekty':
                                                if str(poz[7]).strip() not in ('None',''):
                                                    KontoVatMa=rsglobal.Dekoduj(str(poz[7]))
                                            if Stawka==StawkaP:
                                                # nie ma zmiany stawki vat - jedna pozycja
                                                KwotaMa=rsglobal.Dekoduj(str(psttyp[2]))
                                                KwotaVatMa=rsglobal.Dekoduj(str(psttyp[3]))
                                                KwotaWalMa=rsglobal.Dekoduj(str(psttyp[4]))
                                            else:
                                                # zmiana stawki vat - dwie pozycje.
                                                # najpierw odjcie wartoci starych
                                                KwotaMa=rsglobal.Dekoduj(str(psttyp[6]*(-1)))
                                                KwotaVatMa=rsglobal.Dekoduj(str(psttyp[7]*(-1)))
                                                KwotaWalMa=rsglobal.Dekoduj(str(psttyp[8]*(-1)))

                                                Insert+="INSERT INTO rs_dowody_ksiegowe_pozycje (firma_id,okres_id,dowod_id,nazwa_id,data,numer,nr_poz,nr_ppoz,nr_pozw,"
                                                Insert+="symbol_dok,data_wyst,data_wplywu,termin_plat,data_zapl,kontrahent_id,"
                                                Insert+="konto_wn,kwota_wn,konto_vat_wn,kwota_vat_wn,stawka_vat_wn,tresc_wn,sposob_rozl,waluta_wn,kurs_wn,kwota_wal_wn,"
                                                Insert+="nkup,konto_ma,kwota_ma,konto_vat_ma,kwota_vat_ma,stawka_vat_ma,tresc_ma,waluta_ma,kurs_ma,kwota_wal_ma,pnop,kod_gtu,data_ksieg_kosztow)"
                                                Insert+=" VALUES "
                                                Insert+="('"+self.firma_id+"','"+OkresDekr+"','"+Dowod+"','"+Nazwa+"','"+Data+"',"+str(Numer)+","+str(NrPozDow)+",0,"+rsglobal.Dekoduj(str(poz[0]))+","
                                                Insert+="'"+tresc+" "+NrFakt+"','"+Data+"',NULL,"+TerminPlat+",NULL,'"+Platnik+"',"
                                                Insert+="'',0,'',0,NULL,'','','"+Waluta+"',"+Kurs+",0,"
                                                Insert+="FALSE,'"+KontoMa+"',"+KwotaMa+",'"+KontoVatMa+"',"+KwotaVatMa+",'"+StawkaP+"','"+NrFakt+"','"+Waluta+"',"+Kurs+","+KwotaWalMa+",FALSE,"+rsglobal.ZmienNaNULL(KodGTU)+",'"+DataPrzych+"');"

                                                NrPozDow+=1

                                                if StawkaP in self.SStawkiVAT:
                                                    self.SStawkiVAT[StawkaP]+=psttyp[7]*(-1)
                                                else:
                                                    self.SStawkiVAT[StawkaP]=psttyp[7]*(-1)

                                                # dodanie wartoci nowych
                                                KwotaMa=rsglobal.Dekoduj(str(psttyp[9]))
                                                KwotaVatMa=rsglobal.Dekoduj(str(psttyp[10]))
                                                KwotaWalMa=rsglobal.Dekoduj(str(psttyp[11]))

                                                if Stawka in self.SStawkiVAT:
                                                    self.SStawkiVAT[Stawka]+=psttyp[10]
                                                else:
                                                    self.SStawkiVAT[Stawka]=psttyp[10]

                                            # cz wsplna dla obu wersji
                                            Insert+="INSERT INTO rs_dowody_ksiegowe_pozycje (firma_id,okres_id,dowod_id,nazwa_id,data,numer,nr_poz,nr_ppoz,nr_pozw,"
                                            Insert+="symbol_dok,data_wyst,data_wplywu,termin_plat,data_zapl,kontrahent_id,"
                                            Insert+="konto_wn,kwota_wn,konto_vat_wn,kwota_vat_wn,stawka_vat_wn,tresc_wn,sposob_rozl,waluta_wn,kurs_wn,kwota_wal_wn,"
                                            Insert+="nkup,konto_ma,kwota_ma,konto_vat_ma,kwota_vat_ma,stawka_vat_ma,tresc_ma,waluta_ma,kurs_ma,kwota_wal_ma,pnop,kod_gtu,data_ksieg_kosztow)"
                                            Insert+=" VALUES "
                                            Insert+="('"+self.firma_id+"','"+OkresDekr+"','"+Dowod+"','"+Nazwa+"','"+Data+"',"+str(Numer)+","+str(NrPozDow)+",0,"+rsglobal.Dekoduj(str(poz[0]))+","
                                            Insert+="'"+tresc+" "+NrFakt+"','"+Data+"',NULL,"+TerminPlat+",NULL,'"+Platnik+"',"
                                            Insert+="'',0,'',0,NULL,'','','"+Waluta+"',"+Kurs+",0,"
                                            Insert+="FALSE,'"+KontoMa+"',"+KwotaMa+",'"+KontoVatMa+"',"+KwotaVatMa+",'"+Stawka+"','"+NrFakt+"','"+Waluta+"',"+Kurs+","+KwotaWalMa+",FALSE,"+rsglobal.ZmienNaNULL(KodGTU)+",'"+DataPrzych+"');"

                                            NrPozDow+=1
                                            if Stawka==StawkaP:
                                                if Stawka in self.SStawkiVAT:
                                                    self.SStawkiVAT[Stawka]+=psttyp[3]
                                                else:
                                                    self.SStawkiVAT[Stawka]=psttyp[3]

                            #Nie byo konta ma na wzorze
                            if not Jest:
                                dane.append("BR KMA$"+tresc+" "+NrFakt+" dla stawki "+Stawka)
                                return dane

                    if not PierwszyPrzebieg:
                        Robic= False
                    PierwszyPrzebieg=False
                #pozycje opisowe
                for pozop in DanePozOpis:
                    Stawka=rsglobal.Dekoduj(pozop[0])
                    Jest=False
                    #ptla po pozycjach wzoru dekretacji
                    for poz in PozWz:
                        if not Jest:
                            if poz[3] not in ['','None']:
                                if Stawka==poz[5]:
                                    Jest=True
                                    KontoMa='XXX'
                                    KontoVatMa=rsglobal.Dekoduj(poz[4])
                                    if KontoVatMa =='None':
                                        KontoVatMa=''
                                    KwotaMa=rsglobal.Dekoduj(str(pozop[1]))
                                    KwotaVatMa=rsglobal.Dekoduj(str(pozop[2]))
                                    KwotaWalMa=rsglobal.Dekoduj(str(pozop[3]))
                                    KodGTU = pozop[4]

                                    Insert+="INSERT INTO rs_dowody_ksiegowe_pozycje (firma_id,okres_id,dowod_id,nazwa_id,data,numer,nr_poz,nr_ppoz,nr_pozw,"
                                    Insert+="symbol_dok,data_wyst,data_wplywu,termin_plat,data_zapl,kontrahent_id,"
                                    Insert+="konto_wn,kwota_wn,konto_vat_wn,kwota_vat_wn,stawka_vat_wn,tresc_wn,sposob_rozl,waluta_wn,kurs_wn,kwota_wal_wn,"
                                    Insert+="nkup,konto_ma,kwota_ma,konto_vat_ma,kwota_vat_ma,stawka_vat_ma,tresc_ma,waluta_ma,kurs_ma,kwota_wal_ma,pnop,kod_gtu,data_ksieg_kosztow)"
                                    Insert+=" VALUES "
                                    Insert+="('"+self.firma_id+"','"+OkresDekr+"','"+Dowod+"','"+Nazwa+"','"+Data+"',"+str(Numer)+","+str(NrPozDow)+",0,"+rsglobal.Dekoduj(str(poz[0]))+","
                                    Insert+="'"+tresc+" "+NrFakt+"','"+Data+"',NULL,"+TerminPlat+",NULL,'"+Platnik+"',"
                                    Insert+="'',0,'',0,NULL,'','','"+Waluta+"',"+Kurs+",0,"
                                    Insert+="FALSE,'"+KontoMa+"',"+KwotaMa+",'"+KontoVatMa+"',"+KwotaVatMa+",'"+Stawka+"','"+NrFakt+"','"+Waluta+"',"+Kurs+","+KwotaWalMa+",FALSE,"+rsglobal.ZmienNaNULL(KodGTU)+",'"+DataPrzych+"');"

                                    NrPozDow+=1
                                    if Stawka in self.SStawkiVAT:
                                        self.SStawkiVAT[Stawka]+=pozop[2]
                                    else:
                                        self.SStawkiVAT[Stawka]=pozop[2]

                    #Nie byo konta ma na wzorze
                    if not Jest:
                        dane.append("BR KMA$"+tresc+" "+NrFakt+" dla stawki "+Stawka)
                        return dane

                #policzenie i wstawienie zaokraglen vatu
                #rsglobal.ZapiszDoLogow('Stawki 3 '+str(self.SStawkiVAT),1)
                #rsglobal.ZapiszDoLogow('Stawki 3b '+str(StVat),1)

                # teraz StVat maj inn struktur dla faktur a inn dla korekt
                for Stawka in self.SStawkiVAT:
                    for pst in StVat:
                        if pst[0]==Stawka:
                            if tabela=='rs_faktury':
                                KwZaokr+=rsglobal.RSRound(pst[2]-self.SStawkiVAT[Stawka],2)
                            else:
                                KwZaokr+=rsglobal.RSRound(pst[4]-self.SStawkiVAT[Stawka],2)

                #jeli kwota zaokgle != 0 to dodajemy pozycj
                if KwZaokr != 0:
                    Insert+="INSERT INTO rs_dowody_ksiegowe_pozycje (firma_id,okres_id,dowod_id,nazwa_id,data,numer,nr_poz,nr_ppoz,nr_pozw,"
                    Insert+="symbol_dok,data_wyst,data_wplywu,termin_plat,data_zapl,kontrahent_id,"
                    Insert+="konto_wn,kwota_wn,konto_vat_wn,kwota_vat_wn,stawka_vat_wn,tresc_wn,sposob_rozl,waluta_wn,kurs_wn,kwota_wal_wn,"
                    Insert+="nkup,konto_ma,kwota_ma,konto_vat_ma,kwota_vat_ma,stawka_vat_ma,tresc_ma,waluta_ma,kurs_ma,kwota_wal_ma,pnop,data_ksieg_kosztow)"
                    Insert+=" VALUES "
                    Insert+="('"+self.firma_id+"','"+OkresDekr+"','"+Dowod+"','"+Nazwa+"','"+Data+"',"+str(Numer)+","+str(NrPozDow)+",0,1,"
                    Insert+="'"+tresc+" "+NrFakt+"','"+Data+"',NULL,"+TerminPlat+",NULL,'"+Platnik+"',"
                    Insert+="'',0,'',0,NULL,'','','"+Waluta+"',"+Kurs+",0,"
                    Insert+="FALSE,'"+KontoZaokr+"',"+str(KwZaokr)+",'',0,NULL,'"+NrFakt+"','"+Waluta+"',"+Kurs+","+str(KwZaokr)+",FALSE,'"+DataPrzych+"');"

            else:
            #------------------------------------------------------------
            #--- DEKRETACJA Z ROZBICIEM NA TYPY ARTYKULOW I KODY GTU
            #------------------------------------------------------------

                #pobranie kont z okresu w podziale na typy
                stat,DaneKontaTypow,dump,dump = self.PolSQL.Execute("SELECT konto_01, konto_02, konto_03, konto_04, konto_05, konto_11, konto_12, konto_21, konto_31, konto_99, konto_zaokr FROM rs_wzory_dekretacji_typy WHERE firma_id='"+self.firma_id+"' AND okres_id='"+OkresDekr+"'",3)
                if len(DaneKontaTypow)==0:
                    dane.append("BR KT")
                    return dane

                #stworzenie slownika zaleznego od typow
                SlKTypow={}
                SlKTypow['01']=rsglobal.Dekoduj(DaneKontaTypow[0][0])
                SlKTypow['02']=rsglobal.Dekoduj(DaneKontaTypow[0][1])
                SlKTypow['03']=rsglobal.Dekoduj(DaneKontaTypow[0][2])
                SlKTypow['04']=rsglobal.Dekoduj(DaneKontaTypow[0][3])
                SlKTypow['05']=rsglobal.Dekoduj(DaneKontaTypow[0][4])
                SlKTypow['11']=rsglobal.Dekoduj(DaneKontaTypow[0][5])
                SlKTypow['12']=rsglobal.Dekoduj(DaneKontaTypow[0][6])
                SlKTypow['21']=rsglobal.Dekoduj(DaneKontaTypow[0][7])
                SlKTypow['31']=rsglobal.Dekoduj(DaneKontaTypow[0][8])
                SlKTypow['99']=rsglobal.Dekoduj(DaneKontaTypow[0][9])
                #WS 2020-04-03
                if DaneKontaTypow[0][10].strip() not in ('','None'):
                    KontoZaokr = rsglobal.Dekoduj(DaneKontaTypow[0][10])
                else:
                    dane.append("Brak zdefiniowanego konta do zaokrgle")
                    return dane
                # KontoZaokr=rsglobal.Dekoduj(DaneKontaTypow[0][10])
                KwZaokr=0

                #pobranie sum pozycji w podziale na typy
                if tabela=='rs_korekty':
                    Sumy="SUM(fp.wartosc_netto-fp.wartosc_netto_p)"
                    Sumy+=", SUM(fp.wartosc_vat-fp.wartosc_vat_p)"
                    Sumy+=", SUM(fp.wartosc_wal_netto-fp.wartosc_wal_netto_p)"
                    #pola dodane dla przypadku gdy mamy rozbicie zwizane z rnymi stawkami vat na jednej pozycji
                    Sumy+=",fp.stawka_vat_p,SUM(fp.wartosc_netto_p)"
                    Sumy+=",SUM(fp.wartosc_vat_p)"
                    Sumy+=",SUM(fp.wartosc_wal_netto_p)"
                    Sumy+=",SUM(fp.wartosc_netto)"
                    Sumy+=",SUM(fp.wartosc_vat)"
                    Sumy+=",SUM(fp.wartosc_wal_netto)"
                    PolaGroupBy=",fp.stawka_vat_p"
                    FiltrNaVAT=" HAVING SUM(fp.wartosc_netto-fp.wartosc_netto_p) != 0 OR (fp.stawka_vat!= fp.stawka_vat_p AND (SUM(fp.wartosc_netto)!=0 AND SUM(fp.wartosc_netto_p)!=0))"
                    Pola='fpo.wartosc_netto-fpo.wartosc_netto_p,fpo.wartosc_vat-fpo.wartosc_vat_p,fpo.wartosc_wal_netto-fpo.wartosc_wal_netto_p'
                    FiltrNaSume = ''
                    WherePom='(wartosc_netto-wartosc_netto_p != 0 or stawka_vat!=stawka_vat_p)'
                else:
                    Sumy="SUM(fp.wartosc_netto), ROUND(SUM(fp.wartosc_netto)*CASE WHEN fp.stawka_vat IN ("+str(self.ListaStawekZn)[1:-1]+") THEN 0 ELSE CAST (CAST(fp.stawka_vat AS TEXT) AS NUMERIC)*0.01 END,2)"
                    Sumy+=", SUM(fp.wartosc_wal_netto) "
                    # pomocniczo, dla uoglnienia algorytmu pod stawke_p podstawiamy w fakturze stawke
                    Sumy+=", fp.stawka_vat "
                    Sumy+=",NULL,NULL,NULL,NULL,NULL,NULL"  #WS 2020-04-03 - zalepki na pola 6-11, 12ty bdzie kod_gtu
                    FiltrNaVAT=''
                    PolaGroupBy=""
                    Pola='fpo.wartosc_netto,fpo.wartosc_vat,fpo.wartosc_wal_netto'
                    FiltrNaSume = ' HAVING SUM(fp.wartosc_netto) != 0 '
                    WherePom='wartosc_netto != 0'

                #WS 2020-04-03 - kod_gtu dodany do Select, Group by, Order by
                Sel="SELECT fp.stawka_vat,a.typ_artykulu,"+Sumy+",COALESCE(fp.kod_gtu,'') as gtu FROM "+tabela+"_pozycje fp INNER JOIN rs_artykuly a using (firma_id,artykul_id)"
                Sel+=" WHERE firma_id='"+self.firma_id+"' AND wyznacznik_id='"+Wyzn+"' AND data='"+Data+"' and numer="+Nr+" GROUP BY fp.stawka_vat,"
                Sel+=" a.typ_artykulu"+PolaGroupBy+',gtu '+FiltrNaVAT+FiltrNaSume+" ORDER BY stawka_vat,typ_artykulu,gtu"
                stat,DanePoz,dump,dump = self.PolSQL.Execute(Sel,3,CzyDodawacBlad=True)
                #WS 2020-04-03
                if stat != 'OK':
                    dane.append("Bd pobrania danych pozycji faktury/korekty: "+stat)
                    return dane

                #pobranie pozycji opisowych
                Sel="SELECT fpo.stawka_vat,"+Pola+",COALESCE(fpo.kod_gtu,'') as gtu FROM "+tabela+"_pozycje_opisowe fpo "  #WS 2020-04-03 - kod_gtu
                Sel+=" WHERE "+WherePom+" and wyznacznik_id='"+Wyzn+"' AND data='"+Data+"' and numer="+Nr+" ORDER BY nr_poz"
                # jr dodanie wartosc_netto!=0 - przy korektach moze byc zle
                # zmienione - mr

                stat,DanePozOpis,dump,dump = self.PolSQL.Execute(Sel, 3, CzyDodawacBlad=True)
                #WS 2020-04-03
                if stat != 'OK':
                    dane.append("Bd pobrania danych pozycji opisowych faktury/korekty: "+stat)
                    return dane

                Insert=''
                StawkaP1=''
                TypArtP1=''
                KodGTUP1=''  #WS 2020-04-03
                NrPozDow=2
                self.SStawkiVAT={}

                for poz in PozWz:
                    if Insert=='': #jeszcze nie znaleziono
                        if poz[1] not in ['','None'] or poz[9]: # jeli uzupenione konto wn lub chb z kontem wn z kontrahenta
                            if poz[9]:
                                SQL ="SELECT COALESCE(konto_odbiorca,'') FROM rs_kontrahenci WHERE firma_id='"+self.firma_id+"'"
                                SQL+=" AND kontrahent_id='"+Platnik+"'"
                                stat,DKontoWn,dump,dump = self.PolSQL.Execute(SQL,3)
                                if stat =='OK':
                                    KontoWn=rsglobal.Dekoduj(DKontoWn[0][0])
                            else:
                                KontoWn=rsglobal.Dekoduj(poz[1])
                            if stat =='OK':
                                if poz[2]:
                                    KontoWn+='-'+Platnik

                                Waluta=rsglobal.Dekoduj(dok[5])
                                Kurs=rsglobal.Dekoduj(str(dok[6]))

                                Insert="INSERT INTO rs_dowody_ksiegowe_pozycje (firma_id,okres_id,dowod_id,nazwa_id,data,numer,nr_poz,nr_ppoz,nr_pozw,"
                                Insert+="symbol_dok,data_wyst,data_wplywu,termin_plat,data_zapl,kontrahent_id,"
                                Insert+="konto_wn,kwota_wn,konto_vat_wn,kwota_vat_wn,stawka_vat_wn,tresc_wn,sposob_rozl,waluta_wn,kurs_wn,kwota_wal_wn,"
                                #WS 2020-04-03 - kod_gtu
                                Insert+="nkup,konto_ma,kwota_ma,konto_vat_ma,kwota_vat_ma,stawka_vat_ma,tresc_ma,waluta_ma,kurs_ma,kwota_wal_ma,pnop,kod_gtu,data_ksieg_kosztow)"
                                Insert+=" VALUES "
                                Insert+="('"+self.firma_id+"','"+OkresDekr+"','"+Dowod+"','"+Nazwa+"','"+Data+"',"+str(Numer)+",1,0,"+rsglobal.Dekoduj(str(poz[0]))+","
                                Insert+="'"+tresc+" "+NrFakt+"','"+Data+"',NULL,"+TerminPlat+",NULL,'"+Platnik+"',"
                                Insert+="'"+KontoWn+"',"+rsglobal.Dekoduj(str(dok[7]))+",'',0,NULL,'"+NrFakt+"','','"+Waluta+"',"+Kurs+","+rsglobal.Dekoduj(str(dok[8]))+","

                                #sprawdzenie czy na pozycji z nagwkiem wn jest tez cz ma. Jeli jest to uzupeniana, jak nie to nie.

                                Jest=False
                                if poz[4] not in ['','None']:
                                    StawkaP1=rsglobal.Dekoduj(poz[5])
                                    for psttyp in DanePoz:
                                        if psttyp[0]==StawkaP1 and not Jest:
                                            StawkaPoprzP1=rsglobal.Dekoduj(psttyp[5])
                                            Jest=True
                                            TypArtP1 = psttyp[1]
                                            KodGTUP1 = psttyp[12]  #WS 2020-04-03
                                            #WS 2020-04-03
                                            if TypArtP1 in ('','None',None):
                                                dane.append('Brak typu dla jednego z artykuw faktury/korekty '+NrFakt)
                                                return dane
                                            if SlKTypow[TypArtP1] in ('','None',None):
                                                dane.append("BR KTYP$ dla typu artykuu "+TypArtP1)
                                                return dane
                                            KontoMa=rsglobal.Dekoduj(poz[3]).replace('*',SlKTypow[TypArtP1])
                                            KontoVatMa=rsglobal.Dekoduj(poz[4])
                                            if KontoVatMa =='None':
                                                KontoVatMa=''
                                            if tabela=='rs_korekty':
                                                if str(poz[7]).strip() not in ('None',''):
                                                    KontoVatMa=rsglobal.Dekoduj(str(poz[7]))
                                            # jeli stawka i stawka p s takie same to robimy normalnie bo to faktura lub korekta bez zmiany stawki vat
                                            if psttyp[0]==psttyp[5]:
                                                KwotaMa=rsglobal.Dekoduj(str(psttyp[2]))
                                                KwotaVatMa=rsglobal.Dekoduj(str(psttyp[3]))
                                                KwotaWalMa=rsglobal.Dekoduj(str(psttyp[4]))
                                            else:
                                                # to jest sprytny zabieg. Odejmujemy wartoci_p a dodajemy (w drugiej pozycji) wartosci aktualne.
                                                # do domknicia wykorzystujemy cz wspln obu algorytmw (Insert+"FALSE,'"+KontoMa'" itd)

                                                NrPozDow=3 # wiemy e bd na pocztku dwie pozycje

                                                # zapis kocwki pozycji 1 z danymi wartociami_p
                                                KwotaMa=rsglobal.Dekoduj(str(psttyp[6]*(-1)))
                                                KwotaVatMa=rsglobal.Dekoduj(str(psttyp[7]*(-1)))
                                                KwotaWalMa=rsglobal.Dekoduj(str(psttyp[8]*(-1)))
                                                #WS 2020-04-03 +rsglobal.ZmienNaNULL(KodGTUP1)
                                                Insert+="FALSE,'"+KontoMa+"',"+KwotaMa+",'"+KontoVatMa+"',"+KwotaVatMa+",'"+StawkaPoprzP1+"','"+NrFakt+"','"+Waluta+"',"+Kurs+","+KwotaWalMa+",FALSE,"+rsglobal.ZmienNaNULL(KodGTUP1)+",'"+DataPrzych+"');"
                                                self.SStawkiVAT[StawkaPoprzP1]=psttyp[7]*(-1)

                                                # pocztek drugiej pozycji z danymi wartociami aktualnymi (strona wn jest tu ju pusta)
                                                Insert+="INSERT INTO rs_dowody_ksiegowe_pozycje (firma_id,okres_id,dowod_id,nazwa_id,data,numer,nr_poz,nr_ppoz,nr_pozw,"
                                                Insert+="symbol_dok,data_wyst,data_wplywu,termin_plat,data_zapl,kontrahent_id,"
                                                Insert+="konto_wn,kwota_wn,konto_vat_wn,kwota_vat_wn,stawka_vat_wn,tresc_wn,sposob_rozl,waluta_wn,kurs_wn,kwota_wal_wn,"
                                                #WS 2020-04-03 - kod_gtu
                                                Insert+="nkup,konto_ma,kwota_ma,konto_vat_ma,kwota_vat_ma,stawka_vat_ma,tresc_ma,waluta_ma,kurs_ma,kwota_wal_ma,pnop,kod_gtu,data_ksieg_kosztow)"
                                                Insert+=" VALUES "
                                                Insert+="('"+self.firma_id+"','"+OkresDekr+"','"+Dowod+"','"+Nazwa+"','"+Data+"',"+str(Numer)+",2,0,"+rsglobal.Dekoduj(str(poz[0]))+","
                                                Insert+="'"+tresc+" "+NrFakt+"','"+Data+"',NULL,"+TerminPlat+",NULL,'"+Platnik+"',"
                                                Insert+="'',0,'',0,NULL,'','','"+Waluta+"',"+Kurs+",0,"

                                                KwotaMa=rsglobal.Dekoduj(str(psttyp[9]))
                                                KwotaVatMa=rsglobal.Dekoduj(str(psttyp[10]))
                                                KwotaWalMa=rsglobal.Dekoduj(str(psttyp[11]))
                                                self.SStawkiVAT[StawkaP1]=psttyp[10]

                                            #WS 2020-04-03 +rsglobal.ZmienNaNULL(psttyp[12])
                                            Insert+="FALSE,'"+KontoMa+"',"+KwotaMa+",'"+KontoVatMa+"',"+KwotaVatMa+",'"+StawkaP1+"','"+NrFakt+"','"+Waluta+"',"+Kurs+","+KwotaWalMa+",FALSE,"+rsglobal.ZmienNaNULL(psttyp[12])
                                            if not StawkaP1 in self.SStawkiVAT:
                                                self.SStawkiVAT[StawkaP1]=psttyp[3]
                                if not Jest:
                                    Insert+="FALSE,'',0,'',0,NULL,'','"+Waluta+"',"+Kurs+",0,FALSE,NULL"

                                Insert+=",'"+DataPrzych+"');"

                #Nie byo konta wn na wzorze
                if Insert=='':
                    dane.append("BR KWN$"+tresc+" "+NrFakt)
                    return dane

                #Sprawdzenie pozostaych stawek vat z faktury i wpisanie ich na ma

                #zmienna ta suy do tego by ptla przebiega 2 razy. Za pierwszym razem uzupeniana jest stawka, ktra zostaa
                #ju pobrana w nagwkowej czsci zostaje ew. uzupeniona, potem przebieg drugi uzupeniajcy reszt stawek
                #rsglobal.ZapiszDoLogow('Stawki 1 '+str(self.SStawkiVAT),1)
                PierwszyPrzebieg=True
                Robic=True
                while Robic:
                    for psttyp in DanePoz:
                        Stawka=rsglobal.Dekoduj(psttyp[0])
                        StawkaP=rsglobal.Dekoduj(psttyp[5])
                        TypArt=rsglobal.Dekoduj(psttyp[1])
                        KodGTU=rsglobal.Dekoduj(psttyp[12])  #WS 2020-04-03
                        if (PierwszyPrzebieg and Stawka==StawkaP1 and (StawkaP != StawkaPoprzP1 or TypArt != TypArtP1 or KodGTU != KodGTUP1)) or (not PierwszyPrzebieg and Stawka != StawkaP1):
                            Jest=False
                            #ptla po pozycjach wzoru dekretacji
                            for poz in PozWz:
                                if not Jest:
                                    if poz[3] not in ['','None']:
                                        if Stawka==poz[5]:
                                            Jest=True
                                            #WS 2020-04-03
                                            if TypArt in ('','None',None):
                                                dane.append('Brak typu dla jednego z artykuw faktury/korekty '+NrFakt)
                                                return dane
                                            if SlKTypow[TypArt] in ('','None',None):
                                                dane.append("BR KTYP$ dla typu artykuu "+TypArt)
                                                return dane
                                            KontoMa=rsglobal.Dekoduj(poz[3]).replace('*',SlKTypow[TypArt])
                                            KontoVatMa=rsglobal.Dekoduj(poz[4])
                                            if KontoVatMa =='None':
                                                KontoVatMa=''
                                            if tabela=='rs_korekty':
                                                if str(poz[7]).strip() not in ('None',''):
                                                    KontoVatMa=rsglobal.Dekoduj(str(poz[7]))
                                            if Stawka==StawkaP:
                                                # nie ma zmiany stawki vat - jedna pozycja
                                                KwotaMa=rsglobal.Dekoduj(str(psttyp[2]))
                                                KwotaVatMa=rsglobal.Dekoduj(str(psttyp[3]))
                                                KwotaWalMa=rsglobal.Dekoduj(str(psttyp[4]))
                                            else:
                                                # zmiana stawki vat - dwie pozycje.
                                                # najpierw odjcie wartoci starych
                                                KwotaMa=rsglobal.Dekoduj(str(psttyp[6]*(-1)))
                                                KwotaVatMa=rsglobal.Dekoduj(str(psttyp[7]*(-1)))
                                                KwotaWalMa=rsglobal.Dekoduj(str(psttyp[8]*(-1)))

                                                Insert+="INSERT INTO rs_dowody_ksiegowe_pozycje (firma_id,okres_id,dowod_id,nazwa_id,data,numer,nr_poz,nr_ppoz,nr_pozw,"
                                                Insert+="symbol_dok,data_wyst,data_wplywu,termin_plat,data_zapl,kontrahent_id,"
                                                Insert+="konto_wn,kwota_wn,konto_vat_wn,kwota_vat_wn,stawka_vat_wn,tresc_wn,sposob_rozl,waluta_wn,kurs_wn,kwota_wal_wn,"
                                                #WS 2020-04-03 - kod_gtu
                                                Insert+="nkup,konto_ma,kwota_ma,konto_vat_ma,kwota_vat_ma,stawka_vat_ma,tresc_ma,waluta_ma,kurs_ma,kwota_wal_ma,pnop,kod_gtu,data_ksieg_kosztow)"
                                                Insert+=" VALUES "
                                                Insert+="('"+self.firma_id+"','"+OkresDekr+"','"+Dowod+"','"+Nazwa+"','"+Data+"',"+str(Numer)+","+str(NrPozDow)+",0,"+rsglobal.Dekoduj(str(poz[0]))+","
                                                Insert+="'"+tresc+" "+NrFakt+"','"+Data+"',NULL,"+TerminPlat+",NULL,'"+Platnik+"',"
                                                Insert+="'',0,'',0,NULL,'','','"+Waluta+"',"+Kurs+",0,"
                                                #WS 2020-04-03 +rsglobal.ZmienNaNULL(KodGTU)
                                                Insert+="FALSE,'"+KontoMa+"',"+KwotaMa+",'"+KontoVatMa+"',"+KwotaVatMa+",'"+StawkaP+"','"+NrFakt+"','"+Waluta+"',"+Kurs+","+KwotaWalMa+",FALSE,"+rsglobal.ZmienNaNULL(KodGTU)+",'"+DataPrzych+"');"

                                                NrPozDow+=1

                                                if StawkaP in self.SStawkiVAT:
                                                    self.SStawkiVAT[StawkaP]+=psttyp[7]*(-1)
                                                else:
                                                    self.SStawkiVAT[StawkaP]=psttyp[7]*(-1)

                                                # dodanie wartoci nowych
                                                KwotaMa=rsglobal.Dekoduj(str(psttyp[9]))
                                                KwotaVatMa=rsglobal.Dekoduj(str(psttyp[10]))
                                                KwotaWalMa=rsglobal.Dekoduj(str(psttyp[11]))

                                                if Stawka in self.SStawkiVAT:
                                                    self.SStawkiVAT[Stawka]+=psttyp[10]
                                                else:
                                                    self.SStawkiVAT[Stawka]=psttyp[10]

                                            # cz wsplna dla obu wersji
                                            Insert+="INSERT INTO rs_dowody_ksiegowe_pozycje (firma_id,okres_id,dowod_id,nazwa_id,data,numer,nr_poz,nr_ppoz,nr_pozw,"
                                            Insert+="symbol_dok,data_wyst,data_wplywu,termin_plat,data_zapl,kontrahent_id,"
                                            Insert+="konto_wn,kwota_wn,konto_vat_wn,kwota_vat_wn,stawka_vat_wn,tresc_wn,sposob_rozl,waluta_wn,kurs_wn,kwota_wal_wn,"
                                            #WS 2020-04-03 - kod_gtu
                                            Insert+="nkup,konto_ma,kwota_ma,konto_vat_ma,kwota_vat_ma,stawka_vat_ma,tresc_ma,waluta_ma,kurs_ma,kwota_wal_ma,pnop,kod_gtu,data_ksieg_kosztow)"
                                            Insert+=" VALUES "
                                            Insert+="('"+self.firma_id+"','"+OkresDekr+"','"+Dowod+"','"+Nazwa+"','"+Data+"',"+str(Numer)+","+str(NrPozDow)+",0,"+rsglobal.Dekoduj(str(poz[0]))+","
                                            Insert+="'"+tresc+" "+NrFakt+"','"+Data+"',NULL,"+TerminPlat+",NULL,'"+Platnik+"',"
                                            Insert+="'',0,'',0,NULL,'','','"+Waluta+"',"+Kurs+",0,"
                                            #WS 2020-04-03 +rsglobal.ZmienNaNULL(KodGTU)
                                            Insert+="FALSE,'"+KontoMa+"',"+KwotaMa+",'"+KontoVatMa+"',"+KwotaVatMa+",'"+Stawka+"','"+NrFakt+"','"+Waluta+"',"+Kurs+","+KwotaWalMa+",FALSE,"+rsglobal.ZmienNaNULL(KodGTU)+",'"+DataPrzych+"');"

                                            NrPozDow+=1
                                            if Stawka==StawkaP:
                                                if Stawka in self.SStawkiVAT:
                                                    self.SStawkiVAT[Stawka]+=psttyp[3]
                                                else:
                                                    self.SStawkiVAT[Stawka]=psttyp[3]

                            #Nie byo konta ma na wzorze
                            if not Jest:
                                dane.append("BR KMA$"+tresc+" "+NrFakt+" dla stawki "+Stawka)
                                return dane

                    if not PierwszyPrzebieg:
                        Robic= False
                    PierwszyPrzebieg=False
                #rsglobal.ZapiszDoLogow('Stawki 2 '+str(self.SStawkiVAT),1)
                #pozycje opisowe
                for pozop in DanePozOpis:
                    Stawka=rsglobal.Dekoduj(pozop[0])
                    Jest=False
                    #ptla po pozycjach wzoru dekretacji
                    for poz in PozWz:
                        if not Jest:
                            if poz[3] not in ['','None']:
                                if Stawka==poz[5]:
                                    Jest=True
                                    KontoMa='XXX'
                                    KontoVatMa=rsglobal.Dekoduj(poz[4])
                                    if KontoVatMa =='None':
                                        KontoVatMa=''
                                    KwotaMa=rsglobal.Dekoduj(str(pozop[1]))
                                    KwotaVatMa=rsglobal.Dekoduj(str(pozop[2]))
                                    KwotaWalMa=rsglobal.Dekoduj(str(pozop[3]))
                                    KodGTU = pozop[4]  #WS 2020-04-03

                                    Insert+="INSERT INTO rs_dowody_ksiegowe_pozycje (firma_id,okres_id,dowod_id,nazwa_id,data,numer,nr_poz,nr_ppoz,nr_pozw,"
                                    Insert+="symbol_dok,data_wyst,data_wplywu,termin_plat,data_zapl,kontrahent_id,"
                                    Insert+="konto_wn,kwota_wn,konto_vat_wn,kwota_vat_wn,stawka_vat_wn,tresc_wn,sposob_rozl,waluta_wn,kurs_wn,kwota_wal_wn,"
                                    #WS 2020-04-03 - kod_gtu
                                    Insert+="nkup,konto_ma,kwota_ma,konto_vat_ma,kwota_vat_ma,stawka_vat_ma,tresc_ma,waluta_ma,kurs_ma,kwota_wal_ma,pnop,kod_gtu,data_ksieg_kosztow)"
                                    Insert+=" VALUES "
                                    Insert+="('"+self.firma_id+"','"+OkresDekr+"','"+Dowod+"','"+Nazwa+"','"+Data+"',"+str(Numer)+","+str(NrPozDow)+",0,"+rsglobal.Dekoduj(str(poz[0]))+","
                                    Insert+="'"+tresc+" "+NrFakt+"','"+Data+"',NULL,"+TerminPlat+",NULL,'"+Platnik+"',"
                                    Insert+="'',0,'',0,NULL,'','','"+Waluta+"',"+Kurs+",0,"
                                    #WS 2020-04-03 +rsglobal.ZmienNaNULL(KodGTU)
                                    Insert+="FALSE,'"+KontoMa+"',"+KwotaMa+",'"+KontoVatMa+"',"+KwotaVatMa+",'"+Stawka+"','"+NrFakt+"','"+Waluta+"',"+Kurs+","+KwotaWalMa+",FALSE,"+rsglobal.ZmienNaNULL(KodGTU)+",'"+DataPrzych+"');"

                                    NrPozDow+=1
                                    if Stawka in self.SStawkiVAT:
                                        self.SStawkiVAT[Stawka]+=pozop[2]
                                    else:
                                        self.SStawkiVAT[Stawka]=pozop[2]

                    #Nie byo konta ma na wzorze
                    if not Jest:
                        dane.append("BR KMA$"+tresc+" "+NrFakt+" dla stawki "+Stawka)
                        return dane

                #policzenie i wstawienie zaokraglen vatu
                #rsglobal.ZapiszDoLogow('Stawki 3 '+str(self.SStawkiVAT),1)
                #rsglobal.ZapiszDoLogow('Stawki 3b '+str(StVat),1)

                # teraz StVat maj inn struktur dla faktur a inn dla korekt
                for Stawka in self.SStawkiVAT:
                    for pst in StVat:
                        if pst[0]==Stawka:
                            if tabela=='rs_faktury':
                                KwZaokr+=rsglobal.RSRound(pst[2]-self.SStawkiVAT[Stawka],2)
                            else:
                                KwZaokr+=rsglobal.RSRound(pst[4]-self.SStawkiVAT[Stawka],2)

                #jeli kwota zaokgle != 0 to dodajemy pozycj
                if KwZaokr != 0:
                    Insert+="INSERT INTO rs_dowody_ksiegowe_pozycje (firma_id,okres_id,dowod_id,nazwa_id,data,numer,nr_poz,nr_ppoz,nr_pozw,"
                    Insert+="symbol_dok,data_wyst,data_wplywu,termin_plat,data_zapl,kontrahent_id,"
                    Insert+="konto_wn,kwota_wn,konto_vat_wn,kwota_vat_wn,stawka_vat_wn,tresc_wn,sposob_rozl,waluta_wn,kurs_wn,kwota_wal_wn,"
                    Insert+="nkup,konto_ma,kwota_ma,konto_vat_ma,kwota_vat_ma,stawka_vat_ma,tresc_ma,waluta_ma,kurs_ma,kwota_wal_ma,pnop,data_ksieg_kosztow)"
                    Insert+=" VALUES "
                    Insert+="('"+self.firma_id+"','"+OkresDekr+"','"+Dowod+"','"+Nazwa+"','"+Data+"',"+str(Numer)+","+str(NrPozDow)+",0,1,"
                    Insert+="'"+tresc+" "+NrFakt+"','"+Data+"',NULL,"+TerminPlat+",NULL,'"+Platnik+"',"
                    Insert+="'',0,'',0,NULL,'','','"+Waluta+"',"+Kurs+",0,"
                    Insert+="FALSE,'"+KontoZaokr+"',"+str(KwZaokr)+",'',0,NULL,'"+NrFakt+"','"+Waluta+"',"+Kurs+","+str(KwZaokr)+",FALSE,'"+DataPrzych+"');"

            #zamknicie dokumentu faktury/korekty
            Zam="UPDATE "+tabela+" SET stan='K' WHERE firma_id='"+self.firma_id+"' AND wyznacznik_id='"+Wyzn+"' AND data='"+Data+"' AND numer="+Nr[0:Nr.find('.')]+";"

            stat,dump,dump,dump = self.PolSQL.Execute(InsN+Insert+Zam,1)

            if stat!= 'OK':
                self.PolSQL.Rollback()
                dane.append(stat)

            if stat=='OK':
                stat=fs_dekretacja_automatyczna_projekty.DodajProjektyDzialyDoDowodu(self.PolSQL,self.firma_id,OkresDekr,Dowod,Nazwa,Data,Numer,TypD,NrFakt,WarKontrahent)
                if stat!='OK':
                    self.PolSQL.Rollback()
                    dane.append(stat)

            # OSTATECZNY ZAPIS DEKRETACJI
            if stat=='OK':
                self.PolSQL.Commit()

            return dane
        except Exception,e:
            rsglobal.ZapiszDoLogow('BD '+str(e),1)
            return ['Bd '+str(e)]


#-##############################################################################
#-- DEKRETACJA RӯNIC KURSOWYCH
#-##############################################################################

def DekretacjaAutomatycznaRoznicKursowych(self,Dowod,Nazwa,DataOd,DataDo):
    # sprawdzenie czy DataOd i DataDo s z tego samego miesica
    if DataOd[:7] !=DataDo[:7]:
        return "Do wyznaczenia rnic kursowych naley poda zakres dat z jednego miesica"

    # sprawdzenie czy wszystkie rnice kursowe utworzone wczeniej s rozliczone
    SQL="SELECT DISTINCT konto FROM rs_rozrachunki r "
    SQL+=" INNER JOIN rs_wzory_dekretacji wd ON (wd.firma_id= r.firma_id AND wd.okres_id=r.okres_id "
    SQL+=" AND wd.dowod_id = r.dowod_id AND wd.nazwa_id = r.nazwa_id)"
    #AB 29.03.18r dodanie  AND r.okres_id = '"+self.okres_id+"'
    SQL+=" WHERE r.firma_id = '"+self.firma_id+"' AND r.okres_id = '"+self.okres_id+"' AND r.data_rozlicz IS NULL"
    SQL+=" AND r.waluta != 'PLN' AND wd.typ_wzoru = 'W';"
    stat,DTest,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return 'Wystpiy bdy sprawdzenia rozliczenia rnic kursowych :'+stat

    if len(DTest)>0:
        stat='Na kontach '
        for konto in DTest:
            stat+=konto[0].strip()+','
        stat=stat.strip(',')
        stat+=' s nierozliczone rnice kursowe w walucie. Automatyczna dekretacja rnic kursowych niemoliwa.'
        return stat

    # pobranie sumarycznoci z nagwka wzoru dekretacji
    SQL="SELECT sumarycznie FROM rs_wzory_dekretacji WHERE firma_id='"+self.firma_id+"'"
    SQL+=" AND okres_id='"+self.okres_id+"' AND dowod_id='"+Dowod+"' AND nazwa_id='"+Nazwa+"'"
    stat,DWzorN,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return 'Wystpiy bdy pobierania danych nagwka wzorw dekretacji :'+stat
    if len(DWzorN)==0:
        return "Podano nieprawidowy wzr dekretacji"

    Sumarycznie = DWzorN[0][0]

    # pobranie kont z pozycji wzoru dekretacji
    SQL="SELECT konto_rozr, konto_przec_dod, konto_przec_uj FROM rs_wzory_dekretacji_roznice_kursowe "
    SQL+=" WHERE firma_id='"+self.firma_id+"' AND okres_id='"+self.okres_id+"'"
    SQL+=" AND dowod_id='"+Dowod+"' AND nazwa_id='"+Nazwa+"' ORDER BY nr_poz;"
    stat,DWzorP,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return 'Wystpiy bdy pobierania danych pozycji wzorw dekretacji :'+stat
    if len(DWzorP)==0:
        return "Na podanym wzorze dekretacji brak pozycji"

    # ptla po pozycjach wzoru
    LPoz=[] # lista : [seria, klucz dowodu, konto, waluta, roznica_wn, roznica_ma]
    LSerii={}

    for pozwz in DWzorP:
        KontoRozr=pozwz[0]
        KontoPrzecDod=pozwz[1]
        KontoPrzecUj=pozwz[2]

        # pobranie nieprzetworzonych rnic kursowych z rozrachunkw rozliczonych w  podanym zakresie dat
        SQL="SELECT seria,dowod_id, nazwa_id, data, numer, nr_poz, nr_ppoz, konto, kwota_wn,kwota_ma, roznica_kursowa, "
        SQL+="waluta FROM rs_rozrachunki "
        SQL+=" WHERE firma_id='"+self.firma_id+"' AND okres_id='"+self.okres_id+"' AND data_rozlicz >='"+DataOd+"' "
        SQL+=" AND data_rozlicz <= '"+DataDo+"' AND roznica_kursowa != 0 AND (rk_dowod_id IS NULL OR rk_dowod_id='')  AND konto LIKE '"+KontoRozr+"%'"
        SQL+=" ORDER BY dowod_id,nazwa_id,data,numer;"
        stat,DPozRK,dump,dump = self.PolSQL.Execute(SQL,3)
        if stat != 'OK':
            return 'Wystpiy bdy pobierania danych pozycji rozrachunkw z rnicami kursowymi :'+stat

        # pobranie nierozliczonych rozrachunkw z zerowym saldem walutowym
        SQL="SELECT seria,dowod_id, nazwa_id, data, numer, nr_poz, nr_ppoz, konto,saldo_wn,saldo_ma,waluta FROM rs_rozrachunki "
        SQL+=" WHERE firma_id='"+self.firma_id+"' AND okres_id='"+self.okres_id+"' AND data_rozlicz IS NULL "
        SQL+=" AND ((saldo_wn!= 0 AND saldo_wal_wn = 0) OR (saldo_ma!= 0 AND saldo_wal_ma = 0)) AND NOT (rk_dowod_id IS NULL OR rk_dowod_id='') AND konto LIKE '"+KontoRozr+"%' "
        SQL+=" ORDER BY dowod_id,nazwa_id,data,numer;"
        stat,DPozNroz,dump,dump = self.PolSQL.Execute(SQL,3)
        if stat != 'OK':
            return 'Wystpiy bdy pobierania danych pozycji rozrachunkw z zerowym saldem walutowym :'+stat

        # utworzenie listy pozycji do wstawienia

        for poz in DPozRK:
            if not poz[0] in LSerii: # jeli konta si przecinaj to liczy si pierwsze
                LSerii[poz[0]]=0
                KluczPoz=poz[1]+'/'+poz[2]+'/'+poz[3]+'/'+rsglobal.UsunKropkaZero(poz[4])+'/'+rsglobal.UsunKropkaZero(poz[5])+'/'+rsglobal.UsunKropkaZero(poz[6])
                if poz[8] !=0:
                    RoznWn=poz[10]
                    RoznMa=0
                else:
                    RoznWn=0
                    RoznMa=poz[10]

                LPoz.append([poz[0],KluczPoz,poz[7],poz[11],RoznWn,RoznMa,KontoPrzecDod,KontoPrzecUj])

        # przetworzenie zaokrgle kocowych. dla kadego rozrachunku z zerowym saldem walutowym pobieramy
        # sum niezaznaczonych rnic kursowych i odejmujemy od jego wartoci. To co zostao to zaokrglenie
        # wartoci kursowej i na to trzeba wystawi pozycj
        for poz in DPozNroz:
            if not poz[0] in LSerii:
                LSerii[poz[0]]=0

                KluczPoz=poz[1]+'/'+poz[2]+'/'+poz[3]+'/'+rsglobal.UsunKropkaZero(poz[4])+'/'+rsglobal.UsunKropkaZero(poz[5])+'/'+rsglobal.UsunKropkaZero(poz[6])

                if poz[8] != 0:
                    Strona = 'wn'
                    Kwota = poz[8]
                else:
                    Strona = 'ma'
                    Kwota = poz[9]

                #liczymy sum nierozliczonych rnic kursowych dla dokumentu
                SQL="SELECT COALESCE(SUM(roznica_kursowa),0) FROM rs_rozrachunki WHERE firma_id='"+self.firma_id+"'"
                SQL+=" AND okres_id='"+self.okres_id+"' AND dowod_id='"+poz[1]+"' AND nazwa_id='"+poz[2]+"'"
                SQL+=" AND data='"+poz[3]+"' AND numer="+str(poz[4])+" AND nr_poz="+str(poz[5])
                SQL+=" AND nr_ppoz="+str(poz[6])+" AND konto='"+poz[7]+"' AND NOT czy_utworzona_rk;"
                stat,DRozn,dump,dump = self.PolSQL.Execute(SQL,3)
                if stat != 'OK':
                    return 'Wystpiy bdy pobierania danych do wyliczenia zaokrgle :'+stat

                if DRozn[0][0]!= Kwota:
                    if Strona == 'wn':
                        LPoz.append([poz[0],KluczPoz,poz[7],poz[10],rsglobal.RSRound(Kwota-DRozn[0][0],2),0,KontoPrzecDod,KontoPrzecUj])
                    else:
                        LPoz.append([poz[0],KluczPoz,poz[7],poz[10],0,rsglobal.RSRound(Kwota-DRozn[0][0],2),KontoPrzecDod,KontoPrzecUj])

    # jeli sumarycznie
    if Sumarycznie:
        SPozRK={}
        # posumowanie stron winien i ma z dokadnoci do konta i waluty
        for poz in LPoz:
            KlPoz=(poz[2],poz[3]) # (konto, waluta)
            if not KlPoz in SPozRK:
                SPozRK[KlPoz]=[0,0,poz[6],poz[7]]
            SPozRK[KlPoz][0]+=poz[4]
            SPozRK[KlPoz][1]+=poz[5]

        LPoz=[]
        for kl in SPozRK:
            if SPozRK[kl][0] != 0:
                LPoz.append([0,'',kl[0],kl[1],SPozRK[kl][0],0,SPozRK[kl][2],SPozRK[kl][3]])
            if SPozRK[kl][1] != 0:
                LPoz.append([0,'',kl[0],kl[1],0,SPozRK[kl][1],SPozRK[kl][2],SPozRK[kl][3]])

    # tworzymy nagwek dowodu ksigowego
    NrDowodu = 0

    # te kwoty w przypadku sumarycznym trafiaj na nagwek dokumentu
    SumarycznieKwota = 0

    # dla kadej pozycji rnicy kursowej z listy tworzymy pozycj dowodu ksigowego
    for poz in LPoz:
        if poz[4] != 0:
            Kwota = poz[4]
        else:
            Kwota = poz[5]

        SumarycznieKwota+=abs(Kwota)

        if NrDowodu == 0 or not Sumarycznie:
            # pobranie nowego numeru dowodu ksigowego (dla sumarycznego liczenia robimy raz)
            stat,Test,dump,dump = self.PolSQL.Execute("SELECT rsf_sprawdz_czy_mozna_dodac_dokument('"+self.firma_id+"','[\\\'"+self.okres_id+"\\\',\\\'"+Dowod+"\\\',\\\'"+Nazwa+"\\\',\\\'"+DataDo+"\\\']','D');",3)
            if stat != 'OK':
                return 'Bd sprawdzania moliwoci dodawania dowodu ksigowego :'+stat

            if Test[0][0] != 'OK':
                return 'Bd sprawdzania moliwoci dodawania dowodu ksigowego :'+stat

            stat,DNr,dump,dump = self.PolSQL.Execute("SELECT rsf_podaj_nowy_numer_dokumentu('"+self.firma_id+"','[\\\'"+self.okres_id+"\\\',\\\'"+Dowod+"\\\',\\\'"+Nazwa+"\\\',\\\'"+DataDo+"\\\']','D');",3)
            if stat != 'OK':
                return 'Bd pobierania numeru nowego dowodu ksigowego :'+stat

            if DNr[0][0] in (None,'None',''):
                return 'Bdnie pobrany numer nowego dowodu ksigowego :'+str(DNr[0][0])

            NrDowodu=rsglobal.Dekoduj(DNr[0][0])

            InsN= "INSERT INTO rs_dowody_ksiegowe "
            InsN+="(firma_id,okres_id,dowod_id,nazwa_id,data,numer,"
            InsN+=" stan,kwota_wn,kwota_ma"
            if not Sumarycznie:
                InsN+=",tresc"
            InsN+=") VALUES "
            InsN+="('"+self.firma_id+"','"+self.okres_id+"','"+Dowod+"','"+Nazwa+"','"+DataDo+"',"+str(NrDowodu)+","
            InsN+=" 'W',"+str(abs(Kwota))+","+str(abs(Kwota))
            if not Sumarycznie:
                InsN+=",'rnica kursowa : '||'"+poz[1]+"'"
            InsN+=");"

            stat,dump,dump,dump = self.PolSQL.Execute(InsN,1)
            if stat != 'OK':
                return 'Wystpiy bdy dodawania dowodu ksigowego :'+stat

            NrPoz = 1

        # dodanie pozycji dowodu ksigowego
        if poz[4] != 0:
            # jeli mamy kwot wn to w zalenoci od jej dodatnioci/ujemnoci bierzemy odpowiednie konto przeciwstawne.
            # jeli kwota dodatnia to konto ma jest rozrachunkowe a konto wn bdzie przeciwstawne
            # jeli kwota ujemna to konto ma jest przeciwstawne a konto wn bdzie rozrachunkowe
            if Kwota > 0:
                # cz rozrachunkowa
                KontoMa = poz[2]
                WalMa = 'PLN'
                KwotaWalMa = 0
                SeriaMa=str(poz[0])

                # cz przeciwstawna
                KontoWn = poz[7]
                WalWn = 'PLN'
                KwotaWalWn = abs(Kwota)
                SeriaWn=''
            else:
                # cz rozrachunkowa
                KontoWn = poz[2]
                WalWn = 'PLN'
                KwotaWalWn = 0
                SeriaWn=str(poz[0])

                # cz przeciwstawna
                KontoMa = poz[6]
                WalMa = 'PLN'
                KwotaWalMa = abs(Kwota)
                SeriaMa=''
        else:
            # jeli mamy kwot ma to w zalenoci od jej dodatnioci/ujemnoci bierzemy odpowiednie konto przeciwstawne.
            # jeli kwota dodatnia to konto ma jest przeciwstawne a konto wn bdzie rozrachunkowe
            # jeli kwota ujemna to konto ma jest rozrachunkowe a konto wn bdzie przeciwstawne
            if Kwota > 0:
                # cz rozrachunkowa
                KontoWn = poz[2]
                WalWn = poz[3]
                KwotaWalWn = 0
                SeriaWn=str(poz[0])

                # cz przeciwstawna
                KontoMa = poz[7]
                WalMa = 'PLN'
                KwotaWalMa = abs(Kwota)
                SeriaMa=''
            else:
                # cz rozrachunkowa
                KontoMa = poz[2]
                WalMa = poz[3]
                KwotaWalMa = 0
                SeriaMa=str(poz[0])

                # cz przeciwstawna
                KontoWn = poz[6]
                WalWn = 'PLN'
                KwotaWalWn = abs(Kwota)
                SeriaWn=''

        InsP="INSERT INTO rs_dowody_ksiegowe_pozycje "
        InsP+=" (firma_id,okres_id,dowod_id,nazwa_id,data,numer,nr_poz,nr_ppoz,nr_pozw,"
        if not Sumarycznie:
            InsP+="tresc_wn,tresc_ma,seria_wn,seria_ma,"
        InsP+="konto_wn,kwota_wn,waluta_wn,kurs_wn,kwota_wal_wn,"
        InsP+="konto_ma,kwota_ma,waluta_ma,kurs_ma,kwota_wal_ma)"
        InsP+=" VALUES "
        InsP+="('"+self.firma_id+"','"+self.okres_id+"','"+Dowod+"','"+Nazwa+"','"+DataDo+"',"+str(NrDowodu)+","+str(NrPoz)+",0,1,"
        if not Sumarycznie:
            InsP+="'rnica kursowa : '||'"+poz[1]+"','rnica kursowa : '||'"+poz[1]+"','"+SeriaWn+"','"+SeriaMa+"',"
        InsP+="'"+KontoWn+"',"+str(abs(Kwota))+",'"+WalWn+"',1,"+str(KwotaWalWn)+","
        InsP+="'"+KontoMa+"',"+str(abs(Kwota))+",'"+WalMa+"',1,"+str(KwotaWalMa)+");"

        stat,dump,dump,dump = self.PolSQL.Execute(InsP,1)
        if stat != 'OK':
            return 'Wystpiy bdy dodawania pozycji dowodu ksigowego :'+stat

        LSerii[poz[0]]=NrDowodu

        NrPoz+=1

    # jeli sumarycznie to robimy update na kwot wn i ma nagwka
    if Sumarycznie and NrDowodu > 0:
        SQL="UPDATE rs_dowody_ksiegowe SET kwota_wn="+str(abs(SumarycznieKwota))+",kwota_ma="+str(abs(SumarycznieKwota))
        SQL+=" WHERE firma_id='"+self.firma_id+"' AND okres_id='"+self.okres_id+"' AND dowod_id='"+Dowod+"'"
        SQL+=" AND nazwa_id='"+Nazwa+"' AND data='"+DataDo+"' AND numer="+str(NrDowodu)
        stat,dump,dump,dump = self.PolSQL.Execute(SQL,1)
        if stat != 'OK':
            return 'Wystpiy bdy zmiany kwot sumarycznego dowodu ksigowego :'+stat

        # robimy update na wszystkie rozrachunki, ktre zostay zanalizowane w algorytmie wg serii - dla sumarycznych
        if len(LSerii.keys())>0:
            SQL="UPDATE rs_rozrachunki SET rk_dowod_id='"+Dowod+"',rk_nazwa_id='"+Nazwa+"',rk_data='"+DataDo+"',"
            SQL+=" rk_numer="+str(NrDowodu)+" WHERE seria IN ("+str(LSerii.keys())[1:-1]+");"
            stat,dump,dump,dump = self.PolSQL.Execute(SQL,1)
            if stat != 'OK':
                return 'Wystpiy bdy zapisu informacji o utworzeniu rnic kursowych na rozrachunkach :'+stat

    else:
        # robimy update na wszystkie rozrachunki, ktre zostay zanalizowane w algorytmie wg serii dla pojedyczych
        for kl in LSerii.keys():
            SQL="UPDATE rs_rozrachunki SET rk_dowod_id='"+Dowod+"',rk_nazwa_id='"+Nazwa+"',rk_data='"+DataDo+"',"
            SQL+=" rk_numer="+str(LSerii[kl])+" WHERE seria = "+str(kl)+";"
            stat,dump,dump,dump = self.PolSQL.Execute(SQL,1)
            if stat != 'OK':
                return 'Wystpiy bdy zapisu informacji o utworzeniu rnic kursowych na rozrachunkach :'+stat

    return 'OK'


#-##############################################################################
#-- DEKRETACJA DOKUMENTW MAGAZYNOWYCH
#-##############################################################################

#-------------------------------------------------------------------------------
#-- pobranie sownika z kontami w podziale na typy artykuw
#-------------------------------------------------------------------------------

def PodajSlownikTypowArtykulow(PolSQL,Firma,Okres):
    SQL="SELECT konto_mag_wn_01, konto_mag_wn_02, konto_mag_wn_03, konto_mag_wn_04, konto_mag_wn_05, konto_mag_wn_11, konto_mag_wn_12, konto_mag_wn_21,"
    SQL+="konto_mag_wn_31, konto_mag_wn_99, konto_mag_ma_01, konto_mag_ma_02, konto_mag_ma_03, konto_mag_ma_04, konto_mag_ma_05,"
    SQL+=" konto_mag_ma_11, konto_mag_ma_12, konto_mag_ma_21,konto_mag_ma_31, konto_mag_ma_99 FROM rs_wzory_dekretacji_typy "
    SQL+="WHERE firma_id='"+Firma+"' AND okres_id='"+Okres+"';"
    stat,DaneKontaTypow,dump,dump = PolSQL.Execute(SQL,3)
    if stat!= 'OK':
        return stat,{}

    if len(DaneKontaTypow)==0:
        # zalepka, jeli nie ma wiersza w bazie ( to jest aceptowalne bo mog by konta w pozycjach wzorw dekretacji)
        DaneKontaTypow=[['','','','','','','','','','','','','','','','','','','','']]

    #stworzenie slownika zaleznego od typow
    SlKTypow={}
    SlKTypow['01']=[rsglobal.Dekoduj(DaneKontaTypow[0][0]),rsglobal.Dekoduj(DaneKontaTypow[0][10])]
    SlKTypow['02']=[rsglobal.Dekoduj(DaneKontaTypow[0][1]),rsglobal.Dekoduj(DaneKontaTypow[0][11])]
    SlKTypow['03']=[rsglobal.Dekoduj(DaneKontaTypow[0][2]),rsglobal.Dekoduj(DaneKontaTypow[0][12])]
    SlKTypow['04']=[rsglobal.Dekoduj(DaneKontaTypow[0][3]),rsglobal.Dekoduj(DaneKontaTypow[0][13])]
    SlKTypow['05']=[rsglobal.Dekoduj(DaneKontaTypow[0][4]),rsglobal.Dekoduj(DaneKontaTypow[0][14])]
    SlKTypow['11']=[rsglobal.Dekoduj(DaneKontaTypow[0][5]),rsglobal.Dekoduj(DaneKontaTypow[0][15])]
    SlKTypow['12']=[rsglobal.Dekoduj(DaneKontaTypow[0][6]),rsglobal.Dekoduj(DaneKontaTypow[0][16])]
    SlKTypow['21']=[rsglobal.Dekoduj(DaneKontaTypow[0][7]),rsglobal.Dekoduj(DaneKontaTypow[0][17])]
    SlKTypow['31']=[rsglobal.Dekoduj(DaneKontaTypow[0][8]),rsglobal.Dekoduj(DaneKontaTypow[0][18])]
    SlKTypow['99']=[rsglobal.Dekoduj(DaneKontaTypow[0][9]),rsglobal.Dekoduj(DaneKontaTypow[0][19])]

    return 'OK',SlKTypow

#-------------------------------------------------------------------------------
#-- pobranie listy mozliwych pozycji wzoru do zadekretowania
#-- jesli ZW to korzysta ze wzorw WZ (ale dostaje nagwek ZW)
#-- sprawdzane jest od razu rozbicie na typy
#-------------------------------------------------------------------------------

def PodajPozycjeWzorowDekretacjiDM(PolSQL,Firma,Okres,Dowod,Nazwa):
    # sprawdzenie czy ZW
    SQL="SELECT czy_zw FROM rs_wzory_dekretacji WHERE firma_id='"+Firma+"' AND okres_id='"+Okres+"'"
    SQL+=" AND dowod_id='"+Dowod+"' AND nazwa_id='"+Nazwa+"';"
    stat,DWzoru,dump,dump = PolSQL.Execute(SQL,3)
    if stat!='OK':
        return 'Bd pobierania informacji czy ZW ze wzoru dekretacji '+Dowod+'/'+Nazwa+': '+stat,[]
    if len(DWzoru)==0:
        return 'Brak wzoru dekretacji '+Dowod+'/'+Nazwa,[]

    CzyZW=DWzoru[0][0]

    #pobranie listy mozliwych pozycji wzoru do zadekretowania
    SQL="SELECT nr_poz,konto_wn,konto_ma,magazyn_id,wyznacznik_id,kontrahent_id,z_wylaczeniem,"   #  0-6
    SQL+="pozostali,zlecenie_wn,zlecenie_ma,konto_koszt_wn,konto_koszt_ma,magazyn_wn,magazyn_ma," #  7-13
    SQL+="projekt_wn,projekt_ma,magazyn_dokad,magazyn_dokad_wn,magazyn_dokad_ma, "                # 14-18
    SQL+="konto_mag_wn_01, konto_mag_wn_02, konto_mag_wn_03, konto_mag_wn_04, konto_mag_wn_05,"   # 19-23
    SQL+="konto_mag_wn_11, konto_mag_wn_12, konto_mag_wn_21, konto_mag_wn_31, konto_mag_wn_99,"   # 24-28
    SQL+="konto_mag_ma_01, konto_mag_ma_02, konto_mag_ma_03, konto_mag_ma_04, konto_mag_ma_05,"   # 29-33
    SQL+="konto_mag_ma_11, konto_mag_ma_12, konto_mag_ma_21,konto_mag_ma_31, konto_mag_ma_99"     # 34-38
    SQL+=" FROM rs_wzory_dekretacji_magazynow "
    SQL+=" WHERE firma_id='"+Firma+"' AND okres_id='"+Okres+"' AND dowod_id='"+Dowod+"'"
    if CzyZW:
        SQL+=" AND nazwa_id='WZ';"
    else:
        SQL+=" AND nazwa_id='"+Nazwa+"';"

    stat,PozWz,dump,dump = PolSQL.Execute(SQL,3)
    if stat!='OK':
        return 'Bd pobierania listy pozycji ze wzoru dekretacji '+Dowod+'/'+Nazwa+': '+stat,[]

    if len(PozWz)==0:
        return "BR POZ",[]

    #sprawdzenie czy bd * i podzia na typy artykuw czy nie (po stronie wn i ma osobno sprawdzamy).
    CzyRozbicieWn= (PozWz[0][1][0]=='*')
    CzyRozbicieMa= (PozWz[0][2][0]=='*')
    CzyRozbicie= (PozWz[0][1][0]=='*') or (PozWz[0][2][0]=='*')

    #sprawdzenie str WN
    for poz in PozWz:
        if CzyRozbicieWn:
            if poz[1][0] != '*':
                return "BR GWMWN",[]

    #sprawdzenie str MA
    for poz in PozWz:
        if CzyRozbicieMa:
            if poz[2][0] != '*':
                return "BR GWMMA",[]

    return 'OK',[PozWz,CzyRozbicieWn,CzyRozbicieMa,CzyRozbicie]

#-------------------------------------------------------------------------------
#-- stworzenie dowodu ksigowego dla dekretacji
#-- dokumentu magazynowego
#-------------------------------------------------------------------------------

def ZadekretujDokumentMagazynowy(PolSQL,Firma,Okres,Dowod,Nazwa,DataDK,Mag,DokId,Data,Nr,PozWz,SlKTypow,CzyRozbicieWn,CzyRozbicieMa,CzyRozbicie,Rej,DataRej,CzyZmiany=False):
    # pobranie danych konkretnego dokumentu
    Numer=rsglobal.UsunKropkaZero(Nr)
    KluczDM=Mag+'/'+DokId+'/'+Data+'/'+Numer

    FiltrDokum="WHERE firma_id='"+Firma+"' AND magazyn_id='"+Mag+"' AND dokument_id='"+DokId+"' AND data='"+Data+"' AND numer="+Numer

    SQL="SELECT dm.magazyn_id,dm.dokument_id,dm.data,dm.numer,dm.magazyn_skad,dm.magazyn_dokad,dm.kontrahent_id," # 0-6
    SQL+="dm.nr_zlecenia,dm.konto_kosztow,dm.projekt_id,dm.waluta,dm.kwota,dm.kwota_wal,dm.f_wyznacznik_id,dm.k_wyznacznik_id," #7-14
    SQL+="dm.nr_faktury,dm.k_data,dm.k_numer,COALESCE(dm.numer_dokad::TEXT,''), " #15-18
    SQL+="COALESCE(dm.kor_dm_dokument_id,'') " # 19
    SQL+="FROM rs_dokumenty_magazynowe dm "
    SQL+=FiltrDokum+" FOR UPDATE NOWAIT;"
    stat,DokMag,dump,dump = PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return 'Bdy pobrania danych dokumentu '+KluczDM+' : '+stat,[]

    dok=DokMag[0]

    SQL="SELECT korekta,typ_dokumentu "
    SQL+="FROM rs_typy_dokumentow_m WHERE firma_id='"+Firma+"' AND dokument_id='"+dok[1]+"'"
    stat,DTypDok,dump,dump = PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return 'Bdy pobrania danych typu dokumentu : '+stat,[]

    # sprawdzenie czy ZW
    SQL="SELECT czy_zw FROM rs_wzory_dekretacji WHERE firma_id='"+Firma+"' AND okres_id='"+Okres+"'"
    SQL+=" AND dowod_id='"+Dowod+"' AND nazwa_id='"+Nazwa+"';"
    stat,DWzoru,dump,dump = PolSQL.Execute(SQL,3)
    if stat!='OK':
        return 'Bd pobierania informacji czy ZW ze wzoru dekretacji '+Dowod+'/'+Nazwa+': '+stat,[]
    if len(DWzoru)==0:
        return 'Brak wzoru dekretacji '+Dowod+'/'+Nazwa,[]

    CzyZW=DWzoru[0][0]

    MagS=rsglobal.Dekoduj(dok[4])
    MagD=rsglobal.Dekoduj(dok[5])
    NumerD=rsglobal.Dekoduj(dok[18])
    Kontr=rsglobal.Dekoduj(dok[6])
    DokKoryg=rsglobal.Dekoduj(dok[19])

    Korekta=rsglobal.Dekoduj(DTypDok[0][0])
    TypDok=rsglobal.Dekoduj(DTypDok[0][1])

    # sprawdzenie czy korekta o tym samym typie i czy uzupenia tre pozycji
    ObracacKonta = False
    TrescDodat = ''
    if Korekta and not DokKoryg in ('',None,'None'):
        TrescDodat = ' korekta dla '+DokKoryg
        # pobranie typu zwizanego dokumentu
        SQL="SELECT typ_dokumentu FROM rs_typy_dokumentow_m WHERE firma_id='"+Firma+"'"
        SQL+=" AND dokument_id='"+DokKoryg+"'"
        stat,DTest,dump,dump = PolSQL.Execute(SQL,3)
        if stat != 'OK':
            return 'Bd pobierania typu dokumentw : '+stat,[]

        if len(DTest)>0:
            ObracacKonta= not (DTest[0][0] == TypDok)


    if Kontr=='None':
        Kontr='NULL'
    else:
        Kontr="'"+Kontr+"'"

    Zlec=rsglobal.Dekoduj(dok[7])

    if Zlec=='None':
        Zlec=''

    KontoK=rsglobal.Dekoduj(dok[8])
    if KontoK=='None':
        KontoK=''

    Proj=rsglobal.Dekoduj(dok[9])

    if Proj=='None':
        Proj='NULL'
        ProjK=''
    else:
        ProjK=Proj
        Proj="'"+Proj+"'"

    Wal='PLN'
    Kwota=rsglobal.RSRound(dok[11],2)
    KwotaWal=rsglobal.RSRound(dok[11],2)
    Kurs='1'
    rsglobal.ZapiszDoLogow('Kwota 1 '+str(Kwota),1)
    # jeli to zmiana to kwot naley pobra jako sum zmian dokumentu
    if CzyZmiany:
        SQL="SELECT ROUND(COALESCE(SUM(ROUND(z.wartosc_n-z.wartosc_s,2)),0),2) FROM rs_dokumenty_magazynowe_pozycje_zmiany z "
        SQL+=FiltrDokum+" AND NOT pobrane_do_ksieg;"
        stat,DaneZm,dump,dump = PolSQL.Execute(SQL,3)
        if stat != 'OK':
            return 'Bdy pobierania sumy wartoci zmian dokumentu '+KluczDM+': '+stat,[]

        Kwota=DaneZm[0][0]
        rsglobal.ZapiszDoLogow('Kwota 2 '+str(Kwota),1)
        KwotaWal=DaneZm[0][0]

    # jeli kwota==0 to na kocu dokument po prostu zamieniamy na K i ju
    if Kwota==0:
        if not CzyZmiany:
            UPD="UPDATE rs_dokumenty_magazynowe SET stan='K' "+FiltrDokum+";"
            stat,dump,dump,dump = PolSQL.Execute(UPD,1)
            if stat!= 'OK':
                PolSQL.Rollback()
                return 'Bd zmiany stanu dokumentu magazynowego '+KluczDM+' na K :'+stat,[]
            else:
                PolSQL.Commit()
        rsglobal.ZapiszDoLogow('Wychodze z tytuu kwoty dokument '+KluczDM,1)
        return 'OK',[]

    WyznF=rsglobal.Dekoduj(dok[13])
    WyznK=rsglobal.Dekoduj(dok[14])
    NrFakt=rsglobal.Dekoduj(dok[15])[0:20]
    DataK=rsglobal.Dekoduj(dok[16])
    NumerK=rsglobal.Dekoduj(str(dok[17]))
    Znak=1
    #pobranie odpowiedniego wyznacznika dla zw
    Robic=True
    if CzyZW:
        if WyznK in ('','None',None):
            Robic=False
        else:
            stat,DaneK,dump,dump = PolSQL.Execute("SELECT wyznacznik_id_o FROM rs_korekty WHERE firma_id='"+Firma+"' AND wyznacznik_id='"+WyznK+"' AND data='"+DataK+"' AND numer="+NumerK+";",3)
            if stat != 'OK':
                return 'Bdy pobierania oryginalnego wyznacznika korekty : '+stat,[]

            WyznK=DaneK[0][0]
            Znak=-1

    KlPom=str([Mag,DokId,Data,Numer]).replace("'",'"')
    SQL="SELECT rsf_podaj_numer_dokumentu_na_wydruk('"+Firma+"','"+KlPom+"','M') "
    stat,dm,dump,dump = PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return 'Bdy pobierania numeru dokumentu '+KluczDM+' na wydruk : '+stat,[]

    NrDokM=str(dm[0][0])

    if DokId=='MM':
        NrDokM+=" "+MagD+"/"+NumerD

    if not Robic:
        return "BR KOR$"+NrDokM

    if NrFakt=='None':
        NrFakt='NULL'
    else:
        NrFakt="'"+NrFakt[0:20]+"'"

    if Rej != 'NULL' and not CzyZmiany:
        DataRej="'"+Data+"'"

    #ptla po pozycjach wzoru w celu wybrania najlepszej pozycji
    Wybrany=None
    for poz in PozWz:
        PozMag=poz[3]
        PozMagD=poz[16]
        PozWyzn=poz[4]
        PozKontr=poz[5]
        PozZWyl=poz[6]
        PozPozost=poz[7]
        #rsglobal.ZapiszDoLogow("Wzor "+PozMag+" "+PozWyzn+" "+PozKontr+" "+str(PozZWyl)+" "+str(PozPozost),1)
        OK=True
        if PozMag not in ('','None'):
            if PozMag != Mag:
                OK=False

        if OK:
            if PozMagD not in ('','None'):
                if PozMagD != MagD:
                    OK=False

        if OK:
            if PozWyzn not in ('','None'):
                if PozWyzn not in (WyznF,WyznK):
                    OK=False

        if OK:
            if PozKontr not in ('','None'):
                PozKontr="'"+PozKontr+"'"
                OK= ((PozKontr.strip()==Kontr.strip()) and (not PozZWyl))
                #rsglobal.ZapiszDoLogow("Test 1 dla danych (rowny): '"+PozKontr.strip()+"' '"+Kontr.strip()+"' "+ str(not PozZWyl)+ " OK :"+str(OK),1)
                OK= ((not (PozKontr.strip() in ("''","'None'",Kontr.strip())) and PozZWyl) or OK)
                #rsglobal.ZapiszDoLogow("Test 2 dla danych (rozny): '"+PozKontr.strip()+"' "+ str(PozZWyl)+ " OK :"+str(OK),1)
                OK= (PozPozost or OK)
                #rsglobal.ZapiszDoLogow("Test 3 dla danych  (pozostae) : "+ str(PozPozost)+ " OK :"+str(OK),1)

        #jeli pasuje do schematu
        if OK:
            #rsglobal.ZapiszDoLogow("pasuje. Wybrany wczesniej "+str(Wybrany),1)
            #jak to pierwszy to ok
            if Wybrany==None:
                Wybrany=poz

            #trzeba sprawdzic nadrzdno. Warunki s w kolejnoci od najwaniejszego do najmniej waznego.
            else:
                #ten NOWY przypadek jest najmniej wany, zatem na pewno nie bdziemy podmienia.
                if not PozPozost:
                    #ten STARY przypadek jest najwaniejszy, zatem na pewno nie bdziemy podmienia.
                    if not Wybrany[5]==Kontr:
                        #ten STARY przypadek jest najmniej wany, zatem na pewno bdziemy podmienia.
                        if Wybrany[7]:
                            Wybrany=poz
                        #zosta tylko przypadek e STARY jest rodkowy (i to jest pewne bo pozostae s wyeliminowane wyej), a NOWY najwaniejszy (i to sprawdzamy)
                        elif PozKontr==Kontr:
                            Wybrany=poz


    # jeli si okazao, e nic nie pasuje do wzorca
    if Wybrany==None:
        return "BR POZWZM$"+NrDokM,[]

    #-------------------------------------------
    #tworzymy dowod ksigowy
    #-------------------------------------------

    poz=Wybrany

    stat,Test,dump,dump = PolSQL.Execute("SELECT rsf_sprawdz_czy_mozna_dodac_dokument('"+Firma+"','[\\\'"+Okres+"\\\',\\\'"+Dowod+"\\\',\\\'"+Nazwa+"\\\',\\\'"+DataDK+"\\\']','D');",3)
    if stat != 'OK':
        return 'Bd sprawdzania moliwoci dodania dowodu ksigowego :'+stat,[]

    if Test[0][0]!='OK':
        return 'Bd sprawdzania moliwoci dodania dowodu ksigowego :'+Test[0][0],[]

    stat,DNr,dump,dump = PolSQL.Execute("SELECT rsf_podaj_nowy_numer_dokumentu('"+Firma+"','[\\\'"+Okres+"\\\',\\\'"+Dowod+"\\\',\\\'"+Nazwa+"\\\',\\\'"+DataDK+"\\\']','D');",3)
    if stat != 'OK':
        rsglobal.ZapiszDoLogow('1. Dokument '+NrDokM+' istn poz data dla wzoru '+Dowod+'/'+Nazwa+'.'+stat,1)
        return "PO DOK",[]

    if DNr[0][0] in (None,'None',''):
        rsglobal.ZapiszDoLogow('1. Dokument '+NrDokM+' istn poz data dla wzoru '+Dowod+'/'+Nazwa+'.'+stat,1)
        return "PO DOK",[]

    NumerDK=rsglobal.Dekoduj(DNr[0][0])
    KluczDok=[Okres,Dowod,Nazwa,DataDK,NumerDK]

    #-------------------------------------------------------------------------------
    #przygotowanie nagwka
    #-------------------------------------------------------------------------------

    InsN= "INSERT INTO rs_dowody_ksiegowe "
    InsN+="(firma_id,okres_id,dowod_id,nazwa_id,data,numer,stan,kwota_wn,kwota_ma,"
    InsN+="symbol_dok,data_wyst,data_wplywu,termin_plat,data_zapl,kontrahent_id,rejestr,data_rejestru,nr_faktury,projekt_id,nr_dok_mag)"
    InsN+=" VALUES "
    InsN+="('"+Firma+"','"+Okres+"','"+Dowod+"','"+Nazwa+"','"+DataDK+"',"+str(NumerDK)+",'W',"+rsglobal.Dekoduj(str(Kwota*Znak))+","+rsglobal.Dekoduj(str(Kwota*Znak))+","
    InsN+="'"+NrDokM+"','"+DataDK+"',NULL,NULL,NULL,"+Kontr+","+Rej+","+DataRej+","+NrFakt+","+Proj+",'"+NrDokM+"');"

    #-------------------------------------------------------------------------------
    #przygotowanie pozycji
    #-------------------------------------------------------------------------------

    #  cz wsplna dla obu rodzajw dekretacji
    NrPozW=poz[0]
    KontoWn=rsglobal.Dekoduj(poz[1])
    KontoMa=rsglobal.Dekoduj(poz[2])
    #poz[8:15] zlecenie_wn,zlecenie_ma,konto_koszt_wn,konto_koszt_ma,magazyn_wn,magazyn_ma,projekt_wn,projekt_ma
    if poz[10]:
        KontoWn+="-"+KontoK
    if poz[11]:
        KontoMa+="-"+KontoK
    if poz[12]:
        if MagD not in ('None',''):
            KontoWn+="-"+MagD
        else:
            KontoWn+="-"+Mag
    if poz[13]:
        if MagS not in ('None',''):
            KontoMa+="-"+MagS
        else:
            KontoMa+="-"+Mag

    if poz[17]:
        if MagD not in ('None',''):
            KontoWn+="-"+MagD
    if poz[18]:
        if MagD not in ('None',''):
            KontoWn+="-"+MagD
    if poz[14]:
        KontoWn+="-"+ProjK
    if poz[15]:
        KontoMa+="-"+ProjK
    if poz[8]:
        KontoWn+="-"+Zlec
    if poz[9]:
        KontoMa+="-"+Zlec

    #-------------------------
    # pozycje z rozbicia
    #-------------------------
    if CzyRozbicie:
        #pobranie danych z pozycji typami
        Lp=0
        if not CzyZmiany: # dla normalnej dekretacji
            Wart="CASE WHEN a.jm_kalk='1' THEN dmp.ilosc1*dmp.cena1 ELSE dmp.ilosc2*dmp.cena2 END"
            SQL="SELECT a.typ_artykulu,ROUND(SUM(ROUND("+Wart+",2)),2) FROM rs_dokumenty_magazynowe_pozycje dmp "
            SQL+="INNER JOIN rs_artykuly a USING(firma_id,artykul_id) "+FiltrDokum+" GROUP BY a.typ_artykulu ORDER BY typ_artykulu;"
        else: # dla dekretacji zmian dokumentw
            SQL="SELECT a.typ_artykulu,ROUND(SUM(ROUND(z.wartosc_n-z.wartosc_s,2)),2) FROM rs_dokumenty_magazynowe_pozycje_zmiany z "
            SQL+="INNER JOIN rs_artykuly a USING(firma_id,artykul_id) "+FiltrDokum+" AND NOT pobrane_do_ksieg GROUP BY a.typ_artykulu ORDER BY typ_artykulu;"

        stat,DokPozycje,dump,dump = PolSQL.Execute(SQL,3)
        if stat != 'OK':
            return 'Bd pobierania pozycji dokumentu '+KluczDM+' do dekretacji : '+stat,[]

        if stat=='OK':
            InsP=''
            SlKTypowPoz={'01':[rsglobal.Dekoduj(poz[19]),rsglobal.Dekoduj(poz[29])],
                         '02':[rsglobal.Dekoduj(poz[20]),rsglobal.Dekoduj(poz[30])],
                         '03':[rsglobal.Dekoduj(poz[21]),rsglobal.Dekoduj(poz[31])],
                         '04':[rsglobal.Dekoduj(poz[22]),rsglobal.Dekoduj(poz[32])],
                         '05':[rsglobal.Dekoduj(poz[23]),rsglobal.Dekoduj(poz[33])],
                         '11':[rsglobal.Dekoduj(poz[24]),rsglobal.Dekoduj(poz[34])],
                         '12':[rsglobal.Dekoduj(poz[25]),rsglobal.Dekoduj(poz[35])],
                         '21':[rsglobal.Dekoduj(poz[26]),rsglobal.Dekoduj(poz[36])],
                         '31':[rsglobal.Dekoduj(poz[27]),rsglobal.Dekoduj(poz[37])],
                         '99':[rsglobal.Dekoduj(poz[28]),rsglobal.Dekoduj(poz[38])]}

            for pozdok in DokPozycje:
                Lp=Lp+1
                # jeli s podane konta w pozycji wzoru dekretacji to s uywane,
                # jeli nie to s brane po staremu
                if not SlKTypowPoz[pozdok[0]][0] in ('None',None,''):
                    KontoWnR=rsglobal.Dekoduj(KontoWn).replace('*',SlKTypowPoz[pozdok[0]][0])
                else:
                    KontoWnR=rsglobal.Dekoduj(KontoWn).replace('*',SlKTypow[pozdok[0]][0])
                if not SlKTypowPoz[pozdok[0]][1] in ('None',None,''):
                    KontoMaR=rsglobal.Dekoduj(KontoMa).replace('*',SlKTypowPoz[pozdok[0]][1])
                else:
                    KontoMaR=rsglobal.Dekoduj(KontoMa).replace('*',SlKTypow[pozdok[0]][1])

                if CzyRozbicieWn:
                    KwotaWnR=rsglobal.Dekoduj(str(rsglobal.RSRound(Znak*pozdok[1],2)))
                    KwotaWnWalR=rsglobal.Dekoduj(str(rsglobal.RSRound(Znak*pozdok[1]/float(Kurs),2)))
                else:
                    KwotaWnR=rsglobal.Dekoduj(str(Znak*Kwota))
                    KwotaWnWalR=rsglobal.Dekoduj(str(Znak*KwotaWal))

                if CzyRozbicieMa:
                    KwotaMaR=rsglobal.Dekoduj(str(rsglobal.RSRound(Znak*pozdok[1],2)))
                    KwotaMaWalR=rsglobal.Dekoduj(str(rsglobal.RSRound(Znak*pozdok[1]/float(Kurs),2)))
                else:
                    KwotaMaR=rsglobal.Dekoduj(str(Znak*Kwota))
                    KwotaMaWalR=rsglobal.Dekoduj(str(Znak*KwotaWal))

                KontoWnOst=KontoWnR
                KontoMaOst=KontoMaR
                KwotaWnOst=KwotaWnR
                KwotaMaOst=KwotaMaR
                KwotaWnWalOst=KwotaWnWalR
                KwotaMaWalOst=KwotaMaWalR
                # zamiana kont w przypadku korekty dokumentu magazynowego
                if ObracacKonta:
                    KontoWnOst=KontoMaR
                    KontoMaOst=KontoWnR
                    KwotaWnOst=KwotaMaR
                    KwotaMaOst=KwotaWnR
                    KwotaWnWalOst=KwotaMaWalR
                    KwotaMaWalOst=KwotaWnWalR

                InsP+="INSERT INTO rs_dowody_ksiegowe_pozycje (firma_id,okres_id,dowod_id,nazwa_id,data,numer,nr_poz,nr_ppoz,nr_pozw,"
                InsP+="symbol_dok,data_wyst,data_wplywu,termin_plat,data_zapl,kontrahent_id,projekt_wn,projekt_ma,"
                InsP+="konto_wn,kwota_wn,konto_vat_wn,kwota_vat_wn,stawka_vat_wn,tresc_wn,sposob_rozl,waluta_wn,kurs_wn,kwota_wal_wn,"
                InsP+="nkup,konto_ma,kwota_ma,konto_vat_ma,kwota_vat_ma,stawka_vat_ma,tresc_ma,waluta_ma,kurs_ma,kwota_wal_ma,pnop)"
                InsP+=" VALUES "
                InsP+="('"+Firma+"','"+Okres+"','"+Dowod+"','"+Nazwa+"','"+DataDK+"',"+str(NumerDK)+","+str(Lp)+",0,"+rsglobal.Dekoduj(str(NrPozW))+","
                InsP+="'"+NrDokM+"','"+DataDK+"',NULL,NULL,NULL,"+Kontr+","+Proj+","+Proj+","
                InsP+="'"+KontoWnOst+"',"+KwotaWnOst+",'',0,NULL,'"+NrDokM+TrescDodat+"','','"+Wal+"',"+Kurs+","+KwotaWnWalOst+","
                InsP+="FALSE,'"+KontoMaOst+"',"+KwotaMaOst+",'',0,NULL,'"+NrDokM+TrescDodat+"','"+Wal+"',"+Kurs+","+KwotaMaWalOst+",FALSE);"

                if not CzyRozbicieWn:
                    KontoWn=''

                if not CzyRozbicieMa:
                    KontoMa=''

                Kwota=0
                KwotaWal=0
    else:
    #-------------------------
    # tylko jedna pozycja
    #-------------------------
        KontoWnOst=KontoWn
        KontoMaOst=KontoMa
        # zamiana kont w przypadku korekty dokumentu magazynowego
        if ObracacKonta:
            KontoWnOst=KontoMa
            KontoMaOst=KontoWn

        InsP="INSERT INTO rs_dowody_ksiegowe_pozycje (firma_id,okres_id,dowod_id,nazwa_id,data,numer,nr_poz,nr_ppoz,nr_pozw,"
        InsP+="symbol_dok,data_wyst,data_wplywu,termin_plat,data_zapl,kontrahent_id,projekt_wn,projekt_ma,"
        InsP+="konto_wn,kwota_wn,konto_vat_wn,kwota_vat_wn,stawka_vat_wn,tresc_wn,sposob_rozl,waluta_wn,kurs_wn,kwota_wal_wn,"
        InsP+="nkup,konto_ma,kwota_ma,konto_vat_ma,kwota_vat_ma,stawka_vat_ma,tresc_ma,waluta_ma,kurs_ma,kwota_wal_ma,pnop)"
        InsP+=" VALUES "
        InsP+="('"+Firma+"','"+Okres+"','"+Dowod+"','"+Nazwa+"','"+DataDK+"',"+str(NumerDK)+",1,0,"+rsglobal.Dekoduj(str(NrPozW))+","
        InsP+="'"+NrDokM+"','"+DataDK+"',NULL,NULL,NULL,"+Kontr+","+Proj+","+Proj+","
        InsP+="'"+KontoWnOst+"',"+rsglobal.Dekoduj(str(Znak*Kwota))+",'',0,NULL,'"+NrDokM+TrescDodat+"','','"+Wal+"',"+Kurs+","+rsglobal.Dekoduj(str(Znak*KwotaWal))+","
        InsP+="FALSE,'"+KontoMaOst+"',"+rsglobal.Dekoduj(str(Znak*Kwota))+",'',0,NULL,'"+NrDokM+TrescDodat+"','"+Wal+"',"+Kurs+","+rsglobal.Dekoduj(str(Znak*KwotaWal))+",FALSE);"

    #ostatnie kroki - stworzenie faktyczne dowodu i zmiana stanu dokumentu magazynowego na K
    UpdZam=''
    if not CzyZmiany:
        UpdZam="UPDATE rs_dokumenty_magazynowe SET stan='K' "+FiltrDokum+";"

    stat,dump,dump,dump = PolSQL.Execute(InsN+InsP+UpdZam,1)

    if stat!= 'OK':
        if not CzyZmiany:
            PolSQL.Rollback()
        return 'Bd wstawiania dowodu ksigowego : '+stat,[]

    stat=fs_dekretacja_automatyczna_projekty.DodajProjektyDzialyDoDowodu(PolSQL,Firma,Okres,Dowod,Nazwa,DataDK,str(NumerDK),'M',NrDokM)
    if stat!='OK':
        if not CzyZmiany:
            PolSQL.Rollback()
        return stat,[]

    # OSTATECZNY ZAPIS DEKRETACJI
    if not CzyZmiany:
        #rsglobal.ZapiszDoLogow('Zamknity dokument : '+NrFakt,5)
        PolSQL.Commit()

    return 'OK',KluczDok


#-##############################################################################
#-- DEKRETACJA DOSTAW
#-##############################################################################

def ZadekretujDostawe(self,dane,klucze,DDostN,Rejestr,KontoId):
    stat='OK'

    Dowod= klucze[0]
    Nazwa= klucze[1]
    OkresDekr=self.okres_id
    if len(klucze)==5:
        OkresDekr=klucze[4]

    Data = DDostN[0]
    rsglobal.ZapiszDoLogow('0 '+str(klucze),1)
    rsglobal.ZapiszDoLogow('1 '+str(DDostN),1)
    NrD = rsglobal.UsunKropkaZero(str(DDostN[1]))

    NrDost = Data+'/'+NrD

    #--- pobranie stanu dokumentu w celu zablokowania wielokrotnej dekretacji
    SQL="SELECT stan FROM rs_dostawy WHERE firma_id='"+self.firma_id+"' AND data='"+Data+"'"
    SQL+=" AND numer="+NrD+" FOR UPDATE NOWAIT;"
    stat,DStan,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        dane.append('BLOK STAN$'+NrDost)
        return dane
    if DStan[0][0] !='Z':
        dane.append('BL STAN$'+NrDost)
        return dane

    #--- Pobranie uprawnien do dokumentw ksigowych dla operatora
    UprTotal=True
    if self.operator_id == '0':
        UprDodawanie=True
        LDod=[]
    else:
        SQL = "SELECT dowod_id,nazwa_id,wprowadzanie,edycja,usuwanie,przegladanie,zamykanie,otwieranie " \
              "FROM rs_wzory_dekretacji_upr WHERE firma_id='"+self.firma_id+"' AND operator_id in " \
              ""+str(self.operatorzy).replace("[","(").replace("]",")")+" AND okres_id='"+OkresDekr+"'; "
        stat,daneupr,dump,dump = self.PolSQL.Execute(SQL,3)
        if stat!='OK':
            stat = 'BL SPR UPR'
            dane.append(stat)
        if stat=='OK':
            if len(daneupr)>0:
                UprTotal=False
            else:
                UprTotal=True
            LDod=[]

            UprDodawanie=(self.oddzial_id,'075013') in self.uprawnienia
            if UprDodawanie:
                for d in daneupr:
                    if d[2]:
                        LDod.append((rsglobal.Dekoduj(d[0]).strip(),rsglobal.Dekoduj(d[1]).strip()))

    if not UprDodawanie:
        stat='UPR DOD'
        dane.append(stat)
    if stat=='OK' and not UprTotal:
        if (not(str(Dowod),Nazwa) in LDod) and self.operator_id!='0':
            stat='UPR DOD'
            dane.append(stat)

    if stat != 'OK':
        return dane

    #--- WSTAWIENIE NAGLWKA
    try:
        #kwota_wn - suma kwot wn z pozycji
        #kwota_ma suma kwot_ma z pozycji

        TerminPlat = rsglobal.Dekoduj(DDostN[2])
        if TerminPlat in ('None',None,''):
            TerminPlat = 'NULL'
        else:
            TerminPlat = "'"+TerminPlat+"'"
        Kontr = rsglobal.Dekoduj(DDostN[3])
        Waluta = rsglobal.Dekoduj(DDostN[4])
        Kurs = DDostN[5]
        DoZapl = DDostN[6]
        Wersja = rsglobal.Dekoduj(DDostN[7])
        DataWpl = rsglobal.Dekoduj(DDostN[8])
        NrFakt = rsglobal.Dekoduj(DDostN[9])
        rsglobal.ZapiszDoLogow('31 '+str(len(DDostN)),1)
        DataWyst = rsglobal.Dekoduj(DDostN[10])
        if DataWyst in ('None',None,''):
            DataWyst = 'NULL'
        else:
            DataWyst = "'"+DataWyst+"'"
        Tresc = rsglobal.Dekoduj(DDostN[11])
        #WS 2020-04-05
        OznTrans = rsglobal.Dekoduj(DDostN[13])
        OznNabycie = rsglobal.Dekoduj(DDostN[14])
        Import = 'TRUE' if DDostN[15] is True else 'FALSE'

        #WS 2020-06-23
        # pobranie czy_mpp z danych kontrahenta
        sql = "SELECT czy_mpp FROM rs_kontrahenci"
        sql += " WHERE firma_id='"+self.firma_id+"' AND kontrahent_id='"+Kontr+"';"
        stat, DMPP, dump, dump = self.PolSQL.Execute(sql, 3)
        if stat != 'OK':
            dane.append("Bd pobrania danych kontrahenta")
            return dane
        else:
            CzyMPP = 'TRUE' if DMPP[0][0] is True else 'FALSE'

        WarPlatnik=rsglobal.Dekoduj(Kontr).strip()
        WarWaluta=rsglobal.Dekoduj(Waluta).strip()
        WarProjektId=''
        WarKontrahent=rsglobal.Dekoduj(Kontr).strip()
        WarZamawiajacy=rsglobal.Dekoduj(Kontr).strip()
        WarSposobZaplaty=''
        WarKierowca=''
        WarHandlowiec=''
        WarWersja=rsglobal.Dekoduj(Wersja).strip()
        WarDokumentMag=rsglobal.Dekoduj(DDostN[12]).strip()
        # JR 2024-01-25 (1522)
        Wartosci = {}
        Wartosci['WarPlatnik'] = rsglobal.Dekoduj(Kontr).strip()
        Wartosci['WarWaluta'] = rsglobal.Dekoduj(Waluta).strip()
        Wartosci['WarProjektId'] = ''
        Wartosci['WarKontrahent'] = rsglobal.Dekoduj(Kontr).strip()
        Wartosci['WarZamawiajacy'] = rsglobal.Dekoduj(Kontr).strip()
        Wartosci['WarSposobZaplaty'] = ''
        Wartosci['WarKierowca'] = ''
        Wartosci['WarHandlowiec'] = ''
        Wartosci['WarWersja'] = rsglobal.Dekoduj(Wersja).strip()
        Wartosci['WarDokumentMag'] = rsglobal.Dekoduj(DDostN[12]).strip()

        Rej=rsglobal.Dekoduj(Rejestr)

        #pobranie potencjalnych pozycji ze wzoru dekretacji - sprawdzenie czy nie ma by po prostu zalepkowa (bez dekretacji)
        if stat=='OK':
            SQL="SELECT nr_poz,konto_ma,konto_wn,konto_vat_wn,stawka_vat_wn,czy_kontrahent," \
                " dekretowac,warunek,konto_ma_z_kontrahenta,COALESCE(sposob_rozl,'') FROM rs_wzory_dekretacji_dostaw " \
                " WHERE firma_id='"+self.firma_id+"' AND okres_id='"+OkresDekr+"'" \
                " AND dowod_id='"+Dowod+"' AND nazwa_id='"+Nazwa+"';"
            stat,PozWz,dump,dump = self.PolSQL.Execute(SQL,3)

        if stat=='OK':
            if len(PozWz) == 0:
                dane.append("Brak pozycji wzoru dekretacji dla wzoru "+Dowod+' '+Nazwa+' w dokumencie '+NrDost)
                return dane

        # jeli pole "dekretowa" jest False to zmieniamy stan dostawy na K i koniec
        if stat=='OK':
            if not PozWz[0][6]:
                SQL="UPDATE rs_dostawy SET stan='K' WHERE firma_id='"+self.firma_id+"' AND data='"+Data+"' AND numer="+NrD+";"
                stat,dump,dump,dump = self.PolSQL.Execute(SQL,1)
                if stat!= 'OK':
                    self.PolSQL.Rollback()
                    dane.append(stat)
                else:
                    self.PolSQL.Commit()
                return dane

        #przygotowanie nagwka dowodu ksigowego (samo wstawienie na kocu)
        if stat=='OK':
            stat,Test,dump,dump = self.PolSQL.Execute("SELECT rsf_sprawdz_czy_mozna_dodac_dokument('"+self.firma_id+"','[\\\'"+OkresDekr+"\\\',\\\'"+Dowod+"\\\',\\\'"+Nazwa+"\\\',\\\'"+Data+"\\\']','D');",3)

        if stat=='OK':
            if Test[0][0]=='OK':
                stat,DNr,dump,dump = self.PolSQL.Execute("SELECT rsf_podaj_nowy_numer_dokumentu('"+self.firma_id+"','[\\\'"+OkresDekr+"\\\',\\\'"+Dowod+"\\\',\\\'"+Nazwa+"\\\',\\\'"+Data+"\\\']','D');",3)
                if DNr[0][0] in (None,'None',''):
                    dane.append("PO DOK")
                    return dane
                else:
                    Numer=rsglobal.Dekoduj(DNr[0][0])
            else:
                dane.append("PO DOK")
                return dane

        #WS 2020-04-05 - dodany zapis OznTrans, OznNabycie, Import
        #WS 2020-06-23 - czy_mpp
        InsN= "INSERT INTO rs_dowody_ksiegowe " \
              "(firma_id,okres_id,dowod_id,nazwa_id,data,numer,stan,kwota_wn,kwota_ma," \
              "symbol_dok,data_wyst,data_wplywu,termin_plat,data_zapl,kontrahent_id,rejestr," \
              "data_rejestru,projekt_id,tresc,konto_id,ozn_trans,ozn_nabycie,import,czy_mpp)" \
              " VALUES " \
              "('"+self.firma_id+"','"+OkresDekr+"','"+Dowod+"','"+Nazwa+"','"+Data+"',"+str(Numer)+",'W',"+str(DoZapl)+","+str(DoZapl)+"," \
              "'"+NrFakt+"',"+DataWyst+",'"+DataWpl+"',"+TerminPlat+",NULL,'"+Kontr+"',"+Rej+"," \
              "'"+Data+"',NULL,'"+Tresc+"',"+KontoId+","+rsglobal.ZmienNaNULL(OznTrans)+","+rsglobal.ZmienNaNULL(OznNabycie)+","+Import+"," \
              ""+CzyMPP+");"

        #sprawdzenie czy bd * i podzia na typy artykuw czy nie ( i czy wszystkie maj gwiazdk jeli pierwszy ma. (sprawdzamy w polu konto_wn)
        CzyRozbicie= (PozWz[0][2][0]=='*')
        for poz in PozWz:
            if CzyRozbicie:
                if poz[2][0] != '*':
                    dane.append("BR GW$"+NrDost)
                    return dane

        #sprawdzenie warunku i odrzucenie tych pozycji wzoru dekretacji, ktre nie speniaj
        LPozWzPoWar=[]
        for poz in PozWz:
            War=poz[7]
            if War not in ['',None,'None']:
                try:
                    if evalWarunek(War, Wartosci):
                        #JR LPozWzPoWar.append(eval(str(poz)))
                        LPozWzPoWar.append(poz)
                except Exception,e:
                    dane.append("BL WAR$"+NrFakt+u' WARUNEK: '+War+u' BD: '+str(e))
                    return dane
            else:
                #JR LPozWzPoWar.append(eval(str(poz)))
                LPozWzPoWar.append(poz)

        if len(LPozWzPoWar)==0:
            dane.append("BR POZ WZ$"+NrFakt)
            return dane

        PozWz=LPozWzPoWar

        #pobranie stawek vat z dostawy
        SQL="SELECT stawka_vat,wartosc_netto,wartosc_vat,wartosc_wal_netto,wartosc_wal_vat" \
            " FROM rs_dostawy_vat WHERE firma_id='"+self.firma_id+"' " \
            " AND data='"+Data+"' AND numer="+NrD+";"

        stat,StVat,dump,dump = self.PolSQL.Execute(SQL,3)
        rsglobal.ZapiszDoLogow('4 Dekretacja z rozbiciem? '+str(CzyRozbicie),1)
        #-------------------------
        #--- ZWYKA DEKRETACJA
        #-------------------------
        if not CzyRozbicie:
            #insert pierwszej pozycji (ze stawk domyln lub bez)
            Insert = ''
            KontoMa = ''

            #rsglobal.ZapiszDoLogow('14 Pozycje WZ '+str(PozWz),1)
            #rsglobal.ZapiszDoLogow('14 Pozycje Vat '+str(StVat),1)
            for poz in PozWz:
                if Insert == '' and stat == 'OK': # jeszcze nie znaleziono
                    if poz[1] not in ['','None'] or poz[8]: # jeli uzupenione konto ma lub chb z kontem ma z kontrahenta
                        if poz[8]:
                            SQL ="SELECT COALESCE(konto_odbiorca,'') FROM rs_kontrahenci WHERE firma_id='"+self.firma_id+"'"
                            SQL+=" AND kontrahent_id='"+Kontr+"'"
                            stat,DKontoMa,dump,dump = self.PolSQL.Execute(SQL,3)
                            if stat =='OK':
                                KontoMa = rsglobal.Dekoduj(DKontoMa[0][0])
                        else:
                            KontoMa = rsglobal.Dekoduj(poz[1])

                        if stat == 'OK' and KontoMa != '':
                            if poz[5]:
                                KontoMa += '-'+Kontr

                            SposRozl = poz[9]
                            SposRozl = "'"+SposRozl+"'"

                            Insert+="INSERT INTO rs_dowody_ksiegowe_pozycje (firma_id,okres_id,dowod_id,nazwa_id,data,numer,nr_poz,nr_ppoz,nr_pozw,"
                            Insert+="symbol_dok,data_wyst,data_wplywu,termin_plat,data_zapl,kontrahent_id,"
                            Insert+="konto_ma,kwota_ma,konto_vat_ma,kwota_vat_ma,stawka_vat_ma,tresc_ma,waluta_ma,kurs_ma,kwota_wal_ma,"
                            Insert+="nkup,konto_wn,kwota_wn,konto_vat_wn,kwota_vat_wn,stawka_vat_wn,tresc_wn,sposob_rozl,waluta_wn,kurs_wn,kwota_wal_wn,pnop,data_ksieg_kosztow)"
                            Insert+=" VALUES "
                            Insert+="('"+self.firma_id+"','"+OkresDekr+"','"+Dowod+"','"+Nazwa+"','"+Data+"',"+str(Numer)+",1,0,"+rsglobal.Dekoduj(str(poz[0]))+","
                            Insert+="'"+NrFakt+"',"+DataWyst+",'"+DataWpl+"',"+TerminPlat+",NULL,'"+Kontr+"',"
                            Insert+="'"+KontoMa+"',"+str(DoZapl)+",'',0,NULL,'"+Tresc+"','"+Waluta+"',"+str(Kurs)+","+str(rsglobal.RSRound(DoZapl/Kurs,2))+","

                            #sprawdzenie czy na pozycji z nagwkiem ma jest tez cz wn. Jeli jest to uzupeniana, jak nie to nie.

                            StawkaP1=''
                            Jest=False
                            if poz[3] not in ['','None']:
                                StawkaP1=rsglobal.Dekoduj(poz[4])
                                for pst in StVat:
                                    if pst[0]==StawkaP1:
                                        Jest=True
                                        KontoWn=rsglobal.Dekoduj(poz[2])
                                        KontoVatWn=rsglobal.Dekoduj(poz[3])
                                        if KontoVatWn =='None':
                                            KontoVatWn=''

                                        KwotaWn=rsglobal.Dekoduj(str(pst[1]))
                                        KwotaVatWn=rsglobal.Dekoduj(str(pst[2]))
                                        KwotaWalWn=rsglobal.Dekoduj(str(pst[3]))

                                        Insert+="FALSE,'"+KontoWn+"',"+KwotaWn+",'"+KontoVatWn+"',"+KwotaVatWn+",'"+StawkaP1+"','"+Tresc+"',"+SposRozl+",'"+Waluta+"',"+str(Kurs)+","+KwotaWalWn+",FALSE"

                            if not Jest:
                                Insert+="FALSE,'',0,'',0,NULL,'','','"+Waluta+"',"+str(Kurs)+",0,FALSE"
                                     #AB 04.10.18r
                            Insert+=",'"+Data+"');"

            if stat=='OK':
                #Nie byo konta ma na wzorze
                if Insert=='':
                    dane.append("BR KMA$"+Tresc+" "+NrDost)
                    return dane

                #Sprawdzenie pozostaych stawek vat z dostawy i wpisanie ich na wn
                NrPozDow=2

                for pst in StVat:
                    if pst[0] != StawkaP1:
                        Stawka=rsglobal.Dekoduj(pst[0])
                        Jest=False
                        #ptla po pozycjach wzoru dekretacji
                        for poz in PozWz:
                            if not Jest:
                                #AB 18.02.22r
                                if Stawka==poz[4]:
                                    if poz[3] not in ['','None']:
                                        Jest=True
                                    else:
                                        try:
                                            st = int(Stawka)
                                        except:
                                            st = 0
                                        if st == 0:
                                            Jest = True
                                    if Jest:
                                        KontoWn=rsglobal.Dekoduj(poz[2])
                                        KontoVatWn=rsglobal.Dekoduj(poz[3])
                                        if KontoVatWn =='None':
                                            KontoVatWn=''

                                        KwotaWn=rsglobal.Dekoduj(str(pst[1]))
                                        if KwotaWn !=0:
                                            KwotaVatWn=rsglobal.Dekoduj(str(pst[2]))
                                            KwotaWalWn=rsglobal.Dekoduj(str(pst[3]))

                                            SposRozl = poz[9]
                                            SposRozl = "'"+SposRozl+"'"

                                            Insert+="INSERT INTO rs_dowody_ksiegowe_pozycje (firma_id,okres_id,dowod_id,nazwa_id,data,numer,nr_poz,nr_ppoz,nr_pozw,"
                                            Insert+="symbol_dok,data_wyst,data_wplywu,termin_plat,data_zapl,kontrahent_id,"
                                            Insert+="konto_ma,kwota_ma,konto_vat_ma,kwota_vat_ma,stawka_vat_ma,tresc_ma,waluta_ma,kurs_ma,kwota_wal_ma,"
                                            Insert+="nkup,konto_wn,kwota_wn,konto_vat_wn,kwota_vat_wn,stawka_vat_wn,tresc_wn,sposob_rozl,waluta_wn,kurs_wn,kwota_wal_wn,pnop,data_ksieg_kosztow)"
                                            Insert+=" VALUES "
                                            Insert+="('"+self.firma_id+"','"+OkresDekr+"','"+Dowod+"','"+Nazwa+"','"+Data+"',"+str(Numer)+","+str(NrPozDow)+",0,"+rsglobal.Dekoduj(str(poz[0]))+","
                                            Insert+="'"+NrFakt+"',"+DataWyst+",'"+DataWpl+"',"+TerminPlat+",NULL,'"+Kontr+"',"
                                            Insert+="'',0,'',0,NULL,'','"+Waluta+"',"+str(Kurs)+",0,"                                                                                                      #AB 04.10.18r
                                            Insert+="FALSE,'"+KontoWn+"',"+KwotaWn+",'"+KontoVatWn+"',"+KwotaVatWn+",'"+Stawka+"','"+Tresc+"',"+SposRozl+",'"+Waluta+"',"+str(Kurs)+","+KwotaWalWn+",FALSE,'"+Data+"');"

                                            NrPozDow+=1


                        #Nie byo konta ma na wzorze
                        if not Jest:
                            dane.append("BR KWN$"+Tresc+" "+NrDost+" dla stawki "+Stawka)
                            return dane


        else:

        #------------------------------------------------------
        #--- DEKRETACJA Z ROZBICIEM NA TYPY ARTYKULOW
        #------------------------------------------------------

            #pobranie kont z okresu w podziale na typy
            stat,DaneKontaTypow,dump,dump = self.PolSQL.Execute("SELECT konto_dostawy_01, konto_dostawy_02, konto_dostawy_03, konto_dostawy_04, konto_dostawy_05, konto_dostawy_11, konto_dostawy_12, konto_dostawy_21, konto_dostawy_31, konto_dostawy_99, konto_dostawy_zaokr FROM rs_wzory_dekretacji_typy WHERE firma_id='"+self.firma_id+"' AND okres_id='"+OkresDekr+"'",3)
            if len(DaneKontaTypow)==0:
                dane.append("BR KT")
                return dane
            rsglobal.ZapiszDoLogow('41',1)
            #stworzenie slownika zaleznego od typow
            SlKTypow={}
            SlKTypow['01']=rsglobal.Dekoduj(DaneKontaTypow[0][0])
            SlKTypow['02']=rsglobal.Dekoduj(DaneKontaTypow[0][1])
            SlKTypow['03']=rsglobal.Dekoduj(DaneKontaTypow[0][2])
            SlKTypow['04']=rsglobal.Dekoduj(DaneKontaTypow[0][3])
            SlKTypow['05']=rsglobal.Dekoduj(DaneKontaTypow[0][4])
            SlKTypow['11']=rsglobal.Dekoduj(DaneKontaTypow[0][5])
            SlKTypow['12']=rsglobal.Dekoduj(DaneKontaTypow[0][6])
            SlKTypow['21']=rsglobal.Dekoduj(DaneKontaTypow[0][7])
            SlKTypow['31']=rsglobal.Dekoduj(DaneKontaTypow[0][8])
            SlKTypow['99']=rsglobal.Dekoduj(DaneKontaTypow[0][9])
            KontoZaokr=rsglobal.Dekoduj(DaneKontaTypow[0][10])
            KwZaokr=0

            rsglobal.ZapiszDoLogow('42',1)
            #pobranie sum pozycji w podziale na typy
            Sumy="SUM(dp.wartosc_netto), ROUND(SUM(dp.wartosc_netto)*CASE WHEN dp.stawka_vat IN ("+str(self.ListaStawekZn)[1:-1]+") THEN 0 ELSE CAST (CAST(dp.stawka_vat AS TEXT) AS NUMERIC)*0.01 END,2)"
            Sumy+=", SUM(dp.wartosc_wal_netto) "

            Sel="SELECT dp.stawka_vat,a.typ_artykulu,"+Sumy+" FROM rs_dostawy_pozycje dp INNER JOIN rs_artykuly a USING (firma_id,artykul_id)"
            Sel+=" WHERE firma_id='"+self.firma_id+"' AND data='"+Data+"' and numer="+NrD+" GROUP BY dp.stawka_vat,"
            Sel+=" a.typ_artykulu HAVING SUM(dp.wartosc_netto) != 0 ORDER BY stawka_vat,typ_artykulu"
            stat,DanePoz,dump,dump = self.PolSQL.Execute(Sel,3)
            Insert=''
            StawkaP1=''
            TypArtP1=''
            NrPozDow=2
            self.SStawkiVAT={}
            rsglobal.ZapiszDoLogow('43 przed pozycja 1 '+str(DanePoz),1)
            for poz in PozWz:
                if Insert=='': #jeszcze nie znaleziono
                    if poz[1] not in ['','None'] or poz[8]: # jeli uzupenione konto ma lub chb z kontem ma z kontrahenta
                        if poz[8]:
                            SQL ="SELECT COALESCE(konto_odbiorca,'') FROM rs_kontrahenci WHERE firma_id='"+self.firma_id+"'"
                            SQL+=" AND kontrahent_id='"+Kontr+"'"
                            stat,DKontoMa,dump,dump = self.PolSQL.Execute(SQL,3)
                            if stat =='OK':
                                KontoMa=rsglobal.Dekoduj(DKontoMa[0][0])
                        else:
                            KontoMa=rsglobal.Dekoduj(poz[1])
                        if stat =='OK':
                            if poz[5]:
                                KontoMa+='-'+Kontr

                            SposRozl = poz[9]
                            SposRozl = "'"+SposRozl+"'"

                            Insert+="INSERT INTO rs_dowody_ksiegowe_pozycje (firma_id,okres_id,dowod_id,nazwa_id,data,numer,nr_poz,nr_ppoz,nr_pozw,"
                            Insert+="symbol_dok,data_wyst,data_wplywu,termin_plat,data_zapl,kontrahent_id,"
                            Insert+="konto_ma,kwota_ma,konto_vat_ma,kwota_vat_ma,stawka_vat_ma,tresc_ma,waluta_ma,kurs_ma,kwota_wal_ma,"
                            Insert+="nkup,konto_wn,kwota_wn,konto_vat_wn,kwota_vat_wn,stawka_vat_wn,tresc_wn,sposob_rozl,waluta_wn,kurs_wn,kwota_wal_wn,pnop,data_ksieg_kosztow)"
                            Insert+=" VALUES "
                            Insert+="('"+self.firma_id+"','"+OkresDekr+"','"+Dowod+"','"+Nazwa+"','"+Data+"',"+str(Numer)+",1,0,"+rsglobal.Dekoduj(str(poz[0]))+","
                            Insert+="'"+NrFakt+"',"+DataWyst+",'"+DataWpl+"',"+TerminPlat+",NULL,'"+Kontr+"',"
                            Insert+="'"+KontoMa+"',"+str(DoZapl)+",'',0,NULL,'"+Tresc+"','"+Waluta+"',"+str(Kurs)+","+str(rsglobal.RSRound(DoZapl/Kurs,2))+","

                            #sprawdzenie czy na pozycji z nagwkiem ma jest tez cz wn. Jeli jest to uzupeniana, jak nie to nie.
                            rsglobal.ZapiszDoLogow('45',1)
                            Jest=False
                            if poz[3] not in ['','None']:
                                StawkaP1=rsglobal.Dekoduj(poz[4])
                                for psttyp in DanePoz:
                                    if psttyp[0]==StawkaP1 and not Jest:
                                        StawkaPoprzP1=rsglobal.Dekoduj(psttyp[4])
                                        Jest=True
                                        TypArtP1=psttyp[1]
                                        if SlKTypow[TypArtP1] in ('','None',None):
                                            dane.append("BR KTYP$ dla typu artykuu "+TypArtP1)
                                            return dane

                                        KontoWn=rsglobal.Dekoduj(poz[2]).replace('*',SlKTypow[TypArtP1])
                                        KontoVatWn=rsglobal.Dekoduj(poz[3])
                                        if KontoVatWn =='None':
                                            KontoVatWn=''
                                        rsglobal.ZapiszDoLogow('46',1)
                                        KwotaWn=rsglobal.Dekoduj(str(psttyp[2]))
                                        KwotaVatWn=rsglobal.Dekoduj(str(psttyp[3]))
                                        KwotaWalWn=rsglobal.Dekoduj(str(psttyp[4]))

                                        Insert+="FALSE,'"+KontoWn+"',"+KwotaWn+",'"+KontoVatWn+"',"+KwotaVatWn+",'"+StawkaP1+"','"+Tresc+"',"+SposRozl+",'"+Waluta+"',"+str(Kurs)+","+KwotaWalWn+",FALSE"
                                        if not StawkaP1 in self.SStawkiVAT:
                                            self.SStawkiVAT[StawkaP1]=psttyp[3]
                            if not Jest:
                                Insert+="FALSE,'',0,'',0,NULL,'','','"+Waluta+"',"+str(Kurs)+",0,FALSE"
                                        #AB 04.10.18r
                            Insert+=",'"+Data+"');"

            #Nie byo konta ma na wzorze
            if Insert=='':
                dane.append("BR KMA$"+Tresc+" "+NrDost)
                return dane
            rsglobal.ZapiszDoLogow('47',1)
            #Sprawdzenie pozostaych stawek vat z faktury i wpisanie ich na wn


            #zmienna ta suy do tego by ptla przebiega 2 razy. Za pierwszym razem uzupeniana jest stawka, ktra zostaa
            #ju pobrana w nagwkowej czsci zostaje ew. uzupeniona, potem przebieg drugi uzupeniajcy reszt stawek

            PierwszyPrzebieg=True
            Robic=True
            while Robic:
                for psttyp in DanePoz:
                    Stawka=rsglobal.Dekoduj(psttyp[0])
                    TypArt=rsglobal.Dekoduj(psttyp[1])
                    if (PierwszyPrzebieg and Stawka==StawkaP1 and TypArt != TypArtP1) or (not PierwszyPrzebieg and Stawka != StawkaP1):
                        Jest=False
                        #ptla po pozycjach wzoru dekretacji
                        for poz in PozWz:
                            if not Jest:
                                #AB 18.02.22r
                                if Stawka==poz[4]:
                                    if poz[3] not in ['','None']:
                                        Jest=True
                                    else:
                                        try:
                                            st = int(Stawka)
                                        except:
                                            st = 0
                                        if st == 0:
                                            Jest = True
                                    if Jest:
                                        if SlKTypow[TypArt] in ('','None',None):
                                            dane.append("BR KTYP$ dla typu artykuu "+TypArt)
                                            return dane

                                        KontoWn=rsglobal.Dekoduj(poz[2]).replace('*',SlKTypow[TypArt])
                                        KontoVatWn=rsglobal.Dekoduj(poz[3])
                                        if KontoVatWn =='None':
                                            KontoVatWn=''

                                        KwotaWn=rsglobal.Dekoduj(str(psttyp[2]))
                                        KwotaVatWn=rsglobal.Dekoduj(str(psttyp[3]))
                                        KwotaWalWn=rsglobal.Dekoduj(str(psttyp[4]))

                                        Insert+="INSERT INTO rs_dowody_ksiegowe_pozycje (firma_id,okres_id,dowod_id,nazwa_id,data,numer,nr_poz,nr_ppoz,nr_pozw,"
                                        Insert+="symbol_dok,data_wyst,data_wplywu,termin_plat,data_zapl,kontrahent_id,"
                                        Insert+="konto_ma,kwota_ma,konto_vat_ma,kwota_vat_ma,stawka_vat_ma,tresc_ma,waluta_ma,kurs_ma,kwota_wal_ma,"
                                        Insert+="nkup,konto_wn,kwota_wn,konto_vat_wn,kwota_vat_wn,stawka_vat_wn,tresc_wn,sposob_rozl,waluta_wn,kurs_wn,kwota_wal_wn,pnop,data_ksieg_kosztow)"
                                        Insert+=" VALUES "
                                        Insert+="('"+self.firma_id+"','"+OkresDekr+"','"+Dowod+"','"+Nazwa+"','"+Data+"',"+str(Numer)+","+str(NrPozDow)+",0,"+rsglobal.Dekoduj(str(poz[0]))+","
                                        Insert+="'"+NrFakt+"',"+DataWyst+",'"+DataWpl+"',"+TerminPlat+",NULL,'"+Kontr+"',"
                                        Insert+="'',0,'',0,NULL,'','"+Waluta+"',"+str(Kurs)+",0,"                                                                                                      #AB 04.10.18r
                                        Insert+="FALSE,'"+KontoWn+"',"+KwotaWn+",'"+KontoVatWn+"',"+KwotaVatWn+",'"+Stawka+"','"+Tresc+"',"+SposRozl+",'"+Waluta+"',"+str(Kurs)+","+KwotaWalWn+",FALSE,'"+Data+"');"

                                        NrPozDow+=1

                                        if Stawka in self.SStawkiVAT:
                                            self.SStawkiVAT[Stawka]+=psttyp[3]
                                        else:
                                            self.SStawkiVAT[Stawka]=psttyp[3]

                        #Nie byo konta ma na wzorze
                        if not Jest:
                            dane.append("BR KWM$"+Tresc+" "+NrDost+" dla stawki "+Stawka)
                            return dane

                if not PierwszyPrzebieg:
                    Robic= False
                PierwszyPrzebieg=False

            #policzenie i wstawienie zaokraglen vatu
            for Stawka in self.SStawkiVAT:
                for pst in StVat:
                    if pst[0]==Stawka:
                        KwZaokr+=rsglobal.RSRound(pst[2]-self.SStawkiVAT[Stawka],2)

            #jeli kwota zaokgle != 0 to dodajemy pozycj
            if KwZaokr != 0:
                Insert+="INSERT INTO rs_dowody_ksiegowe_pozycje (firma_id,okres_id,dowod_id,nazwa_id,data,numer,nr_poz,nr_ppoz,nr_pozw,"
                Insert+="symbol_dok,data_wyst,data_wplywu,termin_plat,data_zapl,kontrahent_id,"
                Insert+="konto_ma,kwota_ma,konto_vat_ma,kwota_vat_ma,stawka_vat_ma,tresc_ma,waluta_ma,kurs_ma,kwota_wal_ma,"
                Insert+="nkup,konto_wn,kwota_wn,konto_vat_wn,kwota_vat_wn,stawka_vat_wn,tresc_wn,sposob_rozl,waluta_wn,kurs_wn,kwota_wal_wn,pnop,data_ksieg_kosztow)"
                Insert+=" VALUES "
                Insert+="('"+self.firma_id+"','"+OkresDekr+"','"+Dowod+"','"+Nazwa+"','"+Data+"',"+str(Numer)+","+str(NrPozDow)+",0,1,"
                Insert+="'"+NrFakt+"',"+DataWyst+",'"+DataWpl+"',"+TerminPlat+",NULL,'"+Kontr+"',"
                Insert+="'',0,'',0,NULL,'','"+Waluta+"',"+str(Kurs)+",0,"                                                                   #AB 04.10.18r
                Insert+="FALSE,'"+KontoZaokr+"',"+str(KwZaokr)+",'',0,NULL,'"+Tresc+"','','"+Waluta+"',"+str(Kurs)+","+str(KwZaokr)+",FALSE,'"+Data+"');"

        #zamknicie dokumentu dostawy
        Zam="UPDATE rs_dostawy SET stan='K' WHERE firma_id='"+self.firma_id+"' AND data='"+Data+"' AND numer="+NrD+";"

        stat,dump,dump,dump = self.PolSQL.Execute(InsN+Insert+Zam,1)

        if stat!= 'OK':
            self.PolSQL.Rollback()
            dane.append(stat)

        # OSTATECZNY ZAPIS DEKRETACJI
        if stat=='OK':
            self.PolSQL.Commit()

        return dane
    except Exception,e:
        rsglobal.ZapiszDoLogow('BD '+str(e),1)
        return ['Bd '+str(e)]

#-##############################################################################
#-- DEKRETACJA PAC
#-##############################################################################

def ObliczSumeWartosciSkladnikow(self,ListaSkladnikow,NumerListy,DataDo,Pracownik=None,Komorka=None):
    KomPracSQL = ""
    if Pracownik is not None:
        KomPracSQL=" AND s.pracownik_id='"+Pracownik+"'"
    elif Komorka is not None:
        KomPracSQL=" AND a.komorka_id='"+Komorka+"'"

    skladniki=ListaSkladnikow.replace(",","','")
    skladniki="'"+skladniki+"'"

    SQL ="SELECT SUM(wartosc)"
    SQL+=" FROM rs_kp_listy_plac_osoby_skladniki"
    SQL+=" JOIN ("
    SQL+=" SELECT s.firma_id, s.pracownik_id, s.zatrudnienie_id, s.umowa_id, s.angaz_id, s.numer_listy_id, s.skladnik_id, MAX(data_od) as data_od"
    SQL+=" FROM rs_kp_listy_plac_osoby_skladniki s"
    SQL+=" JOIN rs_kp_angaze a USING (firma_id, pracownik_id, zatrudnienie_id, umowa_id, angaz_id)"
    SQL+=" WHERE s.firma_id='"+self.firma_id+"' AND s.skladnik_id IN ("+skladniki+") AND s.numer_listy_id = '"+NumerListy+"' AND s.data_od <= '"+DataDo+"'"+KomPracSQL
    SQL+=" GROUP BY s.firma_id, s.pracownik_id, s.zatrudnienie_id, s.umowa_id, s.angaz_id, s.numer_listy_id, s.skladnik_id) AS skladniki"
    SQL+=" USING (firma_id, pracownik_id, zatrudnienie_id, umowa_id, angaz_id, numer_listy_id, skladnik_id, data_od);"
    stat,Wartosc,dump,dump = self.PolSQL.Execute(SQL,3,CzyDodawacBlad=True)
    if stat != 'OK':
        stat = 'Wystpiy bd podczas pobierania wartoci skadnikw listy pac'+stat
        return stat, []

    return stat, Wartosc[0][0]

#WS 2021-11-09 - dodany arg OkresDekr, podmiana self.okres_id na OkresDekr (dla harmonogramu)
def DekretacjaAutomatycznaPlac(self, Dowod, Nazwa, ListyPlac, OkresDekr, Warunek=None):
    dane = []
    stat = 'OK'  #WS 2021-11-09

    #Pobranie uprawnien do dokumentw ksigowych dla operatora
    UprTotal = True
    if self.operator_id == '0':
        UprDodawanie = True
        LDod=[]
    else:
        SQL="SELECT dowod_id,nazwa_id,wprowadzanie,edycja,usuwanie,przegladanie,zamykanie,otwieranie " \
            "FROM rs_wzory_dekretacji_upr WHERE firma_id='"+self.firma_id+"' AND operator_id in " \
            +str(self.operatorzy).replace("[","(").replace("]",")")+" AND okres_id='"+OkresDekr+"'; "
        stat, daneupr, dump, dump = self.PolSQL.Execute(SQL,3)
        if stat != 'OK':
            return 'BL SPR UPR', []
        else:
            if len(daneupr) > 0:
                UprTotal = False
            else:
                UprTotal = True
            LDod=[]

            UprDodawanie=(self.oddzial_id,'075013') in self.uprawnienia
            if UprDodawanie:
                for d in daneupr:
                    if d[2]:
                        LDod.append((rsglobal.Dekoduj(d[0]).strip(),rsglobal.Dekoduj(d[1]).strip()))

    if not UprDodawanie:
        return 'UPR DOD', []
    if not UprTotal:
        if (not(str(Dowod),Nazwa) in LDod) and self.operator_id != '0':
            return 'UPR DOD', []

    Zadekretowano = False

    try:
        #dla kadej listy pac tworzymy nowy dokument ksigowy
        for lista in ListyPlac:
            ZAM=''
            INS=''

            NrListyId   = lista[0]
            DataDo = lista[1]
            # WarLista    = lista[2].strip() #WS 2019-06-18 - lista_id (szablon)

            #pobranie nowego numeru dla dowodu ksigowego
            stat, Test, dump, dump = self.PolSQL.Execute("SELECT rsf_sprawdz_czy_mozna_dodac_dokument('"+self.firma_id+"','[''"+OkresDekr+"'',''"+Dowod+"'',''"+Nazwa+"'',''"+DataDo+"'']','D');",3)
            if stat != 'OK':
                return 'Bd sprawdzania moliwoci dodania dowodu ksigowego', []
            else:
                if Test[0][0] == 'OK':
                    stat,DNr,dump,dump = self.PolSQL.Execute("SELECT rsf_podaj_nowy_numer_dokumentu('"+self.firma_id+"','[''"+OkresDekr+"'',''"+Dowod+"'',''"+Nazwa+"'',''"+DataDo+"'']','D');",3)
                    if DNr[0][0] in (None,'None',''):
                        return "PO DOK", []
                    else:
                        NrDowodu=rsglobal.Dekoduj(DNr[0][0])
                else:
                    return "PO DOK", []

            if DataDo in (None,'None',''):
                dane.append("BR D WYP$"+lista[0])
                continue

            INS += "INSERT INTO rs_dowody_ksiegowe ("
            INS += "firma_id,okres_id,dowod_id,nazwa_id,data,numer,stan,symbol_dok"
            INS += ") VALUES ("
            INS += "'"+self.firma_id+"','"+OkresDekr+"','"+Dowod+"','"+Nazwa+"','"+DataDo+"',"+str(NrDowodu)+",'W','"+NrListyId+"');"

            SQL ="SELECT * FROM rs_wzory_dekretacji_plac"
            SQL += " WHERE firma_id = '"+self.firma_id+"' AND okres_id = '"+OkresDekr+"'"
            SQL += " AND dowod_id = '"+Dowod+"' AND nazwa_id = '"+Nazwa+"' ORDER BY nr_poz;"

            stat,WzorDekrPlacPozycje,wyn1,dump = self.PolSQL.Execute(SQL,3,CzyDodawacBlad=True)
            if stat != 'OK':
                return 'Wystpiy bd podczas pobierania pozycji wzoru dekretacji pac'+stat, []

            NrPoz = 1
            for pozycja in WzorDekrPlacPozycje:
                Strona = rsglobal.Dekoduj(pozycja[5]).strip()
                Konto = rsglobal.Dekoduj(pozycja[6]).strip()
                Znak = -1 if pozycja[9] else 1
                ListaSkladnikow = rsglobal.Dekoduj(pozycja[10]).strip()
                Tresc = rsglobal.Dekoduj(pozycja[14]).strip()

                if Strona == '':
                    dane.append('BR STR$ '+str(int(pozycja[4])))

                if Konto == '':
                    dane.append('BR K$ '+str(int(pozycja[4])))

                if len(dane) != 0:
                    continue

                poleStrona = 'wn' if Strona == 'W' else 'ma'

                #ani dzial ani pracownik
                if not pozycja[7] and not pozycja[8]:
                    stat, Kwota = ObliczSumeWartosciSkladnikow(self,ListaSkladnikow,NrListyId,DataDo)

                    if stat != 'OK':
                        return stat, []

                    if Kwota != 0:
                        INS+="INSERT INTO rs_dowody_ksiegowe_pozycje ("
                        INS+="firma_id,okres_id,dowod_id,nazwa_id,data,numer,"
                        INS+="nr_poz,nr_ppoz,symbol_dok,konto_"+poleStrona+",kwota_"+poleStrona+",tresc_"+poleStrona+","
                        INS+="waluta_"+poleStrona+",kwota_wal_"+poleStrona+",kurs_"+poleStrona
                        INS+=") VALUES ("
                        INS+="'"+self.firma_id+"','"+OkresDekr+"','"+Dowod+"','"+Nazwa+"','"+DataDo+"',"+str(NrDowodu)+","
                        INS+=str(NrPoz)+",0,'"+NrListyId+"','"+Konto+"',"+str(Znak*Kwota)+",'"+Tresc+"',"
                        INS+="'PLN',"+str(Znak*Kwota)+",1);"

                else:
                    polaSELECT=[]
                    polaSELECT.append("a.komorka_id"   if pozycja[7] else "NULL")
                    polaSELECT.append("s.pracownik_id" if pozycja[8] else "NULL")

                    #pobranie komorek/pracownikow dla kazdego numeru zatrudnienia, umowy i angazu
                    SQL ="SELECT DISTINCT "+",".join(polaSELECT)
                    SQL+=" FROM rs_kp_listy_plac_osoby_skladniki s"
                    SQL+=" JOIN rs_kp_angaze a"
                    SQL+=" USING (firma_id, zatrudnienie_id, umowa_id, angaz_id, pracownik_id)"
                    SQL+=" WHERE s.firma_id='"+self.firma_id+"' AND s.numer_listy_id='"+NrListyId+"' ORDER BY 1,2;"
                    stat,Podpozycje,dump,dump = self.PolSQL.Execute(SQL,3,CzyDodawacBlad=True)
                    if stat != 'OK':
                        return 'Wystpiy bdy podczas pobierania danych o skadnikach list pac'+stat, []

                    #utworzenie osobnych podpozycji dla komorek/pracownikow
                    NrPp = 1

                    for pp in Podpozycje:
                        KontoPp = Konto
                        if pozycja[7]:
                            KontoPp += "-"+pp[0].strip()
                        if pozycja[8]:
                            KontoPp += "-"+pp[1].strip()

                        if pozycja[8]:
                            stat, Kwota = ObliczSumeWartosciSkladnikow(self,ListaSkladnikow,NrListyId,DataDo,Pracownik=pp[1])
                        else:
                            stat, Kwota = ObliczSumeWartosciSkladnikow(self,ListaSkladnikow,NrListyId,DataDo,Komorka=pp[0])

                        if stat != 'OK':
                            return stat, []

                        if Kwota != 0:
                            INS+="INSERT INTO rs_dowody_ksiegowe_pozycje ("
                            INS+="firma_id,okres_id,dowod_id,nazwa_id,data,numer,"
                            INS+="nr_poz,nr_ppoz,symbol_dok,konto_"+poleStrona+",kwota_"+poleStrona+",tresc_"+poleStrona+","
                            INS +="waluta_" + poleStrona + ",kwota_wal_" + poleStrona + ",kurs_" + poleStrona
                            INS+=") VALUES ("
                            INS+="'"+self.firma_id+"','"+OkresDekr+"','"+Dowod+"','"+Nazwa+"','"+DataDo+"',"+str(NrDowodu)+","
                            INS+=str(NrPoz)+","+str(NrPp)+",'"+NrListyId+"','"+KontoPp+"',"+str(Znak*Kwota)+",'"+Tresc+"',"
                            INS += "'PLN'," + str(Znak * Kwota) + ",1);"

                            NrPp += 1

                NrPoz += 1

            #zamknicie list pac
            ZAM+="UPDATE rs_kp_listy_plac SET stan='K' WHERE firma_id='"+self.firma_id+"' AND numer_listy_id='"+NrListyId+"';"

            if stat == 'OK' and len(dane) == 0:
                stat,dump,dump,dump = self.PolSQL.Execute(INS+ZAM,1,CzyDodawacBlad=True)
                if stat != 'OK':
                    self.PolSQL.Rollback()
                    return 'Wystpiy bd podczas dodawania dowodu ksigowego: '+stat, []
                else:
                    Zadekretowano = True
                    self.PolSQL.Commit()
            else:
                self.PolSQL.Rollback()
                return stat, dane

        if stat == 'OK':
            if not Zadekretowano:
                return 'BR DOK', []
            else:
                # OSTATECZNY ZAPIS DEKRETACJI
                self.PolSQL.Commit()
                return stat, []
    except Exception,e:
        self.PolSQL.Rollback()
        rsglobal.ZapiszDoLogow('BD '+str(e),1)
        return 'Bd '+str(e), []
