# -*- coding: iso8859_2 -*-
#-------------------------------------------------------------------------------
# Name:        fs_dokumenty_magazynowe_import_z_pliku_ttx.py
# Purpose:     obsluga importu dokumentow z pliku ttx
# Author:      Mariusz Rajfura
# Created:     2013/04/18
# RCS-ID:      $Id: fs_dokumenty_magazynowe_import_z_pliku_ttx.py $
# Copyright:   (c) 2013 Record System
# Licence:     GNU GPL
#-------------------------------------------------------------------------------

# 2013-05-08 Mariusz Rajfura (6853) import innych pol: ceny,uwagi,opisy pozycji
# 2013-06-04 Mariusz Rajfura (6918) import zdpw: kategoria 'R' dla dodawanych artykulow
# 2014-02-17 Marcin Natkaski (7613) import dugiej nazwy artykulu

import datetime

#-------------------------------------------------------------------------------

Slownik = {}

#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
# Funkcja podaje podaje wartosc pola nazywanego 'kluczem obcym' ze sparsowanego dokumentu
# Dla rozprawienia sie z roznymi kodowaniami, klucz jest wyszukiwany na podstawie poczatku tekstu
# i/lub konca tekstu w kluczu. To pozwala dokladnie wybrac klucz. W wielu przypadkach
# wystarczy parametr ZaczynaSie, ktory moze byc calym kluczem np 'Magazyn:'
#
# W przyszlosci mozna dodac parametr Zawiera='' lub Zawiera=[]
# Identyczna funkcja jest po stronie klienta.

def PodajKluczObcyZeSlownika(KluczeObce={}, ZaczynaSie='', KonczySie=''):
    jest = False
    wynik = None
    
    for k in KluczeObce:
        if ZaczynaSie <> '' and KonczySie <> '':
            if k.startswith(ZaczynaSie) and k.endswith(KonczySie):
                jest = True
                wynik = KluczeObce[k]
                break
            
        elif ZaczynaSie <> '':
            if k.startswith(ZaczynaSie):
                jest = True
                wynik = KluczeObce[k]
                break
            
        elif KonczySie <> '':
            if k.endswith(KonczySie):
                jest = True
                wynik = KluczeObce[k]
                break
    
    return jest, wynik

#-------------------------------------------------------------------------------



#-------------------------------------------------------------------------------

def _CzyscLinieTTX(linia):
    # Funkcja wewnetrzna, czysci linie tekstu ze znakow nowej linii i powrotu
    # i rozkladajaca na liste pol, ktore byly oddzielone znakiem tabulacji
    wynik = linia.replace('\r','').replace('\n','').replace('"','')
    wynik = wynik.split('\t')
    return wynik

#-------------------------------------------------------------------------------

def _DodajBladTTX(OpisBledu):
    global Slownik
    # Funkcja wewnetrzna, tworzy w ogolnym slowniku informacji dotyczacych analizy 
    # dokumentu liste bledow i dopisuje podany blad.
    if not '__rs_bledy__' in Slownik:
        Slownik['__rs_bledy__'] = []
        
    Slownik['__rs_bledy__'].append(OpisBledu)
    
#-------------------------------------------------------------------------------

def ParsujTTX(Marker, Linie):
    global Slownik, _CzyscLinieTTX, _DodajBladTTX
    Slownik = {}
    # Funkcja analizujaca plik TTX i zwracajaca slownik zawierajacy
    # odczytanie dane. Jesli udalo sie odczytac wszystkie kluczowe informacje,
    # to stat == 'OK'. W innym przypadku stat zawiera komunikat a slownik
    # w kluczu __rs_bledy__ zawiera liste bledow
    #
    # Klucze naszego slownika:
    # __rs_bledy__          Lista bledow jesli stat <> 'OK'
    # __rs_marker__         String identyfikujacy dany dokument, musi byc w tresci
    # __rs_data_dokumentu__ Odczytana data dokumentu zrodlowego, jest przed markerem
    # __rs_nr_dokumentu__   Odczytany nr dokumentu zrodlowego, powinien byc za markerem
    # __rs_kolumny__        Lista nazw kolumn z dokumentu
    # __rs_pozycje__        Lista list zawierajaca pozycje dokumentu w typowym 
    #                       ukladzie wierszy z kolumnami, zgodna z iloscia kolumn
    #                       zapisana w __rs_kolumny__
    #                       [[pole1,pole2,pole3]...]
    #                       [[pole1,pole2,pole3]...]
    # __rs_klucze_obce__    Slownik znalezionych w dokumencie slow kluczowych
    #                       wraz z wartosciami. Slowo kluczowe jest zakonczone
    #                       dwukropkiem i bylo w ukladzie "klucz:" \t "wartosc"
     
    
    # Opis pozycji na kazdym dokumencie zaczyna sie od n/w kolumn.
    # Co wazne, to czyste ASCII i to ulatwia wyszukiwanie.
    ListaKolumnDef = ['LP','Symbol','Opis','JM']
    
    KluczeObce = {}
    wynik = False
    JestMarker = -1
    for i in xrange(len(Linie)):
        l = Linie[i]
        
        if Marker in l:
            # Plik zawiera marker. Mozna dalej analizowac.
            JestMarker = i
            break
    
    if JestMarker == -1:
        # Plik nie zawieral markera. Koniec dziala funkcji.
        wynik = 'Brak znacznika dokumentu '+Marker
        _DodajBladTTX(wynik)
        return wynik,Slownik
    
    
    # Data dokumentu zrodlowego znaduje sie przed markerem.
    for i in xrange(JestMarker-1,-1,-1):
        l = _CzyscLinieTTX(Linie[i])
        
        if len(l) > 0:
            if l[0].startswith('z dnia'):
                data_dokumentu = l[0][-10:]
                # zmiana formatu
                data_dokumentu = data_dokumentu[6:]+'-'+data_dokumentu[3:5]+'-'+data_dokumentu[:2]

                if rsglobal.SprawdzCzyToData(data_dokumentu):
                    Slownik['__rs_data_dokumentu__'] = data_dokumentu
                break
    
    # Dalsza analiza dokumentu zaczyna sie od linii z markerem.
    PierwszaPozycja = -1
    JestLiniaKolumn = False
    
    for i in xrange(JestMarker,len(Linie)):
        l = _CzyscLinieTTX(Linie[i])
        
        if i == JestMarker:
            Slownik['__rs_marker__'] = Marker
            if len(l) > 1:
                Slownik['__rs_nr_dokumentu__'] = l[1]
        
        for p in xrange(len(l)):
            # Przegladamy zawartosc linii

            # Tu wylawiamy kluczowe pola naglowe, stopkowe, ktore sa zakonczone
            # dwukropkiem i za nimi wystepuje nastepne pole.
            # Nazwa klucza jest pobierana z dokumentu
            if l[p].endswith(':') and p<len(l)-1:
                KluczeObce[l[p]] = l[p+1]
            
            if not JestLiniaKolumn:
                # Poszukujemy linii opisujacej kolumny.
                # Linia zawierajaca opisy kolumn ma charakterysteczny uklad,
                # zaczynajacy sie od listy opisow zdefiniowanej na poczatku funkcji.

                if len(l[p]) >= len(ListaKolumnDef):
                    # Sprawdzamy, czy uklad pol jest zgodny z poczatkiem
                    # opisu listy kolumn na dokumentach.
                    
                    ToPoczatekListyKolumn = True
                    for x in xrange(len(ListaKolumnDef)):
                        if l[x] <> ListaKolumnDef[x]:
                            ToPoczatekListyKolumn = False
                            break
                    
                    if ToPoczatekListyKolumn:
                        # To jest startowa linia opisu kolumn na dokumencie
                        LiniaKolumn = l
    
                        # Dalsze przeszukiwanie i dopelnianie linii opisu kolumn.
                        # Dokladamy kolejne opisy kolumn, az trafimy na numer pierwszej
                        # pozycji dokumentu lub osiagniemy koniec tekstu.

                        for jj in xrange(i+1,len(Linie)):
                            ll = _CzyscLinieTTX(Linie[jj])

                            if ll == ['']:
                                # Pusta linia. W tech cwili nic nie znaczy.
                                continue
                            
                            elif ll[0] == '1':
                                # Trafilismy na pierwsza pozycje w dokumencie.
                                # To na pewno oznacza, ze opis kolumn skonczyl sie.
                                PierwszaPozycja = jj
                                break
                            
                            else:
                                # Linia zawiera dalsze nazwy kolumn
                                LiniaKolumn += ll
                                
                        Slownik['__rs_kolumny__'] = LiniaKolumn
                        JestLiniaKolumn = True

    
    if PierwszaPozycja > -1 and '__rs_kolumny__' in Slownik:
        # Mamy uklad kolumn i znamy lokalizacje pierwszej pozycji
        # wiec odczytujemy pozycje dokumentu
        # Nie zakladam, ze zawsze 2 linie opisuja pozycje dokumentu
        Pozycje = []
        
        LiniaKolumn = Slownik['__rs_kolumny__']
        
        LiniaPozycji = []
        LicznikPustych = 0
        for i in xrange(PierwszaPozycja, len(Linie)):
            l = _CzyscLinieTTX(Linie[i])
            if l == ['']:
                LicznikPustych += 1
                if LicznikPustych > 2:
                    # To juz raczej koniec dokumentu, bo kolejne dwie linie sa puste
                    break
                else:
                    # po nastepna linie
                    continue
            elif l[0].startswith("Razem:"):
                # Doszlismy do konca dokumentu posiadajacego podsumowanie :-/
                break
            
            else:
                # To nie jest pusta linia, wiec dokladamy ja do poprzednich
                LicznikPustych = 0    
                LiniaPozycji += l
        
        if len(LiniaPozycji) > 0:
            # Linia zawierajaca pozycje powinna byc wielokrotnoscia linii kolumn
            if len(LiniaPozycji) % len(LiniaKolumn) == 0:
                
                # Tworzymy liste list, ktora przypomina struktura "nasze" dane[[]]
                for i in xrange(len(LiniaPozycji) / len(LiniaKolumn)):
                    Poz = []
                    for j in xrange(len(LiniaKolumn)):
                        Poz.append(LiniaPozycji[i*len(LiniaKolumn)+j])
                
                    # Dodajemy kolejny wiersz z pozycja
                    Pozycje.append(Poz)
            
        # Dopisanie odczytanych pozycji
        if len(Pozycje) > 0:
            Slownik['__rs_pozycje__'] = Pozycje
    
    # Prawidlowo odczytany dokument ma zdefiniowane podstawowe klucze,
    # ktore pozwola utworzyc dokument w anakondzie
    if '__rs_nr_dokumentu__' in Slownik and '__rs_kolumny__' in Slownik and \
        '__rs_pozycje__' in Slownik and '__rs_data_dokumentu__' in Slownik:

        # Dodajewy wykryte klucze obce wraz z wartosciami.
        Slownik['__rs_klucze_obce__'] = KluczeObce
        wynik = 'OK'
    
    else:
        wynik = 'Brak kompletu kluczowych informacji.'
        _DodajBladTTX(wynik)

        if not '__rs_nr_dokumentu__' in Slownik:
            _DodajBladTTX('Brak numeru dokumentu po znaczniku '+Marker)

        if not '__rs_data_dokumentu__' in Slownik:
            _DodajBladTTX('Brak daty dokumentu przed znacznikiem '+Marker)

        if not '__rs_kolumny__' in Slownik:
            _DodajBladTTX('Brak opisu kolumn zaczynajacego sie polami'+str(ListaKolumnDef))

        if not '__rs_pozycje__' in Slownik:
            _DodajBladTTX('Brak pozycji na dokumencie')
    
    return wynik,Slownik

#-------------------------------------------------------------------------------

dane = []
wyn1 = []
wyn2 = []
stat = 'OK'

if not 'Funkcja' in klucze:
    stat='Brak podanej funkcji w kluczach'

if stat=='OK':
    Fun = klucze['Funkcja']
    
    # 1 - parsowanie pliku do slownika w celu pobrania danych wstepnych
    # 2 - dodawanie dokumentu na postawie slownika sparsowanego pliku
    
    SlPar={'Marker':[1,2], 'TrescPliku':[1,2], 'Slownik':[2],
            'Magazyn':[2], 'Dokument':[2], 'Data':[2],'Numer':[2]}
    Lista=rsglobal.SprawdzParametryWKluczach(SlPar,Fun,klucze)
    if Lista<>'':
        stat='Uwaga : wywoanie funkcji fs_dokumenty_magazynowe_import_z_pliku_ttx z numerem '+\
             str(Fun)+' bez parametrw: '+Lista

if stat == 'OK':
    if Fun in (1,2):
        TrescPliku = rsglobal.Dekoduj(klucze['TrescPliku'])
        Marker = "["+rsglobal.Dekoduj(klucze['Marker'])+"]"
        
    if Fun == 2:    
        Slownik = rsglobal.Dekoduj(klucze['Slownik'])
        Magazyn = rsglobal.Dekoduj(klucze['Magazyn'])
        Dokument = rsglobal.Dekoduj(klucze['Dokument'])
        Data = rsglobal.Dekoduj(klucze['Data'])
        Numer = rsglobal.Dekoduj(klucze['Numer'])


#-------------------------------------------------------------------------------
# PARSOWANIE PLIKU (1)
#-------------------------------------------------------------------------------
        
if Fun == 1 and stat == 'OK':
    stat,slownik = ParsujTTX(Marker, TrescPliku)
    dane.append(slownik)
    
#-------------------------------------------------------------------------------
# TWORZENIE DOKUMENTU NA PODSTAWIE SPARSOWANEGO PLIKU (2)
#-------------------------------------------------------------------------------

if Fun == 2 and stat == 'OK':
    # Znacznik usuniecia nowo utworzonego dokumentu jesli wystapi jakis blad
    # jesli mimo to, nie uda sie usunac tego dokumentu, to do okna zostanie
    # zwrocony w liscie dane[] komunikat 'WYKONAJ_DELETE'
    UsunDokument = True

    # Opisowa informacja bledow
    Bledy = ''
    # sql dodajacy nowe artykuly
    INSART = ''
    # sql dodajacy pozycje dokumentu
    INSPOZ = ''
    # sql modyfikujcy naglowek
    UPDNAG = ''
    
    # Zablokowanie nowo utworzonego dokumentu do wpisania importowanych danych
    SQLFU ="SELECT * FROM rs_dokumenty_magazynowe"
    SQLFU+=" WHERE firma_id='"+self.firma_id+"' AND magazyn_id='"+Magazyn+"'"
    SQLFU+=" AND dokument_id='"+Dokument+"' AND data='"+Data+"' AND numer="+str(Numer)
    SQLFU+=" FOR UPDATE;"
    
    stat,d,dump,dump = self.PolSQL.Execute(SQLFU,3)
    if stat <> 'OK':
        stat = 'Dokument jest zablokowany. Dostp niemoliwy'
    elif len(d) == 0:
        stat = 'Brak dokumentu o podanym kluczu'
        

    # Sprawdzenie, czy na importowanym dokumencie nie ma juz pozycji
    # Byc nie powinno ale ...
    if stat == 'OK':
        sql ="SELECT * FROM rs_dokumenty_magazynowe_pozycje"
        sql+=" WHERE firma_id='"+self.firma_id+"' AND magazyn_id='"+Magazyn+"'"
        sql+=" AND dokument_id='"+Dokument+"' AND data='"+Data+"' AND numer="+str(Numer)+";"
        
        stat,d,dump,dump = self.PolSQL.Execute(sql,3)
        if stat <> 'OK':
            stat = 'Bd sprawdzenie pozycji dokumentu przed importem'
        elif len(d) > 0:
            # To troche dziwne zdarzenie ale w tym przypadku nie usuwamy tego dokumentu.
            # Niech operator sam go usunie.
            stat = 'Dokument ju zawiera pozycje. Import niemoliwy.'
            UsunDokument = False

    if stat == 'OK':
        # dane importowanego dokumentu
        OryginalnyNrDokumentu = Slownik['__rs_nr_dokumentu__']
        OryginalnaDataDokumentu = Slownik['__rs_data_dokumentu__']
        KluczeObce = Slownik['__rs_klucze_obce__']
        Kolumny = Slownik['__rs_kolumny__']
        Pozycje = Slownik['__rs_pozycje__']
        
        # Wymagane pola uzywane przy zakladaniu nowych artykulow
        KategArtykulu = ' '
        StawkaVAT = '23'
        # W polu list_przewozowy umieszczamy pole Logo (czyli kontrahenta)
        ListPrzewozowy = ''
        jest,wynik = PodajKluczObcyZeSlownika(KluczeObce, ZaczynaSie='Logo:')
        if jest:
            ListPrzewozowy = wynik
            
        if Marker == '[ZDPW]':
            # produkt
            TypArtykulu = '02'
            KolUwagi = 5
            KategArtykulu = 'R'
            
        elif Marker == '[RWSO]':
            # surowiec
            TypArtykulu = '04'
            KolUwagi = 6
            KolWytop = 7
            KolCenaZakupu = 8
            
            # W przypadku RW sprawdzenie, czy wybrany magazyn stosuje ceny
            sql ="SELECT czy_ceny FROM rs_magazyny"
            sql+=" WHERE firma_id='"+self.firma_id+"' AND magazyn_id='"+Magazyn+"';"
            stat,d,dump,dump = self.PolSQL.Execute(sql,3)
            if stat <> 'OK':
                stat = 'Bd sprawdzenia ustawie magazynu: '+stat
            elif not d[0][0]:
                stat = 'W magazynie '+Magazyn.strip()+' nie stosuje si cen, a dokument ma mie ceny.\nImport niemoliwy.'
            
        else:
            stat = 'Nieznany znacznik dokumentu '+Marker
            
    if stat == 'OK':
        # Sprawdzenie, czy jest w slowniku domyslnie przydzielana kategoria artykulu
        sql ="SELECT * FROM rs_kateg_artykulow"
        sql+=" WHERE firma_id='"+self.firma_id+"' AND kod='"+KategArtykulu+"';"
        stat,d,dump,dump = self.PolSQL.Execute(sql,3)
        
        if stat <> 'OK':
            stat = 'Bd sprawdzenia kategorii artykuu dla importu nowych artykuw: '+stat
            
        elif len(d) == 0:
            stat = "Brak kategorii artykuw '"+KategArtykulu+"' w sowniku dla importu nowych artykuw."

    if stat == 'OK':
        # Numery kolumn w w dokumencie sparsowanym funkcja ParsujTTX().
        # Oprocz n/w wczesniej sa zdefiniowane kolumny zalezne od typu dokumentu.
        KolLP = 0
        KolArtykul = 1
        KolNazwa = 2
        KolJM = 3
        KolIlosc = 4
        
        # Slownik dodawanych artykulow zabezpiecza nas przed wielokrotnym dodawaniem
        # tego samego artykulu w przypadku kolejnego wystapienia w importowanym dokumencie
        DodaneArtykuly = {}
        
        for i in xrange(len(Pozycje)):
            try:
                NrPoz = int(Pozycje[i][KolLP])
                NrPPoz = 0
                Artykul = rsglobal.Dekoduj(Pozycje[i][KolArtykul])
                Nazwa = rsglobal.Dekoduj(Pozycje[i][KolNazwa])
                JM = rsglobal.Dekoduj(Pozycje[i][KolJM])
                Ilosc = float(Pozycje[i][KolIlosc])
                Opis = rsglobal.Dekoduj(Pozycje[i][KolUwagi])
                
                # Pozycje specyficzne dla typu dokumentu
                if Marker == '[RWSO]':
                    Cena = float(Pozycje[i][KolCenaZakupu])
                    NrSerii = rsglobal.Dekoduj(Pozycje[i][KolWytop])
                
            except Exception, e:
                stat = 'Bd danych w pozycji '+str(i)+'  '+str(e)
                break
            
            # Kontrola prawidlowosci numeracji pozycji
            if NrPoz <> i+1:
                Bledy += 'Bdny LP dla pozycji nr '+str(i+1)+'\n'
                
            # Kontrola jednostki miary
            if stat == 'OK':
                sql ="SELECT * FROM rs_jednostki_miar"
                sql+=" WHERE firma_id='"+self.firma_id+"' AND kod='"+JM+"';"
                stat,d,dump,dump = self.PolSQL.Execute(sql,3)
                if stat <> 'OK':
                    stat = 'Bd sprawdzenia jednostki miary: '+stat
                    break
                elif len(d) == 0:
                    Bledy += 'Brak w sowniku jednostki miary '+JM+'\n'
            
            # Sprawdzenie istnienia artykulu. W przypadku braku zakladanie go.'
            if stat == 'OK':
                sql ="SELECT * FROM rs_artykuly WHERE firma_id='"+self.firma_id+"'"
                sql+=" AND artykul_id='"+Artykul+"';"
        
                stat,d,dump,dump = self.PolSQL.Execute(sql,3,CzyDodawacBlad=True)
                if stat <> 'OK':
                    stat = 'Bd sprawdzenia artykuu: '+stat
                    break
                
                elif len(d) == 0:
                    if not Artykul in DodaneArtykuly:
                        DodaneArtykuly[Artykul] = True
                        
                        # Dodajemy nowy artykul
                        INSART+="INSERT INTO rs_artykuly"
                        INSART+=" (firma_id,artykul_id,nazwa,jm1,jm2,"
                        INSART+=" typ_artykulu,kateg_artykulu,stawka_vat,nazwa_dluga)"
                        INSART+=" VALUES"
                        INSART+=" ('"+self.firma_id+"','"+Artykul+"','"+Nazwa[:50]+"','"+JM+"','"+JM+"',"
                        INSART+=" '"+TypArtykulu+"','"+KategArtykulu+"','"+StawkaVAT+"','"+Nazwa+"'); "
                        INSART+="INSERT INTO rs_artykuly_stawki_vat"
                        INSART+=" (firma_id,artykul_id,od_dnia,stawka_vat) "
                        INSART+=" VALUES"
                        INSART+=" ('"+self.firma_id+"','"+Artykul+"','"+Data+"','"+StawkaVAT+"'); "
                
                # Dodajemy pozycje
                INSPOZ+="INSERT INTO rs_dokumenty_magazynowe_pozycje"
                INSPOZ+=" (firma_id,magazyn_id,dokument_id,data,numer,"
                INSPOZ+=" nr_poz,nr_ppoz,artykul_id,ilosc1,ilosc2,opis"

                if Marker == '[RWSO]':
                    INSPOZ += ',nr_serii,cena1,cena2,cena_wal1,cena_wal2'
                
                INSPOZ+=")"
                INSPOZ+=" VALUES"
                INSPOZ+=" ('"+self.firma_id+"','"+Magazyn+"','"+Dokument+"','"+Data+"',"+str(Numer)+","
                INSPOZ+=" "+str(NrPoz)+","+str(NrPPoz)+",'"+Artykul+"',"+str(Ilosc)+","+str(Ilosc)+",'"+Opis+"'"

                if Marker == '[RWSO]':
                    INSPOZ += ",'"+NrSerii+"',"+str(Cena)+","+str(Cena)+","+str(Cena)+","+str(Cena)

                INSPOZ+="); "

    if stat == 'OK' and INSPOZ <> '':
        # Zapisujemy w naglowku dane importowanego dokumentu
        UPDNAG ="UPDATE rs_dokumenty_magazynowe"
        UPDNAG+=" SET nr_faktury='"+OryginalnyNrDokumentu+" z dnia "+OryginalnaDataDokumentu+"'"
        if ListPrzewozowy <> '':
            UPDNAG+=",list_przewozowy='"+ListPrzewozowy+"'"
            
        UPDNAG+=" WHERE firma_id='"+self.firma_id+"' AND magazyn_id='"+Magazyn+"'"
        UPDNAG+=" AND dokument_id='"+Dokument+"' AND data='"+Data+"' AND numer="+str(Numer)+"; "

    if stat == 'OK':
        # Czy byly zwykle(a nie systemowe) bledy w importowanym dokumencie
        if len(Bledy) > 0:
            wyn2 = Bledy
            stat = 'LISTA_BLEDOW'
    
    if stat == 'OK':
        SQL = INSART+INSPOZ+UPDNAG
        if SQL <> '':
            stat,dump,dump,dump = self.PolSQL.Execute(SQL,1,CzyDodawacBlad=True)
            if stat == 'OK':
                # Import sie udal. Nie trzeba usuwac dokumentu.
                UsunDokument = False
            
    if UsunDokument:
        # Komunikat o koniecznosci wykonania usuniecia dokumentu, bo teraz
        # cos sie nie udalo i jest wykonywany ROLLBACK wszystkich operacji
        dane.append('WYKONAJ_DELETE')
