# -*- coding: ISO8859-2 -*-
# -------------------------------------------------------------------------------
# Name:             fs_dokumenty_magazynowe_pom.py
# Purpose:          proste funkcje testujce w dokumentach magazynowych
# Author:           Micha Raburski
# Created:          2010/10/29
# Copyright:        (c) 2010 RecordSystem
# Licence:          GNU GPL
# -------------------------------------------------------------------------------
# 2017-10-31 Rafa Woniak      (10734) zmiana zapisu do logw z 1 na 5
# 2020-03-21 Jan Raburski       (11876) funkcje obslugujace wydanie z magazynow detalicznych
# 2023-01-14 Jan Raburski       (13286) przeliczanie zamknietych d.m. w magazynach detalicznych
# 2023-03-31 Jan Raburski       (13393) pomijanie w przeliczaniu d.m., ktore nie chca sie otworzyc
# 2023-11-27 Aleksander Bargie (13618) Rozliczenie produkcji placwki handlowej: current transaction is aborted,
#                                       commands ignored until end of transaction block
# 2024-01-31 Wojciech Skowron   (13629) Pobieranie receptur dla oddziau pobranego z magazynu PW zamiast self.oddzial_id

from funkcje import fs_magazyny_parametry_dokumentow_pom
import rsglobal
import time


#-##############################################################################
#--- FUNKCJA PRZELICZAJACA ZAMKNIETE DOKUMENTY ROZCHODOWE W MAG.DETALICZNYM
#-##############################################################################

def HDPrzeliczWydania(self, Magazyn, DokId, Data, Numer):
    # Dzialanie:
    # 1. Sprawdza czy na zamknietych dokumentach rozchodowych sa artykuly nie pobrane z artykulow przyjec
    # 2. Robi dla nich update na cene i wartosc w pozycjach d.m. i obrotach

    # pobranie pozycji dokumentu przychodowego
    #SQL = " SELECT nrpoz, nrppoz, artykul_id, ilosc1, ilosc2, cena1, cena2, termin_przyd, nr_serii"
    SQL = " SELECT DISTINCT artykul_id"
    SQL += " FROM rs_dokumenty_magazynowe_pozycje "
    SQL += " WHERE firma_id='" + self.firma_id + "' AND magazyn_id='" + Magazyn +"' AND dokument_id='" + DokId + "'"
    SQL += " AND numer=" + str(Numer) + " AND data='" + Data + "'"
    stat, pozycjepz, dump, dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        stat = 'Bd pobrania pozycji dokumentu przychodowego: '+stat
        return stat
    if len(pozycjepz) == 0:
        return u"Brak pozycji dokumentu przychodowego"
    lartid = '('
    for w in pozycjepz:
        lartid += "'" + w[0] + "',"
    lartid = lartid[:-1] + ")"

    # pobranie listy dokumentow rozchodowych
    SQL = " SELECT dokument_id"
    SQL += " FROM rs_typy_dokumentow_m "
    SQL += " WHERE firma_id='" + self.firma_id + "' AND typ_dokumentu IN ('W')" #,'M')"
    stat, dokumentyid, dump, dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        stat = 'Bd pobrania listy dokumentw rozchodowych: '+stat
        return stat
    if len(dokumentyid) == 0:
        return u"Brak listy dokumentw rozchodowych"
    ldokid = '('
    for w in dokumentyid:
        ldokid += "'" + w[0] + "',"
    ldokid = ldokid[:-1] + ")"

    # pobranie listy dokumentow do ewentualnej zmiany
    SQL = "SELECT DISTINCT dm.dokument_id, dm.data, dm.numer "
    SQL += " FROM rs_dokumenty_magazynowe_pozycje dmp"
    SQL += " INNER JOIN rs_dokumenty_magazynowe dm USING (firma_id,magazyn_id,dokument_id, data, numer)"
    SQL += " WHERE dmp.firma_id='" + self.firma_id + "' AND dmp.dokument_id IN " + ldokid + " AND dmp.magazyn_id='" + Magazyn + "'"
    SQL += " AND dmp.artykul_id IN " + lartid + " AND dmp.data>='" + str(Data) +"'"
    SQL += " AND COALESCE(dokument_przych,'')='' AND dm.stan='Z'"
    # dokumenty tylko bez korekt
    SQL += " AND NOT EXISTS (SELECT 1 FROM rs_dokumenty_magazynowe dmk WHERE dmk.firma_id=dm.firma_id AND dmk.kor_dm_magazyn_id=dm.magazyn_id"
    SQL += " AND dmk.kor_dm_dokument_id=dm.dokument_id AND dmk.kor_dm_data=dm.data AND dmk.kor_dm_numer=dm.numer)"
    SQL += " ORDER BY dm.data"
    rsglobal.ZapiszDoLogow('szukanie wz '+SQL)
    stat, ldozmiany, dump, dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        stat = 'Bd szukania dokumentw do zmiany: '+stat
        return stat
    if len(ldozmiany) == 0:
        return 'OK'

    Klucze = {}
    Klucze['Mag'] = Magazyn
    for dok in ldozmiany:
        # otwarcie dokumentu rozchodowego z Z na W
        Klucze['Typ'] = dok[0]
        Klucze['Data'] = dok[1]
        Klucze['Numer'] = dok[2]
        Klucze['Funkcja'] = 21
        stat, dump, dump, dump = self.WykonajFunkcje("fs_dokumenty_magazynowe", Klucze, [], [], False)
        #if stat != 'OK':  #JR 2023-03-31
        #    return str(u'Bd otwarcia dokumentu ' + Magazyn + '/' + dok[0] + '/' + dok[1] + '/' + str(dok[2]) + ' z Z na W: ' + stat)
        if stat == 'OK':
            # zamknicie dokumentu rozchodowego z W na Z
            Klucze['Funkcja'] = 18
            stat, dump, dump, dump = self.WykonajFunkcje("fs_dokumenty_magazynowe", Klucze, [], [], False)
            if stat != 'OK':
                return str(u'Bd zamknicia dokumentu ' + Magazyn + '/' + dok[0] + '/' + dok[1] + '/' + str(dok[2]) + ' z W na Z: ' + stat)
        stat = 'OK'
    return stat

#-##############################################################################
#--- FUNKCJA WYDAJACA JEDNA POZYCJE Z DOKUMENTU DETAL
#-##############################################################################

def HDWydajArtykulZPrzyjec(self, Artykul, Magazyn, Data, Ilosc1, DaneArtykulu, Najnowsze, DokumentPrzych):
    # DaneArtykulu: zamiennik_id, przelicznik, dynamiczny_przelicznik, jm1.calkowita, jm2.calkowita
    # wraca ze statem (OK lub blad), lista pozycji i iloscia pozostala do wydania lub zerem
    #  Lista: Artykul , Ilosc1, Ilosc2, Cena1, Cena2, Dokument przychodowy, Nr komunikatu - 0 poprawnie (tylko)
    # Dzialanie:
    # 1. Probuje wydac artykul "normalnie" nie zwraca uwagi na cechy, lokacje, terminy przydatnosci itd.
    # 2. Robi update na artykuly, artykuly magazyny i artykuly przyjecia
    Wydania = []
    stat = 'OK'
    if Ilosc1 == 0:
        return stat, Wydania, 0
    przelicznik = DaneArtykulu[0][1]
    calkowita1 = DaneArtykulu[0][3]
    calkowita2 = DaneArtykulu[0][2]
    SQL = " SELECT data, dokument_id, numer, nr_poz, nr_ppoz, ilosc1 - rezerwacja1, ilosc2 - rezerwacja2, rezerwacja1, rezerwacja2,"
    SQL += " cena1, cena2  FROM rs_artykuly_przyjecia "
    SQL += " WHERE not wirtualne and firma_id='" + self.firma_id + "' AND magazyn_id='" + Magazyn +"'"
    SQL += " AND artykul_id='" + Artykul + "' AND data<='" + Data + "' AND ilosc1 - rezerwacja1 >0 "
    if len(DokumentPrzych.strip()) == 30:  # wskazany konkretny dokument
        if DokumentPrzych[22:26] != '0000':  # sprawdzenie czy nie ze spisu
            #SQL += " AND firma_id='" + DokumentPrzych[0:2] + "' AND magazyn_id='" + DokumentPrzych[2:6] + "'"
            SQL += " AND data='" + DokumentPrzych[8:18] + "' AND dokument_id='" + DokumentPrzych[6:8] + "'"
            SQL += " AND numer=" + DokumentPrzych[18:22] + " AND nr_poz=" + DokumentPrzych[22:26] + " AND nr_ppoz=" + DokumentPrzych[26:]
    if Najnowsze:
        SQL += " ORDER BY data DESC FOR UPDATE NOWAIT;"
    else:
        SQL += " ORDER BY data FOR UPDATE NOWAIT;"
    rsglobal.ZapiszDoLogow(SQL)
    stat, przyjecia, dump, dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        stat = 'Bd pobrania przyj: '+stat
        return stat, Wydania, 0
    UPD = ""
    dozdjecia1 = 0
    dozdjecia2 = 0
    for przyjecie in przyjecia:
        if przyjecie[5] >= Ilosc1:
            # calosc / reszta wydania z jednej pozycji
            dozdjecia1 += Ilosc1
            if calkowita2:
                dozdjecia2 += rsglobal.RSRound(Ilosc1 * przelicznik,0)
            else:
                dozdjecia2 += rsglobal.RSRound(Ilosc1 * przelicznik, 3)
            ilo1 = Ilosc1
            if calkowita1:
                ilo1 = rsglobal.RSRound(ilo1, 0)
            ilo2 = rsglobal.RSRound(Ilosc1 * przelicznik, 3)
            if ilo1 == przyjecie[5]:
                ilo2 = przyjecie[6]
            Ilosc1 = 0
        else:
            # calkowite wydanie z pozycji przyjecia
            dozdjecia1 += przyjecie[5]
            dozdjecia2 += przyjecie[6]
            ilo1 = przyjecie[5]
            ilo2 = przyjecie[6]
            Ilosc1 -= przyjecie[5]
        UPD += "UPDATE rs_artykuly_przyjecia SET ilosc1=ilosc1 - " + str(ilo1) + ", ilosc2=ilosc2 - " + str(ilo2)
        UPD += " WHERE firma_id='" + self.firma_id + "' AND magazyn_id='" + Magazyn + "'"
        UPD += " AND artykul_id='" + Artykul + "' AND data='" + str(przyjecie[0]) + "' AND dokument_id='" + przyjecie[1] + "'"
        UPD += " AND numer=" + str(przyjecie[2]) + " AND nr_poz=" + str(przyjecie[3]) + " AND nr_ppoz=" + str(przyjecie[4]) + ";"
        magtmp = Magazyn + '   '
        numer = str(10000 + int(przyjecie[2]))[1:]
        poztmp = str(10000 + int(przyjecie[3]))[1:]
        ppoztmp = str(10000 + int(przyjecie[4]))[1:]
        dok_prz = self.firma_id + magtmp[:4] + przyjecie[1] + str(przyjecie[0]) + numer + poztmp + ppoztmp
        Wydania.append([Artykul, ilo1, ilo2, przyjecie[9], przyjecie[10], dok_prz, 0])
        if Ilosc1 == 0:
            break

    if dozdjecia1 > 0:
        UPD += "UPDATE rs_artykuly_magazyny SET ilosc1=ilosc1 - " + str(dozdjecia1) + ", ilosc2=ilosc2 - " + str(dozdjecia2)
        UPD += " WHERE firma_id='" + self.firma_id + "' AND magazyn_id='" + Magazyn + "'"
        UPD += " AND artykul_id='" + Artykul + "';"
        UPD += "UPDATE rs_artykuly SET ilosc1=ilosc1 - " + str(dozdjecia1) + ", ilosc2=ilosc2 - " + str(dozdjecia2)
        UPD += " WHERE firma_id='" + self.firma_id + "' AND artykul_id='" + Artykul + "';"

    if UPD != "":     # zmniejszenie ilosci w przyjeciach, magazynach i artykulach
        rsglobal.ZapiszDoLogow(UPD)
        stat, dump, dump, dump = self.PolSQL.Execute(UPD, 1)

    return stat, Wydania, Ilosc1

#-##############################################################################
#--- FUNKCJA WYDAJACA JEDNA POZYCJE Z DOKUMENTU DETAL
#-##############################################################################

def HDWydajArtykul(self, Artykul, Magazyn, Data, Ilosc1, Cena1, DokumentPrzych, Najnowsze = False):
    # wraca ze statem i lista pozycji:
    #  Artykul (moze byc zamiennik), Ilosc1, Ilosc2, Cena1, Cena2, Dokument przychodowy, Nr komunikatu - 0 poprawnie
    #    1 - Wydano poprawnie zamiennik, 2 - wydano po ostatniej cenie w magazynie
    #    3 - wydano po ostatniej cenie w firmie
    # Dzialanie:
    # 1. Probuje wydac artykul "normalnie" nie zwraca uwagi na cechy, lokacje, terminy przydatnosci itd.
    # 2. Jak sie nie uda, to sprawdza czy artykul ma zamienniki jesli tak to probuje wydac je "normalnie"
    # 3. Jak sie nie uda, to wydaje artykul w cenie ostatniej dostawy do magazynu (dokument przychodowy = '')
    # 4. Jak sie nie uda, to wydaje artykul w cenie ostatniej dostawy do firmy (dokument przychodowy = '')

    Wydania = []
    stat = 'OK'
    if Ilosc1 == 0:
        return stat, Wydania

    SQL = "SELECT zamiennik_id, przelicznik,a.dynamiczny_przelicznik,j1.calkowita,j2.calkowita FROM rs_artykuly a "
    SQL += " LEFT OUTER JOIN rs_jednostki_miar j1 ON (j1.firma_id=a.firma_id AND j1.kod=a.jm1)"
    SQL += " LEFT OUTER JOIN rs_jednostki_miar j2 ON (j2.firma_id=a.firma_id AND j2.kod=a.jm2)"
    SQL += " WHERE a.firma_id='" + self.firma_id + "' AND a.artykul_id='" + rsglobal.Dekoduj(Artykul) + "'"
    rsglobal.ZapiszDoLogow(SQL)
    stat, daneartykulu, dump, dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        stat = 'Bd pobrania danych artykuu: '+stat
        return stat, Wydania
    zamiennik = daneartykulu[0][0]
    przelicznik = daneartykulu[0][1]
    calkowita1 = daneartykulu[0][3]
    calkowita2 = daneartykulu[0][2]
    if przelicznik <= 0 or daneartykulu[0][2]:
        stat = 'Nieobsugiwany typ przelicznika w artykule: ' + Artykul
        return stat, Wydania

    # proba normalnego wydania
    stat, Wydania, Pozostalo1 = HDWydajArtykulZPrzyjec(self, Artykul, Magazyn, Data, Ilosc1, daneartykulu, Najnowsze, DokumentPrzych)
    rsglobal.ZapiszDoLogow('Wydania:')
    rsglobal.ZapiszDoLogow(Wydania)
    rsglobal.ZapiszDoLogow('Pozostalo1:')
    rsglobal.ZapiszDoLogow(Pozostalo1)
    if stat != 'OK':
        return stat, Wydania
    else:
        if Pozostalo1 == 0:
            return stat, Wydania

    # przygotowanie listy zamiennikow
    zamienniki = []
    if zamiennik.strip() not in ['', 'None', '<None>']:
        # w artykule wpisany jest zamiennik
        SQL = "SELECT artykul_id FROM rs_artykuly WHERE firma_id='" + self.firma_id + "'"
        SQL += " AND (zamiennik_id='" + zamiennik + "' OR artykul_id='" + zamiennik + "')"
        SQL += "  AND artykul_id<>'" + Artykul + "'"
        rsglobal.ZapiszDoLogow(SQL)
        stat, zamienniki, dump, dump = self.PolSQL.Execute(SQL,3)
        if stat != 'OK':
            stat = 'Bd pobrania listy zamiennikow: '+stat
            return stat, Wydania
    else:
        # sprawdzamy czy artykul nie jest wpisany jako zamiennik do innych artykulow
        SQL = "SELECT artykul_id FROM rs_artykuly WHERE firma_id='" + self.firma_id + "'"
        SQL += " AND zamiennik_id='" + Artykul + "'"
        rsglobal.ZapiszDoLogow(SQL)
        stat, zamienniki, dump, dump = self.PolSQL.Execute(SQL,3)
        if stat != 'OK':
            stat = 'Bd pobrania listy zamiennikow: '+stat
            return stat, Wydania
    if len(zamienniki) > 0:
        # proba wydania zamiennikow
        for zam in zamienniki:
            SQL = "SELECT artykul_id, przelicznik,a.dynamiczny_przelicznik,j1.calkowita,j2.calkowita FROM rs_artykuly a "
            SQL += " LEFT OUTER JOIN rs_jednostki_miar j1 ON (j1.firma_id=a.firma_id AND j1.kod=a.jm1)"
            SQL += " LEFT OUTER JOIN rs_jednostki_miar j2 ON (j2.firma_id=a.firma_id AND j2.kod=a.jm2)"
            SQL += " WHERE a.firma_id='" + self.firma_id + "' AND a.artykul_id='" + zam[0] + "'"
            rsglobal.ZapiszDoLogow(SQL)
            stat, daneartykulu, dump, dump = self.PolSQL.Execute(SQL, 3)
            if stat != 'OK':
                stat = 'Bd pobrania danych artykuu: ' + stat
                return stat, Wydania
            stat, ZWydania, Pozostalo1 = HDWydajArtykulZPrzyjec(self, zam[0], Magazyn, Data, Pozostalo1, daneartykulu, Najnowsze, '')
            rsglobal.ZapiszDoLogow('ZWydania:')
            rsglobal.ZapiszDoLogow(ZWydania)
            rsglobal.ZapiszDoLogow('Pozostalo1:')
            rsglobal.ZapiszDoLogow(Pozostalo1)
            if stat != 'OK':
                return stat, Wydania
            else:
                if len(ZWydania) > 0:
                    for w in ZWydania:
                        w[6] = 1  # wydano jako zamiennik
                        Wydania.append(w)
                if Pozostalo1 == 0:
                    return stat, Wydania

    # nie udao si wyda z artykuy przyjcia, reszte wydajemy po ostatniej cenie w magazynie lub firmie
    UPD = ''
    SQL = "SELECT ost_cena1, ost_cena2 FROM rs_artykuly_magazyny "
    SQL += " WHERE firma_id='" + self.firma_id + "' AND magazyn_id='" + Magazyn + "'"
    SQL += " AND artykul_id='" + Artykul + "';"
    rsglobal.ZapiszDoLogow(SQL)
    stat, ceny, dump, dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        stat = 'Bd pobrania cen z magazynu: '+stat
        return stat, Wydania
    if len(ceny) > 0:    # bylo kiedys w magazynie wydajemy reszte towaru
        if Cena1 > 0:   # cena podana przez operatora
            Wydania.append([Artykul, Pozostalo1, rsglobal.RSRound(Pozostalo1 * przelicznik, 3), Cena1, rsglobal.RSRound(Cena1 / przelicznik, 2), '', 2])
        else:           # cena z magazynu
            Wydania.append([Artykul, Pozostalo1, rsglobal.RSRound(Pozostalo1 * przelicznik, 3), ceny[0][0], ceny[0][1], '', 2])
        UPD += "UPDATE rs_artykuly_magazyny SET ilosc1=ilosc1 - " + str(Pozostalo1) + ", "
        UPD += " ilosc2=ilosc2 - " + str(rsglobal.RSRound(Pozostalo1 * przelicznik, 3))
        UPD += " WHERE firma_id='" + self.firma_id + "' AND magazyn_id='" + Magazyn + "'"
        UPD += " AND artykul_id='" + Artykul + "';"
    else:     # wydajemy reszte po cenach z artykulu
        SQL = "SELECT ost_cena1, ost_cena2 FROM rs_artykuly "
        SQL += " WHERE firma_id='" + self.firma_id + "' "
        SQL += " AND artykul_id='" + Artykul + "';"
        rsglobal.ZapiszDoLogow(SQL)
        stat, ceny, dump, dump = self.PolSQL.Execute(SQL, 3)
        if stat != 'OK':
            stat = 'Bd pobrania cen z artykuw: ' + stat
            return stat, Wydania
        if Cena1 > 0:   # cena podana przez operatora
            Wydania.append([Artykul, Pozostalo1, rsglobal.RSRound(Pozostalo1 * przelicznik, 3), Cena1, rsglobal.RSRound(Cena1 / przelicznik, 2), '', 3])
        else:           # cena z artykulu
            Wydania.append([Artykul, Pozostalo1, rsglobal.RSRound(Pozostalo1 * przelicznik, 3), ceny[0][0], ceny[0][1], '', 3])
        # nie ma w rs_magazyny_artykuly robimy insert
        UPD += "INSERT INTO rs_artykuly_magazyny (firma_id,magazyn_id,artykul_id,ilosc1,ilosc2,ost_cena1,ost_cena2)"
        UPD += "VALUES('" + self.firma_id + "','" + Magazyn + "','" + Artykul + "', - " + str(Pozostalo1) + ", - "
        UPD += str(rsglobal.RSRound(Pozostalo1 * przelicznik, 3)) + "," + str(ceny[0][0]) + "," +  str(ceny[0][0]) + ");"

    UPD += "UPDATE rs_artykuly SET ilosc1=ilosc1 - " + str(Ilosc1) + ", "
    UPD += " ilosc2=ilosc2 - " + str(rsglobal.RSRound(Ilosc1 * przelicznik, 3))
    UPD += " WHERE firma_id='" + self.firma_id + "' AND artykul_id='" + Artykul + "';"
    # zmniejszenie ilosci w magazynach i artykulach
    rsglobal.ZapiszDoLogow(UPD)
    stat, dump, dump, dump = self.PolSQL.Execute(UPD, 1)

    return stat, Wydania

#-##############################################################################
#--- FUNKCJA SPRAWDZAJCA POPRAWNO CECHY
#-##############################################################################

def SprawdzPoprawnoscCechy(self,Cecha,Wartosc):
    if Cecha in [None,'None']:
        Cecha=''
    if Wartosc in [None,'None']:
        Wartosc=''
        
    SQL="SELECT rsf_podaj_parametry_cechy('"+self.firma_id+"','"+Cecha+"','"+Wartosc+"','poprawna')"
    stat,d,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        stat = 'Bd sprawdzenia wartoci cechy: '+stat
    else:
        rsglobal.ZapiszDoLogow("rseval: {}".format(d[0][0]))
        Wynik=rsglobal.rseval(d[0][0])
        if not Wynik[0]:
            stat = Wynik[1]
    
    return stat


#-##############################################################################
#--- FUNKCJA TWORZCA DOKUMENT WZ LUB ZW NA PODSTAWIE WYBRANYCH DOWODW DOSTAW
#-##############################################################################

def UtworzDokumentyMagazynoweZDokumentyMagazynoweDowodyDostaw(self,Typ,Data,Magazyn,
              Kontrahent,LKontr,OsobneDokumenty,STryby):

    if Typ in ('WZ','RW'):
        TypDok='wz'
        PoleIlosc='dd.ilosc_dost1'
        PoleMag='magazyn_skad'
    else:
        TypDok='zw'
        PoleIlosc='dd.ilosc_zwr1'
        PoleMag='magazyn_dokad'
        
    # pobranie pozycji z dowodw dostaw. Jeli osobne dostawy to dodatkowo klucz dowodu jest pobierany
    # na kocu jest te kontrahent do wstawienia jeli osobne dokumenty i kontrahent ''
        
    SQL="SELECT dd.artykul_id,"+PoleIlosc+","+PoleIlosc+"*a.przelicznik," # 0-2
    SQL+="d.trasa,d.data,d.tryb_dostawy,d.numer,d.kontrahent_id" # 3-7
    SQL+=" FROM rs_dowody_dostaw_pozycje dd "
    SQL+=" LEFT OUTER JOIN rs_dowody_dostaw d ON (dd.firma_id = d.firma_id AND  d.oddzial_id=dd.oddzial_id "
    SQL+=" AND d.trasa=dd.trasa AND d.data=dd.data AND d.tryb_dostawy=dd.tryb_dostawy AND d.numer=dd.numer) "
    SQL+=" INNER JOIN rs_artykuly a ON (dd.firma_id = a.firma_id AND dd.artykul_id = a.artykul_id) "
    SQL+=" WHERE dd.firma_id='"+self.firma_id+"' AND dd.oddzial_id='"+self.oddzial_id+"' "
    SQL+=" AND (strpos('"+STryby+"',dd.tryb_dostawy)>0 OR '"+STryby+"'='') AND dd.data='"+Data+"'"
    if Kontrahent != '':
        SQL+=" AND d.kontrahent_id='"+Kontrahent+"'"
    elif LKontr != '()':
        SQL+=" AND d.kontrahent_id NOT IN "+LKontr
    SQL+=" AND d."+TypDok+"_magazyn_id IS NULL"
    SQL+=" AND NOT d.korekta AND d.stan >='2'"
    SQL+=" AND a.typ_artykulu NOT IN ('31','99') AND "+PoleIlosc+">0"
    rsglobal.ZapiszDoLogow(100 * 'SQL DPOZ '+str(SQL),5)
    stat,DPoz,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat<>'OK':
        return 'BL SQL POZ DD'
    
    rsglobal.ZapiszDoLogow('Dane POZ podst '+str(DPoz),5)
    # posumowanie 
    # jeli nie s wczone osobne dokumenty to zbierane s informacje dla poszczeglnych dowodw. 
    # jeli wyczone to cao trafia do jednej paczki.
    # do oddzielnej zmiennej zbieramy informacje na ktrych dowodach dostaw to si pojawio
    
    # SPoz - klucz to dokument dostawy lub pusty tekst (dla nieosobnych dokumentw)
    #        warto to sownik z artykuami (klucz) i ich sumami (warto)
    
    SPoz = {}
    LDowodyDost = []
    for poz in DPoz:
        KluczDD=(poz[3],poz[4],poz[5],poz[6],poz[7])
        if OsobneDokumenty:
            KluczSPoz=KluczDD
        else:
            KluczSPoz=''
            
        Art=poz[0]
        if KluczSPoz in SPoz:
            SArt=SPoz[KluczSPoz]
        else:
            SArt={}
            
        if not Art in SArt:
            SArt[Art] = [poz[1],poz[2]]
        else:
            SArt[Art][0] += poz[1]
            SArt[Art][1] += poz[2]
        
        SPoz[KluczSPoz] = SArt
        if not OsobneDokumenty:
            LDowodyDost.append(KluczDD)
            
    #---------------------------------------------------------------------------
    # teraz dla kadego klucza z SPoz robimy jeden dokument magazynowy
    #---------------------------------------------------------------------------

    for klucz in SPoz:
        # pobranie wersji dla dokumentu magazynowego
        Wersja=''
        
        if Typ == 'ZW':
            stat,DWersjaM,dump,dump = self.PolSQL.Execute("SELECT wersja_magazyn FROM rs_oddzialy WHERE firma_id='"+self.firma_id+"' AND oddzial_id = '"+self.oddzial_id+"';", 3)
            if stat<>'OK':
                return 'Bdy pobierania wersji dla oddziau : '+stat
            
            if len(DWersjaM) == 1:
                if str(DWersjaM[0][0]).strip() not in ('NULL','None',None,''):
                    Wersja = "'"+str(wersjam[0][0])+"'"
                else:
                    Wersja = 'NULL'

        SParam = {}
        SParam['wersja'] = Wersja.strip("'")
        SParam['kontrahent_id'] = Kontrahent
        stat,SParam = fs_magazyny_parametry_dokumentow_pom.UzupelnijDomyslneParametryDlaDokumentu(self,Magazyn,Typ,SParam)
        if stat <>'OK':
            return stat

        stat = fs_magazyny_parametry_dokumentow_pom.SprawdzParametryDlaDokumentu(self,Magazyn,Typ,SParam)
        if stat != 'OK' and 'PARAM' in stat:
            # dodanie typu dokumentu dla dokladniejszego komunikatu
            return stat+'_'+Typ

        Wersja = SParam['wersja']
        if Wersja in ('','NULL','None',None):
            Wersja = 'NULL'
        else:
            Wersja = "'"+Wersja+"'"

        # pobranie numeru dokumentu magazynowgo
        stat,test,dump,dump = self.PolSQL.Execute("SELECT rsf_sprawdz_czy_mozna_dodac_dokument('"+self.firma_id+"','[\\\'"+Magazyn+"\\\',\\\'"+Typ+"\\\',\\\'"+Data+"\\\']','M');",3)            
        if test[0][0]<>'OK':    
            return 'N M D D '+Typ

        stat,DNum,dump,dump = self.PolSQL.Execute("SELECT rsf_podaj_nowy_numer_dokumentu('"+self.firma_id+"','[\\\'"+Magazyn+"\\\',\\\'"+Typ+"\\\',\\\'"+Data+"\\\']','M');",3)        
        if stat<>'OK':
            return 'Bdy podawania nowego numeru dokumentu '+Typ+': '+stat

        if DNum[0][0] in (None,'None',''):
            return 'B N D '+Typ 

        NrDok=DNum[0][0]

        # stworzenie nagwka dokumentu
        KontrPom = "'"+Kontrahent+"'"
        if KontrPom == "''":
            if OsobneDokumenty:
                KontrPom="'"+rsglobal.Dekoduj(klucz[4])+"'"
            elif Kontrahent <>'':
                KontrPom="'"+Kontrahent+"'"
            else:
                KontrPom="NULL"
        
        INS="INSERT INTO rs_dokumenty_magazynowe"
        INS+=" (firma_id,magazyn_id,dokument_id,data,numer,"
        INS+=PoleMag+" ,kontrahent_id,wersja,waluta)"
        INS+=" VALUES"
        INS+=" ('"+self.firma_id+"','"+Magazyn+"','"+Typ+"','"+str(Data)+"',"+str(NrDok)+","
        INS+="  '"+Magazyn+"',"+KontrPom+","+Wersja+",'PLN');"
        
        # robimy ptl po pozycjach z artykuw 
        KlArt = SPoz[klucz].keys()
        KlArt.sort()
        NrPoz = 1

        for art in KlArt:
            PozArt=SPoz[klucz][art]
            
            # pobieramy ceny dla dokumentu ZW
            Cena1='0'
            Cena2='0'
            if Typ == 'ZW' and not Wersja in ['NULL','']:
                Cena1="rsf_podaj_cene_n1('"+self.firma_id+"','"+rsglobal.Dekoduj(art)+"',"+Wersja+",'PLN','"+Data+"')"
                Cena2="rsf_podaj_cene_n2('"+self.firma_id+"','"+rsglobal.Dekoduj(art)+"',"+Wersja+",'PLN','"+Data+"')"
            
            # robimy insert pozycji
            INS+= "INSERT INTO rs_dokumenty_magazynowe_pozycje "
            INS+= "(firma_id,magazyn_id,dokument_id,data,numer,nr_poz,nr_ppoz,artykul_id,ilosc1,ilosc2, " 
            INS+= "cena1,cena2,cena_wal1,cena_wal2) "
            INS+= " VALUES('"+self.firma_id+"','"+Magazyn+"','"+Typ+"','"+Data+"',"+str(NrDok)+","
            INS+= str(NrPoz)+",0,'"+rsglobal.Dekoduj(art)+"',"+str(PozArt[0])+","+str(rsglobal.RSRound(PozArt[1],3))+", "
            INS+= Cena1+","+Cena2+","+Cena1+","+Cena2+");"
            NrPoz=NrPoz+1

        rsglobal.ZapiszDoLogow(100*'INS: ' + str(SQL), 5)
        # faktyczne utworzenie dokumentu
        stat,dump,dump,dump = self.PolSQL.Execute(INS,1)
        if stat != 'OK':
            return 'Wystpiy bdy podczas tworzenia dokumentu '+Typ+' :'+stat
        
        # zapis utworzonego dokumentu na dowodach dostaw
        if OsobneDokumenty:
            SQL="UPDATE rs_dowody_dostaw SET "+TypDok+"_magazyn_id='"+Magazyn+"',"
            SQL+=TypDok+"_dokument_id='"+Typ+"',"+TypDok+"_data='"+Data+"',"+TypDok+"_numer="+str(NrDok)
            SQL+=" WHERE firma_id='"+self.firma_id+"' AND oddzial_id='"+self.oddzial_id+"'"
            SQL+=" AND trasa='"+klucz[0]+"' AND data='"+klucz[1]+"'"
            SQL+=" AND tryb_dostawy='"+klucz[2]+"' AND numer="+str(klucz[3])+";"
        else:
            SQL=''
            for ddost in LDowodyDost:
                SQL+="UPDATE rs_dowody_dostaw SET "+TypDok+"_magazyn_id='"+Magazyn+"',"
                SQL+=TypDok+"_dokument_id='"+Typ+"',"+TypDok+"_data='"+Data+"',"+TypDok+"_numer="+str(NrDok)
                SQL+=" WHERE firma_id='"+self.firma_id+"' AND oddzial_id='"+self.oddzial_id+"'"
                SQL+=" AND trasa='"+ddost[0]+"' AND data='"+ddost[1]+"'"
                SQL+=" AND tryb_dostawy='"+ddost[2]+"' AND numer="+str(ddost[3])+";"
            
        if SQL <>'':
            stat,dump,dump,dump = self.PolSQL.Execute(SQL,1)
            if stat != 'OK':
                return 'Wystpiy bdy podczas zapisu dokumentu na dowodach dostaw :'+stat

        # pobranie opakowa i palet na dokument
        stat = UzupelnijOpakowaniaIPaletyDokumentu(self,Magazyn,Typ,Data,NrDok)
        if stat != 'OK':
            return stat
        
        self.PolSQL.Commit()
        
        # zamknicie dokumentu z W na G
        Klucze = PobierzKlucze(Magazyn,Typ,Data,NrDok,18)
        spom='rs_artykuly'
        while spom.find('rs_artykuly')>-1:
            spom, dump, dump, dump = self.WykonajFunkcje("fs_dokumenty_magazynowe", Klucze, [], [],False)
            rsglobal.ZapiszDoLogow('!!! spom '+spom,5)
            if spom.find('rs_artykuly')>-1:
                self.PolSQL.Rollback()
                time.sleep(1)
        
        if spom == 'OK':
            self.PolSQL.Commit()
            # zamknicie dokumentu z G na Z
            spom='rs_artykuly'
            while spom.find('rs_artykuly')>-1:
                Klucze['Funkcja']=19
                spom, dump, dump, dump = self.WykonajFunkcje("fs_dokumenty_magazynowe", Klucze, [], [],False)
                rsglobal.ZapiszDoLogow('spom 2 '+spom,5)
                if spom.find('rs_artykuly')>-1:
                    self.PolSQL.Rollback()
                    time.sleep(1)
                    
        self.PolSQL.Commit()
                    
    return 'OK'


#-##############################################################################
#--- FUNKCJA TWORZCA KOREKTY DM NA PODSTAWIE KOREKT DOWODW DOSTAW
#-##############################################################################

## UWAGA - zostao uzgldnia informacj o pierwotnym DM dla korekty i ew tworzy WZ/ZW po prostu jeli nie 
## rady inaczej

# TODO !!!!! UWAGA kawaek kodu poniej naley zastpi funkcj pomocnicz PodajDokumentKorekty
# w ramach ewentualnych zmian w tym algorytmie to zrobi przy okazji
def UtworzKorektyDMZDokumentyMagazynoweDowodyDostaw(self,Data,MagazynWZ,MagazynZW,MagazynRW,
              Kontrahent,LKontr,STryby):

    # pobranie potencjalnych typw dokumentw dla WZ+, WZ-, ZW+ i ZW-
    # na tym etapie brak definicji to nie problem. Dopiero przy samym wykorzystaniu
    STypyDok={}
    for kl in [('RW','P'),('RW','W'),('WZ','P'),('WZ','W'),('ZW','P'),('ZW','W')]:
        # szukamy dokumentu z dokadnym typem korekty_do
        SQL="SELECT dokument_id FROM rs_typy_dokumentow_m WHERE firma_id='"+self.firma_id+"'"
        SQL+="AND typ_dokumentu='"+kl[1]+"' AND korekta AND STRING_TO_ARRAY(korekta_do,',') @> ARRAY['"+kl[0]+"']"
        stat,DDok,dump,dump = self.PolSQL.Execute(SQL,3)
        if stat<>'OK':
            return 'Bdy sprawdzania typw korekt dokumentw magazynowych : '+stat
        if len(DDok)>1:
            STypyDok[kl]='BLAD'
        elif len(DDok)==1:
            STypyDok[kl]=DDok[0][0]
        # szukamy dokumentu bez typu do
        else:
            SQL="SELECT dokument_id FROM rs_typy_dokumentow_m WHERE firma_id='"+self.firma_id+"'"
            SQL+="AND typ_dokumentu='"+kl[1]+"' AND korekta AND COALESCE(TRIM(korekta_do),'')=''"
            stat,DDok,dump,dump = self.PolSQL.Execute(SQL,3)
            if stat<>'OK':
                return 'Bdy drugiego sprawdzania typw korekt dokumentw magazynowych : '+stat

            if len(DDok)==1:
                STypyDok[kl]=DDok[0][0]
            else:
                STypyDok[kl]='BLAD'
            
    # pobranie pozycji z dowodw dostaw. Klucz dowodu jest pobierany
    # na kocu jest te kontrahent do wstawienia jeli osobne dokumenty i kontrahent ''
        
    SQL="SELECT dd.artykul_id,dd.ilosc_dost1,dd.ilosc_dost1*a.przelicznik," # 0-2
    SQL+=" dd.ilosc_zwr1,dd.ilosc_zwr1*a.przelicznik," # 3-4
    SQL+=" d.trasa,d.data,d.tryb_dostawy,d.numer,d.kontrahent_id," # 5-9
    SQL+=" dp.wz_magazyn_id,COALESCE(dp.wz_dokument_id,''),dp.wz_data,dp.wz_numer," # 10-13
    SQL+=" dp.zw_magazyn_id,COALESCE(dp.zw_dokument_id,''),dp.zw_data,dp.zw_numer," # 14-17
    SQL+=" dp.trasa,dp.data,dp.tryb_dostawy,dp.numer" # 18-21
    SQL+=" FROM rs_dowody_dostaw_pozycje dd "
    SQL+=" LEFT OUTER JOIN rs_dowody_dostaw d ON (dd.firma_id = d.firma_id AND  d.oddzial_id=dd.oddzial_id "
    SQL+=" AND d.trasa=dd.trasa AND d.data=dd.data AND d.tryb_dostawy=dd.tryb_dostawy AND d.numer=dd.numer) "
    SQL+=" LEFT OUTER JOIN rs_dowody_dostaw dp ON (dp.firma_id = d.firma_id AND  dp.oddzial_id=d.oddzial_id_p "
    SQL+=" AND dp.trasa=d.trasa_p AND dp.data=d.data_p AND dp.tryb_dostawy=d.tryb_dostawy_p AND dp.numer=d.numer_p) "
    SQL+=" INNER JOIN rs_artykuly a ON (dd.firma_id = a.firma_id AND dd.artykul_id = a.artykul_id) "
    SQL+=" WHERE dd.firma_id='"+self.firma_id+"' AND dd.oddzial_id='"+self.oddzial_id+"' "
    SQL+=" AND (strpos('"+STryby+"',dd.tryb_dostawy)>0 OR '"+STryby+"'='') AND dd.data='"+Data+"'"
    if Kontrahent != '':
        SQL+=" AND d.kontrahent_id='"+Kontrahent+"'"
    elif LKontr != '()':
        SQL+=" AND d.kontrahent_id NOT IN "+LKontr
    SQL+=" AND d.korekta AND NOT d.czy_automatycznie AND d.wz_magazyn_id IS NULL AND d.zw_magazyn_id IS NULL"
    SQL+=" AND a.typ_artykulu NOT IN ('31','99') AND (dd.ilosc_dost1!=0 OR dd.ilosc_zwr1!=0) AND d.stan >='2'"
    rsglobal.ZapiszDoLogow('SQLPOZ '+SQL,5)
    stat,DPoz,dump,dump = self.PolSQL.Execute(SQL,3)
    rsglobal.ZapiszDoLogow('DPoz '+str(DPoz),5)
    if stat<>'OK':
        return 'BL SQL POZ DD'

    # posumowanie 
    #  poniewa s wczone osobne dokumenty to zbierane s informacje dla poszczeglnych dowodw. 
    
    # SPoz - klucz to dokument dostawy 
    #                kontrahent i klucze pierwotnych wz i zw
    #        warto to sownik z artykuami (klucz) i ich sumami (warto)
    #        na kocu informacja jakie typy korekt bd wystawione
    
    SPoz = {}
    LDowodyDost = []
    for poz in DPoz:
        KluczDD=(poz[5],poz[6],poz[7],poz[8],poz[9], # 0-4 klucz dowodu dostawy i kontrahent
                 poz[10],poz[11],poz[12],poz[13],    # 5-8 klucz pierwotnego WZ (RW)
                 poz[14],poz[15],poz[16],poz[17],    # 9-12 klucz pierwotnego ZW
                 poz[18],poz[19],poz[20],poz[21])    # 13-16 klucz pierwotnego dowodu dostawy
        Art=poz[0]
        if KluczDD in SPoz:
            SArt,LTypow=SPoz[KluczDD]
        else:
            SArt,LTypow={},[]
            
        if not Art in SArt:
            # osobno odkadamy iloci+ i  iloci-  - razem 8 kolumn
            # il_dost1+,il_dost2+, il_dost1-, il_dost2-, il_zwr1+,il_zwr2+, il_zwr1-, il_zwr2-
            SArt[Art] = [0,0,0,0,0,0,0,0,[]]
        
        if poz[1]>0:
            SArt[Art][0] += poz[1]
            SArt[Art][1] += poz[2]
            if MagazynRW <>'    ':
              if not ('RW','W') in LTypow:
                LTypow.append(('RW','W'))  
            elif not ('WZ','W') in LTypow:
                LTypow.append(('WZ','W'))
        elif poz[1]<0:
            SArt[Art][2] += -1*poz[1]
            SArt[Art][3] += -1*poz[2]
            if MagazynRW <>'    ':
              if not ('RW','P') in LTypow:
                LTypow.append(('RW','P'))  
            elif not ('WZ','P') in LTypow:
                LTypow.append(('WZ','P'))
        if poz[3]>0:
            SArt[Art][4] += poz[3]
            SArt[Art][5] += poz[4]
            if not ('ZW','P') in LTypow:
                LTypow.append(('ZW','P'))
        elif poz[3]<0:
            SArt[Art][6] += -1*poz[3]
            SArt[Art][7] += -1*poz[4]
            if not ('ZW','W') in LTypow:
                LTypow.append(('ZW','W'))
        
        SPoz[KluczDD] = [SArt,LTypow]
    
    #---------------------------------------------------------------------------
    # teraz dla kadego klucza z SPoz robimy odpowiednie korekty do DM
    #---------------------------------------------------------------------------

    for klucz in SPoz:
        SArt,LTypow=SPoz[klucz]
        if len(LTypow)>2:
            return 'Na dowodzie dostawy '+klucz[0:5]+' wypeniono pozycje generujce wicej ni dwa typy korekt dokumentw magazynowych'

        # funkcje pomocnicze, ktre pomagaj stwierdzi jak wypenia nagwek dowodu dostawy 
        # jeli s dwa typy WZ/RW lub dwa typy ZW
        NaglWZ=''
        NaglZW=''
        
        # ptla po typach dokumentw
        for typ in LTypow:
            TypDok= STypyDok[typ]
            if TypDok == 'BLAD':
                return 'Nie zdefiniowano jednoznacznie dokumentu korekty dla typu '+typ[0]
            
            # sprawdzamy czy przypadkiem nie bdzie tworzony normalny dokument WZ/ZW (bo go nie byo wczeniej)
            MagP='NULL'
            DokP='NULL'
            DataP='NULL'
            NumerP='NULL'
            if typ[0] in  ['WZ','RW']:
                if klucz[6] == '': # nie ma dokumentu pierwotnego WZ na dowodzie pierwotnym
                    
                    # sprawdzamy, czy nie byo do niego korekty, na ktrej powsta WZ/RW
                    SQL="SELECT wz_magazyn_id,wz_dokument_id,wz_data,wz_numer"
                    SQL+=" FROM rs_dowody_dostaw WHERE firma_id='"+self.firma_id+"' AND oddzial_id_p='"+self.oddzial_id+"'"
                    SQL+=" AND trasa_p='"+klucz[13]+"' AND data_p='"+klucz[14]+"' AND tryb_dostawy_p='"+klucz[15]+"'"
                    SQL+=" AND numer_p="+str(klucz[16])+" AND korekta AND wz_dokument_id='"+typ[0]+"';"
                    stat,DKorWZ,dump,dump = self.PolSQL.Execute(SQL,3)
                    if stat<>'OK':
                        return 'Bdy sprawdzania istnienia WZ na korektach dowodw dostaw :'+stat
                    
                    if len(DKorWZ)>0: # by WZ na korekcie
                        MagP="'"+DKorWZ[0][0]+"'"
                        DokP="'"+DKorWZ[0][1]+"'"
                        DataP="'"+DKorWZ[0][2]+"'"
                        NumerP="'"+str(DKorWZ[0][3])+"'"
                    else: # nie byo jeszcze WZ
                        TypDok=typ[0]
                    
                else: # jest dokument pierwotny WZ/RW
                    MagP="'"+klucz[5]+"'"
                    DokP="'"+klucz[6]+"'"
                    DataP="'"+klucz[7]+"'"
                    NumerP="'"+str(klucz[8])+"'"
                    
            else: # typ ZW
                if klucz[10] == '': # nie ma dokumentu pierwotnego ZW
                    
                    # sprawdzamy, czy nie byo do niego korekty, na ktrej powsta ZW
                    SQL="SELECT zw_magazyn_id,zw_dokument_id,zw_data,zw_numer"
                    SQL+=" FROM rs_dowody_dostaw WHERE firma_id='"+self.firma_id+"' AND oddzial_id_p='"+self.oddzial_id+"'"
                    SQL+=" AND trasa_p='"+klucz[13]+"' AND data_p='"+klucz[14]+"' AND tryb_dostawy_p='"+klucz[15]+"'"
                    SQL+=" AND numer_p="+str(klucz[16])+" AND korekta AND zw_dokument_id='ZW';"
                    stat,DKorZW,dump,dump = self.PolSQL.Execute(SQL,3)
                    if stat<>'OK':
                        return 'Bdy sprawdzania istnienia ZW na korektach dowodw dostaw :'+stat
                    
                    if len(DKorZW)>0: # by ZW na korekcie
                        MagP="'"+DKorZW[0][0]+"'"
                        DokP="'"+DKorZW[0][1]+"'"
                        DataP="'"+DKorZW[0][2]+"'"
                        NumerP="'"+str(DKorZW[0][3])+"'"
                    else: # nie byo jeszcze ZW
                        if typ[1]=='P':
                            TypDok='ZW'
                        else:
                            TypDok='WZ'
                    
                else: # jest dokument pierwotny ZW
                    MagP="'"+klucz[9]+"'"
                    DokP="'"+klucz[10]+"'"
                    DataP="'"+klucz[11]+"'"
                    NumerP="'"+str(klucz[12])+"'"
            
            # pobranie wersji dla dokumentu magazynowego
            Wersja=''
            
            # jeli jest to prbujemy pobra wersj z dokumentu pierwotnego
            if DokP != 'NULL':
                SQL="SELECT COALESCE(wersja,'') FROM rs_dokumenty_magazynowe WHERE firma_id='"+self.firma_id+"'"
                SQL+=" AND magazyn_id="+MagP+" AND dokument_id="+DokP+" AND data="+DataP+" AND numer="+NumerP
                stat,DWersji,dump,dump = self.PolSQL.Execute(SQL,3)
                if stat != 'OK':
                    return 'Wystpiy bdy podczas pobierania wersji z dokumentu pierwotnego :'+stat
                Wersja = DWersji[0][0]
                
                
            # wybieramy magazyn na ktrym to bdzie tworzone. Jeli nie da si go wybra jednoznacznie to wychodzimy z funkcji
            if MagP <>'NULL':
                Magazyn=MagP.strip("'")
            else:
                if typ[0] == 'RW':
                    Magazyn=MagazynRW
                elif typ[0] == 'WZ':
                    Magazyn=MagazynWZ
                elif typ[0] == 'ZW':
                    Magazyn=MagazynZW
            
            if Magazyn.strip() =='':
                return "Nie udao si dobra magazynu dla dokumentu "+typ[0]+' dla kontrahenta "'+Kontrahent+'"'

            # jeli w dokumencie nie ma wersji (albo nie ma dokumentu pierwotnego)
            # to sprawdzamy wersj dla dokumentu ZW. sprawdzamy tylko dla przychodowych
            if Wersja == '' and typ[1] == 'P':
                SParam = {}
                SParam['wersja'] = ''
                SParam['kontrahent_id'] = klucz[4]
                stat,SParam = fs_magazyny_parametry_dokumentow_pom.UzupelnijDomyslneParametryDlaDokumentu(self,Magazyn,typ[0],SParam)
                if stat <>'OK':
                    return stat
    
                stat = fs_magazyny_parametry_dokumentow_pom.SprawdzParametryDlaDokumentu(self,Magazyn,typ[0],SParam)
                if stat != 'OK' and 'PARAM' in stat:
                    # dodanie typu dokumentu dla dokladniejszego komunikatu
                    return stat+'_'+typ[0]
    
                Wersja = SParam['wersja']
            
                
            if Wersja in ('','NULL','None',None):
                Wersja = 'NULL'
            else:
                Wersja = "'"+Wersja+"'"

            # pobranie numeru dokumentu magazynowgo
            stat,test,dump,dump = self.PolSQL.Execute("SELECT rsf_sprawdz_czy_mozna_dodac_dokument('"+self.firma_id+"','[\\\'"+Magazyn+"\\\',\\\'"+TypDok+"\\\',\\\'"+klucz[1]+"\\\']','M');",3)            
            if test[0][0]<>'OK':    
                return 'N M D D '+typ[0]
    
            stat,DNum,dump,dump = self.PolSQL.Execute("SELECT rsf_podaj_nowy_numer_dokumentu('"+self.firma_id+"','[\\\'"+Magazyn+"\\\',\\\'"+TypDok+"\\\',\\\'"+klucz[1]+"\\\']','M');",3)        
            if stat<>'OK':
                return 'Bdy podawania nowego numeru dokumentu '+TypDok+': '+stat
    
            if DNum[0][0] in (None,'None',''):
                return 'B N D '+typ[0] 
    
            NrDok=DNum[0][0]

            # stworzenie nagwka dokumentu
            KontrPom = "'"+Kontrahent+"'"
            if KontrPom == "''":
                KontrPom="'"+rsglobal.Dekoduj(klucz[4])+"'"
            
            if typ[1] == 'P':
                PoleMag='magazyn_dokad'
            else:
                PoleMag='magazyn_skad'
            
            INS="INSERT INTO rs_dokumenty_magazynowe"
            INS+=" (firma_id,magazyn_id,dokument_id,data,numer,"
            INS+=PoleMag+" ,kontrahent_id,wersja,waluta,"
            INS+="kor_dm_magazyn_id,kor_dm_dokument_id,kor_dm_data,kor_dm_numer,opis)"
            INS+=" VALUES"
            INS+=" ('"+self.firma_id+"','"+Magazyn+"','"+TypDok+"','"+str(klucz[1])+"',"+str(NrDok)+","
            INS+="  '"+Magazyn+"',"+KontrPom+","+Wersja+",'PLN',"
            INS+=MagP+","+DokP+","+DataP+","+NumerP+",'Korekta dowodu dostawy');"
        
            # robimy ptl po pozycjach z artykuw 
            KlArt = SArt.keys()
            KlArt.sort()
            NrPoz = 1
        
            for art in KlArt:
                PozArt=SArt[art]
                
                # pobieramy ceny dla typu przychodowego
                Cena1='0'
                Cena2='0'
                if typ[1] == 'P' and not Wersja in ['NULL','']:
                    Cena1="rsf_podaj_cene_n1('"+self.firma_id+"','"+rsglobal.Dekoduj(art)+"',"+Wersja+",'PLN','"+Data+"')"
                    Cena2="rsf_podaj_cene_n2('"+self.firma_id+"','"+rsglobal.Dekoduj(art)+"',"+Wersja+",'PLN','"+Data+"')"
            
                if typ in [('RW','W'),('WZ','W')]:
                    Ilosc1=PozArt[0]
                    Ilosc2=PozArt[1]
                elif typ in [('RW','P'),('WZ','P')]:
                    Ilosc1=PozArt[2]
                    Ilosc2=PozArt[3]
                elif typ == ('ZW','P'):
                    Ilosc1=PozArt[4]
                    Ilosc2=PozArt[5]
                elif typ == ('ZW','W'):
                    Ilosc1=PozArt[6]
                    Ilosc2=PozArt[7]
                    
                # robimy insert pozycji
                if Ilosc1>0:
                    INS+= "INSERT INTO rs_dokumenty_magazynowe_pozycje "
                    INS+= "(firma_id,magazyn_id,dokument_id,data,numer,nr_poz,nr_ppoz,artykul_id,ilosc1,ilosc2, " 
                    INS+= "cena1,cena2,cena_wal1,cena_wal2) "
                    INS+= " VALUES('"+self.firma_id+"','"+Magazyn+"','"+TypDok+"','"+klucz[1]+"',"+str(NrDok)+","
                    INS+= str(NrPoz)+",0,'"+rsglobal.Dekoduj(art)+"',"+str(Ilosc1)+","+str(rsglobal.RSRound(Ilosc2,3))+", "
                    INS+= Cena1+","+Cena2+","+Cena1+","+Cena2+");"
                    NrPoz=NrPoz+1
        
            # faktyczne utworzenie dokumentu
            stat,dump,dump,dump = self.PolSQL.Execute(INS,1)
            if stat != 'OK':
                return 'Wystpiy bdy podczas tworzenia dokumentu '+TypDok+' :'+stat
        
            # zapis utworzonego dokumentu na dowodzie dostawy
            if (typ[0] in ['RW','WZ'] and NaglWZ=='') or(typ[0]=='ZW' and NaglZW<>''):
                TypDokPom='wz'
            else:
                TypDokPom='zw'
                
            SQL="UPDATE rs_dowody_dostaw SET "+TypDokPom+"_magazyn_id='"+Magazyn+"',"
            SQL+=TypDokPom+"_dokument_id='"+TypDok+"',"+TypDokPom+"_data='"+klucz[1]+"',"+TypDokPom+"_numer="+str(NrDok)
            SQL+=" WHERE firma_id='"+self.firma_id+"' AND oddzial_id='"+self.oddzial_id+"'"
            SQL+=" AND trasa='"+klucz[0]+"' AND data='"+klucz[1]+"'"
            SQL+=" AND tryb_dostawy='"+klucz[2]+"' AND numer="+str(klucz[3])+";"
                
            if SQL <>'':
                stat,dump,dump,dump = self.PolSQL.Execute(SQL,1)
                if stat != 'OK':
                    return 'Wystpiy bdy podczas zapisu dokumentu na dowodach dostaw :'+stat
    
            # pobranie opakowa i palet na dokument
            stat = UzupelnijOpakowaniaIPaletyDokumentu(self,Magazyn,TypDok,klucz[1],NrDok)
            if stat != 'OK':
                return stat
    
            self.PolSQL.Commit()
            
            # zamknicie dokumentu z W na G
            Klucze = PobierzKlucze(Magazyn,TypDok,klucz[1],NrDok,18)
            spom, dump, dump, dump = self.WykonajFunkcje("fs_dokumenty_magazynowe", Klucze, [], [],False)
            if spom == 'OK':
                self.PolSQL.Commit()
                # zamknicie dokumentu z G na Z
                Klucze['Funkcja']=29
                spom, dump, dump, dump = self.WykonajFunkcje("fs_dokumenty_magazynowe", Klucze, [], [],False)
                if spom != 'OK':
                    self.PolSQL.Rollback()
                else:
                    self.PolSQL.Commit()

    return 'OK'


#-------------------------------------------------------------------------------
#-- FUNKCJA POMOCNICZA - PODANIE RECEPTURY DLA ARTYKUU
#-------------------------------------------------------------------------------

def PodajReceptureDlaArtykulu(PolSQL,Firma,Oddzial,Artykul,Data):
    import rsglobal
    
    ReceptNag = []
    ReceptPoz = []

    Oddz = Oddzial
    # pobieram recepture dla oddzialu
    # jako wazniejsza niz recepture dla wszystkich oddzialow
    sql ="SELECT data_od,ilosc1,ilosc2"
    sql+=" FROM rs_receptury"
    sql+=" WHERE firma_id='"+Firma+"' AND oddzial_id='"+Oddz+"'"
    sql+=" AND stan='Z' AND artykul_id='"+Artykul+"' AND data_od<='"+Data+"'"
    sql+=" ORDER BY data_od DESC LIMIT 1;"
        
    stat,ReceptNag,dump,dump = PolSQL.Execute(sql,3)
    if stat != 'OK':
        return 'Bd pobrania nagwka receptury: '+stat,RecepNag,ReceptPoz
    
    if stat == 'OK' and len(ReceptNag) == 0:
        # jesli nie bylo receptury dla oddzialu, w ktorym zalogowany jest operator
        # probujemy pobrac recepture ogolna dla wszystkich oddzialow
        
        Oddz = '**'
        sql ="SELECT data_od,ilosc1,ilosc2"
        sql+=" FROM rs_receptury"
        sql+=" WHERE firma_id='"+Firma+"' AND oddzial_id='"+Oddz+"'"
        sql+=" AND stan='Z' AND artykul_id='"+Artykul+"' AND data_od<='"+Data+"'"
        sql+=" ORDER BY data_od DESC LIMIT 1;"

        stat,ReceptNag,dump,dump = PolSQL.Execute(sql,3)
        if stat != 'OK':
            return 'Bd pobrania nagwka receptury: '+stat,RecepNag,ReceptPoz
    
    if stat=='OK' and len(ReceptNag) > 0: 
        # pobranie pozycji receptury dla artykulu
        sql ="SELECT skladnik_id,COALESCE(magazyn_id,''::TEXT),ilosc1,ilosc2,0,0"
        sql+=" FROM rs_receptury_pozycje"
        sql+=" WHERE firma_id='"+Firma+"' AND oddzial_id='"+Oddz+"'"
        sql+=" AND artykul_id='"+Artykul+"'"
        sql+=" AND data_od='"+rsglobal.Dekoduj(ReceptNag[0][0])+"'"
        sql+=" ORDER BY skladnik_id;"
        
        stat,ReceptPoz,dump,dump = PolSQL.Execute(sql,3)
        if stat != 'OK':
            return 'Bd pobrania pozycji receptury: '+stat,RecepNag,ReceptPoz
    
    return 'OK',ReceptNag,ReceptPoz

#-------------------------------------------------------------------------------
#-- FUNKCJA POMOCNICZA - PODAJE DOKUMENT KOREKTY TYPU P/W DLA DOKUMENTU PIERWOTNEGO 
#-------------------------------------------------------------------------------

def PodajDokumentKorekty(self,DokPierw,Typ):
    SQL="SELECT dokument_id FROM rs_typy_dokumentow_m WHERE firma_id='"+self.firma_id+"'"
    SQL+="AND typ_dokumentu='"+Typ+"' AND korekta AND STRING_TO_ARRAY(korekta_do,',') @> ARRAY['"+DokPierw+"']"
    stat,DDok,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat<>'OK':
        return 'Bdy sprawdzania typw korekt dokumentw magazynowych : '+stat,''
    if len(DDok)>1:
        return 'Znaleziono wicej ni jeden dokument korekty typu '+Typ+' dla dokumentu pierwotnego '+DokPierw,''
    elif len(DDok)==1:
        return 'OK',DDok[0][0]

    # szukamy dokumentu bez typu do ktrego jest korekta

    SQL="SELECT dokument_id FROM rs_typy_dokumentow_m WHERE firma_id='"+self.firma_id+"'"
    SQL+="AND typ_dokumentu='"+Typ+"' AND korekta AND COALESCE(TRIM(korekta_do),'')=''"
    stat,DDok,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat<>'OK':
        return 'Bdy drugiego sprawdzania typw korekt dokumentw magazynowych : '+stat,''

    if len(DDok)==1:
        return 'OK',DDok[0][0]
    else:
        return 'Znaleziono wicej ni jeden dokument korekty typu '+Typ+' bez podania konkretnego dokumentu',''


# -------------------------------------------------------------------------------
# -- FUNKCJA WSTAWIAJACA DOKUMENT MAGAZYNOWY
# -------------------------------------------------------------------------------

# PARAMETRY
# Magazyn,Typ,Data - klucz wstawianego dokumentu magazynowego (numer pobierany jako kolejny)
# Kontrahent_id (lub '')
# Pozycje do wstawenia, lista [artykul_id, ilosc1, ilosc2]
# powrot: (stat, numer)
# Uwwaga: tylko dla dokumentow rozchodowych

def WstawDokumentMagazynowy(self, Magazyn, Typ, Data, Kontrahent, Pozycje):
    Numer = 0
    if Kontrahent != '':
        Kontrahent = "'" + Kontrahent + "'"
    else:
        Kontrahent = 'NULL'
    # sprawdzenie czy mona wystawic dokument z t dat
    stat, test, dump, dump = self.PolSQL.Execute("SELECT rsf_sprawdz_czy_mozna_dodac_dokument('"+self.firma_id+"','[\\\'"+Magazyn+"\\\',\\\'"+Typ+"\\\',\\\'"+Data+"\\\']','M');", 3)
    if stat != 'OK':
        stat = 'Bd sprawdzenia czy mona dodawa dokumenty z dat ' + str(Data)
    else:
        stat = test[0][0]
    if stat == 'OK':
        # pobranie numeru dokumentu
        stat, dane, dump, dump = self.PolSQL.Execute("SELECT rsf_podaj_nowy_numer_dokumentu('" + self.firma_id + "','[\\\'" + Magazyn + "\\\',\\\'" + Typ + "\\\',\\\'" + Data + "\\\']','M');", 3)
        if dane[0][0] in (None, 'None', ''):
            stat = "Bdy numeracji dokumentu."
        else:
            Numer = dane[0][0]
    if stat == 'OK':
        # wstawienie nagwka dokumentu
        sql = "INSERT INTO rs_dokumenty_magazynowe (firma_id,magazyn_id,dokument_id,data,numer,kontrahent_id,magazyn_skad) VALUES ('%s','%s','%s','%s',%s,%s,'%s');" % (
            self.firma_id, Magazyn, Typ, str(Data), str(Numer), Kontrahent,Magazyn)
        nrpoz = 0
        for p in Pozycje:
            nrpoz +=1
            sql += "INSERT INTO rs_dokumenty_magazynowe_pozycje (firma_id,magazyn_id,dokument_id,data,numer,nr_poz,nr_ppoz,artykul_id,ilosc1,ilosc2) VALUES ('%s','%s','%s','%s',%s,%s,%s,'%s',%s,%s);" % (
                self.firma_id, Magazyn, Typ, str(Data), str(Numer), str(nrpoz), 0, p[0], str(p[1]),str(p[2]))
        stat, dump, dump, dump = self.PolSQL.Execute(sql, 1)

    return (stat, Numer)

#-------------------------------------------------------------------------------
#-- FUNKCJA ZAMYKAJCA DOKUMENT MAGAZYNOWY
#-------------------------------------------------------------------------------

# PARAMETRY
# Mag,Dok,Data,Nr - klucz zamykanego dokumentu magazynowego
# StanP - stan pocztkowy dokumentu, StanK - stan kocowy dokumentu
# IgnorowacStat - jeli True to niezamknicie dokumentu jest dopuszczalne i OK

def ZamknijDokumentMagazynowy(self,Mag,Dok,Data,Nr,StanP,StanK,IgnorowacStat=False):
    Klucze = PobierzKlucze(Mag,Dok,Data,Nr,18)
    
    # prba zmiany stanu dokumentu z W na G
    if StanP =='W':
        stat, dump, dump, dump = self.WykonajFunkcje("fs_dokumenty_magazynowe", Klucze, [], [],False)
        if stat<>'OK':
            if IgnorowacStat:
                return 'OK'
            else:
                return stat
            
    # prba zmiany stanu dokumentu z G na Z
    if StanK == 'Z':
        Klucze['Funkcja']=19
        stat, dump, dump, dump = self.WykonajFunkcje("fs_dokumenty_magazynowe", Klucze, [], [],False)        
        if stat<>'OK':
            if IgnorowacStat:
                return 'OK'
            else:
                return stat
            
    return 'OK'

#-------------------------------------------------------------------------------
#-- FUNKCJA OTWIERAJCA DOKUMENT MAGAZYNOWY
#-------------------------------------------------------------------------------

# PARAMETRY
# Mag,Dok,Data,Nr - klucz otwieranego dokumentu magazynowego
# StanP - stan pocztkowy dokumentu, StanK - stan kocowy dokumentu
# IgnorowacStat - jeli True to nieotwarcie dokumentu jest dopuszczalne i OK

def OtworzDokumentMagazynowy(self,Mag,Dok,Data,Nr,StanP,StanK,IgnorowacStat=False):
    Klucze = PobierzKlucze(Mag,Dok,Data,Nr,21)
    # prba zmiany stanu dokumentu z Z na G
    if StanP =='Z':
        stat, dump, dump, dump = self.WykonajFunkcje("fs_dokumenty_magazynowe", Klucze, [], [],False)
        if stat<>'OK':
            if IgnorowacStat:
                return 'OK'
            else:
                return stat
            
    # prba zmiany stanu dokumentu z G na W
    if StanK == 'W':
        Klucze['Funkcja']=20
        stat, dump, dump, dump = self.WykonajFunkcje("fs_dokumenty_magazynowe", Klucze, [], [],False)        
        if stat<>'OK':
            if IgnorowacStat:
                return 'OK'
            else:
                return stat
            
    return 'OK'


#-------------------------------------------------------------------------------
#-- FUNKCJA USUWAJCA DOKUMENT MAGAZYNOWY
#-------------------------------------------------------------------------------

# PARAMETRY
# Mag,Dok,Data,Nr - klucz zamykanego dokumentu magazynowego
# IgnorowacStat - jeli True to nieusunicie dokumentu jest dopuszczalne i OK

def UsunDokumentMagazynowy(self,Mag,Dok,Data,Nr,IgnorowacStat=False):
    # prba usunicia dokumentu
    
    # do filtru podany klucz filtrowania tylko usuwanego dokumentu aby nie pobierac calego grida
    # po usunieciu dokumentu grid z pobrania dokumentow magazynowych powinien byc pusty
    Klucze = PobierzKlucze(Mag,Dok,Data,Nr,6)
    Klucze['StanF'] = ''
    Klucze['MagF'] = Mag
    Klucze['DokF'] = Dok
    Klucze['DataOdF'] = Data
    Klucze['DataDoF'] = Data
    Klucze['NrF'] = Nr
    
    Klucze['KontrF'] = ''
    Klucze['MiejDostF'] = ''
    Klucze['FaktF'] = ''
    Klucze['ProjektF'] = ''
    Klucze['KluczeF'] = []
    Klucze['NiezamknieteF'] = False
    Klucze['ZmienioneF'] = False
    Klucze['NiewyslaneTowF']=False
    Klucze['CzyPrzesylki']=False       

    stat, dump, dump, dump = self.WykonajFunkcje("fs_dokumenty_magazynowe", Klucze, [], [],False)
    if stat<>'OK':
        if IgnorowacStat:
            return 'OK'
        else:
            return stat

    return 'OK'


#-------------------------------------------------------------------------------
#-- FUNKCJA TWORZCA RW Z PW
#-------------------------------------------------------------------------------

def UtworzRWzPW(self,Mag,Dok,Data,Numer):
    # deklaracja podstawowych zmiennych
    rsglobal.ZapiszDoLogow('!!!12345',5)
    # sownik surowcw do wydania. klucz to magazyn,produkt,artyku a wynik to ilosc1
    SSurowce={}
    DaneRW=[]
    LKorPW=[]
    LKorRW=[]
    DSurowceRW=[]
    LMagazZRW=[]  # po uzupenieniu dla tych magazynw bd robione korekty RW, dla innych po prostu RW
    LUtwDokum = []  # lista utworzonych dokumentw RW/korekt RW
    
    # pobranie oglnego magazynu RW - jeli go nie ma to algorytm wogle nie ma dziaa
    SQL="SELECT COALESCE(magazyn_rw,'') FROM rs_dokumenty_magazyny "
    SQL+=" WHERE firma_id='"+self.firma_id+"'"
    SQL+=" AND magazyn_id='"+Mag+"' AND dokument_id='PW';"
    stat,DMagRW,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return 'Wystpiy bdy pobierania danych magazynu RW :'+stat

    # jeli go nie ma to koniec
    if len(DMagRW) == 0:
        return 'OK'
    
    if DMagRW[0][0] =='':
        return 'OK'
    
    OgolnyMagazynRW=DMagRW[0][0]
    
    # sprawdzamy czy przypadkiem nasz dokument nie ma ju wystawionego RW lub korekty RW
    SQL="SELECT firma_id FROM rs_dokumenty_magazynowe WHERE firma_id='"+self.firma_id+"' AND pw_magazyn_id='"+Mag+"'"
    SQL+=" AND pw_dokument_id='"+Dok+"' AND pw_data='"+Data+"' AND pw_numer="+str(Numer)
    stat,DTest,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return 'Wystpiy bdy pobierania danych z dokumentw magazynowych :'+stat
    if len(DTest) >0:
        return 'OK'
    
    # pobieramy z podanego dokumentu stan, informacj czy jest korekt i do jakiego dokumentu
    
    SQL="SELECT stan,COALESCE(kor_dm_dokument_id,''),kor_dm_magazyn_id,kor_dm_data,kor_dm_numer,COALESCE(opis,'')"
    SQL+=" FROM rs_dokumenty_magazynowe WHERE firma_id='"+self.firma_id+"' AND magazyn_id='"+Mag+"'"
    SQL+=" AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+str(Numer)
    stat,DDok,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return 'Wystpiy bdy pobierania danych z nagowka dokumentu :'+stat

    # podano zy klucz dokumentu - bd
    if len(DDok)==0:
        return 'Dokument o podanym kluczu nie istnieje'
    
    # dokument nie jest PW ani korekt PW - wychodzimy
    if not (Dok == 'PW' or DDok[0][1] == 'PW'):
        return 'OK'
    
    # dokument nie jest zamknity - wychodzimy
    if DDok[0][0] not in ('Z','K'):
        return 'OK'

    OpisKor=DDok[0][5]
    
    # pobieramy list zmienionych produktw. Tylko te warto analizowa (bo wszystko jest liczone z dokadnoci do produktu)
    SQL="SELECT DISTINCT artykul_id  FROM rs_dokumenty_magazynowe_pozycje WHERE firma_id='"+self.firma_id+"' AND magazyn_id='"+Mag+"'"
    SQL+=" AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+str(Numer)
    stat,DArt,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return 'Wystpiy bdy pobierania danych produktw z pozycji dokumentu :'+stat
    
    LArt=[]
    for poz in DArt:
        LArt.append(str(poz[0]))

    
    #-################
    #KOREKTA
    #-################
    
    # zakadamy, e nasz dokument to PW. Jeli nie, to poniej bdzie ponowne 
    #przypisanie wartoci zmiennym w sekcji korekt
    DokPW=Dok
    MagPW=Mag
    DataPW=Data
    NrPW=str(Numer)
    
    if not DDok[0][1] == '':
        # jeli to korekta to sprawdzamy, czy dokument PW mia wystawione RW
        # pobieramy list tych RW (moe by wicej ni 1)
        DokPW=DDok[0][1]
        MagPW=DDok[0][2]
        DataPW=DDok[0][3]
        NrPW=str(DDok[0][4])
        SQL="SELECT magazyn_id,dokument_id,data,numer,COALESCE(kontrahent_id,''),stan "
        SQL+=" FROM rs_dokumenty_magazynowe WHERE firma_id='"+self.firma_id+"'"
        SQL+=" AND pw_magazyn_id='"+MagPW+"' AND pw_dokument_id='"+DokPW+"'"
        SQL+=" AND pw_data='"+DataPW+"' AND pw_numer="+NrPW
        stat,DaneRW,dump,dump = self.PolSQL.Execute(SQL,3)
        if stat != 'OK':
            return 'Wystpiy bdy pobierania danych pierwotnego dokumentu RW :'+stat
    
        # dokument pierwotny PW nie mia RW - wychodzimy
        if len(DaneRW)==0:
            return 'OK'

        # sprawdzenie czy te dokumenty maj rne magazyny (jeli nie to bd)
        # i czy s zamknite  i od razu uzupeniamy zmienn LMagazZRW
        for dok in DaneRW:
            if dok[0] in LMagazZRW:
                return 'Magazyn '+dok[0]+' wystpi na 2 dokumentach RW dla PW. Tworzenie RW zatrzymane'
            if not dok[5] in ['Z','K']:
                return 'Dokument pierwotny '+dok[0]+'/'+dok[1]+'/'+dok[2]+'/'+str(dok[3])+' nie jest zamknity. Tworzenie RW zatrzymane'
            LMagazZRW.append(dok[0])

        # pobieramy list korekt do PW 
        SQL="SELECT magazyn_id,dokument_id,data,numer"
        SQL+=" FROM rs_dokumenty_magazynowe WHERE firma_id='"+self.firma_id+"'"
        SQL+=" AND kor_dm_magazyn_id='"+MagPW+"' AND kor_dm_dokument_id='"+DokPW+"'"
        SQL+=" AND kor_dm_data='"+DataPW+"' AND kor_dm_numer="+NrPW
        stat,LKorPW,dump,dump = self.PolSQL.Execute(SQL,3)
        if stat != 'OK':
            return 'Wystpiy bdy pobierania listy korekt PW :'+stat

        # pobieramy list korekt do RW 
        SQL="SELECT magazyn_id,dokument_id,data,numer"
        SQL+=" FROM rs_dokumenty_magazynowe WHERE firma_id='"+self.firma_id+"' AND ("
        PrzedL=''
        for pozRW in DaneRW:
            MagRW=pozRW[0]
            DokRW=pozRW[1]
            DataRW=pozRW[2]
            NrRW=str(pozRW[3])
            SQL+=PrzedL+" (kor_dm_magazyn_id='"+MagRW+"' AND kor_dm_dokument_id='"+DokRW+"'"
            SQL+=" AND kor_dm_data='"+DataRW+"' AND kor_dm_numer="+NrRW+")"    
            PrzedL=' OR '
        SQL+=")"
        
        stat,LKorRW,dump,dump = self.PolSQL.Execute(SQL,3)
        if stat != 'OK':
            return 'Wystpiy bdy pobierania listy korekt RW :'+stat
    
        # pobieramy ju wydane surowce (na podstawie pozycji RW i jego korekt)
        SQL="SELECT dmp.produkt_id,dmp.artykul_id,dmp.ilosc1,dmp.dokument_id,dmp.magazyn_id," # 0-4
        SQL+=" dmp.data,dmp.numer,tdm.typ_dokumentu,dmp.cena1,dmp.cena2 " # 5-9
        SQL+=" FROM rs_dokumenty_magazynowe_pozycje dmp "
        SQL+=" INNER JOIN rs_typy_dokumentow_m tdm USING(firma_id,dokument_id) WHERE dmp.firma_id='"+self.firma_id+"'"
        SQL+=" AND dmp.produkt_id IN ("+str(LArt)[1:-1]+") AND ("
        PrzedL=''
        for pozRW in DaneRW+LKorRW:
            MagRW=pozRW[0]
            DokRW=pozRW[1]
            DataRW=pozRW[2]
            NrRW=str(pozRW[3])
            SQL+=PrzedL+" (dmp.magazyn_id='"+MagRW+"' AND dmp.dokument_id='"+DokRW+"'"
            SQL+=" AND dmp.data='"+DataRW+"' AND dmp.numer="+NrRW+")"    
            PrzedL=' OR '
        SQL+=") ORDER BY dmp.produkt_id,dmp.artykul_id,dmp.data;"
        stat,DSurowceRW,dump,dump = self.PolSQL.Execute(SQL,3)
        rsglobal.ZapiszDoLogow('SQL \n' + repr(SQL))
        rsglobal.ZapiszDoLogow('DSurowceRW \n' + repr(DSurowceRW))
        if stat != 'OK':
            return 'Wystpiy bdy pobierania listy surowcw z RW i korekt :'+stat
        
    #-################
    # WSPLNIE
    #-################
    # pobranie pozycji produkcji z PW i (ewentualnie) jego korekt
    SQL="SELECT dmp.artykul_id,dmp.data,SUM(CASE WHEN tdm.typ_dokumentu='P' THEN dmp.ilosc1 ELSE -dmp.ilosc1 END) "
    SQL+=" FROM rs_dokumenty_magazynowe_pozycje dmp "
    SQL+=" INNER JOIN rs_typy_dokumentow_m tdm USING(firma_id,dokument_id) WHERE dmp.firma_id='"+self.firma_id+"'"
    SQL+=" AND dmp.magazyn_id='"+MagPW+"' AND dmp.artykul_id IN ("+str(LArt)[1:-1]+")"
    if len(LKorPW)==0:
        SQL+=" AND dmp.dokument_id='"+DokPW+"' AND dmp.data='"+DataPW+"' AND dmp.numer="+NrPW
    else:
        SQL+=" AND ((dmp.dokument_id='"+DokPW+"' AND dmp.data='"+DataPW+"' AND dmp.numer="+NrPW+")"
        for kor in LKorPW:
            SQL+=" OR (dmp.dokument_id='"+kor[1]+"' AND dmp.data='"+kor[2]+"' AND dmp.numer="+str(kor[3])+")"
        SQL+=")"
    SQL+=" GROUP BY dmp.artykul_id,dmp.data ORDER BY dmp.artykul_id,dmp.data;"

    stat,DProdukcjaPW,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return 'Wystpiy bdy pobierania listy produktw z PW i jego korekt :'+stat

    #WS 2024-01-31 - pobranie oddziau z magazynu
    sql ="SELECT oddzial_id FROM rs_magazyny"
    sql+=" WHERE firma_id='"+self.firma_id+"' AND magazyn_id='"+MagPW+"';"
    stat,d,dump,dump = self.PolSQL.Execute(sql,3)
    if stat != 'OK':
        return 'Bd pobrania oddziau dla magazynu: '+stat
    else:
        OddzialMag = d[0][0]
    
    #--- wyliczenie zuycia surowcw dla PW i (ewentualnie) jego korekt
    rsglobal.ZapiszDoLogow('SQL \n' + repr(SQL))
    rsglobal.ZapiszDoLogow('DProdukcjaPW \n' + repr(DProdukcjaPW))
    for poz in DProdukcjaPW:
        #WS 2024-01-31 - OddzialMag zamiast self.oddzial_id
        stat,DNagl,DSur=PodajReceptureDlaArtykulu(self.PolSQL,self.firma_id,OddzialMag,poz[0],poz[1])
        rsglobal.ZapiszDoLogow('DNagl \n' + repr(DNagl))
        rsglobal.ZapiszDoLogow('DSur \n' + repr(DSur))
        if stat != 'OK':
            return "Bd pobierania receptury dla artykuu "+poz[0].strip()+" na dzie "+poz[1]+" :"+stat
        if len(DNagl) == 0:
            return "Brak receptury dla artykuu "+poz[0].strip()+" na dzie "+poz[1]
        
        # przeliczenie surowcw na ilo z nagwka
        PrzelNagl=DNagl[0][1]

        for sklad in DSur:
            IloscSur=rsglobal.RSRound((poz[2]*sklad[2])/PrzelNagl,5) # (ilo prod * ilo sur)/ilo nag receptury
            MagTmp=sklad[1]
            if MagTmp =='':
                MagTmp = OgolnyMagazynRW
                
            KluczP=(MagTmp,poz[0],sklad[0]) # magazyn, produkt, surowiec
    
            # umieszczenie tej informacji w Sowniku surowcw
            if KluczP in SSurowce:
                SSurowce[KluczP] += IloscSur
            else:
                SSurowce[KluczP] = IloscSur
                
    #-################
    # KOREKTA
    #-################
    
    # jeli to korekta to porwnujemy surowce do wydania z tymi ju wydanymi
    # i zmieniamy iloci surowcowe do przyjcia/wydania

    rsglobal.ZapiszDoLogow('DSurowceRW \n' + repr(DSurowceRW))
    for surRW in DSurowceRW:
        KluczSur=(surRW[4],surRW[0],surRW[1]) # magazyn, produkt, surowiec
        if not KluczSur in SSurowce:
            SSurowce[KluczSur]=0
        
        # jeli typ przychodowy to dodajemy ilo do wydania, a jeli rozchodowy to odejmujemy
        if surRW[7]=='P':
            SSurowce[KluczSur]+=surRW[2]
        else:
            SSurowce[KluczSur]-=surRW[2]
        
    #-################
    #WSPLNIE
    #-################
        
    # tworzymy list magazynw do zmiany
    LMag=[]
    for kl in SSurowce.keys():
        if not kl[0] in LMag:
            LMag.append(kl[0])
    
    KluczeSur = SSurowce.keys()
    KluczeSur.sort()

    #-################
    # PTLA PO KOLEJNYCH MAGAZYNACH. DLA KADEGO POWSTANIE JEDEN DOKUMENT
    #-################

    for magsur in LMag:
        # pobranie do listy konkretnych danych do wydania/przyjcia na magazyn
        # jeli s zmieszane (to jaka awaria bo pocztkowy dokument zwiksza albo zmniejsza ilo surowcw) to wyjcie
        # jeli jest przyjcie a nie byo RW to te wyjcie (np. po zmianie magazynu RW dla PW)
        LPozZm=[]
        Plus=False
        Minus=False
        rsglobal.ZapiszDoLogow('SSurowce \n'+repr(SSurowce))
        for klsur in KluczeSur:
            if klsur[0]==magsur and SSurowce[klsur] != 0:
                rsglobal.ZapiszDoLogow('LPozZm.append([klsur[1],klsur[2],SSurowce[klsur],0,0]) '+repr(([klsur[1],klsur[2],SSurowce[klsur],0,0])))
                LPozZm.append([klsur[1],klsur[2],SSurowce[klsur],0,0]) # produkt, artykul, ilosc1,cena1,cena2
                if SSurowce[klsur] > 0:
                    Plus=True
                else:
                    Minus=True
            else:
                rsglobal.ZapiszDoLogow('Nie weszo \n klsur[0]==magsur'+repr(klsur[0]==magsur)+'SSurowce[klsur] != 0'+repr(SSurowce[klsur] != 0))
            
            if Plus and Minus:
                return "Bdy danych dla magazynu "+magsur+" - program chce na raz stworzy dokument przychodowy i rozchodowy "
            if Minus and DDok[0][1] == '': # to nie korekta
                return "Bdy danych dla magazynu "+magsur+" - program chce stworzy dokument przychodowy tworzc podstawowy dokument RW "
    
        # jeli dokument przychodowy - przygotowanie pozycji rozbitych w stosunku do tego co byo wydawane
        # analizujemy pozycje z RW od tyu i wykorzystujemy kolejne ceny. Jeli przekroczymy ilo do przyjcia 
        # i nie moemy wyznaczy ceny - bd krytyczny
        if Minus:
            SPozZmTmp={}
            # kad pozycj do produkcji rozpatrujemy osobno
            DSurowceRW.reverse() # bdziemy je przeglda "od koca"
            for poz in LPozZm:
                SPrzyjec={}
                Produkt=poz[0]
                Art=poz[1]
                IloscP=-1*poz[2]
                for pozRW in DSurowceRW:
                    if IloscP >0:
                        if Produkt == pozRW[0] and Art == pozRW[1]:
                            # korekta przychodowa RW
                            KlRW = (pozRW[0],pozRW[1],pozRW[8],pozRW[9])
                            if pozRW[7] == 'P':
                                if not KlRW in SPrzyjec:
                                    SPrzyjec[KlRW] = 0
                                    
                                SPrzyjec[KlRW] += pozRW[2]
                            # RW lub korekta rozchodowa
                            else:
                                Ilosc = rsglobal.RSRound(pozRW[2],8)
                                # jeli bya na przyjciu to zmniejszamy i sprawdzamy czy jeszcze co zostao na wydaniu
                                if KlRW in SPrzyjec:
                                    if Ilosc < SPrzyjec[KlRW]:
                                        SPrzyjec[KlRW] -= Ilosc
                                        Ilosc = 0
                                    else:
                                        Ilosc -= SPrzyjec[KlRW]
                                        del SPrzyjec[KlRW]
                                
                                # uwzgldniamy wydanie na przyjciu
                                if Ilosc > 0:
                                    if not KlRW in SPozZmTmp:
                                        SPozZmTmp[KlRW] = 0
                                    
                                    SPozZmTmp[KlRW] += min(Ilosc,IloscP)
                                    IloscP -= Ilosc
                            
                if IloscP>0:
                    stat = "Bd pobierania cen na korekt przychodow dla magazynu "+magsur+". Artyku "+Art.strip()+" dla produktu "+Produkt.strip()+" nie zosta wydany w odpowiedniej iloci."
                    stat+= " Zostao "+str(IloscP)+". Moliwe, e nastpia prba wydania artykuu iloci wikszej ni wystpuje na PW."
                    return stat
                
                
            # zmieniamy sownik SPozZmTmp na liste pozycji do utworzenia
            KluczePozTmp = SPozZmTmp.keys()
            KluczePozTmp.sort()
            rsglobal.ZapiszDoLogow('SPozZmTmp \n'+repr(SPozZmTmp))
            LPozZm=[]
            for kl in KluczePozTmp:
                LPozZm.append([kl[0],kl[1],SPozZmTmp[kl],kl[2],kl[3]])

        Opis=''
        PodstMagRW='NULL'
        PodstDokRW='NULL'
        PodstDataRW='NULL'
        PodstNumerRW='NULL'
        
        #-################
        # KOREKTA
        #-################
        
        if magsur in LMagazZRW: # mamy RW wic bdzie korekta
            # podanie klucza podstawowego RW do ktrego bdzie korekta. Wyszukujemy po magazynie
            for dokRW in DaneRW:
                if dokRW[0] == magsur:
                    PodstMagRW="'"+dokRW[0]+"'"
                    PodstDokRW="'"+dokRW[1]+"'"
                    PodstDataRW="'"+dokRW[2]+"'"
                    PodstNumerRW=str(dokRW[3])
        
            Opis=OpisKor
            
            # pobranie dokumentu korekty RW (przyjcia lub wydania)
            # jeli nie ma dokumentu lub jest kilka to wychodzimy z algorytmu z bdem
            TypTmp='W'
            if Minus:
                TypTmp='P'
            
            stat,DokDoc=PodajDokumentKorekty(self,'RW',TypTmp)
            if stat<>'OK': # bdy analizowane s w funkcji
                return stat
            
            # pobranie kontrahenta z RW - analiza DaneRW
            Kontrahent='NULL'
            for poz in DaneRW:
                if poz[0] == magsur:
                    if poz[4] <>'':
                        Kontrahent = "'"+poz[4]+"'"
            
        else:
            #-################
            # ZWYKE RW
            #-################
            DokDoc='RW'            
            
            # pobranie kontrahenta na RW
            SParam = {}
            SParam['wersja'] = ''
            SParam['kontrahent_id'] = ''
            rsglobal.ZapiszDoLogow(magsur,5)
            stat,SParam = fs_magazyny_parametry_dokumentow_pom.UzupelnijDomyslneParametryDlaDokumentu(self,magsur,'RW',SParam,True,False)
            if stat != 'OK':
                return stat 
            
            Kontrahent = SParam['kontrahent_id']
            
            if Kontrahent in ('','NULL'):
                Kontrahent = 'NULL'
            else:
                Kontrahent = "'"+Kontrahent+"'"

        #-################
        #WSPLNIE
        #-################
        rsglobal.ZapiszDoLogow('!!! Kontrahent '+str(Kontrahent),5)
        rsglobal.ZapiszDoLogow('!!! Kontrahent '+str(Kontrahent),5)
        # sprawdzenie czy mona doda dokument zgodnie z numeracj
        SQL ="SELECT rsf_sprawdz_czy_mozna_dodac_dokument('"+self.firma_id+"',"
        SQL+="'[\\\'"+magsur+"\\\',\\\'"+DokDoc+"\\\',\\\'"+Data+"\\\']','M');"            

        stat,dane,dump,dump = self.PolSQL.Execute(SQL,3)
        if stat != 'OK':
            return "Wystpiy bdy podczas sprawdzania moliwoci wystawienia dokumentu "+DokDoc+" z magazynu "+magsur+" z dat "+Data+": "+stat
        if dane[0][0] != 'OK':    
            return 'Do magazynu '+magsur+' nie mona doda dokumentu z dat '+Data

        # pobranie kolejnego numeru dokumentu
        SQL ="SELECT rsf_podaj_nowy_numer_dokumentu('"+self.firma_id+"',"
        SQL+="'[\\\'"+magsur+"\\\',\\\'"+DokDoc+"\\\',\\\'"+Data+"\\\']','M');"

        stat,DNumer,dump,dump = self.PolSQL.Execute(SQL,3)
        if stat != 'OK':
            return "Wystpiy bdy podczas pobierania nowego numeru dokumentu "+DokDoc+" z magazynu "+magsur+" z dat "+Data+": "+stat
        if DNumer[0][0] in (None,'None',''):
            return 'Bd pobrania nowego numeru dla '+Dokument+' z magazynu '+Magazyn
        
        NumerDoc=str(DNumer[0][0])
        
        if Plus:
            PoleMag='magazyn_skad'
        else:
            PoleMag='magazyn_dokad'

        if len(LPozZm):  # s pozycje do zmiany
            # utworzenie nagwka nowego dokumentu RW (kor RW)
            INS="INSERT INTO rs_dokumenty_magazynowe"
            INS+=" (firma_id, magazyn_id, dokument_id,"
            INS+=" data, numer, "+PoleMag+", kontrahent_id, opis,"
            INS+=" pw_magazyn_id, pw_dokument_id, pw_data, pw_numer,"
            INS+=" kor_dm_magazyn_id, kor_dm_dokument_id, kor_dm_data, kor_dm_numer)"
            INS+=" VALUES "
            INS+=" ('"+self.firma_id+"','"+magsur+"','"+DokDoc+"',"
            INS+="'"+Data+"',"+NumerDoc+",'"+magsur+"',"+Kontrahent+",'"+Opis+"',"
            INS+="'"+Mag+"','"+Dok+"','"+Data+"',"+str(Numer)+","
            INS+=PodstMagRW+","+PodstDokRW+","+PodstDataRW+","+str(PodstNumerRW)+"); "
            stat,dump,dump,dump = self.PolSQL.Execute(INS,1)
            rsglobal.ZapiszDoLogow('INS \n'+INS)
            if stat != 'OK':
                return "Wystpiy bdy podczas tworzenia nagwka dokumentu "+DokDoc+" z magazynu "+magsur+" z dat "+Data+": "+stat

            # utworzenie pozycji nowego dokumentu RW (korekty RW)
            Lp=1
            INS=''
            for poz in LPozZm:
                Ilosc1=str(poz[2])
                Ilosc2="ROUND("+Ilosc1+"*(SELECT przelicznik FROM rs_artykuly WHERE firma_id='"+self.firma_id+"' AND artykul_id='"+poz[1]+"'),4)"
                Cena1=str(poz[3])
                Cena2=str(poz[4])
                INS+="INSERT INTO rs_dokumenty_magazynowe_pozycje"
                INS+=" (firma_id, magazyn_id, dokument_id, data,"
                INS+="  numer, nr_poz, nr_ppoz, artykul_id,"
                INS+="  ilosc1, ilosc2,"
                INS+="  cena1, cena2,"
                INS+="  cena_wal1, cena_wal2,produkt_id)"
                INS+=" VALUES"
                INS+=" ('"+self.firma_id+"','"+magsur+"','"+DokDoc+"','"+Data+"',"
                INS+="  "+NumerDoc+","+str(Lp)+",0,'"+poz[1]+"',"
                INS+="  "+Ilosc1+","+Ilosc2+","
                INS+="  "+Cena1+","+Cena2+","
                INS+="  "+Cena1+","+Cena2+",'"+poz[0]+"'); "

                Lp += 1

            stat,dump,dump,dump = self.PolSQL.Execute(INS,1)
            rsglobal.ZapiszDoLogow('INS2 \n'+INS)
            if stat != 'OK':
                return "Wystpiy bdy podczas wstawiania pozycji dokumentu "+DokDoc+" z magazynu "+magsur+" z dat "+Data+": "+stat
        
            LUtwDokum.append([magsur,DokDoc,Data,NumerDoc])
    
    # !!! ten commit do usunicia przy przywracaniu blokady zamykania PW bez RW
    self.PolSQL.Commit()
    
    # dokumenty zostay utworzone. W ptli prbujemy zamkn utworzone dokumenty RW/korekty RW z W na Z
    rsglobal.ZapiszDoLogow('zamykanie \n')
    for dok in LUtwDokum:
        # wersja bez blokady
        # ZamknijDokumentMagazynowy(self,dok[0],dok[1],dok[2],dok[3],'W','Z',True)

        # wersja z blokad tylko gdy wynika z blokady w bazie  #AB 2023-11-27
        stat = ZamknijDokumentMagazynowy(self,dok[0],dok[1],dok[2],dok[3],'W','Z',True)
        if stat.startswith("Dostp do tabeli bazy danych jest zablokowany"):
            return stat
        
        # wersja z blokad
        #stat=ZamknijDokumentMagazynowy(self,dok[0],dok[1],dok[2],dok[3],'W','Z',False)        
        #if stat<>'OK':
        #    return 'Bd przygotowania/zamykania dokumentu '+dok[0]+'/'+dok[1]+'/'+dok[2]+'/'+str(dok[3])+' :\n '+stat
    
    # !!! Na koniec robimy Commit - do przywrcenia przy przywracaniu blokady RW z PW
    # self.PolSQL.Commit()
    
    return 'OK'


#-------------------------------------------------------------------------------
#-- FUNKCJA ZMIENIAJCA ODPOWIEDNIE ILOCI NA -1, DLA ARTYKUW PROPORCJONALNYCH
#-------------------------------------------------------------------------------

def UstawIlosciProporcjonalneNaMinus1(self,Mag,Dok,Data,Numer):
    # pobranie danych pozycji
    SQL="SELECT dmp.nr_poz,dmp.nr_ppoz,a.jm_kalk"
    SQL+=" FROM rs_dokumenty_magazynowe_pozycje dmp INNER JOIN rs_artykuly a USING (firma_id,artykul_id)"
    SQL+=" WHERE dmp.firma_id='"+self.firma_id+"' AND dmp.magazyn_id='"+Mag+"' AND dmp.dokument_id='"+Dok+"'"
    SQL+=" AND dmp.data='"+Data+"' AND dmp.numer="+str(Numer)
    SQL+=" AND a.dynamiczny_przelicznik AND a.wydanie_proporcjonalne"
    stat,DPozycje,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return "Wystpiy bdy podczas pobierania pozycji do zmiany iloci na -1 :"+stat
        
    # zmiana wybranych wierszy
    for poz in DPozycje:
        if poz[2] =='1':
            pole='ilosc2'
        else:
            pole='ilosc1'
        
        SQL="UPDATE rs_dokumenty_magazynowe_pozycje SET "+pole+"=-1 "
        SQL+=" WHERE firma_id='"+self.firma_id+"' AND magazyn_id='"+Mag+"' AND dokument_id='"+Dok+"'"
        SQL+=" AND data='"+Data+"' AND numer="+str(Numer)+" AND nr_poz="+str(poz[0])+" AND nr_ppoz="+str(poz[1])+";"
        stat,dump,dump,dump = self.PolSQL.Execute(SQL,1)
        if stat != 'OK':
            return "Wystpiy bdy podczas zmiany iloci pozycji na -1 :"+stat
        
    return 'OK'

#-------------------------------------------------------------------------------
#-- FUNKCJA POBIERAJCA LIMITY W TERMINIE PRZYDATNOCI
# Art - artyku dla ktrego prowadzimy analiz
# Kontr - kontrahent dla ktrego prowadzimy analiz
# Typ - typ dokumentu (P - przychodowy, W - rozchodowy)
# 
# UWAGA!!! Zmiana tej funkcji musi te zmieni funkcj w rs_soap_funkcje_pomocnicze
# o tej samej nazwie
#
#-------------------------------------------------------------------------------

def PodajLimityTerminuPrzydatnosci(self,Art,Kontr,Typ):
    LimitDni = 0
    LimitProc = 0
    if Art.strip() == '' or Kontr.strip() == '':
        return 'OK',[0,0]
    
    Art = rsglobal.Dekoduj(Art)
    Kontr = rsglobal.Dekoduj(Kontr)
    Typ = rsglobal.Dekoduj(Typ)
    
    # sprawdzamy czy artyku ma wpisany termin i pobieramy liczb dni
    SQL="SELECT czy_termin,dni_przydatnosci FROM rs_artykuly "
    SQL+=" WHERE firma_id='"+self.firma_id+"' AND artykul_id='"+Art+"';"
    stat,DArt,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return u"Wystpiy bdy pobierania danych artykuu :"+stat,[0,0]
    
    if not DArt[0][0]:
        return 'OK',[0,0]
    
    if len(DArt) == 0:
        return u"Brak artykuu o symbolu "+Art.strip(),[0,0]
    
    DniPrzyd = DArt[0][1]
    
    # sprawdzamy czy artyku ma wpisany limit w rs_artykuly_limity_przydatnosci
    SQL="SELECT limit_procent,limit_dni FROM rs_artykuly_limity_przydatnosci "
    SQL+=" WHERE firma_id='"+self.firma_id+"' AND artykul_id='"+Art+"'"
    SQL+=" AND kontrahent_id='"+Kontr+"' AND typ='"+Typ+"';"
    stat,DLimit,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return u"Wystpiy bdy pobierania limitw przydatnoci artykuu :"+stat,[0,0]
    
    if len(DLimit) == 0:
        return 'OK',[0,0]
    
    LimitProc = DLimit[0][0]
    LimitDni = DLimit[0][1]
    
    # sprawdzamy czy jeli limit jest % to ma wpisane dni w artykule
    if LimitProc > 0:
        if DniPrzyd == 0:
            return u"Dla artykuu "+Art.strip()+" istnieje limit procentowy, ale nie podano dni przydatnoci w danych artykuu",[0,0]
    
        # wyliczamy liczb dni do limitu jeli limit %
        LimitDni = rsglobal.RSRound((LimitProc * 0.01) * DniPrzyd,2)
        
    return 'OK',[LimitDni,LimitProc]


#-------------------------------------------------------------------------------
#-- FUNKCJA POBIERAJCA OPISY I ILOCI DLA OPAKOWANIA/PALETY
# Art - artyku dla ktrego prowadzimy analiz
# Cecha - cecha analizowanego artykuu
# Kontr - kontrahent dla ktrego prowadzimy analiz
# Opak - symbol analizowanego opakowania
# Paleta - symbol analizowanej palety
# DopuszczacPuste - czy ma by sprawdzenie tego co zostao podane (False) czy te wyszukanie domylnego (True)
#
# LDane - lista z danymi [opakowanie_id, opis opakowania, ilo w opakowaniu,paleta_id, opis palety, ilo opakowa w palecie]
#-------------------------------------------------------------------------------


def PodajOpisIIlosciOpakowaniaIPalety(self,Art,Cecha,Kontr,Opak,Paleta,DopuszczacPuste=False):
    import rsglobal
    stat = 'OK'
    Art = rsglobal.Dekoduj(Art)
    Cecha = rsglobal.Dekoduj(Cecha)
    Kontr = rsglobal.Dekoduj(Kontr)
    Opak = rsglobal.Dekoduj(Opak)
    Paleta = rsglobal.Dekoduj(Paleta)

    LDane = [Opak,'',0,Paleta,'',0]
    # sprawdzenie czy artyku ma zaznaczon obsug opakowa. Jeli nie to nic nie sprawdzamy
    SQL= "SELECT COALESCE(tylko_w_opakowaniach,FALSE) FROM rs_artykuly "
    SQL+=" WHERE firma_id='"+self.firma_id+"' AND artykul_id='"+Art+"'"
    stat,DArt,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return u'Bd pobierania danych artykuu : '+stat,LDane
    elif len(DArt) == 0:
        return u'Podano bdny kod artykuu : '+Art,LDane
    elif not DArt[0][0]:
        return 'OK',LDane
    
    # pobieranie, jeli opakowanie/paleta musz by podane
    if not DopuszczacPuste:
        # pobranie opisu opakowania (sprawdzenie poprawnoci)
        if Opak != '':
            SQL = "SELECT opis FROM rs_opakowania_zbiorcze WHERE firma_id='"+self.firma_id+"' AND kod='"+Opak+"';" 
            stat,DOpak,dump,dump = self.PolSQL.Execute(SQL,3)
            if stat != 'OK':
                return u'Bd pobierania opisu opakowania : '+stat,LDane
            if len(DOpak) > 0:
                LDane[1] = DOpak[0][0]
            
        # pobranie opisu palety (sprawdzenie poprawnoci)
        if Paleta != 'OK':
            SQL = "SELECT opis FROM rs_palety WHERE firma_id='"+self.firma_id+"' AND paleta_id='"+Paleta+"';" 
            stat,DPal,dump,dump = self.PolSQL.Execute(SQL,3)
            if stat != 'OK':
                return u'Bd pobierania opisu palety : '+stat,LDane
            if len(DPal) > 0:
                LDane[4] = DPal[0][0]

    # jeli oba s podane i prawidowe to prbujemy znale par z rs_artykuly_opakowania_zbiorcze
    # z uwzgldnieniem daty do 
    if (LDane[1] != '' and LDane[4] != '') or DopuszczacPuste:
        SQL = "SELECT COALESCE(kontrahent_id,''),opakowanie_id,ilosc1,paleta_id,pojemnosc_paletowa,data_do"
        SQL+=" FROM rs_artykuly_opakowania_zbiorcze "
        SQL+=" WHERE firma_id = '"+self.firma_id+"' AND artykul_id = '"+Art+"' "
        SQL+=" AND NOT opakowanie_id IS NULL AND NOT paleta_id IS NULL "
        if Cecha != '':
            SQL+=" AND CASE WHEN COALESCE(cecha1_wartosc,'') = '' THEN '"+Cecha+"' ELSE cecha1_wartosc END = '"+Cecha+"'"
        if Kontr != '':
            SQL+=" AND COALESCE(kontrahent_id,'"+Kontr+"') = '"+Kontr+"'"
        if Opak != '':
            SQL+=" AND opakowanie_id = '"+Opak+"'"
        if Paleta != '':
            SQL+=" AND paleta_id = '"+Paleta+"'"
        SQL+=" ORDER BY COALESCE(kontrahent_id,'') DESC,COALESCE(cecha1_wartosc,'') DESC, data_do DESC"
        stat,DZbior,dump,dump = self.PolSQL.Execute(SQL,3)
        rsglobal.ZapiszDoLogow('SQL ZBIOR '+SQL,5)
        rsglobal.ZapiszDoLogow('DZbior '+str(DZbior),5)
        if stat != 'OK':
            return u'Bd pobierania danych z artykuw opakowa zbiorczych : '+stat,LDane
        if len(DZbior) > 0:
            LDane[0] = DZbior[0][1]
            LDane[2] = DZbior[0][2]
            LDane[3] = DZbior[0][3]
            LDane[5] = DZbior[0][4]
            
    # pobieranie, jeli mogy by puste opakowanie/paleta
    if DopuszczacPuste:
        # pobranie opisu opakowania
        if LDane[0] != '':
            SQL = "SELECT opis FROM rs_opakowania_zbiorcze WHERE firma_id='"+self.firma_id+"' AND kod='"+LDane[0]+"';" 
            stat,DOpak,dump,dump = self.PolSQL.Execute(SQL,3)
            if stat != 'OK':
                return u'Bd pobierania opisu opakowania : '+stat,LDane
            if len(DOpak) > 0:
                LDane[1] = DOpak[0][0]
            
        # pobranie opisu palety
        if LDane[3] != '':
            SQL = "SELECT opis FROM rs_palety WHERE firma_id='"+self.firma_id+"' AND paleta_id='"+LDane[3]+"';" 
            stat,DPal,dump,dump = self.PolSQL.Execute(SQL,3)
            if stat != 'OK':
                return u'Bd pobierania opisu palety : '+stat,LDane
            if len(DPal) > 0:
                LDane[4] = DPal[0][0]
            
    return stat,LDane

#-------------------------------------------------------------------------------
# FUNKCJA POBIERAJCA OPISY I ILOCI DLA OPAKOWANIA/PALETY 
# NA CAY DOKUMENT
#-------------------------------------------------------------------------------

def UzupelnijOpakowaniaIPaletyDokumentu(self,Mag,Dok,Data,Numer):
    # pobranie kontrahenta
    SQL = "SELECT COALESCE(kontrahent_id,'') FROM rs_dokumenty_magazynowe WHERE firma_id='"+self.firma_id+"'"
    SQL+= " AND magazyn_id='"+Mag+"' AND dokument_id = '"+Dok+"' AND data='"+Data+"' AND numer = "+str(Numer)
    stat,DDok,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return u'Bd pobierania danych dokumentu magazynowego: '+stat
    Kontr = DDok[0][0]
    
    # pobranie pozycji dokumentu
    SQL = "SELECT nr_poz,nr_ppoz,COALESCE(artykul_id,''),COALESCE(cecha1_wartosc,''),COALESCE(opakowanie_id,''),"
    SQL+= " COALESCE(paleta_id,'') FROM rs_dokumenty_magazynowe_pozycje WHERE firma_id='"+self.firma_id+"'"
    SQL+= " AND magazyn_id='"+Mag+"' AND dokument_id = '"+Dok+"' AND data='"+Data+"' AND numer = "+str(Numer)
    stat,DPoz,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return u'Bd pobierania danych pozycji dokumentu magazynowego: '+stat
    
    # uycie funkcji pomocniczej pobierajcej dane opakowania/palety
    for poz in DPoz:
        if poz[2] != '':
            stat,LOpak = PodajOpisIIlosciOpakowaniaIPalety(self,poz[2],poz[3],Kontr,poz[4],poz[5],True)
            if stat != 'OK':
                return u'Bd pobierania informacji o opakowaniach: '+stat
    
            Bylo = False
            UPD = "UPDATE rs_dokumenty_magazynowe_pozycje SET "
            
            # jeli zmienione opakowanie
            if poz[4] != LOpak[0]:
                if LOpak[0] == '':
                    UPD +=" opakowanie_id = NULL"
                else:
                    UPD +=" opakowanie_id = '"+LOpak[0]+"'"
                Bylo = True
            
            # jeli zmieniona paleta    
            if poz[5] != LOpak[3]:
                if Bylo:
                    UPD+=","
                if LOpak == '':
                    UPD +=" paleta_id = NULL"
                else:
                    UPD +=" paleta_id = '"+LOpak[3]+"'"
                Bylo = True
            
            if Bylo:
                UPD+=","
            UPD += "wsp_ilosci_opakowanie = "+str(LOpak[2])
            UPD += ",wsp_ilosci_paleta = "+str(LOpak[5])
            UPD += " WHERE firma_id='"+self.firma_id+"' AND magazyn_id='"+Mag+"' AND dokument_id = '"+Dok+"'"
            UPD += " AND data='"+Data+"' AND numer = "+str(Numer)+" AND nr_poz="+str(poz[0])+" AND nr_ppoz="+str(poz[1])
            stat,dump,dump,dump = self.PolSQL.Execute(UPD,1)
            if stat != 'OK':
                return u'Bd zmiany opakowania na pozycji dokumentu magazynowego: '+stat
            
    # ustawienie iloci lokacyjnej na caym dokumencie
    from funkcje import fs_dokumenty_magazynowe_lokacje_pom
    stat = fs_dokumenty_magazynowe_lokacje_pom.UstawWspIIloscLokacyjnaNaDM(self,Mag,Dok,Data,Numer,0,0,False)
    
    return stat
    
#-------------------------------------------------------------------------------
# FUNKCJA ZMIENIAJCA RӯNE ILOCI NA ZLECENIACH PODCZAS OTWIERANIA/ZAMYKANIA DOKUMENTU
#-------------------------------------------------------------------------------

def ZmienIlosciNaZleceniachDlaDokumentu(self,Mag,Dok,Data,Numer,Znak):
    stat = 'OK'
    LZlecen = []
    
    #ewentualne cignicie iloci rezerwowanych/produkowanych itp na zleceniu z zamwienia odbiorcy
    SQL = "SELECT dmp.typ_zlec,dmp.zo_wyznacznik_id,dmp.zo_data,dmp.zo_numer,"
    SQL+=" dmp.zo_nr_poz,dmp.zo_nr_ppoz,dmp.zo_nr_zlec,dmp.ilosc1,zoz.ilosc_calk "
    SQL+=" FROM rs_dokumenty_magazynowe_pozycje dmp"
    SQL+=" INNER JOIN rs_zamowienia_odbiorcy_pozycje_zlecenia zoz "
    SQL+=" ON (zoz.firma_id=dmp.firma_id AND zoz.wyznacznik_id = dmp.zo_wyznacznik_id "
    SQL+=" AND zoz.data = dmp.zo_data AND zoz.numer = dmp.zo_numer AND zoz.nr_poz = dmp.zo_nr_poz "
    SQL+=" AND zoz.nr_ppoz = dmp.zo_nr_ppoz AND zoz.nr_zlec = dmp.zo_nr_zlec) "
    SQL+=" WHERE dmp.firma_id='"+self.firma_id+"' AND dmp.magazyn_id='"+Mag+"'"
    SQL+=" AND dmp.dokument_id='"+Dok+"' AND dmp.data='"+Data+"' AND dmp.numer="+str(Numer)
    SQL+=" ORDER BY dmp.nr_poz,dmp.nr_ppoz;"
    stat,DPozZlec,dump,dump = self.PolSQL.Execute(SQL,3)   
    if stat != 'OK':
        return 'Bd pobierania informacji o zleceniach na pozycji dokumentu magazynowego: '+stat
        
    UPD = ''
    for poz in DPozZlec:
        TypZl = poz[0]
        IloscZl = str(poz[7])
        # Wspczynnik iloci wzgldem caoci pozwala atwo dziaa na podzleceniach
        WspIl = '('+str(poz[7])+'/'+str(poz[8])+')'
        
        FiltrZl = " WHERE firma_id='"+self.firma_id+"' AND wyznacznik_id='"+poz[1]+"' AND data='"+poz[2]+"'"
        FiltrZl+= " AND numer="+str(poz[3])+" AND nr_poz="+str(poz[4])+" AND nr_ppoz="+str(poz[5])
        KlZlec = [poz[1],poz[2],str(poz[3]),str(poz[4]),str(poz[5]),poz[6].strip()]
        if not KlZlec in LZlecen:
            LZlecen.append(KlZlec)
            
        # zmieniamy ilo zarezerwowan na zleceniu i zrealizowan na zleceniu i podlzeceniach
        if poz[0] == 'R':
            UPD += "UPDATE rs_zamowienia_odbiorcy_pozycje_zlecenia SET ilosc_rezerwacja = ilosc_rezerwacja "+Znak+" "+IloscZl
            UPD += FiltrZl+" AND nr_zlec = '"+poz[6]+"';"
            UPD += "UPDATE rs_zamowienia_odbiorcy_pozycje_zlecenia SET ilosc_zrealizowana = ilosc_zrealizowana "+Znak+" (ilosc_calk * "+WspIl+")"
            UPD += FiltrZl+" AND TRIM(nr_zlec) LIKE '"+poz[6].strip()+"%';"
        
        # zmieniamy ilo wykonan i zrealizowan na zleceniu
        if poz[0] == 'W':
            UPD += "UPDATE rs_zamowienia_odbiorcy_pozycje_zlecenia SET ilosc_wykonana = ilosc_wykonana "+Znak+" "+IloscZl+","
            UPD += "ilosc_zrealizowana = ilosc_zrealizowana "+Znak+" "+IloscZl
            UPD += FiltrZl+" AND nr_zlec = '"+poz[6]+"';"
    
    if UPD != '':
        stat,dump,dump,dump = self.PolSQL.Execute(UPD,1)   
        if stat != 'OK':
            return 'Bd zmiany zlece zapytaniem :\n'+UPD+' \n\n '+stat
        
        # sprawdzenie poprawnoci zlece, ktre zostay zmienione
        from funkcje import fs_zamowienia_odbiorcy_pozycje_zlecenia_pom
        for KlZlec in LZlecen:
            stat = fs_zamowienia_odbiorcy_pozycje_zlecenia_pom.SprawdzPoprawnoscZlecen(self,KlZlec[0],KlZlec[1],KlZlec[2],KlZlec[3],KlZlec[4],KlZlec[5])
            if stat != 'OK':
                return stat
    
    return 'OK'

#-------------------------------------------------------------------------------
#- PODAJE NUMER PRZYJCIA Z KLUCZA DOKUMENTU MAGAZYNOWEGO
#-------------------------------------------------------------------------------

def PodajNumerPrzyjecia(self,Mag,Dok,Data,Numer,NrPoz,NrPPoz):
    nr = "0000"+str(Numer)
    nr = nr[len(nr)-6:len(nr)-2]

    poztmp = "0000"+str(NrPoz)
    poztmp = poztmp[len(poztmp)-6:len(poztmp)-2]

    ppoztmp = "0000"+str(NrPPoz)
    ppoztmp = ppoztmp[len(ppoztmp)-6:len(ppoztmp)-2]
    
    magtmp=Mag+'    '
    
    dok_prz = self.firma_id+magtmp[:4]+Dok+str(Data)+nr+poztmp+ppoztmp
    
    return dok_prz

#-------------------------------------------------------------------------------

def PobierzKlucze(Mag, Typ, Data, Numer, Funkcja):
    Klucze={}
    Klucze['Mag'] = Mag
    Klucze['Typ'] = Typ
    Klucze['Data'] = Data
    Klucze['Numer'] = Numer
    Klucze['Funkcja'] = Funkcja
    return Klucze
