# -*- coding: iso-8859-2 -*-
#-------------------------------------------------------------------------------
# Name:        fs_edeklaracje_aktualizacja.py
# Purpose:     Pobiera z rzadowej witryny liste schematow edeklaracji i pliki .xsd
#
# Author:      Micha Raburski
#
# Created:     2012-08-07
# RCS-ID:      $Id: fs_edeklaracje_aktualizacja.py $
# Copyright:   (c) 2004-2020 Record System
# Licence:     GNU GPL
#-------------------------------------------------------------------------------
# 2020-08-25 Aleksander Bargie(05984) Edeklaracje - lista wersji konkretnych typw deklaracji(udronienie po zmianach)
#  	2022-09-13 Aleksander Bargie(12192) Czysta - bd funkcji w harmonogramie

import os
import urllib
import requests
import datetime


from lxml import etree
#AB 24.08.20r. przejscie na bs4 bo bs nie mialo aktualizacji od 2011r. i walilo bledami
#from BeautifulSoup import BeautifulSoup
from bs4 import BeautifulSoup


def AktualizujDeklaracje(Wzorce, Sciezka, BazaDanych):
    u = Aktualizator(Sciezka, Wzorce, BazaDanych)
    u.Aktualizuj()
    return u.NOWE

#-#############################################################################
#--- Klasa: Aktualizator
#-#############################################################################

class Aktualizator:
    '''
    Klasa aktualizujca deklaracje z serwisu eDeklaracje. 
    Parametrami s: 
        Sciezka    - Gdzie deklaracje maj si mieci na dysku (tworzy ciek jeli nie istnieje),
        Wzorce     - Nazwy deklaracji branych pod uwag (np. ['CIT-6AR', 'PIT-39']); Wzorce musz by podawane w licie,
        BazaDanych - Wypis z bazy danych o deklaracjach ju dodanych w postaci listy list. 
    '''

    NOWE       = [] #Trzyma list elementw, ktre zostan dodane do bazy danych, a nastpnie zapisane na dysk.
    WSZYSTKIE  = [] #Trzyma list wszytkich elementw pobranych w aktualnym sprawdzaniu aktualizacji.
  
    def __init__(self, Sciezka, Wzorce, BazaDanych):
        #AB 24.08.20r. usuniety zbedny import
        # import rsglobal

        #AB 24.08.20r.
        URL = "https://www.podatki.gov.pl/e-deklaracje/dokumentacja-it/struktury-dokumentow-xml/"

        #print 9,URL
        html = requests.get(URL).text.encode('utf-8')
        #AB 24.08.20r. replace rozszerzony o inne dodatkowe biale znaki
        html = html.replace('&nbsp;', '').replace('\xa0', '').replace('\xc2', '').replace('\n', '') #Usuwa brzydkie rzeczy z kodu html pobranego z eDeklaracje, ktre mogy spowodowa bdy.

        #AB 24.08.20r. skorzystanie z bs by pobrac tabele
        # html_tabelka = self.PobierzTabele(html)
        # # print 12,'html_tabelka' ,html_tabelka
        # self.tabelka = BeautifulSoup(
        #     html_tabelka).__str__()  # BeautifulSoup wykorzystany jest aby wczyta tabelk i poprawi budow html'a
        # # print 13 ,'self.tabelka',self.tabelka     #pod uycie z lxml.
        self.tabelka  = BeautifulSoup(html, 'lxml').find('table')

        self.sciezka = Sciezka
        self.wzorce = Wzorce
        self.bazadanych = BazaDanych

#------------------------------------------------------------------------------
#--- Funkcja: Aktualizuj
#------------------------------------------------------------------------------

    def Aktualizuj(self):
        '''
        Funkcja wywolujaca wszyskie inne funkcje potrzebne do aktualizacji, 
        sciagniecia i sprawdzenia
        '''

        self.PobierzDeklaracje()
        self.FiltrujDeklaracje()
        self.SprawdzCzyNowe()
        self.ZapiszNaDysk()

#------------------------------------------------------------------------------
#--- Funkcja: PobierzDeklaracje
#------------------------------------------------------------------------------

    def PobierzDeklaracje(self):           
        '''
        Pobiera liste deklaracji ze strony eDeklaracje
        Liste zapisuje w polu klasy WSZYSTKIE
        Struktura elementu z listy WSZYSTKIE wyglada tak: [Nazwa, Tytul, DataOd, URL] 
        '''
        #AB 24.08.20r iteracja z uzyciem find_all zamiast getchildren,
        # wiersze = etree.fromstring(self.tabelka).getchildren() #Wiersze tabeli, ktre zdobywam pobierajc dzieci z tabelki.
        # for wiersz in wiersze:
        #     if str(wiersz[1].text)[:-1].isdigit(): #Jeli to liczba to mamy doczynienia z dokumentem.
        #         tytul = wiersz[2].getchildren()[0].text #Pobieramy wersje pliku z ktorej mamy wszystkie informacje prcz dat.
        naglowki = [col.text for col in self.tabelka.find_all('th')]
        wiersze = self.tabelka.find_all('tr')[1:]
        for wiersz in wiersze:
            wiersz = [col for col in wiersz.find_all('td')]
            if len(wiersz) != 4:  # wiersze nie z deklaracjami, a np. tytuy czesci tabeli
                continue
            tytul = wiersz[0].text


            nazwa = tytul

            try: #W tabelce moe nie by kulumn z datami (np. dla ePUAP) dlatego prbuje odczyta w try
                #AB 24.08.20r numer kolumny zmienil sie na 1 z 3
                dataod = wiersz[1].text #kolumna 1 zawiera dat od
                if dataod is None:
                    dataod = str(datetime.datetime.now().year) + '-01-01' #jeli nie ma daty ustalono, e bdzie z obecnego roku
            except:
                dataod = str(datetime.datetime.now().year) + '-01-01'

            znalezione = nazwa.find('(') #Szukam '(' aby wydoby nazw dokumentu
            if znalezione != -1:
                nazwa = nazwa[:znalezione]
            #AB 24.08.20r skorzystanie z bibliotekowego sposobu pobrania adresu pliku
            #url = self.PobierzWartoscAtrybutu(wiersz[2].getchildren()[0], 'href')
            #AB 13.09.22r
            url = [x.find('a') for x in wiersz[0] if (hasattr(x.find('a'), 'attrs') and 'href' in x.find('a').attrs)]

            if url:
                url = url[0].attrs['href']
            else:
                continue

            if url.startswith('/'):

                #url = 'http://e-deklaracje.mf.gov.pl' + url[1:] #Adresy mog by podane w caoci (na pewno ePUAP) albo zawierajce
                url = 'http://www.finanse.mf.gov.pl/' + url[1:] #Adresy mog by podane w caoci (na pewno ePUAP) albo zawierajce
                                                                #adres pod katalogu i dlatego trzeba zczy z adresem eDeklaracje

            self.WSZYSTKIE.append([nazwa, tytul, dataod, url])  #Wszystkie znalezione deklaracje
        #print self.WSZYSTKIE
                
#------------------------------------------------------------------------------
#--- Funkcja: FiltrujDeklaracje
#------------------------------------------------------------------------------

    def FiltrujDeklaracje(self): 
        '''
        Filtruje deklaracje WSZYSTKIE wedlug self.wzorce
        oraz sprawdza za pomoca funkcji CzyPasuje fakt bycia zalocznikiem czy
        normalna deklaracja
        '''

        def CzyPasuje(Nazwa):

            if Nazwa[Nazwa.find(')') + 2] == 'v': #Sprawdzam czy jest to deklaracja czy zalcznik. Jeeli znajd ')' a dwa znaki pniej jest 'v'
                return True                       #to mam doczynienia z normalna deklaracj a nie zacznikiem.
            else:
                return False

        self.WSZYSTKIE = [x for x in self.WSZYSTKIE if (x[0] in self.wzorce)] #Lista WSZYSTKIE jest filtrowana i zostawiam w niej tylko te co, spelniaj
        self.WSZYSTKIE = [x for x in self.WSZYSTKIE if CzyPasuje(x[1])]       #zaoenia.

#------------------------------------------------------------------------------
#--- Funkcja: CheckUpdates
#------------------------------------------------------------------------------

    def SprawdzCzyNowe(self):
        '''
        Sprawdza czy sa nowe deklaracje i jesli tak zwraca liste do insertow
        Deklaracje nowe zapisuje do zmiennej NOWE
        '''

        for item1 in self.WSZYSTKIE:
            istnieje = False
            for item2 in self.bazadanych:
                if item1[1] == item2[1]:
                    istnieje = True                     

            if not istnieje:
                self.NOWE.append(item1)

#------------------------------------------------------------------------------
#--- Funkcja: ZapiszNaDysk
#------------------------------------------------------------------------------

    def ZapiszNaDysk(self):
        '''
        Zapisuje na dysk wszystkie pliki ze zmiennej NOWE na dysk oraz wszystkie
        pliki importowane w tych plikach (pliki bazowe plikow xsd)
        '''

        try:
            os.makedirs(self.sciezka) #Tworzy pen ciek dla zapisu deklaracji na dysk
        except:
            pass

        for item in self.NOWE:
            sciezka = os.path.join(self.sciezka, item[0])
            if not os.path.exists(sciezka): #Tworzymy pod katalog dla deklaracji jednego typu (np. PIT-39)
                os.mkdir(sciezka)
    
            sciezka = os.path.join(sciezka, item[1])
            if not os.path.exists(sciezka): #Tworzymy pod katalog dla deklaracji konkretnej i jej plikw
                os.mkdir(sciezka)
            #print 'CIEKA ',item[3]
            tekst = urllib.urlopen(item[3]).read()
        
            f = open(os.path.join(sciezka, item[1] + '.xsd'), 'w')
            f.write(tekst)
            f.close()

            self.odwiedzone = []
            self.PobierzPlikiDodatkowe(tekst, sciezka) #Szukamy plikw dodatkowych w pliku XSD aby je dogra do katalogu

#------------------------------------------------------------------------------
#--- Funkcja: PobierzPlikiDodatkowe
#------------------------------------------------------------------------------

    def PobierzPlikiDodatkowe(self, Tekst, Sciezka):
        '''
        Wspiera funkcje zapisujaca na dysk w pobieraniu importowanych plikow i zapis
        ich na dysk w tym samym katalogu co plik glowny XSD
        '''

        root = etree.fromstring(Tekst)
        for child in root.getchildren():
            try:
                tag = child.tag[34:]
            except:
                continue
            
            if tag in ('include', 'import'): #Szukamy tych elementw co oznaczaj e mamy doczynienia z plikami wczanymi
                url = self.PobierzWartoscAtrybutu(child, 'schemaLocation')
                if not url in self.odwiedzone:
                    self.odwiedzone.append(url)                                    
                    f = open(os.path.join(Sciezka, url[url.rfind('/') + 1:]), 'w')
                    tekst = urllib.urlopen(url).read()
                    f.write(tekst)
                    f.close()
                    self.PobierzPlikiDodatkowe(tekst, Sciezka)

#------------------------------------------------------------------------------
#--- Funkcja: PobierzWartoscAtrybutu
#------------------------------------------------------------------------------
    
    def PobierzWartoscAtrybutu(self, Element, Nazwa):
        '''
        Funkcja wyciagajaca wartosc atrybutu elementu etree dla podanej nazwy, jesli
        nie ma elementu zwraca pusty '' 
        '''

        for item in Element.items():
            if item[0] == Nazwa:
                return item[1]        
        return ''

#--------------------------------------------------------------------------------------------------
