# -*- coding: iso-8859-2  -*-

# 2021-11-03 Wojciech Skowron - (12174) Poprawiona obsuga pliku

import os
import sys
import glob
import urllib

from jinja2 import Template
from lxml import etree

#------------------------------------------------------------------------------
#--- Funkcje pomocnicze
#------------------------------------------------------------------------------

def PobierzPolaDeklaracji(sciezka, elementy_pomijane = []):
    parser = Parser(sciezka, elementy_pomijane) 
    if not sciezka.endswith('E'):
        parser.epuap = False
    else:
        parser.epuap = True
    pomocnik = Pomocnik(parser.Uruchom())
    
    return pomocnik.PobierzPola()
    
#-------------------------------------------------------------------------------
    
def PobierzSlownikiDeklaracji(sciezka, nazwa):
    parser = Parser(sciezka) 
    if not sciezka.endswith('E'):
        parser.epuap = False
    else:
        parser.epuap = True
        
    pomocnik = Pomocnik(parser.Uruchom())
    
    return pomocnik.PobierzSlowniki(nazwa)

#-------------------------------------------------------------------------------

def StworzPrzykladowaDeklaracje(sciezka, elementy_pomijane = []):
    parser = Parser(sciezka, elementy_pomijane) 
    pomocnik = Pomocnik(parser.Uruchom())
    
    return pomocnik.StworzPustySzablon()[0]

#-------------------------------------------------------------------------------

def StworzDeklaracje(sciezka, elementy_pomijane, wartosci, wartosci_podpis):
    parser = Parser(sciezka, elementy_pomijane)
    if not sciezka.endswith('E'):
        parser.epuap = False
    else:
        parser.epuap = True
         
    pomocnik = Pomocnik(parser.Uruchom())
    
    pola = pomocnik.PobierzPola()
    pola = [x['nazwa'] for x in pola]
    xml1 = pomocnik.StworzXML(pola, wartosci, wartosci_podpis).encode('UTF-8')
        
    xml2 = etree.fromstring(xml1)
    pomocnik.UsunZbedneElementy(xml2)

    return etree.tostring(xml2)

#-#############################################################################
#--- Klasa: Parser
#-#############################################################################

class Parser:
    def __init__(self, sciezka, elementy_pomijane = []):
        '''
        Parametry:
            sciezka: cieka do katalogu z wybran deklaracj
            elementy_pomijane: lista nazw elementw, ktre bd pomijane
              np. ['AdresZagr'] - zostanie pominity element AdresZagr
              skutkiem jest te pominicie elementw podrzdnych
        '''

        self.przestrzenie = []
        self.elementy_pomijane = elementy_pomijane

        self.Inicjuj(sciezka)
        #Wczytuje wszytkie pliki z katalogu deklaracji
        self.PobierzPrzestrzenie(sciezka, self.przestrzenie)
        #Pobiera przestrzenie nazw elementw: 'etd:', 'zzu:', itd...

#------------------------------------------------------------------------------
#--- Funkcja: Inicjuj
#------------------------------------------------------------------------------

    def Inicjuj(self, sciezka):
        '''
        Wczytuje pliki z katalogu zmiennej Sciezka i za pomoc biblioteki lxml
        tworzy drzewa plikw XSD-XML. Drzewa nastpnie zapisuje w licie
        self.root, a drzewo pliku gwnego w self._mainroot.
        '''

        #Wyszukuje pliki w katalogu z rozszerzeniem 'xsd'.
        wyniki = glob.glob(os.path.join(sciezka, '*.xsd'))
        self.root = []

        for item in wyniki:
            #WS 2021-11-03
            with open(item) as p:
                root = etree.fromstring(p.read())
            # root = etree.fromstring(open(item).read())

            #Sprawdza czy plik wczytywany jest plikiem gwnym.
            if os.path.split(item)[1] == os.path.split(sciezka)[1] + '.xsd':
                self._mainroot = root

            self.root.append(root)

#------------------------------------------------------------------------------
#--- Funkcja: PobierzPrzestrzenie
#------------------------------------------------------------------------------

    def PobierzPrzestrzenie(self, sciezka, przestrzenie = []):
        '''
        Pobiera przestrzenie dla pliku dziki czemu w pliku XML wystpi
        nagowek wczajcy odpowiednie przestrzenie.
        Parametr:
            sciezka: jest ciek do katalogu z plikami 'xsd'.
        '''

        f = open(os.path.join(sciezka, os.path.split(sciezka)[1] + '.xsd'))
        tekst = f.read()

        while True:
            start = tekst.find('xmlns')
            if start == -1:
                break

            #Liczba 25 jest wzita z powietrza ale dziki niej to dziaa
            koniec = tekst[start:].find('"', 25) + 1

            nazwa   = tekst[start:tekst[start:start + koniec].find('=') + 1]
            wartosc = tekst[tekst[start:start + koniec].find('=') + 3:start + koniec - 1]

            przestrzenie.append({  'nazwa':nazwa,
                                 'wartosc':wartosc})

            tekst = tekst[start + koniec:]

        f.close()


#------------------------------------------------------------------------------
#--- Funkcja: PobierzWartoscAtrybutu
#------------------------------------------------------------------------------

    def PobierzWartoscAtrybutu(self, element, nazwa):
        '''
        Funkcja wycigajca warto atrybutu elementu etree dla podanej nazwy, jeli
        nie ma elementu, zwraca pusty string.
        '''

        for item in element.items():
            if item[0] == nazwa:
                return item[1]

        return ''

#------------------------------------------------------------------------------
#--- Funkcja: SzukajW
#------------------------------------------------------------------------------

    def SzukajW(self, wezel, tagi, atrybuty = {}):
        '''
        Szuka w Wezel elementu o tagu wystpujcego w licie Tagi, ktry
        ma atrybuty ze zmiennej atrybuty.
        '''

        for item in wezel.getchildren():
            try:
                tag = item.tag[34:]
            except:
                continue

            if tag in tagi:
                poprawny = True

                for attr in atrybuty:
                    if self.PobierzWartoscAtrybutu(item, attr) != atrybuty[attr]:
                        poprawny = False

                if poprawny:
                    return item

            wynik = self.SzukajW(item, tagi, atrybuty)
            if wynik is not None:
                return wynik

        return None

#------------------------------------------------------------------------------
#--- Funkcja: SzukajWKorzeniach
#------------------------------------------------------------------------------

    def SzukajWKorzeniach(self, korzenie, tagi, atrybuty = {}):
        '''
        To samo co funkcja SzukajW lecz ta przeszukuje we wszystkich podanych
        korzeniach.
        '''

        for item in korzenie:
            wynik = self.SzukajW(item, tagi, atrybuty)
            if wynik is not None:
                return wynik

        return None

#------------------------------------------------------------------------------
#--- Funkcja: PobierzAtrybutyZRozszerzenia
#------------------------------------------------------------------------------

    def PobierzAtrybutyZRozszerzenia(self, element_rozszerzenia):
        '''
        Funkcja, ktra pobiera atrybuty jakie naley doda do rozszerzanego
        elementu.
        '''

        atrybuty = []
        for item in element_rozszerzenia.getchildren():
            if item.tag[34:] != 'attribute':
                continue
            else:
                nazwa   = self.PobierzWartoscAtrybutu(item,  'name')
                wartosc = self.PobierzWartoscAtrybutu(item, 'fixed')
                atrybuty.append({  'nazwa':nazwa,
                                 'wartosc':wartosc})
        return atrybuty

#------------------------------------------------------------------------------
#--- Funkcja: _Element
#------------------------------------------------------------------------------

    def Element(self, element, rodzic, deklaracja = True, zalacznik = False, przestrzen_zalacznika = ''):
        '''
        Wywouje si w momencie napotkania elementu o tagu 'element'
        Parametry:
            element: napotkany element
            rodzic: slownik rodzica z kluczami
            deklaracja: True oznacza, e to gwna cz, False, e poza
              nie oznacza to, e jestemy w zaczniku
            przestrzen_zalacznika: przestrze jak bd miay wszytkie elementy
              jeli bdziemy w zaczniku
        '''

        e = {'nazwa': '', 'wartosc': '', 'opis': '', 'atrybuty': [], 'elementy': [], 'wyliczenia': []}
        nazwa = self.PobierzWartoscAtrybutu(element, 'name')
        if nazwa in self.elementy_pomijane:
            return

        if nazwa != '':
            e['nazwa'] = nazwa

            if (deklaracja == False) and (zalacznik == False):
                if self.epuap == True:
                    e['nazwa'] = 'etd:' + e['nazwa']

            if zalacznik:
                e['nazwa'] = przestrzen_zalacznika + e['nazwa']

            wartosc_domyslna = self.PobierzWartoscAtrybutu(element, 'fixed')
            if wartosc_domyslna != '':
                e['wartosc'] = wartosc_domyslna
                e['wyliczenia'].append((wartosc_domyslna, ''))

            typ = self.PobierzWartoscAtrybutu(element, 'type')
            if typ != '':
                wynik = typ.find(':')
                if typ[:wynik] == 'xsd':
                    pass
                else:
                    wynik2 = None
                    wynik2 = self.SzukajW(self._mainroot, ('simpleType', 'complexType'), {'name': typ[wynik + 1: ]})
                    if wynik2 is not None:
                        self.Spaceruj(wynik2, e, True, zalacznik, przestrzen_zalacznika)
                    else:
                        wynik2 = self.SzukajWKorzeniach(self.root, ('simpleType', 'complexType'), {'name': typ[wynik + 1: ]})
                        if wynik2 is not None:
                            self.Spaceruj(wynik2, e, False, zalacznik, przestrzen_zalacznika)

        else:
            referencja = self.PobierzWartoscAtrybutu(element, 'ref')
            nazwa_referencji = referencja[referencja.find(':') + 1:]
            przestrzen_referencji = referencja[:referencja.find(':') + 1]

            wynik = None
            if nazwa_referencji == 'PozycjeSzczegolowe':
                wynik = self.SzukajW(self._mainroot, 'element', {'name': nazwa_referencji})
                e['nazwa'] = 'PozycjeSzczegolowe'
                self.Spaceruj(wynik, e, True, False, przestrzen_referencji)
            else:
                wynik = self.SzukajWKorzeniach(self.root, 'element', {'name': nazwa_referencji})
                e['nazwa'] = przestrzen_referencji + self.PobierzWartoscAtrybutu(wynik, 'name')
                e['atrybuty'] = [x for x in self.przestrzenie if x['nazwa'] == 'xmlns:' + przestrzen_referencji[:-1]]
                self.Spaceruj(wynik, e, False, True, przestrzen_referencji)

        rodzic['elementy'].append(e)
        self.Spaceruj(element, e, deklaracja, zalacznik, przestrzen_zalacznika)

#------------------------------------------------------------------------------
#--- Funkcja: Opis
#------------------------------------------------------------------------------

    def Opis(self, element, rodzic, deklaracja = True, zalacznik = False, przestrzen_zalacznika = ''):
        '''
        Wywouje si w momencie napotkania elementu o tagu 'documentation'
        Parametry:
            element: napotkany element
            rodzic: sownik rodzica elementu
        '''

        rodzic['opis'] = element.text

#------------------------------------------------------------------------------
#--- Funkcja: Wyliczenia
#------------------------------------------------------------------------------

    def Wyliczenia(self, element, rodzic, deklaracja = True, zalacznik = False, przestrzen_zalacznika = ''):
        '''
        Wywouje si w momencie napotkania elementu o tagu 'enumeration'
        Parametry:
            element: napotkany element
            rodzic: sownik rodzica elementu
        '''

        wartosc = self.PobierzWartoscAtrybutu(element, 'value')

        rodzic['wartosc'] = wartosc
        try:
            pomoc = element.getchildren()[0].getchildren()[0].text
        except:
            pomoc = ''
        rodzic['wyliczenia'].append((wartosc, pomoc))

#------------------------------------------------------------------------------
#--- Funkcja: Rozszerzenie
#------------------------------------------------------------------------------

    def Rozszerzenie(self, element, rodzic, deklaracja = True, zalacznik = False, przestrzen_zalacznika = ''):
        '''
        Wywouje si w momencie napotkania elementu o tagu 'extension'.
        Parametry:
            element: napotkany element
            rodzic: sownik rodzica elementu
            deklaracja: True oznacza, e to gwna cz, False, e poza
              nie oznacza to, e jestemy w zaczniku
            przestrzen_zalacznika: przestrze jak bd miay wszytkie elementy
              jeli bdziemy w zaczniku
        '''

        baza = self.PobierzWartoscAtrybutu(element, 'base')
        baza = baza[baza.find(':') + 1:]
        wynik = self.SzukajWKorzeniach(self.root, ('simpleType', 'complexType'), {'name': baza})

        if wynik is not None:
            self.Spaceruj(wynik, rodzic, False, zalacznik, przestrzen_zalacznika)

        rodzic['atrybuty'] = self.PobierzAtrybutyZRozszerzenia(element)

#------------------------------------------------------------------------------
#--- Funkcja: Spaceruj
#------------------------------------------------------------------------------

    def Spaceruj(self, element, rodzic = {}, deklaracja = True, zalacznik = False, przestrzen_zalacznika = ''):
        '''
        Przechodzi przez kady element drzewa i wykonuje odpowiednie operacje
        kiedy spotka odpowiednie tagi.
        '''

        for child in element.getchildren():
            tag = child.tag[34:]

            if tag   == 'element':
                self.Element(child, rodzic, deklaracja, zalacznik, przestrzen_zalacznika)

            elif tag == 'extension':
                self.Rozszerzenie(child, rodzic, deklaracja, zalacznik, przestrzen_zalacznika)

            elif tag == 'enumeration':
                self.Wyliczenia(child, rodzic, deklaracja, zalacznik, przestrzen_zalacznika)

            elif tag == 'documentation':
                self.Opis(child, rodzic, deklaracja, zalacznik, przestrzen_zalacznika)

            else:
                self.Spaceruj(child, rodzic, deklaracja, zalacznik, przestrzen_zalacznika)

#------------------------------------------------------------------------------
#--- Funkcja: Uruchom
#------------------------------------------------------------------------------

    def Uruchom(self):
        '''
        Uruchamia parser i zwracam drzewo wyniku
        '''

        self.drzewo = {'nazwa': 'Deklaracja', 'wartosc': '', 'opis': '', 'atrybuty': [], 'elementy': [], 'wyliczenia': []}
        self.drzewo['atrybuty'] = [x for x in self.przestrzenie if (x['nazwa'] == 'xmlns') or (x['nazwa'] == 'xmlns:etd')]

        e = self.SzukajW(self._mainroot, 'element', {'name': 'Deklaracja'})
        self.Spaceruj(e, self.drzewo)

        return self.drzewo

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------

#-#############################################################################
#--- Klasa: Pomocnik
#-#############################################################################
class Pomocnik:
    def __init__(self, sparsowane_drzewo):
        self.wynik = []
        self.drzewo = sparsowane_drzewo

        self.Spaceruj(self.drzewo, drzewo_rodzic = None)

#------------------------------------------------------------------------------
#--- Funkcja: Spaceruj
#------------------------------------------------------------------------------

    def Spaceruj(self, drzewo, drzewo_rodzic, zalaczniki = False):

        e_nazwa = drzewo['nazwa']
        e_opis = drzewo['opis']
        e_elementy = drzewo['elementy']
        e_wyliczenia = drzewo['wyliczenia']

        if e_nazwa == 'Zalaczniki':
            zalaczniki = True

        if len(e_elementy) == 0:
            #if len(e_wyliczenia) != 1:
            nazwa = e_nazwa[e_nazwa.find(':') + 1:]
            if drzewo_rodzic != None:
                e_opis = e_opis + ' ; ' + unicode(drzewo_rodzic['opis'])

            self.wynik.append([nazwa, e_nazwa, e_opis, e_wyliczenia])

        for element in e_elementy:
            self.Spaceruj(element, drzewo, zalaczniki)

#------------------------------------------------------------------------------
#--- Funkcja: PobierzSlowniki
#------------------------------------------------------------------------------

    def PobierzSlowniki(self, nazwa):
        '''
        Pobiera list sownikw, czyli spis moliwoci dla pl deklaracji.
        '''

        try:
            for pole in self.wynik:
                if pole[1] == nazwa:
                    return pole[3]
        except:
            return ('', '')

#------------------------------------------------------------------------------
#--- Funkcja: PobierzPola
#------------------------------------------------------------------------------

    def PobierzPola(self):
        '''
        Zwraca pola deklaracji, moliwe do wypenienia
        '''

        a = []
        w = {}
        t = []
        for item in self.wynik:
            if item[0].endswith('KodFormularza'):
                t = []
                w = {}
                a.append(w)
                w['nazwa'] = item[3][0][0]
                w['pola'] = t
            try: 
                if len(item[-1]) == 1:
                    wartosc = item[-1][0][0]
                else:
                    wartosc = ''
            except:
                wartosc = ''
            it = item[:-1] + [wartosc]
            t.append(it)

        return a


#------------------------------------------------------------------------------
#--- Funkcja: UsunZbedneElementy
#------------------------------------------------------------------------------

    def UsunZbedneElementy(self, korzen):
        '''
        Usuwa zbdne elementy, ktre nie maj wartoci bd elementw wewntrz.
        '''

        def Usun(korzen):
            for child in korzen.getchildren():
                if (len(child.getchildren()) == 0) and (str(child.text) == 'None' or str(child.text) == ''):
                    korzen.remove(child)
                    self.wartosc += 1
                else:
                    Usun(child)

        self.wartosc = 1
        while self.wartosc != 0:
            self.wartosc = 0
            Usun(korzen)

#------------------------------------------------------------------------------
#--- Funkcja: GenerujSzablony
#------------------------------------------------------------------------------

    def GenerujPusteSzablony(self):
        lista = self.StworzPustySzablon()[1]
        #print lista
        licznik1 = 0
        licznik2 = 0

        for i in lista:
            if i.find('<Zalaczniki') != -1:
                break
            licznik1 += 1

        for i in lista:
            if i.find('</Zalaczniki>') != -1:
                break
            licznik2 += 1
        glowna = []
        glowna.append(lista[:licznik1] + lista[-1:])
        zalaczniki = []
        lista = lista[licznik1 + 2:-2]
        indeks = 0
        while True:
            if indeks == len(lista):
                break

            indeks2 = 0
            for i in lista:
                if i.find(lista[indeks][:1] + '/' + lista[indeks][1:] + '>') != -1:
                    break
                indeks2 += 1

            zalaczniki.append(lista[indeks:indeks2 + 1])

            indeks = indeks2 + 1

        return (glowna + zalaczniki)


#------------------------------------------------------------------------------
#--- Funkcja: StworzPustySzablon
#------------------------------------------------------------------------------

    def StworzPustySzablon(self):
        XML = []
        def Stworz(drzewo, XML):
            e_nazwa = drzewo['nazwa']
            e_wartosc = drzewo['wartosc']
            e_atrybuty = drzewo['atrybuty']
            e_elementy = drzewo['elementy']
            e_wyliczenia = drzewo['wyliczenia']

            XML.append('<' + e_nazwa)

            if len(e_atrybuty) > 0:
                for attr in e_atrybuty:
                    XML.append(' ' + attr['nazwa'])
                    XML.append('=')

                    if attr['wartosc'] != '':
                        XML.append('"' + attr['wartosc'] + '"')
                    else:
                        XML.append('"' + '{{' + attr['nazwa'].replace(':', '_') + '}}' + '"')

            XML.append('>')

            if len(e_elementy) == 0:
                if len(e_wyliczenia) == 1:
                    if e_wartosc != '':
                        if e_nazwa.find('P_') == -1:
                            XML.append(e_wartosc)
                        else:
                            XML.append('{{' + e_nazwa.replace(':', '_') + '}}')
                else:
                    XML.append('{{' + e_nazwa.replace(':', '_') + '}}')

            for element in e_elementy:
                Stworz(element, XML)

            XML.append('</' + e_nazwa + '>')

        Stworz(self.drzewo, XML)

        return (''.join(XML),XML)

#------------------------------------------------------------------------------
#--- Funkcja: StworzXML
#------------------------------------------------------------------------------

    def StworzXML(self, nazwy_deklaracji, wartosci = [], wartosci_podpis = []):
        szablony = self.GenerujPusteSzablony()
        szablony = [''.join(x) for x in szablony]
        szablony = zip(nazwy_deklaracji, szablony)

        lista_do_wypelnienia = []
        slownik_wartosci = {}

        for i in wartosci:
            slownik_wartosci = {}
            lista_do_wypelnienia.append((i['nazwa'], slownik_wartosci))

            for j in i['pola']:
                slownik_wartosci[j[0].replace(':', '_')] = j[1]

        wynik = []
        for i in lista_do_wypelnienia:
            nazwa = i[0]
            slownik = i[1]
            for j in szablony:
                nazwa2 = j[0]
                szablon = j[1]

                if nazwa == nazwa2:
                    wynik.append(Template(szablon).render(slownik))
        XML = ''
        if len(wartosci) == 0:
             XML = ''
        elif len(wartosci) == 1:            
            XML = '<?xml version="1.0" encoding="UTF-8"?>' + wynik[0]
        else:
            XML += '<?xml version="1.0" encoding="UTF-8"?>'
            XML += wynik[0][:-13]
            XML +='<Zalaczniki>'
            for i in wynik[1:]:
                XML += i
            XML +='</Zalaczniki>'
            XML +='</Deklaracja>'

        if len(wartosci_podpis) != 0:
            XML = XML[:-13] + '<podp:DaneAutoryzujace' + wartosci_podpis[0] + ' xmlns:podp="http://e-deklaracje.mf.gov.pl/Repozytorium/Definicje/Podpis/">'
            for i in wartosci_podpis[1:]:
                XML += '<podp:' + i[0] + '>' + i[1] + '</podp:' + i[0] + '>'

            XML += '</podp:DaneAutoryzujace' + wartosci_podpis[0] +'>'
            XML += '</Deklaracja>'
        return XML
        
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
