# -*- coding: iso8859_2 -*-

# -----------------------------------------------------------------------------
# Name:        fs_ftp_obsluga.py
# Purpose:     obsluga polaczen FTP(S)
# Author:
# Created:     2019/09/10
# RCS-ID:      $Id: fs_ftp_obsluga.py $
# Copyright:   (c) 2019 RecordSystem
# Licence:     GNU GPL
# -----------------------------------------------------------------------------
# Changes:
# 2019-10-23 (11707) Aleksander Bargie - Uwagi do eksportu na ftp
# 2020-10-27 (12172) Aleksander Bargie - Przeniesienie sklepw Doroty do harmonogramu

"""
fs zawiera klas RSFTP suc do obsugi pocze z serwerami plikw, moliw do wykorzystania bezporednio w innych
fsach, albo wywoania z poziomu klienta
metody klasy RSFTP:
    -pobierzPlik
    -pobierzKatalog
    -listuj
    -wyslijPlik
    -wylijKatalog
    -usun
    -zmienNazwe
    -close

Funkcje:
1. Wysanie pliku
2. Wysanie katalogu
3. Pobranie pliku
4. Pobranie katalogu
5. Pobranie listy plikw(listuj)
6. Inicjacja folderw dla wszystkich kontrahentw z uzupenionym FTPNazwa(tworzenie katalogw)
7. Zapisanie pliku przesanego jako argument
8. Wysanie zawartoci folderw tworzonych w 6. na odpowiednie serwery
9. Pobranie raportw
15. Pobranie listy lokalizacji z pliku konfiguracyjnego"""
import re

stat = 'OK'

try:
    if 'Funkcja' not in klucze:
        stat = 'Brak podanej funkcji w kluczach'
except:
    stat = 'Brak podanej funkcji w kluczach'
    Fun = 9999

if stat == 'OK':
    Fun = klucze['Funkcja']
    SlPar = {'Argumenty': [1, 2, 3, 4, 5, 7, ]}
    Lista = rsglobal.SprawdzParametryWKluczach(SlPar, Fun, klucze)
    if Lista != '':
        stat = 'Uwaga : wywoanie funkcji fs_ftp_obsluga z numerem '+str(Fun)+' bez parametrw: '+Lista

if stat == 'OK':
    if Fun in [1, 2, 3, 4, 5, 7, ]:
        Argumenty = klucze['Argumenty']

dane = []
wyn1 = []
wyn2 = []

global pobierzConfig, listuj, plik_z_configiem, RSFTP, pobierzKatalog, pobierzPlik, sprawdzSciezke, wyslijPlik, \
    wyslijKatalog, dl_nazwy, inicjujKatalogi, pobierzNazwy, loguj, wyslijWszystko, katalogWyslano, trzymacPlikiPrzez

plik_z_configiem = 'ftp.conf'
dl_nazwy = 10
katalogWyslano = 'wyslane'
trzymacPlikiPrzez = 15

def loguj(tekst, inflevel=5):
    """obsluga zapisu do logow ktora nie rzuca bledow jesli odpalilismy plik bezposrednio"""
    try:
        rsglobal.ZapiszDoLogow(tekst, inflevel)
    except:
        print tekst


def pobierzConfig():
    """Metoda przerabiajaca zawartosc pliku konfiguracyjnego na liste slownikow o kluczach:
    "nazwa"
    "serwer"
    "port"
    "uzytkownik"
    "haslo"
    "protokol"
     "raporty"
     "faktury"
     "korekty"
     "dowody_dostaw"
     "zamowienia"
     "z_faktury"
     "z_korekty"
     "z_dowody_dostaw"
     "z_zamowienia" """
    import os
    if not os.path.exists(plik_z_configiem):
        raise Exception("nie odnaleziono pliku {}, nie mona odczyta konfiguracji ftp.".format(plik_z_configiem))
    loguj("Pobieranie konfiguracji")
    out = []
    with open(plik_z_configiem) as config:
        server = {     "nazwa": '',    "serwer": '',      "port": '',      "uzytkownik": '',         "haslo": '',
                    "protokol": '',   "raporty": '',   "faktury": '',         "korekty": '', "dowody_dostaw": '',
                  "zamowienia": '', "z_faktury": '', "z_korekty": '', "z_dowody_dostaw": '',  "z_zamowienia": ''}
        for line in config:
            line = unicode(line, 'iso8859-2')
            if not line.strip().startswith('#'):
                found = re.findall('^((?:\w|\s)+)=(.*)$', line)
                if found:
                    for pair in found:
                        if pair[0] in server.keys():
                            server[pair[0]] = pair[1]
                else:
                    found = re.findall('^\[(.+)\]$', line)
                    if found:
                        if server['nazwa'] != '':
                            out.append(server)
                        server = {'nazwa': found[0],        "serwer": '',          "port": '', "uzytkownik": '',
                                      "haslo": '',        "protokol": '',       "raporty": '',    "faktury": '',
                                    "korekty": '',   "dowody_dostaw": '',    "zamowienia": '',  "z_faktury": '',
                                  "z_korekty": '', "z_dowody_dostaw": '',  "z_zamowienia": ''}
        if server['nazwa'] != '':
            out.append(server)
    #AB 27.10.20r
    # loguj("Odczytano {}".format(repr(out)))
    return out


def pobierzNazwy():
    """Zwraca same nazwy z pliku konfiguracyjnego, ktorych dlugosc jest zgodna z limitem"""
    return [x['nazwa'] for x in pobierzConfig() if len(x['nazwa']) <= dl_nazwy]

class RSFTP(object):
    def __init__(self, serwer='', port=21, uzytkownik='', haslo='', protokol='FTP', passive=True, ftp_conn=None,
                 z_configu=u'', **kwargs):
        """Klasa suca do komunikacji z serwerami plikw, majca obsugiwa wewntrz rne protokoy, udostpniajc
        jednolite metody

        argumenty:
          -zdalna_sciezka    obowiazkowa lokalizacja pliku do pobrania
          -sciezka_zapisu    opcjonalna lokalizacja do zapisu na dysku lokalnym, jeli '',  to zwrcony zostanie plik
                             jako zmienna
          -nadpisywac        czy plik ma zostac nadpisany jesli juz istnieje
          """
        loguj("Tworzenie obiektu RSFTP")
        self.serwer = serwer
        self.port = port
        self.uzytkownik = uzytkownik
        self.haslo = haslo
        self.protokol = protokol
        self.z_configu = z_configu
        self.bledy = []
        self.udane = []
        if z_configu != '':
            self.__wczytajConfigDla(z_configu)
        elif serwer == '':
            raise Exception("Nie podano adresu serwera ani nazwy konfiguracji do odczytania.")
        if self.protokol.upper() in ('FTP', 'FTPS'):
            loguj("Wybrany protokol FTP")
            self.__typyFolderow__ = ['d', ]
            self.__typyPlikow__ = ['-', ]
            try:
                from ftpretty import ftpretty
            except ImportError:
                raise Exception("Nie zainstalowano biblioteki ftpretty. Skontaktuj si z administratorem.")
            self.ftp = ftpretty(host     = self.serwer,
                                user     = self.uzytkownik,
                                password = self.haslo,
                                secure   = True if self.protokol.upper() == 'FTPS' else False,
                                passive  = passive,
                                ftp_conn = ftp_conn,  # polaczenie FTP/FTP_TLS z ftplib jesli juz istnieje
                                **kwargs  # dodatkowe argumenty do klas FTP/FTP_TLS z ftplib
                                )
            if self.__FTPGetConn__().sock is None:
                raise Exception('Nie udao si nawiza poczenia')
        else:
            raise Exception('Nieobsugiwany protok: ' + self.protokol)

        loguj("Nawiazano polaczenie z {}".format(self.serwer))

    def __wczytajConfigDla(self, nazwa):
        """Prywatna metoda pobrania ustawien polaczenia z configu.

        argumenty:
          -nazwa    nazwa serwera w pliku konfiguracyjnym
          """
        loguj("Odczyt configu dla {} z pliku {}".format(nazwa, plik_z_configiem))
        import unicodedata
        if isinstance(nazwa, unicode):
            pass
        elif isinstance(nazwa, str):
            nazwa = unicode(nazwa, 'iso8859-2')
        else:
            raise Exception('Bdny format nazwy: {}'.format(nazwa))

        conf = pobierzConfig()
        for line in conf:
            if unicodedata.normalize('NFKC', line['nazwa']).lower() == unicodedata.normalize('NFKC', nazwa).lower():
                self.serwer = line['serwer']
                self.protokol = unicodedata.normalize('NFKC', line['protokol']).upper().strip()
                self.uzytkownik = line['uzytkownik']
                self.haslo = line['haslo']
                self.protokol = unicodedata.normalize('NFKC', line['protokol']).upper().strip()
                if line['port'] != '':
                    self.port = line['port']
                else:
                    self.port = 21
                self.raporty         = line['raporty']
                self.faktury         = line['faktury']
                self.korekty         = line['korekty']
                self.dowody_dostaw   = line['dowody_dostaw']
                self.zamowienia      = line['zamowienia']
                self.z_faktury       = line['z_faktury']
                self.z_korekty       = line['z_korekty']
                self.z_dowody_dostaw = line['z_dowody_dostaw']
                self.z_zamowienia    = line['z_zamowienia']
                return
        raise Exception('Nie odnaleziono konfiguracji dla {} w pliku {}'.format(nazwa, plik_z_configiem))

    def pobierzPlik(self, zdalna_sciezka, sciezka_zapisu='', nadpisywac=False, usuwac=False):
        """Pobieranie plikw z serwera.

        argumenty:
          -zdalna_sciezka    obowiazkowa lokalizacja pliku do pobrania
          -sciezka_zapisu    opcjonalna lokalizacja do zapisu na dysku lokalnym, jeli '',  to zwrcony zostanie plik
                             jako zmienna
          -nadpisywac        czy plik ma zostac nadpisany jesli juz istnieje
          -usuwac            czy plik ma zostac usuniety po pobraniu
          """
        import posixpath
        import os.path
        nazwa = posixpath.basename(zdalna_sciezka)
        if nazwa == '':
            raise Exception('Nie podano nazwy pliku do pobrania.')

        if sciezka_zapisu != '':
            if not os.path.isdir(sciezka_zapisu):
                raise Exception('Niepoprawna cieka zapisu: ' + self.protokol)
            do_zapisu = os.path.join(sciezka_zapisu, nazwa)
            if not nadpisywac and os.path.exists(do_zapisu):
                raise Exception('Plik ju istnieje: ' + do_zapisu)

        loguj("Pobieranie {} do {}".format(zdalna_sciezka, sciezka_zapisu))
        if self.protokol.upper() in ('FTP', 'FTPS'):
            plik = self.__FTPGet__(zdalna_sciezka)
        else:
            raise Exception('Funkcja pobierzPlik jest nieobsugiwana dla protokou {}'.format(self.protokol))
        if sciezka_zapisu != '':
            with open(do_zapisu, 'wb+') as obiektDoZapisu:
                plik = obiektDoZapisu.write(plik)
        if usuwac:
            self.usunPlik(zdalna_sciezka)
        return plik

    def usunPlik(self, zdalna_sciezka):
        """Pobieranie plikw z serwera.

        argumenty:
          -zdalna_sciezka    obowiazkowa lokalizacja pliku do usuniecia

        zwraca True jeli usunito, False jeli napotkano bd
          """
        loguj("Usuwanie {}".format(zdalna_sciezka))

        if self.protokol.upper() in ('FTP', 'FTPS'):
            return self.__FTPDelete__(zdalna_sciezka)
        else:
            raise Exception('Funkcja pobierzPlik jest nieobsugiwana dla protokou {}'.format(self.protokol))

    def wyslijPlik(self, plik=None, zawartosc=None, zdalna_sciezka='', nadpisywac=False, tworzycKatalogi=False,
                   przenosic=False):
        """Wysyanie plikw na serwer

        argumenty:
          -plik              plik jaki ma zosta przesany, moze byc otwarty plik lub string ze sciezka/nazwa pliku
          -zawartosc         zawartosc pliku do wyslania na serwer
          -zdalna_sciezka    cieka gdzie zapisa plik na serwerze, domylnie ''
          -nadpisywac        czy plik ma zostac nadpisany jesli juz istnieje
          -tworzycKatalogi   czy tworzyc katalog jesli nie istnieje
          -przenosic         czy po udanym przeslaniu przeniesc lokalny plik do podfolderu katalogWyslano
          """
        loguj("Wysyanie {} do {}".format(plik, zdalna_sciezka))
        import posixpath
        import os
        if plik == '' and zawartosc is not None:
            raise Exception('Nie podano nazwy pliku do wysania.')

        zdalny_katalog = os.path.dirname(zdalna_sciezka)
        
        if zawartosc is None:
            if not isinstance(plik, file):
                with open(plik, 'rb') as tmp:
                    zdalny_plik = os.path.basename(plik)
                    zawartosc = tmp.read()
            else:
                zdalny_plik = os.path.basename(plik.name)
                zawartosc = plik.read()
        else:
            zdalny_plik = plik if zdalna_sciezka.endswith('/') else os.path.basename(zdalna_sciezka)
        if przenosic:
            typ_upd = ''
            if os.path.split(zdalny_katalog)[1] == self.korekty:
                zdalny_katalog = posixpath.join(os.path.split(zdalny_katalog)[0], self.z_korekty)
                typ_upd = 'KO'
            elif os.path.split(zdalny_katalog)[1] == self.faktury:
                zdalny_katalog = posixpath.join(os.path.split(zdalny_katalog)[0], self.z_faktury)
                typ_upd = 'FA'
            elif os.path.split(zdalny_katalog)[1] == self.dowody_dostaw:
                zdalny_katalog = posixpath.join(os.path.split(zdalny_katalog)[0], self.z_dowody_dostaw)
                typ_upd = 'DD'
            loguj("Zmieniona sciezka:  {} ".format(zdalna_sciezka))
        zdalna_sciezka = posixpath.join(zdalny_katalog, zdalny_plik)
        if not tworzycKatalogi and not self.sprawdzSciezke(zdalny_katalog):
            self.bledy.append('Bd przesania pliku do lokalizacji {}'.format(zdalna_sciezka))
            raise Exception('Katalog {} nie istnieje'.format(zdalny_katalog))
        if not nadpisywac and self.sprawdzSciezke(zdalna_sciezka):
            raise Exception('Plik {} ju istnieje'.format(zdalna_sciezka))

        if self.protokol.upper() in ('FTP', 'FTPS'):
            wynik = self.__FTPPut__(remote=zdalna_sciezka, contents=zawartosc)
            if wynik == 1:
                self.bledy.append('Bd przesania pliku do lokalizacji {}'.format(zdalna_sciezka))
                raise Exception('Bd przesania pliku do lokalizacji {}'.format(zdalna_sciezka))
        else:
            self.bledy.append(zdalny_plik + ' funkcja wyslijPlik jest nieobsugiwana dla protokou {}'.format(self.protokol))
            raise Exception('Funkcja wyslijPlik jest nieobsugiwana dla protokou {}'.format(self.protokol))

        # Przenoszenie tyczy si wysyania plikw do PC Marketu, to zachowanie raczej nie bdzie uyteczne w innych
        # przypadkach, nie zadziala jesli wykorzystywana jest zawartosc
        if przenosic:
            import re
            tmp = os.path.join(os.path.dirname(plik), katalogWyslano)
            if not os.path.exists(tmp):
                os.mkdir(tmp)
            loguj("Przenoszenie {} do {}".format(plik, os.path.join(tmp, os.path.basename(plik))))
            os.rename(plik, os.path.join(tmp, os.path.basename(plik)))
            loguj("Plik {}".format(plik))
            UPD = ''
            if typ_upd == 'FA':
                wyznacznik, data, numer = re.findall(r'([^_/\\]{1,2})_([12][910]\d\d-(?:(?:[0][0-9])|(?:1[0-2]))-(?:(?:[0-2][0-9])|(?:3[01])))_(\d+)',
                                                     os.path.basename(zdalny_plik))[0]
                UPD="UPDATE rs_faktury SET data_exportu=CURRENT_DATE WHERE firma_id='"+self.firma_id+"' AND wyznacznik_id='"\
                    +wyznacznik+"' AND data='"+data+"' AND numer="+numer+"; "
            elif typ_upd == 'KO':
                wyznacznik, data, numer = re.findall(r'([^_/\\]{1,2})_([12][910]\d\d-(?:(?:[0][0-9])|(?:1[0-2]))-(?:(?:[0-2][0-9])|(?:3[01])))_(\d+)',
                               os.path.basename(zdalny_plik))[0]
                UPD=" UPDATE rs_korekty SET data_exportu=CURRENT_DATE WHERE firma_id='"+self.firma_id+"' AND wyznacznik_id='"\
                    +wyznacznik+"' AND data='"+data+"' AND numer="+numer+"; "
            elif typ_upd == 'DD':
                oddzial, trasa, data, tryb, numer = re.findall(r'([^_/\\]{1,2})_([^_/\\]{1,4})_([12][910]\d\d-(?:(?:[0][0-9])|(?:1[0-2]))-(?:(?:[0-2][0-9])|(?:3[01])))_([^_/\\]{1,2})_(\d+)',
                               os.path.basename(zdalny_plik))[0]
                UPD = """UPDATE rs_dowody_dostaw SET data_eksportu=CURRENT_DATE WHERE firma_id='{}' AND oddzial_id='{}'
                     AND trasa='{}' AND data='{}' AND tryb_dostawy='{}' AND numer={}; 
                     """.format(self.firma_id, oddzial, trasa, data, tryb, numer)
            if UPD != '':
                loguj("Wykonanie {}".format(UPD))
                stat, dump, dump, dump = self.PolSQL.Execute(UPD, 1)
                if stat != "OK":
                    self.bledy.append('Bdy zapisu daty eksportu dla ' + zdalny_plik)
                    loguj("Bdy zapisu daty eksportu: "+stat)
                    raise Exception("Bdy zapisu daty eksportu: "+stat)
                else:
                    self.udane.append("Wysano poprawnie " + zdalny_plik)
            else:
                loguj("Nie rozpoznano typu dokumentu dla pliku {}".format(os.path.basename(zdalny_plik)))

        return True

    def pobierzKatalog(self, zdalna_sciezka, sciezka_zapisu, nadpisywac=False, zawartosc=True, usuwac=False):
        """Pobieranie rekursywnie plikw z serwera.

        argumenty:
          -zdalna_sciezka    lokalizacja do pobrania, obowiazkowa
          -sciezka_zapisu    lokalizacja do zapisu, obowiazkowa
          -nadpisywac        czy plik ma zostac nadpisany jesli juz istnieje
          -usuwac            czy plik ma zostac usuniety po pobraniu, pozostawia katalogi
          """
        import os.path
        import posixpath
        zdalna_sciezka = '/'+zdalna_sciezka.strip('/')+'/'

        if zawartosc:
            loguj("Pobieranie katalogu {} do {}".format(zdalna_sciezka, sciezka_zapisu))

        if sciezka_zapisu != '':
            try:
                os.mkdir(sciezka_zapisu)
                loguj("Tworzenie lokalnego katalogu {}".format(sciezka_zapisu))
            except:
                pass
            if not os.path.isdir(sciezka_zapisu):
                raise Exception('Niepoprawna cieka zapisu: ' + self.protokol)

        lista = self.listuj(zdalna_sciezka, tylko_nazwy=False)
        for wpis in lista:
            if wpis['folder']:
                podkatalog_zdalny  = posixpath.join(zdalna_sciezka, wpis['nazwa'])
                podkatalog_lokalny = os.path.join(sciezka_zapisu, wpis['nazwa'])
                self.pobierzKatalog(zdalna_sciezka=podkatalog_zdalny, sciezka_zapisu=podkatalog_lokalny,
                                    nadpisywac=nadpisywac, zawartosc=zawartosc, usuwac=usuwac)
            elif zawartosc:
                do_pobrania = posixpath.join(zdalna_sciezka, wpis['nazwa'])
                self.pobierzPlik(zdalna_sciezka=do_pobrania, sciezka_zapisu=sciezka_zapisu, nadpisywac=nadpisywac, usuwac=usuwac)

    def wyslijKatalog(self, zdalna_sciezka, lokalna_sciezka, nadpisywac=False, omijac=(), przenosic=False):
        """Wysyanie rekursywnie plikw na serwer.

        argumenty:
          -zdalna_sciezka    lokalizacja do wysania, obowiazkowa
          -lokalna_sciezka   lokalizacja do odczytu, obowiazkowa
          -nadpisywac        czy plik ma zostac nadpisany jesli juz istnieje
          -omijac            tupla nazw do pomijania
          -przenosic         przekazywane do wyslijPlik
          """
        import os.path
        import posixpath
        zdalna_sciezka = '/'+zdalna_sciezka.strip('/')+'/'

        loguj("Przesanie katalogu {} do {}".format(lokalna_sciezka, zdalna_sciezka))
        sukces = True

        if not os.path.exists(lokalna_sciezka):
            if not przenosic:  # przenosic oznacza wysylanie wszystkiego, brak katalogu nam nie przeszkadza
                raise Exception("katalog {} nie istnieje".format(lokalna_sciezka))
            else:
                return True

        for nazwa in os.listdir(lokalna_sciezka):
            if nazwa not in omijac:
                if os.path.isdir(os.path.join(lokalna_sciezka, nazwa)):
                    podkatalog_zdalny  = posixpath.join(zdalna_sciezka, nazwa)
                    podkatalog_lokalny = os.path.join(lokalna_sciezka, nazwa)
                    wyn = self.wyslijKatalog(zdalna_sciezka=podkatalog_zdalny, lokalna_sciezka=podkatalog_lokalny,
                                             nadpisywac=nadpisywac,  omijac=omijac, przenosic=przenosic)
                    sukces = wyn and sukces
                elif os.path.isfile(os.path.join(lokalna_sciezka, nazwa)):
                    do_wyslania = os.path.join(lokalna_sciezka, nazwa)
                    try:
                        wyn = self.wyslijPlik(plik=do_wyslania, zdalna_sciezka=zdalna_sciezka, nadpisywac=nadpisywac,
                                              przenosic=przenosic, tworzycKatalogi=True)
                    except Exception, e:
                        wyn = False
                        import traceback
                        loguj(traceback.format_exc(), 1)
                        if not przenosic:  # przenosic oznacza, ze korzystamy z wyslijwszystko, a wyjatki zbieramy w self.bledy
                            raise e
                    sukces = wyn and sukces

        if not sukces:
            loguj("Przesanie katalogu {} do {} nie powiodo si.".format(lokalna_sciezka, zdalna_sciezka))
        return sukces

    def listuj(self, sciezka='', typ='wszystko', tylko_nazwy=True, info=True, typ_dat='czas', dodatkowe_kolumny=(),
               plik=False):
        """Wypisuje zawarto scieki na serwerze.

        argumenty:
          -sciezka            dla jakiej sciezki wypisac zawartosc, domyslnie '.'
          -typ                co ma si znale na licie, do wyboru wszystko|foldery|pliki, domylnie wszystko
          -tylko_nazwy        czy wypisywa same nazwy plikw/folderw, jeli False to wypisana bdzie caa cieka
          -typ_dat            opcje 'dzien' np. '2019-08-08', czas np. '2019-08-08 12:23' oraz 'datetime' czyli,
                              zmienna typu datetime
          -dodatkowe_kolumny  dostepne w ftpretty.list(extende=True), a nie pobierane tutaj: ['perms','owner', 'group',
                              'items']
          -plik               domyslnie False, True jesli potrzeba pobrac info o pliku
          """
        if plik:
            loguj("Pobieranie informacji o {}".format(sciezka))
        else:
            loguj("Listowanie zawartoci katalogu {} dla typu {}".format(sciezka, typ))
        if self.protokol.upper() in ('FTP', 'FTPS'):
            lista = self.__FTPListuj__(sciezka=sciezka, typ=typ, tylko_nazwy=tylko_nazwy, info=info,
                                      dodatkowe_kolumny=dodatkowe_kolumny, plik=plik)
        else:
            raise Exception('Funkcja listuj jest nieobsugiwana dla protokou {}'.format(self.protokol))
        if not tylko_nazwy:
            if typ_dat == 'dzien':
                for x in lista:
                    x['data'] = str(x['data'].date())
            elif typ_dat == 'czas':
                for x in lista:
                    x['data'] = str(x['data'].strftime('%Y-%m-%d %H:%M:%S'))
        return lista

    def close(self):
        """Zamykanie polaczen."""
        loguj("Zamykanie polaczenia")
        if self.protokol.upper() in ('FTP', 'FTPS'):
            self.ftp.close()
        else:
            raise Exception('Funkcja close jest nieobsugiwana dla protokou {}'.format(self.protokol))

    def sprawdzSciezke(self, sciezka, czyKatalog=True):
        """Sprawdza na serwerze czy cieka istnieje i jest odpowieniego typu(plik lub katalog.

        argumenty:
          -sciezka           dla jakiej sciezki wypisac zawartosc, domyslnie '.'
          -czyKatalog        czy cieka powinna by katalogiem, domylnie tak
          """
        loguj("Test sciezki {}".format(sciezka))
        import posixpath
        sciezka, plik = posixpath.split(sciezka)
        lista = self.listuj(sciezka=sciezka, tylko_nazwy=False)
        for zawartosc in lista:
            if zawartosc['nazwa'].strip('/') == plik.strip('/'):
                return zawartosc['folder'] == czyKatalog
        return False


    def __FTPGet__(self, remote, local=None):

        size = self.listuj(remote, tylko_nazwy=False, plik=True)[0]['rozmiar']
        temp = self.ftp.get(remote)
        if len(temp) != size: # filesize in bytes, 1 if error
            raise Exception('Bd pobrania, pobrano {} zamiast {}'.format(len(temp), size))

        if isinstance(local, file):  # open file, leave open
            local.write(temp)
        elif local is None:
            return temp
        else:  # path to file, open, write/close return None
            local = open(local, 'wb+')
            local.write(temp)
            local.close()

    def __FTPPut__(self, local=None, remote='', contents=None, quiet=False):
        try:
            self.ftp.cd('/')
            temp = self.ftp.put(remote=remote, quiet=False, contents=contents, local=None)
        except:
            temp = 1
        # len(temp) = filesize in bytes, 1 if error
        return temp

    def __FTPList__(self, remote='.', extra=True, remove_relative_paths=True, extra_columns=(), plik=False):
        columns = ['name', 'datetime', 'size', 'directory']
        columns.extend(extra_columns)

        self.ftp.cd(remote)
        if not plik and self.__FTPGetConn__().lastresp != '257':
            raise Exception('Nie udao si przej do lokalizacji {}, prosimy upewni si e jest poprawna.'.format(remote))
        content  = self.ftp.list(remote=remote if plik else '.', extra=extra, remove_relative_paths=remove_relative_paths)
        out = []
        for val in content:
            tmp = {}
            for key in columns:
                tmp[key] = val[key]
            out.append(tmp)
        return out

    def __FTPRename__(self, remote_from, remote_to):
        return self.ftp.rename(remote_from, remote_to)

    def __FTPClose__(self):
        self.ftp.close()

    def __FTPDelete__(self, remote):
        return self.ftp.delete(remote)

    def __str__(self):
        return self.ftp.__str__()

    def __FTPGetConn__(self):
        return self.ftp.conn

    def __FTPListuj__(self, sciezka, typ, tylko_nazwy, info, dodatkowe_kolumny=(), plik=False):
        """implemenatacja metody listuj() dla protokow FTP i FTPS z wykorzystaniem biblioteki ftpretty
          """
        if typ == 'wszystko':
            typy = ['d', '-' ]
        elif typ == 'foldery':
            typy = self.__typyFolderow__
        elif typ == 'pliki':
            typy = self.__typyPlikow__
        else:
            raise Exception("Podano bdny typ do wypisania: {}".format(typ))

        zawartosc = self.__FTPList__(remote=sciezka, extra=info, extra_columns=dodatkowe_kolumny, plik=plik)
        out = []
        for wiersz in zawartosc:
            if wiersz['directory'] in typy:
                if tylko_nazwy:
                    out.append({'nazwa': wiersz['name'] + ('/' if wiersz['directory'] == 'd' else '')})
                else:
                    tmp = {}
                    for key in wiersz.keys():
                        if key == 'name' and wiersz['directory'] == 'd':
                            tmp['nazwa'] = wiersz[key] + '/'
                            tmp['folder'] = True
                        elif key == 'name':
                            tmp['nazwa'] = wiersz[key]
                            tmp['folder'] = False
                        elif key == 'size':
                            tmp['rozmiar'] = wiersz[key]
                        elif key == 'datetime':
                            tmp['data'] = wiersz[key]
                        elif key != 'directory':
                            tmp[key] = wiersz[key]
                    out.append(tmp)
        return out


def listuj(serwer='', port=21, uzytkownik='', haslo='', protokol='FTP', passive=True, ftp_conn=None, z_configu='',
           sciezka='', typ='wszystko', tylko_nazwy=True, info=True, typ_dat='czas', dodatkowe_kolumny=(), plik=False):
    """Metoda jednorazowego listowania plikw, tworzy poczenie, pobiera list i rozcza si"""
    pol = RSFTP(serwer=serwer, port=port, uzytkownik=uzytkownik, haslo=haslo, passive=passive, ftp_conn=ftp_conn,
                z_configu=z_configu, protokol=protokol)
    lista = pol.listuj(sciezka=sciezka, typ=typ, tylko_nazwy=tylko_nazwy, info=info, typ_dat=typ_dat,
                       dodatkowe_kolumny=dodatkowe_kolumny, plik=plik)
    pol.close()
    return lista


def pobierzPlik(zdalna_sciezka, serwer='', port=21, uzytkownik='', haslo='', protokol='FTP', passive=True,
                ftp_conn=None, z_configu='', sciezka_zapisu='', nadpisywac=False, usuwac=False):
    """Metoda jednorazowego pobrania pliku, tworzy poczenie, pobiera plik i rozcza si"""
    pol = RSFTP(serwer=serwer, port=port, uzytkownik=uzytkownik, haslo=haslo, passive=passive, ftp_conn=ftp_conn,
                z_configu=z_configu, protokol=protokol)
    plik = pol.pobierzPlik(zdalna_sciezka=zdalna_sciezka, sciezka_zapisu=sciezka_zapisu, nadpisywac=nadpisywac)
    pol.close()
    return plik


def pobierzKatalog(zdalna_sciezka, sciezka_zapisu, serwer='', port=21, uzytkownik='', haslo='', protokol='FTP', passive=True,
                ftp_conn=None, z_configu='', nadpisywac=False, zawartosc=True, usuwac=False):
    """Metoda jednorazowego pobrania katalogu, tworzy poczenie, pobiera rekursywnie zawartosc zdalnej sciezki i
    rozcza si"""
    pol = RSFTP(serwer=serwer, port=port, uzytkownik=uzytkownik, haslo=haslo, passive=passive, ftp_conn=ftp_conn,
                z_configu=z_configu, protokol=protokol)
    plik = pol.pobierzKatalog(zdalna_sciezka=zdalna_sciezka, sciezka_zapisu=sciezka_zapisu, nadpisywac=nadpisywac,
                              zawartosc=zawartosc, usuwac=usuwac)
    pol.close()
    return plik


def wyslijPlik(serwer='', port=21, uzytkownik='', haslo='', protokol='FTP', passive=True,
                ftp_conn=None, z_configu='', plik=None, zawartosc=None, zdalna_sciezka='', nadpisywac=False,
               tworzycKatalogi=False):
    """Metoda jednorazowego wysania pliku, tworzy poczenie, przesya plik i rozcza si"""
    pol = RSFTP(serwer=serwer, port=port, uzytkownik=uzytkownik, haslo=haslo, passive=passive, ftp_conn=ftp_conn,
                z_configu=z_configu, protokol=protokol)
    plik = pol.wyslijPlik(plik=plik, zawartosc=zawartosc, zdalna_sciezka=zdalna_sciezka, nadpisywac=nadpisywac,
                          tworzycKatalogi=tworzycKatalogi)
    pol.close()
    return plik


def wyslijKatalog(zdalna_sciezka, lokalna_sciezka, serwer='', port=21, uzytkownik='', haslo='', protokol='FTP', passive=True,
                ftp_conn=None, z_configu='', nadpisywac=False, omijac=(), przenosic=False):
    """Metoda jednorazowego wysania katalogu, tworzy poczenie, przesya rekursywnie pliki z wskazanego katalogu a
    nastpnie rozcza si"""
    pol = RSFTP(serwer=serwer, port=port, uzytkownik=uzytkownik, haslo=haslo, passive=passive, ftp_conn=ftp_conn,
                z_configu=z_configu, protokol=protokol)
    plik = pol.wyslijKatalog(zdalna_sciezka=zdalna_sciezka, lokalna_sciezka=lokalna_sciezka, nadpisywac=nadpisywac,
                             omijac=omijac, przenosic=przenosic)
    pol.close()
    return plik


def sprawdzSciezke(sciezka, serwer='', port=21, uzytkownik='', haslo='', protokol='FTP', passive=True,
                ftp_conn=None, z_configu='', czyKatalog=True, rsgl=None):
    """Metoda jednorazowego sprawdzenia, tworzy poczenie, sprawdza istnienie cieki i rozcza si"""
    if rsgl is not None:
        global rsglobal
        rsglobal = rsgl
    pol = RSFTP(serwer=serwer, port=port, uzytkownik=uzytkownik, haslo=haslo, passive=passive, ftp_conn=ftp_conn,
                z_configu=z_configu, protokol=protokol)
    plik = pol.sprawdzSciezke(sciezka=sciezka, czyKatalog=czyKatalog)
    pol.close()
    return plik


def inicjujKatalogi(firma_id=''):
    """Metoda pobiera list nazw serwerw FTP i tworzy lokalnie katalogi ktre si w nich znajduj"""
    loguj("Inicjacja katalogow FTP na serwerze")
    import os
    if not os.path.exists(plik_z_configiem):
        loguj("Brak konfiguracji {}".format(plik_z_configiem))
        return
    else:
        serwery = pobierzNazwy()
        for serwer in serwery:
            katalog = os.path.join(firma_id, serwer)
            if not os.path.exists(katalog):
                os.makedirs(katalog)
                pol = RSFTP(z_configu=serwer)
                pol.pobierzKatalog('', katalog, zawartosc=False)
                pol.close()


def zapiszPlik(plik, sciezka, typ='', nazwa=''):
    """Metoda przyjmuje binarny plik i zapisuje we wskazanej lokalizacji, sciezka zawiera nazw pliku"""
    loguj("Zapis pliku jako {} typ {}".format(sciezka, typ))
    import os
    if typ != '':
        dane = {}
        conf = pobierzConfig()
        for wpis in conf:
            if wpis['nazwa'] == nazwa:
                dane = wpis
        loguj("Wybrane dane {}".format(dane))
        if typ == 'faktura' and dane:
            sciezka = os.path.join(os.path.dirname(sciezka), dane['faktury'], os.path.basename(sciezka))
        elif typ == 'korekta' and dane:
            sciezka = os.path.join(os.path.dirname(sciezka), dane['korekty'], os.path.basename(sciezka))
        elif typ == 'dd' and dane:
            sciezka = os.path.join(os.path.dirname(sciezka), dane['dowody_dostaw'], os.path.basename(sciezka))
    if not os.path.exists(os.path.dirname(sciezka)):
        os.makedirs(os.path.dirname(sciezka))
    with open(sciezka, 'wb+') as lok:
        lok.write(plik)


def wyslijWszystko(firma, polSQL):
    """Metoda pobiera list nazw serwerw FTP, w oparciu o t list przeszukuje lokalne katalogi(wczeniej powinno
    zosta wywoane inicjujKatalogi()) i wysya ich zawartoc na serwer ftp"""
    loguj("Wysylanie wszystkiego co sie da w oparciu o config.")
    import os
    serwery = pobierzNazwy()
    sukces = True
    udane = []
    bledy = []
    for serwer in serwery:
        sciezka = os.path.join(os.getcwd(), firma, serwer)
        pol = RSFTP(z_configu=serwer)
        pol.firma_id = firma
        pol.PolSQL = polSQL
        try:
            wyn = pol.wyslijKatalog(lokalna_sciezka=sciezka, zdalna_sciezka='', nadpisywac=True,
                                    omijac=(katalogWyslano, os.path.basename(pol.raporty)), przenosic=True)
        except:
            import traceback
            loguj(traceback.format_exc(), 1)
            loguj("Blad wyslania katalogu {}".format(sciezka))
            wyn = False

        sukces = sukces and wyn
        udane += pol.udane
        bledy += pol.bledy
        pol.close()
    return udane, bledy


def pobierzRaporty(firma):
    """Metoda pobiera list nazw serwerw FTP, w oparciu o t list przeszukuje zdalne katalogi
    raportw(podane w configu), pobiera z nich pliki, usuwa zdalne pliki, usuwa lokalne pliki starsze ni 15 dni oraz
    zwraca list z wierszami w postaci: nazwa_pliku, czas_modyfikacji, zawartosc"""
    import os
    from datetime import datetime
    conf = pobierzConfig()
    stat = 'OK'
    dane = []
    for wpis in conf:
        tmp = os.path.join(os.getcwd(), firma, wpis['nazwa'], wpis['raporty'])
        loguj("Pobieranie raportow do {}".format(tmp))
        ftp = RSFTP(z_configu=wpis['nazwa'])
        ftp.pobierzKatalog(zdalna_sciezka=wpis['raporty'], nadpisywac=True, sciezka_zapisu=tmp, usuwac=True)
        ftp.close()
        for plik in os.listdir(tmp):
            sciezka = tmp + '/' + plik
            if os.path.isfile(sciezka):
                dataModPliku = datetime.fromtimestamp(os.path.getmtime(sciezka))
                teraz = datetime.now()
                roznica = teraz - dataModPliku
                if (roznica.days > trzymacPlikiPrzez) or (roznica.days == trzymacPlikiPrzez and roznica.seconds > 0):
                    loguj("Usuwanie starego pliku {}".format(sciezka))
                    os.remove(sciezka)
                else:
                    with open(sciezka, 'rb+') as zawartosc:
                        dane.append([dataModPliku.strftime('%Y-%m-%d %H:%M:%S'), plik, zawartosc.read()])

    return stat, dane

if Fun == 1:
    try:
        dane.append(pobierzPlik(**Argumenty))
    except Exception, e:
        import traceback
        loguj(traceback.format_exc(), 1)
        stat = str(e)

if Fun == 2:
    try:
        dane.append(pobierzKatalog(**Argumenty))
    except Exception, e:
        import traceback
        loguj(traceback.format_exc(), 1)
        stat = str(e)
if Fun == 3:
    try:
        dane.append(wyslijPlik(**Argumenty))
    except Exception, e:
        import traceback
        loguj(traceback.format_exc(), 1)
        stat = str(e)

if Fun == 4:
    try:
        dane.append(wyslijKatalog(**Argumenty))
    except Exception, e:
        import traceback
        loguj(traceback.format_exc(), 1)
        stat = str(e)

if Fun == 5:
    try:
        dane.append(listuj(**Argumenty))
    except Exception, e:
        import traceback
        loguj(traceback.format_exc(), 1)
        stat = str(e)

if Fun == 6:
    try:
        try:  # zaslepka dla odpalenia metody bez self.firma_id
            firma_id = self.firma_id
        except:
            firma_id = ''
        inicjujKatalogi(firma_id=firma_id)
    except Exception, e:
        import traceback
        loguj(traceback.format_exc(), 1)
        stat = str(e)

if Fun == 7:
    try:
        zapiszPlik(**Argumenty)
    except Exception, e:
        import traceback
        loguj(traceback.format_exc(), 1)
        stat = str(e)

if Fun == 8:
    try:
        udane, bledy = wyslijWszystko(firma=self.firma_id, polSQL=self.PolSQL)
        if len(bledy) > 0:
            stat = "Podczas przesyania plikw napotkano problem/y"
        dane = [udane, bledy]
    except Exception, e:
        import traceback
        loguj(traceback.format_exc(), 1)
        stat = str(e)

if Fun == 9:
    try:
        stat, dane = pobierzRaporty(firma=self.firma_id)
    except Exception, e:
        import traceback
        loguj(traceback.format_exc(), 1)
        stat = str(e)

if Fun == 15:
    try:
        dane = pobierzNazwy() + ['']  # dodana pusta opcja by na oknie mc wyczyci nazw
    except Exception, e:
        import traceback
        loguj(traceback.format_exc(), 1)
        stat = str(e)
