#!/usr/bin/python
# -*- coding: iso8859_2 -*-
#-------------------------------------------------------------------------------
# Name:             fs_generator_obsluga_definicji.py
# Purpose:          Zapisywanie raportw do definicji/pobieranie z definicji/eksport i import definicji z pliku
# Author:           Wojciech Skowron
# Created:          2021/01/15
# RCS-ID:           $Id: fs_generator_obsluga_definicji.py $
# Copyright:        (c) 2021 RecordSystem
# Licence:          GNU GPL
#-------------------------------------------------------------------------------

# 2021-07-28 Aleksander Bargie (12478) Problem z importem definicji raportw
# 2024-02-29 Wojciech Skowron   (13686) Poprawka w zapytaniach przy imporcie/eksporcie tabel _parametry

import os
import rsglobal
from funkcje import fs_pliki_global

naglowek_pola = ['raport_id', 'nazwa', 'szer_okna', 'wys_okna', 'naglowek', 'kolumny', 'tabela',
                 'sql_przed', 'sql_raport', 'sql_po', 'autor', 'modul_id', 'czcionka_pdf',
                 'czy_znakowy', 'czy_graficzny', 'czy_agata', 'produkcyjny']

parametry_pola = ['raport_id', 'lp', 'nazwa', 'opis', 'typ', 'dlugosc', 'po_przec',
                  'wsp_gora', 'wsp_lewa', 'wsp_dlug', 'wartosc_domyslna']

stat = 'OK'
dane = []
wyn1 = []
wyn2 = []

if not 'Funkcja' in klucze:
    stat = 'Brak podanej funkcji w kluczach'

if stat == 'OK':
    Funkcja = klucze['Funkcja']
    SlPar = {'Symbol': [1,3], 'CzyPrzeslac': [5], 'PlikRaporty': [6], 'PlikRaportyParam': [6], }

    Lista = rsglobal.SprawdzParametryWKluczach(SlPar, Funkcja, klucze)
    if Lista != '':
        stat = 'Uwaga : wywoanie funkcji fs_generator_obsluga_definicji z numerem ' +str(Funkcja)+' bez parametrw: '+Lista

if stat == 'OK':
    if Funkcja in [1,3]:
        Symbol = rsglobal.Dekoduj(klucze['Symbol'])
    if Funkcja in [5]:
        CzyPrzeslac = klucze['CzyPrzeslac']
    if Funkcja in [6]:
        PlikRaporty = klucze['PlikRaporty']
        PlikRaportyParam = klucze['PlikRaportyParam']

UPR = False
if self.operator_id == STANDARD_OPERATOR_ID:
    UPR = True
else:
    stat = 'Brak uprawnie'

sciezka = os.path.join('raporty', 'definicje')
nazwa_pliku_r = "rs_raporty_definicje.copy"
nazwa_pliku_rp = "rs_raporty_parametry_definicje.copy"

naglowek = ','.join(naglowek_pola)
naglowek_r = ','.join(['r.'+pole for pole in naglowek_pola])
parametry = ','.join(parametry_pola)
parametry_rp = ','.join(['rp.'+pole for pole in parametry_pola])

# -------------------------------------------------------------------------------
# ---   ZAPISZ DEFINICJʠRAPORTU (1)
# -------------------------------------------------------------------------------

if stat == 'OK' and Funkcja == 1 and UPR:
    # najpierw wykonywany update - jeli definicja raportu istnieje, zostanie nadpisana,
    # potem insert - jeli definicja raportu nie istnieje
    SQL = """UPDATE rs_raporty_definicje
             SET    ({naglowek}) = ({naglowek_r})
             FROM   (SELECT {naglowek} FROM rs_raporty 
                     WHERE firma_id='{self.firma_id}' AND raport_id='{Symbol}') AS r
             WHERE  rs_raporty_definicje.raport_id = r.raport_id;
             INSERT INTO rs_raporty_definicje ({naglowek})
             SELECT {naglowek}
             FROM   rs_raporty
             WHERE  firma_id='{self.firma_id}' AND raport_id='{Symbol}' 
                    AND NOT EXISTS (SELECT 1 FROM rs_raporty_definicje
                                    WHERE raport_id='{Symbol}');
             ---
             UPDATE rs_raporty_parametry_definicje
             SET    ({parametry}) = ({parametry_rp})
             FROM   (SELECT {parametry} FROM rs_raporty_parametry 
                     WHERE firma_id='{self.firma_id}' AND raport_id='{Symbol}') AS rp
             WHERE  rs_raporty_parametry_definicje.raport_id = rp.raport_id
                    AND rs_raporty_parametry_definicje.lp = rp.lp;
             INSERT INTO rs_raporty_parametry_definicje ({parametry})
             SELECT {parametry}
             FROM   rs_raporty_parametry
             WHERE  firma_id='{self.firma_id}' AND raport_id='{Symbol}' 
                    AND NOT EXISTS (SELECT 1 FROM rs_raporty_parametry_definicje
                                    WHERE raport_id='{Symbol}'
                                          AND rs_raporty_parametry_definicje.lp = rs_raporty_parametry.lp);"""
    SQL = SQL.format(**locals())
    rsglobal.ZapiszDoLogow(SQL)
    stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1, CzyDodawacBlad=True)
    if stat != 'OK':
        stat = 'Bd zapisu raportu do tabeli definicji: '+stat

# -------------------------------------------------------------------------------
# ---   ZAPISZ BRAKUJCE DEFINICJE RAPORTW (2)
# -------------------------------------------------------------------------------

if stat == 'OK' and Funkcja == 2 and UPR:
    SQL = """INSERT INTO rs_raporty_definicje ({naglowek})
             SELECT {naglowek}
             FROM   rs_raporty
             WHERE  firma_id='{self.firma_id}' 
                    AND NOT EXISTS (SELECT 1 FROM rs_raporty_definicje WHERE rs_raporty_definicje.raport_id = rs_raporty.raport_id);
             ---
             INSERT INTO rs_raporty_parametry_definicje ({parametry})
             SELECT {parametry}
             FROM   rs_raporty_parametry
             WHERE  firma_id='{self.firma_id}'
                    AND NOT EXISTS (SELECT 1 FROM rs_raporty_parametry_definicje WHERE rs_raporty_parametry_definicje.raport_id = rs_raporty_parametry.raport_id
                                                                                       AND rs_raporty_parametry_definicje.lp = rs_raporty_parametry.lp);"""
    SQL = SQL.format(**locals())
    rsglobal.ZapiszDoLogow(SQL)
    stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1, CzyDodawacBlad=True)
    if stat != 'OK':
        stat = 'Bd zapisu raportw do tabeli definicji: '+stat

# -------------------------------------------------------------------------------
# ---   POBIERZ DEFINICJʠRAPORTU (3)
# -------------------------------------------------------------------------------

if stat == 'OK' and Funkcja == 3 and UPR:
    # najpierw wykonywany update - jeli raport istnieje, zostanie nadpisany,
    # potem insert - jeli raport nie istnieje
    SQL = """UPDATE rs_raporty
             SET    ({naglowek}) = ({naglowek_r})
             FROM   (SELECT {naglowek} FROM rs_raporty_definicje
                     WHERE raport_id='{Symbol}') AS r
             WHERE  rs_raporty.raport_id = r.raport_id AND rs_raporty.firma_id='{self.firma_id}';
             INSERT INTO rs_raporty (firma_id, {naglowek})
             SELECT '{self.firma_id}', {naglowek}
             FROM   rs_raporty_definicje
             WHERE  raport_id='{Symbol}' AND NOT EXISTS (SELECT 1 FROM rs_raporty
                                                         WHERE firma_id='{self.firma_id}' AND raport_id='{Symbol}');
             ---
             UPDATE rs_raporty_parametry
             SET    ({parametry}) = ({parametry_rp})
             FROM   (SELECT {parametry} FROM rs_raporty_parametry_definicje
                     WHERE raport_id='{Symbol}') AS rp
             WHERE  rs_raporty_parametry.firma_id='{self.firma_id}' AND rs_raporty_parametry.raport_id = rp.raport_id 
                    AND rs_raporty_parametry.lp = rp.lp;
             INSERT INTO rs_raporty_parametry (firma_id, {parametry})
             SELECT '{self.firma_id}', {parametry}
             FROM   rs_raporty_parametry_definicje
             WHERE  raport_id='{Symbol}' 
                    AND NOT EXISTS (SELECT 1 FROM rs_raporty_parametry
                                    WHERE firma_id='{self.firma_id}' AND raport_id='{Symbol}' 
                                          AND rs_raporty_parametry.lp = rs_raporty_parametry_definicje.lp);"""
    SQL = SQL.format(**locals())
    rsglobal.ZapiszDoLogow(SQL)
    stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1, CzyDodawacBlad=True)
    if stat != 'OK':
        stat = 'Bd zapisu raportu pobranego z tabeli definicji: '+stat

# -------------------------------------------------------------------------------
# ---   POBIERZ BRAKUJCE DEFINICJE RAPORTW (4)
# -------------------------------------------------------------------------------

if stat == 'OK' and Funkcja == 4 and UPR:
    SQL = """INSERT INTO rs_raporty (firma_id, {naglowek})
             SELECT '{self.firma_id}', {naglowek}
             FROM   rs_raporty_definicje
             WHERE  NOT EXISTS (SELECT 1 FROM rs_raporty WHERE firma_id='{self.firma_id}' 
                                                               AND rs_raporty.raport_id = rs_raporty_definicje.raport_id);
             INSERT INTO rs_raporty_parametry (firma_id, {parametry})
             SELECT '{self.firma_id}', {parametry}
             FROM   rs_raporty_parametry_definicje
             WHERE  NOT EXISTS (SELECT 1 FROM rs_raporty_parametry WHERE firma_id='{self.firma_id}' 
                                                                         AND rs_raporty_parametry.raport_id = rs_raporty_parametry_definicje.raport_id
                                                                         AND rs_raporty_parametry.lp = rs_raporty_parametry_definicje.lp);"""
    SQL = SQL.format(**locals())
    rsglobal.ZapiszDoLogow(SQL)
    stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1, CzyDodawacBlad=True)
    if stat != 'OK':
        stat = 'Bd zapisu raportw pobranych z tabeli definicji: '+stat

# -------------------------------------------------------------------------------
# ---   EKSPORTUJ DEFINICJE RAPORTW DO PLIKU (5)
# -------------------------------------------------------------------------------

if stat == 'OK' and Funkcja == 5 and UPR:
    try:
        if not os.path.exists(sciezka):
            os.mkdir(sciezka)
    except OSError, exc:
        stat = 'Bd tworzenia katalogu: '+str(exc)

    if stat == 'OK':
        SQL = "COPY rs_raporty_definicje TO '"+os.path.abspath(os.path.join(sciezka, nazwa_pliku_r))+"';"
        SQL+= "COPY rs_raporty_parametry_definicje TO '"+os.path.abspath(os.path.join(sciezka, nazwa_pliku_rp))+"';"
        rsglobal.ZapiszDoLogow(SQL)
        stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1, CzyDodawacBlad=True)
        if stat != 'OK':
            stat = 'Bd zapisu definicji raportw do pliku: '+stat

    if stat == 'OK':
        if CzyPrzeslac:
            # pliki zostan przesane do klienta
            PlikRaporty = fs_pliki_global.PodajTrescPliku(os.path.join(sciezka, nazwa_pliku_r))
            PlikRaportyParam = fs_pliki_global.PodajTrescPliku(os.path.join(sciezka, nazwa_pliku_rp))

            dane = [PlikRaporty, PlikRaportyParam]

# -------------------------------------------------------------------------------
# ---   IMPORTUJ DEFINICJE RAPORTW Z PLIKU (6)
# -------------------------------------------------------------------------------

if stat == 'OK' and Funkcja == 6 and UPR:
    #AB 29.07.21r
    try:
        if not os.path.exists(sciezka):
            os.mkdir(sciezka)
    except OSError, exc:
        stat = 'Bd tworzenia katalogu: '+str(exc)
    # zapisanie przesanych plikw po stronie serwera
    if len(PlikRaporty) > 0 and len(PlikRaportyParam) > 0:
        stat = fs_pliki_global.ZapiszPlik(Tresc=PlikRaporty, Nazwa=nazwa_pliku_r, Katalog=sciezka, CzyZIP=True)
        if stat != 'OK':
            stat = 'Wystpi bd podczas przesyania pliku na serwer: '+stat
        stat = fs_pliki_global.ZapiszPlik(Tresc=PlikRaportyParam, Nazwa=nazwa_pliku_rp, Katalog=sciezka, CzyZIP=True)
        if stat != 'OK':
            stat = 'Wystpi bd podczas przesyania pliku na serwer: '+stat

    if stat == 'OK':
        # sprawdzenie czy istniej pliki z tabelami w katalogu sciezka
        if not os.path.exists(os.path.join(sciezka, nazwa_pliku_r)):
            stat = 'Brak pliku z definicjami na serwerze: '+os.path.join(sciezka, nazwa_pliku_r)
        elif not os.path.exists(os.path.join(sciezka, nazwa_pliku_rp)):
            stat = 'Brak pliku z definicjami na serwerze: '+os.path.join(sciezka, nazwa_pliku_rp)

    if stat == 'OK':
        SQL = "DELETE FROM rs_raporty_definicje; DELETE FROM rs_raporty_parametry_definicje;"
        rsglobal.ZapiszDoLogow(SQL)
        stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1, CzyDodawacBlad=True)
        if stat != 'OK':
            stat = 'Bd usuwania definicji raportw: '+stat

    if stat == 'OK':
        SQL = "COPY rs_raporty_definicje FROM '"+os.path.abspath(os.path.join(sciezka, nazwa_pliku_r))+"';"
        SQL+= "COPY rs_raporty_parametry_definicje FROM '"+os.path.abspath(os.path.join(sciezka, nazwa_pliku_rp))+"';"
        rsglobal.ZapiszDoLogow(SQL)
        stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1, CzyDodawacBlad=True)
        if stat != 'OK':
            stat = 'Bd wczytania definicji raportw z pliku: '+stat
