# -*- coding: iso8859_2 -*-
#-------------------------------------------------------------------------------
# Name:        fs_hd_artykuly_import_pom.py
# Purpose:     Import artykuw z PC Market - funkcje pomocnicze
# Author:      Wojciech Skowron
# Created:     2019/07/29
# RCS-ID:      $Id: fs_hd_artykuly_import_pom.py $
# Copyright:   (c) 2019 Record System
# Licence:     GNU GPL
#-------------------------------------------------------------------------------

# 2020-04-30 Wojciech Skowron - (11960) Podczas importu artykuw, na sztywno wpisywa grup towarow '!20'
# 2020-05-05 Wojciech Skowron - (11966) Import artykuw, ktre wystpiy na paragonach od 2020-01-01
# 2020-09-10 Wojciech Skowron - (12107) typ_artykulu = '01'

import rsmssql
import rsglobal
from rsglobal import STANDARD_OPERATOR_ID

stat = "OK"

#-------------------------------------------------------------------------------
#---   SPRAWDZENIE DANYCH
#-------------------------------------------------------------------------------

def Polacz(self):
    self.UprImport = self.ZHarmonogramu or ((self.oddzial_id,'092115') in self.uprawnienia) or (self.operator_id == STANDARD_OPERATOR_ID)

    if not self.UprImport:
        return 'BL_UPR_IMP', None

    plikKonf = "rs-and.conf"
    param    = ('pcmoperator','pcmhaslo','pcmhost','pcmport','pcmbaza')
    lista    = rsglobal.PodajZKonf(plikKonf, param)

    Logowanie = {}

    for par,wart in zip(param,lista):
        if wart == '':
            return "Brak parametru '"+par+"' w pliku konfiguracyjnym serwera.", None
        else:
            Logowanie[par] = wart

    RSMSSQL = rsmssql.RSMSSQL()
    stat = RSMSSQL.PolaczZSQL(Logowanie['pcmoperator'], Logowanie['pcmhaslo'], Logowanie['pcmhost'],
                              Logowanie['pcmport'], Baza=Logowanie['pcmbaza'])

    if stat == 'OK':
        rsglobal.ZapiszDoLogow('Pomylne poczono z serwerem PC Market')
        return stat, RSMSSQL
    else:
        stat = 'Dane logowania niepoprawne lub wystpi bd poczenia z serwerem PC Market:\n'+stat
        rsglobal.ZapiszDoLogow(stat)
        return stat, None

def Rozlacz(self):
    self.UprImport = self.ZHarmonogramu or ((self.oddzial_id,'092115') in self.uprawnienia) or (self.operator_id == STANDARD_OPERATOR_ID)

    if not self.UprImport:
        return 'BL_UPR_IMP'

    if self.RSMSSQL is not None:
        self.RSMSSQL.RozlaczZSQL()

def ImportujSlowniki(self):
    self.UprImport = self.ZHarmonogramu or ((self.oddzial_id,'092115') in self.uprawnienia) or (self.operator_id == STANDARD_OPERATOR_ID)

    if not self.UprImport:
        return 'BL_UPR_IMP'

    #JEDNOSTKI MIAR

    SEL = "SELECT Nazwa, Precyzja FROM JM;"
    stat,PCMDane,nagl = self.RSMSSQL.Execute(SEL)
    if stat != 'OK':
        return 'Pobieranie danych o jednostkach miar z PC Market nie powiodo si: '+stat

    SEL = """SELECT kod FROM rs_jednostki_miar
             WHERE firma_id='{}';""".format(self.firma_id)
    stat,RSDane,nagl,dump = self.PolSQL.Execute(SEL,3,CzyDodawacBlad=True)
    if stat != 'OK':
        return 'Pobieranie danych z rs_jednostki_miar nie powiodo si: '+stat

    INS = ''

    for PCMWiersz in PCMDane:
        Nazwa    = rsglobal.Dekoduj(PCMWiersz[0]).strip()
        Precyzja = PCMWiersz[1]

        Znaleziono = False
        for RSWiersz in RSDane:
            kod = rsglobal.Dekoduj(RSWiersz[0]).strip()

            if Nazwa[:5] == kod:
                Znaleziono = True

        if not Znaleziono:
            INS+= """INSERT INTO rs_jednostki_miar (firma_id, kod, opis, calkowita)
                     VALUES ('{firma_id}','{kod}','{opis}',{calkowita});
                  """.format(firma_id  = self.firma_id,
                             kod       = Nazwa[:5],
                             opis      = Nazwa,
                             calkowita = True if Precyzja <= 0 else False)

    if INS != '':
        # rsglobal.ZapiszDoLogow(INS)
        stat,dump,dump,dump = self.PolSQL.Execute(INS,1,CzyDodawacBlad=True)
        if stat != 'OK':
            return 'Importowanie sownika jednostek miar nie powiodo si: '+stat

    #STAWKI VAT

    # SEL = "SELECT Stawka FROM Vat;"
    # stat,PCMDane,nagl = self.RSMSSQL.Execute(SEL)
    # rsglobal.ZapiszDoLogow(str(stat) +'\n'+ str(PCMDane) +'\n'+ str(nagl))
    # if stat != 'OK':
    #     return 'Pobieranie danych o jednostkach miar z PC Market nie powiodo si: '+stat
    #
    # SEL = "SELECT kod FROM rs_stawki_vat;"
    # stat,RSDane,nagl,dump = self.PolSQL.Execute(SEL,3,CzyDodawacBlad=True)
    # rsglobal.ZapiszDoLogow(str(stat) +'\n'+ str(RSDane) +'\n'+ str(nagl))
    # if stat != 'OK':
    #     return 'Pobieranie danych z rs_stawki_vat nie powiodo si: '+stat
    #
    # INS = ''
    #
    # for PCMWiersz in PCMDane:
    #     Stawka = PCMWiersz[0]
    #
    #     Znaleziono = False
    #     for RSWiersz in RSDane:
    #         kod = rsglobal.Dekoduj(RSWiersz[0]).strip()
    #
    #         if str(Stawka/100).rjust(2,'0')[0:2] == kod:
    #             Znaleziono = True
    #
    #     if not Znaleziono:
    #         INS+= """INSERT INTO rs_stawki_vat
    #                  VALUES ('{kod}','{opis}');
    #               """.format(firma_id  = self.firma_id,
    #                          kod       = str(Stawka/100).rjust(2,'0')[0:2],
    #                          opis      = Stawka)
    #
    # if INS != '':
    #     rsglobal.ZapiszDoLogow(INS)
    #     stat,dump,dump,dump = self.PolSQL.Execute(INS,1,CzyDodawacBlad=True)
    #     if stat != 'OK':
    #         return 'Importowanie sownika stawek VAT nie powiodo si: '+stat

    #GRUPY TOWAROWE

    #WS 2020-04-30
    # SEL = "SELECT AsId, Nazwa FROM Asort;"
    # stat,PCMDane,nagl = self.RSMSSQL.Execute(SEL)
    # if stat != 'OK':
    #     return 'Pobieranie danych o grupach towarowych z PC Market nie powiodo si: '+stat
    #
    # SEL = """SELECT kod FROM rs_hd_grupy_towarowe
    #          WHERE firma_id='{}';""".format(self.firma_id)
    # stat,RSDane,nagl,dump = self.PolSQL.Execute(SEL,3,CzyDodawacBlad=True)
    # if stat != 'OK':
    #     return 'Pobieranie danych z rs_hd_grupy_towarowe nie powiodo si: '+stat
    #
    # INS = ''
    # #Zaimportowane kody s zawsze liczb, zostan uzupenione zerami aby poprawnie si sortoway
    # MaxDlKodu = len(str(len(PCMDane)))
    #
    # for PCMWiersz in PCMDane:
    #     AsId = int(PCMWiersz[0])
    #     Nazwa = rsglobal.Dekoduj(PCMWiersz[1]).strip()
    #
    #     Znaleziono = False
    #     for RSWiersz in RSDane:
    #         kod = rsglobal.Dekoduj(RSWiersz[0]).strip()
    #
    #         if str(AsId).rjust(MaxDlKodu,'0')[:MaxDlKodu] == kod:
    #             Znaleziono = True
    #
    #     if not Znaleziono:
    #         INS+= """INSERT INTO rs_hd_grupy_towarowe
    #                  VALUES ('{firma_id}','{kod}','{opis}');
    #               """.format(firma_id  = self.firma_id,
    #                          kod       = str(AsId).rjust(MaxDlKodu,'0')[:MaxDlKodu],
    #                          opis      = Nazwa)
    #
    # if INS != '':
    #     # rsglobal.ZapiszDoLogow(INS)
    #     stat,dump,dump,dump = self.PolSQL.Execute(INS,1,CzyDodawacBlad=True)
    #     if stat != 'OK':
    #         return 'Importowanie sownika grup towarowych nie powiodo si: '+stat

    #WS 2020-04-30 - wszystkie importowane artykuy maj na sztywno wpisywan grup towarow '!20',
    # jeli nie ma jej w sowniku, zostanie dodana
    SEL = """SELECT kod FROM rs_hd_grupy_towarowe
             WHERE firma_id='{}' AND kod='!20';""".format(self.firma_id)
    stat,RSDane,nagl,dump = self.PolSQL.Execute(SEL,3,CzyDodawacBlad=True)
    if stat != 'OK':
        return 'Pobieranie danych z rs_hd_grupy_towarowe nie powiodo si: '+stat
    elif len(RSDane) == 0:
        INS = """INSERT INTO rs_hd_grupy_towarowe
                 VALUES ('{}','!20','Import z PC Market');""".format(self.firma_id)
        stat,dump,dump,dump = self.PolSQL.Execute(INS,1,CzyDodawacBlad=True)
        if stat != 'OK':
            return 'Importowanie sownika grup towarowych nie powiodo si: '+stat

    return stat

def ImportujArtykuly(self):
    self.UprImport = self.ZHarmonogramu or ((self.oddzial_id,'092115') in self.uprawnienia) or (self.operator_id == STANDARD_OPERATOR_ID)

    if not self.UprImport:
        return 'BL_UPR_IMP', [], []

    dane = []
    wyn2 = []

    # ------------------------------------------------------------------------------------------------------------------
    # Pobranie wszystkich artykuw z PC Market, ktre wystpiy na paragonach od 2020-01-01
    # ------------------------------------------------------------------------------------------------------------------

    SEL = """SELECT T.TowId, T.Kod, T.Nazwa, T.Skrot, T.AsId, T.Stawka, T.Indeks2, T.Aktywny, J.Nazwa, T.UkrycNaKasie, A.Nazwa
             FROM Towar T 
             JOIN JM J ON T.JMId = J.JMId
             JOIN Asort A ON T.AsId = A.AsId
             WHERE T.TowId IN (
                 SELECT DISTINCT TowId
                 FROM PozDok P
                 JOIN Dok D on D.DokId = P.DokId
                 WHERE D.Data >= '2020-01-01 00:00:00' AND D.TypDok = 21);"""  #WS 2020-05-05 - 2019-01-01 -> 2020-01-01
    stat,PCMArtykuly,dump = self.RSMSSQL.Execute(SEL)
    if stat != 'OK':
        return 'Pobieranie artykuw z PC Market nie powiodo si: '+stat, [], []
    elif len(PCMArtykuly) == 0:
        stat = "Nie znaleziono artykuw do zaimportowania."
        LogZapisz(self, '---'+stat.upper()+'---')
        return stat, [], []

    # ------------------------------------------------------------------------------------------------------------------
    # Pobranie wszystkich artykuw z rs_artykuly w celu porwnania
    # ------------------------------------------------------------------------------------------------------------------

    SEL = """SELECT artykul_id FROM rs_artykuly WHERE firma_id='{}';""".format(self.firma_id)
    stat,RSArtykuly,dump,dump = self.PolSQL.Execute(SEL,3,CzyDodawacBlad=True)
    if stat != 'OK':
        return 'Pobieranie symboli artykuw z rs_artykuly nie powiodo si: '+stat, [], []

    LWszystkich = 0
    LZaimportowanych = 0
    LBlad = 0

    # ------------------------------------------------------------------------------------------------------------------
    # Porwnanie z rs_artykuly
    # ------------------------------------------------------------------------------------------------------------------

    PCMArtykulyNowe = [] # Artykuy jeszcze niewystpujce w rs_artykuly - do zaimportowania

    for PCMWiersz in PCMArtykuly:
        Znaleziono = False

        Kod = rsglobal.Dekoduj(PCMWiersz[1]).strip()

        for RSWiersz in RSArtykuly:
            Symbol = RSWiersz[0].strip()

            if Kod == Symbol:
                Znaleziono = True

        if not Znaleziono:
            # ----------------------------------------------------------------------------------------------------------
            # Sprawdzenie, czy jeli kodem artykuu jest EAN, to czy nie wystpuje ju w rs_artykuly_ean
            # ----------------------------------------------------------------------------------------------------------

            if Kod.isdigit() and len(Kod) in (7,8,12,13):
                SEL = """SELECT ean FROM rs_artykuly_ean WHERE firma_id='{}' AND ean='{}' AND artykul_id IS NOT NULL;""".format(self.firma_id, Kod)
                stat,RSEan,dump,dump = self.PolSQL.Execute(SEL,3,CzyDodawacBlad=True)
                if stat != 'OK':
                    return 'Pobieranie kodu ean z rs_artykuly_ean nie powiodo si: '+stat, [], []
                elif len(RSEan) > 0:
                    continue # przejcie do nastpnego artykuu

            LWszystkich += 1
            PCMArtykulyNowe.append(PCMWiersz)

    if LWszystkich > 0:
        LogZapisz(self, '---ROZPOCZYNAM IMPORTOWANIE---')

    for PCMWiersz in PCMArtykulyNowe:
        TowId        = str(int(PCMWiersz[0]))
        Kod          = rsglobal.Dekoduj(PCMWiersz[1]).strip()
        Nazwa        = rsglobal.Dekoduj(PCMWiersz[2]).replace("'", "`").strip()
        Skrot        = rsglobal.Dekoduj(PCMWiersz[3]).replace("'", "`").strip()
        AsId         = int(PCMWiersz[4])
        Stawka       = int(PCMWiersz[5])
        Indeks2      = rsglobal.Dekoduj(PCMWiersz[6]).strip()
        Aktywny      = PCMWiersz[7]
        JMNazwa      = rsglobal.Dekoduj(PCMWiersz[8]).strip()
        UkrycNaKasie = PCMWiersz[9]
        AsortNazwa   = PCMWiersz[10]

        # --------------------------------------------------------------------------------------------------------------
        # Sprawdzenie zgodnoci ze sownikami Anakondy (nie powinno by bdw, jeli poprawnie zaimportowane)
        # --------------------------------------------------------------------------------------------------------------

        wyn2 = [LZaimportowanych, LWszystkich]

        # JEDNOSTKI MIAR

        SEL = """SELECT kod FROM rs_jednostki_miar
                 WHERE firma_id='{}' AND TRIM(kod)='{}';""".format(self.firma_id, JMNazwa[:5])
        stat,RSDane,dump,dump = self.PolSQL.Execute(SEL,3,CzyDodawacBlad=True)
        if stat != 'OK':
            return 'Pobieranie danych z rs_jednostki_miar nie powiodo si: '+stat, dane, wyn2
        elif len(RSDane) == 0:
            return 'Niezgodno importowanych danych ze sownikiem jednostek miar (Nazwa: '+str(JMNazwa)+')', dane, wyn2
        else:
            JMNazwa = RSDane[0][0]

        # STAWKI VAT

        VAT = str(Stawka/100).rjust(2,'0')[:2] if Stawka >=0 else 'zw'

        SEL = """SELECT kod FROM rs_stawki_vat WHERE TRIM(kod)='{}';""".format(VAT)
        stat,RSDane,dump,dump = self.PolSQL.Execute(SEL,3,CzyDodawacBlad=True)
        if stat != 'OK':
            return 'Pobieranie danych z rs_stawki_vat nie powiodo si: '+stat, dane, wyn2
        elif len(RSDane) == 0:
            return 'Niezgodno importowanych danych ze sownikiem stawek VAT (Stawka: '+str(Stawka)+')', dane, wyn2
        else:
            VAT = RSDane[0][0]

        # GRUPY TOWAROWE

        #WS 2020-04-30
        # SEL = "SELECT AsId FROM Asort;"
        # stat,PCMDane,dump = self.RSMSSQL.Execute(SEL)
        # if stat != 'OK':
        #     return 'Pobieranie danych o grupach towarowych z PC Market nie powiodo si: '+stat, dane, wyn2
        # else:
        #     MaxDlKoduGrupaTow = len(str(len(PCMDane)))
        #
        # KodGrupaTow = str(AsId).rjust(MaxDlKoduGrupaTow,'0')[:MaxDlKoduGrupaTow]
        #
        # SEL = """SELECT kod FROM rs_hd_grupy_towarowe WHERE firma_id='{}' AND TRIM(kod)='{}';""".format(self.firma_id, KodGrupaTow)
        # stat,RSDane,dump,dump = self.PolSQL.Execute(SEL,3,CzyDodawacBlad=True)
        # if stat != 'OK':
        #     return 'Pobieranie danych z rs_hd_grupy_towarowe nie powiodo si: '+stat, dane, wyn2
        # elif len(RSDane) == 0:
        #     return 'Niezgodno importowanych danych ze sownikiem grup towarowych (AsId: '+str(AsId)+')', dane, wyn2
        # else:
        #     KodGrupaTow = RSDane[0][0]

        # --------------------------------------------------------------------------------------------------------------
        # Sprawdzenie unikalnoci pary nazwa skrcona - stawka VAT wrd aktywnych artykuw
        # --------------------------------------------------------------------------------------------------------------

        if Skrot not in ('','None',None):
            SEL = """SELECT artykul_id
                     FROM rs_artykuly
                     WHERE firma_id='{}' AND aktywny=TRUE AND nazwa_skrocona='{}' AND artykul_id!='{}';""".format(self.firma_id, Skrot, Kod)
            stat,d,dump,dump = self.PolSQL.Execute(SEL,3,CzyDodawacBlad=True)
            if stat != 'OK':
                return 'Bd pobierania informacji z rs_artykuly: '+stat, dane, wyn2
            elif len(d) != 0:
                info = 'TowId: '+TowId.ljust(9)+' Kod: '+Kod.ljust(20)+'BD: Istniej inne aktywne artykuy o tej samej stawce VAT i nazwie skrconej: '+', '.join([art[0].strip() for art in d])
                dane.append(info)
                LogZapisz(self, info)

                LBlad += 1

                continue # przejcie do nastpnego artykuu

        # --------------------------------------------------------------------------------------------------------------
        # Import
        # --------------------------------------------------------------------------------------------------------------

        INS = """INSERT INTO rs_artykuly 
                             (  firma_id,    artykul_id,    nazwa,    nazwa_skrocona,    typ_artykulu,    grupa_towarowa,    stawka_vat,    stawka_vat_import,    sww_pkwiu,   aktywny,   jm1,    jm2,   nie_na_kasy,  wprowadzil, data_wprowadzenia,tylko_detal)
                      VALUES ('{firma_id}','{artykul_id}','{nazwa}','{nazwa_skrocona}','{typ_artykulu}','{grupa_towarowa}','{stawka_vat}','{stawka_vat_import}','{sww_pkwiu}',{aktywny},'{jm1}','{jm2}',{nie_na_kasy},{wprowadzil},NOW(),            TRUE);
              """.format(firma_id          = self.firma_id,
                         artykul_id        = Kod,
                         nazwa             = Nazwa[:50],
                         nazwa_skrocona    = Skrot[:35],
                         typ_artykulu      = '01',  #WS 2020-09-10
                         grupa_towarowa    = '!20',  #WS 2020-04-30
                         stawka_vat        = VAT,
                         stawka_vat_import = VAT,
                         sww_pkwiu         = Indeks2[:20],
                         aktywny           = bool(Aktywny),
                         jm1               = JMNazwa[:5],
                         jm2               = JMNazwa[:5],
                         nie_na_kasy       = bool(UkrycNaKasie) or AsortNazwa == '_USU Z KAS',
                         wprowadzil        = 0 if self.ZHarmonogramu else self.operator_id)
        # LogZapisz(self, INS)
        stat,dump,dump,dump = self.PolSQL.Execute(INS,1,CzyDodawacBlad=True)
        if stat != 'OK':
            info = 'TowId: '+TowId.ljust(9)+' Kod: '+Kod.ljust(20)+stat
            dane.append(info)
            LogZapisz(self, info)

            LBlad += 1

            continue # przejcie do nastpnego artykuu

        INS = """INSERT INTO rs_artykuly_stawki_vat
                             (  firma_id,    artykul_id,    stawka_vat,  od_dnia,      sww_pkwiu,    stawka_vat_import,    nazwa_skrocona)
                      VALUES ('{firma_id}','{artykul_id}','{stawka_vat}',NOW()::DATE,'{sww_pkwiu}','{stawka_vat_import}','{nazwa_skrocona}');
              """.format(firma_id          = self.firma_id,
                         artykul_id        = Kod,
                         stawka_vat        = VAT,
                         sww_pkwiu         = Indeks2[:20],
                         stawka_vat_import = VAT,
                         nazwa_skrocona    = Skrot[:35])
        # LogZapisz(self, INS)
        stat,dump,dump,dump = self.PolSQL.Execute(INS,1,CzyDodawacBlad=True)
        if stat != 'OK':
            info = 'TowId: '+TowId.ljust(9)+' Kod: '+Kod.ljust(20)+stat
            dane.append(info)
            LogZapisz(self, info)

            LBlad += 1
        else:
            self.PolSQL.Commit()
            LZaimportowanych += 1

            # ------------------------------------------------------------------------------------------------------
            # Sprawdzenie zaimportowanych danych pod ktem zgodnoci z rs_artykuly
            # ------------------------------------------------------------------------------------------------------

            if len(Nazwa) > 50:
                info = 'TowId: '+TowId.ljust(9)+' Kod: '+Kod.ljust(20)+'Nazwa artykuu zostaa skrcona (przekracza 50 znakw): '+"'"+Nazwa+"' -> '"+Nazwa[:50]+"'"
                dane.append(info)
                LogZapisz(self, info)
            if Skrot == '':
                info = 'TowId: '+TowId.ljust(9)+' Kod: '+Kod.ljust(20)+'Pusta nazwa skrcona (Skrot)'
                dane.append(info)
                LogZapisz(self, info)
            elif len(Skrot) > 50:
                info = 'TowId: '+TowId.ljust(9)+' Kod: '+Kod.ljust(20)+'Nazwa skrcona artykuu (Skrot) zostaa skrcona (przekracza 35 znakw): '+"'"+Skrot+"' -> '"+Skrot[:35]+"'"
                dane.append(info)
                LogZapisz(self, info)
            if len(Indeks2) > 20:
                info = 'TowId: '+TowId.ljust(9)+' Kod: '+Kod.ljust(20)+'Kod SWW/PKWiU (Indeks2) zosta skrcony (przekracza 20 znakw)'
                dane.append(info)
                LogZapisz(self, info)
            # if Stawka != 2300 and Indeks2 == '':
            #     info = 'TowId: '+TowId.ljust(9)+' Kod: '+Kod.ljust(20)+'Brak kodu SWW/PKWiU (Indeks2) przy stawce VAT <> 23%'
            #     dane.append(info)
            #     LogZapisz(self, info)

            # ------------------------------------------------------------------------------------------------------
            # Jeli kodem artykuu jest EAN, zostaje wpisany do rs_artykuly_ean
            # ------------------------------------------------------------------------------------------------------

            if Kod.isdigit() and len(Kod) in (7,8,12,13):
                INS = """INSERT INTO rs_artykuly_ean
                                     (  firma_id,    artykul_id,    ean  )
                              VALUES ('{firma_id}','{artykul_id}','{ean}')
                      """.format(firma_id       = self.firma_id,
                                 artykul_id     = Kod,
                                 ean            = Kod)

                # LogZapisz(self, INS)
                stat,dump,dump,dump = self.PolSQL.Execute(INS,1,CzyDodawacBlad=True)
                if stat != 'OK':
                    info = 'TowId: '+TowId.ljust(9)+' Kod: '+Kod.ljust(20)+'Bd dodawania kodu EAN: '+stat
                    dane.append(info)
                    LogZapisz(self, info)

                    LBlad += 1
                else:
                    self.PolSQL.Commit()

    if LZaimportowanych == 0 and LBlad == 0:
        dane.append("Brak nowych artykuw do zaimportowania.")
        LogZapisz(self, '---'+stat.upper()+'---')
    else:
        LogZapisz(self, '---ZAIMPORTOWANO NOWYCH ARTYKUW: '+str(LZaimportowanych)+'/'+str(LWszystkich)+'---')
        wyn2 = [LZaimportowanych, LWszystkich]

    return 'OK', dane, wyn2

def Import(self, ZHarmonogramu = False, ZapiszLog = None):
    self.ZHarmonogramu = ZHarmonogramu
    self.ZapiszLog = ZapiszLog
    self.UprImport = self.ZHarmonogramu or ((self.oddzial_id,'092115') in self.uprawnienia) or (self.operator_id == STANDARD_OPERATOR_ID)

    stat = 'OK'
    dane = []
    wyn2 = []

    stat, self.RSMSSQL = Polacz(self)
    if self.RSMSSQL is not None and stat == 'OK':
        stat = ImportujSlowniki(self)
        stat, dane, wyn2 = ImportujArtykuly(self) if stat == 'OK' else (stat, [], [])
        Rozlacz(self)

    return stat, dane, wyn2

def LogZapisz(self, s):
    if not self.ZHarmonogramu:
        rsglobal.ZapiszDoLogow(s)
    elif self.ZapiszLog is not None:
        self.ZapiszLog(s)
