# -*- coding: iso8859_2 -*-
#-------------------------------------------------------------------------------
# Name:        fs_hd_faktury_do_paragonow_import_pom.py
# Purpose:     Import faktur do paragonw fiskalnych z PC Market do dowodw ksigowych - funkcje pomocnicze
# Author:      Wojciech Skowron
# Created:     2020/08/21
# RCS-ID:      $Id: fs_hd_faktury_do_paragonow_import_pom.py $
# Copyright:   (c) 2020 Record System
# Licence:     GNU GPL
#-------------------------------------------------------------------------------

#WS 2020-09-25 - zaokrglenia wszystkich kwot do 2 miejsc po przecinku
#WS 2020-11-19 - (12193) zmiana numerw kont wpisywanych na dowod ksiegowy oraz ozn_sprzedaz='FP'
#WS 2023-06-05 - (12105) Porwnanie nazwy/adresu kontrahentw niewraliwe na polskie znaki
#WS 2024-02-13 - (13694) Porwnanie kontrahentw tylko po NIP
#WS 2024-02-27 - (13707) Przy porwnaniu kontrahentw - usunity warunek AND aktywny=TRUE

import rsglobal

stat = "OK"

def ImportujFakturyDoParagonow(self, DataImport, Sklep, WynikEmail, DokKontr, FunkcjaZapisuDoLogow=rsglobal.ZapiszDoLogow):
    """Funkcja importujca faktury do paragonw fiskalnych z PC Market do dowodw ksigowych, z dnia DataImport dla sklepu (placwki) Sklep

       Parameters:
           DataImport (str):                data 'RRRR-MM-DD'
           Sklep (tuple):                   tupla (sklep_id, pcmarket), czyli pola z rs_hd_sklepy
           WynikEmail (list):               tre e-maila z wynikami importu
           DokKontr (dict):                 klucz - NrDok, warto - kontrahent_id
           FunkcjaZapisuDoLogow (function): funkcja wykorzystywana do zapisu do logw, w przypadku harmonogramu trzeba przekazaZapiszLog"""

    info = 'Placwka {} z dnia {}: '.format(Sklep[0].strip(), DataImport)

    SklepRS = Sklep[0]
    SklepPCM = int(Sklep[1])

    # ----------------------------------------------------------------------------------------------------------
    # Znalezienie okresu ksigowego pasujcego do daty faktur
    # ----------------------------------------------------------------------------------------------------------

    SEL = """SELECT okres_id FROM rs_definicje_okresow
             WHERE firma_id='{self.firma_id}'
             AND '{DataImport}' BETWEEN data_od AND data_do;""".format(**locals())
    stat, RSOkres, dump, dump = self.PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
    if stat != 'OK':
        blad = 'import przerwany, bd pobrania danych z rs_definicje_okresow   \r\n' + stat
        FunkcjaZapisuDoLogow(info + blad)
        WynikEmail.append(info + blad)
        return blad
    elif len(RSOkres) == 0:
        blad = 'import przerwany, brak okresu ksigowego pasujcego do daty faktur'
        FunkcjaZapisuDoLogow(info + blad)
        WynikEmail.append(info + blad)
        return blad
    else:
        okres_id = RSOkres[0][0]

    # ----------------------------------------------------------------------------------------------------------
    # Pobranie symboli faktur ju zaimportowanych
    # ----------------------------------------------------------------------------------------------------------

    # symbol_dok w rs_dowody_ksiegowe odpowiada NrDok z tabeli Dok PC Marketu

    SEL = """SELECT symbol_dok FROM rs_dowody_ksiegowe
             WHERE firma_id='{self.firma_id}'
             AND dowod_id = 'FR' AND rejestr = 'FR' AND tresc = 'Faktura do paragonu'
             AND data >= '2020-08-01 00:00:00'
             AND data >= '{DataImport}'::DATE AND data < '{DataImport}'::DATE + interval '1 day'
             AND nazwa_id = '{SklepRS}';""".format(**locals())
    stat, RSFaktury, dump, dump = self.PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
    if stat != 'OK':
        blad = "import przerwany, bd pobrania listy numerw zaimportowanych faktur z rs_dowody_ksiegowe   \r\n" + stat
        FunkcjaZapisuDoLogow(info + blad)
        WynikEmail.append(info + blad)
        return blad
    else:
        FakturyWczesniejZaimportowane = [faktura[0] for faktura in RSFaktury]

    # ----------------------------------------------------------------------------------------------------------
    # Pobranie danych nagwkw faktur z PC Market
    # ----------------------------------------------------------------------------------------------------------

    NrDokWar = "AND D.NrDok not in ('"+"','".join(FakturyWczesniejZaimportowane)+"')" if len(FakturyWczesniejZaimportowane) > 0 else ''

    #WS 2020-09-25 - dodany ROUND na D.Razem
    SEL = """SELECT D.DokId, D.Data, D.NrDok, ROUND(D.Razem,2), TD1.Tekst AS NIP, TD2.Tekst AS PlatNIP
             FROM Dok D
             LEFT OUTER JOIN TekstDok TD1 ON TD1.DokId = D.DokId AND TD1.Znaczenie = 93
             LEFT OUTER JOIN TekstDok TD2 ON TD2.DokId = D.DokId AND TD2.Znaczenie = 162
             WHERE D.Data >= '2020-08-01 00:00:00' 
             AND D.Data >= '{DataImport} 00:00:00' AND D.Data < DATEADD(day, 1, '{DataImport}')
             AND D.MagId = {SklepPCM} AND D.TypDok = 36
             {NrDokWar};""".format(**locals())
    stat, PCMFakturyNagl, dump = self.RSPCMarketSQL.Execute(SEL)
    if stat != 'OK':
        blad = "import przerwany, bd pobrania danych nagwkw faktur   \r\n" + stat
        FunkcjaZapisuDoLogow(info + blad)
        WynikEmail.append(info + blad)
        return blad
    elif len(PCMFakturyNagl) == 0:
        # FunkcjaZapisuDoLogow(info + 'brak nowych faktur')
        return 'OK'

    LNowych = 0
    LBrakNIP = 0

    # ----------------------------------------------------------------------------------------------------------

    for faktura in PCMFakturyNagl:
        if faktura[4] is None and faktura[5] is None:
            # pominicie faktur, ktrych kontrahent i patnik nie ma NIP
            LBrakNIP += 1
            continue
        else:
            LNowych += 1

        # Dane faktury z Dok
        DokId = int(faktura[0])
        Data  = faktura[1]  # Data wystawienia faktury
        NrDok = faktura[2]  # Symbol faktury
        Razem = faktura[3]  # Brutto

        # ----------------------------------------------------------------------------------------------------------
        # Pobranie numeru dla nowego dowodu ksigowego
        # ----------------------------------------------------------------------------------------------------------

        SEL_SPR = """SELECT rsf_podaj_nowy_numer_dokumentu('{self.firma_id}',
                     '[''{okres_id}'',''FR'',''{SklepRS}'',''{Data}'']', 'D');""".format(**locals())
        stat, RSNumer, dump, dump = self.PolSQL.Execute(SEL_SPR, 3, CzyDodawacBlad=True)

        if stat != 'OK':
            blad = "import przerwany, bd pobrania nowego numeru dowodu ksigowego dla faktury {}   \r\n".format(NrDok) + stat
            FunkcjaZapisuDoLogow(info + blad)
            WynikEmail.append(info + blad)
            return blad
        elif RSNumer[0][0].strip() in ('None', 'BRAK??', ''):
            blad = "import przerwany, bdnie pobrany numer nowego dowodu ksigowego dla faktury {}   \r\n".format(NrDok)
            FunkcjaZapisuDoLogow(info + blad)
            WynikEmail.append(info + blad)
            return blad
        else:
            numer = RSNumer[0][0]

        # ----------------------------------------------------------------------------------------------------------
        # Zapis nagwka do rs_dowody_ksiegowe
        # ----------------------------------------------------------------------------------------------------------

        # kwota_wn i kwota_ma uzupeniane pniej, po imporcie pozycji

        kontrahent_id = DokKontr[NrDok]

        #WS 2020-11-19 - ozn_sprzedaz='FP'
        INS = """INSERT INTO rs_dowody_ksiegowe
                 (firma_id, okres_id, dowod_id, nazwa_id, data, numer, 
                 stan, symbol_dok, data_wyst, kontrahent_id, rejestr, 
                 data_rejestru, tresc, ozn_sprzedaz)
                 VALUES 
                 ('{self.firma_id}','{okres_id}','FR','{SklepRS}','{Data}',{numer},
                 'W','{NrDok}','{Data}','{kontrahent_id}','FR',
                 '{Data}','Faktura do paragonu','FP');
              """.format(**locals())
        # FunkcjaZapisuDoLogow(INS)
        stat, dump, dump, dump = self.PolSQL.Execute(INS, 1, CzyDodawacBlad=True)
        if stat != 'OK':
            blad = "import przerwany, bd zapisu dowodu ksigowego dla faktury  {}   \r\n".format(NrDok) + stat
            FunkcjaZapisuDoLogow(info + blad)
            WynikEmail.append(info + blad)
            return blad

        # ------------------------------------------------------------------------------------------------------
        # Pobranie danych pozycji faktury z PC Market
        # ------------------------------------------------------------------------------------------------------

        # Pobranie danych potrzebnych do wypenienia rs_dowody_ksiegowe_pozycje
        # Sumy netto i VAT w podziale na stawki

        #WS 2020-09-25 - dodany ROUND
        SEL = """SELECT PD.Stawka,
                        ROUND(SUM(PD.Wartosc),2),
                        ROUND(SUM((CAST(PD.Stawka as decimal)/CAST(10000 as decimal))*PD.Wartosc),2)
                 FROM Dok D
                 JOIN PozDok PD ON (D.DokId = PD.DokId)
                 WHERE D.DokId={DokId}
                 GROUP BY PD.Stawka;""".format(**locals())
        stat, PCMFakturyPoz, dump = self.RSPCMarketSQL.Execute(SEL)
        if stat != 'OK':
            blad = "import przerwany, bd pobrania pozycji faktury {}   \r\n".format(NrDok) + stat
            FunkcjaZapisuDoLogow(info + blad)
            WynikEmail.append(info + blad)
            return blad

        # ------------------------------------------------------------------------------------------------------
        # Zapis pozycji do rs_dowody_ksiegowe_pozycje
        # ------------------------------------------------------------------------------------------------------

        LPozycji = 1 + len(PCMFakturyPoz)

        for nr_poz in range(LPozycji):
            if nr_poz == 0:  # Pozycja brutto
                #WS 2020-11-19 - zmiana konta z 200-01 na 275-04
                INS_POZ = """INSERT INTO rs_dowody_ksiegowe_pozycje
                             (firma_id, okres_id, dowod_id, nazwa_id, data, numer,
                             nr_poz, nr_ppoz, nr_pozw, pozycja_ze_wzoru, symbol_dok, data_wyst,
                             konto_wn, kwota_wn, konto_vat_wn, tresc_wn, waluta_wn, kwota_wal_wn, kurs_wn)
                             VALUES 
                             ('{self.firma_id}','{okres_id}','FR','{SklepRS}','{Data}',{numer},
                             {nr_poz}+1,0,1,TRUE,'{NrDok}','{Data}',
                             '275-04',{Razem},'','Faktura do paragonu','PLN',{Razem},1.0);
                          """.format(**locals())

            else:  # Pozycje dla stawek
                Stawka = int(PCMFakturyPoz[nr_poz-1][0])  # 0, 500, 800, 2300
                Netto  = PCMFakturyPoz[nr_poz-1][1]
                VAT    = PCMFakturyPoz[nr_poz-1][2]

                stawka_vat_ma = str(Stawka/100).rjust(2, '0')[:2] if Stawka >= 0 else 'zw'

                #WS 2020-11-19 - zmiana konta z 732-01 na 997-01 i 222-02 na 992-01
                INS_POZ+= """INSERT INTO rs_dowody_ksiegowe_pozycje
                             (firma_id, okres_id, dowod_id, nazwa_id, data, numer,
                             nr_poz, nr_ppoz, nr_pozw, pozycja_ze_wzoru, symbol_dok, data_wyst,
                             konto_ma, kwota_ma, konto_vat_ma, kwota_vat_ma, stawka_vat_ma, tresc_ma, 
                             waluta_ma, kurs_ma, kwota_wal_ma)
                             VALUES 
                             ('{self.firma_id}','{okres_id}','FR','{SklepRS}','{Data}',{numer},
                             {nr_poz}+1,0,2,TRUE,'{NrDok}','{Data}',
                             '997-01-{SklepRS}',{Netto},'992-01',{VAT},'{stawka_vat_ma}','Faktura do paragonu',
                             'PLN',1.0,{Netto});
                          """.format(**locals())

        # FunkcjaZapisuDoLogow(INS_POZ)
        stat, dump, dump, dump = self.PolSQL.Execute(INS_POZ, 1, CzyDodawacBlad=True)
        if stat != 'OK':
            blad = "import przerwany, bd zapisu pozycji dowodu ksigowego dla faktury {}   \r\n".format(NrDok) + stat
            FunkcjaZapisuDoLogow(info + blad)
            WynikEmail.append(info + blad)
            return blad

        # dopisanie sum z pozycji do nagwka dowodu
        INS_KWOTY = """UPDATE rs_dowody_ksiegowe dk
                       SET kwota_wn = dkp.suma_kwot_wn,
                           kwota_ma = dkp.suma_kwot_ma
                       FROM (SELECT firma_id, okres_id, dowod_id, nazwa_id, data, numer,
                                    SUM(kwota_wn)+SUM(kwota_vat_wn) AS suma_kwot_wn, 
                                    SUM(kwota_ma)+SUM(kwota_vat_ma) AS suma_kwot_ma
                             FROM rs_dowody_ksiegowe_pozycje
                             GROUP BY firma_id, okres_id, dowod_id, nazwa_id, data, numer) dkp
                       WHERE (dk.firma_id, dk.okres_id, dk.dowod_id, dk.nazwa_id, dk.data, dk.numer)=
                       (dkp.firma_id, dkp.okres_id, dkp.dowod_id, dkp.nazwa_id, dkp.data, dkp.numer)
                       AND (dk.firma_id, dk.okres_id, dk.dowod_id, dk.nazwa_id, dk.data, dk.numer)=
                       ('{self.firma_id}','{okres_id}','FR','{SklepRS}','{Data}',{numer});""".format(**locals())
        # FunkcjaZapisuDoLogow(INS_KWOTY)
        stat, dump, dump, dump = self.PolSQL.Execute(INS_KWOTY, 1, CzyDodawacBlad=True)
        if stat != 'OK':
            blad = "import przerwany, bd zapisu dowodu ksigowego dla faktury {}   \r\n".format(NrDok) + stat
            FunkcjaZapisuDoLogow(info + blad)
            WynikEmail.append(info + blad)
            return blad

    # ------------------------------------------------------------------------------------------------------

    wynik = info + 'zaimportowano '+str(LNowych)+' nowych faktur'
    if LBrakNIP > 0:
        wynik += ', nie zaimportowano '+str(LBrakNIP)+' faktur z powodu braku NIP'
    FunkcjaZapisuDoLogow(wynik)
    WynikEmail.append(wynik+'   \r\n')

    return 'OK'


def PrzygotujImport(self, Daty, Sklepy, FunkcjaZapisuDoLogow=rsglobal.ZapiszDoLogow):
    """Funkcja sprawdzajca par rzeczy przed rozpoczciem importu

       Parameters:
           Daty (list):          lista dat dla ktrych importujemy faktury
           Sklepy (list):        lista tupli (sklep_id, pcmarket), czyli pl z rs_hd_sklepy"""

    # ----------------------------------------------------------------------------------------------------------
    # Sprawdzenie, czy dla firmy istnieje rejestr VAT o kodzie 'FR'
    # ----------------------------------------------------------------------------------------------------------

    SEL_SPR = """SELECT 1 FROM rs_rejestry_vat
                 WHERE firma_id='{self.firma_id}' AND kod='FR';""".format(**locals())
    stat, RSRejestr, dump, dump = self.PolSQL.Execute(SEL_SPR, 3, CzyDodawacBlad=True)
    if stat != 'OK':
        stat = 'bd pobrania danych z rs_rejestry_vat   \r\n' + stat
        return stat
    elif len(RSRejestr) == 0:
        stat = "brak rejestru VAT 'FR'"
        return stat

    # ----------------------------------------------------------------------------------------------------------
    # Znalezienie okresw ksigowych pasujcych do dat importu
    # ----------------------------------------------------------------------------------------------------------

    Okresy = []
    DatyNieWOkresie = []
    for data in Daty:
        SEL_SPR = """SELECT okres_id FROM rs_definicje_okresow
                     WHERE firma_id='{self.firma_id}'
                     AND '{data}' BETWEEN data_od AND data_do;""".format(**locals())
        stat, RSOkres, dump, dump = self.PolSQL.Execute(SEL_SPR, 3, CzyDodawacBlad=True)
        if stat != 'OK':
            stat = 'bd pobrania danych z rs_definicje_okresow   \r\n' + stat
            return stat
        elif len(RSOkres) == 0:
            DatyNieWOkresie.append(data)
        else:
            Okresy.append(RSOkres[0][0])

    if len(DatyNieWOkresie) > 0:
        return "brak okresu ksigowego pasujcego do dat importu: '"+"', '".join(DatyNieWOkresie)+"'"

    # ----------------------------------------------------------------------------------------------------------
    # Sprawdzenie, czy istnieje wzr dekretacji dla (firma_id, okres_id, dowod_id, nazwa_id)
    # ----------------------------------------------------------------------------------------------------------

    # dowod_id to zawsze 'FR'
    # nazwa_id odpowiada sklep_id

    BrakujaceWzory = []
    for okres in set(Okresy):
        for sklep in Sklepy:
            SklepRS = sklep[0]

            SEL_SPR = """SELECT 1 FROM rs_wzory_dekretacji
                         WHERE firma_id='{self.firma_id}' AND okres_id='{okres}'
                         AND dowod_id='FR' AND nazwa_id='{SklepRS}';""".format(**locals())
            stat, RSWzor, dump, dump = self.PolSQL.Execute(SEL_SPR, 3, CzyDodawacBlad=True)
            if stat != 'OK':
                stat = 'bd pobrania danych z rs_wzory_dekretacji   \r\n' + stat
                return stat
            elif len(RSWzor) == 0:
                BrakujaceWzory.append((okres, SklepRS))

    if len(BrakujaceWzory) > 0:
        stat = ''
        for wzor in BrakujaceWzory:
            okres = wzor[0]
            nazwa = wzor[1]
            stat += "w okresie '{okres}' brak wzoru dekretacji dla dowodu 'FR', nazwa id '{nazwa}'   \r\n".format(**locals())
        return stat

    return 'OK'


def ZnajdzKontrahentow(self, Daty, Sklepy, FunkcjaZapisuDoLogow=rsglobal.ZapiszDoLogow):
    """Funkcja pobierajca dane kontrahentw z faktur PC Marketu i na podstawie tych danych
       szukajca odpowiedniego kontrahenta w rs_kontrahenci

       Parameters:
           Daty (list):          lista dat dla ktrych importujemy faktury
           Sklepy (list):        lista tupli (sklep_id, pcmarket), czyli pl z rs_hd_sklepy"""

    import collections

    # Sownik DokKontr bdzie zawiera znalezione w rs_kontrahenci odpowiedniki
    # dla danych kontrahentw z faktur PC Marketu (brak nru w sowniku oznacza brak znalezionego dopasowania)
    # klucz - NrDok, warto - kontrahent_id
    DokKontr = {}
    # Sownik Kontrahenci bdzie zawiera dane kontrahentw z faktur PC Marketu
    # oraz numery faktur, na ktych te dane si pojawiy
    # klucz - dane kontrahenta (SklepRS, Nazwa, Ulica, UlicaNry, Kod, Miejscowosc, NIP), warto - lista NrDok
    Kontrahenci = collections.OrderedDict()
    # Jeli ktrykolwiek z kontrahentw nie bdzie mia odpowiednika w rs_kontrahenci,
    # import nie zostanie wykonany, zamiast tego zostanie wywoana funkcja WyslijEmailZListaBrakujacychKontrahentow
    importowac = True

    ZbiorDat = "','".join(data for data in Daty)
    ZbiorSklepow = ",".join(str(sklep[1]) for sklep in Sklepy)

    # ------------------------------------------------------------------------------------------------------
    # Pobranie wszystkich numerw faktur z danego okresu
    # ------------------------------------------------------------------------------------------------------

    SEL = """SELECT D.DokId, D.NrDok, M.Nazwa AS SklepRS
             FROM Dok D
             JOIN Magazyn M ON (D.MagId = M.MagId)
             WHERE D.Data >= '2020-08-01 00:00:00' AND D.Data in ('{ZbiorDat}')
             AND D.MagId in ({ZbiorSklepow}) AND D.TypDok = 36
             ORDER BY SklepRS;""".format(**locals())
    # FunkcjaZapisuDoLogow(SEL)
    stat, PCMFaktury, dump = self.RSPCMarketSQL.Execute(SEL)
    if stat != 'OK':
        stat = 'bd pobrania danych faktur z PC Market   \r\n' + stat
        return stat, [], [], False

    for faktura in PCMFaktury:
        DokId   = int(faktura[0])
        NrDok   = faktura[1]
        SklepRS = faktura[2][:5]

        # ------------------------------------------------------------------------------------------------------
        # Pobranie danych kontrahenta/patnika dla kadej faktury
        # ------------------------------------------------------------------------------------------------------

        SEL = """SELECT
                 (SELECT Tekst FROM TekstDok WHERE DokId={DokId} AND Znaczenie = 88)  AS Nazwa,
                 (SELECT Tekst FROM TekstDok WHERE DokId={DokId} AND Znaczenie = 89)  AS Ulica,
                 (SELECT Tekst FROM TekstDok WHERE DokId={DokId} AND Znaczenie = 145) AS NrUlicy,
                 (SELECT Tekst FROM TekstDok WHERE DokId={DokId} AND Znaczenie = 146) AS NrLokalu,
                 (SELECT Tekst FROM TekstDok WHERE DokId={DokId} AND Znaczenie = 90)  AS Kod,
                 (SELECT Tekst FROM TekstDok WHERE DokId={DokId} AND Znaczenie = 91)  AS Miejscowosc,
                 (SELECT Tekst FROM TekstDok WHERE DokId={DokId} AND Znaczenie = 93)  AS NIP,
                 (SELECT Tekst FROM TekstDok WHERE DokId={DokId} AND Znaczenie = 157) AS PlatNazwa,
                 (SELECT Tekst FROM TekstDok WHERE DokId={DokId} AND Znaczenie = 158) AS PlatUlica,
                 (SELECT Tekst FROM TekstDok WHERE DokId={DokId} AND Znaczenie = 165) AS PlatNrUlicy,
                 (SELECT Tekst FROM TekstDok WHERE DokId={DokId} AND Znaczenie = 166) AS PlatNrLokalu,
                 (SELECT Tekst FROM TekstDok WHERE DokId={DokId} AND Znaczenie = 159) AS PlatKod,
                 (SELECT Tekst FROM TekstDok WHERE DokId={DokId} AND Znaczenie = 160) AS PlatMiejscowosc,
                 (SELECT Tekst FROM TekstDok WHERE DokId={DokId} AND Znaczenie = 162) AS PlatNIP;
              """.format(**locals())
        # FunkcjaZapisuDoLogow(SEL)
        stat, PCMKontrahent, dump = self.RSPCMarketSQL.Execute(SEL)
        if stat != 'OK':
            stat = 'bd pobrania danych kontrahentw faktur z PC Market   \r\n' + stat
            return stat, [], [], False

        # dane kontrahenta
        Nazwa       = PCMKontrahent[0][0]
        Ulica       = PCMKontrahent[0][1] if PCMKontrahent[0][1] is not None else ''
        NrUlicy     = PCMKontrahent[0][2] if PCMKontrahent[0][2] is not None else ''
        NrLokalu    = PCMKontrahent[0][3] if PCMKontrahent[0][3] is not None else ''
        Kod         = PCMKontrahent[0][4] if PCMKontrahent[0][4] is not None else ''
        Miejscowosc = PCMKontrahent[0][5] if PCMKontrahent[0][5] is not None else ''
        NIP         = PCMKontrahent[0][6]

        # dane patnika
        PlatNazwa       = PCMKontrahent[0][7]  if PCMKontrahent[0][7]  is not None else ''
        PlatUlica       = PCMKontrahent[0][8]  if PCMKontrahent[0][8]  is not None else ''
        PlatNrUlicy     = PCMKontrahent[0][9]  if PCMKontrahent[0][9]  is not None else ''
        PlatNrLokalu    = PCMKontrahent[0][10] if PCMKontrahent[0][10] is not None else ''
        PlatKod         = PCMKontrahent[0][11] if PCMKontrahent[0][11] is not None else ''
        PlatMiejscowosc = PCMKontrahent[0][12] if PCMKontrahent[0][12] is not None else ''
        PlatNIP         = PCMKontrahent[0][13]

        if NIP is None and PlatNIP is None:  # pominicie faktur, na ktrych kontrahent i patnik nie ma NIP
            continue
        elif NIP is None:
            NIP = ''

        if NrUlicy != '':
            UlicaNry = NrUlicy + (('/' + NrLokalu) if NrLokalu not in ('', 'None') else '')
        else:
            UlicaNry = ''

        if PlatNrUlicy != '':
            PlatUlicaNry = PlatNrUlicy + (('/' + PlatNrLokalu) if PlatNrLokalu not in ('', 'None') else '')
        else:
            PlatUlicaNry = ''

        Nazwa = Nazwa.replace("'", "`")
        Ulica = Ulica.replace("'", "`")
        UlicaNry = UlicaNry.replace("'", "`")
        Miejscowosc = Miejscowosc.replace("'", "`")
        PlatNazwa = PlatNazwa.replace("'", "`")
        PlatUlica = PlatUlica.replace("'", "`")
        PlatUlicaNry = PlatUlicaNry.replace("'", "`")
        PlatMiejscowosc = PlatMiejscowosc.replace("'", "`")

        # ------------------------------------------------------------------------------------------------------
        # Znalezienie kontrahenta w rs_kontrahenci pasujcego do danych z faktury
        # ------------------------------------------------------------------------------------------------------

        firma_id = self.firma_id

        #WS 2024-02-27 - usunite AND aktywny=TRUE
        #WS 2024-02-13
        # Wyszukanie porwnujc tylko NIP
        SEL = """SELECT kontrahent_id
                 FROM rs_kontrahenci
                 WHERE firma_id='{firma_id}'
                 AND rsf_zamien_polskie_znaki(regexp_replace(COALESCE(nip,''), '[^0-9]', '', 'g'))=rsf_zamien_polskie_znaki(regexp_replace('{NIP}', '[^0-9]', '', 'g'));"""
        # #WS 2023-06-05 - dodany rsf_zamien_polskie_znaki
        # # Wyszukanie porwnujc:
        # # NIP - wycznie cyfry
        # # nazw - pomijajc biae znaki, wielko liter oraz .,:;"'`/-
        # # kod pocztowy, miejscowo - j.w.
        # # nr ulicy / lokalu - j.w. (jeli numery puste, to bierzemy je z koca ulicy)
        # # ulic - j.w., bez prefiksu 'UL. ' lub 'UL ' i z wyciciem nr ulicy/lokalu na kocu
        # SEL = """SELECT kontrahent_id
        #          FROM rs_kontrahenci
        #          WHERE firma_id='{firma_id}' AND aktywny=TRUE
        #          AND rsf_zamien_polskie_znaki(regexp_replace(COALESCE(nip,''), '[^0-9]', '', 'g'))=rsf_zamien_polskie_znaki(regexp_replace('{NIP}', '[^0-9]', '', 'g'))
        #          AND (rsf_zamien_polskie_znaki(UPPER(regexp_replace(COALESCE(nazwa_pelna,''), '[\\\\s.,:;"''`/-]', '', 'g')))=rsf_zamien_polskie_znaki(UPPER(regexp_replace('{Nazwa}', '[\\\\s.,:;"''`/-]', '', 'g'))) OR
        #               rsf_zamien_polskie_znaki(UPPER(regexp_replace(COALESCE(CONCAT(nazwa_f1, nazwa_f2),''), '[\\\\s.,:;"''`/-]', '', 'g')))=rsf_zamien_polskie_znaki(UPPER(regexp_replace('{Nazwa}', '[\\\\s.,:;"''`/-]', '', 'g'))) OR
        #               rsf_zamien_polskie_znaki(UPPER(regexp_replace(COALESCE(nazwa,''), '[\\\\s.,:;"''`/-]', '', 'g')))=rsf_zamien_polskie_znaki(UPPER(regexp_replace('{Nazwa}', '[\\\\s.,:;"''`/-]', '', 'g'))))
        #          AND rsf_zamien_polskie_znaki(UPPER(regexp_replace(COALESCE(adr_nry,''), '[\\\\s.,:;"''`/-]', '', 'g')))=
        #              rsf_zamien_polskie_znaki((CASE WHEN '{UlicaNry}' != ''
        #               THEN UPPER(regexp_replace('{UlicaNry}', '[\\\\s.,:;"''`/-]', '', 'g'))
        #               ELSE COALESCE(substring(UPPER(regexp_replace('{Ulica}', '[\\\\s.,:;"''`/-]', '', 'g')), '([0-9].*)$'), '') END))
        #          AND rsf_zamien_polskie_znaki(regexp_replace(regexp_replace(regexp_replace(COALESCE(UPPER(adr_ulica),''), '^(UL.)', ''), '^(UL )', ''), '[\\\\s.,:;"''`/-]', '', 'g'))=
        #              rsf_zamien_polskie_znaki((CASE WHEN '{UlicaNry}' != ''
        #               THEN regexp_replace(regexp_replace(regexp_replace(regexp_replace(UPPER('{Ulica}'), '^(UL.)', ''), '^(UL )', ''), '[\\\\s.,:;"''`/-]', '', 'g'),UPPER(regexp_replace('{UlicaNry}', '[\\\\s.,:;"''`/-]', '', 'g')), '')
        #               ELSE regexp_replace(regexp_replace(regexp_replace(regexp_replace(UPPER('{Ulica}'), '^(UL.)', ''), '^(UL )', ''), '[\\\\s.,:;"''`/-]', '', 'g'),COALESCE(substring(UPPER(regexp_replace('{Ulica}', '[\\\\s.,:;"''`/-]', '', 'g')), '([0-9].*)$'), ''), '') END))
        #          AND regexp_replace(COALESCE(adr_kod,''), '[\\\\s.,:;"''`/-]', '', 'g')=regexp_replace('{Kod}', '[\\\\s.,:;"''`/-]', '', 'g')
        #          AND rsf_zamien_polskie_znaki(UPPER(regexp_replace(COALESCE(adr_miej,''), '[\\\\s.,:;"''`/-]', '', 'g')))=rsf_zamien_polskie_znaki(UPPER(regexp_replace('{Miejscowosc}', '[\\\\s.,:;"''`/-]', '', 'g')))
        #          ;"""

        SEL_KONTR = SEL.format(**locals())
        # FunkcjaZapisuDoLogow(SEL)
        stat, RSKontrahent, dump, dump = self.PolSQL.Execute(SEL_KONTR, 3, CzyDodawacBlad=True)
        if stat != 'OK':
            stat = 'bd pobrania danych z rs_kontrahenci   \r\n' + stat
            return stat, [], [], False

        # ------------------------------------------------------------------------------------------------------
        # Znalezienie patnika w rs_kontrahenci pasujcego do danych z faktury
        # ------------------------------------------------------------------------------------------------------

        SEL_PLAT = SEL.format(firma_id=firma_id, Nazwa=PlatNazwa, Ulica=PlatUlica, UlicaNry=PlatUlicaNry,
                              Kod=PlatKod, Miejscowosc=PlatMiejscowosc, NIP=PlatNIP)
        # FunkcjaZapisuDoLogow(SEL)
        stat, RSPlatnik, dump, dump = self.PolSQL.Execute(SEL_PLAT, 3, CzyDodawacBlad=True)
        if stat != 'OK':
            stat = 'bd pobrania danych z rs_kontrahenci   \r\n' + stat
            return stat, [], [], False

        # ------------------------------------------------------------------------------------------------------

        DaneKontrahenta = (Nazwa, Ulica, UlicaNry, Kod, Miejscowosc, NIP)
        DanePlatnika = (PlatNazwa, PlatUlica, PlatUlicaNry, PlatKod, PlatMiejscowosc, PlatNIP)

        if (SklepRS, DaneKontrahenta, DanePlatnika) not in Kontrahenci:
            Kontrahenci[(SklepRS, DaneKontrahenta, DanePlatnika)] = []
        Kontrahenci[(SklepRS, DaneKontrahenta, DanePlatnika)].append(NrDok)

        if len(RSPlatnik) > 0 and PlatNIP:
            DokKontr[NrDok] = RSPlatnik[0][0]
        elif len(RSKontrahent) > 0 and NIP:
            DokKontr[NrDok] = RSKontrahent[0][0]
        else:  # nie udao si dopasowa ani kontrahenta, ani patnika
            importowac = False

    return 'OK', DokKontr, Kontrahenci, importowac


def WyslijEmailZListaBrakujacychKontrahentow(self, DokKontr, Kontrahenci, FunkcjaZapisuDoLogow=rsglobal.ZapiszDoLogow):
    """Funkcja wysyajca e-mail z list kontrahentw,
       ktrych znaleziono na fakturach PC Marketu, a nie znaleziono w rs_kontrahenci

       Parameters:
           DokKontr (dict):           opis w funkcji ZnajdzKontrahentow
           Kontrahenci (dict):        opis w funkcji ZnajdzKontrahentow"""

    from datetime import datetime
    import rsglobal

    stat = 'OK'

    temat = 'Import faktur do paragonw fiskalnych z PC Market - lista brakujcych kontrahentw'
    tresc = ''

    self.WynikEmail.append('Lista brakujcych kontrahentw:   \r\n\r\n')

    for (sklep, kontrahent, platnik), NumeryDok in Kontrahenci.items():
        if NumeryDok[0] not in DokKontr:
            SklepRS     = sklep
            Nazwa       = kontrahent[0]
            Ulica       = kontrahent[1]
            UlicaNry    = kontrahent[2]
            Kod         = kontrahent[3]
            Miejscowosc = kontrahent[4]
            NIP         = kontrahent[5] if kontrahent[5] != '' else 'BRAK'
            PlatNazwa       = platnik[0]
            PlatUlica       = platnik[1]
            PlatUlicaNry    = platnik[2]
            PlatKod         = platnik[3]
            PlatMiejscowosc = platnik[4]
            PlatNIP         = platnik[5]
            ZbiorNryDok = ', '.join(NumeryDok)

            DaneKontrahenta = (Nazwa, Ulica, UlicaNry, Kod, Miejscowosc, NIP)
            DanePlatnika = (PlatNazwa, PlatUlica, PlatUlicaNry, PlatKod, PlatMiejscowosc, PlatNIP)

            if DanePlatnika == ('', '', '', '', '', None) or DaneKontrahenta == DanePlatnika:
                # jeeli danych patnika brak lub patnik jest tosamy z kontrahentem
                dane_tekst = """Placwka: {SklepRS}   \r\nNr faktury: {ZbiorNryDok}   \r\nNazwa: {Nazwa}   \r\nAdres: {Ulica} {UlicaNry}, {Kod} {Miejscowosc}   \r\nNIP: {NIP}   \r\n--------------------------------------------------   \r\n""".format(**locals())
                self.WynikEmail.append(dane_tekst)
            else:
                dane_tekst  = """Placwka: {SklepRS}   \r\nNr faktury: {ZbiorNryDok}   \r\nNazwa kontrahenta: {Nazwa}   \r\nAdres kontrahenta: {Ulica} {UlicaNry}, {Kod} {Miejscowosc}   \r\nNIP kontrahenta: {NIP}   \r\n""".format(**locals())
                dane_tekst += """Nazwa patnika: {PlatNazwa}   \r\nAdres patnika: {PlatUlica} {PlatUlicaNry}, {PlatKod} {PlatMiejscowosc}   \r\nNIP patnika: {PlatNIP}   \r\n--------------------------------------------------   \r\n""".format(**locals())
                self.WynikEmail.append(dane_tekst)

    for wiersz in self.WynikEmail:
        tresc += wiersz

    stat = rsglobal.RSWyslijEmail(self.AdresatEmail, temat, tresc, [], 0, polsql=self.PolSQL)
    if stat != 'OK':
        FunkcjaZapisuDoLogow('Bd podczas wysyania wiadomoci e-mail   \r\n' + stat + '   \r\n')
    else:
        FunkcjaZapisuDoLogow('Wysano e-mail na adres {}   \r\n'.format(self.AdresatEmail))

    for wiersz in self.WynikEmail[2:]:
        FunkcjaZapisuDoLogow(wiersz)
    
    return stat


def WyslijEmailZWynikiemImportu(self, Powodzenie, FunkcjaZapisuDoLogow=rsglobal.ZapiszDoLogow):
    import rsglobal

    if Powodzenie:
        temat = 'Import faktur do paragonw fiskalnych z PC Market zakoczony powodzeniem'
    else:
        temat = 'Import faktur do paragonw fiskalnych z PC Market przerwany bdem'
    tresc = ''
    for wiersz in self.WynikEmail[:2]:
        tresc += wiersz

    if len(self.WynikEmail) == 2:
        tresc += 'Brak nowych faktur.   \r\n\r\n'
        FunkcjaZapisuDoLogow('Brak nowych faktur.   \r\n\r\n')
    else:
        for wiersz in self.WynikEmail[2:]:
            tresc += wiersz

    stat = rsglobal.RSWyslijEmail(self.AdresatEmail, temat, tresc, [], 0, polsql=self.PolSQL)
    if stat != 'OK':
        FunkcjaZapisuDoLogow('   \r\nBd podczas wysyania wiadomoci e-mail z wynikami importu   \r\n' + stat)
    else:
        FunkcjaZapisuDoLogow('   \r\nWysano e-mail z wynikami na adres {}'.format(self.AdresatEmail))

    return stat


def WyslijEmailZBledamiPrzedImportem(self, blad, FunkcjaZapisuDoLogow=rsglobal.ZapiszDoLogow):
    import rsglobal

    temat = 'Niepowodzenie importu faktur do paragonw fiskalnych z PC Market'
    tresc = ''
    for wiersz in self.WynikEmail:
        tresc += wiersz
    tresc += 'Import nie rozpocz si, wystpiy problemy:   \r\n'+blad
    FunkcjaZapisuDoLogow('Import nie rozpocz si, wystpiy problemy:   \r\n'+blad)

    stat = rsglobal.RSWyslijEmail(self.AdresatEmail, temat, tresc, [], 0, polsql=self.PolSQL)
    if stat != 'OK':
        FunkcjaZapisuDoLogow('   \r\nBd podczas wysyania wiadomoci e-mail z wynikami importu   \r\n' + stat)
    else:
        FunkcjaZapisuDoLogow('   \r\nWysano e-mail z wynikami na adres {}'.format(self.AdresatEmail))

    return stat
