# -*- coding: iso8859_2 -*-
#-------------------------------------------------------------------------------
# Name:        fs_hd_paragony_import_pom.py
# Purpose:     Import paragonw fiskalnych z PC Market - funkcje pomocnicze
# Author:      Wojciech Skowron
# Created:     2020/02/13
# RCS-ID:      $Id: fs_hd_paragony_import_pom.py $
# Copyright:   (c) 2020 Record System
# Licence:     GNU GPL
#-------------------------------------------------------------------------------

# 2020-05-12 Wojciech Skowron - (11973) Poprawki
# 2023-04-07 Wojciech Skowron - (12694) Import zwrotw

import rsglobal

stat = "OK"


def ImportujParagony(self, DataImport, Sklep, FunkcjaZapisuDoLogow=rsglobal.ZapiszDoLogow):
    """Funkcja importujca paragony z PC Market z dnia DataImport dla sklepu (placwki) Sklep

       Parameters:
           DataImport (str):                data 'RRRR-MM-DD'
           Sklep (tuple):                   tupla (sklep_id, pcmarket), czyli pola z rs_hd_sklepy
           FunkcjaZapisuDoLogow (function): funkcja wykorzystywana do zapisu do logw, w przypadku harmonogramu trzeba
           przekazaZapiszLog"""

    info = 'Placwka {} z dnia {}: '.format(Sklep[0].strip(), DataImport)

    #WS 2020-05-12
    # ----------------------------------------------------------------------------------------------------------
    # Sprawdzenie, czy istnieje zdefiniowane stoisko/kasa dla sklepu
    # ----------------------------------------------------------------------------------------------------------

    RSTypStoiska = None
    RSDataOdStoiska = None

    SEL = """SELECT typ_stoiska_id, data_od FROM rs_hd_sklepy_stoiska
             WHERE firma_id='{firma_id}' AND sklep_id='{sklep_id}';"""
    SEL = SEL.format(firma_id  = self.firma_id,
                     sklep_id  = Sklep[0])
    stat, Stoiska, dump, dump = self.PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
    if stat != 'OK':
        FunkcjaZapisuDoLogow(info + DekodujBlad(stat) + stat)
        return 'Bd pobrania stoisk dla placwki:\n ' + stat
    elif len(Stoiska) == 0:
        stat = 'BL_PRZERW_BR_STOISKA${}'.format(Sklep[0].strip())
        FunkcjaZapisuDoLogow("Pominito placwk {}: brak zdefiniowanego stoiska".format(Sklep[0].strip()))
        return stat
    # istnieje jedno stoisko dla placwki - zostanie wpisane na paragony
    elif len(Stoiska) == 1:
        RSTypStoiska = Stoiska[0][0]
        RSDataOdStoiska = Stoiska[0][1]

    RSListaKas = []  # bdzie zawiera wszystkie kasy, ktre mona wpisa na paragon dla znalezionego wyej stoiska

    SEL = """SELECT DISTINCT kasa_id FROM rs_hd_kasy_stoiska
             WHERE firma_id='{firma_id}' AND sklep_id='{sklep_id}' 
             AND ({typ_stoiska_id} IS NULL OR typ_stoiska_id={typ_stoiska_id}) 
             AND ({data_od_stoiska} IS NULL OR data_od_stoiska={data_od_stoiska});"""
    SEL = SEL.format(firma_id        = self.firma_id,
                     sklep_id        = Sklep[0],
                     typ_stoiska_id  = rsglobal.ZmienNaNULL(RSTypStoiska),
                     data_od_stoiska = rsglobal.ZmienNaNULL(RSDataOdStoiska))
    stat, Kasy, dump, dump = self.PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
    if stat != 'OK':
        FunkcjaZapisuDoLogow(info + DekodujBlad(stat) + stat)
        return 'Bd pobrania kas powizanych ze stoiskiem placwki:\n ' + stat
    elif len(Kasy) == 0:
        stat = "BL_PRZERW_BR_KASY${}".format(Sklep[0].strip())
        FunkcjaZapisuDoLogow("Pominito placwk {}: brak kas powizanych ze stoiskami placwki".format(Sklep[0].strip()))
        return stat
    elif RSTypStoiska is not None and RSDataOdStoiska is not None:  # istnieje jedno stoisko
        RSListaKas = [k[0] for k in Kasy]

    # ----------------------------------------------------------------------------------------------------------
    # Pobranie id paragonw ju zaimportowanych
    # ----------------------------------------------------------------------------------------------------------

    # import_id w rs_hd_paragony odpowiada str(DokId) z tabeli Dok PC Marketu

    SEL = """SELECT import_id FROM rs_hd_paragony
             WHERE firma_id='{firma_id}'
             AND czas >= '2020-01-01 00:00:00'
             AND czas >= '{czas} 00:00:00' AND czas < '{czas}'::DATE + interval '1 day'
             AND sklep_id='{sklep_id}';"""
    SEL = SEL.format(firma_id  = self.firma_id,
                     czas      = DataImport,
                     sklep_id  = Sklep[0])
    stat, RSParagony, dump, dump = self.PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
    if stat != 'OK':
        FunkcjaZapisuDoLogow(info + DekodujBlad(stat) + stat)
        return 'Bd pobrania paragonw z rs_hd_paragony:\n ' + stat
    else:
        ParagonyWczesniejZaimportowane = [paragon[0] for paragon in RSParagony]

    # ----------------------------------------------------------------------------------------------------------
    # Pobranie danych nagwkw i patnoci z PC Market
    # ----------------------------------------------------------------------------------------------------------

    # Pobranie danych potrzebnych do wypenienia rs_hd_paragony oraz rs_hd_paragony_platnosci

    SEL = """SELECT D.DokId, D.Data, D.Kwota3, D.Param1, TP.Tekst, TC.Tekst,
                    DK.KasaId, U.NrKasjera, D.Zaplacono, D.Kwota5
             FROM Dok D
             JOIN DokKasa DK ON (D.DokId = DK.DokId)
             JOIN Uzytkownik U ON (D.UzId = U.UzId)
             LEFT OUTER JOIN TekstDok TP ON (D.DokId = TP.DokId AND TP.Znaczenie = 16)
             LEFT OUTER JOIN TekstDok TC ON (D.DokId = TC.DokId AND TC.Znaczenie = 186)
             WHERE D.Data >= '2020-01-01 00:00:00' 
             AND D.Data >= '{Data} 00:00:00' AND D.Data < DATEADD(day, 1, '{Data}')
             AND D.MagId = {MagId} AND D.TypDok = 21
             {DokId};"""
    SEL = SEL.format(Data  = DataImport,
                     MagId = int(Sklep[1]),
                     DokId = 'AND D.DokId not in ('+",".join(ParagonyWczesniejZaimportowane)+')' if len(ParagonyWczesniejZaimportowane) > 0 else '')
    stat, PCMParagonyNagl, dump = self.RSPCMarketSQL.Execute(SEL)
    if stat != 'OK':
        FunkcjaZapisuDoLogow(info + DekodujBlad(stat) + stat)
        return 'Bd pobrania nagwkw paragonw z PC Market:\n ' + stat
    # elif len(PCMParagonyNagl) == 0:
    #     FunkcjaZapisuDoLogow(info + 'brak nowych paragonw')
    #     return 'OK'

    LNowychParagonow = 0

    # ----------------------------------------------------------------------------------------------------------

    for paragon in PCMParagonyNagl:
        LNowychParagonow += 1

        # Nagwek - dane do wstawienia w rs_hd_paragony
        DokId     = int(paragon[0])
        Data      = paragon[1]
        Kwota3    = paragon[2]       # Rabat
        Param1    = int(paragon[3])  # Nr paragonu
        TPTekst   = paragon[4]       # Nr transakcji (po 2 redniku)
        TCTekst   = paragon[5]       # Data i dokadny czas w formacie RRRRMMDDggmmss (niczym nie przedzielone)
        KasaId    = int(paragon[6])
        NrKasjera = int(paragon[7])

        # Patnoci - dane do wstawienia w rs_hd_platnosci
        Zaplacono = paragon[8]       # Zapacona kwota brutto
        Kwota5    = paragon[9]       # Kwota zapacona kart (Zaplacono-Kwota5 - kwota zapacona gotwk)

        #WS 2020-05-12
        RSKasa = None  # id kasy, ktr ostatecznie wpiszemy na paragon
        if len(RSListaKas) == 1:  # ze stoiskiem powizana jest tylko 1 kasa - wpiszemy j bez wzgldu na id z pcmarketu
            RSKasa = RSListaKas[0]
        elif KasaId in RSListaKas:  # jest powizane kilka kas - sprawdzamy czy ktra ma id zgodne z id w pcmarkecie
            RSKasa = KasaId

        # ----------------------------------------------------------------------------------------------------------
        # Pobranie kolejnego numeru z sekwencji
        # ----------------------------------------------------------------------------------------------------------

        SEL = """SELECT nextval('rs_hd_paragony_paragon_id_seq');"""
        stat, RSParagonySekw, dump, dump = self.PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
        if stat != 'OK':
            FunkcjaZapisuDoLogow(info + DekodujBlad(stat) + stat)
            return 'Bd pobrania danych paragonw z rs_hd_paragony:\n ' + stat

        paragon_id = RSParagonySekw[0][0]

        # ----------------------------------------------------------------------------------------------------------
        # Zapis nagwka do rs_hd_paragony
        # ----------------------------------------------------------------------------------------------------------

        INS = """INSERT INTO rs_hd_paragony
                 (firma_id, paragon_id, nr_transakcji, nr_paragonu, czas, 
                 sklep_id, typ_stoiska_id, data_od_stoiska, kasa_id, kasjer_id, 
                 wartosc, rabat, import_id)
                 VALUES 
                 ('{firma_id}',{paragon_id},'{nr_transakcji}','{nr_paragonu}','{czas}',
                 '{sklep_id}',{typ_stoiska_id},{data_od_stoiska},{kasa_id},{kasjer_id},
                 {wartosc},{rabat},'{import_id}');
              """.format(firma_id        = self.firma_id,
                         paragon_id      = paragon_id,
                         nr_transakcji   = TPTekst.split(";")[2] if len(TPTekst.split(";")) >= 3 else '',
                         nr_paragonu     = Param1,
                         czas            = Data[:10]+' '+TCTekst[8:10]+':'+TCTekst[10:12]+':'+TCTekst[12:14],
                         sklep_id        = Sklep[0],
                         typ_stoiska_id  = rsglobal.ZmienNaNULL(RSTypStoiska),  #WS 2020-05-12
                         data_od_stoiska = rsglobal.ZmienNaNULL(RSDataOdStoiska),  #WS 2020-05-12
                         kasa_id         = rsglobal.ZmienNaNULL(RSKasa),  #WS 2020-05-12
                         kasjer_id       = NrKasjera,
                         wartosc         = Zaplacono,
                         rabat           = Kwota3,
                         import_id       = DokId)

        # FunkcjaZapisuDoLogow(INS)
        stat, dump, dump, dump = self.PolSQL.Execute(INS, 1, CzyDodawacBlad=True)
        if stat != 'OK':
            FunkcjaZapisuDoLogow(info + DekodujBlad(stat) + stat)
            return 'Bd zapisu nagwka paragonu do rs_hd_paragony:\n ' + stat

        # ----------------------------------------------------------------------------------------------------------
        # Zapis danych o patnoci do rs_hd_paragony_platnosci
        # ----------------------------------------------------------------------------------------------------------

        INS_PLAT = """INSERT INTO rs_hd_paragony_platnosci
                      (firma_id, paragon_id, platnosc, wartosc)
                      VALUES """

        if Zaplacono - Kwota5 > 0:  # zapacono gotwk
            INS_PLAT += """('{firma_id}',{paragon_id},'{platnosc}',{wartosc}),
                        """.format(firma_id   = self.firma_id,
                                   paragon_id = paragon_id,
                                   platnosc   = 'G',
                                   wartosc    = Zaplacono - Kwota5)

        if Kwota5 > 0:  # zapacono kart
            INS_PLAT += """('{firma_id}',{paragon_id},'{platnosc}',{wartosc}),
                         """.format(firma_id   = self.firma_id,
                                    paragon_id = paragon_id,
                                    platnosc   = 'KAR',
                                    wartosc    = Kwota5)

        INS_PLAT = INS_PLAT.strip()[:-1] + ";"
        stat, dump, dump, dump = self.PolSQL.Execute(INS_PLAT, 1, CzyDodawacBlad=True)
        if stat != 'OK':
            FunkcjaZapisuDoLogow(info + DekodujBlad(stat) + stat)
            return 'Bd zapisu danych o patnoci do rs_hd_paragony_platnosci:\n ' + stat

        # ------------------------------------------------------------------------------------------------------
        # Pobranie danych pozycji paragonu z PC Market
        # ------------------------------------------------------------------------------------------------------

        # Pobranie danych potrzebnych do wypenienia rs_hd_paragony_pozycje

        SEL = """SELECT PD.DokId, PD.NrPozycji, T.Kod, PD.IloscPlus, PD.Stawka,
                        PD.CenaPoRab+(CAST(PD.Stawka as decimal)/CAST(10000 as decimal))*PD.CenaPoRab,
                        T.CenaEw,
                        PD.Wartosc+(CAST(PD.Stawka as decimal)/CAST(10000 as decimal))*PD.Wartosc
                 FROM Dok D
                 JOIN PozDok PD ON (D.DokId = PD.DokId)
                 JOIN Towar T ON (PD.TowId = T.TowId)
                 WHERE D.DokId={DokId};"""
        SEL = SEL.format(Data  = DataImport,
                         MagId = int(Sklep[1]),
                         DokId = DokId)
        stat, PCMParagonyPoz, dump = self.RSPCMarketSQL.Execute(SEL)
        if stat != 'OK':
            FunkcjaZapisuDoLogow(info + DekodujBlad(stat) + stat)
            return 'Bd pobrania pozycji paragonu z PC Market:\n ' + stat

        # ------------------------------------------------------------------------------------------------------
        # Zapis pozycji do rs_hd_paragony_pozycje
        # ------------------------------------------------------------------------------------------------------

        INS_POZ = """INSERT INTO rs_hd_paragony_pozycje
                     (firma_id, paragon_id, lp, typ, artykul_id,
                     proc_vat, ilosc, cena_spr, cena_ew, wartosc)
                     VALUES """

        for pozycja in PCMParagonyPoz:
            DokId     = int(pozycja[0])
            NrPozycji = int(pozycja[1])
            Kod       = pozycja[2].strip()  # artykul_id z rs_artykuly lub kod EAN z rs_artykuly_ean
            IloscPlus = pozycja[3]
            Stawka    = int(pozycja[4])     # 0, 500, 800, 2300
            Cena      = pozycja[5]          # CenaPoRab (jednostkowa netto) + podatek
            CenaEw    = pozycja[6]          # CenaEw z tabeli Towar
            Brutto    = pozycja[7]          # Warto cakowita netto dla pozycji + podatek

            # ------------------------------------------------------------------------------------------------------
            # Kod z tabeli Towar jest z reguy taki sam jak artykul_id z rs_artykuly,
            # ale jeli jest EANem, wtedy moe by powizany z innym artykul_id ktry trzeba znale

            # Kod jest EANem
            if Kod.isdigit() and len(Kod) in (7, 8, 12, 13):
                SEL = """SELECT 
                         COALESCE((SELECT artykul_id FROM rs_artykuly WHERE firma_id = '{firma_id}' AND artykul_id='{Kod}'), 
                                  (SELECT artykul_id FROM rs_artykuly_ean WHERE firma_id = '{firma_id}' AND ean='{Kod}'));"""
                SEL = SEL.format(firma_id=self.firma_id, Kod=Kod)
                stat, RSArtykuly, dump, dump = self.PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
                if stat != 'OK':
                    FunkcjaZapisuDoLogow(info + DekodujBlad(stat) + stat)
                    return 'Bd pobrania id artykuu:\n ' + stat
                elif len(RSArtykuly) == 0:
                    FunkcjaZapisuDoLogow(info + 'artyku wystpujcy na paragonie nie istnieje w kartotece artykuw')
                    return 'artyku wystpujcy na paragonie nie istnieje w kartotece artykuw'
                else:
                    artykul_id = RSArtykuly[0][0]
            # Kod nie jest EANem
            else:
                artykul_id = Kod

            # ------------------------------------------------------------------------------------------------------

            INS_POZ += """('{firma_id}',{paragon_id},{lp},{typ},'{artykul_id}',
                           '{proc_vat}',{ilosc},{cena_spr},{cena_ew},{wartosc}),
                       """.format(firma_id   = self.firma_id,
                                  paragon_id = paragon_id,
                                  lp         = NrPozycji,
                                  typ        = '1', # na razie zawsze 1 - jeli si zmieni, uwzgldni w DekodujBlad()
                                  artykul_id = artykul_id,
                                  proc_vat   = str(Stawka / 100).rjust(2, '0')[:2] if Stawka >= 0 else 'zw',
                                  ilosc      = IloscPlus,
                                  cena_spr   = Cena,
                                  cena_ew    = CenaEw,  #WS 2020-05-12
                                  wartosc    = Brutto)

        INS_POZ = INS_POZ.strip()[:-1] + ";"
        # FunkcjaZapisuDoLogow(INS)
        stat, dump, dump, dump = self.PolSQL.Execute(INS_POZ, 1, CzyDodawacBlad=True)
        if stat != 'OK':
            FunkcjaZapisuDoLogow(info + DekodujBlad(stat) + stat)
            return 'Bd zapisu pozycji paragonw do rs_hd_paragony_pozycje:\n ' + stat

    # ----------------------------------------------------------------------------------------------------------
    # Pobranie id zwrotw ju zaimportowanych
    # ----------------------------------------------------------------------------------------------------------

    # import_id w rs_hd_paragony odpowiada str(DokId) z tabeli Dok PC Marketu

    SEL = """SELECT import_id FROM rs_hd_paragony
             WHERE firma_id='{firma_id}'
             AND czas >= '2020-01-01 00:00:00'
             AND czas >= '{czas} 00:00:00' AND czas < '{czas}'::DATE + interval '1 day'
             AND sklep_id='{sklep_id}';"""
    SEL = SEL.format(firma_id  = self.firma_id,
                     czas      = DataImport,
                     sklep_id  = Sklep[0])
    stat, RSParagonyZwroty, dump, dump = self.PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
    if stat != 'OK':
        FunkcjaZapisuDoLogow(info + DekodujBlad(stat) + stat)
        return 'Bd pobrania zwrotw z rs_hd_paragony:\n ' + stat
    else:
        ZwrotyWczesniejZaimportowane = [zwrot[0] for zwrot in RSParagonyZwroty]

    # ----------------------------------------------------------------------------------------------------------
    # Pobranie danych nagwkw i patnoci z PC Market
    # ----------------------------------------------------------------------------------------------------------

    # Pobranie danych potrzebnych do wypenienia rs_hd_paragony oraz rs_hd_paragony_platnosci

    SEL = """SELECT D.DokId, D.Data, T1.Tekst, T2.Tekst, T3.Tekst, DP.Param1,
                    DK.KasaId, U.NrKasjera, D.Zaplacono, 
                    DP.Zaplacono, DP.Kwota5
             FROM Dok D
             JOIN DokKasa DK ON (D.DokId = DK.DokId)
             JOIN Uzytkownik U ON (D.UzId = U.UzId)
             LEFT OUTER JOIN TekstDok T1 ON (D.DokId = T1.DokId AND T1.Znaczenie = 16)
             LEFT OUTER JOIN TekstDok T2 ON (D.DokId = T2.DokId AND T2.Znaczenie = 27)
             LEFT OUTER JOIN TekstDok T3 ON (D.DokId = T3.DokId AND T3.Znaczenie = 28)
             LEFT OUTER JOIN Dok DP ON (DP.NrDok = T3.Tekst)
             WHERE D.Data >= '2020-01-01 00:00:00' 
             AND D.Data >= '{Data} 00:00:00' AND D.Data < DATEADD(day, 1, '{Data}')
             AND D.MagId = {MagId} AND D.TypDok = 8 AND D.NrDok LIKE 'ZWPAR/%/%/%'
             {DokId};"""
    SEL = SEL.format(Data  = DataImport,
                     MagId = int(Sklep[1]),
                     DokId = 'AND D.DokId not in ('+",".join(ZwrotyWczesniejZaimportowane)+')' if len(ZwrotyWczesniejZaimportowane) > 0 else '')
    stat, PCMZwrotyNagl, dump = self.RSPCMarketSQL.Execute(SEL)
    if stat != 'OK':
        FunkcjaZapisuDoLogow(info + DekodujBlad(stat) + stat)
        return 'Bd pobrania nagwkw zwrotw z PC Market:\n ' + stat
    elif len(PCMZwrotyNagl) == 0 and len(PCMParagonyNagl) == 0:
        FunkcjaZapisuDoLogow(info + 'brak nowych paragonw i zwrotw')
        return 'OK'

    LNowychZwrotow = 0

    # ----------------------------------------------------------------------------------------------------------

    for zwrot in PCMZwrotyNagl:
        LNowychZwrotow += 1

        # Nagwek - dane do wstawienia w rs_hd_paragony
        DokId     = int(zwrot[0])
        Data      = zwrot[1]
        T1Tekst   = zwrot[2]  # Nr transakcji (po 2 redniku)
        T2Tekst   = zwrot[3]  # Kwota zwrotu (pierwsza kart, po 2 redniku gotwk)
        T3Tekst   = zwrot[4]  # NrDok paragonu
        Param1    = int(zwrot[5])  # Nr paragonu
        KasaId    = int(zwrot[6])
        NrKasjera = int(zwrot[7])

        # Patnoci - dane do wstawienia w rs_hd_platnosci
        Zwrocono  = zwrot[8]       # Zwrocona kwota brutto
        Zaplacono = zwrot[9]       # Zapacona kwota brutto na paragonie
        Kwota5    = zwrot[10]      # Kwota zapacona kart na paragonie (Zaplacono-Kwota5 - kwota zapacona gotwk)

        RSKasa = None  # id kasy, ktr ostatecznie wpiszemy na paragon
        if len(RSListaKas) == 1:  # ze stoiskiem powizana jest tylko 1 kasa - wpiszemy j bez wzgldu na id z pcmarketu
            RSKasa = RSListaKas[0]
        elif KasaId in RSListaKas:  # jest powizane kilka kas - sprawdzamy czy ktra ma id zgodne z id w pcmarkecie
            RSKasa = KasaId

        # ----------------------------------------------------------------------------------------------------------
        # Pobranie kolejnego numeru z sekwencji
        # ----------------------------------------------------------------------------------------------------------

        SEL = """SELECT nextval('rs_hd_paragony_paragon_id_seq');"""
        stat, RSParagonySekw, dump, dump = self.PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
        if stat != 'OK':
            FunkcjaZapisuDoLogow(info + DekodujBlad(stat) + stat)
            return 'Bd pobrania danych paragonw z rs_hd_paragony:\n ' + stat

        zwrot_id = RSParagonySekw[0][0]

        # ----------------------------------------------------------------------------------------------------------
        # Zapis nagwka do rs_hd_paragony
        # ----------------------------------------------------------------------------------------------------------

        INS = """INSERT INTO rs_hd_paragony
                 (firma_id, paragon_id, nr_transakcji, nr_paragonu, czas, 
                 sklep_id, typ_stoiska_id, data_od_stoiska, kasa_id, kasjer_id, 
                 wartosc, rabat, import_id)
                 VALUES 
                 ('{firma_id}',{paragon_id},'{nr_transakcji}','{nr_paragonu}','{czas}',
                 '{sklep_id}',{typ_stoiska_id},{data_od_stoiska},{kasa_id},{kasjer_id},
                 {wartosc},{rabat},'{import_id}');
              """.format(firma_id        = self.firma_id,
                         paragon_id      = zwrot_id,
                         nr_transakcji   = T1Tekst.split(";")[2],
                         nr_paragonu     = Param1,
                         czas            = Data[:10],
                         sklep_id        = Sklep[0],
                         typ_stoiska_id  = rsglobal.ZmienNaNULL(RSTypStoiska),
                         data_od_stoiska = rsglobal.ZmienNaNULL(RSDataOdStoiska),
                         kasa_id         = rsglobal.ZmienNaNULL(RSKasa),
                         kasjer_id       = NrKasjera,
                         wartosc         = Zwrocono,
                         rabat           = 0,
                         import_id       = DokId)

        # FunkcjaZapisuDoLogow('Nagwek zwrotu '+str(DokId)+': '+INS)
        stat, dump, dump, dump = self.PolSQL.Execute(INS, 1, CzyDodawacBlad=True)
        if stat != 'OK':
            FunkcjaZapisuDoLogow(info + DekodujBlad(stat) + stat)
            return 'Bd zapisu nagwka zwrotu do rs_hd_paragony:\n ' + stat

        # ----------------------------------------------------------------------------------------------------------
        # Zapis danych o patnoci do rs_hd_paragony_platnosci
        # ----------------------------------------------------------------------------------------------------------

        INS_PLAT = """INSERT INTO rs_hd_paragony_platnosci
                      (firma_id, paragon_id, platnosc, wartosc)
                      VALUES """

        Karta = Gotowka = '0'

        if len(T2Tekst.split(";")) >= 3 and T2Tekst.split(";")[2] not in ('', '0'):  # zapacono kart
            Karta = T2Tekst.split(";")[2]
            INS_PLAT += """('{firma_id}',{paragon_id},'{platnosc}',{wartosc}),
                         """.format(firma_id   = self.firma_id,
                                    paragon_id = zwrot_id,
                                    platnosc   = 'KAR',
                                    wartosc    = float(Karta[:-2]+'.'+Karta[-2:]))

        if len(T2Tekst.split(";")) > 0 and T2Tekst.split(";")[0] not in ('', '0'):  # zapacono gotwk
            Gotowka = T2Tekst.split(";")[0]
            INS_PLAT += """('{firma_id}',{paragon_id},'{platnosc}',{wartosc}),
                        """.format(firma_id   = self.firma_id,
                                   paragon_id = zwrot_id,
                                   platnosc   = 'G',
                                   wartosc    = float(Gotowka[:-2]+'.'+Gotowka[-2:]))

        if Karta == '0' and Gotowka == '0':
            INS_PLAT += """('{firma_id}',{paragon_id},'{platnosc}',{wartosc}),
                        """.format(firma_id   = self.firma_id,
                                   paragon_id = zwrot_id,
                                   platnosc   = 'KAR' if Kwota5 > 0 else 'G',
                                   wartosc    = Zwrocono)

        INS_PLAT = INS_PLAT.strip()[:-1] + ";"
        # FunkcjaZapisuDoLogow('Patno zwrotu '+str(DokId)+': '+INS_PLAT)
        stat, dump, dump, dump = self.PolSQL.Execute(INS_PLAT, 1, CzyDodawacBlad=True)
        if stat != 'OK':
            FunkcjaZapisuDoLogow(info + DekodujBlad(stat) + stat)
            return 'Bd zapisu danych o patnoci do rs_hd_paragony_platnosci:\n ' + stat

        # ------------------------------------------------------------------------------------------------------
        # Pobranie danych pozycji paragonu z PC Market
        # ------------------------------------------------------------------------------------------------------

        # Pobranie danych potrzebnych do wypenienia rs_hd_paragony_pozycje

        SEL = """SELECT PD.DokId, PD.NrPozycji, T.Kod, PD.IloscPlus, PD.Stawka,
                        PD.CenaPoRab+(CAST(PD.Stawka as decimal)/CAST(10000 as decimal))*PD.CenaPoRab,
                        T.CenaEw,
                        PD.Wartosc+(CAST(PD.Stawka as decimal)/CAST(10000 as decimal))*PD.Wartosc
                 FROM Dok D
                 JOIN PozDok PD ON (D.DokId = PD.DokId)
                 JOIN Towar T ON (PD.TowId = T.TowId)
                 WHERE D.DokId={DokId};"""
        SEL = SEL.format(Data  = DataImport,
                         MagId = int(Sklep[1]),
                         DokId = DokId)
        stat, PCMZwrotyPoz, dump = self.RSPCMarketSQL.Execute(SEL)
        if stat != 'OK':
            FunkcjaZapisuDoLogow(info + DekodujBlad(stat) + stat)
            return 'Bd pobrania pozycji zwrotu z PC Market:\n ' + stat

        # ------------------------------------------------------------------------------------------------------
        # Zapis pozycji do rs_hd_paragony_pozycje
        # ------------------------------------------------------------------------------------------------------

        INS_POZ = """INSERT INTO rs_hd_paragony_pozycje
                     (firma_id, paragon_id, lp, typ, artykul_id,
                     proc_vat, ilosc, cena_spr, cena_ew, wartosc)
                     VALUES """

        for pozycja in PCMZwrotyPoz:
            DokId     = int(pozycja[0])
            NrPozycji = int(pozycja[1])
            Kod       = pozycja[2].strip()  # artykul_id z rs_artykuly lub kod EAN z rs_artykuly_ean
            IloscPlus = pozycja[3]
            Stawka    = int(pozycja[4])     # 0, 500, 800, 2300
            Cena      = pozycja[5]          # CenaPoRab (jednostkowa netto) + podatek
            CenaEw    = pozycja[6]          # CenaEw z tabeli Towar
            Brutto    = pozycja[7]          # Warto cakowita netto dla pozycji + podatek

            # ------------------------------------------------------------------------------------------------------
            # Kod z tabeli Towar jest z reguy taki sam jak artykul_id z rs_artykuly,
            # ale jeli jest EANem, wtedy moe by powizany z innym artykul_id ktry trzeba znale

            # Kod jest EANem
            if Kod.isdigit() and len(Kod) in (7, 8, 12, 13):
                SEL = """SELECT 
                         COALESCE((SELECT artykul_id FROM rs_artykuly WHERE firma_id = '{firma_id}' AND artykul_id='{Kod}'), 
                                  (SELECT artykul_id FROM rs_artykuly_ean WHERE firma_id = '{firma_id}' AND ean='{Kod}'));"""
                SEL = SEL.format(firma_id=self.firma_id, Kod=Kod)
                stat, RSArtykuly, dump, dump = self.PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
                if stat != 'OK':
                    FunkcjaZapisuDoLogow(info + DekodujBlad(stat) + stat)
                    return 'Bd pobrania id artykuu:\n ' + stat
                elif len(RSArtykuly) == 0:
                    FunkcjaZapisuDoLogow(info + 'artyku wystpujcy na zwrocie nie istnieje w kartotece artykuw')
                    return 'artyku wystpujcy na zwrocie nie istnieje w kartotece artykuw'
                else:
                    artykul_id = RSArtykuly[0][0]
            # Kod nie jest EANem
            else:
                artykul_id = Kod

            # ------------------------------------------------------------------------------------------------------

            INS_POZ += """('{firma_id}',{paragon_id},{lp},{typ},'{artykul_id}',
                           '{proc_vat}',{ilosc},{cena_spr},{cena_ew},{wartosc}),
                       """.format(firma_id   = self.firma_id,
                                  paragon_id = zwrot_id,
                                  lp         = NrPozycji,
                                  typ        = '1', # na razie zawsze 1 - jeli si zmieni, uwzgldni w DekodujBlad()
                                  artykul_id = artykul_id,
                                  proc_vat   = str(Stawka / 100).rjust(2, '0')[:2] if Stawka >= 0 else 'zw',
                                  ilosc      = -1*IloscPlus,
                                  cena_spr   = Cena,
                                  cena_ew    = CenaEw,  #WS 2020-05-12
                                  wartosc    = -1*Brutto)

        INS_POZ = INS_POZ.strip()[:-1] + ";"
        # FunkcjaZapisuDoLogow('Pozycja zwrotu '+str(DokId)+': '+INS_POZ)
        stat, dump, dump, dump = self.PolSQL.Execute(INS_POZ, 1, CzyDodawacBlad=True)
        if stat != 'OK':
            FunkcjaZapisuDoLogow(info + DekodujBlad(stat) + stat)
            return 'Bd zapisu pozycji zwrotw do rs_hd_paragony_pozycje:\n ' + stat

    # ------------------------------------------------------------------------------------------------------

    FunkcjaZapisuDoLogow(info + DekodujBlad(stat) + 'zaimportowano '+str(LNowychParagonow)+' nowych paragonw')
    FunkcjaZapisuDoLogow(info + DekodujBlad(stat) + 'zaimportowano '+str(LNowychZwrotow)+' nowych zwrotw')

    return 'OK'


def DekodujBlad(stat):
    """Funkcja ktra wyapuje ze stat sowa kluczowe i na ich podstawie podpowiada po ludzku, co mogo pj nie tak"""

    if 'deadlock' in stat:
        return u'dostp do danych jest chwilowo zablokowany\n'
    if 'fk_rs_hd_paragony_pozycje_rs_artykuly' in stat:
        return u'artyku wystpujcy na paragonie nie istnieje w kartotece artykuw\n'
    if 'fk_rs_hd_paragony_platnosci_rs_sposoby_zaplaty' in stat:
        return u"brak definicji odpowiednich kodw ('G', 'KAR') w sowniku Sposoby zapaty\n"
    if 'fk_rs_hd_paragony_pozycje_rs_hd_paragony_typy_pozycji' in stat:
        return u"brak definicji odpowiedniego kodu ('1') w sowniku Typy pozycji paragonw fiskalnych\n"

    return ''
