# -*- coding: iso8859_2 -*-

# -----------------------------------------------------------------------------
# Name:        fs_hd_placowki_handlowe_stoiska.py
# Purpose:     przegladanie i usuwanie stoisk przypisanych do placwki handlowej
# Author:
# Created:     2019/08/20
# RCS-ID:      $Id: fs_hd_placowki_handlowe_stoiska.py $
# Copyright:   (c) 2019 RecordSystem
# Licence:     GNU GPL
# -----------------------------------------------------------------------------

# 2019-11-13 Wojciech Skowron - (11711) Filtr na aktywne stoiska
# 2019-11-28 Wojciech Skowron - (11739) Poprawka w kontroli dat
# 2019-12-04 Wojciech Skowron - (11739) Sprawdzenie czy dla tej samej placwki nie ma ju dodanego innego stoiska
#                                       o tym samym typie i pokrywajcych si datach
# 2019-12-05 Wojciech Skowron - (11773) W edycji dat kas i stoisk doda kontrol na obecne powizania kas do stoisk
# 2019-12-18 Wojciech Skowron - (11783) Kontrola dat midzy nagwkiem placwki handlowej i stoiskami placwki handlowej
# 2019-01-09 Wojciech Skowron - (11711) Komunikat przy usuwaniu o istniejcych powizaniach z kasami lub paragonach

import rsglobal

"""
Funkcje:
1 - Dodawanie, Modyfikacja i Usuwanie danych(grid edytowalny)
5 - Pobranie danych
 """

"""
091330 Stoiska
091332 Stoiska modyfikacja
"""

stat = 'UPR PRZ'
TylkoAktywne = False #WS 2019-11-13

# -------------------------------------------------------------------------------
# -----------------------   UPRAWNIENIA   ---------------------------------------
# -------------------------------------------------------------------------------

UprPrzegl = ((self.oddzial_id, '091330') in self.uprawnienia) or (self.operator_id == STANDARD_OPERATOR_ID)
UprMod    = ((self.oddzial_id, '091332') in self.uprawnienia) or (self.operator_id == STANDARD_OPERATOR_ID)

if UprPrzegl > 0:
    stat = 'OK'

if stat == 'OK' and not 'Funkcja' in klucze:
    stat = 'Brak podanej funkcji w kluczach'

if stat == "OK":
    Funkcja = klucze['Funkcja']

if stat == 'OK' and Funkcja in (1, 5, ):
    sklep_id = klucze['sklep_id']
    TylkoAktywne = klucze['TylkoAktywne'] #WS 2019-11-13

if stat == 'OK' and Funkcja in (1, ):
    if not UprMod > 0:
        stat = 'UPR MOD'

#-------------------------------------------------------------------------------
#-------- EDYCJA GRIDA EDYTOWALNEGO (1)
#-------------------------------------------------------------------------------

Insert = []
Update = []
Delete = []
dane   = []
wyn2   = []

if stat == 'OK' and Funkcja == 1:
    if not UprMod:
        stat = 'BR UPR MOD'
        dane.append(stat)
    Update = klucze['Update']
    Insert = klucze['Insert']
    Delete = klucze['Delete']

#-------------------------------------------------------------------------------
#-----------------------------   DELETE (1) ------------------------------------
#-------------------------------------------------------------------------------

if stat == 'OK' and Funkcja == 1:
    if len(Delete[0]) > 0:
        # wykonanie DELETE
        if stat == 'OK':
            DEL = ''
            pola = Delete[1]

            for wartosci in Delete[0]:
                #WS 2019-01-09
                # sprawdzenie, czy nie istniej powizania z kasami
                SQL = """SELECT FROM rs_hd_kasy_stoiska WHERE firma_id='{firma_id}' AND sklep_id='{sklep_id}'
                         AND typ_stoiska_id='{typ_stoiska_id}' AND data_od_stoiska='{data_od_stoiska}';"""
                SQL = SQL.format(firma_id        = self.firma_id,
                                 sklep_id        = sklep_id,
                                 typ_stoiska_id  = wartosci[0],
                                 data_od_stoiska = wartosci[1])
                stat, powiazania, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
                if stat != 'OK':
                    stat = 'Bd sprawdzania powiza kas ze stoiskami: ' + stat
                    dane.append(stat)
                elif len(powiazania) != 0:
                    stat = 'Nie mona usun stoiska {} od {}, poniewa istniej powizania z kasami'.format(wartosci[0], wartosci[1])
                    dane.append(stat)
                else:
                    # sprawdzenie, czy nie istniej paragony dla tego stoiska
                    SQL = """SELECT FROM rs_hd_paragony WHERE firma_id='{firma_id}' AND sklep_id='{sklep_id}'
                             AND typ_stoiska_id='{typ_stoiska_id}' AND data_od_stoiska='{data_od_stoiska}';"""
                    SQL = SQL.format(firma_id        = self.firma_id,
                                     sklep_id        = sklep_id,
                                     typ_stoiska_id  = wartosci[0],
                                     data_od_stoiska = wartosci[1])
                    stat, paragony, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
                    if stat != 'OK':
                        stat = 'Bd sprawdzania paragonw: ' + stat
                        dane.append(stat)
                    elif len(paragony) != 0:
                        stat = 'Nie mona usun stoiska {} od {}, poniewa istniej przypisane do niego paragony fiskalne'.format(wartosci[0], wartosci[1])
                        dane.append(stat)
                    else:
                        DEL += """DELETE FROM rs_hd_sklepy_stoiska WHERE firma_id='{firma_id}' AND sklep_id='{sklep_id}'
                                 AND typ_stoiska_id='{typ_stoiska_id}' AND data_od='{data_od}';"""
                        DEL = DEL.format(firma_id        = self.firma_id,
                                         sklep_id        = sklep_id,
                                         typ_stoiska_id  = wartosci[0],
                                         data_od         = wartosci[1])

                # DEL += """DELETE FROM rs_hd_sklepy_stoiska WHERE firma_id='{}' AND sklep_id='{}'""".format(self.firma_id, sklep_id)
                # for pole, wart in zip(pola, wartosci):
                #     DEL += " AND "+pole+"='"+wart+"'"
                #
                # DEL += ";"

            if stat == "OK" and DEL != '':
                rsglobal.ZapiszDoLogow(DEL)
                stat,dump,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(DEL),1,CzyDodawacBlad=True)
                if stat != 'OK':
                    stat = "Bd przy usuwaniu danych: "+stat
                    dane.append(stat)

# -------------------------------------------------------------------------------
# ------------------------   SPRAWDZANIE DANYCH(1)   ----------------------------
# -------------------------------------------------------------------------------

if Funkcja == 1:
    if stat == 'OK':
        UPD_powiazanie = ''

        # UPDATE
        if len(Update[0]) > 0:
            klucz = Update[3]

            for pola, wartosci, przed, wiersz in zip(Update[0], Update[1], Update[2], Update[-1]):
                po = list(przed)

                od       = None
                do       = None #WS 2019-12-04
                do_przed = None #WS 2019-12-05

                for pole, wart in zip(pola, wartosci):
                    if pole == 'typ_stoiska_id':
                        if wart.strip() == '':
                            stat = 'Nie podano typu stoiska'
                            dane.append(stat)
                        else:
                            po[klucz.index('typ_stoiska_id')] = wart
                            SQL = "SELECT 1 FROM rs_hd_typy_stoisk " \
                                  "WHERE firma_id='{}' AND typ_stoiska_id='{}';".format(self.firma_id, wart)
                            stat, stoisko, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
                            if stat != 'OK':
                                stat = 'Bd sprawdzania typu stoiska: ' + stat
                                dane.append(stat)
                            elif len(stoisko) == 0:
                                stat = 'Niepoprawny typ stoiska: {}'.format(wart)
                                dane.append(stat)
                            if stat != 'OK':
                                wyn2.append((wiersz, 0))
                                stat = 'OK'

                    elif pole == 'data_od':
                        if wart in ('', '-  -', '    -  -  '):
                            stat = 'Nie podano daty od'
                            dane.append(stat)
                        elif not rsglobal.SprawdzCzyToData(wart):
                            stat = 'Niepoprawna data od'
                            dane.append(stat)
                        else:
                            po[klucz.index('data_od')] = wart
                            if 'data_do' not in pola:
                                #WS 2019-11-28
                                SQL = """SELECT data_do FROM rs_hd_sklepy_stoiska WHERE firma_id='{firma_id}' AND sklep_id='{sklep_id}'
                                         AND typ_stoiska_id='{typ_stoiska_id}' AND data_od='{data_od}';"""
                                SQL = SQL.format(firma_id       = self.firma_id,
                                                 sklep_id       = sklep_id,
                                                 typ_stoiska_id = przed[klucz.index('typ_stoiska_id')],
                                                 data_od        = przed[klucz.index('data_od')])
                                # SQL = "SELECT data_do FROM rs_hd_sklepy_stoiska WHERE firma_id='{}' AND sklep_id='{}'""".format(
                                #     self.firma_id, sklep_id)
                                stat, datado, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
                                if stat != 'OK':
                                    stat = 'Bd sprawdzania poprawnoci dat: ' + stat
                                    dane.append(stat)
                                else:
                                    do = datado[0][0]

                                if stat == 'OK' and rsglobal.SprawdzCzyToData(do) and do < wart:
                                    stat = 'Data do jest wczeniejsza ni data od.'
                                    # stat = 'Data od jest pniejsza ni data do.'
                                    dane.append(stat)
                        if stat != 'OK':
                            wyn2.append((wiersz, 1))
                            stat = 'OK'

                    elif pole == 'data_do':
                        if wart not in ('', '-  -', '    -  -  '):
                            if not rsglobal.SprawdzCzyToData(wart):
                                stat = 'Niepoprawna data do'
                                dane.append(stat)
                            else:
                                if 'data_od' in pola:
                                    od = wartosci[pola.index('data_od')]

                                    if rsglobal.SprawdzCzyToData(od) and od > wart:
                                        stat = 'Data od jest pniejsza ni data do.'
                                        dane.append(stat)
                                else:
                                    rsglobal.ZapiszDoLogow('else')
                                    SQL = """SELECT data_od FROM rs_hd_sklepy_stoiska WHERE firma_id='{firma_id}' AND sklep_id='{sklep_id}'
                                             AND typ_stoiska_id='{typ_stoiska_id}' AND data_od='{data_od}';"""
                                    SQL = SQL.format(firma_id       = self.firma_id,
                                                     sklep_id       = sklep_id,
                                                     typ_stoiska_id = przed[klucz.index('typ_stoiska_id')],
                                                     data_od        = przed[klucz.index('data_od')])
                                    # SQL = "SELECT data_od FROM rs_hd_sklepy_stoiska WHERE firma_id='{}' AND sklep_id='{}'""".format(self.firma_id, sklep_id)
                                    stat, dataod, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
                                    if stat != 'OK':
                                        stat = 'Bd sprawdzania poprawnoci dat: ' + stat
                                        dane.append(stat)
                                    else:
                                        od = dataod[0][0]

                                    if stat == 'OK' and rsglobal.SprawdzCzyToData(od) and od > wart:
                                        # rsglobal.ZapiszDoLogow(wart)
                                        # rsglobal.ZapiszDoLogow(od)
                                        stat = 'Data do jest wczeniejsza, ni data od.'
                                        dane.append(stat)
                                    else:
                                        #WS 2019-12-05
                                        # pobranie wczeniejszej daty do
                                        SQL = """SELECT data_do FROM rs_hd_sklepy_stoiska WHERE firma_id='{firma_id}' AND sklep_id='{sklep_id}'
                                                 AND typ_stoiska_id='{typ_stoiska_id}' AND data_od='{data_od}';"""
                                        SQL = SQL.format(firma_id       = self.firma_id,
                                                         sklep_id       = sklep_id,
                                                         typ_stoiska_id = przed[klucz.index('typ_stoiska_id')],
                                                         data_od        = przed[klucz.index('data_od')])
                                        stat, datadoprzed, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
                                        if stat != 'OK':
                                            stat = 'Bd sprawdzania poprawnoci dat: ' + stat
                                            dane.append(stat)
                                        elif len(datadoprzed) == 1:
                                            do_przed = datadoprzed[0][0]

                        if stat != 'OK':
                            wyn2.append((wiersz, 2))
                            stat = 'OK'
                        #WS 2019-12-04
                        else:
                            do = wart

                if do_przed is None:
                    do_przed = do

                #WS 2019-12-04
                # pobranie daty do, jeli adna z dat nie bya modyfikowana
                if do is None:
                    SQL = """SELECT data_od, data_do FROM rs_hd_sklepy_stoiska WHERE firma_id='{firma_id}' AND sklep_id='{sklep_id}'
                             AND typ_stoiska_id='{typ_stoiska_id}' AND data_od='{data_od}';"""
                    SQL = SQL.format(firma_id       = self.firma_id,
                                     sklep_id       = sklep_id,
                                     typ_stoiska_id = przed[klucz.index('typ_stoiska_id')],
                                     data_od        = przed[klucz.index('data_od')])
                    # SQL = "SELECT data_od FROM rs_hd_sklepy_stoiska WHERE firma_id='{}' AND sklep_id='{}'""".format(self.firma_id, sklep_id)
                    stat, dataod, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
                    if stat != 'OK':
                        stat = 'Bd sprawdzania poprawnoci dat: ' + stat
                        dane.append(stat)
                    else:
                        do = do_przed = dataod[0][1]

                elif do_przed and do != do_przed and do not in ('', '    -  -  ','None'):
                    # wprowadzono now dat do - jest rna od wczeniejszej i nie jest pusta
                    # jeli istniej powizania z pust dat do - trzeba je zamkn
                    SQL = """UPDATE rs_hd_kasy_stoiska SET data_do='{data_do}'
                             WHERE firma_id='{firma_id}' AND sklep_id='{sklep_id}'
                             AND typ_stoiska_id='{typ_stoiska_id}' AND data_od_stoiska='{data_od_klucz}'
                             AND data_od >= '{data_od}' AND data_do IS NULL;"""
                    SQL = SQL.format(firma_id       = self.firma_id,
                                     sklep_id       = sklep_id,
                                     typ_stoiska_id = przed[klucz.index('typ_stoiska_id')],
                                     data_od_klucz  = przed[klucz.index('data_od')],
                                     data_od        = po[klucz.index('data_od')],
                                     data_do        = do)
                    UPD_powiazanie += SQL

                #WS 2019-12-18
                # kontrola z datami uruchomienia/zamkniecia sklepu
                SQL = """SELECT data_od, data_do FROM rs_hd_sklepy
                         WHERE firma_id='{firma_id}' AND sklep_id='{sklep_id}';
                      """.format(firma_id=self.firma_id,
                                 sklep_id=sklep_id)
                stat, daty_sklep, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
                if stat != 'OK':
                    stat = 'Bd pobierania danych placowki handlowej: ' + stat
                    dane.append(stat)
                else:
                    data_od_placowki = daty_sklep[0][0] if daty_sklep[0][0] not in ('', '    -  -  ', 'None') else None
                    data_do_placowki = daty_sklep[0][1] if daty_sklep[0][1] not in ('', '    -  -  ', 'None') else None

                    data_od_stoiska = po[klucz.index('data_od')] if po[klucz.index('data_od')] not in ('', '    -  -  ', 'None', None) else None
                    data_do_stoiska = do if do not in ('', '    -  -  ', 'None', None) else None

                    if data_od_stoiska and data_od_placowki and data_od_stoiska < data_od_placowki:
                        stat = 'Data od stoiska nie moe by wczeniejsza ni data od placwki.'
                        wyn2.append((wiersz, 1))
                        dane.append(stat)
                    if data_od_stoiska and data_do_placowki and data_od_stoiska > data_do_placowki:
                        stat = 'Data od stoiska nie moe by pniejsza ni data do placwki.'
                        wyn2.append((wiersz, 1))
                        dane.append(stat)
                    if data_do_stoiska and data_od_placowki and data_do_stoiska < data_od_placowki:
                        stat = 'Data do stoiska nie moe by wczeniejsza ni data od placwki.'
                        wyn2.append((wiersz, 2))
                        dane.append(stat)
                    if data_do_stoiska and data_do_placowki and data_do_stoiska > data_do_placowki:
                        stat = 'Data do stoiska nie moe by pniejsza ni data do placwki.'
                        wyn2.append((wiersz, 2))
                        dane.append(stat)

                #WS 2019-12-04
                # sprawdzenie czy dla tej samej placwki nie ma ju dodanego innego stoiska o tym samym typie
                # i pokrywajcych si datach
                SQL = """SELECT 1 FROM rs_hd_sklepy_stoiska
                         WHERE firma_id='{firma_id}' AND sklep_id='{sklep_id}' AND typ_stoiska_id='{typ_stoiska_id_nowy}' 
                         AND NOT (sklep_id='{sklep_id}' AND typ_stoiska_id='{typ_stoiska_id}' AND data_od='{data_od}')
                         AND
                         (((data_od<=COALESCE({data_do_nowa},'2501-01-01')::DATE AND data_od>='{data_od_nowa}')
                         OR 
                         (data_do<=COALESCE({data_do_nowa},'2501-01-01')::DATE AND data_do>='{data_od_nowa}'))
                         OR
                         (data_od<='{data_od_nowa}' AND data_do>=COALESCE({data_do_nowa},'2501-01-01')::DATE)
                         OR
                         (data_od IS NOT NULL AND data_do IS NULL AND data_od<'{data_od_nowa}'))
                         ORDER BY data_od DESC LIMIT 1;"""
                SQL = SQL.format(firma_id            = self.firma_id,
                                 sklep_id            = sklep_id,
                                 typ_stoiska_id      = przed[klucz.index('typ_stoiska_id')],
                                 data_od             = przed[klucz.index('data_od')],
                                 typ_stoiska_id_nowy = po[klucz.index('typ_stoiska_id')],
                                 data_od_nowa        = po[klucz.index('data_od')],
                                 data_do_nowa        = "'"+do+"'" if do not in ('', '    -  -  ','None',None) else 'NULL')
                stat, spr, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
                if stat != 'OK':
                    stat = 'Bd sprawdzania poprawnoci dat: ' + stat
                    dane.append(stat)
                elif len(spr) > 0:
                    stat = 'Istnieje inny wiersz o tym samym typie stoiska "{}" i pokrywajcych si datach.'.format(po[klucz.index('typ_stoiska_id')])
                    wyn2.append((wiersz, 0))
                    dane.append(stat)

                #WS 2019-12-05
                # sprawdzenie, czy wprowadzona data od nie jest pniejsza ni data od ktregokolwiek z powiza
                SQL = """SELECT 1 FROM rs_hd_kasy_stoiska
                         WHERE firma_id='{firma_id}' AND sklep_id='{sklep_id}'
                         AND typ_stoiska_id='{typ_stoiska_id}' AND data_od_stoiska='{data_od_klucz}'
                         AND data_od < '{data_od}';"""
                SQL = SQL.format(firma_id = self.firma_id,
                                 sklep_id       = sklep_id,
                                 typ_stoiska_id = przed[klucz.index('typ_stoiska_id')],
                                 data_od_klucz  = przed[klucz.index('data_od')],
                                 data_od        = po[klucz.index('data_od')],
                                 data_do        = do)
                stat, kasy_stoiska, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
                if stat != 'OK':
                    stat = 'Bd sprawdzania dat powiza z kasami: ' + stat
                    dane.append(stat)
                elif len(kasy_stoiska) > 0:
                    stat = 'Nie mona wprowadzi daty od {}, poniewa istnieje rozpoczynajce si wczeniej powizanie z kas.'.format(po[klucz.index('data_od')])
                    wyn2.append((wiersz, 1))
                    dane.append(stat)

                #WS 2019-12-05
                # sprawdzenie, czy wprowadzona data do nie jest wczeniejsza ni data od lub do ktregokolwiek z powiza
                if do not in ('', '    -  -  ','None'):
                    SQL = """SELECT 1 FROM rs_hd_kasy_stoiska
                             WHERE firma_id='{firma_id}' AND sklep_id='{sklep_id}'
                             AND typ_stoiska_id='{typ_stoiska_id}' AND data_od_stoiska='{data_od_klucz}'
                             AND (data_do > '{data_do}' OR data_od > '{data_do}');"""
                    SQL = SQL.format(firma_id       = self.firma_id,
                                     sklep_id       = sklep_id,
                                     typ_stoiska_id = przed[klucz.index('typ_stoiska_id')],
                                     data_od_klucz  = przed[klucz.index('data_od')],
                                     data_od        = po[klucz.index('data_od')],
                                     data_do        = do)
                    stat, kasy_stoiska, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
                    if stat != 'OK':
                        stat = 'Bd sprawdzania dat powiza ze stoiskami: ' + stat
                        dane.append(stat)
                    elif len(kasy_stoiska) > 0:
                        stat = 'Nie mona wprowadzi daty do {}, poniewa istnieje rozpoczynajce lub koczce si po tej dacie powizanie z kas.'.format(do)
                        wyn2.append((wiersz, 2))
                        dane.append(stat)

                if 'typ_stoiska_id' in pola or 'data_od' in pola:
                    SQL = "SELECT 1 FROM rs_hd_sklepy_stoiska " \
                          "WHERE firma_id='{}'  AND sklep_id='{}' AND typ_stoiska_id='{}' AND data_od='{}'" \
                          ";".format(self.firma_id, sklep_id, po[klucz.index('typ_stoiska_id')],
                                     po[klucz.index('data_od')])
                    stat, stoisko, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
                    if stat != 'OK':
                        stat = 'Bd sprawdzania unikalnoci: ' + stat
                        wyn2.append((wiersz, 0))
                        dane.append(stat)
                    elif stoisko not in ([[]], []):
                        stat = 'Typ stoiska "{}" od {} jest ju wprowadzony'.format(po[klucz.index('typ_stoiska_id')], po[klucz.index('data_od')])
                        dane.append(stat)
                if stat != 'OK':
                    # wyn2.append((wiersz, 0))
                    stat = 'OK'


        # INSERT
        if len(Insert[0]) > 0:
            pola = Insert[1]
            for wartosci, wiersz in zip(Insert[0], Insert[-1]):
                typ = wartosci[pola.index('typ_stoiska_id')]
                od = wartosci[pola.index('data_od')]
                do = wartosci[pola.index('data_do')]
                poprawnyWiersz = True

                #Typ
                if typ.strip() == '':
                    stat = 'Nie podano typu stoiska'
                    dane.append(stat)
                else:
                    SQL = "SELECT 1 FROM rs_hd_typy_stoisk " \
                          "WHERE firma_id='{}' AND typ_stoiska_id='{}';".format(self.firma_id, typ)
                    stat, stoisko, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
                    if stat != 'OK':
                        stat = 'Bd sprawdzania typu stoiska: ' + stat
                        dane.append(stat)
                    elif len(stoisko) == 0:
                        stat = 'Niepoprawny typ stoiska: {}'.format(typ)
                        dane.append(stat)
                if stat != 'OK':
                    wyn2.append((wiersz, 0))
                    poprawnyWiersz = False
                    stat = 'OK'

                #Data od
                if od in ('', '-  -', '    -  -  '):
                    stat = 'Nie podano daty od'
                    dane.append(stat)
                else:
                    if rsglobal.SprawdzCzyToData(do) and do < od:
                        stat = 'Data od jest pniejsza ni data do.'
                        dane.append(stat)
                if stat != 'OK':
                    wyn2.append((wiersz, 1))
                    poprawnyWiersz = False
                    stat = 'OK'

                if poprawnyWiersz:
                    SQL = "SELECT 1 FROM rs_hd_sklepy_stoiska " \
                          "WHERE firma_id='{}' AND sklep_id = '{}' AND typ_stoiska_id='{}' AND data_od='{}';".format(self.firma_id, sklep_id, typ, od)
                    stat, stoisko, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
                    rsglobal.ZapiszDoLogow(SQL)
                    if stat != 'OK':
                        stat = 'Bd sprawdzania typu stoiska: ' + stat
                        dane.append(stat)
                    elif stoisko not in ([[]], []):
                        rsglobal.ZapiszDoLogow(stoisko)
                        stat = 'Typ stoiska "{}" od {} jest ju wprowadzony'.format(typ, od)
                        dane.append(stat)
                if stat != 'OK':
                    wyn2.append((wiersz, 0))
                    stat = 'OK'

                #WS 2019-12-18
                # kontrola z datami uruchomienia/zamkniecia sklepu
                SQL = """SELECT data_od, data_do FROM rs_hd_sklepy
                         WHERE firma_id='{firma_id}' AND sklep_id='{sklep_id}';
                      """.format(firma_id        = self.firma_id,
                                 sklep_id        = sklep_id)
                stat, daty_sklep, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
                if stat != 'OK':
                    stat = 'Bd pobierania danych placowki handlowej: ' + stat
                    dane.append(stat)
                else:
                    data_od_placowki = daty_sklep[0][0] if daty_sklep[0][0] not in ('', '    -  -  ','None') else None
                    data_do_placowki = daty_sklep[0][1] if daty_sklep[0][1] not in ('', '    -  -  ','None') else None

                    data_od_stoiska = wartosci[pola.index('data_od')] if wartosci[pola.index('data_od')] not in ('', '    -  -  ','None',None) else None
                    data_do_stoiska = wartosci[pola.index('data_do')] if wartosci[pola.index('data_do')] not in ('', '    -  -  ','None',None) else None

                    if data_od_stoiska and data_od_placowki and data_od_stoiska < data_od_placowki:
                        stat = 'Data od stoiska nie moe by wczeniejsza ni data od placwki.'
                        wyn2.append((wiersz, 1))
                        dane.append(stat)
                    if data_od_stoiska and data_do_placowki and data_od_stoiska > data_do_placowki:
                        stat = 'Data od stoiska nie moe by pniejsza ni data do placwki.'
                        wyn2.append((wiersz, 1))
                        dane.append(stat)
                    if data_do_stoiska and data_od_placowki and data_do_stoiska < data_od_placowki:
                        stat = 'Data do stoiska nie moe by wczeniejsza ni data od placwki.'
                        wyn2.append((wiersz, 2))
                        dane.append(stat)
                    if data_do_stoiska and data_do_placowki and data_do_stoiska > data_do_placowki:
                        stat = 'Data do stoiska nie moe by pniejsza ni data do placwki.'
                        wyn2.append((wiersz, 2))
                        dane.append(stat)

                #WS 2019-12-04
                # sprawdzenie czy dla tej samej placwki nie ma ju dodanego innego stoiska o tym samym typie
                # i pokrywajcych si datach
                SQL = """SELECT 1 FROM rs_hd_sklepy_stoiska
                         WHERE firma_id='{firma_id}' AND sklep_id='{sklep_id}' AND typ_stoiska_id='{typ_stoiska_id}' 
                         AND NOT (sklep_id='{sklep_id}' AND typ_stoiska_id='{typ_stoiska_id}' AND data_od='{data_od}')
                         AND
                         (((data_od<=COALESCE({data_do},'2501-01-01')::DATE AND data_od>='{data_od}')
                         OR 
                         (data_do<=COALESCE({data_do},'2501-01-01')::DATE AND data_do>='{data_od}'))
                         OR
                         (data_od<='{data_od}' AND data_do>=COALESCE({data_do},'2501-01-01')::DATE)
                         OR
                         (data_od IS NOT NULL AND data_do IS NULL AND data_od<'{data_od}'))
                         ORDER BY data_od DESC LIMIT 1;"""
                SQL = SQL.format(firma_id       = self.firma_id,
                                 sklep_id       = sklep_id,
                                 typ_stoiska_id = wartosci[pola.index('typ_stoiska_id')],
                                 data_od        = wartosci[pola.index('data_od')],
                                 data_do        = "'"+wartosci[pola.index('data_do')]+"'" if wartosci[pola.index('data_do')] not in ('', '    -  -  ','None',None) else 'NULL')
                stat, spr, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
                if stat != 'OK':
                    stat = 'Bd sprawdzania poprawnoci dat: ' + stat
                    dane.append(stat)
                elif len(spr) > 0:
                    stat = 'Istnieje inny wiersz o tym samym typie stoiska "{}" i pokrywajcych si datach.'.format(wartosci[pola.index('typ_stoiska_id')])
                    wyn2.append((wiersz, 0))
                    dane.append(stat)

        if stat == 'OK':
            if len(dane) > 0:
                stat = dane[0]
            #WS 2019-12-05
            elif UPD_powiazanie:
                rsglobal.ZapiszDoLogow(UPD_powiazanie, 1)
                stat, dump, dump, dump = self.PolSQL.Execute(UPD_powiazanie, 1, CzyDodawacBlad=True)
                if stat != 'OK':
                    stat = 'Bdy zamykania powiza: ' + stat
                    dane.append(stat)

#-------------------------------------------------------------------------------
#-------------------------------  UPDATE (1)  ----------------------------------
#-------------------------------------------------------------------------------

if stat == 'OK' and Funkcja == 1 and len(dane) == 0:
    if len(Update[0]) > 0:

        # wykonanie UPDATE
        if stat == 'OK':
            UPD = ''

            for pola, wartosci, klucz in zip(Update[0], Update[1], Update[2]):
                UPD += "UPDATE rs_hd_sklepy_stoiska SET "
                for pole, wart in zip(pola, wartosci):
                    if wart in ('', '-  -', '    -  -  '):
                        UPD += pole+"=NULL,"
                    else:
                        UPD += pole+"='"+str(wart)+"',"

                UPD = UPD.strip(",")
                UPD += " WHERE firma_id='"+self.firma_id+"' AND sklep_id='"+sklep_id+"'"

                for pole, wart in zip(Update[3], klucz): # petla po kluczu
                    UPD+= " AND "+pole+"='"+str(rsglobal.Dekoduj(wart))+"'"

                UPD += ";"
            if UPD != '':
                rsglobal.ZapiszDoLogow(UPD)
                stat,dump,dump,dump=self.PolSQL.Execute(rsglobal.Dekoduj(UPD),1,CzyDodawacBlad=True)
                if stat != 'OK':
                    stat = "Bd przy modyfikowaniu danych: "+stat
                    dane.append(stat)

#-------------------------------------------------------------------------------
#------------------------------  INSERT (1)  -----------------------------------
#-------------------------------------------------------------------------------

if stat == 'OK' and Funkcja == 1 and len(dane) == 0:
    if len(Insert[0]) > 0:

        # wykonanie INSERT
        INS = ''
        pola = Insert[1]

        for wartosci in Insert[0]:
            INSERT  = "INSERT INTO rs_hd_sklepy_stoiska "
            KOLUMNY = "(firma_id, sklep_id"
            VALUES  = " VALUES ('"+self.firma_id+"','"+sklep_id+"'"
            for pole, wart in zip(pola, wartosci):
                KOLUMNY += ","+pole
                if wart in ('', '-  -', '    -  -  '):
                    VALUES+=",NULL"
                else:
                    VALUES+=",'"+str(wart)+"'"

            KOLUMNY+=")"
            VALUES+=");"
            INS += INSERT+KOLUMNY+VALUES
        if INS != '':
            rsglobal.ZapiszDoLogow(INS, 5)
            stat,dump,dump,dump=self.PolSQL.Execute(rsglobal.Dekoduj(INS),1,CzyDodawacBlad=True)
            if stat != 'OK':
                stat = "Bd przy dodawaniu danych: "+stat
                dane.append(stat)


if stat == 'OK' and len(dane) > 0:
    stat = dane[0]
# -------------------------------------------------------------------------------
# ------------------------   POBRANIE DANYCH(1, 5)   ----------------------------
# -------------------------------------------------------------------------------

if stat == 'OK' and Funkcja in (1, 5, ):
    #WS 2019-11-13
    if TylkoAktywne:
        aktywne = "AND (data_do IS NULL OR data_do >= NOW()::DATE)"
    else:
        aktywne = ""

    SQL = "SELECT typ_stoiska_id, data_od, data_do FROM rs_hd_sklepy_stoiska " \
          "WHERE firma_id='{}' {} AND sklep_id='{}' ORDER BY typ_stoiska_id, data_od;"
    SQL = SQL.format(self.firma_id, aktywne, sklep_id)
    stat, dane, wyn1, wyn2 = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
    if stat != 'OK':
        rsglobal.ZapiszDoLogow(stat, 1)
        stat = 'BL SQL PRZ'

