# -*- coding: iso8859_2 -*-
# -------------------------------------------------------------------------------
# Name:        fs_jpk_ewidencja_zakupu_sprzedazy_z_deklaracja.py
# Purpose:     Tworzenie JPK_V7
# Author:      Wojciech Skowron
# Created:     2020/03/13
# RCS-ID:      $Id: fs_jpk_ewidencja_zakupu_sprzedazy_z_deklaracja.py $
# Copyright:   (c) 2020 RecordSystem
# Licence:     GNU GPL
# -------------------------------------------------------------------------------

# 2020-05-25 Wojciech Skowron - (19902) aktualizacja schematu do v1-2E
# 2020-07-20 Wojciech Skowron - (12055) zliczanie dokumentw z oznaczeniami na potrzeby raportu
# 2020-09-07 Wojciech Skowron - (12096) JPK_V7K
# 2020-09-09 Wojciech Skowron - (12103) Zmiana warunkw dla pl K_11-K_20
# 2020-09-24 Wojciech Skowron - (12133) Zaokraglenia dla pl z kwota_ma i kwota_vat_ma
# 2020-10-23 Wojciech Skowron - (12169) War. 10 - jeli 'MPP' to ignorowane czy_mpp
# 2020-11-24 Wojciech Skowron - (12197) NIP zagranicznego kontrahenta bez przedrostka z kodem kraju
# 2020-12-03 Wojciech Skowron - (12203) Uwzgldnienie wart. NULL w data_wplywu przy pobieraniu dk do rejestru zakupu
# 2022-01-19 Wojciech Skowron - (12568) aktualizacja schematu do (2) 1-0E
# 2023-04-27 Wojciech Skowron - (13402) Pobranie danych kontrahenta z rs_dowody_ksiegowe_kontrahent
# 2023-05-05 Wojciech Skowron - (12103) Pobranie i uwzgldnienie pola sprzedaz_uslug w K_12
# 2023-07-20 Wojciech Skowron - (13489) W rejestrze zakupu pobiera dk tylko w stanie Z
# 2023-10-20 Wojciech Skowron - (13587) Pusty string zamiast 'BRAK' jeli brak NrDostawcy/NazwaDostawcy
# 2023-10-26 Wojciech Skowron - (13589) Lista dowodw bez NIP dla jpk_v7m/k
# 2024-01-22 Wojciech Skowron - (13675) Lista dowodw bez kontrahentw dla jpk_v7m/k
# 2024-02-23 Wojciech Skowron - (13700) Pole sprzedaz_uslug dla DK, zmiana w P/K_11 i P/K_12
# 2024-02-27 Wojciech Skowron - (13708) Zmiana w P/K_11, P/K_12 i P/K_13

import rsglobal


class EwidencjaZakupuSprzedazyZDeklaracja:
    #WS 2020-09-07
    #WS 2020-10-23 - parametry
    def __init__(self, data_od, data_do, cel, warunkisprzedazy, deklaracja, email, wariant, parametry, deklzakwar):
    # def __init__(self, data_od, data_do, cel, warunkisprzedazy, deklaracja, email=''):
        from funkcje import fs_xml
        self.ewidencja_jpk = fs_xml.RSToXml()
        self.ewidencja_jpk_deklaracja = fs_xml.RSToXml()

        self.data_od = data_od
        self.data_do = data_do
        self.cel = cel
        self.warunkisprzedazy = warunkisprzedazy
        self.email = email
        self.deklaracja = deklaracja
        #WS 2020-09-07
        self.wariant = wariant
        self.deklzakwar = deklzakwar
        #WS 2020-10-23
        self.parametry = parametry

    # -------------------------------------------------------------------------------

    def ZapiszEwidencje(self, rejestr):
        import collections
        from datetime import datetime
        import math
        import os
        import time

        from funkcje import fs_jpk_pom

        stat = 'OK'
        dane = []
        ListaDowodowBezNIP = []  #WS 2023-10-26 - uzupeniane w ZakupWiersze
        ListaDowodowBezKontrahenta = []  #WS 2024-01-22 - uzupeniane w ZakupWiersze

        sumy_k_mies = []
        for i in xrange(60):
            sumy_k_mies.append(0)

        #WS 2020-09-07
        sumy_k_kwar = []
        for i in xrange(60):
            sumy_k_kwar.append(0)

        P = []  # lista bdzie zawiera wartoci wszystkich  pl deklaracji
        for i in range(661):
            P.append(0)

        self.czy_deklaracja = False

        #WS 2020-07-20
        LiczbaDokumentow = collections.OrderedDict()  # sownik bdzie zawiera liczby dokumentw z poszczeglnymi oznaczeniami
        LiczbaDokumentow["S_RO"] = LiczbaDokumentow["S_WEW"] = LiczbaDokumentow["S_FP"] = 0
        for i in range(1, 14):
            LiczbaDokumentow["S_GTU_"+str(i).rjust(2, '0')] = 0
        #WS 2022-01-19 - nowe kody WSTO_EE, IED
        LiczbaDokumentow["S_WSTO_EE"] = LiczbaDokumentow["S_IED"] = \
        LiczbaDokumentow["S_SW"] = LiczbaDokumentow["S_EE"] = LiczbaDokumentow["S_TP"] = LiczbaDokumentow["S_TT_WNT"] = \
        LiczbaDokumentow["S_TT_D"] = LiczbaDokumentow["S_MR_T"] = LiczbaDokumentow["S_MR_UZ"] = LiczbaDokumentow["S_I_42"] = \
        LiczbaDokumentow["S_I_63"] = LiczbaDokumentow["S_B_SPV"] = LiczbaDokumentow["S_B_SPV_DOSTAWA"] = \
        LiczbaDokumentow["S_B_MPV_PROWIZJA"] = LiczbaDokumentow["S_MPP"] = 0
        LiczbaDokumentow["S_SprzedazVAT_Marza"] = 0
        LiczbaDokumentow["Z_MK"] = LiczbaDokumentow["Z_VAT_RR"] = LiczbaDokumentow["Z_WEW"] = 0
        LiczbaDokumentow["Z_MPP"] = 0
        LiczbaDokumentow["Z_IMP"] = 0
        LiczbaDokumentow["Z_ZakupVAT_Marza"] = 0

        # ----------------
        # Naglowek
        # ----------------

        p1 = rejestr['Podmiot1']
        
        od = datetime.strptime(self.data_od, "%Y-%m-%d")
        do = datetime.strptime(self.data_do, "%Y-%m-%d")

        Rok = str(do.year)
        Miesiac = str(do.month)
        if do.month < 7 and do.year <= 2020:  #WS 2020-05-25
            DataWytworzeniaJPK = '2020-07-01T00:00:00Z'
        else:
            DataWytworzeniaJPK = time.strftime("%Y-%m-%d") + 'T' + datetime.now().time().isoformat()[:-7] + 'Z'

        self.ewidencja_jpk.Inicjuj("JPK", 2)
        self.ewidencja_jpk.WstawWRoot("tns:Naglowek")
        if do.year < 2022:  #WS 2022-01-19
            if self.wariant == 'M':
                self.ewidencja_jpk.WstawTekst('<tns:KodFormularza kodSystemowy="JPK_V7M (1)" wersjaSchemy="1-2E">JPK_VAT</tns:KodFormularza>')
            else:  #WS 2020-09-07
                self.ewidencja_jpk.WstawTekst('<tns:KodFormularza kodSystemowy="JPK_V7K (1)" wersjaSchemy="1-2E">JPK_VAT</tns:KodFormularza>')
            self.ewidencja_jpk.WstawNowyWZWartoscia("Naglowek", "tns:WariantFormularza", '1')
        else:
            if self.wariant == 'M':
                self.ewidencja_jpk.WstawTekst('<tns:KodFormularza kodSystemowy="JPK_V7M (2)" wersjaSchemy="1-0E">JPK_VAT</tns:KodFormularza>')
            else:  #WS 2020-09-07
                self.ewidencja_jpk.WstawTekst('<tns:KodFormularza kodSystemowy="JPK_V7K (2)" wersjaSchemy="1-0E">JPK_VAT</tns:KodFormularza>')
            self.ewidencja_jpk.WstawNowyWZWartoscia("Naglowek", "tns:WariantFormularza", '2')
        self.ewidencja_jpk.WstawNowyWZWartoscia("Naglowek", "tns:DataWytworzeniaJPK",        DataWytworzeniaJPK)
        self.ewidencja_jpk.WstawNowyWZWartoscia("Naglowek", "tns:NazwaSystemu",              "Anakonda")
        self.ewidencja_jpk.WstawTekst('<tns:CelZlozenia poz="P_7">'+str(self.cel)+'</tns:CelZlozenia>')
        self.ewidencja_jpk.WstawNowyWZWartoscia("Naglowek", "tns:KodUrzedu",                 p1['urzad_skarbowy_id'])
        self.ewidencja_jpk.WstawNowyWZWartoscia("Naglowek", "tns:Rok",                       Rok)
        self.ewidencja_jpk.WstawNowyWZWartoscia("Naglowek", "tns:Miesiac",                   Miesiac)
        #WS 2020-05-25 - przeniesione do wza Deklaracja zgodnie ze schematem 1-2E
        # if not (self.cel == '2' and self.deklaracja['Kor.cz. ewid.']):
        #     self.ewidencja_jpk.WstawTekst('<tns:KodFormularzaDekl kodSystemowy="VAT-7 (21)" kodPodatku="VAT" rodzajZobowiazania="Z" wersjaSchemy="1-2E">VAT-7</tns:KodFormularzaDekl>')
        #     self.ewidencja_jpk.WstawNowyWZWartoscia("Naglowek", "tns:WariantFormularzaDekl", '21')
        self.ewidencja_jpk.Zamknij("tns:Naglowek")

        # ----------------
        # Podmiot1
        # ----------------

        self.ewidencja_jpk.WstawTekst('<tns:Podmiot1 rola="Podatnik">')
        if p1['osoba_fizyczna']:
            self.ewidencja_jpk.WstawNowyW("tns:Podmiot1", "tns:OsobaFizyczna")
            self.ewidencja_jpk.WstawNowyWZWartoscia("tns:OsobaFizyczna", "etd:NIP",           str(p1['nip']).strip())
            self.ewidencja_jpk.WstawNowyWZWartoscia("tns:OsobaFizyczna", "etd:ImiePierwsze",  p1['imie'].encode('utf-8'))
            self.ewidencja_jpk.WstawNowyWZWartoscia("tns:OsobaFizyczna", "etd:Nazwisko",      p1['nazwisko'].encode('utf-8'))
            self.ewidencja_jpk.WstawNowyWZWartoscia("tns:OsobaFizyczna", "etd:DataUrodzenia", p1['data_urodzenia'])
            self.ewidencja_jpk.WstawNowyWZWartoscia("tns:OsobaFizyczna", "tns:Email",         self.email.encode('utf-8'))
            if p1['tel1'] != '':
                self.ewidencja_jpk.WstawNowyWZWartoscia("tns:OsobaFizyczna", "tns:Telefon",   p1['tel1'])
            self.ewidencja_jpk.Zamknij("tns:OsobaFizyczna")
        else:
            self.ewidencja_jpk.WstawNowyW("tns:Podmiot1", "tns:OsobaNiefizyczna")
            self.ewidencja_jpk.WstawNowyWZWartoscia("tns:OsobaNieFizyczna", "tns:NIP",         str(p1['nip']).strip())
            self.ewidencja_jpk.WstawNowyWZWartoscia("tns:OsobaNieFizyczna", "tns:PelnaNazwa",  p1['pelna_nazwa'].encode('utf-8'))
            self.ewidencja_jpk.WstawNowyWZWartoscia("tns:OsobaNieFizyczna", "tns:Email",       self.email.encode('utf-8'))
            if p1['tel1'] != '':
                self.ewidencja_jpk.WstawNowyWZWartoscia("tns:OsobaNieFizyczna", "tns:Telefon", p1['tel1'])
            self.ewidencja_jpk.Zamknij("tns:OsobaNiefizyczna")
        self.ewidencja_jpk.Zamknij("tns:Podmiot1")

        self.ewidencja_jpk.WstawTekst('<!--DEKLARACJA-->')  # Tu zostanie pniej wstawiona deklaracja

        self.ewidencja_jpk.WstawWRoot("tns:Ewidencja")  #WS 2020-05-25

        podatek = 0.0
        num = 0
        if 'SprzedazWiersze' in rejestr.keys():

            # ----------------
            # SprzedazWiersze
            # ----------------

            for i in rejestr['SprzedazWiersze']:
                num += 1
                TypDokumentu = i[24].strip()

                if TypDokumentu != 'FP':
                    podatek += i[17] + i[19] + i[21]
                #WS 2020-05-25
                self.ewidencja_jpk.WstawNowyW("tns:Ewidencja", "tns:SprzedazWiersz")
                # self.ewidencja_jpk.WstawTekst('<tns:SprzedazWiersz>')
                self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:LpSprzedazy", num)
                if i[9].strip() not in ('', 'None'):
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:KodKrajuNadaniaTIN", i[9])
                else:
                    ListaDowodowBezKontrahenta.append([i[34], i[35], i[36], i[37]])  #WS 2024-01-22
                if i[23].strip() == '':
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:NrKontrahenta", 'BRAK')
                else:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:NrKontrahenta", str(i[23]).encode('utf-8'))
                if i[4].strip() == '':
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:NazwaKontrahenta", 'BRAK')
                else:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:NazwaKontrahenta", i[4].encode('utf-8'))
                if i[7].strip() == '':
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:DowodSprzedazy", 'BRAK')
                else:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:DowodSprzedazy", str(i[7]).encode('utf-8'))
                self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:DataWystawienia", i[1])
                self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:DataSprzedazy", i[3])
                if i[24] != '':  # ozn_sprzedaz
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:TypDokumentu", TypDokumentu)
                    LiczbaDokumentow["S_"+TypDokumentu] += 1  #WS 2020-07-20

                if i[28] != 'None':
                    kody_gtu = i[28].split(';')  # kody gtu pobrane z pozycji s oddzielone znakiem ;
                    kody_gtu = list(set(kody_gtu))  # usunicie duplikatw
                    kody_gtu.sort()
                    for kod in kody_gtu:
                        self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:GTU_"+kod, '1')
                        LiczbaDokumentow["S_GTU_"+kod] += 1  #WS 2020-07-20

                if i[25] != '' and i[25].strip() != 'MPP':  # ozn_trans
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:"+i[25].strip(), '1')
                    LiczbaDokumentow["S_"+i[25].strip()] += 1  #WS 2020-07-20
                if do.year < 2022:  #WS 2022-01-19
                    if i[25].strip() == 'MPP':
                        self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:MPP", '1')
                        LiczbaDokumentow["S_MPP"] += 1  #WS 2020-07-20
                    #WS 2020-10-23
                    elif i[29] and self.parametry[9].strip() != 'MPP':
                    # elif i[29]:
                        self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:MPP", '1')
                        LiczbaDokumentow["S_MPP"] += 1  #WS 2020-07-20

                if i[26] == 'K':  # 1 jeli korekta
                    if do.year < 2022:  #WS 2022-01-19
                        self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:KorektaPodstawyOpodt", '1')
                    else:
                        #WS 2022-01-19
                        if i[31] != 'None' or i[32] != 'None':
                            self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:KorektaPodstawyOpodt", '1')
                        if i[31] != 'None':
                            self.ewidencja_jpk.WstawNowyWZWartoscia("tns:KorektaPodstawyOpodt", "tns:TerminPlatnosci", i[31])
                        if i[32] != 'None':
                            self.ewidencja_jpk.WstawNowyWZWartoscia("tns:KorektaPodstawyOpodt", "tns:DataZaplaty", i[32])

                if i[9].strip() == 'PL' and i[22] in self.warunkisprzedazy[13]:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_10", i[12])
                    sumy_k_mies[10] += i[12] if TypDokumentu != 'FP' else 0
                #WS 2024-02-27 - nie_podlega_vat or stawka=='00'  #WS 2024-02-23  #WS 2023-05-05  #WS 2020-09-09
                if (i[26] == 'DK' and i[9].strip() != 'PL' and i[38]) or (i[26] != 'DK' and (i[9].strip() != 'PL' and i[11].strip() != 'PL' and (i[30] or i[39]) and i[22] in self.warunkisprzedazy[13])):
                # if i[26] != 'DK' and (i[9].strip() != 'PL' and i[11].strip() != 'PL' and i[30] and i[22] in self.warunkisprzedazy[13]):
                # if i[9].strip() != 'PL' and i[11].strip() != 'PL' and (i[30] or i[26] == 'DK') and i[22] in self.warunkisprzedazy[13]:
                # if i[9].strip() != 'PL' and i[22] in self.warunkisprzedazy[13]:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_11", i[13])
                    sumy_k_mies[11] += i[13] if TypDokumentu != 'FP' else 0
                #WS 2024-02-27 - nie_podlega_vat or stawka=='00'  #WS 2024-02-23  #WS 2023-05-05  #WS 2020-09-09
                if (i[26] == 'DK' and i[9].strip() != 'PL' and i[38] and i[33]) or (i[26] != 'DK' and (i[9].strip() != 'PL' and i[11].strip() != 'PL' and (i[30] or i[39]) and i[33] and i[22] in self.warunkisprzedazy[13])):
                # if i[26] != 'DK' and (i[9].strip() != 'PL' and i[11].strip() != 'PL' and i[30] and i[33] and i[22] in self.warunkisprzedazy[13]):
                # if i[9].strip() != 'PL' and i[11].strip() != 'PL' and (i[30] or i[26] == 'DK') and i[10] and i[22] in self.warunkisprzedazy[13]:
                # if i[9].strip() != 'PL' and i[10] and i[22] in self.warunkisprzedazy[13]:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_12", i[14])
                    sumy_k_mies[12] += i[14] if TypDokumentu != 'FP' else 0
                #WS 2024-02-27  #WS 2023-05-05  #WS 2020-09-09
                if (i[26] == 'DK' and i[9].strip() == 'PL' and i[39]) or ((i[26] != 'DK' and (i[9].strip() == 'PL' or (i[9].strip() != 'PL' and i[11].strip() == 'PL' and not (i[30] or i[39])))) and i[22] in self.warunkisprzedazy[13]):
                # if (i[26] == 'DK' or i[9].strip() == 'PL' or (i[9].strip() != 'PL' and i[11].strip() == 'PL' and not i[30])) and i[22] in self.warunkisprzedazy[13]:
                # if (i[9].strip() == 'PL' or (i[9].strip() != 'PL' and i[11].strip() == 'PL' and (not i[30] or i[26] == 'DK'))) and i[22] in self.warunkisprzedazy[13]:
                # if i[9].strip() == 'PL' and i[22] in self.warunkisprzedazy[13]:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_13", i[15])
                    sumy_k_mies[13] += i[15] if TypDokumentu != 'FP' else 0
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_15", i[16])
                    sumy_k_mies[15] += i[16] if TypDokumentu != 'FP' else 0
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_16", i[17])
                    sumy_k_mies[16] += i[17] if TypDokumentu != 'FP' else 0
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_17", i[18])
                    sumy_k_mies[17] += i[18] if TypDokumentu != 'FP' else 0
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_18", i[19])
                    sumy_k_mies[18] += i[19] if TypDokumentu != 'FP' else 0
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_19", i[20])
                    sumy_k_mies[19] += i[20] if TypDokumentu != 'FP' else 0
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_20", i[21])
                    sumy_k_mies[20] += i[21] if TypDokumentu != 'FP' else 0
                if i[8].strip() == 'ww' or i[22] in self.warunkisprzedazy[0]:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_21", i[13])
                    sumy_k_mies[21] += i[13] if TypDokumentu != 'FP' else 0
                else:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_21", 0.0)
                if i[22] in self.warunkisprzedazy[1]:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_22", i[13])
                    sumy_k_mies[22] += i[13] if TypDokumentu != 'FP' else 0
                else:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_22", 0.0)
                if i[22] in self.warunkisprzedazy[2]:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_23", i[13])
                    sumy_k_mies[23] += i[13] if TypDokumentu != 'FP' else 0
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_24", i[17] + i[19] + i[21])
                    sumy_k_mies[24] += (i[17] + i[19] + i[21]) if TypDokumentu != 'FP' else 0
                else:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_23", 0.0)
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_24", 0.0)
                if i[22] in self.warunkisprzedazy[3]:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_25", i[13])
                    sumy_k_mies[25] += i[13]  if TypDokumentu != 'FP' else 0
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_26", i[17] + i[19] + i[21])
                    sumy_k_mies[26] += (i[17] + i[19] + i[21]) if TypDokumentu != 'FP' else 0
                else:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_25", 0.0)
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_26", 0.0)
                if i[22] in self.warunkisprzedazy[4]:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_27", i[13])
                    sumy_k_mies[27] += i[13] if TypDokumentu != 'FP' else 0
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_28", i[17] + i[19] + i[21])
                    sumy_k_mies[28] += (i[17] + i[19] + i[21]) if TypDokumentu != 'FP' else 0
                else:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_27", 0.0)
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_28", 0.0)
                if i[22] in self.warunkisprzedazy[5]:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_29", i[13])
                    sumy_k_mies[29] += i[13] if TypDokumentu != 'FP' else 0
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_30", i[17] + i[19] + i[21])
                    sumy_k_mies[30] += (i[17] + i[19] + i[21]) if TypDokumentu != 'FP' else 0
                else:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_29", 0.0)
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_30", 0.0)
                if i[22] in self.warunkisprzedazy[7]:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_31", i[13])
                    sumy_k_mies[31] += i[13] if TypDokumentu != 'FP' else 0
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_32", i[17] + i[19] + i[21])
                    sumy_k_mies[32] += (i[17] + i[19] + i[21]) if TypDokumentu != 'FP' else 0
                else:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_31", 0.0)
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_32", 0.0)
                if i[22] in self.warunkisprzedazy[9]:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_33", i[13])
                    sumy_k_mies[33] += i[13] if TypDokumentu != 'FP' else 0
                else:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_33", 0.0)
                if i[22] in self.warunkisprzedazy[10]:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_34", i[13])
                    sumy_k_mies[34] += i[13] if TypDokumentu != 'FP' else 0
                else:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_34", 0.0)
                if i[22] in self.warunkisprzedazy[11]:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_35", i[13])
                    sumy_k_mies[35] += i[13] if TypDokumentu != 'FP' else 0
                else:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_35", 0.0)
                if i[22] in self.warunkisprzedazy[12]:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_36", i[13])
                    sumy_k_mies[36] += i[13] if TypDokumentu != 'FP' else 0
                else:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:K_36", 0.0)
                if i[25].strip() in ('MR_T', 'MR_UZ'):  # ozn_trans
                    if i[26] == 'DK':
                        self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:SprzedazVAT_Marza", i[27])  # kwota_wn
                        LiczbaDokumentow["S_SprzedazVAT_Marza"] += 1  #WS 2020-07-20
                    else:  # jeli faktura lub korekta
                        self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazWiersz", "tns:SprzedazVAT_Marza", i[5])  # do_zaplaty
                        LiczbaDokumentow["S_SprzedazVAT_Marza"] += 1  #WS 2020-07-20
                self.ewidencja_jpk.Zamknij("tns:SprzedazWiersz")

        # -------------
        # SprzedazCtrl
        # -------------

        #WS 2020-05-25
        self.ewidencja_jpk.WstawNowyW("tns:Ewidencja", "tns:SprzedazCtrl")
        # self.ewidencja_jpk.WstawWRoot("tns:SprzedazCtrl")
        self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazCtrl", "tns:LiczbaWierszySprzedazy", num)
        self.ewidencja_jpk.WstawNowyWZWartoscia("SprzedazCtrl", "tns:PodatekNalezny", podatek)
        self.ewidencja_jpk.Zamknij("tns:SprzedazCtrl")

        suma_kwot = 0.0
        num = 0
        if 'ZakupWiersze' in rejestr.keys():

            # -------------
            # ZakupWiersze
            # -------------

            for i in rejestr['ZakupWiersze']:
                #WS 2020-05-25
                self.ewidencja_jpk.WstawNowyW("tns:Ewidencja", "tns:ZakupWiersz")
                # self.ewidencja_jpk.WstawTekst('<tns:ZakupWiersz>')
                num = num + 1
                suma_kwot += i[6] + i[8] + i[9] + i[10]

                self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:LpZakupu", num)
                if i[12].strip() != '':
                    self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:KodKrajuNadaniaTIN", i[12])
                #WS 2023-10-20
                # if i[2].strip() == '':
                #     self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:NrDostawcy", 'BRAK')
                # else:
                if str(i[2]).strip() != '':
                    self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:NrDostawcy", str(i[2]).encode('utf-8'))
                else:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:NrDostawcy", 'BRAK')
                    if i[0].strip() != '':
                        ListaDowodowBezNIP.append([i[17], i[18], i[19], i[20], i[0]])  #WS 2023-10-26
                #WS 2024-01-22
                # if i[0].strip() == '':
                #     self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:NazwaDostawcy", 'BRAK')
                # else:
                if i[0].strip() != '':
                    self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:NazwaDostawcy", i[0].encode('utf-8'))
                else:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:NazwaDostawcy", 'BRAK')
                    ListaDowodowBezKontrahenta.append([i[17], i[18], i[19], i[20]])  #WS 2024-01-22
                if i[3].strip() == '':
                    self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:DowodZakupu", 'BRAK')
                else:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:DowodZakupu", str(i[3]).encode('utf-8'))
                self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:DataZakupu", i[11])
                if i[4] != 'None':  #WS 2020-12-03
                    self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:DataWplywu", i[4])
                if i[13] != '':
                    self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:DokumentZakupu", i[13].strip())
                    LiczbaDokumentow["Z_"+i[13].strip()] += 1  #WS 2020-07-20
                if do.year < 2022:  #WS 2022-01-19
                    if i[14].strip() == 'MPP':
                        self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:MPP", '1')
                        LiczbaDokumentow["Z_MPP"] += 1  #WS 2020-07-20
                    #WS 2020-10-23
                    elif i[21] and self.parametry[9].strip() != 'MPP':
                    # elif i[21]:
                        self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:MPP", '1')
                        LiczbaDokumentow["Z_MPP"] += 1  #WS 2020-07-20
                if i[16]:
                    self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:IMP", '1')
                    LiczbaDokumentow["Z_IMP"] += 1  #WS 2020-07-20

                self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:K_40", i[5])
                sumy_k_mies[40] += i[5]
                self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:K_41", i[6])
                sumy_k_mies[41] += i[6]
                self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:K_42", i[7])
                sumy_k_mies[42] += i[7]
                self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:K_43", i[8])
                sumy_k_mies[43] += i[8]
                self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:K_44", i[9])
                sumy_k_mies[44] += i[9]
                self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:K_45", i[10])
                sumy_k_mies[45] += i[10]
                self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:K_46", 0.0)
                # self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:K_47", 0.0)  # ?
                if i[14].strip() in ('MR_T', 'MR_UZ'):  # ozn_trans
                    self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupWiersz", "tns:ZakupVAT_Marza", i[15])  # kwota_ma
                    LiczbaDokumentow["Z_ZakupVAT_Marza"] += 1  #WS 2020-07-20
                self.ewidencja_jpk.Zamknij("tns:ZakupWiersz")

        # ----------
        # ZakupCtrl
        # ----------

        #WS 2020-05-25
        self.ewidencja_jpk.WstawNowyW("tns:Ewidencja", "tns:ZakupCtrl")
        # self.ewidencja_jpk.WstawWRoot("tns:ZakupCtrl")
        self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupCtrl", "tns:LiczbaWierszyZakupow", num)
        self.ewidencja_jpk.WstawNowyWZWartoscia("ZakupCtrl", "tns:PodatekNaliczony", suma_kwot)
        self.ewidencja_jpk.Zamknij("tns:ZakupCtrl")

        self.ewidencja_jpk.Zamknij("tns:Ewidencja")  #WS 2020-05-25
        self.ewidencja_jpk.Zamknij("tns:JPK")

        #WS 2020-09-07
        # ---------------------------------------------------------------------------------
        # Sumowanie kwot dla sprzeday/zakupu za kwarta na potrzeby deklaracji dla JPK_V7K
        # ---------------------------------------------------------------------------------

        if 'SprzedazKwar' in rejestr.keys():
            for i in rejestr['SprzedazKwar']:
                TypDokumentu = i[24].strip()

                if i[9].strip() == 'PL' and i[22] in self.warunkisprzedazy[13]:
                    sumy_k_kwar[10] += i[12] if TypDokumentu != 'FP' else 0
                #WS 2024-02-27 - nie_podlega_vat or stawka=='00'  #WS 2024-02-23  #WS 2023-05-05  #WS 2020-09-09
                if (i[26] == 'DK' and i[9].strip() != 'PL' and i[38]) or (i[26] != 'DK' and (i[9].strip() != 'PL' and i[11].strip() != 'PL' and (i[30] or i[39]) and i[22] in self.warunkisprzedazy[13])):
                # if i[26] != 'DK' and (i[9].strip() != 'PL' and i[11].strip() != 'PL' and i[30] and i[22] in self.warunkisprzedazy[13]):
                # if i[9].strip() != 'PL' and i[11].strip() != 'PL' and (i[30] or i[26] == 'DK') and i[22] in self.warunkisprzedazy[13]:
                # if i[9].strip() != 'PL' and i[22] in self.warunkisprzedazy[13]:
                    sumy_k_kwar[11] += i[13] if TypDokumentu != 'FP' else 0
                #WS 2024-02-27 - nie_podlega_vat or stawka=='00'  #WS 2024-02-23  #WS 2023-05-05  #WS 2020-09-09
                if (i[26] == 'DK' and i[9].strip() != 'PL' and i[38] and i[33]) or (i[26] != 'DK' and (i[9].strip() != 'PL' and i[11].strip() != 'PL' and (i[30] or i[39]) and i[33] and i[22] in self.warunkisprzedazy[13])):
                # if i[26] != 'DK' and (i[9].strip() != 'PL' and i[11].strip() != 'PL' and i[30] and i[33] and i[22] in self.warunkisprzedazy[13]):
                # if i[9].strip() != 'PL' and i[11].strip() != 'PL' and (i[30] or i[26] == 'DK') and i[10] and i[22] in self.warunkisprzedazy[13]:
                # if i[9].strip() != 'PL' and i[10] and i[22] in self.warunkisprzedazy[13]:
                    sumy_k_kwar[12] += i[14] if TypDokumentu != 'FP' else 0
                #WS 2024-02-27  #WS 2023-05-05  #WS 2020-09-09
                if (i[26] == 'DK' and i[9].strip() == 'PL' and i[39]) or ((i[26] != 'DK' and (i[9].strip() == 'PL' or (i[9].strip() != 'PL' and i[11].strip() == 'PL' and not (i[30] or i[39])))) and i[22] in self.warunkisprzedazy[13]):
                # if (i[9].strip() == 'PL' or (i[9].strip() != 'PL' and i[11].strip() == 'PL' and (not i[30] or i[26] == 'DK'))) and i[22] in self.warunkisprzedazy[13]:
                # if i[9].strip() == 'PL' and i[22] in self.warunkisprzedazy[13]:
                    sumy_k_kwar[13] += i[15] if TypDokumentu != 'FP' else 0
                    sumy_k_kwar[15] += i[16] if TypDokumentu != 'FP' else 0
                    sumy_k_kwar[16] += i[17] if TypDokumentu != 'FP' else 0
                    sumy_k_kwar[17] += i[18] if TypDokumentu != 'FP' else 0
                    sumy_k_kwar[18] += i[19] if TypDokumentu != 'FP' else 0
                    sumy_k_kwar[19] += i[20] if TypDokumentu != 'FP' else 0
                    sumy_k_kwar[20] += i[21] if TypDokumentu != 'FP' else 0
                if i[8].strip() == 'ww' or i[22] in self.warunkisprzedazy[0]:
                    sumy_k_kwar[21] += i[13] if TypDokumentu != 'FP' else 0
                if i[22] in self.warunkisprzedazy[1]:
                    sumy_k_kwar[22] += i[13] if TypDokumentu != 'FP' else 0
                if i[22] in self.warunkisprzedazy[2]:
                    sumy_k_kwar[23] += i[13] if TypDokumentu != 'FP' else 0
                    sumy_k_kwar[24] += (i[17] + i[19] + i[21]) if TypDokumentu != 'FP' else 0
                if i[22] in self.warunkisprzedazy[3]:
                    sumy_k_kwar[25] += i[13] if TypDokumentu != 'FP' else 0
                    sumy_k_kwar[26] += (i[17] + i[19] + i[21]) if TypDokumentu != 'FP' else 0
                if i[22] in self.warunkisprzedazy[4]:
                    sumy_k_kwar[27] += i[13] if TypDokumentu != 'FP' else 0
                    sumy_k_kwar[28] += (i[17] + i[19] + i[21]) if TypDokumentu != 'FP' else 0
                if i[22] in self.warunkisprzedazy[5]:
                    sumy_k_kwar[29] += i[13] if TypDokumentu != 'FP' else 0
                    sumy_k_kwar[30] += (i[17] + i[19] + i[21]) if TypDokumentu != 'FP' else 0
                if i[22] in self.warunkisprzedazy[7]:
                    sumy_k_kwar[31] += i[13] if TypDokumentu != 'FP' else 0
                    sumy_k_kwar[32] += (i[17] + i[19] + i[21]) if TypDokumentu != 'FP' else 0
                if i[22] in self.warunkisprzedazy[9]:
                    sumy_k_kwar[33] += i[13] if TypDokumentu != 'FP' else 0
                if i[22] in self.warunkisprzedazy[10]:
                    sumy_k_kwar[34] += i[13] if TypDokumentu != 'FP' else 0
                if i[22] in self.warunkisprzedazy[11]:
                    sumy_k_kwar[35] += i[13] if TypDokumentu != 'FP' else 0
                if i[22] in self.warunkisprzedazy[12]:
                    sumy_k_kwar[36] += i[13] if TypDokumentu != 'FP' else 0

        if 'ZakupKwar' in rejestr.keys():
            for i in rejestr['ZakupKwar']:
                sumy_k_kwar[40] += i[5]
                sumy_k_kwar[41] += i[6]
                sumy_k_kwar[42] += i[7]
                sumy_k_kwar[43] += i[8]
                sumy_k_kwar[44] += i[9]
                sumy_k_kwar[45] += i[10]

        # -----------
        # Deklaracja
        # -----------

        #WS 2020-09-07
        # Tworzymy wze deklaracji,
        # jeli nie jest to korekta samej czci ewidencyjnej lub JPK_V7K bez deklaracji kwartalnej
        if not (self.cel == '2' and self.deklaracja['Kor.cz. ewid.']) and (self.wariant == 'M' or (self.wariant == 'K' and self.deklzakwar)):
        # if not (self.cel == '2' and self.deklaracja['Kor.cz. ewid.']):
            self.czy_deklaracja = True

            self.ewidencja_jpk_deklaracja.Inicjuj("tns:Deklaracja", 2)

            #WS 2020-05-25
            self.ewidencja_jpk_deklaracja.WstawWRoot("tns:Naglowek")
            if do.year < 2022:  #WS 2022-01-19
                if self.wariant == 'M':
                    self.ewidencja_jpk_deklaracja.WstawTekst('<tns:KodFormularzaDekl kodSystemowy="VAT-7 (21)" kodPodatku="VAT" rodzajZobowiazania="Z" wersjaSchemy="1-2E">VAT-7</tns:KodFormularzaDekl>')
                    self.ewidencja_jpk_deklaracja.WstawNowyWZWartoscia("Naglowek", "tns:WariantFormularzaDekl", '21')
                else:  #WS 2020-09-07
                    self.ewidencja_jpk_deklaracja.WstawTekst('<tns:KodFormularzaDekl kodSystemowy="VAT-7K (15)" kodPodatku="VAT" rodzajZobowiazania="Z" wersjaSchemy="1-2E">VAT-7K</tns:KodFormularzaDekl>')
                    self.ewidencja_jpk_deklaracja.WstawNowyWZWartoscia("Naglowek", "tns:WariantFormularzaDekl", '15')
                    self.ewidencja_jpk_deklaracja.WstawNowyWZWartoscia("Naglowek", "tns:Kwartal", str(int(math.ceil(float(Miesiac)/float(3)))))
            else:
                if self.wariant == 'M':
                    self.ewidencja_jpk_deklaracja.WstawTekst('<tns:KodFormularzaDekl kodSystemowy="VAT-7 (22)" kodPodatku="VAT" rodzajZobowiazania="Z" wersjaSchemy="1-0E">VAT-7</tns:KodFormularzaDekl>')
                    self.ewidencja_jpk_deklaracja.WstawNowyWZWartoscia("Naglowek", "tns:WariantFormularzaDekl", '22')
                else:  #WS 2020-09-07
                    self.ewidencja_jpk_deklaracja.WstawTekst('<tns:KodFormularzaDekl kodSystemowy="VAT-7K (16)" kodPodatku="VAT" rodzajZobowiazania="Z" wersjaSchemy="1-0E">VAT-7K</tns:KodFormularzaDekl>')
                    self.ewidencja_jpk_deklaracja.WstawNowyWZWartoscia("Naglowek", "tns:WariantFormularzaDekl", '16')
                    self.ewidencja_jpk_deklaracja.WstawNowyWZWartoscia("Naglowek", "tns:Kwartal", str(int(math.ceil(float(Miesiac)/float(3)))))
            self.ewidencja_jpk_deklaracja.Zamknij("tns:Naglowek")

            self.ewidencja_jpk_deklaracja.WstawWRoot("tns:PozycjeSzczegolowe")

            rsglobal.ZapiszDoLogow(str(self.deklaracja))

            # P - lista bdzie zawiera wartoci wszystkich pl deklaracji

            # Wartoci P_10-P_36 oraz P_40-P_47 uzyskujemy z sum wartoci pl wzw SprzedazWiersz i ZakupWiersz
            if self.deklzakwar:  #WS 2020-09-07
                for i in range(10, 36+1)+range(40, 47+1):
                    P[i] = int(round(sumy_k_kwar[i], 0))  # wszystkie kwoty deklaracji zaokrglamy do liczb cakowitych
            else:
                for i in range(10, 36+1)+range(40, 47+1):
                    P[i] = int(round(sumy_k_mies[i], 0))  # wszystkie kwoty deklaracji zaokrglamy do liczb cakowitych

            # Przepisujemy wartoci wpisane przez operatora na oknie tworzenia JPK
            # None - niewypenione
            if do.year < 2022:  #WS 2022-01-19
                for i in [39, 49, 50, 52, 54, 55, 56, 57, 58, 59, 60, 61, 63, 64, 65, 66, 67, 68, 69]:
                    P[i] = self.deklaracja['P_'+str(i)]
            else:
                for i in [39, 49, 50, 52, 54, 540, 55, 56, 560, 57, 58, 59, 60, 61, 63, 64, 65, 66, 660, 67, 68, 69]:  #WS 2022-01-19 - dodane 540, 560, 660
                    P[i] = self.deklaracja['P_'+str(i)]

            # Wyliczamy wartosci pozostaych pl wg wzorw podanych w schemacie:
            # ---------- P_37 ----------

            P[37] = P[10]+P[11]+P[13]+P[15]+P[17]+P[19]+P[21]+P[22]+P[23]+P[25]+P[27]+P[29]+P[31]

            # ---------- P_38 ----------

            P[38] = P[16]+P[18]+P[20]+P[24]+P[26]+P[28]+P[30]+P[32]+P[33]+P[34]
            P[38] = P[38]-P[35]-P[36]

            # ---------- P_48 ----------

            P[48] = (P[39] or 0)+P[41]+P[43]+P[44]+P[45]+P[46]+P[47]
            
            # ---------- P_49 ----------

            if P[38]-P[48] <= 0:
                P[49] = 0
            elif P[49] is not None and P[38]-P[48] < P[49]:
                P[49] = 'BLAD'
                stat = 'Kwota wykazana w P_49 nie moe by wysza od rnicy kwot z P_38 i P_48.'
                dane.append(stat)

            # ---------- P_50 ----------

            if P[38]-P[48]-(P[49] or 0) < 0:
                P[50] = 0
            elif P[50] is not None and P[38]-(P[48]+(P[49] or 0)) < P[50]:
                P[50] = 'BLAD'
                stat = 'Kwota wykazana w P_50 nie moe by wysza ni rnica pomidzy kwot z P_38 a sum kwot z P_48 i P_49.'
                dane.append(stat)

            # ---------- P_51 ----------

            if P[38]-P[48] > 0:
                P[51] = P[38]-P[48]-(P[49] or 0)-(P[50] or 0)
            else:
                P[51] = 0

            # ---------- P_53 ----------

            if do.year < 2022:  #WS 2022-01-19
                if P[48]-P[38] > 0:
                    P[53] = P[48]-P[38]+(P[52] or 0)
                else:
                    P[53] = 0
            else:
                if P[51] > 0:
                    P[53] = 0
                elif (P[48]+(P[49] or 0)+(P[50] or 0)+(P[52] or 0))-P[38] > 0:
                    P[53] = P[48]-P[38]+(P[49] or 0)+(P[50] or 0)+(P[52] or 0)
                else:
                    P[53] = 0

            # ---------- P_62 ----------

            P[62] = P[53]-(P[54] or 0)

            # --------------------------

            # sprawdzenie zalenoci dla pl P_540-P_58
            liczba_wypelnionych = 0
            if do.year < 2022:  #WS 2022-01-19
                for i in [55, 56, 57, 58]:
                    if P[i] is not None:
                        liczba_wypelnionych += 1
            else:
                for i in [540, 55, 56, 560, 57, 58]:  #WS 2022-01-19 - dodane 540, 560
                    if P[i] is not None:
                        liczba_wypelnionych += 1

            if liczba_wypelnionych == 0 and P[54] is not None:
                stat = 'Jeeli wypeniono pole P_54, naley rwnie wypeni jedno z pl P_540-P_58.'  #WS 2022-01-19
                dane.append(stat)
            if liczba_wypelnionych > 0 and P[54] is None:
                stat = 'Jeeli wypeniono jedno z pl P_540-P_58, naley wypeni take P_54.'  #WS 2022-01-19
                dane.append(stat)
            if liczba_wypelnionych > 1:
                stat = 'Moliwe jest wypenienie tylko jednego z pl P_540-P_58.'  #WS 2022-01-19
                dane.append(stat)

            # sprawdzenie zalenoci dla pl P_59-P_61
            liczba_wypelnionych = 0
            for i in [59, 60, 61]:
                if P[i] is not None and P[i] != '':
                    liczba_wypelnionych += 1

            if 0 < liczba_wypelnionych < 3:
                stat = 'Naley wypeni wszystkie 3 pola P_59-P-61.'
                dane.append(stat)

            # Wypeniamy pola deklaracji:
            if do.year < 2022:  #WS 2022-01-19
                for i in range(10, 70):
                    if P[i] is not None and P[i] != '':
                        self.ewidencja_jpk_deklaracja.WstawNowyWZWartoscia("tns:PozycjeSzczegolowe", "tns:P_"+str(i), P[i])
            else:
                for i in range(10, 54+1)+[540, 55, 56, 560]+range(57, 66+1)+[660, 67, 68, 69]:  #WS 2022-01-19 - dodane 540, 560, 660
                    if P[i] is not None and P[i] != '':
                        self.ewidencja_jpk_deklaracja.WstawNowyWZWartoscia("tns:PozycjeSzczegolowe", "tns:P_"+str(i), P[i])

            if self.deklaracja['P_ORDZU'].strip() != '':
                self.ewidencja_jpk_deklaracja.WstawNowyWZWartoscia("tns:PozycjeSzczegolowe", "tns:P_ORDZU", self.deklaracja['P_ORDZU'].encode('utf-8'))
            self.ewidencja_jpk_deklaracja.Zamknij("tns:PozycjeSzczegolowe")

            if self.deklaracja['Pouczenia'] == 1:
                self.ewidencja_jpk_deklaracja.WstawNowyWZWartoscia("tns:Deklaracja", "tns:Pouczenia", self.deklaracja['Pouczenia'])
            else:
                stat = 'Niewypenione pole Pouczenia.'
                dane.append(stat)

            self.ewidencja_jpk_deklaracja.Zamknij("tns:Deklaracja")

        if self.wariant == 'M':  # JPK_V7M
            nazwa_pliku = 'jpk_v7m_' + datetime.now().strftime("%Y_%m_%d_%H_%M_%S")+'.xml'
        else:  #WS 2020-09-07 - JPK_V7K
            nazwa_pliku = 'jpk_v7k_' + datetime.now().strftime("%Y_%m_%d_%H_%M_%S")+'.xml'

        self.ewidencja_jpk.ZapiszPlik(nazwa_pliku, False, '<?xml version="1.0" encoding="utf-8"?>')
        plik = open(nazwa_pliku, 'r')
        if do.year < 2022:  #WS 2022-01-19
            if self.wariant == 'M':  # JPK_V7M
                tresc = plik.read().replace('&','&amp;').replace('<JPK>', '<tns:JPK xmlns:etd="http://crd.gov.pl/xml/schematy/dziedzinowe/mf/2020/03/11/eD/DefinicjeTypy/" xmlns:tns="http://crd.gov.pl/wzor/2020/05/08/9393/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">')
            else:  #WS 2020-09-07 - JPK_V7K
                tresc = plik.read().replace('&','&amp;').replace('<JPK>', '<tns:JPK xmlns:etd="http://crd.gov.pl/xml/schematy/dziedzinowe/mf/2020/03/11/eD/DefinicjeTypy/" xmlns:tns="http://crd.gov.pl/wzor/2020/05/08/9394/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">')
        else:
            if self.wariant == 'M':  # JPK_V7M
                tresc = plik.read().replace('&','&amp;').replace('<JPK>', '<tns:JPK xmlns:etd="http://crd.gov.pl/xml/schematy/dziedzinowe/mf/2021/06/08/eD/DefinicjeTypy/" xmlns:tns="http://crd.gov.pl/wzor/2021/12/27/11148/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">')
            else:  #WS 2020-09-07 - JPK_V7K
                tresc = plik.read().replace('&','&amp;').replace('<JPK>', '<tns:JPK xmlns:etd="http://crd.gov.pl/xml/schematy/dziedzinowe/mf/2021/06/08/eD/DefinicjeTypy/" xmlns:tns="http://crd.gov.pl/wzor/2021/12/27/11149/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">')
        plik.close()

        #WS 2020-09-07
        # wstawienie wza deklaracji
        if self.czy_deklaracja:
            self.ewidencja_jpk_deklaracja.doc = ''.join(self.ewidencja_jpk_deklaracja.wiersze)
            tresc_deklaracja = self.ewidencja_jpk_deklaracja.doc.replace('&','&amp;')
            tresc = tresc.replace('<!--DEKLARACJA-->', tresc_deklaracja)
        else:
            P = []
            tresc = tresc.replace('<!--DEKLARACJA-->', '')

        plik = open(nazwa_pliku, 'w')

        plik.write(tresc)
        plik.close()

        # przed sprawdzeniem xml ze schematem, najpierw sprawdzamy czy po drodze nie zostay wyapane bdy
        if stat != 'OK':
            os.system('mv ' + nazwa_pliku + ' ' + 'xsd/Blad_' + nazwa_pliku)
            return stat, dane, None

        # sprawdzenie ze schematem
        if do.year < 2022:  #WS 2022-01-19
            if self.wariant == 'M':  # JPK_V7M
                stat = fs_jpk_pom.SprawdzJpk(nazwa_pliku, 'xsd/jpk_v7m(1).xsd')
            else:  #WS 2020-09-07 - JPK_V7K
                stat = fs_jpk_pom.SprawdzJpk(nazwa_pliku, 'xsd/jpk_v7k(1).xsd')
        else:
            if self.wariant == 'M':  # JPK_V7M
                stat = fs_jpk_pom.SprawdzJpk(nazwa_pliku, 'xsd/jpk_v7m.xsd')
            else:  #WS 2020-09-07 - JPK_V7K
                stat = fs_jpk_pom.SprawdzJpk(nazwa_pliku, 'xsd/jpk_v7k.xsd')

        if stat == 'OK':
            os.remove(nazwa_pliku)
            #WS 2020-07-20
            #WS 2023-10-26 - ListaDowodowBezNIP
            #WS 2024-01-22 - ListaDowodowBezKontrahenta
            return stat, [tresc], (podatek, suma_kwot, sumy_k_mies, P, LiczbaDokumentow, ListaDowodowBezNIP, ListaDowodowBezKontrahenta)
            # return stat, [tresc], (podatek, suma_kwot, P)
        else:
            stat = stat.replace('{http://crd.gov.pl/wzor/2020/05/08/9393/}', 'tns:').replace('{http://crd.gov.pl/wzor/2020/05/08/9394/}', 'tns:').replace('{http://crd.gov.pl/wzor/2021/12/27/11148/}', 'tns:').replace('{http://crd.gov.pl/wzor/2021/12/27/11149/}', 'tns:')
            os.system('mv ' + nazwa_pliku + ' ' + 'xsd/Blad_' + nazwa_pliku)
            return stat, ['BLAD_SPR'], None


stat = 'OK'
dane = []
wyn1 = []
wyn2 = []

if not 'Funkcja' in klucze:
    stat = 'Brak podanej funkcji w kluczach'

if stat == 'OK':
    Funkcja = klucze['Funkcja']

    SlPar = {'DataOd': [1, ], 'DataDo': [1, ], 'Cel': [1, ], 'Email': [1, ], 'Deklaracja': [1, ], 'Wariant': [1, ]}

    Lista = rsglobal.SprawdzParametryWKluczach(SlPar, Funkcja, klucze)
    if Lista != '':
        stat = 'Uwaga : wywoanie funkcji fs_jpk_ewidencja_zakupu_sprzedazy z numerem ' + str(
            Funkcja) + ' bez parametrw: ' + Lista

    if Funkcja in [1, 3]:
        DataOd = rsglobal.Dekoduj(klucze['DataOd'])
        DataDo = rsglobal.Dekoduj(klucze['DataDo'])
        Cel = rsglobal.Dekoduj(klucze['Cel'])
        Email = rsglobal.Dekoduj(klucze['Email'])
        Deklaracja = klucze['Deklaracja']
        #WS 2020-09-07
        Wariant = klucze['Wariant']

# -------------------------------------------------------------------------------
# Pobranie danych nagwka  (2)
# -------------------------------------------------------------------------------
if Funkcja == 1 and stat == 'OK':
    #WS 2020-09-07
    DeklZaKwar = False
    # Jeli dla JPK_V7K daty w parametrach obejmuj 3 miesice, to generowana jest deklaracja za kwarta
    if Wariant == 'K' and int(DataDo[5:7])-int(DataOd[5:7]) == 2:
        DeklZaKwar = True
        # DataOdKwar i DataDoKwar - za ten okres pobierane s drugi raz dane ewidencji,
        # aby nastpnie je zsumowa i wykorzysta w deklaracji za kwarta
        DataOdKwar = DataOd
        DataDoKwar = DataDo
        # DataOd i DataDo - za ten okres sporzdzana jest ewidencja
        DataOd = DataDo[:8]+'01'

    # Pobieramy parametry stae
    SQL = """SELECT war_1, war_2, war_3, war_4, war_5, war_6, war_7, war_8, war_9, war_10 
             FROM rs_jpk_parametry
             WHERE firma_id='{firma_id}' AND jpk_id='JPK_VAT' AND data_od<='{DataDo}' ORDER BY data_od DESC LIMIT 1
          """.format(firma_id=self.firma_id, DataDo=DataDo)
    stat, dane_parametry, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
    if stat != 'OK':
        stat = "Bd pobierania parametrw: "+stat
        dane.append(stat)
    elif len(dane_parametry) == 0:
        stat = "Dla podanych dat brak zarejestrowanych parametw staych"
        dane.append(stat)
    else:
        parametry = []
        for wiersz in dane_parametry:
            par = []
            for parametr in wiersz:
                par.append(parametr.replace("`", "'"))
            parametry.append(par)

    if parametry[0][7].strip() != '' and parametry[0][5].strip() != '<None>':
        warunkisprzedazy = rsglobal.rseval(parametry[0][7].strip())
        if len(warunkisprzedazy) != 14:
            stat = 'Nieprawidlowa liczba warunkow sprzedazy.'
            dane.append(stat)
    else:
        warunkisprzedazy = [[], [], [], [], [], [], [], [], [], [], [], [], [], ['111']]

    # Pobieramy dane do nagwka
    if stat == 'OK':
        SQL = """SELECT osoba_fizyczna, nip, 
                        imie, nazwisko, data_urodzenia,
                        CASE WHEN (nazwa1 IS NULL OR TRIM(nazwa1) = '') AND (nazwa2 IS NULL OR TRIM(nazwa2) = '') 
                        THEN nazwa 
                        ELSE COALESCE(nazwa1,'') || ' ' || COALESCE(nazwa2,'') 
                        END AS pelna_nazwa,
                        adr_kraj, adr_wojew, adr_powiat, adr_gmina, adr_ulica, adr_nry, adr_kod, adr_poczta, adr_miej, 
                        COALESCE(tel1,'') AS tel1, urzad_skarbowy_id, regon
                 FROM rs_oddzialy o
                 JOIN rs_firmy f USING(firma_id)
                 WHERE firma_id='{firma_id}' AND centrala""".format(firma_id=self.firma_id)
        stat, dane_podmiot1, wyn1_podmiot1, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
        if stat != 'OK':
            stat = "Bd pobierania danych oddziau: "+stat
            dane.append(stat)
        elif len(dane_podmiot1) == 0:
            stat = "Brak oddziau oznaczonego jako centrala "
            dane.append(stat)

    SprzedazWiersze = None
    # Dane do galazki SprzedazWiersz dla faktur
    if stat == 'OK':
        if parametry[0][5].strip() != '' and parametry[0][5].strip() != '<None>':
            SQL = " SELECT a.wyznacznik_id, a.data, a.numer, a.data_przychodu, b.nazwa, a.do_zaplaty,"  # [0-5]
            SQL += "COALESCE(b.adr_kod,'')||' '||COALESCE(b.adr_miej,'')||' '||COALESCE(b.adr_ulica,'')||' '||COALESCE(b.adr_nry,''),"  # [6]
            SQL += """ rsf_podaj_numer_dokumentu_na_wydruk('""" + self.firma_id + """','["'||a.wyznacznik_id||'", "'||a.data||'", "'||a.numer||'"]','F'), """  # [7]
            SQL += " w.typ_faktury, b.kraj, k.czy_ue, a.kraj_opodatkowania,  "  # [8-11]
            SQL += "(SELECT sum(wartosc_netto) FROM rs_faktury_vat WHERE stawka_vat in ('zw','np') AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [12]
            SQL += "(SELECT sum(wartosc_netto) FROM rs_faktury_vat WHERE (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [13]
            SQL += "(SELECT sum(wartosc_netto) FROM rs_faktury_vat WHERE stawka_vat NOT IN ('zw') AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [14]
            SQL += "(SELECT sum(wartosc_netto) FROM rs_faktury_vat WHERE stawka_vat='00' AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [15]
            SQL += "(SELECT sum(wartosc_netto) FROM rs_faktury_vat WHERE stawka_vat='05' AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [16]
            SQL += "(SELECT sum(wartosc_vat) FROM rs_faktury_vat WHERE stawka_vat='05' AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [17]
            SQL += "(SELECT sum(wartosc_netto) FROM rs_faktury_vat WHERE stawka_vat in ('07','08') AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [18]
            SQL += "(SELECT sum(wartosc_vat) FROM rs_faktury_vat WHERE stawka_vat in ('07','08') AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [19]
            SQL += "(SELECT sum(wartosc_netto) FROM rs_faktury_vat WHERE stawka_vat in ('22','23') AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [20]
            SQL += "(SELECT sum(wartosc_vat) FROM rs_faktury_vat WHERE stawka_vat in ('22','23') AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [21]
            #WS 2020-11-24
            # SQL += " '111', CASE WHEN b.kraj = 'PL' THEN REPLACE(COALESCE(b.nip,'BRAK'), ' ', '') ELSE (SELECT kraj FROM rs_kraje kr WHERE kr.kraj = b.kraj AND kr.czy_ue) || REPLACE(b.nip, ' ', '') END AS nip, "
            SQL += " '111', REPLACE(COALESCE(b.nip,'BRAK'), ' ', '') AS nip, "  # [22],[23] zaslepka dla symblu rejestru z dowodow ksiegowych
            SQL += " COALESCE(a.ozn_sprzedaz,''), COALESCE(a.ozn_trans,''), 'F', '',"  # [24-27]
            SQL += "(SELECT STRING_AGG(kod_gtu,';') FROM rs_faktury_pozycje WHERE firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer AND kod_gtu IS NOT NULL),"  # [28]
            SQL += "a.czy_mpp, nie_podlega_vat, NULL, NULL, sprzedaz_uslug,"  # [29-33]  #WS 2020-09-09 - nie_podlega_vat  #WS 2023-05-05 - sprzedaz_uslug
            SQL += "NULL, NULL, NULL, NULL, NULL,"  # [34-38]  #WS 2024-01-22 - zalepka na klucz dowodu
            SQL += "(SELECT BOOL_AND(stawka_vat='00') FROM rs_faktury_vat WHERE firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer)"  # [39]  #WS 2024-02-27
            SQL += " FROM rs_faktury a  INNER JOIN rs_kontrahenci b ON (a.firma_id=b.firma_id AND a.platnik_id=b.kontrahent_id) "
            SQL += " INNER JOIN rs_wyznaczniki w ON (a.firma_id=w.firma_id AND a.wyznacznik_id=w.wyznacznik_id) "
            SQL += " INNER JOIN rs_kraje k ON (b.kraj=k.kraj ) "
            SQL += parametry[0][5].replace('$firma', self.firma_id).replace('$$data_od$$', DataOd).replace(
                '$$data_do$$', DataDo)
            rsglobal.ZapiszDoLogow('JPK SQL FAKTURY:' + str(SQL), 1)
            stat, SprzedazWierszeFaktury, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
            if stat != 'OK':
                stat = "Bd pobierania wierszy dla faktur: "+stat
                dane.append(stat)
            else:
                SprzedazWiersze = SprzedazWierszeFaktury

    # Dane do galazki SprzedazWiersz dla korekt
    if stat == 'OK':
        if parametry[0][4].strip() != '' and parametry[0][4].strip() != '<None>':
            SQL = " SELECT a.wyznacznik_id, a.data, a.numer, a.data_przychodu, b.nazwa, a.do_zaplaty,"  # [0-5]
            SQL += "COALESCE(b.adr_kod,'')||' '||COALESCE(b.adr_miej,'')||' '||COALESCE(b.adr_ulica,'')||' '||COALESCE(b.adr_nry,''),"  # [6]
            SQL += """ rsf_podaj_numer_dokumentu_na_wydruk('""" + self.firma_id + """','["'||a.wyznacznik_id||'", "'||a.data||'", "'||a.numer||'"]','F'), """  # [7]
            SQL += " w.typ_faktury, b.kraj, k.czy_ue, a.kraj_opodatkowania,  "  # [8-11]
            SQL += "(SELECT sum(wartosc_netto-wartosc_netto_p) FROM rs_korekty_vat WHERE stawka_vat in ('zw','np') AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [12]
            SQL += "(SELECT sum(wartosc_netto-wartosc_netto_p) FROM rs_korekty_vat WHERE (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [13]
            SQL += "(SELECT sum(wartosc_netto-wartosc_netto_p) FROM rs_korekty_vat WHERE stawka_vat NOT IN ('zw') AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [14]
            SQL += "(SELECT sum(wartosc_netto-wartosc_netto_p) FROM rs_korekty_vat WHERE stawka_vat='00' AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [15]
            SQL += "(SELECT sum(wartosc_netto-wartosc_netto_p) FROM rs_korekty_vat WHERE stawka_vat='05' AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [16]
            SQL += "(SELECT sum(wartosc_vat-wartosc_vat_p) FROM rs_korekty_vat WHERE stawka_vat='05' AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [17]
            SQL += "(SELECT sum(wartosc_netto-wartosc_netto_p) FROM rs_korekty_vat WHERE stawka_vat in ('07','08') AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [18]
            SQL += "(SELECT sum(wartosc_vat-wartosc_vat_p) FROM rs_korekty_vat WHERE stawka_vat in ('07','08') AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [19]
            SQL += "(SELECT sum(wartosc_netto-wartosc_netto_p) FROM rs_korekty_vat WHERE stawka_vat in ('22','23') AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [20]
            SQL += "(SELECT sum(wartosc_vat-wartosc_vat_p) FROM rs_korekty_vat WHERE stawka_vat in ('22','23') AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [21]
            #WS 2020-11-24
            # SQL += " '111', CASE WHEN b.kraj = 'PL' THEN REPLACE(COALESCE(b.nip,'BRAK'), ' ', '') ELSE (SELECT kraj FROM rs_kraje kr WHERE kr.kraj = b.kraj AND kr.czy_ue) || REPLACE(b.nip, ' ', '') END AS nip, "
            SQL += " '111', REPLACE(COALESCE(b.nip,'BRAK'), ' ', '') AS nip, "  # [22] [23] zaslepka dla symblu rejestru z dowodow ksiegowych
            SQL += " COALESCE(a.ozn_sprzedaz,''), COALESCE(a.ozn_trans,''), 'K', '',"  # [24-27]
            SQL += "(SELECT STRING_AGG(kod_gtu,';') FROM rs_korekty_pozycje WHERE firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer AND kod_gtu IS NOT NULL),"  # [28]
            SQL += "a.czy_mpp, nie_podlega_vat, termin_platnosci_ulga, data_zaplaty_ulga, sprzedaz_uslug,"  # [29-33]  #WS 2020-09-09 - nie_podlega_vat  #WS 2022-01-19 - termin_platnosci_ulga, data_zaplaty_ulga  #WS 2023-05-05 - sprzedaz_uslug
            SQL += "NULL, NULL, NULL, NULL, NULL,"  # [34-38]  #WS 2024-01-22 - zalepka na klucz dowodu
            SQL += "(SELECT BOOL_AND(stawka_vat='00') FROM rs_korekty_vat WHERE firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer)"  # [39]  #WS 2024-02-27
            SQL += " FROM rs_korekty a  INNER JOIN rs_kontrahenci b ON (a.firma_id=b.firma_id AND a.platnik_id=b.kontrahent_id) "
            SQL += " INNER JOIN rs_wyznaczniki w ON (a.firma_id=w.firma_id AND a.wyznacznik_id=w.wyznacznik_id) "
            SQL += " INNER JOIN rs_kraje k ON (b.kraj=k.kraj ) "
            SQL += parametry[0][4].replace('$firma', self.firma_id).replace('$$data_od$$', DataOd).replace(
                '$$data_do$$', DataDo)

            # print 'SQL pozycje KOREKTY:'
            # print SQL
            rsglobal.ZapiszDoLogow('JPK SQL KOREKTY:' + str(SQL), 1)
            stat, SprzedazWierszeKorekty, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
            if stat != 'OK':
                stat = "Bd pobierania wierszy dla korekt: "+stat
                dane.append(stat)
            else:
                if SprzedazWiersze is None:
                    SprzedazWiersze = SprzedazWierszeKorekty
                else:
                    SprzedazWiersze += SprzedazWierszeKorekty

    # Dane do galazki SprzedazWiersz z dowodow ksiegowych
    if stat == 'OK':
        if parametry[0][6].strip() != '' and parametry[0][6].strip() != '<None>':
            #WS 2020-09-24 - ROUND dla 5 oraz sum z 12-21
            #WS 2023-04-27 - pobranie danych kontrahenta z rs_dowody_ksiegowe_kontrahent do pl 4, 6, 9, 10, 11, 23; LEFT OUTER zamiast INNER JOIN na rs_kontrahenci i rs_kraje
            SQL = " SELECT '', a.data, a.numer, a.data_rejestru, CASE WHEN b.firma_id IS NOT NULL THEN COALESCE(b.nazwa,'BRAK') WHEN kontr.firma_id IS NOT NULL THEN COALESCE(kontr.nazwa,'BRAK') END, ROUND(a.kwota_ma,2),"  # [0-5]
            #WS 2023-04-27
            SQL += "CASE WHEN b.firma_id IS NOT NULL THEN COALESCE(b.adr_kod,'')||' '||COALESCE(b.adr_miej,'')||' '||COALESCE(b.adr_ulica,'')||' '||COALESCE(b.adr_nry,'') "  # [6]
            SQL += "WHEN kontr.firma_id IS NOT NULL THEN COALESCE(kontr.adr_kod,'')||' '||COALESCE(kontr.adr_miej,'')||' '||COALESCE(kontr.adr_ulica,'')||' '||COALESCE(kontr.adr_nry,'') END,"  # [6]
            # SQL += "COALESCE(b.adr_kod,'')||' '||COALESCE(b.adr_miej,'')||' '||COALESCE(b.adr_ulica,'')||' '||COALESCE(b.adr_nry,''),"  # [6]
            SQL += " a.symbol_dok,  "  # [7]
            SQL += " 'KR', CASE WHEN b.firma_id IS NOT NULL THEN b.kraj WHEN kontr.firma_id IS NOT NULL THEN kontr.kraj END, "  # [8-9]
            SQL += " CASE WHEN b.firma_id IS NOT NULL THEN kb.czy_ue WHEN kontr.firma_id IS NOT NULL THEN kkontr.czy_ue END, "  # [10]
            SQL += " CASE WHEN b.firma_id IS NOT NULL THEN b.kraj WHEN kontr.firma_id IS NOT NULL THEN kontr.kraj END,  "  # [11]
            SQL += "(SELECT ROUND(sum(kwota_ma),2) FROM rs_dowody_ksiegowe_pozycje WHERE stawka_vat_ma in ('zw','np') AND (firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer) ),"  # [12]
            SQL += "(SELECT ROUND(sum(kwota_ma),2) FROM rs_dowody_ksiegowe_pozycje WHERE (firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer) ),"  # [13]
            SQL += "(SELECT ROUND(sum(kwota_ma),2) FROM rs_dowody_ksiegowe_pozycje WHERE stawka_vat_ma NOT IN ('zw') AND (firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer) ),"  # [14]
            SQL += "(SELECT ROUND(sum(kwota_ma),2) FROM rs_dowody_ksiegowe_pozycje WHERE stawka_vat_ma='00' AND (firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer) ),"  # [15]
            SQL += "(SELECT ROUND(sum(kwota_ma),2) FROM rs_dowody_ksiegowe_pozycje WHERE stawka_vat_ma='05' AND (firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer) ),"  # [16]
            SQL += "(SELECT ROUND(sum(kwota_vat_ma),2) FROM rs_dowody_ksiegowe_pozycje WHERE stawka_vat_ma='05' AND (firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer) ),"  # [17]
            SQL += "(SELECT ROUND(sum(kwota_ma),2) FROM rs_dowody_ksiegowe_pozycje WHERE stawka_vat_ma IN ('07','08') AND (firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer) ),"  # [18]
            SQL += "(SELECT ROUND(sum(kwota_vat_ma),2) FROM rs_dowody_ksiegowe_pozycje WHERE stawka_vat_ma IN ('07','08') AND (firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer) ),"  # [19]
            SQL += "(SELECT ROUND(sum(kwota_ma),2) FROM rs_dowody_ksiegowe_pozycje WHERE stawka_vat_ma IN ('22','23') AND (firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer) ),"  # [20]
            SQL += "(SELECT ROUND(sum(kwota_vat_ma),2) FROM rs_dowody_ksiegowe_pozycje WHERE stawka_vat_ma IN ('22','23') AND (firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer) ),"  # [21]
            #WS 2020-11-24
            # SQL += " a.rejestr, CASE WHEN b.kraj = 'PL' THEN REPLACE(COALESCE(b.nip,'BRAK'), ' ', '') ELSE (SELECT kraj FROM rs_kraje kr WHERE kr.kraj = b.kraj AND kr.czy_ue) || REPLACE(b.nip, ' ', '') END AS nip, "
            SQL += " a.rejestr, CASE WHEN b.firma_id IS NOT NULL THEN REPLACE(COALESCE(b.nip,'BRAK'), ' ', '') WHEN kontr.firma_id IS NOT NULL THEN REPLACE(COALESCE(kontr.nip,'BRAK'), ' ', '') END AS nip, "  # [22] [23]
            SQL += " COALESCE(a.ozn_sprzedaz,''), COALESCE(a.ozn_trans,''), 'DK', a.kwota_wn,"  # [24-27]
            SQL += "(SELECT STRING_AGG(kod_gtu,';') FROM rs_dowody_ksiegowe_pozycje WHERE firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer AND kod_gtu IS NOT NULL),"  # [28]
            SQL += "a.czy_mpp, NULL, NULL, NULL, a.sprzedaz_uslug,"  # [29-33]  #WS 2020-09-09 - 30 to zalepka na nie_podlega_vat z faktur/korekt  #WS 2024-02-23 - sprzedaz_uslug
            SQL += "a.dowod_id, a.nazwa_id, a.data, a.numer,"  # [34-37]  #WS 2024-01-22 - klucz dowodu do przekazania w przypadku braku kontrahenta
            SQL += "(SELECT BOOL_AND(stawka_vat_ma IN ('00','np')) FROM rs_dowody_ksiegowe_pozycje WHERE firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer ),"  # [38]  #WS 2024-02-23
            SQL += "(SELECT BOOL_AND(stawka_vat_ma='00') FROM rs_dowody_ksiegowe_pozycje WHERE firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer )"  # [39]  #WS 2024-02-27
            SQL += " FROM rs_dowody_ksiegowe a  LEFT OUTER JOIN rs_kontrahenci b ON (a.firma_id=b.firma_id AND a.kontrahent_id=b.kontrahent_id) "
            SQL += " LEFT OUTER JOIN rs_dowody_ksiegowe_kontrahent kontr ON (a.firma_id=kontr.firma_id AND a.okres_id=kontr.okres_id AND a.dowod_id=kontr.dowod_id AND a.nazwa_id=kontr.nazwa_id AND a.data=kontr.data AND a.numer=kontr.numer) "  #WS 2023-04-27
            SQL += " LEFT OUTER JOIN rs_kraje kb ON (b.kraj=kb.kraj ) "
            SQL += " LEFT OUTER JOIN rs_kraje kkontr ON (kontr.kraj=kkontr.kraj ) "  #WS 2023-04-27
            SQL += parametry[0][6].replace('$firma', self.firma_id).replace('$$data_od$$', DataOd).replace(
                '$$data_do$$', DataDo)
            rsglobal.ZapiszDoLogow('JPK SQL DOW.SPRZEDAZ:' + str(SQL), 1)
            stat, SprzedazWierszeDowody, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
            if stat != 'OK':
                stat = "Bd pobierania wierszy dla dowodow: "+stat
                dane.append(stat)
            else:
                if SprzedazWiersze is None:
                    SprzedazWiersze = SprzedazWierszeDowody
                else:
                    SprzedazWiersze += SprzedazWierszeDowody

    # Pobieranie danych do rejestru zakupu (wykomentowany kod to jest funkjca pobierajca rajestr zakupu w ANAKONDZIE) - NIE USUWA!!!
    if stat == 'OK':
        #WS 2023-04-27 - pobranie danych z rs_dowody_ksiegowe_kontrahent do pl 0, 1, 2, 12
        SQL = " SELECT CASE WHEN k.kontrahent_id IS NOT NULL THEN TRIM(COALESCE(k.nazwa,'')) "  # [0]
        SQL += " WHEN dkk.firma_id IS NOT NULL THEN TRIM(COALESCE(dkk.nazwa,'')) ELSE '' END,"  # [0]
        SQL += " CASE WHEN k.kontrahent_id IS NOT NULL THEN COALESCE(k.adr_kod,'')||' '||TRIM(COALESCE(k.adr_miej,''))||', '||TRIM(COALESCE(k.adr_ulica,''))||' '||TRIM(COALESCE(k.adr_nry,'')) "  # [1]
        SQL += " WHEN dkk.firma_id IS NOT NULL THEN COALESCE(dkk.adr_kod,'')||' '||TRIM(COALESCE(dkk.adr_miej,''))||', '||TRIM(COALESCE(dkk.adr_ulica,''))||' '||TRIM(COALESCE(dkk.adr_nry,'')) ELSE 'BRAK' END,"  # [1]
        #AB 04.12.18r
        #WS 2020-11-24
        # SQL += "  CASE WHEN k.kontrahent_id IS NOT NULL THEN TRIM(CASE WHEN k.kraj = 'PL' THEN "  # [2]
        # SQL += "REPLACE(COALESCE(k.nip,'BRAK'), ' ', '') ELSE (SELECT kraj FROM rs_kraje kr WHERE kr.kraj = k.kraj AND "
        # SQL += "kr.czy_ue) || REPLACE(k.nip, ' ', '') END) ELSE 'BRAK' END AS nipp, "
        SQL += "  REPLACE(COALESCE(CASE WHEN k.kontrahent_id IS NOT NULL THEN k.nip WHEN dkk.firma_id IS NOT NULL THEN dkk.nip END,''), ' ', '')  AS nipp, "  # [2]
        SQL += " dk.symbol_dok, "  # [3]
        SQL += "dk.data_wplywu, "  # [4]
        #  [5]
        SQL += " CASE WHEN dk.rejestr IN " + str(
            tuple(parametry[0][0].encode('utf-8').split(','))) + u" THEN SUM(COALESCE(dkp.kwota_wn,0)) ELSE 0 END, "
        #  [6]
        SQL += " CASE WHEN dk.rejestr IN " + str(
            tuple(parametry[0][0].encode('utf-8').split(','))) + u" THEN SUM(COALESCE(dkp.kwota_vat_wn,0)) ELSE 0 END, "
        #        SQL+=" CASE WHEN dk.rejestr='"+parametry[0][0].encode('utf-8')+"' THEN SUM(COALESCE(dkp.kwota_wn,0)) ELSE 0 END, "
        #        SQL+=" CASE WHEN dk.rejestr='"+parametry[0][0].encode('utf-8')+"' THEN SUM(COALESCE(dkp.kwota_vat_wn,0)) ELSE 0 END, "
        #  [7]
        SQL += u" CASE WHEN dk.rejestr not in" + str(
            tuple(parametry[0][1].encode('utf-8').split(','))) + u"  THEN SUM(COALESCE(dkp.kwota_wn,0)) ELSE 0 END, "
        #  [8]
        SQL += u" CASE WHEN dk.rejestr not in " + str(
            tuple(parametry[0][1].encode('utf-8').split(','))) + u" THEN SUM(COALESCE(dkp.kwota_vat_wn,0)) ELSE 0 END, "
        #  [9]
        SQL += " CASE WHEN dk.rejestr IN " + str(tuple(
            parametry[0][2].encode('utf-8').split(','))) + u"  THEN SUM(COALESCE(dkp.kwota_vat_wn,0)) ELSE 0 END, "
        #  [10]
        SQL += u" CASE WHEN dk.rejestr IN " + str(
            tuple(parametry[0][3].encode('utf-8').split(','))) + u" THEN SUM(COALESCE(dkp.kwota_vat_wn,0)) ELSE 0 END, "
        #WS 2020-09-24 - ROUND dla kwota_ma
        SQL += "dk.data_wyst, COALESCE(CASE WHEN k.kontrahent_id IS NOT NULL THEN k.kraj WHEN dkk.firma_id IS NOT NULL THEN dkk.kraj END,''), COALESCE(dk.ozn_nabycie,'') as ozn_nabycie, dk.ozn_trans, ROUND(dk.kwota_ma,2), dk.import,"  # [11-16]
        SQL += "dk.dowod_id,dk.nazwa_id,dk.data,dk.numer,dk.czy_mpp"  # [17-21] #WS 2020-05-25
        # tabele i filtry wydruku
        SQL += " FROM rs_dowody_ksiegowe dk INNER JOIN rs_dowody_ksiegowe_pozycje dkp USING (firma_id,dowod_id,nazwa_id,data,numer)"
        SQL += " LEFT OUTER JOIN rs_kontrahenci k ON (dk.firma_id=k.firma_id AND dk.kontrahent_id=k.kontrahent_id)"
        SQL += " LEFT OUTER JOIN rs_dowody_ksiegowe_kontrahent dkk "
        SQL += " ON (dk.firma_id=dkk.firma_id AND dk.dowod_id=dkk.dowod_id AND dk.nazwa_id=dkk.nazwa_id AND dk.data=dkk.data AND dk.numer=dkk.numer) "
        SQL += " INNER JOIN rs_rejestry_vat rv ON (dk.firma_id=rv.firma_id AND dk.rejestr=rv.kod) "
        if parametry[0][8].strip() != '' and parametry[0][8].strip() != '<None>':
            SQL += parametry[0][8].replace('$firma', self.firma_id).replace('$$data_od$$', DataOd).replace(
                '$$data_do$$', DataDo)
        else:
            SQL += " WHERE dk.firma_id='" + self.firma_id + "' AND COALESCE(dk.data_rejestru,dk.data)>='" + str(
                DataOd) + "' AND COALESCE(dk.data_rejestru,dk.data)<='" + str(DataDo) + "' "
            SQL += " AND NOT dkp.stawka_vat_wn IS NULL AND NOT dkp.sposob_rozl IN (' ','N') AND NOT dk.rejestr IS NULL AND rv.typ='Z' "
            SQL += " AND NOT dkp.zamkniecie_kregu AND dkp.pobierac_na_rejestr AND NOT dkp.stawka_vat_wn IS NULL "  #JR 2019-06-07 11504
            SQL += " AND dk.stan='Z' "  #WS 2023-07-20
            # SQL+=" GROUP BY dk.data_wplywu,dk.symbol_dok,dk.dowod_id,dk.nazwa_id,dk.data,dk.numer,dk.rejestr,dkp.kwota_wn, dkp.kwota_vat_wn,"
            SQL += " GROUP BY dk.data_wplywu,dk.symbol_dok,dk.dowod_id,dk.nazwa_id,dk.data,dk.numer,dk.rejestr,"
            SQL += "ozn_nabycie,dk.ozn_trans,dk.kwota_ma,dk.import,dk.czy_mpp,"
            #AB 04.12.18r
            SQL += " k.kontrahent_id,k.nazwa,nipp,k.adr_kod,k.adr_miej,k.adr_ulica,k.adr_nry,k.kraj,"
            SQL += "dkk.firma_id,dkk.nazwa,dkk.nip,dkk.adr_kod,dkk.adr_miej,dkk.adr_ulica,dkk.adr_nry,dkk.kraj,dk.data_wyst,dk.data_rejestru"
            SQL += " ORDER BY dk.data,dk.dowod_id,dk.numer;"

        rsglobal.ZapiszDoLogow('JPK SQL DOW.ZAKUP:' + str(SQL), 1)
        stat, ZakupWierszePrzed, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
        if stat != 'OK':
            stat = "Bd pobierania danych do rejestru zakupu: " + stat
            dane.append(stat)
        # Dopiero tutaj bdziemy filrowa wynik z zapytania bo bdzie szybciej ni w SQL
        ZakupWiersze = []
        if parametry[0][8].strip() != '' and parametry[0][8].strip() != '<None>':  #WS 2020-12-03
            for b in ZakupWierszePrzed:
                # if (b[4] >=DataOd and b[4] <=DataDo and b[4].strip()!=''):
                #WS 2020-12-03
                if (b[4] <= DataDo and b[4].strip() != '') or b[4] == 'None':
                # if b[4] <= DataDo and b[4].strip() != '':
                    ZakupWiersze.append(b)
                else:
                    rsglobal.ZapiszDoLogow('Odrzucony:' + str(b), 1)
        else:
            # rezygnacja z odrzucenia
            ZakupWiersze=ZakupWierszePrzed

        # wyn2= LStawek

    #WS 2020-09-07
    # Pobranie danych jak do wypenienia ewidencji sprzeday/zakupu, ale za kwarta
    SprzedazKwar = []
    ZakupKwar = []
    if DeklZaKwar:
        # Ew. sprzeday dla faktur
        if stat == 'OK':
            if parametry[0][5].strip() != '' and parametry[0][5].strip() != '<None>':
                SQL = " SELECT a.wyznacznik_id, a.data, a.numer, a.data_przychodu, b.nazwa, a.do_zaplaty,"  # [0-5]
                SQL += "COALESCE(b.adr_kod,'')||' '||COALESCE(b.adr_miej,'')||' '||COALESCE(b.adr_ulica,'')||' '||COALESCE(b.adr_nry,''),"  # [6]
                SQL += """ rsf_podaj_numer_dokumentu_na_wydruk('""" + self.firma_id + """','["'||a.wyznacznik_id||'", "'||a.data||'", "'||a.numer||'"]','F'), """  # [7]
                SQL += " w.typ_faktury, b.kraj, k.czy_ue, a.kraj_opodatkowania,  "  # [8-11]
                SQL += "(SELECT sum(wartosc_netto) FROM rs_faktury_vat WHERE stawka_vat in ('zw','np') AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [12]
                SQL += "(SELECT sum(wartosc_netto) FROM rs_faktury_vat WHERE (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [13]
                SQL += "(SELECT sum(wartosc_netto) FROM rs_faktury_vat WHERE stawka_vat NOT IN ('zw') AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [14]
                SQL += "(SELECT sum(wartosc_netto) FROM rs_faktury_vat WHERE stawka_vat='00' AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [15]
                SQL += "(SELECT sum(wartosc_netto) FROM rs_faktury_vat WHERE stawka_vat='05' AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [16]
                SQL += "(SELECT sum(wartosc_vat) FROM rs_faktury_vat WHERE stawka_vat='05' AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [17]
                SQL += "(SELECT sum(wartosc_netto) FROM rs_faktury_vat WHERE stawka_vat in ('07','08') AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [18]
                SQL += "(SELECT sum(wartosc_vat) FROM rs_faktury_vat WHERE stawka_vat in ('07','08') AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [19]
                SQL += "(SELECT sum(wartosc_netto) FROM rs_faktury_vat WHERE stawka_vat in ('22','23') AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [20]
                SQL += "(SELECT sum(wartosc_vat) FROM rs_faktury_vat WHERE stawka_vat in ('22','23') AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [21]
                #WS 2020-11-24
                # SQL += " '111', CASE WHEN b.kraj = 'PL' THEN REPLACE(COALESCE(b.nip,'BRAK'), ' ', '') ELSE (SELECT kraj FROM rs_kraje kr WHERE kr.kraj = b.kraj AND kr.czy_ue) || REPLACE(b.nip, ' ', '') END AS nip, "
                SQL += " '111', REPLACE(COALESCE(b.nip,'BRAK'), ' ', '') AS nip, "  # [22],[23] zaslepka dla symblu rejestru z dowodow ksiegowych
                SQL += " COALESCE(a.ozn_sprzedaz,''), COALESCE(a.ozn_trans,''), 'F', '',"  # [24-27]
                SQL += "(SELECT STRING_AGG(kod_gtu,';') FROM rs_faktury_pozycje WHERE firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer AND kod_gtu IS NOT NULL),"  # [28]
                SQL += "a.czy_mpp, nie_podlega_vat, NULL, NULL, sprzedaz_uslug,"  # [29-33]  #WS 2020-09-09 - nie_podlega_vat  #WS 2023-05-05 - sprzedaz_uslug
                SQL += "NULL, NULL, NULL, NULL, NULL,"  # [34-38]  #WS 2024-01-22 - zalepka na klucz dowodu
                SQL += "(SELECT BOOL_AND(stawka_vat='00') FROM rs_faktury_vat WHERE firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer)"  # [39]  #WS 2024-02-27
                SQL += " FROM rs_faktury a  INNER JOIN rs_kontrahenci b ON (a.firma_id=b.firma_id AND a.platnik_id=b.kontrahent_id) "
                SQL += " INNER JOIN rs_wyznaczniki w ON (a.firma_id=w.firma_id AND a.wyznacznik_id=w.wyznacznik_id) "
                SQL += " INNER JOIN rs_kraje k ON (b.kraj=k.kraj ) "
                SQL += parametry[0][5].replace('$firma', self.firma_id).replace('$$data_od$$', DataOdKwar).replace(
                    '$$data_do$$', DataDoKwar)
                rsglobal.ZapiszDoLogow('JPK SQL FAKTURY KWAR:' + str(SQL), 1)
                stat, SprzedazKwarFaktury, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
                if stat != 'OK':
                    stat = "Bd pobierania wierszy dla faktur: "+stat
                    dane.append(stat)
                else:
                    SprzedazKwar = SprzedazKwarFaktury

        # Ew. sprzeday dla korekt
        if stat == 'OK':
            if parametry[0][4].strip() != '' and parametry[0][4].strip() != '<None>':
                SQL = " SELECT a.wyznacznik_id, a.data, a.numer, a.data_przychodu, b.nazwa, a.do_zaplaty,"  # [0-5]
                SQL += "COALESCE(b.adr_kod,'')||' '||COALESCE(b.adr_miej,'')||' '||COALESCE(b.adr_ulica,'')||' '||COALESCE(b.adr_nry,''),"  # [6]
                SQL += """ rsf_podaj_numer_dokumentu_na_wydruk('""" + self.firma_id + """','["'||a.wyznacznik_id||'", "'||a.data||'", "'||a.numer||'"]','F'), """  # [7]
                SQL += " w.typ_faktury, b.kraj, k.czy_ue, a.kraj_opodatkowania,  "  # [8-11]
                SQL += "(SELECT sum(wartosc_netto-wartosc_netto_p) FROM rs_korekty_vat WHERE stawka_vat in ('zw','np') AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [12]
                SQL += "(SELECT sum(wartosc_netto-wartosc_netto_p) FROM rs_korekty_vat WHERE (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [13]
                SQL += "(SELECT sum(wartosc_netto-wartosc_netto_p) FROM rs_korekty_vat WHERE stawka_vat NOT IN ('zw') AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [14]
                SQL += "(SELECT sum(wartosc_netto-wartosc_netto_p) FROM rs_korekty_vat WHERE stawka_vat='00' AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [15]
                SQL += "(SELECT sum(wartosc_netto-wartosc_netto_p) FROM rs_korekty_vat WHERE stawka_vat='05' AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [16]
                SQL += "(SELECT sum(wartosc_vat-wartosc_vat_p) FROM rs_korekty_vat WHERE stawka_vat='05' AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [17]
                SQL += "(SELECT sum(wartosc_netto-wartosc_netto_p) FROM rs_korekty_vat WHERE stawka_vat in ('07','08') AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [18]
                SQL += "(SELECT sum(wartosc_vat-wartosc_vat_p) FROM rs_korekty_vat WHERE stawka_vat in ('07','08') AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [19]
                SQL += "(SELECT sum(wartosc_netto-wartosc_netto_p) FROM rs_korekty_vat WHERE stawka_vat in ('22','23') AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [20]
                SQL += "(SELECT sum(wartosc_vat-wartosc_vat_p) FROM rs_korekty_vat WHERE stawka_vat in ('22','23') AND (firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer) ),"  # [21]
                #WS 2020-11-24
                # SQL += " '111', CASE WHEN b.kraj = 'PL' THEN REPLACE(COALESCE(b.nip,'BRAK'), ' ', '') ELSE (SELECT kraj FROM rs_kraje kr WHERE kr.kraj = b.kraj AND kr.czy_ue) || REPLACE(b.nip, ' ', '') END AS nip, "
                SQL += " '111', REPLACE(COALESCE(b.nip,'BRAK'), ' ', '') AS nip, "  # [22] [23] zaslepka dla symblu rejestru z dowodow ksiegowych
                SQL += " COALESCE(a.ozn_sprzedaz,''), COALESCE(a.ozn_trans,''), 'K', '',"  # [24-27]
                SQL += "(SELECT STRING_AGG(kod_gtu,';') FROM rs_korekty_pozycje WHERE firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer AND kod_gtu IS NOT NULL),"  # [28]
                SQL += "a.czy_mpp, nie_podlega_vat, termin_platnosci_ulga, data_zaplaty_ulga, sprzedaz_uslug,"  # [29-33]  #WS 2020-09-09 - nie_podlega_vat  #WS 2022-01-19 - termin_platnosci_ulga, data_zaplaty_ulga  #WS 2023-05-05 - sprzedaz_uslug
                SQL += "NULL, NULL, NULL, NULL, NULL,"  # [34-38]  #WS 2024-01-22 - zalepka na klucz dowodu
                SQL += "(SELECT BOOL_AND(stawka_vat='00') FROM rs_korekty_vat WHERE firma_id=a.firma_id AND wyznacznik_id=a.wyznacznik_id AND data=a.data AND numer=a.numer)"  # [39]  #WS 2024-02-27
                SQL += " FROM rs_korekty a  INNER JOIN rs_kontrahenci b ON (a.firma_id=b.firma_id AND a.platnik_id=b.kontrahent_id) "
                SQL += " INNER JOIN rs_wyznaczniki w ON (a.firma_id=w.firma_id AND a.wyznacznik_id=w.wyznacznik_id) "
                SQL += " INNER JOIN rs_kraje k ON (b.kraj=k.kraj ) "
                SQL += parametry[0][4].replace('$firma', self.firma_id).replace('$$data_od$$', DataOdKwar).replace(
                    '$$data_do$$', DataDoKwar)

                # print 'SQL pozycje KOREKTY:'
                # print SQL
                rsglobal.ZapiszDoLogow('JPK SQL KOREKTY KWAR:' + str(SQL), 1)
                stat, SprzedazKwarKorekty, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
                if stat != 'OK':
                    stat = "Bd pobierania wierszy dla korekt: "+stat
                    dane.append(stat)
                else:
                    if SprzedazKwar is None:
                        SprzedazKwar = SprzedazKwarKorekty
                    else:
                        SprzedazKwar += SprzedazKwarKorekty

        # Ew. sprzeday z dowodow ksiegowych
        if stat == 'OK':
            if parametry[0][6].strip() != '' and parametry[0][6].strip() != '<None>':
                #WS 2023-04-27 - pobranie danych kontrahenta z rs_dowody_ksiegowe_kontrahent do pl 4, 6, 9, 10, 11, 23; LEFT OUTER zamiast INNER JOIN na rs_kontrahenci i rs_kraje
                SQL = " SELECT '', a.data, a.numer, a.data_rejestru, CASE WHEN b.firma_id IS NOT NULL THEN COALESCE(b.nazwa,'BRAK') WHEN kontr.firma_id IS NOT NULL THEN COALESCE(kontr.nazwa,'BRAK') END, ROUND(a.kwota_ma,2),"  # [0-5]
                #WS 2023-04-27
                SQL += "CASE WHEN b.firma_id IS NOT NULL THEN COALESCE(b.adr_kod,'')||' '||COALESCE(b.adr_miej,'')||' '||COALESCE(b.adr_ulica,'')||' '||COALESCE(b.adr_nry,'') "  # [6]
                SQL += "WHEN kontr.firma_id IS NOT NULL THEN COALESCE(kontr.adr_kod,'')||' '||COALESCE(kontr.adr_miej,'')||' '||COALESCE(kontr.adr_ulica,'')||' '||COALESCE(kontr.adr_nry,'') END,"  # [6]
                # SQL += "COALESCE(b.adr_kod,'')||' '||COALESCE(b.adr_miej,'')||' '||COALESCE(b.adr_ulica,'')||' '||COALESCE(b.adr_nry,''),"  # [6]
                SQL += " a.symbol_dok,  "  # [7]
                SQL += " 'KR', CASE WHEN b.firma_id IS NOT NULL THEN b.kraj WHEN kontr.firma_id IS NOT NULL THEN kontr.kraj END, "  # [8-9]
                SQL += " CASE WHEN b.firma_id IS NOT NULL THEN kb.czy_ue WHEN kontr.firma_id IS NOT NULL THEN kkontr.czy_ue END, "  # [10]
                SQL += " CASE WHEN b.firma_id IS NOT NULL THEN b.kraj WHEN kontr.firma_id IS NOT NULL THEN kontr.kraj END,  "  # [11]
                SQL += "(SELECT ROUND(sum(kwota_ma),2) FROM rs_dowody_ksiegowe_pozycje WHERE stawka_vat_ma in ('zw','np') AND (firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer) ),"  # [12]
                SQL += "(SELECT ROUND(sum(kwota_ma),2) FROM rs_dowody_ksiegowe_pozycje WHERE (firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer) ),"  # [13]
                SQL += "(SELECT ROUND(sum(kwota_ma),2) FROM rs_dowody_ksiegowe_pozycje WHERE stawka_vat_ma NOT IN ('zw') AND (firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer) ),"  # [14]
                SQL += "(SELECT ROUND(sum(kwota_ma),2) FROM rs_dowody_ksiegowe_pozycje WHERE stawka_vat_ma='00' AND (firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer) ),"  # [15]
                SQL += "(SELECT ROUND(sum(kwota_ma),2) FROM rs_dowody_ksiegowe_pozycje WHERE stawka_vat_ma='05' AND (firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer) ),"  # [16]
                SQL += "(SELECT ROUND(sum(kwota_vat_ma),2) FROM rs_dowody_ksiegowe_pozycje WHERE stawka_vat_ma='05' AND (firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer) ),"  # [17]
                SQL += "(SELECT ROUND(sum(kwota_ma),2) FROM rs_dowody_ksiegowe_pozycje WHERE stawka_vat_ma IN ('07','08') AND (firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer) ),"  # [18]
                SQL += "(SELECT ROUND(sum(kwota_vat_ma),2) FROM rs_dowody_ksiegowe_pozycje WHERE stawka_vat_ma IN ('07','08') AND (firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer) ),"  # [19]
                SQL += "(SELECT ROUND(sum(kwota_ma),2) FROM rs_dowody_ksiegowe_pozycje WHERE stawka_vat_ma IN ('22','23') AND (firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer) ),"  # [20]
                SQL += "(SELECT ROUND(sum(kwota_vat_ma),2) FROM rs_dowody_ksiegowe_pozycje WHERE stawka_vat_ma IN ('22','23') AND (firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer) ),"  # [21]
                #WS 2020-11-24
                # SQL += " a.rejestr, CASE WHEN b.kraj = 'PL' THEN REPLACE(COALESCE(b.nip,'BRAK'), ' ', '') ELSE (SELECT kraj FROM rs_kraje kr WHERE kr.kraj = b.kraj AND kr.czy_ue) || REPLACE(b.nip, ' ', '') END AS nip, "
                SQL += " a.rejestr, CASE WHEN b.firma_id IS NOT NULL THEN REPLACE(COALESCE(b.nip,'BRAK'), ' ', '') WHEN kontr.firma_id IS NOT NULL THEN REPLACE(COALESCE(kontr.nip,'BRAK'), ' ', '') END AS nip, "  # [22] [23]
                SQL += " COALESCE(a.ozn_sprzedaz,''), COALESCE(a.ozn_trans,''), 'DK', a.kwota_wn,"  # [24-27]
                SQL += "(SELECT STRING_AGG(kod_gtu,';') FROM rs_dowody_ksiegowe_pozycje WHERE firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer AND kod_gtu IS NOT NULL),"  # [28]
                SQL += "a.czy_mpp, NULL, NULL, NULL, a.sprzedaz_uslug, "  # [29-33]  #WS 2024-02-23 - sprzedaz_uslug
                SQL += "a.dowod_id, a.nazwa_id, a.data, a.numer,"  # [34-37]  #WS 2024-01-22 - klucz dowodu do przekazania w przypadku braku kontrahenta
                SQL += "(SELECT BOOL_AND(stawka_vat_ma IN ('00','np')) FROM rs_dowody_ksiegowe_pozycje WHERE firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer ),"  # [38]  #WS 2024-02-23
                SQL += "(SELECT BOOL_AND(stawka_vat_ma='00') FROM rs_dowody_ksiegowe_pozycje WHERE firma_id=a.firma_id AND okres_id=a.okres_id AND dowod_id=a.dowod_id AND nazwa_id=a.nazwa_id AND data=a.data AND numer=a.numer )"  # [39]  #WS 2024-02-27
                SQL += " FROM rs_dowody_ksiegowe a  LEFT OUTER JOIN rs_kontrahenci b ON (a.firma_id=b.firma_id AND a.kontrahent_id=b.kontrahent_id) "
                SQL += " LEFT OUTER JOIN rs_dowody_ksiegowe_kontrahent kontr ON (a.firma_id=kontr.firma_id AND a.okres_id=kontr.okres_id AND a.dowod_id=kontr.dowod_id AND a.nazwa_id=kontr.nazwa_id AND a.data=kontr.data AND a.numer=kontr.numer) "  #WS 2023-04-27
                SQL += " LEFT OUTER JOIN rs_kraje kb ON (b.kraj=kb.kraj ) "
                SQL += " LEFT OUTER JOIN rs_kraje kkontr ON (kontr.kraj=kkontr.kraj ) "  #WS 2023-04-27
                SQL += parametry[0][6].replace('$firma', self.firma_id).replace('$$data_od$$', DataOdKwar).replace(
                    '$$data_do$$', DataDoKwar)
                rsglobal.ZapiszDoLogow('JPK SQL DOW.SPRZEDAZ KWAR:' + str(SQL), 1)
                stat, SprzedazKwarDowody, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
                if stat != 'OK':
                    stat = "Bd pobierania wierszy dla dowodow: "+stat
                    dane.append(stat)
                else:
                    if SprzedazKwar is None:
                        SprzedazKwar = SprzedazKwarDowody
                    else:
                        SprzedazKwar += SprzedazKwarDowody

        # Rejestr zakupu (wykomentowany kod to jest funkjca pobierajca rajestr zakupu w ANAKONDZIE) - NIE USUWA!!!
        if stat == 'OK':
            #WS 2023-04-27 - pobranie danych z rs_dowody_ksiegowe_kontrahent do pl 0, 1, 2, 12
            SQL = " SELECT CASE WHEN k.kontrahent_id IS NOT NULL THEN TRIM(COALESCE(k.nazwa,'')) "  # [0]
            SQL += " WHEN dkk.firma_id IS NOT NULL THEN TRIM(COALESCE(dkk.nazwa,'')) ELSE '' END,"  # [0]
            SQL += " CASE WHEN k.kontrahent_id IS NOT NULL THEN COALESCE(k.adr_kod,'')||' '||TRIM(COALESCE(k.adr_miej,''))||', '||TRIM(COALESCE(k.adr_ulica,''))||' '||TRIM(COALESCE(k.adr_nry,'')) "  # [1]
            SQL += " WHEN dkk.firma_id IS NOT NULL THEN COALESCE(dkk.adr_kod,'')||' '||TRIM(COALESCE(dkk.adr_miej,''))||', '||TRIM(COALESCE(dkk.adr_ulica,''))||' '||TRIM(COALESCE(dkk.adr_nry,'')) ELSE 'BRAK' END,"  # [1]
            #AB 04.12.18r
            #WS 2020-11-24
            # SQL += "  CASE WHEN k.kontrahent_id IS NOT NULL THEN TRIM(CASE WHEN k.kraj = 'PL' THEN "  # [2]
            # SQL += "REPLACE(COALESCE(k.nip,'BRAK'), ' ', '') ELSE (SELECT kraj FROM rs_kraje kr WHERE kr.kraj = k.kraj AND "
            # SQL += "kr.czy_ue) || REPLACE(k.nip, ' ', '') END) ELSE 'BRAK' END AS nipp, "
            SQL += "  REPLACE(COALESCE(CASE WHEN k.kontrahent_id IS NOT NULL THEN k.nip WHEN dkk.firma_id IS NOT NULL THEN dkk.nip END,''), ' ', '')  AS nipp, "  # [2]
            SQL += " dk.symbol_dok, "  # [3]
            SQL += "dk.data_wplywu, "  # [4]
            #  [5]
            SQL += " CASE WHEN dk.rejestr IN " + str(
                tuple(parametry[0][0].encode('utf-8').split(','))) + u" THEN SUM(COALESCE(dkp.kwota_wn,0)) ELSE 0 END, "
            #  [6]
            SQL += " CASE WHEN dk.rejestr IN " + str(
                tuple(parametry[0][0].encode('utf-8').split(','))) + u" THEN SUM(COALESCE(dkp.kwota_vat_wn,0)) ELSE 0 END, "
            #        SQL+=" CASE WHEN dk.rejestr='"+parametry[0][0].encode('utf-8')+"' THEN SUM(COALESCE(dkp.kwota_wn,0)) ELSE 0 END, "
            #        SQL+=" CASE WHEN dk.rejestr='"+parametry[0][0].encode('utf-8')+"' THEN SUM(COALESCE(dkp.kwota_vat_wn,0)) ELSE 0 END, "
            #  [7]
            SQL += u" CASE WHEN dk.rejestr not in" + str(
                tuple(parametry[0][1].encode('utf-8').split(','))) + u"  THEN SUM(COALESCE(dkp.kwota_wn,0)) ELSE 0 END, "
            #  [8]
            SQL += u" CASE WHEN dk.rejestr not in " + str(
                tuple(parametry[0][1].encode('utf-8').split(','))) + u" THEN SUM(COALESCE(dkp.kwota_vat_wn,0)) ELSE 0 END, "
            #  [9]
            SQL += " CASE WHEN dk.rejestr IN " + str(tuple(
                parametry[0][2].encode('utf-8').split(','))) + u"  THEN SUM(COALESCE(dkp.kwota_vat_wn,0)) ELSE 0 END, "
            #  [10]
            SQL += u" CASE WHEN dk.rejestr IN " + str(
                tuple(parametry[0][3].encode('utf-8').split(','))) + u" THEN SUM(COALESCE(dkp.kwota_vat_wn,0)) ELSE 0 END, "
            SQL += "dk.data_wyst, COALESCE(CASE WHEN k.kontrahent_id IS NOT NULL THEN k.kraj WHEN dkk.firma_id IS NOT NULL THEN dkk.kraj END,''), COALESCE(dk.ozn_nabycie,'') as ozn_nabycie, dk.ozn_trans, dk.kwota_ma, dk.import,"  # [11-16]
            SQL += "dk.dowod_id,dk.nazwa_id,dk.data,dk.numer,dk.czy_mpp"  # [17-21] #WS 2020-05-25
            # tabele i filtry wydruku
            SQL += " FROM rs_dowody_ksiegowe dk INNER JOIN rs_dowody_ksiegowe_pozycje dkp USING (firma_id,dowod_id,nazwa_id,data,numer)"
            SQL += " LEFT OUTER JOIN rs_kontrahenci k ON (dk.firma_id=k.firma_id AND dk.kontrahent_id=k.kontrahent_id)"
            SQL += " LEFT OUTER JOIN rs_dowody_ksiegowe_kontrahent dkk "
            SQL += " ON (dk.firma_id=dkk.firma_id AND dk.dowod_id=dkk.dowod_id AND dk.nazwa_id=dkk.nazwa_id AND dk.data=dkk.data AND dk.numer=dkk.numer) "
            SQL += " INNER JOIN rs_rejestry_vat rv ON (dk.firma_id=rv.firma_id AND dk.rejestr=rv.kod) "
            if parametry[0][8].strip() != '' and parametry[0][8].strip() != '<None>':
                SQL += parametry[0][8].replace('$firma', self.firma_id).replace('$$data_od$$', DataOdKwar).replace(
                    '$$data_do$$', DataDoKwar)
            else:
                SQL += " WHERE dk.firma_id='" + self.firma_id + "' AND COALESCE(dk.data_rejestru,dk.data)>='" + str(
                    DataOdKwar) + "' AND COALESCE(dk.data_rejestru,dk.data)<='" + str(DataDoKwar) + "' "
                SQL += " AND NOT dkp.stawka_vat_wn IS NULL AND NOT dkp.sposob_rozl IN (' ','N') AND NOT dk.rejestr IS NULL AND rv.typ='Z' "
                SQL += " AND NOT dkp.zamkniecie_kregu AND dkp.pobierac_na_rejestr AND NOT dkp.stawka_vat_wn IS NULL "  #JR 2019-06-07 11504
                SQL += " AND dk.stan='Z' "  #WS 2023-07-20
                # SQL+=" GROUP BY dk.data_wplywu,dk.symbol_dok,dk.dowod_id,dk.nazwa_id,dk.data,dk.numer,dk.rejestr,dkp.kwota_wn, dkp.kwota_vat_wn,"
                SQL += " GROUP BY dk.data_wplywu,dk.symbol_dok,dk.dowod_id,dk.nazwa_id,dk.data,dk.numer,dk.rejestr,"
                SQL += "ozn_nabycie,dk.ozn_trans,dk.kwota_ma,dk.import,dk.czy_mpp,"
                #AB 04.12.18r
                SQL += " k.kontrahent_id,k.nazwa,nipp,k.adr_kod,k.adr_miej,k.adr_ulica,k.adr_nry,k.kraj,"
                SQL += "dkk.firma_id,dkk.nazwa,dkk.nip,dkk.adr_kod,dkk.adr_miej,dkk.adr_ulica,dkk.adr_nry,dkk.kraj,dk.data_wyst,dk.data_rejestru"
                SQL += " ORDER BY dk.data,dk.dowod_id,dk.numer;"

            rsglobal.ZapiszDoLogow('JPK SQL DOW.ZAKUP KWAR:' + str(SQL), 1)
            stat, ZakupKwarPrzed, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
            if stat != 'OK':
                stat = "Bd pobierania danych do rejestru zakupu: " + stat
                dane.append(stat)
            # Dopiero tutaj bdziemy filrowa wynik z zapytania bo bdzie szybciej ni w SQL
            if parametry[0][8].strip() != '' and parametry[0][8].strip() != '<None>':  #WS 2020-12-03
                for b in ZakupKwarPrzed:
                    # if (b[4] >=DataOd and b[4] <=DataDo and b[4].strip()!=''):
                    #WS 2020-12-03
                    if (b[4] <= DataDoKwar and b[4].strip() != '') or b[4] == 'None':
                    # if b[4] <= DataDo and b[4].strip() != '':
                        ZakupKwar.append(b)
                    else:
                        rsglobal.ZapiszDoLogow('Odrzucony:' + str(b), 1)
            else:
                # rezygnacja z odrzucenia
                ZakupWiersze=ZakupKwarPrzed

            # wyn2= LStawek

if Funkcja == 1 and stat == 'OK':
    REJESTR = {}
    REJESTR['Podmiot1'] = rssql.RSTableRowset2(dane_podmiot1, wyn1_podmiot1)[0]

    if len(SprzedazWiersze) > 0:
        REJESTR['SprzedazWiersze'] = SprzedazWiersze
    if len(ZakupWiersze) > 0:
        REJESTR['ZakupWiersze'] = ZakupWiersze
    #WS 2020-09-07
    if len(SprzedazKwar) > 0:
        REJESTR['SprzedazKwar'] = SprzedazKwar
    #WS 2020-09-07
    if len(ZakupKwar) > 0:
        REJESTR['ZakupKwar'] = ZakupKwar

    Parametry = parametry[0]  #WS 2020-10-23

    #WS 2020-09-07
    rejestr = EwidencjaZakupuSprzedazyZDeklaracja(DataOd, DataDo, Cel, warunkisprzedazy, Deklaracja, Email, Wariant, Parametry, DeklZaKwar)
    # rejestr = EwidencjaZakupuSprzedazyZDeklaracja(DataOd, DataDo, Cel, warunkisprzedazy, Deklaracja, Email)
    stat, dane, wyn1 = rejestr.ZapiszEwidencje(REJESTR)
