# -*- coding: iso8859_2 -*-
#-------------------------------------------------------------------------------
# Name:        fs_jpk_ksiegi_rachunkowe.py
# Purpose:     pobranie danych do JPL
# Author:      Marcin Natkaski
# Created:     2016/06/09
# RCS-ID:      $Id: fs_jpk_ksiegi_rachunkowe.py $
# Copyright:   (c) 2016 RecordSystem
# Licence:     GNU GPL
#-------------------------------------------------------------------------------
# FUNKCJE
# 1 - Pobranie danych do JPK - Ksiegi Rachunkowe
# 2018-08-17 Aleksander Bargie - 11072 - JPK_KR - uzupenienie
# 2020-01-14 Wojciech Skowron   - 11814 - Pobieranie danych firmy z centrali
# 2020-01-23 Wojciech Skowron   - 11775 - Optymalizacja
# 2020-02-05 Wojciech Skowron   - 11775 - Poprawki
# 2022-10-17 Wojciech Skowron   - 13210 - Poprawki - kodowanie KodOperatora
# 2023-02-23 Aleksander Bargie - 13362 - bdy w generowaniu jpk_kr


class KsiegiRachunkowe:
    #print 'wchodzi w ksiegi rachunkowe'
    def __init__(self,data_od,data_do,konto_od,konto_do,cel_zlozenia):
        from funkcje import fs_xml
        self.ksiegi = fs_xml.RSToXml()
        self.cel_zlozenia = cel_zlozenia
        self.data_od = data_od
        self.data_do = data_do
        self.konto_od = konto_od
        self.konto_do = konto_do

    def ZapiszKsiegi(self,ksiegi):
        #print 'wchodzi w zapiszksiegu'
        from funkcje import fs_jpk_pom

        import datetime
        import os
        import rsglobal
        import time

        dane = ksiegi
        nagl = ksiegi['dane_naglowka'][0]
        self.nazwa_dluga=nagl[0].encode('utf-8')
        self.nip=nagl[1].encode('utf-8')
        self.regon=nagl[2].encode('utf-8')
        self.kraj=nagl[3].encode('utf-8')
        self.woje=nagl[4].encode('utf-8')
        self.powiat=nagl[5].encode('utf-8')
        self.gmina=nagl[6].encode('utf-8')
        self.ulica=nagl[7].encode('utf-8')
        self.nrdom=nagl[8].encode('utf-8')
        self.miejscowosc=nagl[9].encode('utf-8')
        self.kod_pocztowy=nagl[10].encode('utf-8')
        self.poczta=nagl[11].encode('utf-8')
        self.urzad=nagl[12].encode('utf-8')
        # dane do ZOiS
        self.pozycje_zois = ksiegi['zois']
        #dane do dziennika
        self.pozycje_dziennik = ksiegi['dziennik'][0]
        self.pozycje_dziennik_sumy = ksiegi['dziennik'][2][0]

        self.ksiegi.Inicjuj("JPK",2)
        self.ksiegi.WstawWRoot("Naglowek")
        self.ksiegi.WstawTekst('<KodFormularza wersjaSchemy="1-0" kodSystemowy="JPK_KR (1)">JPK_KR</KodFormularza>')
        self.ksiegi.WstawNowyWZWartoscia("Naglowek","WariantFormularza",'1')
        self.ksiegi.WstawNowyWZWartoscia("Naglowek","CelZlozenia",self.cel_zlozenia.encode('utf-8'))
        self.ksiegi.WstawNowyWZWartoscia("Naglowek","DataWytworzeniaJPK",time.strftime("%Y-%m-%d")+'T'+datetime.datetime.now().time().isoformat()[:-7]+'Z')
        self.ksiegi.WstawNowyWZWartoscia("Naglowek","DataOd",self.data_od)
        self.ksiegi.WstawNowyWZWartoscia("Naglowek","DataDo",self.data_do)
        self.ksiegi.WstawNowyWZWartoscia("Naglowek","DomyslnyKodWaluty","PLN")
        self.ksiegi.WstawNowyWZWartoscia("Naglowek","KodUrzedu",self.urzad)
        self.ksiegi.Zamknij("Naglowek")
        self.ksiegi.WstawWRoot("Podmiot1")
        self.ksiegi.WstawNowyW("Podmiot1","IdentyfikatorPodmiotu")
        self.ksiegi.WstawNowyWZWartoscia("IdentyfikatorPodmiotu","tns:NIP",self.nip.strip())
        self.ksiegi.WstawNowyWZWartoscia("IdentyfikatorPodmiotu","tns:PelnaNazwa",self.nazwa_dluga)
        self.ksiegi.WstawNowyWZWartoscia("IdentyfikatorPodmiotu","tns:REGON",self.regon.strip())
        self.ksiegi.Zamknij("IdentyfikatorPodmiotu")
        self.ksiegi.WstawNowyW("Podmiot1","AdresPodmiotu")
        self.ksiegi.WstawNowyWZWartoscia("AdresPodmiotu","tns:KodKraju",self.kraj)
        self.ksiegi.WstawNowyWZWartoscia("AdresPodmiotu","tns:Wojewodztwo",self.woje)
        self.ksiegi.WstawNowyWZWartoscia("AdresPodmiotu","tns:Powiat",self.powiat)
        self.ksiegi.WstawNowyWZWartoscia("AdresPodmiotu","tns:Gmina",self.gmina)
        self.ksiegi.WstawNowyWZWartoscia("AdresPodmiotu","tns:Ulica",self.ulica)
        self.ksiegi.WstawNowyWZWartoscia("AdresPodmiotu","tns:NrDomu",self.nrdom)
        self.ksiegi.WstawNowyWZWartoscia("AdresPodmiotu","tns:NrLokalu",'0')
        self.ksiegi.WstawNowyWZWartoscia("AdresPodmiotu","tns:Miejscowosc",self.miejscowosc)
        self.ksiegi.WstawNowyWZWartoscia("AdresPodmiotu","tns:KodPocztowy",self.kod_pocztowy)
        self.ksiegi.WstawNowyWZWartoscia("AdresPodmiotu","tns:Poczta",self.poczta)
        self.ksiegi.Zamknij("AdresPodmiotu")
        self.ksiegi.Zamknij("Podmiot1")




        # self.ksiegi.WstawWRoot("ZOiS")
        # self.ksiegi.WstawAtrybutW("ZOiS",'typ','G')

        #Zaczynamy wtawia ZOiS
        for i in self.pozycje_zois:
            typ_konta = ''
            #TUTAJ TRZEBA WSTAWIA W PTLI !!!!111ONEONE - for i in lista_obrotow
            self.ksiegi.WstawTekst('<ZOiS typ="G" >')


            if i[0]:
                self.ksiegi.WstawNowyWZWartoscia("ZOiS","KodKonta",str(i[0]).encode('utf-8'))
            else:
                self.ksiegi.WstawNowyWZWartoscia("ZOiS","KodKonta",'BRAK')
            if i[1]:
                self.ksiegi.WstawNowyWZWartoscia("ZOiS","OpisKonta",i[1].encode('utf-8'))
            else:
                self.ksiegi.WstawNowyWZWartoscia("ZOiS","OpisKonta",'BRAK')
            #WS 2020-02-05
            # if i[12] in ('W','N','R'):
            #     typ_konta+=' Wchodzi do bilansu,'
            # if i[2]==True:
            #     typ_konta+=' Pozabilansowe,'
            # if i[13]==True:
            #     typ_konta+=' Rozrachunkowe,'
            if i[12] in ('W'):
                typ_konta += ' bilansowe,'
            if i[12] in ('N'):
                typ_konta += ' wynikowe,'
            if i[12] in ('R'):
                typ_konta += ' rozliczeniowe,'
            if i[2] == True:
                typ_konta += ' pozabilansowe,'
            self.ksiegi.WstawNowyWZWartoscia("ZOiS","TypKonta",typ_konta[:-1])
            self.ksiegi.WstawNowyWZWartoscia("ZOiS","KodZespolu",str(i[14][0][0]).encode('utf-8'))
            self.ksiegi.WstawNowyWZWartoscia("ZOiS","OpisZespolu",str(i[14][0][1]).encode('utf-8'))
            self.ksiegi.WstawNowyWZWartoscia("ZOiS","KodKategorii",str(i[16][0][0]).encode('utf-8'))
            self.ksiegi.WstawNowyWZWartoscia("ZOiS","OpisKategorii",str(i[16][0][1]).encode('utf-8'))
            if len(i[15])>0:
                if i[15][0][0]:
                    self.ksiegi.WstawNowyWZWartoscia("ZOiS","KodPodkategorii",str(i[15][0][0]).encode('utf-8'))
                self.ksiegi.WstawNowyWZWartoscia("ZOiS","OpisPodkategorii",str(i[15][0][1]).encode('utf-8'))
            self.ksiegi.WstawNowyWZWartoscia("ZOiS","BilansOtwarciaWinien",str(i[3]))
            self.ksiegi.WstawNowyWZWartoscia("ZOiS","BilansOtwarciaMa",str(i[4]))
            self.ksiegi.WstawNowyWZWartoscia("ZOiS","ObrotyWinien",str(i[5]))
            self.ksiegi.WstawNowyWZWartoscia("ZOiS","ObrotyMa",str(i[6]))
            self.ksiegi.WstawNowyWZWartoscia("ZOiS","ObrotyWinienNarast",str(i[7]))
            self.ksiegi.WstawNowyWZWartoscia("ZOiS","ObrotyMaNarast",str(i[8]))
            self.ksiegi.WstawNowyWZWartoscia("ZOiS","SaldoWinien",str(i[9]))
            self.ksiegi.WstawNowyWZWartoscia("ZOiS","SaldoMa",str(i[10]))
            self.ksiegi.Zamknij("ZOiS")

        #Zaczynamy wstawiac dziennik
        #self.ksiegi.WstawTekst('<Dziennik typ="G" />')

        # self.ksiegi.WstawWRoot("Dziennik")
        # self.ksiegi.WstawAtrybutW("Dziennik",'typ','G')


        suma_dziennika = 0
        for j in self.pozycje_dziennik:
            self.ksiegi.WstawTekst('<Dziennik typ="G" >')
            self.ksiegi.WstawNowyWZWartoscia("Dziennik","LpZapisuDziennika",str(int(j[0])))
            self.ksiegi.WstawNowyWZWartoscia("Dziennik","NrZapisuDziennika",str(int(j[0])))
            self.ksiegi.WstawNowyWZWartoscia("Dziennik","OpisDziennika",'BRAK')
            self.ksiegi.WstawNowyWZWartoscia("Dziennik","NrDowoduKsiegowego",(str(j[1])+'/'+str(j[2])+'/'+str(j[3])+'/'+str(j[4])).encode('utf-8'))
            self.ksiegi.WstawNowyWZWartoscia("Dziennik","RodzajDowodu",str(j[16]).encode('utf-8'))
            if j[6]=='None':
                self.ksiegi.WstawNowyWZWartoscia("Dziennik","DataOperacji",j[5])
                self.ksiegi.WstawNowyWZWartoscia("Dziennik","DataDowodu",j[5])
            else:
                self.ksiegi.WstawNowyWZWartoscia("Dziennik","DataOperacji",j[6])
                self.ksiegi.WstawNowyWZWartoscia("Dziennik","DataDowodu",j[6])
            self.ksiegi.WstawNowyWZWartoscia("Dziennik","DataKsiegowania",j[5])
            self.ksiegi.WstawNowyWZWartoscia("Dziennik","KodOperatora",(j[12]+' '+j[13]).encode('utf-8'))  #WS 2022-10-17
            self.ksiegi.WstawNowyWZWartoscia("Dziennik","OpisOperacji",j[7].replace('<',',').replace('>',',').encode('utf-8'))
            if j[8]!=0:
                self.ksiegi.WstawNowyWZWartoscia("Dziennik","DziennikKwotaOperacji",str(j[8]))
                suma_dziennika = suma_dziennika + j[8]
            else:
                self.ksiegi.WstawNowyWZWartoscia("Dziennik","DziennikKwotaOperacji",str(j[9]))
                suma_dziennika = suma_dziennika + j[9]
            self.ksiegi.Zamknij("Dziennik")
        # dane kontrolne dziennika
        self.ksiegi.WstawWRoot("DziennikCtrl")
        self.ksiegi.WstawNowyWZWartoscia("DziennikCtrl","LiczbaWierszyDziennika",str(len(self.pozycje_dziennik)))
        self.ksiegi.WstawNowyWZWartoscia("DziennikCtrl","SumaKwotOperacji",str(suma_dziennika))
        self.ksiegi.Zamknij("DziennikCtrl")
        # dane konto - zapis
        #self.ksiegi.WstawTekst('<KontoZapis typ="G" />')
        # self.ksiegi.WstawWRoot("KontoZapis")
        # self.ksiegi.WstawAtrybutW("KontoZapis",'typ','G')
        # tutaj mgbym to wstawia t sam ptl co dziennik (bo dane z tego samego rda), ale ze wzgldu
        # na mechanik wstawiania danych do gazek pozwol sobie zrobi dwie ptle nawet kosztem wydajnoci algorytmu
        #print self.pozycje_dziennik_sumy
        for l in self.pozycje_dziennik:
            self.ksiegi.WstawTekst('<KontoZapis typ="G" >')
            #WS 2020-02-05
            self.ksiegi.WstawNowyWZWartoscia("KontoZapis", "LpZapisu", str(int(l[0])))
            self.ksiegi.WstawNowyWZWartoscia("KontoZapis", "NrZapisu", str(int(l[0])))
            # self.ksiegi.WstawNowyWZWartoscia("KontoZapis","LpZapisu",str(int(j[0])))
            # self.ksiegi.WstawNowyWZWartoscia("KontoZapis","NrZapisu",str(int(j[0])))
            if str(l[8])!='0.0':
                self.ksiegi.WstawNowyWZWartoscia("KontoZapis","KodKontaWinien",str(l[10]).encode('utf-8'))
                self.ksiegi.WstawNowyWZWartoscia("KontoZapis","KwotaWinien",str(l[8]))
                self.ksiegi.WstawNowyWZWartoscia("KontoZapis","KwotaWinienWaluta",str(l[8]))
                self.ksiegi.WstawNowyWZWartoscia("KontoZapis","KodWalutyWinien","PLN")
                self.ksiegi.WstawNowyWZWartoscia("KontoZapis","OpisZapisuWinien",str(l[7]).replace('<',',').replace('>',',').encode('utf-8'))

                self.ksiegi.WstawNowyWZWartoscia("KontoZapis","KodKontaMa",0)
                self.ksiegi.WstawNowyWZWartoscia("KontoZapis","KwotaMa",0)
                self.ksiegi.WstawNowyWZWartoscia("KontoZapis","KwotaMaWaluta",0)
                self.ksiegi.WstawNowyWZWartoscia("KontoZapis","KodWalutyMa","PLN")
                self.ksiegi.WstawNowyWZWartoscia("KontoZapis","OpisZapisuMa",0)

            else:
                self.ksiegi.WstawNowyWZWartoscia("KontoZapis","KodKontaWinien",0)
                self.ksiegi.WstawNowyWZWartoscia("KontoZapis","KwotaWinien",0)
                self.ksiegi.WstawNowyWZWartoscia("KontoZapis","KwotaWinienWaluta",0)
                self.ksiegi.WstawNowyWZWartoscia("KontoZapis","KodWalutyWinien","PLN")
                self.ksiegi.WstawNowyWZWartoscia("KontoZapis","OpisZapisuWinien",0)



                self.ksiegi.WstawNowyWZWartoscia("KontoZapis","KodKontaMa",str(l[10]).encode('utf-8'))
                self.ksiegi.WstawNowyWZWartoscia("KontoZapis","KwotaMa",str(l[9]))
                self.ksiegi.WstawNowyWZWartoscia("KontoZapis","KwotaMaWaluta",str(l[9]))
                self.ksiegi.WstawNowyWZWartoscia("KontoZapis","KodWalutyMa","PLN")
                self.ksiegi.WstawNowyWZWartoscia("KontoZapis","OpisZapisuMa",str(l[7]).replace('<',',').replace('>',',').encode('utf-8'))




            self.ksiegi.Zamknij("KontoZapis")
        self.ksiegi.WstawWRoot("KontoZapisCtrl")
        self.ksiegi.WstawNowyWZWartoscia("KontoZapisCtrl","LiczbaWierszyKontoZapisj",str(len(self.pozycje_dziennik)))
        self.ksiegi.WstawNowyWZWartoscia("KontoZapisCtrl","SumaWinien",str(self.pozycje_dziennik_sumy[0]))
        self.ksiegi.WstawNowyWZWartoscia("KontoZapisCtrl","SumaMa",str(self.pozycje_dziennik_sumy[1]))
        self.ksiegi.Zamknij("KontoZapisCtrl")
        self.ksiegi.Zamknij("JPK")



        plik = 'jpk_kr_'+time.strftime("%Y-%m-%d_%H_%M_%S", time.gmtime())+'.xml'



        self.ksiegi.ZapiszPlik(plik,False,'<?xml version="1.0" encoding="utf-8"?>')
        file = open(plik,'r')

        tresc = file.read().replace('&','&amp;').replace('<JPK>', '<JPK xmlns="http://jpk.mf.gov.pl/wzor/2016/03/09/03091/"  xmlns:tns="http://crd.gov.pl/xml/schematy/dziedzinowe/mf/2016/01/25/eD/DefinicjeTypy/">')

        file.close()

        file = open(plik,'w')

        file.write(tresc)
        file.close()

        #WS 2020-01-23
        stat = fs_jpk_pom.SprawdzJpkKR(plik, 'xsd/jpk_kr.xsd')
        # stat = fs_jpk_pom.SprawdzJpk(plik,'xsd/jpk_kr.xsd')

        if stat=='OK':
            #WS 2020-01-23
            from funkcje import fs_pliki_global

            wynik = fs_pliki_global.PodajTrescPliku(plik)
            if wynik is None:
                stat = "Wystpi bd podczas przygotowania do przesania pliku xml"
                return stat,'BLAD'
            else:
                os.remove(plik)
                return stat,wynik

        else:
            os.system('mv '+plik+' '+'xsd/Blad_'+plik)
            return stat,'BLAD'



from funkcje import fs_rozrachunki_pom


stat='OK'
dane=[]
wyn1 = []
wyn2 = []
zois = []
dziennik = []
#AB 21.08.18r
Zespoly = {'0': 'Aktywa trwae',
           '1': 'rodki pienine, rachunki bankowe oraz inne krtkoterminowe aktywa finansowe',
           '2': 'Rozrachunki i roszczenia',
           '3': 'Materiay i towary',
           '4': 'Koszty wedug rodzajw i ich rozliczenie',
           '5': 'Koszty wedug typw dziaalnoci i ich rozliczenie',
           '6': 'Produkty i rozliczenia midzyokresowe',
           '7': 'Przychody i koszty zwizane z ich osigniciem',
           '8': 'Kapitay (fundusze) wasne, fundusze specjalne i wynik finansowy',
           '9': 'Pozostae'}


if not 'Funkcja' in klucze:
    stat='Brak podanej funkcji w kluczach'

if stat=='OK':
    Funkcja = klucze['Funkcja']

    SlPar={'DataOd':[1,],'DataDo':[1,],'KontoOd':[1,],'KontoDo':[1,],'Cel':[1,]}

    Lista=rsglobal.SprawdzParametryWKluczach(SlPar,Funkcja,klucze)
    if Lista!='':
        stat='Uwaga : wywoanie funkcji fs_jpk z numerem '+str(Funkcja)+' bez parametrw: '+Lista

    if Funkcja in [1,]:
        DataOd = rsglobal.Dekoduj(klucze['DataOd'])
        DataDo = rsglobal.Dekoduj(klucze['DataDo'])
        KontoOd = rsglobal.Dekoduj(klucze['KontoOd'])
        KontoDo = rsglobal.Dekoduj(klucze['KontoDo'])
        Cel = rsglobal.Dekoduj(klucze['Cel'])

    Rok = DataOd[:4]
    Mc = DataDo[5:7]
    Poziom = 99999
    NieDrukowacNazw = False
    ObrotyZBO = True
#-------------------------------------------------------------------------------
# POBIERANIE DANYCH  (5)
#-------------------------------------------------------------------------------
if Funkcja == 1 and stat=='OK':

    #Pobieramy dane do nagwka
    #WS 2019-01-14
    SQL = "SELECT CASE WHEN (nazwa1 IS NULL OR TRIM(nazwa1) = '') AND (nazwa2 IS NULL OR TRIM(nazwa2) = '') THEN nazwa ELSE COALESCE(nazwa1,'') || ' ' || COALESCE(nazwa2,'') END,"
    # SQL = "SELECT nazwa1,"
    SQL+= "nip,regon,adr_kraj,adr_wojew,adr_powiat,adr_gmina,adr_ulica,adr_nry,adr_miej,adr_kod,adr_poczta,urzad_skarbowy_id "
    SQL+= " FROM rs_oddzialy "
    #WS 2020-01-14
    SQL+=" WHERE firma_id='"+self.firma_id+"' AND centrala"
    # SQL+=" WHERE firma_id='"+self.firma_id+"' AND oddzial_id='01'"
    stat,dane_naglowka,dump,dump=self.PolSQL.Execute(SQL,3)
    if stat!='OK':
        dane.append("Bd pobierania danych oddziau "+stat)
    #WS 2020-01-14
    elif len(dane_naglowka) == 0:
        stat = "Brak oddziau oznaczonego jako centrala"
        dane.append(stat)

# pobieramy pozycje do ZOiS
if Funkcja == 1 and stat=='OK':
    #sprawdzenie poprawnosci kluczy
    #sprawdzenie roku
    try:
        if int(Rok)<1950 or int(Rok)>2100:
            stat='Niepoprawny rok wykonania raportu'
    except:
        stat='Niepoprawny rok wykonania raportu: '+str(Rok)
    #sprawdzenie miesiaca
    if stat=='OK':
        if Mc not in ['01','02','03','04','05','06','07','08','09','10','11','12']:
            stat='Niepoprawny miesic wykonania raportu: '+str(Mc)

    if stat=='OK':
        Data = Rok+'-'+Mc+'-'+'01'
        sql = "SELECT data_od, data_do FROM rs_definicje_okresow"
        sql+=" WHERE firma_id = '"+self.firma_id+"' AND okres_id = '"+self.okres_id+"';"
        stat,d,dump,dump = self.PolSQL.Execute(sql, 2)
        if stat != 'OK':
            stat = 'Bd pobrania danych okresu ksigowego : '+stat
            dane.append(stat)
        else:
            if len(d) == 1:
                if DataOd < str(d[0][0]):
                    stat = 'Data od jest wczeniejsza od daty pocztku okresu ksigowego'
                    dane.append(stat)

                if DataDo > str(d[0][1]):
                    stat = 'Data do jest pniejsza ni data koca okresu okresu ksigowego'
                    dane.append(stat)

            else:
                stat = 'Brak zdefiniowanego okresu ksigowego '+str(self.okres_id)
                dane.append(stat)

    if stat=='OK':
        if not '_' in KontoOd:
            if KontoOd=='':
                stat='Nie podano konta od'

            if stat=='OK':
                if KontoDo=='':
                    stat='Nie podano konta do'

    if stat =='OK':
        if '_' in KontoOd:
            FiltrKont=" AND konto LIKE '"+KontoOd+"%' "
        else:
            FiltrKont=" AND (((konto>='"+KontoOd+"') AND (konto<='"+KontoDo+"')) OR (konto LIKE '"+KontoDo+"%')) "

    if stat =='OK':
        #pobranie listy kont z tabeli rs_stany_kont
        SQL="SELECT DISTINCT konto FROM rs_stany_kont WHERE firma_id='"+self.firma_id+"'"
        SQL+=" AND okres_id='"+self.okres_id+"' "+FiltrKont
        SQL+=" ORDER BY konto ASC;"
        stat,DaneKont,dump,dump = self.PolSQL.Execute(SQL,3)
        if stat != 'OK':
            stat='Bdy pobierania list kont do analizy : '+stat
    if stat=='OK':
        if len(DaneKont)==0:
            stat="Brak kont mieszczcych si w zakresie o podanych parametrach"

    if stat=='OK':
        #przerabiamy DaneKont na list stringw (konta)
        ListaKont = []
        for n in DaneKont:
            ListaKont.extend(n)
        #WYKOMENTOWANA CZʦ JEST DO TWORZENIA SYNTETYK Z KONT ANALITYCZNYCH - MOE BDZIE POTRZEBNA
        # #ListaKont = DaneKont
        # #stworzenie pelnej list kont w oparciu o pobrana liste
        # for konto in DaneKont:
        #     Konto=rsglobal.Dekoduj(konto[0])
        #     if not Konto in ListaKont:
        #         Poz=Konto.count("-")
        #
        #         if Poz <= Poziom:
        #             if not Konto in ListaKont:
        #                 ListaKont.append(Konto)
        #
        #         bufor = Konto
        #         indeks = 0
        #         while indeks != -1:
        #             indeks = bufor.rfind("-")
        #             if indeks != -1:
        #                 bufor = bufor[0:indeks]
        #                 Poz = Poz-1
        #             if not bufor in ListaKont:
        #                  if (bufor >= KontoOd or '_' in KontoOd) and Poz <=Poziom:
        #                     ListaKont.append(bufor)
        #pobranie sum kont dla wszystkich kont (i potem wyszukiwanie)

        #WS 2020-01-23
        SQL = """SELECT konto,
                 sum(CASE WHEN miesiac = '00' THEN kwota_wn ELSE NULL END) AS bown, sum(CASE WHEN miesiac = '00' THEN kwota_ma ELSE NULL END) AS boma,
                 sum(CASE WHEN miesiac = '{miesiac}' AND rok = '{rok}' THEN kwota_wn ELSE NULL END) AS obrwn,sum(CASE WHEN miesiac = '{miesiac}' AND rok = '{rok}' THEN kwota_ma ELSE NULL END) AS obrma,
                 sum(CASE WHEN miesiac != '00' AND (rok||miesiac) <= ('{rok}{miesiac}') THEN kwota_wn ELSE NULL END) AS narwn,sum(CASE WHEN miesiac != '00' AND (rok||miesiac) <= ('{rok}{miesiac}') THEN kwota_ma else NULL END) AS narma,
                 sum(CASE WHEN miesiac = '{miesiac}' AND rok = '{rok}' THEN saldo_wn ELSE NULL END) AS salwn,sum(CASE WHEN miesiac = '{miesiac}'  AND rok = '{rok}' THEN saldo_ma ELSE NULL END) AS salma
                 FROM rs_stany_kont
                 WHERE firma_id = '{firma_id}' and okres_id = '{okres_id}' {FiltrKont}
                 GROUP BY konto ORDER BY konto;"""
        SQL = SQL.format(firma_id  = self.firma_id,
                         okres_id  = self.okres_id,
                         miesiac   = Mc,
                         rok       = Rok,
                         FiltrKont = FiltrKont)
        stat, DaneStanyKont, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
        if stat != 'OK':
            stat = 'Bd pobierania stanw kont : ' + stat

        #Suma bilansu otwarcia
    #     SQL="SELECT konto,sum(kwota_wn) AS bown,sum(kwota_ma) AS boma FROM rs_stany_kont "
    #     SQL+=" WHERE firma_id='"+self.firma_id+"' AND okres_id='"+self.okres_id+"' "
    #     SQL+=" AND miesiac = '00' "+FiltrKont
    #     SQL+=" GROUP BY konto ORDER BY konto;"
    #     stat,DaneBO,dump,dump = self.PolSQL.Execute(SQL,3)
    #
    # if stat=='OK':
    #     #Obroty w miesiacu
    #     SQL="SELECT konto,sum(kwota_wn) AS obrwn,sum(kwota_ma) AS obrma FROM rs_stany_kont "
    #     SQL+=" WHERE firma_id='"+self.firma_id+"' AND okres_id='"+self.okres_id+"' AND rok = '"+Rok+"' "
    #     SQL+=" AND miesiac = '"+Mc+"' "+FiltrKont
    #     SQL+=" GROUP BY konto ORDER BY konto;"
    #     stat,DaneOBR,dump,dump = self.PolSQL.Execute(SQL,3)
    #
    # if stat=='OK':
    #     #Suma obrotow narastajaca
    #     SQL="SELECT konto,sum(kwota_wn) AS narwn,sum(kwota_ma) AS narma FROM rs_stany_kont "
    #     SQL+=" WHERE firma_id='"+self.firma_id+"' AND okres_id='"+self.okres_id+"' AND (rok||miesiac) <= ('"+Rok+"'||'"+Mc+"') "
    #     SQL+=" AND miesiac != '00' "+FiltrKont
    #     SQL+=" GROUP BY konto ORDER BY konto;"
    #     stat,DaneNAR,dump,dump = self.PolSQL.Execute(SQL,3)
    #
    # if stat=='OK':
    #     #Saldo
    #     SQL="SELECT konto,sum(saldo_wn) AS salwn,sum(saldo_ma) AS salma FROM rs_stany_kont "
    #     SQL+=" WHERE firma_id='"+self.firma_id+"' AND okres_id='"+self.okres_id+"' AND rok = '"+Rok+"' "
    #     SQL+=" AND miesiac = '"+Mc+"'"+FiltrKont
    #     SQL+=" GROUP BY konto ORDER BY konto;"
    #     stat,DaneSAL,dump,dump = self.PolSQL.Execute(SQL,3)

    if stat=='OK':
       # sprawdzenie istnienia BO
        stat,Wyn=fs_rozrachunki_pom.PodajOkresyDlaRozrachunkow(self,self.okres_id)
        if stat != 'OK':
            stat = 'Bd pobierania okresw dla rozrachunkw : '+stat

    if stat == 'OK':
        LOkresow=Wyn[0]
        DataGr=Wyn[1]
        IstniejeBO=(LOkresow.find(',')==-1)

    if stat=='OK':
        #pobranie danych dla kolejnych kont
        for Konto in ListaKont:
            if stat=='OK':
                KontoS = Konto[0:3]
                KontoA = ""
                if len(Konto)>3:
                    KontoA = Konto[4:len(Konto)]
                KontoA = KontoA.strip()

                #Pobranie opisow
                SQL="SELECT rsf_podaj_parametry_konta('"+self.firma_id+"','"+self.okres_id+"','"+Konto+"');"
                stat,DaneParK,dump,dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)

            if stat != 'OK':
                stat = "Bd wykonania rsf_podaj_parametry_konta: " + stat
            elif len(DaneParK)==0:
                stat="Bdy w definicji konta : "+KontoS+"-"+KontoA
            #WS 2020-02-05
            else:
                Nazwa=""
                KontoPodzielone = Konto.split('-')
                KontoTmp = ''

                for el in KontoPodzielone:
                    KontoTmp = '-'.join([KontoTmp, el]) if KontoTmp != '' else el
                    SQL = "SELECT rsf_podaj_nazwe_konta('" + self.firma_id + "','" + self.okres_id + "','" + KontoTmp + "');"
                    stat, DaneNazwaK, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
                    if stat != 'OK':
                        stat = "Bd wykonania rsf_podaj_nazwe_konta: "+stat
                        break
                    elif len(DaneNazwaK) == 0:
                        stat = "Bdy w definicji konta : "+Konto
                        break
                    else:
                        Nazwa = ', '.join([Nazwa, DaneNazwaK[0][0].strip()]) if Nazwa != '' else DaneNazwaK[0][0].strip()

            #pobranie danych zespou kont syntetycznych
            #AB 21.08.18r
            if stat == 'OK':
                DaneKontSynt = [[Konto[:1], Zespoly[Konto[:1]]]]
                # SQL="SELECT DISTINCT kontos_id, nazwa FROM rs_plan_kont WHERE firma_id='"+self.firma_id+"' AND okres_id='"+self.okres_id+"' AND "
                # SQL+=" kontos_id = substring('"+Konto+"' from 1 for 1) AND trim(konto_id)='';"
                # stat,DaneKontSynt,dump,dump = self.PolSQL.Execute(SQL,3)
                # if stat!='OK':
                #     stat='Bdy w pobieraniu opisu grupy kont systetycznych.'
                #
                # elif len(DaneKontSynt)==0:
                #     DaneKontSynt=[['Brak konta','Brak nazwy']]


            #pobranie danych grupy kont analitycznych (podkategoria w JPK)
            if stat=='OK':
                a = '-'.join(Konto.split('-')[1:])
                SQL="SELECT DISTINCT konto_id, nazwa FROM rs_plan_kont WHERE firma_id='"+self.firma_id+"' AND okres_id='"+self.okres_id+"' AND "
                SQL+="kontos_id = substring('"+Konto+"' from 1 for 3) and konto_id='"+a+"';"
                stat,DaneKontAnal,dump,dump = self.PolSQL.Execute(SQL,3)
                if stat!='OK':
                    stat='Bdy w pobieraniu danych grupy kont analitycznych.'
                #AB 21.08.18r Zmieniony komunikat o bdzie i zakomentowane poniej
                # elif len(DaneKontSynt)==0:
                #     DaneKontSynt=[['Brak konta','Brak nazwy']]


            #pobranie kategorii (JPK)
            if stat=='OK':
                SQL="SELECT DISTINCT kontos_id, nazwa FROM rs_plan_kont WHERE firma_id='"+self.firma_id+"' AND okres_id='"+self.okres_id+"' AND "
                SQL+="kontos_id ='"+Konto.split('-')[0]+"' LIMIT 1;"
                stat,DaneKategorii,dump,dump = self.PolSQL.Execute(SQL,3)
                if stat!='OK':
                    stat='Bdy w pobieraniu kategorii JPK.'
                #AB 21.08.18r Zmieniony komunikat o bdzie i zakomentowane poniej
                # elif len(DaneKontSynt)==0:
                #     DaneKontSynt=[['Brak konta','Brak nazwy']]

            if stat=='OK':
                DaneKonta=eval(DaneParK[0][0])
                if NieDrukowacNazw:
                    Nazwa = '     '
                #WS 2020-02-05
                # else:
                    # Nazwa = DaneKonta[0]
                    # Nazwa = DaneKonta[1]

                Rozrachunkowe = DaneKonta[3]
                Pozabilansowe = DaneKonta[4]
                Jednostronne  = DaneKonta[7]
                Waluta = DaneKonta[5]
                DoBilansu = DaneKonta[6]

                BOWn=0
                BOMa=0
                OBRWn=0
                OBRMa=0
                NARWn=0
                NARMa=0
                SALWn=0
                SALMa=0

                #WS 2020-01-23
                for d in DaneStanyKont:
                    BO  = d[1:3]
                    OBR = d[3:5]
                    NAR = d[5:7]
                    SAL = d[7:9]

                    if (Konto==d[0]) or (d[0][0:len(Konto)+1] == Konto+'-'):
                        if BO[0] not in (None,'None',0):
                            BOWn+=rsglobal.RSRound(BO[0],2)
                        if BO[1] not in (None,'None',0):
                            BOMa+=rsglobal.RSRound(BO[1],2)

                        if OBR[0] not in (None,'None',0):
                            OBRWn+=rsglobal.RSRound(OBR[0],2)
                        if OBR[1] not in (None,'None',0):
                            OBRMa+=rsglobal.RSRound(OBR[1],2)

                        if NAR[0] not in (None,'None',0):
                            NARWn+=rsglobal.RSRound(NAR[0],2)
                        if NAR[1] not in (None,'None',0):
                            NARMa+=rsglobal.RSRound(NAR[1],2)

                        if Rozrachunkowe and IstniejeBO:
                            if SAL[0] not in (None, 'None', 0):
                                SALWn += rsglobal.RSRound(SAL[0], 2)
                            if SAL[1] not in (None, 'None', 0):
                                SALMa += rsglobal.RSRound(SAL[1], 2)

                if not Rozrachunkowe or not IstniejeBO:
                    if (BOWn + NARWn) > (BOMa + NARMa):
                        SALWn = rsglobal.RSRound(BOWn + NARWn - BOMa - NARMa, 2)
                        SALMa = 0
                    else:
                        SALWn = 0
                        SALMa = rsglobal.RSRound(BOMa + NARMa - BOWn - NARWn, 2)

                    if BOWn > BOMa:
                        BOWn = rsglobal.RSRound(BOWn - BOMa, 2)
                        BOMa = 0
                    else:
                        BOMa = rsglobal.RSRound(BOMa - BOWn, 2)
                        BOWn = 0

                if Jednostronne:
                    if BOWn>BOMa:
                        BOWn=rsglobal.RSRound(BOWn-BOMa,2)
                        BOMa=0
                    else:
                        BOMa=rsglobal.RSRound(BOMa-BOWn,2)
                        BOWn=0

                    if SALWn>SALMa:
                        SALWn=rsglobal.RSRound(SALWn-SALMa,2)
                        SALMa=0
                    else:
                        SALMa=rsglobal.RSRound(SALMa-SALWn,2)
                        SALWn=0

                # BOWn=0
                # BOMa=0
                # for BO in DaneBO:
                #     if (Konto==BO[0]) or (BO[0][0:len(Konto)+1] == Konto+'-'):
                #         if BO[1] not in (None,'None',0):
                #             BOWn+=rsglobal.RSRound(BO[1],2)
                #         if BO[2] not in (None,'None',0):
                #             BOMa+=rsglobal.RSRound(BO[2],2)
                #
                # OBRWn=0
                # OBRMa=0
                # for OBR in DaneOBR:
                #     if (Konto==OBR[0]) or (OBR[0][0:len(Konto)+1] == Konto+'-'):
                #         if OBR[1] not in (None,'None',0):
                #             OBRWn+=rsglobal.RSRound(OBR[1],2)
                #         if OBR[2] not in (None,'None',0):
                #             OBRMa+=rsglobal.RSRound(OBR[2],2)
                #
                # NARWn=0
                # NARMa=0
                # for NAR in DaneNAR:
                #     if (Konto==NAR[0]) or (NAR[0][0:len(Konto)+1] == Konto+'-'):
                #         if NAR[1] not in (None,'None',0):
                #             NARWn+=rsglobal.RSRound(NAR[1],2)
                #         if NAR[2] not in (None,'None',0):
                #             NARMa+=rsglobal.RSRound(NAR[2],2)
                #
                # if Rozrachunkowe and IstniejeBO:
                #     SALWn=0
                #     SALMa=0
                #     for SAL in DaneSAL:
                #         if (Konto==SAL[0]) or (SAL[0][0:len(Konto)+1] == Konto+'-'):
                #             if SAL[1] not in (None,'None',0):
                #                 SALWn+=rsglobal.RSRound(SAL[1],2)
                #             if SAL[2] not in (None,'None',0):
                #                 SALMa+=rsglobal.RSRound(SAL[2],2)
                # else:
                #     if (BOWn+NARWn) > (BOMa+NARMa):
                #         SALWn = rsglobal.RSRound(BOWn+NARWn-BOMa-NARMa,2)
                #         SALMa = 0
                #     else:
                #         SALWn = 0
                #         SALMa = rsglobal.RSRound(BOMa+NARMa-BOWn-NARWn,2)
                #
                #     if BOWn>BOMa:
                #         BOWn=rsglobal.RSRound(BOWn-BOMa,2)
                #         BOMa=0
                #     else:
                #         BOMa=rsglobal.RSRound(BOMa-BOWn,2)
                #         BOWn=0
                #
                # if Jednostronne:
                #     if BOWn>BOMa:
                #         BOWn=rsglobal.RSRound(BOWn-BOMa,2)
                #         BOMa=0
                #     else:
                #         BOMa=rsglobal.RSRound(BOMa-BOWn,2)
                #         BOWn=0
                #
                #     if SALWn>SALMa:
                #         SALWn=rsglobal.RSRound(SALWn-SALMa,2)
                #         SALMa=0
                #     else:
                #         SALMa=rsglobal.RSRound(SALMa-SALWn,2)
                #         SALWn=0

                #obsuga obrotw z BO dziaa tylko dla zestawienia bez miesicznych
                # BO jest dodawane do sumy narastajco po uwzgldnieniu sald jednostronnych
                # if Fun==5:
                #     if ObrotyZBO:
                #         NARWn=NARWn+BOWn
                #         NARMa=NARMa+BOMa

                if (BOWn != 0) or (BOMa != 0) or (OBRWn != 0) or (OBRMa != 0) or (NARWn != 0) or (NARMa != 0) or (SALWn != 0) or (SALMa != 0):
                    zois.append([Konto,Nazwa,Pozabilansowe,BOWn,BOMa,OBRWn,OBRMa,NARWn,NARMa,SALWn,SALMa,Waluta,DoBilansu,Rozrachunkowe,DaneKontSynt,DaneKontAnal,DaneKategorii])



#pobieranie danych do dziennika

if Funkcja==1 and stat=='OK':
    if '_' in KontoOd:
        FiltrKont=" AND o.konto LIKE '"+KontoOd+"%' "
    else:
        FiltrKont=" AND (((o.konto>='"+KontoOd+"') AND (o.konto<='"+KontoDo+"')) OR (o.konto LIKE '"+KontoDo+"%')) "


    #pobranie danych z operacji i danych operatora
    #WS 2020-01-23
    SQL = """SELECT podzap.nr_dziennika,
                    podzap.dowod_id,
                    podzap.nr_dowodu,
                    podzap.nr_poz,
                    podzap.nr_ppoz,
                    podzap.data,
                    podzap.data_wyst,
                    podzap.tresc,
                    podzap.kwota_wn,
                    podzap.kwota_ma,
                    podzap.konto,
                    podzap.konto_prz,
                    substring(u.imie from 1 for 1) as imie,
                    u.nazwisko,
                    NULL,
                    podzap.nazwa_id,
                    m.nazwa
             FROM (
                 SELECT CASE WHEN o.dziennik IS NOT NULL THEN o.dziennik ELSE 0 END as nr_dziennika,
                        o.dowod_id,
                        cast(o.numer as text) as nr_dowodu,
                        cast(o.nr_poz as text) as nr_poz,
                        cast(o.nr_ppoz as text) as nr_ppoz,
                        o.data,
                        o.data_wyst,
                        CASE WHEN (TRIM(o.tresc) = '' OR o.tresc IS NULL) THEN '.' ELSE o.tresc END AS tresc,  --#AB 23.02.23r
                        o.kwota_wn,
                        o.kwota_ma,
                        o.konto,
                        o.konto_prz,
                        o.z_op_id,
                        o.firma_id,
                        o.okres_id,
                        o.nazwa_id
                 FROM rs_operacje o
                 WHERE o.firma_id = '{firma_id}'
                       and o.okres_id = '{okres_id}'
                       and o.data >= '{DataOd}'
                       and o.data <= '{DataDo}'
                       and o.dziennik is not null
                       {FiltrKont}
             ) as podzap
             LEFT JOIN rs_operatorzy u ON
                 (podzap.z_op_id = u.operator_id)
             INNER JOIN rs_wzory_dekretacji m ON
                 (podzap.firma_id = m.firma_id
                 and podzap.okres_id = m.okres_id
                 and podzap.dowod_id = m.dowod_id
                 and podzap.nazwa_id = m.nazwa_id)
             ORDER BY podzap.nr_dziennika;"""
    SQL = SQL.format(firma_id  = self.firma_id,
                     okres_id  = self.okres_id,
                     DataOd    = DataOd,
                     DataDo    = DataDo,
                     FiltrKont = FiltrKont)
    rsglobal.ZapiszDoLogow(SQL)
    stat, daneO, naglO, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
    # Pola =" CASE WHEN o.dziennik IS NOT NULL THEN o.dziennik ELSE 0 END as nr_dziennika, o.dowod_id,  CAST(o.numer AS TEXT) as nr_dowodu, CAST(o.nr_poz AS TEXT) as nr_poz, CAST(o.nr_ppoz AS TEXT) as nr_ppoz"
    # Pola+=", o.data, o.data_wyst, o.tresc, o.kwota_wn, o.kwota_ma, o.konto, o.konto_prz, SUBSTRING(u.imie FROM 1 FOR 1) as imie , u.nazwisko, p.pozabilansowe,o.nazwa_id, m.nazwa  "
    # From ="rs_operacje o LEFT OUTER JOIN rs_operatorzy u on(o.z_op_id=u.operator_id)"
    # From+=" LEFT OUTER JOIN rs_plan_kont p on (o.firma_id=p.firma_id AND o.okres_id=p.okres_id AND (STRING_TO_ARRAY(o.konto,'-'))[1] = p.kontos_id)"
    # From+=" INNER JOIN rs_wzory_dekretacji m ON (o.firma_id=m.firma_id AND o.okres_id=m.okres_id AND o.dowod_id=m.dowod_id AND o.nazwa_id=m.nazwa_id )"
    # Where=" o.firma_id='"+self.firma_id+"' AND o.okres_id='"+self.okres_id+"' AND o.data>='"+DataOd+"' AND o.data<='"+DataDo+"' AND o.dziennik IS NOT NULL"
    # Where+=FiltrKont
    # GroupBy=" o.dziennik, o.dowod_id,o.numer,o.nr_poz,o.nr_ppoz,o.data,o.data_wyst,o.tresc,o.kwota_wn, o.kwota_ma,o.konto,o.konto_prz,u.imie, u.nazwisko,p.pozabilansowe,o.nazwa_id, m.nazwa "
    # stat,daneO,naglO,dump=self.PolSQL.Execute("SELECT "+Pola+" FROM "+From+" WHERE "+Where+" GROUP BY "+GroupBy+" ORDER BY o.dziennik;",3)
    if stat=='OK':
        if len(daneO)==0:
            stat='Brak danych dziennika do tworzenia JPK'
            dane.append(stat)
    #WS 2020-01-23
    else:
        stat = 'Bd pobierania danych dziennika: '+stat
        dane.append(stat)

    if stat=="OK":
        if '_' in KontoOd:
            FiltrKont=" AND konto LIKE '"+KontoOd+"%' "
        else:
            FiltrKont=" AND (((konto>='"+KontoOd+"') AND (konto<='"+KontoDo+"')) OR (konto LIKE '"+KontoDo+"%')) "
        #pobieranie sum kwota_wn, kwota_ma za okres_id
        SQL="SELECT COALESCE(SUM(kwota_wn),0) AS KwotaWn, COALESCE(SUM(kwota_ma),0) AS KwotaMa "
        SQL+=" FROM rs_operacje "
        SQL+=" WHERE firma_id='"+self.firma_id+"' AND okres_id='"+self.okres_id+"' AND data>='"+DataOd+"' AND data<='"+DataDo+"' "
        SQL+=FiltrKont
        SQL+=" AND dziennik IS NOT NULL"
        stat,daneS,naglS,dump=self.PolSQL.Execute(SQL,3)
        if stat != 'OK':
            stat='Bdy w pobieraniu sumy kont  :'+stat
            dane.append(stat)

    if stat=='OK':
        #pobranie danych do obliczenia liczby wierszy
        stat,daneW,naglW,dump=self.PolSQL.Execute("SELECT COUNT(*)::NUMERIC as ile_wierszy FROM rs_operacje WHERE firma_id='"+self.firma_id+"' AND okres_id='"+self.okres_id+"' AND data<'"+DataOd+"' AND SUBSTRING(data::TEXT from 6 for 2) in ('01','02','03','04','05','06','07','08','09','10','11','12') AND dziennik IS NOT NULL GROUP BY SUBSTRING(data::TEXT from 6 for 2) ORDER BY SUBSTRING(data::TEXT from 6 for 2);",3)
        #rsglobal.ZapiszDoLogow(daneW,1)
    if stat=='OK':
        dziennik=[daneO,naglO,daneS,naglS,[],[],daneW,naglW]


if stat=='OK':
    KSIEGI = {}
    KSIEGI['dane_naglowka'] = dane_naglowka
    KSIEGI['zois'] = zois
    KSIEGI['dziennik'] = dziennik
    ksiegi = KsiegiRachunkowe(DataOd, DataDo,KontoOd,KontoDo, Cel)
    stat,wynik = ksiegi.ZapiszKsiegi(KSIEGI)
    if stat=='OK':
        dane = [wynik]
    else:
        dane.append(stat)
