#!/usr/bin/python
# -*- coding: iso8859_2 -*-
#-------------------------------------------------------------------------------
# Name:			fs_jpk_pom.py
# Purpose:		Funkcja pomocnicza dla JPK
# Author:		Marcin Natkaski
# Created:		2016/09/20
# RCS-ID:		$Id: fs_jpk_pom.py $
# Copyright:	(c) 2016 RecordSystem
#-------------------------------------------------------------------------------

# 2020-01-20 Wojciech Skowron   - 11815 - Funkcja podmiany zastrzeonych znakw dla xml (&,<,>,',")
# 2020-01-23 Wojciech Skowron   - 11775 - Zmiana sposobu walidacji xml dla JPK_KR

def SprawdzJpk(xml,xsd):
    import os
    stat = 'OK'

    # pokamy bardziej przyjazny komunikat gdyby nie byo tutaj lxml
    try:
        from lxml import etree
    except ImportError:
        stat='Brak biblioteki lxml'
        return stat

    # adowanie pliku xsd
    schema_doc = etree.parse(xsd)
    schema = etree.XMLSchema(schema_doc)

    if not os.path.exists(xml):
        stat='Brak pliku xml'
        return stat

    # adowanie pliku inicjalizacji sesji (???)
    try:
        doc = etree.parse(xml)
    except etree.XMLSyntaxError, exc:
        stat='EE Plik %s nie jest prawidowym dokumentem xml' % xml
        return stat

    # sprawdzanie struktury xml
    if schema.validate(doc):
        stat ='OK'
    else:
        stat=''
        for e in schema.error_log:
            stat+='  {0.level_name} at {0.filename}:{0.line}:{0.column}\n  {0.message}\n'.format(e)

    return stat

#WS 2020-01-23
def SprawdzJpkKR(xml,xsd):
    import os
    import rsglobal
    import subprocess
    stat = 'OK'

    if not os.path.exists(xsd):
        stat='Brak pliku xsd'
        return stat

    if not os.path.exists(xml):
        stat='Brak pliku xml'
        return stat

    s1 = subprocess.Popen(["xmlstarlet"], shell=True)
    s1.communicate()
    wynik = s1.returncode
    if wynik == 127:
        try:
            from lxml import etree
        except ImportError:
            stat = 'Brak biblioteki lxml'
            return stat

        if os.path.getsize(xml) > 105000000:
            stat = 'Rozmiar pliku xml przekracza 100 MB, wymagana instalacja pakietu XMLStarlet. Skontaktuj si z administratorem.'
            return stat

        schema_doc = etree.parse(xsd)
        schema = etree.XMLSchema(schema_doc)

        try:
            doc = etree.parse(xml)
        except etree.XMLSyntaxError, exc:
            stat = 'EE Plik %s nie jest prawidowym dokumentem xml' % xml
            return stat
        except Exception, exc:
            stat = 'Bd parsowania pliku xml: '+str(exc)
            return stat

        # sprawdzanie struktury xml
        if schema.validate(doc):
            stat = 'OK'
        else:
            stat = ''
            for e in schema.error_log:
                stat += '  {0.level_name} at {0.filename}:{0.line}:{0.column}\n  {0.message}\n'.format(e)
    else:
        s2 = subprocess.Popen(["xmlstarlet val --xsd {} -e {} ".format(xsd,xml)],
                              stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
        stdout, stderr = s2.communicate()
        # wynik = s2.returncode - nie zwraca prawidowego wyniku
        if '- valid' in stdout and not stderr:
            stat = 'OK'
        else:
            stat = stderr.replace(xml+":", "") + stdout

    return stat

#WS 2020-01-20
def ZamienZastrzezoneZnaki(wartosc):
    try:  # kodowanie wartosci jesli to mozliwe plus podmiana znakow specjalnych
        wartosc = str(wartosc).strip().encode('utf-8').replace('&', '&amp;') \
            .replace('<', '&lt;') \
            .replace('>', '&gt;') \
            .replace("'", '&apos;') \
            .replace('"', '&quot;')
    except:
        pass

    return wartosc
