# -*- coding: iso8859_2 -*-
# -------------------------------------------------------------------------------
# Name:        fs_jpk_ewidencja_faktur_vat.py
# Purpose:     pobranie danych do JPK
# Author:      Marcin Natkaski
# Created:     2016/06/09
# RCS-ID:      $Id: fs_jpk_wyciagi_bankowe.py $
# Copyright:   (c) 2016 RecordSystem
# Licence:     GNU GPL
# -------------------------------------------------------------------------------

# 2020-01-14 Wojciech Skowron - (11815) Pobieranie danych firmy z centrali

# -------------------------------------------------------------------------------
# -----Tworzenie XML
# -------------------------------------------------------------------------------

class WyciagiBankowe:
    def __init__(self, data_od, data_do, cel, bank):
        from funkcje import fs_xml
        self.wyciagi_jpk = fs_xml.RSToXml()
        self.data_od = data_od
        self.data_do = data_do
        self.cel = cel
        self.bank = bank.split(',')

    # -------------------------------------------------------------------------------

    def ZapiszWyciag(self,wyciagi):
        import time
        import tempfile
        import datetime
        from funkcje import fs_jpk_pom
        import os

        dane_naglowka = wyciagi['dane_naglowka']
        dane = wyciagi


        self.nip = dane_naglowka[0][0]
        self.nazwa = dane_naglowka[0][1]
        self.kraj = dane_naglowka[0][2]
        self.woje = dane_naglowka[0][3]
        self.powiat = dane_naglowka[0][4]
        self.gmina = dane_naglowka[0][5]
        self.ulica = dane_naglowka[0][6]
        self.numer = dane_naglowka[0][7]
        self.kod = dane_naglowka[0][8]
        self.poczta = dane_naglowka[0][9]
        self.miejsc = dane_naglowka[0][10]
        self.urzad = dane_naglowka[0][11]
        self.regon = dane_naglowka[0][12]
        self.adres = self.kod + ' ' + self.miejsc + ' ' + self.ulica + ' ' + self.numer

        # ----SKLADAMY JPK__NAGLOWEK
        self.wyciagi_jpk.Inicjuj("JPK",2)
        self.wyciagi_jpk.WstawWRoot("Naglowek")
        self.wyciagi_jpk.WstawTekst('<KodFormularza wersjaSchemy="1-0" kodSystemowy="JPK_WB (1)">JPK_WB</KodFormularza>')
        self.wyciagi_jpk.WstawNowyWZWartoscia("Naglowek", "WariantFormularza", '1')
        self.wyciagi_jpk.WstawNowyWZWartoscia("Naglowek", "CelZlozenia", self.cel.encode('utf-8'))
        self.wyciagi_jpk.WstawNowyWZWartoscia("Naglowek","DataWytworzeniaJPK",time.strftime("%Y-%m-%d")+'T'+datetime.datetime.now().time().isoformat()[:-7]+'Z')
        self.wyciagi_jpk.WstawNowyWZWartoscia("Naglowek", "DataOd", self.data_od)
        self.wyciagi_jpk.WstawNowyWZWartoscia("Naglowek", "DataDo", self.data_do)
        self.wyciagi_jpk.WstawNowyWZWartoscia("Naglowek", "DomyslnyKodWaluty", 'PLN')
        self.wyciagi_jpk.WstawNowyWZWartoscia("Naglowek", "KodUrzedu", self.urzad)
        self.wyciagi_jpk.Zamknij("Naglowek")
        self.wyciagi_jpk.WstawWRoot("Podmiot1")
        self.wyciagi_jpk.WstawNowyW("Podmiot1", "IdentyfikatorPodmiotu")
        self.wyciagi_jpk.WstawNowyWZWartoscia("IdentyfikatorPodmiotu", "tns:NIP", str(self.nip).strip())
        self.wyciagi_jpk.WstawNowyWZWartoscia("IdentyfikatorPodmiotu", "tns:PelnaNazwa", self.nazwa.encode('utf-8'))
        self.wyciagi_jpk.WstawNowyWZWartoscia("IdentyfikatorPodmiotu", "tns:REGON", str(self.regon).strip())
        self.wyciagi_jpk.Zamknij("IdentyfikatorPodmiotu")
        self.wyciagi_jpk.WstawNowyW("Podmiot1", "AdresPodmiotu")
        self.wyciagi_jpk.WstawNowyWZWartoscia("AdresPodmiotu", "tns:KodKraju", self.kraj.encode('utf-8'))
        self.wyciagi_jpk.WstawNowyWZWartoscia("AdresPodmiotu", "tns:Wojewodztwo", self.woje.encode('utf-8'))
        self.wyciagi_jpk.WstawNowyWZWartoscia("AdresPodmiotu", "tns:Powiat", self.powiat.encode('utf-8'))
        self.wyciagi_jpk.WstawNowyWZWartoscia("AdresPodmiotu", "tns:Gmina", self.gmina.encode('utf-8'))
        self.wyciagi_jpk.WstawNowyWZWartoscia("AdresPodmiotu", "tns:Ulica", self.ulica.encode('utf-8'))
        self.wyciagi_jpk.WstawNowyWZWartoscia("AdresPodmiotu", "tns:NrDomu", str(self.numer))
        self.wyciagi_jpk.WstawNowyWZWartoscia("AdresPodmiotu", "tns:NrLokalu", 0)
        self.wyciagi_jpk.WstawNowyWZWartoscia("AdresPodmiotu", "tns:Miejscowosc", self.miejsc.encode('utf-8'))
        self.wyciagi_jpk.WstawNowyWZWartoscia("AdresPodmiotu", "tns:KodPocztowy", str(self.kod))
        self.wyciagi_jpk.WstawNowyWZWartoscia("AdresPodmiotu", "tns:Poczta", self.poczta.encode('utf-8'))
        self.wyciagi_jpk.Zamknij("AdresPodmiotu")
        self.wyciagi_jpk.Zamknij("Podmiot1")

        # Wstawiamy Numer rachunku
        if dane['konto_numer']:
            self.wyciagi_jpk.WstawNowyWZWartoscia("JPK", "NumerRachunku", str(dane['konto_numer'][0][0]).replace(' ', ''))
        else:
            self.wyciagi_jpk.WstawNowyWZWartoscia("JPK", "NumerRachunku", 'BRAK')

        if dane['wiersze_ctrl'][0] > 0:
            # wstawioamy salda
            self.wyciagi_jpk.WstawWRoot("Salda")
            self.wyciagi_jpk.WstawNowyWZWartoscia("Salda", "SaldoPoczatkowe", str(dane['saldo'][0][0][0]))
            self.wyciagi_jpk.WstawNowyWZWartoscia("Salda", "SaldoKoncowe", str(dane['saldo'][1][0][0]))
            self.wyciagi_jpk.Zamknij("Salda")

            # wstawiamy wyciag wiersz
            a = 0
            poprzednie_saldo = dane['saldo'][0][0][0]
            for g in dane['wiersze']:
                a = a + 1
                self.wyciagi_jpk.WstawTekst('<WyciagWiersz typ="G">')
                self.wyciagi_jpk.WstawNowyWZWartoscia("WyciagWiersz", "NumerWiersza", str(a))
                self.wyciagi_jpk.WstawNowyWZWartoscia("WyciagWiersz", "DataOperacji", g[0])
                self.wyciagi_jpk.WstawNowyWZWartoscia("WyciagWiersz", "NazwaPodmiotu", g[1].encode('utf-8'))

                if len(g[2])==0:
                    self.wyciagi_jpk.WstawNowyWZWartoscia("WyciagWiersz", "OpisOperacji", 'BRAK')
                else:
                    self.wyciagi_jpk.WstawNowyWZWartoscia("WyciagWiersz", "OpisOperacji", g[2].encode('utf-8'))

                if g[4].strip() == 'P':
                    #self.wyciagi_jpk.WstawNowyWZWartoscia("WyciagWiersz", "KwotaOperacji", '+' + str(g[3]))
                    self.wyciagi_jpk.WstawNowyWZWartoscia("WyciagWiersz", "KwotaOperacji",str(round(g[3],2)))
                    self.wyciagi_jpk.WstawNowyWZWartoscia("WyciagWiersz", "SaldoOperacji",
                                                          str(round(poprzednie_saldo + g[3],2)))
                    poprzednie_saldo = poprzednie_saldo + g[3]
                else:
                    #self.wyciagi_jpk.WstawNowyWZWartoscia("WyciagWiersz", "KwotaOperacji", '-' + str(g[3]))
                    self.wyciagi_jpk.WstawNowyWZWartoscia("WyciagWiersz", "KwotaOperacji",str(round(g[3],2)))
                    self.wyciagi_jpk.WstawNowyWZWartoscia("WyciagWiersz", "SaldoOperacji",
                                                          str(round(poprzednie_saldo - g[3],2)))
                    poprzednie_saldo = poprzednie_saldo - g[3]
                self.wyciagi_jpk.Zamknij("WyciagWiersz")

            # wstawiamy wyciag ctrl
            self.wyciagi_jpk.WstawWRoot("WyciagCtrl")
            self.wyciagi_jpk.WstawNowyWZWartoscia("WyciagCtrl", "LiczbaWierszy", str(dane['wiersze_ctrl'][0]))
            self.wyciagi_jpk.WstawNowyWZWartoscia("WyciagCtrl", "SumaObciazen", str(dane['wiersze_ctrl'][1]))
            self.wyciagi_jpk.WstawNowyWZWartoscia("WyciagCtrl", "SumaUznan", str(dane['wiersze_ctrl'][2]))
            self.wyciagi_jpk.Zamknij("WyciagCtrl")

        else:
            self.wyciagi_jpk.WstawWRoot("WyciagCtrl")
            self.wyciagi_jpk.WstawNowyWZWartoscia("WyciagCtrl", "LiczbaWierszy", '0')
            self.wyciagi_jpk.WstawNowyWZWartoscia("WyciagCtrl", "SumaObciazen", '0')
            self.wyciagi_jpk.WstawNowyWZWartoscia("WyciagCtrl", "SumaUznan", '0')
            self.wyciagi_jpk.Zamknij("WyciagCtrl")
        self.wyciagi_jpk.Zamknij("JPK")



        plik = 'jpk_wb_'+time.strftime("%Y-%m-%d_%H_%M_%S", time.gmtime())+'.xml'

        self.wyciagi_jpk.ZapiszPlik(plik,False,'<?xml version="1.0" encoding="utf-8"?>')
        file = open(plik,'r')

        tresc = file.read().replace('&','&amp;').replace('<JPK>', '<JPK xmlns="http://jpk.mf.gov.pl/wzor/2016/03/09/03092/" xmlns:tns="http://crd.gov.pl/xml/schematy/dziedzinowe/mf/2016/01/25/eD/DefinicjeTypy/">')

        file.close()

        file = open(plik,'w')


        file.write(tresc)
        file.close()

        stat = fs_jpk_pom.SprawdzJpk(plik,'xsd/jpk_wb.xsd')

        if stat=='OK':
            os.remove(plik)
            return stat,tresc

        else:
            os.system('mv '+plik+' '+'xsd/Blad_'+plik)
            return stat,'BLAD'



# FUNKCJE
# 1 - Pobranie danych do JPK - Wycigi


stat = 'OK'
dane = []
wyn1 = []
wyn2 = []

if not 'Funkcja' in klucze:
    stat = 'Brak podanej funkcji w kluczach'

if stat == 'OK':
    Funkcja = klucze['Funkcja']

    SlPar = {'DataOd': [1, 3], 'DataDo': [1, 3], 'Bank': [1, 3], 'Cel': [1,3]}

    Lista = rsglobal.SprawdzParametryWKluczach(SlPar, Funkcja, klucze)
    if Lista <> '':
        stat = 'Uwaga : wywoanie funkcji fs_jpk_ewidencja_faktur_vat z numerem ' + str(
            Funkcja) + ' bez parametrw: ' + Lista

    if Funkcja in [1, 3]:
        DataOd = rsglobal.Dekoduj(klucze['DataOd'])
        DataDo = rsglobal.Dekoduj(klucze['DataDo'])
        Bank = rsglobal.Dekoduj(klucze['Bank'])
        Cel = rsglobal.Dekoduj(klucze['Cel'])

# -------------------------------------------------------------------------------
# Walidacja parametrw  (3)
# -------------------------------------------------------------------------------

if Funkcja == 1 and stat == 'OK':
    SQL = "SELECT kasa_bank_id FROM rs_kasy_banki WHERE firma_id='" + self.firma_id + "' AND kasa_bank_id='" + Bank + "';"
    stat, bank, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat <> 'OK':
        stat = "Bd sprawdzania poprawnoci banku "
        dane.append(stat)
    elif len(bank) == 0:
        stat='W bazie nie ma banku: '+Bank
        dane.append(stat)

    # sprawdzamy poprawnosc z okresem
    if stat == 'OK':
        sql = "SELECT data_od, data_do FROM rs_definicje_okresow"
        sql += " WHERE firma_id = '" + self.firma_id + "' AND okres_id = '" + self.okres_id + "';"
        stat, d, dump, dump = self.PolSQL.Execute(sql, 2)
        if stat <> 'OK':
            stat = 'Bd pobrania danych okresu ksigowego : ' + stat
            dane.append(stat)
        else:
            if len(d) == 1:
                if DataOd < str(d[0][0]):
                    stat = 'Data od jest wczeniejsza od daty pocztku okresu ksigowego'
                    dane.append(stat)

                if DataDo > str(d[0][1]):
                    stat = 'Data do jest pniejsza ni data koca okresu okresu ksigowego'
                    dane.append(stat)

            else:
                stat = 'Brak zdefiniowanego okresu ksigowego ' + str(self.okres_id)
                dane.append(stat)

# -------------------------------------------------------------------------------
# Pobranie danych nagwka  (2)
# -------------------------------------------------------------------------------
if Funkcja == 1 and stat == 'OK':
    # Pobieramy dane do nagwka
    #WS 2019-01-14
    SQL = " SELECT nip, CASE WHEN (nazwa1 IS NULL OR TRIM(nazwa1) = '') AND (nazwa2 IS NULL OR TRIM(nazwa2) = '') THEN nazwa ELSE COALESCE(nazwa1,'') || ' ' || COALESCE(nazwa2,'') END, "
    # SQL = " SELECT nip, nazwa"
    SQL+="adr_kraj,adr_wojew,adr_powiat,adr_gmina,adr_ulica,adr_nry,adr_kod,adr_poczta,adr_miej,urzad_skarbowy_id,regon "
    #WS 2020-01-14
    SQL += " FROM rs_oddzialy WHERE firma_id='" + self.firma_id + "' AND centrala"
    # SQL += " FROM rs_oddzialy WHERE firma_id='" + self.firma_id + "' AND oddzial_id='" + self.oddzial_id + "'"
    stat, dane_naglowka, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat <> 'OK':
        stat = "Bd pobierania danych oddziau "
        dane.append(stat)
    #WS 2020-01-14
    elif len(dane_naglowka) == 0:
        stat = "Brak oddziau oznaczonego jako centrala"
        dane.append(stat)

# -------------------------------------------------------------------------------
# Pobranie danych (1)
# -------------------------------------------------------------------------------
if Funkcja == 1 and stat == 'OK':
    # pobieramy uznania, obciazenia  i liczbe wierszy
    if stat == 'OK':
        SQL = " SELECT count(b.firma_id),sum(a.wplaty),sum(a.wyplaty) "
        SQL += " FROM rs_raporty_wyciagi a INNER JOIN rs_raporty_wyciagi_pozycje b "
        SQL += " USING (firma_id, okres_id, kasa_bank_id, data, numer) "
        SQL += " WHERE a.firma_id='" + self.firma_id + "' AND a.data>='" + DataOd + "' AND a.data<='" + DataDo + "' AND a.kasa_bank_id='" + Bank + "' AND a.stan='Z'"
        stat, wiersze_ctrl, dump, dump = self.PolSQL.Execute(SQL, 3)
        if stat <> 'OK':
            stat = "Bd danych kontrolnych wierszy wycigw bankowych "
            dane.append(stat)
        #WS 2020-01-14
        elif wiersze_ctrl[0][0] == 0:
            stat = "Brak danych do wykonania zestawienia "
            dane.append(stat)

        # ---------Pobieramy numer konta
    if stat == 'OK':
        SQL = " SELECT bk.iban FROM rs_kasy_banki kb INNER JOIN rs_konta_bankowe bk USING(firma_id,konto_id) "
        SQL+=" WHERE firma_id='" + self.firma_id + "' AND kasa_bank_id='"+Bank+"' AND okres_id='"+self.okres_id+"'"
        #print SQL
        stat, konto_numer, dump, dump = self.PolSQL.Execute(SQL, 3)
        if stat <> 'OK':
            stat = "Bd pobierania numeru konta "
            dane.append(stat)

        # ---------Pobieramy saldo pocztkowe i staldo kocowe zgodznie z datami i bankiem
    if stat == 'OK':
        # pobieramy saldo poczatkowe pierwszego raportu z listy
        SQL = " SELECT saldo_pocz FROM rs_raporty_wyciagi WHERE firma_id='" + self.firma_id + "' "
        SQL += " AND data>='" + DataOd + "' AND data<='" + DataDo + "' AND kasa_bank_id='" + Bank + "' AND stan='Z' "
        SQL += " ORDER BY data,numer LIMIT 1"
        stat, saldo_p, dump, dump = self.PolSQL.Execute(SQL, 3)
        if stat <> 'OK':
            stat = "Bd pobierania salda pocztkowego "
            dane.append(stat)

    if stat == 'OK':
        # pobieramy saldo koncowe ostatniego raportu z listy
        SQL = " SELECT saldo_konc FROM rs_raporty_wyciagi WHERE firma_id='" + self.firma_id + "' "
        SQL += " AND data>='" + DataOd + "' AND data<='" + DataDo + "' AND kasa_bank_id='" + Bank + "' AND stan='Z' "
        SQL += " ORDER BY data,numer DESC LIMIT 1"
        stat, saldo_k, dump, dump = self.PolSQL.Execute(SQL, 3)
        if stat <> 'OK':
            stat = "Bd pobierania salda pocztkowego "
            dane.append(stat)
    # czymy dane
    if stat == 'OK':
        saldo = [saldo_p, saldo_k]


    # ---------Pobieramy pozycje do wyciagu
    # ---------2018-06-07 - uzupelnienie opisu operacji o dane faktur i korekt - mk (mantis 11073)
    if stat == 'OK':
        SQL = " SELECT a.data,COALESCE(b.nazwa, 'BRAK'), COALESCE(a.tresc, (a.wyznacznik_id||'/'||a.data_fakt||'/'||a.numer_fakt)), a.kwota,a.typ FROM rs_raporty_wyciagi_pozycje a LEFT JOIN "
        SQL += " rs_kontrahenci b ON (a.firma_id=b.firma_id AND a.platnik_id=b.kontrahent_id) "
        SQL += " WHERE a.firma_id='" + self.firma_id + "'  AND a.data>='" + DataOd + "' AND a.data<='" + DataDo + "' AND a.kasa_bank_id='" + Bank + "' AND a.stan_poz='Z'"
        SQL += " AND (nr_ppoz >0 OR (nr_ppoz=0 AND (numer_fakt>0 OR trim(tresc)<>'' OR trim(konto)<>'' OR platnosc_id >0))) "
        SQL += " ORDER BY a.data,a.numer,a.nr_poz,a.nr_ppoz"
        stat, wiersze, dump, dump = self.PolSQL.Execute(SQL, 3)
        if stat <> 'OK':
            stat = "Bd pobierania wierszy wycigu bankowego. "
            dane.append(stat)

if Funkcja == 1 and stat == 'OK':
    WYCIAGI = {}

    WYCIAGI['konto_numer'] = konto_numer
    WYCIAGI['dane_naglowka'] = dane_naglowka

    if wiersze_ctrl[0][0] > 0:
        WYCIAGI['saldo'] = saldo
        WYCIAGI['wiersze_ctrl'] = wiersze_ctrl[0]
        WYCIAGI['wiersze'] = wiersze
    else:
        WYCIAGI['wiersze_ctrl'] = [0, 0]
    wyciagi = WyciagiBankowe(DataOd, DataDo, Cel, Bank)
    stat,wynik = wyciagi.ZapiszWyciag(WYCIAGI)
    if stat=='OK':
        dane = [wynik]
    else:
        dane.append(stat)

