#!/usr/bin/python
# -*- coding: iso8859_2 -*-
#-----------------------------------------------------------------------------
# Name:             fs_kolejkowanie_pom.py
# Purpose:          funkcje pomocnicze do kolejkowania wydawania towarw z magazynw
# Author:           Micha Raburski
# Created:          2013/05/23
# RCS-ID:           $Id: fs_kolejkowanie_pom.py $
# Copyright:        (c) 2013 RecordSystem
# Licence:          GNU GPL
#-----------------------------------------------------------------------------

#-------------------------------------------------------------------------------
# FUNKCJA SPRAWDZAJCA POPRAWNO WPROWADZONEGO KOLEJKOWANIA
#-------------------------------------------------------------------------------

def SprawdzPoprawnoscListyKolejkowania(Kolejkowanie):
    # Aktualnie mamy dostpne typy :
    # FIFO - First In First Out
    # LIFO - Last In First Out
    # LOFO - Lowest In First Out
    # HIFO - Highest In First Out
    # FEFO - First Expired First Out
    
    # parametry
    # Kolejkowanie - tekst do sprawdzenia poprawnoci kolejkowania
    
    # obrobienie parametru
    LKolejkowanie=Kolejkowanie.upper().replace(' ','').split(',')
    
    # sprawdzenie czy nie ma sprzecznych par w Kolejkowaniu
    if 'FIFO' in LKolejkowanie and 'LIFO' in LKolejkowanie:
        return ('Uwaga, opcje kolejkowania FIFO i LIFO s sprzeczne i nie mog wystpowa jednoczenie na jednej licie','')
    if 'LOFO' in LKolejkowanie and 'HIFO' in LKolejkowanie:
        return ('Uwaga, opcje kolejkowania LOFO i HIFO s sprzeczne i nie mog wystpowa jednoczenie na jednej licie','')
    
    Wynik=''    
    # zoenie nowego tekstu listy
    for kol in LKolejkowanie:
        if not kol in ['FIFO','LIFO','LOFO','HIFO','FEFO']:
            return ('Brak opcji '+kol+' w typach kolejkowania','')
        
        Wynik+=kol+','
        
    Wynik=Wynik.strip(',')
    
    return ('OK',Wynik)


#-------------------------------------------------------------------------------
# FUNKCJA PODAJCA ORDER BY DLA DOKUMENTU LUB PODANEGO TEKSTU
#-------------------------------------------------------------------------------

def PodajOrderByDokumentuLubTekstu(PolSQL,Firma,Dokument=None,Kolejkowanie='',PoleData='',Odwrotnie= False):
    # Aktualnie mamy dostpne typy :
    # FIFO - First In First Out
    # LIFO - Last In First Out
    # LOFO - Lowest In First Out
    # HIFO - Highest In First Out
    # FEFO - First Expired First Out
    
    # parametry
    # PolSQL - obiekt pozwalajcy na komunikacj z baz danych
    # Firma - firma dla ktrej wywoujemy funkcj
    # Dokument - dokument dla ktrego podajemy Order By   (None lub [Magazyn] lub [Magazyn,Dokument,Data,Numer])
    # Kolejkowanie - zamiast dokumentu mona poda konkretny tekst do obrobienia
    # Odwrotnie - jeli True wwczas "zostawia" w magazynie dobre kolejkowanie. Uywane np. w spisie z natury
    if PoleData=='':
        PoleData='data'
        
        SQL="SELECT wg_daty_wejscia FROM rs_firmy WHERE firma_id='"+Firma+"'"
        stat, DaneDaty, dump, dump = PolSQL.Execute(SQL, 3)
        if stat <> 'OK':
            return ('Bdy pobierania daty wejcia z firmy: '+stat,'')
            
        if len(DaneDaty) == 0:
            return ('Podana firma nie istnieje','')
            
        if DaneDaty[0][0]:
            PoleData="COALESCE(wejscie_data,data)"
    
    # jeli na dokumencie, w magazynie i firmie nic nie podano to wstawiane jest FIFO
    
    
    # jeli nie podano rcznego kolejkowania to staramy si pobra kolejkowanie z bazy danych
    if Kolejkowanie=='':
        # jeli to peny dokument
        if len(Dokument)==4:
            SQL="SELECT COALESCE(kolejkowanie,'') FROM rs_dokumenty_magazynowe "
            SQL+=" WHERE firma_id='"+Firma+"' AND magazyn_id='"+Dokument[0]+"' AND dokument_id='"+Dokument[1]+"'"
            SQL+=" AND data='"+Dokument[2]+"' AND numer="+str(Dokument[3])+';'
            stat,DaneDM,dump,dump=PolSQL.Execute(SQL,3)
            if stat<>'OK':
                return ('Bdy pobierania kolejkowania z dokumentu magazynowego: '+stat,'')
            
            if len(DaneDM)==0:
                return ('Jako parametr podano klucz nieistniejcego dokumentu','')
            
            if DaneDM[0][0]<>'':
                Kolejkowanie=DaneDM[0][0]
            
        # nie znalelimy kolejkowania na dokumencie. Sprawdzamy magazyn (moe jest podany).
        if Kolejkowanie=='':
            if len(Dokument) >= 1:
                SQL="SELECT COALESCE(kolejkowanie,'') FROM rs_magazyny "
                SQL+=" WHERE firma_id='"+Firma+"' AND magazyn_id='"+Dokument[0]+"';"
                stat,DaneM,dump,dump=PolSQL.Execute(SQL,3)
                if stat<>'OK':
                    return ('Bdy pobierania kolejkowania z magazynu: '+stat,'')
                
                if len(DaneM)==0:
                    return ('Jako parametr podano klucz nieistniejcego magazynu','')
                
                if DaneM[0][0]<>'':
                    Kolejkowanie=DaneM[0][0]
                    
        # nie znalelimy kolejkowania na magazynie. Sprawdzamy firm.
        if Kolejkowanie=='':
            SQL="SELECT COALESCE(kolejkowanie,'') FROM rs_firmy "
            SQL+=" WHERE firma_id='"+Firma+"';"
            stat,DaneF,dump,dump=PolSQL.Execute(SQL,3)
            if stat<>'OK':
                return ('Bdy pobierania kolejkowania z firmy: '+stat,'')
            
            if len(DaneF)==0:
                return ('Jako parametr podano klucz nieistniejcej firmy','')
            
            if DaneF[0][0]<>'':
                Kolejkowanie=DaneF[0][0]
            else:
                Kolejkowanie='FIFO'
       
    
    # obrobienie parametru
    LKolejkowanie=Kolejkowanie.upper().replace(' ','').split(',')
    
    # sprawdzenie czy nie ma sprzecznych par w Kolejkowaniu
    if 'FIFO' in LKolejkowanie and 'LIFO' in LKolejkowanie:
        return ('Uwaga, opcje kolejkowania FIFO i LIFO s sprzeczne i nie mog wystpowa jednoczenie na jednej licie','')
    if 'LOFO' in LKolejkowanie and 'HIFO' in LKolejkowanie:
        return ('Uwaga, opcje kolejkowania LOFO i HIFO s sprzeczne i nie mog wystpowa jednoczenie na jednej licie','')
    
    Wynik=''    
    # zoenie odpowiedniego order by
    
    PierwKol = ''
    for kol in LKolejkowanie:
        if PierwKol == '':
            PierwKol = kol
        if not kol in ['FIFO','LIFO','LOFO','HIFO','FEFO']:
            return 'Brak opcji '+kol+' w typach kolejkowania'
        if not Odwrotnie:
            # normalne kolejowanie
            if kol=='FIFO':
                Wynik+=PoleData+' ASC,'
            if kol=='LIFO':
                Wynik+=PoleData+' DESC,'
            if kol=='LOFO':
                Wynik+='cena1 ASC,'
            if kol=='HIFO':
                Wynik+='cena1 DESC,'
            if kol=='FEFO':
                Wynik+='termin_przyd ASC,'
        else:
            # kolejkowanie odwrotne
            if kol=='FIFO':
                Wynik+=PoleData+' DESC,'
            if kol=='LIFO':
                Wynik+=PoleData+' ASC,'
            if kol=='LOFO':
                Wynik+='cena1 DESC,'
            if kol=='HIFO':
                Wynik+='cena1 ASC,'
            if kol=='FEFO':
                Wynik+='termin_przyd DESC,'
    
    if len(Dokument) == 4:
        if (PierwKol in ['FIFO','FEFO'] and not Odwrotnie) or (PierwKol in ['LIFO','LOFO','HIFO'] and Odwrotnie):
            # najstarsze
            Wynik+="numer,nr_poz,nr_ppoz"
        else:
            # najmodsze
            Wynik+="numer DESC,nr_poz DESC,nr_ppoz DESC"
    Wynik=Wynik.strip(',')
    
    return ('OK',Wynik)
            
            
            
    