# -*- coding: iso8859_2 -*-
# -------------------------------------------------------------------------------
# Name:        fs_kp_angaze_dane
# Purpose:     dodawanie/edycja danych angazu
# Author:      Mariusz Rajfura
# Created:     2009/09/26
# RCS-ID:      $Id: fs_kp_angaze_dane.py $
# Copyright:   (c) 2009 Record System
# Licence:     GNU GPL
# -------------------------------------------------------------------------------

# 2012-02-27 (3951) Mariusz Rajfura     - nowe pole 'kalendarz_id'
# 2015-08-04 (9116) Marcin Natkaski    - nowe pole 'stanowisko_miejsce_id'
# 2015-09-03 (9003) Micha Raburski     - nowe pole 'lista_id'
# 2018-06-11 (11087) Aleksander Bargie - kopiowanie skadnikw specjalnych do kolejnego angau na umowie.
# 2019-10-10 (11683) Aleksander Bargie - [KP] zerowy_pit w angau
# 2021-12-21 (12777) Bartosz Pagan     - Dodatkowe checkboxy w angau dot. ulgi dla klasy redniej od 2022-01-01
# 2022-01-21 (12722) Bartosz Pagan     - Kopiowanie skadnikw dodatkowych podczas dodawania nowej umowy
# 2022-02-02 (12722) Aleksander Bargie - dokoczenie
# 2022-09-01 (13144) Aleksander Bargie - [KP] blokada zmiany etatu jeli s listy
# 2022-10-03 (13186) Aleksander Bargie - [KP] Bd dodawania nowego angau


def verification_after_update(self, WHERE, dane):
    verification_required = """SELECT
    CASE WHEN up.od_kiedy > a.od_kiedy THEN 'BL_DATA1' ELSE '' END,
    CASE WHEN (up.od_kiedy > a.do_kiedy and a.do_kiedy is not null) THEN 'BL_DATA2' ELSE '' END,
    CASE WHEN (up.do_kiedy < a.od_kiedy and up.do_kiedy is not null) THEN 'BL_DATA3' ELSE '' END,
    CASE WHEN (up.do_kiedy < a.do_kiedy and a.do_kiedy is not null and up.do_kiedy is not null)
        THEN 'BL_DATA4' ELSE '' END,
    CASE WHEN a.od_kiedy ISNULL THEN 'BR_OD_KIEDY' ELSE '' END,
    CASE WHEN a.miejsce_id ISNULL or a.miejsce_id = '' THEN 'BR_MIEJSCE' ELSE '' END,
    CASE WHEN a.komorka_id ISNULL or a.komorka_id = '' THEN 'BR_KOMORKA' ELSE '' END,
    CASE WHEN a.rodzaj_pracownika_id ISNULL or a.rodzaj_pracownika_id = '' THEN 'BR_RODZAJ_PRACOWNIKA' ELSE '' END,
    CASE WHEN a.grupa_pracownikow_id ISNULL or a.grupa_pracownikow_id = '' THEN 'BR_GRUPA_PRACOWNIKOW' ELSE '' END,
    CASE WHEN a.stanowisko_id ISNULL or a.stanowisko_id = '' THEN 'BR_STANOWISKO' ELSE '' END,
    CASE WHEN a.stanowisko_miejsce_id ISNULL or a.stanowisko_miejsce_id = '' THEN 'BR_STANOWISKO_MIEJSCE' ELSE '' END,
    CASE WHEN a.kalendarz_id ISNULL or a.kalendarz_id = '' THEN 'BR_KALENDARZ' ELSE '' END
    FROM rs_kp_angaze a
    JOIN rs_kp_umowy_o_prace up
        ON a.firma_id=up.firma_id AND a.pracownik_id=up.pracownik_id
        AND a.zatrudnienie_id = up.zatrudnienie_id AND a.umowa_id = up.umowa_id
    """
    sql = verification_required + WHERE.replace("WHERE ", "WHERE a.").replace("AND ", "AND a.").replace(";", " ")
    stat, check_field, dump, dump = self.PolSQL.Execute(sql, 3)
    if stat != 'OK':
        stat = 'Bd sprawdzenia danych po zapisie: ' + stat
        dane.append(stat)
    else:
        for field in check_field[0]:
            if field:
                dane.append(field)
                stat = field
        if len(dane) > 1:
            stat = dane
    return stat, dane


def SprNakladanieDat(self, Fun, Pracownik, Zatrudnienie, Umowa, Angaz, Data_Od, Data_Do):
    stat = 'OK'
    SELECT = "SELECT umowa_id FROM rs_kp_angaze"
    SELECT += " WHERE firma_id = '" + self.firma_id + "' AND pracownik_id = '" + Pracownik + "'"
    SELECT += " AND zatrudnienie_id = " + str(Zatrudnienie) + " AND umowa_id = " + str(Umowa)

    # sprawdzenie unikalnosci zakresow przed INSERT
    if Fun in (1, 2):
        # sprawdzamy, czy data poczatkowa nie miesci sie
        # w jakims dokladnie okreslonym zakresie od-do
        SELECT += " AND ("
        SELECT += " ( do_kiedy IS NOT NULL AND '" + Data_Od + "' >= od_kiedy AND '" + Data_Od + "' <= do_kiedy )"

        # jesli podano date do
        if Data_Do != 'NULL':
            # czy w podanym zakresie od-do zlapiemy jakas istniejeca date_od
            SELECT += " OR (od_kiedy > '" + Data_Od + "' AND od_kiedy <= '" + Data_Do + "' )"

            # czy w podanym zakresie od-do zlapiemy jakas istniejaca date_do
            SELECT += " OR (do_kiedy IS NOT NULL AND do_kiedy >= '" + Data_Od + "' AND do_kiedy <= '" + Data_Do + "' )"

            # czy data koncowa nie znajduje sie w jakims dokladnie okreslonym zakresie
            SELECT += " OR (do_kiedy IS NOT NULL AND '" + Data_Do + "' > od_kiedy AND '" + Data_Do + "' <= do_kiedy)"
        SELECT += ");"

    # sprawdzenie unikalnosci zakresow przed UPDATE
    if Fun in (3, 4):
        # w przypadku edycji moze sie zdarzyc, ze jedna z dat nie jest modyfikowana
        # wtedy nie mamy jej wartosci. poniewaz mamy pelne dane klucza, to mozemy
        # pobrac te niemodyfikowana wartosc z tabeli
        if Data_Od == None or Data_Do == None:
            sql = "SELECT od_kiedy, do_kiedy FROM rs_kp_angaze"
            sql += " WHERE firma_id = '" + self.firma_id + "' AND pracownik_id = '" + Pracownik + "'"
            sql += " AND zatrudnienie_id = " + str(Zatrudnienie) + " AND umowa_id = " + str(
                Umowa) + " AND angaz_id = " + str(Angaz) + ";"
            stat, d, dump, dump = self.PolSQL.Execute(sql, 2)
            if stat != 'OK':
                stat = 'Bd pobrania dat z angau : ' + stat
            else:
                if len(d) != 1:
                    stat = 'Brak danych angau : ' + stat
                else:
                    if Data_Od == None:
                        Data_Od = str(d[0][0])
                        if Data_Od == 'None':
                            Data_Od = 'NULL'
                    if Data_Do == None:
                        Data_Do = str(d[0][1])
                        if Data_Do == 'None':
                            Data_Do = 'NULL'

        # wykluczenie z SELECT modyfikowanej pozycji
        SELECT += " AND angaz_id != " + str(Angaz) + " AND ("

        # sprawdzamy, czy data poczatkowa nie miesci sie
        # w jakims dokladnie okreslonym zakresie od-do
        SELECT += "( do_kiedy IS NOT NULL AND '" + Data_Od + "' >= od_kiedy AND '" + Data_Od + "' <= do_kiedy )"

        if Data_Do != 'NULL':
            SELECT += " AND ("
            # czy w podanym zakresie od-do zlapiemy jakas istniejeca date_od
            SELECT += " (od_kiedy >= '" + Data_Od + "' AND od_kiedy <= '" + Data_Do + "' )"

            # czy w podanym zakresie od-do zlapiemy jakas istniejaca date_do
            SELECT += " OR (do_kiedy IS NOT NULL AND do_kiedy >= '" + Data_Od + "' AND do_kiedy <= '" + Data_Do + "' )"

            # czy data koncowa nie znajduje sie w jakims dokladnie okreslonym zakresie
            SELECT += " OR (do_kiedy IS NOT NULL AND '" + Data_Do + "' >= od_kiedy AND '" + Data_Do + "' <= do_kiedy)"
            SELECT += ")"

        SELECT += ");"

    stat, d, dump, dump = self.PolSQL.Execute(SELECT, 2)
    if stat != 'OK':
        stat = 'Bd sprawdzenia pokrywania si zakresw dat angay : ' + stat

    if stat == 'OK':
        if len(d) > 0:
            stat = 'BL_POKRYWANIA_DAT'

    return stat


def NastepnyDzien(Data):
    if not rsglobal.SprawdzCzyToData(Data):
        return Data

    Rok = int(Data[0:4])
    Mies = int(Data[5:7])
    Dzien = int(Data[8:10])

    Dzien += 1
    if Mies in (1, 3, 5, 7, 8, 10, 12):
        if Dzien > 31:
            Dzien = 1
            Mies += 1
            if Mies > 12:
                Mies = 1
                Rok += 1
    elif Mies in (4, 6, 9, 11):
        if Dzien > 30:
            Dzien = 1
            Mies += 1
            if Mies > 12:
                Mies = 1
                Rok += 1
    else:
        # jesli byl 29 lutego, to teraz 1 marca
        if Dzien > 29:
            Dzien = 1
            Mies = 3
        elif Dzien > 28:
            # jesli zwyczajny rok, to 1 marca
            if not (Rok % 4 == 0) and ((Rok % 100 != 0) or (Rok % 400 == 0)):
                Dzien = 1
                Mies = 3

    return str(Rok) + '-' + str(Mies).zfill(2) + '-' + str(Dzien).zfill(2)


def PoprzedniDzien(Data):
    if not rsglobal.SprawdzCzyToData(Data):
        return Data

    Rok = int(Data[0:4])
    Mies = int(Data[5:7])
    Dzien = int(Data[8:10])

    Dzien -= 1
    if Dzien < 1:
        Mies -= 1
        if Mies == 2:
            # jesli przestepny rok, to 29 lutego
            if (Rok % 4 == 0) and ((Rok % 100 != 0) or (Rok % 400 == 0)):
                Dzien = 29
            else:
                Dzien = 28

        elif Mies == 0:
            Dzien = 31
            Mies = 12
            Rok -= 1

        elif Mies in (1, 3, 5, 7, 8, 10):
            Dzien = 31
        else:
            Dzien = 30

    return str(Rok) + '-' + str(Mies).zfill(2) + '-' + str(Dzien).zfill(2)


# -------------------------------------------------------------------------------

Pracownik = klucze[0]
Zatrudnienie = klucze[1]
Umowa = klucze[2]
Angaz = klucze[3]
#AB 02.02.22r
if len(klucze) == 6:
    dodawanie = klucze[4]
else:
    dodawanie = False

Fun = klucze[-1]

stat = 'OK'
dane = []
wyn1 = []
wyn2 = []

sql = "SELECT rsf_kadry_dozwolony_modul('" + self.firma_id + "', "
sql += "'" + str(self.operator_id) + "', 'angaze', '" + Pracownik + "');"
stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
if stat == 'OK':
    stat = d[0][0]

UprDodawanie = (((self.oddzial_id, '052043') in self.uprawnienia) or (self.operator_id == STANDARD_OPERATOR_ID))
UprEdycja = (((self.oddzial_id, '052042') in self.uprawnienia) or UprDodawanie
             or (self.operator_id == STANDARD_OPERATOR_ID))

# -------------------------------------------------------------------------------
# sprawdzenie uprawnien
# -------------------------------------------------------------------------------

if Fun in (1, 2) and stat == 'OK':
    if not UprDodawanie:
        stat = 'BR UPR INS'

if Fun in (3, 4) and stat == 'OK':
    if not UprEdycja:
        stat = 'BR UPR UPD'

# -------------------------------------------------------------------------------
# sprawdzenie danych
# -------------------------------------------------------------------------------

OdKiedy = None
DoKiedy = None

if stat == 'OK':
    for i in range(len(pola)):
        pole = pola[i].lower()
        wart = wartosci[i]

        if pole == 'angaz_id':
            continue

        elif pole == 'umowa_id':
            if wart == 0:
                dane.append('BR_UMOWA')
            else:
                sql = "SELECT umowa_id FROM rs_kp_umowy_o_prace"
                sql += " WHERE firma_id = '" + self.firma_id + "' AND pracownik_id = '" + Pracownik + "'"
                sql += " AND zatrudnienie_id = " + str(Zatrudnienie) + " AND umowa_id = " + str(wart) + ";"
                stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
                if stat != 'OK':
                    dane.append('Bd pobrania numeru umowy o prac : ' + stat)
                elif len(d) == 0:
                    dane.append('BL_UMOWA')
            wartosci[i] = str(wart)

        elif pole == 'od_kiedy':
            if wart == '    -  -  ':
                dane.append('BR_OD_KIEDY')
            else:
                if rsglobal.SprawdzCzyToData(wart):
                    OdKiedy = wart
                else:
                    dane.append('BL_OD_KIEDY')
            wartosci[i] = "'" + wart + "'"

        elif pole == 'do_kiedy':
            if wart == '    -  -  ':
                wartosci[i] = 'NULL'
                DoKiedy = wartosci[i]
            else:
                if rsglobal.SprawdzCzyToData(wart):
                    DoKiedy = wart
                else:
                    dane.append('BL_DO_KIEDY')
                wartosci[i] = "'" + wart + "'"

        elif pole == 'miejsce_id':
            if wart == '':
                dane.append('BR_MIEJSCE')
            else:
                sql = "SELECT kod FROM rs_miejsca_pracy"
                sql += " WHERE firma_id = '" + self.firma_id + "' AND kod = '" + wart + "';"
                stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
                if stat != 'OK':
                    dane.append('Bd pobrania kodu miejsca pracy : ' + stat)
                elif len(d) == 0:
                    dane.append('BL_MIEJSCE')
            wartosci[i] = "'" + wart + "'"

        elif pole == 'komorka_id':
            if wart == '':
                dane.append('BR_KOMORKA')
            else:
                sql = "SELECT kod FROM rs_komorki_zatrudniajace"
                sql += " WHERE firma_id = '" + self.firma_id + "' AND kod = '" + wart + "';"
                stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
                if stat != 'OK':
                    dane.append('Bd pobrania kodu komrki zatrudniajacej : ' + stat)
                elif len(d) == 0:
                    dane.append('BL_KOMORKA')
            wartosci[i] = "'" + wart + "'"

        elif pole == 'rodzaj_pracownika_id':
            if wart == '':
                dane.append('BR_RODZAJ_PRACOWNIKA')
            else:
                sql = "SELECT kod FROM rs_rodzaje_pracownikow"
                sql += " WHERE firma_id = '" + self.firma_id + "' AND kod = '" + wart + "';"
                stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
                if stat != 'OK':
                    dane.append('Bd pobrania kodu rodzaju pracownika : ' + stat)
                elif len(d) == 0:
                    dane.append('BL_RODZAJ_PRACOWNIKA')
            wartosci[i] = "'" + wart + "'"

        elif pole == 'grupa_pracownikow_id':
            if wart == '':
                dane.append('BR_GRUPA_PRACOWNIKOW')
            else:
                sql = "SELECT kod FROM rs_grupy_pracownikow"
                sql += " WHERE firma_id = '" + self.firma_id + "' AND kod = '" + wart + "';"
                stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
                if stat != 'OK':
                    dane.append('Bd pobrania kodu grupy pracownikw : ' + stat)
                elif len(d) == 0:
                    dane.append('BL_GRUPA_PRACOWNIKOW')
            wartosci[i] = "'" + wart + "'"

        elif pole == 'stanowisko_id':
            if wart == '':
                dane.append('BR_STANOWISKO')
            else:
                sql = "SELECT kod FROM rs_stanowiska_pracy"
                sql += " WHERE firma_id = '" + self.firma_id + "' AND kod = '" + wart + "';"
                stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
                if stat != 'OK':
                    dane.append('Bd pobrania kodu stanowiska pracy : ' + stat)
                elif len(d) == 0:
                    dane.append('BL_STANOWISKO')
            wartosci[i] = "'" + wart + "'"

        elif pole == 'stanowisko_miejsce_id':
            if wart == '':
                dane.append('BR_STANOWISKO_MIEJSCE')
            else:
                sql = "SELECT kod FROM rs_kp_stanowiska_pracy_miejsca"
                sql += " WHERE firma_id = '" + self.firma_id + "' AND kod = '" + wart + "';"
                stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
                if stat != 'OK':
                    dane.append('Bd pobrania kodu stanowiska pracy-miejsca : ' + stat)
                elif len(d) == 0:
                    dane.append('BL_STANOWISKO_MIEJSCE')
            wartosci[i] = "'" + wart + "'"

        elif pole == 'zawod_wykonywany_id':
            if wart == '':
                wartosci[i] = 'NULL'
            else:
                sql = "SELECT kod FROM rs_zawody_specjalnosci_klasyfikacja"
                sql += " WHERE firma_id = '" + self.firma_id + "' AND kod = '" + wart + "';"
                stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
                if stat != 'OK':
                    dane.append('Bd pobrania kodu zawodu wykonywanego : ' + stat)
                elif len(d) == 0:
                    dane.append('BL_ZAWOD_WYKONYWANY')
                wartosci[i] = "'" + wart + "'"

        elif pole == 'stanowisko_kosztow_id':
            if wart == '':
                wartosci[i] = 'NULL'
            else:
                sql = "SELECT kod FROM rs_stanowiska_kosztow"
                sql += " WHERE firma_id = '" + self.firma_id + "' AND kod = '" + wart + "';"
                stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
                if stat != 'OK':
                    dane.append('Bd pobrania kodu stanowiska kosztw : ' + stat)
                elif len(d) == 0:
                    dane.append('BL_STANOWISKO_KOSZTOW')
                wartosci[i] = "'" + wart + "'"

        elif pole == 'kalendarz_id':
            if wart == '':
                dane.append('BR_KALENDARZ')
            else:
                sql = "SELECT kalendarz_id FROM rs_kp_kalendarze"
                sql += " WHERE firma_id = '" + self.firma_id + "' AND kalendarz_id = '" + wart + "';"
                stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
                if stat != 'OK':
                    dane.append('Bd pobrania kodu kalendarza : ' + stat)
                elif len(d) == 0:
                    dane.append('BL_KALENDARZ')
                wartosci[i] = "'" + wart + "'"

        elif pole == 'lista_id':
            if wart != '':
                sql = "SELECT lista_id FROM rs_kp_szablony_list_plac"
                sql += " WHERE firma_id = '" + self.firma_id + "' AND lista_id = '" + wart + "';"
                stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
                if stat != 'OK':
                    dane.append('Bd pobrania kodu szablonu listy pac : ' + stat)
                elif len(d) == 0:
                    dane.append('BL_LISTAPLAC')
                wartosci[i] = "'" + wart + "'"
            else:
                wartosci[i] = "NULL"

        elif pole == 'etat':
            if wart < 0 or wart > 1.0:
                dane.append('BL_ETAT')
            elif not Fun in (1, 2):  #AB 03.10.22r Nie sprawdza podczas dodawania
                SQL = """SELECT *
                         FROM rs_kp_listy_plac_osoby 
                         WHERE (firma_id, pracownik_id, zatrudnienie_id, umowa_id, angaz_id) =
                               (%s,       %s,           %s,              %s,       %s);"""
                args = [self.firma_id, Pracownik, Zatrudnienie, Umowa, Angaz]

                rsglobal.ZapiszDoLogow(self.PolSQL.Mogrify(SQL, args))
                stat, listy, dump, dump = self.PolSQL.Execute(SQL, 3, args, CzyDodawacBlad=True)
                if stat != 'OK':
                    stat = 'Bd szukania list pac: '+stat
                elif len(listy) > 0:
                    dane.append(u'ISTN_L_PLAC')


            wartosci[i] = str(wart)
        #AB 10.10.19r
        elif pole in ('placa', 'stawka', 'dodatek_fun', 'ryczalt', 'dodatek1', 'dodatek2', 'zerowy_pit',
                      'ulga_kl_sr', 'ulga_rodz_plus','ulga_na_powrot','ulga_dla_emeryta'): #BP 2021-12-21
            wartosci[i] = str(wart)

        else:
            # pozostale pola
            wartosci[i] = "'" + wart + "'"

    # jesli byl jakis blad
    if len(dane) > 0:
        stat = dane[0]

if stat == 'OK':
    if OdKiedy != None and DoKiedy != None:
        if OdKiedy > DoKiedy:
            stat = 'BL_OD_DO_KIEDY'
            dane.append(stat)

    if stat == 'OK':
        if OdKiedy != None or DoKiedy != None:
            stat = SprNakladanieDat(self, Fun, Pracownik, Zatrudnienie, Umowa, Angaz, OdKiedy, DoKiedy)
            if stat != 'OK':
                dane.append(stat)

# -------------------------------------------------------------------------------
# --- DODAWANIE DANYCH (1 -z pobraniem select  2 - bez pobrania
# -------------------------------------------------------------------------------

if Fun in (1, 2) and stat == 'OK':
    # ustalenie numeru nowego angazu
    Angaz = 0
    sql = "SELECT MAX(angaz_id) FROM rs_kp_angaze"
    sql += " WHERE firma_id = '" + self.firma_id + "' AND pracownik_id = '" + Pracownik + "'"
    sql += " AND zatrudnienie_id = " + str(Zatrudnienie) + " AND umowa_id = " + str(Umowa) + ";"

    stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
    if stat != 'OK':
        stat = 'Bd pobrania numeru z kartoteki angay : ' + stat
    else:
        if d[0][0] in ['None', '']:
            Angaz = 1
        else:
            Angaz = int(d[0][0]) + 1

    # jesli to nastepny angaz w ramach umowy, sprawdzamy ciaglosc tzn. data od_kiedy ma byc nastepnym dniem
    # po zakonczeniu poprzednieno angazu czyli data do_kiedy z poprzeniego angazu
    if Angaz > 1:
        sql = "SELECT do_kiedy FROM rs_kp_angaze"
        sql += " WHERE firma_id = '" + self.firma_id + "' AND pracownik_id = '" + Pracownik + "'"
        sql += " AND zatrudnienie_id = " + str(Zatrudnienie) + " AND umowa_id = " + str(
            Umowa) + " AND angaz_id = " + str(Angaz - 1) + ";"

        stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
        if stat != 'OK':
            stat = 'Bd pobrania daty kocowej z poprzeniego angau : ' + stat
        else:
            if len(d) != 1:
                stat = 'Brak danych dot.poprzenieggo angau'
            else:
                if PoprzedniDzien(OdKiedy) != str(d[0][0]):
                    stat = 'BL_BRAK_CIAGLOSCI'

    if stat == 'OK':
        UmowaOdKiedy = ''
        UmowaDoKiedy = ''
        sql = "SELECT od_kiedy, do_kiedy FROM rs_kp_umowy_o_prace"
        sql += " WHERE firma_id = '" + self.firma_id + "' AND pracownik_id = '" + Pracownik + "'"
        sql += " AND zatrudnienie_id = " + str(Zatrudnienie) + " AND umowa_id = " + str(Umowa) + ";"
        stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
        if stat != 'OK':
            stat = 'Bd pobrania danych z umowy o prac : ' + stat
        else:
            if len(d) == 0:
                stat = 'Brak umowy o prac o podanych parametrach'
            else:
                UmowaOdKiedy = str(d[0][0])
                UmowaDoKiedy = str(d[0][1])

    if stat == 'OK':
        # pierwszy angaz ma date poczatku umowy
        if Angaz == 1:
            if OdKiedy != UmowaOdKiedy:
                stat = 'BL_DATA_PIERWSZY_ANGAZ'

    if stat == 'OK':
        # czy koniec angazu nie przekracza konca umowy, jesli koniec umowy jest podany
        if UmowaDoKiedy != 'None':
            if DoKiedy == 'NULL':
                stat = 'BL_NIEOKRESLONA_DATADO_ANGAZ'
            else:
                if DoKiedy > UmowaDoKiedy:
                    stat = 'BL_DATADO_ANGAZ_UMOWA'

    if stat == 'OK':
        inspola = "firma_id, pracownik_id, zatrudnienie_id"
        inswart = "'" + self.firma_id + "', '" + Pracownik + "', " + str(Zatrudnienie)

        for i in xrange(len(pola)):
            if pola[i] == 'angaz_id':
                wartosci[i] = str(Angaz)

            inspola += ", " + pola[i]
            inswart += ", " + wartosci[i]

        INSERT = "INSERT INTO rs_kp_angaze (" + inspola + ") VALUES (" + inswart + ");"
        stat, dump, dump, dump = self.PolSQL.Execute(INSERT, 1)
        if stat != 'OK':
            stat = 'Bd dodawania nowego angau : ' + stat
            dane.append(stat)

        #AB 11.06.18r kopiowanie skadnikw dodajc kolejny anga

    if stat == 'OK' and Angaz > 1:
        INSSKL = "INSERT INTO rs_kp_angaze_skladniki"
        INSSKL += "(firma_id, pracownik_id, zatrudnienie_id, umowa_id, angaz_id, skladnik_id, data_od,"
        INSSKL += " data_do, kwota)"
        INSSKL += "SELECT firma_id, pracownik_id, zatrudnienie_id, umowa_id, " + str(Angaz) + ", skladnik_id, data_od,"
        INSSKL += " data_do, kwota "
        INSSKL += "FROM rs_kp_angaze_skladniki "
        INSSKL += "WHERE firma_id = '" + self.firma_id + "' AND pracownik_id = '" + Pracownik + "' AND zatrudnienie_id "
        INSSKL += "= " + str(Zatrudnienie) + " AND umowa_id = " + str(Umowa) + " AND angaz_id = " + str(Angaz - 1) + ";"
        stat, dump, dump, dump = self.PolSQL.Execute(INSSKL, 1)
        if stat != 'OK':
            stat = 'Bd kopiowania skadnikw z poprzedniego angau : ' + stat
            dane.append(stat)

# -------------------------------------------------------------------------------
# --- UPDATE DANYCH ( 3-select, 4 -update z pobraniem)
# -------------------------------------------------------------------------------

if Fun in (3, 4) and stat == 'OK':
    sqlupd = ''
    for i in xrange(len(pola)):
        sqlupd += pola[i] + " = " + wartosci[i] + ","

    if len(sqlupd) > 0:
        UPDATE = "UPDATE rs_kp_angaze SET " + sqlupd[:-1]
        WHERE = " WHERE firma_id = '" + self.firma_id + "' AND pracownik_id = '" + Pracownik + "'"
        WHERE += " AND zatrudnienie_id = " + str(Zatrudnienie) + " "
        WHERE += " AND umowa_id = " + str(Umowa) + " AND angaz_id = " + str(Angaz) + ";"
        stat, d, dump, dump = self.PolSQL.Execute(UPDATE + WHERE, 1)
        if stat != 'OK':
            stat, dane = verification_after_update(self, WHERE, dane)
            if stat == 'OK':
                stat = 'Bd aktualizacji danych angau : ' + stat
                dane.append(stat)
        else:
            stat, dane = verification_after_update(self, WHERE, dane)

    if stat == 'OK':
        sql = "SELECT * FROM rs_kp_angaze "
        sql += " WHERE firma_id = '" + self.firma_id + "' AND pracownik_id = '" + Pracownik + "'"
        sql += " AND zatrudnienie_id = " + str(Zatrudnienie) + " "
        sql += " AND umowa_id = " + str(Umowa) + " AND angaz_id = " + str(Angaz)
        sql += " AND od_kiedy > do_kiedy" + ";"
        stat, d, wyn1, dump = self.PolSQL.Execute(sql, 3)
        if stat == 'OK' and len(d):
            stat = "BL_OD_DO_KIEDY"


# -------------------------------------------------------------------------------
# --- POBRANIE DANYCH ( 5-select, 3 -update z pobraniem)
# -------------------------------------------------------------------------------

if stat == 'OK' and Fun in (1, 3, 5, 9):
    SELECT = "SELECT * FROM rs_kp_angaze"
    SELECT += " WHERE firma_id='" + self.firma_id + "' AND pracownik_id='" + Pracownik + "'"
    SELECT += " AND zatrudnienie_id=" + str(Zatrudnienie) + " AND umowa_id=" + str(Umowa)
    SELECT += " AND angaz_id=" + str(Angaz) + ";"

    stat, dane, wyn1, dump = self.PolSQL.Execute(SELECT, 3)
    if stat != 'OK':
        stat = 'Bd pobrania danych angau : ' + stat
        dane.append(stat)

    if stat == 'OK':
        tr = rssql.RSTableRowset(dane, wyn1)

        # [0] miejsce_id
        if (self.oddzial_id, '011700') in self.uprawnienia or self.operator_id == STANDARD_OPERATOR_ID:
            sql = "SELECT opis FROM rs_miejsca_pracy"
            sql += " WHERE firma_id='" + self.firma_id + "' AND kod='" + tr.GetVal(0, "miejsce_id") + "';"
            stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
            if stat != 'OK':
                wyn2.append('Bd pobrania opisu ze sownika miejsc pracy : ' + stat)
            else:
                if len(d) == 0:
                    wyn2.append('Bdny kod miejsca pracy.')
                else:
                    wyn2.append(d[0][0])
        else:
            wyn2.append('Brak uprawnie do sownika. Poinformuj administratora.')

        # [1] komorka_id
        if (self.oddzial_id, '011710') in self.uprawnienia or self.operator_id == STANDARD_OPERATOR_ID:
            sql = "SELECT opis FROM rs_komorki_zatrudniajace"
            sql += " WHERE firma_id='" + self.firma_id + "' AND kod='" + tr.GetVal(0, "komorka_id") + "';"
            stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
            if stat != 'OK':
                wyn2.append('Bd pobrania opisu ze sownika komrek zatrudniajcych : ' + stat)
            else:
                if len(d) == 0:
                    wyn2.append('Bdny kod komrki zatrudniajcej.')
                else:
                    wyn2.append(d[0][0])
        else:
            wyn2.append('Brak uprawnie do sownika. Poinformuj administratora.')

        # [2] rodzaj_pracownika_id
        if (self.oddzial_id, '011720') in self.uprawnienia or self.operator_id == STANDARD_OPERATOR_ID:
            sql = "SELECT opis FROM rs_rodzaje_pracownikow"
            sql += " WHERE firma_id='" + self.firma_id + "' AND kod='" + tr.GetVal(0, "rodzaj_pracownika_id") + "';"
            stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
            if stat != 'OK':
                wyn2.append('Bd pobrania opisu ze sownika rodzajw pracownikw : ' + stat)
            else:
                if len(d) == 0:
                    wyn2.append('Bdny kod rodzaju pracownika.')
                else:
                    wyn2.append(d[0][0])
        else:
            wyn2.append('Brak uprawnie do sownika. Poinformuj administratora.')

        # [3] grupa_pracownikow_id
        if (self.oddzial_id, '011730') in self.uprawnienia or self.operator_id == STANDARD_OPERATOR_ID:
            sql = "SELECT opis FROM rs_grupy_pracownikow"
            sql += " WHERE firma_id='" + self.firma_id + "' AND kod='" + tr.GetVal(0, "grupa_pracownikow_id") + "';"
            stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
            if stat != 'OK':
                wyn2.append('Bd pobrania opisu ze sownika grup pracownikw : ' + stat)
            else:
                if len(d) == 0:
                    wyn2.append('Bdny kod grupy pracownikw.')
                else:
                    wyn2.append(d[0][0])
        else:
            wyn2.append('Brak uprawnie do sownika. Poinformuj administratora.')

        # [4] stanowisko_id
        if (self.oddzial_id, '011740') in self.uprawnienia or self.operator_id == STANDARD_OPERATOR_ID:
            sql = "SELECT opis FROM rs_stanowiska_pracy"
            sql += " WHERE firma_id='" + self.firma_id + "' AND kod='" + tr.GetVal(0, "stanowisko_id") + "';"
            stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
            if stat != 'OK':
                wyn2.append('Bd pobrania opisu ze sownika stanowisk pracy : ' + stat)
            else:
                if len(d) == 0:
                    wyn2.append('Bdny kod stanowiska pracy.')
                else:
                    wyn2.append(d[0][0])
        else:
            wyn2.append('Brak uprawnie do sownika. Poinformuj administratora.')

        # [5] zawod_wykonywany_id   moze byc pusty
        wart = rsglobal.Dekoduj(tr.GetVal(0, "zawod_wykonywany_id").strip())
        if wart in ('', 'None'):
            wyn2.append('')
        else:
            if (self.oddzial_id, '011640') in self.uprawnienia or self.operator_id == STANDARD_OPERATOR_ID:
                sql = "SELECT opis FROM rs_zawody_specjalnosci_klasyfikacja"
                sql += " WHERE firma_id='" + self.firma_id + "' AND kod='" + wart + "';"
                stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
                if stat != 'OK':
                    wyn2.append('Bd pobrania opisu ze sownika zawodw : ' + stat)
                else:
                    if len(d) == 0:
                        wyn2.append('Bdny kod zawodu wykonywanego.')
                    else:
                        wyn2.append(d[0][0])
            else:
                wyn2.append('Brak uprawnie do sownika. Poinformuj administratora.')

        # [6] stanowisko_kosztow_id   moze byc puste
        wart = rsglobal.Dekoduj(tr.GetVal(0, "stanowisko_kosztow_id").strip())
        if wart in ('', 'None'):
            wyn2.append('')
        else:
            if (self.oddzial_id, '011750') in self.uprawnienia or self.operator_id == STANDARD_OPERATOR_ID:
                sql = "SELECT opis FROM rs_stanowiska_kosztow"
                sql += " WHERE firma_id='" + self.firma_id + "' AND kod='" + wart + "';"
                stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
                if stat != 'OK':
                    wyn2.append('Bd pobrania opisu ze sownika stanowisk kosztw : ' + stat)
                else:
                    if len(d) == 0:
                        wyn2.append('Bdny kod stanowiska kosztw.')
                    else:
                        wyn2.append(d[0][0])
            else:
                wyn2.append('Brak uprawnie do sownika. Poinformuj administratora.')

        # [7] kalendarz_id
        if (self.oddzial_id, '051030') in self.uprawnienia or self.operator_id == STANDARD_OPERATOR_ID:
            sql = "SELECT nazwa FROM rs_kp_kalendarze"
            sql += " WHERE firma_id='" + self.firma_id + "' AND kalendarz_id='" + tr.GetVal(0, "kalendarz_id") + "';"
            stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
            if stat != 'OK':
                wyn2.append('Bd pobrania opisu kalendarza: ' + stat)
            else:
                if len(d) == 0:
                    wyn2.append('Bdny kod kalendarza.')
                else:
                    wyn2.append(d[0][0])
        else:
            wyn2.append('Brak uprawnie do kalendarzy. Poinformuj administratora.')

        # [8] stanowisko_miejsce_id   moze byc puste
        wart = rsglobal.Dekoduj(tr.GetVal(0, "stanowisko_miejsce_id").strip())
        if wart in ('', 'None'):
            wyn2.append('')
        else:
            if (self.oddzial_id, '211220') in self.uprawnienia or self.operator_id == STANDARD_OPERATOR_ID:
                sql = "SELECT opis FROM rs_kp_stanowiska_pracy_miejsca"
                sql += " WHERE firma_id='" + self.firma_id + "' AND kod='" + wart + "';"
                stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
                if stat != 'OK':
                    wyn2.append('Bd pobrania opisu ze sownika stanowisk pracy miejsca : ' + stat)
                else:
                    if len(d) == 0:
                        wyn2.append('Bdny kod stanowiska pracy miejsca.')
                    else:
                        wyn2.append(d[0][0])
            else:
                wyn2.append('Brak uprawnie do sownika. Poinformuj administratora.')

        # [9] lista_id
        if (self.oddzial_id, '061020') in self.uprawnienia or self.operator_id == STANDARD_OPERATOR_ID:
            sql = "SELECT opis FROM rs_kp_szablony_list_plac"
            sql += " WHERE firma_id='" + self.firma_id + "' AND lista_id='" + tr.GetVal(0, "lista_id") + "';"
            stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
            if stat != 'OK':
                wyn2.append('Bd pobrania opisu szablonu listy pac: ' + stat)
            else:
                if len(d) == 0:
                    wyn2.append('Bdny kod szablonu listy pac.')
                else:
                    wyn2.append(d[0][0])
        else:
            wyn2.append('Brak uprawnie do szablonw list pac. Poinformuj administratora.')

        #AB 10.10.19r
        # [10] czy modszy ni 26 lat
        sql = """SELECT data_urodzenia + INTERVAL '26 years' > NOW() AS mlodszy_niz_26 
        FROM rs_kp_dane_osobowe 
        WHERE firma_id='{}' AND pracownik_id='{}';""".format(self.firma_id, Pracownik)
        stat, d, dump, dump = self.PolSQL.Execute(sql, 3, CzyDodawacBlad=True)
        if stat != 'OK':
            wyn2.append('Bd pobrania informacji o wieku pracownika : ' + stat)
        else:
            wyn2.append(d[0][0])

        #JR 2023-05-20 (13422)
        sql = "SELECT pracownik FROM rs_kp_dane_osobowe "
        sql += " WHERE firma_id = '" + self.firma_id + "' AND pracownik_id = '" + Pracownik + "'"
        stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
        if stat != 'OK':
            wyn2.append('Bd pobrania informacji czy pracownik : ' + stat)
        else:
            wyn2.append(d[0][0])
