# -*- coding: iso8859_2 -*-
#-------------------------------------------------------------------------------
# Name:        fs_kp_dane_osobowe_absencje.py
# Purpose:     grid z absencjami pracownika
# Author:      Piotr Witkowski
# Created:     2015/12/06
# RCS-ID:      $Id: fs_kp_dane_osobowe_absencje.py $
# Copyright:   (c) 2016 Record System
# Licence:     GNU GPL
#-------------------------------------------------------------------------------

# 2016-02-06 - Piotr Witkowski   (9554) Okno Edycja absencji pracownika - poprawki
# 2018-09-24 Aleksander Bargie (11234) - [KP] rodzina i data poprzedniej absencji w absencjach kartoteki osobowej
# 2018-10-26 Aleksander Bargie (11270) - Dodatkowe pole w pozycjach absencji w pracy biecej
# 2018-11-27 Aleksander Bargie (11297) - [KP] podstawy dla zasikw
# 2019-06-03 Wojciech Skowron   (11434) - Edycja podstaw dla zasikw
# 2019-10-08 Wojciech Skowron   (11679) - Podstawy dla zasikw - przy zapisie obcina grosze w polach numerycznych
# 2020-05-06 Mikoaj Kozowski  (11947) - obsuga filtrw do wywietlania samych urlopw, samych zwolnie lub wszystkich absencji
# 2020-06-29 Aleksander Bargie (12029) - [KP] Obsuga urlopu na danie w pracy biecej
# 2020-07-03 Aleksander Bargie (12035) - Bdne dziaanie filtrw w kartotece osobowej absencje
# 2020-07-21 Aleksander Bargie (12059) - Bdna podstawa do zasiku
# 2021-02-13 Jan Raburski (12332) - blokada nieaktywnych rodzajow absencji
# 2022-01-20 Bartosz Pagan     (12833) - Dane osobowe absencje - edytowalnosc pl
# 2022-03-02 Aleksander Bargie (12912 ) - Dane osobowe absencje - podstawy - bedne wyswietlanie
# 2023-02-15 Aleksander Bargie (13355) - Limity urlopw na gridzie z absencjami pracownika
# 2023-08-28 Jan Raburski (13512) - przejcie w dniach roboczych na kalendarz
# 2023-09-05 Wojciech Skowron   (13524) - Przy spr czy daty od kiedy/do kiedy robocze dodatkowo spr czy_nieobecnosc
# 2023-09-08 Wojciech Skowron   (13541) - Blokada usunicia absencji w danych osobowych jeli ma wpisan list pac
# 2023-12-22 Wojciech Skowron   (13547) - Sprawdzenie dat z 13524 dodatkowo przy zmianie rodzaju

import datetime
import rs_anakonda_daty
import rsglobal

#AB 26.10.18r
# Zawarto self.Etykiety w \RSKPDaneOsoboweAbsencje.py .py, do najdowania numerow kolumn
Etykiety = ['typ_absencji_id','absencja_opis','od_kiedy','do_kiedy', 'data_dostarczenia','data_wystawienia','kod_zus',
            'dni_szpital', 'procent_wyplaty','kontynuacja', 'kontynuacja_data', 'rodzina_id','dni_kalendarzowe',
                                              #AB 29.06.20r
            'dni_robocze', 'godziny_robocze', 'na_zadanie', 'dni_zalegle','godziny_zalegle', 'dni_tegoroczne','godziny_tegoroczne',
            'dni_okr', 'dni_rok',
            #AB 27.11.18r
            'podstawa', 'lmcy', 'zasilek_dzien', 'zasilek_razem', 'numer_listy_id',
            'data_wyplaty', 'proc_zwolnienia', 'miesiace', 'podstawa_obliczenia']
# przerobka na slownik w celu wygodniejszego dostepu do danych
kolGrid = {}
for i in range(len(Etykiety)):
    kolGrid[Etykiety[i]] = i

dane  = []
wyn1  = []
wyn2  = []
stat  = 'OK'
#AB 26.09.18r
puste = (None, '', '    -  -  ', 'None', '-  -')

if not 'Funkcja' in klucze:
    stat='Brak podanej funkcji w kluczach'

if stat=='OK':
    Funkcja = klucze['Funkcja']

    SlPar={'Pracownik':[1, 5, 99],'RodzajAbsencjiF':[1,5],'DataOdF':[1,5],
    'Klucze_Id':[1,5],'TypAbsencji':[10], 'dodane':[1,5], 'TylkoUrlopyF':[1,5], 'TylkoZwolnieniaF':[1,5], 'WszystkieF':[1,5]} #MK 2020-05-06

    Lista=rsglobal.SprawdzParametryWKluczach(SlPar,Funkcja,klucze)
    if Lista<>'':
        stat='Uwaga : wywoanie funkcji fs_kp_dane_osobowe_absencje z numerem '+str(Funkcja)+' bez parametrw: '+Lista

if Funkcja in (1, 5, 11, 99):
    Pracownik = rsglobal.Dekoduj(klucze['Pracownik'])

if Funkcja in (1,5):    
    # filtry dla grida
    TypAbsencjiF = rsglobal.Dekoduj(klucze['RodzajAbsencjiF'])
    DataOdF      = rsglobal.Dekoduj(klucze['DataOdF'])
    DataDoF      = rsglobal.Dekoduj(klucze['DataDoF'])
    TylkoUrlopyF = rsglobal.Dekoduj(klucze['TylkoUrlopyF'])  #MK 2020-05-06
    TylkoZwolnieniaF = rsglobal.Dekoduj(klucze['TylkoZwolnieniaF']) #MK 2020-05-06
    WszystkieF = rsglobal.Dekoduj(klucze['WszystkieF']) #MK 2020-05-06
    Klucze_Id    = rsglobal.Dekoduj(klucze['Klucze_Id'])
    dodane       = rsglobal.Dekoduj(klucze['dodane'])

if Funkcja in (10,):
    # typ zwolnienia dla kodu absencji i listy podobnych typow absencji
    Absencja = rsglobal.Dekoduj(klucze['TypAbsencji'])
if Funkcja in (11,):
    # typ zwolnienia dla kodu absencji i listy podobnych typow absencji
    DataOdF = rsglobal.Dekoduj(klucze['DataOdF'])

if Funkcja in (1,5,99) and stat == 'OK':
    # sprawdzenie uprawnien do modulu
    sql ="SELECT rsf_kadry_dozwolony_modul('"+self.firma_id+"', '"+str(self.operator_id)+"', 'absencje', '"+Pracownik+"');"
    stat,Dane,dump,dump = self.PolSQL.Execute(sql,3)
    if stat == 'OK':
        if Dane[0][0]<>'OK':       
            stat = Dane[0][0]
            dane.append(stat)
    else:
        stat='Problemy przy sprawdzaniu uprawnie do wybranego moduu : '+stat
        dane.append(stat)


#-------------------------------------------------------------------------------
#---   Pobranie procentu zwrotu i kodu zus
#-------------------------------------------------------------------------------

if Funkcja == 10 and stat=="OK":
    sql ="SELECT proc_zwolnienia, kod_zus FROM rs_kp_absencje_rodzaje"
    sql+=" WHERE firma_id='"+self.firma_id+"' AND typ_absencji_id='"+Absencja+"';"
    stat,dane,dump,dump = self.PolSQL.Execute(sql,3)
    if stat != 'OK':
        stat = 'Bd pobrania opisu absencji: '+stat

#-------------------------------------------------------------------------------
#----------   UPRAWNIENIA
#-------------------------------------------------------------------------------

#UprModyfikowanie=(((self.oddzial_id,'052046') in self.uprawnienia) or (self.operator_id == STANDARD_OPERATOR_ID)) 

UprUsuwanie  = ((self.oddzial_id,'052134') in self.uprawnienia or self.operator_id == STANDARD_OPERATOR_ID)
UprDodawanie = (((self.oddzial_id,'052133') in self.uprawnienia) or (self.operator_id == STANDARD_OPERATOR_ID))
UprEdycja    = (((self.oddzial_id,'052132') in self.uprawnienia) or (self.operator_id == STANDARD_OPERATOR_ID)) or UprDodawanie

UprPZasilkow = ((self.oddzial_id,'052136') in self.uprawnienia or self.operator_id == STANDARD_OPERATOR_ID) #WS 2019-06-03

UprIns = UprDodawanie
UprUpd = UprEdycja
UprDel = UprUsuwanie

Insert = []
Update = [] 
Delete = []

if stat=='OK' and Funkcja == 1:
    Update = klucze['Update']
    Insert = klucze['Insert']
    Delete = klucze['Delete']


#-------------------------------------------------------------------------------
#--- SPRAWDZENIE DAT W FILTRZE - 1,5
#-------------------------------------------------------------------------------

if Funkcja in (1,5) and stat == 'OK':
    if DataOdF != '    -  -  ':
        if not rsglobal.SprawdzCzyToData(DataOdF):
            stat='Bdna data "od kiedy" w filtrze'
            dane.append(stat)  

            
#-------------------------------------------------------------------------------
#---   DELETE (1)
#-------------------------------------------------------------------------------

if Funkcja == 1 and stat=="OK":
    if len(Delete[0])>0:
        if not UprDel:
            stat='UPR BR DEL'
            dane.append(stat)
        # sprawdzanie DELETE 
        if stat == 'OK':
            for i in xrange(len(Delete[0])):
                if stat == 'OK':
                    #WS 2023-09-08 - sprawdzenie numer_listy_id, jeli wpisany to blokowa usuwanie
                    SELECT = "SELECT numer_listy_id, data_wyplaty FROM rs_kp_dane_osobowe_absencje "
                    WHERE = " WHERE firma_id='"+self.firma_id+"' AND pracownik_id='"+str(Pracownik)+"' "
                    for j in xrange(len(Delete[0][i])):
                        pole = Delete[1][j]
                        wart = Delete[0][i][j]
                        if pole in ['typ_absencji_id','od_kiedy']:
                            if wart<>'' and stat=='OK':
                                WHERE+=" AND "+str(pole)+"='"+str(wart)+"'"
                    WHERE+=";"
                    stat,lista,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(SELECT+WHERE),3)
                    if stat != 'OK':
                        stat = "Bd przy sprawdzaniu danych przed usuniciem: "+stat
                        dane.append(stat)
                    elif lista[0][0].strip() not in ('', 'None'):
                        stat = "Absencja wykorzystywana jest w licie pac "+lista[0][0].strip()+" z dat wypaty "+lista[0][1]+", usunicie niemoliwe."
                        dane.append(stat)
                    else:
                        DELETE = "DELETE FROM rs_kp_dane_osobowe_absencje "
                        stat,dump,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(DELETE+WHERE),1)
                        if stat != 'OK':
                            stat = "Bd przy usuwaniu danych: "+stat
                            dane.append(stat)


#-------------------------------------------------------------------------------
#---   UPDATE (1)
#-------------------------------------------------------------------------------

if Funkcja == 1 and stat=="OK":
    if len(Update[0])>0 and UprUpd:
            
        if stat == 'OK':
            WierszeNr = Update[-1]
            
            for i in xrange(len(Update[0])):
                WHERE = " WHERE firma_id='"+self.firma_id+"' AND pracownik_id='"+str(Pracownik)+"' "
                for j in xrange(len(Update[2][i])): # petla po kluczu
                    pole=Update[3][j]
                    wart=rsglobal.Dekoduj(Update[2][i][j])
                    if pole in ['typ_absencji_id','od_kiedy']:
                        WHERE+=" AND "+pole+"='"+wart+"'"   
                # WHERE+=";"

                #WS 2023-09-05
                # TUTAJ POBIERA SI INFORMACJE O WYMAGANYCH POLACH DLA DANEGO TYPU ABSENCJI, JELI KIEDY
                # Z JAKIEGO POWODU KOLEJNO KOLUMN ULEGNIE ZMIANIE, TO SPRAWDZENIE PRZESTANIE DZIAA.
                spr ="SELECT zus_wymagany, dostarczono, wystawiono, czy_glowna, czy_nieobecnosc FROM rs_kp_absencje_rodzaje "
                spr+="WHERE firma_id='"+str(self.firma_id)+"' AND typ_absencji_id = (SELECT typ_absencji_id FROM rs_kp_dane_osobowe_absencje " + WHERE + ");"
                stat, wymagania, wyn1, dump = self.PolSQL.Execute(spr, 3)
                if stat != 'OK' or len(wymagania) == 0:
                    stat = 'Bd odczytu wymaga dla wybranego typu absencji.'
                else:
                    for j in xrange(len(Update[0][i])):
                        pole = Update[0][i][j]
                        wart = Update[1][i][j]

                        if pole == 'typ_absencji_id':
                            CzyKlucz=True
                            SPR2=" AND "+pole+"='"+wart+"'"
                            if len(wart.strip())==0:
                                #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('RODZAJ BR')
                            else: # TUTAJ POBIERA SI INFORMACJE O WYMAGANYCH POLACH DLA DANEGO TYPU ABSENCJI, JELI KIEDY
                                    # Z JAKIEGO POWODU KOLEJNO KOLUMN ULEGNIE ZMIANIE, TO SPRAWDZENIE PRZESTANIE DZIAA.
                                spr ="SELECT zus_wymagany, dostarczono, wystawiono, czy_glowna, czy_nieobecnosc FROM rs_kp_absencje_rodzaje "
                                spr+="WHERE firma_id='"+str(self.firma_id)+"' AND typ_absencji_id = '"+wart+"' AND czy_aktywna;"  #MK 2020-05-06
                                stat, wymagania, wyn1, dump=self.PolSQL.Execute(spr, 3)
                                rsglobal.ZapiszDoLogow(spr, 5)
                                rsglobal.ZapiszDoLogow(wymagania, 5)
                                if len(wymagania)==0:
                                   #AB 26.10.18r
                                    wyn2.append((WierszeNr[i], kolGrid[pole]))
                                    dane.append('RODZAJ BL')

                                #WS 2023-12-22
                                if 'od_kiedy' not in Update[0][i] or 'do_kiedy' not in Update[0][i]:
                                    spr ="SELECT od_kiedy, do_kiedy FROM rs_kp_dane_osobowe_absencje "+WHERE+';'
                                    stat,d,dump,dump = self.PolSQL.Execute(spr, 3)
                                    if stat != 'OK':
                                        wyn2.append((WierszeNr[i], kolGrid[pole]))
                                        dane.append('Bd kontroli dat przy zmianie rodzaju absencji')
                                    else:
                                        od_kiedy = d[0][0]
                                        do_kiedy = d[0][1]
                                        if 'od_kiedy' not in Update[0][i]:
                                            if wymagania[0][4] and not rs_anakonda_daty.CzyRoboczyWgKalendarza(self, Pracownik, datetime.date(int(od_kiedy[0:4]), int(od_kiedy[5:7]), int(od_kiedy[8:10]))):
                                                wyn2.append((WierszeNr[i], kolGrid['od_kiedy']))
                                                dane.append('DATA OD BL ROB')
                                        if 'do_kiedy' not in Update[0][i]:
                                            if wymagania[0][4] and not rs_anakonda_daty.CzyRoboczyWgKalendarza(self, Pracownik, datetime.date(int(do_kiedy[0:4]), int(do_kiedy[5:7]), int(do_kiedy[8:10]))):
                                                wyn2.append((WierszeNr[i], kolGrid['do_kiedy']))
                                                dane.append('DATA DO BL ROB')

                        elif pole == 'numer_listy_id':
                            if len(wart.strip())!=0:
                                spr ="SELECT 1 FROM rs_kp_listy_plac "
                                spr+="WHERE firma_id='"+str(self.firma_id)+"' AND numer_listy_id = '"+wart+"' ;"
                                stat, lista, wyn1, dump=self.PolSQL.Execute(spr, 3)
                                rsglobal.ZapiszDoLogow(spr, 5)
                                rsglobal.ZapiszDoLogow(lista, 5)
                                if len(lista) == 0:
                                    wyn2.append((WierszeNr[i], kolGrid[pole]))
                                    dane.append('NUMER LISTY BL')

                        elif pole=='od_kiedy':
                            if  wart not in (None,'None') and wart.strip()<>'':
                                if not rsglobal.SprawdzCzyToData(wart.strip()):
                                                        #AB 26.10.18r
                                    wyn2.append((WierszeNr[i], kolGrid[pole]))
                                    dane.append('DATA OD BL')
                                else:
                                    Rok = int(wart[0:4])
                                    Mies = int(wart[5:7])
                                    Dzien = int(wart[8:10])
                                    #WS 2023-09-05 - if wymagania[0][4] (czy_nieobecnosc)
                                    if wymagania[0][4] and not rs_anakonda_daty.CzyRoboczyWgKalendarza(self, Pracownik, datetime.date(Rok, Mies, Dzien)):
                                        #AB 26.10.18r
                                        wyn2.append((WierszeNr[i], kolGrid[pole]))
                                        dane.append('DATA OD BL ROB')
                                    else:
                                        data_od = wart.strip()
                            else:
                                                        #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('DATA OD BR')

                        elif pole=='do_kiedy':
                            if wart not in (None,'None') and wart.strip()<>'':
                                if not rsglobal.SprawdzCzyToData(wart.strip()):
                                                        #AB 26.10.18r
                                    wyn2.append((WierszeNr[i], kolGrid[pole]))
                                    dane.append('DATA DO BL')
                                else:
                                    Rok = int(wart[0:4])
                                    Mies = int(wart[5:7])
                                    Dzien = int(wart[8:10])
                                    #WS 2023-09-05 - if wymagania[0][4] (czy_nieobecnosc)
                                    if wymagania[0][4] and not rs_anakonda_daty.CzyRoboczyWgKalendarza(self, Pracownik, datetime.date(Rok, Mies, Dzien)):
                                        wyn2.append((WierszeNr[i], kolGrid[pole]))
                                        dane.append('DATA DO BL ROB')
                            else:
                                                        #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('DATA DO BR')

                        elif pole=='data_wyplaty':
                            if wart not in puste:
                                if not rsglobal.SprawdzCzyToData(wart.strip()):
                                                        #AB 26.10.18r
                                    wyn2.append((WierszeNr[i], kolGrid[pole]))
                                    dane.append('DATA WYPL BL')

                        elif pole=='data_dostarczenia':
                            if wart not in (None,'None') and wart.strip()<>'':
                                if not rsglobal.SprawdzCzyToData(wart.strip()):
                                                        #AB 26.10.18r
                                    wyn2.append((WierszeNr[i], kolGrid[pole]))
                                    dane.append('DATA DOST BL')
                            else:
                                                        #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('DATA DOST BR')

                        elif pole=='data_wystawienia':
                            if wart not in (None,'None') and wart.strip()<>'':
                                if not rsglobal.SprawdzCzyToData(wart.strip()):
                                                        #AB 26.10.18r
                                    wyn2.append((WierszeNr[i], kolGrid[pole]))
                                    dane.append('DATA WYST BL')
                            else:
                                                        #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('DATA WYST BR')

                        elif pole== 'kod_zus':
                            if len(wart) >6:
                                                        #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('ZUS BL')

                        elif pole == 'rodzina_id' and wart != '':
                            try:
                                wart = int(float(wart))

                                SPRRODZ = "SELECT * FROM rs_kp_rodzina "
                                SPRRODZ += " WHERE firma_id = '{firma}' AND pracownik_id = '{prac}' AND rodzina_id={rodz}"
                                SPRRODZ = SPRRODZ.format(firma=self.firma_id, prac=Pracownik, rodz=wart)
                                stat, rodz, dump, dump = self.PolSQL.Execute(SPRRODZ, 3)
                                if stat == 'OK' and len(rodz) == 0:
                                                        #AB 26.10.18r
                                    wyn2.append((WierszeNr[i], kolGrid[pole]))
                                    dane.append('RODZ_BL')
                            except ValueError:
                                import traceback
                                rsglobal.ZapiszDoLogow(str(traceback.format_exc()))
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('RODZ_BL')

                        elif pole == 'dni_kalendarzowe':
                            if len(str(int(wart)))>4:
                                                        #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('KAL BL')
                            elif wart == 0:
                                                        #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid['od_kiedy']))
                                wyn2.append((WierszeNr[i], kolGrid['do_kiedy']))
                                dane.append('KAL BR')

                        elif pole == 'dni_robocze':
                            if int(wart)>9999:
                                                        #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('ROBOCZE DNI BL')
                            elif wart == 0:
                                                        #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('ROBOCZE DNI BR')

                        elif pole == 'procent_wyplaty':
                            if float(wart) < 0 or float(wart) > 100:
                                                        #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('PROCENT BL')

                        elif pole == 'godziny_robocze':
                            if len(str(int(wart)))>6:
                                                        #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('ROBOCZE GODZ BL')
                        #WS 2019-06-03
                        elif pole == 'miesiace':
                            for w in wart.strip().split('/'):
                                if w != '':
                                    if not w.isdigit() or not int(w) <= 12:
                                        wyn2.append((WierszeNr[i], kolGrid[pole]))
                                        dane.append('MIESIACE BL')

                if stat == 'OK':
                    if len(dane) > 0:
                        stat = dane[0]

                EdycjaPodstaw = False #WS 2019-06-03

                # wykonanie UPDATE
                if stat=='OK':
                    sql=''
                    UPDATE="UPDATE rs_kp_dane_osobowe_absencje "
                    SET='SET '
                    for j in xrange(len(Update[0][i])): 
                        pole=Update[0][i][j]
                        wart=Update[1][i][j]

                        if pole in ['typ_absencji_id','od_kiedy','do_kiedy', 'numer_listy_id', 'data_wyplaty',
                                    'data_dostarczenia','data_wystawienia', 'kod_zus',
                                    'procent_wyplaty','kontynuacja', 'kontynuacja_data',
                                    #WS 2019-10-08                                         #AB 29.06.20r
                                    'podstawa','zasilek_dzien','zasilek_razem','miesiace', 'na_zadanie', 'dni_zalegle',
                                    'godziny_zalegle','dni_tegoroczne','godziny_tegoroczne', 'dni_okr', 'dni_rok']: #BP 2022-01-20
                            #AB 26.09.18r
                            if str(wart).strip() in puste:
                                SET+=pole+"=NULL,"
                            else:
                                SET+=pole+"='"+str(wart)+"',"

                        elif pole in ['dni_kalendarzowe','dni_robocze',
                                                                         #AB 26.10.18r
                                    'godziny_robocze','dni_okr', 'dni_rok', 'dni_szpital',
                                    #WS 2019-10-08
                                    'lmcy']:
                            SET+=pole+"='"+str(int(wart))+"',"
                        elif pole in ['rodzina_id']:
                            if wart.strip() in ('0', '0.0', ''):
                                SET += pole + "=NULL,"
                            else:
                                wart = int(float(wart))
                                SET+=pole+"="+str(wart)+","

                        if pole in ['podstawa','lmcy','zasilek_dzien','zasilek_razem','miesiace', 'zmienil_kiedy']:
                            EdycjaPodstaw = True

                    #WS 2019-06-03
                    if EdycjaPodstaw:
                        if not UprPZasilkow:
                            stat = 'UPR BR P ZAS'
                            dane.append(stat)
                        else:
                            #WS 2019-06-10
                            for j in xrange(len(Update[0][i])):
                                pole=Update[0][i][j]
                                wart=Update[1][i][j]

                                if pole in ['podstawa','lmcy','zasilek_dzien','zasilek_razem','procent_wyplaty','miesiace','zmienil_kiedy']:
                                    if wart in ('', 0.0):
                                        stat = 'P ZAS PUSTE'

                            if stat == 'OK':
                                SET+="zmienil_kto='"+str(self.operator_id)+"',"
                                SET+="zmienil_kiedy=now()"
                            else:
                                dane.append(stat)

                    if stat == 'OK':
                        SET=SET.strip(",")
                        sql+=UPDATE+SET+WHERE
                        stat,dump,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(sql),1,CzyDodawacBlad=True)
                        if stat<>'OK':
                            stat = "Bd przy modyfikowaniu danych: "+stat
                            dane.append(stat)


#-------------------------------------------------------------------------------
#---   INSERT (1)
#-------------------------------------------------------------------------------

if Funkcja == 1 and stat == 'OK' and UprIns:
    if len(Insert[0]) > 0 and Insert[0][0][kolGrid['typ_absencji_id']] != '':
        if stat == 'OK':
            WierszeNr = Insert[-1]

            for i in xrange(len(Insert[0])):
                for j in xrange(len(Insert[0][i])): 
                    
                    pole=Insert[1][j]
                    wart=Insert[0][i][j]

                    if pole == 'typ_absencji_id':
                        if len(wart.strip())==0:
                                                    #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('RODZAJ BR')
                        else: # TUTAJ POBIERA SI INFORMACJE O WYMAGANYCH POLACH DLA DANEGO TYPU ABSENCJI, JELI KIEDY
                                # Z JAKIEGO POWODU KOLEJNO KOLUMN ULEGNIE ZMIANIE, TO SPRAWDZENIE PRZESTANIE DZIAA.
                                                                            #AB 18.09.18r   #WS 2023-09-05
                            spr ="SELECT zus_wymagany, dostarczono, wystawiono, czy_glowna, czy_nieobecnosc FROM rs_kp_absencje_rodzaje "
                            spr+="WHERE firma_id='"+str(self.firma_id)+"' AND typ_absencji_id = '"+wart+"' AND czy_aktywna;" #MK 2020-05-06
                            stat, wymagania, wyn1, dump=self.PolSQL.Execute(spr, 3)
                            rsglobal.ZapiszDoLogow(spr, 5)
                            rsglobal.ZapiszDoLogow(wymagania, 5)
                            if len(wymagania)==0:
                               #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('RODZAJ BL')

                    elif pole == 'numer_listy_id':
                        if len(wart.strip())!=0:
                            spr ="SELECT 1 FROM rs_kp_listy_plac "
                            spr+="WHERE firma_id='"+str(self.firma_id)+"' AND numer_listy_id = '"+wart+"' ;"
                            stat, lista, wyn1, dump=self.PolSQL.Execute(spr, 3)
                            rsglobal.ZapiszDoLogow(spr, 5)
                            rsglobal.ZapiszDoLogow(lista, 5)
                            if len(lista) == 0:
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('NUMER LISTY BL')

                    elif pole=='od_kiedy':
                        if wart not in (None,'None') and wart.strip()<>'':
                            if not rsglobal.SprawdzCzyToData(wart.strip()):
                                                    #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('DATA OD BL')
                            else:
                                Rok = int(wart[0:4])
                                Mies = int(wart[5:7])
                                Dzien = int(wart[8:10])
                                #WS 2023-09-05 - if wymagania[0][4] (czy_nieobecnosc)
                                if wymagania[0][4] and not rs_anakonda_daty.CzyRoboczyWgKalendarza(self, Pracownik, datetime.date(Rok, Mies, Dzien)):
                                    # AB 26.10.18r
                                    wyn2.append((WierszeNr[i], kolGrid[pole]))
                                    dane.append('DATA OD BL ROB')
                        else:
                                                    #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('DATA OD BR')

                    elif pole=='do_kiedy':               
                        if wart not in (None,'None') and wart.strip()<>'':
                            if not rsglobal.SprawdzCzyToData(wart.strip()):
                                                    #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('DATA DO BL')
                            else:
                                Rok = int(wart[0:4])
                                Mies = int(wart[5:7])
                                Dzien = int(wart[8:10])
                                #WS 2023-09-05 - if wymagania[0][4] (czy_nieobecnosc)
                                if wymagania[0][4] and not rs_anakonda_daty.CzyRoboczyWgKalendarza(self, Pracownik, datetime.date(Rok, Mies, Dzien)):
                                    wyn2.append((WierszeNr[i], kolGrid[pole]))
                                    dane.append('DATA DO BL ROB')
                        else:
                                                    #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('DATA DO BR')

                    elif pole=='data_wyplaty':
                        if wart not in puste:
                            if not rsglobal.SprawdzCzyToData(wart.strip()):
                                #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('DATA WYPL BL')

                    elif pole=='data_dostarczenia':     
                        if wart not in (None,'None') and wart.strip()<>'':
                            if not rsglobal.SprawdzCzyToData(wart.strip()):
                                                    #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('DATA DOST BL')
                        else:
                                                    #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('DATA DOST BR')

                    elif pole=='data_wystawienia':     
                        if wart not in (None,'None') and wart.strip()<>'':
                            if not rsglobal.SprawdzCzyToData(wart.strip()):
                                                    #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('DATA WYST BL')
                        else:
                                                    #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('DATA WYST BR')

                    elif pole=='kod_zus':
                        if len(wart) > 7:
                                                    #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('ZUS BL')

                    elif pole == 'rodzina_id' and wart != '':
                        try:
                            wart = int(float(wart))

                            SPRRODZ = "SELECT * FROM rs_kp_rodzina "
                            SPRRODZ += " WHERE firma_id = '{firma}' AND pracownik_id = '{prac}' AND rodzina_id={rodz}"
                            SPRRODZ = SPRRODZ.format(firma=self.firma_id, prac=Pracownik, rodz=wart)
                            stat, rodz, dump, dump = self.PolSQL.Execute(SPRRODZ, 3)
                            if stat == 'OK' and len(rodz) == 0:
                                                    #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('RODZ_BL')
                        except ValueError:
                            import traceback
                            rsglobal.ZapiszDoLogow(str(traceback.format_exc()))
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('RODZ_BL')

                    elif pole =='dni_kalendarzowe':
                        if len(str(int(wart)))>4:
                                                    #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('KAL BL')
                        elif wart == 0:
                                                    #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid['od_kiedy']))
                            wyn2.append((WierszeNr[i], kolGrid['do_kiedy']))
                            dane.append('KAL BR')

                    elif pole =='dni_robocze':
                        if int(wart)>9999:
                                                    #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('ROBOCZE DNI BL')
                        elif wart == 0:
                                                    #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('ROBOCZE DNI BR')

                    elif pole == 'procent_wyplaty':
                        if float(wart) < 0 or float(wart) > 100:
                                                    #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('PROCENT BL')

                    elif pole =='godziny_robocze':
                        if len(str(int(wart)))>6:
                                                    #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('ROBOCZE GODZ BL')
                    #WS 2019-06-03
                    elif pole == 'miesiace':
                        for w in wart.strip().split('/'):
                            if w != '':
                                if not w.isdigit() or not int(w) <= 12:
                                    wyn2.append((WierszeNr[i], kolGrid[pole]))
                                    dane.append('MIESIACE BL')
        if stat == 'OK':
            if len(dane) > 0:
                stat = dane[0]

        # wykonanie INSERT
        if stat == 'OK':    
            sqlIns=''
            for i in xrange(len(Insert[0])):
                INSERT="INSERT INTO rs_kp_dane_osobowe_absencje"
                KOLUMNY=" (firma_id, pracownik_id"
                VALUES=" VALUES ('"+str(self.firma_id)+"', '"+str(Pracownik)+"'"
                for j in xrange(len(Insert[0][i])):
                    pole=Insert[1][j]
                    wart=Insert[0][i][j]
                                                                        #AB 29.06.20r
                    if pole in ['pracownik_id','od_kiedy','kontynuacja', 'na_zadanie', 'numer_listy_id', 'data_wyplaty', 'miesiace']:
                        KOLUMNY+=","+pole
                        VALUES+=', '+rsglobal.ZmienNaNULL(wart)  #BP 2022-01-20

                    elif pole in ['do_kiedy','kod_zus','typ_absencji_id','data_wystawienia',
                                  'data_dostarczenia','dni_kalendarzowe',
                                  'dni_robocze','procent_wyplaty',
                                                                                             #AB 26.10.18r
                                  'godziny_robocze','dni_okr', 'dni_rok', 'kontynuacja_data', 'dni_szpital', 'dni_zalegle',
                                    'godziny_zalegle','dni_tegoroczne','godziny_tegoroczne', 'dni_okr', 'dni_rok']: #BP 2022-01-20

                        KOLUMNY+=","+pole
                        #AB 26.09.18r
                        if str(wart).strip() in puste:
                            VALUES+=","+str('NULL')
                        else:
                            VALUES+=", '"+str(wart)+"'"
                    elif pole in ['rodzina_id']:
                        KOLUMNY+=", "+pole
                        if wart.strip() in ('0', '0.0', ''):
                            VALUES+=", NULL"
                        else:
                            VALUES+=", " + wart

                KOLUMNY+=")"
                VALUES+=");"
                sqlIns+=INSERT+KOLUMNY+VALUES
            if sqlIns<>'':
                stat,dump,dump,dump=self.PolSQL.Execute(rsglobal.Dekoduj(sqlIns),1)
                if stat<>'OK':
                    rsglobal.ZapiszDoLogow(rsglobal.Dekoduj(sqlIns)) #BP 2022-01-20
                    stat = "Bd przy dodawaniu danych: "+stat
                    dane.append(stat)  


#-------------------------------------------------------------------------------
#---   Sprawdzenie pokrywania si dat
#-------------------------------------------------------------------------------

if Funkcja == 1 and stat == 'OK':
    sql ="Select DISTINCT a.typ_absencji_id, a.od_kiedy, b.od_kiedy, b.do_kiedy From rs_kp_dane_osobowe_absencje a, rs_kp_dane_osobowe_absencje b "
    sql+="WHERE a.firma_id='"+self.firma_id+"' AND a.pracownik_id='"+str(Pracownik)+"' AND a.firma_id=b.firma_id AND a.pracownik_id=b.pracownik_id "
    sql+="AND (a.od_kiedy BETWEEN b.od_kiedy and b.do_kiedy) AND a.od_kiedy!=b.od_kiedy;"
    stat,D,wyn1,dump=self.PolSQL.Execute(sql,3)
    if stat=='OK' and len(D)>0:
        stat="Istniej pokrywajce si daty w pozycjach: "
        for i in xrange(len(D)):
            rodz=str(D[i][0])
            dataA=str(D[i][1])
            dataB=str(D[i][2])
            dataC=str(D[i][3])
            stat+= u"\nRodzaj: "+str(rodz).strip()+u", data od: "+str(dataA)+"\n\tpokrywa si z zakresem od "+str(dataB)+" do "+str(dataC)
        wyn2.append((0,1))
        D.append(stat)  


#-------------------------------------------------------------------------------
#---   POBRANIE DANYCH (1,5)
#-------------------------------------------------------------------------------

if stat=='OK' and (Funkcja in [1,5]):
    sql ="SELECT p.typ_absencji_id, r.nazwa, od_kiedy, do_kiedy,"
                                                                        #AB 26.10.18r
    sql+="data_dostarczenia, data_wystawienia, coalesce(p.kod_zus, ' '), dni_szpital, procent_wyplaty, kontynuacja, kontynuacja_data, rodzina_id::TEXT, dni_kalendarzowe, dni_robocze, "
                         #AB 29.06.20r
    sql+="godziny_robocze, na_zadanie, dni_zalegle, godziny_zalegle, dni_tegoroczne, godziny_tegoroczne, dni_okr, dni_rok,"
    #AB 02.03.22r
    sql+=""" CASE WHEN podstawa != 0 
        THEN podstawa
    ELSE
        (SELECT podstawa FROM 
            (SELECT podstawa, od_kiedy 
             FROM rs_kp_dane_osobowe_absencje pp
             WHERE pp.firma_id = p.firma_id AND podstawa != 0 AND pp.od_kiedy < p.od_kiedy 
                AND pp.pracownik_id = p.pracownik_id 
                AND pp.zasilek_dzien = p.zasilek_dzien 
            UNION 
            SELECT 0 AS podstawa, '1970-01-01' AS od_kiedy) x
        ORDER BY od_kiedy DESC
        LIMIT 1)
    END AS podstawa, """
    #WS 2019-05-31
    sql+="lmcy, zasilek_dzien, zasilek_razem, numer_listy_id, data_wyplaty, r.proc_zwolnienia, miesiace, podstawa_obliczenia, zmienil_kto, zmienil_kiedy "
    # sql+="lmcy, zasilek_dzien, zasilek_razem, numer_listy_id, data_wyplaty, r.proc_zwolnienia, miesiace, podstawa_obliczenia "
    sql+="FROM rs_kp_dane_osobowe_absencje p JOIN rs_kp_dane_osobowe o ON p.firma_id=o.firma_id AND p.pracownik_id = o.pracownik_id "
    sql+="JOIN rs_kp_absencje_rodzaje r ON p.typ_absencji_id = r.typ_absencji_id AND p.firma_id = r.firma_id "
    sql+=" WHERE p.firma_id='"+self.firma_id+"' AND p.pracownik_id='"+str(Pracownik)+"' "
    sql+=" AND p.typ_absencji_id like '"+str(TypAbsencjiF)+"%' "
    # data od bez daty do
    if (DataOdF != '    -  -  ' and rsglobal.SprawdzCzyToData(DataOdF)) and not (DataDoF != '    -  -  ' and rsglobal.SprawdzCzyToData(DataDoF)):
        sql+=" AND do_kiedy >= '" + DataOdF + "'"
    # data do bez daty od
    if (DataDoF != '    -  -  ' and rsglobal.SprawdzCzyToData(DataDoF)) and not (DataOdF != '    -  -  ' and rsglobal.SprawdzCzyToData(DataOdF)):
        sql+=" AND od_kiedy <= '" + DataDoF + "'"
    # data do i data od
    if (DataDoF != '    -  -  ' and rsglobal.SprawdzCzyToData(DataDoF)) and (DataOdF != '    -  -  ' and rsglobal.SprawdzCzyToData(DataOdF)):
        #AB 03.07.20r
        #         sql += " AND ( '{od}' BETWEEN p.od_kiedy AND p.do_kiedy OR '{do}' BETWEEN od_kiedy AND do_kiedy OR p.do_kiedy IS NULL AND p.od_kiedy <= '{do}')".format(od=DataOdF, do=DataDoF)
        sql += " AND ( '{od}' BETWEEN p.od_kiedy AND p.do_kiedy OR '{do}' BETWEEN od_kiedy AND do_kiedy OR p.do_kiedy between '{od}' and '{do}' or p.od_kiedy between '{od}' and '{do}')".format(od=DataOdF, do=DataDoF)
    if TylkoUrlopyF:  #MK 2020-05-06
        sql += " AND r.czy_nieobecnosc='TRUE'"
    elif TylkoZwolnieniaF: #MK 2020-05-06
        sql += " AND (r.czy_zwolnienie='TRUE' OR r.nieobecnosc_inna='TRUE')"
    if dodane != []:
        sql += " OR p.firma_id='"+self.firma_id+"' AND p.pracownik_id='"+str(Pracownik)+"' AND ("+''.join(["p.typ_absencji_id = '%s' AND od_kiedy = '%s' OR "%(wiersz[0], wiersz[1]) for wiersz in dodane])[:-3] +") "

    if TylkoUrlopyF:  #AB 17.02.23r
        sql += """UNION
                  SELECT u.urlop_id AS typ_absencji_id, r2.nazwa, u.od_dnia AS od_kiedy, NULL AS do_kiedy, NULL AS data_dostarczenia, NULL AS data_wystawienia, '' AS kod_zus, NULL AS dni_szpital, NULL AS procent_wyplaty, NULL AS kontynuacja, NULL AS kontynuacja_data, 
                  NULL AS rodzina_id, NULL AS dni_kalendarzowe, NULL AS dni_robocze, NULL AS godziny_robocze, NULL AS na_zadanie, u.limit_dni_zalegle, u.limit_godziny_zalegle, u.limit_dni AS dni_tegoroczne, u.limit_godziny AS godziny_tegoroczne, NULL AS dni_okr, 
                  NULL AS dni_rok, NULL AS podstawa, NULL AS lmcy, NULL AS zasilek_dzien, NULL AS zasilek_razem, 'BO LIMITY' AS numer_listy_id, NULL AS data_wyplaty, NULL AS proc_zwolnienia, NULL AS miesiace, NULL AS podstawa_obliczenia, NULL AS zmienil_kto, 
                  NULL AS zmienil_kiedy 
                  FROM rs_kp_dane_osobowe_limity_urlopow u
                  JOIN rs_kp_absencje_rodzaje r2 ON u.urlop_id = r2.typ_absencji_id AND u.firma_id = r2.firma_id  
                  WHERE u.firma_id = '{}' AND u.pracownik_id='{}' and r2.czy_nieobecnosc
                  ORDER BY od_kiedy DESC """.format(self.firma_id, Pracownik)
    sql+=";"

    rsglobal.ZapiszDoLogow(sql, 5)
    stat,dane,wyn1,dump=self.PolSQL.Execute(sql,3)
    if stat<>'OK':
        stat="Bd przy pobieraniu danych : "+stat   
        dane.append(stat)