# -*- coding: iso8859_2 -*-

# -----------------------------------------------------------------------------
# Name:        fs_kp_dane_osobowe_ppk.py
# Purpose:     Obsuga grida z danymi pracownika do PPK
# Author:      Wojciech Skowron
# Created:     2020/10/01
# RCS-ID:      $Id: fs_kp_dane_osobowe_ppk.py $
# Copyright:   (c) 2020 RecordSystem
# Licence:     GNU GPL
# -----------------------------------------------------------------------------
# 2020-10-12 Aleksander Bargie - (12163) PPK - automatyczne uzupelnianie wysokosci skladek podstawowych
# 2021-03-05 Aleksander Bargie - (12376) Rozwizanie stosunku pracy - rezygnacja z PPK

import rsglobal

"""
052250 - PPK
    052251 - Przegldanie
    052252 - Modyfikacja
    052253 - Dodawanie
    052254 - Usuwanie
"""

"""Funkcje:
1. Update, Delete i Insert z RSEdit
5. Pobranie danych"""

stat = "OK"

if 'Funkcja' not in klucze:
    stat = 'Brak podanej funkcji w kluczach'

if stat == 'OK':
    Funkcja = klucze['Funkcja']

    SlPar = {'Pracownik': [1,5]}
    Lista = rsglobal.SprawdzParametryWKluczach(SlPar, Funkcja, klucze)
    if Lista != '':
        stat = 'Uwaga : wywoanie funkcji fs_kp_dane_osobowe_ppk z numerem ' + str(Funkcja)
        stat += ' bez parametrw: ' + Lista

dane = []
wyn1 = []
wyn2 = []

#-------------------------------------------------------------------------------
#---   UPRAWNIENIA
#-------------------------------------------------------------------------------

UprDel = ((self.oddzial_id,'052254') in self.uprawnienia) or (self.operator_id == STANDARD_OPERATOR_ID)
UprIns = ((self.oddzial_id,'052253') in self.uprawnienia) or (self.operator_id == STANDARD_OPERATOR_ID)
UprUpd = ((self.oddzial_id,'052252') in self.uprawnienia) or (self.operator_id == STANDARD_OPERATOR_ID) or UprIns
UprPrzegl = ((self.oddzial_id,'052251') in self.uprawnienia) or (self.operator_id == STANDARD_OPERATOR_ID) or UprIns or UprUpd or UprDel

#-------------------------------------------------------------------------------
#-------- EDYCJA GRIDA EDYTOWALNEGO (1)
#-------------------------------------------------------------------------------

Insert = []
Update = []
Delete = []

if stat == 'OK' and Funkcja == 1:
    Update = klucze['Update']
    Insert = klucze['Insert']
    Delete = klucze['Delete']

if stat == 'OK' and Funkcja in (1,5):
    Pracownik = klucze['Pracownik']

#-------------------------------------------------------------------------------
#-----------------------------   DELETE (1) ------------------------------------
#-------------------------------------------------------------------------------

if stat == 'OK' and Funkcja == 1:
    if len(Delete[0]) > 0:
        if not UprDel:
            stat = 'BR_UPR_DEL'
            dane.append(stat)

        if stat == 'OK':
            DEL = ''
            pola = Delete[1]

            for wartosci in Delete[0]:
                DEL += """DELETE FROM rs_kp_dane_osobowe_ppk WHERE firma_id='{}' AND pracownik_id='{}'
                       """.format(self.firma_id, Pracownik)
                for pole, wart in zip(pola, wartosci):
                    DEL += " AND "+pole+"="+rsglobal.ZmienNaNULL(wart)

                DEL += ";"

            if stat == "OK" and DEL != '':
                rsglobal.ZapiszDoLogow(DEL)
                stat,dump,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(DEL),1,CzyDodawacBlad=True)
                if stat != 'OK':
                    stat = "Bd przy usuwaniu danych: "+stat
                    dane.append(stat)

#-------------------------------------------------------------------------------
#---------------------------  SPRAWDZENIE DANYCH  ------------------------------
#-------------------------------------------------------------------------------


def SprawdzPole(pole, wart, nrWiersza, dane, wyn2):
    if pole == 'data_przystapienia':
        if wart.strip() in ('', '-  -'):
            dane.append('BR_DATA_PRZYST')
            wyn2.append((nrWiersza, 0))

    if pole == 'skladka_prac_podst':
        if wart < 0:
            dane.append('BL_MIN_WART_PRAC_PODST')
            wyn2.append((nrWiersza, 2))
        elif wart > 999.99:
            dane.append('BL_MAX_WART_PRAC_PODST')
            wyn2.append((nrWiersza, 2))

    if pole == 'skladka_prac_dodat':
        if wart < 0:
            dane.append('BL_MIN_WART_PRAC_DODAT')
            wyn2.append((nrWiersza, 3))
        elif wart > 999.99:
            dane.append('BL_MAX_WART_PRAC_DODAT')
            wyn2.append((nrWiersza, 3))

    if pole == 'skladka_firma_podst':
        if wart < 0:
            dane.append('BL_MIN_WART_FIRMA_PODST')
            wyn2.append((nrWiersza, 4))
        elif wart > 999.99:
            dane.append('BL_MAX_WART_FIRMA_PODST')
            wyn2.append((nrWiersza, 4))

    if pole == 'skladka_firma_dodat':
        if wart < 0:
            dane.append('BL_MIN_WART_FIRMA_DODAT')
            wyn2.append((nrWiersza, 5))
        elif wart > 999.99:
            dane.append('BL_MAX_WART_FIRMA_DODAT')
            wyn2.append((nrWiersza, 5))


#-------------------------------------------------------------------------------

if stat == 'OK' and Funkcja == 1:
    if len(Update[0]) > 0:
        if not UprUpd:
            stat = 'BR_UPR_UPD'
            dane.append(stat)
        else:
            for pola, wartosci, nrWiersza in zip(Update[0], Update[1], Update[-1]):
                for pole, wart in zip(pola, wartosci):
                    SprawdzPole(pole, wart, nrWiersza, dane, wyn2)

    if len(Insert[0]) > 0:
        if not UprIns:
            stat = 'BR_UPR_INS'
            dane.append(stat)
        else:
            pola = Insert[1]

            for wartosci, nrWiersza in zip(Insert[0], Insert[-1]):
                for pole, wart in zip(pola, wartosci):
                    SprawdzPole(pole, wart, nrWiersza, dane, wyn2)

    if stat == 'OK':
        if len(dane) > 0:
            stat = dane[0]

    if len(dane) > 0:
        dane = list(set(dane))  # usunicie zduplikowanych komunikatw bdw

#-------------------------------------------------------------------------------
#-------------------------------  UPDATE (1)  ----------------------------------
#-------------------------------------------------------------------------------

if stat == 'OK' and Funkcja == 1 and len(Update[0]) > 0:
    UPD = ''

    for pola, wartosci, klucz in zip(Update[0], Update[1], Update[2]):
        UPD += "UPDATE rs_kp_dane_osobowe_ppk SET "
        for pole, wart in zip(pola, wartosci):
            if pole in ['skladka_obnizona']:  #AB 12.10.20r
                if isinstance(wart, (basestring)):
                    UPD += pole+"="+str(wart)+","
                else:
                    UPD += pole+"="+str(bool(wart))+","

            elif pole not in ['data_przystapienia_wyslanie', 'data_rezygnacji_wyslanie']:
                if isinstance(wart, (int, long, float)):
                    UPD += pole+"="+str(wart)+","
                else:
                    UPD += pole+"="+rsglobal.ZmienNaNULL(wart.strip())+","

        UPD = UPD.strip(",")
        UPD += " WHERE firma_id='"+self.firma_id+"' AND pracownik_id='"+Pracownik+"'"

        for pole, wart in zip(Update[3], klucz):  # petla po kluczu
            UPD += " AND "+pole+"='"+str(rsglobal.Dekoduj(wart))+"'"

        UPD += ";"

    if UPD != '':
        rsglobal.ZapiszDoLogow(UPD)
        stat, dump, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(UPD), 1, CzyDodawacBlad=True)
        if stat != 'OK':
            stat = "Bd przy modyfikowaniu danych: "+stat
            dane.append(stat)

#-------------------------------------------------------------------------------
#------------------------------  INSERT (1)  -----------------------------------
#-------------------------------------------------------------------------------

if stat == 'OK' and Funkcja == 1 and len(Insert[0]) > 0:
    INS = ''
    pola = Insert[1]

    for wartosci in Insert[0]:
        INSERT  = "INSERT INTO rs_kp_dane_osobowe_ppk "
        KOLUMNY = "(firma_id,pracownik_id"
        VALUES  = " VALUES ('"+self.firma_id+"','"+Pracownik+"'"
        for pole, wart in zip(pola, wartosci):
            if pole in ['skladka_obnizona']:  #AB 12.10.20r
                KOLUMNY += ","+pole
                if isinstance(wart, (basestring)):
                    VALUES += ","+str(wart)
                else:
                    VALUES += ","+str(bool(wart))

            elif pole not in ['data_przystapienia_wyslanie', 'data_rezygnacji_wyslanie']:
                if isinstance(wart, (int, long, float)):
                    KOLUMNY += ","+pole
                    VALUES += ","+str(wart)
                else:
                    KOLUMNY += ","+pole
                    VALUES += ","+rsglobal.ZmienNaNULL(wart.strip())

        KOLUMNY += ")"
        VALUES += ");"
        INS += INSERT+KOLUMNY+VALUES

    if INS != '':
        rsglobal.ZapiszDoLogow(INS)
        stat, dump, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(INS), 1, CzyDodawacBlad=True)
        if stat != 'OK':
            stat = "Bd przy dodawaniu danych: "+stat
            dane.append(stat)

#-------------------------------------------------------------------------------
#---   POBRANIE DANYCH (1- insert 1 -select, 1 -delete, 5 -select)
#-------------------------------------------------------------------------------

if stat == 'OK' and Funkcja in (1, 5):
    if not UprPrzegl:
        stat = 'BR_UPR_PRZ'
        dane.append(stat)
    else:  #AB 05.03.21r
        SEL = """SELECT data_przystapienia, data_rezygnacji, data_konca_zatrudnienia, 
                        skladka_prac_podst, skladka_prac_dodat, skladka_firma_podst, skladka_firma_dodat, skladka_obnizona, 
                        ppk_pfr_id, ppk_tfi_id, data_przystapienia_wyslanie, data_rezygnacji_wyslanie
                 FROM rs_kp_dane_osobowe_ppk WHERE firma_id='{}' AND pracownik_id='{}'
                 ORDER BY data_przystapienia DESC;""".format(self.firma_id, Pracownik)
        stat,dane,wyn1,wyn2 = self.PolSQL.Execute(SEL,3,CzyDodawacBlad=True)
        if stat != 'OK':
            stat = "Bd przy pobieraniu danych: "+stat
            dane.append(stat)
