# -*- coding: iso8859_2 -*-
#-----------------------------------------------------------------------------
# Name:        fs_kp_deklaracje_us.py
# Purpose:     Generowanie PIT-11 (13154)
# Author:      Wojciech Skowron
# Created:     2022/11/16
# RCS-ID:      $Id: fs_kp_deklaracje_us.py $
# Copyright:   (c) 2022 Record System
# Licence:     GNU GPL
#-----------------------------------------------------------------------------

# 2022-12-30 Wojciech Skowron - (13283) Zbiorcze generowanie xml
# 2023-01-16 Wojciech Skowron - (13250) Imi i nazwisko operatora w polu 124
# 2023-01-17 Wojciech Skowron - (13313) Poprawka z kodowaniem imienia i nazwiska
# 2023-01-18 Wojciech Skowron - (13315) Parametr NrInformacji
# 2023-01-20 Wojciech Skowron - (13320) UPO w PDF
# 2024-01-16 Wojciech Skowron - (13667) Poprawka w zczeniach w zapytaniu pobierajacym adres

import os

from funkcje.fs_kp_deklaracje_us_pom import USXML, wykonaj_zapytanie
import rsglobal

stat = 'OK'
dane = []
wyn1 = []
wyn2 = []

if not 'Funkcja' in klucze:
    stat = 'Brak podanej funkcji w kluczach'

if stat == 'OK':
    Funkcja = klucze['Funkcja']

    SlPar = {'NumerListy': [1, 2, 3], 'Pracownik': [2], 'Zatrudnienie': [2], 'Rok': [1, 2, 3],
             'NrInformacji': [2, 3], 'CelZlozenia': [2, 3], 'Format': [2, 3], 'Pouczenie': [2, 3],
             'TrescUpoXml': [4]}
    Lista = rsglobal.SprawdzParametryWKluczach(SlPar, Funkcja, klucze)
    if Lista != '':
        stat = u'Uwaga : wywoanie funkcji fs_kp_deklaracje_us z numerem '+str(Funkcja)+u' bez parametrw: '+Lista

Upr = (((self.oddzial_id, '057200') in self.uprawnienia) or (self.operator_id == STANDARD_OPERATOR_ID))
if not Upr:
    stat = 'BR_UPR'

if stat == 'OK':
    if Funkcja in (1, 2, 3):
        NumerListy   = rsglobal.Dekoduj(klucze['NumerListy'])
        Rok          = rsglobal.Dekoduj(klucze['Rok'])

    if Funkcja == 2:
        Pracownik    = rsglobal.Dekoduj(klucze['Pracownik'])
        Zatrudnienie = rsglobal.Dekoduj(klucze['Zatrudnienie'])

    if Funkcja in (2, 3):
        NrInformacji = rsglobal.Dekoduj(klucze['NrInformacji'])  #WS 2023-01-18
        CelZlozenia  = rsglobal.Dekoduj(klucze['CelZlozenia'])
        Pouczenie    = rsglobal.Dekoduj(klucze['Pouczenie'])
        Format       = rsglobal.Dekoduj(klucze['Format'])

    #WS 2023-01-20
    if Funkcja == 4:
        TrescUpoXml  = rsglobal.Dekoduj(klucze['TrescUpoXml'])


class PIT11(USXML):
    def __init__(self, rodzic):
        self.PIT11Z_ADRESY = {
        'TNS': "http://crd.gov.pl/wzor/2022/09/12/11784/",
        'ZR': "http://crd.gov.pl/xml/schematy/dziedzinowe/mf/2022/03/15/eD/PITR/",
        'ETD': "http://crd.gov.pl/xml/schematy/dziedzinowe/mf/2022/03/15/eD/DefinicjeTypy/",
        'SCHEMALOCATION': "http://crd.gov.pl/wzor/2022/09/12/11784/ http://crd.gov.pl/wzor/2022/09/12/11784/schemat.xsd"}

        self.PIT11_ADRESY = {
        'TNS': "http://crd.gov.pl/wzor/2022/11/09/11890/",
        'ZR': "http://crd.gov.pl/xml/schematy/dziedzinowe/mf/2022/03/15/eD/PITR/",
        'ETD': "http://crd.gov.pl/xml/schematy/dziedzinowe/mf/2022/03/15/eD/DefinicjeTypy/",
        'SCHEMALOCATION': "http://crd.gov.pl/wzor/2022/11/09/11890/ http://crd.gov.pl/wzor/2022/11/09/11890/schemat.xsd"}

        from funkcje.fs_kp_deklaracje_us_pom import USXML
        USXML.__init__(self, rodzic)

    def ZapiszXML(self, dane, NrInformacji, CelZlozenia, Rok, Zbiorczy=False):
        try:
            self.WstawTekst('<tns:Deklaracja xmlns:tns="{TNS}" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:etd="{ETD}" xmlns:zr="{ZR}" xsi:schemaLocation="{SCHEMALOCATION}">'.format(**(self.PIT11Z_ADRESY if Zbiorczy else self.PIT11_ADRESY)))
            self.WstawNowy("tns:Naglowek")
            if Zbiorczy:
                self.WstawTekst('<tns:KodFormularza kodPodatku="PIT" kodSystemowy="PIT-11 (29)" rodzajZobowiazania="Z" wersjaSchemy="2-0E">PIT-11Z</tns:KodFormularza>')
            else:
                self.WstawTekst('<tns:KodFormularza kodPodatku="PIT" kodSystemowy="PIT-11 (29)" rodzajZobowiazania="Z" wersjaSchemy="1-1E">PIT-11</tns:KodFormularza>')
            self.WstawNowyZWartoscia("tns:WariantFormularza", '29')
            self.WstawTekst('<tns:CelZlozenia poz="P_7">'+str(CelZlozenia)+'</tns:CelZlozenia>')
            self.WstawNowyZWartoscia("tns:Rok", Rok)
            if Zbiorczy:
                self.WstawNowyZWartoscia("tns:KodUrzedu", '0000')
            else:
                self.WstawNowyZWartoscia("tns:KodUrzedu", dane[0]['urzad_skarbowy']['kod'])
            self.Zamknij("tns:Naglowek")

            self.WstawTekst('<tns:Podmiot1 rola="Patnik/Skadajcy">'.encode('utf-8'))
            self.WstawNowy("tns:OsobaNiefizyczna")
            self.WstawNowyZWartoscia("tns:NIP", dane[0]['oddzial']['nip'])
            self.WstawNowyZWartoscia("tns:PelnaNazwa", dane[0]['oddzial']['nazwa'])
            self.Zamknij("tns:OsobaNiefizyczna")
            self.Zamknij("tns:Podmiot1")

            for d in dane:
                if Zbiorczy:
                    self.WstawTekst('<tns:Pozycja typ="G">'.encode('utf-8'))
                    self.WstawNowyZWartoscia("tns:KodUrzeduP", d['urzad_skarbowy']['kod'])
                self.WstawTekst('<tns:Podmiot2 rola="Podatnik">')
                self.WstawNowy("tns:OsobaFizyczna")
                self.WstawNowyZWartoscia("tns:PESEL", d['osobowe']['pesel'])
                self.WstawNowyZWartoscia("tns:ImiePierwsze", d['osobowe']['imie'])
                self.WstawNowyZWartoscia("tns:Nazwisko", d['osobowe']['nazwisko'])
                self.WstawNowyZWartoscia("tns:DataUrodzenia", d['osobowe']['data_urodzenia'])
                if d['osobowe']['obywatelstwo_kraj'] != 'PL':
                    if d['osobowe']['pesel'] not in ('', 'None'):
                        self.WstawTekst('<tns:NrId poz="P_13">'+d['osobowe']['pesel']+'</tns:NrId>')
                    else:
                        self.WstawTekst('<tns:NrId poz="P_13">'+d['osobowe']['dokument_nr']+'</tns:NrId>')
                    self.WstawTekst('<tns:RodzajNrId poz="P_14">'+d['osobowe']['rodzaj_numeru_ident']+'</tns:RodzajNrId>')
                    self.WstawTekst('<tns:KodKrajuWydania poz="P_15A">'+d['osobowe']['obywatelstwo_kraj']+'</tns:KodKrajuWydania>')
                self.Zamknij("tns:OsobaFizyczna")
                self.WstawTekst('<tns:AdresZamieszkania rodzajAdresu="RAD">')
                self.WstawNowyZWartoscia('tns:KodKraju', d['adresowe']['kraj_id'], 'poz="P_19A"')
                if d['adresowe']['wojewodztwo'] != '':
                    self.WstawNowyZWartoscia('tns:Wojewodztwo', d['adresowe']['wojewodztwo'])
                if d['adresowe']['powiat'] != '':
                    self.WstawNowyZWartoscia('tns:Powiat', d['adresowe']['powiat'])
                if d['adresowe']['gmina'] != '':
                    self.WstawNowyZWartoscia('tns:Gmina', d['adresowe']['gmina'])
                if d['adresowe']['ulica'] != '':
                    self.WstawNowyZWartoscia('tns:Ulica', d['adresowe']['ulica'], 'poz="P_23"')
                if d['adresowe']['nr_domu'] != '':
                    self.WstawNowyZWartoscia('tns:NrDomu', d['adresowe']['nr_domu'], 'poz="P_24"')
                if d['adresowe']['nr_mieszkania'] != '':
                    self.WstawNowyZWartoscia('tns:NrLokalu', d['adresowe']['nr_mieszkania'], 'poz="P_25"')
                self.WstawNowyZWartoscia('tns:Miejscowosc', d['adresowe']['miejscowosc'], 'poz="P_26"')
                if d['adresowe']['kod_pocztowy'] != '':
                    self.WstawNowyZWartoscia('tns:KodPocztowy', d['adresowe']['kod_pocztowy'], 'poz="P_27"')
                self.Zamknij("tns:AdresZamieszkania")
                self.Zamknij("tns:Podmiot2")

                self.WstawNowy("tns:PozycjeSzczegolowe")
                self.WstawNowyZWartoscia("tns:P_5", NrInformacji)  #WS 2023-01-18
                self.WstawNowyZWartoscia("tns:P_11", '2' if d['osobowe']['nie_rezydent'] else '1')

                import collections

                dane_pol = collections.OrderedDict()
                for nr_pola, wart_skl in enumerate(d['skladnikow'][0]):
                    dane_pol[d['skladnikow'].GetFieldName(nr_pola)] = wart_skl
                    rsglobal.ZapiszDoLogow(d['skladnikow'].GetFieldName(nr_pola))

                if dane_pol['28_1'] == 1:
                    self.WstawNowyZWartoscia("tns:P_28", '1')
                elif dane_pol['28_3'] == 1:
                    self.WstawNowyZWartoscia("tns:P_28", '3')

                for pole, wart_skl in dane_pol.iteritems():
                    if pole not in ('28_1', '28_3'):
                        # checkboxy (podstawa prawna zwolnienia od podatku)
                        if pole in ('118', '119', '120'):
                            if '114' in dane_pol and wart_skl == 1:
                                self.WstawNowyZWartoscia("tns:P_"+str(pole), '1')
                        # czy doczony PIT-R
                        elif pole == '121':
                            self.WstawNowyZWartoscia("tns:P_"+str(pole), '1' if wart_skl == 1 else '2')
                        # kwoty cakowite
                        elif pole in ('33', '40', '57', '61', '65', '83', '86'):
                            self.WstawNowyZWartoscia("tns:P_"+str(pole), rsglobal.UsunKropkaZero(rsglobal.RSRound(wart_skl)))
                        # kwoty z gr
                        else:
                            self.WstawNowyZKwota("tns:P_"+str(pole), wart_skl)
                self.Zamknij("tns:PozycjeSzczegolowe")
                if Zbiorczy:
                    self.Zamknij("tns:Pozycja")

            self.WstawNowyZWartoscia("tns:Pouczenie", '1')
            self.Zamknij("tns:Deklaracja")
            self.ZakonczTworzenie()

            import re
            self.tresc = re.sub('>\s*<', '>\n<', self.tresc)
            if Zbiorczy:
                rsglobal.ZapiszDoLogow('Koniec tworzenia zbiorczego PIT11 w xml')
            else:
                rsglobal.ZapiszDoLogow('Koniec tworzenia PIT11 w xml dla pracownika '+dane[0]['osobowe']['pracownik_id'].strip())
        except:
            import traceback
            return traceback.format_exc(), self.PodajTresc()
        else:
            from datetime import datetime
            import tempfile

            from funkcje.fs_kp_deklaracje_us_pom import sprawdz_xml

            plik_tmp_nazwa = tempfile.mktemp('.xml')
            self.ZapiszPlik(plik_tmp_nazwa)
            stat = sprawdz_xml(plik_tmp_nazwa, 'xsd/pit11.xsd' if not Zbiorczy else 'xsd/pit11z.xsd')
            if stat != 'OK' and os.path.exists(plik_tmp_nazwa):
                rsglobal.ZapiszDoLogow('Bdy weryfikacji PIT11 w xml ze schematem')
                if Zbiorczy:
                    os.system('mv '+plik_tmp_nazwa+' '+'xsd/Blad_PIT11Z_'+str(Rok)+'_'+datetime.now().strftime("%Y_%m_%d_%H_%M_%S")+'.xml')
                else:
                    os.system('mv '+plik_tmp_nazwa+' '+'xsd/Blad_PIT11_'+str(Rok)+'_'+dane[0]['osobowe']['pracownik_id'].strip()+'_'+datetime.now().strftime("%Y_%m_%d_%H_%M_%S")+'.xml')

            return stat, self.PodajTresc()

    def ZapiszPDF(self, dane, NrInformacji, CelZlozenia, Rok):
        tresc = ''

        stat, xml = self.ZapiszXML(dane, NrInformacji, CelZlozenia, Rok)
        if stat == 'OK':
            try:
                xml = xml.replace('<?xml version="1.0" encoding="utf-8"?>', '')

                import pdfrw
                import zlib

                # trzeba wykona na pliku formularza cignitym ze strony, inaczej pdfrw go nie przeczyta
                # qpdf --password='' --decrypt input.pdf output.pdf
                reader = pdfrw.PdfReader('xsd/pit11.pdf')
                # podmiana xml zawartego w pdfie
                xml_dekl = zlib.decompress(reader.Root.AcroForm.XFA[7].stream)
                xml_dekl = self.ZnajdzFragment(xml_dekl, ['<Deklaracja '], '</Deklaracja\n>', zastap=xml)
                reader.Root.AcroForm.XFA[7].stream = zlib.compress(xml_dekl)
                # reader.Root.AcroForm.update(pdfrw.PdfDict(NeedAppearances=pdfrw.PdfObject('true')))

                xml_form = zlib.decompress(reader.Root.AcroForm.XFA[5].stream)

                # po podmianie xml w niektre pola z list rozwijan wpisuje si sam kod z xml zamiast odpowiadajcego
                # mu opisu, aby opis si wpisa trzeba przenie definicj pl listy rozwijanej bezporednio
                # do wza z definicj pola
                urzedy = self.ZnajdzFragment(xml_form, ['name="Zalaczniki"', 'name="KodUrzedu"', '<items\n>'], ['</items\n>']*2)
                kraje = self.ZnajdzFragment(xml_form, ['name="Podmiot2"', 'name="AdresZamieszkania"', 'name="Kraj"', '<items\n>'], ['</items\n>']*2)

                xml_form = self.ZnajdzFragment(xml_form, ['name="Strona1"', 'name="KodUrzedu"', '</validate\n>', ''], '', zastap=urzedy)
                xml_form = self.ZnajdzFragment(xml_form, ['name="Podmiot2"', 'name="P_14"', '</validate\n>', ''], '', zastap=kraje)

                # pola na NIP i PESEL nie pokazuj si po podmianie xml, trzeba im zmieni definicj
                xml_form = self.ZnajdzFragment(xml_form, ['name="Podmiot1"', 'name="Organizacja"', '</assist\n>', ''], '', zastap='<value\n><integer\n>2</integer\n></value\n>')  # zaznaczamy checkbox 'skadajcy niebdcy osob fizyczn' w polu B.8.

                xml_form = self.ZnajdzFragment(xml_form, ['name="Podmiot1"', 'name="ID1"', 'name="NIP1"', '<occur min="0"'], '<occur min="0"', zastap='<occur min="1"')  # pole staje si widoczne
                xml_form = self.ZnajdzFragment(xml_form, ['name="Podmiot1"', 'name="Identyfikator"', 'name="NIP"', '</assist\n>', ''], '', zastap='<value\n><boolean\n>1</boolean\n></value\n>')  # zaznaczamy checkbox 'numer NIP'
                xml_form = self.ZnajdzFragment(xml_form, ['name="Podmiot1"', 'name="LineP"', 'presence="invisible"'], 'presence="invisible"', zastap='presence="visible"')  # niepotrzebne skreli - skrelamy PESEL
                xml_form = self.ZnajdzFragment(xml_form, ['name="Podmiot1"', 'name="ID1"', 'name="NIP1"', '</assist\n>', ''], '', zastap='<value\n><text\n>'+dane[0]['oddzial']['nip']+'</text\n></value\n>')  # wpisujemy NIP

                xml_form = self.ZnajdzFragment(xml_form, ['name="Podmiot2"', 'name="ID2"', 'name="PESEL1"', '<occur min="0"'], '<occur min="0"', zastap='<occur min="1"')  # pole staje si widoczne
                xml_form = self.ZnajdzFragment(xml_form, ['name="Podmiot2"', 'name="Identyfikator"', 'name="PESEL"', '</assist\n>', ''], '', zastap='<value\n><integer\n>2</integer\n></value\n>')  # zaznaczamy checkbox 'numer PESEL'
                xml_form = self.ZnajdzFragment(xml_form, ['name="Podmiot2"', 'name="LineN"', 'presence="invisible"'], 'presence="invisible"', zastap='presence="visible"')  # niepotrzebne skreli - skrelamy NIP
                xml_form = self.ZnajdzFragment(xml_form, ['name="Podmiot2"', 'name="ID2"', 'name="PESEL1"', '</assist\n>', ''], '', zastap='<value\n><text\n>'+dane[0]['osobowe']['pesel']+'</text\n></value\n>')  # wpisujemy PESEL

                xml_form = xml_form.replace('Nazwa pelna', 'Nazwa pe\xc5\x82na')  # poprawka literwki

                #WS 2023-01-16 - imi i nazwisko operatora w polu 124
                xml_form = self.ZnajdzFragment(xml_form, ['<draw name="T272"', '</draw\n>', ''], '', zastap='<draw name="T315" x="8.343mm" y="14.0mm" w="111.981mm" h="0.115in"><value><text>'+dane[0]['operator']['imie'].encode('utf-8')+' '+dane[0]['operator']['nazwisko'].encode('utf-8')+'</text></value><ui><textEdit/></ui><para vAlign="middle"/><font weight="normal" size="11pt" typeface="Myriad Pro"/><traversal><traverse ref="Sekcja_I[0]"/></traversal></draw>')

                reader.Root.AcroForm.XFA[5].stream = zlib.compress(xml_form)

                import tempfile
                plik_tmp_nazwa = tempfile.mktemp('.pdf')
                pdfrw.PdfWriter().write(plik_tmp_nazwa, reader)
                with open(plik_tmp_nazwa, 'rb') as plik_tmp:
                    tresc = plik_tmp.read()

                if xml_dekl == '' or xml_form == '':
                    return 'Bd tworzenia pliku pdf', ''
            except:
                import traceback
                stat = traceback.format_exc()
                rsglobal.ZapiszDoLogow('Bd tworzenia PIT11 w pdf dla pracownika '+dane[0]['osobowe']['pracownik_id'].strip()+': '+stat)
                rsglobal.ZapiszDoLogow('Bd tworzenia PIT11 w pdf dla pracownika '+dane[0]['osobowe']['pracownik_id'].strip()+', plik xml: '+xml)
            else:
                stat = 'OK'
        return stat, tresc


#WS 2023-01-20
class UPO(USXML):
    def __init__(self, rodzic):
        from funkcje.fs_kp_deklaracje_us_pom import USXML
        USXML.__init__(self, rodzic)

    def ZapiszPDF(self, tresc_upo_xml):
        tresc = ''

        try:
            import pdfrw
            import re
            import tempfile
            import zlib

            reader = pdfrw.PdfReader('xsd/upo.pdf')
            xml_upo_pdf = zlib.decompress(reader.Root.AcroForm.XFA[11].stream)
            xml_upo = re.search('<Potwierdzenie[\s\S]*</Potwierdzenie>', tresc_upo_xml).group(0)
            xml_upo_pdf = re.sub('<Potwierdzenie[\s\S]*</Potwierdzenie\n>', xml_upo, xml_upo_pdf)
            reader.Root.AcroForm.XFA[11].stream = zlib.compress(xml_upo_pdf)

            xml_form = zlib.decompress(reader.Root.AcroForm.XFA[5].stream)
            SigningTime = re.search('<xades:SigningTime[\s\S]*</xades:SigningTime>', tresc_upo_xml)
            xml_form = xml_form.replace('Dokument wystawiony automatycznie przez system teleinformatyczny Ministerstwa Finansw'.encode('utf-8'), 'Wygenerowano na podstawie dokumentu pobranego z sys. teleinformatycznego Ministerstwa Finansw'.encode('utf-8'))
            if SigningTime:
                xml_form = xml_form.replace('Data i czas wystawienia dokumentu:'.encode('utf-8'), 'Data i czas wystawienia dokumentu: '+SigningTime.group(0).encode('utf-8').strip('<xades:SigningTime>').strip('</xades:SigningTime>'))

            xml_form = xml_form.replace('<field name="WyswietlUPO" ', '<field name="WyswietlUPO" presence="invisible" ')
            xml_form = xml_form.replace('<field name="CheckOther" ', '<field name="CheckOther" presence="invisible" ')
            xml_form = xml_form.replace('<field name="SavaXML_UPO" ', '<field name="SavaXML_UPO" presence="invisible" ')
            xml_form = xml_form.replace('<field name="ShowXML" ', '<field name="ShowXML" presence="invisible" ')
            xml_form = xml_form.replace('<draw name="Image1" ', '<draw name="Image1" presence="invisible" ')

            PESEL = re.search('<PESEL1[\s\S]*</PESEL1>', tresc_upo_xml)
            NIP = re.search('<NIP1[\s\S]*</NIP1>', tresc_upo_xml)
            KodUrzedu = re.search('<KodUrzedu[\s\S]*</KodUrzedu>', tresc_upo_xml)
            if PESEL:
                xml_form = self.ZnajdzFragment(xml_form, ['<field name="Text1" y="138.828mm"', '</calculate\n>', ''], '', zastap='<value\n><text\n>numer PESEL</text\n></value\n>')
            elif NIP:
                xml_form = self.ZnajdzFragment(xml_form, ['<field name="Text1" y="138.828mm"', '</calculate\n>', ''], '', zastap='<value\n><text\n>NIP</text\n></value\n>')
            if KodUrzedu:
                xml_form = self.ZnajdzFragment(xml_form, ['<name="KodUrzedu"', '</calculate\n>', ''], '', zastap='<value\n><text\n>'+KodUrzedu.group(0).strip('<KodUrzedu>').strip('</KodUrzedu>')+'</text\n></value\n>')
            reader.Root.AcroForm.XFA[5].stream = zlib.compress(xml_form)

            plik_tmp_nazwa = tempfile.mktemp('.pdf')
            pdfrw.PdfWriter().write(plik_tmp_nazwa, reader)
            with open(plik_tmp_nazwa, 'rb') as plik_tmp:
                tresc = plik_tmp.read()

            if xml_upo_pdf == '' or xml_form == '':
                return 'Bd tworzenia UPO w pdf', ''
        except:
            import traceback
            stat = traceback.format_exc()
            rsglobal.ZapiszDoLogow('Bd tworzenia UPO w pdf: '+stat)
        else:
            stat = 'OK'
        return stat, tresc


# -------------------------------------------------------------------------------
# --- 1 - Pobranie pracownikw dla listy pac z parametrw
# --- 2 - Tworzenie deklaracji dla pojedynczego pracownika z parametrw
# --- 3 - Pobranie list pac (jeli pusta w param) i tworzenie zbiorcze
# -------------------------------------------------------------------------------

if stat == 'OK' and Funkcja == 2:
    dane_pracownikow = [{'pracownik_id': Pracownik, 'zatrudnienie_id': Zatrudnienie, 'numer_listy_id': NumerListy}]

if stat == 'OK' and Funkcja in (1, 3):
    if NumerListy != '':
        numery_list = [NumerListy]
    else:
        zapytanie = """SELECT numer_listy_id
                       FROM rs_kp_listy_plac
                       WHERE firma_id=%s AND EXTRACT(YEAR FROM data_wyplaty)::TEXT=%s::TEXT AND lista_id='PIT11'
                       ORDER BY numer_listy_id;"""
        zmienne = [self.firma_id, Rok]
        rsglobal.ZapiszDoLogow('Pobranie list pac do PIT11: '+self.PolSQL.Mogrify(zapytanie, zmienne))
        stat, dane_listy, wyn1, dump = self.PolSQL.Execute(zapytanie, 3, zmienne)
        if stat != 'OK':
            stat = u'Bd pobrania numerw list pac'
        else:
            if len(dane_listy) == 0:
                stat = u'Brak list pac dla ktrych mona utworzy PIT11 za rok '+str(Rok)
            else:
                numery_list = [d[0] for d in dane_listy]

    if stat == 'OK':
        zmienne = [self.firma_id]
        numery_list_zap = ''
        for numer in numery_list:
            numery_list_zap += '%s,'
            zmienne.append(str(numer))
        numery_list_zap = numery_list_zap.strip(',')

        zapytanie = """SELECT pracownik_id, MIN(zatrudnienie_id) AS zatrudnienie_id, imie, nazwisko, liczba_list, MIN(numer_listy_id) AS numer_listy_id
                       FROM (SELECT lpo.pracownik_id, zatrudnienie_id, o.imie, o.nazwisko, numer_listy_id, 
                                    COUNT(numer_listy_id) OVER (PARTITION BY pracownik_id) AS liczba_list
                             FROM rs_kp_listy_plac_osoby lpo
                             JOIN rs_kp_dane_osobowe o USING (firma_id, pracownik_id)
                             WHERE lpo.firma_id=%s AND numer_listy_id IN ({numery_list_zap})
                             GROUP BY lpo.pracownik_id, o.imie, o.nazwisko, zatrudnienie_id, numer_listy_id
                             ORDER BY lpo.pracownik_id) AS zap
                       GROUP BY pracownik_id, imie, nazwisko, liczba_list;""".format(**locals())
        rsglobal.ZapiszDoLogow('Pobranie pracownikw z listy pac: ' + self.PolSQL.Mogrify(zapytanie, zmienne))
        stat, dane, wyn1, dump = self.PolSQL.Execute(zapytanie, 3, zmienne)
        if stat == 'OK':
            if len(dane) == 0:
                stat = u'Nieprawidowy numer listy'
            else:
                dane_pracownikow = rssql.RSTableRowset2(dane, wyn1)
                for pracownik in dane_pracownikow:
                    if pracownik['liczba_list'] > 1:
                        stat = u"Pracownik "+pracownik['pracownik_id']+u" wystpuje na wicej ni jednej licie PIT11"
                        break


if stat == 'OK' and Funkcja in (2, 3):
    #WS 2023-01-16
    zapytanie = "SELECT imie, nazwisko FROM rs_operatorzy WHERE operator_id="+str(self.operator_id)
    rsglobal.ZapiszDoLogow('Pobranie danych operatora: ' + self.PolSQL.Mogrify(zapytanie, []))
    stat, dane_operator = wykonaj_zapytanie(self, zapytanie, [], u'Bd pobrania danych operatora')

    if stat == 'OK':
        zapytanie = """SELECT nip, CASE WHEN (nazwa1 IS NULL OR TRIM(nazwa1) = '') AND (nazwa2 IS NULL OR TRIM(nazwa2) = '') THEN nazwa ELSE COALESCE(nazwa1,'') || ' ' || COALESCE(nazwa2,'') END AS nazwa
                       FROM rs_oddzialy 
                       WHERE firma_id=%s AND centrala;"""
        zmienne = [self.firma_id]
        rsglobal.ZapiszDoLogow('Pobranie danych centrali: '+self.PolSQL.Mogrify(zapytanie, zmienne))
        stat, dane_oddzial = wykonaj_zapytanie(self, zapytanie, zmienne, u'Bd pobrania danych centrali')
        if len(dane_oddzial) == 0:
            stat = u"Brak oddziau oznaczonego jako centrala "

    if stat == 'OK':
        lista_danych = []

        for pracownik in dane_pracownikow:
            zapytanie = """SELECT COALESCE(us.kod, '') as kod, us.opis
                           FROM rs_kp_zatrudnienie z
                           LEFT OUTER JOIN rs_urzedy_skarbowe us ON (z.firma_id=us.firma_id AND z.urzad_skarbowy_id=us.kod)
                           WHERE z.firma_id=%s AND pracownik_id=%s AND numer_id=%s;"""
            zmienne = [self.firma_id, pracownik['pracownik_id'], pracownik['zatrudnienie_id']]
            rsglobal.ZapiszDoLogow('Pobranie urzdu skarbowego dla zatrudnienia: '+self.PolSQL.Mogrify(zapytanie, zmienne))
            stat, dane_urzad_skarbowy = wykonaj_zapytanie(self, zapytanie, zmienne, u'Bd pobrania urzdu skarbowego dla zatrudnienia')

            if stat == 'OK':
                zapytanie = """SELECT pracownik_id, imie, nazwisko, data_urodzenia, pesel, 
                                      dokument_nr, rodzaj_numeru_ident, obywatelstwo_kraj, nie_rezydent
                               FROM rs_kp_dane_osobowe
                               WHERE firma_id=%s AND pracownik_id=%s;"""
                zmienne = [self.firma_id, pracownik['pracownik_id']]
                # rsglobal.ZapiszDoLogow('Pobranie danych osobowych: '+self.PolSQL.Mogrify(zapytanie, zmienne))
                stat, dane_osobowe = wykonaj_zapytanie(self, zapytanie, zmienne, u'Bd pobrania danych osobowych')

            if stat == 'OK':
                #WS 2024-01-16 - poprawka aliasow w zlaczeniach z rs_gminy - g.firma_id -> woj./pow.firma_id
                zapytanie = """SELECT COALESCE(ulica, '') AS ulica, COALESCE(nr_domu, '') AS nr_domu, COALESCE(nr_mieszkania, '') AS nr_mieszkania, COALESCE(miejscowosc, '') AS miejscowosc, COALESCE(kod_pocztowy, '') AS kod_pocztowy, kraj_id,
                                      COALESCE(g.opis, '') AS gmina, COALESCE(woj.opis, '') AS wojewodztwo, COALESCE(pow.opis, '') AS powiat
                               FROM rs_kp_adresy a
                               LEFT OUTER JOIN rs_gminy g ON (a.firma_id=g.firma_id AND a.gmina_id=g.kod)
                               LEFT OUTER JOIN rs_gminy woj ON (a.firma_id=woj.firma_id AND SUBSTRING(a.gmina_id, 1, 2)=woj.kod)
                               LEFT OUTER JOIN rs_gminy pow ON (a.firma_id=pow.firma_id AND SUBSTRING(a.gmina_id, 1, 4)=pow.kod)
                               WHERE a.firma_id=%s AND pracownik_id=%s
                               ORDER BY CASE adres_id
                                        WHEN '2' THEN 1
                                        WHEN '3' THEN 2
                                        WHEN '4' THEN 3
                                        WHEN '1' THEN 4 END;"""
                zmienne = [self.firma_id, pracownik['pracownik_id']]
                rsglobal.ZapiszDoLogow('Pobranie adresu: '+self.PolSQL.Mogrify(zapytanie, zmienne))
                stat, dane_adresowe = wykonaj_zapytanie(self, zapytanie, zmienne, u'Bd pobrania adresu')
                if stat == 'OK' and len(dane_adresowe) == 0:
                    stat = u'Brak danych adresowych dla pracownika '+pracownik['pracownik_id']

            if stat == 'OK':
                # lista pl dla ktrych maj zosta pobrane wartoci skadnikw
                pola = []
                pola.extend(range(29, 40+1))
                pola.extend(range(54, 65+1))
                pola.extend(range(81, 86+1))
                pola.extend(range(95, 97+1))
                pola.extend(range(109, 114+1))
                pola.extend(range(118, 122+1))

                zapytanie_wart  = """SUM(CASE WHEN TRIM(skladnik_id)='PIT11028_1' THEN COALESCE(wartosc,0) ELSE 0 END) AS "28_1","""
                zapytanie_wart += """SUM(CASE WHEN TRIM(skladnik_id)='PIT11028_3' THEN COALESCE(wartosc,0) ELSE 0 END) AS "28_3","""
                for pole in pola:
                    zapytanie_wart += """SUM(CASE WHEN TRIM(skladnik_id)='PIT11{}' THEN COALESCE(wartosc,0) ELSE 0 END) AS "{}",""".format(str(pole).rjust(3, '0'), str(pole))
                zapytanie_wart = zapytanie_wart.strip(',')

                zapytanie = """SELECT {zapytanie_wart}
                               FROM rs_kp_listy_plac_osoby_skladniki
                               WHERE firma_id=%s AND numer_listy_id=%s AND pracownik_id=%s;""".format(**locals())
                zmienne = [self.firma_id, pracownik['numer_listy_id'], pracownik['pracownik_id']]
                rsglobal.ZapiszDoLogow('Pobranie skadnikw: '+self.PolSQL.Mogrify(zapytanie, zmienne))
                stat, dane_skladnikow = wykonaj_zapytanie(self, zapytanie, zmienne, u'Bd pobrania skadnikw')

            if stat == 'OK':
                d = {}
                d['operator'] = dane_operator[0]
                d['oddzial'] = dane_oddzial[0]
                d['osobowe'] = dane_osobowe[0]
                d['adresowe'] = dane_adresowe[0]
                d['urzad_skarbowy'] = dane_urzad_skarbowy[0]
                d['skladnikow'] = dane_skladnikow

                lista_danych.append(d)

            if stat != 'OK':
                break


if stat == 'OK' and Funkcja == 2:
    pit11 = PIT11(self)
    if Format == 'XML':
        stat, dane = pit11.ZapiszXML(lista_danych, NrInformacji, CelZlozenia, Rok)
    elif Format == 'PDF':
        stat, dane = pit11.ZapiszPDF(lista_danych, NrInformacji, CelZlozenia, Rok)


if stat == 'OK' and Funkcja == 3:
    pit11 = PIT11(self)
    stat, dane = pit11.ZapiszXML(lista_danych, NrInformacji, CelZlozenia, Rok, Zbiorczy=True)

#WS 2023-01-20
if stat == 'OK' and Funkcja == 4:
    upo = UPO(self)
    stat, dane = upo.ZapiszPDF(TrescUpoXml)
