# -*- coding: iso8859_2 -*-
#-----------------------------------------------------------------------------
# Name:        fs_kp_deklaracje_us_pom.py
# Purpose:     Funkcje pomocnicze do tworzenia deklacji do US (13154)
# Author:      Wojciech Skowron
# Created:     2022/11/16
# RCS-ID:      $Id: fs_kp_deklaracje_us_pom.py $
# Copyright:   (c) 2022 Record System
# Licence:     GNU GPL
#-----------------------------------------------------------------------------

from funkcje import fs_xml
import rssql


def wykonaj_zapytanie(self, zapytanie, zmienne, tresc_bledu):
    dane_rowset = None

    zapytanie = zapytanie.format(**locals())
    zmienne = [str(z) for z in zmienne]
    stat, dane_zapytanie, wyn1, dump = self.PolSQL.Execute(zapytanie, 3, zmienne)
    if stat != 'OK':
        stat = tresc_bledu
    else:
        dane_rowset = rssql.RSTableRowset2(dane_zapytanie, wyn1)

    return stat, dane_rowset


def sprawdz_xml(xml, xsd):
    import os
    import subprocess
    stat = 'OK'

    if not os.path.exists(xsd):
        stat='Brak pliku xsd'
        return stat

    if not os.path.exists(xml):
        stat='Brak pliku xml'
        return stat

    s1 = subprocess.Popen(["xmlstarlet"], shell=True)
    s1.communicate()
    wynik = s1.returncode
    if wynik == 127:
        try:
            from lxml import etree
        except ImportError:
            stat = 'Brak biblioteki lxml'
            return stat

        if os.path.getsize(xml) > 105000000:
            stat = 'Rozmiar pliku xml przekracza 100 MB, wymagana instalacja pakietu XMLStarlet. Skontaktuj si z administratorem.'
            return stat

        schema_doc = etree.parse(xsd)
        schema = etree.XMLSchema(schema_doc)

        try:
            doc = etree.parse(xml)
        except etree.XMLSyntaxError, exc:
            stat = 'EE Plik %s nie jest prawidowym dokumentem xml' % xml
            return stat
        except Exception, exc:
            stat = 'Bd parsowania pliku xml: '+str(exc)
            return stat

        # sprawdzanie struktury xml
        if schema.validate(doc):
            stat = 'OK'
        else:
            stat = ''
            for e in schema.error_log:
                stat += '  {0.level_name} at {0.filename}:{0.line}:{0.column}\n  {0.message}\n'.format(e)
    else:
        s2 = subprocess.Popen(["xmlstarlet val --xsd {} -e {} ".format(xsd,xml)],
                              stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
        stdout, stderr = s2.communicate()
        # wynik = s2.returncode - nie zwraca prawidowego wyniku
        if '- valid' in stdout and not stderr:
            stat = 'OK'
        else:
            stat = stderr.replace(xml+":", "") + stdout

    return stat


class USXML(fs_xml.RSToXml):
    def __init__(self, rodzic, naglowek='?xml version="1.0" encoding="utf-8"?'):
        self.rodzic = rodzic
        fs_xml.RSToXml.__init__(self)
        self.Inicjuj(naglowek, 2)
        self.tresc = None

    def WstawNowyZWartoscia(self, nazwa, wartosc, atrybut=''):
        try:
            wartosc = str(wartosc).strip()
            wartosc = wartosc.encode('utf-8').replace('&', '&amp;')\
                                             .replace('<', '&lt;')\
                                             .replace('>', '&gt;')\
                                             .replace("'", '&apos;')\
                                             .replace('"', '&quot;')
        except:
            pass
        if atrybut != '':
            atrybut = ' '+atrybut
        self.wiersze.append(' <'+nazwa+atrybut+'>'+str(wartosc)+'</'+nazwa+'> ')

    def WstawNowy(self, nazwa, zamknac=0):
        """Dla trybu 2 argument wco i tak nic nie robi, metoda przeciona bez niego"""
        fs_xml.RSToXml.WstawNowyW(self, '', nazwa, zamknac)

    def WstawNowyZKwota(self, nazwa, kwota):
        self.WstawNowyZWartoscia(nazwa, '{:.2f}'.format(kwota))

    def ZakonczTworzenie(self):
        self.tresc = ''.join(self.wiersze)

    def PodajTresc(self):
        """Zwraca tresc, lub None jesli nie skonczono tworzenia"""
        return self.tresc

    def ZnajdzPoz(self, tekst, szukane, poz=0):
        """Idzie po licie fragmentw tekstu (szukane) i zwraca pozycj pocztkow ostatniego"""
        poz = tekst.find(szukane[0], poz)
        if len(szukane) > 1:
            s = szukane.pop(0)
            poz = self.ZnajdzPoz(tekst, szukane, poz+len(s))

        return poz

    def ZnajdzFragment(self, tekst, start, koniec, zastap=''):
        """Zwraca szukany fragment tekstu lub zastpuje go podanym w arg (zastap) i zwraca cao"""
        if isinstance(start, basestring):
            start = [start]
        if isinstance(koniec, basestring):
            koniec = [koniec]

        start_poz = self.ZnajdzPoz(tekst, start)
        if start_poz == -1:
            return ''

        koniec_poz = self.ZnajdzPoz(tekst, koniec, start_poz)
        if koniec_poz == -1:
            return ''

        if zastap != '':
            return tekst[:start_poz]+zastap+tekst[koniec_poz+len(koniec[0]):]
        else:
            return tekst[start_poz:koniec_poz+len(koniec[0])]
