# -*- coding: iso8859_2 -*-
# -------------------------------------------------------------------------------
# Name:        fs_kp_deklaracje_zus_dra.py
# Purpose:     pobranie danych do ZUS DRA
# Author:      Aleksander Bargie
# Created:     2019/04/23
# RCS-ID:      $Id: fs_kp_deklaracje_zus_dra.py $
# Copyright:   (c) 2019 RecordSystem
# Licence:     GNU GPL
# -------------------------------------------------------------------------------A
# 2020-12-15 Wojciech Skowron - (10852) Deklaracje w Kartotece osobowej
# 2021-04-15 Wojciech Skowron - (12452) Poprawka w zapytaniu pobierajcym pracownikw i dane do wza IV
# 2021-11-05 Wojciech Skowron - (12699) Zmiana pobierania pl VIIp01, VIIp02 oraz Vp02
# 2021-11-17 Wojciech Skowron - (12714) Poprawione Vp05
# 2022-01-26 Wojciech Skowron - (12800) Dostosowanie do KEDU 5_4
# 2022-03-02 Wojciech Skowron - (12876) RPA
# 2022-03-08 Wojciech Skowron - (12910) Poprawka w pobieraniu VIIp01, VIIp02, VIIp03
# 2022-07-25 Wojciech Skowron - (13107) Poprawka w IIIp3

from funkcje.fs_kp_deklaracje_zus_pom import ZUSXML


class ZUS(ZUSXML):
    def __init__(self, rodzic):
        from funkcje.fs_kp_deklaracje_zus_pom import ZUSXML
        ZUSXML.__init__(self, rodzic)

    def ZapiszDra(self, dane_inne, lp, skl_spol, skl_zdr, termin_przys, nr_raportu, rok, miesiac, data_wypelnienia):
        self.ZrobNaglowek()

        self.WstawTekst('<ZUSDRA id_dokumentu="1">')

        self.WstawNowy("I")
        self.WstawNowyZWartoscia("p1", termin_przys)
        self.WstawNowy("p2")
        self.WstawNowyZWartoscia("p1", str(nr_raportu).zfill(2))
        self.WstawNowyZWartoscia("p2", str(rok)+'-'+str(miesiac))
        self.Zamknij("p2")
        self.Zamknij("I")

        stat = self.WstawDanePlatnika()

        self.WstawNowy("III")
        self.WstawNowyZWartoscia("p1", dane_inne['IIIp01'])  # Liczba ubezpieczonych
        self.WstawNowyZKwota("p3", dane_inne['IIIp02'])  # Stopa procentowa skadek na ubezpieczenie wypadkowe
        self.Zamknij("III")

        self.WstawNowy("IV")  # Zestawienie nalenych skadek na ubezpieczenia spoeczne oraz rde finansowania
        # Sumy
        self.WstawNowyZKwota("p1", abs(skl_spol['IVp01']))  # Suma skladek na ubezp. emerytalne
        self.WstawNowyZKwota("p2", abs(skl_spol['IVp02']))  # Suma skladek na ubezp. rentowe
        self.WstawNowyZKwota("p3", abs(skl_spol['IVp03']))  # Suma skladek na ubezp. emerytalne i rentowe

        # Finansowane przez ubezpieczonch
        self.WstawNowyZKwota("p4", abs(skl_spol['IVp04']))  # skladki na ubezp. emerytalne
        self.WstawNowyZKwota("p5", abs(skl_spol['IVp05']))  # skladki na ubezp. rentowe
        self.WstawNowyZKwota("p6", abs(skl_spol['IVp06']))  # suma skladek na ubezp. rentowe i emerytalne

        # Finansowane przez platnika
        self.WstawNowyZKwota("p7", abs(skl_spol['IVp07']))  # skladki na ubezp. emerytalne
        self.WstawNowyZKwota("p8", abs(skl_spol['IVp08']))  # skladki na ubezp. rentowe
        self.WstawNowyZKwota("p9", abs(skl_spol['IVp09']))  # suma skladek na ubezp. rentowe i emerytalne

        # Finansowane przez budzet panstwa
        self.WstawNowyZKwota("p10", abs(skl_spol['IVp10']))  # skladki na ubezp. emerytalne
        self.WstawNowyZKwota("p11", abs(skl_spol['IVp11']))  # skladki na ubezp. rentowe
        self.WstawNowyZKwota("p12", abs(skl_spol['IVp12']))  # suma skladek na ubezp. rentowe i emerytalne

        # Finansowane przez Funcusz Koscielny
        self.WstawNowyZKwota("p16", abs(skl_spol['IVp16']))  # skladki na ubezp. emerytalne
        self.WstawNowyZKwota("p17", abs(skl_spol['IVp17']))  # skladki na ubezp. rentowe
        self.WstawNowyZKwota("p18", abs(skl_spol['IVp18']))  # suma skladek na ubezp. rentowe i emerytalne

        # Sumy 2
        self.WstawNowyZKwota("p19", abs(skl_spol['IVp19']))  # Suma skladek na ubezp. chorobowe
        self.WstawNowyZKwota("p20", abs(skl_spol['IVp20']))  # Suma skladek na ubezp. wypadkowe
        self.WstawNowyZKwota("p21", abs(skl_spol['IVp21']))  # Suma skladek na ubezp. chorobowe i wypadkowe

        # Finansowane przez ubezpieczonych
        self.WstawNowyZKwota("p22", abs(skl_spol['IVp22']))  # Suma skladek na ubezp. chorobowe
        self.WstawNowyZKwota("p23", abs(skl_spol['IVp23']))  # Suma skladek na ubezp. wypadkowe
        self.WstawNowyZKwota("p24", abs(skl_spol['IVp24']))  # Suma skladek na ubezp. chorobowe i wypadkowe

        # Finansowane przez platnika
        self.WstawNowyZKwota("p25", abs(skl_spol['IVp25']))  # Suma skladek na ubezp. chorobowe
        self.WstawNowyZKwota("p26", abs(skl_spol['IVp26']))  # Suma skladek na ubezp. wypadkowe
        self.WstawNowyZKwota("p27", abs(skl_spol['IVp27']))  # Suma skladek na ubezp. chorobowe i wypadkowe

        # Finansowane przez budzet panstwa
        # self.WstawNowyZWartoscia("p28", skl_spol['IVp28'])  # Suma skladek na ubezp. chorobowe
        self.WstawNowyZKwota("p29", abs(skl_spol['IVp29']))  # Suma skladek na ubezp. wypadkowe
        self.WstawNowyZKwota("p30", abs(skl_spol['IVp30']))  # Suma skladek na ubezp. chorobowe i wypadkowe

        # Kwota do przekazania przez platnika na ubezp. spoleczne
        self.WstawNowyZKwota("p37", abs(skl_spol['IVp37']))
        self.Zamknij("IV")

        self.WstawNowy("V")
        self.WstawNowyZKwota("p1", abs(lp['Vp01']))
        #WS 2021-11-05
        self.WstawNowyZKwota("p2", rsglobal.RSRound(abs(lp['Vp01'])*0.1*0.01,2))
        # self.WstawNowyZKwota("p2", abs(lp['Vp02']))
        self.WstawNowyZKwota("p3", abs(lp['Vp03']))
        self.WstawNowyZKwota("p4", abs(lp['Vp04']))
        #WS 2021-11-17
        self.Vp05 = rsglobal.RSRound(abs(lp['Vp01'])+rsglobal.RSRound(abs(lp['Vp01'])*0.1*0.01,2)+abs(lp['Vp03'])+abs(lp['Vp04']), 2)
        self.WstawNowyZKwota("p5", self.Vp05)
        self.Zamknij("V")

        self.WstawNowy("VI")
        self.WstawNowyZKwota("p1", abs(skl_zdr['VIp01']))
        self.WstawNowyZKwota("p2", abs(skl_zdr['VIp02']))
        self.WstawNowyZKwota("p3", abs(skl_zdr['VIp03']))
        self.WstawNowyZKwota("p4", abs(skl_zdr['VIp04']))
        self.WstawNowyZKwota("p5", abs(skl_zdr['VIp05']))
        self.WstawNowyZKwota("p7", abs(skl_zdr['VIp05']))
        self.Zamknij("VI")

        self.WstawNowy("VII")
        #WS 2021-11-05
        self.WstawNowyZKwota("p1", skl_spol['VIIp01'])
        self.WstawNowyZKwota("p2", skl_spol['VIIp02'])
        self.WstawNowyZKwota("p3", skl_spol['VIIp03'])
        # self.WstawNowyZKwota("p1", abs(lp['VIIp01']))
        # self.WstawNowyZKwota("p2", abs(lp['VIIp02']))
        # self.WstawNowyZKwota("p3", abs(lp['VIIp03']))
        self.Zamknij("VII")

        self.WstawNowy("IX")
        #WS 2021-11-05
        self.WstawNowyZKwota("p2", rsglobal.RSRound(abs(skl_spol['IVp37'])+abs(skl_zdr['VIp05'])+rsglobal.RSRound(abs(skl_spol['VIIp01'])+abs(skl_spol['VIIp02']),2)-self.Vp05, 2))
        # self.WstawNowyZKwota("p2", rsglobal.RSRound(abs(skl_spol['IVp37'])+abs(skl_zdr['VIp05'])+abs(lp['VIIp03'])-abs(lp['Vp05']), 2))
        self.Zamknij("IX")

        self.WstawNowy("XIII")  #WS 2022-01-26 - XI -> XIII
        self.WstawNowyZWartoscia("p1", data_wypelnienia)
        self.Zamknij("XIII")

        self.Zamknij("ZUSDRA")
        self.ZakonczTworzenie(False)

        tresc = self.PodajTresc()

        return stat, tresc


#-----------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------

stat='OK'
dane=[]
wyn1 = []
wyn2 = []


if not 'Funkcja' in klucze:
    stat='Brak podanej funkcji w kluczach'

if stat=='OK':
    Funkcja = klucze['Funkcja']

    SlPar = {'NrRaportu': [1, ], 'Rok': [1, ], 'Miesiac': [1, ],  'DataWypelnienia': [1, ], 'TerminPrzys': [1, ], 'ListaPracownikow': [1, ]}

    Lista = rsglobal.SprawdzParametryWKluczach(SlPar, Funkcja, klucze)
    if Lista != '':
        stat = 'Uwaga : wywoanie funkcji fs_kp_deklaracje_zus_dra '+str(Funkcja)+' bez parametrw: '+Lista

    if Funkcja in [1, ]:
        TerminPrzys = str(rsglobal.Dekoduj(klucze['TerminPrzys']))
        NrRaportu = str(rsglobal.Dekoduj(klucze['NrRaportu'])).zfill(2)
        Rok = str(rsglobal.Dekoduj(klucze['Rok']))
        Miesiac = str(rsglobal.Dekoduj(klucze['Miesiac'])).zfill(2)
        DataWypelnienia = rsglobal.Dekoduj(klucze['DataWypelnienia'])
        ListaPracownikow = klucze['ListaPracownikow']

#-----------------------------------------------------------------------------------------------------------------------

if Funkcja == 1 and stat == 'OK':
    # pobranie danych do zrobienia deklaracji DRA
    zus = ZUS(self)

    dane_inne = {}

    SQL = """SELECT DISTINCT pracownik_id
             FROM rs_kp_dane_osobowe_zus_skladki
             WHERE firma_id='{self.firma_id}' AND rodzaj in ('RCA','RZA')
                   AND rok='{Rok}' AND miesiac='{Miesiac}';
          """.format(**locals())
    rsglobal.ZapiszDoLogow(SQL)
    stat, dane_prac, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat != 'OK':
        stat = 'Bd pobierania listy pracownikw ubezpieczonych: ' + stat
    elif len(dane_prac) == 0:
        stat = 'Brak ubezpieczonych pracownikw dla ktrych mona by utworzy DRA'
    else:
        dane_inne['IIIp01'] = len([prac[0] for prac in dane_prac])
        war_pracownik = " AND pracownik_id in ('" + "','".join([prac[0] for prac in dane_prac]) + "')"
        if ListaPracownikow == []:
            klucze['ListaPracownikow'] = [prac[0] for prac in dane_prac]

        SQL = """SELECT wart_num FROM rs_kp_stale s WHERE s.firma_id='{firma}' AND s.stala_id='SZ8' ORDER BY data_od DESC LIMIT 1;
              """.format(firma=self.firma_id)
        rsglobal.ZapiszDoLogow(SQL)
        stat, dane_stale, dump, dump = self.PolSQL.Execute(SQL, 3)
        if stat != 'OK':
            stat = 'Bd pobierania danych z wartoci staych: '+stat
        elif len(dane_stale) == 0:
            stat = 'Brak zdefiniowanej stopy procentowej skadek na ubezpieczenie wypadkowe (SZ8) w wartociach staych'
        else:
            dane_inne['IIIp02'] = dane_stale[0][0]

    rsglobal.ZapiszDoLogow(war_pracownik)

    if stat == 'OK':
        kolumny = []
        #V
        kolumny.append(zus.SumySkladnikow(pole='V.p1', nazwa='Vp01'))
        # kolumny.append(zus.SumySkladnikow(pole='V.p2', nazwa='Vp02'))
        kolumny.append(zus.SumySkladnikow(pole='V.p3', nazwa='Vp03'))
        kolumny.append(zus.SumySkladnikow(pole='V.p4', nazwa='Vp04'))
        # kolumny.append(zus.SumySkladnikow(pola=['V.p1', 'V.p2', 'V.p3', 'V.p4'], nazwa='Vp05'))
        #VII
        # kolumny.append(zus.SumySkladnikow(pole='VII.p1', nazwa='VIIp01'))
        # kolumny.append(zus.SumySkladnikow(pole='VII.p2', nazwa='VIIp02'))
        # kolumny.append(zus.SumySkladnikow(pola=['VII.p1', 'VII.p2'], nazwa='VIIp03'))
        #IX
        kolumny.append(zus.SumySkladnikow(pole='IX.p1', nazwa='IXp01'))

        SQL = """SELECT {kolumny}
                 FROM rs_kp_listy_plac l 
                 JOIN rs_kp_listy_plac_osoby_skladniki s USING (firma_id, numer_listy_id) 
                 LEFT JOIN (SELECT firma_id, typ_id, platnik_id, kolumna_id, pole_deklaracji,
                            (SELECT * FROM regexp_split_to_array(skladniki, ',\\\\s*')) AS skladniki
                            FROM rs_edeklaracje_skladniki s2
                            JOIN rs_edeklaracje_wersje w1 USING (typ_id, wersja_id) 
                            WHERE firma_id='{firma}' AND wersja_id=(SELECT wersja_id FROM rs_edeklaracje_wersje w2 
                                                                    WHERE w2.typ_id=w1.typ_id AND data_od<='{od}'
                                                                    ORDER BY w2.data_od DESC  LIMIT 1)) 
                            AS a ON a.firma_id=l.firma_id AND a.typ_id='DRA' AND s.skladnik_id = ANY(a.skladniki)
                 WHERE l.firma_id='{firma}' AND l.stan in ('Z','K') AND data_wyplaty BETWEEN '{od}' AND '{do}' {war_pracownik};
              """.format(kolumny='\n, '.join(kolumny), firma=self.firma_id,
                         od=Rok+'-'+Miesiac+'-'+'01', do=rsglobal.OstatniDzienMiesiaca(Rok+'-'+Miesiac+'-'+'01'),
                         war_pracownik=war_pracownik)
        rsglobal.ZapiszDoLogow(SQL)
        stat, dane_lp, wyn1_lp, dump = self.PolSQL.Execute(SQL, 3)
        if stat != 'OK':
            stat = 'Bd pobierania sum z list pac: ' + stat

    if stat == 'OK':
        #WS 2021-11-05 - pobranie VIIp01 i VIIp02
        #WS 2022-03-08 - poprawka w VIIp01, VIIp02, VIIp03
        SQL = """SELECT SUM(ABS(skl_emeryt_ubez)+ABS(skl_emeryt_plat)+ABS(skl_emeryt_budzet)) AS IVp01,
                        SUM(ABS(skl_rent_ubez)+ABS(skl_rent_plat)+ABS(skl_rent_budzet)) AS IVp02,
                        SUM(ABS(skl_emeryt_ubez)+ABS(skl_emeryt_plat)+ABS(skl_emeryt_budzet)+ABS(skl_rent_ubez)+ABS(skl_rent_plat)+ABS(skl_rent_budzet)) AS IVp03,
                        SUM(ABS(skl_emeryt_ubez)) AS IVp04, SUM(ABS(skl_rent_ubez)) AS IVp05, 
                        SUM(ABS(skl_emeryt_ubez)+ABS(skl_rent_ubez)) AS IVp06,
                        SUM(ABS(skl_emeryt_plat)) AS IVp07, SUM(ABS(skl_rent_plat)) AS IVp08,
                        SUM(ABS(skl_emeryt_plat)+ABS(skl_rent_plat)) AS IVp09,
                        SUM(ABS(skl_emeryt_budzet)) AS IVp10, SUM(ABS(skl_rent_budzet)) AS IVp11,
                        SUM(ABS(skl_emeryt_budzet)+ABS(skl_rent_budzet)) AS IVp12,
                        0 AS IVp16, 0 AS IVp17, 0 AS IVp18,
                        SUM(ABS(skl_chor_ubez)+ABS(skl_chor_plat)) AS IVp19,
                        SUM(ABS(skl_wyp_ubez)+ABS(skl_wyp_plat)+ABS(skl_wyp_budzet)) AS IVp20,
                        SUM(ABS(skl_chor_ubez)+ABS(skl_chor_plat)+ABS(skl_wyp_ubez)+ABS(skl_wyp_plat)+ABS(skl_wyp_budzet)) AS IVp21,
                        SUM(ABS(skl_chor_ubez)) AS IVp22, SUM(ABS(skl_wyp_ubez)) AS IVp23, 
                        SUM(ABS(skl_chor_ubez)+ABS(skl_wyp_ubez)) AS IVp24,
                        SUM(ABS(skl_chor_plat)) AS IVp25, SUM(ABS(skl_wyp_plat)) AS IVp26,
                        SUM(ABS(skl_chor_plat)+ABS(skl_wyp_plat)) AS IVp27,
                        0 AS IVp28, SUM(ABS(skl_wyp_budzet)) AS IVp29,
                        SUM(ABS(skl_wyp_budzet)) AS IVp30,
                        SUM(ABS(skl_emeryt_ubez)+ABS(skl_rent_ubez)+ABS(skl_emeryt_plat)+ABS(skl_rent_plat)+ABS(skl_chor_ubez)+ABS(skl_wyp_ubez)+ABS(skl_chor_plat)+ABS(skl_wyp_plat)) AS IVp37,
                        ROUND(SUM(ABS(podstawa_fp)*2.45*0.01), 2) AS VIIp01,
                        ROUND(SUM(ABS(podstawa_fgsp)*0.1*0.01), 2) AS VIIp02,
                        ROUND(SUM(ABS(podstawa_fp)*2.45*0.01), 2)+ROUND(SUM(ABS(podstawa_fgsp)*0.1*0.01), 2) AS VIIp03
                 FROM rs_kp_dane_osobowe_zus_skladki
                 WHERE firma_id='{self.firma_id}' AND rodzaj='RCA' {war_pracownik}
                 AND rok='{Rok}' AND miesiac='{Miesiac}';"""
        SQL = SQL.format(**locals())
        rsglobal.ZapiszDoLogow(SQL)
        stat, dane_skl_spol, wyn1_skl_spol, dump = self.PolSQL.Execute(SQL, 3)
        if stat != 'OK':
            stat = 'Bd pobierania informacji o skadkach na ubezpieczenia spoeczne'

    if stat == 'OK':
        SQL = """SELECT SUM(skl_zdr_plat) AS VIp01, SUM(skl_zdr_ubez) AS VIp02,
                        0 AS VIp03,
                        SUM(skl_zdr_budzet) AS VIp04,
                        SUM(skl_zdr_plat+skl_zdr_ubez) AS VIp05
                 FROM rs_kp_dane_osobowe_zus_skladki
                 WHERE firma_id='{self.firma_id}' AND rodzaj in ('RCA','RZA') {war_pracownik}
                 AND rok='{Rok}' AND miesiac='{Miesiac}';"""
        SQL = SQL.format(**locals())
        rsglobal.ZapiszDoLogow(SQL)
        stat, dane_skl_zdr, wyn1_skl_zdr, dump = self.PolSQL.Execute(SQL, 3)
        if stat != 'OK':
            stat = 'Bd pobierania informacji o skadkach na ubezpieczenie zdrowotne'

    if stat == 'OK':
        lp = rssql.RSTableRowset2(dane_lp, wyn1_lp)[0]
        skl_spol = rssql.RSTableRowset2(dane_skl_spol, wyn1_skl_spol)[0]
        skl_zdr = rssql.RSTableRowset2(dane_skl_zdr, wyn1_skl_zdr)[0]
        stat, dane = zus.ZapiszDra(dane_inne, lp, skl_spol, skl_zdr, TerminPrzys, NrRaportu, Rok, Miesiac, DataWypelnienia)
        klucze["IDDok"] = 2

    if stat == 'OK':
        stat, dane_rca, dump, dump = self.WykonajFunkcje("fs_kp_deklaracje_zus_rca", klucze, [], [], False)
        if stat == 'OK' and len(dane_rca) > 0:
            dane += dane_rca
            klucze["IDDok"] += 1

    if stat == 'OK':
        stat, dane_rsa, dump, dump = self.WykonajFunkcje("fs_kp_deklaracje_zus_rsa", klucze, [], [], False)
        if stat == 'OK' and len(dane_rsa) > 0:
            dane += dane_rsa
            klucze["IDDok"] += 1

    if stat == 'OK':
        stat, dane_rza, dump, dump = self.WykonajFunkcje("fs_kp_deklaracje_zus_rza", klucze, [], [], False)
        if stat == 'OK' and len(dane_rza) > 0:
            dane += dane_rza
            klucze["IDDok"] += 1

    #WS 2022-03-02
    if stat == 'OK':
        stat, dane_rpa, dump, dump = self.WykonajFunkcje("fs_kp_deklaracje_zus_rpa", klucze, [], [], False)
        if stat == 'OK' and len(dane_rpa) > 0:
            dane += dane_rpa
            klucze["IDDok"] += 1

    if stat == 'OK':
        dane += "</KEDU>"
