# -*- coding: iso8859_2 -*-
# -----------------------------------------------------------------------------
# Name:        fs_kp_deklaracje_zus_pom.py
# Purpose:     funkcje i zmienne pomocnicze do tworzenia deklacji do ZUS
#
# Author:      Aleksander Bargie
# Created:     2019/04/15
# RCS-ID:      $Id: fs_kp_deklaracje_zus_pom.py$
# Copyright:   (c) 2019 Record System
# Licence:     GNU GPL
# -----------------------------------------------------------------------------

# 2020-10-12 Wojciech Skowron - (10852) Nowa wersja KEDU_5_2, zmiany w funkcji SumySkladnikow()
# 2021-03-02 Wojciech Skowron - (12370) Strip we WstawNowyZWartoscia()
# 2021-03-08 Wojciech Skowron - (12379) WstawDaneOsobowe() i WstawDaneEwidencyjne()
# 2021-03-09 Wojciech Skowron - (12387) WstawDaneOsoboweRozl()
# 2021-03-18 Wojciech Skowron - (12394) Poprawka we wpisywaniu obywatelstwa
# 2021-03-24 Wojciech Skowron - (12412) Pobieranie obywatelstwa z rs_kraje
# 2021-04-21 Wojciech Skowron - (12452) Opcja dodatnie w SumySkladnikow()
# 2021-05-18 Wojciech Skowron - (12492) Nowa wersja KEDU_5_3
# 2022-01-26 Wojciech Skowron - (12800) Nowa wersja KEDU_5_4, opcja wyboru wersji w inicie
# 2022-03-02 Wojciech Skowron - (12876) Arg. czy_zanykac w WstawDaneOsoboweRozl() na potrzeby generowania RPA

from funkcje import fs_xml
import rssql

# #WS 2020-10-12
# # NaglowekKEDU  = 'KEDU wersja_schematu="1" xmlns="http://www.zus.pl/2018/KEDU_5" '
# NaglowekKEDU  = 'KEDU wersja_schematu="1" xmlns="http://www.zus.pl/2021/KEDU_5_3" '
# NaglowekKEDU += 'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
# #WS 2020-10-12
# # NaglowekKEDU += 'xsi:schemaLocation="http://www.zus.pl/2018/KEDU_5 kedu_5.xsd"'
# NaglowekKEDU += 'xsi:schemaLocation="http://www.zus.pl/2021/KEDU_5_3 kedu_5_3.xsd"'


class ZUSXML(fs_xml.RSToXml):
    def __init__(self, rodzic, KEDU=True, wersja='5_4'):  #WS 2022-01-26 - arg wersja
        """kazda deklaracja ZUS bedzie inicjoiwana naglowkiem KEDU"""
        #WS 2022-01-26
        if wersja == '5_3':
            NaglowekKEDU  = 'KEDU wersja_schematu="1" xmlns="http://www.zus.pl/2021/KEDU_5_3" '
            NaglowekKEDU += 'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
            NaglowekKEDU += 'xsi:schemaLocation="http://www.zus.pl/2021/KEDU_5_3 kedu_5_3.xsd"'
        else:
            NaglowekKEDU  = 'KEDU wersja_schematu="1" xmlns="http://www.zus.pl/2021/KEDU_5_4" '
            NaglowekKEDU += 'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
            NaglowekKEDU += 'xsi:schemaLocation="http://www.zus.pl/2021/KEDU_5_4 kedu_5_4.xsd"'

        self.rodzic = rodzic
        fs_xml.RSToXml.__init__(self)
        if KEDU:
            self.Inicjuj('?xml version="1.0" encoding="utf-8"?', 2)
            self.WstawNowy(NaglowekKEDU)
        else:
            self.wiersze = []
            self.Tryb = 2
        self.tresc = None

    def ZrobNaglowek(self):
        """Naglowek w deklaracjach ZUS wyglada zawsze tak samo, wiec robimy z tego metode"""
        self.WstawNowy("naglowek.KEDU")
        self.WstawNowy("program")
        self.WstawNowyZWartoscia("producent","RECORD SYSTEM")
        self.WstawNowyZWartoscia("symbol","RECORD SYSTEM")
        self.WstawNowyZWartoscia("wersja","RECORD SYSTEM")
        self.Zamknij("program")
        self.Zamknij("naglowek.KEDU")

    def WstawNowy(self, nazwa, zamknac=0):
        """Dla trybu 2 argument wco i tak nic nie robi, metoda przeciona bez niego"""
        fs_xml.RSToXml.WstawNowyW(self, '', nazwa, zamknac)

    def WstawNowyZWartoscia(self, nazwa, wartosc):
        """Dla trybu 2 argument wco i tak nic nie robi, metoda przeciona bez niego"""
        try:  # kodowanie wartosci jesli to mozliwe plus podmiana znakow specjalnych
            wartosc = wartosc.strip()  #WS 2021-03-02
            wartosc = wartosc.encode('utf-8').replace('&', '&amp;')\
                                             .replace('<', '&lt;')\
                                             .replace('>', '&gt;')\
                                             .replace("'", '&apos;')\
                                             .replace('"', '&quot;')
        except:
            pass
        fs_xml.RSToXml.WstawNowyWZWartoscia(self, '', nazwa, wartosc)

    def WstawNowyZKwota(self, nazwa, kwota):
        self.WstawNowyZWartoscia(nazwa, '{:.2f}'.format(kwota))

    def WstawDanePlatnika(self, wezel='II'):
        # pobranie danych z bazy
        SQL  = "SELECT nip, regon, nazwa, osoba_fizyczna, nazwisko, imie, data_urodzenia FROM rs_oddzialy o "
        SQL += "INNER JOIN rs_firmy f USING(firma_id) WHERE o.firma_id='"+self.rodzic.firma_id+"' "
        SQL += " AND oddzial_id='"+self.rodzic.oddzial_id+"';"
        stat, D2, N2, dump = self.rodzic.PolSQL.Execute(SQL, 3)
        if stat != 'OK':
            stat = 'Bd pobierania informacji z danych oddziau'
        else:
            D = rssql.RSTableRowset2(D2, N2)[0]

            self.WstawNowy(wezel)
            self.WstawNowyZWartoscia("p1", D['nip'].strip(' '))
            self.WstawNowyZWartoscia("p2", D['regon'].strip(' '))
            self.WstawNowyZWartoscia("p6", D['nazwa'])
            if D['osoba_fizyczna']:
                self.WstawNowyZWartoscia("p7", D['nazwisko'])
                self.WstawNowyZWartoscia("p8", D['imie'])
                self.WstawNowyZWartoscia("p9", D['data_urodzenia'])
            self.Zamknij(wezel)

        return stat

    #WS 2021-03-08
    def WstawDaneOsobowe(self, pracownik_id, wezel='III'):
        SQL = "SELECT imie, nazwisko, TRIM(COALESCE(pesel, '')) AS pesel, data_urodzenia, "
        SQL += " CASE WHEN dokument_id='D' THEN 1 WHEN dokument_id='P' THEN 2 ELSE 0 END AS dok_typ, "
        SQL += " CASE WHEN dokument_id in ('D','P') THEN dokument_nr ELSE '' END AS dok_nr "
        SQL += " FROM rs_kp_dane_osobowe os "
        SQL += " WHERE firma_id='"+self.rodzic.firma_id+"' AND pracownik_id='"+pracownik_id+"';"
        stat, D3, N3, dump = self.rodzic.PolSQL.Execute(SQL, 3)
        if stat != 'OK':
            stat = 'Bd pobierania danych osobowych pracownika'
        elif len(D3) == 0:
            stat = 'Brak danych dla pracownika '+pracownik_id
        else:
            D = rssql.RSTableRowset2(D3, N3)[0]

            self.WstawNowy(wezel)
            if D['pesel'] != '':
                self.WstawNowyZWartoscia("p1", D['pesel'])
            else:
                if D['dok_typ'] != 0:
                    self.WstawNowyZWartoscia("p3", D['dok_typ'])
                if D['dok_nr'] != '':
                    self.WstawNowyZWartoscia("p4", D['dok_nr'].replace(' ', ''))
            self.WstawNowyZWartoscia("p5", D['nazwisko'])
            self.WstawNowyZWartoscia("p6", D['imie'])
            self.WstawNowyZWartoscia("p7", D['data_urodzenia'])
            self.Zamknij(wezel)

        return stat

    #WS 2021-03-08
    def WstawDaneEwidencyjne(self, pracownik_id, wezel='IV'):
        SQL = "SELECT TRIM(COALESCE(imie_drugie,'')) AS imie_drugie, TRIM(COALESCE(rodowe,'')) AS rodowe, COALESCE(obywatelstwo,'') AS obywatelstwo, plec"
        SQL += " FROM rs_kp_dane_osobowe os "
        SQL += " LEFT OUTER JOIN rs_kraje kr ON kr.kraj=os.obywatelstwo_kraj "  #WS 2021-03-24
        SQL += " WHERE firma_id='"+self.rodzic.firma_id+"' AND pracownik_id='"+pracownik_id+"';"
        stat, D4, N4, dump = self.rodzic.PolSQL.Execute(SQL, 3)
        if stat != 'OK':
            stat = 'Bd pobierania danych ewidencyjnych pracownika'
        elif len(D4) == 0:
            stat = 'Brak danych dla pracownika '+pracownik_id
        else:
            D = rssql.RSTableRowset2(D4, N4)[0]
            
            self.WstawNowy(wezel)
            if D['imie_drugie'] != '':
                self.WstawNowyZWartoscia("p1", D['imie_drugie'])
            if D['rodowe'] != '':
                self.WstawNowyZWartoscia("p2", D['rodowe'])
            #WS 2021-03-24
            self.WstawNowyZWartoscia("p3", D['obywatelstwo'].upper())
            # #WS 2021-03-18
            # if D['obywatelstwo_kraj'].strip() == 'PL':
            #     self.WstawNowyZWartoscia("p3", 'POLSKIE')
            # else:
            #     self.WstawNowyZWartoscia("p3", D['obywatelstwo_kraj'])
            # # self.WstawNowyZWartoscia("p3", D['obywatelstwo_kraj'])
            self.WstawNowyZWartoscia("p4", D['plec'])
            self.Zamknij(wezel)

        return stat

    #WS 2021-03-09
    #WS 2022-03-02 - czy_zamykac
    def WstawDaneOsoboweRozl(self, pracownik_id, wezel='A', czy_zamykac=True):
        SQL = "SELECT imie, nazwisko, TRIM(COALESCE(pesel, '')) AS pesel, data_urodzenia, "
        SQL += " CASE WHEN dokument_id='D' THEN 1 WHEN dokument_id='P' THEN 2 ELSE 0 END AS dok_typ, "
        SQL += " CASE WHEN dokument_id in ('D','P') THEN dokument_nr ELSE '' END AS dok_nr "
        SQL += " FROM rs_kp_dane_osobowe os "
        SQL += " WHERE firma_id='"+self.rodzic.firma_id+"' AND pracownik_id='"+pracownik_id+"';"
        stat, D3, N3, dump = self.rodzic.PolSQL.Execute(SQL, 3)
        if stat != 'OK':
            stat = 'Bd pobierania danych osobowych pracownika'
        elif len(D3) == 0:
            stat = 'Brak danych dla pracownika '+pracownik_id
        else:
            D = rssql.RSTableRowset2(D3, N3)[0]

            self.WstawNowy(wezel)
            self.WstawNowyZWartoscia("p1", D['nazwisko'])
            self.WstawNowyZWartoscia("p2", D['imie'])
            if D['pesel'] != '':
                self.WstawNowyZWartoscia("p3", 'P')
                self.WstawNowyZWartoscia("p4", D['pesel'])
            else:
                if D['dok_typ'] != 0:
                    self.WstawNowyZWartoscia("p3", D['dok_typ'])
                if D['dok_nr'] != '':
                    self.WstawNowyZWartoscia("p4", D['dok_nr'].replace(' ', ''))
            if czy_zamykac:  #WS 2022-03-02
                self.Zamknij(wezel)

        return stat

    def ZakonczTworzenie(self, KEDU=True):
        """Z tablicy wierszy tworzy string"""
        if KEDU:
            self.Zamknij("KEDU")
        self.tresc = ''.join(self.wiersze)

    def PodajTresc(self):
        """Zwraca tresc, lub None jesli nie skonczono tworzenia"""
        return self.tresc

    #WS 2020-10-12
    def SumySkladnikow(self, kolumna=None, platnik=None, kolumny=None, platnicy=None, pole=None, pola=None, nazwa='', dodatnie=True):
    # def SumySkladnikow(self, kolumna=None, platnik=None, kolumny=None, platnicy=None, nazwa=''):
        warunki = []
        if pole is not None:  #WS 2020-10-12
            warunki.append(" COALESCE(('"+pole+"' = pole_deklaracji), False) ")
        elif pola is not None:  #WS 2020-10-12
            warunki.append(" COALESCE((pole_deklaracji in ('"+"', '".join(pola)+"')), False) ")
        else:
            warunki.append(" COALESCE(pole_deklaracji, '') = '' ")

        if kolumna is not None:
            warunki.append(" COALESCE(('"+kolumna+"' = kolumna_id), False) ")
        if platnik is not None:
            warunki.append(" COALESCE(('"+platnik+"' = platnik_id), False) ")
        if kolumny is not None:
            warunki.append(" COALESCE((kolumna_id in ('"+"', '".join(kolumny)+"')), False) ")
        if platnicy is not None:
            warunki.append(" COALESCE((platnik_id in ('"+"', '".join(platnicy)+"')), False) ")

        for i in range(len(warunki)):
            warunki[i] = warunki[i].ljust(80)

        if len(warunki) == 0:
            if dodatnie:
                sql = " ABS(wartosc)"
            else:
                sql = " wartosc"
        else:
            sql = " CASE WHEN ("
            sql += ' AND '.join(warunki)
            if dodatnie:
                sql += ' ) THEN ABS(wartosc) ELSE 0 END '
            else:
                sql += ' ) THEN wartosc ELSE 0 END '

        sql = " SUM("+sql+") " + ('AS ' + nazwa) if nazwa.strip() != '' else ''
        return sql
