# -*- coding: iso8859_2 -*-
#-------------------------------------------------------------------------------
# Name:        fs_kp_deklaracje_zus_rca.py
# Purpose:     tworzenie deklaracji ZUS RCA
# Author:      Wojciech Skowron
# Created:     2020/12/15
# RCS-ID:      $Id: fs_kp_deklaracje_zus_rca.py $
# Copyright:   (c) 2020 RecordSystem
# Licence:     GNU GPL
#-------------------------------------------------------------------------------

# 2021-02-03 Wojciech Skowron   - 12370 - Usunicie spacji z nr dokumentu tosamoci
# 2021-03-09 Wojciech Skowron   - 12387 - Pobranie danych osobowych przeniesione do _pom
# 2021-05-13 Wojciech Skowron   - 12452 - Pobranie ppk_wplaty do P28
# 2022-01-26 Wojciech Skowron   - 12800 - Dostosowanie do KEDU 5_4
# 2022-02-17 Wojciech Skowron   - 12884 - Sumowanie etatu i wartoci jeli ten sam kod ubezpieczenia
# 2022-09-08 Wojciech Skowron   - 13143 - Etat z angau
# 2022-09-27 Aleksander Bargie - 13181 - Poprawka w RCA
# 2022-10-28 Wojciech Skowron   -       - Poprawka w RCA (w zczeniu z rs_pk_listy_plac_osoby)
# 2023-01-31 Wojciech Skowron   - 13335 - Usunity warunek przy zczeniu skl z etat

from funkcje.fs_kp_deklaracje_zus_pom import ZUSXML
import rssql


class ZUS(ZUSXML):
    def __init__(self, rodzic, KEDU):
        from funkcje.fs_kp_deklaracje_zus_pom import ZUSXML
        ZUSXML.__init__(self, rodzic, KEDU)

    def ZapiszRca(self, id, skladki, nr_raportu, rok, miesiac, data_wypelnienia):
        import fractions

        if not id > 1:
            self.ZrobNaglowek()

        self.WstawTekst('<ZUSRCA id_dokumentu="'+str(id)+'">')

        self.WstawNowy("I")
        self.WstawNowy("p1")
        self.WstawNowyZWartoscia("p1", str(nr_raportu).zfill(2))
        self.WstawNowyZWartoscia("p2", str(rok)+'-'+str(miesiac))
        self.Zamknij("p1")
        self.Zamknij("I")

        stat = self.WstawDanePlatnika()
        if stat != 'OK':
            return stat, ''

        id_bloku = 0

        for skl in skladki:
            etat_ulamek = fractions.Fraction.from_float(skl['etat']).limit_denominator()
            etat_licznik = etat_ulamek.numerator
            etat_mianownik = etat_ulamek.denominator

            id_bloku += 1
            self.WstawTekst('<III id_bloku="'+str(id_bloku)+'">')

            #WS 2021-03-09
            stat = self.WstawDaneOsoboweRozl(skl['pracownik_id'])
            if stat != 'OK':
                return stat, ''
            # self.WstawNowy("A")
            # self.WstawNowyZWartoscia("p1", skl['nazwisko'])
            # self.WstawNowyZWartoscia("p2", skl['imie'])
            # self.WstawNowyZWartoscia("p3", skl['dok_typ'])
            # self.WstawNowyZWartoscia("p4", skl['dok_nr'].replace(' ', ''))
            # self.Zamknij("A")

            self.WstawNowy("B")
            self.WstawNowy("p1")  # Kod tytuu ubezpieczenia
            self.WstawNowyZWartoscia("p1", skl['ubezpieczenie_id'][:4])
            if len(skl['ubezpieczenie_id'].strip(' ')) > 4:
                self.WstawNowyZWartoscia("p2", skl['ubezpieczenie_id'][4:5])
            else:
                self.WstawNowyZWartoscia("p2", "0")
            if len(skl['ubezpieczenie_id'].strip(' ')) > 5:
                self.WstawNowyZWartoscia("p3", skl['ubezpieczenie_id'][5:6])
            else:
                self.WstawNowyZWartoscia("p3", "0")
            self.Zamknij("p1")
            if skl['czy_przekroczenie']:
                self.WstawNowyZWartoscia("p2", '2')
            self.WstawNowy("p3")
            self.WstawNowyZWartoscia("p1", etat_licznik)
            self.WstawNowyZWartoscia("p2", etat_mianownik)
            self.Zamknij("p3")

            # Skadki na ubezpieczenia spoeczne
            # Podstawy wymiaru skadek
            self.WstawNowyZKwota("p4",  abs(skl['podst_emeryt_rent']))    # emerytalne i rentowe
            self.WstawNowyZKwota("p5",  abs(skl['podst_chor']))    # chorobowe
            self.WstawNowyZKwota("p6",  abs(skl['podst_wyp']))    # wypadkowe

            # Finansowane przez ubezpieczonych
            self.WstawNowyZKwota("p7",  abs(skl['skl_emeryt_ubez']))   # skladki na ubezp. emerytalne
            self.WstawNowyZKwota("p8",  abs(skl['skl_rent_ubez']))   # skladki na ubezp. rentowe
            self.WstawNowyZKwota("p9",  abs(skl['skl_chor_ubez']))   # skladki na ubezp. chorobowe
            self.WstawNowyZKwota("p10", abs(skl['skl_wyp_ubez']))   # skladki na ubezp. wypadkowe

            # Finansowane przez platnika
            self.WstawNowyZKwota("p11", abs(skl['skl_emeryt_plat']))   # skladki na ubezp. emerytalne
            self.WstawNowyZKwota("p12", abs(skl['skl_rent_plat']))   # skladki na ubezp. rentowe
            self.WstawNowyZKwota("p13", abs(skl['skl_chor_plat']))   # skladki na ubezp. chorobowe
            self.WstawNowyZKwota("p14", abs(skl['skl_wyp_plat']))   # skladki na ubezp. wypadkowe

            # Finansowane przez budzet panstwa
            self.WstawNowyZKwota("p15", abs(skl['skl_emeryt_budzet']))  # skladki na ubezp. emerytalne
            self.WstawNowyZKwota("p16", abs(skl['skl_rent_budzet']))  # skladki na ubezp. rentowe
            self.WstawNowyZKwota("p18", abs(skl['skl_wyp_budzet']))  # skladki na ubezp. wypadkowe

            # Finansowane przez Fundusz Kocielny
            # self.WstawNowyZWartoscia("p23", skl['skl_emeryt_budzet'])  # skladki na ubezp. emerytalne
            # self.WstawNowyZWartoscia("p24", skl['skl_rent_budzet'])  # skladki na ubezp. rentowe
            # self.WstawNowyZWartoscia("p26", skl['skl_wyp_budzet'])  # skladki na ubezp. wypadkowe

            self.WstawNowyZKwota("p27", abs(skl['kwota_obnizenia_podstawy']))
            self.WstawNowyZKwota("p28", abs(skl['ppk_wplaty']))  #WS 2021-05-13

            # Suma
            self.WstawNowyZKwota("p29", abs(skl['suma']))
            self.Zamknij("B")

            self.WstawNowy("C")
            # Skadki na ubezpieczenie zdrowotne
            self.WstawNowyZKwota("p1", abs(skl['podstawa_zdr']))    # Podstawa wymiaru skadki na ubezp. zdr.
            self.WstawNowyZKwota("p2", abs(skl['skl_zdr_plat']))    # finansowane przez platnika
            self.WstawNowyZKwota("p3", abs(skl['skl_zdr_budzet']))  # finansowane przez budzet panstwa
            self.WstawNowyZKwota("p4", abs(skl['skl_zdr_ubez']))    # finansowane przez ubezpieczonych
            # self.WstawNowyZWartoscia("p5", skl['skl_zdr_ubez'])  # finansowane przez Fundusz Kocielny
            self.Zamknij("C")

            self.Zamknij("III")

        self.WstawNowy("IV")  #WS 2022-01-26 - V -> IV
        self.WstawNowyZWartoscia("p1", data_wypelnienia)
        self.Zamknij("IV")

        self.Zamknij("ZUSRCA")
        self.ZakonczTworzenie(not id > 1)

        tresc = self.PodajTresc()

        return stat, tresc


#-----------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------

stat = 'OK'
dane = []
wyn1 = []
wyn2 = []


if not 'Funkcja' in klucze:
    stat='Brak podanej funkcji w kluczach'

if stat=='OK':
    Funkcja = klucze['Funkcja']

    SlPar={'NrRaportu': [1, ], 'Rok': [1, 2], 'Miesiac': [1, 2], 'DataWypelnienia': [1, ], 'ListaPracownikow': [1, ], 'IDDok': [1, ]}

    Lista=rsglobal.SprawdzParametryWKluczach(SlPar, Funkcja, klucze)
    if Lista!='':
        stat='Uwaga : wywoanie funkcji fs_kp_deklaracje_zus_rca '+str(Funkcja)+' bez parametrw: '+Lista

    if Funkcja in [1, 2]:
        Rok = rsglobal.Dekoduj(klucze['Rok'])
        Miesiac = str(rsglobal.Dekoduj(klucze['Miesiac'])).zfill(2)

    if Funkcja in [1, ]:
        IDDok = klucze['IDDok']
        NrRaportu = str(rsglobal.Dekoduj(klucze['NrRaportu'])).zfill(2)
        DataWypelnienia = rsglobal.Dekoduj(klucze['DataWypelnienia'])
        ListaPracownikow = klucze['ListaPracownikow']

#-----------------------------------------------------------------------------------------------------------------------

if Funkcja == 1 and stat == 'OK':
    zus = ZUS(self, not IDDok > 1)

    war_pracownik = " AND skl.pracownik_id in ('"+"','".join(ListaPracownikow)+"')"

    # pobranie danych o skadkach pracownika
    #WS 2022-02-17 - sumowanie etatu i wartoci przy grupowaniu po ubezpieczenie_id
    #WS 2022-09-08 - etat z angau
    #AB 2022-09-27 - Poprawka w RCA
    #WS 2023-01-31 - Usunite warunki 'AND rl=EXTRACT(YEAR FROM etat.data_do)::TEXT AND ml=to_char(EXTRACT(MONTH FROM etat.data_do), 'fm00')' przy zczeniu skl z etat
    SQL = """SELECT skl.pracownik_id, rok, miesiac, rodzaj, ubezpieczenie_id, czy_przekroczenie, podst_emeryt_rent, podst_chor, podst_wyp, podstawa_zdr, podstawa_fp, podstawa_fgsp, skl_emeryt_ubez, skl_rent_ubez,
                   skl_chor_ubez, skl_wyp_ubez, skl_zdr_ubez, skl_emeryt_plat, skl_rent_plat, skl_chor_plat, skl_wyp_plat, skl_zdr_plat, skl_emeryt_budzet, skl_rent_budzet, skl_wyp_budzet, skl_zdr_budzet, 
                   kwota_obnizenia_podstawy, ppk_wplaty, suma, max(etat) AS etat
             FROM (SELECT skl.pracownik_id, rok, miesiac, rodzaj, ubezpieczenie_id, max(rok_lista) AS rl, max(miesiac_lista) AS ml,
                       BOOL_OR(czy_przekroczenie) AS czy_przekroczenie,
                       SUM(ABS(podst_emeryt_rent)) AS podst_emeryt_rent, SUM(ABS(podst_chor)) AS podst_chor,
                       SUM(ABS(podst_wyp)) AS podst_wyp, SUM(ABS(podstawa_zdr)) AS podstawa_zdr, SUM(ABS(podstawa_fp)) AS podstawa_fp, 
                       SUM(ABS(podstawa_fgsp)) AS podstawa_fgsp,
                       SUM(ABS(skl_emeryt_ubez)) AS skl_emeryt_ubez, SUM(ABS(skl_rent_ubez)) AS skl_rent_ubez,
                       SUM(ABS(skl_chor_ubez)) AS skl_chor_ubez, SUM(ABS(skl_wyp_ubez)) AS skl_wyp_ubez, 
                       SUM(ABS(skl_zdr_ubez)) AS skl_zdr_ubez, SUM(ABS(skl_emeryt_plat)) AS skl_emeryt_plat, 
                       SUM(ABS(skl_rent_plat)) AS skl_rent_plat, SUM(ABS(skl_chor_plat)) AS skl_chor_plat, 
                       SUM(ABS(skl_wyp_plat)) AS skl_wyp_plat, SUM(ABS(skl_zdr_plat)) AS skl_zdr_plat, 
                       SUM(ABS(skl_emeryt_budzet)) AS skl_emeryt_budzet, SUM(ABS(skl_rent_budzet)) AS skl_rent_budzet, 
                       SUM(ABS(skl_wyp_budzet)) AS skl_wyp_budzet, SUM(ABS(skl_zdr_budzet)) AS skl_zdr_budzet, 
                       SUM(ABS(kwota_obnizenia_podstawy)) AS kwota_obnizenia_podstawy, SUM(ABS(ppk_wplaty)) AS ppk_wplaty,
                       SUM(ABS(skl_emeryt_ubez))+SUM(ABS(skl_rent_ubez))+SUM(ABS(skl_chor_ubez))+SUM(ABS(skl_wyp_ubez))+
                       SUM(ABS(skl_emeryt_plat))+SUM(ABS(skl_rent_plat))+SUM(ABS(skl_chor_plat))+SUM(ABS(skl_wyp_plat))+
                       SUM(ABS(skl_emeryt_budzet))+SUM(ABS(skl_rent_budzet))+SUM(ABS(skl_wyp_budzet)) AS suma
                   FROM rs_kp_dane_osobowe_zus_skladki skl
                   WHERE skl.firma_id='{self.firma_id}' AND rodzaj='RCA' {war_pracownik} 
                         AND rok='{Rok}' AND miesiac='{Miesiac}'
                   GROUP BY skl.firma_id, skl.pracownik_id, rok, miesiac, rodzaj, ubezpieczenie_id) AS skl 
            JOIN          
                (SELECT lp.firma_id, a.pracownik_id, etat, data_wyplaty, os.data_do 
                 FROM rs_kp_listy_plac lp 
                 JOIN rs_kp_listy_plac_osoby os
                    ON lp.firma_id=os.firma_id AND lp.numer_listy_id=os.numer_listy_id
                 JOIN rs_kp_angaze a ON a.firma_id=os.firma_id AND a.pracownik_id=os.pracownik_id
                      AND a.zatrudnienie_id=os.zatrudnienie_id AND a.umowa_id=os.umowa_id AND a.angaz_id=os.angaz_id
                 WHERE lp.firma_id = '{self.firma_id}' 
                ) etat
              ON etat.pracownik_id = skl.pracownik_id
                  AND '{Rok}'=EXTRACT(YEAR FROM etat.data_wyplaty)::TEXT AND '{Miesiac}'=to_char(EXTRACT(MONTH FROM etat.data_wyplaty), 'fm00')
            GROUP BY skl.pracownik_id, rok, miesiac, rodzaj, ubezpieczenie_id, czy_przekroczenie, podst_emeryt_rent, podst_chor, podst_wyp, podstawa_zdr, podstawa_fp, podstawa_fgsp, skl_emeryt_ubez, skl_rent_ubez,
                     skl_chor_ubez, skl_wyp_ubez, skl_zdr_ubez, skl_emeryt_plat, skl_rent_plat, skl_chor_plat, skl_wyp_plat, skl_zdr_plat, skl_emeryt_budzet, skl_rent_budzet, skl_wyp_budzet, skl_zdr_budzet, 
                     kwota_obnizenia_podstawy, ppk_wplaty, suma             
             ORDER BY ubezpieczenie_id;"""
    SQL = SQL.format(**locals())
    rsglobal.ZapiszDoLogow(SQL)
    stat, dane_skl, wyn1_skl, dump = self.PolSQL.Execute(SQL, 3)
    if stat != 'OK':
        stat = 'Bd pobierania informacji o skadkach pracownikw'

    if stat == 'OK' and len(dane_skl) > 0:
        Skladki = rssql.RSTableRowset2(dane_skl, wyn1_skl)
        stat, dane = zus.ZapiszRca(IDDok, Skladki, NrRaportu, Rok, Miesiac, DataWypelnienia)

#-----------------------------------------------------------------------------------------------------------------------

if Funkcja == 2 and stat == 'OK':
    SQL = """SELECT pracownik_id FROM rs_kp_dane_osobowe_zus_skladki
             WHERE firma_id='{self.firma_id}' AND rodzaj='RCA' 
             AND rok='{Rok}' AND miesiac='{Miesiac}';"""
    SQL = SQL.format(**locals())
    rsglobal.ZapiszDoLogow(SQL)
    stat, dane, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat != 'OK':
        stat = 'Bd pobierania informacji o skadkach pracownikw'
