# -*- coding: iso8859_2 -*-
#-------------------------------------------------------------------------------
# Name:        fs_kp_deklaracje_zus_rpa.py
# Purpose:     tworzenie deklaracji ZUS RPA
# Author:      Wojciech Skowron
# Created:     2022/03/02
# RCS-ID:      $Id: fs_kp_deklaracje_zus_rpa.py $
# Copyright:   (c) 2022 RecordSystem
# Licence:     GNU GPL
#-------------------------------------------------------------------------------

from funkcje.fs_kp_deklaracje_zus_pom import ZUSXML
import rssql

class ZUS(ZUSXML):
    def __init__(self, rodzic, KEDU):
        from funkcje.fs_kp_deklaracje_zus_pom import ZUSXML
        ZUSXML.__init__(self, rodzic, KEDU)

    def ZapiszRpa(self, id, skladki, nr_raportu, rok, miesiac, data_wypelnienia):
        if not id > 1:
            self.ZrobNaglowek()

        self.WstawTekst('<ZUSRPA id_dokumentu="'+str(id)+'">')

        self.WstawNowy("I")
        self.WstawNowy("p1")
        self.WstawNowyZWartoscia("p1", str(nr_raportu).zfill(2))
        self.WstawNowyZWartoscia("p2", str(rok)+'-'+str(miesiac))
        self.Zamknij("p1")
        self.Zamknij("I")

        stat = self.WstawDanePlatnika()
        if stat != 'OK':
            return stat, ''

        id_bloku = 0

        for skl in skladki:
            id_bloku += 1
            self.WstawTekst('<III id_bloku="'+str(id_bloku)+'">')

            #WS 2021-03-09
            stat = self.WstawDaneOsoboweRozl(skl['pracownik_id'], czy_zamykac=False)
            if stat != 'OK':
                return stat, ''
            self.WstawNowy("p5")  # Kod tytuu ubezpieczenia
            self.WstawNowyZWartoscia("p1", skl['ubezpieczenie_id'][:4])
            if len(skl['ubezpieczenie_id'].strip(' ')) > 4:
                self.WstawNowyZWartoscia("p2", skl['ubezpieczenie_id'][4:5])
            else:
                self.WstawNowyZWartoscia("p2", "0")
            if len(skl['ubezpieczenie_id'].strip(' ')) > 5:
                self.WstawNowyZWartoscia("p3", skl['ubezpieczenie_id'][5:6])
            else:
                self.WstawNowyZWartoscia("p3", "0")
            self.Zamknij("p5")
            self.Zamknij("A")

            self.WstawNowy("B")
            self.WstawNowyZWartoscia("p1",  str(rok-1))
            self.WstawNowyZKwota("p2",  abs(skl['podst_emeryt_rent']))
            self.Zamknij("B")

            self.WstawNowy("C")
            self.WstawNowyZWartoscia("p1",  str(rok-1))
            self.WstawNowyZKwota("p2",  abs(skl['podst_wyp']))
            self.Zamknij("C")

            self.Zamknij("III")

        self.WstawNowy("IV")
        self.WstawNowyZWartoscia("p1", data_wypelnienia)
        self.Zamknij("IV")

        self.Zamknij("ZUSRPA")
        self.ZakonczTworzenie(not id > 1)

        tresc = self.PodajTresc()

        return stat, tresc


#-----------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------

stat = 'OK'
dane = []
wyn1 = []
wyn2 = []


if not 'Funkcja' in klucze:
    stat='Brak podanej funkcji w kluczach'

if stat=='OK':
    Funkcja = klucze['Funkcja']

    SlPar={'NrRaportu': [1, ], 'Rok': [1, 2], 'Miesiac': [1, 2], 'DataWypelnienia': [1, ], 'ListaPracownikow': [1, ], 'IDDok': [1, ]}

    Lista=rsglobal.SprawdzParametryWKluczach(SlPar, Funkcja, klucze)
    if Lista!='':
        stat='Uwaga : wywoanie funkcji fs_kp_deklaracje_zus_rpa '+str(Funkcja)+' bez parametrw: '+Lista

    if Funkcja in [1, 2]:
        Rok = rsglobal.Dekoduj(klucze['Rok'])
        Miesiac = str(rsglobal.Dekoduj(klucze['Miesiac'])).zfill(2)

    if Funkcja in [1, ]:
        IDDok = klucze['IDDok']
        NrRaportu = str(rsglobal.Dekoduj(klucze['NrRaportu'])).zfill(2)
        DataWypelnienia = rsglobal.Dekoduj(klucze['DataWypelnienia'])
        ListaPracownikow = klucze['ListaPracownikow']

#-----------------------------------------------------------------------------------------------------------------------

if Funkcja == 1 and stat == 'OK':
    zus = ZUS(self, not IDDok > 1)

    war_pracownik = " AND pracownik_id in ('"+"','".join(ListaPracownikow)+"')"

    # pobranie danych o skadkach pracownika
    SQL = """SELECT pracownik_id, rok, miesiac, rodzaj, ubezpieczenie_id,
                    SUM(ABS(podst_emeryt_rent)) AS podst_emeryt_rent, SUM(ABS(podst_wyp)) AS podst_wyp
             FROM rs_kp_dane_osobowe_zus_skladki skl
             WHERE firma_id='{self.firma_id}' AND rodzaj='RCA' {war_pracownik}
             AND rok='{Rok}' AND miesiac='{Miesiac}'
             AND rok_lista<rok AND rok_lista IS NOT NULL AND (podst_emeryt_rent > 0 OR podst_wyp > 0)
             GROUP BY firma_id, pracownik_id, rok, miesiac, rodzaj, ubezpieczenie_id
             ORDER BY ubezpieczenie_id;"""
    SQL = SQL.format(**locals())
    rsglobal.ZapiszDoLogow(SQL)
    stat, dane_skl, wyn1_skl, dump = self.PolSQL.Execute(SQL, 3)
    if stat != 'OK':
        stat = 'Bd pobierania informacji o skadkach pracownikw'

    if stat == 'OK' and len(dane_skl) > 0:
        Skladki = rssql.RSTableRowset2(dane_skl, wyn1_skl)
        stat, dane = zus.ZapiszRpa(IDDok, Skladki, NrRaportu, Rok, Miesiac, DataWypelnienia)

#-----------------------------------------------------------------------------------------------------------------------

if Funkcja == 2 and stat == 'OK':
    SQL = """SELECT pracownik_id FROM rs_kp_dane_osobowe_zus_skladki
             WHERE firma_id='{self.firma_id}' AND rodzaj='RCA' 
             AND rok='{Rok}' AND miesiac='{Miesiac}'
             AND rok_lista<rok AND rok_lista IS NOT NULL AND (podst_emeryt_rent > 0 OR podst_wyp > 0);"""
    SQL = SQL.format(**locals())
    rsglobal.ZapiszDoLogow(SQL)
    stat, dane, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat != 'OK':
        stat = 'Bd pobierania informacji o skadkach pracownikw'
