# -*- coding: iso8859_2 -*-
#-------------------------------------------------------------------------------
# Name:        fs_kp_deklaracje_zus_rsa.py
# Purpose:     tworzenie deklaracji ZUS RSA
# Author:      Wojciech Skowron
# Created:     2020/12/15
# RCS-ID:      $Id: fs_kp_deklaracje_zus_rsa.py $
# Copyright:   (c) 2020 RecordSystem
# Licence:     GNU GPL
#-------------------------------------------------------------------------------

# 2021-02-03 Wojciech Skowron   - 12370 - Usunicie spacji z nr dokumentu tosamoci
# 2021-03-09 Wojciech Skowron   - 12387 - Pobranie danych osobowych przeniesione do _pom
# 2021-04-20 Wojciech Skowron   - 12452 - czenie wierszy absencji jeli daty nastpuj dzie po dniu,
#                                         pobieranie dni i kwot z rs_kp_dane_osobowe_absencje zamiast ze skadnikw
# 2021-10-29 Wojciech Skowron   - 12684 - Usunicie warunku na zwolnienie_data
# 2021-10-29 Wojciech Skowron   - 12684 - Dodanie warunku na zakoczenie absencji przed zakoczeniem umowy
# 2021-11-17 Wojciech Skowron   - 12714 - ORDER BY min_od_kiedy


from funkcje.fs_kp_deklaracje_zus_pom import ZUSXML


class ZUS(ZUSXML):
    def __init__(self, rodzic, KEDU):
        from funkcje.fs_kp_deklaracje_zus_pom import ZUSXML
        ZUSXML.__init__(self, rodzic, KEDU)

    def ZapiszRsa(self, id, absencje, nr_raportu, rok, miesiac, data_wypelnienia):
        if not id > 1:
            self.ZrobNaglowek()

        self.WstawTekst('<ZUSRSA id_dokumentu="'+str(id)+'">')

        self.WstawNowy("I")
        self.WstawNowy("p1")
        self.WstawNowyZWartoscia("p1", str(nr_raportu).zfill(2))
        self.WstawNowyZWartoscia("p2", str(rok) + '-' + str(miesiac))
        self.Zamknij("p1")
        self.Zamknij("I")

        stat = self.WstawDanePlatnika()
        if stat != 'OK':
            return stat, ''

        id_bloku = 0

        for abs in absencje:
            id_bloku += 1
            self.WstawTekst('<III id_bloku="'+str(id_bloku)+'">')

            #WS 2021-03-09
            stat = self.WstawDaneOsoboweRozl(abs['pracownik_id'])
            if stat != 'OK':
                return stat, ''
            # self.WstawNowy("A")
            # self.WstawNowyZWartoscia("p1", abs['nazwisko'])
            # self.WstawNowyZWartoscia("p2", abs['imie'])
            # self.WstawNowyZWartoscia("p3", abs['dok_typ'])
            # self.WstawNowyZWartoscia("p4", abs['dok_nr'].replace(' ', ''))
            # self.Zamknij("A")

            self.WstawNowy("B")
            self.WstawNowy("p1")  # Kod tytuu ubezpieczenia
            self.WstawNowyZWartoscia("p1", abs['ubezpieczenie_id'][:4])
            if len(abs['ubezpieczenie_id'].strip(' ')) > 4:
                self.WstawNowyZWartoscia("p2", abs['ubezpieczenie_id'][4:5])
            else:
                self.WstawNowyZWartoscia("p2", "0")
            if len(abs['ubezpieczenie_id'].strip(' ')) > 5:
                self.WstawNowyZWartoscia("p3", abs['ubezpieczenie_id'][5:6])
            else:
                self.WstawNowyZWartoscia("p3", "0")
            self.Zamknij("p1")

            self.WstawNowyZWartoscia("p2", abs['kod_zus'])
            self.WstawNowyZWartoscia("p3", abs['od_kiedy'])
            self.WstawNowyZWartoscia("p4", abs['do_kiedy'])
            self.WstawNowyZWartoscia("p5", int(abs['IIIBp5']))
            if abs['IIIBp6'] != 0:
                self.WstawNowyZKwota("p6", abs['IIIBp6'])

            self.Zamknij("B")
            self.Zamknij("III")

        self.WstawNowy("XI")
        self.WstawNowyZWartoscia("p1", data_wypelnienia)
        self.Zamknij("XI")

        self.Zamknij("ZUSRSA")
        self.ZakonczTworzenie(not id > 1)

        tresc = self.PodajTresc()

        return stat, tresc


#-----------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------

stat='OK'
dane=[]
wyn1 = []
wyn2 = []


if not 'Funkcja' in klucze:
    stat='Brak podanej funkcji w kluczach'

if stat == 'OK':
    Funkcja = klucze['Funkcja']

    SlPar={'NrRaportu': [1, ], 'Rok': [1,2], 'Miesiac': [1,2],  'DataWypelnienia': [1, ], 'ListaPracownikow': [1, ], 'IDDok': [1, ]}

    Lista=rsglobal.SprawdzParametryWKluczach(SlPar, Funkcja, klucze)
    if Lista!='':
        stat='Uwaga : wywoanie funkcji fs_kp_deklaracje_zus_rsa '+str(Funkcja)+' bez parametrw: '+Lista

    if Funkcja in [1,2]:
        Rok = str(rsglobal.Dekoduj(klucze['Rok']))
        Miesiac = str(rsglobal.Dekoduj(klucze['Miesiac'])).zfill(2)
        DataOd = Rok + '-' + Miesiac + '-' + '01'
        DataDo = rsglobal.OstatniDzienMiesiaca(Rok + '-' + Miesiac + '-' + '01')

    if Funkcja in [1, ]:
        IDDok = klucze['IDDok']
        NrRaportu = str(rsglobal.Dekoduj(klucze['NrRaportu'])).zfill(2)
        DataWypelnienia = rsglobal.Dekoduj(klucze['DataWypelnienia'])
        ListaPracownikow = klucze['ListaPracownikow']

#-----------------------------------------------------------------------------------------------------------------------

if Funkcja == 1 and stat=='OK':
    zus = ZUS(self, not IDDok > 1)
    #pobranie danych do zrobienia deklaracji RSA
    kolumny = []

    #WS 2021-10-29 - usunicie warunku AND (z.zwolnienie_data IS NULL OR z.zwolnienie_data >= l.data_wyplaty)
    #WS 2021-10-29 - dodanie JOIN na rs_kp_umowy_o_prace oraz warunku (u.do_kiedy IS NULL OR ab.do_kiedy <= u.do_kiedy)
    #WS 2021-11-17 - ORDER BY min_od_kiedy
    # III
    SQL = """SELECT wyn.pracownik_id, wyn.ubezpieczenie_id, wyn.kod_zus, 
                    MIN(wyn.od_kiedy) AS od_kiedy, MAX(wyn.do_kiedy) AS do_kiedy, 
                    SUM(wyn.dni_kalendarzowe) AS IIIBp5, SUM(wyn.zasilek_razem) AS IIIBp6,
                    wyn.min_od_kiedy
             FROM (SELECT ab.pracownik_id, z.ubezpieczenie_id, ab.kod_zus, ab.od_kiedy, ab.do_kiedy, ab.grp,
                          ab.dni_kalendarzowe, ab.zasilek_razem, MIN(ab.od_kiedy) OVER (PARTITION BY ab.pracownik_id) AS min_od_kiedy
                   FROM rs_kp_listy_plac l
                   LEFT JOIN (SELECT firma_id, typ_id, typ_absencji_id
                               FROM rs_edeklaracje_skladniki s2
                               JOIN rs_edeklaracje_wersje w1 USING (typ_id, wersja_id) 
                               WHERE s2.firma_id='{firma}' AND wersja_id=(SELECT wersja_id FROM rs_edeklaracje_wersje w2 
                                                                     WHERE w2.typ_id=w1.typ_id 
                                                                           AND data_od<='{od}'
                                                                     ORDER BY w2.data_od DESC  LIMIT 1)
                              ) AS a ON a.firma_id=l.firma_id AND a.typ_id='RSA'
                   JOIN (SELECT doa.*, SUM(startflag) OVER (PARTITION BY firma_id, pracownik_id, kod_zus ORDER BY firma_id, pracownik_id, kod_zus, od_kiedy) AS grp
                         FROM 
                         (SELECT doa.*, 
                                 (CASE WHEN LAG(do_kiedy) OVER (PARTITION BY firma_id, pracownik_id, kod_zus ORDER BY firma_id, pracownik_id, kod_zus, od_kiedy)=(od_kiedy-INTERVAL '1 day') THEN 0 ELSE 1 END) AS startflag 
                          FROM rs_kp_dane_osobowe_absencje doa
                         ) doa
                        ) ab ON (ab.firma_id, ab.typ_absencji_id, ab.numer_listy_id)=(l.firma_id, a.typ_absencji_id, l.numer_listy_id)
                   JOIN rs_kp_zatrudnienie z ON (z.firma_id, z.pracownik_id)=(l.firma_id, ab.pracownik_id)
                   JOIN rs_kp_umowy_o_prace u ON (u.firma_id, u.pracownik_id, u.zatrudnienie_id)=(z.firma_id, z.pracownik_id, z.numer_id)
                   WHERE l.firma_id='{firma}' AND l.stan in ('Z','K') AND l.data_wyplaty BETWEEN '{od}' AND '{do}'
                         {war_pracownik} AND ab.kod_zus IS NOT NULL 
                         AND (CASE WHEN ab.typ_absencji_id != '3' THEN ab.zasilek_razem != 0 ELSE True END)
                         AND z.czy_absencje = TRUE 
                         AND (u.do_kiedy IS NULL OR ab.do_kiedy <= u.do_kiedy)
                   GROUP BY ab.pracownik_id, z.ubezpieczenie_id, ab.kod_zus, ab.od_kiedy, ab.do_kiedy, ab.grp, ab.dni_kalendarzowe, ab.zasilek_razem) wyn
             GROUP BY wyn.pracownik_id, wyn.ubezpieczenie_id, wyn.kod_zus, wyn.grp, wyn.min_od_kiedy
             ORDER BY min_od_kiedy;
          """.format(firma=self.firma_id, od=DataOd, do=DataDo,
                     war_pracownik=" AND ab.pracownik_id in ('"+"','".join(ListaPracownikow)+"')")
    rsglobal.ZapiszDoLogow(SQL)
    stat, dane_abs, wyn1_abs, dump = self.PolSQL.Execute(SQL, 3)
    if stat != 'OK':
        stat = 'Bd pobierania informacji o absencjach: '+stat

    if stat == 'OK' and len(dane_abs) > 0:
        Absencje = rssql.RSTableRowset2(dane_abs, wyn1_abs)
        stat, dane = zus.ZapiszRsa(IDDok, Absencje, NrRaportu, Rok, Miesiac, DataWypelnienia)

#-----------------------------------------------------------------------------------------------------------------------

if Funkcja == 2 and stat == 'OK':
    #WS 2021-10-29 - usunicie warunku AND (z.zwolnienie_data IS NULL OR z.zwolnienie_data >= l.data_wyplaty)
    #WS 2021-10-29 - dodanie JOIN na rs_kp_umowy_o_prace oraz warunku (u.do_kiedy IS NULL OR ab.do_kiedy <= u.do_kiedy)
    SQL = """SELECT DISTINCT ab.pracownik_id
             FROM rs_kp_dane_osobowe_absencje ab
             JOIN rs_kp_listy_plac l USING (firma_id, numer_listy_id)
             LEFT JOIN (SELECT DISTINCT firma_id, typ_id, typ_absencji_id
                        FROM rs_edeklaracje_skladniki s2
                        JOIN rs_edeklaracje_wersje w1 USING (typ_id, wersja_id)
                        WHERE firma_id='{firma}' AND wersja_id=(SELECT wersja_id FROM rs_edeklaracje_wersje w2
                                                           WHERE w2.typ_id=w1.typ_id AND data_od<='{od}'
                                                           ORDER BY w2.data_od DESC  LIMIT 1))
                        AS a ON a.firma_id=l.firma_id AND a.typ_id='RSA' AND a.typ_absencji_id=ab.typ_absencji_id
             JOIN rs_kp_zatrudnienie z ON (z.firma_id, z.pracownik_id)=(l.firma_id, ab.pracownik_id)
             JOIN rs_kp_umowy_o_prace u ON (u.firma_id, u.pracownik_id, u.zatrudnienie_id)=(z.firma_id, z.pracownik_id, z.numer_id)
             WHERE l.firma_id ='{firma}' AND l.stan in ('Z','K') AND l.data_wyplaty BETWEEN '{od}' AND '{do}'
                   AND ab.kod_zus IS NOT NULL
                   AND (CASE WHEN ab.typ_absencji_id != '3' THEN ab.zasilek_razem != 0 ELSE True END)
                   AND z.czy_absencje = TRUE
                   AND (u.do_kiedy IS NULL OR ab.do_kiedy <= u.do_kiedy);
          """.format(firma=self.firma_id, od=DataOd, do=DataDo)
    rsglobal.ZapiszDoLogow(SQL)
    stat,dane,dump,dump = self.PolSQL.Execute(SQL, 3)
    if stat != 'OK':
        stat = 'Bd pobierania informacji o pracownikach'
