# -*- coding: iso8859_2 -*-
#-------------------------------------------------------------------------------
# Name:        fs_kp_deklaracje_zus_rza.py
# Purpose:     tworzenie deklaracji ZUS RZA
# Author:      Wojciech Skowron
# Created:     2020/12/15
# RCS-ID:      $Id: fs_kp_deklaracje_zus_rza.py $
# Copyright:   (c) 2020 RecordSystem
# Licence:     GNU GPL
#-------------------------------------------------------------------------------

# 2021-02-03 Wojciech Skowron   - 12370 - Usunicie spacji z nr dokumentu tosamoci
# 2021-03-05 Wojciech Skowron   - 12379 - WstawNowyZKwota dla pola III.B.p2
# 2021-03-09 Wojciech Skowron   - 12387 - Pobranie danych osobowych przeniesione do _pom
# 2022-01-26 Wojciech Skowron   - 12800 - Przekazanie wersji KEDU w inicie


from funkcje.fs_kp_deklaracje_zus_pom import ZUSXML


class ZUS(ZUSXML):
    def __init__(self, rodzic, KEDU):
        from funkcje.fs_kp_deklaracje_zus_pom import ZUSXML
        ZUSXML.__init__(self, rodzic, KEDU, wersja='5_3')  #WS 2022-01-26

    def ZapiszRza(self, id, skladki, nr_raportu, rok, miesiac, data_wypelnienia):
        if not id > 1:
            self.ZrobNaglowek()

        self.WstawTekst('<ZUSRZA id_dokumentu="'+str(id)+'">')

        self.WstawNowy("I")
        self.WstawNowy("p1")
        self.WstawNowyZWartoscia("p1", str(nr_raportu).zfill(2))
        self.WstawNowyZWartoscia("p2", str(rok)+'-'+str(miesiac))
        self.Zamknij("p1")
        self.Zamknij("I")

        stat = self.WstawDanePlatnika()
        if stat != 'OK':
            return stat, ''

        id_bloku = 0

        for skl in skladki:
            id_bloku += 1
            self.WstawTekst('<III id_bloku="'+str(id_bloku)+'">')

            #WS 2021-03-09
            stat = self.WstawDaneOsoboweRozl(skl['pracownik_id'])
            if stat != 'OK':
                return stat, ''
            # self.WstawNowy("A")
            # self.WstawNowyZWartoscia("p1", skl['nazwisko'])
            # self.WstawNowyZWartoscia("p2", skl['imie'])
            # self.WstawNowyZWartoscia("p3", skl['dok_typ'])
            # self.WstawNowyZWartoscia("p4", skl['dok_nr'].replace(' ', ''))
            # self.Zamknij("A")

            self.WstawNowy("B")
            self.WstawNowy("p1")  # Kod tytuu ubezpieczenia
            self.WstawNowyZWartoscia("p1", skl['ubezpieczenie_id'][:4])
            if len(skl['ubezpieczenie_id'].strip(' ')) > 4:
                self.WstawNowyZWartoscia("p2", skl['ubezpieczenie_id'][4:5])
            else:
                self.WstawNowyZWartoscia("p2", "0")
            if len(skl['ubezpieczenie_id'].strip(' ')) > 5:
                self.WstawNowyZWartoscia("p3", skl['ubezpieczenie_id'][5:6])
            else:
                self.WstawNowyZWartoscia("p3", "0")
            self.Zamknij("p1")
            self.WstawNowyZKwota("p2", abs(skl['podstawa_zdr']))

            # Skadki na ubezpieczenia zdrowotne
            self.WstawNowyZKwota("p3", abs(skl['skl_zdr_plat']))    # finansowane przez patnika
            self.WstawNowyZKwota("p4", abs(skl['skl_zdr_budzet']))  # finansowane przez budet pastwa
            self.WstawNowyZKwota("p5", abs(skl['skl_zdr_ubez']))    # finansowane przez ubezpieczonego
            # self.WstawNowyZWartoscia("p6", skl['skl_zdr_ubez'])    # finansowane przez Fundusz Kocielny

            self.Zamknij("B")
            self.Zamknij("III")

        self.WstawNowy("VIII")
        self.WstawNowyZWartoscia("p1", data_wypelnienia)
        self.Zamknij("VIII")

        self.Zamknij("ZUSRZA")
        self.ZakonczTworzenie(not id > 1)

        tresc = self.PodajTresc()

        return stat, tresc


#-----------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------------

stat = 'OK'
dane = []
wyn1 = []
wyn2 = []


if not 'Funkcja' in klucze:
    stat='Brak podanej funkcji w kluczach'

if stat=='OK':
    Funkcja = klucze['Funkcja']

    SlPar={'NrRaportu': [1, ], 'Rok': [1, 2], 'Miesiac': [1, 2], 'DataWypelnienia': [1, ], 'ListaPracownikow': [1, ], 'IDDok': [1, ]}

    Lista=rsglobal.SprawdzParametryWKluczach(SlPar, Funkcja, klucze)
    if Lista!='':
        stat='Uwaga : wywoanie funkcji fs_kp_deklaracje_zus_rza '+str(Funkcja)+' bez parametrw: '+Lista

    if Funkcja in [1, 2]:
        Rok = rsglobal.Dekoduj(klucze['Rok'])
        Miesiac = str(rsglobal.Dekoduj(klucze['Miesiac'])).zfill(2)

    if Funkcja in [1, ]:
        IDDok = klucze['IDDok']
        NrRaportu = str(rsglobal.Dekoduj(klucze['NrRaportu'])).zfill(2)
        DataWypelnienia = rsglobal.Dekoduj(klucze['DataWypelnienia'])
        ListaPracownikow = klucze['ListaPracownikow']

#-----------------------------------------------------------------------------------------------------------------------

if Funkcja == 1 and stat == 'OK':
    zus = ZUS(self, not IDDok > 1)

    war_pracownik = " AND pracownik_id in ('" + "','".join(ListaPracownikow) + "')"

    # pobranie danych o skadkach pracownika
    SQL = """SELECT skl.* 
             FROM rs_kp_dane_osobowe_zus_skladki skl
             WHERE firma_id='{self.firma_id}' AND rodzaj='RZA' {war_pracownik}
             AND rok='{Rok}' AND miesiac='{Miesiac}'
             ORDER BY ubezpieczenie_id;"""
    SQL = SQL.format(**locals())
    rsglobal.ZapiszDoLogow(SQL)
    stat, dane_skl, wyn1_skl, dump = self.PolSQL.Execute(SQL, 3)
    if stat != 'OK':
        stat = 'Bd pobierania informacji o skadkach pracownikw'

    if stat == 'OK' and len(dane_skl) > 0:
        Skladki = rssql.RSTableRowset2(dane_skl, wyn1_skl)
        stat, dane = zus.ZapiszRza(IDDok, Skladki, NrRaportu, Rok, Miesiac, DataWypelnienia)

#-----------------------------------------------------------------------------------------------------------------------

if Funkcja == 2 and stat == 'OK':
    SQL = """SELECT pracownik_id FROM rs_kp_dane_osobowe_zus_skladki
             WHERE firma_id='{self.firma_id}' AND rodzaj='RZA' 
             AND rok='{Rok}' AND miesiac='{Miesiac}';"""
    SQL = SQL.format(**locals())
    rsglobal.ZapiszDoLogow(SQL)
    stat, dane, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat != 'OK':
        stat = 'Bd pobierania informacji o skadkach pracownikw'
