# -*- coding: iso8859_2 -*-
#-------------------------------------------------------------------------------
# Name:        fs_kp_listy_plac.py
# Purpose:     obsluga grida edytowalnego z listami pac, osobami z list i ich skadnikami
# Author:      Micha Raburski
# Created:     2015/10/27
# RCS-ID:      $Id: fs_kp_listy_plac.py$
# Copyright:   (c) 2015 Record System
# Licence:     GNU GPL
#-------------------------------------------------------------------------------

# 2019-10-01 - Jan Raburski       (11324) - Zamykanie i otwieranie listy
# 2019-03-26 - Aleksander Bargie (11449) - Na listach pac w Hali nie wywietlaj si skadniki
# 2021-02-18 - Jan Raburski       (12346) - rozbicie skladek ppk na listy
# 2021-08-26 - Aleksander Bargie (12624) - Opcja modyfikacji niemodyfikowalnych skadnikw na listach pac
# 2021-11-17 - Jan Raburski       (12692) - Uwzgldnianie wg_dat_wypaty przy pobieraniu skladnikow i stalych
# 2022-03-22 - Jan Raburski       (12925) - zmiana obslugi listy plac w absencjach (funkcja 17) i czyszczenie po usunieciu angazu
# 2022-05-05 - Jan Raburski       (13008) - dodanie obslugi F_ABP - absencje z poprzedniego miesiaca
# 2023-06-05 - Jan Raburski       (13460) - dodawanie nowego skladnika do listy
# 2024-01-23 - Jan Raburski       (13677) - zmiana w obliczeniach 26 lat do PIT11

# FUNKCJE
#
# 1 - update danych
# 5 - pobranie danych
# 6 - usunicie listy
# 7 - zamknicie listy
# 8 - otworzenie listy
# 9 -  zmiana opisu listy
# 10 - pobranie opisu listy
# 11 - usunicie angau
# 15 - pobranie danych do przeliczenia angau
# 16 - zapis danych po przeliczeniu angau
# 17 - zapis numeru listy i daty wuplaty do absencji po policzeniu pracownika
# 18 - grupowy zapis modyfikowalnych skadnikw
# 25 - Funkcja F_005 -  suma skadnikw za okres wg daty listy
# 26 - Funkcja F_006 -  rednia skadnikw za okres
# 27 - Funkcja F_007 -  suma skadnikw z listy biecej dla angau
# 28 - Funkcja F_008 -  suma skadnikw z listy biecej dla pracownika
# 29 - Funkcja F_009 -  suma skadnikw z polem w wersji biecej dla angau
# 30 - Funkcja F_010 -  suma skadnikw z polem w wersji biecej dla pracownika
# 31 - Funkcja F_05W -  suma skadnikw za okres wg daty wyplaty
# 32 - Funkcja F_05P -  suma skladnikow od poczatku roku do miesiaca poprzedniego wg daty wyplaty dla pracownika
# 33 - Funkcja F_05I -  suma skladnikow w miesiacu daty wyplaty dla pozostalych list/angazy dla pracownika
# 34 - Funkcja F_RK1 -  suma skladnikow w roku daty wyplaty dla pracownika do 26 lat
# 35 - Funkcja F_RK2 -  suma skladnikow w roku daty wyplaty dla pracownika powyej 26 lat

import rs_anakonda_daty

stat='OK'

#rsglobal.ZapiszDoLogow('KLUCZE L '+str(klucze),1)
if not 'Funkcja' in klucze:
    stat='Brak podanej funkcji w kluczach'

if stat=='OK':
    Funkcja = klucze['Funkcja']
        
    SlPar={'StanF':[5,6,7,8,11],'SzablonF':[5,6,7,8,11],'ListaF':[5,6,7,8,11],
           'OpisF':[5,6,7,8,11], 'PracownikF':[5,6,7,8,11,11], 'DataWyplatyOdF':[5,6,7,8,11],
           'DataWyplatyDoF':[5,6,7,8,11], 'RokF':[5,6,7,8,11],'MiesiacF':[5,6,7,8,11],'LKluczyF':[5,6,7,8,11],
           'Lista':[1,5,6,7,8,9,10,11,15,16,17,18,27,28,29,30,32,33,34,35],
           'Pracownik':[1,11,15,16,17,25,26,27,28,29,30,31,32,33,34,35],
           'Zatrudnienie':[1,11,16,27,29,32,33],'Umowa':[1,11,16,27,29,33],'Angaz':[1,11,16,27,29,33],'DataA':[1,11,16,27,29],
           'SSkladnikiZ':[16], 'Usuwanie':[16],'Rozliczenie':[16], 'LSkladniki':[18,25,26,27,28,31,32,33,34,35],'Pole':[29,30],
           'DataOd':[25,26,31],'IleMc':[25,26,31],'TylkoAngaz':[5],'Opis':[9], 'SzablonTak':[34, 35], 'SzablonNie':[34, 35],
           'Kasa':[7],'KasaSk':[7],'Bank':[7],'BankSk':[7],'Absencje':[17],'AbsencjePM':[17],'NowySkladnik':[15, 16]}
    
    ListaBledow=rsglobal.SprawdzParametryWKluczach(SlPar,Funkcja,klucze)
    if ListaBledow<>'':
        stat='Uwaga : wywoanie funkcji fs_kp_listy_plac z numerem '+str(Funkcja)+' bez parametrw: '+ListaBledow
        
if stat == 'OK':
    if Funkcja in [5,6,7,8,11]: 
        StanF=rsglobal.Dekoduj(klucze['StanF'])
        SzablonF=rsglobal.Dekoduj(klucze['SzablonF'])
        ListaF=rsglobal.Dekoduj(klucze['ListaF'])
        OpisF=rsglobal.Dekoduj(klucze['OpisF'])
        PracownikF=rsglobal.Dekoduj(klucze['PracownikF']).strip()
        if PracownikF != '':
            PracownikF = '000000'+PracownikF
            PracownikF = PracownikF[-6:]
        DataWyplOdF=rsglobal.Dekoduj(klucze['DataWyplatyOdF'])
        DataWyplDoF=rsglobal.Dekoduj(klucze['DataWyplatyDoF'])
        RokF=rsglobal.Dekoduj(klucze['RokF'])
        McF=rsglobal.Dekoduj(klucze['MiesiacF'])
        LKluczyF=klucze['LKluczyF']
    
    if Funkcja in [1,5,6,7,8,9,10,11,15,16,17,18,27,28,29,30,32,33,34,35]:
        Lista=rsglobal.Dekoduj(klucze['Lista'])
    
    if Funkcja in  [1,11,15,16,17,25,26,27,28,29,30,31,32,33,34,35]:
        Pracownik = rsglobal.Dekoduj(klucze['Pracownik'])

    if Funkcja in  [1,11,16,27,29]:
        Zatrudnienie = str(klucze['Zatrudnienie'])
        Umowa = str(klucze['Umowa'])
        Angaz = str(klucze['Angaz'])
        DataA = str(klucze['DataA'])

    if Funkcja in  [32]:
        Zatrudnienie = str(klucze['Zatrudnienie'])

    if Funkcja in  [15, 16]:
        NowySkladnik = str(klucze['NowySkladnik'])

    if Funkcja in  [33]:
        Zatrudnienie = str(klucze['Zatrudnienie'])
        Umowa = str(klucze['Umowa'])
        Angaz = str(klucze['Angaz'])

    if Funkcja == 9:
        Opis = klucze['Opis']

    if Funkcja == 5:
        TylkoAngaz = klucze['TylkoAngaz']
    else:
        TylkoAngaz = False

    if Funkcja == 16:
        SSkladnikiZ = klucze['SSkladnikiZ']
        Usuwanie = klucze['Usuwanie']
        Rozliczenie = klucze['Rozliczenie']

    if Funkcja == 17:
        SAbsencje = klucze['Absencje']
        SAbsencjePM = klucze['AbsencjePM']

    if Funkcja in [18, 25, 26, 27, 28, 31, 32, 33, 34, 35]:
        LSkladniki = klucze['LSkladniki']
        
    if Funkcja in [29,30]:
        Pole = klucze['Pole']

    if Funkcja in [25,26,31]:
        DataOd = klucze['DataOd']
        IleMc = klucze['IleMc']

    if Funkcja in [34, 35]:
        SzablonTak = klucze['SzablonTak']
        SzablonNie = klucze['SzablonNie']

    if Funkcja == 7:
        Kasa = klucze['Kasa']
        KasaSk = klucze['KasaSk']
        Bank = klucze['Bank']
        BankSk = klucze['BankSk']

dane = []
wyn1 = []
wyn2 = []

UprIns=((self.oddzial_id,'062003') in self.uprawnienia) or self.operator_id == STANDARD_OPERATOR_ID
UprUpd=UprIns or ((self.oddzial_id,'062002') in self.uprawnienia) or self.operator_id == STANDARD_OPERATOR_ID
UprDel=((self.oddzial_id,'062004') in self.uprawnienia) or self.operator_id == STANDARD_OPERATOR_ID
UprZam=((self.oddzial_id,'062005') in self.uprawnienia) or self.operator_id == STANDARD_OPERATOR_ID
UprOtw=((self.oddzial_id,'062006') in self.uprawnienia) or self.operator_id == STANDARD_OPERATOR_ID
UprPrzel=((self.oddzial_id,'062007') in self.uprawnienia) or self.operator_id == STANDARD_OPERATOR_ID

Insert = []
Update = []
Delete = []

if stat=='OK' and Funkcja == 1:
    Update=klucze['Update']
    Insert=klucze['Insert']
    Delete=klucze['Delete']

#-------------------------------------------------------------------------------
#---   FUNKCJA F_005 - suma skadnikw za okres (25)
#-------------------------------------------------------------------------------

if Funkcja == 25 and stat == 'OK':
    DataOd = DataOd[:7]+'-01'

    SQL = "SELECT COALESCE(SUM(wartosc),0) FROM rs_kp_listy_plac_osoby_skladniki lpsk INNER JOIN rs_kp_listy_plac lp USING(firma_id, numer_listy_id) "
    SQL+= " WHERE lpsk.firma_id='"+self.firma_id+"' AND pracownik_id='"+Pracownik+"'"
    SQL+=" AND skladnik_id IN ("+str(LSkladniki.split(","))[1:-1]+") AND lp.data_od < '"+DataOd+"' AND lp.data_od >= '"+DataOd+"' ::DATE - INTERVAL '"+str(IleMc)+" MONTH' - INTERVAL '1 DAY'"
    rsglobal.ZapiszDoLogow('SQL25 '+SQL,1)
    stat,DSuma,dump,dump = self.PolSQL.Execute(SQL,3)
    #rsglobal.ZapiszDoLogow('D25 '+str(DSuma),1)
    if stat=='OK':
        dane=[DSuma[0][0]]

#-------------------------------------------------------------------------------
#---   FUNKCJA F_006 - rednia skadnikw za okres (26)
#-------------------------------------------------------------------------------

if Funkcja == 26 and stat == 'OK':
    # pobieramy daty - koniec poprzedniego miesica i pocztek pierwszego sprawdzanego miesica
    DataOd = DataOd[:7]+'-01'
    SQL = "SELECT '"+DataOd+"'::DATE - INTERVAL '1 DAY' - INTERVAL '"+str(IleMc)+" MONTH','"+DataOd+"'::DATE - INTERVAL '1 DAY'"
    stat,DDaty,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat == 'OK':
        DataOd = DDaty[0][0][:7]+'-01'
        DataDo = DDaty[0][1]

        # pobieramy sumy skadnikw w podziale na rok i mc
        SQL = "SELECT EXTRACT(YEAR FROM data_od), EXTRACT(MONTH FROM data_od), COALESCE(SUM(wartosc),0) "
        SQL+=" FROM rs_kp_listy_plac_osoby_skladniki WHERE firma_id='"+self.firma_id+"' AND pracownik_id='"+Pracownik+"'"
        SQL+=" AND skladnik_id IN ("+str(LSkladniki.split(","))[1:-1]+") AND data_od >='"+DataOd+"' AND data_od <= '"+DataDo+"'"
        SQL+=" GROUP BY EXTRACT(YEAR FROM data_od), EXTRACT(MONTH FROM data_od)"
        rsglobal.ZapiszDoLogow('SQL26 '+SQL,1)
        stat,DSuma,dump,dump = self.PolSQL.Execute(SQL,3)
        #rsglobal.ZapiszDoLogow('D26 '+str(DSuma),1)
        if stat=='OK':
            # sumujemy w ptli niezerowe miesice
            Suma = 0
            Ile = 0
            for poz in DSuma:
                if poz[2] != 0:
                    Suma += poz[2]
                    Ile += 1

            if Ile == 0:
                dane = [0]
            else:
                dane=[rsglobal.RSRound(Suma/Ile,2)]


#-------------------------------------------------------------------------------
#---   FUNKCJA F_007 - suma skadnikw z listy biecj dla angau (27)
#-------------------------------------------------------------------------------

if Funkcja == 27 and stat == 'OK':
    SQL = "SELECT COALESCE(SUM(wartosc),0) FROM rs_kp_listy_plac_osoby_skladniki WHERE firma_id='"+self.firma_id+"'"
    SQL+=" AND numer_listy_id='"+Lista+"'"+" AND pracownik_id = '"+Pracownik+"'"
    SQL+=" AND zatrudnienie_id="+Zatrudnienie+" AND umowa_id="+Umowa+" AND angaz_id="+Angaz
    SQL+=" AND data_od='"+DataA+"' AND skladnik_id IN ("+str(LSkladniki.split(","))[1:-1]+")"
    rsglobal.ZapiszDoLogow('SQL27 '+SQL,1)
    stat,DSuma,dump,dump = self.PolSQL.Execute(SQL,3)
    #rsglobal.ZapiszDoLogow('D27 '+str(DSuma),1)
    if stat=='OK':
        dane=[DSuma[0][0]]

#-------------------------------------------------------------------------------
#---   FUNKCJA F_008 - suma skadnikw z listy biecj dla pracownika (28)
#-------------------------------------------------------------------------------

if Funkcja == 28 and stat == 'OK':
    SQL = "SELECT COALESCE(SUM(wartosc),0) FROM rs_kp_listy_plac_osoby_skladniki WHERE firma_id='"+self.firma_id+"'"
    SQL+=" AND numer_listy_id='"+Lista+"'"+" AND pracownik_id = '"+Pracownik+"'"
    SQL+=" AND skladnik_id IN ("+str(LSkladniki.split(","))[1:-1]+")"
    rsglobal.ZapiszDoLogow('SQL28 '+SQL,1)
    stat,DSuma,dump,dump = self.PolSQL.Execute(SQL,3)
    #rsglobal.ZapiszDoLogow('D28 '+str(DSuma),1)
    if stat=='OK':
        dane=[DSuma[0][0]]

#-------------------------------------------------------------------------------
#---   FUNKCJA F_009 - suma skadnikw dla pola w wersjach dla angau (29)
#-------------------------------------------------------------------------------

if Funkcja == 29 and stat == 'OK':
    if not Pole in ['do_podatku_dochodowego', 'do_skladki_zus',
        'do_grupy_statystycznej', 'do_brutto', 'do_sumy_potracen',
        'do_sumy_zasilkow', 'z_zus_do_podatku_dochodowego',
        'do_nagrody_z_zysku', 'do_podstawy_zasilku', 'druk_na_zbiorowce']:
        stat='Prba uruchomienia funkcji F_009 z nieobsugiwanym polem '+Pole
           
    if stat =='OK':   
        SQL = "SELECT COALESCE(SUM(wartosc),0) FROM rs_kp_listy_plac_osoby_skladniki ps " \
           " WHERE firma_id='"+self.firma_id+"' " \
           " AND numer_listy_id='"+Lista+"'"+" AND pracownik_id = '"+Pracownik+"'" \
           " AND zatrudnienie_id="+Zatrudnienie+" AND umowa_id="+Umowa+" AND angaz_id="+Angaz +"" \
           " AND data_od='"+DataA+"' AND (SELECT "+Pole+" FROM rs_kp_skladniki_placowe_wersje " \
           " WHERE firma_id=ps.firma_id AND skladnik_id=ps.skladnik_id AND data_od<='"+DataA+"' " \
           " AND (data_do >= '"+DataA+"' OR data_do IS NULL) ORDER BY data_od DESC LIMIT 1)"
           
    rsglobal.ZapiszDoLogow('SQL29 '+SQL,1)
    stat,DSuma,dump,dump = self.PolSQL.Execute(SQL,3)
    #rsglobal.ZapiszDoLogow('D29 '+str(DSuma),1)
    if stat=='OK':
        dane=[DSuma[0][0]]

#-------------------------------------------------------------------------------
#---   FUNKCJA F_010 - suma skadnikw dla pola w wersjach dla listy (30)
#-------------------------------------------------------------------------------

if Funkcja == 30 and stat == 'OK':
    if not Pole in ['do_podatku_dochodowego', 'do_skladki_zus',
        'do_grupy_statystycznej', 'do_brutto', 'do_sumy_potracen',
        'do_sumy_zasilkow', 'z_zus_do_podatku_dochodowego',
        'do_nagrody_z_zysku', 'do_podstawy_zasilku', 'druk_na_zbiorowce']:
        stat='Prba uruchomienia funkcji F_010 z nieobsugiwanym polem '+Pole

    if stat =='OK':    
        # pobranie daty listy pac
        SQL="SELECT data_od FROM rs_kp_listy_plac WHERE firma_id='"+self.firma_id+"' AND numer_listy_id='"+Lista+"'"
        stat,DListy,dump,dump = self.PolSQL.Execute(SQL,3)
        if stat<>'OK':
            stat = u"Bd pobrania danych listy pac :"+stat
        elif len(DListy) == 0:
            stat = u"Brak listy o podanym numerze :"+Lista
        else:
            DataL=DListy[0][0]
    
    if stat =='OK':    
        SQL = "SELECT COALESCE(SUM(wartosc),0) FROM rs_kp_listy_plac_osoby_skladniki ps " \
           " WHERE firma_id='"+self.firma_id+"' " \
           " AND numer_listy_id='"+Lista+"'"+" AND pracownik_id = '"+Pracownik+"'" \
           " AND (SELECT "+Pole+" FROM rs_kp_skladniki_placowe_wersje " \
           " WHERE firma_id=ps.firma_id AND skladnik_id=ps.skladnik_id AND data_od<='"+DataL+"' " \
           " AND (data_do >= '"+DataL+"' OR data_do IS NULL) ORDER BY data_od DESC LIMIT 1)"
           
    rsglobal.ZapiszDoLogow('SQL29 '+SQL,1)
    stat,DSuma,dump,dump = self.PolSQL.Execute(SQL,3)
    #rsglobal.ZapiszDoLogow('D29 '+str(DSuma),1)
    if stat=='OK':
        dane=[DSuma[0][0]]

#-------------------------------------------------------------------------------
#---   FUNKCJA F_05W - suma skadnikw za okres wg daty wyplaty (30)
#-------------------------------------------------------------------------------

if Funkcja == 31 and stat == 'OK':
    DataOd = DataOd[:7]+'-01'

    SQL = "SELECT COALESCE(SUM(wartosc),0) FROM rs_kp_listy_plac_osoby_skladniki lpsk INNER JOIN rs_kp_listy_plac lp USING(firma_id, numer_listy_id) "
    SQL+= " WHERE lpsk.firma_id='"+self.firma_id+"' AND pracownik_id='"+Pracownik+"'"
    SQL+=" AND skladnik_id IN ("+str(LSkladniki.split(","))[1:-1]+") AND lp.data_wyplaty < '"+DataOd+"' AND lp.data_wyplaty >= '"+DataOd+"' ::DATE - INTERVAL '"+str(IleMc)+" MONTH' - INTERVAL '1 DAY'"
    rsglobal.ZapiszDoLogow('SQL31 '+SQL,1)
    stat,DSuma,dump,dump = self.PolSQL.Execute(SQL,3)
    #rsglobal.ZapiszDoLogow('D25 '+str(DSuma),1)
    if stat=='OK':
        dane=[DSuma[0][0]]

#-------------------------------------------------------------------------------
#---   FUNKCJA F_05P - suma skadnikw od poczatku roku do miesiaca z daty wyplaty (32)
#-------------------------------------------------------------------------------

if Funkcja == 32 and stat == 'OK':
    SQL = "SELECT data_wyplaty FROM rs_kp_listy_plac WHERE firma_id='"+self.firma_id+"' AND numer_listy_id='"+Lista+"'"
    stat,DSuma,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat == 'OK':
        rok = DSuma[0][0][0:4]
        mc = DSuma[0][0][5:7]
    if stat == 'OK':
        SQL = "SELECT COALESCE(SUM(wartosc),0) FROM rs_kp_listy_plac_osoby_skladniki lpsk INNER JOIN rs_kp_listy_plac lp USING(firma_id, numer_listy_id) "
        SQL+= " WHERE lpsk.firma_id='"+self.firma_id+"' AND pracownik_id='"+Pracownik+"' AND zatrudnienie_id="+Zatrudnienie
        SQL+=" AND skladnik_id IN ("+str(LSkladniki.split(","))[1:-1]+") AND EXTRACT(YEAR FROM lp.data_wyplaty) = "+rok
        SQL += " AND EXTRACT(MONTH FROM lp.data_wyplaty) < "+mc
        rsglobal.ZapiszDoLogow('SQL32 '+SQL,1)
        stat,DSuma,dump,dump = self.PolSQL.Execute(SQL,3)
        #rsglobal.ZapiszDoLogow('D25 '+str(DSuma),1)
        if stat=='OK':
            dane=[DSuma[0][0]]

#-------------------------------------------------------------------------------
#---   FUNKCJA F_05I - suma skadnikw z miesiaca  wyplaty z wyjatkiem obliczanego angazu (33)
#-------------------------------------------------------------------------------

if Funkcja == 33 and stat == 'OK':
    SQL = "SELECT data_wyplaty FROM rs_kp_listy_plac WHERE firma_id='"+self.firma_id+"' AND numer_listy_id='"+Lista+"'"
    stat,DSuma,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat == 'OK':
        rok = DSuma[0][0][0:4]
        mc = DSuma[0][0][5:7]
    if stat == 'OK':
        SQL = "SELECT COALESCE(SUM(wartosc),0) FROM rs_kp_listy_plac_osoby_skladniki lpsk INNER JOIN rs_kp_listy_plac lp USING(firma_id, numer_listy_id) "
        SQL+= " WHERE lpsk.firma_id='"+self.firma_id+"' AND pracownik_id='"+Pracownik+"'"
        SQL+=" AND skladnik_id IN ("+str(LSkladniki.split(","))[1:-1]+") AND EXTRACT(YEAR FROM lp.data_wyplaty) = "+rok
        SQL += " AND EXTRACT(MONTH FROM lp.data_wyplaty) = "+mc+ " AND lpsk.zatrudnienie_id="+str(Zatrudnienie)
        SQL += " AND (lp.numer_listy_id<>'"+Lista+"' OR lpsk.umowa_id<>"+str(Umowa)+" OR lpsk.angaz_id<>"+str(Angaz)+")"
        rsglobal.ZapiszDoLogow('SQL33 '+SQL,1)
        stat,DSuma,dump,dump = self.PolSQL.Execute(SQL,3)
        #rsglobal.ZapiszDoLogow('D25 '+str(DSuma),1)
        if stat=='OK':
            dane=[DSuma[0][0]]

#-------------------------------------------------------------------------------
#---   FUNKCJA F_RK1 - suma skadnikw za caly rok z roku z daty wyplaty do 26 lat (34)
#---   FUNKCJA F_RK2 - suma skadnikw za caly rok z roku z daty wyplaty powyzej 26 lat (35)
#-------------------------------------------------------------------------------

if Funkcja in [34, 35] and stat == 'OK':  #JR 13677
    SQL = "SELECT data_wyplaty FROM rs_kp_listy_plac WHERE firma_id='"+self.firma_id+"' AND numer_listy_id='"+Lista+"'"
    stat, DSuma, dump, dump = self.PolSQL.Execute(SQL,3)
    if stat == 'OK':
        rokwyplaty = DSuma[0][0][0:4]
        SQL = "SELECT data_urodzenia FROM rs_kp_dane_osobowe WHERE firma_id='"+self.firma_id+"' AND pracownik_id='"+Pracownik+"'"
        stat, DSuma, dump, dump = self.PolSQL.Execute(SQL,3)
    if stat == 'OK':
        dataurodzenia = DSuma[0][0]
        dataprzelomu = str(int(dataurodzenia[0:4])+10026)[1:] + dataurodzenia[4:]
        #rokprzelomu = dataprzelomu[0:4]
        #mcprzelomu = dataprzelomu[5:7]
        if Funkcja == 34:  # do 26 lat
            #warunek = " AND (EXTRACT(YEAR from lp.data_od) < " + rokprzelomu
            #warunek += " OR (EXTRACT(YEAR from lp.data_od) = " + rokprzelomu + " AND EXTRACT(MONTH FROM lp.data_od) <'" + mcprzelomu + "'))"
            warunek = " AND lp.data_wyplaty <= '" + dataprzelomu + "'"
        else:  # powyzej 26 lat
            #warunek = " AND (EXTRACT(YEAR from lp.data_od) > " + rokprzelomu
            #warunek += " OR (EXTRACT(YEAR from lp.data_od) = " + rokprzelomu + " AND EXTRACT(MONTH FROM lp.data_od) >='" + mcprzelomu + "'))"
            warunek = " AND lp.data_wyplaty > '" + dataprzelomu + "'"
        if SzablonTak != '':
            warunek += " AND lp.lista_id IN ("+str(SzablonTak.split(","))[1:-1]+")"
            #warunek += " AND lp.lista_id ='" + SzablonTak + "'"
        if SzablonNie != '':
            #warunek += " AND lp.lista_id !='" + SzablonNie + "'"
            warunek += " AND NOT lp.lista_id IN (" + str(SzablonNie.split(","))[1:-1] + ")"
    if stat == 'OK':
        SQL = "SELECT COALESCE(SUM(wartosc),0) FROM rs_kp_listy_plac_osoby_skladniki lpsk INNER JOIN rs_kp_listy_plac lp USING(firma_id, numer_listy_id) "
        SQL += " WHERE lpsk.firma_id='"+self.firma_id+"' AND pracownik_id='"+Pracownik+"'"
        SQL += " AND skladnik_id IN ("+str(LSkladniki.split(","))[1:-1]+") AND EXTRACT(YEAR FROM lp.data_wyplaty) = "+rokwyplaty
        SQL += warunek
        rsglobal.ZapiszDoLogow('SQL34|35 '+SQL,1)
        stat, DSuma, dump, dump = self.PolSQL.Execute(SQL,3)
        #rsglobal.ZapiszDoLogow('D25 '+str(DSuma),1)
        if stat == 'OK':
            dane = [DSuma[0][0]]

#-------------------------------------------------------------------------------
#---   ZMIANA OPISU (9)
#-------------------------------------------------------------------------------

if Funkcja == 9 and stat == 'OK':
    if not UprUpd:
        stat=u'Brak uprawnie do edycji list'
    if stat=='OK':
        sql = "SELECT stan FROM rs_kp_listy_plac WHERE firma_id = '"+self.firma_id+"' AND numer_listy_id='"+str(Lista)+"';"
        stat,DListy,dump,dump = self.PolSQL.Execute(sql,3)
    if stat=='OK':
        if len(DListy)==0:
            stat = u'Brak wybranej listy'
        elif DListy[0][0] != 'W':
            stat = u'Mona edytowa jedynie list w stanie W'

    if stat=='OK':
        SQL = "UPDATE rs_kp_listy_plac SET opis = '"+Opis+"' WHERE firma_id = '"+self.firma_id+"' AND numer_listy_id='"+str(Lista)+"';"
        stat,dump,dump,dump = self.PolSQL.Execute(SQL,1)

#-------------------------------------------------------------------------------
#---   POBIERANIE OPISU (10)
#-------------------------------------------------------------------------------

if Funkcja == 10 and stat == 'OK':
    sql = "SELECT opis FROM rs_kp_listy_plac WHERE firma_id = '"+self.firma_id+"' AND numer_listy_id='"+str(Lista)+"';"
    stat,DListy,dump,dump = self.PolSQL.Execute(sql,3)
    if stat=='OK':
        if len(DListy)>0:
            dane=[DListy[0][0]]

#-------------------------------------------------------------------------------
#---   POMOCNICZY FILTR I STAN
#-------------------------------------------------------------------------------

if Funkcja in [1,6,7,8,11,15,16,18]:
    if stat == 'OK':
        FiltrL = " WHERE firma_id='"+self.firma_id+"' AND numer_listy_id='"+Lista+"';"
        FiltrLM = " AND firma_id='"+self.firma_id+"' AND numer_listy_id='"+Lista+"' "

        # pobranie stanu listy pac
        SQL = "SELECT stan,data_od,data_do,lista_id,data_wyplaty,tekst_na_przelewie FROM rs_kp_listy_plac "+FiltrL
        stat,DStan,dump,dump = self.PolSQL.Execute(SQL,3)
        if stat != 'OK':
            stat = "Bd przy pobieraniu stanu: "+stat   
        if stat == 'OK':
            if len(DStan) == 0:
                Czynnosc = {6:'usunicia',7:'zamknicia',8:'otworzenia',11:'usunicia angau z ',15:'przeliczenia',16:'przeliczenia',18:'grupowej zmiany skadnikw modyfikowalnych'}
                stat = "Prba "+Czynnosc[Funkcja]+" nieistniejcej listy pac"
        if stat == 'OK':
            Stan = DStan[0][0]
            DataOdL = DStan[0][1]
            DataDoL = DStan[0][2]
            SzablonL = DStan[0][3]
            DataWypl = DStan[0][4]
            TekstNaPrzelewie = DStan[0][5].strip()


#-------------------------------------------------------------------------------
#---   DELETE LISTY (6)
#-------------------------------------------------------------------------------
            
if Funkcja == 6 and stat=="OK":
    if not UprDel:
        stat='BR UPR DEL'

    if stat == 'OK':
        if Stan != 'W':
            stat = 'Nie mona usun listy w stanie innym ni W'

    # sprawdzenie czy w patnociach nie wystpuje klucz z listy pac
    if stat == 'OK':
        SQL = "SELECT numer_listy_id FROM rs_platnosci " + FiltrL.strip(';') + " LIMIT 1;"
        stat, DPlat, dump, dump = self.PolSQL.Execute(SQL,3)
        if stat != 'OK':
            stat = "Bd przy pobieraniu patnoci dla listy pac: "+stat
        else:
            if len(DPlat) > 0:
                stat = "Nie mona usun wybranej listy pac, bo jej numer jest uyty w kartotece patnoci"

    # usuniecie numeru listy z absencji
    if stat == 'OK':
        SQL = "UPDATE rs_kp_dane_osobowe_absencje SET numer_listy_id=NULL, data_wyplaty=NULL  " + FiltrL
        stat, dump, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL), 1)
        if stat != 'OK':
            stat = "Bd przy usuwaniu danych listy z absencji: " + stat


    # waciwe usuwanie
    if stat == 'OK':
        SQL = "DELETE FROM rs_kp_listy_plac " + FiltrL
        stat, dump, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL), 1)
        if stat != 'OK':
            stat = "Bd przy usuwaniu danych: " + stat

#-------------------------------------------------------------------------------
#---   ZAMYKANIE LISTY (7)
#-------------------------------------------------------------------------------
            
if Funkcja == 7 and stat == "OK":
    if not UprZam:
        stat='BR UPR ZAM'
        
    if stat == 'OK':
        if Stan != 'W':
            stat = 'Nie mona zamyka listy w stanie innym ni W.'

    beneficjenci = []  # lista platnosci z zobowiazan dla beneficjentow
    # DataOdL, DataDoL, SzablonL, DataWypl pobrane wczesniej

    # sprawdzenie czy wszystkie osoby sa przeliczone
    if stat == 'OK':
        SQL = "SELECT COUNT(*) FROM rs_kp_listy_plac_osoby "
        SQL += " WHERE NOT przeliczony " + FiltrLM
        stat,wynik,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),3)
        if stat != 'OK':
            stat = "Bd przy sprawdzaniu nieprzeliczonych angay: "+stat
        elif wynik[0][0] > 0:
            stat = "Nie wszystkie angae s przeliczone. Zamknicie listy niemoliwe."

    # pobranie skladnikow zobowiazan dla osob z listy i zapis zmian w zobowiazaniach
    if stat == 'OK':
        SQL = "SELECT lpo.pracownik_id, lpo.skladnik_id, wartosc, z.wartosc_poczatkowa, z.beneficjent_id,"
        SQL += " TRIM(dos.imie) || ' ' || TRIM(dos.nazwisko), TRIM(zk.opis), z.data_od "
        SQL += " FROM rs_kp_listy_plac_osoby_skladniki lpo "
        SQL += " INNER JOIN rs_kp_dane_osobowe_zobowiazania_ratalne z USING (firma_id, pracownik_id, skladnik_id) "
        SQL += " INNER JOIN rs_kp_dane_osobowe dos USING (firma_id, pracownik_id) "
        SQL += " INNER JOIN rs_kp_zobowiazania_ratalne zk ON (z.firma_id=zk.firma_id AND z.zobowiazanie_id=zk.kod) "
        SQL += " WHERE (z.wartosc_poczatkowa = 0 OR (z.wartosc_poczatkowa > 0 AND wartosc_aktualna > 0))"
        SQL += " AND z.data_od<='" + str(DataDoL) + "' AND wartosc != 0 AND lpo.firma_id='"+self.firma_id+"' AND lpo.numer_listy_id='"+Lista+"' "
        SQL += " ORDER BY lpo.pracownik_id, lpo.skladnik_id"
        stat, wynik, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),3)
        if stat != 'OK':
            stat = "Bd przy pobieraniu skadnikw do zobowiza: "+stat
        elif len(wynik) > 0:
            # zapis skladnikow do zobowiazan ratalnych
            for zob in wynik:
                SQL = "UPDATE rs_kp_dane_osobowe_zobowiazania_ratalne SET wartosc_aktualna = wartosc_aktualna"
                if zob[3] == 0:
                    SQL += " - "
                else:
                    SQL += " + "
                SQL += str(zob[2]) + " WHERE firma_id='"+self.firma_id+"' AND pracownik_id='" + zob[0]
                SQL += "' AND skladnik_id='" + zob[1] + "' AND data_od='" + str(zob[7]) + "'"
                stat, dump, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL), 1)
                if stat != 'OK':
                    stat = "Bd przy aktualizacji zobowiza: " + stat
                else:
                    # zebranie platnosci dla beneficjentow jesli sa w zobowiazaniach
                    if not (zob[4].strip() in ['', 'None', '<None>']):
                        beneficjenci.append([zob[4],zob[2],zob[5],zob[6]])

    # zapis numeru listy do absencji jeli jest glowna i ma czy absencje
    #if stat == 'OK':
    #    SQL = "SELECT a.pracownik_id, a.lista_id FROM rs_kp_listy_plac_osoby lpo "
    #    SQL += " INNER JOIN rs_kp_angaze a USING (firma_id, pracownik_id, zatrudnienie_id, umowa_id, angaz_id) "
    #    SQL += " INNER JOIN rs_kp_zatrudnienie za ON (lpo.firma_id=za.firma_id AND lpo.pracownik_id=za.pracownik_id AND lpo.zatrudnienie_id=za.numer_id) "
    #    SQL += " WHERE za.czy_absencje AND lpo.firma_id='"+self.firma_id+"' AND lpo.numer_listy_id='"+Lista+"' "
    #    stat, wynik, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),3)
    #    if stat != 'OK':
    #        stat = "Bd przy pobieraniu szablonw list plac z angay: "+stat
    #    else:
    #        SQL = ''
    #        for angaz in wynik:
    #            if angaz[1] == SzablonL:
    #                PomOd = rs_anakonda_daty.DodajKilkaDni(DataOdL, -31)
    #                SQL += "UPDATE rs_kp_dane_osobowe_absencje SET numer_listy_id='"
    #                SQL += Lista + "', data_wyplaty='" + str(DataWypl) + "'"
    #                SQL += " WHERE firma_id='"+self.firma_id+"' AND pracownik_id='"+angaz[0]+"'"
    #                SQL += " AND COALESCE(numer_listy_id,'')=''"
    #                SQL += " AND od_kiedy>='"+str(PomOd)+"' AND do_kiedy<='"+str(DataDoL)+"'; "
    #        if SQL != '':
    #            stat, dump, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL), 1)
    #        if stat != 'OK':
    #            stat = "Bd przy aktualizacji absencji: " + stat

    # zapis platnosci
    if stat == 'OK':
        # beneficjenci ze zobowiazan
        INS = ''
        for ben in beneficjenci:
            INS += "INSERT INTO rs_platnosci (firma_id, stan, konto_id, kontrahent_id, nr_faktury,"
            INS += " kwota_oryg,kwota_platnosci,termin_plat, op_wpr,numer_listy_id) VALUES ("
            INS += " '" + self.firma_id + "', 'W', '" + str(Bank) + "','" + ben[0] + "', "
            INS += " '" + ben[2]+" "+ben[3] + "', " + str(ben[1]) + "," + str(ben[1]) + ",'" + str(DataWypl) + "', "
            INS += str(self.operator_id) + ",'" + Lista + "');"
        if INS != '':
            stat, dump, dump, dump = self.PolSQL.Execute(INS, 1)
        if stat != 'OK':
            stat = "Bd przy zapisie patnoci dla beneficjentw: "+stat

        if stat == 'OK':
            # pracownicy
            SQL = "SELECT pracownik_id, ABS(SUM( CASE WHEN skladnik_id IN ("+str(BankSk.split(","))[1:-1]+") THEN COALESCE(wartosc,0)"
            SQL += " ELSE 0 END)) AS wyplata_bank, "
            SQL += " ABS(SUM( CASE WHEN skladnik_id IN (" + str(KasaSk.split(","))[1:-1] + ") THEN COALESCE(wartosc,0)"
            SQL += " ELSE 0 END)) AS wyplata_kasa "
            SQL += " FROM rs_kp_listy_plac_osoby_skladniki "
            SQL += " WHERE firma_id='"+self.firma_id+"' AND numer_listy_id='"+Lista+"'"
            SQL += "  GROUP BY pracownik_id ORDER BY pracownik_id"
            stat, wynik, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL), 3)
            if stat != 'OK':
                stat = "Bd przy pobieraniu sum skadnikw do wypaty: " + stat
            else:
                INS = ''
                for prac in wynik:
                    if prac[1] > 0:
                        #bank
                        INS += "INSERT INTO rs_platnosci (firma_id, stan, konto_id, pracownik_id, nr_faktury,"
                        INS += " kwota_oryg,kwota_platnosci,termin_plat, op_wpr,numer_listy_id) VALUES ("
                        INS += " '" + self.firma_id + "', 'W', '" + Bank + "','" + prac[0] + "','"+TekstNaPrzelewie+"', "
                        INS += str(prac[1]) + ", " + str(prac[1]) + ",'" + str(DataWypl)
                        INS += "'," + str(self.operator_id) + ",'" + Lista + "');"
                    elif prac[2] > 0:
                        # kasa
                        INS += "INSERT INTO rs_platnosci (firma_id, stan, konto_id, pracownik_id, nr_faktury,"
                        INS += " kwota_oryg,kwota_platnosci,termin_plat, op_wpr,numer_listy_id) VALUES ("
                        INS += " '" + self.firma_id + "', 'W', '" + Kasa + "','" + prac[0] + "','" + TekstNaPrzelewie + "', "
                        INS += str(prac[2]) + ", " + str(prac[2]) + ",'" + str(DataWypl)
                        INS += "'," + str(self.operator_id) + ",'" + Lista + "');"
                if INS != '':
                    stat, dump, dump, dump = self.PolSQL.Execute(INS, 1)
                    if stat != 'OK':
                        stat = "Bd przy zapisie patnoci dla pracownikw: " + stat
                #else:
                #    stat = "Brak patnoci dla pracownikw: " + stat

    if stat == 'OK':
        # skladki PPK
        Rok = DataDoL[0:4]
        Mc = DataDoL[5:7]
        SQL = "SELECT pracownik_id,"
        SQL += " SUM( CASE WHEN TRIM(skladnik_id)='PPK00' THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id)='PPK01' THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id)='PPK02' THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id)='PPK03' THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id)='PPK04' THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id)='PPKOBN' THEN COALESCE(wartosc,0) ELSE 0 END) "
        SQL += " FROM rs_kp_listy_plac_osoby_skladniki "
        SQL += " WHERE firma_id='" + self.firma_id + "' AND numer_listy_id='" + Lista + "'"
        SQL += "  GROUP BY pracownik_id ORDER BY pracownik_id"
        stat, wynik, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL), 3)
        if stat != 'OK':
            stat = "Bd przy pobieraniu sum skadnikw do PPK: " + stat
        else:
            INSUPD = ''
            for prac in wynik:
                if prac[2] != 0: # sa skladki na PPK
                    INSUPD += "INSERT INTO rs_kp_dane_osobowe_ppk_skladki (firma_id, pracownik_id, rok, mc, numer_listy_id,"
                    INSUPD += " podstawa,skladka_firma_podst,skladka_firma_dodat,skladka_prac_podst,skladka_prac_dodat,skladka_obnizona)"
                    INSUPD += " VALUES ('" + self.firma_id + "', '" + prac[0] + "','" + Rok + "','" + Mc + "','" + Lista + "',"
                    INSUPD += str(prac[1]) + ", " + str(prac[2]) + ",- " + str(prac[3]) + ",- " + str(prac[4]) + ",- " + str(prac[5]) + "," + str(prac[6]==1.0) + ");"
            if stat == 'OK' and INSUPD != '':
                stat, dump, dump, dump = self.PolSQL.Execute(INSUPD, 1)
                if stat != 'OK':
                    stat = "Bd przy zapisie skadek PPK dla pracownikw: " + stat

    if stat == 'OK':
        # skladki ZUS RCA i RZA
        Rok = DataWypl[0:4]
        Mc = DataWypl[5:7]
        RokL = DataOdL[0:4]
        McL = DataOdL[5:7]
        # tutaj powinno byc pobieranie z rs_edeklaracje_skladniki listy skladnikow, ktore powinny byc zaliczana
        # odpowiednich kolumn w rs_kp_dane_osobowe_zus_skladki, ale jest zaslepka dla Hali Wola
        RCASK = []
        RCASK.append("'26A'")  # 0 - podst_chor
        RCASK.append("'261'")  # 1 - skl_chor_ubez
        RCASK.append("'250'")   # 2 - podst_emeryt_rent
        RCASK.append("'255'")  # 3 - skl_emeryt_budzet
        RCASK.append("'253'")  # 4 - skl_emeryt_plat
        RCASK.append("'252'")  # 5 - skl_emeryt_ubez
        RCASK.append("'258'")  # 6 - skl_rent_budzet
        RCASK.append("'257'")  # 7 - skl_rent_plat
        RCASK.append("'256'")  # 8 - skl_rent_ubez
        RCASK.append("'260'")  # 9 - podst_wyp
        RCASK.append("'263'")  # 10 - skl_wyp_ubez
        RCASK.append("'263'")  # 11 - skl_wyp_ubez
        RCASK.append("'265'")  # 12 - podstawa_zdr
        RCASK.append("'55Y','55Z'")  # 13 -  skl_zdr_plat
        RCASK.append("'555'")  # 14 - skl_zdr_ubez
        RCASK.append("'251'")  # 0 - czy przekroczenie
        RCASK.append("'259'")  # 1 - podstawa fp
        RCASK.append("'25G'")  # 2 - podstawa fgsp
        RCASK.append("'556'")  # 3 - skl_zdr_budzet
        RCASK.append("'PPK01'")  # ??
        SQL = "SELECT lps.pracownik_id,"
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[0] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "  # 1
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[1] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[2] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[3] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[4] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[5] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[6] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[7] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[8] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[9] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "  # 10
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[10] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[11] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[12] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[13] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[14] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "  # 15
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[15] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[16] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[17] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "  # 18
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[18] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "  # 19
        SQL += " CASE WHEN kpz.ubezpieczenie_rsa IS NOT NULL AND (lp.data_od,lp.data_do) OVERLAPS "
        SQL += " (kpz.data_obowiazywania_od,kpz.data_obowiazywania_do) THEN kpz.ubezpieczenie_rsa ELSE kpz.ubezpieczenie_id END as kod_ubezp,"  # 20
        SQL += " kpa.etat,"
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[19] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "  # 22
        SQL += " COALESCE(kpz.zwolnienie_data, '9999-12-31') as zdata, kpz.ubezpieczenie_id "  #23, 24
        SQL += " FROM rs_kp_listy_plac_osoby_skladniki lps"
        SQL += " INNER JOIN rs_kp_listy_plac lp USING(firma_id,numer_listy_id)"
        SQL += " INNER JOIN rs_kp_angaze kpa USING(firma_id,pracownik_id,zatrudnienie_id,umowa_id,angaz_id)"
        SQL += " INNER JOIN rs_kp_zatrudnienie kpz ON (lps.firma_id=kpz.firma_id AND lps.pracownik_id=kpz.pracownik_id AND lps.zatrudnienie_id=kpz.numer_id)"
        SQL += " INNER JOIN rs_kp_szablony_list_plac szab ON (lps.firma_id=szab.firma_id AND lp.lista_id=szab.lista_id)"
        SQL += " WHERE lps.firma_id='" + self.firma_id + "' AND lps.numer_listy_id='" + Lista + "' AND TRIM(COALESCE(kpz.ubezpieczenie_id,''))<>''"
        SQL += " AND szab.do_deklaracji_zus_z_data_wyplaty"
        SQL += "  GROUP BY lps.pracownik_id,kod_ubezp, kpa.etat, zdata, kpz.ubezpieczenie_id ORDER BY lps.pracownik_id"
        rsglobal.ZapiszDoLogow('RZA??' + SQL,1)
        stat, wynik, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL), 3)
        if stat != 'OK':
            stat = "Bd przy pobieraniu sum skadnikw do RCA/RZA: " + stat
        else:
            INSUPD = ''
            for prac in wynik:
                #if prac[3] == 0 and prac[13] == 0:  # nie ma skladek na ZUS:  # 12444
                #    continue
                if Rok+Mc>='202201':
                    typdekl = 'RCA'
                elif prac[3] == 0 and prac[13] != 0:  # RZA
                    typdekl = 'RZA'
                else:
                    typdekl = 'RCA'
                kodubez = prac[20]
                if prac[23][0:7] < DataWypl[0:7]:  # rok i miesiac
                    kodubez = '3000' + prac[24][4:6]
                warunekzus = " WHERE firma_id='" + self.firma_id + "' AND pracownik_id='" + prac[0] + "'"
                warunekzus += " AND rok='" + Rok + "' AND miesiac='" + Mc + "' AND ubezpieczenie_id='" + kodubez + "'"  # prac[20]
                warunekzus += " AND rok_lista='" + RokL + "' AND miesiac_lista='" + McL + "'"
                warunekzus += " AND etat=" + str(prac[21]) + " AND rodzaj='" + typdekl + "';"
                SEL = "SELECT count(*) FROM rs_kp_dane_osobowe_zus_skladki" + warunekzus
                stat, czyjest, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(SEL), 3)
                if stat != 'OK':
                    stat = "Bd przy sprawdzaniu " + typdekl + "dla pracownika: " + stat
                    break
                if czyjest[0][0] == 0:  # nie ma dodajemy wiersz
                    INSUPD += "INSERT INTO rs_kp_dane_osobowe_zus_skladki (firma_id,pracownik_id,rok,miesiac,rok_lista,miesiac_lista,rodzaj,ubezpieczenie_id,etat ) VALUES"
                    INSUPD += " ('" + self.firma_id + "', '" + prac[0] + "','" + Rok + "','" + Mc + "','" + RokL + "','" + McL  + "','" + typdekl + "','" + kodubez + "'," + str(prac[21]) + ");"
                INSUPD += "UPDATE rs_kp_dane_osobowe_zus_skladki SET "
                INSUPD += "podst_chor=podst_chor+ " + str(prac[1]) + ",skl_chor_ubez=skl_chor_ubez- " + str(prac[2])
                INSUPD += ",podst_emeryt_rent=podst_emeryt_rent+ " + str(prac[3]) + ",skl_emeryt_budzet=skl_emeryt_budzet- " + str(prac[4])
                INSUPD += ",skl_emeryt_plat=skl_emeryt_plat- " + str(prac[5]) + ",skl_emeryt_ubez=skl_emeryt_ubez- " + str(prac[6])
                INSUPD += ",skl_rent_budzet=skl_rent_budzet- " + str(prac[7]) + ",skl_rent_plat=skl_rent_plat- " + str(prac[8])
                INSUPD += ",skl_rent_ubez=skl_rent_ubez- " + str(prac[9]) + ",podst_wyp=podst_wyp+" + str(prac[10])
                INSUPD += ",skl_wyp_plat=skl_wyp_plat- " + str(prac[11]) + ",podstawa_zdr=podstawa_zdr+" + str(prac[13])
                # INSUPD += ",skl_zdr_plat=skl_zdr_plat+" + str(prac[14]) + ",skl_zdr_ubez=skl_zdr_ubez+" + str(prac[15])
                INSUPD += ",skl_zdr_plat=skl_zdr_plat+" + "0" + ",skl_zdr_ubez=skl_zdr_ubez- " + str(prac[14])
                INSUPD += ",czy_przekroczenie=" + str(prac[16] > 0) + ",podstawa_fp=podstawa_fp+" + str(prac[17])
                INSUPD += ",podstawa_fgsp=podstawa_fgsp+" + str(prac[18]) + ",skl_zdr_budzet=skl_zdr_budzet- " + str(prac[19])
                INSUPD += ", ppk_wplaty= ppk_wplaty+ " + str(prac[22])
                INSUPD += warunekzus

            if stat == 'OK' and INSUPD != '':
                stat, dump, dump, dump = self.PolSQL.Execute(INSUPD, 1)
                if stat != 'OK':
                    stat = "Bd przy zapisie skadek ZUS dla pracownikw: " + stat
    # zmiana stanu
    if stat == 'OK':
        SQL = "UPDATE rs_kp_listy_plac SET stan = 'Z' "+FiltrL
        stat, dump, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL), 1)
        if stat != 'OK':
            stat = "Bd przy zmianie stanu listy: " + stat

#-------------------------------------------------------------------------------
#---   OTWIERANIE LISTY (8)
#-------------------------------------------------------------------------------

if Funkcja == 8 and stat == "OK":
    if not UprOtw:
        stat='BR UPR OTW'

    if stat =='OK':
        if Stan == 'Z':
            NStan = 'W'
        else:
            stat = 'Nie mona otwiera listy w stanie innym ni Z.'

    if stat == 'OK':
        SQL = "SELECT count(*) FROM rs_platnosci "
        SQL += " WHERE firma_id='"+self.firma_id+"' AND numer_listy_id='"+Lista+"' AND stan NOT IN ('W','U');"
        stat, wynik, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),3)
        if stat != 'OK':
            stat = "Bd przy sprawdzaniu patnoci: "+stat
        elif wynik[0][0] > 0:
            stat = "Nie mona otworzy listy - wystpuj patnoci w stanie rnym od W lub U"

    if stat == 'OK':
        SQL = "DELETE FROM rs_platnosci " + FiltrL
        stat, dump, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL), 1)
        if stat != 'OK':
            stat = "Bd przy usuwaniu patnoci: " + stat

    #if stat == 'OK':
    #    SQL = "UPDATE rs_kp_dane_osobowe_absencje SET numer_listy_id=NULL, data_wyplaty=NULL " + FiltrL
    #    stat, dump, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL), 1)
    #    if stat != 'OK':
    #        stat = "Bd przy usuwaniu numeru listy z absencji: " + stat

    if stat == 'OK':
        # zobowizania, skadki, komornik
        SQL = "SELECT lpo.pracownik_id, lpo.skladnik_id, wartosc, z.wartosc_poczatkowa, z.data_od"
        SQL += " FROM rs_kp_listy_plac_osoby_skladniki lpo "
        SQL += " INNER JOIN rs_kp_dane_osobowe_zobowiazania_ratalne z USING (firma_id, pracownik_id, skladnik_id) "
        SQL += " WHERE z.data_od<='" + str(DataDoL) + "' AND wartosc != 0" + FiltrLM
        SQL += " ORDER BY lpo.pracownik_id, lpo.skladnik_id, z.data_od DESC"
        stat, wynik, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),3)
        if stat != 'OK':
            stat = "Bd przy pobieraniu skadnikw do zobowiza: "+stat
        elif len(wynik) > 0:
            # wycofanie skladnikow ze zobowiazan ratalnych - tylko jedno najnowsze zobowiazanie w ramach skladnika
            pracownik = 'abc'
            skladnik = 'abc'
            for zob in wynik:
                if pracownik != zob[0] or skladnik != zob[1]:
                    SQL = "UPDATE rs_kp_dane_osobowe_zobowiazania_ratalne SET wartosc_aktualna = wartosc_aktualna"
                    if zob[3] == 0:
                        SQL += " + "
                    else:
                        SQL += " - "
                    SQL += str(zob[2]) + " WHERE firma_id='"+self.firma_id+"' AND pracownik_id='" + zob[0]
                    SQL += "' AND skladnik_id='" + zob[1] + "' AND data_od='"+str(zob[4])+"'"
                    stat, dump, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL), 1)
                    if stat != 'OK':
                        stat = "Bd przy aktualizacji zobowiza: " + stat
                pracownik = zob[0]
                skladnik = zob[1]

    if stat == 'OK':
        # skladki PPK
        Rok = DataDoL[0:4]
        Mc = DataDoL[5:7]
        # sprawdzenie czy nie poszy do deklaracji
        SEL = "SELECT count(*) FROM rs_kp_dane_osobowe_ppk_skladki WHERE firma_id='"+self.firma_id+"'"
        SEL += " AND numer_listy_id='" + Lista + "' AND data_wyslania IS NOT NULL;"
        stat, czydeklaracja, dump, dump = self.PolSQL.Execute(SEL, 3)
        if stat != 'OK':
            stat = "Bd przy sprawdzaniu skadek PPK dla pracownikw: " + stat
        elif czydeklaracja[0][0] > 0:
            stat = "Z tej listy zostaa utworzona deklaracja PPK "
        else:
            UPD = "DELETE FROM rs_kp_dane_osobowe_ppk_skladki WHERE firma_id='"+self.firma_id+"' AND numer_listy_id='" + Lista + "';"
            stat, dump, dump, dump = self.PolSQL.Execute(UPD, 1)
            if stat != 'OK':
                 stat = "Bd przy usuwaniu skadek PPK dla pracownikw: " + stat


    if stat == 'OK':
        # RCA / RZA
        Rok = DataWypl[0:4]
        Mc = DataWypl[5:7]
        RokL = DataOdL[0:4]
        McL = DataOdL[5:7]
        # tutaj powinno byc pobieranie z rs_edeklaracje_skladniki listy skladnikow, ktore powinny byc zaliczana
        # odpowiednich kolumn w rs_kp_dane_osobowe_zus_skladki, ale jest zaslepka dla Hali Wola
        RCASK = []
        RCASK.append("'26A'")  # 0 - podst_chor
        RCASK.append("'261'")  # 1 - skl_chor_ubez
        RCASK.append("'250'")   # 2 - podst_emeryt_rent
        RCASK.append("'255'")  # 3 - skl_emeryt_budzet
        RCASK.append("'253'")  # 4 - skl_emeryt_plat
        RCASK.append("'252'")  # 5 - skl_emeryt_ubez
        RCASK.append("'258'")  # 6 - skl_rent_budzet
        RCASK.append("'257'")  # 7 - skl_rent_plat
        RCASK.append("'256'")  # 8 - skl_rent_ubez
        RCASK.append("'260'")  # 9 - podst_wyp
        RCASK.append("'263'")  # 10 - skl_wyp_ubez
        RCASK.append("'263'")  # 11 - skl_wyp_ubez
        RCASK.append("'265'")  # 12 - podstawa_zdr
        RCASK.append("'55Y','55Z'")  # 13 -  skl_zdr_plat
        RCASK.append("'555'")  # 14 - skl_zdr_ubez
        RCASK.append("'251'")  # 0 - czy przekroczenie
        RCASK.append("'259'")  # 1 - podstawa fp
        RCASK.append("'25G'")  # 2 - podstawa fgsp
        RCASK.append("'556'")  # 2 - skl_zdr_budzet
        RCASK.append("'PPK01'")  # ??
        SQL = "SELECT lps.pracownik_id,"
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[0] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "  # 1
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[1] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[2] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[3] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[4] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[5] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[6] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[7] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[8] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[9] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "  # 10
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[10] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[11] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[12] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[13] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[14] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "  # 15
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[15] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[16] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[17] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "  # 18
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[18] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "  # 19
        SQL += " CASE WHEN kpz.ubezpieczenie_rsa IS NOT NULL AND (lp.data_od,lp.data_do) OVERLAPS "
        SQL += " (kpz.data_obowiazywania_od,kpz.data_obowiazywania_do) THEN kpz.ubezpieczenie_rsa ELSE kpz.ubezpieczenie_id END as kod_ubezp,"  # 20
        SQL += " kpa.etat,"  # 21
        SQL += " SUM( CASE WHEN TRIM(skladnik_id) IN (" + RCASK[19] + ") THEN COALESCE(wartosc,0) ELSE 0 END), "  # 22
        SQL += " COALESCE(kpz.zwolnienie_data, '9999-12-31') as zdata, kpz.ubezpieczenie_id "  #23, 24
        SQL += " FROM rs_kp_listy_plac_osoby_skladniki lps"
        SQL += " INNER JOIN rs_kp_listy_plac lp USING(firma_id,numer_listy_id)"
        SQL += " INNER JOIN rs_kp_angaze kpa USING(firma_id,pracownik_id,zatrudnienie_id,umowa_id,angaz_id)"
        SQL += " INNER JOIN rs_kp_zatrudnienie kpz ON (lps.firma_id=kpz.firma_id AND lps.pracownik_id=kpz.pracownik_id AND lps.zatrudnienie_id=kpz.numer_id)"
        SQL += " INNER JOIN rs_kp_szablony_list_plac szab ON (lps.firma_id=szab.firma_id AND lp.lista_id=szab.lista_id)"
        SQL += " WHERE lps.firma_id='" + self.firma_id + "' AND lps.numer_listy_id='" + Lista + "' AND TRIM(COALESCE(kpz.ubezpieczenie_id,''))<>''"
        SQL += " AND szab.do_deklaracji_zus_z_data_wyplaty"
        SQL += "  GROUP BY lps.pracownik_id,kod_ubezp, kpa.etat, kpz.ubezpieczenie_id, zdata ORDER BY lps.pracownik_id"
        stat, wynik, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL), 3)
        if stat != 'OK':
            stat = "Bd przy pobieraniu sum skadnikw do ZUS: " + stat
        else:
            UPD = ''
            for prac in wynik:
                #if prac[3] == 0 and prac[13] == 0:  # nie ma skladek na ZUS:  # 12444
                #    continue
                if Rok+Mc>='202201':
                    typdekl = 'RCA'
                elif prac[3] == 0 and prac[13] != 0:  # RZA
                    typdekl = 'RZA'
                else:
                    typdekl = 'RCA'
                kodubez = prac[20]
                if prac[23][0:7] < DataWypl[0:7]:  # rok i miesiac
                    kodubez = '3000' + prac[24][4:6]
                warunekzus = " WHERE firma_id='" + self.firma_id + "' AND pracownik_id='" + prac[0] + "'"
                warunekzus += " AND rok='" + Rok + "' AND miesiac='" + Mc + "' AND ubezpieczenie_id='" + kodubez + "'"
                warunekzus += " AND ((rok_lista='" + RokL + "' AND miesiac_lista='" + McL + "') OR (rok_lista='' AND miesiac_lista=''))"
                warunekzus += " AND etat=" + str(prac[21]) + " AND rodzaj='" + typdekl + "';"
                UPD += "UPDATE rs_kp_dane_osobowe_zus_skladki SET "
                UPD += "podst_chor=podst_chor- " + str(prac[1]) + ",skl_chor_ubez=skl_chor_ubez+ " + str(prac[2])
                UPD += ",podst_emeryt_rent=podst_emeryt_rent- " + str(prac[3]) + ",skl_emeryt_budzet=skl_emeryt_budzet+ " + str(prac[4])
                UPD += ",skl_emeryt_plat=skl_emeryt_plat+ " + str(prac[5]) + ",skl_emeryt_ubez=skl_emeryt_ubez+ " + str(prac[6])
                UPD += ",skl_rent_budzet=skl_rent_budzet+ " + str(prac[7]) + ",skl_rent_plat=skl_rent_plat+ " + str(prac[8])
                UPD += ",skl_rent_ubez=skl_rent_ubez+ " + str(prac[9]) + ",podst_wyp=podst_wyp- " + str(prac[10])
                UPD += ",skl_wyp_plat=skl_wyp_plat+ " + str(prac[11]) + ",podstawa_zdr=podstawa_zdr- " + str(prac[13])
                # UPD += ",skl_zdr_plat=skl_zdr_plat- " + str(prac[14]) + ",skl_zdr_ubez=skl_zdr_ubez- " + str(prac[15])
                UPD += ",skl_zdr_plat=skl_zdr_plat+ " + "0" + ",skl_zdr_ubez=skl_zdr_ubez+ " + str(prac[14])
                UPD += ",czy_przekroczenie=False,podstawa_fp=podstawa_fp- " + str(prac[17])
                UPD += ",podstawa_fgsp=podstawa_fgsp- " + str(prac[18]) + ", skl_zdr_budzet=skl_zdr_budzet+ " + str(prac[19])
                UPD += ", ppk_wplaty= ppk_wplaty- " + str(prac[22])
                UPD += warunekzus
            if stat == 'OK' and UPD != '':
                stat, dump, dump, dump = self.PolSQL.Execute(UPD, 1)
                if stat != 'OK':
                    stat = "Bd przy zapisie skadek ZUS dla pracownikw: " + stat

    # waciwa zmiana stanu
    if stat =='OK':
        SQL = "UPDATE rs_kp_listy_plac SET stan = '"+NStan+"'"+FiltrL
        stat,dump,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),1)
        if stat != 'OK':
            stat = "Bd przy zmianie stanu listy: "+stat

#-------------------------------------------------------------------------------
#---   DELETE ANGAU (11)
#-------------------------------------------------------------------------------
            
if Funkcja == 11 and stat == "OK":
    if not UprDel:
        stat = 'BR UPR DEL'
        
    if stat == 'OK':
        if Stan != 'W':
            stat = 'Nie mona usun angau z listy w stanie innym ni W'
            
    # waciwe usuwanie
    if stat == 'OK':
        SQL = "DELETE FROM rs_kp_listy_plac_osoby "+FiltrL.strip(';')+" AND pracownik_id = '"+Pracownik+"'"
        SQL += " AND zatrudnienie_id="+Zatrudnienie+" AND umowa_id="+Umowa+" AND angaz_id="+Angaz
        SQL += " AND data_od='"+DataA+"'"
        stat, dump, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),1)
        if stat != 'OK':
            stat = "Bd przy usuwaniu danych angau: "+stat

    # sprawdzenie czy sa inne angaze dla pracownika na liscie, jak nie ma czyszczenie absencji
    if stat == 'OK':
        SQL = "SELECT count(*) FROM rs_kp_listy_plac_osoby "+FiltrL.strip(';')+" AND pracownik_id = '"+Pracownik+"'"
        stat, ile, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),3)
        if stat != 'OK':
            stat = "Bd przy sprawdzanie obecnoci pracownika na licie : "+stat
        elif ile[0][0] == 0:
            SQL = "UPDATE rs_kp_dane_osobowe_absencje SET numer_listy_id=NULL, data_wyplaty=NULL " + FiltrL.strip(';')+" AND pracownik_id = '"+Pracownik+"'"
            stat, dump, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL), 1)
            if stat != 'OK':
                stat = "Bd przy usuwaniu numeru listy z absencji: " + stat



#-------------------------------------------------------------------------------
#---   UPDATE (1)
#-------------------------------------------------------------------------------

if Funkcja == 1 and stat == "OK":
    if len(Update[0])>0:
        if not UprUpd:
            stat='BR UPR UPD'
            dane.append(stat)

        if stat =='OK':
            if Stan != 'W':
                stat = 'Nie mona zmienia wartoci skadnikw z listy w stanie innym ni W'

        if stat == 'OK':
            WierszeNr = Update[-1]
            
            for i in xrange(len(Update[0])):
                WHERE=" WHERE firma_id='"+self.firma_id+"' AND numer_listy_id='"+Lista+"'"+" AND pracownik_id = '"+Pracownik+"'"
                WHERE+=" AND zatrudnienie_id="+Zatrudnienie+" AND umowa_id="+Umowa+" AND angaz_id="+Angaz
                WHERE+=" AND data_od='"+DataA+"'"

                # update angau na nieprzeliczony
                SQLAng="UPDATE rs_kp_listy_plac_osoby SET przeliczony = FALSE "+WHERE+';'

                for j in xrange(len(Update[2][i])): # petla po kluczu
                    pole=Update[3][j]
                    wart=rsglobal.Dekoduj(Update[2][i][j])
                    if pole in ['skladnik_id',]:
                        WHERE+=" AND "+pole+"='"+wart+"'"

                if stat == 'OK':
                    if len(dane) > 0:
                        stat = dane[0]
                        
                # wykonanie UPDATE
                if stat=='OK':
                    sql=''
                    UPDATE="UPDATE rs_kp_listy_plac_osoby_skladniki "
                    SET='SET '
                    for j in xrange(len(Update[0][i])): 
                        pole=Update[0][i][j]
                        wart=Update[1][i][j]
                        if pole == 'wartosc':
                            SET+=pole+"='"+str(wart).strip().replace("\\","\\\\").replace("'","`")+"',"
                            SET+=" modyfikowane_przez="+str(self.operator_id)+",modyfikowane_kiedy=NOW(), "
                            #AB 26.08.21r
                            SET+=" zmodyfikowal=(SELECT login FROM rs_operatorzy WHERE operator_id="+str(self.operator_id)+") "
                        if pole == 'mod':
                            SET+="modyfikowalny='"+str(wart)+"',"
                            SET+=" zmodyfikowal=(SELECT login FROM rs_operatorzy WHERE operator_id="+str(self.operator_id)+") "

                    SET=SET.strip(",")
                    sql+=UPDATE+SET+WHERE+';'+SQLAng
                    rsglobal.ZapiszDoLogow('sql '+sql,1)
                    stat,dump,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(sql),1)
                    if stat<>'OK':
                        stat = "Bd przy modyfikowaniu danych: "+stat   
                        dane.append(stat)  
    

#-------------------------------------------------------------------------------
#---   POBRANIE DANYCH DO PRZELICZENIA (15)
#-------------------------------------------------------------------------------
            
if Funkcja == 15 and stat=="OK":
    if not UprPrzel:
        stat='BR UPR PRZEL'

    if stat =='OK':
        if Stan != 'W' and NowySkladnik == '':
            stat = 'Nie mona przelicza angay z listy w stanie innym ni W'

    # sprawdzenie czy sa karty pracy
    if stat =='OK':
        SQL  = "SELECT lp.pracownik_id  FROM rs_kp_listy_plac_osoby lp"
        SQL += " INNER JOIN rs_kp_listy_plac p USING (firma_id, numer_listy_id)"
        SQL += " INNER JOIN rs_kp_szablony_list_plac sz ON sz.firma_id=p.firma_id AND sz.lista_id=p.lista_id"
        SQL += " WHERE lp.firma_id='"+self.firma_id+"' AND numer_listy_id='"+Lista+"' AND sz.czy_karty_pracy"
        SQL += " AND NOT EXISTS (SELECT 1 FROM rs_kp_angaze_karty_pracy kp "
        SQL += " WHERE lp.firma_id=kp.firma_id AND lp.zatrudnienie_id=kp.zatrudnienie_id AND lp.umowa_id=kp.umowa_id AND lp.angaz_id=kp.angaz_id"
        SQL += " AND kp.rok=SUBSTRING(CAST(lp.data_od AS TEXT),1,4) AND kp.mc=SUBSTRING(CAST(lp.data_od AS TEXT),6,2))"
        stat,DKarty,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),3)
        if stat != 'OK':
            stat = "Bd przy sprawdzaniu kart pracy dla listy: "+stat
        elif len(DKarty)>0:
            stat = "Brak kart pracy dla pracownikw: "
            for p in DKarty:
                stat += str(p[0])+"  "


    # pobranie listy angay do przeliczenia dla pracownika
    if stat =='OK':
        SQL = "SELECT zatrudnienie_id,umowa_id,angaz_id,data_od,COALESCE(data_do::TEXT,'') FROM rs_kp_listy_plac_osoby "
        SQL+= " WHERE firma_id='"+self.firma_id+"' AND numer_listy_id='"+Lista+"'"+" AND pracownik_id = '"+Pracownik+"'"
        SQL+= " ORDER BY zatrudnienie_id,umowa_id,angaz_id "
        stat,DAngaze,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),3)
        if stat != 'OK':
            stat = "Bd przy pobieraniu angay dla listy: "+stat

    if stat =='OK' and NowySkladnik == '':
        # usuniecie skladnikow niemodyfikowalnych recznie dla pracownika w celu uzyskania powtarzalnosci obliczen
        SQL = "DELETE FROM rs_kp_listy_plac_osoby_skladniki lp WHERE firma_id='"+self.firma_id+"'"
        SQL+=" AND numer_listy_id='"+Lista+"' AND pracownik_id = '"+Pracownik+"' AND modyfikowalny NOT IN ('1','2')"    #JR 2021-09-01 (12628)
        #SQL+=" AND EXISTS ( SELECT 1 FROM rs_kp_skladniki_placowe sk WHERE sk.firma_id=lp.firma_id AND sk.skladnik_id=lp.skladnik_id"
        #SQL+=" AND NOT sk.modyfikowalny_recznie );"
        stat,dump,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),1)
        if stat != 'OK':
            stat = u"Bd przy usuwaniu skadnikw z angau listy: "+stat   

    # pobranie czy dywidenda
    CzyDywidenda = False
    if stat =='OK':
        SQL = "SELECT sz.czy_dywidenda FROM rs_kp_listy_plac "
        SQL +=" INNER JOIN rs_kp_szablony_list_plac sz USING (firma_id, lista_id)"
        SQL+= " WHERE firma_id='"+self.firma_id+"' AND numer_listy_id='"+Lista+"'"
        stat,D,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),3)
        if stat != 'OK':
            stat = "Bd przy sprawdzaniu dywidendy dla listy: "+stat
        else:
            CzyDywidenda = D[0][0]

    if stat =='OK':
        # to jest kluczowa zmienna, ktra na koniec zwraca sownik gdzie luczem jest anga a wynikiem - wszystkie dane potrzebne do wyliczenia skadnikw listy pac
        SDaneAngazy = {}

        for pozangazu in DAngaze:
            if stat =='OK':
                Zatrudnienie = str(pozangazu[0])
                Umowa = str(pozangazu[1])
                Angaz = str(pozangazu[2])
                DataOdA = pozangazu[3]
                DataDoA = pozangazu[4]

                DaneTmp = [] # zmienna gdzie s trzymane dane jednego angau

                # pobieramy list skadnikw dla listy. Jeli wicej ni jedna to koczymy przeliczanie.
                SQL= "SELECT slpw.skladniki,slpw.data_od,COALESCE(slpw.data_do::TEXT,'brak') FROM rs_kp_szablony_list_plac_wersje slpw "
                SQL+="INNER JOIN rs_kp_listy_plac lp ON (lp.firma_id=slpw.firma_id AND lp.lista_id = slpw.lista_id)"
                SQL+=" WHERE lp.firma_id='"+self.firma_id+"' AND lp.numer_listy_id='"+Lista+"'"
                SQL+=" AND slpw.data_od <= lp.data_od AND (slpw.data_do >= lp.data_do OR slpw.data_do IS NULL)"
                stat,DSzabl,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),3)
                if stat != 'OK':
                    stat = "Bd przy pobieraniu skadnikw z szablonu listy: "+stat

                if stat == 'OK':
                    rsglobal.ZapiszDoLogow('select skladnikow obliczanych:' + str(DSzabl))
                    if len(DSzabl) == 0:
                        stat = u"Brak wersji szablonu o parametrach z listy pac "+Lista+' dla dat '+DataOdL+' - '+DataDoL
                    else:
                        # pobieramy list skadnikw wymaganych, ktre maj by na liscie - skadniki spoza tej listy pomijamy
                        LSkladWym = []
                        DSzabl[0][0] = DSzabl[0][0].replace('\n','').replace('\r','')
                        for skl in DSzabl[0][0].split(','):
                            if skl.strip() != '':
                                LSkladWym.append(skl.strip())
                        rsglobal.ZapiszDoLogow('lista skladnikow obliczanych:' + str(LSkladWym))


            # pobranie dostpnych skadnikw pacowych i ich wzorw oraz kolejnosci wykonania
            if stat == 'OK':
                warunek1 = "CAST (CASE WHEN sp.wg_daty_wyplaty THEN '" + DataWypl + "' ELSE '" + DataDoA + "' END AS DATE)"
                warunek2 = "CAST (CASE WHEN sp.wg_daty_wyplaty THEN '" + DataWypl + "' ELSE '" + DataOdA + "' END AS DATE)"
                SQL = "SELECT TRIM(skladnik_id),data_od,algorytm,zapis_do_listy_plac "
                SQL += " FROM rs_kp_skladniki_placowe_wersje "
                SQL += " INNER JOIN rs_kp_skladniki_placowe sp USING (firma_id,skladnik_id)"
                SQL += " WHERE firma_id='"+self.firma_id+"' AND data_od <= "+warunek1  #JR 2021-11-17 (12692)
                SQL += " AND (data_do IS NULL OR data_do >= "+warunek2+")"
                #SQL += " WHERE firma_id='"+self.firma_id+"' AND data_od <= '"+DataDoA+"'"
                #SQL += " AND (data_do IS NULL OR data_do >= '"+DataOdA+"')"
                rsglobal.ZapiszDoLogow('skladniki wg dat: ' + str(SQL), 1)
                stat, DSklad, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),3)
                if stat != 'OK':
                    stat = "Bd przy pobieraniu skadnikw z wersji skadnikw: "+stat

                if stat == 'OK':
                    if len(DSklad) == 0:
                        stat = u"Brak wersji skadnikw dla daty "+DataOdL

                if stat == 'OK':
                    # jeli jedna wersja = False i skadnik wykorzystany to zgasza bd
                    SSkladWersje ={}
                    for poz in DSklad:
                        # jeli skadnika nie ma na liscie wymaganych skadnikw to go zerujemy
                        if not poz[0].strip() in LSkladWym:
                            SSkladWersje[poz[0]] = [DataOdA,'wart=0',False,True]
                        if not poz[0] in SSkladWersje:
                            SSkladWersje[poz[0]] = [poz[1],poz[2],poz[3],True] # data_od, algorytm, czy zapis do listy,jedna wersja
                        elif poz[1] > SSkladWersje[poz[0]][0]: # odwieamy bo modsza wersja
                            SSkladWersje[poz[0]] = [poz[1],poz[2],poz[3],False] # data_od, algorytm, czy zapis do listy,jedna wersja

                    # dodatkowo czycimy treci algorytmw z komentarzy i zamieniamy round na RSGlobal.RSRound
                    for sklad in SSkladWersje:
                        Alg = SSkladWersje[sklad][1].splitlines()
                        rsglobal.ZapiszDoLogow('1 Alg '+str(Alg),1)
                        AlgTmp = ''
                        for w in Alg:
                            if '#' in w:
                                AlgTmp += w[:w.find('#')]+'\n'
                            else:
                                AlgTmp += w+'\n'

                        rsglobal.ZapiszDoLogow('2 AlgTmp '+str(AlgTmp),1)
                        SSkladWersje[sklad][1] = AlgTmp.replace('round', 'RSGlobal.RSRound')

            # pobranie wartoci modyfikowalnych z listy
            if stat == 'OK':
                SQL = "SELECT TRIM(skladnik_id),wartosc FROM rs_kp_listy_plac_osoby_skladniki lpos "
                #SQL+= " INNER JOIN rs_kp_skladniki_placowe sp USING (firma_id,skladnik_id)"    #JR 2021-09-01 (12628)
                SQL+= " WHERE lpos.firma_id='"+self.firma_id+"' AND lpos.numer_listy_id='"+Lista+"'"+" AND lpos.pracownik_id = '"+Pracownik+"'"
                SQL+= " AND lpos.zatrudnienie_id="+Zatrudnienie+" AND lpos.umowa_id="+Umowa+" AND lpos.angaz_id="+Angaz
                SQL+= " AND data_od='"+DataOdA+"' AND modyfikowalny IN ('1','2')"   #JR 2021-09-01 (12628)  AND sp.modyfikowalny_recznie"
                stat,DMod,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),3)
                if stat != 'OK':
                    stat = "Bd przy pobieraniu skadnikw modyfikowalnych z listy pac "+Lista+": "+stat

                if stat == 'OK':
                    SMod = {}
                    for poz in DMod:
                        SMod[poz[0]] = poz[1]

            # pobranie wartoci modyfikowalnych oglnych
            if stat == 'OK':
                SQL = "SELECT TRIM(skladnik_id) FROM rs_kp_skladniki_placowe sp "
                SQL+= " WHERE firma_id='"+self.firma_id+"' AND sp.modyfikowalny_recznie"
                stat,DModR,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),3)
                if stat != 'OK':
                    stat = "Bd przy pobieraniu skadnikw modyfikowalnych z definicji skadnikw: "+stat

                if stat == 'OK':
                    for poz in DModR:
                        if not poz[0] in SMod:
                            SMod[poz[0]] = 0

            # pobranie skadnikw dodanych do angay (s traktowane jak stae)
            if stat == 'OK':
                SQL= "SELECT TRIM(sa.skladnik_id),sa.data_od,sa.kwota,sk.staly,sk.jednorazowy FROM rs_kp_angaze_skladniki sa "
                SQL+= " INNER JOIN rs_kp_skladniki_placowe sk USING (firma_id,skladnik_id) "
                SQL+= " WHERE firma_id='"+self.firma_id+"' AND pracownik_id = '"+Pracownik+"'"
                SQL+= " AND zatrudnienie_id="+Zatrudnienie+" AND umowa_id="+Umowa+" AND angaz_id="+Angaz
                SQL+= " AND data_od <= '"+DataDoA+"'"
                SQL+= " AND (data_do IS NULL OR data_do >= '"+DataOdA+"')"
                stat,DSkladAn,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),3)
                if stat != 'OK':
                    stat = "Bd przy pobieraniu skadnikw z angau pracownika: "+stat

                if stat == 'OK':
                    SSkladAn = {}
                    for poz in DSkladAn:
                        if stat == 'OK':
                            if not poz[0].strip() in LSkladWym: # skadnik nie jest na licie skadnikw wymienionych na szablonie
                                SSkladAn[poz[0]] = [[DataOdA,0, False,poz[3],poz[4]]] # data_od, warto, czy na list
                            elif not poz[0] in SSkladAn:
                                SSkladAn[poz[0]] = [[poz[1], poz[2], False,poz[3],poz[4]]] # data_od, warto, czy na list
                            else: # dokladamy kolejn pozycj dla skadnika
                                SSkladAn[poz[0]].append([poz[1], poz[2], False,poz[3],poz[4]])

                # uzupenienie informacji czy na list dla staych z angay
                if stat == 'OK':
                    for kl in SSkladAn:
                        for poz in SSkladAn[kl]:
                            if kl in SSkladWersje:
                                poz[2] = SSkladWersje[kl][2]

            # pobranie staych pacowych
            if stat == 'OK':
                SQL ="SELECT TRIM(s.stala_id), s.data_od,CASE WHEN ts.format = 'I' THEN s.wart_int::TEXT WHEN ts.format = 'N' THEN s.wart_num::TEXT "
                SQL+=" WHEN ts.format = 'T' THEN s.wart_text ELSE s.wart_bool::TEXT END "
                SQL+=" FROM rs_kp_stale s INNER JOIN rs_kp_typy_stalych ts USING (firma_id,stala_id)"
                SQL+=" WHERE s.firma_id='"+self.firma_id+"' AND s.data_od <= '"+DataOdA+"' AND NOT wg_daty_wyplaty;"   #JR 2021-11-17 (12692)
                stat,DStale,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),3)
                if stat != 'OK':
                    stat = "Bd przy pobieraniu staych pacowych: "+stat

                if stat == 'OK':
                    SQL = "SELECT TRIM(s.stala_id), s.data_od,CASE WHEN ts.format = 'I' THEN s.wart_int::TEXT WHEN ts.format = 'N' THEN s.wart_num::TEXT "
                    SQL += " WHEN ts.format = 'T' THEN s.wart_text ELSE s.wart_bool::TEXT END "
                    SQL += " FROM rs_kp_stale s INNER JOIN rs_kp_typy_stalych ts USING (firma_id,stala_id)"
                    SQL += " WHERE s.firma_id='" + self.firma_id + "' AND s.data_od <= '" + DataWypl + "' AND wg_daty_wyplaty;"   #JR 2021-11-17 (12692)
                    stat, DStaleW, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL), 3)
                    if stat != 'OK':
                        stat = "Bd przy pobieraniu staych pacowych wg daty wypaty: " + stat

                rsglobal.ZapiszDoLogow('DStale ' + str(DStale), 1)
                rsglobal.ZapiszDoLogow('DStaleW ' + str(DStaleW), 1)
                if stat == 'OK':
                    SStale = {}
                    for poz in DStale:
                        if not poz[0] in SStale:
                            SStale[poz[0]] = [poz[1], poz[2],poz[1],poz[2]] # data_od, warto (jako string), data od, wartosc
                        elif poz[1] > SStale[poz[0]][0]: # odwieamy bo modsza wersja
                            SStale[poz[0]] = [poz[1], poz[2],poz[1],poz[2]] # data_od, warto (jako string)s, data od, wartosc
                    for poz in DStaleW:  # stale dla daty wyplaty
                        if not poz[0] in SStale:
                            SStale[poz[0]] = [poz[1], poz[2],poz[1],poz[2]] # data_od, warto (jako string), data od, wartosc
                        elif poz[1] > SStale[poz[0]][0]: # odwieamy bo modsza wersja
                            SStale[poz[0]] = [poz[1], poz[2],poz[1],poz[2]] # data_od, warto (jako string)s, data od, wartosc
                        #elif poz[1] > SStale[poz[0]][2]: # odwieamy bo modsza wersja
                        #    SStale[poz[0]] = [SStale[poz[0]][0], SStale[poz[0]][1], poz[1], poz[2]] # data_od, warto (jako string)s, data od, wartosc

            # pobranie zmiennych pracownika/zatrudnienia/umowy/angau
            if stat == 'OK':
                if CzyDywidenda:  # dla dywidendy pobieramy tylko ko.rorproc i ko.rorkwota
                    SZmienne = {}
                    SQL = "SELECT sum(procent), sum(kwota) FROM rs_kp_dane_osobowe_ror WHERE firma_id='" + self.firma_id + "' AND pracownik_id='" + str(Pracownik) + "'"
                    stat, ror, dump, dump = self.PolSQL.Execute(SQL, 3)
                    if stat != 'OK':
                        SZmienne['ko.rorproc'] = '0'
                        SZmienne['ko.rorproc'] = '0'
                    else:
                        SZmienne['ko.rorproc'] = str(ror[0][0])
                        SZmienne['ko.rorkwota'] = str(ror[0][1])
                else:  # pobranie zmiennych dla pracownika
                    from funkcje import fs_kp_zmienne_placowe_pom
                    import rs_anakonda_daty
                    stat,SZmienne = fs_kp_zmienne_placowe_pom.PodmienDanymi(self,'',Pracownik,Zatrudnienie,Umowa,Angaz,Lista,True)
                    rsglobal.ZapiszDoLogow('Stat + Zmienne placowe ' + stat + '|' + str(SZmienne), 1)
                    if stat != 'OK':
                        stat = 'Bd pobierania zmiennych pracownika :'+stat

            # pobranie skadnikw i ich kolejnoci oraz informacji czy stale i jednorazowe
            if stat == 'OK':
                SQL ="SELECT TRIM(skladnik_id),COALESCE(CASE WHEN kolejnosc_obliczen = 0 THEN 1000 ELSE kolejnosc_obliczen END,1000),"
                SQL+=" staly, jednorazowy"
                SQL+=" FROM rs_kp_skladniki_placowe "
                SQL+=" WHERE firma_id='"+self.firma_id+"';"
                stat,DSkladnikiKolej,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),3)
                if stat != 'OK':
                    stat = "Bd przy pobieraniu kolejnoci skadnikw: "+stat

                if stat == 'OK':
                    SSkladKolej = {}
                    SSkladStalyJedn = {}
                    for poz in DSkladnikiKolej:
                        SSkladKolej[poz[0]] = poz[1] # kolejno
                        SSkladStalyJedn[poz[0]] = [poz[2],poz[3]] # staly, jednorazowy


            # pobranie daty poczatku angazu
            if stat == 'OK':
                SQL = "SELECT od_kiedy FROM rs_kp_angaze "
                SQL += " WHERE firma_id='" + self.firma_id + "' AND pracownik_id = '" + Pracownik + "'"
                SQL += " AND zatrudnienie_id=" + str(Zatrudnienie) + " AND umowa_id=" + str(Umowa) + " AND angaz_id=" + str(Angaz)
                stat, test, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL), 3)
                if stat != 'OK':
                    stat = "Bd przy pobieraniu daty od angau dla listy: " + stat
                else:
                    PomOd = test[0][0]

            # pobranie liczby dni absencji dla angazu
            if stat == 'OK':
                if PomOd <= DataOdL:
                    PomOd = "'" + rs_anakonda_daty.DodajKilkaDni(DataOdL, -31) + "'"
                else:
                    PomOd = "'" + max(rs_anakonda_daty.DodajKilkaDni(DataOdL, -31), PomOd) + "'"
                rsglobal.ZapiszDoLogow('!!!PomOd ' + str(DataOdL)+"|"+str(DataOdA)+"|"+str(PomOd), 1)
                if DataDoL < DataDoA or DataDoA == '    -  -  ':
                    PomDo = "'"+DataDoL+"'"
                else:
                    PomDo = "'"+DataDoA+"'"
                SQL ="SELECT typ_absencji_id,COALESCE(kod_zus,'') as kod,"
                SQL+=" SUM (CASE "
                SQL+=" WHEN "+PomDo+"<od_kiedy OR "+PomOd+">do_kiedy THEN 0"
                SQL+=" WHEN "+PomOd+"<= od_kiedy AND "+PomDo+">= do_kiedy THEN do_kiedy- od_kiedy + 1"
                SQL+=" WHEN "+PomOd+"> od_kiedy AND "+PomDo+">= do_kiedy THEN do_kiedy- CAST (TEXT "+PomOd+" AS DATE) + 1"
                SQL+=" WHEN "+PomOd+"<= od_kiedy AND "+PomDo+"< do_kiedy THEN CAST (TEXT "+PomDo+" AS DATE) - od_kiedy + 1"
                SQL+=" END ) AS  dnik,"
                SQL+=" SUM (CASE "
                SQL+=" WHEN "+PomDo+"<od_kiedy OR "+PomOd+">do_kiedy THEN 0"
                SQL+=" WHEN "+PomOd+"<= od_kiedy AND "+PomDo+">= do_kiedy THEN (do_kiedy- od_kiedy + 1) * 8"
                SQL+=" WHEN "+PomOd+"> od_kiedy AND "+PomDo+">= do_kiedy THEN (do_kiedy- CAST (TEXT "+PomOd+" AS DATE) + 1) * 8"
                SQL+=" WHEN "+PomOd+"<= od_kiedy AND "+PomDo+"< do_kiedy THEN (CAST (TEXT "+PomDo+" AS DATE) - od_kiedy + 1) * 8"
                SQL+=" END ) AS godzk, "
                SQL+=" SUM (CASE "
                SQL+=" WHEN "+PomDo+"<od_kiedy OR "+PomOd+">do_kiedy THEN 0"
                SQL+=" ELSE dni_robocze"
                SQL+=" END ) AS dnir, "
                SQL+=" SUM (CASE "
                SQL+=" WHEN "+PomDo+"<od_kiedy OR "+PomOd+">do_kiedy THEN 0"
                SQL+=" ELSE godziny_robocze"
                SQL+=" END ) AS godzr, "
                SQL+=" SUM (CASE "
                SQL+=" WHEN "+PomDo+"<od_kiedy OR "+PomOd+">do_kiedy THEN 0"
                SQL+=" WHEN "+PomOd+"<od_kiedy AND "+PomDo+"<=do_kiedy THEN ROUND((CAST (TEXT "+PomDo+" AS DATE) - od_kiedy + 1) * zasilek_dzien,2)"
                SQL+=" WHEN "+PomOd+">=od_kiedy AND "+PomDo+">do_kiedy THEN ROUND((do_kiedy- CAST (TEXT "+PomOd+" AS DATE) + 1) * zasilek_dzien,2)"
                SQL+=" ELSE zasilek_razem"
                SQL+=" END ) AS zasilek "
                SQL+=" FROM rs_kp_dane_osobowe_absencje "
                SQL+=" WHERE firma_id='"+self.firma_id+"' AND pracownik_id = '"+Pracownik+"'"
                SQL+=" AND (TRIM(COALESCE(numer_listy_id,''))='' OR numer_listy_id='"+Lista+"')"
                SQL+=" GROUP BY typ_absencji_id,kod; "
                rsglobal.ZapiszDoLogow('!!!DAbsencje ' + str(SQL), 1)
                stat,DAbsencje,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),3)
                if stat != 'OK':
                    stat = "Bd przy pobieraniu liczby dni absencji: "+stat

                if stat == 'OK':
                    SAbsencje = {}
                    for poz in DAbsencje:
                            SAbsencje[poz[0].strip(), poz[1].strip()] = [poz[2], poz[3], poz[4], poz[5], poz[6], 0] # liczba dni i zasilek razem, 0 - czy policzony
                    rsglobal.ZapiszDoLogow('!!!Absencje '+str(SAbsencje),1)

            # pobranie liczby dni absencji dla angazu z data wczesniejsza niz poczatek listy
            if stat == 'OK':
                PomDo = "'" + rs_anakonda_daty.DodajKilkaDni(DataOdL, -1) + "'"  # PomOd ustawiony wczesniej
                SQL ="SELECT typ_absencji_id,COALESCE(kod_zus,'') as kod,"
                SQL+=" SUM (CASE "
                SQL+=" WHEN "+PomDo+"<od_kiedy OR "+PomOd+">do_kiedy THEN 0"
                SQL+=" WHEN "+PomOd+"<= od_kiedy AND "+PomDo+">= do_kiedy THEN do_kiedy- od_kiedy + 1"
                SQL+=" WHEN "+PomOd+"> od_kiedy AND "+PomDo+">= do_kiedy THEN do_kiedy- CAST (TEXT "+PomOd+" AS DATE) + 1"
                SQL+=" WHEN "+PomOd+"<= od_kiedy AND "+PomDo+"< do_kiedy THEN CAST (TEXT "+PomDo+" AS DATE) - od_kiedy + 1"
                SQL+=" END ) AS  dnik,"
                SQL+=" SUM (CASE "
                SQL+=" WHEN "+PomDo+"<od_kiedy OR "+PomOd+">do_kiedy THEN 0"
                SQL+=" WHEN "+PomOd+"<= od_kiedy AND "+PomDo+">= do_kiedy THEN (do_kiedy- od_kiedy + 1) * 8"
                SQL+=" WHEN "+PomOd+"> od_kiedy AND "+PomDo+">= do_kiedy THEN (do_kiedy- CAST (TEXT "+PomOd+" AS DATE) + 1) * 8"
                SQL+=" WHEN "+PomOd+"<= od_kiedy AND "+PomDo+"< do_kiedy THEN (CAST (TEXT "+PomDo+" AS DATE) - od_kiedy + 1) * 8"
                SQL+=" END ) AS godzk, "
                SQL+=" SUM (CASE "
                SQL+=" WHEN "+PomDo+"<od_kiedy OR "+PomOd+">do_kiedy THEN 0"
                SQL+=" ELSE dni_robocze"
                SQL+=" END ) AS dnir, "
                SQL+=" SUM (CASE "
                SQL+=" WHEN "+PomDo+"<od_kiedy OR "+PomOd+">do_kiedy THEN 0"
                SQL+=" ELSE godziny_robocze"
                SQL+=" END ) AS godzr, "
                SQL+=" SUM (CASE "
                SQL+=" WHEN "+PomDo+"<od_kiedy OR "+PomOd+">do_kiedy THEN 0"
                SQL+=" WHEN "+PomOd+"<od_kiedy AND "+PomDo+"<=do_kiedy THEN ROUND((CAST (TEXT "+PomDo+" AS DATE) - od_kiedy + 1) * zasilek_dzien,2)"
                SQL+=" WHEN "+PomOd+">=od_kiedy AND "+PomDo+">do_kiedy THEN ROUND((do_kiedy- CAST (TEXT "+PomOd+" AS DATE) + 1) * zasilek_dzien,2)"
                SQL+=" ELSE zasilek_razem"
                SQL+=" END ) AS zasilek "
                SQL+=" FROM rs_kp_dane_osobowe_absencje "
                SQL+=" WHERE firma_id='"+self.firma_id+"' AND pracownik_id = '"+Pracownik+"'"
                SQL+=" AND (TRIM(COALESCE(numer_listy_id,''))='' OR numer_listy_id='"+Lista+"')"
                SQL+=" GROUP BY typ_absencji_id,kod; "
                rsglobal.ZapiszDoLogow('!!!DAbsencjePM ' + str(SQL), 1)
                stat,DAbsencje,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),3)
                if stat != 'OK':
                    stat = "Bd przy pobieraniu liczby dni absencji: "+stat

                if stat == 'OK':
                    SAbsencjePM = {}
                    for poz in DAbsencje:
                            SAbsencjePM[poz[0].strip(), poz[1].strip()] = [poz[2], poz[3], poz[4], poz[5], poz[6], 0] # liczba dni i zasilek razem, 0 - czy policzony
                    rsglobal.ZapiszDoLogow('!!!AbsencjePM '+str(SAbsencjePM), 1)

            # pobranie liczby miesiecy od ostatniej absencji
            if stat == 'OK':
                SQL ="SELECT typ_absencji_id,max(do_kiedy)"
                SQL+=" FROM rs_kp_dane_osobowe_absencje "
                SQL+=" WHERE firma_id='"+self.firma_id+"' AND pracownik_id = '"+Pracownik+"'"
                SQL+=" AND do_kiedy < '" + str(DataOdL)+"' GROUP BY typ_absencji_id"
                stat,DAbsencje,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),3)
                if stat != 'OK':
                    stat = "Bd przy pobieraniu liczby miesicy absencji: "+stat

                if stat == 'OK':
                    SAbsencjeMc = {}
                    for poz in DAbsencje:
                        pom = rs_anakonda_daty.OkresPomiedzy(poz[1],DataOdL)
                        SAbsencjeMc[poz[0].strip()] = int(pom[0:4]) * 12 + int(pom[5:7])
                    rsglobal.ZapiszDoLogow('!!!AbsencjeMc '+str(SAbsencjeMc),1)

            # pobranie zobowiazan i skladek
            if stat == 'OK':
                if DataOdL < DataOdA:
                    PomOd = "'"+DataOdA+"'"
                else:
                    PomOd = "'"+DataOdL+"'"
                if DataDoL < DataDoA or DataDoA=='    -  -  ':
                    PomDo = "'"+DataDoL+"'"
                else:
                    PomDo = "'"+DataDoA+"'"
                SQL ="SELECT skladnik_id,"
                SQL += " CASE WHEN wartosc_poczatkowa = 0 THEN rata ELSE LEAST(rata,wartosc_aktualna) END as kwota, "
                SQL += " CASE WHEN wartosc_poczatkowa = 0 AND wartosc_aktualna = 0 AND rata > 0 THEN 1 ELSE 0 END as wpisowe "
                SQL += " FROM rs_kp_dane_osobowe_zobowiazania_ratalne "
                SQL += " WHERE firma_id='"+self.firma_id+"' AND pracownik_id = '"+Pracownik+"'"
                SQL += " AND data_od<="+PomDo+" AND (zawieszone_do<"+PomDo+" OR zawieszone_do IS NULL)"
                SQL += " AND NOT (wartosc_poczatkowa>0 AND wartosc_aktualna=0); "
                stat,DRaty,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),3)
                if stat != 'OK':
                    stat = "Bd przy pobieraniu zobowiza ratalnych: "+stat

                if stat == 'OK':
                    SRaty = {}
                    for poz in DRaty:
                            SRaty[poz[0].strip()] = [poz[1],poz[2]] # kwota, wpisowe dla skladnika
                    rsglobal.ZapiszDoLogow('!!!Raty '+str(SRaty),1)

            if stat == 'OK':
                SDaneAngazy[(Zatrudnienie,Umowa,Angaz,DataOdA)] =[DataOdL, DataDoL, DSzabl[0][0], SSkladWersje,  # lista [data listy OD, data listy DO, skadniki pocztkowe,dostpne skadniki bazowe,
                                SMod, SSkladAn, SStale, SZmienne,SSkladKolej,        # skadniki zmodyfikowane z listy, stae z angau, stae pacowe, zmienne pacowe pracownika, sownik z kolejnoci skadnikw]
                                [DataOdA, DataDoA], [SAbsencje, SRaty, SAbsencjeMc, SSkladStalyJedn, SAbsencjePM]]   # Data od angau w licie pac, Data do angau w licie pac, slownik absencji, slownik rat
        
    if stat == 'OK':
        dane = [SDaneAngazy]


#-------------------------------------------------------------------------------
#---   ZAPISU DANYCH PO PRZELICZENIU (16)
#-------------------------------------------------------------------------------
            
if Funkcja == 16 and stat=="OK":
    if not UprPrzel:
        stat='BR UPR PRZEL'
        
    if stat == 'OK':
        if Stan != 'W' and NowySkladnik == '':
            stat = 'Nie mona przelicza angay z listy w stanie innym ni W'
            
    # ewentualne usunicie poprzednich danych
    if stat == 'OK' and Usuwanie and NowySkladnik == '':
        SQL = "DELETE FROM rs_kp_listy_plac_osoby_skladniki WHERE firma_id='"+self.firma_id+"'"
        SQL+=" AND numer_listy_id='"+Lista+"' AND pracownik_id = '"+Pracownik+"'"
        SQL+=" AND zatrudnienie_id="+Zatrudnienie+" AND umowa_id="+Umowa+" AND angaz_id="+Angaz
        SQL+=" AND data_od='"+DataA+"' AND modyfikowane_kiedy IS NULL;"
        stat,dump,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),1)
        if stat != 'OK':
            stat = u"Bd przy usuwaniu skadnikw z angau listy: "+stat   
            
    # pobieramy skadniki ktre zostay - od tego zaley czy robimy insert
    if stat == 'OK':
        SQL = "SELECT skladnik_id FROM rs_kp_listy_plac_osoby_skladniki WHERE firma_id='"+self.firma_id+"'"
        SQL+=" AND numer_listy_id='"+Lista+"' AND pracownik_id = '"+Pracownik+"'"
        SQL+=" AND zatrudnienie_id="+Zatrudnienie+" AND umowa_id="+Umowa+" AND angaz_id="+Angaz
        SQL+=" AND data_od='"+DataA+"'"
        stat,DStareS,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),3)
        if stat != 'OK':
            stat = u"Bd przy pobieraniu starych skadnikw z angau listy: "+stat   
    if stat =='OK':
        LStare = []
        for poz in DStareS:
            LStare.append(poz[0].strip())

    # pobieramy list skadnikw, ktre maj by na liscie - skadniki spoza tej listy pomijamy
    if stat == 'OK':
        SQL = "SELECT skladniki FROM rs_kp_szablony_list_plac_wersje WHERE firma_id='"+self.firma_id+"' AND lista_id='"+SzablonL+"'" \
                " AND data_od <='"+DataOdL+"' AND (data_do >= '"+DataDoL+"' OR data_do IS NULL) ORDER BY data_od DESC LIMIT 1"

        stat,DSkladniki,dump,dump = self.PolSQL.Execute(SQL,3)
        if stat != 'OK':
            stat = u"Bd przy pobieraniu skadnikw z szablonu listy: "+stat
        if stat =='OK':
            LSkladniki = []
            #for w in DSkladniki[0][0].splitlines():
            DSkladniki[0][0] = DSkladniki[0][0].replace('\n','').replace('\r','')
            for skl in DSkladniki[0][0].split(','):
                if skl.strip() != '':
                    LSkladniki.append(skl.strip())
            rsglobal.ZapiszDoLogow('skladniki na liscie:' + str(LSkladniki))

    # waciwy zapis danych
    if stat == 'OK':
        SQL = ''
        for skladnik in SSkladnikiZ:
            rsglobal.ZapiszDoLogow('skladnik '+str(skladnik.strip()))
            if NowySkladnik != '':
                if skladnik.strip() != NowySkladnik.strip():  # chcemy zapisac tylko nowy skladnik
                    continue
            if not skladnik.strip() in LStare and skladnik.strip() in LSkladniki and SSkladnikiZ[skladnik][1]: # jeli przeliczony
                SQL+="INSERT INTO rs_kp_listy_plac_osoby_skladniki "
                #AB 26.08.21r
                SQL+=" (firma_id,numer_listy_id,pracownik_id,zatrudnienie_id,umowa_id,angaz_id,data_od,skladnik_id,wartosc, modyfikowalny) "
                SQL+=" SELECT '"+self.firma_id+"','"+Lista+"','"+Pracownik+"',"+Zatrudnienie+","
                SQL+=Umowa+","+Angaz+",'"+DataA+"','"+skladnik+"',"+str(SSkladnikiZ[skladnik][0])+", CASE WHEN " \
                     " modyfikowalny_recznie THEN '1' ELSE '0' END " \
                     "FROM rs_kp_skladniki_placowe WHERE firma_id='"+self.firma_id+"' AND skladnik_id= '"+skladnik+"';"

        if SQL != '':
            stat,dump,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),1)
            if stat != 'OK':
                stat = "Bd przy wstawieniu wyliczonych skadnikw do angau: "+stat+" SQL="+str(SQL)   

    # ustawienie angau jako przeliczonego/nieprzeliczonego
    if stat == 'OK' and Rozliczenie and NowySkladnik == '':
        SQL = "UPDATE rs_kp_listy_plac_osoby SET przeliczony = TRUE WHERE firma_id='"+self.firma_id+"'"
        SQL+=" AND numer_listy_id='"+Lista+"' AND pracownik_id = '"+Pracownik+"'"
        SQL+=" AND zatrudnienie_id="+Zatrudnienie+" AND umowa_id="+Umowa+" AND angaz_id="+Angaz
        SQL+=" AND data_od='"+DataA+"'"
        stat,dump,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),1)
        if stat != 'OK':
            stat = "Bd przy zmianie przeliczenia angau: "+stat

# -------------------------------------------------------------------------------
# ---   ZAPISU NUMRU LISTY I DATY WYPLATY DO ABSENCJI (17)
# -------------------------------------------------------------------------------

if Funkcja == 17 and stat == "OK":
    if not UprPrzel:
        stat = 'BR UPR PRZEL'
    if stat == 'OK':
        SQL = "SELECT data_wyplaty, data_od, data_do FROM rs_kp_listy_plac WHERE firma_id='"+self.firma_id+"' AND numer_listy_id='"+Lista+"'"
        stat, dane, dump, dump = self.PolSQL.Execute(SQL, 3)
        if stat != 'OK':
            stat = u"Bd przy pobieraniu daty wypaty z listy: "+stat
        elif len(dane) > 0:
            DataWypl = dane[0][0]
            DataOdL = dane[0][1]
            DataDoL = dane[0][2]
        else:
            stat = "Niepoprawny numer listy przy zapisie absencji"
    if stat == 'OK':
        SQL = ''
        PomOd = rs_anakonda_daty.DodajKilkaDni(DataOdL, -31)
        for a in SAbsencje.keys():
            if SAbsencje[a][5] == 1:
                # pobrane do listy plac zapisujemy do absencji
                SQL += "UPDATE rs_kp_dane_osobowe_absencje SET numer_listy_id='"
                SQL += Lista + "', data_wyplaty='" + str(DataWypl) + "'"
                SQL += " WHERE firma_id='" + self.firma_id + "' AND pracownik_id='" + Pracownik + "'"
                SQL += " AND COALESCE(numer_listy_id,'')=''"
                SQL += " AND typ_absencji_id='" + a[0] + "' AND COALESCE(kod_zus,'')='" + a[1] + "'"
                SQL += " AND od_kiedy>='" + str(PomOd) + "' AND do_kiedy<='" + str(DataDoL) + "'; "
        PomDo = rs_anakonda_daty.DodajKilkaDni(DataOdL, -1)
        for a in SAbsencjePM.keys():
            if SAbsencjePM[a][5] == 1:
                # pobrane do listy plac zapisujemy do absencji
                SQL += "UPDATE rs_kp_dane_osobowe_absencje SET numer_listy_id='"
                SQL += Lista + "', data_wyplaty='" + str(DataWypl) + "'"
                SQL += " WHERE firma_id='" + self.firma_id + "' AND pracownik_id='" + Pracownik + "'"
                SQL += " AND COALESCE(numer_listy_id,'')=''"
                SQL += " AND typ_absencji_id='" + a[0] + "' AND COALESCE(kod_zus,'')='" + a[1] + "'"
                SQL += " AND od_kiedy>='" + str(PomOd) + "' AND do_kiedy<='" + str(PomDo) + "'; "
        if SQL != '':
            stat, dump, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL), 1)
        if stat != 'OK':
            stat = "Bd przy aktualizacji absencji: " + stat

#-------------------------------------------------------------------------------
#---   GRUPOWA MODYFKACJA SKADNIKW MODYFIKOWALNYCH (18)
#-------------------------------------------------------------------------------
            
if Funkcja == 18 and stat == "OK":
    if not UprUpd:
        stat='BR UPR UPD'
        
    if stat == 'OK':
        if Stan != 'W':
            stat = 'Nie mona zmienia skadnikw na licie o stanie innym ni W'
            
    #zmiana skadnikw i angay (na nieprzeliczone)
    if stat == 'OK':
        LAng = []
        SQL = ''
        for poz in LSkladniki:
            SQL+= " UPDATE rs_kp_listy_plac_osoby_skladniki SET wartosc="+str(poz[6])+","
            SQL+= " modyfikowane_przez="+str(self.operator_id)+",modyfikowane_kiedy=NOW() "
            SQL+= " WHERE firma_id='"+self.firma_id+"' AND numer_listy_id='"+Lista+"' AND pracownik_id='"+poz[0]+"'"
            SQL+= " AND zatrudnienie_id="+str(poz[1])+" AND umowa_id="+str(poz[2])
            SQL+= " AND angaz_id="+str(poz[3])+" AND data_od = '"+poz[4]+"' AND skladnik_id='"+poz[5]+"';"
            
            KlAng = poz[:5]
            if not KlAng in LAng:
                SQL+= "UPDATE rs_kp_listy_plac_osoby SET przeliczony = FALSE"
                SQL+= " WHERE firma_id='"+self.firma_id+"' AND numer_listy_id='"+Lista+"' AND pracownik_id='"+poz[0]+"'"
                SQL+= " AND zatrudnienie_id="+str(poz[1])+" AND umowa_id="+str(poz[2])
                SQL+= " AND angaz_id="+str(poz[3])+" AND data_od = '"+poz[4]+"';"
                LAng.append(KlAng)
        
        if SQL != '':
            stat,dump,dump,dump = self.PolSQL.Execute(rsglobal.Dekoduj(SQL),1)
            if stat != 'OK':
                stat = u"Bd przy grupowej zmianie skadnikw modyfikowalnych: "+stat   
            
        
#-------------------------------------------------------------------------------
#---   POBRANIE DANYCH (1- update skadnikw, 5-select, 6 - delete listy, 7 -zamykanie, 8 - otwieranie, 11 - usuwanie angau)
#-------------------------------------------------------------------------------

if Funkcja in (5,6,7,8,11) and stat == 'OK':
    # pobranie danych do grida list pac

    # nie robimy tego po usuniciu angau (jeli nie filtrujemy po pracowniku)
    rsglobal.ZapiszDoLogow('!!! '+str(TylkoAngaz)+' '+Lista,1)
    if (Funkcja == 11 and PracownikF == '') or (TylkoAngaz and Lista != ''):
        DListy = [['','','',Lista]]

    # a teraz robimy (pozostae przypadki)
    else:
        SQL = "SELECT DISTINCT lp.stan,lp.lista_id,slp.opis,lp.numer_listy_id,COALESCE(lp.opis,''),lp.data_wyplaty,"
        SQL += " SUBSTRING(lp.numer_listy_id FROM 1 FOR 4),SUBSTRING(lp.numer_listy_id FROM 6 FOR 2),"
        SQL += "lp.data_od,lp.data_do,COALESCE(lp.grupa_pracownikow,''),COALESCE(lp.komorka_id,'')"
        SQL += " FROM rs_kp_listy_plac lp "
        SQL += " INNER JOIN rs_kp_szablony_list_plac slp USING(firma_id,lista_id)"
        if PracownikF != '':
            SQL+=" INNER JOIN rs_kp_listy_plac_osoby lpo USING(firma_id,numer_listy_id)"
        SQL += " WHERE (firma_id='"+self.firma_id+"' "
        if StanF != '':
            SQL += " AND lp.stan = '"+StanF+"'"
        if SzablonF != '':
            SQL += " AND lp.lista_id = '"+SzablonF+"'"
        if ListaF != '':
            SQL += " AND lp.numer_listy_id LIKE '"+ListaF+"%'"
        if OpisF != '':
            SQL += " AND lp.opis LIKE '"+OpisF+"%'"
        if PracownikF != '':
            SQL += " AND lpo.pracownik_id LIKE '"+PracownikF+"%'"
        if DataWyplOdF.strip() != '-  -':
            SQL += " AND lp.data_wyplaty >= '"+DataWyplOdF+"'"
        if DataWyplDoF.strip() != '-  -':
            SQL += " AND lp.data_wyplaty <= '"+DataWyplDoF+"'"
        if RokF.strip() != '':
            SQL += " AND SUBSTRING(lp.numer_listy_id FROM 1 FOR 4) = '"+RokF+"'"
        if McF.strip() != '':
            SQL += " AND SUBSTRING(lp.numer_listy_id FROM 6 FOR 2) = '"+McF+"'"
        SQL += ")"
        if len(LKluczyF) > 0:
            for nrlisty in LKluczyF:
                SQL += " OR (lp.firma_id = '"+self.firma_id+"' AND lp.numer_listy_id = '"+nrlisty+"') "

        SQL += " ORDER BY lp.data_od DESC,lp.numer_listy_id;"
        stat, DListy, dump, dump = self.PolSQL.Execute(SQL,3)
        rsglobal.ZapiszDoLogow('lista1: ' + SQL, 5)
        if stat != 'OK':
            stat = u"Bd przy pobieraniu list: "+stat
            dane.append(stat)

    if stat=='OK':
        # pobranie angay dla list
        rsglobal.ZapiszDoLogow('!!!2 '+str(DListy),1)
        SAngaze = {}
        if len(DListy)>0:
            SQL="SELECT lpo.numer_listy_id,lpo.pracownik_id, dos.imie, dos.nazwisko, dos.pesel, "
            SQL+=" lpo.zatrudnienie_id, lpo.umowa_id, lpo.angaz_id, lpo.data_od,lpo.data_do, lpo.przeliczony"
            SQL+=" FROM rs_kp_listy_plac_osoby lpo "
            SQL+=" INNER JOIN rs_kp_dane_osobowe dos USING (firma_id,pracownik_id)"
            SQL+=" WHERE lpo.firma_id='"+self.firma_id+"' "
            if PracownikF != '':
                SQL+=" AND lpo.pracownik_id LIKE '"+PracownikF+"%'"
            SQL+=" AND lpo.numer_listy_id IN ("
            for lista in DListy:
                 SQL+="'"+lista[3]+"',"
            SQL = SQL.strip(',')
            SQL+=") ORDER BY lpo.numer_listy_id,dos.nazwisko,dos.imie,lpo.data_od,lpo.data_do"
            stat,DAngaze,dump,dump=self.PolSQL.Execute(SQL,3)
            rsglobal.ZapiszDoLogow('lista2: '+ SQL, 5)

            if stat != 'OK':
                stat = u'Bd przy pobieraniu angay : '+stat
                dane.append(stat)
            
            if stat == 'OK':
                for poz in DAngaze:
                    if not poz[0] in SAngaze:
                        SAngaze[poz[0]] = []
                        
                    SAngaze[poz[0]].append(poz[1:])
                    
    # pobranie skadnikw dla angay
    if stat=='OK':
        SSkladniki = {}
        for lista in DListy:
            if lista[3] in SAngaze:
                for angaz in SAngaze[lista[3]]:
                    #AB 26.08.21r
                    SQL = "SELECT lpos.skladnik_id,sp.opis,lpos.wartosc,lpos.modyfikowalny::INT > 0 as modyfikowalny, "\
                          "zmodyfikowal, modyfikowalny AS mod FROM rs_kp_listy_plac_osoby_skladniki lpos "
                    SQL+=" INNER JOIN rs_kp_skladniki_placowe sp USING (firma_id,skladnik_id) "
                    SQL+=" WHERE firma_id='"+self.firma_id+"'"
                    SQL+=" AND numer_listy_id='"+lista[3]+"' AND pracownik_id = '"+angaz[0]+"'"
                    SQL+=" AND zatrudnienie_id="+str(angaz[4])+" AND umowa_id="+str(angaz[5])+" AND angaz_id="+str(angaz[6])
                    #AB 26.03.19r.
                    #SQL+=" AND data_od='"+str(angaz[7]) +"' "
                    SQL+=";"
                    stat,DSkladniki,dump,dump=self.PolSQL.Execute(SQL,3)
        
                    if stat != 'OK':
                        stat = u'Bd przy pobieraniu skadnikw : '+stat
                        dane.append(stat)
                    
                    if stat == 'OK':
                        SSkladniki[(lista[3],angaz[0],str(angaz[4]),str(angaz[5]),str(angaz[6]),str(angaz[7]))] = eval(str(DSkladniki))
        
    if stat=='OK':        
        dane =[DListy,SAngaze,SSkladniki]