# -*- coding: iso8859_2 -*-
#-------------------------------------------------------------------------------
# Name:        fs_kp_praca_biezaca_absencje_pom.py
# Purpose:     funkcje pomocnicze dla absencji z pracy biecej
# Author:      Jan Raburski
# Created:     2018/09/04
# RCS-ID:      $Id: fs_kp_praca_biezaca_absencje_pom.py $
# Copyright:   (c) 2018 Record System
# Licence:     GNU GPL
#-------------------------------------------------------------------------------
# 2018-10-03 Aleksander Bargie (11116) - [KP] Podzia absencji miesicami
# 2021-01-20 Aleksander Bargie (12234) Dni nominalne w kartach pracy - uwzglednia wieta przypadajce w sobot
# 2023-08-28 Jan Raburski (13512) - przejcie w dniach roboczych na kalendarz
# 2024-02-27 Jan Raburski (13709) - nie wliczac niepelnego pierwszego miesiaca zatrudnienia do liczenia podstawy

import datetime
import rsglobal
import rs_anakonda_daty
from string import *

def LiczDniIGodzinyRobocze(self, DataOd, DataDo, Pracownik):
    stat = 'OK'
    ldni = rs_anakonda_daty.LiczbaDniPracujacychWgKalendarza(self, Pracownik, rs_anakonda_daty.ZamienDate(DataOd), rs_anakonda_daty.ZamienDate(DataDo))
    dane = [[ldni, ldni*8, '01']]
    return stat, dane


#-------------------------------------------------------------------------------

def PodajZatrudnienieUmoweAngaz(self, Data, Pracownik):
    # podaje zatrudnienie, umowe i angaz dla ktorego maja byc uwzgledniane absencje
    SQL ="SELECT a.zatrudnienie_id, a.umowa_id, a.angaz_id, z.staz_do_urlopu, a.etat  FROM rs_kp_angaze a"
    SQL+=" INNER JOIN rs_kp_zatrudnienie z ON a.firma_id=z.firma_id AND a.pracownik_id = z.pracownik_id"
    SQL+=" AND a.zatrudnienie_id=z.numer_id"
    SQL+=" WHERE a.firma_id='"+self.firma_id+"' "
    SQL+=" AND a.pracownik_id='"+Pracownik+"' AND z.czy_absencje AND a.od_kiedy<='"+str(Data)+"'"
    SQL+="  ORDER BY a.od_kiedy DESC LIMIT 1;"
    stat, dane, dump, dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return stat, []
    if len(dane) == 0:
        return ' Brak danych angau dla pracownika:' + Pracownik + 'na dzie:' + str(Data), []
    return 'OK', dane

#-------------------------------------------------------------------------------

def ZmianaLimituUrlopu(self, Data, Pracownik, TypAbsencji):
    # sprawdza czy zmienil sie limit do wykorzystania dla pracownika w ramach urlopu
    # jesli przekroczone zostalo 10 lat stazu to do urlopu dodawane jest 6 dni
    # w zaleznosci od ostatniego urlopu albo do zaleglego albo tegorocznego
    # UWAGA: Data jest dat koca urlopu

    zalegly = 0
    tegoroczny = 0

    # pobranie daty od kiedy liczyc staz
    stat, dane = PodajZatrudnienieUmoweAngaz(self, Data, Pracownik)
    if stat != 'OK':
        return stat, zalegly, tegoroczny
    datastazu = dane[0][3]

    # pobieramy date konca poprzedniego urlopu
    SQL = "SELECT do_kiedy "
    SQL += " FROM rs_kp_dane_osobowe_absencje WHERE firma_id='" + self.firma_id + "' "
    SQL += " AND pracownik_id='" + Pracownik + "' AND typ_absencji_id LIKE '" + TypAbsencji.strip() + "%'"
    SQL += " AND do_kiedy<'" + str(Data) + "' ORDER BY do_kiedy DESC LIMIT 1;"
    stat, Poprzedni, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat != 'OK':
        return stat, zalegly, tegoroczny
    if len(Poprzedni) == 0:
        return stat, zalegly, tegoroczny
    datapoprzednia = Poprzedni[0][0]
    poprzednie = rs_anakonda_daty.OkresPomiedzy(datastazu, datapoprzednia)[0:4]
    aktualne = rs_anakonda_daty.OkresPomiedzy(datastazu, Data)[0:4]
    if poprzednie == '0009' and aktualne == '0010':
        # jest zmiana limitu
        if datapoprzednia[0:4] == Data[0:4]:
            # ten sam rok
            tegoroczny = 6
        else:
            # w ktorym roku zmiana (sprawdzamy z data konca roku poprzedniego urlopu)
            if rs_anakonda_daty.OkresPomiedzy(datastazu, datapoprzednia[0:4] + '-12-31')[0:4] == '0009':
                tegoroczny = 6
            else:
                zalegly = 6

    return stat, zalegly, tegoroczny

#-------------------------------------------------------------------------------

def PodajLimitDoWykorzystania(self, Data, Pracownik, TypAbsencji, Rodzina, NaZadanie = False, DataDo = None):
    # podaje limit do wykorzystania dla pracownika w ramach typu absencji

    from funkcje import fs_kp_zmienne_placowe_pom as zm

    stat = 'OK'
    wart = 0
    Rok = Data[0:4]
    if NaZadanie:
        # sprawdzamy czy nie przekroczone 4 dni w roku
        # jesli tak, wracamy z zerem, jesli nie wracamy z min z urlopu i na zadanie
        SQL = "SELECT COALESCE(SUM(dni_robocze),0) "
        SQL += " FROM rs_kp_dane_osobowe_absencje WHERE firma_id='" + self.firma_id + "' "
        SQL += " AND pracownik_id='" + Pracownik + "' AND typ_absencji_id LIKE '" + TypAbsencji.strip() + "%'"
        SQL += " AND na_zadanie AND EXTRACT(YEAR from od_kiedy)=" + Data[0:4] + ";"
        stat, Wykorzystane, dump, dump = self.PolSQL.Execute(SQL, 3)
        if stat != 'OK':
            return stat, 0
        ZostaloNaZadanie = 4 - Wykorzystane[0][0]
        return stat, ZostaloNaZadanie

    SQL = "SELECT przechodni "
    SQL += " FROM rs_kp_absencje_rodzaje WHERE firma_id='"+self.firma_id+"' AND typ_absencji_id='"+TypAbsencji+"';"
    stat, Parametr, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat != 'OK':
        return stat, 0
    if Parametr[0][0]:
        SQL = "SELECT COALESCE(dni_zalegle,0)+COALESCE(dni_tegoroczne,0), od_kiedy "
        SQL += " FROM rs_kp_dane_osobowe_absencje WHERE firma_id='"+self.firma_id+"' "
        SQL += " AND pracownik_id='"+Pracownik+"' AND typ_absencji_id LIKE '"+TypAbsencji.strip()+"%'"
        SQL += " AND od_kiedy<'"+str(Data) + "' AND EXTRACT(YEAR from od_kiedy)="+Data[0:4] + " ORDER BY od_kiedy DESC LIMIT 1;"
        stat,Zostalo,dump,dump = self.PolSQL.Execute(SQL,3)
        if stat != 'OK':
            return stat, 0
        rsglobal.ZapiszDoLogow('Zostalo:'+str(Zostalo)+' len:'+str(len(Zostalo)), 1)

        # sprawdzamy czy nie ma pozniejszego limitu
        SQL = "SELECT COALESCE(limit_dni_zalegle,0)+COALESCE(limit_dni,0), od_dnia"
        SQL += " FROM rs_kp_dane_osobowe_limity_urlopow WHERE firma_id='" + self.firma_id + "' "
        # SQL += " AND pracownik_id='" + Pracownik + "' AND urlop_id LIKE '" + TypAbsencji.strip() + "%' AND rok='" + Rok + "';"  #JR 2021-05-29
        SQL += " AND pracownik_id='" + Pracownik + "' AND urlop_id LIKE '" + TypAbsencji.strip() + "%'"
        SQL += " AND EXTRACT(YEAR FROM od_dnia)=" + Data[0:4] + " ORDER BY od_dnia DESC LIMIT 1;"
        stat, ZostaloZL, dump, dump = self.PolSQL.Execute(SQL, 3)
        if stat != 'OK':
            return stat, 0

        # do dalszych obliczen biezemy nowsze
        if ZostaloZL != [] and Zostalo != []:
            if ZostaloZL[0][1] > Zostalo[0][1]:
                Zostalo = ZostaloZL
        elif ZostaloZL != [] and Zostalo == []:
            Zostalo = ZostaloZL

        if Zostalo != []:
            if DataDo != None:
                stat, stazzal, stazteg  = ZmianaLimituUrlopu(self, DataDo, Pracownik, TypAbsencji)
            else:
                stat, stazzal, stazteg = ZmianaLimituUrlopu(self, Data, Pracownik, TypAbsencji)
            if stat != 'OK':
                return stat, 0
            Zostalo[0][0] += stazzal + stazteg
            if Zostalo[0][0] > 0:
                #return stat, min(Zostalo[0][0], ZostaloNaZadanie)
                return stat, Zostalo[0][0]

    wykorzystane = 0
    # pobranie wykorzystanych
    SQL ="SELECT sum(dni_robocze) "
    SQL+=" FROM rs_kp_dane_osobowe_absencje WHERE firma_id='"+self.firma_id+"' "
    if Rodzina > 0:
        SQL+=" AND rodzina_id="+str(Rodzina)
    SQL+=" AND pracownik_id='"+Pracownik+"' AND typ_absencji_id LIKE '"+TypAbsencji.strip()+"%' AND EXTRACT(YEAR from od_kiedy)="+Data[0:4]+";"
    stat,Wykorzystane,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return stat, 0 
    if len(Wykorzystane)>0:
        wykorzystane = Wykorzystane[0][0]
    
    # sprawdzenie czy sa limity wprowadzone
    SQL = "SELECT limit_dni,limit_dni_zalegle, limit_godziny,limit_godziny_zalegle "
    SQL += " FROM rs_kp_dane_osobowe_limity_urlopow WHERE firma_id='"+self.firma_id+"' "
    #SQL+=" AND pracownik_id='"+Pracownik+"' AND urlop_id LIKE '"+TypAbsencji.strip()+"%' AND rok='"+Data[0:4]+"';"  #JR 2021-05-29
    SQL += " AND pracownik_id='"+Pracownik+"' AND urlop_id LIKE '"+TypAbsencji.strip()+"%'"
    SQL += " AND EXTRACT(YEAR FROM od_dnia)="+Data[0:4]+" AND od_dnia<='"+Data+"' ORDER BY od_dnia DESC LIMIT 1;"
    stat,Limit,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return stat, 0
    if len(Limit) > 0:
        return stat, Limit[0][0] + Limit[0][1] - wykorzystane

    # sprawdzenie czy przechodni - pobierany wtedy z zaleglych i tegorocznych
    zostalo = 0
    SQL ="SELECT czy_zwolnienie, przechodni "
    SQL+=" FROM rs_kp_absencje_rodzaje WHERE firma_id='"+str(self.firma_id)+"'AND typ_absencji_id = '"+TypAbsencji.strip()+"';"
    stat,Parametry,wyn1,dump=self.PolSQL.Execute(SQL,3)
    if stat !='OK':
        return stat, 0
    else:
        czyzwolnienie = Parametry[0][0]
        czyprzechodni = Parametry[0][1]
    if czyprzechodni:
        SQL = "SELECT COALESCE(dni_zalegle,0) + COALESCE(dni_tegoroczne,0), od_kiedy "
        SQL += " FROM rs_kp_dane_osobowe_absencje WHERE firma_id='"+self.firma_id+"' "
        SQL += " AND pracownik_id='"+Pracownik+"' AND typ_absencji_id LIKE '"+TypAbsencji.strip()+"%'"
        SQL += " AND od_kiedy<'"+str(Data)+"' ORDER BY od_kiedy DESC LIMIT 1;"
        stat, Zostalo, dump, dump = self.PolSQL.Execute(SQL, 3)
        if stat != 'OK':
            return stat, 0
        elif len(Zostalo) > 0:
            if str(Zostalo[0][1])[0:4] == str(Data)[0:4]:
                # ten sam rok
                return stat, Zostalo[0][0]
            else:
                # poprzedni rok, dodajemy limit z nowego roku
                zostalo = Zostalo[0][0]

    wart= 0
    stat = 'OK'
    # pobranie algorytmu limitu
    SQL ="SELECT algorytm_limit "
    SQL+=" FROM rs_kp_absencje_rodzaje WHERE firma_id='"+self.firma_id+"' "
    SQL+=" AND typ_absencji_id='"+TypAbsencji+"';"
    stat,dane,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return stat, 0
    if len(dane) > 0:
        tresc = dane[0][0].strip()
        if tresc in ['', 'None', '<None>']:
            wart = 0
        else:
            stat,angaz = PodajZatrudnienieUmoweAngaz(self, Data, Pracownik)
            if stat != 'OK':
                return stat,0
            stat,tresc = zm.PodmienDanymi(self,tresc,Pracownik,str(angaz[0][0]),str(angaz[0][1]),str(angaz[0][2]),'',False,[TypAbsencji,Data],Rodzina)
            if stat == 'OK':
                try:
                    rsglobal.ZapiszDoLogow("limity tresc:"+tresc,1)
                    exec(tresc.replace(chr(13),''))
                    wart = wart - wykorzystane + zostalo
                except:
                    stat = "Bd liczenia limitow: "+tresc.replace(chr(13),'')
                    wart = 0

    return stat, wart

# -------------------------------------------------------------------------------

def PodajLimitNowyRok(self, Rok, Pracownik, TypAbsencji):
    # podaje limit na nowy rok do wykorzystania dla pracownika w ramach typu absencji
    # uwzgledniajac zalegly z poprzedniego roku
    # zwraca stat, liczba dni zaleglych i liczba dni tegorocznych dp wykorzystania
    # jesli jest urlop w tym roku to wraca z zerami

    from funkcje import fs_kp_zmienne_placowe_pom as zm

    stat = 'OK'
    Rok = str(int(Rok))
    wart = 0

    # ma juz wpisany limit - wracamy z zerem
    SQL = "SELECT * "
    SQL += " FROM rs_kp_dane_osobowe_limity_urlopow WHERE firma_id='" + self.firma_id + "' "
    SQL += " AND pracownik_id='" + Pracownik + "' AND urlop_id LIKE '" + TypAbsencji.strip() + "%' AND od_dnia='" + Rok + "-01-01';"
    stat, Limit, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat != 'OK':
        return stat, [0, 0]
    if len(Limit) > 0:
        return stat, [0, 0]

    # dni z zeszlego roku
    SQL = "SELECT COALESCE(dni_zalegle,0)+COALESCE(dni_tegoroczne,0), od_kiedy "
    SQL += " FROM rs_kp_dane_osobowe_absencje WHERE firma_id='" + self.firma_id + "' "
    SQL += " AND pracownik_id='" + Pracownik + "' AND typ_absencji_id LIKE '" + TypAbsencji.strip() + "%'"
    SQL += " AND EXTRACT(YEAR from od_kiedy)=" + str(int(Rok) - 1) + " ORDER BY od_kiedy DESC LIMIT 1;"
    stat, Zostalo, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat != 'OK':
        return stat, [0, 0]
    # sprawdzamhy czy byl limit
    SQL = "SELECT COALESCE(limit_dni,0) +  COALESCE(limit_dni_zalegle,0), od_dnia"
    SQL += " FROM rs_kp_dane_osobowe_limity_urlopow WHERE firma_id='" + self.firma_id + "' "
    SQL += " AND pracownik_id='" + Pracownik + "' AND urlop_id LIKE '" + TypAbsencji.strip() + "%'"
    #SQL += " AND rok='" + str(int(Rok) - 1) + "';"  #JR 2021-05-29
    SQL += " AND EXTRACT(YEAR FROM od_dnia)=" + str(int(Rok) - 1) + " ORDER BY od_dnia DESC LIMIT 1;"
    stat, ZostaloZL, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat != 'OK':
        return stat, [0, 0]
    if len(Zostalo) > 0 and len(ZostaloZL) > 0:
        if Zostalo[0][1] > ZostaloZL[0][1]:
            zostalo = Zostalo[0][0]
        else:
            zostalo = ZostaloZL[0][0]
    elif len(Zostalo) > 0:
        zostalo = Zostalo[0][0]
    elif len(ZostaloZL) > 0:
        zostalo = ZostaloZL[0][0]
    else:
        zostalo = 0

    # pobranie algorytmu limitu
    SQL = "SELECT algorytm_limit "
    SQL += " FROM rs_kp_absencje_rodzaje WHERE firma_id='" + self.firma_id + "' "
    SQL += " AND typ_absencji_id='" + TypAbsencji + "';"
    stat, dane, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat != 'OK':
        return stat, [0, 0]
    if len(dane) > 0:
        tresc = dane[0][0].strip()
        if tresc not in ['', 'None', '<None>']:
            stat, angaz = PodajZatrudnienieUmoweAngaz(self, Rok + '.01.01', Pracownik)
            if stat != 'OK':
                return stat, [0, 0]
            stat, tresc = zm.PodmienDanymi(self, tresc, Pracownik, str(angaz[0][0]), str(angaz[0][1]), str(angaz[0][2]),
                                           '', False, [TypAbsencji, Rok + '.01.01'], False)
            if stat == 'OK':
                try:
                    exec (tresc.replace(chr(13), ''))
                    # exec ustawia zmienna wart
                except:
                    stat = "Bd liczenia limitow: " + tresc.replace(chr(13), '')
                    return stat, [0, 0]

    return stat, [zostalo, wart]

# -------------------------------------------------------------------------------

def PodajZalegleTegoroczneUrlop(self, DataOd, Pracownik, TypAbsencji):
    # podaje aktualne liczby dni zaleglych i tegorocznychdo wykorzystania dla pracownika w ramach typu absencji
    # uwzgledniajac aktualny stan absencji i limity
    # uzywac tylko dla przechodnich

    from funkcje import fs_kp_zmienne_placowe_pom as zm

    stat = 'OK'
    Rok = DataOd[0:4]
    puste = [[0, 0, '    -  -  ']]
    Zostalo = []
    
    # ma wczesniejsze juz absencje w tym roku - wracamy z danymi
    SQL = "SELECT COALESCE(dni_zalegle,0), COALESCE(dni_tegoroczne,0), od_kiedy "
    SQL += " FROM rs_kp_dane_osobowe_absencje WHERE firma_id='" + self.firma_id + "' "
    SQL += " AND pracownik_id='" + Pracownik + "' AND typ_absencji_id LIKE '" + TypAbsencji.strip()+"%'"
    SQL += " AND od_kiedy<'" + str(DataOd) + "' AND od_kiedy>='" + Rok + "-01-01' ORDER BY od_kiedy DESC LIMIT 1;"
    stat, Zostalo, dump, dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return stat, puste
    #if len(Zostalo) > 0:
    #    return stat, Zostalo

    # ma wpisany limit - wracamy z danymi
    SQL = "SELECT COALESCE(limit_dni_zalegle,0), COALESCE(limit_dni,0), od_dnia"
    SQL += " FROM rs_kp_dane_osobowe_limity_urlopow WHERE firma_id='" + self.firma_id + "' "
    #SQL += " AND pracownik_id='" + Pracownik + "' AND urlop_id LIKE '" + TypAbsencji.strip() + "%' AND rok='" + Rok + "';"  #JR 2021-05-29
    SQL += " AND pracownik_id='" + Pracownik + "' AND urlop_id LIKE '" + TypAbsencji.strip() + "%'"
    SQL += " AND EXTRACT(YEAR FROM od_dnia)=" + Rok + " ORDER BY od_dnia DESC LIMIT 1;"
    stat, ZostaloZL, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat != 'OK':
        return stat, puste
    if len(Zostalo) > 0 and len(ZostaloZL) > 0:
        if Zostalo[0][2] > ZostaloZL[0][2]:
            return stat, Zostalo
        else:
            return stat, ZostaloZL
    elif len(Zostalo) > 0:
        return stat, Zostalo
    elif len(ZostaloZL) > 0:
        return stat, ZostaloZL

    # liczymy limity i juz
    stat, Zostalo = PodajLimitNowyRok(self, Rok, Pracownik, TypAbsencji)
    if stat != 'OK':
        return stat, puste
    Zostalo.append(Rok + '-01-01')
    return stat, [Zostalo]

#-------------------------------------------------------------------------------

def PodajPodAbsencje(self, Data, Pracownik, TypAbsencji, Rodzina):
    # podaje podabsencje w ramach typu absencji

    from funkcje import fs_kp_zmienne_placowe_pom as zm

    wart = [['', 0]]
    # pobranie algorytmu
    SQL = "SELECT COALESCE(algorytm_rodzaj,'') "
    SQL += " FROM rs_kp_absencje_rodzaje WHERE firma_id='"+self.firma_id+"' "
    SQL += " AND typ_absencji_id='"+TypAbsencji+"';"
    stat, dane, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat != 'OK':
        return stat, wart
    if len(dane) > 0:
        tresc = dane[0][0].strip()
        if tresc == '':
            wart = [[TypAbsencji, 9999]]
        else:
            stat, angaz = PodajZatrudnienieUmoweAngaz(self, Data, Pracownik)
            if stat != 'OK':
                return stat, wart
            stat, tresc = zm.PodmienDanymi(self,tresc,Pracownik,str(angaz[0][0]),str(angaz[0][1]),str(angaz[0][2]),'',False,[TypAbsencji,Data],Rodzina)
            if stat == "OK":
                try:
                    rsglobal.ZapiszDoLogow("podabsencje tresc:"+tresc,1)
                    exec(tresc.replace(chr(13), ''))  # ustawia zmienna wart [TypAbsencji, liczbadni]
                except:
                    stat = 'Bd pobrania podabsencji'
                    wart = [['', 0]]

    return stat, wart


#-------------------------------------------------------------------------------

def PodajPodZwolnienie(self, Data, Pracownik, TypAbsencji, Rodzina, Szpital):
    # podaje podabsencje w ramach typu absencji

    from funkcje import fs_kp_zmienne_placowe_pom as zm
    
    wart=''
    stat = 'OK'
    # pobranie algorytmu
    SQL ="SELECT algorytm_rodzaj, czy_zwolnienie "
    SQL+=" FROM rs_kp_absencje_rodzaje WHERE firma_id='"+self.firma_id+"' "
    SQL+=" AND typ_absencji_id='"+TypAbsencji+"';"
    stat,dane,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return stat, [['',0,0]]
    if len(dane) > 0:
        if not dane[0][1]:
            # obsluga pierwszego wywolania dla podzialu wstepnego absencji dotyczy tylko zwolnien
            return stat,0
        tresc = dane[0][0].strip()
        if tresc in ['','None','<None>']:
            wart =  [[TypAbsencji,9999,0]]
        else:
            stat,angaz = PodajZatrudnienieUmoweAngaz(self, Data, Pracownik)
            if stat != 'OK':
                return stat,0
            stat,tresc = zm.PodmienDanymi(self,tresc,Pracownik,str(angaz[0][0]),str(angaz[0][1]),str(angaz[0][2]),'',False,[TypAbsencji,Data],Rodzina)
            if stat == "OK":
                try:
                    rsglobal.ZapiszDoLogow("podzwolnienie tresc:"+tresc,1)
                    exec(tresc.replace(chr(13),''))
                except:
                    stat = 'Bd pobrani podzwolnienia'
                    wart = [['',0,0]]
                if wart[0][0] != '':
                    # pobranie procentu wynagrodzenia
                    for w in wart:
                        w.append(0)
                        SQL ="SELECT proc_zwolnienia "
                        SQL+=" FROM rs_kp_absencje_rodzaje WHERE firma_id='"+self.firma_id+"' "
                        SQL+=" AND typ_absencji_id='"+w[0]+"';"
                        stat,dane,dump,dump = self.PolSQL.Execute(SQL,3)
                        if stat != 'OK':
                            stat = 'Bd pobrania procentu wynagrodzenia '+stat
                        elif len(dane) != 1:
                            stat = 'Bd pobrania procentu wynagrodzenia '+stat
                        else:
                            w[2] = dane[0][0]
    return stat, wart

# -------------------------------------------------------------------------------

def PodzielZwolnienieDo2022(self, Dane):
    # dzieli zwolnienia w zaleznosci od tgo kto placi i szpitala
    # Dane:
    # pracownik_id, typ_absencji_id, data_od, data_do,data_dostarczenia,data_wystawienia,
    # COALESCE(kod_zus,''), dni_kalendarzowe, dni_robocze, procent_wyplaty, kontynuacja,godziny_robocze,
    # dni_zalegle, godziny_zalegle, dni_tegoroczne, godziny_tegoroczne, rodzina_id, kontynuacja_data, dni_szpital,
    # na zadanie, czy wypadkowe, kod (A,B,C,D,E)

    from funkcje import fs_kp_zmienne_placowe_pom as zm

    wart = ''
    stat = 'OK'
    # pobranie algorytmu
    SQL = "SELECT algorytm_rodzaj, czy_zwolnienie "
    SQL += " FROM rs_kp_absencje_rodzaje WHERE firma_id='" + self.firma_id + "' "
    SQL += " AND typ_absencji_id='" + Dane[1] + "';"
    stat, dane, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat != 'OK':
        return stat, [['', 0, 0]]
    if len(dane) == 0:
        return 'Brak definicji absencji', [['', 0, 0]]
    if not dane[0][1]:
        # obsluga pierwszego wywolania dla podzialu wstepnego absencji dotyczy tylko zwolnien
        return "nie zwolnienie", 0
    tresc = dane[0][0].strip()
    if tresc in ['', 'None', '<None>']:
        wart = [[Dane[1], 9999, 0]]
        return "Brak algorytmu", wart

    Pracownik = Dane[0]
    TypAbsencji = Dane[1]
    DataOd = Dane[2]
    DataDo = Dane[3]
    KodZus = Dane[6]
    ProcWyn = Dane[9]
    Kontynuacja = Dane[10]
    Rodzina = Dane[16]
    KontynuacjaData = Dane[17]
    Szpital = Dane[18]
    Wypadkowe = Dane[20]
    DniZwolnienia = rsglobal.IleDniWZakresieDat(DataOd, DataDo)

    PrzelomRok = False
    PoprzedniTyp = ''
    nowe = []
    if DataOd[0:4] != DataDo[0:4]:
        # zwolnienie na przelomie roku liczymy jako dwa odobne zwolnienia z uwzglednieniem limitow
        w = []
        w.extend(Dane)
        w[3] = DataOd[0:4] + '-12-31'
        w[18] = min(rsglobal.IleDniWZakresieDat(w[2], w[3]), Szpital)
        Szpital = max(Szpital - rsglobal.IleDniWZakresieDat(w[2], w[3]), 0)  # pozostalo ze szpitala
        stat, wynik = PodzielZwolnienie(self, w)
        if stat != 'OK':
            return stat, wynik
        nowe.extend(wynik)
        PrzelomRok = True
        PoprzedniTyp = nowe[len(nowe)-1][1]
        Dane[2] = DataDo[0:4] + '-01-01'
        #Dane[3] = DataDo
        DataOd = DataDo[0:4] + '-01-01'
        DniZwolnienia = rsglobal.IleDniWZakresieDat(DataOd, DataDo)
        rsglobal.ZapiszDoLogow('na nowy rok:'+str(Dane)+' poprzedni typ:'+PoprzedniTyp, 1)

    stat, tresc = zm.PodmienDanymi(self, tresc, Pracownik, '0', '0', '0', '', False, [TypAbsencji, DataOd], Rodzina)
    if stat != "OK":
        return "Blad podmiany danych", [['', 0, 0]]

    try:
        rsglobal.ZapiszDoLogow("podzwolnienie tresc:" + tresc, 1)
        exec (tresc.replace(chr(13), ''))
    except:
        stat = 'Bd pobrania podzwolnienia'
        wart = [['', 0, 0]]
        return stat, wart

    if (len(wart) != 7) and (len(wart) != 9) and (len(wart) != 11):
        stat = 'Niepoprawny algorytm'
        wart = [['', 0, 0]]
        return stat, wart

    TypZwZakl = wart[0]
    TypZwZus = wart[1]
    TypSzpZakl = wart[2]
    TypSzpZus = wart[3]
    TypWypZakl = wart[4]
    TypWypZus = wart[5]
    DniPodzialu = wart[6]
    DniPoprzednie = 0
    DniPodzialuSzpital = 0
    ProcentSzpital = 0
    DniPodzialuWypadkowe = 0
    ProcentWypadkowe = 0
    if len(wart) > 7:
        DniPodzialuSzpital = wart[7]
        ProcentSzpital = wart[8]
    if len(wart) > 9:
        DniPodzialuWypadkowe = wart[9]
        ProcentWypadkowe = wart[10]
    #TypyZakl = [wart[0],wart[2],wart[4]]

    if not PrzelomRok and DataOd[5:] == '01-01':
        # sprawdzenie czy nie ma ciaglosci na przelomie roku
        SQL = "SELECT typ_absencji_id, od_kiedy "
        SQL += " FROM rs_kp_dane_osobowe_absencje WHERE firma_id='" + self.firma_id + "' "
        SQL += " AND pracownik_id='" + Pracownik + "'"
        SQL += " AND do_kiedy='" + str(int(DataOd[0:4])-1) + "-12-31'"
        SQL += " AND typ_absencji_id LIKE '" + TypAbsencji[0] + "%'"
        rsglobal.ZapiszDoLogow("przelom roku select:" + SQL, 1)
        stat, w, dump, dump = self.PolSQL.Execute(SQL, 3)
        if stat != 'OK':
            stat = 'Bd pobierania daty do sprawdzenia przeomu roku. ' + stat
            wart = [['', 0, 0]]
            return stat, wart
        elif len(w) == 1:  # jest ciaglosc na przelomie roku
            #if w[0][0] in TypyZakl:  # jesli poprzednie placone przez zaklad to zerujemy licznik dni
            PrzelomRok = True
            KontynuacjaData = w[0][1]
            PoprzedniTyp = w[0][0]

    PierwszeWRoku = False
    if not PrzelomRok:
        rsglobal.ZapiszDoLogow("if not PrzelomRok", 0)
        # obliczenie daty poczatku okresu, przewaznie pierwszy styczen,
        # chyba ze bylo zwolnienie na przelomie roku ktore mialo ciaglosc bez zadnego odstepu
        SQL = "SELECT od_kiedy, do_kiedy, dni_kalendarzowe, typ_absencji_id "
        SQL += " FROM rs_kp_dane_osobowe_absencje WHERE firma_id='" + self.firma_id + "' "
        SQL += " AND pracownik_id='" + Pracownik + "'"
        SQL += " AND (EXTRACT(YEAR FROM do_kiedy)=" + DataOd[0:4] + " OR do_kiedy ='" + str(int(DataOd[0:4])-1) + "-12-31')"
        SQL += " AND typ_absencji_id LIKE '" + TypAbsencji[0] + "%'"
        SQL += " ORDER BY do_kiedy;"
        rsglobal.ZapiszDoLogow("podzwolnienie select:" + SQL, 1)
        stat, Ciaglosc, dump, dump = self.PolSQL.Execute(SQL, 3)
        if stat != 'OK':
            stat = 'Bd pobierania dni do sprawdzenia ciagoci zwolnienia. ' + stat
            wart = [['', 0, 0]]
            return stat, wart
        else:
            rsglobal.ZapiszDoLogow("podzwolnienie1 wynik:" + str(Ciaglosc), 1)
            KontynuacjaData = ''
            DoKiedyPrzerwa = DataOd
            DniPoprzednie = 0
            PoczatekOkresu = DataOd[0:4] + '-01-01'
            #for w in Ciaglosc:
            #    if rs_anakonda_daty.DodajKilkaDni(w[1], 60) < DoKiedyPrzerwa:
            #        break
            #    KontynuacjaData = w[0]
            #    DoKiedyPrzerwa = w[0]
            #    DniPoprzednie += w[2]
            DDo = ''
            for w in Ciaglosc:
                if w[1][5:] == '12-31':  # ostatnie zwolnienie z poprzedniego roku
                    PrzelomRok = True
                elif PrzelomRok:
                    if w[0][5:] == '01-01':  # kontynuacja na przelomie roku
                        DDo = w[1]
                        PoprzedniTyp = w[3]
                        PoczatekOkresu = rs_anakonda_daty.DodajKilkaDni(DDo, 1)
                        PierwszeWRoku = True
                    elif DDo == '':
                        PrzelomRok = False
                        break
                    elif DDo != '':  # czy dalsza kontynuacja
                        PierwszeWRoku = False
                        if rs_anakonda_daty.DodajKilkaDni(DDo, 1) == w[0]:  # ciaglosc od poprzedniego roku - sprawdzamy dalej
                            DDo = w[1]
                            PoprzedniTyp = w[3]
                        else:
                            PrzelomRok = False
                            PoczatekOkresu = w[0]
                            break
                else:
                    PoczatekOkresu = w[0]
                    break

                KontynuacjaData = w[0]
                DoKiedyPrzerwa = w[0]
                DniPoprzednie += w[2]
            rsglobal.ZapiszDoLogow("podzwolnienie1 sumy:" + str(KontynuacjaData) + '|' + str(DoKiedyPrzerwa) + '|' + str(DniPoprzednie), 0)

    if PierwszeWRoku and PoczatekOkresu <= DataDo:
        PrzelomRok = False

    if not PrzelomRok:
        # sprawdzenie czy nie ma odstepu 60 dni czyli czy jest ciaglosc
        # ustawienie zmiennych
        SQL = "SELECT od_kiedy, do_kiedy, dni_kalendarzowe, typ_absencji_id "
        SQL += " FROM rs_kp_dane_osobowe_absencje WHERE firma_id='" + self.firma_id + "' "
        SQL += " AND pracownik_id='" + Pracownik + "'"
        SQL += " AND od_kiedy >='" + str(PoczatekOkresu) + "'"
        SQL += " AND typ_absencji_id LIKE '" + TypAbsencji[0] + "%'"
        SQL += " ORDER BY do_kiedy DESC;"
        rsglobal.ZapiszDoLogow("podzwolnienie select:" + SQL, 0)
        stat, Ciaglosc, dump, dump = self.PolSQL.Execute(SQL, 3)
        if stat != 'OK':
            stat = 'Bd pobierania dni do sprawdzenia ciagoci zwolnienia. ' + stat
            wart = [['', 0, 0]]
            return stat, wart
        else:
            rsglobal.ZapiszDoLogow("podzwolnienie2 wynik:" + str(Ciaglosc), 1)
            KontynuacjaData = ''
            DoKiedyPrzerwa = DataOd
            DniPoprzednie = 0
            for w in Ciaglosc:
                #if rs_anakonda_daty.DodajKilkaDni(w[1], 60) < DoKiedyPrzerwa:
                #    break
                KontynuacjaData = w[0]
                DoKiedyPrzerwa = w[0]
                DniPoprzednie += w[2]
            rsglobal.ZapiszDoLogow("podzwolnienie2 sumy:" + str(KontynuacjaData) + '|' + str(DoKiedyPrzerwa) + '|' + str(DniPoprzednie), 0)


    # wlasciwe obliczenia

    if Wypadkowe:
        if PrzelomRok and PoprzedniTyp == TypWypZus:  # przelom roku i w starym roku placil zus
            Dane[1] = TypWypZus
            Dane[9] = ProcentWypadkowe
            nowe.append(Dane)
        elif DniPoprzednie + DniZwolnienia <= DniPodzialuWypadkowe:  # cale placi zaklad
            Dane[1] = TypWypZakl
            Dane[9] = ProcentWypadkowe
            nowe.append(Dane)
        elif DniPoprzednie > DniPodzialuWypadkowe:  # calosc placi ZUS
            Dane[1] = TypWypZus
            Dane[9] = ProcentWypadkowe
            nowe.append(Dane)
        else:  # dzielimy
            w = []
            w.extend(Dane)
            w[1] = TypWypZakl
            w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd, DniPodzialuWypadkowe - DniPoprzednie - 1)
            w[9] = ProcentWypadkowe
            nowe.append(w)
            w = []
            w.extend(Dane)
            w[1] = TypWypZus
            w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd, DniPodzialuWypadkowe - DniPoprzednie)
            w[9] = ProcentWypadkowe
            nowe.append(w)

    elif PrzelomRok and PoprzedniTyp in [TypZwZus, TypSzpZus]:  # kontynuacja placenia przez ZUS
        rsglobal.ZapiszDoLogow('przelom roku dalej zus', 0)
        if Szpital > 0:
            w = []
            w.extend(Dane)
            w[1] = TypSzpZus
            w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd, Szpital - 1)
            w[9] = ProcentSzpital
            nowe.append(w)
            if Szpital < Dni:
                w = []
                w.extend(Dane)
                w[1] = TypZwZus
                w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd, Szpital)
                nowe.append(w)
        else:
            w = []
            w.extend(Dane)
            w[1] = TypZwZus
            nowe.append(w)
            rsglobal.ZapiszDoLogow("przelom roku:" + str(nowe), 0)

    elif DniPoprzednie + DniZwolnienia > DniPodzialu:
        if DniPoprzednie >= DniPodzialu:  # kontynuacja, nie dotyczy przelomu roku gdzie DniPoprzdenie = 0
            if Szpital > 0:
                rsglobal.ZapiszDoLogow( "Dni kontynuacja:" + str(DniPoprzednie) + '/' + str(DniZwolnienia) + '/' + str(DniPodzialu) + '/' + str(DniPodzialuSzpital) + '/', 0)
                rsglobal.ZapiszDoLogow( "Dni kontynuacja:" + str(DniZus) + '/' + str(DniZaklad) + '/' + str(DniSzpitalZus) + '/' + str( DniSzpitalZaklad) + '/', 0)
                if DataDo == rs_anakonda_daty.DodajKilkaDni(DataOd,Szpital-1):
                    # cale zwolnienie w szpitalu
                    if DniPoprzednie + DniZwolnienia <= DniPodzialuSzpital:
                        # cale zwolnienie w szpitalu
                        Dane[1] = TypSzpZus
                        Dane[9] = ProcentSzpital
                        nowe.append(Dane)
                    elif DniPoprzednie >= DniPodzialuSzpital:
                        Dane[1] = TypSzpZus
                        nowe.append(Dane)
                    else:
                        # czesc platna 80 czesc 70
                        w = []
                        w.extend(Dane)
                        w[1] = TypSzpZus
                        w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd,DniPodzialuSzpital-DniPoprzednie-1)
                        w[9] = ProcentSzpital
                        nowe.append(w)
                        w = []
                        w.extend(Dane)
                        w[1] = TypSzpZus
                        w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd,DniPodzialuSzpital-DniPoprzednie)
                        nowe.append(w)
                else:
                    # pierwsze dni w szpitalu reszta normalnie
                    if DniPoprzednie + Szpital <= DniPodzialuSzpital:
                        # cale zwolnienie 80
                        w = []
                        w.extend(Dane)
                        w[1] = TypSzpZus
                        w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd,Szpital-1)
                        w[9] = ProcentSzpital
                        nowe.append(w)
                        w = []
                        w.extend(Dane)
                        w[1] = TypZwZus
                        w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd,Szpital)
                        w[18] = 0
                        nowe.append(w)
                    elif DniPoprzednie >= DniPodzialuSzpital:
                        w = []
                        w.extend(Dane)
                        w[1] = TypSzpZus
                        w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd,Szpital-1)
                        nowe.append(w)
                        w = []
                        w.extend(Dane)
                        w[1] = TypZwZus
                        w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd,Szpital)
                        w[18] = 0
                        nowe.append(w)
                    else:
                        # czesc platna 80 czesc 70
                        w = []
                        w.extend(Dane)
                        w[1] = TypSzpZus  # chyba TypSzpZak ??
                        w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd,min(DniPodzialuSzpital-DniPoprzednie, Szpital) -1)
                        w[9] = ProcentSzpital
                        w[18] = min(DniPodzialuSzpital-DniPoprzednie, Szpital)
                        nowe.append(w)
                        w = []
                        w.extend(Dane)
                        w[1] = TypSzpZus
                        w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd,min(DniPodzialuSzpital-DniPoprzednie, Szpital))
                        w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd,Szpital -1)
                        w[18] = Szpital - min(DniPodzialuSzpital-DniPoprzednie, Szpital)
                        if w[18] > 0:
                            nowe.append(w)
                        w = []
                        w.extend(Dane)
                        w[1] = TypZwZus
                        w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd,Szpital)
                        w[18] = 0
                        nowe.append(w)
            else:
                # bez podzialu wszystko placi ZUS
                w = []
                w.extend(Dane)
                w[1] = TypZwZus
                nowe.append(w)

        else:
            # podzial na placone przez zaklad i zus w tym zwolnieniu
            #nowe = []
            DniZus = DniPoprzednie + DniZwolnienia - DniPodzialu
            DniZaklad = DniZwolnienia - DniZus
            DniSzpitalZus = 0
            DniSzpitalZaklad = 0
            if Szpital > 0:
                if DniZaklad >= Szpital:
                    DniSzpitalZaklad = Szpital
                else:
                    DniSzpitalZaklad = DniZaklad
                    DniSzpitalZus = Szpital - DniZaklad
            rsglobal.ZapiszDoLogow("Dni:"+str(DniPoprzednie)+'/'+str(DniZwolnienia)+'/'+str(DniPodzialu)+'/'+str(DniPodzialuSzpital)+'/',0)
            rsglobal.ZapiszDoLogow("Dni:"+str(DniZus)+'/'+str(DniZaklad)+'/'+str(DniSzpitalZus)+'/'+str(DniSzpitalZaklad)+'/',0)
            if DniZaklad > 0:
                if DniSzpitalZaklad == 0:
                    rsglobal.ZapiszDoLogow("Dni:1", 0)
                    w = []
                    w.extend(Dane)
                    w[1] = TypZwZakl
                    w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd,DniZaklad-1)
                    nowe.append(w)
                elif DniZaklad == DniSzpitalZaklad:
                    rsglobal.ZapiszDoLogow("Dni:2", 0)
                    w = []
                    w.extend(Dane)
                    w[1] = TypSzpZakl
                    w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd,DniZaklad-1)
                    nowe.append(w)
                else:
                    rsglobal.ZapiszDoLogow("Dni:3", 0)
                    w = []
                    w.extend(Dane)
                    w[1] = TypSzpZakl
                    w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd,DniSzpitalZaklad-1)
                    w[18] = DniSzpitalZaklad
                    nowe.append(w)
                    w = []
                    w.extend(Dane)
                    w[1] = TypZwZakl
                    w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd,DniSzpitalZaklad)
                    w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd,DniZaklad-1)
                    w[18] = 0
                    nowe.append(w)
            if DniZus > 0:
                if DniSzpitalZus == 0:
                    rsglobal.ZapiszDoLogow("Dni:4",1)
                    w = []
                    w.extend(Dane)
                    w[1] = TypZwZus
                    w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd,DniZaklad)
                    nowe.append(w)
                elif DniSzpitalZus == DniZus:
                    rsglobal.ZapiszDoLogow("Dni:5",1)
                    w = []
                    w.extend(Dane)
                    w[1] = TypSzpZus
                    w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd,DniZaklad)
                    w[9] = ProcentSzpital
                    w[18] = DniSzpitalZus
                    nowe.append(w)
                else:
                    rsglobal.ZapiszDoLogow("Dni:6",1)
                    w = []
                    w.extend(Dane)
                    w[1] = TypSzpZus
                    w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd,DniZaklad)
                    w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd,DniZaklad+DniSzpitalZus-1)
                    w[9] = ProcentSzpital
                    w[18] = DniSzpitalZus
                    nowe.append(w)
                    w = []
                    w.extend(Dane)
                    w[1] = TypZwZus
                    w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd,DniZaklad+DniSzpitalZus)
                    w[18] = 0
                    nowe.append(w)
    else:
        # bez podzialu wszystko placi zaklad
        if Szpital == 0:
            w = []
            w.extend(Dane)
            w[1] = TypZwZakl
            nowe.append(w)
        else:
            if DataDo == rs_anakonda_daty.DodajKilkaDni(DataOd,Szpital-1):
                w = []
                w.extend(Dane)
                w[1] = TypSzpZakl
                nowe.append(w)
            else:
                w = []
                w.extend(Dane)
                w[1] = TypSzpZakl
                w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd,Szpital-1)
                nowe.append(w)
                w = []
                w.extend(Dane)
                w[1] = TypZwZakl
                w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd,Szpital)
                w[18] = 0
                nowe.append(w)


    # pobranie procentu wynagrodzenia, kodu zus, wpisanie daty konynuacji
    i = 0
    for w in nowe:
        SQL ="SELECT proc_zwolnienia, kod_zus "
        SQL+=" FROM rs_kp_absencje_rodzaje WHERE firma_id='"+self.firma_id+"' "
        SQL+=" AND typ_absencji_id='"+w[1]+"';"
        stat,dane,dump,dump = self.PolSQL.Execute(SQL,3)
        if stat != 'OK':
            stat = 'Bd pobrania procentu wynagrodzenia '+stat
        elif len(dane) != 1:
            stat = 'Bd pobrania procentu wynagrodzenia '+stat
        else:
            if w[9] == 0:
                w[9] = dane[0][0]
            if w[6].strip() == '':
                w[6] = dane[0][1]
                #w[6] = 'ble'
            if i>0:
                # ewentualna kontynacja w nastepnych wierszach
                if KontynuacjaData in ['', None, 'None']:
                    w[10] = 'True'
                    w[17] = DataOd
            i += 1

    rsglobal.ZapiszDoLogow("podzwolnienie wynik ostateczny:" + str(nowe), 0)

    return stat, nowe

# -------------------------------------------------------------------------------

def PodzielZwolnienie(self, Dane):
    # dzieli zwolnienia w zaleznosci od tgo kto placi i szpitala
    # Dane:
    # pracownik_id, typ_absencji_id, data_od, data_do,data_dostarczenia,data_wystawienia,
    # COALESCE(kod_zus,''), dni_kalendarzowe, dni_robocze, procent_wyplaty, kontynuacja,godziny_robocze,
    # dni_zalegle, godziny_zalegle, dni_tegoroczne, godziny_tegoroczne, rodzina_id, kontynuacja_data, dni_szpital,
    # na zadanie, czy wypadkowe, kod (A,B,C,D,E)

    from funkcje import fs_kp_zmienne_placowe_pom as zm

    wart = ''
    # pobranie algorytmu
    SQL = "SELECT COALESCE(algorytm_rodzaj,''), czy_zwolnienie "
    SQL += " FROM rs_kp_absencje_rodzaje WHERE firma_id='" + self.firma_id + "' "
    SQL += " AND typ_absencji_id='" + Dane[1] + "';"
    stat, dane, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat != 'OK':
        return stat, [['', 0, 0]]
    if len(dane) == 0:
        return 'Brak definicji absencji', [['', 0, 0]]
    if not dane[0][1]:
        # obsluga pierwszego wywolania dla podzialu wstepnego absencji dotyczy tylko zwolnien
        return "nie zwolnienie", 0
    Algorytm = dane[0][0].strip()
    if Algorytm == '':
        wart = [[Dane[1], 9999, 0]]
        return "Brak algorytmu", wart

    Pracownik = Dane[0]
    TypAbsencji = Dane[1]
    DataOd = Dane[2]
    DataDo = Dane[3]
    #KodZus = Dane[6]
    #ProcWyn = Dane[9]
    #Kontynuacja = Dane[10]
    Rodzina = Dane[16]
    KontynuacjaData = Dane[17]
    Szpital = Dane[18]
    Wypadkowe = Dane[20]
    Kod = Dane[21]
    MaksLiczbaDni = 182
    DniZwolnienia = rsglobal.IleDniWZakresieDat(DataOd, DataDo)
    if Kod != '':
        if Kod in ['A', 'E']:
            pass
        elif Kod in ['B', 'D']:
            MaksLiczbaDni = 270
            Szpital = 0
            Wypadkowe = False
        else:
            return 'Kod ' + Kod + '- nie wiem co robic', wart


    PrzelomRok = False
    PoprzedniTyp = ''
    nowe = []
    if DataOd[0:4] != DataDo[0:4]:
        # zwolnienie na przelomie roku liczymy jako dwa odobne zwolnienia z uwzglednieniem limitow
        w = []
        w.extend(Dane)
        w[3] = DataOd[0:4] + '-12-31'
        w[18] = min(rsglobal.IleDniWZakresieDat(w[2], w[3]), Szpital)
        Szpital = max(Szpital - rsglobal.IleDniWZakresieDat(w[2], w[3]), 0)  # pozostalo ze szpitala
        stat, wynik = PodzielZwolnienie(self, w)  # wywolanie rekurencyjne dla konca roku
        if stat != 'OK':
            return stat, wynik
        nowe.extend(wynik)
        PrzelomRok = True
        PoprzedniTyp = nowe[len(nowe)-1][1]
        Dane[2] = DataDo[0:4] + '-01-01'
        DataOd = DataDo[0:4] + '-01-01'
        DniZwolnienia = rsglobal.IleDniWZakresieDat(DataOd, DataDo)
        rsglobal.ZapiszDoLogow('na nowy rok:'+str(Dane)+' poprzedni typ:'+PoprzedniTyp, 1)

    stat, Algorytm = zm.PodmienDanymi(self, Algorytm, Pracownik, '0', '0', '0', '', False, [TypAbsencji, DataOd], Rodzina)
    if stat != "OK":
        return "Blad podmiany danych", [['', 0, 0]]

    try:
        rsglobal.ZapiszDoLogow("podzwolnienie tresc algorytmu:" + Algorytm, 1)
        exec (Algorytm.replace(chr(13), ''))
    except:
        stat = 'Bd pobrania podzwolnienia'
        wart = [['', 0, 0]]
        return stat, wart

    if (len(wart) != 7) and (len(wart) != 9) and (len(wart) != 11):
        stat = 'Niepoprawny algorytm'
        wart = [['', 0, 0]]
        return stat, wart

    TypZwZakl = wart[0]  # 6A (B-6C) (D-6G)
    TypZwZus = wart[1]   # 6B (B-6D) (D-6H)
    TypSzpZakl = wart[2]  # 6S
    TypSzpZus = wart[3]   # 6T
    TypWypZakl = wart[4]  # 6W
    TypWypZus = wart[5]   # 6Z
    DniPodzialu = wart[6]
    DniPoprzednie = 0
    DniPodzialuSzpital = 0
    ProcentSzpital = 0
    DniPodzialuWypadkowe = 0
    ProcentWypadkowe = 0
    if len(wart) > 7:
        DniPodzialuSzpital = wart[7]
        ProcentSzpital = wart[8]
    if len(wart) > 9:
        DniPodzialuWypadkowe = wart[9]
        ProcentWypadkowe = wart[10]
    if Kod != '':
        if Kod == 'B':  # ciaza
            TypZwZakl = '6C'
            TypZwZus = '6D'
        elif Kod == 'D':  # gruzlica
            TypZwZakl = '6G'
            TypZwZus = '6H'

    PierwszeWRoku = False
    if not PrzelomRok and DataOd[5:] == '01-01':
        # sprawdzenie czy nie ma ciaglosci na przelomie roku
        SQL = "SELECT typ_absencji_id, od_kiedy, dni_kalendarzowe "
        SQL += " FROM rs_kp_dane_osobowe_absencje WHERE firma_id='" + self.firma_id + "' "
        SQL += " AND pracownik_id='" + Pracownik + "'"
        SQL += " AND do_kiedy='" + str(int(DataOd[0:4])-1) + "-12-31'"
        SQL += " AND typ_absencji_id LIKE '" + TypAbsencji[0] + "%'"
        rsglobal.ZapiszDoLogow("przelom roku select 01-01:" + SQL, 1)
        stat, w, dump, dump = self.PolSQL.Execute(SQL, 3)
        if stat != 'OK':
            stat = 'Bd pobierania daty do sprawdzenia przeomu roku. ' + stat
            wart = [['', 0, 0]]
            return stat, wart
        elif len(w) == 1:  # jest ciaglosc na przelomie roku
            #if w[0][0] in TypyZakl:  # jesli poprzednie placone przez zaklad to zerujemy licznik dni
            PrzelomRok = True
            #PierwszeWRoku = True
            KontynuacjaData = w[0][1]
            PoprzedniTyp = w[0][0]
            DniPoprzednie = w[0][2]
            DoKiedyPrzerwa = DataOd
            rsglobal.ZapiszDoLogow("podzwolnienie0 sumy:" + str(KontynuacjaData) + '|' + str(DoKiedyPrzerwa) + '|' + str(DniPoprzednie), 0)

    if not PrzelomRok:
        rsglobal.ZapiszDoLogow("if not PrzelomRok", 0)
        # obliczenie daty poczatku okresu, przewaznie pierwszy styczen,
        # chyba ze bylo zwolnienie na przelomie roku ktore mialo ciaglosc bez zadnego odstepu
        SQL = "SELECT od_kiedy, do_kiedy, dni_kalendarzowe, typ_absencji_id "
        SQL += " FROM rs_kp_dane_osobowe_absencje WHERE firma_id='" + self.firma_id + "' "
        SQL += " AND pracownik_id='" + Pracownik + "'"
        SQL += " AND (EXTRACT(YEAR FROM do_kiedy)=" + DataOd[0:4] + " OR do_kiedy ='" + str(int(DataOd[0:4])-1) + "-12-31')"
        SQL += " AND typ_absencji_id LIKE '" + TypAbsencji[0] + "%'"
        SQL += " ORDER BY do_kiedy;"
        rsglobal.ZapiszDoLogow("podzwolnienie1 select:" + SQL, 1)
        stat, Ciaglosc, dump, dump = self.PolSQL.Execute(SQL, 3)
        if stat != 'OK':
            stat = 'Bd pobierania dni do sprawdzenia ciagoci zwolnienia. ' + stat
            wart = [['', 0, 0]]
            return stat, wart
        else:
            rsglobal.ZapiszDoLogow("podzwolnienie1 wynik:" + str(Ciaglosc), 1)
            KontynuacjaData = ''
            DoKiedyPrzerwa = DataOd
            DniPoprzednie = 0
            PoczatekOkresu = DataOd[0:4] + '-01-01'
            DDo = ''
            for w in Ciaglosc:
                if w[1][5:] == '12-31':  # ostatnie zwolnienie z poprzedniego roku
                    PrzelomRok = True
                elif PrzelomRok:
                    if w[0][5:] == '01-01':  # kontynuacja na przelomie roku
                        DDo = w[1]
                        PoprzedniTyp = w[3]
                        PoczatekOkresu = rs_anakonda_daty.DodajKilkaDni(DDo, 1)
                        PierwszeWRoku = True
                    elif DDo == '':
                        PrzelomRok = False
                        break
                    elif DDo != '':  # czy dalsza kontynuacja
                        PierwszeWRoku = False
                        if rs_anakonda_daty.DodajKilkaDni(DDo, 1) == w[0]:  # ciaglosc od poprzedniego roku - sprawdzamy dalej
                            DDo = w[1]
                            PoprzedniTyp = w[3]
                        else:
                            PrzelomRok = False
                            PoczatekOkresu = w[0]
                            break
                else:
                    PoczatekOkresu = w[0]
                    break

                KontynuacjaData = w[0]
                DoKiedyPrzerwa = w[0]
                DniPoprzednie += w[2]
            rsglobal.ZapiszDoLogow("podzwolnienie1 sumy:" + str(KontynuacjaData) + '|' + str(DoKiedyPrzerwa) + '|' + str(DniPoprzednie), 0)

    if PierwszeWRoku and PoczatekOkresu <= DataDo:
        PrzelomRok = False

    if not PrzelomRok:
        # sprawdzenie czy nie ma odstepu 60 dni czyli czy jest ciaglosc
        # ustawienie zmiennych
        SQL = "SELECT od_kiedy, do_kiedy, dni_kalendarzowe, typ_absencji_id "
        SQL += " FROM rs_kp_dane_osobowe_absencje WHERE firma_id='" + self.firma_id + "' "
        SQL += " AND pracownik_id='" + Pracownik + "'"
        SQL += " AND od_kiedy >='" + str(PoczatekOkresu) + "'"
        SQL += " AND typ_absencji_id LIKE '" + TypAbsencji[0] + "%'"
        SQL += " ORDER BY do_kiedy DESC;"
        rsglobal.ZapiszDoLogow("podzwolnienie select:" + SQL, 0)
        stat, Ciaglosc, dump, dump = self.PolSQL.Execute(SQL, 3)
        if stat != 'OK':
            stat = 'Bd pobierania dni do sprawdzenia ciagoci zwolnienia. ' + stat
            wart = [['', 0, 0]]
            return stat, wart
        else:
            rsglobal.ZapiszDoLogow("podzwolnienie2 wynik:" + str(Ciaglosc), 1)
            KontynuacjaData = ''
            DoKiedyPrzerwa = DataOd
            DniPoprzednie = 0
            for w in Ciaglosc:
                #if rs_anakonda_daty.DodajKilkaDni(w[1], 60) < DoKiedyPrzerwa:
                #    break
                KontynuacjaData = w[0]
                DoKiedyPrzerwa = w[0]
                DniPoprzednie += w[2]
            rsglobal.ZapiszDoLogow("podzwolnienie2 sumy:" + str(KontynuacjaData) + '|' + str(DoKiedyPrzerwa) + '|' + str(DniPoprzednie), 0)


    # wlasciwe obliczenia

    if Wypadkowe:
        if PrzelomRok and PoprzedniTyp == TypWypZus:  # przelom roku i w starym roku placil zus
            Dane[1] = TypWypZus
            Dane[9] = ProcentWypadkowe
            nowe.append(Dane)
        elif DniPoprzednie + DniZwolnienia <= DniPodzialuWypadkowe:  # cale placi zaklad
            Dane[1] = TypWypZakl
            Dane[9] = ProcentWypadkowe
            nowe.append(Dane)
        elif DniPoprzednie > DniPodzialuWypadkowe:  # calosc placi ZUS
            Dane[1] = TypWypZus
            Dane[9] = ProcentWypadkowe
            nowe.append(Dane)
        else:  # dzielimy
            w = []
            w.extend(Dane)
            w[1] = TypWypZakl
            w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd, DniPodzialuWypadkowe - DniPoprzednie - 1)
            w[9] = ProcentWypadkowe
            nowe.append(w)
            w = []
            w.extend(Dane)
            w[1] = TypWypZus
            w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd, DniPodzialuWypadkowe - DniPoprzednie)
            w[9] = ProcentWypadkowe
            nowe.append(w)

    elif PrzelomRok and PoprzedniTyp in [TypZwZus, TypSzpZus]:  # kontynuacja placenia przez ZUS
        rsglobal.ZapiszDoLogow('przelom roku dalej zus', 0)
        if Szpital > 0:
            w = []
            w.extend(Dane)
            w[1] = TypSzpZus
            w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd, Szpital - 1)
            w[9] = ProcentSzpital
            nowe.append(w)
            if Szpital < Dni:
                w = []
                w.extend(Dane)
                w[1] = TypZwZus
                w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd, Szpital)
                nowe.append(w)
        else:
            w = []
            w.extend(Dane)
            w[1] = TypZwZus
            nowe.append(w)
            rsglobal.ZapiszDoLogow("przelom roku:" + str(nowe), 0)

    elif DniPoprzednie + DniZwolnienia > DniPodzialu:
        if DniPoprzednie >= DniPodzialu:  # kontynuacja, nie dotyczy przelomu roku gdzie DniPoprzdenie = 0
            if Szpital > 0:
                rsglobal.ZapiszDoLogow( "Dni kontynuacja:" + str(DniPoprzednie) + '/' + str(DniZwolnienia) + '/' + str(DniPodzialu) + '/' + str(DniPodzialuSzpital) + '/', 0)
                #rsglobal.ZapiszDoLogow( "Dni kontynuacja:" + str(DniZus) + '/' + str(DniZaklad) + '/' + str(DniSzpitalZus) + '/' + str( DniSzpitalZaklad) + '/', 0)
                if DataDo == rs_anakonda_daty.DodajKilkaDni(DataOd,Szpital-1):
                    # cale zwolnienie w szpitalu
                    if DniPoprzednie + DniZwolnienia <= DniPodzialuSzpital:
                        # cale zwolnienie w szpitalu
                        Dane[1] = TypSzpZus
                        Dane[9] = ProcentSzpital
                        nowe.append(Dane)
                    elif DniPoprzednie >= DniPodzialuSzpital:
                        Dane[1] = TypSzpZus
                        nowe.append(Dane)
                    else:
                        # czesc platna 80 czesc 70
                        w = []
                        w.extend(Dane)
                        w[1] = TypSzpZus
                        w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd,DniPodzialuSzpital-DniPoprzednie-1)
                        w[9] = ProcentSzpital
                        nowe.append(w)
                        w = []
                        w.extend(Dane)
                        w[1] = TypSzpZus
                        w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd,DniPodzialuSzpital-DniPoprzednie)
                        nowe.append(w)
                else:
                    # pierwsze dni w szpitalu reszta normalnie
                    if DniPoprzednie + Szpital <= DniPodzialuSzpital:
                        # cale zwolnienie 80
                        w = []
                        w.extend(Dane)
                        w[1] = TypSzpZus
                        w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd,Szpital-1)
                        w[9] = ProcentSzpital
                        nowe.append(w)
                        w = []
                        w.extend(Dane)
                        w[1] = TypZwZus
                        w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd,Szpital)
                        w[18] = 0
                        nowe.append(w)
                    elif DniPoprzednie >= DniPodzialuSzpital:
                        w = []
                        w.extend(Dane)
                        w[1] = TypSzpZus
                        w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd,Szpital-1)
                        nowe.append(w)
                        w = []
                        w.extend(Dane)
                        w[1] = TypZwZus
                        w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd,Szpital)
                        w[18] = 0
                        nowe.append(w)
                    else:
                        # czesc platna 80 czesc 70
                        w = []
                        w.extend(Dane)
                        w[1] = TypSzpZus  # chyba TypSzpZak ??
                        w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd,min(DniPodzialuSzpital-DniPoprzednie, Szpital) -1)
                        w[9] = ProcentSzpital
                        w[18] = min(DniPodzialuSzpital-DniPoprzednie, Szpital)
                        nowe.append(w)
                        w = []
                        w.extend(Dane)
                        w[1] = TypSzpZus
                        w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd,min(DniPodzialuSzpital-DniPoprzednie, Szpital))
                        w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd,Szpital -1)
                        w[18] = Szpital - min(DniPodzialuSzpital-DniPoprzednie, Szpital)
                        if w[18] > 0:
                            nowe.append(w)
                        w = []
                        w.extend(Dane)
                        w[1] = TypZwZus
                        w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd,Szpital)
                        w[18] = 0
                        nowe.append(w)
            else:
                # bez podzialu wszystko placi ZUS
                w = []
                w.extend(Dane)
                w[1] = TypZwZus
                nowe.append(w)

        else:
            # podzial na placone przez zaklad i zus w tym zwolnieniu
            #nowe = []
            DniZus = DniPoprzednie + DniZwolnienia - DniPodzialu
            DniZaklad = DniZwolnienia - DniZus
            DniSzpitalZus = 0
            DniSzpitalZaklad = 0
            if Szpital > 0:
                if DniZaklad >= Szpital:
                    DniSzpitalZaklad = Szpital
                else:
                    DniSzpitalZaklad = DniZaklad
                    DniSzpitalZus = Szpital - DniZaklad
            rsglobal.ZapiszDoLogow("Dni:"+str(DniPoprzednie)+'/'+str(DniZwolnienia)+'/'+str(DniPodzialu)+'/'+str(DniPodzialuSzpital)+'/',0)
            rsglobal.ZapiszDoLogow("Dni:"+str(DniZus)+'/'+str(DniZaklad)+'/'+str(DniSzpitalZus)+'/'+str(DniSzpitalZaklad)+'/',0)
            if DniZaklad > 0:
                if DniSzpitalZaklad == 0:
                    rsglobal.ZapiszDoLogow("Dni:1", 0)
                    w = []
                    w.extend(Dane)
                    w[1] = TypZwZakl
                    w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd,DniZaklad-1)
                    nowe.append(w)
                elif DniZaklad == DniSzpitalZaklad:
                    rsglobal.ZapiszDoLogow("Dni:2", 0)
                    w = []
                    w.extend(Dane)
                    w[1] = TypSzpZakl
                    w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd,DniZaklad-1)
                    nowe.append(w)
                else:
                    rsglobal.ZapiszDoLogow("Dni:3", 0)
                    w = []
                    w.extend(Dane)
                    w[1] = TypSzpZakl
                    w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd,DniSzpitalZaklad-1)
                    w[18] = DniSzpitalZaklad
                    nowe.append(w)
                    w = []
                    w.extend(Dane)
                    w[1] = TypZwZakl
                    w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd,DniSzpitalZaklad)
                    w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd,DniZaklad-1)
                    w[18] = 0
                    nowe.append(w)
            if DniZus > 0:
                if DniSzpitalZus == 0:
                    rsglobal.ZapiszDoLogow("Dni:4",1)
                    w = []
                    w.extend(Dane)
                    w[1] = TypZwZus
                    w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd,DniZaklad)
                    nowe.append(w)
                elif DniSzpitalZus == DniZus:
                    rsglobal.ZapiszDoLogow("Dni:5",1)
                    w = []
                    w.extend(Dane)
                    w[1] = TypSzpZus
                    w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd,DniZaklad)
                    w[9] = ProcentSzpital
                    w[18] = DniSzpitalZus
                    nowe.append(w)
                else:
                    rsglobal.ZapiszDoLogow("Dni:6",1)
                    w = []
                    w.extend(Dane)
                    w[1] = TypSzpZus
                    w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd,DniZaklad)
                    w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd,DniZaklad+DniSzpitalZus-1)
                    w[9] = ProcentSzpital
                    w[18] = DniSzpitalZus
                    nowe.append(w)
                    w = []
                    w.extend(Dane)
                    w[1] = TypZwZus
                    w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd,DniZaklad+DniSzpitalZus)
                    w[18] = 0
                    nowe.append(w)
    else:
        # bez podzialu wszystko placi zaklad
        if Szpital == 0:
            w = []
            w.extend(Dane)
            w[1] = TypZwZakl
            nowe.append(w)
        else:
            if DataDo == rs_anakonda_daty.DodajKilkaDni(DataOd,Szpital-1):
                w = []
                w.extend(Dane)
                w[1] = TypSzpZakl
                nowe.append(w)
            else:
                w = []
                w.extend(Dane)
                w[1] = TypSzpZakl
                w[3] = rs_anakonda_daty.DodajKilkaDni(DataOd,Szpital-1)
                nowe.append(w)
                w = []
                w.extend(Dane)
                w[1] = TypZwZakl
                w[2] = rs_anakonda_daty.DodajKilkaDni(DataOd,Szpital)
                w[18] = 0
                nowe.append(w)


    # pobranie procentu wynagrodzenia, kodu zus, wpisanie daty konynuacji
    i = 0
    for w in nowe:
        SQL ="SELECT proc_zwolnienia, kod_zus "
        SQL+=" FROM rs_kp_absencje_rodzaje WHERE firma_id='"+self.firma_id+"' "
        SQL+=" AND typ_absencji_id='"+w[1]+"';"
        stat,dane,dump,dump = self.PolSQL.Execute(SQL,3)
        if stat != 'OK':
            stat = 'Bd pobrania procentu wynagrodzenia '+stat
        elif len(dane) != 1:
            stat = 'Bd pobrania procentu wynagrodzenia '+stat
        else:
            if w[9] == 0:
                w[9] = dane[0][0]
            if w[6].strip() == '':
                w[6] = dane[0][1]
                #w[6] = 'ble'
            if i>0:
                # ewentualna kontynacja w nastepnych wierszach
                if KontynuacjaData in ['', None, 'None']:
                    w[10] = 'True'
                    w[17] = DataOd
            i += 1

    rsglobal.ZapiszDoLogow("podzwolnienie wynik ostateczny:" + str(nowe), 0)

    return stat, nowe

#-------------------------------------------------------------------------------

def PodajPodstaweZwolnienia(self, Data, Pracownik, Absencja, Kontynuacja, DataDo):
    # podaje podstawe z miesiaca lub zero

    rsglobal.ZapiszDoLogow('PodajPodstaweParametry:' + str(Data)+','+Pracownik+','+Absencja+','+str(Kontynuacja)+','+str(DataDo), 1)
    wart = [0, '', 0, 0, '']
    stat = 'OK'
    # sprawdzenie czy w angazach nie ma zmiany etatu
    SQL = "SELECT a.zatrudnienie_id, a.umowa_id, a.angaz_id, a.od_kiedy, a.etat  FROM rs_kp_angaze a"
    SQL += " INNER JOIN rs_kp_zatrudnienie z ON a.firma_id=z.firma_id AND a.pracownik_id = z.pracownik_id"
    SQL += " AND a.zatrudnienie_id=z.numer_id"
    SQL += " WHERE a.firma_id='"+self.firma_id+"' "
    SQL += " AND a.pracownik_id='"+Pracownik+"' AND z.czy_absencje AND a.od_kiedy<='"+str(Data)+"'"
    SQL += "  ORDER BY a.od_kiedy DESC;"
    stat, dane, dump, dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return stat, wart
    if len(dane) == 0:
        return 'Brak danych angau', wart
    poprz = dane[0]
    datazmianyetatu = ''
    for w in dane:
        if poprz[4] != w[4]:
            datazmianyetatu = poprz[3]
            break
        poprz = w
    if datazmianyetatu != '':
        # byla zmiana etatu, sprawdzamy czy mozna liczyc
        if datazmianyetatu[0:7] == Data[0:7]:
            wart[4] = 'Niemoliwe policzenie podstawy: zmiana etatu w miesicu zwolnienia. Dane naley wprowadzi w Kartotece osobowej / Absencje.'
            return stat, wart

    # sprawdzenie czy nie policzone
    Absencja = Absencja[0]
    # podstawa obowiazuje przez 1 miesiace od daty policzenia  #JR 2022-02-03 (12670)
    # czyli jesli byla policzona w lutym to obowiazuje do konca marca wlacznie
    # bez wzgldu na rodzaj absencji
    # BYLO:
    # podstawa obowiazuje przez 3 miesiace od daty policzenia
    # czyli jesli byla policzona w lutym to obowiazuje do konca maja wlacznie
    # chyba e kontynnuacja absencji, wtedy obowizuje dluzej
    odkad = rs_anakonda_daty.ZamienDate(str(Data))
    rok = odkad.year
    if odkad.month < 2:
        mc = 12
        rok += - 1
    else:
        mc = odkad.month - 1
    odkad = str(rok) + '-' + str(100+mc)[1:] + '-01'
    SQL = "SELECT lmcy, miesiace, podstawa, zasilek_dzien, procent_wyplaty "
    SQL += " FROM rs_kp_dane_osobowe_absencje ab"
    SQL += " INNER JOIN rs_kp_absencje_rodzaje ar USING(firma_id, typ_absencji_id)"
    SQL += " WHERE ab.firma_id='" + self.firma_id + "' "
    SQL += " AND pracownik_id='"+Pracownik+"' AND (czy_zwolnienie OR nieobecnosc_inna) and procent_wyplaty>0"
    if datazmianyetatu != '':
        SQL += " AND od_kiedy >= '" + str(datazmianyetatu) + "'"
    # if not Kontynuacja:
    #     SQL += " AND od_kiedy >= '" + str(odkad) + "'"
    SQL += " AND od_kiedy >= '" + str(odkad) + "'"
    SQL += "  ORDER BY od_kiedy DESC LIMIT 1"
    stat, dane, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat != 'OK':
        wart[4] = '1'
        return stat, wart
    if len(dane) > 0:
        return stat, [dane[0][0], dane[0][1], 0, dane[0][3], 'Podstawa dla numeru: '+Pracownik+' policzona wczeniej\n\n', dane[0][4]]


    # pobranie parametrow obliczen
    SQL ="SELECT algorytm "
    SQL+=" FROM rs_kp_skladniki_placowe_wersje WHERE firma_id='"+self.firma_id+"' "
    SQL+=" AND skladnik_id='RSPodstZUS' AND data_od<='"+str(Data)+"' ORDER BY data_od DESC LIMIT 1 ;"
    stat,dane,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        wart[4] = '2'
        return stat, wart
    if len(dane) == 0:
        wart[4] = '3'
        return 'Brak skadnika z parametrami', wart
    tresc = dane[0][0].strip()
    if tresc in ['','None','<None>']:
        wart[4] = '4'
        return 'Brak parametrw', wart
    wiersze = tresc.splitlines()
    szablony = ''
    powinien = ''
    pracowal = ''
    nieuzupelniac = ''
    pelne = ''
    nieuzupelnione_pelne = ''
    dni_zwolnienia = ''
    for w in wiersze:
        if w[0:8] == 'szablony':
            szablony = w[9:]
        if w[0:8] == 'powinien':
            powinien = w[9:]
        if w[0:8] == 'pracowal':
            pracowal = w[9:]
        if w[0:13] == 'nieuzupelniac':
            nieuzupelniac = w[14:]
        if w[0:5] == 'pelne':
            pelne = w[6:]
        if w[0:20] == 'nieuzupelnione_pelne':
            nieuzupelnione_pelne = w[21:]
        if w[0:14] == 'dni_zwolnienia':
            dni_zwolnienia = w[15:]

    SQL ="SELECT skladnik_id "
    SQL+=" FROM rs_kp_skladniki_placowe_wersje WHERE firma_id='"+self.firma_id+"' "
    SQL+=" AND do_podstawy_zasilku AND data_od<='"+str(Data)+"' ORDER BY skladnik_id ;"
    stat,dane,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        wart[4] = '5'
        return stat, wart
    if len(dane) == 0:
        wart[4] = '6'
        return 'Brak skadnikow z podstaw ZUS', wart
    dopodstawy="("
    for w in dane:
        dopodstawy += "'" + w[0].strip() + "',"
    dopodstawy = dopodstawy[:-1] + ')'

    # pobranie daty pocztku zatrudnienia
    SQL = "SELECT um.od_kiedy  FROM rs_kp_umowy_o_prace um"
    SQL += " INNER JOIN rs_kp_zatrudnienie z ON um.firma_id=z.firma_id AND um.pracownik_id = z.pracownik_id"
    SQL += " AND um.zatrudnienie_id=z.numer_id"
    SQL += " WHERE um.firma_id='"+self.firma_id+"' "
    SQL += " AND um.pracownik_id='"+Pracownik+"' AND z.czy_absencje AND z.stan_pracownika_id='A'"
    SQL += "  ORDER BY um.od_kiedy LIMIT 1;"
    stat, dane, dump, dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return stat, wart
    if len(dane) == 0:
        return 'Brak danych umowy', wart
    DataZatr = str(dane[0][0])
    if DataZatr[-3:] == "-01":  #JR nie pomijamy miesiaca z<atrudnienia
        DataZatr = '1990-01-01'

    odkad = rs_anakonda_daty.ZamienDate(str(Data))
    rok = odkad.year - 1
    mc = odkad.month
    odkad = str(rok)+'-'+str(100+mc)[1:]+'-01'
    if odkad < DataZatr:  #JR 13709 nie wliczamy miesiaca zatrudnienia jesli nie jest od 1-go
        rok = int(DataZatr[0:4])
        mc = int(DataZatr[5:7])
        if mc == 12:
            rok += 1
            mc = 1
        else:
            mc += 1
        odkad = str(rok)+'-'+str(100+mc)[1:]+'-01'

    SQL ="SELECT EXTRACT (YEAR FROM lp.data_od) AS rok, EXTRACT (MONTH FROM lp.data_od) AS mc, "
    SQL += " SUM (CASE WHEN lpos.skladnik_id IN " + powinien + " THEN lpos.wartosc ELSE 0.0 END) as powinien,"
    SQL += " SUM (CASE WHEN lpos.skladnik_id IN " + pracowal + " THEN lpos.wartosc ELSE 0.0 END) as pracowal,"
    SQL += " SUM (CASE WHEN lpos.skladnik_id IN " + dopodstawy + " THEN lpos.wartosc ELSE 0.0 END) as podstawa, "
    SQL += " SUM (CASE WHEN lpos.skladnik_id IN " + nieuzupelniac + " THEN lpos.wartosc ELSE 0.0 END) as nieuzupelniac, "
    SQL += " SUM (CASE WHEN lpos.skladnik_id IN " + pelne + " THEN lpos.wartosc ELSE 0.0 END) as pelne, "
    SQL += " SUM (CASE WHEN lpos.skladnik_id IN " + nieuzupelnione_pelne + " THEN lpos.wartosc ELSE 0.0 END) as nieuzupelnione_pelne, "
    SQL += " SUM (CASE WHEN lpos.skladnik_id IN " + dni_zwolnienia + "  THEN lpos.wartosc ELSE 0.0 END) as dni_zwolnienia "
    SQL += " FROM rs_kp_listy_plac_osoby_skladniki lpos INNER JOIN rs_kp_listy_plac lp USING(firma_id,numer_listy_id) "
    SQL += "  WHERE firma_id='"+self.firma_id+"' "
    SQL += " AND pracownik_id='"+Pracownik+"' AND lp.data_do<'"+str(Data)+"' AND lp.data_od>='"+str(odkad)+"'"
    SQL += " AND stan != 'W'"
    if datazmianyetatu != '':
        SQL += " AND lp.data_od >= '" + str(datazmianyetatu) + "'"
    if szablony != '':
        SQL += " AND lp.lista_id NOT IN "+szablony
    SQL += " GROUP BY rok,mc ORDER BY rok,mc ;"
    rsglobal.ZapiszDoLogow('do podstawy:'+SQL,1)
    stat,dane,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        wart[4] = '7'
        return stat, wart
    if len(dane) == 0:
        wart[4] = '8'
        return stat, wart
    SQL ="SELECT trim(imie), trim(nazwisko), data_urodzenia, an.od_kiedy "
    SQL+=" FROM rs_kp_dane_osobowe dos INNER JOIN rs_kp_angaze an "
    SQL+=" ON dos.firma_id=an.firma_id AND dos.pracownik_id=an.pracownik_id "
    SQL+="  WHERE dos.firma_id='" + self.firma_id + "' "
    SQL+=" AND dos.pracownik_id='" + Pracownik + "'"
    SQL+=" ORDER BY an.od_kiedy LIMIT 1"
    stat, imienaz, dump, dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        wart[4] = '9'
        return stat, wart
    if len(dane) == 0:
        wart[4] = '10'
        return stat, wart
    lmcy = 0.0
    sumapodstawy = 0.0
    mce = ''
    wiek = rs_anakonda_daty.OkresPomiedzy(str(imienaz[0][2]),Data)[0:4]
    tekst = 'Obliczenia podstawy dla: ' + Pracownik + ' - ' + imienaz[0][0] + ' ' + imienaz[0][1] + ' na dzie: ' + str(Data) + '\n'
    tekst += 'Wiek:'+str(int(wiek)) + ' Zatrudnienie od:' + str(imienaz[0][3]) + ' Zwolnienie za okres od:' + str(Data) + ' do:' + str(DataDo) + '\n'
    tekst += '---------------------------------------------------------\n'
    tekst += ' Rok-mc  Przeprac. Nominalne Do podstawy  Po uzupenieniu  \n'
    tekst += '---------------------------------------------------------\n'
    for w in dane:
        tekst += str(int(w[0]))+'-'+str(100+int(w[1]))[1:]
        if (w[2] <= 0) or (w[3] <= 0):
            # nie pracowal wcale
            tekst += '  Nie pracowa(a)\n'
            continue
        elif w[3] / w[2] < 0.5:
            # nie przepracowal polowy miesiaca
            tekst += '  Nie przepracowa(a) poowy miesica\n'
            continue
        #if w[3] <= w[2]:  #JR 2021-09-10 (12610)
        #    kwota = round((w[4] / w[3]) * w[2], 2)
        #else:
        #    kwota = w[4]
        elif w[3] < w[2]:  #JR 2021-09-10 (12610) i (12742)
            #kwota = (w[4] - w[5]) * (w[2] / w[3]) + w[5]
            #kwota = w[6] + w[5]
            kwota = w[8] * rsglobal.RSRound(w[6] / 30.0,2) + w[5] + w[7]
            kwota = rsglobal.RSRound(kwota * (1.0 - 0.1371),2)
            w[4] = rsglobal.RSRound(w[4] * (1.0 - 0.1371),2)
        else:
            kwota = rsglobal.RSRound(w[4] * (1.0 - 0.1371),2)
            w[4] = kwota
        sumapodstawy += kwota
        tekst += rjust(str(int(w[3])), 7) + rjust(str(int(w[2])), 10) + rjust(str(w[4]+0.0001)[:-2], 15) + rjust(str(round(kwota, 2) + 0.0001)[:-2], 17) + '\n'
        lmcy += 1.0
        mce += str(int(w[1])) + '/'
    if lmcy == 0:
        wart[4] = '11'
        return 'OK', wart
    tekst += '---------------------------------------------------------\n'
    tekst += 'Suma                                   '+rjust(str(round(sumapodstawy,2) + 0.0001)[:-2], 17)+'\n'
    sumapodstawy = rsglobal.RSRound(sumapodstawy / lmcy, 2) # podstawa miesieczna
    dzienpodstawy = rsglobal.RSRound(sumapodstawy / 30, 2)  # zasiek za dzien
    tekst += '---------------------------------------------------------\n'
    tekst += 'Podstawa: ' + str(sumapodstawy) + ', zasiek dzie: 100% - ' + str(dzienpodstawy) + ", 80% - " + str(rsglobal.RSRound(dzienpodstawy * 0.8, 2)) + ", 70% - " + str(rsglobal.RSRound(dzienpodstawy * 0.7, 2)) + '\n\n\n'
    rsglobal.ZapiszDoLogow('do podstawy:'+str([lmcy, mce, sumapodstawy, dzienpodstawy]),1)
    return 'OK', [lmcy, mce, sumapodstawy, dzienpodstawy, str(tekst)]



