# -*- coding: iso8859_2 -*-
#-------------------------------------------------------------------------------
# Name:        fs_kp_praca_biezaca_absencje_pozycje.py
# Purpose:     grid z pozycjami absencjami pracy biecej
# Author:      Piotr Witkowski
# Created:     2015/11/02
# RCS-ID:      $Id: fs_kp_praca_biezaca_absencje_pozycje.py $
# Copyright:   (c) 2016 Record System
# Licence:     GNU GPL
#-------------------------------------------------------------------------------

# 2015-12-10 - Piotr Witkowski   (9442) Podtabela danych osobowych - absencje ZMIANY
# 2016-02-06 - Piotr Witkowski   (9554) Okno Edycja absencji pracownika - poprawki
# 2018-09-03 - Jan Raburski - Przepisanie od poczatku
# 2018-09-18 Aleksander Bargie (11223) - [KP] czy_glowna w rs_kp_absencje_rodzaje
# 2018-10-22 Aleksander Bargie (11265) - Praca bieca absencje - dodatkowa kontrola daty od
# 2018-10-26 Aleksander Bargie (11270) - Dodatkowe pole w pozycjach absencji w pracy biecej
# 2018-11-06 Aleksander Bargie (11275) - Pozycje absencji w pracy biecej - sortowanie
# 2018-11-15 Aleksander Bargie (11285) - Praca bieca absencje - dodatkowa kontrola daty od - zmiana
# 2018-12-06 Aleksander Bargie (11313) - Absencje - kontrola na aktywny anga
# 2019-06-11 Wojciech Skowron   (11493) - Wczytywanie e-zwolnie lekarskich
# 2020-06-29 Aleksander Bargie (12029) - [KP] Obsuga urlopu na danie w pracy biecej
# 2020-02-13 Jan Raburski (12332)       - blokada niektywnych rodzajow absencji
# 2021-03-01 Aleksander Bargie (12358) - [KP] Do pozycji absencji doda 'wypadkowe'
# 2021-03-16 Aleksander Bargie (12395) - Doda trim na imi i nazwisko
# 2022-03-03 Aleksander Bargie (12913) - Praca bieca - zwolnienia - dodatkowe pole
# 2023-02-02 Wojciech Skowron   (13337) - Pokrywanie si dat absencji sprawdzane tylko z paczkami w stanie W
# 2023-08-28 Jan Raburski (13512) - przejcie w dniach roboczych na kalendarz

import binascii
import datetime

from funkcje import fs_kp_praca_biezaca_absencje_pom as abpom
from lxml import etree
from rsglobal import SprawdzCzyToData,SprPesel

import rs_anakonda_daty

#AB 10.12.18r
def SprawdzZatrudnienie(self, pracownik, data):
    sql = """SELECT a.od_kiedy, a.do_kiedy, *
    FROM rs_kp_zatrudnienie z
        JOIN rs_kp_umowy_o_prace u ON z.firma_id=u.firma_id AND z.pracownik_id=u.pracownik_id AND z.numer_id=u.zatrudnienie_id
        JOIN rs_kp_angaze        a ON z.firma_id=a.firma_id AND z.pracownik_id=a.pracownik_id AND z.numer_id=a.zatrudnienie_id AND u.umowa_id = a.umowa_id
    WHERE z.firma_id='{f}' AND z.pracownik_id='{p}' AND z.stan_pracownika_id='A' AND z.czy_absencje AND ('{d}' BETWEEN a.od_kiedy AND a.do_kiedy OR '{d}'>= a.od_kiedy AND a.do_kiedy IS NULL);"""
    sql = sql.format(f=self.firma_id, p=pracownik, d=data)
    rsglobal.ZapiszDoLogow(sql, 5)
    stat, dane, dump, dump = self.PolSQL.Execute(sql, 3)
    if stat != 'OK':
        stat = 'Bd sprawdzania zatrudnienia: ' + stat
    elif len(dane) == 0:
        stat = 'Pracownik '+pracownik+' nie posiada angau obejmujcego ' + data + ' w zatrudnieniu uwzgldniajcym absencje.'
    return stat

#AB 26.10.18r
# Zawarto self.Etykiety w \RSKPPracaBiezacaAbsencjePozycje.py, do znajdowania numerow kolumn
#AB 03.03.22r dodanie kolumny kod
Etykiety = ['kolejnosc', 'pracownik_id','nazwisko_imie','typ_absencji_id','absencja_opis','data_od','data_do',               #0-6
            'data_dostarczenia','data_wystawienia','kod_zus', 'kod',  'dni_szpital',                                         #7-11
            'procent_wyplaty','kontynuacja', 'kontynuacja_data', 'wypadkowe', 'rodzina_id','dni_kalendarzowe','dni_robocze', #12-18
            'godziny_robocze', 'na_zadanie', 'dni_zalegle','godziny_zalegle',                                                #19-22
            'dni_tegoroczne','godziny_tegoroczne', ]
# przerobka na slownik w celu wygodniejszego dostepu do danych
kolGrid = {}
for i in range(len(Etykiety)):
    kolGrid[Etykiety[i]] = i


puste = (None, '', '    -  -  ', 'None')
stat='OK'
if not 'Funkcja' in klucze:
    stat='Brak podanej funkcji w kluczach'

if stat=='OK':
    Funkcja = klucze['Funkcja']
#AB 10.12.18r
    SlPar={'Numer':[1, 5, 13], 'Data':[1, 5], 'Oddzial':[1, 5], 'Absencja':[10] , 'DataOd':[11, 12], 'DataDo':[11], 'Pracownik':[11, 12],
           #WS 2019-06-11
           'TrescXML':[13],
           #WS 2020-02-03
           'Tryb':[1, 14]
           }
     
    Lista=rsglobal.SprawdzParametryWKluczach(SlPar, Funkcja, klucze)
    if Lista<>'':
        stat='Uwaga : wywoanie funkcji fs_kp_praca_biezaca_absencje_pozycje z numerem ' +str(Funkcja)+' bez parametrw: '+Lista
    
    if stat=='OK': 
        if Funkcja in [1, 5, 13]:
            Numer = str(klucze['Numer'])
            Data = str(klucze['Data']) 
            Oddzial = str(klucze['Oddzial'])
        if Funkcja in [10]:             
            Absencja = str(klucze['Absencja'])
        if Funkcja in [11]:
            DataOd = str(klucze['DataOd'])
            DataDo = str(klucze['DataDo'])
            Pracownik = str(klucze['Pracownik'])
#AB 10.12.18r
        if Funkcja in [12]:
            DataOd = str(klucze['DataOd'])
            Pracownik = str(klucze['Pracownik'])
        #WS 2019-06-11
        if Funkcja in [13]:
            TrescXML = str(klucze['TrescXML'])
        #WS 2020-02-03
        if Funkcja in [1, 14]:
            Tryb = klucze['Tryb']

dane = []
wyn1=[]
wyn2=[]

#-------------------------------------------------------------------------------
#---  OPIS ABSENCJI (10)
#-------------------------------------------------------------------------------

if Funkcja == 10 and stat=="OK":
    sql ="SELECT proc_zwolnienia, kod_zus FROM rs_kp_absencje_rodzaje"
    sql+=" WHERE firma_id='"+self.firma_id+"' AND typ_absencji_id='"+Absencja+"';"

    stat, dane, dump, dump = self.PolSQL.Execute(sql, 3)
    if stat != 'OK':
        stat = 'Bd pobrania opisu absencji: '+stat

#-------------------------------------------------------------------------------
#--- DNI WEDUG KALENDARZA PRACOWNIKA (11)
#-------------------------------------------------------------------------------

if Funkcja == 11 and stat == "OK":
    stat, dane = abpom.LiczDniIGodzinyRobocze(self, DataOd, DataDo, Pracownik)


#AB 10.12.18r
#-------------------------------------------------------------------------------
#---   Sprawdzenie czy absencje mog wystpowa w zatrudnieniach pracownika z daty
#-------------------------------------------------------------------------------

if Funkcja == 12 and stat=="OK":
    stat = SprawdzZatrudnienie(self, Pracownik, DataOd)

#WS 2020-02-03
#-------------------------------------------------------------------------------
#---   Pobranie typw absencji zgodnych z trybem:
#---   Urlopy (1) - czy_nieobecnosc=TRUE AND czy_glowna=TRUE
#---   Zwolnienia (2) - (czy_zwolnienie=TRUE OR nieobecnosc_inna=TRUE) AND czy_glowna=TRUE
#-------------------------------------------------------------------------------

if Funkcja == 14 and stat == "OK":
    SQL = """SELECT typ_absencji_id FROM rs_kp_absencje_rodzaje 
             WHERE"""

    WHERE = """ firma_id='{firma_id}' AND"""
    WHERE = WHERE.format(firma_id=self.firma_id)
    if Tryb == 1:
        WHERE += " czy_nieobecnosc=TRUE AND czy_glowna=TRUE"
    else:
        WHERE += " (czy_zwolnienie=TRUE OR nieobecnosc_inna=TRUE) AND czy_glowna=TRUE"
    WHERE += " AND czy_aktywna;"     #JR 21-02-13


    stat, dane, dump, dump = self.PolSQL.Execute(SQL+WHERE, 3, CzyDodawacBlad=True)
    if stat != 'OK':
        stat = "Bd pobierania typw absencji: "+stat

#-------------------------------------------------------------------------------
#----------   UPRAWNIENIA
#-------------------------------------------------------------------------------

UprModyfikowanie=True

UprIns=UprModyfikowanie
UprUpd=UprModyfikowanie
UprDel=UprModyfikowanie

Insert = []
Update = [] 
Delete = []

if stat=='OK' and Funkcja == 1:
    Update=klucze['Update']
    Insert=klucze['Insert']
    Delete=klucze['Delete']

#-------------------------------------------------------------------------------
#---   DELETE (1)
#-------------------------------------------------------------------------------

if Funkcja == 1 and stat=="OK":
    if len(Delete[0])>0:
        if not UprDel:
            stat='UPR BR DEL'
            dane.append(stat)
        # sprawdzanie DELETE 
        if stat == 'OK':
            for i in xrange(len(Delete[0])):
                DELETE = "DELETE FROM rs_kp_absencje_pozycje "
                WHERE = "WHERE firma_id='"+self.firma_id+"' AND numer='"+str(Numer) 
                WHERE+= "' AND data='"+str(Data)+"' AND oddzial_id='"+str(Oddzial)+"'"

                for j in xrange(len(Delete[0][i])):
                    pole = Delete[1][j]
                    wart = Delete[0][i][j]
                    if pole in ['pracownik_id', 'typ_absencji_id', 'data_od']:
                        if wart<>'':
                            if stat=='OK':
                                WHERE+=" AND "+str(pole)+"='"+str(wart)+"'"
                WHERE+=";"
                if stat == 'OK':
                    stat, dump, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(DELETE+WHERE), 1)
                    if stat != 'OK':
                        stat = "Bd przy usuwaniu danych: "+stat
                        dane.append(stat)


#-------------------------------------------------------------------------------
#---   UPDATE (1)
#-------------------------------------------------------------------------------

if Funkcja == 1 and stat=="OK":
    SprawdzenieBlad=u'Nie mona zaktualizowa danych - istnieje ju absencja o podanym kluczu: '
    Uwagi=['pracownik', 'typ', 'data od']
    if len(Update[0])>0:
        if not UprUpd:
            stat='UPR BR UPD'
            dane.append(stat)
            
        if stat == 'OK':
            rsglobal.ZapiszDoLogow(Update, 5)
            WierszeNr = Update[-1]
            for i in xrange(len(Update[0])):
                CzyKlucz=False
                Sprawdzac = True
                WHERE=" WHERE firma_id='"+self.firma_id+"' AND oddzial_id='"+str(Oddzial)
                WHERE+="' AND data='"+str(Data)+"' AND numer='"+str(Numer)+"'"
                pracownik_id = ''
                typ = ''
                rok = ''
                Sprawdzenie="SELECT pracownik_id, data_od FROM rs_kp_absencje_pozycje WHERE firma_id='"+self.firma_id+"' "
                for j in xrange(len(Update[2][i])): # petla po kluczu
                    pole=Update[3][j]
                    wart=rsglobal.Dekoduj(Update[2][i][j])
                    if pole =='pracownik_id':
                        wart=('000000' + str(wart))[-6:]
                        WHERE+=" AND "+pole+"='"+wart+"'"
                        pracownik_id = wart
                        SPR1=" AND "+pole+"='"+wart+"'"
                        opis = "SELECT ' '||pracownik_id::int||'-'|| nazwisko ||' '|| imie FROM rs_kp_dane_osobowe "
                        opis += "WHERE firma_id='" + str(self.firma_id) + "' AND pracownik_id ='" + wart.strip() + "';"
                        stat, s, wyn1, dump = self.PolSQL.Execute(opis, 3)
                        if stat=='OK' and len(s)>0:
                            Uwagi[0]=s[0][0]

                    if pole =='typ_absencji_id':
                        WHERE+=" AND "+pole+"='"+wart+"'"
                        SPR2=" AND "+pole+"='"+wart+"'"
                        Uwagi[1]=u' typ absencji-'+wart
                        typ = wart
                    if pole =='data_od':
                        rok = wart[:4]
                        #AB 10.12.18r
                        data_od = wart
                        WHERE+=" AND "+pole+"='"+wart+"'"
                        SPR3=" AND "+pole+"='"+wart+"'"
                        Uwagi[2]=u' od kiedy-'+wart

                data_do=''
                dkal = None                                                     #AB 15.11.18r
                spr ="SELECT zus_wymagany, dostarczono, wystawiono, czy_glowna, czy_nieobecnosc=FALSE FROM rs_kp_absencje_rodzaje "
                spr+="WHERE firma_id='"+str(self.firma_id)+"' AND typ_absencji_id = (SELECT typ_absencji_id FROM rs_kp_absencje_pozycje " + WHERE + ");"
                stat, wymagania, wyn1, dump = self.PolSQL.Execute(spr, 3)
                if stat != 'OK' or len(wymagania) == 0:
                    stat = 'Bd odczytu wymaga dla wybranego typu absencji.'

                for j in xrange(len(Update[0][i])):
                    pole = Update[0][i][j]
                    wart = Update[1][i][j]

                    if pole == 'pracownik_id':
                        wart=('000000' + str(wart))[-6:]
                        #WHERE+=" AND "+pole+"='"+wart+"'"
                        opis = "SELECT ' '||pracownik_id::int||'-'|| nazwisko ||' '|| imie FROM rs_kp_dane_osobowe "
                        opis += "WHERE firma_id='" + str(self.firma_id) + "' AND pracownik_id ='" + wart.strip() + "';"
                        stat, s, wyn1, dump = self.PolSQL.Execute(opis, 3)
                        if stat=='OK' and len(s)>0:
                            Uwagi[0]=s[0][0]
                        CzyKlucz=True
                        SPR1=" AND "+pole+"='"+wart+"'"
                        if wart.strip()=='':
                            dane.append('PRACOWNIK BR')
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                        #elif len(wart.strip())>6:
                        #    dane.append('PRACOWNIK DL')
                        #    wyn2.append((WierszeNr[i], 0))
                        else:
                            spr ="SELECT 1 FROM rs_kp_dane_osobowe "
                            spr+="WHERE firma_id='"+str(self.firma_id)+"' AND pracownik_id ='"+wart.strip()+"';"
                            stat, s, wyn1, dump=self.PolSQL.Execute(spr, 3)
                            if len(s)==0:
                            #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('PRACOWNIK BL')
                            #AB 10.12.18r
                            else:
                                pracownik_id = wart.strip()

                    elif pole == 'typ_absencji_id':
                        CzyKlucz=True
                        SPR2=" AND "+pole+"='"+wart+"'"
                        Uwagi[1]=u' typ absencji-'+wart
                        if len(wart.strip())==0:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('RODZAJ BR')
                        else: # TUTAJ POBIERA SI INFORMACJE O WYMAGANYCH POLACH DLA DANEGO TYPU ABSENCJI, JELI KIEDY
                                # Z JAKIEGO POWODU KOLEJNO KOLUMN ULEGNIE ZMIANIE, TO SPRAWDZENIE PRZESTANIE DZIAA.
                                                                                            #AB 15.11.18r          #WS 2020-02-03   #WS 2020-02-03  #WS 2020-02-03
                            spr ="SELECT zus_wymagany, dostarczono, wystawiono, czy_glowna, czy_nieobecnosc=FALSE, czy_nieobecnosc, czy_zwolnienie, nieobecnosc_inna FROM rs_kp_absencje_rodzaje "
                            spr+="WHERE firma_id='"+str(self.firma_id)+"' AND typ_absencji_id = '"+wart.strip()+"';"
                            stat, wymagania, wyn1, dump=self.PolSQL.Execute(spr, 3)
                            rsglobal.ZapiszDoLogow(spr, 5)
                            rsglobal.ZapiszDoLogow(wymagania, 5)
                            if len(wymagania)==0:
                               #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('RODZAJ BL')

                            #AB 18.09.18r
                            elif not wymagania[0][3]:
                                  #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('GLOWNY BL')

                            #WS 2020-02-03
                            # Sprawdzenie czy typ absencji prawidowy:
                            # Urlopy - czy_nieobecnosc=TRUE AND czy_glowna=TRUE
                            # Zwolnienia - (czy_zwolnienie=TRUE OR nieobecnosc_inna=TRUE) AND czy_glowna=TRUE
                            elif Tryb == 1 and not wymagania[0][5]:
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('RODZAJ URLOP BL')
                            elif Tryb == 2 and not wymagania[0][6] and not wymagania[0][7]:
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('RODZAJ ZWOLNIENIE BL')

                    elif pole=='data_od':
                        CzyKlucz=True
                        SPR3=" AND "+pole+"='"+wart+"'"
                        Uwagi[2]=u' od kiedy-'+wart
                        if  wart not in (None, 'None') and wart.strip()<>'':
                            if not rsglobal.SprawdzCzyToData(wart.strip()):
                            #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('DATA OD BL1')
                            else:
                                Rok = int(wart[0:4])
                                Mies = int(wart[5:7])
                                Dzien = int(wart[8:10])
                                #AB 15.11.18r
                                if not wymagania[0][4] and not rs_anakonda_daty.CzyRoboczyWgKalendarza(self, pracownik_id, datetime.date(Rok, Mies, Dzien)):
                                     #AB 26.10.18r
                                    wyn2.append((WierszeNr[i], kolGrid[pole]))
                                    dane.append('DATA OD BL ROB')
                                else:
                                    data_od=wart.strip()
                        else:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('DATA OD BR')

                    elif pole=='data_do':
                        if wart not in (None, 'None') and wart.strip()<>'':
                            if not rsglobal.SprawdzCzyToData(wart.strip()):
                                #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('DATA DO BL')
                            if data_od != '':
                                if wart.strip()<data_od:
                                   #AB 26.10.18r
                                    wyn2.append((WierszeNr[i], kolGrid[pole]))
                                    dane.append('DATA DO BL3')
                            else:#AB 15.11.18r
                                Rok = int(wart[0:4])
                                Mies = int(wart[5:7])
                                Dzien = int(wart[8:10])
                                if not wymagania[0][4] and not rs_anakonda_daty.CzyRoboczyWgKalendarza(self, pracownik_id, datetime.date(Rok, Mies, Dzien)):
                                    wyn2.append((WierszeNr[i], kolGrid[pole]))
                                    dane.append('DATA DO BL ROB')
                                else:
                                    data_do=wart.strip()

                        else:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('DATA DO BR')

                    elif pole=='data_dostarczenia':     
                        if wart not in (None, 'None') and wart.strip()<>'':
                            if not rsglobal.SprawdzCzyToData(wart.strip()):
                                #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('DATA DOST BL')

                    elif pole=='data_wystawienia':     
                        if wart not in (None, 'None') and wart.strip()<>'':
                            if not rsglobal.SprawdzCzyToData(wart.strip()):
                              #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('DATA WYST BL')

                    elif pole=='kod_zus':
                        if len(wart) >7:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('ZUS BL')

                    elif pole=='kod':  #AB 03.03.22r
                        if wart not in ('A', 'B', 'C', 'D', 'E', ''):
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('KOD BL')

                    elif pole =='dni_kalendarzowe':
                        dkal = int(wart)
                        if int(wart) < 0:
                            #AB 18.09.18r
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('KAL BL')
                        elif wart == 0:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid['data_od']))
                            wyn2.append((WierszeNr[i], kolGrid['data_do']))
                            dane.append('KAL BR')
                            
                    elif pole =='dni_robocze':
                        if dkal is not None and int(wart) > dkal:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('ROBOCZE BL')
                        elif wart == 0:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('ROBOCZE BR')

                    #AB 26.10.18r
                    elif pole =='dni_szpital':
                        if dkal is not None and int(wart) > dkal:
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('SZPITAL BL')


                    elif pole =='procent_wyplaty':
                        if len(str(wart))>6:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('PROCENT BL')

                    elif pole =='godziny_robocze':
                        if len(str(int(wart)))>4:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('ROBOCZE GODZ BL')

                            #AB 18.09.18r
                    elif pole == 'rodzina_id' and wart != '':
                        try:
                            wart = int(float(wart))

                            SPRRODZ = "SELECT * FROM rs_kp_rodzina "
                            SPRRODZ += " WHERE firma_id = '{firma}' AND pracownik_id = '{prac}' AND rodzina_id={rodz}"
                            SPRRODZ = SPRRODZ.format(firma=self.firma_id, prac=pracownik_id, rodz=wart)
                            stat, rodz, dump, dump = self.PolSQL.Execute(SPRRODZ, 3)
                            if stat == 'OK' and len(rodz) == 0:
                                 #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('RODZ_BL')
                        except ValueError:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('RODZ_BL')
                if stat == 'OK' and wymagania[0][0]:
                    if 'kod_zus' in Update[0][i]:
                        if len(Update[1][i][Update[0][i].index('kod_zus')].strip()) == 0:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('ZUS BR')
                    else:
                        stat, kod, dump, dump = self.PolSQL.Execute('select kod_zus from rs_kp_absencje_pozycje ' + WHERE, 3)
                        if stat != 'OK':
                            stat = 'Bd sprawdzania kodu ZUS przed aktualizacj: ' + stat
                        elif kod[0][0].strip() in puste:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('ZUS BR')

                if stat == 'OK' and wymagania[0][1]:
                    if 'data_dostarczenia' in Update[0][i]:
                        if len(Update[1][i][Update[0][i].index('data_dostarczenia')]) in puste:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('DATA DOST BR')
                    else:
                        stat, kod, dump, dump = self.PolSQL.Execute('select data_dostarczenia from rs_kp_absencje_pozycje ' + WHERE, 3)
                        if stat != 'OK':
                            stat = 'Bd sprawdzania daty dostarczenia przed aktualizacj: ' + stat
                        elif kod[0][0].strip() in puste:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('DATA DOST BR')

                if stat == 'OK' and wymagania[0][2]:
                    if 'data_wystawienia' in Update[0][i]:
                        if len(Update[1][i][Update[0][i].index('data_wystawienia')]) in puste:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('DATA WYST BR')
                    else:
                        stat, kod, dump, dump = self.PolSQL.Execute('select data_wystawienia from rs_kp_absencje_pozycje ' + WHERE, 3)
                        if stat != 'OK':
                            stat = 'Bd sprawdzania daty wystawienia przed aktualizacj: ' + stat
                        elif kod[0][0].strip() in puste:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('DATA WYST BR')

                if stat == 'OK' and CzyKlucz and Sprawdzac:
                    Sprawdzenie+=SPR1+SPR2+SPR3
                    stat, s, wyn1, dump = self.PolSQL.Execute(Sprawdzenie, 3)
                    if len(s) > 0:
                        wyn2.append((WierszeNr[i], 0))
                        dane.append(SprawdzenieBlad+Uwagi[0]+Uwagi[1]+Uwagi[2])


                if stat == 'OK':
                    if len(dane) > 0:
                        stat = dane[0]

                #AB 10.12.18r
                if stat == 'OK':
                    stat = SprawdzZatrudnienie(self, pracownik_id, data_od)

                # wykonanie UPDATE
                if stat=='OK':
                    sql=''
                    UPDATE="UPDATE rs_kp_absencje_pozycje "
                    SET='SET '
                    for j in xrange(len(Update[0][i])): 
                        pole=Update[0][i][j]
                        wart=Update[1][i][j]
                        #
                        # OLD = "SELECT dni_robocze, godziny_robocze, dni_zalegle, godziny_zalegle, dni_tegoroczne, "
                        # OLD += "godziny_tegoroczne FROM rs_kp_absencje_pozycje " + WHERE
                        # stat, old, dump, dump = self.PolSQL.Execute(OLD, 3)
                        # rsglobal.ZapiszDoLogow('OLDOLDOLDOLDOLDOLDOLDOLDOLDOLDOLDOLDOLDOLDOLDOLDOLDOLD', 5)
                        # rsglobal.ZapiszDoLogow(str(OLD), 5)
                        # rsglobal.ZapiszDoLogow(str(old), 5)
                        # rsglobal.ZapiszDoLogow(str(Update[1][i]), 5)
                        # rsglobal.ZapiszDoLogow(str(Update[0][i]), 5)
                        # if stat != 'OK':
                        #     stat = 'Bd pobierania danych absencji. ' + stat
                        # else:
                        #
                        #     dni_rob = Update[1][i][Update[0][i].index('dni_robocze')] if 'dni_robocze' in Update[0][i] else old[0][0]
                        #     godz_rob = Update[1][i][Update[0][i].index('godziny_robocze')] if 'godziny_robocze' in Update[0][i] else old[0][1]
                            # dni_zal = Update[1][i][Update[0][i].index('dni_zalegle')] if 'dni_zalegle' in Update[0][i] else old[0][2]
                            # godz_zal = Update[1][i][Update[0][i].index('godziny_zalegle')] if 'godziny_zalegle' in Update[0][i] else old[0][3]
                            # dni_tegorocz = Update[1][i][Update[0][i].index('dni_tegoroczne')] if 'dni_tegoroczne' in Update[0][i] else old[0][4]
                            # godz_tegorocz = Update[1][i][Update[0][i].index('godziny_tegoroczne')] if 'godziny_tegoroczne' in Update[0][i] else old[0][5]


                            # SPRURL = "select sum(CASE WHEN rok < " + rok + " THEN 0 ELSE pozostalo_dni END) AS dni_tegoroczne, "
                            # SPRURL += "sum(CASE WHEN rok < " + rok + " THEN 0 ELSE pozostalo_godziny END) AS godziny_tegoroczne, "
                            # SPRURL += "sum(CASE WHEN rok = " + rok + " THEN 0 ELSE pozostalo_dni END) AS dni_zalegle, "
                            # SPRURL += "sum(CASE WHEN rok = " + rok + " THEN 0 ELSE pozostalo_godziny END) AS godziny_zalegle "
                            # SPRURL += "FROM rs_kp_dane_osobowe_limity_urlopow "
                            # SPRURL += "WHERE firma_id = '" + self.firma_id + "' AND pracownik_id = '" + pracownik_id + "' "
                            # SPRURL += "AND urlop_id = '" + typ + "';"
                            # stat, url, dump, dump = self.PolSQL.Execute(SPRURL, 3)
                            # rsglobal.ZapiszDoLogow('AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAABBBBBBBBB', 5)
                            # rsglobal.ZapiszDoLogow(str(SPRURL), 5)
                            # rsglobal.ZapiszDoLogow(str(url), 5)
                            #
                            # if stat != 'OK':
                            #     stat = 'Bd sprawdzania zalegych urlopw. ' + stat
                            # elif url[0][0] == 0 and url[0][1] == 0 and url[0][2] == 0 and url[0][3] == 0:
                            #     stat = 'Brak pozostaych limitw urlopw dla pracownika: ' + pracownik_id + ', typ absencji: '
                            #     stat += typ
                            # else:
                            #
                            #     d_tegor = int(url[0][0])
                            #     g_tegor = int(url[0][1])
                            #     d_zal = int(url[0][2])
                            #     g_zal = int(url[0][3])
                            #     upddzal = 0
                            #     upddtegor = 0
                            #     updgzal = 0
                            #     updgtegor = 0
                            #     rsglobal.ZapiszDoLogow('Update', 5)
                            #     rsglobal.ZapiszDoLogow(str(Update), 5)
                            #     rsglobal.ZapiszDoLogow(50 * ' LOCALS ', 5)
                            #     rsglobal.ZapiszDoLogow(str(locals()), 5)
                            #     rsglobal.ZapiszDoLogow(50 * ' GLOBALS ', 5)
                            #     rsglobal.ZapiszDoLogow(str(globals()), 5)
                            #
                            #
                            #     upddzal = min(dni_rob, d_zal)
                            #     upddtegor = dni_rob - upddzal
                            #     updgzal = min(godz_rob, g_zal)
                            #     updgtegor = godz_rob - updgzal

                        #\/TO BYO WCITE WEWNTRZ ELSE W ZAHASZOWANYM KAWAKU POWYEJ \/
                        if pole =='pracownik_id':
                            wart = ('000000' + str(wart))[-6:]
                            SET+=pole+"='"+str(wart)+"', "
                        elif pole in ['data_od', 'typ_absencji_id', 'data_do',
                                    'procent_wyplaty', 'kontynuacja', 'wypadkowe']: #AB 01.03.21r
                            SET+=pole+"='"+str(wart)+"', "                              #AB 18.09.18r
                        elif pole in ['data_dostarczenia', 'data_wystawienia', 'kod_zus', 'kontynuacja_data', 'kod']:  #AB 03.03.22r
                            if str(wart).strip() in puste:
                                SET+=pole+"=NULL, "
                            else:
                                SET+=pole+"='"+str(wart)+"', "
                                #AB 18.09.18r
                        elif pole in ['rodzina_id']:
                            if wart.strip() in ('0', '0.0', ''):
                                SET += pole + "=NULL, "
                            else:
                                wart = int(float(wart))
                                SET+=pole+"="+str(wart)+", "

                        elif pole in ['dni_kalendarzowe', 'dni_robocze',
                                     #AB 26.10.18r
                                    'godziny_robocze', 'dni_szpital']:
                            SET+=pole+"='"+str(int(wart))+"', "
                        #AB 29.06.20r
                        elif pole in ['na_zadanie']:
                            SET+=pole+"="+str(bool(wart))+", "

                        #/\TO BYO WCITE WEWNTRZ ELSE W ZAHASZOWANYM KAWAKU POWYEJ /\

                            # if (upddtegor + upddzal > int(d_tegor + d_zal)) or (updgtegor + updgzal > int(g_tegor + g_zal)):
                            #     stat = 'Przekroczono limity urlopw dla pracownika: ' + pracownik_id
                            # if SET.find('dni_zalegle') == -1 and ('dni_zalegle' in Update[0][i] or 'godziny_zalegle' in Update[0][i] or 'dni_tegoroczne'
                            #       in Update[0][i] or 'godziny_tegoroczne' in Update[0][i] or 'dni_robocze' in Update[0][i] or 'godziny_robocze' in Update[0][i]):
                            #     SET+="dni_zalegle="+str(upddzal)+", "
                            #     SET+="godziny_zalegle="+str(updgzal)+", "
                            #     SET+="dni_tegoroczne="+str(upddtegor)+", "
                            #     SET+="godziny_tegoroczne="+str(updgtegor)+", "
                                
                    SET=SET.strip(", ")
                    WHERE+=";"
                    sql+=UPDATE+SET+WHERE
                    rsglobal.ZapiszDoLogow('MMMMMMMMMMMMMMMMMMMM', 5)
                    rsglobal.ZapiszDoLogow(sql, 5)

                    if stat == 'OK':
                        stat, dump, dump, dump = self.PolSQL.Execute(rsglobal.Dekoduj(sql), 1, CzyDodawacBlad=True)
                        if stat !='OK':
                            stat = "Bd przy modyfikowaniu danych: "+stat
                            dane.append(stat)


#-------------------------------------------------------------------------------
#---   INSERT (1)
#-------------------------------------------------------------------------------

if Funkcja == 1 and stat == 'OK':
    SprawdzenieBlad=u'Nie mona doda - istnieje ju absencja o podanym kluczu: '
    Uwagi=['pracownik', 'typ', 'data od']
    rsglobal.ZapiszDoLogow(Insert, 5)

    if len(Insert[0]) > 0 and Insert[0][0][kolGrid['pracownik_id']] != '':
        if not UprIns:
            stat='UPR BR INS'
            dane.append(stat)
 
        if stat == 'OK':
            WierszeNr = Insert[-1]

            for i in xrange(len(Insert[0])):
                Sprawdzenie="SELECT 1 FROM rs_kp_absencje_pozycje WHERE firma_id='"+self.firma_id+"' "
                wymagania = [[]]
                dkal = None
                for j in xrange(len(Insert[0][i])):
                    pole=Insert[1][j]
                    wart=Insert[0][i][j]
                    if pole == 'pracownik_id':
                        wart=('000000' + str(wart))[-6:]
                        SPR1=" AND "+pole+"='"+wart+"'"
                        if wart.strip()=='':
                            dane.append('PRACOWNIK BR')
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                        elif len(wart.strip())>6:
                            dane.append('PRACOWNIK DL')
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                        else:
                            #AB 18.09.18r
                            pracownik_id = wart.strip()
                            opis = "SELECT ' '||pracownik_id::int||'-'|| nazwisko ||' '|| imie FROM rs_kp_dane_osobowe "
                            opis += "WHERE firma_id='" + str(self.firma_id) + "' AND pracownik_id ='" + pracownik_id + "';"
                            stat, s, wyn1, dump = self.PolSQL.Execute(opis, 3)
                            if stat=='OK' and len(s)>0:
                                Uwagi[0]=s[0][0]
                            if len(s)==0:
                                #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('Nie ma takiego pracownika')

                            #AB 18.09.18r
                    elif pole == 'rodzina_id' and wart != '':
                        try:
                            wart = int(float(wart))
                            SPRRODZ = "SELECT * FROM rs_kp_rodzina "
                            SPRRODZ += " WHERE firma_id = '{firma}' AND pracownik_id = '{prac}' AND rodzina_id={rodz}"
                            SPRRODZ = SPRRODZ.format(firma=self.firma_id, prac=pracownik_id, rodz=wart)
                            stat, rodz, dump, dump = self.PolSQL.Execute(SPRRODZ, 3)
                            if stat == 'OK' and len(rodz) == 0:
                               #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('RODZ_BL')
                        except ValueError:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('RODZ_BL')

                    elif pole == 'typ_absencji_id':
                        Uwagi[1]=u' typ absencji-'+wart
                        SPR2 =" AND "+pole+"='"+wart+"'"
                        rsglobal.ZapiszDoLogow(wart, 5)
                        if len(wart.strip())==0:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('RODZAJ BR')
                        else:  # TUTAJ POBIERA SI INFORMACJE O WYMAGANYCH POLACH DLA DANEGO TYPU ABSENCJI, JELI KIEDY
                                # Z JAKIEGO POWODU KOLEJNO KOLUMN ULEGNIE ZMIANIE, TO SPRAWDZENIE PRZESTANIE DZIAA.
                                                                            #AB 18.09.18r   #AB 15.11.18r          #WS 2020-02-03   #WS 2020-02-03  #WS 2020-02-03
                            spr ="SELECT zus_wymagany, dostarczono, wystawiono, czy_glowna, czy_nieobecnosc=FALSE, czy_nieobecnosc, czy_zwolnienie, nieobecnosc_inna FROM rs_kp_absencje_rodzaje "
                            spr+="WHERE firma_id='"+str(self.firma_id)+"' AND typ_absencji_id = '"+wart.strip()+"';"   
                            stat, wymagania, wyn1, dump=self.PolSQL.Execute(spr, 3)
                            rsglobal.ZapiszDoLogow(spr, 5)
                            rsglobal.ZapiszDoLogow(wymagania, 5)
                            if len(wymagania)==0:
                               #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('RODZAJ BL')

                            #AB 18.09.18r
                            elif not wymagania[0][3]:
                                  #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('GLOWNY BL')

                            #WS 2020-02-03
                            # Sprawdzenie czy typ absencji prawidowy:
                            # Urlopy - czy_nieobecnosc=TRUE AND czy_glowna=TRUE
                            # Zwolnienia - (czy_zwolnienie=TRUE OR nieobecnosc_inna=TRUE) AND czy_glowna=TRUE
                            elif Tryb == 1 and not wymagania[0][5]:
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('RODZAJ URLOP BL')
                            elif Tryb == 2 and not wymagania[0][6] and not wymagania[0][7]:
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('RODZAJ ZWOLNIENIE BL')

                    elif pole=='data_od':
                        Uwagi[2]=u' data od-'+wart
                        SPR3 =" AND "+pole+"='"+wart+"'"
                        if  wart not in (None, 'None') and wart.strip()<>'':
                            if not rsglobal.SprawdzCzyToData(wart.strip()):
                                #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('DATA OD BL4')
                            #AB 22.10.18r
                            else:
                                Rok = int(wart[0:4])
                                Mies = int(wart[5:7])
                                Dzien = int(wart[8:10])
                                #AB 10.12.18r
                                data_od = wart.strip()
                                    #AB 15.11.18r
                                if not wymagania[0][4] and not rs_anakonda_daty.CzyRoboczyWgKalendarza(self, pracownik_id, datetime.date(Rok, Mies, Dzien)):
                                    #AB 26.10.18r
                                    wyn2.append((WierszeNr[i], kolGrid[pole]))
                                    dane.append('DATA OD BL ROB')
                        else:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('DATA OD BR')

                    elif pole=='data_do':               
                        if wart not in (None, 'None') and wart.strip()<>'':
                            if not rsglobal.SprawdzCzyToData(wart.strip()):
                                #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('DATA DO BL')
                                #AB 15.11.18r
                            else:
                                Rok = int(wart[0:4])
                                Mies = int(wart[5:7])
                                Dzien = int(wart[8:10])
                                rsglobal.ZapiszDoLogow(1000*' ', 5)
                                rsglobal.ZapiszDoLogow(wart, 5)
                                rsglobal.ZapiszDoLogow(rs_anakonda_daty.CzyRoboczyWgKalendarza(self, pracownik_id, datetime.date(Rok, Mies, Dzien)), 5)
                                if not wymagania[0][4] and not rs_anakonda_daty.CzyRoboczyWgKalendarza(self, pracownik_id, datetime.date(Rok, Mies, Dzien)):
                                    wyn2.append((WierszeNr[i], kolGrid[pole]))
                                    dane.append('DATA DO BL ROB')
                        else:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('DATA DO BR')

                    elif pole=='data_dostarczenia':     
                        if wart not in (None, 'None') and wart.strip()<>'':
                            if not rsglobal.SprawdzCzyToData(wart.strip()):
                                #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('DATA DOST BL')
                        elif wymagania[0][1] and wart in puste:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('DATA DOST BR')

                    elif pole=='data_wystawienia':     
                        if wart not in (None, 'None') and wart.strip()<>'':
                            if not rsglobal.SprawdzCzyToData(wart.strip()):
                                #AB 26.10.18r
                                wyn2.append((WierszeNr[i], kolGrid[pole]))
                                dane.append('DATA WYST BL')
                        elif wymagania[0][2] and wart in puste:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('DATA WYST BR')

                    elif pole=='kod':  #AB 03.03.22r
                        if wart not in ('A', 'B', 'C', 'D', 'E', ''):
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('KOD BL')

                    elif pole=='kod_zus':
                        if len(wart) > 7:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('ZUS BL')
                        elif wymagania[0][0] and wart in puste:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('ZUS BR')

                    elif pole =='dni_kalendarzowe':
                        dkal = int(wart)
                        if int(wart) < 0:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('KAL BL')
                        elif wart == 0:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('KAL BR')

                    elif pole =='dni_robocze':
                        if dkal is not None and int(wart) > dkal:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('ROBOCZE BL')
                        elif wart == 0:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('ROBOCZE BR')

                    #AB 26.10.18r
                    elif pole =='dni_szpital':
                        if dkal is not None and int(wart) > dkal:
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('SZPITAL BL')

                    elif pole =='procent_wyplaty':
                        if len(str(wart))>6:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('PROCENT BL')

                    elif pole =='godziny_robocze':
                        if len(str(int(wart)))>4:
                            #AB 26.10.18r
                            wyn2.append((WierszeNr[i], kolGrid[pole]))
                            dane.append('ROBOCZE GODZ BL')
                #AB 10.12.18r
                if stat == 'OK' and len(dane) == 0:
                    stat = SprawdzZatrudnienie(self, pracownik_id, data_od)
                rsglobal.ZapiszDoLogow(dane, 5)
                rsglobal.ZapiszDoLogow(wyn2, 5)
                if stat == 'OK' and len(dane) == 0:
                    Sprawdzenie+=SPR1+SPR2+SPR3
                    rsglobal.ZapiszDoLogow(Sprawdzenie, 5)
                    stat, s, wyn1, dump = self.PolSQL.Execute(Sprawdzenie, 3)
                    if len(s) > 0:
                        wyn2.append((WierszeNr[i], 0))
                        dane.append(SprawdzenieBlad+Uwagi[0]+Uwagi[1]+Uwagi[2])

        if stat == 'OK':
            if len(dane) > 0:
                stat = dane[0]


        # wykonanie INSERT
        if stat == 'OK':    
            sqlIns=''
            for i in xrange(len(Insert[0])):
                INSERT="INSERT INTO rs_kp_absencje_pozycje"
                KOLUMNY="(firma_id, oddzial_id, data, numer"
                VALUES=" VALUES ('"+self.firma_id+"', '"+str(Oddzial)+"', '"+str(Data)+"', '"+str(Numer)+"'"

                # dt = Insert[1].index('dni_tegoroczne')
                # gt = Insert[1].index('godziny_tegoroczne')
                # dz = Insert[1].index('dni_zalegle')
                # # gz = Insert[1].index('godziny_zalegle')
                # dr = Insert[1].index('dni_robocze')
                # gr = Insert[1].index('godziny_robocze')
                # pr = Insert[1].index('pracownik_id')
                # typ = Insert[1].index('typ_absencji_id')
                # rok = Data[:4]
                #
                # SPRURL = "select sum(CASE WHEN rok < " + rok + " THEN 0 ELSE pozostalo_dni END) AS dni_tegoroczne, "
                # SPRURL += "sum(CASE WHEN rok < " + rok + " THEN 0 ELSE pozostalo_godziny END) AS godziny_tegoroczne, "
                # SPRURL += "sum(CASE WHEN rok = " + rok + " THEN 0 ELSE pozostalo_dni END) AS dni_zalegle, "
                # SPRURL += "sum(CASE WHEN rok = " + rok + " THEN 0 ELSE pozostalo_godziny END) AS godziny_zalegle "
                # SPRURL += "FROM rs_kp_dane_osobowe_limity_urlopow "
                # SPRURL += "WHERE firma_id = '"+self.firma_id+"' AND pracownik_id = '" + Insert[0][i][pr] + "' "
                # SPRURL += "AND urlop_id = '" + Insert[0][i][typ] + "';"
                # stat, url, dump, dump = self.PolSQL.Execute(SPRURL, 3)
                #
                # if stat != 'OK':
                #     stat = 'Bd sprawdzania zalegych urlopw. ' + stat
                # elif url[0][0] == 0 and url[0][1] == 0 and url[0][2] == 0 and url[0][3] == 0:
                #     stat = 'Brak pozostaych limitw urlopw dla pracownika: ' + Insert[0][i][pr] + ', typ absencji: '
                #     stat += Insert[0][i][typ]
                # else:
                #
                #     d_tegor = int(url[0][0])
                #     g_tegor = int(url[0][1])
                #     d_zal = int(url[0][2])
                #     g_zal = int(url[0][3])
                #     upddzal = 0
                #     upddtegor = 0
                #     updgzal = 0
                #     updgtegor = 0
                #
                #     if Insert[0][i][gr] != 0:
                #         if Insert[0][i][gz] == 0 and Insert[0][i][gt] == 0:
                #             updgzal = min(int(Insert[0][i][gr]), g_zal)
                #             updgtegor = int(Insert[0][i][gr]) - updgzal
                #
                #         elif Insert[0][i][gz] != 0 and Insert[0][i][gt] == 0:
                #             updgzal = Insert[0][i][gz]
                #             updgtegor = int(Insert[0][i][gr]) - updgzal
                #         elif Insert[0][i][gz] == 0 and Insert[0][i][gt] != 0:
                #             updgzal = min(int(Insert[0][i][gr]) - updgtegor, g_zal)
                #             updgtegor = int(Insert[0][i][gr]) - updgzal
                #         else:
                #             updgzal = Insert[0][i][gz]
                #             updgtegor = Insert[0][i][gt]
                #
                #
                #     #dni robocze nie moga byc puste, nie sprawdzamy tego tutaj
                #     if Insert[0][i][dz] == 0 and Insert[0][i][dt] == 0:
                #         upddzal = min(int(Insert[0][i][dr]), d_zal)
                #         upddtegor = int(Insert[0][i][dr]) - upddzal
                #
                #     elif Insert[0][i][dz] != 0 and Insert[0][i][dt] == 0:
                #         upddzal = Insert[0][i][dz]
                #         upddtegor = int(Insert[0][i][dr]) - upddzal
                #     elif Insert[0][i][dz] == 0 and Insert[0][i][dt] != 0:
                #         upddzal = min(int(Insert[0][i][dr]) - upddtegor, d_zal)
                #         upddtegor = int(Insert[0][i][dr]) - upddzal
                #     else:
                #         upddzal = Insert[0][i][dz]
                #         upddtegor = Insert[0][i][dt]
                #     if upddtegor > d_tegor or updgtegor > g_tegor:
                #         stat = 'Przekroczono limity urlopw dla pracownika: ' + Insert[0][i][pr]


                for j in xrange(len(Insert[0][i])):
                    pole=Insert[1][j]
                    wart=Insert[0][i][j]

                    if pole in ['pracownik_id']:
                        KOLUMNY+=", "+pole
                        wart=('000000' + str(wart))[-6:]
                        VALUES+=", '"+str(wart)+"'"
                    #AB 18.09.18r
                    elif pole in ['data_od', 'kontynuacja', 'kontynuacja_data', 'wypadkowe']:  #AB 01.03.21r
                        KOLUMNY+=", "+pole
                        #AB 18.09.18r
                        if str(wart) == '':
                            VALUES+=", NULL"
                        else:
                            VALUES+=", '"+str(wart)+"'"
                    # elif pole == 'dni_zalegle':
                    #     KOLUMNY+=", "+pole
                    #     VALUES+=", '"+str(upddzal)+"'"
                    # elif pole == 'godziny_zalegle':
                    #     KOLUMNY+=", "+pole
                    #     VALUES+=", '"+str(updgzal)+"'"
                    # elif pole == 'dni_tegoroczne':
                    #     KOLUMNY+=", "+pole
                    #     VALUES+=", '"+str(upddtegor)+"'"
                    # elif pole == 'godziny_tegoroczne':
                    #     KOLUMNY+=", "+pole
                    #     VALUES+=", '"+str(updgtegor)+"'"
                    elif pole in ['data_do', 'typ_absencji_id', 'data_wystawienia', 
                                  'data_dostarczenia', 'kod_zus', 'kod', 'dni_kalendarzowe',
                                  'dni_robocze', 'procent_wyplaty', 
                                                    #AB 26.10.18r
                                  'godziny_robocze', 'dni_szpital']:

                        KOLUMNY+=", "+pole
                        if str(wart).strip() in puste:
                            VALUES+=", "+str('NULL')
                        else:
                            VALUES+=", '"+str(wart)+"'"
                                #AB 18.09.18r
                    elif pole in ['rodzina_id']:
                        KOLUMNY+=", "+pole
                        if wart.strip() in ('0', '0.0', ''):
                            VALUES+=", NULL"
                        else:
                            VALUES+=", " + wart
                                #AB 29.06.20r
                    elif pole in ['na_zadanie']:
                        KOLUMNY+=", "+pole
                        VALUES+=", " + str(bool(wart))
                KOLUMNY+=")"
                VALUES+=");"
                sqlIns+=INSERT+KOLUMNY+VALUES
            if sqlIns != '' and stat == 'OK':
                rsglobal.ZapiszDoLogow(sqlIns)
                stat, dump, dump, dump=self.PolSQL.Execute(rsglobal.Dekoduj(sqlIns), 1, CzyDodawacBlad=True)
                if stat != 'OK':
                    stat = "Bd przy dodawaniu danych: "+stat
                    dane.append(stat)  


#-------------------------------------------------------------------------------
#---   Sprawdzenie pokrywania si dat
#-------------------------------------------------------------------------------

if Funkcja == 1 and stat == 'OK':
    #sprawdzenie na oknie
    #WS 2023-02-02
    sql ="Select DISTINCT a.typ_absencji_id, a.data_od, b.data_od, b.data_do, a.pracownik_id FROM rs_kp_absencje_pozycje a "
    sql+="JOIN rs_kp_absencje_pozycje b "
    sql+="ON a.firma_id=b.firma_id AND a.pracownik_id=b.pracownik_id "
    sql+="AND ((a.data_od BETWEEN b.data_od and b.data_do) OR (a.data_do BETWEEN b.data_od and b.data_do)) "
    sql+="AND a.data_od!=b.data_od "
    sql+="JOIN rs_kp_absencje c "
    sql+="ON b.firma_id=c.firma_id AND b.oddzial_id=c.oddzial_id AND b.data=c.data AND b.numer=c.numer "
    sql+="WHERE a.firma_id='"+self.firma_id+"' AND a.oddzial_id='"+Oddzial+"' AND a.data='"+Data+"' AND a.numer='"+Numer+"' AND c.stan='W';"
    # sql ="SELECT DISTINCT a.typ_absencji_id, a.data_od, b.data_od, b.data_do, a.pracownik_id FROM rs_kp_absencje_pozycje a, rs_kp_absencje_pozycje b "
    # sql+="WHERE a.firma_id='"+self.firma_id+"' AND a.firma_id=b.firma_id AND a.pracownik_id=b.pracownik_id "
    # sql+="AND (a.data_od BETWEEN b.data_od and b.data_do)"
    # sql+="AND a.data_od!=b.data_od AND a.oddzial_id='"+Oddzial+"' AND a.data='"+Data+"' AND a.numer='"+Numer+"';"
    stat, D, wyn1, dump=self.PolSQL.Execute(sql, 3)
    if stat=='OK' and len(D)>0:
        stat="Istniej pokrywajce si daty w pozycjach: "
        for i in xrange(len(D)):
            rodz=str(D[i][0])
            dataA=str(D[i][1])
            dataB=str(D[i][2])
            dataC=str(D[i][3])            
            prac=str(D[i][4])
            stat+= u"\nPracownik: "+ prac+" , rodzaj: "+str(rodz).strip()+u", data od: "+str(dataA)+"\n\tpokrywa si z zakresem od "+str(dataB)+" do "+str(dataC)
        wyn2.append((0, 1))
        D.append(stat)  

    if stat == 'OK':
        #w poczeniu z rs_kp_dane_osobowe_absencje
        sql ="Select DISTINCT a.typ_absencji_id, a.data_od, b.od_kiedy, b.do_kiedy From rs_kp_absencje_pozycje a, rs_kp_dane_osobowe_absencje b "
        sql+="WHERE a.firma_id='"+self.firma_id+"' AND a.firma_id=b.firma_id AND a.pracownik_id=b.pracownik_id "
        sql+="AND ((a.data_od BETWEEN b.od_kiedy and b.do_kiedy) OR (a.data_do BETWEEN b.od_kiedy and b.do_kiedy)) "
        sql+="AND a.data_od!=b.od_kiedy AND a.oddzial_id='"+Oddzial+"' AND a.data='"+Data+"' AND a.numer='"+Numer+"';"
        stat, D, wyn1, dump=self.PolSQL.Execute(sql, 3)
        if stat=='OK' and len(D)>0:
            stat="Istniej pokrywajce si daty w danych osobowych absencji: "
            for i in xrange(len(D)):
                rodz=str(D[i][0])
                dataA=str(D[i][1])
                dataB=str(D[i][2])
                dataC=str(D[i][3])
                stat+= u"\nRodzaj: "+str(rodz).strip()+u", data od: "+str(dataA)+"\n\tpokrywa si z zakresem od "+str(dataB)+" do "+str(dataC)
            wyn2.append((0, 1))
            D.append(stat)


#-------------------------------------------------------------------------------
#---   POBRANIE DANYCH (1, 5)
#-------------------------------------------------------------------------------

if stat=='OK' and (Funkcja in [1, 5]):  #AB 03.03.22r dodanie kolumny kod
    sql ="SELECT kolejnosc, p.pracownik_id, TRIM(nazwisko)||' '||TRIM(imie), p.typ_absencji_id, r.nazwa, data_od, data_do, "
    sql+="data_dostarczenia, data_wystawienia, coalesce(p.kod_zus, ' '), coalesce(p.kod, ''), dni_szpital, procent_wyplaty, kontynuacja, kontynuacja_data, wypadkowe, rodzina_id::TEXT, dni_kalendarzowe, dni_robocze, "
    sql+="godziny_robocze, na_zadanie, dni_zalegle, godziny_zalegle, dni_tegoroczne, godziny_tegoroczne "
    sql+="FROM rs_kp_absencje_pozycje p JOIN rs_kp_dane_osobowe o ON p.firma_id=o.firma_id AND p.pracownik_id = o.pracownik_id "
    sql+="JOIN rs_kp_absencje_rodzaje r ON p.typ_absencji_id = r.typ_absencji_id AND p.firma_id = r.firma_id "
    sql+="WHERE p.firma_id='"+str(self.firma_id)+"' AND oddzial_id = '"+str(Oddzial)
    sql+="' AND data = '"+str(Data)+"' AND numer = '"+str(Numer)+"' ORDER BY kolejnosc;"
    rsglobal.ZapiszDoLogow(sql, 5)
    stat, dane, wyn1, dump=self.PolSQL.Execute(sql, 3)
    if stat<>'OK':
        stat="Bd przy pobieraniu danych : "+stat   
        dane.append(stat)

#-------------------------------------------------------------------------------
#---   E-ZLA (13)
#-------------------------------------------------------------------------------

def PobierzWartosc(wezel,listatagow):
    for tag in listatagow:
        if wezel.find(tag) is not None:
            wezel = wezel.find(tag)
        else:
            return None

    return wezel.text


#WS 2019-06-11
if stat=='OK' and Funkcja in [13]:
    Tresc = binascii.unhexlify(TrescXML)

    #Usunicie wiersza z namespace, by nazwy tagw nie miay go w prefixie i atwiej byo uy find()
    namespace = Tresc[Tresc.find('xmlns="'):Tresc.find('"', Tresc.find('xmlns="')+7)+1]
    Tresc = Tresc.replace(namespace, '')

    root = etree.fromstring(Tresc).find('ZUSZLA')

    DaneXML = {'PESEL':          PobierzWartosc(root,["II", "p1"]),
               'Imie':           PobierzWartosc(root,["II", "p2"]),
               'Nazwisko':       PobierzWartosc(root,["II", "p3"]),
               'DataOd':         PobierzWartosc(root,["IV", "p1", "p1"]),
               'DataDo':         PobierzWartosc(root,["IV", "p1", "p2"]),
               'DataWystawienia':PobierzWartosc(root,["VIII", "p1"])}

    tmpSzpitalOd = PobierzWartosc(root,["IV", "p2", "p1"])
    tmpSzpitalDo = PobierzWartosc(root,["IV", "p2", "p2"])

    if tmpSzpitalOd is not None and tmpSzpitalDo is not None:
        DaneXML['SzpitalOd'] = tmpSzpitalOd
        DaneXML['SzpitalDo'] = tmpSzpitalDo
        Szpital = True
    else:
        Szpital = False

    for pole, wartosc in DaneXML.iteritems():
        if pole in ['Imie', 'Nazwisko']:
            if wartosc is None:
                stat = 'XML BL'
                break
        elif pole in ['DataOd', 'DataDo', 'DataWystawienia']:
            if wartosc is None:
                stat = 'XML BL'
                break
            else:
                if not SprawdzCzyToData(wartosc):
                    stat = 'Bdna warto pola '+pole
        elif pole in ['PESEL']:
            if wartosc is None:
                stat = 'XML BL'
                break
            else:
                if SprPesel(wartosc) != 'OK':
                    stat = 'Bdna warto pola '+pole

    if Szpital:
        if not SprawdzCzyToData(DaneXML['SzpitalOd']) or not SprawdzCzyToData(DaneXML['SzpitalDo']):
            stat = 'Bdna warto pola SzpitalOd lub SzpitalDo'

    if stat == 'OK':
        if DaneXML['DataDo'] < DaneXML['DataOd']:
            stat = 'XML DATA BL'

        if Szpital:
            if DaneXML['SzpitalDo'] < DaneXML['SzpitalOd']:
                stat = 'XML SZ DATA BL'

    if stat == 'OK':
        sql ="SELECT pracownik_id, imie, nazwisko"
        sql+=" FROM rs_kp_dane_osobowe"
        sql+=" WHERE firma_id='"+self.firma_id+"' AND pesel='"+DaneXML['PESEL']+"';"
        rsglobal.ZapiszDoLogow(sql, 5)
        stat,d,dump,dump=self.PolSQL.Execute(sql, 3)
        if stat != 'OK':
            stat = "XML PRAC SQL"
        else:
            if len(d) > 0:
                if unicode(d[0][1]).lower() != unicode(DaneXML['Imie']).lower() or unicode(d[0][2]).lower() != unicode(DaneXML['Nazwisko']).lower():
                    stat = "XML PRAC BL"
            else:
                stat = "XML PRAC BR"

    if stat == 'OK':
        Pracownik = d[0][0]

        sql ="SELECT typ_absencji_id, kod_zus, proc_zwolnienia"
        sql+=" FROM rs_kp_absencje_rodzaje"
        sql+=" WHERE firma_id='"+self.firma_id+"' AND czy_zwolnienie = TRUE AND czy_glowna = TRUE;"
        rsglobal.ZapiszDoLogow(sql, 5)
        stat,d,dump,dump=self.PolSQL.Execute(sql, 3)
        if stat != 'OK':
            stat = "XML R ABS SQL"
        else:
            if len(d) != 1:
                stat = "XML R ABS"

    if stat == 'OK':
        TypAbsencji = d[0][0]
        KodZUS = "'"+str(d[0][1])+"'" if d[0][1] != 'None' else 'NULL'
        ProcZwol = "'"+str(d[0][2])+"'"

        sql ="SELECT pracownik_id FROM rs_kp_absencje_pozycje WHERE firma_id='"+self.firma_id+"' "
        sql+=" AND pracownik_id='"+str(Pracownik)+"' AND typ_absencji_id='"+str(TypAbsencji)+"' AND data_od='"+DaneXML['DataOd']+"';"
        rsglobal.ZapiszDoLogow(sql, 5)
        stat,s,dump,dump = self.PolSQL.Execute(sql, 3)
        if len(s) > 0:
            stat =u'Nie mona doda - istnieje ju absencja o podanym kluczu:\n' \
                  u'Pracownik: '+str(Pracownik)+u' | Rodz. absencji: '+str(TypAbsencji)+u' | Data od: '+DaneXML['DataOd']

    if stat == 'OK':
        stat = SprawdzZatrudnienie(self, Pracownik, DaneXML['DataOd'])

    if stat == 'OK':
        d = abpom.LiczDniIGodzinyRobocze(self, DaneXML['DataOd'], DaneXML['DataDo'], Pracownik)[1]
        DniRob = d[0][0]
        GodzRob = d[0][1]
        DniKal = rsglobal.IleDniWZakresieDat(DaneXML['DataOd'],DaneXML['DataDo'])

        sqlIns ="INSERT INTO rs_kp_absencje_pozycje ("
        sqlIns+="firma_id, oddzial_id, data, numer, typ_absencji_id, kod_zus, procent_wyplaty,"
        sqlIns+="pracownik_id, data_od, data_do, data_wystawienia, data_dostarczenia,"
        if Szpital:
            sqlIns+="dni_szpital,"
        sqlIns+="dni_kalendarzowe, dni_robocze, godziny_robocze"
        sqlIns+=") VALUES ("
        sqlIns+="'"+self.firma_id+"', '"+str(Oddzial)+"', '"+str(Data)+"', '"+str(Numer)+"', '"+str(TypAbsencji)+"',"+str(KodZUS)+","+str(ProcZwol)+","
        sqlIns+="'"+str(Pracownik)+"', '"+DaneXML['DataOd']+"', '"+DaneXML['DataDo']+"', '"+DaneXML['DataWystawienia']+"', '"+DaneXML['DataWystawienia']+"',"
        if Szpital:
            sqlIns+=str(rsglobal.IleDniWZakresieDat(DaneXML['SzpitalOd'],DaneXML['SzpitalDo']))+","
        sqlIns+=str(DniKal)+","+str(DniRob)+","+str(GodzRob)
        sqlIns+=");"

        rsglobal.ZapiszDoLogow(sqlIns, 5)
        stat,dump,dump,dump=self.PolSQL.Execute(rsglobal.Dekoduj(sqlIns), 1, CzyDodawacBlad=True)
        if stat != 'OK':
            stat = "Bd przy dodawaniu danych: "+stat

