# -*- coding: iso8859_2 -*-
#-------------------------------------------------------------------------------
# Name:        fs_magazyny_parametry_dokumentow_pom.py
# Purpose:     funkcje pomocnicze do sprawdzen parametrow z dokumentow magazynowych
# Author:      Mariusz Rajfura
# Created:     2011/05/11
# RCS-ID:      $Id: fs_magazyny_parametry_dokumentow_pom$
# Copyright:   (c) 2011 Record System
# Licence:     GNU GPL
#-------------------------------------------------------------------------------

# 2011-06-06 Mariusz Rajfura - wielokrotne definicje parametrow dla dok.w magazynie (4638)

def PodajDomyslneParametryDlaDokumentu(self,Mag,Dok,JedenWiersz = False,Pojedyncze = True):
    # Funkcja podaje liste domyslnych parametrow dla dokumentu w podanym magazynie
    # w formie slownika: pole:wartosc
    # Oprocz tego w slowniku dla klucza "ZESTAWY_PARAMETROW" jest lista
    # dozwolonych zestawow parametrow.
    # Domyslne parametry to takie, kiedy jest tylko jeden zestaw pojedynczych parametrow
    # dla dokumentu w magazynie
    
    import rsglobal

    SDomyslne = {}
    SDomyslne['wersja'] = ''
    SDomyslne['kontrahent_id'] = ''
    SDomyslne['ZESTAWY_PARAMETROW'] = []

    Mag = rsglobal.Dekoduj(Mag).strip()
    Dok = rsglobal.Dekoduj(Dok).strip()
    
    if Mag == '' or Dok == '':
        return 'Brak magazynu lub dokumentu',SDomyslne

    sql ="SELECT"
    sql+=" COALESCE(TRIM(wersje_cenowe),''::TEXT),"
    sql+=" COALESCE(TRIM(kontrahenci),''::TEXT)"
    sql+=" FROM rs_magazyny_dokumenty_parametry"
    sql+=" WHERE firma_id='"+self.firma_id+"' AND magazyn_id='"+Mag+"'"
    sql+=" AND dokument_id='"+Dok+"';"
    
    stat,dParam,dump,dump = self.PolSQL.Execute(sql,3)
    if stat <> 'OK':
        return 'Bd pobrania parametrw dla dokumentu '+Dok+' w magazynie '+Mag+': '+stat,SDomyslne

    if len(dParam) == 0:
        # brak zdefiniowanych parametrow dla dokumentu w magazynie
        return 'OK',SDomyslne

    # lista dozwolonych zestawow do innych sprawdzen/wyszukiwan
    SDomyslne['ZESTAWY_PARAMETROW'] = dParam
    
    # szukamy domyslnego zestawu parametrow dla dokumentu w magazynie
    Licz = 0
    Wersja = ''
    Kontrahent = ''
    
    for i in xrange(len(dParam)):
        Wersje = rsglobal.Dekoduj(dParam[i][0]).split(',')
        Kontrahenci = rsglobal.Dekoduj(dParam[i][1]).split(',')
        
        # zestaw pojedynczych parametrow
        if Pojedyncze:
            if len(Wersje) == 1 and len(Kontrahenci) == 1:
                if len(Wersje[0])<=2:
                    Wersja = Wersje[0]
                if len(Kontrahenci[0])<=5:
                    Kontrahent = Kontrahenci[0]
        else:
            if len(Wersje) > 0:
                Wersja = Wersje[0]
            if len(Kontrahenci) > 0:
                Kontrahent = Kontrahenci[0]
                
        Licz += 1
    rsglobal.ZapiszDoLogow(str(JedenWiersz)+' '+str(Licz),1)
    if Licz == 1:
        # jesli tylko jeden zestaw pojedynczych parametrow
        SDomyslne['wersja'] = Wersja
        SDomyslne['kontrahent_id'] = Kontrahent
    elif JedenWiersz:   # brak lub wicej ni jeden - bd
        return 'Niejednoznacznie zdefiniowane parametry dla dokumentu '+Dok,SDomyslne

    return 'OK',SDomyslne

#-------------------------------------------------------------------------------

def SprawdzParametryDlaDokumentu(self,Mag,Dok,SParam):
    # funkcja sprawdza domyslne parametry dla dokumentu z magazynu
    # ze slownikiem[pole]=wartosc
    import rsglobal

    stat,SDomyslne = PodajDomyslneParametryDlaDokumentu(self,Mag,Dok)
    if stat <> 'OK':
        return stat
    
    # lista wszystkich dozwolonych zestawow dla dokumentu w magazynie
    LZestawy = SDomyslne['ZESTAWY_PARAMETROW']
    
    if len(LZestawy) == 0:
        # brak zdefiniowanych parametrow oznacza, ze wszystkie akceptowane
        return 'OK'
    
    # parametry z dokumentu do sprawdzenia
    Wersja = rsglobal.Dekoduj(SParam['wersja']).strip()
    Kontrahent = rsglobal.Dekoduj(SParam['kontrahent_id']).strip()
    if Wersja in ('None','NULL'):
        Wersja = ''
    
    if Kontrahent in ('None','NULL'):
        Kontrahent = ''
    
    # listy wszystkich dozwolonych zestawow dla dokumentu w magazynie
    LZestawy = SDomyslne['ZESTAWY_PARAMETROW']
    # przegladamy zdefiniowane zestawy
    # pierwsze trafienie parametrow spelniajacych warunki konczy funkcje z wynikiem OK
    for i in xrange(len(LZestawy)):
        WersjaOK = False
        KontrahentOK = False
        
        # sprawdzenie wersji, jesli podane dozwolone wersje w komplecie
        if LZestawy[i][0] <> '':
            if Wersja <> '' and Wersja in LZestawy[i][0]:
                WersjaOK = True
        else:
            WersjaOK = True

        # sprawdzenie kontrahenta, jesli podani dozwoloni kontrahenci w komplecie
        if LZestawy[i][1] <> '':
            if Kontrahent <> '' and Kontrahent in LZestawy[i][1]:
                KontrahentOK = True
        else:
            KontrahentOK = True
        
        if WersjaOK and KontrahentOK:
            # udalo sie znalezc dozwolona kombinacje parametrow
            return 'OK'

    # po sprawdzeniu calej listy zestawow dozwolonych parametrow nie znaleziono
    # ani jednego, ktory zgadzal sie z parametrami podanymi dla dokumentu
    if not WersjaOK:
        if Wersja == '':
            return 'BR_PARAM_WERSJA'
        else:
            return 'BL_PARAM_WERSJA'

    if not KontrahentOK:
        if Kontrahent == '':
            return 'BR_PARAM_KONTRAHENT'
        else:
            return 'BL_PARAM_KONTRAHENT'

#-------------------------------------------------------------------------------

def UzupelnijDomyslneParametryDlaDokumentu(self,Mag,Dok,SParam,JedenWiersz = False,Pojedyncze = True):
    # Funkcja uzupelnia puste miejsca slownika z proponowanymi parametrami
    # Jesli wszystkie miejsca sa puste, to wypelnia je parametrami domyslnymi
    # dla dokumentu w magazynie.
    # Jezeli ktorys z parametrow jest podany, to probuje uzupelnic pozostale,
    # jesli znajdziemy zestaw dla istniejacego parametru, dla ktorego pozostale
    # parametry sa pojedynczymi wartosciami, aby byla jednoznacznosc

    import rsglobal
    Wersja = rsglobal.Dekoduj(SParam['wersja']).strip()
    Kontrahent = rsglobal.Dekoduj(SParam['kontrahent_id']).strip()
    if Wersja in ('None','NULL'):
        Wersja = ''
        
    if Kontrahent in ('None','NULL'):
        Kontrahent = ''
    if Wersja <> '' and Kontrahent <> '':
        # nie ma nic do uzupelniania
        return 'OK',SParam

    # pobieramy parametry domyslne i zestawy parametrow
    stat,SDomyslne = PodajDomyslneParametryDlaDokumentu(self,Mag,Dok,JedenWiersz,Pojedyncze)
    if stat <> 'OK':
        return stat,''
    
    LZestawy = SDomyslne['ZESTAWY_PARAMETROW']
    
    if len(LZestawy) == 0:
        # brak definicji parametrow dla dokumentu w magazynie -> wszystkie dozwolone,
        # a tym samym nie ma czym uzupelniac
        return 'OK',SParam

    if Wersja == '' and Kontrahent == '':
        # jesli wszystkie parametry puste, to wypelniamy domyslnymi i koniec
        SParam['wersja'] = SDomyslne['wersja']
        SParam['kontrahent_id'] = SDomyslne['kontrahent_id']
        return 'OK',SParam

    # przyjete rozwiazanie : najpierw kontrahent, potem wersja
    
    # podany kontrahent -> uzupelnic wersje, jesli jest tylko jedna wersja dla podanego kontrahenta
    if Kontrahent <> '' and Wersja == '':
        # sprawdzenie dla konkretnych kontrahentw
        for i in xrange(len(LZestawy)):
            LKontr=LZestawy[i][1].split(',')
            if Kontrahent in LKontr:
                Wersje = LZestawy[i][0].split(',')
                if len(Wersje) > 0:
                    SParam['wersja'] = Wersje[0]
                    break

        if SParam['wersja'] in ('','NULL'):
            # sprawdzenie dla pustych kontrahentw (mniej wane)
            for i in xrange(len(LZestawy)):
                if LZestawy[i][1].strip()=='':
                    Wersje = LZestawy[i][0].split(',')
                    if len(Wersje) > 0:
                        SParam['wersja'] = Wersje[0]
                        break

    # podana wersja -> uzupelnic kontrahenta, jesli jest tylko jeden kontrahent dla podanej wersji
    if Wersja <> '' and Kontrahent == '':
        for i in xrange(len(LZestawy)):
            if Wersja in LZestawy[i][0]:
                Kontrahenci = LZestawy[i][1].split(',')
                if len(Komtrahenci) == 1:
                    Kontrahent = Kontrahenci[0]
                    break
                
    return 'OK',SParam

#-------------------------------------------------------------------------------

def UtworzSqlUpdZParametrow(Firma,Mag,Dok,Data,Numer,SParametry):
    # funkcja tworzy sql do aktualizacji dokumentu magazynowego
    # z domyslnych parametrow na podstawie slownika[pole]=wartosc
    import rsglobal
    
    SqlUpd = ''
    sql = ''
    for pole in SParametry.keys():
        if pole <> 'ZESTAWY_PARAMETROW':
            wart = rsglobal.Dekoduj(SParametry[pole])
            if type(wart) == type(str()):
                if wart <> '':
                    sql+=pole+"='"+wart+"',"
            else:
                sql+=pole+"="+str(wart)+","
            
    if len(sql) > 0:
        SqlUpd ="UPDATE rs_dokumenty_magazynowe SET "+sql[:-1]
        SqlUpd+=" WHERE firma_id='"+rsglobal.Dekoduj(Firma)+"'"
        SqlUpd+=" AND magazyn_id='"+rsglobal.Dekoduj(Mag)+"'"
        SqlUpd+=" AND dokument_id='"+rsglobal.Dekoduj(Dok)+"'"
        SqlUpd+=" AND data='"+rsglobal.Dekoduj(Data)+"'"
        SqlUpd+=" AND numer="+str(Numer)+";"
    
    return SqlUpd