# -*- coding: iso8859_2 -*-

#-----------------------------------------------------------------------------
# Name:        fs_pliki_global.py
# Purpose:     funkcje do zarzdzania plikami zapisywanymi na dysku serwera
# Author:      Micha Raburski
# Created:     2010/12/27
# RCS-ID:      $Id: fs_pliki_global.py $
# Copyright:   (c) 2010 RecordSystem
# Licence:     GNU GPL
#-----------------------------------------------------------------------------

# 2021-01-18 Wojciech Skowron - (12202) funkcja ZapiszPlik
# 2021-11-03 Wojciech Skowron - (12174) Poprawiona obsuga pliku
# 2023-07-21 Aleksander Bargie (12552) Czy tworzy katalog jeli cieka do zapisu pliku nie istnieje?
# 2023-08-23 Aleksander Bargie (13503) Proba o wyjanienie

import rsglobal
import os
import zipfile
import binascii
import random
import os.path

#-------------------------------------------------------------------------------
# podaje sciek do pliku wzgldem numeru
#-------------------------------------------------------------------------------

def PodajNazwePliku(NrPliku):
    Plik=str(NrPliku)
    # uparzystowienie pliku
    if len(Plik) % 2==1:
        Plik='0'+Plik
    
    KatPlik=''
    while len(Plik)>2:
        KatPlik+='/'+Plik[:2]
        Plik=Plik[2:]
    
    Plik=Plik+'.ext'
    return KatPlik,Plik
    
#-------------------------------------------------------------------------------
# funkcja rozszyfrowuje katalog wzgl numeru pliku i zapisuje go pod nazw .ext
#-------------------------------------------------------------------------------

def WstawPlikNaSerwer(Katalog,NrPliku,TrescPliku):
    try:
        if TrescPliku=='':
            return 'OK'
        
        # sprawdzamy czy gwny katalog plikw istnieje. Jeli nie to nie pozwalamy zapisa.
        # uwaga! z automatu wiemy, e ostatnia cz katalogu to baza danych i musimy j wyci z badania.
        KatalogTest=Katalog[:Katalog.rfind('/')]
        if not os.path.isdir(KatalogTest):
            return 'UWAGA! Gowny katalog plikw ('+KatalogTest+') jest niedostpny. Poinformuj administratora'
        
        #zakadamy, e plik jest hexlified
        import binascii
        TrescPliku=binascii.unhexlify(TrescPliku)
        
        # pobranie cieki pliku
        KatPlik,Plik=PodajNazwePliku(NrPliku)
        KatPlik=Katalog+KatPlik
        
        # ewentualne utworzenie katalogu
        if not os.path.isdir(KatPlik):
            os.makedirs(KatPlik)

        #WS 2021-11-03
        with open(KatPlik+'/'+Plik, 'wb') as p:
            p.write(TrescPliku)
        # open(KatPlik+'/'+Plik, 'wb').write(TrescPliku)
    except Exception,e:
        return 'Bd zapisu pliku na serwerze : '+str(e)
    
    return 'OK'


#-------------------------------------------------------------------------------
# funkcja pobiera tre pliku o podanym numerze i koduje szesnastkowo do przesyki do klienta
#-------------------------------------------------------------------------------
#AB 2023-08-23 Zmiana opisu i nazwy funkcji
def PrzygotujPlikDoPrzeslaniaPoNumerze(Katalog, NrPliku):
    try:
        # pobranie cieki pliku 
        KatPlik,Plik=PodajNazwePliku(NrPliku)
        KatPlik=Katalog+KatPlik 
        
        #mamy plik i katalog do pobrania
        import binascii
        #WS 2021-11-03
        with open(KatPlik+'/'+Plik,'rb') as p:
            Tresc=binascii.hexlify(buffer(p.read()))
        # Tresc=binascii.hexlify(buffer(open(KatPlik+'/'+Plik,'rb').read()))
    except Exception,e:
        return 'Bd wczytania pliku z serwera : '+str(e),''
    
    return 'OK',Tresc

#AB 2023-09-06
def PobierzPlikZSerwera(Katalog,NrPliku):  # przestarzaa nazwa pliku
    return PrzygotujPlikDoPrzeslaniaPoNumerze(Katalog, NrPliku)

#-------------------------------------------------------------------------------
# funkcja pobiera tre pliku z podanego numeru
#-------------------------------------------------------------------------------

def PobierzZwyklyPlikZSerwera(Plik):
    try:
        #mamy plik i katalog do pobrania
        import binascii
        #WS 2021-11-03
        with open(Plik,'rb') as p:
            Tresc=binascii.hexlify(buffer(p.read()))
        # Tresc=binascii.hexlify(buffer(open(Plik,'rb').read()))
    except Exception,e:
        return 'Bd wczytania zwykego pliku z serwera : '+str(e),''
    
    return 'OK',Tresc

#-------------------------------------------------------------------------------
# funkcja usuwajca plik z dysku
#-------------------------------------------------------------------------------

def UsunPlikZSerwera(Katalog,NrPliku):
    try:
        # pobranie cieki pliku 
        KatPlik,Plik=PodajNazwePliku(NrPliku)
        KatPlik=Katalog+KatPlik 
        
        # usuwamy plik
        os.remove(KatPlik+'/'+Plik)
    except Exception,e:
        return 'Bd usuwania pliku z serwera : '+str(e)
    
    return 'OK'

#-------------------------------------------------------------------------------
# odczytuje plik o wskazanej sciezce, pakuje do zipa i koduje szsnastkowo do przesylki do klienta
#-------------------------------------------------------------------------------
#AB 2023-08-23 Zmiana opisu i nazwy funkcji oraz argumentu
def PrzygotujPlikDoPrzeslania(sciezka_pliku):
    # spakowanie pliku zipem
    try:
        NrPliku=str(random.randint(100,10000))
        z=zipfile.ZipFile(NrPliku+'.zip', "w")
        z.write(sciezka_pliku,'YouWontKnow',zipfile.ZIP_DEFLATED)
        z.close()
    
        # przygotowanie pliku do wysania
        #WS 2021-11-03
        with open(NrPliku+'.zip','rb') as p:
            Tresc = binascii.hexlify(buffer(p.read()))
        # Tresc=binascii.hexlify(buffer(open(NrPliku+'.zip','rb').read()))
        
        # usunicie pliku zip
        os.remove(NrPliku+'.zip')
    except Exception,e:
        if os.path.exists(NrPliku+'.zip'):
            os.remove(NrPliku+'.zip')
        rsglobal.ZapiszDoLogow('Bd pobierania danych z wybranego pliku '+sciezka_pliku+' : '+str(e))
        return None

    return Tresc

#AB 2023-09-06
def PodajTrescPliku(Plik):  # przestarzaa nazwa metody
    return PrzygotujPlikDoPrzeslania(Plik)

#-------------------------------------------------------------------------------
# zapisuje tre do pliku o podanej nazwie, w podanym katalogu
#-------------------------------------------------------------------------------

#WS 2021-01-18
def ZapiszPlik(Tresc, Nazwa, Katalog, CzyZIP=True):
    if len(Tresc) == 0:
        return 'Plik jest pusty'

    if Katalog.strip() == '':
        Katalog = os.getcwd()
    if Nazwa.strip() == '':
        return 'Brak nazwy pliku'

    #AB 2023-07-24
    try:
        if not os.path.exists(Katalog):
            os.mkdir(Katalog)
    except OSError, exc:
        return 'Bd tworzenia katalogu: '+str(exc)

    KatalogPlik = os.path.join(Katalog, Nazwa)

    Tresc = binascii.unhexlify(Tresc)

    if CzyZIP:
        try:
            # zapisanie pliku zip
            NrPliku = str(random.randint(100, 10000))
            with open(os.path.join(Katalog, NrPliku+'.zip'), 'wb') as tmpz:
                tmpz.write(Tresc)

            # rozpakowanie pliku zip
            with zipfile.ZipFile(os.path.join(Katalog, NrPliku+'.zip'), "r") as z:
                Tresc = z.read('YouWontKnow')
        except zipfile.BadZipfile, e:
            return 'Bd rozpakowania pliku: '+str(e)
        except OSError, e:
            return 'Bd dostpu do pliku: '+str(e)

    # zapisanie treci do pliku
    try:
        with open(KatalogPlik, "wb") as p:
            p.write(Tresc)
    except OSError, e:
        return 'Bd zapisu do pliku: ' + str(e)

    # usunicie pliku zip
    if CzyZIP:
        os.remove(os.path.join(Katalog, NrPliku+'.zip'))

    return 'OK'


# def ZapiszPlikXLSZTresciCSV(Tresc,Nazwa=''):
#     if len(Tresc)==0:
#         return
#
#     Katalog=os.getcwd()
#
#
#
# def ZapiszPlik(Tresc,Nazwa=''):
#     if len(Tresc)==0:
#         return
#
#
#     if KatalogPlik=='':
#         Katalog=os.getcwd()
#     elif Nazwa<>'':
#         Katalog=KatalogPlik
#     else:
#         Katalog=os.path.dirname(KatalogPlik)
#         Nazwa=os.path.basename(KatalogPlik)
#
#     if Katalog=='' or Nazwa=='':
#         return
#
#     # mamy tre pliku zip
#     Tresc=binascii.unhexlify(Tresc)
#
#     # zapisanie pliku zip
#     NrPliku=str(random.randint(100,10000))
#     open(Katalog+'/'+NrPliku+'.zip', 'wb').write(Tresc)
#
#     # rozpakowanie pliku zip
#     z=zipfile.ZipFile(Katalog+'/'+str(NrPliku)+'.zip', "r")
#     Tresc=z.read('YouWontKnow')
#     z.close()
#
#     # zapisanie treci pliku - albo na dysk albo do pliku tmp
#     if KatalogPlik<>'':
#         open(Katalog+'/'+Nazwa, 'wb').write(Tresc)
#     else:
#         # jeli miao by tylko sprawdzenie to jest zapis tmp i odpalenie programu
#         Rozszerz=Nazwa[-3:]
#         if RSLogin.System == 1 and not Rozszerz in ['csv','pdf','html','sxw','xml']:
#             RSMess.RSFakt('Nieobsugiwany typ pliku - zapisz go na dysk')
#         else:
#             PlikTmp=tempfile.mktemp('.'+Rozszerz)
#             open(PlikTmp, 'wb').write(Tresc)
#
#             if RSLogin.System == 2:
#                 # windowsy
#                 os.startfile(PlikTmp)
#             elif Rozszerz=='csv':
#                 os.spawnlp(os.P_NOWAIT,RSLogin.programcsv,RSLogin.programcsv,PlikTmp)
#             elif Rozszerz=='pdf':
#                 os.spawnlp(os.P_NOWAIT,RSLogin.programpdf,RSLogin.programpdf,PlikTmp)
#             elif Rozszerz=='html':
#                 os.spawnlp(os.P_NOWAIT,RSLogin.programhtml,RSLogin.programhtml,PlikTmp)
#             elif Rozszerz=='sxw':
#                 os.spawnlp(os.P_NOWAIT,RSLogin.programsxw,RSLogin.programsxw,PlikTmp)
#             elif Rozszerz=='xml':
#                 os.spawnlp(os.P_NOWAIT,RSLogin.programxml,RSLogin.programxml,PlikTmp)
#
#
#     # usunicie pliku zip
#     os.remove(Katalog+'/'+NrPliku+'.zip')
