# -*- coding: iso8859_2 -*-

# -------------------------------------------------------------------------------------
# Name:        fs_pliki_office.py
# Purpose:     funkcje do zarzdzania plikami pakietu office
# Author:      Micha Raburski
# Created:     2012/06/26
# RCS-ID:      $Id: fs_pliki_office.py $
# Copyright:   (c) 2012 RecordSystem
# Licence:     GNU GPL
# -------------------------------------------------------------------------------------
#
# 2018-03-19, Aleksander Bargie 10961 - przygotowanie do bdw w tworzeniu pliku .xlsx
# 2018-03-29, Aleksander Bargie 10961 - przygotowanie do tworzenia pliku .xlsx w nowszych wersjach openpyxxl
# 2018-03-29, Aleksander Bargie 10961 - przygotowanie do tworzenia pliku .xlsx w nowszych wersjach openpyxxl - dodany cast dla poprawnego typu danych
# 2022-05-04  Wojciech Skowron   12969 - klasa RSDOCX
# 2023-11-08  Wojciech Skowron   13254 - przeniesienie klas RSRamka, RSStyl i UtworzXLS do UtworzXLS2 z klienta ze zmianami
# 2024-02-15  Wojciech Skowron   13695 - zwracany stat (traceback) z UtworzXLS
# 2024-02-20  Wojciech Skowron   13695 - zwracany stat z ZapiszXLSZDanych i ZapiszXLSGenerator

import datetime
import openpyxl
import os
import os.path
import rsglobal
import string
import sys

from openpyxl.styles import Alignment, Side, Border
from openpyxl.styles.numbers import FORMAT_GENERAL, FORMAT_NUMBER, FORMAT_NUMBER_COMMA_SEPARATED1, FORMAT_DATE_YYYYMMDD2, FORMAT_TEXT
from openpyxl.utils import get_column_letter

#AB 29.03.18r Dodanie informacji o wersji biblioteki
pyxlver = [int(x) for x in openpyxl.__version__.split('.')]
# Funkcja tworzca obiekt book wypeniajc go treci
# tre to lista = [[sheet1, ListaPl1],[sheet2, ListaPl2]]
# gdzie ListaPl = [[wiersz1,kolumna1,wartosc1,typpola1],[wiersz2,kolumna2,wartosc2,typpola2]]

# Niej nowsza wersja UtworzXLS2
def UtworzXLS(tresc,kodowanie='iso8859_2'):
    row = 0
    col = 0
    try:
    # utworzenie arkusza
        #AB 29.03.18r rna funkcja zalenie od wersji openpyxl
        if pyxlver[0] > 2 or (pyxlver[0] == 2 and pyxlver[1] >= 1):
            book = openpyxl.Workbook()
            # usunicie pierwszej zakadki utworzonej automatycznie podczas tworzenia workbooka
            book.remove_sheet(book.active)
        else:
            book = openpyxl.Workbook()
            # usunicie pierwszej zakadki utworzonej automatycznie podczas tworzenia workbooka w starej skadni
            book.remove_sheet(book.get_active_sheet())
        # kodowanie arkusza
        book.encoding = kodowanie
        # zapis kolejnych zakadek i ich danych
        ListaZakladek=[]
        i=0
        for zakladka in tresc:
            #AB 30.03.18r dodanie castowania na str i potem unicode bo nie zawsze w kodzie trafia tytul w odpowiednim formacie
            ListaZakladek.append(book.create_sheet(title=unicode(str(zakladka[0]))))
            for poz in zakladka[1]:
                #AB 29.03.18r od wersji 2 inna nazwa funkcji, liczenie rzdw  i kolumn od 1
                if pyxlver[0] <= 1:
                    row = poz[0]
                    col = poz[1]
                    ListaZakladek[i].cell(row=row, column=col).set_value_explicit(str(poz[2]), data_type=poz[3])
                    if len(poz)>4:
                        # ze stylem (nieuywane i pewnie wymaga poznania definiowania stylw)
                        ListaZakladek[i].cell(row=row, column=col).style = poz[4]
                else:
                    row = poz[0]+1
                    col = poz[1]+1
                    ListaZakladek[i].cell(row=row, column=col).set_explicit_value(str(poz[2]), data_type=poz[3])
                    if len(poz)>4:
                        # ze stylem (nieuywane i pewnie wymaga poznania definiowania stylw)
                        ListaZakladek[i].cell(row=row, column=col).style = poz[4]
            i+=1
        return 'OK', book
    except Exception, e: #AB zamiana print na zapiszdologow
        #WS 2024-02-15
        import traceback
        rsglobal.ZapiszDoLogow('Bd tworzenia arkusza XLS (wiersz: '+str(row)+', kolumna: '+str(col)+'): '+str(traceback.format_exc()))
        stat = 'Bd tworzenia arkusza XLS (wiersz: '+str(row)+', kolumna: '+str(col)+'): '+type(e).__name__+(': '+str(e) if str(e) != '' else '')
        return stat, None
        # rsglobal.ZapiszDoLogow('Bd tworzenia arkusza XLS : '+str(e), 1)
        # return None

#-------------------------------------------------------------------------------

#WS 2023-11-08
# Funkcja przeniesiona z RSPlikiOffice i dodana moliwo:
# - ustawienia formatu komrki n_00 (2 miejsca po przecinku) lub innego formatu zapisanego po 'n_', np. 'n_0.000'
# - ustawienia wysokoci wierszy (3 element listy w formie listy list, np. [[11, 80]] - wysoko 80 dla wiersza 11)
def UtworzXLS2(tresc,kodowanie='iso8859_2'):
    row = 0
    col = 0
    try:
        # utworzenie arkusza
        #AB 29.03.18r rna funkcja zalenie od wersji openpyxl
        if pyxlver[0] > 2 or (pyxlver[0] == 2 and pyxlver[1] >= 1):
            book = openpyxl.Workbook()
            # usunicie pierwszej zakadki utworzonej automatycznie podczas tworzenia workbooka
            book.remove_sheet(book.active)
        else:
            book = openpyxl.Workbook()
            # usunicie pierwszej zakadki utworzonej automatycznie podczas tworzenia workbooka w starej skadni
            book.remove_sheet(book.get_active_sheet())
        # kodowanie arkusza
        book.encoding = kodowanie
        # zapis kolejnych zakadek i ich danych
        ListaZakladek=[]
        i=0
        komorki = 0
        for zakladka in tresc:
            #AB 30.03.18r dodanie castowania na str i potem unicode bo nie zawsze w kodzie trafia tytul w odpowiednim formacie
            sheet = book.create_sheet(title=unicode(str(zakladka[0])))
            if len(zakladka) > 2:
                szerokosci = zakladka[2]
                for szer, kol_nr in zip(szerokosci, zakladka[1]):
                    sheet.column_dimensions[get_column_letter(int(kol_nr[1])+1)].width = float(szer/6)

            if len(zakladka) > 3:
                wysokosci = zakladka[3]
                for wiersz in wysokosci:
                    wiersz_nr = wiersz[0]
                    wys = wiersz[1]
                    sheet.row_dimensions[wiersz_nr].height = wys
                # sheet.row_dimensions[1].height = 30

            if len(zakladka) > 4:
                for merge in zakladka[4]:
                    sheet.merge_cells(merge)

            ListaZakladek.append(sheet)
            for poz in zakladka[1]:
                komorki += 1
                #AB 29.03.18r od wersji 2 inna nazwa funkcji, liczenie rzdw  i kolumn od 1
                if pyxlver[0] <= 1:
                    row = poz[0]
                    col = poz[1]
                    ListaZakladek[i].cell(row=row, column=col).set_value_explicit(str(poz[2]), data_type=poz[3])
                    if isinstance(poz[2], datetime.datetime):
                        ListaZakladek[i].cell(row=row, column=col).number_format = "YYYY-MM-dd"
                    if len(poz)>4:
                        styl = poz[4]
                        if styl.kolor_tla:
                            ListaZakladek[i].cell(row=row, column=col).fill = openpyxl.styles.fills.PatternFill(patternType='solid', fgColor=openpyxl.styles.colors.Color(rgb=styl.kolor_tla))

                        new_font           = ListaZakladek[i].cell(row=row, column=col).font
                        new_font           = new_font.copy(color=openpyxl.styles.colors.Color(rgb=styl.kolor_tekstu))
                        new_font.name      = styl.czcionka
                        new_font.bold      = styl.pogrubienie
                        new_font.italic    = styl.pochylenie
                        new_font.underline = styl.podkreslenie
                        new_font.size      = styl.rozmiar_tekstu
                        new_font.wrap      = styl.zawijac_tekst

                        ListaZakladek[i].cell(row=row, column=col).font = new_font
                        ListaZakladek[i].cell(row=row, column=col).border = styl.ramka.Border()

                else:
                    row = poz[0]+1
                    col = poz[1]+1
                    wart = poz[2]
                    if isinstance(poz[2], datetime.datetime):
                        wart = wart.strftime('%Y-%m-%d')
                    wart = str(wart)
                    if wart.startswith('='):
                        ListaZakladek[i].cell(row=row, column=col).value = wart
                    else:
                        if poz[3] in ('s', 'n'):
                            ListaZakladek[i].cell(row=row, column=col).set_explicit_value(wart, data_type=poz[3])
                        else:
                            ListaZakladek[i].cell(row=row, column=col).set_explicit_value(wart)
                    if len(poz)>4:
                        styl = poz[4]
                        typ_komorki = poz[3]
                        if styl.kolor_tla:
                            ListaZakladek[i].cell(row=row, column=col).fill = openpyxl.styles.fills.PatternFill(patternType='solid', fgColor=openpyxl.styles.colors.Color(rgb=styl.kolor_tla))

                        new_font = ListaZakladek[i].cell(row=row, column=col).font
                        new_font = new_font.copy(color=openpyxl.styles.colors.Color(rgb=styl.kolor_tekstu))
                        new_font.name = styl.czcionka
                        new_font.bold = styl.pogrubienie
                        new_font.italic = styl.pochylenie
                        if styl.podkreslenie:
                            new_font.underline = 'single'
                        new_font.size = styl.rozmiar_tekstu
                        zmiany_wyrownania = {}
                        if styl.zawijac_tekst:
                            zmiany_wyrownania['wrap_text']  = True
                        if styl.wyrownanie_w_pionie:
                            zmiany_wyrownania['vertical']   = styl.wyrownanie_w_pionie
                        if styl.wyrownanie_w_poziomie:
                            zmiany_wyrownania['horizontal'] = styl.wyrownanie_w_poziomie
                        new_font.wrap = styl.zawijac_tekst

                        ListaZakladek[i].cell(row=row, column=col).font = new_font
                        ListaZakladek[i].cell(row=row, column=col).border = styl.ramka.Border()
                        if zmiany_wyrownania:
                            ListaZakladek[i].cell(row=row, column=col).alignment = Alignment(**zmiany_wyrownania)

                        if isinstance(poz[2], datetime.datetime):
                            ListaZakladek[i].cell(row=row,
                                                  column=col).number_format = FORMAT_DATE_YYYYMMDD2
                        elif typ_komorki == 'n':
                            ListaZakladek[i].cell(row=row, column=col).number_format = FORMAT_NUMBER
                        elif typ_komorki == 'n_00':
                            ListaZakladek[i].cell(row=row, column=col).number_format = FORMAT_NUMBER_COMMA_SEPARATED1
                        elif typ_komorki == 't':
                            ListaZakladek[i].cell(row=row, column=col).number_format = FORMAT_TEXT
                        elif typ_komorki.startswith('n_'):
                            ListaZakladek[i].cell(row=row, column=col).number_format = typ_komorki[2:]


            i += 1
        return 'OK', book
    except Exception, e:
        #WS 2024-02-15 - wiersz i kolumna
        import traceback
        rsglobal.ZapiszDoLogow('Bd tworzenia arkusza XLS (wiersz: '+str(row)+', kolumna: '+str(col)+'): '+str(traceback.format_exc()))
        stat = 'Bd tworzenia arkusza XLS (wiersz: '+str(row)+', kolumna: '+str(col)+'): '+type(e).__name__+(': '+str(e) if str(e) != '' else '')
        return stat, None

#-------------------------------------------------------------------------------

def ZapiszXLS(tresc,plik,kodowanie='iso8859_2'):
    stat, book = UtworzXLS(tresc, kodowanie)  #WS 2024-02-15
    if stat == 'OK':
        try:  #AB save otoczone w try/except
            book.save(plik)
        except Exception, e:
            rsglobal.ZapiszDoLogow('Bd zapisu arkusza XLS: '+str(e), 1)

    return stat  #WS 2024-02-15
    
#-------------------------------------------------------------------------------
    
def PrzygotujTrescZCSV(tresccsv,Kodowanie='iso8859_2',ZAgaty=False):
    import csv
    import cStringIO
    
    Tresc=[]
    wier=0
    for wiersz in tresccsv:
        Wtmp = wiersz.replace('"','')
        LPol=csv.reader(cStringIO.StringIO(Wtmp),delimiter=';')
        kol=0
        # wiemy e jest jeden wiersz ale LPol[0] nie dziaa wic robimy to troch dziwnie
        for w in LPol:
            Pola=w
        
        for pole in Pola:
            pole=pole.strip('\n\r')
            # jeli z Agaty to robimy Dekoduj, oraz usuwamy cudzysw umieszczony dookoa tekstu
            if ZAgaty:
                pole=pole.strip('"')
            pole=pole.strip()
            # poniewa liczby przychodz jako teksty to sprawdzamy czy nie da si ich przerobi na liczb
            try:
                # data lub "nazwa z mylnikiem"
                if pole.find('-')>0:
                    Tresc.append([wier,kol,pole,'s'])
                else:
                    #powyej z analizy odrzucamy daty (liczby z minusem przejd)
                    if pole.find('.')<>-1 or pole.find(',')<>-1:
                        pole=float(pole.replace(' ','').replace('.','').replace(',','.'))
                        Tresc.append([wier,kol,pole,'n'])
                    else:
                        test=float(pole.strip())
                        # to jest liczba bez przecinka. Zakadamy e to pole tekstowe takie jak np. symbol artykuu
                        Tresc.append([wier,kol,pole,'s'])
            except Exception,e:
                Tresc.append([wier,kol,pole,'s'])
            
            kol+=1
        wier+=1
    return Tresc
    
#-------------------------------------------------------------------------------

def ZmienCSVwXLS(PlikCSV,PlikXLS,Kodowanie,Usuwac=False,CzyZAgaty=False):
    try:
        if os.path.exists(PlikCSV):
            f=open(PlikCSV,'r')
            Dane=f.readlines()
            f.close()
            if Usuwac:
                os.remove(PlikCSV)

            Tresc=[[u'Zakadka1',PrzygotujTrescZCSV(Dane,Kodowanie,CzyZAgaty)]]
            stat = ZapiszXLS(Tresc,PlikXLS,Kodowanie)
    except IOError,e:
        stat = 'Plik csv '+PlikCSV+' nie zosta otwarty: '+str(e)
        rsglobal.ZapiszDoLogow(stat)
    finally:
        return stat  #WS 2024-02-20

# -------------------------------------------------------------------------------

def ZapiszXLSZDanych(dane,plik, wyn1, naglowek='', kodowanie='iso8859_2',Kolumny=[]):
    rsglobal.ZapiszDoLogow('11',1)
    # parametry: dane, wyn1 bezporednio z funkcji self.PolSQL.Execute(sql,3),
    # plik-nazwa pliku z katalogiem,
    # naglowek - lista [wiersz0, wiersz1] gdzie kolejne elementy wstawiaj w kolumnach
    Tresc = [[u'Zakladka1',[]]]
    Typy = []
    rsglobal.ZapiszDoLogow('12',1)
    if Kolumny == []:
        TworzycKolumny = True
    else:
        TworzycKolumny = False
    rsglobal.ZapiszDoLogow('13',1)
    for typ in wyn1:
        if (typ[1] in ('str', 'text', 'varchar', 'char')):
            Typy.append('s')
        elif typ[1] == 'bool':
            Typy.append('b')
        elif (typ[1] in ('numeric', 'int')):
            Typy.append('n')
        elif typ[1] == 'date':
            Typy.append('d')

        if TworzycKolumny:
            Kolumny.append(typ[0])
    rsglobal.ZapiszDoLogow('14 '+str(Typy),1)
    i = 0
    for w in naglowek:
        Tresc[0][1].append([i,0,str(w),'s'])
        i += 1

    for w in range(len(Kolumny)):
        Tresc[0][1].append([i,w,Kolumny[w],'s'])
    i += 1
    rsglobal.ZapiszDoLogow('15',1)
    for wiersz in dane:
        j=0
        for kolumna in wiersz:
            wart = ZapiszPoleXml(str(kolumna),Typy[j])
            if Typy[j]=='d':
                Typy[j]='s'
            Tresc[0][1].append([i,j,wart,Typy[j]])
            j+=1
        i+=1
    rsglobal.ZapiszDoLogow('16',1)
    try:
        rsglobal.ZapiszDoLogow('TTT'+str(Tresc),1)
        stat, book = UtworzXLS(Tresc, kodowanie)  #WS 2024-02-15
        rsglobal.ZapiszDoLogow('1 '+plik,1)
        if stat == 'OK':
            book.save(plik)
            rsglobal.ZapiszDoLogow('2', 1)
    except Exception, e:
        stat = 'Bd tworzenia pliku xlsx: '+str(e)
        rsglobal.ZapiszDoLogow(stat)
        #AB 27.03.18r None na False
        # return False
    finally:
        return stat  #WS 2024-02-20

# -------------------------------------------------------------------------------

def ZapiszXLSGenerator(dane,plik, Cechy, naglowek='', kodowanie='iso8859_2'):
    # plik-nazwa pliku z katalogiem,
    # naglowek - lista [wiersz0, wiersz1] gdzie kolejne elementy wstawiaj w kolumnach
    # Cechy = [Nazwa,Typ,Dlugosc,PoPrzecinku,Kolejnosc,Grupowanie,Sumowac]

    Tresc = [['Zakladka1',[]]]
    i = 0
    for w in naglowek:
        Tresc[0][1].append([i,0,str(w),'s'])
        i += 1

    for w in range(len(Cechy[0])):
        Tresc[0][1].append([i,w,Cechy[0][w],'s'])
    i += 1

    for wiersz in dane:
        j=0
        for kolumna in wiersz:
            wart = ZapiszPoleXml(str(kolumna),Cechy[1][j].strip())
            if Cechy[1][j] in ['d','D']:
                Cechy[1][j]='s'
            if Cechy[1][j] in ['s','S']:
                wart = wart.strip()
            Tresc[0][1].append([i,j,wart,Cechy[1][j].strip()])
            j+=1
        i+=1
    try:
        stat, book = UtworzXLS(Tresc, kodowanie)  #WS 2024-02-15
        if stat == 'OK':
            book.save(plik)  #AB dodano zwracanie Tru przy sukcesie
            # return True
    except Exception, e:
        #AB: zmieniony print na zapiszdologow, zwracamy stat zamiast None
        stat = 'Bd tworzenia pliku xlsx: ' + str(e)
        rsglobal.ZapiszDoLogow(stat, 1)
        #AB 27.03.18r None na False
        # return False
    finally:
        return stat  #WS 2024-02-20

# -------------------------------------------------------------------------------

def ZapiszPoleXml(wart, typ):
    if typ in ['s','S']:
        t = str(wart)
        if t == 'None':
            t = ''
        return t
    elif typ in ['n','N']:
        if wart == 'None':
            wart = 0
        return str(wart)
    elif typ in ['b','B']:
        if wart == 1:
            t = 'T'
        else:
            t = 'F'
        return t
    elif typ in ['d','D']:
        t = str(wart)[0:10]
        return t
    elif typ in ['t','T]']:
        t = str(wart)[0:16]
        return t

# -------------------------------------------------------------------------------

def ZapiszCSVGenerator(dane,plik, Cechy, naglowek=[''], kodowanie='iso8859_2'):
    # plik-nazwa pliku z katalogiem,
    # naglowek - lista [wiersz0, wiersz1] gdzie kolejne elementy wstawiaj w kolumnach
    # Cechy = [Nazwa,Typ,Dlugosc,PoPrzecinku,Kolejnosc,Grupowanie,Sumowac]
    try:
        Plik = open(plik, 'w+b')
        sys.stdout = Plik
        #zapisuje nagwek
        for w in naglowek:
            ZapiszWierszCsv([w,],'s')

        #zapisuje kolumny
        str = []
        for w in range(len(Cechy[0])):
            str.append('s')
        ZapiszWierszCsv(Cechy[0],str)

        #zapisuje Dane
        for w in dane:
            ZapiszWierszCsv(w, Cechy[1])

        if Plik != None:
            Plik.close()
    except:
        print u'Bd zapisu do pliku csv'
        sys.stdout = sys.__stdout__

    sys.stdout = sys.__stdout__

#-------------------------------------------------------------------------------

def ZapiszWierszCsv(wiersz, typ):
    for i in range(len(wiersz)):
        ZapiszPoleCsv(wiersz[i],typ[i].strip())
    print chr(13)

#-------------------------------------------------------------------------------

def ZapiszPoleCsv(wart,typ):
    if typ in ['s','S']:
        t = str(wart)
        if t=='None':
            t=''
        print t+';',
    elif typ in ['n','N']:
        if wart == 'None':
            wart=0
        print str(wart).replace('.',',')+';',
    elif typ in ['b','B']:
        if wart==1:
            t='T'
        else:
            t='F'
        print t+';',
    elif typ in ['d','D']:
        t=str(wart)

#-------------------------------------------------------------------------------

#WS 2022-05-04
class RSDOCX:
    LEFT = 0
    CENTER = 1
    RIGHT = 2
    JUSTIFY = 3

    def __init__(self):
        from docx import Document

        self.document = Document()

    def Paragraf(self, tekst, styl, wyrownanie=-1):
        paragraf = self.document.add_paragraph('', style=styl)
        if wyrownanie > -1:
            paragraf.alignment = wyrownanie
        self._InterpretujZnaczniki(paragraf, tekst.replace('&nbsp;', ' '))
        return paragraf

    def DodajStylParagrafu(self, nazwa, font='DejaVu Serif', rozmiar=9, odstep_przed=0, odstep_za=0, wyrownanie=JUSTIFY):
        from docx.shared import Pt
        from docx.enum.style import WD_STYLE_TYPE
        from docx.enum.text import WD_LINE_SPACING

        style = self.document.styles.add_style(nazwa, WD_STYLE_TYPE.PARAGRAPH)
        style.font.name = font
        style.font.size = Pt(rozmiar)
        style.paragraph_format.space_before = Pt(odstep_przed)
        style.paragraph_format.space_after = Pt(odstep_za)
        style.paragraph_format.line_spacing_rule = WD_LINE_SPACING.SINGLE
        style.paragraph_format.alignment = wyrownanie

        return style

    def ZrobDok(self, firma_id, nazwa_pliku, gorny_margines, dolny_margines, lewy_margines, prawy_margines, numeruj_strony=True, stopka_anakondy=True):
        from docx.shared import Cm, Mm

        sections = self.document.sections
        sections[0].page_height = Mm(297)
        sections[0].page_width = Mm(210)
        sections[0].top_margin = Cm(gorny_margines)
        sections[0].bottom_margin = Cm(dolny_margines)
        sections[0].left_margin = Cm(lewy_margines)
        sections[0].right_margin = Cm(prawy_margines)

        if stopka_anakondy:
            footer = sections[0].footer
            sections[0].footer_distance = Cm(dolny_margines)
            styl = self.DodajStylParagrafu(nazwa='Stopka', rozmiar=8, wyrownanie=self.CENTER)
            footer.add_paragraph('Dokument wygenerowany przez system Anakonda.', styl)

        if numeruj_strony:
            footer = sections[0].footer
            sections[0].footer_distance = Cm(dolny_margines)
            styl = self.DodajStylParagrafu(nazwa='Strony', rozmiar=8, wyrownanie=self.RIGHT)
            footer.add_paragraph('', styl)
            self._DodajNumeryStron(self.document.sections[0].footer.paragraphs[-1])

        self.document.save(nazwa_pliku)

    def _InterpretujZnaczniki(self, paragraf, tekst, znaczniki=[]):
        import re
        x = re.search('^</*([biu0]|sup)>', tekst)
        if x is not None:
            pocz = x.end()
            znaczniki.append(x.group())
        else:
            pocz = 0

        fragm = tekst[pocz:]

        y = re.search('</*([biu]|sup)>', fragm)
        if y is not None:
            koniec = y.start()
            fragm = fragm[:koniec]
        else:
            koniec = -1

        para_run = paragraf.add_run(fragm)
        for znacznik in znaczniki:
            if znacznik == '<b>':
                para_run.bold = True
            elif znacznik == '<i>':
                para_run.italic = True
            elif znacznik == '<u>':
                para_run.underline = True
            elif znacznik == '</b>':
                para_run.bold = False
            elif znacznik == '</i>':
                para_run.italic = False
            elif znacznik == '</u>':
                para_run.underline = False
            elif znacznik == '<sup>':
                para_run.font.superscript = True
            elif znacznik == '</sup>':
                para_run.font.superscript = False

        if koniec != -1:
            self._InterpretujZnaczniki(paragraf, tekst[pocz:][koniec:], znaczniki)

    def _UtworzElement(self, name):
        from docx.oxml import OxmlElement
        return OxmlElement(name)

    def _UstawAtrybut(self, element, name, value):
        from docx.oxml import ns
        element.set(ns.qn(name), value)

    def _DodajNumeryStron(self, paragraf):
        page_run = paragraf.add_run()
        t1 = self._UtworzElement('w:t')
        self._UstawAtrybut(t1, 'xml:space', 'preserve')
        t1.text = 'Strona '
        page_run._r.append(t1)

        page_num_run = paragraf.add_run()

        fldChar1 = self._UtworzElement('w:fldChar')
        self._UstawAtrybut(fldChar1, 'w:fldCharType', 'begin')

        instrText = self._UtworzElement('w:instrText')
        self._UstawAtrybut(instrText, 'xml:space', 'preserve')
        instrText.text = "PAGE"

        fldChar2 = self._UtworzElement('w:fldChar')
        self._UstawAtrybut(fldChar2, 'w:fldCharType', 'end')

        page_num_run._r.append(fldChar1)
        page_num_run._r.append(instrText)
        page_num_run._r.append(fldChar2)

        of_run = paragraf.add_run()
        t2 = self._UtworzElement('w:t')
        self._UstawAtrybut(t2, 'xml:space', 'preserve')
        t2.text = ' / '
        of_run._r.append(t2)

        fldChar3 = self._UtworzElement('w:fldChar')
        self._UstawAtrybut(fldChar3, 'w:fldCharType', 'begin')

        instrText2 = self._UtworzElement('w:instrText')
        self._UstawAtrybut(instrText2, 'xml:space', 'preserve')
        instrText2.text = "NUMPAGES"

        fldChar4 = self._UtworzElement('w:fldChar')
        self._UstawAtrybut(fldChar4, 'w:fldCharType', 'end')

        num_pages_run = paragraf.add_run()
        num_pages_run._r.append(fldChar3)
        num_pages_run._r.append(instrText2)
        num_pages_run._r.append(fldChar4)

#WS 2023-11-08
class RSRamka:
    def __init__(self,
                 lewa=None, lewa_kolor='FF000000', prawa=None, prawa_kolor='FF000000', gora=None,
                 gora_kolor='FF000000', dol=None, dol_kolor='FF000000'):
        self.lewa         = lewa
        self.lewa_kolor   = lewa_kolor
        self.prawa        = prawa
        self.prawa_kolor  = prawa_kolor
        self.gora         = gora
        self.gora_kolor   = gora_kolor
        self.dol          = dol
        self.dol_kolor    = dol_kolor

    def Border(self):
        argumenty = {}
        if self.lewa is not None:
            argumenty['left'] = Side(style=self.lewa)
        if self.prawa is not None:
            argumenty['right'] = Side(style=self.prawa)
        if self.gora is not None:
            argumenty['top'] = Side(style=self.gora)
        if self.dol is not None:
            argumenty['bottom'] = Side(style=self.dol)
        return Border(**argumenty)

#WS 2023-11-08
class RSStyl:
    def __init__(self, wzor=None, kolor_tekstu=None, czcionka=None, pogrubienie=None, ramka=None, grubosc_ramki=None,
                 kolor_tla=None, pochylenie=None, podkreslenie=None, rozmiar_tekstu=None, zawijac_tekst=None,
                 wyrownanie_w_poziomie=None, wyrownanie_w_pionie=None):
        self.kolor_tekstu          = 'FF000000'
        self.czcionka              = 'DejaVu'
        self.pogrubienie           = False
        self.ramka                 = RSRamka()
        self.grubosc_ramki         = None
        self.kolor_tla             = None
        self.pochylenie            = False
        self.podkreslenie          = False
        self.rozmiar_tekstu        = 11
        self.zawijac_tekst         = False
        self.wyrownanie_w_pionie   = None  # bottom, center, justify, top, distributed
        self.wyrownanie_w_poziomie = None  # center, left, general, justify, centerContinuous, right, fill, distributed
        if wzor:
            self.kolor_tekstu           = wzor.kolor_tekstu
            self.czcionka               = wzor.czcionka
            self.pogrubienie            = wzor.pogrubienie
            self.ramka                  = wzor.ramka
            self.grubosc_ramki          = wzor.grubosc_ramki
            self.kolor_tla              = wzor.kolor_tla
            self.pochylenie             = wzor.pochylenie
            self.podkreslenie           = wzor.podkreslenie
            self.rozmiar_tekstu         = wzor.rozmiar_tekstu
            self.zawijac_tekst          = wzor.zawijac_tekst
            self.wyrownanie_w_pionie    = wzor.wyrownanie_w_pionie
            self.wyrownanie_w_poziomie  = wzor.wyrownanie_w_poziomie
        self.kolor_tekstu           = kolor_tekstu           if kolor_tekstu            is not None else self.kolor_tekstu
        self.czcionka               = czcionka               if czcionka                is not None else self.czcionka
        self.pogrubienie            = pogrubienie            if pogrubienie             is not None else self.pogrubienie
        self.ramka                  = ramka                  if ramka                   is not None else self.ramka
        self.grubosc_ramki          = grubosc_ramki          if grubosc_ramki           is not None else self.grubosc_ramki
        self.kolor_tla              = kolor_tla              if kolor_tla               is not None else self.kolor_tla
        self.pochylenie             = pochylenie             if pochylenie              is not None else self.pochylenie
        self.podkreslenie           = podkreslenie           if podkreslenie            is not None else self.podkreslenie
        self.rozmiar_tekstu         = rozmiar_tekstu         if rozmiar_tekstu          is not None else self.rozmiar_tekstu
        self.zawijac_tekst          = zawijac_tekst          if zawijac_tekst           is not None else self.zawijac_tekst
        self.wyrownanie_w_pionie    = wyrownanie_w_pionie    if wyrownanie_w_pionie     is not None else self.wyrownanie_w_pionie
        self.wyrownanie_w_poziomie  = wyrownanie_w_poziomie  if wyrownanie_w_poziomie   is not None else self.wyrownanie_w_poziomie

    def __str__(self):
        return """                self.kolor_tekstu   = {self.kolor_tekstu}
                  self.czcionka       = {self.czcionka}
                  self.pogrubienie    = {self.pogrubienie}
                  self.ramka          = {self.ramka}
                  self.grubosc_ramki  = {self.grubosc_ramki}
                  self.kolor_tla      = {self.kolor_tla}
                  self.pochylenie     = {self.pochylenie}
                  self.podkreslenie   = {self.podkreslenie}
                  self.rozmiar_tekstu = {self.rozmiar_tekstu}
                  self.zawijac_tekst  = {self.zawijac_tekst}""".format(**locals())
