# -*- coding: iso8859_2 -*-
#-------------------------------------------------------------------------------
# Name:        fs_reklamacje_dane.py
# Purpose:     edycja pojedynczej reklamacji
# Author:      Mariusz Rajfura
# Created:     2012/03/21
# RCS-ID:      $Id: fs_reklamacje_dane.py $
# Copyright:   (c) 2012 Record System
# Licence:     GNU GPL
#-------------------------------------------------------------------------------
from funkcje import fs_pliki_global

# 2012-04-01 Mariusz Rajfura  popr.numer.nowej, reklamacja konsumencka (5607,5595)
# 2012-04-23 Mariusz Rajfura  modyfikacja edycji (5670)
# 2020-10-25 Wojciech Skowron (12170) dodawanie reklamacji tylko z kontrahentem zdefiniowanym dla operatora,
#                                     tworzenie i zamykanie MM dla sklepw firmowych
# 2020-12-14 Jan Raburski (12223) rezygnacja z zamykania MM, tworzenie MM dla oddzialu produkujacegi,
#                                     przepinanie reklamacji do oddzialu produkujacego.
#     w kluczu pojawil sie oddzialid, generalnie tozsamy z self.oddzial_id, moze sie roznic
#     reklamacji ze sklepow firmowych, ktore edytuja reklamacje z oddzialow do ktorych nie maja uprawnien
#     dodawania reklamacji zawsze w aktywnym oddziale
# 2021-02-12 Aleksander Bargie (12327) [SP] Zmiany w reklamacjach odbiorcw
# 2023-03-20 Wojciech Skowronn  (13380) Wysanie informacji e-mail przy zamkniciu reklamacji
# 2023-04-17 Aleksander Bargie (13379) Reklamacja firmowe: zdjcia

Fun = klucze[-1]

if Fun == 15:
    PlikId = rsglobal.Dekoduj(klucze[0])
else:
    Data = rsglobal.Dekoduj(klucze[0])
    Numer = rsglobal.Dekoduj(klucze[1])
    OddzialId = rsglobal.Dekoduj(klucze[2])
    firma_id = self.firma_id
    if not OddzialId:
        OddzialId = self.oddzial_id
    if Fun == 11:
        zawartoscPliku = klucze[3]
        nazwaPliku = klucze[4]


stat = 'OK'
dane = []
wyn1 = []
wyn2 = []
BladWysylaniaEmail = None  #WS 2023-03-20
zmiana_uznana = False  #WS 2023-03-20

IloscDostarczona = 0
IloscReklamowana = 0

FunWyslaniaMaila = 0
JestMM = False  #WS 2020-10-25
BladTworzeniaMM = None  #WS 2020-10-25

# mimo OddzialId jako parametru uprawnienia sa sprawdzane dla self.oddzial_id
# inny oddzial moze pojawic sie jedynie w przypadku operatora ktory ma wpisanego kontrahenta
# i reklamacja zostala przepieta do innego oddzialu przy tworzeniu MM
UprDodawanie = ((self.oddzial_id, '027103') in self.uprawnienia or (self.operator_id == STANDARD_OPERATOR_ID))
UprEdycja = ((self.oddzial_id, '027102') in self.uprawnienia or UprDodawanie or (self.operator_id == STANDARD_OPERATOR_ID))
UprZmianaStanu = ((self.oddzial_id, '027106') in self.uprawnienia or (self.operator_id == STANDARD_OPERATOR_ID))

#-------------------------------------------------------------------------------
# SPRAWDZENIE UPRAWNIEN
#-------------------------------------------------------------------------------

if Fun in (1,2) and stat == 'OK':
    if not UprDodawanie:
        stat = 'BR_UPR_INS'

if Fun in (3,4) and stat == 'OK':
    if not UprEdycja:
        stat = 'BR_UPR_UPD'


#-------------------------------------------------------------------------------
# KONTROLA DANYCH
#-------------------------------------------------------------------------------

#WS 2020-10-25 - pobranie kontrahent_id z rs_operatorzy
if Fun in (1,2,3,4,5) and stat == 'OK':
    KontrahentOp = ''

    SEL = "SELECT kontrahent_id FROM rs_operatorzy WHERE operator_id={}".format(self.operator_id)
    stat,dane_op,dump,dump = self.PolSQL.Execute(SEL, 3)
    if stat != 'OK':
        stat = 'Bd pobrania kontrahent_id dla zalogowanego operatora: '+stat
    elif len(dane_op) > 0:
        KontrahentOp = dane_op[0][0] if dane_op[0][0].strip() not in ('None', '') else ''


if Fun in (1,2) and stat == 'OK':
    if Data == '   -  -  ':
        dane.append('BR_DATA')
    elif not rsglobal.SprawdzCzyToData(Data):
        dane.append('BL_DATA')

#WS 2020-10-25 - Fun 6 - sama prba utworzenia i zamknicia MM
if Fun in (3,4,6) and stat == 'OK':
# if Fun in (3,4) and stat == 'OK':
    # blokowanie i pobranie potrzebnych do sprawdzenia danych przy modyfikacji
    sql ="SELECT ilosc_dostarczona,ilosc_reklamowana,"
    sql+=" CASE WHEN uznana IS NULL THEN 'R'::TEXT WHEN uznana THEN 'U'::TEXT"
    sql+="  ELSE 'N'::TEXT END,konsumencka,TRIM(COALESCE(nr_paragonu,''::TEXT)),"
    sql+=" COALESCE(kontrahent_id,''::TEXT),"
    sql+=" artykul_id,"  #WS 2020-10-25
    sql+=" magazyn_id IS NOT NULL AND dokument_id IS NOT NULL AND data_dm IS NOT NULL AND numer_dm IS NOT NULL"  #WS 2020-10-25
    sql+=" FROM rs_reklamacje"
    sql+=" WHERE firma_id='"+self.firma_id+"' AND oddzial_id='"+OddzialId+"'"
    sql+=" AND data='"+Data+"' AND numer="+str(Numer)+" FOR UPDATE NOWAIT;"
    
    stat,Bylo,nag,dump = self.PolSQL.Execute(sql, 3)
    if stat != 'OK':
        stat = 'Bd sprawdzenia istnienia reklamacji: '+stat
    elif len(Bylo) == 0:
        stat = 'BL_KLUCZ_NIST'
    else:
        IloscDostarczona = Bylo[0][0]
        IloscReklamowana = Bylo[0][1]
        Uznana = rsglobal.Dekoduj(Bylo[0][2])
        Konsumencka = rsglobal.Dekoduj(Bylo[0][3])
        NrParagonu = rsglobal.Dekoduj(Bylo[0][4])
        Kontrahent = rsglobal.Dekoduj(Bylo[0][5])
        Artykul = rsglobal.Dekoduj(Bylo[0][6])  #WS 2020-10-25
        JestMM = Bylo[0][7]  #WS 2020-10-25
        
if Fun in (1,2,3,4) and stat == 'OK':
    # sprawdzenie zawartoci pl
    for i in xrange(len(pola)):
        pole = pola[i]
        wart = wartosci[i]
        
        if pole == 'forma_zgloszenia':
            wart = wart.strip()
            if wart == '':
                dane.append('BR_FORMA_ZGLOSZENIA')
            else:
                wart = "'"+wart+"'"
                sql = "SELECT kod FROM rs_reklamacje_formy_zgloszenia"
                sql += " WHERE firma_id='"+self.firma_id+"' AND kod="+wart+";"
                stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
                if stat != 'OK':
                    dane.append('Bd sprawdzenia kodu formy zgoszenia: '+stat)
                elif len(d) == 0:
                    dane.append('BL_FORMA_ZGLOSZENIA')

        elif pole == 'data_dostawy':
            if wart == '    -  -  ':
                dane.append('BR_DATA_DOSTAWY')
            elif not rsglobal.SprawdzCzyToData(wart):
                dane.append('BL_DATA_DOSTAWY')
            else:
                sql = "SELECT '"+wart+"'::DATE>'"+Data+"'::DATE;"
                stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
                if stat != 'OK':
                    dane.append('Bd sprawdzenia daty dostawy: '+stat)
                elif d[0][0]:
                    dane.append('BL_DATA_DOSTAWY_ZA_POZNO')
            wart = "'"+wart+"'"

        elif pole == 'kontrahent_id':
            if KontrahentOp == '':  #WS 2020-10-25
                wart = wart.strip()
                Kontrahent = wart
                if wart == '':
                    wart = "NULL"
                else:
                    wart = "'"+wart+"'"
                    sql = "SELECT nazwa FROM rs_kontrahenci"
                    sql += " WHERE firma_id='" + self.firma_id + "' AND kontrahent_id="+wart
                    sql += " AND aktywny;"
                    stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
                    if stat != 'OK':
                        dane.append('Bd sprawdzenia kontrahenta: '+stat)
                    elif len(d) == 0:
                        dane.append('BL_KONTRAHENT')
            #WS 2020-10-25
            elif Fun in (1,2):
                wart = "'"+KontrahentOp.strip()+"'"
                Kontrahent = KontrahentOp

        elif pole == 'artykul_id':
            wart = wart.strip()
            Artykul = wart  #WS 2020-10-25
            if wart == '':
                dane.append('BR_ARTYKUL')
            else:
                wart = "'"+wart+"'"
                sql = "SELECT nazwa FROM rs_artykuly"
                sql += " WHERE firma_id='"+self.firma_id+"' AND artykul_id="+wart
                sql += " AND aktywny;"
                stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
                if stat != 'OK':
                    dane.append('Bd sprawdzenia artykuu: '+stat)
                elif len(d) == 0:
                    dane.append('BL_ARTYKUL')

        elif pole == 'zastrzezenie':
            wart = wart.strip()
            if wart == '':
                dane.append('BR_ZASTRZEZENIE')
            else:
                wart = "'"+wart+"'"
                sql = "SELECT kod FROM rs_reklamacje_typy_zastrzezen"
                sql += " WHERE firma_id='"+self.firma_id+"' AND kod="+wart+";"
                stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
                if stat != 'OK':
                    dane.append('Bd sprawdzenia kodu typu zastrzeenia: '+stat)
                elif len(d) == 0:
                    dane.append('BL_ZASTRZEZENIE')

        elif pole == 'tryb_postepowania':
            wart = wart.strip()
            if wart == '':
                wart = "NULL"
            else:
                wart = "'"+wart+"'"
                sql = "SELECT kod FROM rs_reklamacje_tryby_postepowania"
                sql += " WHERE firma_id='"+self.firma_id+"' AND kod="+wart+";"
                stat, d, dump, dump = self.PolSQL.Execute(sql,3)
                if stat != 'OK':
                    dane.append('Bd sprawdzenia kodu trybu postpowania: '+stat)
                elif len(d) == 0:
                    dane.append('BL_TRYB_POSTEPOWANIA')

        elif pole == 'przyczyna_niezgodnosci':
            wart = wart.strip()
            if wart == '':
                wart = "NULL"
            else:
                wart = "'"+wart+"'"
                sql = "SELECT kod FROM rs_reklamacje_przyczyny_niezgodnosci"
                sql += " WHERE firma_id='"+self.firma_id+"' AND kod="+wart+";"
                stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
                if stat != 'OK':
                    dane.append('Bd sprawdzenia kodu przyczyny niezgodnoci: '+stat)
                elif len(d) == 0:
                    dane.append('BL_PRZYCZYNA_NIEZGODNOSCI')

        elif pole == 'ilosc_dostarczona':
            IloscDostarczona = wart
            if wart <= 0:
                dane.append('BL_ILOSC_DOSTARCZONA')
            wart = str(wart)

        elif pole == 'ilosc_reklamowana':
            IloscReklamowana = wart
            if wart <= 0:
                dane.append('BL_ILOSC_REKLAMOWANA')
            wart = str(wart)

        elif pole == 'zglaszajacy':
            wart =  wart.strip()
            if wart == '':
                dane.append('BR_ZGLASZAJACY')
            wart = "'"+wart+"'"

        elif pole == 'przyjmujacy':
            wart = wart.strip()
            if wart == '':
                dane.append('BR_PRZYJMUJACY')
            wart = "'"+wart+"'"

        elif pole == 'uznana':
            wart = wart.strip()
            
            if Fun in [3,4] or wart in ['U','N']:
                if not UprZmianaStanu :
                    dane.append('BR_UPR_INS')
                    
            if not 'BR_UPR_INS' in dane:
                if wart == 'U':
                    wart = "TRUE"
                    FunWyslaniaMaila=10
                elif wart == 'N':
                    wart = "FALSE"
                    FunWyslaniaMaila=11
                else:
                    wart = "NULL"
                    FunWyslaniaMaila=9
            
        elif pole == 'konsumencka':
            Konsumencka = wart
            wart = str(wart)

        elif pole in ('zastrzezenie_opis','sposob_zalatwienia',
                      'arkusz_niezgodnosci','informacja_zwrotna'):
            if wart == '':
                wart = 'NULL'
            else:
                wart = "'"+wart+"'"

        elif pole == 'nr_paragonu':
            wart = wart.strip()
            NrParagonu = wart
            if wart == '':
                wart = 'NULL'
            else:
                wart = "'"+wart+"'"

        #WS 2023-03-20
        elif pole == 'email_zwrotny':
            wart = wart.strip()
            if wart == '':
                wart = 'NULL'
            elif not rsglobal.SprawdzAdresEmail(wart):
                dane.append('BL_EMAIL_ZWROTNY')
            else:
                wart = "'"+wart+"'"
        
        # zachowanie wart do tworzenia sql-a
        wartosci[i] = wart


    if stat == 'OK':
        if Konsumencka:
            if NrParagonu == '':
                dane.append('BR_NR_PARAGONU')
        else:
            if Kontrahent == '':
                dane.append('BR_KONTRAHENT')

    # czy byly jakies bledy
    if stat == 'OK':
        if len(dane) > 0:
            stat = dane[0]

    if stat == 'OK':
        if IloscReklamowana > IloscDostarczona:
            stat = 'BL_IL_REKL_WIEKSZ_IL_DOST'
            dane.append(stat)


#-------------------------------------------------------------------------------
#---   DODAWANIE DANYCH (1 -select, 2 -update z pobraniem)
#-------------------------------------------------------------------------------

if Fun in (1,2) and stat == 'OK':
    sql = "SELECT COALESCE(MAX(numer),0)+1 FROM rs_reklamacje"
    sql += " WHERE firma_id='"+self.firma_id+"' AND oddzial_id='"+self.oddzial_id+"';"
    stat,d,dump,dump = self.PolSQL.Execute(sql,3)
    if stat != 'OK':
        stat = 'Bd pobrania nowego numeru dla reklamacji: '+stat
    else:
        Numer = d[0][0]
    #AB 15.02.21r
    if 'artykul_id' not in pola:
        stat = u"Nie podano artykuu."  # argument wymagany wiec jak nie ma to cos siadlo

    if stat == 'OK':
        art  = wartosci[pola.index('artykul_id')]
        data = wartosci[pola.index('data_dostawy')]
        sql  = """SELECT oddzial_id 
                  FROM rs_dowody_dostaw_pozycje rddp 
                  WHERE firma_id ='{}' AND artykul_id ={} AND data <= {} AND kontrahent_id='{}'
                  ORDER BY data DESC ;""".format(self.firma_id, art, data, Kontrahent)
        stat, Nowy_OddzialId, dump, dump = self.PolSQL.Execute(sql, 5)
        rsglobal.ZapiszDoLogow("Zmiana oddzialu z {} na {}".format(self.oddzial_id, OddzialId))
        if stat == 'OK' and Nowy_OddzialId == '':
            stat = 'BR_DD${} nie byo dostaw dla kontrahenta \'{}\' do {}, nie mona wprowadzi reklamacji.'.format(art, Kontrahent, data.strip("'"))

        if stat == 'OK' and KontrahentOp != '' and Nowy_OddzialId != '':
            OddzialId = Nowy_OddzialId

    if stat == 'OK':
        inspola = 'firma_id,oddzial_id,data_modyfikacji'
        inswart = "'"+self.firma_id+"','"+OddzialId+"',NOW()::DATE"
    
        for i in xrange(len(pola)):
            pole = pola[i]
            wart = wartosci[i]

            #WS 2020-10-25
            #WS 2020-12-21 - magazyn_dokad, stan
            if pole in ('magazyn_id', 'magazyn_dokad', 'stan', 'dokument_id', 'data_dm', 'numer_dm'):
                continue

            #WS 2023-03-20
            if pole in ('uznana', 'email_zwrotny'):
                if wart != 'NULL':
                    zmiana_uznana = True

            if pole == 'data':
                wart = "'"+Data+"'"
            elif pole == 'numer':
                wart = str(Numer)
            
            inspola += ","+pole
            inswart += ","+wart
    
    if stat == 'OK':
        INSERT = "INSERT INTO rs_reklamacje ("+inspola+") VALUES ("+inswart+");"
        stat,d,dump,dump = self.PolSQL.Execute(INSERT, 1)
        if stat != 'OK':
            stat = 'Bd dodawania nowej reklamacji: '+stat

        rsglobal.ZapiszDoLogow("Insert rekl {} \n stat{}".format(INSERT, stat))


#-------------------------------------------------------------------------------
#---   UPDATE DANYCH ( 3-select, 4 -update z pobraniem)
#-------------------------------------------------------------------------------

if Fun in (3,4) and stat == 'OK':
    # sprawdzamy czy mona edytowa reklamacj (jeli nie modyfikowano od 30 dni to nie)
    SQL="SELECT NOW()::DATE - COALESCE(data_modyfikacji,NOW()::DATE) FROM rs_reklamacje "
    SQL+=" WHERE firma_id='"+self.firma_id+"' AND oddzial_id='"+OddzialId+"'"
    SQL+=" AND data='"+Data+"' AND numer="+str(Numer)+";"
    stat,DModyf,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        stat = 'Bd pobrania informacji o datach reklamacji: '+stat
    elif len(DModyf) == 0:
        stat = 'Reklamacja o podanym kluczu nie istnieje.'
    elif DModyf[0][0] > 30:
        stat = 'Nie mona dokonywa zmian w reklamacji, bo bya ona modyfikowana wicej ni 30 dni temu'
    
    # dalsza cz modyfikacji
    UPD = 'data_modyfikacji = NOW()::DATE,'
    for i in xrange(len(pola)):
        pole = pola[i]
        wart = wartosci[i]

        #WS 2020-10-25
        #WS 2020-12-21 - magazyn_dokad, stan
        if pole in ('magazyn_id', 'magazyn_dokad', 'stan', 'dokument_id', 'data_dm', 'numer_dm'):
            continue

        #WS 2023-03-20
        if pole in ('uznana', 'email_zwrotny'):
            if wart != 'NULL':
                zmiana_uznana = True

        if pole != 'kontrahent_id' or KontrahentOp == '':  #WS 2020-10-25
            UPD += pole+"="+wart+","
            
    if stat == 'OK' and len(UPD) > 0:
        UPD = UPD[:-1]
        UPDATE ="UPDATE rs_reklamacje SET "+UPD
        UPDATE += " WHERE firma_id='"+self.firma_id+"' AND oddzial_id='"+OddzialId+"'"
        UPDATE += " AND data='"+Data+"' AND numer="+str(Numer)+";"

        stat, d, dump, dump = self.PolSQL.Execute(UPDATE, 1)
        if stat != 'OK':
            stat = 'Bd aktualizacji danych reklamacji: '+stat

#-------------------------------------------------------------------------------
#---   DODANIE ZACZNIKA (11)
#-------------------------------------------------------------------------------

#WS 2023-03-20 - jeli status zmieniony na uznan lub nie, wysyany e-mail
if Fun in (11, ) and stat == 'OK':
    if not UprEdycja:
        stat = "Brak uprawnie do modyfikacji reklamacji"

    if stat == 'OK':
        INSPLIKI = """INSERT INTO rs_pliki (  firma_id,    nazwa,   tabela,            kompresja)
                                 VALUES ('{firma_id}','{nazwaPliku}','rs_reklamacje_zalaczniki', FALSE);""".format(
            **locals())
        rsglobal.ZapiszDoLogow(INSPLIKI)
        stat, dump, dump, dump = self.PolSQL.Execute(INSPLIKI, 1, CzyDodawacBlad=True)
        if stat != 'OK':
            stat = "Bd dodawania pliku: {stat}".format(**locals())

    if stat == 'OK':
        SQLNumerPliku = """SELECT max(plik_id) 
                           FROM rs_pliki 
                           WHERE firma_id='{firma_id}' AND tabela='rs_reklamacje_zalaczniki';""".format(
            **locals())
        rsglobal.ZapiszDoLogow(SQLNumerPliku)
        stat, nrp, dump, dump = self.PolSQL.Execute(SQLNumerPliku, 3, CzyDodawacBlad=True)
        rsglobal.ZapiszDoLogow("pobrano numer: " + str(nrp))
        if stat != 'OK':
            stat = "Bd pobierania numeru zacznika: {stat}".format(**locals())

    if stat == 'OK':
        NumerPliku = nrp[0][0]
        stat = fs_pliki_global.WstawPlikNaSerwer(self.KatalogPlikow, NumerPliku, zawartoscPliku)

        if stat != 'OK':
            stat = "Bd wstawiania pliku na serwer: {stat}".format(**locals())
        else:
            INSZALACZNIKI = """INSERT INTO rs_reklamacje_zalaczniki
                                      (  firma_id,    oddzial_id,    data,     numer,   nr_pliku,        opis  )
                                VALUES ('{firma_id}','{OddzialId}','{Data}',  {Numer}, {NumerPliku},'{nazwaPliku}');""".format(
                **locals())

            rsglobal.ZapiszDoLogow(INSZALACZNIKI, 5)
            stat, dump, dump, dump = self.PolSQL.Execute(INSZALACZNIKI, 1, CzyDodawacBlad=True)
            if stat != 'OK':
                stat = 'Bd dodawania rekordu o nowym pliku: ' + stat
            else:
                dane = [{'nr_pliku': NumerPliku}]


#-------------------------------------------------------------------------------
#---   POBRANIE PLIKU (15)
#-------------------------------------------------------------------------------

if stat == 'OK' and Fun in (15, ):
    if stat == 'OK':
        stat, Tresc = fs_pliki_global.PobierzPlikZSerwera(self.KatalogPlikow, PlikId)
        dane = [Tresc]

#-------------------------------------------------------------------------------
#---   WYSANIE E-MAIL (1,2,3,4)
#-------------------------------------------------------------------------------

#WS 2023-03-20 - jeli status zmieniony na uznan lub nie, wysyany e-mail
if Fun in (1,2,3,4) and stat == 'OK' and zmiana_uznana:
    SELECT_EMAIL = """SELECT data, artykul_id, a.nazwa, ilosc_reklamowana, COALESCE(n.opis, 'brak') AS opis, 
                             CASE WHEN uznana IS NULL THEN 'R' ELSE (CASE WHEN uznana IS TRUE THEN 'T' ELSE 'F' END) END AS uznana, 
                             COALESCE(sposob_zalatwienia, 'brak') AS sposob_zalatwienia, email_zwrotny
                      FROM rs_reklamacje r
                      JOIN rs_artykuly a USING (firma_id, artykul_id)
                      LEFT OUTER JOIN rs_reklamacje_przyczyny_niezgodnosci n ON (n.firma_id=r.firma_id AND n.kod=r.przyczyna_niezgodnosci)
                      WHERE r.firma_id=%s AND r.oddzial_id=%s AND r.data=%s AND r.numer=%s;"""
    zmienne = [self.firma_id, OddzialId, Data, Numer]
    rsglobal.ZapiszDoLogow('Tre maila dane: '+self.PolSQL.Mogrify(SELECT_EMAIL, zmienne))
    stat_email, d_email, wyn_email, dump = self.PolSQL.Execute(SELECT_EMAIL, 3, zmienne)
    if stat_email != 'OK':
        BladWysylaniaEmail = "Bd przy pobieraniu danych do e-mail: "+stat_email
    else:
        dane_email = rssql.RSTableRowset2(d_email, wyn_email)[0]

        if dane_email['email_zwrotny'].strip() not in  ('None', '') and dane_email['uznana'] != 'R':
            email_temat = "Decyzja w sprawie reklamacji z dnia: "+dane_email['data']
            email_tresc = "Reklamacja dotyczya: "+dane_email['artykul_id'].strip()+' '+dane_email['nazwa']+'\n'
            email_tresc += "Reklamowana ilo: {:.2f}".format(dane_email['ilosc_reklamowana'])+'\n'
            email_tresc += "Opis niezgodnoci: "+dane_email['opis']+'\n'
            email_tresc += "Zostaa uznana\n" if dane_email['uznana'] else "Nie zostaa uznana\n"
            email_tresc += "Sposb zaatwienia reklamacji: "+dane_email['sposob_zalatwienia']
            stat_email = rsglobal.RSWyslijEmail(adresat=dane_email['email_zwrotny'],
                                                temat=email_temat, tresc=email_tresc, zalaczniki=[],
                                                operator_id=0, polsql=self.PolSQL)
            if stat_email != 'OK':
                BladWysylaniaEmail = "Bd przy wysyaniu wiadomoci e-mail: "+stat_email

#-------------------------------------------------------------------------------
#---   UTWORZENIE I ZAMKNICIE MM DLA SKLEPW FIRMOWYCH
#-------------------------------------------------------------------------------

#WS 2020-10-25
if Fun in (1,2,3,4,6) and stat == 'OK' and not JestMM:
    self.PolSQL.Commit()

    # Sprawdzenie, czy istnieje magazyn 'H'+kontrahent_id
    #JR 2023-01-10 (13303) zmiana z 'H' na 'D'
    # Magazyn = 'H'+Kontrahent.strip()
    Magazyn = 'D'+Kontrahent.strip()
    OddzialProd = OddzialId
    NowyNumer = 0
    statdm = 'OK'

    SEL = "SELECT 1 FROM rs_magazyny " \
          "WHERE firma_id='{}' AND magazyn_id='{}'".format(self.firma_id, Magazyn)
    stat, dane_mag, dump, dump = self.PolSQL.Execute(SEL, 2)
    if stat != 'OK':
        stat = 'Bd pobrania danych z rs_magazyny: '+stat
    elif len(dane_mag) > 0:
        #JR znalezienie oddzialu produkujacego  12223
        SEL = "SELECT data_dostawy FROM rs_reklamacje WHERE firma_id='" + self.firma_id + "'"
        SEL += " AND oddzial_id='" + OddzialId + "' AND data='" + Data + "' And numer=" + str(Numer)
        stat, dostawa, dump, dump = self.PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
        if stat != 'OK':
            stat = 'Bd pobrania daty dostawy z rs_reklamacje: '+stat
        elif len(dostawa) == 0:
            stat = 'Brak reklamacji przy tworzeniu MM - co nie tak.'
        else:
            datadostawy = str(dostawa[0][0])
            SEL = "SELECT oddzial_prod FROM rs_artykuly_producenci WHERE firma_id='" + self.firma_id + "'"
            SEL += " AND oddzial_zam='" + OddzialId + "' AND (data_od IS NULL OR data_od<='" + datadostawy + "')"
            SEL += " AND (data_do IS NULL OR data_do>='" + datadostawy + "') AND artykul_id='" + Artykul + "';"
            stat, oddzialy, dump, dump = self.PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
            if stat != 'OK':
                stat = 'Bd pobrania danych z rs_artykuly_producenci: '+stat
            elif len(oddzialy) > 1:
                stat = 'Bd tworzenia MM: wicej ni 1 producent dla oddziau '+ OddzialId
            elif len(oddzialy) == 1:
                OddzialProd = oddzialy[0][0]
            rsglobal.ZapiszDoLogow("producent:"+OddzialProd+" dostawa:"+str(datadostawy)+"sel:"+SEL)

        if stat == 'OK':
            # Znalazienie magazynu brakw dla oddziau produkujacego
            SEL = "SELECT magazyn_id FROM rs_magazyny " \
                  "WHERE firma_id='{}' AND oddzial_id='{}' AND brakow=TRUE".format(self.firma_id, OddzialProd)
            stat, dane_mag_braki, dump, dump = self.PolSQL.Execute(SEL, 2, CzyDodawacBlad=True)
            if stat != 'OK':
                stat = 'Bd pobrania danych z rs_magazyny: '+stat
            elif len(dane_mag_braki) == 0:
                stat = 'Bd tworzenia MM: brak magazynu brakw dla oddziau '+OddzialProd
            else:
                MagazynDokad = dane_mag_braki[0][0]

        if stat == 'OK':
            # Sprawdzenie, czy w magazynie jest ilo reklamowana
            #JR 12223 - zdjcie sprawdzania - MM ma nie by zamykane.
            #SEL = "SELECT ilosc1 FROM rs_artykuly_magazyny " \
            #      "WHERE firma_id='{}' AND magazyn_id='{}' AND artykul_id='{}'".format(self.firma_id, Magazyn, Artykul)
            #stat,dane_mag_ilosc,dump,dump = self.PolSQL.Execute(SEL, 2, CzyDodawacBlad=True)
            #if stat != 'OK':
            #    stat = 'Bd pobrania danych z rs_artykuly_magazyny: '+stat
            #elif len(dane_mag_ilosc) == 0:
            #    stat = 'Bd tworzenia MM: brak artykuu w magazynie '+Magazyn
            #elif dane_mag_ilosc[0][0] < IloscReklamowana:
            #    stat = 'Bd tworzenia MM: brak reklamowanej iloci artykuu w magazynie '+Magazyn
            #else:
                # Tworzenie MM
            if True:  #JR 12223 - zeby sie wciecia zgadzaly, nie wiadomo czy nie trzeba bedzie wrocic do sprawdzania
                # pobranie numeru dla nowego dokumentu
                sql = "SELECT rsf_podaj_nowy_numer_dokumentu('"+self.firma_id+"',"
                sql += "'[''"+Magazyn+"'',''MM'',''"+Data+"'']','MM');"

                stat, dane_nr_mag, dump, dump = self.PolSQL.Execute(sql, 2, CzyDodawacBlad=True)
                if stat != 'OK':
                    stat = 'Bd tworzenia MM: bd pobrania nowego numeru dla MM z magazynu ' + Magazyn
                else:
                    NumerDok = dane_nr_mag[0][0]

                if stat == 'OK':
                    # pobranie numeru w magazynie dokad
                    sql = "SELECT rsf_podaj_nowy_numer_dokumentu('"+self.firma_id+"',"
                    sql += "'[''"+MagazynDokad+"'',''MM'',''"+Data+"'']','MM');"

                    stat, dane_nr_mag_dokad, dump, dump = self.PolSQL.Execute(sql, 2, CzyDodawacBlad=True)
                    if stat != 'OK':
                        stat = 'Bd tworzenia MM: bd pobrania nowego numeru dla MM z magazynu ' + MagazynDokad
                    else:
                        NumerDokDokad = dane_nr_mag_dokad[0][0]

                if stat == 'OK':
                    DwieJM = False
                    # czy dwie jednostki miar
                    sql = "SELECT dwie_jedn_miar FROM rs_firmy WHERE firma_id='" + self.firma_id + "';"
                    stat, d, dump, dump = self.PolSQL.Execute(sql, 2, CzyDodawacBlad=True)
                    if stat != 'OK':
                        stat = 'Bd pobrania informacji o stosowaniu dwch jednostek miar: '+stat
                    elif len(d) == 0:
                        stat = 'Brak firmy o podanym kodzie ' + self.firma_id
                    else:
                        DwieJM = d[0][0]

                if stat == 'OK' and DwieJM:
                    sql = "SELECT przelicznik FROM rs_artykuly"
                    sql += " WHERE firma_id='"+self.firma_id+"' AND artykul_id='"+Artykul+"'"
                    sql += " AND aktywny;"
                    stat, d, dump, dump = self.PolSQL.Execute(sql, 2, CzyDodawacBlad=True)
                    if stat != 'OK':
                        stat = 'Bd pobrania przelicznika dla artykuu: ' + stat
                    elif len(d) == 0:
                        stat = 'Brak artykuu o kodzie '+Artykul
                    else:
                        Przelicznik = d[0][0]

                if stat == 'OK':
                    Ilosc1 = IloscReklamowana
                    Ilosc2 = Ilosc1
                    if DwieJM:
                        Ilosc2 = round(Ilosc1*Przelicznik, 4)

                    INS_Dok = "INSERT INTO rs_dokumenty_magazynowe"
                    INS_Dok += " (firma_id, magazyn_id, dokument_id,"
                    INS_Dok += " data, numer, numer_dokad,"
                    INS_Dok += " magazyn_skad, magazyn_dokad, kontrahent_id)"
                    INS_Dok += " VALUES "
                    INS_Dok += " ('" + self.firma_id + "','" + Magazyn + "','MM',"
                    INS_Dok += "'" + Data+"'," + str(NumerDok) + "," + str(NumerDokDokad) + ","
                    INS_Dok += "'"+Magazyn + "','" + MagazynDokad + "','" + Kontrahent + "'); "

                    INS_Poz = "INSERT INTO rs_dokumenty_magazynowe_pozycje"
                    INS_Poz += " (firma_id, magazyn_id, dokument_id, data,"
                    INS_Poz += "  numer, nr_poz, nr_ppoz, artykul_id,"
                    INS_Poz += "  ilosc1, ilosc2)"
                    INS_Poz += " VALUES"
                    INS_Poz += " ('" + self.firma_id + "','" + Magazyn + "','MM','" + Data + "',"
                    INS_Poz += "  "+str(NumerDok) + ",1,0,'" + Artykul + "',"
                    INS_Poz += "  " + str(Ilosc1) + "," + str(Ilosc2)+"); "

                    stat, d, dump, dump = self.PolSQL.Execute(INS_Dok+INS_Poz, 1, CzyDodawacBlad=True)
                    if stat != 'OK':
                        stat = 'Bd tworzenia MM: '+stat
                    else:
                        self.PolSQL.Commit()
                        rsglobal.ZapiszDoLogow('reklamacje MM:' + Magazyn + '/' + str(NumerDok))

                #JR tylko proba zamykania MM - 12223
                if stat == 'OK':
                    # Zamknicie MM, jak si nie uda to pomijamy bdy, operator w magazynie zrobi to kiedys

                    # zamknicie dokumentu z W na G
                    Klucze = {}
                    Klucze['Mag'] = Magazyn
                    Klucze['Typ'] = 'MM'
                    Klucze['Data'] = Data
                    Klucze['Numer'] = NumerDok
                    Klucze['Funkcja'] = 18
                    statdm,dump,dump,dump = self.WykonajFunkcje("fs_dokumenty_magazynowe", Klucze, [], [], False)
                    if statdm == 'OK':
                        # zamknicie dokumentu z G na Z
                        Klucze['Funkcja'] = 19
                        statdm,dump,dump,dump = self.WykonajFunkcje("fs_dokumenty_magazynowe", Klucze, [], [], False)

                if stat == 'OK':
                    # Udao si utworzy  MM, klucz dokumentu zostaje wpisany na reklamacji
                    UPDATE ="UPDATE rs_reklamacje SET magazyn_id='"+Magazyn+"', dokument_id='MM', "
                    UPDATE+=" data_dm='"+Data+"', numer_dm="+str(NumerDok)
                    UPDATE+=" WHERE firma_id='"+self.firma_id+"' AND oddzial_id='"+OddzialId+"'"
                    UPDATE+=" AND data='"+Data+"' AND numer="+str(Numer)+";"
                    rsglobal.ZapiszDoLogow('reklamacje update:' + UPDATE)
                    stat, d ,dump, dump = self.PolSQL.Execute(UPDATE, 1, CzyDodawacBlad=True)
                    if stat != 'OK':
                        stat = 'Bd przypisania dokumentu MM do reklamacji: '+stat

                if stat == 'OK' and OddzialProd != OddzialId:
                    #JR 12223 2020-12-14 przepiecie reklamacji do innego oddzialu
                    sql = "SELECT COALESCE(MAX(numer),0)+1 FROM rs_reklamacje"
                    sql += " WHERE firma_id='" + self.firma_id + "' AND oddzial_id='" + OddzialProd + "';"
                    stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
                    if stat != 'OK':
                        stat = 'Bd pobrania nowego numeru dla reklamacji: ' + stat
                    else:
                        NowyNumer = d[0][0]
                        UPDATE = "UPDATE rs_reklamacje SET oddzial_id='" + OddzialProd + "', "
                        UPDATE += " numer=" + str(NowyNumer)
                        UPDATE += " WHERE firma_id='"+self.firma_id+"' AND oddzial_id='"+OddzialId+"'"
                        UPDATE += " AND data='"+Data+"' AND numer="+str(Numer)+";"
                        # rsglobal.ZapiszDoLogow(UPDATE)
                        stat, d, dump, dump = self.PolSQL.Execute(UPDATE, 1, CzyDodawacBlad=True)
                        if stat != 'OK':
                            stat = 'Bd zmiany oddziau reklamacji: '+stat
                        else: #AB 14.01.21r
                            OddzialId = OddzialProd
                            Numer = NowyNumer

    # Bd przy tworzeniu MM nie blokuje dodania/aktualizacji reklamacji
    #if statdm != 'OK':
    #    BladTworzeniaMM = statdm
    #    stat = 'OK'
    #    dane = [BladTworzeniaMM]
    #else:  #JR 2020-12-21 12223 przekazanie zmiany klucza
    if OddzialProd != OddzialId:
        dane = [OddzialProd, NowyNumer]

# jeli trzeba wysya maila bo zmieni si status reklamacji
if stat == 'OK' and FunWyslaniaMaila >0:
    # uruchamiamy funkcj tworzc PDF i wysyajc maila
    stat,dump,dump,dump=self.WykonajFunkcje('fs_reklamacje',[[Data,str(int(Numer)),'',0, OddzialId],['','','','','','',''],[],FunWyslaniaMaila],[],[],False)


#-------------------------------------------------------------------------------
#---   POBRANIE DANYCH ( 5-select, 3 -update z pobraniem  1-insert z pobraniem )
#-------------------------------------------------------------------------------

if Fun in (5, 3, 1) and stat == 'OK':
    # pobranie pol z tabeli rs-reklamacje umozliwi dodatkowo pobranie wartosci
    # pola 'uznana' przetlumaczone na litery wg schematu zastosowanego w
    # wyswietleniu grida: True->U(znana), False->N(ieuznana), NULL->R(ozpatrywana)
    # poniewaz zwykly SELECT zwraca False, jesli uznana jest FALSE lub NULL
    # pola 'techniczne' zostaly dodane w SELECT aby RSEditF() nie mia problemow

    stat,Pola = rsglobal.PolaTabeli(self.PolSQL,'rs_reklamacje')
    if stat == 'OK':
        SELECT ="SELECT "
        for i in xrange(len(Pola)):
            SELECT += "r."+Pola[i]+","
        warunek  = "r.firma_id='"+self.firma_id+"' AND r.oddzial_id='"+OddzialId+"'"
        warunek +=" AND r.data='"+Data+"' AND r.numer="+str(Numer)+""
            
        SELECT+=" CASE WHEN uznana IS NULL THEN 'R'::TEXT WHEN uznana THEN 'U'::TEXT"
        SELECT+="  ELSE 'N'::TEXT END AS uznana,r.z_czas,r.z_typ,r.z_op_id,"
        SELECT+=" dm.magazyn_dokad,dm.stan"  #WS 2020-12-21
        SELECT+=" FROM rs_reklamacje r"
        SELECT+=" LEFT OUTER JOIN rs_dokumenty_magazynowe dm "  #WS 2020-12-21
        SELECT+=" ON (r.firma_id=dm.firma_id AND r.magazyn_id=dm.magazyn_id AND r.dokument_id=dm.dokument_id"
        SELECT+=" AND r.data_dm=dm.data AND r.numer_dm=dm.numer)"
        SELECT+=" WHERE " + warunek + ';'
    
        stat,dane,wyn1,dump = self.PolSQL.Execute(SELECT,3)
        if stat != 'OK':
            stat = 'Bd pobrania danych reklamacji.'

    # uzupelnie danych dodatkowych i opisow slownikowych
    if stat == 'OK' and len(dane) == 1:
        tr = rssql.RSTableRowset(dane,wyn1)
        
        # [0] stan reklamacji - nie uzywany obecnie
        wyn2.append(tr.GetVal(0, "forma_zgloszenia"))

        # [1] forma_zgloszenia
        kod = tr.GetVal(0, "forma_zgloszenia").strip()
        if kod in ('None', ''):
            wyn2.append('')
        elif (self.oddzial_id, '011970') in self.uprawnienia or self.operator_id == STANDARD_OPERATOR_ID:
            sql = "SELECT opis FROM rs_reklamacje_formy_zgloszenia"
            sql +=" WHERE firma_id='"+self.firma_id+"' AND kod='"+kod+"';"
            stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
            if stat != 'OK':
                wyn2.append('Bd pobrania opisu ze sownika form zgosze: '+stat)
            elif len(d) == 0:
                wyn2.append('Bdny kod formy zgoszenia reklamacji.')
            else:
                wyn2.append(rsglobal.Dekoduj(d[0][0]))
        else:
            wyn2.append('Brak uprawnie do sownika form zgosze. Poinformuj administratora.')

        # [2] kontrahent_id
        kod = tr.GetVal(0,"kontrahent_id").strip()
        if kod in ('None', ''):
            wyn2.append('')
        elif (self.oddzial_id,'012000') in self.uprawnienia or self.operator_id == STANDARD_OPERATOR_ID:
            sql = "SELECT nazwa FROM rs_kontrahenci"
            sql+=" WHERE firma_id='"+self.firma_id+"' AND kontrahent_id='"+kod+"';"
            stat,d,dump,dump = self.PolSQL.Execute(sql, 3)
            if stat != 'OK':
                wyn2.append('Bd pobrania opisu kontrahenta: '+stat)
            elif len(d) == 0:
                wyn2.append('Bdny kod kontrahenta.')
            else:
                wyn2.append(rsglobal.Dekoduj(d[0][0]))
        else:
            wyn2.append('Brak uprawnie do katalogu kontrahentw. Poinformuj administratora.')

        # [3] artykul_id
        kod = tr.GetVal(0, "artykul_id").strip()
        if kod in ('None', ''):
            wyn2.append('')
        elif (self.oddzial_id, '013000') in self.uprawnienia or self.operator_id == STANDARD_OPERATOR_ID:
            sql = "SELECT nazwa FROM rs_artykuly"
            sql +=" WHERE firma_id='"+self.firma_id+"' AND artykul_id='"+kod+"';"
            stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
            if stat != 'OK':
                wyn2.append('Bd pobrania opisu artykuu: '+stat)
            elif len(d) == 0:
                wyn2.append('Bdny kod artykuu.')
            else:
                wyn2.append(rsglobal.Dekoduj(d[0][0]))
        else:
            wyn2.append('Brak uprawnie do katalogu artykuw. Poinformuj administratora.')

        # [4] zastrzezenie
        kod = tr.GetVal(0, "zastrzezenie").strip()
        if kod in ('None', ''):
            wyn2.append('')
        elif (self.oddzial_id, '011980') in self.uprawnienia or self.operator_id == STANDARD_OPERATOR_ID:
            sql = "SELECT opis FROM rs_reklamacje_typy_zastrzezen"
            sql+=" WHERE firma_id='"+self.firma_id+"' AND kod='"+kod+"';"
            stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
            if stat != 'OK':
                wyn2.append('Bd pobrania opisu ze sownika typw zastrzee: '+stat)
            elif len(d) == 0:
                wyn2.append('Bdny kod typu zastrzeenia.')
            else:
                wyn2.append(rsglobal.Dekoduj(d[0][0]))
        else:
            wyn2.append('Brak uprawnie do sownika typw zastrzee. Poinformuj administratora.')

        # [5] tryb_postepowania
        kod = tr.GetVal(0,"tryb_postepowania").strip()
        if kod in ('None',''):
            wyn2.append('')
        elif (self.oddzial_id, '011990') in self.uprawnienia or self.operator_id == STANDARD_OPERATOR_ID:
            sql = "SELECT opis FROM rs_reklamacje_tryby_postepowania"
            sql +=" WHERE firma_id='"+self.firma_id+"' AND kod='"+kod+"';"
            stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
            if stat != 'OK':
                wyn2.append('Bd pobrania opisu ze sownika trybw postpowa: '+stat)
            elif len(d) == 0:
                wyn2.append('Bdny kod trybu postpowania.')
            else:
                wyn2.append(rsglobal.Dekoduj(d[0][0]))
        else:
            wyn2.append('Brak uprawnie do sownika trybw postpowa. Poinformuj administratora.')

        # [6] przyczyna_niezgodnosci
        kod = tr.GetVal(0,"przyczyna_niezgodnosci").strip()
        if kod in ('None',''):
            wyn2.append('')
        elif (self.oddzial_id, '211010') in self.uprawnienia or self.operator_id == STANDARD_OPERATOR_ID:
            sql = "SELECT opis FROM rs_reklamacje_przyczyny_niezgodnosci"
            sql +=" WHERE firma_id='"+self.firma_id+"' AND kod='"+kod+"';"
            stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
            if stat != 'OK':
                wyn2.append('Bd pobrania opisu ze sownika przyczyn niezgodnoci: '+stat)
            elif len(d) == 0:
                wyn2.append('Bdny kod przyczyny niezgodnoci.')
            else:
                wyn2.append(rsglobal.Dekoduj(d[0][0]))
        else:
            wyn2.append('Brak uprawnie do sownika przyczyn niezgodnoci. Poinformuj administratora.')
            
        # [7] sprawdzenie czy mona modyfikowa reklamacj (ostatnio modyfikowana mniej ni 30 dni temu)
        SQL="SELECT NOW()::DATE - COALESCE(data_modyfikacji,NOW()::DATE) FROM rs_reklamacje "
        SQL +=" WHERE firma_id='"+self.firma_id+"' AND oddzial_id='"+OddzialId+"'"
        SQL +=" AND data='"+Data+"' AND numer="+str(Numer)+";"
        stat, DModyf, dump, dump = self.PolSQL.Execute(SQL,3)
        if stat != 'OK':
            stat = 'Bd pobrania informacji o datach reklamacji: '+stat
        elif len(DModyf) == 0:
            stat = 'Reklamacja o podanym kluczu nie istnieje.'
        else :
            wyn2.append(DModyf[0][0] <= 30)

        #WS 2020-10-25
        wyn2.append(BladTworzeniaMM)
        #WS 2023-03-20
        wyn2.append(BladWysylaniaEmail)

        if stat == 'OK':
            SQLPliki = """SELECT *
                          FROM rs_reklamacje_zalaczniki r JOIN rs_pliki p ON p.firma_id=r.firma_id  AND p.plik_id=r.nr_pliku 
                          AND p.tabela ='rs_reklamacje_zalaczniki'
                          WHERE {warunek};""".format(**locals())

            rsglobal.ZapiszDoLogow(SQLPliki)
            stat, Pliki, PlikiWyn1, dump = self.PolSQL.Execute(SQLPliki, 3, CzyDodawacBlad=True)
            if stat != 'OK':
                stat = u"Bd pobierania plikw dla zgoszenia: {stat}".format(**locals())

            wyn2.append([Pliki, PlikiWyn1])  # kody[10]
