# -*- coding: iso8859_2 -*-
#-----------------------------------------------------------------------------
# Name:        fs_reklamacje_klientow_pdf.py
# Purpose:     Pobranie danych do PDF ze stanem zgoszenia reklamacyjnego
# Author:      Wojciech Skowron
# Created:     2019/12/12
# RCS-ID:      $Id: fs_reklamacje_klientow_pdf.py $
# Copyright:   (c) 2019 Record System
# Licence:     GNU GPL
#-----------------------------------------------------------------------------
# Aleksander Bargie 2020-02-11 - Reklamacje RODO - przypominajki w harmonogramie (11748)
# Wojciech Skowron   2021-06-29 - Imi i nazwisko operatora przy informacjach oraz pytaniach i odpowiedziach (12620)
# Wojciech Skowron   2021-11-03 - Poprawiona obsuga pliku (12174)
# Aleksander Bargie 2022-07-13 - Komunikat gdy tworzenie raportu pdf nie powiedzie si (12989)

class ReklamacjeKilentowPDF:
    def __init__(self, selfSerwera, ZgloszenieID, NazwaPliku=''):
        import rs_print_util
        import rsglobal
        rsglobal.ZapiszDoLogow(', '.join("%s: %s" % item for item in vars(selfSerwera).items()))
        rs_print_util.PrzygotujCzcionki()
        self.ZgloszenieID = ZgloszenieID
        self.NazwaPliku = NazwaPliku
        self.firma_id = selfSerwera.firma_id
        self.PolSQL = selfSerwera.PolSQL
        self.KatalogPlikow = selfSerwera.KatalogPlikow

    # Funkcje pomocnicze

    def PobierzDane(self):
        import binascii
        from funkcje import fs_pliki_global
        import rsglobal
        import tempfile
        import traceback
        import zipfile

        stat = "OK"

        # Nagwek
        SQL = """SELECT zgloszenie_id, data_zgloszenia, stan, tresc_zgloszenia, tresc_odpowiedzi, 
                        to_char(rozpatrzone, 'YYYY-MM-DD HH24:MI/SS.US') as rozpatrzone
                 FROM rs_rodo_reklamacje_zgloszenia
                 WHERE firma_id='{firma_id}' and zgloszenie_id='{zgloszenie_id}';
              """.format(firma_id = self.firma_id,
                         zgloszenie_id = self.ZgloszenieID)
        rsglobal.ZapiszDoLogow(SQL)
        stat, daneN, naglN, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
        if stat != 'OK':
            stat = "Bd pobrania danych nagwka reklamacji: " + stat

        if stat == 'OK':
            #WS 2021-06-29 - pobranie pytanie_kto, odpowiedz_kto
            # Pytania dodatkowe
            SQL = """SELECT to_char(pytanie_kiedy, 'YYYY-MM-DD HH24:MI/SS.US') as pytanie_kiedy, 
                            CONCAT(op1.imie, ' ', op1.nazwisko) as pytanie_kto,
                            pytanie, 
                            to_char(odpowiedz_kiedy, 'YYYY-MM-DD HH24:MI/SS.US') as odpowiedz_kiedy,
                            CONCAT(op2.imie, ' ', op2.nazwisko) as odpowiedz_kto,
                            odpowiedz
                     FROM rs_rodo_reklamacje_zgloszenia_pytania_dodatkowe pd
                     LEFT OUTER JOIN rs_operatorzy op1 ON (op1.operator_id=pd.pytanie_kto)
                     LEFT OUTER JOIN rs_operatorzy op2 ON (op2.operator_id=pd.odpowiedz_kto)
                     WHERE firma_id='{firma_id}' and zgloszenie_id='{zgloszenie_id}';
                  """.format(firma_id = self.firma_id,
                             zgloszenie_id = self.ZgloszenieID)
            rsglobal.ZapiszDoLogow(SQL)
            stat, danePD, naglPD, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
            if stat != 'OK':
                stat = "Bd pobrania pyta dodatkowych reklamacji: " + stat

        if stat == 'OK':
            #WS 2021-06-29 - pobranie informacja_kto
            # Przebieg reklamacji
            SQL = """SELECT to_char(wprowadzone, 'YYYY-MM-DD HH24:MI/SS.US') as wprowadzone, 
                            produkt, obsluga, transport, inne, informacja, 
                            to_char(informacja_kiedy, 'YYYY-MM-DD HH24:MI/SS.US') as informacja_kiedy, 
                            CONCAT(op.imie, ' ', op.nazwisko) as informacja_kto,
                            (CASE WHEN COALESCE(p.oddzial_id,'') = '' THEN 'DOWOLNY ODDZIA' ELSE 'ODDZIA: ' || p.oddzial_id || ' - ' || o.opis END) as oddzial
                     FROM rs_rodo_reklamacje_zgloszenia_przebieg_reklamacji p
                     LEFT OUTER JOIN rs_oddzialy o USING (firma_id, oddzial_id)
                     LEFT OUTER JOIN rs_operatorzy op ON (op.operator_id=p.informacja_kto)
                     WHERE firma_id='{firma_id}' and zgloszenie_id='{zgloszenie_id}'
                     ORDER BY wprowadzone ASC;
                  """.format(firma_id = self.firma_id,
                             zgloszenie_id = self.ZgloszenieID)
            rsglobal.ZapiszDoLogow(SQL)
            stat, danePRZ, naglPRZ, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
            if stat != 'OK':
                stat = "Bd pobrania danych przebiegu reklamacji: " + stat

        if stat == 'OK':
            # Zaczniki
            Zal = []

            SQL = """SELECT nr_pliku, opis
                     FROM rs_rodo_reklamacje_zgloszenia_zalaczniki
                     WHERE firma_id='{firma_id}' and zgloszenie_id='{zgloszenie_id}';
                  """.format(firma_id = self.firma_id,
                             zgloszenie_id = self.ZgloszenieID)
            rsglobal.ZapiszDoLogow(SQL)
            stat, NrPlikow, dump, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
            if stat != 'OK':
                stat = "Bd pobrania zacznikw do reklamacji: " + stat
            else:
                for numer, opis in NrPlikow:
                    rsglobal.ZapiszDoLogow("pobranie pliku " + str(numer))
                    stat, Tresc = fs_pliki_global.PobierzPlikZSerwera(self.KatalogPlikow, numer)
                    rsglobal.ZapiszDoLogow("stat " + str(stat))
                    rsglobal.ZapiszDoLogow("dl tresci " + str(len(Tresc)))

                    try:
                        temp = tempfile.mktemp('.zip')

                        Tresc = binascii.unhexlify(Tresc)
                        #WS 2021-11-03
                        with open(temp, 'wb') as p:
                            p.write(Tresc)
                        # open(temp, 'wb').write(Tresc)
                        z = zipfile.ZipFile(temp, "r")
                        tresc = z.read('YouWontKnow')
                        z.close()
                    except Exception, e:
                        rsglobal.ZapiszDoLogow(traceback.format_exc())
                        stat = str(e)
                        tresc = ''

                    Zal.append([stat, tresc, opis])

        if stat == 'OK':
            return 'OK', [daneN, naglN, danePD, naglPD, danePRZ, naglPRZ, Zal]
        else:
            return stat, None

    def UtworzPoleTekstowe(self, opis, data, tresc):
        import rs_print_util

        TPoleTekstowe = rs_print_util.Table([[opis, data]] + [[rs_print_util.Paragraf(tresc, 'Tekst')]], [14 * rs_print_util.cm, 5 * rs_print_util.cm])
        TPoleTekstowe.setStyle(rs_print_util.TableStyle([('TEXTCOLOR', (0, 0), (-1, -1), rs_print_util.colors.black),
                                                         ('BOX', (0, 1), (-1, -1), 0.5, rs_print_util.colors.lightgrey),
                                                         ('FONT', (0, 0), (-1, -1), 'DejaVu', 7),
                                                         ('ALIGN', (0, 0), (0, -1), 'LEFT'),
                                                         ('ALIGN', (1, 0), (1, -1), 'RIGHT'),
                                                         ('SPAN', (0, 1), (-1, -1)),
                                                         ('NOSPLIT', (0, 0), (-1, -1))]))
        return TPoleTekstowe

    def UtworzLinie(self):
        import rs_print_util

        TLinia = rs_print_util.Table([[""], [""]], [19 * rs_print_util.cm])
        TLinia.setStyle(rs_print_util.TableStyle([("LINEBELOW", (0, 0), (-1, 0), 0.5, rs_print_util.colors.black)]))

        return TLinia

    def UtworzTabele(self, tresc):
        import rs_print_util

        T = rs_print_util.Table(tresc, [3 * rs_print_util.cm, 16 * rs_print_util.cm])
        T.setStyle(rs_print_util.TableStyle([('TEXTCOLOR', (0, 0), (-1, -1), rs_print_util.colors.black),
                                             ('FONT', (0, 0), (0, -1), 'DejaVu-Bold', 7),
                                             ('FONT', (1, 0), (1, -1), 'DejaVu', 7),
                                             ('ALIGN', (0, 0), (-1, -1), 'LEFT'),
                                             ('NOSPLIT', (0, 0), (-1, -1))]))

        return T

    def UtworzObrazek(self, img, opis):
        import rs_print_util
        import rsglobal

        from reportlab.platypus.flowables import Image
        from StringIO import StringIO

        if img is not None and img not in ('None', ''):
            if img != 'BRAK':
                try:
                    img = Image(StringIO(img), 19 * rs_print_util.cm, 12 * rs_print_util.cm, kind='proportional')
                    img.hAlign = 'CENTER'
                except Exception, e:
                    img = u"Nie udao si wczyta zacznika:"
                    blad = u'Bd wczytania zacznika "' + opis + '": ' + str(e)
                    self.Bledy.append(blad)
                    rsglobal.ZapiszDoLogow(blad)
            else:
                img = ''
        else:
            img = u"Nie udao si wczyta zacznika:"

        TImg = rs_print_util.Table([[img], [opis]])
        TImg.setStyle(rs_print_util.TableStyle([('TEXTCOLOR', (0, 0), (-1, -1), rs_print_util.colors.black),
                                                ('FONT', (0, 0), (-1, -1), 'DejaVu-Italic', 7),
                                                ('ALIGN', (0, 0), (-1, -1), 'CENTER'),
                                                ('NOSPLIT', (0, 0), (-1, -1))]))

        return TImg

    def ZamienRSTableRowsetNaSlownikowy(self, rowset):
        nowyRowset = []

        for wiersz in rowset.dane:
            slownik = {}
            for nrPola in range(len(wiersz)):
                wart = rowset.GetVal(rowset.dane.index(wiersz), nrPola)

                if isinstance(wart, bool):
                    slownik[rowset.GetFieldName(nrPola)] = wart
                else:
                    slownik[rowset.GetFieldName(nrPola)] = unicode(wart)

            nowyRowset.append(slownik)

        return nowyRowset

    # Generowanie PDF

    def GenerujPDFZeStanemZgloszenia(self):
        import rsglobal
        import rssql
        import rs_print_util
        import tempfile
        import traceback
        from PIL import Image
        from StringIO import StringIO

        stat, Dane = self.PobierzDane()
        if stat != 'OK':
            return [stat], None
        self.Bledy = []

        # Dane nagwka
        self.DaneN = self.ZamienRSTableRowsetNaSlownikowy(rssql.RSTableRowset(Dane[0], Dane[1]))
        # Dane pyta dodatkowych
        self.DanePD = self.ZamienRSTableRowsetNaSlownikowy(rssql.RSTableRowset(Dane[2], Dane[3]))
        # Dane przebiegu reklamacji
        self.DanePRZ = self.ZamienRSTableRowsetNaSlownikowy(rssql.RSTableRowset(Dane[4], Dane[5]))

        # Zaczniki
        self.Zal = Dane[6]

        try:
            Tresc = []

            # --- Nagwek ---

            Nagl = [[u'Zgoszenie reklamacyjne nr ' + str(self.DaneN[0]["zgloszenie_id"])],
                    [u"Data zgoszenia: ", self.DaneN[0]["data_zgloszenia"]],
                    [u"Rozpatrzone: ", 'TAK' if self.DaneN[0]["stan"] == 'Z' else 'NIE']]

            TNagl = self.UtworzTabele(Nagl)

            TNagl.setStyle(rs_print_util.TableStyle([('FONT', (0, 0), (-1, 0), 'DejaVu-Bold', 12),
                                                   ('SPAN', (0, 0), (-1, 0)),
                                                   ('ALIGN', (0, 0), (-1, 0), 'CENTER')]))

            # --- Tre zgoszenia ---

            TTrescZgloszenia = None
            if self.DaneN[0]["tresc_zgloszenia"] not in ('', 'None'):
                TTrescZgloszenia = self.UtworzPoleTekstowe(u"Tre zgoszenia: ",
                                                           self.DaneN[0]["data_zgloszenia"],
                                                           self.DaneN[0]["tresc_zgloszenia"])

            TrescDodatkowa = []  # Pytania dodatkowe + przebieg reklamacji, wraz z datami (eby dao si to posortowa)

            # --- Pytania dodatkowe ---

            for pytanie in self.DanePD:
                TrescPytanie = []

                if not pytanie["pytanie"].strip() in ('', 'None'):
                    TrescPytanie.append(self.UtworzLinie())
                    #WS 2021-06-29 - dodane pytanie_kto
                    TrescPytanie.append(self.UtworzPoleTekstowe(u"Pytanie dodatkowe: ",
                                                                pytanie["pytanie_kto"]+', '+pytanie["pytanie_kiedy"].split('/')[0],
                                                                pytanie["pytanie"]))

                    if not pytanie["odpowiedz"].strip() in ('', 'None'):
                        TrescPytanie.append(rs_print_util.Spacer(1, 0.5 * rs_print_util.cm))
                        #WS 2021-06-29 - dodane odpowiedz_kto
                        TrescPytanie.append(self.UtworzPoleTekstowe(u"Odpowied: ",
                                                                    pytanie["odpowiedz_kto"]+', '+pytanie["odpowiedz_kiedy"].split('/')[0],
                                                                    pytanie["odpowiedz"]))

                    TrescDodatkowa.append([pytanie["pytanie_kiedy"], TrescPytanie])

            # --- Przebieg reklamacji ---

            # Pierwsze przekazanie

            if len(self.DanePRZ) > 0:
                for zmianaPrzebiegu in self.DanePRZ:
                    TrescZmianaPrzebiegu = []

                    if zmianaPrzebiegu["informacja"].strip() not in ('', 'None'):
                        TrescZmianaPrzebiegu.append(self.UtworzLinie())
                        #WS 2021-06-29 - dodane informacja_kto
                        TrescZmianaPrzebiegu.append(self.UtworzPoleTekstowe(u"Informacja: ",
                                                                            zmianaPrzebiegu["informacja_kto"]+', '+zmianaPrzebiegu["informacja_kiedy"].split('/')[0],
                                                                            zmianaPrzebiegu["informacja"]))
                        TrescZmianaPrzebiegu.append(rs_print_util.Spacer(1, 0.5 * rs_print_util.cm))

                    if zmianaPrzebiegu["produkt"] or zmianaPrzebiegu["obsluga"] or \
                       zmianaPrzebiegu["transport"] or zmianaPrzebiegu["inne"] or zmianaPrzebiegu["oddzial"] not in ('', 'None', 'DOWOLNY ODDZIA'):
                        TZ = ''
                        if zmianaPrzebiegu["produkt"]:
                            TZ += u' PRODUKT,'
                        if zmianaPrzebiegu["obsluga"]:
                            TZ += u' OBSUGA,'
                        if zmianaPrzebiegu["transport"]:
                            TZ += u' TRANSPORT,'
                        if zmianaPrzebiegu["inne"]:
                            TZ += u' INNE,'

                        TZ += ' ' + unicode(zmianaPrzebiegu["oddzial"])

                        TypZastrzezenia = [[u"Przekazano dalej do:", TZ]]
                        TrescZmianaPrzebiegu.append(self.UtworzTabele(TypZastrzezenia))

                    TrescDodatkowa.append([zmianaPrzebiegu["wprowadzone"], TrescZmianaPrzebiegu])

            # --- Odpowied ostateczna ---

            TOdpOstateczna = None

            if self.DaneN[0]["tresc_odpowiedzi"].strip() not in ('', 'None'):
                TOdpOstateczna = self.UtworzPoleTekstowe(u"Odpowied ostateczna: ",
                                                         self.DaneN[0]["rozpatrzone"].split('/')[0],
                                                         self.DaneN[0]["tresc_odpowiedzi"])

            # --- Zaczniki ---

            TrescZalaczniki = []

            for zalacznik in self.Zal:
                stat = zalacznik[0]

                TrescZalaczniki.append(rs_print_util.Spacer(1, 0.5 * rs_print_util.cm))
                if stat != 'OK':
                    blad = u'Bd wczytania zacznika "'+zalacznik[2]+'"'
                    self.Bledy.append(blad+': '+stat)
                    rsglobal.ZapiszDoLogow(blad+': '+stat)
                    TrescZalaczniki.append(self.UtworzObrazek('BRAK', blad))
                else:
                    try:
                        im = Image.open(StringIO(zalacznik[1]))
                        im.verify()
                    except:
                        TrescZalaczniki.append(self.UtworzObrazek('BRAK', u'Brak podgldu dla zacznika "'+zalacznik[2]+'"'))
                    else:
                        TrescZalaczniki.append(self.UtworzObrazek(zalacznik[1], zalacznik[2]))

            # --- Zoenie pliku w cao ---

            Tresc.append(TNagl)
            Tresc.append(rs_print_util.Spacer(1, 0.5 * rs_print_util.cm))

            if TTrescZgloszenia is not None:
                Tresc.append(TTrescZgloszenia)

            for tresc in sorted(TrescDodatkowa):
                Tresc.extend(tresc[1])

            if TOdpOstateczna is not None:
                Tresc.append(self.UtworzLinie())
                Tresc.append(TOdpOstateczna)

            Tresc.extend(TrescZalaczniki)

            if self.NazwaPliku == '':
                self.NazwaPliku = tempfile.mktemp('.pdf')
            #AB 13.07.22r dodana zwrotka z ZrobPdf
            stat = rs_print_util.ZrobPdf(self.firma_id, self.NazwaPliku, '', Tresc, ozdobniki=135,
                                         opcje={'topMargin': rs_print_util.cm, 'bottomMargin': 1.2 * rs_print_util.cm})
            if stat != 'OK':
                return ["Wystpi problem podczas generowania pliku w formacie PDF"], None

        except Exception, e:
            rsglobal.ZapiszDoLogow(str(traceback.format_exc()))
            return [str(e)], None
        else:
            if self.Bledy != []:
                return self.Bledy, self.NazwaPliku
            else:
                return ['OK'], self.NazwaPliku
