# -*- coding: iso8859_2 -*-
#-------------------------------------------------------------------------------
# Name:        fs_rozstawienie_asortymentu.py
# Purpose:     obsluga rozstawienia asortymentu na dowody dostaw
# Author:      Mariusz Rajfura
# Created:     2010/09/24
# RCS-ID:      $Id: fs_rozstawienie_asortymentu.py$
# Copyright:   (c) 2010 Record System
# Licence:     GNU GPL
#-------------------------------------------------------------------------------

# Uwaga: w przypadku zmiany klucza nalezy poprawic wywolania
#        anakondy: RSRozstawienieAsortymentu, RSRozstawienieAsortymentuDane
#        PANELU: RSRozstawienie, RSPanRozstawienieArtykuly

#2011-02-06 Mariusz Rajfura - rozstawienie na wywietlaczu (4259)
#2011-03-02 Mariusz Rajfura - rozstawienie asortymentu z uwzgl.priorytetow (4325)
#2011-03-08 Mariusz Rajfura - dostos.do wywolania z panelu i przygot.danych dla wyswietlaczy (4326)
#2011-03-18 Mariusz Rajfura - obsluga pola 'ilosc_wysw1,ilosc_wysw2 (4326)
#2011-03-22 Mariusz Rajfura - obsluga pol zapamietujacych stan wyswietlaczy w boksach (4326)
#2011-06-07 Mariusz Rajfura - pobranie informacji o rozstawieniu na danym wierszu (4661)
#2011-08-29 Mariusz Rajfura - sumow.ilosci z tras w boksie, popr.PodajWyswietloneArtykuly (4906)
#2011-10-04 Mariusz Rajfura - popr.bledu przy braku rozstawionych pozycji (5082)
#2011-10-06 Mariusz Rajfura - rzut zamiast trybu dostawy na filtrze (5088)
#2011-10-19 Mariusz Rajfura - zmiana filtru pobierajacego pozycje dow.dost. dla zatw/anul:
#                             firma_id, oddzial_id, artykul_id, data, ilosc_wysw1>0
#                             zdjeta kontrola przy zatwierdz/anulow (5117)
#2011-12-28 Mariusz Rajfura - rozstawienie dow.dost.z innych oddzialow na pdst rs_boksy_trasy_dni (5330)
#2012-03-09 Mariusz Rajfura - mozliwosc opcji pobrania ilosci z pola: ilosc_zam1/ilosc_pobr1 (5526)
#2012-04-12 Mariusz Rajfura - rozstawienie ulamkowe dla kilogramow (5641)
#2012-10-05 Mariusz Rajfura (6136) uwzglednienie pola 'osobny_przerzut' w rs_boksy_trasy_dni
#2012-11-29 Michal Raburski (6319) optymalizacja zapytan
#2012-12-11 Mariusz Rajfura (bez mantisa-HotLine) wydruk rozstawienia wg tras (jak bylo poprzednio)
#2013-08-20 Mariusz Rajfura (7127) popr.warunku w filtrze przed grupowaniem Fun==12
#2013-11-15 Mariusz Rajfura (7387) popr.zliczania artykulow z boksow (PodajWyswietloneArtykuly)
#2013-11-20 Mariusz Rajfura (7405) poprawka poprawki (PodajWyswietloneArtykuly)
#2013-11-21 Mariusz Rajfura (7409) rozstawienie z uwzglednieniem pomieszczen
#2017-10-31 Rafa Woniak (10734) zmiana zapisu do logw z 1 na 5


#-------------------------------------------------------------------------------

def PodajTrasyITrybyZPomieszczenia(obj, Data, ListaTrybow, Pomieszczenie, JakoSQL=False):
    # Funkcja zwraca liste tras, ktore sa obslugiwane przez 
    # boksy z podanego pomieszczenia z uwzglednieniem innych filtrow
    # Opcja JakoSQL zwraca wynik w formie stringa dla SQL-a
    
    import rsglobal

    sql ="SELECT bt.boks_id,bt.trasa,TRIM(b.boks_ip),bt.tryb_dostawy"
    sql+=" FROM rs_boksy_trasy bt"
    sql+=" INNER JOIN rs_boksy b USING(firma_id,oddzial_id,boks_id)"
    sql+=" WHERE bt.firma_id='"+obj.firma_id+"' AND bt.oddzial_id='"+obj.oddzial_id+"'"
    sql+=" AND b.pomieszczenie='"+Pomieszczenie+"'"
    sql+=" AND bt.tryb_dostawy IN "+ListaTrybow
    sql+=" AND bt.dzien_tygodnia=CASE WHEN EXTRACT (DOW FROM '"+Data+"'::DATE)=0"
    sql+="  THEN 7 ELSE EXTRACT (DOW FROM '"+Data+"'::DATE) END"
    sql+=" ORDER BY bt.trasa,bt.boks_id;"

    stat,dBoksyTrasy,dump,dump = obj.PolSQL.Execute(sql,3)
    if stat <> 'OK':
        stat = 'Bd pobrania tras dla boksw z pomieszczenia: '+stat
        return stat,[]

    #lista boksow odpowiadajacych wybranemu dniowi
    sql ="SELECT btd.boks_id,btd.trasa,TRIM(b.boks_ip),btd.tryb_dostawy"
    sql+=" FROM rs_boksy_trasy_dni btd"
    sql+=" INNER JOIN rs_boksy b USING(firma_id,oddzial_id,boks_id)"
    sql+=" WHERE btd.firma_id='"+obj.firma_id+"' AND btd.oddzial_id='"+obj.oddzial_id+"'"
    sql+=" AND b.pomieszczenie='"+Pomieszczenie+"'"
    sql+=" AND btd.tryb_dostawy IN "+ListaTrybow+" AND btd.data<='"+Data+"' AND (btd.data_do>='"+Data+"' OR data_do IS NULL)"
    sql+=" AND NOT btd.osobny_przerzut"
    sql+=" ORDER BY btd.trasa,btd.boks_id;"
    
    stat,dBoksyTrasyDni,dump,dump = obj.PolSQL.Execute(sql,3)
    if stat <> 'OK':
        stat = 'Bd pobrania tras dla boksw-dni z pomieszczenia: '+stat
        return stat,[]

    TrasyITrybyZPomieszczenia = []
    for i in xrange(len(dBoksyTrasy)):
        trasa = rsglobal.Dekoduj(dBoksyTrasy[i][1])
        tryb = rsglobal.Dekoduj(dBoksyTrasy[i][3])
        if not (trasa,tryb) in TrasyITrybyZPomieszczenia:
            TrasyITrybyZPomieszczenia.append((trasa,tryb))
            
    for i in xrange(len(dBoksyTrasyDni)):
        trasa = rsglobal.Dekoduj(dBoksyTrasyDni[i][1])
        tryb = rsglobal.Dekoduj(dBoksyTrasyDni[i][3])
        if not (trasa,tryb) in TrasyITrybyZPomieszczenia:
            TrasyITrybyZPomieszczenia.append((trasa,tryb))
            
    if stat == 'OK':
        if JakoSQL:
            if len(TrasyITrybyZPomieszczenia) > 0:
                wynik = ''
                # zwraca list par - trasa tryb i tylko takie dane zostan zaakceptowane
                for para in TrasyITrybyZPomieszczenia:
                    wynik += "'"+para[0]+para[1]+"',"
                TrasyITrybyZPomieszczenia= '('+wynik.strip(',')+')'
            else:
                TrasyITrybyZPomieszczenia = "('')"
    
    return stat, TrasyITrybyZPomieszczenia

#-------------------------------------------------------------------------------

def PodajNrWyswietlacza(obj, Rzut, Data, Artykul, Pomieszczenie, Opak = ''):
    #zwraca (stat,nr_wiersza)
    import rsglobal

    sql ="SELECT DISTINCT"
    sql+=" CASE WHEN rzut_wysw_w1='"+Rzut+"' AND data_wysw_w1='"+Data+"'"
    sql+="  AND artykul_wysw_w1='"+Artykul+"' AND ilosc_wysw_w1>0"
    sql+="  AND COALESCE(opakowanie_wysw_w1,'') = '"+Opak+"'"
    sql+="  THEN 1 ELSE 0 END,"

    sql+=" CASE WHEN rzut_wysw_w2='"+Rzut+"' AND data_wysw_w2='"+Data+"'"
    sql+="  AND artykul_wysw_w2='"+Artykul+"' AND ilosc_wysw_w2>0"
    sql+="  AND COALESCE(opakowanie_wysw_w2,'') = '"+Opak+"'"
    sql+="  THEN 1 ELSE 0 END,"

    sql+=" CASE WHEN rzut_wysw_w3='"+Rzut+"' AND data_wysw_w3='"+Data+"'"
    sql+="  AND artykul_wysw_w3='"+Artykul+"' AND ilosc_wysw_w3>0"
    sql+="  AND COALESCE(opakowanie_wysw_w3,'') = '"+Opak+"'"
    sql+="  THEN 1 ELSE 0 END,"

    sql+=" CASE WHEN rzut_wysw_w4='"+Rzut+"' AND data_wysw_w4='"+Data+"'"
    sql+="  AND artykul_wysw_w4='"+Artykul+"' AND ilosc_wysw_w4>0"
    sql+="  AND COALESCE(opakowanie_wysw_w4,'') = '"+Opak+"'"
    sql+="  THEN 1 ELSE 0 END"

    sql+=" FROM rs_boksy"
    sql+=" WHERE firma_id='"+obj.firma_id+"' AND oddzial_id='"+obj.oddzial_id+"'"
    
    #--POMIESZCZENIE--
    if Pomieszczenie <> '':
        sql+=" AND pomieszczenie='"+Pomieszczenie+"'"
    sql+=";"
    
    stat,dane,dump,dump = obj.PolSQL.Execute(sql,3)
    if stat <> 'OK':
        return 'Bd pobrania danych z boksw: '+stat,0

    for d in dane:
        if d[0] > 0:
            return stat,1
        if d[1] > 0:
            return stat,2
        if d[2] > 0:
            return stat,3
        if d[3] > 0:
            return stat,4
    
    #komplet podanych danych nie wystepuje w zadnym boksie
    return stat,0

#-------------------------------------------------------------------------------

def PodajWyswietloneArtykuly(obj, Rzut, Data, Pomieszczenie):
    #zwraca slownik: [artykul]=[wiersz,nazwa_artykulu,ilosc,czy_rozstawiany_w_kg,opakowanie]
    import rsglobal

    # SELECT sklada sie z 4 wynikow dla 4 selectow (dla kazdego wiersza z wyswietlaczy)

    sql = ""
    for i in ['1','2','3','4']:
        sql+=" SELECT b.artykul_wysw_w"+i+",a.nazwa,b.ilosc_wysw_w"+i+","+i+",COALESCE(b.opakowanie_wysw_w"+i+",''),"
        sql+=" COALESCE(ob.opis,''),COALESCE(a.tylko_w_opakowaniach,FALSE)"
        sql+=" FROM rs_boksy b"
        sql+=" INNER JOIN rs_artykuly a ON (a.firma_id=b.firma_id AND a.artykul_id=b.artykul_wysw_w"+i+")"
        sql+=" LEFT OUTER JOIN rs_opakowania_zbiorcze ob ON (ob.firma_id=b.firma_id AND ob.kod=b.opakowanie_wysw_w"+i+")"
        sql+=" WHERE b.firma_id='"+obj.firma_id+"' AND b.oddzial_id='"+obj.oddzial_id+"'"
        sql+=" AND b.rzut_wysw_w"+i+"='"+Rzut+"' AND b.data_wysw_w"+i+"='"+Data+"'"
        
        #--POMIESZCZENIE--
        if Pomieszczenie <> '':
            sql+=" AND b.pomieszczenie='"+Pomieszczenie+"'"
        if i <> '4':
            sql+=" UNION ALL"
    sql+=";"
    stat,dArt,dump,dump = obj.PolSQL.Execute(sql,3)
    if stat <> 'OK':
        return 'Bd pobrania danych z boksw: '+stat,{}

    sArt = {}
    
    for i in xrange(len(dArt)):
        Artykul = dArt[i][0]
        Nazwa = dArt[i][1]
        Ilosc = dArt[i][2]
        Wiersz = dArt[i][3]
        Opakowanie = dArt[i][4]
        NazwaOpak = dArt[i][5]
        TylkoWOpak = dArt[i][6]

        if not (Artykul,Opakowanie) in sArt:
            sArt[(Artykul,Opakowanie)] = [Wiersz, Nazwa, 0, False,Opakowanie,NazwaOpak,0,TylkoWOpak]
        
        if Opakowanie =='':
            # suma ilosci dla artykulu
            sArt[(Artykul,Opakowanie)][2] += Ilosc
        else:
            # suma opakowan dla artykulu
            sArt[(Artykul,Opakowanie)][6] += Ilosc
            
    if stat == 'OK' and len(sArt.keys()) > 0:
        # pobranie informacji o artykulach rozstawianych w kg i uzupelnienie zwracanego slownika
        Artykuly = ''
        kArt = sArt.keys()
        for k in sArt.keys():
            Artykuly += "'"+rsglobal.Dekoduj(k[0])+"',"
        Artykuly = Artykuly[:-1]
            
        sql ="SELECT artykul_id FROM rs_artykuly"
        sql+=" WHERE firma_id='"+obj.firma_id+"'"
        sql+=" AND artykul_id IN ("+Artykuly+") AND UPPER(TRIM(jm1))='KG';"
        
        stat,d,dump,dump = obj.PolSQL.Execute(sql,3)
        if stat <> 'OK':
            return 'Bd pobrania informacji o artykuach rozstawianych w kg.',{}

        for i in xrange(len(d)):
            Artykul = d[i][0]
            if (Artykul,'') in sArt:
                sArt[(Artykul,'')][3] = True

    return stat,sArt

#-------------------------------------------------------------------------------

def PrzefiltrujDaneWzgledemCzyRozstawiac(dane,kolumna,usuwac):
    # funkcja filtruje dane wzgldem wartoci w kolumnie czy_rozstawia
    # dodatkowo jeli jest usuwa to wyrzuca ca kolumn z danych
    DaneF=[]
    for poz in dane:
        if poz[kolumna]:
            if usuwac:
                pozpom=[]
                if kolumna>0:
                    pozpom=poz[:kolumna]
                if len(pozpom)<len(poz)-1:
                    pozpom+=poz[kolumna:]
                DaneF.append(pozpom)
            else:
                DaneF.append(poz)
    return DaneF

#-------------------------------------------------------------------------------

def SprawdzZajetoscKoloruNaPomieszczeniu(self,Pomieszczenie,Kolor):
    SQL="SELECT data_wysw_w"+Kolor+",rzut_wysw_w"+Kolor+" FROM rs_boksy "
    SQL+=" WHERE firma_id='"+self.firma_id+"' AND oddzial_id='"+self.oddzial_id+"'"
    SQL+=" AND pomieszczenie='"+Pomieszczenie+"' AND ilosc_wysw_w"+Kolor+" <> 0"    
    stat,d,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat <>'OK':
        return 'Wystpiy bdy zajtoci koloru w pomieszczeniu na wywietlaczach : '+stat
    if len(d)>0:
        return 'Nie mona rozstawi artykuu, bo na wybranym kolorze s rozstawione dane w dniu '+d[0][0]+' dla rzutu '+d[0][1]
    
    return 'OK'
    

    
#-##############################################################################
#-##############################################################################
#-- WACIWE CIAO FUNKCJI
#-##############################################################################
#-##############################################################################

import math

from funkcje import fs_artykuly_pom

Artykul = rsglobal.Dekoduj(klucze[0]).strip()
IloscDoR = klucze[1]

RzutF = rsglobal.Dekoduj(klucze[2][0]).strip()
DataF = rsglobal.Dekoduj(klucze[2][1])
TrasaF = rsglobal.Dekoduj(klucze[2][2]).strip()
KategArtF = rsglobal.Dekoduj(klucze[2][3]).strip()
PomieszczenieF = rsglobal.Dekoduj(klucze[2][4]).strip()

Fun = klucze[-1]


#FIXME: blad przy wywolywaniu fukcji z panelu
#       to chyba w rs_server_pan
self.firma_id = rsglobal.Dekoduj(self.firma_id)
self.oddzial_id = rsglobal.Dekoduj(self.oddzial_id)

stat = 'OK'
dane = []
wyn1 = []
wyn2 = []
LBoksIP = []

# tu mozna zdecydowac o opcjonalnym wyborze pola
Opcja = 'POBRANE'

if Opcja == 'ZAMOWIONE':
    PoleIlosci = 'ilosc_zam1'
elif Opcja == 'POBRANE':    
    PoleIlosci = 'ilosc_pobr1'
else:
    stat = 'BEDNA OPCJA'


UprEdycja = (self.oddzial_id,'049002') in self.uprawnienia or self.operator_id == STANDARD_OPERATOR_ID 

UprEdycja = True

#-------------------------------------------------------------------------------
# KONTROLA OBOWIAZKOWYCH FILTROW, POBRANIE OBLIGATORYJNYCH WARTOSCI
#-------------------------------------------------------------------------------

if stat == 'OK' and Fun == 5:
    if RzutF == '':
        stat = 'BR RZUT'

    if not RzutF in ('1','2','3'):
        stat = 'BL RZUT'

if stat == 'OK' and Fun == 5:
    if DataF == '    -  -  ':
        stat = 'BR DATA'
    elif not rsglobal.SprawdzCzyToData(DataF):
        stat = 'BL DATA'

if stat == 'OK' and Fun == 5:
    sql ="SELECT magazyn_mm FROM rs_produkcja_dokumenty"
    sql+=" WHERE firma_id='"+self.firma_id+"' AND oddzial_id='"+self.oddzial_id+"';"

    stat,MagMM,dump,dump = self.PolSQL.Execute(sql,3)
    if stat <> 'OK':
        stat = 'Bd pobrania informacji o magazynie dla dokumentw MM: '+stat
    elif len(MagMM) == 0:
        stat = 'BR DEF DOK PROD'
    elif MagMM[0][0] == 'None':
        stat = 'BR DEF MAG MM'
    else:
        MagMM = rsglobal.Dekoduj(MagMM[0][0])

# (5) wywietlanie danych, (12) pobranie sumy iloci do rozstawienia z DD , (13) rozstawienie, 
#(14) zatwierdzenie rozstawienia, (15) anulowanie rozstawienia, (16) informacje o artykuach, (17) informacje o rozstawieniu na trasy
if stat == 'OK' and Fun in (5,12,13,14,15,16,17,23):
    if PomieszczenieF =='':
        stat='BR POMIESZCZENIA'

# lista trybow do filtrowania
ListaTrybow = "('')"

if stat == 'OK' and not Fun in [23,26]:
    #pobranie listy trybow dla podanego rzutu
    sql ="SELECT kod FROM rs_tryby_dostaw"
    sql+=" WHERE firma_id='"+self.firma_id+"' AND rzut='"+RzutF+"';"
    
    stat,dTryby,dump,dump = self.PolSQL.Execute(sql,3)
    if stat <> 'OK':
        stat = 'Bd pobrania listy trybw dostaw dla podanego rzutu: '+stat
        dane.append(stat)
    elif len(dTryby) > 0:
        lista = []
        for i in xrange(len(dTryby)):
            lista.append(rsglobal.Dekoduj(dTryby[i][0]))
        ListaTrybow = str(lista).replace("[","(").replace("]",")")

#-------------------------------------------------------------------------------
# POBRANIE BOKSW DLA POMIESZCZENIA (23)
#-------------------------------------------------------------------------------
   
if stat == 'OK' and Fun == 23:
    SQL="SELECT boks_id,boks_ip FROM rs_boksy WHERE firma_id='"+self.firma_id+"' AND pomieszczenie = '"+PomieszczenieF+"'"
    stat,dane,dump,dump = self.PolSQL.Execute(SQL,3)

#-------------------------------------------------------------------------------
# POBRANIE OPISU OPAKOWANIA (26)
#-------------------------------------------------------------------------------
   
if stat == 'OK' and Fun == 26:
    SQL="SELECT opis FROM rs_opakowania_zbiorcze WHERE firma_id='"+self.firma_id+"' AND kod = '"+klucze[3]+"'"
    stat,dane,dump,dump = self.PolSQL.Execute(SQL,3)
    
#-------------------------------------------------------------------------------
# POBRANIE SUMY ILOSCI DO ROZSTAWIENIA Z DOWODOW DOSTAW (12)
#-------------------------------------------------------------------------------

if Fun == 12 and stat == 'OK':
    sql ="SELECT artykul_id FROM rs_artykuly"
    sql+=" WHERE firma_id='"+self.firma_id+"' AND artykul_id='"+Artykul+"'"
        
    stat,d,dump,dump = self.PolSQL.Execute(sql,3)
    if stat <> 'OK':
        stat = 'Bd sprawdzenia artykuu: '+stat
        dane.append(stat)
    elif len(d) == 0:
        stat = 'BL ARTYKUL'
        dane.append(stat)

    if stat == 'OK':    
        sql ="SELECT SUM(ddp."+PoleIlosci+"- ddp.ilosc_rozst1- ddp.ilosc_wysw1),"
        sql+=" rsf_czy_rozstawiac_asortyment(ddp.firma_id,ddp.oddzial_id,ddp.trasa,"
        sql+=" ddp.data,ddp.tryb_dostawy,'"+self.oddzial_id+"')"
        sql+=" FROM rs_dowody_dostaw_pozycje ddp"
        sql+=" INNER JOIN rs_dowody_dostaw dd USING(firma_id,oddzial_id,trasa,data,tryb_dostawy,numer)"
        sql+=" WHERE ddp.firma_id='"+self.firma_id+"'"

        sql+=" AND ddp.tryb_dostawy IN "+ListaTrybow+" AND ddp.data='"+DataF+"'"
        sql+=" AND ddp.artykul_id='"+Artykul+"'"
        sql+=" AND ddp."+PoleIlosci+"- ddp.ilosc_rozst1- ddp.ilosc_wysw1<>0"
    
        if TrasaF<>'':
            sql+= " AND ddp.trasa='"+TrasaF+"' "
        if KategArtF<>'':
            sql+= " AND a.kateg_artykulu LIKE '"+KategArtF+"%'"
        
        #--POMIESZCZENIE--
        if PomieszczenieF <> '':
            stat,TrasyPSql = PodajTrasyITrybyZPomieszczenia(self,DataF,ListaTrybow,PomieszczenieF,JakoSQL=True)
            if stat == 'OK':
                sql+=" AND SUBSTRING((ddp.trasa||'    '),1,4)||ddp.tryb_dostawy IN "+TrasyPSql
        
        sql+=" GROUP BY rsf_czy_rozstawiac_asortyment(ddp.firma_id,ddp.oddzial_id,ddp.trasa,"
        sql+=" ddp.data,ddp.tryb_dostawy,'"+self.oddzial_id+"')"
        sql+=";"
        
        stat,dane,dump,dump = self.PolSQL.Execute(sql,3)
        if stat <> 'OK':
            stat = 'Bd pobrania sumy iloci z dowodw dostaw: '+stat
            dane.append(stat)
        else:
            dane=PrzefiltrujDaneWzgledemCzyRozstawiac(dane,1,True)
            if len(dane) == 0:
                dane=[[0]]


#-------------------------------------------------------------------------------
# ROZSTAWIENIE ASORTYMENTU Z UWZGLEDNIENIEM PRIORYTETOW (13)
#-------------------------------------------------------------------------------

if Fun == 13 and stat == 'OK':
    if not UprEdycja:
        stat = 'UPR EDY'

    # sprawdzamy, czy artykul juz jest gdzies wyswietlony w podanym rzucie
    # i dacie, czyli tak, jak byo dotychczas czyli w ramach oddzialu nie mozna
    # ponownie rozstawic tego samego artykulu, jesli jest juz rozstawiony
    NrWyswietlacza = klucze[3]
    PoprzIloscDD = klucze[4]
    Opak = klucze[5]
    
    if stat == 'OK':
        stat,NrWysw = PodajNrWyswietlacza(self,RzutF,DataF,Artykul,PomieszczenieF,Opak)
        if stat <> 'OK':
            dane.append(stat)
        elif NrWysw > 0:
            stat = 'Artykul '+Artykul+' jest ju rozstawiony w wierszu nr '+rsglobal.UsunKropkaZero(NrWysw)+'.'
            stat+= 'Rozstawienie tego artykuu niemoliwe.'
            dane.append(stat)
            
    if stat == 'OK':
        stat = SprawdzZajetoscKoloruNaPomieszczeniu(self,PomieszczenieF,str(NrWyswietlacza))
   
    if stat == 'OK':
        RozstawKG = False
        
        WartOpak = "NULL"
        #przelicznik artykulu dla jm2
        sql ="SELECT przelicznik, UPPER(TRIM(jm1))='KG'"
        sql+=" FROM rs_artykuly"
        sql+=" WHERE firma_id='"+self.firma_id+"' AND artykul_id='"+Artykul+"';"
        
        stat,dPrzelicz,dump,dump = self.PolSQL.Execute(sql,3)
        if stat <> 'OK':
            stat = 'Bd pobrania danych artykuu: '+stat
            dane.append(stat)
        elif len(dPrzelicz) == 0:
            stat = 'BL ARTYKUL'
            dane.append(stat)
        elif Opak == '':
            RozstawKG = dPrzelicz[0][1]
                
        if Opak <> '':
            # sprawdzenie poprawnoci opakowania
            WartOpak = "'"+Opak+"'"
            SQL = "SELECT kod FROM rs_opakowania_zbiorcze WHERE firma_id='"+self.firma_id+"' AND kod = '"+Opak+"'"
            stat,DTestOpak,dump,dump = self.PolSQL.Execute(SQL,3)
            if stat <> 'OK':
                stat = 'Bd pobrania danych opakowania: '+stat
                dane.append(stat)
            elif len(DTestOpak) == 0:
                stat = 'BL OPAK'
                dane.append(stat)
                
            # sprawdzenie czy artyku ma wczone "tylko w opakowaniach"
            if stat == 'OK':
                sql ="SELECT COALESCE(tylko_w_opakowaniach,FALSE) FROM rs_artykuly"
                sql+=" WHERE firma_id='"+self.firma_id+"' AND artykul_id='"+Artykul+"';"
                stat,DTest,dump,dump = self.PolSQL.Execute(sql,3)
                if stat <> 'OK':
                    stat = 'Bd pobrania danych artykuu: '+stat
                    dane.append(stat)
                elif not DTest[0][0]:
                    stat = 'BL ART NIE W OPAK'
                    dane.append(stat)

    #--------------------------------------------------------
    LInfo = [] #lista informacyjna na czas testow priorytetow
    #--------------------------------------------------------

    if stat == 'OK':
        if IloscDoR <= 0:
            stat = 'BL ILOSC'
            dane.append(stat)

    if stat == 'OK':
        if not NrWyswietlacza in xrange(1,5):
            stat = 'BL NR_WYSWIETLACZA'
            dane.append(stat)

    if stat == 'OK':
        # dla opakowa (poniewa od razu posugujemy si w algorytmie caymi opakowaniami)
        # przyjmujemy pojemno opakowania w opakowaniu = 1
        if Opak <> '':
            dOpak=[[1]]
        else:
            #opakowanie zbiorcze artykulu - pierwsze z listy wg kodu opakowania
            sql ="SELECT ilosc1 FROM rs_artykuly_opakowania_zbiorcze"
            sql+=" WHERE firma_id='"+self.firma_id+"' AND artykul_id='"+Artykul+"'"
            sql+=" ORDER BY opakowanie_id LIMIT 1;"
            
            stat,dOpak,dump,dump = self.PolSQL.Execute(sql,3)
            if stat <> 'OK':
                stat = 'Bd pobrania danych opakowania zbiorczego: '+stat
            elif len(dOpak) == 0:
                stat = 'BR OPAK_ZBIOR'
            elif RozstawKG:
                # jesli to kilogramy, to opakowanie zbiorcze == 1kg
                dOpak[0][0] = 1
            
        
    if stat == 'OK' and dOpak[0][0] > 100:
        # jesli opakowanie zbiorcze ma pojemnosc wieksza niz 100,
        # bo wyswietlacz na dwie cyfry na ilosc luzem
        stat = 'BL OPAK_ZBIOR_PONAD_100'

    if stat == 'OK':
        if Opak <> '':
            # pole z informacj ile mamy ilo pocztkow. Potem rcznie wyliczamy ile jakich opakowa powinno by
            # a nastpnie rcznie odejmujemy ju wystawione opakowania
            sql ="SELECT ddp."+PoleIlosci+","  # 0
        else:
            #nierozstawione pozycje z dowodow dostaw (ilociowa metoda rozliczania)
            sql ="SELECT ddp."+PoleIlosci+"-ddp.ilosc_rozst1- ddp.ilosc_wysw1," # 0 - pole z informacj ile mamy faktycznie rozliczy
        
        sql+=" ddp.trasa,dd.numer,ddp.nr_poz,0," # 1- 4
        sql+=" dd.kontrahent_id, COALESCE(dd.platnik_id,''::TEXT),0,0,0,ddp.tryb_dostawy,ddp.oddzial_id," # 5 - 11
        sql+=" rsf_czy_rozstawiac_asortyment(ddp.firma_id,ddp.oddzial_id,ddp.trasa," # 12
        sql+=" ddp.data,ddp.tryb_dostawy,'"+self.oddzial_id+"')" 
        if Opak <> '':
            sql+=",(SELECT COALESCE(ddpo.ilosc_rozstawiona,0) FROM rs_dowody_dostaw_pozycje_opakowania ddpo WHERE ddpo.firma_id = ddp.firma_id"
            sql+=" AND ddpo.oddzial_id = ddp.oddzial_id AND ddpo.trasa = ddp.trasa AND ddpo.data = ddp.data"
            sql+=" AND ddpo.tryb_dostawy = ddp.tryb_dostawy AND ddpo.numer = ddp.numer "
            sql+=" AND ddpo.nr_poz = ddp.nr_poz AND ddpo.opakowanie_id = '"+Opak+"')" # 13
        else:
            sql+=",0" # 13
            
        sql+=" FROM rs_dowody_dostaw_pozycje ddp"
        sql+=" INNER JOIN rs_dowody_dostaw dd USING(firma_id,oddzial_id,trasa,data,tryb_dostawy,numer)"
        sql+=" WHERE ddp.firma_id='"+self.firma_id+"'"
        sql+=" AND ddp.tryb_dostawy IN "+ListaTrybow+" AND ddp.data='"+DataF+"'"
        sql+=" AND ddp.artykul_id='"+Artykul+"'"
        sql+=" AND ddp."+PoleIlosci+"-ddp.ilosc_rozst1-ddp.ilosc_wysw1>0"

        # --POMIESZCZENIE--
        if PomieszczenieF <> '':
            stat,TrasyPSql = PodajTrasyITrybyZPomieszczenia(self,DataF,ListaTrybow,PomieszczenieF,JakoSQL=True)
            if stat == 'OK':
                sql+=" AND SUBSTRING((ddp.trasa||'    '),1,4)||ddp.tryb_dostawy IN "+TrasyPSql

        if TrasaF<>'':
            sql+= " AND ddp.trasa='"+TrasaF+"' "
        if KategArtF<>'':
            sql+= " AND a.kateg_artykulu LIKE '"+KategArtF+"%' "
        
        sql+=" FOR UPDATE NOWAIT;"
        
        stat,dPozDD,dump,dump = self.PolSQL.Execute(sql,3,CzyDodawacBlad=True)
        if stat <> 'OK':
            stat = "Bd pobrania pozycji z dowodw dostaw.\nMoliwe, e jakie dowody dostaw s zablokowane przez innego operatora lub proces.\n\nSprbuj jeszcze raz."
            dane.append(stat)
            
        if stat=='OK':
            dPozDD=PrzefiltrujDaneWzgledemCzyRozstawiac(dPozDD,12,True)
            if len(dPozDD) == 0:
                stat = 'Brak wolnych pozycji do rozstawienia.'
                dane.append(stat)
                
            rsglobal.ZapiszDoLogow('dPozDD 1 '+str(dPozDD),5)
    # dla rozstawienia w opakowaniach sprawdzamy ile tak napraw naszych opakowa powinno by w sumie do wydania
    if stat =='OK' and Opak <> '':
        dPozDDTmp = []
        for poz in dPozDD:
            if stat == 'OK':
                IloscTmp = poz[0]
                KontrTmp = poz[5]
                stat,SOpak = fs_artykuly_pom.PodajOpakowaniaDlaArtykulu(self,Artykul,IloscTmp,KontrTmp)
                rsglobal.ZapiszDoLogow('SOpak '+str((Artykul,IloscTmp,KontrTmp))+' '+str(SOpak),5)
                if stat == 'OK':
                    if Opak in SOpak:
                        # uwzgldniamy informacj czy nie wydano czego wczeniej
                        if SOpak[Opak] - poz[13] > 0:
                            dPozDDTmp.append(eval(str(poz)))
                            dPozDDTmp[-1][0] = SOpak[Opak] - poz[13]
        
        dPozDD = eval(str(dPozDDTmp))
        if stat == 'OK':        
            if len(dPozDD) == 0:
                stat = u'Nie ma adnego zapotrzebowania na artyku '+Artykul+u' w opakowaniach '+Opak+'. Sprawdzanie nastpio na podstawie danych z pozycji dowodw dostaw.'
        
    if stat == 'OK':
        #czy podana ilo do rozstawienia nie przekroczya
        #dostpnej iloci na dowodach (test dla rozstawienia ilociowego)
        IloscDD = 0
        for i in xrange(len(dPozDD)):
            IloscDD += dPozDD[i][0]
        
        if Opak =='':
            IloscDD = rsglobal.RSRound(IloscDD,4)
    
            if IloscDoR > IloscDD: 
                stat = "Podana ilo do rozstawienia ("+str(IloscDoR)+")\n"
                stat+= "przekracza dostpn na dowodach dostaw ilo do rozstawienia ("
                stat+= str(int(IloscDD))+"). Prosz odwiey dane."
                dane.append(stat)
        
            if stat == 'OK':
                # czy zmienila sie ilosc z dowodow dostaw pomiedzy pobraniem
                # informacji, a wykonaniem rozstawienia (test dla rozstawienia ilociowego)
                if rsglobal.RSRound(IloscDD,4) <> rsglobal.RSRound(PoprzIloscDD,4):
                    stat = 'BL ZMIANA_ILOSCI'
                    dane.append(stat)

    if stat == 'OK' and RozstawKG:
        # czy rozstawiana caa ilo w kilogramach
        if rsglobal.RSRound(IloscDoR,4) <> rsglobal.RSRound(IloscDD,4):
            stat = 'Rozstawienie w kilogramach dostpne tylko dla caej iloci.'
            dane.append(stat)

    if stat == 'OK':
        # uzupelnienie priorytetow w dowodach dostaw
        SKontrah = {}
        
        for i in xrange(len(dPozDD)):
            Kontrah = rsglobal.Dekoduj(dPozDD[i][5])
            Platnik = rsglobal.Dekoduj(dPozDD[i][6]).strip()
            
            if not Kontrah in SKontrah:
                # pobranie priorytetow z danych do fakturowania dla kontrahenta lub platnika
                sql ="SELECT priorytet FROM rs_kontrahenci_fakturowanie"
                sql+=" WHERE firma_id='"+self.firma_id+"' AND kontrahent_id='"+Kontrah+"'"
                sql+=" AND data_od<='"+DataF+"' ORDER BY data_od DESC LIMIT 1;"

                stat,dPrior,dump,dump = self.PolSQL.Execute(sql,3)
                if stat <> 'OK':
                    stat = 'Bd pobrania danych fakturowych kontrahenta: '+stat
                    dane.append(stat)
                    break
                
                elif len(dPrior) == 0:
                    if Platnik == '':
                        stat = 'Brak priorytetu w danych do fakturowania dla kontrahenta '+Kontrah
                        dane.append(stat)
                        break
                    
                    # pobranie priorytetu danych do fakturowania dla platnika
                    sql ="SELECT priorytet FROM rs_kontrahenci_fakturowanie"
                    sql+=" WHERE firma_id='"+self.firma_id+"' AND kontrahent_id='"+Platnik+"'"
                    sql+=" AND data_od<='"+DataF+"' ORDER BY data_od DESC LIMIT 1;"
                    
                    stat,dPrior,dump,dump = self.PolSQL.Execute(sql,3)
                    if stat <> 'OK':
                        stat = 'Bd pobrania danych fakturowych patnika: '+stat
                        dane.append(stat)
                        break
                    elif len(dPrior) == 0:
                        stat = 'Brak priorytetu w danych do fakturowania dla kontrahenta '+Kontrah
                        dane.append(stat)
                        break

                SKontrah[Kontrah] = dPrior[0][0]
            
            # uzupelnienie priorytetu
            dPozDD[i][7] = SKontrah[Kontrah]
            
    if stat == 'OK':
        # wszyscy dostaj max, potem bdziemy ciga
        for poz in dPozDD:
            poz[8] = poz[0]
            #informacja o wielkosci limitu na czas testow
            if RozstawKG:
                poz[9] = poz[0]-rsglobal.RSRound(poz[0]*(10-poz[7])*0.1, 4)
            else:
                poz[9] = poz[0]-math.ceil(poz[0]*(10-poz[7])*0.1)
        
        # tyle brakuje do rozstawienia penego
        IlBrakuje = IloscDD-IloscDoR  
        
        if IlBrakuje>0:
            # brak ilosci na wszystkie dowody dostaw. Wiemy, e musimy zdj IlBrakuje
            # zatem najpierw zdejmujemy z priorytetw <> 0 tak, by nie przekroczy % priorytetu
            Zdjeto=True
            while Zdjeto:
                Zdjeto=False
                for poz in dPozDD:
                    if IlBrakuje>0:
                        MinLimit=math.ceil(poz[0]*(10-poz[7])*0.1)
                        if MinLimit<poz[8]:
                            poz[8]=poz[8]-1
                            IlBrakuje=IlBrakuje-1
                            Zdjeto=True
            
        # jeli si nie udao nie przekroczy priorytetw to zdejmujemy w ptli ze wszystkich kontrahentw 
        # prcz tych z priorytetem 0
        if IlBrakuje>0:
            Zdjeto=True
            while Zdjeto:
                Zdjeto=False
                for poz in dPozDD:
                    if IlBrakuje>0 and poz[7]>0 and poz[8]>0:
                        poz[8]=poz[8]-1
                        IlBrakuje=IlBrakuje-1
                        Zdjeto=True

        # zostay priorytety 0 do ograniczenia
        if IlBrakuje>0:
            Zdjeto=True
            while Zdjeto:
                Zdjeto=False
                for poz in dPozDD:
                    if IlBrakuje>0 and poz[8]>0:
                        poz[8]=poz[8]-1
                        IlBrakuje=IlBrakuje-1
                        Zdjeto=True
    
        rsglobal.ZapiszDoLogow('dPozDD 2 '+str(dPozDD)+' Brakuje '+str(IlBrakuje),5)
    
    if stat == 'OK':
        RozstDDPS = {}  #slownik z rozstawionymi ilosciami z dowodow dostaw
                        #wg tras: [trasa]=ilosc
                        
        for i in xrange(len(dPozDD)):
            if dPozDD[i][8] > 0:
                #laczna ilosc na trasie
                if rsglobal.Dekoduj(dPozDD[i][1]) in RozstDDPS:
                    RozstDDPS[rsglobal.Dekoduj(dPozDD[i][1])] += dPozDD[i][8]
                else:
                    RozstDDPS[rsglobal.Dekoduj(dPozDD[i][1])] = dPozDD[i][8]
        
        
        sql ="SELECT bt.boks_id,bt.trasa,TRIM(b.boks_ip),bt.tryb_dostawy,COALESCE(b.pomieszczenie,'')"
        sql+=" FROM rs_boksy_trasy bt"
        sql+=" INNER JOIN rs_boksy b USING(firma_id,oddzial_id,boks_id)"
        sql+=" WHERE bt.firma_id='"+self.firma_id+"' AND bt.oddzial_id='"+self.oddzial_id+"'"
        sql+=" AND bt.tryb_dostawy IN "+ListaTrybow
        sql+=" AND bt.dzien_tygodnia=CASE WHEN EXTRACT (DOW FROM '"+DataF+"'::DATE)=0"
        sql+="  THEN 7 ELSE EXTRACT (DOW FROM '"+DataF+"'::DATE) END"
        sql+=" AND bt.trasa IN "+str(RozstDDPS.keys()).replace("[","(").replace("]",")")
        
        #--POMIESZCZENIE--
        if PomieszczenieF <> '':
            sql+=" AND b.pomieszczenie='"+PomieszczenieF+"'"
        
        sql+=" ORDER BY bt.trasa,bt.boks_id;"

        stat,dBoksyTrasy,dump,dump = self.PolSQL.Execute(sql,3)
        if stat <> 'OK':
            stat = 'Bd pobrania danych z tras dla boksw: '+stat
            dane.append(stat)
    
    if stat == 'OK':
        #lista boksow odpowiadajacych wybranemu dniowi
        sql ="SELECT btd.boks_id,btd.trasa,TRIM(b.boks_ip),btd.tryb_dostawy,COALESCE(b.pomieszczenie,'')"
        sql+=" FROM rs_boksy_trasy_dni btd"
        sql+=" INNER JOIN rs_boksy b USING(firma_id,oddzial_id,boks_id)"
        sql+=" WHERE btd.firma_id='"+self.firma_id+"' AND btd.oddzial_id='"+self.oddzial_id+"'"
        sql+=" AND btd.tryb_dostawy IN "+ListaTrybow+" AND btd.data<='"+DataF+"' AND (btd.data_do>='"+DataF+"' OR data_do IS NULL)"
        sql+=" AND NOT btd.osobny_przerzut"
        sql+=" AND btd.trasa IN "+str(RozstDDPS.keys()).replace("[","(").replace("]",")")

        #--POMIESZCZENIE--
        if PomieszczenieF <> '':
            sql+=" AND b.pomieszczenie='"+PomieszczenieF+"'"

        sql+=" ORDER BY btd.trasa,btd.boks_id;"
        
        stat,dBoksyTrasyDni,dump,dump = self.PolSQL.Execute(sql,3)
        if stat <> 'OK':
            stat = 'Bd pobrania danych boksw: '+stat
            dane.append(stat)

    #Tabele wynikowe dla rozstawienia: Rozst, NRozst
    #  LRozst[trasa,boks,ilosc,ilosc_opakowan_zbiorczych,ilosc_luzem,ip,nr_wiersza] - rozstawione
    #  LNRozst[trasa,ilosc,ilosc_opakowan_zbiorczych,ilosc_luzem] - nierozstawione (brak boksu)
    if stat == 'OK':
        dBoksy = []
        LRozst = []
        LNRozst = []
        
        #-------------------------------------------------------------------
        # LDD = lista pomocnicza z dowodow dostaw na czas testow priorytetow
        # [kontrahent,zamowiono,rozstawiono,priorytet,maks.redukcja]
        LDD = []
        for i in xrange(len(dPozDD)):
            LDD.append([dPozDD[i][5],
                       dPozDD[i][0],
                       dPozDD[i][8],
                       dPozDD[i][7],
                       dPozDD[i][9],
                       ])
        #-------------------------------------------------------------------
        
        #
        sBoksySumy = {}
    
        dBoksy.extend(dBoksyTrasy)
        dBoksy.extend(dBoksyTrasyDni)
        
        Trasy = RozstDDPS.keys()
        #sortowanie ulozy trasy - ulatwienie dla sprawdzania
        Trasy.sort()
        
        if len(dBoksy) == 0:
            #wszystko nierozstawione
            for t in Trasy:
                Ilosc = RozstDDPS[t]
                IloscOpakZb = int(Ilosc/dOpak[0][0])
                if RozstawKG:
                    IloscLuzem = Ilosc-IloscOpakZb*dOpak[0][0]
                else:
                    IloscLuzem = int(Ilosc-IloscOpakZb*dOpak[0][0])
                    
                LNRozst.append([t,Ilosc,IloscOpakZb,IloscLuzem])        
        else:
            #wyszukiwanie wsrod boksow spelniajacych warunki filtru
            for t in Trasy:
                Ilosc = RozstDDPS[t]
                for i in xrange(len(dBoksy)):
                    if dBoksy[i][1] == t:
                        #jesli jest boks dla trasy, to wstawiamy cala ilosc
                        IloscOpakZb = int(Ilosc/dOpak[0][0])
                        if RozstawKG:
                            IloscLuzem = Ilosc-IloscOpakZb*dOpak[0][0]
                        else:
                            IloscLuzem = int(Ilosc-IloscOpakZb*dOpak[0][0])
                            
                        LRozst.append([t,dBoksy[i][0],Ilosc,IloscOpakZb,IloscLuzem,dBoksy[i][2],NrWyswietlacza])
                        Ilosc = 0
                        break
                if Ilosc > 0:
                    #jesli wstawiono ilosci do boksu, to znaczy, ze nierozstawiono
                    IloscOpakZb = int(Ilosc/dOpak[0][0])
                    if RozstawKG:
                        IloscLuzem = Ilosc-IloscOpakZb*dOpak[0][0]
                    else:
                        IloscLuzem = int(Ilosc-IloscOpakZb*dOpak[0][0])
                        
                    LNRozst.append([t,Ilosc,IloscOpakZb,IloscLuzem])

    if stat == 'OK' and len(LRozst) > 0:
        #modyfikacja pozycji rozstawionych dowodow dostaw
        
        #pobieramy liste uzytych boksow, blokujemy je i sprawdzamy
        #czy nie zostalo tam poprzednie rozstawienie
        sql ="SELECT boks_id,"
        sql+=" artykul_wysw_w"+str(NrWyswietlacza)+","
        sql+=" data_wysw_w"+str(NrWyswietlacza)+","
        sql+=" rzut_wysw_w"+str(NrWyswietlacza)+","
        sql+=" ilosc_wysw_w"+str(NrWyswietlacza)+","
        sql+=" COALESCE(pomieszczenie,'')"
        sql+=" FROM rs_boksy"
        sql+=" WHERE firma_id='"+self.firma_id+"'"
        sql+=" AND oddzial_id='"+self.oddzial_id+"' AND boks_id IN ("

        for i in xrange(len(LRozst)):
            if i > 0:
                sql+=","
            sql+="'"+rsglobal.Dekoduj(LRozst[i][1])+"'"
        sql+=") FOR UPDATE NOWAIT;"

        stat,dBoksyDoR,dump,dump = self.PolSQL.Execute(sql,3)
        if stat <> 'OK':
            stat = 'Zablokowane boksy. Sprbuj ponownie za chwil.'
            dane.append(stat)
        elif len(dBoksyDoR) == 0:  #<> len(LRozst):
            stat = 'Brak boksw podanych do rozstawienia.'
            dane.append(stat)
        else:
            for b in dBoksyDoR:
                if b[4] > 0:
                    stat ='Rozstawienie w wierszu '+rsglobal.UsunKropkaZero(NrWyswietlacza)
                    stat+=' jest niemoliwe, poniewa'
                    stat+=' boks '+rsglobal.Dekoduj(b[0]).strip()+' zawiera rozstawienie'
                    stat+=' z dnia '+rsglobal.Dekoduj(b[2]).strip()
                    stat+=' dla rzutu '+rsglobal.Dekoduj(b[3]).strip()
                    stat+=' dla artykuu '+rsglobal.Dekoduj(b[1]).strip()
                    stat+=' w iloci '+rsglobal.UsunKropkaZero(b[4]).strip()+'.'
                    
                dane.append(stat)
        
    if stat == 'OK' and len(LRozst) > 0:
        UPD = ''
        
        sTrasyPomieszczenia = {}
        for i in xrange(len(dBoksy)):
            Trasa = dBoksy[i][1]
            PomieszczenieBoksu = dBoksy[i][4]
            if not Trasa in sTrasyPomieszczenia:
                sTrasyPomieszczenia[Trasa] = PomieszczenieBoksu
        
        
        for i in xrange(len(LRozst)):
            Trasa = LRozst[i][0]
            Ilosc = LRozst[i][2]
            
            #--POMIESZCZENIE--
            PomieszczenieDD = sTrasyPomieszczenia[Trasa]
            if PomieszczenieDD == '':
                PomieszczenieDD = "NULL"
            
            for j in xrange(len(dPozDD)):
                if rsglobal.Dekoduj(dPozDD[j][1]) == Trasa and dPozDD[j][8]>0 and stat == 'OK':
                    Ilosc1 = min(Ilosc, dPozDD[j][8])
                    if Ilosc1 > 0:
                        # jeli opakowanie to przeliczamy iloci do wydania wzgl pojemnoci opakowania
                        IloscSzt1 = Ilosc1
                        if Opak <> '':
                            stat,PojOpak = fs_artykuly_pom.PodajPojemnoscOpakowania(self,Artykul,Opak,dPozDD[j][5])
                            if stat == 'OK':
                                IloscSzt1 = Ilosc1*PojOpak
                        
                        if stat == 'OK':
                            IloscSzt2 = rsglobal.RSRound(IloscSzt1*dPrzelicz[0][0],4)
                            Ilosc2 = rsglobal.RSRound(Ilosc1*dPrzelicz[0][0],4)
    
                            UPD+="UPDATE rs_dowody_dostaw_pozycje"
                            UPD+=" SET ilosc_wysw1=ilosc_wysw1+"+str(IloscSzt1)+","
                            UPD+=" ilosc_wysw2=ilosc_wysw2+"+str(IloscSzt2)+","
                            UPD+=" pomieszczenie="+PomieszczenieDD
                            
                            UPD+=" WHERE firma_id='"+self.firma_id+"'"
                            UPD+=" AND oddzial_id='"+rsglobal.Dekoduj(dPozDD[j][11])+"'"
                            UPD+=" AND trasa='"+rsglobal.Dekoduj(dPozDD[j][1])+"'"
                            UPD+=" AND data='"+DataF+"'"
                            UPD+=" AND tryb_dostawy='"+str(dPozDD[j][10])+"'"
                            UPD+=" AND numer="+str(dPozDD[j][2])
                            UPD+=" AND nr_poz="+str(dPozDD[j][3])+";"
                            
                            # dla opakowa wstawiamy te liczb opakowa wywietlonych do podtabeli
                            if Opak <> '':
                                SQL = "SELECT pojemnosc FROM rs_dowody_dostaw_pozycje_opakowania WHERE "
                                SQL+=" firma_id = '"+self.firma_id+"'"
                                SQL+=" AND oddzial_id='"+rsglobal.Dekoduj(dPozDD[j][11])+"'"
                                SQL+=" AND trasa='"+rsglobal.Dekoduj(dPozDD[j][1])+"'"
                                SQL+=" AND data='"+DataF+"'"
                                SQL+=" AND tryb_dostawy='"+str(dPozDD[j][10])+"'"
                                SQL+=" AND numer="+str(dPozDD[j][2])
                                SQL+=" AND nr_poz="+str(dPozDD[j][3])
                                SQL+=" AND opakowanie_id='"+Opak+"';"
                                stat,DOpakDD,dump,dump = self.PolSQL.Execute(SQL,3)
                                if stat <> 'OK':
                                    stat = 'Bd pobrania podpozycji opakowa z dowodw dostaw: '+stat
                                if stat == 'OK':
                                    if len(DOpakDD) == 0:
                                        # wstawienie nowej podpozycji opakowania
                                        UPD+="INSERT INTO rs_dowody_dostaw_pozycje_opakowania (firma_id,"
                                        UPD+="oddzial_id,trasa,data,tryb_dostawy,numer,nr_poz,"
                                        UPD+="opakowanie_id,pojemnosc,ilosc_wyswietlona) VALUES ("
                                        UPD+="'"+self.firma_id+"','"+rsglobal.Dekoduj(dPozDD[j][11])+"','"+rsglobal.Dekoduj(dPozDD[j][1])+"',"
                                        UPD+="'"+DataF+"','"+str(dPozDD[j][10])+"',"+str(dPozDD[j][2])+","+str(dPozDD[j][3])+","
                                        UPD+="'"+Opak+"',"+str(PojOpak)+","+str(Ilosc1)+");"
                                    else:
                                        # sprawdzenie czy pojemnoci s rwne
                                        if not PojOpak == DOpakDD[0][0]:
                                            stat = 'W pozycji dowodu '+rsglobal.Dekoduj(dPozDD[j][1])+'/'+DataF+'/'+str(dPozDD[j][10])+'/'+str(dPozDD[j][2])+'/'+str(dPozDD[j][3])
                                            stat+=' dla opakowania '+Opak+' pojemno z podpozycji ('+str(DOpak[0][0])+') nie zgadza si z pojemnoci wyliczon z definicji opakowa ('+str(PojOpak)+')'
                                        if stat == 'OK':
                                            # zmiana istniejcej pozycji opakowania
                                            UPD+="UPDATE rs_dowody_dostaw_pozycje_opakowania SET ilosc_wyswietlona = ilosc_wyswietlona+"+str(Ilosc1)
                                            UPD+=" WHERE firma_id = '"+self.firma_id+"'"
                                            UPD+=" AND oddzial_id='"+rsglobal.Dekoduj(dPozDD[j][11])+"'"
                                            UPD+=" AND trasa='"+rsglobal.Dekoduj(dPozDD[j][1])+"'"
                                            UPD+=" AND data='"+DataF+"'"
                                            UPD+=" AND tryb_dostawy='"+str(dPozDD[j][10])+"'"
                                            UPD+=" AND numer="+str(dPozDD[j][2])
                                            UPD+=" AND nr_poz="+str(dPozDD[j][3])
                                            UPD+=" AND opakowanie_id='"+Opak+"';"
                                        
                            Ilosc -= Ilosc1
                        
                if Ilosc == 0:
                    #na tej trasie wszystko rozstawiono na dowody i nie ma
                    #sensu przeszukiwanie dalszych boksow
                    break
        
        # zapisujemy informacje dla wyswietlaczy w rs_boksy
        
        # ilosci w boksach sumujemy, poniewaz w jednym boksie mozna
        # rozstawic sume z kilku tras
        # wczesniejsze sprawdzenie daje pewnosc, ze wybrane boksy sa puste
        
        # sumy ilosci dla wyswietlenia w boksie:
        # kolejnosc taka sama jak w liscie LRozst
        # tylko pole trasy zawiera string z wszystkimi trasami w tym boksie
        # dla tego numeru wiersza
        
        # sBoksySumy[boks] = [trasy, boks, ilosc, ilosc_opak_zb, ilosc_luzem, adres_ip]
        # slownik sBoksySumy zadeklarowny wczesniej, bo moze byc tak,
        # ze nic nie udalo sie rozstawic
        #
        for i in xrange(len(LRozst)):
            # boksy beda posortowane wg numeru na koncu operacji
            Boks = LRozst[i][1].rjust(4)
            AdresIP = 'adres_ip'
            for j in xrange(len(dBoksy)):
                if dBoksy[j][0].rjust(4) == Boks:
                    AdresIP = dBoksy[j][2]
                    break
        
            if not Boks in sBoksySumy:
                # nowy boks w slowniku sum
                sBoksySumy[Boks] = ['',Boks,0,0,0,AdresIP]

            # string z kodami tras
            if sBoksySumy[Boks][0] <> '':
                sBoksySumy[Boks][0] += ','
            sBoksySumy[Boks][0] += LRozst[i][0].strip()

            sBoksySumy[Boks][2] += LRozst[i][2]
            sBoksySumy[Boks][3] += LRozst[i][3]
            sBoksySumy[Boks][4] += LRozst[i][4]
            
            UPD+="UPDATE rs_boksy SET"
            UPD+=" data_wysw_w"+str(NrWyswietlacza)+"='"+DataF+"',"
            UPD+=" rzut_wysw_w"+str(NrWyswietlacza)+"='"+RzutF+"',"
            UPD+=" artykul_wysw_w"+str(NrWyswietlacza)+"='"+Artykul+"',"
            UPD+=" opakowanie_wysw_w"+str(NrWyswietlacza)+"="+WartOpak+","
            UPD+=" ilosc_wysw_w"+str(NrWyswietlacza)+"=ilosc_wysw_w"+str(NrWyswietlacza)+"+"+str(LRozst[i][2])
            UPD+=" WHERE firma_id='"+self.firma_id+"' AND oddzial_id='"+self.oddzial_id+"'"
            UPD+=" AND boks_id='"+rsglobal.Dekoduj(LRozst[i][1])+"'; "
        
        stat,dump,dump,dump = self.PolSQL.Execute(UPD,1)
        if stat <> 'OK':
            stat = 'Bd aktualizacji dowodw dostaw lub boksw: '+stat

    if stat == 'OK':
        # zalaczamy listy wynikowe

        # lista boksow do wyswietlenia uporzadkowana wg numerow boksow
        # dla wygodnego sprawdzania
        LBoksyWysw = []
        kBoksySumy = sBoksySumy.keys()
        for i in xrange(len(kBoksySumy)):
            kBoksySumy[i] = kBoksySumy[i].rjust(4)
            
        kBoksySumy.sort()
        
        for Boks in kBoksySumy:
            Trasy = sBoksySumy[Boks][0]
            Ilosc = sBoksySumy[Boks][2]

            # ilosc_opak_zb i ilosc_luzem wyliczona z calej ilosci
            IloscOpakZb = int(Ilosc/dOpak[0][0])
            if RozstawKG:
                IloscLuzem = Ilosc-IloscOpakZb*dOpak[0][0]
            else:
                IloscLuzem = int(Ilosc-IloscOpakZb*dOpak[0][0])

            AdresIP = sBoksySumy[Boks][5]

            LBoksyWysw.append([Trasy,Boks,Ilosc,IloscOpakZb,IloscLuzem,AdresIP,NrWyswietlacza])
        
        wyn2.append(LRozst)
        wyn2.append(LNRozst)
        wyn2.append(LBoksyWysw)
        
        #---------------------------------------------------------------
        # LDD - lista analizowanych dow.dost.na czas testow priorytetow
        wyn2.append(LDD)
        # LInfo - lista dodatkowych informacji tekstowych na czas testow
        wyn2.append(LInfo)
        #---------------------------------------------------------------


#-------------------------------------------------------------------------------
# ZATWIERDZENIE WYSWIETLONEGO ROZSTAWIENIA (14)
#-------------------------------------------------------------------------------

if Fun == 14 and stat == 'OK':
    if not UprEdycja:
        stat = 'UPR EDY'
        
    if stat == 'OK':
        Opak = klucze[5]
        
        stat,NrWyswietlacza = PodajNrWyswietlacza(self, RzutF, DataF, Artykul, PomieszczenieF, Opak)
        if stat <> 'OK':
            dane.append(stat)
        elif NrWyswietlacza == 0:
            stat = 'Nie znaleziono wrd boksw danych, pasujcych do wywietlenia. Prosz odwiey dane.'
            dane.append(stat)

    if stat == 'OK':
        #wyswietlone pozycje z dowodow dostaw do modyfikacji
        sql ="SELECT ddp.ilosc_wysw1,ddp.ilosc_wysw2,ddp.trasa,dd.numer,ddp.nr_poz,ddp.tryb_dostawy,ddp.oddzial_id," # 0 - 6 
        sql+=" rsf_czy_rozstawiac_asortyment(ddp.firma_id,ddp.oddzial_id,ddp.trasa,"
        sql+=" ddp.data,ddp.tryb_dostawy,'"+self.oddzial_id+"')" # 7
        if Opak <> '':
            sql+=",(SELECT COALESCE(ddpo.ilosc_wyswietlona,-1) FROM rs_dowody_dostaw_pozycje_opakowania ddpo WHERE ddpo.firma_id = ddp.firma_id"
            sql+=" AND ddpo.oddzial_id = ddp.oddzial_id AND ddpo.trasa = ddp.trasa AND ddpo.data = ddp.data"
            sql+=" AND ddpo.tryb_dostawy = ddp.tryb_dostawy AND ddpo.numer = ddp.numer "
            sql+=" AND ddpo.nr_poz = ddp.nr_poz AND ddpo.opakowanie_id = '"+Opak+"')" # 8
        else:
            sql+=",0" # 8
        sql+=",a.przelicznik"     # 9
        sql+=" FROM rs_dowody_dostaw_pozycje ddp"
        sql+=" INNER JOIN rs_dowody_dostaw dd USING(firma_id,oddzial_id,trasa,data,tryb_dostawy,numer)"
        sql+=" INNER JOIN rs_artykuly a USING(firma_id,artykul_id)"
        sql+=" WHERE ddp.firma_id='"+self.firma_id+"'"

        sql+=" AND ddp.data='"+DataF+"'"
        sql+=" AND ddp.artykul_id='"+Artykul+"'"
        sql+=" AND ddp.ilosc_wysw1>0"
        
        #--POMIESZCZENIE--
        if PomieszczenieF <> '':
            sql+=" AND ddp.pomieszczenie='"+PomieszczenieF+"'"

        sql+=" FOR UPDATE NOWAIT;"
        rsglobal.ZapiszDoLogow('!!! sql '+sql,5)
        stat,dPozDD,dump,dump = self.PolSQL.Execute(sql,3)
        rsglobal.ZapiszDoLogow('!!! dPozDD '+str(dPozDD),5)
        if stat=='OK':
            dPozDD=PrzefiltrujDaneWzgledemCzyRozstawiac(dPozDD,7,True)
        rsglobal.ZapiszDoLogow('!!! dPozDD2 '+str(dPozDD),5)
        if stat <> 'OK':
            stat = "Bd pobrania pozycji z dowodw dostaw.\nMoliwe, e jakie dowody dostaw s zablokowane przez innego operatora lub proces.\n\nSprbuj jeszcze raz."
            dane.append(stat)
        elif len(dPozDD) == 0:
            stat = 'Brak pozycji do zatwierdzenia. Prosz odwiey dane.'
            dane.append(stat)

    if stat == 'OK':
        #pobranie i zablokowanie boksow, zawierajacych wyswietlone dane
        #do usuniecia danych wyswietlanych
        sql ="SELECT boks_id,ilosc_wysw_w"+str(NrWyswietlacza)+",boks_ip"
        sql+=" FROM rs_boksy"
        sql+=" WHERE firma_id='"+self.firma_id+"'"
        sql+=" AND data_wysw_w"+str(NrWyswietlacza)+"='"+DataF+"'"
        sql+=" AND rzut_wysw_w"+str(NrWyswietlacza)+"='"+RzutF+"'"
        sql+=" AND artykul_wysw_w"+str(NrWyswietlacza)+"='"+Artykul+"'"
        sql+=" AND COALESCE(opakowanie_wysw_w"+str(NrWyswietlacza)+",'')='"+Opak+"'"
        
        #--POMIESZCZENIE--
        if PomieszczenieF <> '':
            sql+=" AND pomieszczenie='"+PomieszczenieF+"'"
        
        sql+=" FOR UPDATE NOWAIT;"
        
        stat,dBoksy,dump,dump = self.PolSQL.Execute(sql,3)
        if stat <> 'OK':
            stat = 'Zablokowane boksy. Sprbuj ponownie za chwil.'
            dane.append(stat)
        elif len(dBoksy) == 0:
            stat = 'Brak boksw pasujcych do podanego wywietlenia.'
            dane.append(stat)

    if stat == 'OK':
        UPD = ''
        for i in xrange(len(dPozDD)):
            if stat == 'OK':
                # wiersze dla opakowania nie zawierajce tego opakowania ignorujemy
                if Opak <> '':
                    if dPozDD[i][8] == -1:
                        continue
                    
                IloscDD = dPozDD[i][0]
                Ilosc2DD = dPozDD[i][1]
                PojOpak = 1
                if Opak <> '':
                    # liczba wywietlonych opakowa
                    IloscDD = dPozDD[i][8]
                    Ilosc2DD = rsglobal.RSRound(dPozDD[i][8]* dPozDD[i][9],4)
                    
                    # przeliczamy ilo wzgldem pojemnoci z rs_dowody_dostaw_pozycje_opakowania
                    SQL = "SELECT pojemnosc FROM rs_dowody_dostaw_pozycje_opakowania "
                    SQL+=" WHERE firma_id='"+self.firma_id+"'"
                    SQL+=" AND oddzial_id='"+rsglobal.Dekoduj(dPozDD[i][6])+"'"
                    SQL+=" AND trasa='"+rsglobal.Dekoduj(dPozDD[i][2])+"'"
                    SQL+=" AND data='"+DataF+"'"
                    SQL+=" AND tryb_dostawy='"+str(dPozDD[i][5])+"'"
                    SQL+=" AND numer="+str(dPozDD[i][3])
                    SQL+=" AND nr_poz="+str(dPozDD[i][4])
                    SQL+=" AND opakowanie_id='"+str(Opak)+"'"
                    stat,DPojOpak,dump,dump = self.PolSQL.Execute(SQL,3)
                    if stat <> 'OK':
                        stat = 'Bd pobrania pojemnoci opakowania z podtabeli opakowa pozycji dowodw dostaw :'+stat
                        dane.append(stat)
                    elif len(DPojOpak) == 0:
                        stat = 'Brak pojemnoci dla opakowania pozycji dowodu dostawy'
                        dane.append(stat)
                    if stat == 'OK':
                        PojOpak = DPojOpak[0][0]
                        
            if stat == 'OK':
                Ilosc = min(IloscDoR, IloscDD) 
                
                if Ilosc == 0:
                    dane.append(stat)
                    break
                
                if Ilosc > 0:
                    IloscDoR -= Ilosc
                    
                    Ilosc1Szt = rsglobal.RSRound(IloscDD * PojOpak,4)
                    Ilosc2Szt = rsglobal.RSRound(Ilosc2DD * PojOpak,4)
                    
                    UPD+="UPDATE rs_dowody_dostaw_pozycje SET"
                    UPD+=" ilosc_rozst1=ilosc_rozst1 + "+str(Ilosc1Szt)+","
                    UPD+=" ilosc_rozst2=ilosc_rozst2 + "+str(Ilosc2Szt)+","
                    UPD+=" ilosc_wysw1= ilosc_wysw1 - "+str(Ilosc1Szt)+","
                    UPD+=" ilosc_wysw2= ilosc_wysw2 - "+str(Ilosc2Szt)+","
                    UPD+=" pomieszczenie = CASE WHEN ilosc_wysw1 - "+str(Ilosc1Szt)+" = 0 THEN NULL ELSE pomieszczenie END"
                    UPD+=" WHERE firma_id='"+self.firma_id+"'"
                    UPD+=" AND oddzial_id='"+rsglobal.Dekoduj(dPozDD[i][6])+"'"
                    UPD+=" AND trasa='"+rsglobal.Dekoduj(dPozDD[i][2])+"'"
                    UPD+=" AND data='"+DataF+"'"
                    UPD+=" AND tryb_dostawy='"+str(dPozDD[i][5])+"'"
                    UPD+=" AND numer="+str(dPozDD[i][3])
                    UPD+=" AND nr_poz="+str(dPozDD[i][4])+"; "
                
            
                # dla opakowa zmieniamy wiersz z opakowaniami przenoszc ilo z wywietlonej do rozstawionej
                if Opak <> '':
                    UPD+="UPDATE rs_dowody_dostaw_pozycje_opakowania SET ilosc_wyswietlona = 0, ilosc_rozstawiona = ilosc_rozstawiona + ilosc_wyswietlona"
                    UPD+=" WHERE firma_id = '"+self.firma_id+"'"
                    UPD+=" AND oddzial_id='"+rsglobal.Dekoduj(dPozDD[i][6])+"'"
                    UPD+=" AND trasa='"+rsglobal.Dekoduj(dPozDD[i][2])+"'"
                    UPD+=" AND data='"+DataF+"'"
                    UPD+=" AND tryb_dostawy='"+str(dPozDD[i][5])+"'"
                    UPD+=" AND numer="+str(dPozDD[i][3])
                    UPD+=" AND nr_poz="+str(dPozDD[i][4])
                    UPD+=" AND opakowanie_id='"+Opak+"';"
    
    if stat == 'OK':
        for i in xrange(len(dBoksy)):
            LBoksIP.append([dBoksy[i][0],dBoksy[i][2]])
            
            UPD+="UPDATE rs_boksy SET"
            UPD+=" data_wysw_w"+str(NrWyswietlacza)+"=NULL,"
            UPD+=" rzut_wysw_w"+str(NrWyswietlacza)+"=NULL,"
            UPD+=" artykul_wysw_w"+str(NrWyswietlacza)+"=NULL,"
            UPD+=" opakowanie_wysw_w"+str(NrWyswietlacza)+"=NULL,"
            UPD+=" ilosc_wysw_w"+str(NrWyswietlacza)+"=0"
            UPD+=" WHERE firma_id='"+self.firma_id+"' AND oddzial_id='"+self.oddzial_id+"'"
            UPD+=" AND boks_id='"+rsglobal.Dekoduj(dBoksy[i][0])+"'; "

    if stat == 'OK':
        stat,dump,dump,dump = self.PolSQL.Execute(UPD,1)
        if stat <> 'OK':
            stat = 'Bd zatwierdzania wywietlonego rozstawienia: '+stat
            dane.append(stat)
        else:
            #dla ponownego pobrania calego grida
            Artykul = ''


#-------------------------------------------------------------------------------
# ANULACJA WYSWIETLONEGO ROZSTAWIENIA (15)
#-------------------------------------------------------------------------------

if Fun == 15 and stat == 'OK':
    if not UprEdycja:
        stat = 'UPR EDY'

    if stat == 'OK':
        Opak = klucze[5]
        
        stat,NrWyswietlacza = PodajNrWyswietlacza(self, RzutF, DataF, Artykul, PomieszczenieF, Opak)
        if stat <> 'OK':
            dane.append(stat)
        elif NrWyswietlacza == 0:
            stat = 'Nie znaleziono wrd boksw danych, pasujcych do wywietlenia. Prosz odwiey dane.'
            dane.append(stat)

    if stat == 'OK':
        #wyswietlone pozycje z dowodow dostaw do modyfikacji
        sql ="SELECT ddp.ilosc_wysw1,ddp.ilosc_wysw2,ddp.trasa,dd.numer,ddp.nr_poz,ddp.tryb_dostawy,ddp.oddzial_id," # 0 - 6 
        sql+=" rsf_czy_rozstawiac_asortyment(ddp.firma_id,ddp.oddzial_id,ddp.trasa,"
        sql+=" ddp.data,ddp.tryb_dostawy,'"+self.oddzial_id+"')" # 7
        if Opak <> '':
            sql+=",(SELECT COALESCE(ddpo.ilosc_wyswietlona,-1) FROM rs_dowody_dostaw_pozycje_opakowania ddpo WHERE ddpo.firma_id = ddp.firma_id"
            sql+=" AND ddpo.oddzial_id = ddp.oddzial_id AND ddpo.trasa = ddp.trasa AND ddpo.data = ddp.data"
            sql+=" AND ddpo.tryb_dostawy = ddp.tryb_dostawy AND ddpo.numer = ddp.numer "
            sql+=" AND ddpo.nr_poz = ddp.nr_poz AND ddpo.opakowanie_id = '"+Opak+"')" # 8
        else:
            sql+=",0" # 8
        sql+=",a.przelicznik"     # 9
        sql+=" FROM rs_dowody_dostaw_pozycje ddp"
        sql+=" INNER JOIN rs_dowody_dostaw dd USING(firma_id,oddzial_id,trasa,data,tryb_dostawy,numer)"
        sql+=" INNER JOIN rs_artykuly a USING(firma_id,artykul_id)"
        sql+=" WHERE ddp.firma_id='"+self.firma_id+"'"

        sql+=" AND ddp.data='"+DataF+"'"
        sql+=" AND ddp.artykul_id='"+Artykul+"'"
        sql+=" AND ddp.ilosc_wysw1>0"
        
        #--POMIESZCZENIE--
        if PomieszczenieF <> '':
            sql+=" AND ddp.pomieszczenie='"+PomieszczenieF+"'"

        sql+=" FOR UPDATE NOWAIT;"
        
        stat,dPozDD,dump,dump = self.PolSQL.Execute(sql,3)
        if stat=='OK':
            dPozDD=PrzefiltrujDaneWzgledemCzyRozstawiac(dPozDD,7,True)
            
        if stat <> 'OK':
            stat = "Bd pobrania pozycji z dowodw dostaw.\nMoliwe, e jakie dowody dostaw s zablokowane przez innego operatora lub proces.\n\nSprbuj jeszcze raz."
            dane.append(stat)
        elif len(dPozDD) == 0:
            stat = 'Brak pozycji do anulacji. Prosz odwiey dane.'
            dane.append(stat)

    if stat == 'OK':
        #pobranie i zablokowanie boksow, zawierajacych wyswietlone dane
        #do usuniecia danych wyswietlanych
        sql ="SELECT boks_id,ilosc_wysw_w"+str(NrWyswietlacza)+",boks_ip"
        sql+=" FROM rs_boksy"
        sql+=" WHERE firma_id='"+self.firma_id+"'"
        sql+=" AND data_wysw_w"+str(NrWyswietlacza)+"='"+DataF+"'"
        sql+=" AND rzut_wysw_w"+str(NrWyswietlacza)+"='"+RzutF+"'"
        sql+=" AND artykul_wysw_w"+str(NrWyswietlacza)+"='"+Artykul+"'"
        sql+=" AND COALESCE(opakowanie_wysw_w"+str(NrWyswietlacza)+",'') = '"+Opak+"'"

        #--POMIESZCZENIE--
        if PomieszczenieF <> '':
            sql+=" AND pomieszczenie='"+PomieszczenieF+"'"
        
        sql+=" FOR UPDATE NOWAIT;"
        
        stat,dBoksy,dump,dump = self.PolSQL.Execute(sql,3)
        if stat <> 'OK':
            stat = 'Zablokowane boksy. Sprbuj ponownie za chwil.'
            dane.append(stat)
        elif len(dBoksy) == 0:
            stat = 'Brak boksw pasujcych do podanego wywietlenia.'
            dane.append(stat)

    if stat == 'OK':
        UPD = ''
        for i in xrange(len(dPozDD)):
            if stat == 'OK':
                # wiersze dla opakowania nie zawierajce tego opakowania ignorujemy
                if Opak <> '':
                    if dPozDD[i][8] == -1:
                        continue
                    
                IloscDD = dPozDD[i][0]
                Ilosc2DD = dPozDD[i][1]
                PojOpak = 1
                if Opak <> '':
                    # liczba wywietlonych opakowa
                    IloscDD = dPozDD[i][8]
                    Ilosc2DD = rsglobal.RSRound(dPozDD[i][8]* dPozDD[i][9],4)
                    
                    # przeliczamy ilo wzgldem pojemnoci z rs_dowody_dostaw_pozycje_opakowania
                    SQL = "SELECT pojemnosc FROM rs_dowody_dostaw_pozycje_opakowania "
                    SQL+=" WHERE firma_id='"+self.firma_id+"'"
                    SQL+=" AND oddzial_id='"+rsglobal.Dekoduj(dPozDD[i][6])+"'"
                    SQL+=" AND trasa='"+rsglobal.Dekoduj(dPozDD[i][2])+"'"
                    SQL+=" AND data='"+DataF+"'"
                    SQL+=" AND tryb_dostawy='"+str(dPozDD[i][5])+"'"
                    SQL+=" AND numer="+str(dPozDD[i][3])
                    SQL+=" AND nr_poz="+str(dPozDD[i][4])
                    SQL+=" AND opakowanie_id='"+str(Opak)+"'"
                    stat,DPojOpak,dump,dump = self.PolSQL.Execute(SQL,3)
                    if stat <> 'OK':
                        stat = 'Bd pobrania pojemnoci opakowania z podtabeli opakowa pozycji dowodw dostaw :'+stat
                        dane.append(stat)
                    elif len(DPojOpak) == 0:
                        stat = 'Brak pojemnoci dla opakowania pozycji dowodu dostawy'
                        dane.append(stat)
                    if stat == 'OK':
                        PojOpak = DPojOpak[0][0]
                        
            if stat == 'OK':
                Ilosc = min(IloscDoR, IloscDD) 
                
                if Ilosc == 0:
                    dane.append(stat)
                    break
                
                if Ilosc > 0:
                    IloscDoR -= Ilosc
                    
                    Ilosc1Szt = rsglobal.RSRound(IloscDD * PojOpak,4)
                    Ilosc2Szt = rsglobal.RSRound(Ilosc2DD * PojOpak,4)
                    
                    UPD+="UPDATE rs_dowody_dostaw_pozycje SET "
                    UPD+=" ilosc_wysw1= ilosc_wysw1 - "+str(Ilosc1Szt)+","
                    UPD+=" ilosc_wysw2= ilosc_wysw2 - "+str(Ilosc2Szt)+","
                    UPD+=" pomieszczenie = CASE WHEN ilosc_wysw1 - "+str(Ilosc1Szt)+" = 0 THEN NULL ELSE pomieszczenie END"
                    UPD+=" WHERE firma_id='"+self.firma_id+"'"
                    UPD+=" AND oddzial_id='"+rsglobal.Dekoduj(dPozDD[i][6])+"'"
                    UPD+=" AND trasa='"+rsglobal.Dekoduj(dPozDD[i][2])+"'"
                    UPD+=" AND data='"+DataF+"'"
                    UPD+=" AND tryb_dostawy='"+str(dPozDD[i][5])+"'"
                    UPD+=" AND numer="+str(dPozDD[i][3])
                    UPD+=" AND nr_poz="+str(dPozDD[i][4])+"; "
            
                # dla opakowa zmieniamy wiersz z opakowaniami czyszczc ilo wywietlon
                if Opak <> '':
                    UPD+="UPDATE rs_dowody_dostaw_pozycje_opakowania SET ilosc_wyswietlona = 0"
                    UPD+=" WHERE firma_id = '"+self.firma_id+"'"
                    UPD+=" AND oddzial_id='"+rsglobal.Dekoduj(dPozDD[i][6])+"'"
                    UPD+=" AND trasa='"+rsglobal.Dekoduj(dPozDD[i][2])+"'"
                    UPD+=" AND data='"+DataF+"'"
                    UPD+=" AND tryb_dostawy='"+str(dPozDD[i][5])+"'"
                    UPD+=" AND numer="+str(dPozDD[i][3])
                    UPD+=" AND nr_poz="+str(dPozDD[i][4])
                    UPD+=" AND opakowanie_id='"+Opak+"';"


    if stat == 'OK':   
        # usunicie podpozycji z opakowaniami z zerowymi ilosciami
        if Opak <> '':
            UPD+="DELETE FROM rs_dowody_dostaw_pozycje_opakowania WHERE firma_id='"+self.firma_id+"'"
            UPD+=" AND opakowanie_id='"+Opak+"' AND ilosc_wyswietlona = 0 AND ilosc_rozstawiona = 0;"
        
        # wyczyszczenie boksw
        for i in xrange(len(dBoksy)):
            LBoksIP.append([dBoksy[i][0],dBoksy[i][2]])
                        
            UPD+="UPDATE rs_boksy SET"
            UPD+=" data_wysw_w"+str(NrWyswietlacza)+"=NULL,"
            UPD+=" rzut_wysw_w"+str(NrWyswietlacza)+"=NULL,"
            UPD+=" artykul_wysw_w"+str(NrWyswietlacza)+"=NULL,"
            UPD+=" ilosc_wysw_w"+str(NrWyswietlacza)+"=0"
            UPD+=" WHERE firma_id='"+self.firma_id+"' AND oddzial_id='"+self.oddzial_id+"'"
            UPD+=" AND boks_id='"+rsglobal.Dekoduj(dBoksy[i][0])+"'; "
            
        
    if stat == 'OK':
        stat,dump,dump,dump = self.PolSQL.Execute(UPD,1)
        if stat <> 'OK':
            stat = 'Bd anulacji wywietlonego rozstawienia: '+stat
            dane.append(stat)
        else:
            #dla ponownego pobrania calego grida
            Artykul = ''


#-------------------------------------------------------------------------------
# POBRANIE INFORMACJI O WYSWIETLONYCH ARTYKULACH (16)
#-------------------------------------------------------------------------------

if Fun == 16 and stat == 'OK':
    stat,dane = PodajWyswietloneArtykuly(self,RzutF,DataF,PomieszczenieF)
    if stat <> 'OK':
        dane.append(stat)
    else:
        # slownik musi byc zwracany z fs-a w formie stringu
        # na oknie robimy eval(dane) i dostajemy slownik
        dane = str(dane)        

#-------------------------------------------------------------------------------
# POBRANIE INFORMACJI O ROZSTAWIENIU NA TRASY (17)
#-------------------------------------------------------------------------------

if Fun == 17 and stat == 'OK':
    if stat == 'OK':
        Opak = klucze[5]
    stat,NrWyswietlacza = PodajNrWyswietlacza(self, RzutF, DataF, Artykul, PomieszczenieF, Opak)
    if stat <> 'OK':
        dane.append(stat)
    elif NrWyswietlacza == 0:
        stat = 'Nie znaleziono wrd boksw danych, pasujcych do wywietlenia. Prosz odwiey dane.'
        dane.append(stat)

    if stat == 'OK':    
        #pobranie boksow zawierajacych wyswietlone dane - do sprawdzenia zgodnosci
        sql ="SELECT boks_id,ilosc_wysw_w"+str(NrWyswietlacza)
        sql+=" FROM rs_boksy"
        sql+=" WHERE firma_id='"+self.firma_id+"'"
        sql+=" AND data_wysw_w"+str(NrWyswietlacza)+"='"+DataF+"'"
        sql+=" AND rzut_wysw_w"+str(NrWyswietlacza)+"='"+RzutF+"'"
        sql+=" AND artykul_wysw_w"+str(NrWyswietlacza)+"='"+Artykul+"'"
        sql+=" AND COALESCE(opakowanie_wysw_w"+str(NrWyswietlacza)+",'')='"+Opak+"'"
        
        #--POMIESZCZENIE--
        if PomieszczenieF <> '':
            sql+=" AND pomieszczenie='"+PomieszczenieF+"'"
        sql+=";"
        
        stat,dBoksy,dump,dump = self.PolSQL.Execute(sql,3)
        if stat <> 'OK':
            stat = 'Bd pobrania danych do wyswietlenia z boksw : '+stat
            dane.append(stat)
        elif len(dBoksy) == 0:
            stat = 'Brak boksw pasujcych do podanego wywietlenia.'
            dane.append(stat)

    if stat == 'OK':
        # pobranie informacji o trasach z dowodow dostaw
        sql ="SELECT ddp.trasa,SUM(ddp.ilosc_wysw1)," # 1
        sql+=" rsf_czy_rozstawiac_asortyment(ddp.firma_id,ddp.oddzial_id,ddp.trasa,"
        sql+=" ddp.data,ddp.tryb_dostawy,'"+self.oddzial_id+"')" # 2
        if Opak <> '':
            sql+=",SUM(SELECT COALESCE(ddpo.ilosc_wyswietlona,0) FROM rs_dowody_dostaw_pozycje_opakowania ddpo WHERE ddpo.firma_id = ddp.firma_id"
            sql+=" AND ddpo.oddzial_id = ddp.oddzial_id AND ddpo.trasa = ddp.trasa AND ddpo.data = ddp.data"
            sql+=" AND ddpo.tryb_dostawy = ddp.tryb_dostawy AND ddpo.numer = ddp.numer "
            sql+=" AND ddpo.nr_poz = ddp.nr_poz AND ddpo.opakowanie_id = '"+Opak+"')" # 3
        else:
            sql+=",0" # 3
        sql+=" FROM rs_dowody_dostaw_pozycje ddp"
        sql+=" INNER JOIN rs_dowody_dostaw dd USING(firma_id,oddzial_id,trasa,data,tryb_dostawy,numer)"
        sql+=" WHERE ddp.firma_id='"+self.firma_id+"'"
        
        sql+=" AND ddp.tryb_dostawy IN "+ListaTrybow+" AND ddp.data='"+DataF+"'"
        sql+=" AND ddp.artykul_id='"+Artykul+"'"
        sql+=" AND ddp.ilosc_wysw1>0"
        
        if PomieszczenieF <> '':
            stat, TrasyPSql = PodajTrasyITrybyZPomieszczenia(self,DataF,ListaTrybow,PomieszczenieF,JakoSQL=True)
            if stat == 'OK':
                sql+=" AND SUBSTRING((ddp.trasa||'    '),1,4)||ddp.tryb_dostawy IN "+TrasyPSql

        sql+=" GROUP BY ddp.trasa,rsf_czy_rozstawiac_asortyment(ddp.firma_id,ddp.oddzial_id,ddp.trasa,"
        sql+=" ddp.data,ddp.tryb_dostawy,'"+self.oddzial_id+"')"
        sql+=" ORDER BY ddp.trasa;"

    if stat == 'OK':
        stat,dPozDD,dump,dump = self.PolSQL.Execute(sql,3)
        if stat=='OK':
            dPozDD=PrzefiltrujDaneWzgledemCzyRozstawiac(dPozDD,2,True)
        
        if stat <> 'OK':
            stat = 'Bd pobrania pozycji z dowodw dostaw: '+stat
            dane.append(stat)
        elif len(dPozDD) == 0:
            stat = 'Brak pozycji na dowodach dostaw.'
            dane.append(stat)

    if stat == 'OK':
        if Opak == '':
            #czy artykul rozstawiany w kg
            sql ="SELECT UPPER(TRIM(jm1))='KG' FROM rs_artykuly"
            sql+=" WHERE firma_id='"+self.firma_id+"' AND artykul_id='"+Artykul+"';"
            
            stat,RozstawKG,dump,dump = self.PolSQL.Execute(sql,3)
            if stat <> 'OK':
                stat = 'Bd pobrania danych artykuu: '+stat
            elif len(RozstawKG) == 0:
                stat = 'Brak artykuu o podanym kodzie.'
            
            #opakowanie zbiorcze artykulu - pierwsze z listy wg kodu opakowania
            sql ="SELECT ilosc1 FROM rs_artykuly_opakowania_zbiorcze"
            sql+=" WHERE firma_id='"+self.firma_id+"' AND artykul_id='"+Artykul+"'"
            sql+=" ORDER BY opakowanie_id LIMIT 1;"
            
            stat,dOpak,dump,dump = self.PolSQL.Execute(sql,3)
            if stat <> 'OK':
                stat = 'Bd pobrania danych opakowania zbiorczego: '+stat
            elif len(dOpak) == 0:
                stat = 'BR OPAK_ZBIOR'
            elif RozstawKG[0][0]:
                # jesli rozstawienie w kg, to opakowanie zbiorcze rwne 1kg
                dOpak[0][0] = 1
        else:
            # dane ustawione dla opakowania
            RozstawKG=[[False]]
            dOpak = [[1]]

    if stat == 'OK':
        # utworzenie grida z wynikami:
        # 0 - trasa
        # 1 - pelne kosze (kg)
        # 2 - ostatni kosz (reszta z ostatniego kg)
        
        SIlosc = 0
        dane = []
        for i in xrange(len(dPozDD)):
            Trasa = rsglobal.Dekoduj(dPozDD[i][0])
            if Opak == '':
                Ilosc = dPozDD[i][1]
            else:
                Ilosc = dPozDD[i][3]
                
            Pelne = int(Ilosc/dOpak[0][0])
            
            if RozstawKG[0][0]:
                Ostatni = Ilosc-Pelne*dOpak[0][0]
            else:
                Ostatni = int(Ilosc-Pelne*dOpak[0][0])
                
            SIlosc += Ilosc
                
            dane.append([Trasa,Pelne,Ostatni])
        
        # sprawdzenie z iloscia zapisana w boksach
        SIloscWBoksach = 0
        for i in xrange(len(dBoksy)):
            SIloscWBoksach += dBoksy[i][1]
        
        if rsglobal.RSRound(SIlosc,4) <> rsglobal.RSRound(SIloscWBoksach,4):
            stat = 'Ilo zapisana w boksach niezgodna z iloci pobran z dowodw dostaw. Powiadom administratora.'
            dane.append(stat)
    
    if stat == 'OK':
        # informacja o tym, czy rozstawiany w kg, przyda sie do napisow
        wyn2.append(RozstawKG[0][0])
        

#-------------------------------------------------------------------------------
#---   POBRANIE DANYCH (5)
#-------------------------------------------------------------------------------

if stat == 'OK' and Fun in (5,14,15):
    sql ="SELECT magazyn_mm FROM rs_produkcja_dokumenty"
    sql+=" WHERE firma_id='"+self.firma_id+"' AND oddzial_id='"+self.oddzial_id+"';"
    
    stat,MagMM,dump,dump = self.PolSQL.Execute(sql,3)
    if stat <> 'OK':
        stat = 'Bd pobrania informacji o magazynie dla dokumentw MM: '+stat
    elif len(MagMM) == 0:
        stat = 'BR DEF DOK PROD'
    elif MagMM[0][0] == 'None':
        stat = 'BR DEF MAG MM'
    else:
        MagMM = rsglobal.Dekoduj(MagMM[0][0])
    
    if stat == 'OK':
        #filtr obligatoryjny
        Filtr =" ddp.firma_id='"+self.firma_id+"'"
        
        Filtr+=" AND ddp.tryb_dostawy IN "+ListaTrybow+" AND ddp.data='"+DataF+"'"
        
        #filtr wykorzystywany przy pobraniu danych dla konkretnego artykulu
        if Artykul<>'':
            Filtr+= " AND ddp.artykul_id='"+Artykul+"' "
        
        #filtr opcjonalny
        if TrasaF<>'':
            Filtr+= " AND ddp.trasa='"+TrasaF+"' "
        if KategArtF<>'':
            Filtr+= " AND a.kateg_artykulu LIKE '"+KategArtF+"%' "

        # --POMIESZCZENIE--
        if PomieszczenieF <> '':
            stat,TrasyPSql = PodajTrasyITrybyZPomieszczenia(self, DataF, ListaTrybow, PomieszczenieF, JakoSQL=True)
            
            if stat == 'OK':
                Filtr+= " AND SUBSTRING((ddp.trasa||'    '),1,4)||ddp.tryb_dostawy IN "+TrasyPSql
                
    if stat == 'OK':
        sql ="SELECT ddp.artykul_id,a.nazwa,SUM(ddp.ilosc_rozst1),"
        sql+=" SUM(ddp."+PoleIlosci+"- ddp.ilosc_rozst1- ddp.ilosc_wysw1),"
        sql+=" SUM(ddp.ilosc_wysw1),SUM(0),a.jm1,SUM(0),"
        sql+=" rsf_czy_rozstawiac_asortyment(ddp.firma_id,ddp.oddzial_id,ddp.trasa,"
        sql+=" ddp.data,ddp.tryb_dostawy,'"+self.oddzial_id+"')"
        sql+=" FROM rs_dowody_dostaw_pozycje ddp"
        sql+=" INNER JOIN rs_artykuly a USING(firma_id,artykul_id)"
        sql+=" INNER JOIN rs_dowody_dostaw dd USING(firma_id,oddzial_id,trasa,data,tryb_dostawy,numer)"
        sql+=" LEFT JOIN rs_artykuly_magazyny am"
        sql+="  ON (am.firma_id=ddp.firma_id AND am.magazyn_id='"+MagMM+"' AND am.artykul_id=ddp.artykul_id)"
        sql+=" WHERE "+Filtr
        sql+=" GROUP BY ddp.artykul_id,a.nazwa,am.ilosc1,a.jm1,"
        sql+=" rsf_czy_rozstawiac_asortyment(ddp.firma_id,ddp.oddzial_id,ddp.trasa,"
        sql+=" ddp.data,ddp.tryb_dostawy,'"+self.oddzial_id+"')"
        sql+=" ORDER BY ddp.artykul_id;"
        stat,dane,wyn1,dump = self.PolSQL.Execute(sql,3)
        if stat=='OK':
            dane=PrzefiltrujDaneWzgledemCzyRozstawiac(dane,8,True)
        
        if stat <> 'OK':
            stat = "Bd pobrania asortymentu do rozstawienia: "+stat
            dane = []
            dane.append(stat)
        
        #pobieramy ilosci zakonczonych procesow, ktore jeszcze nie maja
        #wpisanego dok.MM - czyli nie zostaly przeniesione do magazynu
        if stat == 'OK' and len(dane) > 0:
            
            #przy okazji tworzenia sql-a tworzymy slownik zawierajacy pozycje
            #poszczegolnych artykulow, pobranych z dowodow dostaw
            #wykorzystamy go przy dopisaniu ilosci pobranych z planu produkcji
            ArtS = {}
            
            sql ="SELECT DISTINCT artykul_id,partia_id,odebrano"
            sql+=" FROM rs_produkcja_plan"
            sql+=" WHERE firma_id='"+self.firma_id+"' AND oddzial_id='"+self.oddzial_id+"' "
            sql+=" AND odebrano>0 AND data_na_kiedy='"+DataF+"'"
            sql+=" AND mm_dokument_id IS NULL"
            sql+=" AND mm_data IS NULL and (mm_numer IS NULL OR mm_numer=0)"
            sql+=" AND (mm_nr_poz IS NULL OR mm_nr_poz=0)"
            sql+=" AND (mm_nr_ppoz IS NULL OR mm_nr_ppoz=0)"
            sql+=" AND artykul_id IN ("
    
            for i in xrange(len(dane)):
                #pozycja w danych z dowodow dostaw
                ArtS[dane[i][0]] = i
                if i > 0:
                    sql+=","
                sql+="'"+rsglobal.Dekoduj(dane[i][0].strip())+"'"
                
            sql+=") ORDER BY artykul_id, partia_id;"
            
            stat,dZPlanu,dump,dump = self.PolSQL.Execute(sql,3)
            if stat <> 'OK':
                stat = 'Bd pobrania iloci z planu produkcji: '+stat
    
        #powiekszamy ilosc do rozstawienia o odebrane ilosci z planu produkcji
        if stat == 'OK' and len(dane) > 0 and len(dZPlanu) > 0:
            for i in xrange(len(dZPlanu)):
                Art = dZPlanu[i][0]
                if Art in ArtS:
                    dane[ArtS[Art]][5] += dZPlanu[i][2]

##------------------------------------------------------------------------------
##        Blokada wyliczenia ilosci w kolumnnie 'Do rozstawienia' do czasu
##        opracowania funkcjonalnosci aby w gridzie nie pojawialy sie
##        ilosci ujemne.
##
##        #pomniejszamy ilosc do rozstawienia o ilosci juz rozstawione
##        if stat == 'OK' and len(dane) > 0:
##            for i in xrange(len(dane)):
##                dane[i][5] -= dane[i][2]
##------------------------------------------------------------------------------

        #pojemnosc opakowan zbiorczych dla artykulow wybranych z dowodow dostaw
        #pobieramy pierwszy rekord z listy uprzadkowanej wg kodu opakowania
        if stat == 'OK' and len(dane) > 0:
            sql ="SELECT artykul_id, ilosc1"
            sql+=" FROM rs_artykuly_opakowania_zbiorcze"
            sql+=" WHERE firma_id='"+self.firma_id+"'"
            sql+=" AND artykul_id IN ("
            for i in xrange(len(dane)):
                if i > 0:
                    sql+=","
                sql+="'"+rsglobal.Dekoduj(dane[i][0].strip())+"'"
            sql+=") ORDER BY artykul_id, opakowanie_id;"
            
            stat,dOpakZbior,dump,dump = self.PolSQL.Execute(sql,3)
            if stat <> 'OK':
                stat = 'Bd pobrania danych opakowa zbiorczych: '+stat
    
        #przeliczamy ilosc do rozstawienia na opakowania zbiorcze
        #opakowania zbiorcze zaokraglamy do 2 miejsc po przecinku i wstawiamy do grida z danymi
        if stat == 'OK' and len(dane) > 0 and len(dOpakZbior) > 0:
            OpakZbior = {}
            for i in dOpakZbior:
                Art = i[0]
                if not Art in OpakZbior:
                    OpakZbior[Art] = i[1]
            
            for i in xrange(len(dane)):
                Art = dane[i][0]
                if Art in OpakZbior:
                    Pojemnosc = OpakZbior[Art]
                    if Pojemnosc > 0:
                        dane[i][7] = round(dane[i][5]/Pojemnosc,2)
    
    if stat == 'OK':
        stat,wyn2 = PodajWyswietloneArtykuly(self,RzutF,DataF,PomieszczenieF)
        if stat <> 'OK':
            dane.append(stat)
            
    if stat == 'OK':
        # slownik musi byc zwracany z fs-a w formie stringu
        # na oknie robimy eval(wyn2) i dostajemy slownik
        wyn2 = [str(wyn2)]
        wyn2.append(LBoksIP)
    
    if stat == 'OK' and Artykul <> '' and len(dane) == 1:
        # jesli pobieralismy informacje o konkretnym artykule
        # to dobierzemy jeszcze info, czy ten artykul rozstawiac w kilogramach
        sql ="SELECT UPPER(TRIM(jm1))='KG',COALESCE(tylko_w_opakowaniach,FALSE) FROM rs_artykuly"
        sql+=" WHERE firma_id='"+self.firma_id+"' AND artykul_id='"+Artykul+"';"
        stat,d,dump,dump = self.PolSQL.Execute(sql,3)
        if stat <> 'OK':
            stat = 'Bd pobrania danych uzupelniajcych artykuu: '+stat
        else:
            wyn2.append(d[0][0])
            wyn2.append(d[0][1])
    else:
        wyn2.append(False)
        wyn2.append(False)
        
#-------------------------------------------------------------------------------
# POBRANIE POJEDYNCZEGO POMIESZCZENIA DLA ODDZIAU (25)
#-------------------------------------------------------------------------------

if stat == 'OK' and Fun == 25:
    SQL ="SELECT DISTINCT COALESCE(pomieszczenie,'') FROM rs_boksy"
    SQL+=" WHERE firma_id='"+self.firma_id+"' AND oddzial_id='"+self.oddzial_id+"';"
    
    stat,DPomieszcz,dump,dump = self.PolSQL.Execute(SQL,3)
    if stat <> 'OK':
        stat = 'Bd pobrania pomieszcze z boksw dla wybranego oddziau: '+stat
    elif len(DPomieszcz)==1:
        dane=[DPomieszcz[0][0]]
    else:
        dane=['']