# -*- coding: iso8859_2 -*-

# -----------------------------------------------------------------------------
# Name:        fs_wf_korespondencja_przychodzaca.py
# Purpose:     Obsuga grida korespondencji przychodzcej
# Author:      Wojciech Skowron
# Created:     2021/07/26
# RCS-ID:      $Id: fs_wf_korespondencja_przychodzaca.py $
# Copyright:   (c) 2021 RecordSystem
# Licence:     GNU GPL
# -----------------------------------------------------------------------------

# 2022-07-20 Aleksander Bargie (13099) Zmiany w korespondencji przychodzcej
# 2022-08-18 Wojciech Skowron   (13122) Kopiowanie ostatniej pozycji (10)
# 2022-10-05 Aleksander Bargie (13193) [WF] Korespondencja przychodzca - ZPO
# 2023-02-21 Aleksander Bargie (13359) Korespondencja przychodzca - drobiazgi

import rsglobal

"""
302110 Przegldanie
302120 Modyfikacja
"""

stat = 'OK'
dane = []
wyn1 = []
wyn2 = []

if 'Funkcja' not in klucze:
    stat = 'Brak podanej funkcji w kluczach'

if stat == 'OK':
    Funkcja = klucze['Funkcja']

    SlPar = {'Oddzial':       [5,10],
             'DataOdF':       [5],
             'DataDoF':       [5],
             'ZagadnienieF':  [5],
             'KomorkaF':      [5],
             'KontrahentF':   [5],
             'ZPOF':          [5]}  #AB 05.10.22r
    Lista = rsglobal.SprawdzParametryWKluczach(SlPar, Funkcja, klucze)
    if Lista != '':
        stat = 'Uwaga : wywoanie funkcji fs_wf_korespondencja_przychodzaca z numerem ' + str(Funkcja)
        stat += ' bez parametrw: ' + Lista

if stat == 'OK' and Funkcja in (5,):
    Oddzial       = rsglobal.Dekoduj(klucze['Oddzial'])
    DataOdF       = rsglobal.Dekoduj(klucze['DataOdF'])
    DataDoF       = rsglobal.Dekoduj(klucze['DataDoF'])
    ZagadnienieF  = rsglobal.Dekoduj(klucze['ZagadnienieF'])
    KomorkaF      = rsglobal.Dekoduj(klucze['KomorkaF'])
    KontrahentF   = rsglobal.Dekoduj(klucze['KontrahentF'])
    ZPOF          = rsglobal.Dekoduj(klucze['ZPOF'])  #AB 05.10.22r

if stat == 'OK' and Funkcja in (10,):
    Oddzial       = rsglobal.Dekoduj(klucze['Oddzial'])

# -------------------------------------------------------------------------------
# -----------------------   UPRAWNIENIA   ---------------------------------------
# -------------------------------------------------------------------------------

UprMod = ((Oddzial, '302120') in self.uprawnienia) or (self.operator_id == STANDARD_OPERATOR_ID)
UprPrz = ((Oddzial, '302110') in self.uprawnienia) or (self.operator_id == STANDARD_OPERATOR_ID) or UprMod

# -------------------------------------------------------------------------------
# -----------------   KOPIOWANIE OSTATNIEJ POZYCJI(10)   ------------------------
# -------------------------------------------------------------------------------

#WS 2022-08-18
if stat == 'OK' and Funkcja in (10,):
    if not UprMod > 0:
        stat = 'UPR_MOD'
        dane.append(stat)
    else:
        stat, pola_tabeli = rsglobal.PolaTabeli(self.PolSQL, 'rs_wf_korespondencja_przychodzaca')
        if stat == 'OK':
            dane = [None, None]

            pola_tabeli.remove('numer')
            pola_tabeli.remove('zagadnienie_id')
            pola = ",".join(pola_tabeli)

            SEL = """SELECT rok, MAX(numer)+1 FROM rs_wf_korespondencja_przychodzaca 
                     WHERE firma_id='{self.firma_id}' AND oddzial_id='{Oddzial}'
                     GROUP BY rok ORDER BY rok DESC LIMIT 1;"""
            SEL = SEL.format(**locals())
            stat, dane_numer, dump, dump = self.PolSQL.Execute(SEL, 3)
            if stat != 'OK':
                stat = 'Bad pobrania numeru nowej pozycji:' + stat
            elif len(dane_numer) > 0:
                rok = dane_numer[0][0]
                numer = dane_numer[0][1]
                dane = [rok, numer]

                INS = """INSERT INTO rs_wf_korespondencja_przychodzaca ({pola},numer)
                         SELECT {pola},{numer} FROM rs_wf_korespondencja_przychodzaca 
                         WHERE firma_id='{self.firma_id}' AND oddzial_id='{Oddzial}'
                         ORDER BY rok DESC, numer DESC LIMIT 1;"""
                INS = INS.format(**locals())
                stat, dump, dump, dump = self.PolSQL.Execute(INS, 1)
                if stat != 'OK':
                    stat = 'Bd kopiowania pozycji: '+stat

# -------------------------------------------------------------------------------
# ----------------------   POBRANIE DANYCH(5,10)   ------------------------------
# -------------------------------------------------------------------------------

if stat == 'OK' and Funkcja in (5,):
    if not UprPrz > 0:
        stat = 'UPR_PRZ'
        dane.append(stat)
    else:  #AB 2022-07-14 dodany join na komorki zeby pobierac opis
        filtry = ''
        filtry_lista = []
        if DataOdF not in ('', '    -  -  ','None') and rsglobal.SprawdzCzyToData(DataOdF):
            filtry_lista.append(" kp.data>='"+DataOdF+"'")
        if DataDoF not in ('', '    -  -  ','None') and rsglobal.SprawdzCzyToData(DataDoF):
            filtry_lista.append(" kp.data<='"+DataDoF+"'")
        if ZagadnienieF != 'None':
            filtry_lista.append(" kp.zagadnienie_id="+str(ZagadnienieF))
        if KomorkaF != '':
            filtry_lista.append(" upper(kp.komorka_id) LIKE upper('"+KomorkaF+"%')")
        if KontrahentF != '':
            filtry_lista.append(" upper(kp.kontrahent_id) LIKE upper('"+KontrahentF+"%')")
        if ZPOF:
            filtry_lista.append(" czy_zpo")  #AB 05.10.22r
        if len(filtry_lista) > 0:
            filtry = " AND "+" AND ".join(filtry_lista)
        #AB 2023-02-21
        SEL = """SELECT data, rok, numer, 
                        kp.komorka_id || ' - ' || kz.opis, kp.kontrahent_id, kp.nadawca_nazwa, 
                        kp.nadawca_adres, kp.polecony_nr, kp.czy_zpo, kp.temat, 
                        kp.dokument_id, 
                        EXISTS(select 1 
                               from rs_wf_zagadnienia_pliki p 
                               where (firma_id, zagadnienie_id) = (kp.firma_id, kp.zagadnienie_id)) AS czy_zalaczniki,
                        kp.zagadnienie_id
                 FROM rs_wf_korespondencja_przychodzaca kp
                 LEFT JOIN rs_komorki_zatrudniajace kz ON (kz.firma_id, kz.kod) = (kp.firma_id, kp.komorka_id) 
                 WHERE kp.firma_id='{self.firma_id}' AND kp.oddzial_id='{Oddzial}' {filtry}
                 ORDER BY data DESC, numer DESC;
              """.format(**locals())
        stat, dane, wyn1, dump = self.PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
        rsglobal.ZapiszDoLogow(SEL)
        if stat != 'OK':
            rsglobal.ZapiszDoLogow(stat)
            stat = 'Bd pobrania danych: '+stat
        else:
            #WS 2022-08-18
            SEL = """SELECT rok, MAX(numer) FROM rs_wf_korespondencja_przychodzaca 
                     WHERE firma_id='{self.firma_id}' AND oddzial_id='{Oddzial}'
                     GROUP BY rok ORDER BY rok DESC LIMIT 1;"""
            SEL = SEL.format(**locals())
            stat, dane_numer, dump, dump = self.PolSQL.Execute(SEL, 3)
            if stat == 'OK' and len(dane_numer) > 0:
                wyn2 = dane_numer[0]
