# -*- coding: iso8859_2 -*-

# -----------------------------------------------------------------------------
# Name:        fs_wf_ksef.py
# Purpose:     Obsuga grida KSeF - Dokumenty
# Author:      Wojciech Skowron
# Created:     2023/12/14
# RCS-ID:      $Id: fs_wf_ksef.py $
# Copyright:   (c) 2023 RecordSystem
# Licence:     GNU GPL
# -----------------------------------------------------------------------------

# 2023-12-20 Wojciech Skowron - (13650) Funkcje 12, 35, 36 - tworzenie zagadnienia

import ast
import binascii
import os
from StringIO import StringIO
import tempfile
import traceback
import zipfile

from funkcje import fs_pliki_global, fs_wf_zagadnienia_pom
import rsglobal

"""
306000 KSeF - Dokumenty
"""

pola_z_zagadnienia = ('kontrahent_id', 'kontrahent_nazwa', 'kontrahent_nip',
                      'numer_faktury', 'data_wystawienia', 'kwota_brutto')
stat = 'UPR PRZ'

# -------------------------------------------------------------------------------
# -----------------------   UPRAWNIENIA   ---------------------------------------
# -------------------------------------------------------------------------------

UprPrz = ((self.oddzial_id, '306000') in self.uprawnienia) or (self.operator_id == STANDARD_OPERATOR_ID)

if UprPrz > 0:
    stat = 'OK'

if stat == 'OK' and not 'Funkcja' in klucze:
    stat = 'Brak podanej funkcji w kluczach'

if stat == "OK":
    Funkcja = klucze['Funkcja']

    SlPar = {'StanF':[5], 'RodzajDokumentuF':[5],
             'KontrahentF':[5], 'KontrahentNazwaF':[5], 'KontrahentNIPF':[5],
             'DataOdF':[5], 'DataDoF':[5],
             'NrFakturyF':[5], 'KwotaOdF':[5], 'KwotaDoF':[5], 'NrKsefF':[5],
             'Numer':[11,12], 'Podglad':[11],
             'PrzypnijDo':[12], 'StanZagadnienia':[12], 'TematZagadnienia':[12], 'OpisZagadnienia':[12],
             'Dokument':[12,35,36], 'Komorka':[12,35,36], 'Operator':[36]}
    Lista = rsglobal.SprawdzParametryWKluczach(SlPar,Funkcja,klucze)
    if Lista != '':
        stat = 'Uwaga : wywoanie funkcji fs_wf_ksef z numerem '+str(Funkcja)+' bez parametrw: '+Lista

if stat == "OK":
    if Funkcja == 5:
        StanF = rsglobal.Dekoduj(klucze['StanF'])
        RodzajDokumentuF = rsglobal.Dekoduj(klucze['RodzajDokumentuF'])
        KontrahentF = rsglobal.Dekoduj(klucze['KontrahentF'])
        KontrahentNazwaF = rsglobal.Dekoduj(klucze['KontrahentNazwaF'])
        KontrahentNIPF = rsglobal.Dekoduj(klucze['KontrahentNIPF'])
        DataOdF = rsglobal.Dekoduj(klucze['DataOdF'])
        DataDoF = rsglobal.Dekoduj(klucze['DataDoF'])
        NrFakturyF = rsglobal.Dekoduj(klucze['NrFakturyF'])
        KwotaOdF = rsglobal.Dekoduj(klucze['KwotaOdF'])
        KwotaDoF = rsglobal.Dekoduj(klucze['KwotaDoF'])
        NrKsefF = rsglobal.Dekoduj(klucze['NrKsefF'])

    if Funkcja == 11:
        Numer = rsglobal.Dekoduj(klucze['Numer'])
        Podglad = rsglobal.Dekoduj(klucze['Podglad'])

    if Funkcja in (12, 35, 36):
        Dokument = rsglobal.Dekoduj(klucze['Dokument'])
        Komorka = rsglobal.Dekoduj(klucze['Komorka'])

    if Funkcja == 12:
        Numer = rsglobal.Dekoduj(klucze['Numer'])
        PrzypnijDo = rsglobal.Dekoduj(klucze['PrzypnijDo'])
        StanZagadnienia = rsglobal.Dekoduj(klucze['StanZagadnienia'])
        TematZagadnienia = rsglobal.Dekoduj(klucze['TematZagadnienia'])
        OpisZagadnienia = rsglobal.Dekoduj(klucze['OpisZagadnienia'])

    if Funkcja == 36:
        Operator = rsglobal.Dekoduj(klucze['Operator'])

# -------------------------------------------------------------------------------
# -----------------------   POBRANIE DANYCH (5)   -------------------------------
# -------------------------------------------------------------------------------

if stat == 'OK' and Funkcja in (5, ):
    filtr = ''
    args = [self.firma_id]

    if StanF != '':
        filtr += " AND upper(d.stan) = upper(%s)"
        args.append(StanF)
    if RodzajDokumentuF.strip() != '':
        filtr += " AND upper(d.rodzaj_faktury) = upper(%s)"
        args.append(RodzajDokumentuF.strip())
    if KontrahentF != '':
        filtr += " AND upper(d.kontrahent_id) LIKE upper(%s)"
        args.append(KontrahentF.strip()+'%')
    if KontrahentNazwaF.strip() != '':
        filtr += " AND upper(d.kontrahent_nazwa) LIKE upper(%s)"
        args.append('%'+KontrahentNazwaF.strip()+'%')
    if KontrahentNIPF != '':
        filtr += " AND regexp_replace(d.kontrahent_nip, '[^0-9]', '', 'g') LIKE %s"
        args.append(rsglobal.re.sub('[^0-9]', '', KontrahentNIPF)+'%')
    if DataOdF not in ('', '    -  -  ','None') and rsglobal.SprawdzCzyToData(DataOdF):
        filtr += " AND d.data_wystawienia >= %s"
        args.append(DataOdF)
    if DataDoF not in ('', '    -  -  ','None') and rsglobal.SprawdzCzyToData(DataDoF):
        filtr += " AND d.data_wystawienia <= %s"
        args.append(DataDoF)
    if NrFakturyF.strip() != '':
        filtr += " AND upper(d.numer_faktury) LIKE upper(%s)"
        args.append(NrFakturyF.strip()+'%')
    if KwotaOdF is not None:
        filtr += " AND d.kwota_brutto >= %s"
        args.append(KwotaOdF)
    if KwotaDoF is not None:
        filtr += " AND d.kwota_brutto <= %s"
        args.append(KwotaDoF)
    if NrKsefF != '':
        filtr += " AND d.numer_ksef = %s"
        args.append(NrKsefF)

    SEL = """SELECT d.numer, d.stan, d.rodzaj_faktury, d.kontrahent_id || ' - ' || d.kontrahent_nazwa AS kontrahent, d.kontrahent_nip,
                    d.data_wystawienia, d.data_dostawy, d.miejsce_wystawienia, d.numer_faktury, d.waluta, d.kwota_brutto,
                    d.oznaczenia, d.numer_ksef, d.zagadnienie_id,
                    COALESCE(d.fk_dowod_id ||'/'|| d.fk_nazwa_id ||'/'|| d.fk_data ||'/'|| d.fk_numer, '') AS dowod_ksiegowy,
                    to_char(d.czas_pobrania, 'YYYY-MM-DD HH24:MI:SS') AS czas_pobrania
             FROM rs_wf_ksef_dokumenty d
             WHERE d.firma_id=%s {filtr} ORDER BY d.data_wystawienia;""".format(**locals())
    rsglobal.ZapiszDoLogow('Pobranie dokumentw KSeF: '+self.PolSQL.Mogrify(SEL, args))
    stat, dane, wyn1, dump = self.PolSQL.Execute(SEL, 3, args)
    if stat != 'OK':
        stat = 'Bd pobierania danych: '+stat

# -------------------------------------------------------------------------------
# -----------------------   POBRANIE DANYCH (10)   ------------------------------
# -------------------------------------------------------------------------------

# -------------------------------------------------------------------------------
# -----------------------   PODGLD DOKUMENTU (11)   ----------------------------
# -------------------------------------------------------------------------------

if stat == 'OK' and Funkcja in (11, ):
    SEL = """SELECT d.plik_id, d.numer_ksef
             FROM rs_wf_ksef_dokumenty d
             WHERE d.firma_id=%s AND d.numer=%s;""".format(**locals())
    args = [self.firma_id, Numer]
    stat, d, dump, dump = self.PolSQL.Execute(SEL, 3, args)
    if stat != 'OK':
        stat = 'Bd pobierania danych: '+stat
    elif len(d) == 0 or d[0][0] is None:
        stat = 'BR_PLIKU'
    else:
        stat, Tresc = fs_pliki_global.PobierzPlikZSerwera(self.KatalogPlikow, d[0][0])
        if stat == 'OK' and Podglad:
            try:
                TempTresc = binascii.unhexlify(Tresc)
                with zipfile.ZipFile(StringIO(TempTresc), "r") as z:
                    TempTresc = z.read('YouWontKnow')
                    TrescHtml = rsglobal.TransformujZXSL(xml_tresc=TempTresc, xsl_sciezka='xsd/fa.xsl')

                    # przygotowanie do przesania
                    TmpPlik = tempfile.mktemp('.html')
                    with open(TmpPlik, 'wb') as p:
                        p.write(TrescHtml)
                        Tresc = fs_pliki_global.PrzygotujPlikDoPrzeslania(TmpPlik)
                    os.remove(TmpPlik)

                    if Tresc is None:
                        stat = 'Bd przetwarzania pliku xml'
            except:
                stat = 'Bd przetwarzania pliku xml'
                rsglobal.ZapiszDoLogow(stat+': '+traceback.format_exc())

        numer_ksef = d[0][1]
        dane = [Tresc, numer_ksef+'.html' if Podglad else numer_ksef+'.xml']

# -------------------------------------------------------------------------------
# -----------------------   TWORZENIE ZAGADNIENIA (12)   ------------------------
# -------------------------------------------------------------------------------

if stat == 'OK' and Funkcja in (12,):
    plik_id = 0
    nazwa_pliku = ''

    # sprawdzenie czy nie ma ju wpisanego zagadnienia na dokumencie, pobranie id pliku i jego nazwy
    SQL = """SELECT d.zagadnienie_id, d.plik_id, p.nazwa 
             FROM rs_wf_ksef_dokumenty d
             LEFT OUTER JOIN rs_pliki p USING (plik_id)
             WHERE d.firma_id=%s AND d.numer=%s;"""
    args = [self.firma_id, Numer]
    stat, d, dump, dump = self.PolSQL.Execute(SQL, 3, args)
    if stat != 'OK':
        stat = 'Bd sprawdzania danych dokumentu przed tworzeniem zagadnienia'
    elif d[0][0] != 0:
        stat = 'BL_ZAG_UTWORZONE'
    else:
        plik_id = d[0][1]
        nazwa_pliku = d[0][2]

    # sprawdzenie poprawnoci rodzaju dokumentu
    if stat == 'OK':
        SQL = "SELECT 1 FROM rs_wf_rodzaje_dokumentow WHERE firma_id=%s AND kod=%s;"
        args = [self.firma_id, Dokument]
        stat, d, dump, dump = self.PolSQL.Execute(SQL, 3, args)
        if stat != 'OK':
            stat = 'Bd sprawdzania rodzaju dokumentu przed tworzeniem zagadnienia'
        elif len(d) == 0:
            stat = 'BL_ZAG_RODZ_DOK'

    # sprawdzenie poprawnoci komrki zatrudniajcej
    if stat == 'OK':
        SQL = "SELECT 1 FROM rs_komorki_zatrudniajace WHERE firma_id=%s AND kod=%s;"
        args = [self.firma_id, Komorka]
        stat, d, dump, dump = self.PolSQL.Execute(SQL, 3, args)
        if stat != 'OK':
            stat = 'Bd sprawdzania rodzaju dokumentu przed tworzeniem zagadnienia'
        elif len(d) == 0:
            stat = 'BL_ZAG_KOMORKA'

    if stat == 'OK':
        stat, zagadnienie_id = fs_wf_zagadnienia_pom.UtworzZagadnienie(self=self, firma_id=self.firma_id, tabela='rs_wf_ksef_dokumenty',
                                                                       pola_klucz=['firma_id', 'numer'],
                                                                       wartosci_klucz=[self.firma_id, Numer],
                                                                       pola_z_zagadnienia=pola_z_zagadnienia,
                                                                       przypnij_do=PrzypnijDo,
                                                                       stan=StanZagadnienia,
                                                                       temat=TematZagadnienia,
                                                                       opis=OpisZagadnienia,
                                                                       pola_dodatkowe_odpowiedniki={'nip': 'kontrahent_nip',
                                                                                                    'symbol_dokumentu': 'numer_faktury',
                                                                                                    'data_dokumentu': 'data_wystawienia'},
                                                                       dodatkowe_wartosci={'dokument_id': Dokument, 'komorka_id': Komorka})
    if stat == 'OK':
        UPD = """UPDATE rs_wf_ksef_dokumenty SET zagadnienie_id=%s WHERE firma_id=%s AND numer=%s;"""
        args = [zagadnienie_id, self.firma_id, Numer]
        stat, dump, dump, dump = self.PolSQL.Execute(UPD, 1, args)
        if stat != 'OK':
            stat = 'Bd dodawania numeru zagadnienia na dokumencie:'+stat

    # doczenie pliku dokumentu do zagadnienia
    if stat == 'OK' and plik_id != 0:
        INSZALACZNIK = """INSERT INTO rs_wf_zagadnienia_pliki
                                   (firma_id, zagadnienie_id, nr_pliku)
                            VALUES (%s,       %s,             %s);""".format(**locals())
        args = [self.firma_id, zagadnienie_id, plik_id]
        stat, dump, dump, dump = self.PolSQL.Execute(INSZALACZNIK, 1, args)
        if stat != 'OK':
            stat = 'Bd dodawania rekordu o nowym pliku: '+stat
        else:
            wzor_hist = """INSERT INTO rs_wf_zagadnienia_historia (firma_id, zagadnienie_id, zmiana_czas, zmiana_kto, zmiana_co, zmiana_wartosc) 
                           VALUES(%s, %s, NOW() + INTERVAL '{} microseconds', rsf_getuserid(), '{}', '{}');
                        """.format('{}', '{}', '{}')
            args = [self.firma_id, zagadnienie_id]
            npl = nazwa_pliku if len(nazwa_pliku) <= 35 else (nazwa_pliku[:32] + '...')
            INSHISTORIA = wzor_hist.format(99, "Doczono plik {}".format(npl), " ")
            rsglobal.ZapiszDoLogow('Aktualizacja historii zagadnienia po dodaniu pliku: '+self.PolSQL.Mogrify(INSHISTORIA, args))
            stat, dump, dump, dump = self.PolSQL.Execute(INSHISTORIA, 1, args)
            if stat != 'OK':
                stat = "Bd aktualizacji historii zagadnienia: "+stat

# -------------------------------------------------------------------------------
# -----------------------   POBRANIE OPERATORW (35)   --------------------------
# -------------------------------------------------------------------------------

if stat == 'OK' and Funkcja in (35, ):
    SQLOperatorzy = """SELECT DISTINCT operator_id || ' - ' || o.nazwisko || ' ' || o.imie AS op, operator_id
                       FROM rs_wf_uprawnienia_szczegoly 
                       JOIN rs_operatorzy o USING (operator_id)
                       WHERE  firma_id = %s AND dokument_id = %s AND komorka_id = %s
                       ORDER BY operator_id;""".format()

    args = [self.firma_id, Dokument, Komorka]
    stat, dane, wyn1, dump = self.PolSQL.Execute(SQLOperatorzy, 3, args)
    if stat != 'OK':
        stat = u"Bd pobierania operatorw do przekazania: {stat}".format(**locals())


# -------------------------------------------------------------------------------
# -----------------------   POBRANIE STANW (36)   ------------------------------
# -------------------------------------------------------------------------------

if stat == 'OK' and Funkcja in (36, ):
    SQLOperatorzy = """SELECT stany
                       FROM rs_wf_uprawnienia_szczegoly 
                       JOIN rs_operatorzy o USING (operator_id)
                       WHERE firma_id = %s AND dokument_id = %s AND komorka_id = %s AND operator_id = %s
                       ORDER BY operator_id;"""

    args = [self.firma_id, Dokument, Komorka, Operator]
    stat, dane, wyn1, dump = self.PolSQL.Execute(SQLOperatorzy, 3, args)
    if stat != 'OK':
        stat = u"Bd pobierania stanw: {stat}".format(**locals())
    else:
        dane = ast.literal_eval(dane[0][0])
