# -*- coding: iso8859_2 -*-

# -----------------------------------------------------------------------------
# Name:        fs_wf_zagadnienia.py
# Purpose:     Wywietlanie listy zagadnie
# Author:      Aleksander Bargie
# Created:     2021/05/25
# RCS-ID:      $Id: fs_wf_zagadnienia.py $
# Copyright:   (c) 2021 RecordSystem
# Licence:     GNU GPL
# -----------------------------------------------------------------------------
# 2021-10-14 Aleksander Bargie (12669) [WF] rozszerzenia funkcjonalnoci
# 2021-11-17 Aleksander Bargie (12709) [WF] Opisy kodw w Zagadnieniach
# 2021-11-22 Aleksander Bargie (12710) [WF] Przeka zagadnienie w edycji danych zagadnienia
# 2021-11-22 Aleksander Bargie (12708) [WF] Grupowe akceptowanie zagadnie
# 2021-12-02 Aleksander Bargie (12752) [WF] Zagadnienia - automatyczne filtrowanie, operator na gridzie, poprawka filtrowania po operatorze
# 2021-12-02 Aleksander Bargie (12750) [WF] Podgld skanu i data dokumentu w Zagadnieniach
# 2021-12-06 Wojciech Skowron   (12759) [WF] Pozwoli Anakondzie wywietla/edytowa wszystkie zagadnienia
# 2021-12-06 Bartosz Pagan     (12774) [WF] symbol dokumentu plus drobiazgi
# 2022-06-17 Aleksander Bargie (13056) [WF] problem ksigowoci i przekazywania przez zarzad <Zagadnienia>
# 2022-10-05 Aleksander Bargie (13189) [WF] zagadnienia - zmiana filtra na dat wprowadzenia

import rsglobal
from funkcje import fs_pliki_global
from funkcje import fs_wf_zagadnienia_pom as pom
import re

"""
303000 Przegldanie
"""

"""Funkcje:
5. Pobranie danych
15. Pobranie stanw na jakie mona przepisa zagadnienie(lub wszystkich jeli wiele zagadnie)
25. Pobranie operatorw do ktrych mona przypisa zagadnienie(lub wszystkich jeli wiele zagadnie)
35. Pobranie pierwszego pliku na zagadnieniu
45. Pobranie stanw do jakich uprawnienia posiada operator
"""

stat = "OK"
Puste = ('', '    -  -  ', None, 'None', 'NULL')

if 'Funkcja' not in klucze:
    stat = 'Brak podanej funkcji w kluczach'

if stat == 'OK':
    Fun = klucze['Funkcja']

    SlPar = {'Kolumny': [5], 'Filtry': [5], 'Stan': [25], 'Zagadnienia': [15, 25], 'zagadnienie_id': [35]}  #AB 17.11.21r
    Lista = rsglobal.SprawdzParametryWKluczach(SlPar, Fun, klucze)
    if Lista != '':
        import os
        stat = 'Uwaga : wywoanie funkcji '+os.path.basename(__file__)+' z numerem '+str(Fun)+' bez parametrw: '+Lista

if stat == 'OK' and Fun in (5, ):
    Kolumny = klucze['Kolumny']  #AB 17.11.21r
    Filtry = klucze['Filtry']
    for kol in Kolumny:  #AB 14.10.21r
        if re.findall(r'[^a-z_.0-9, ]+', kol):
            stat = "Nazwa kolumny {} nie spelnia wymaga.".format(kol)
    for kl, wart in Filtry.iteritems():
        if isinstance(wart, basestring):
            Filtry[kl] = wart.replace("\\", "\\\\").replace("'", "\\\'")
if stat == 'OK' and Fun in (15, 25, ):  #AB 17.11.21r
    Zagadnienia = klucze['Zagadnienia']
    if 'Stan' in klucze:
        nowy_stan = rsglobal.Dekoduj(klucze['Stan'].strip()).replace("\\", "\\\\").replace("'", "\\\'")
    else:
        nowy_stan = ''

if stat == 'OK' and Fun in (35, ):
    zagadnienie_id = str(rsglobal.Dekoduj(klucze['zagadnienie_id'])).replace("\\", "\\\\").replace("'", "\\\'")

dane = []
wyn1 = []
wyn2 = []
firma_id = self.firma_id


def Execute(self, zapytanie, tryb):
    rsglobal.ZapiszDoLogow(zapytanie)
    return self.PolSQL.Execute(zapytanie, tryb, CzyDodawacBlad=True)

#-------------------------------------------------------------------------------
#---   UPRAWNIENIA
#-------------------------------------------------------------------------------

UprPrz = (((self.oddzial_id, '303000') in self.uprawnienia) or (self.operator_id == STANDARD_OPERATOR_ID))

#-------------------------------------------------------------------------------
#---   POBRANIE DANYCH (5 -select)
#-------------------------------------------------------------------------------

if stat == 'OK' and Fun in (5, ):
    if not UprPrz:
        stat = 'UPR_PRZ'
        dane.append(stat)
    else:
        for x in xrange(len(Kolumny)):  #AB 17.11.21r
            kol        = Kolumny[x]
            if kol in ('dokument_id', 'stan'):
                Kolumny[x] = "COALESCE(z."+kol+"||' - '||"+kol[0:3]+".opis, '') AS "+kol
            elif kol == 'operator_id':
                Kolumny[x] = "COALESCE("+kol[0:3]+".login, '') AS "+kol
            elif kol == 'zalaczniki':
                Kolumny[x] = "EXISTS(SELECT 1 FROM rs_wf_zagadnienia_pliki rwzp WHERE rwzp.zagadnienie_id=z.zagadnienie_id) AS "+kol
            elif kol not in ('false', 'k.nazwa'):
                Kolumny[x] = "z."+kol

        Kolumny = ', '.join(Kolumny)
        where = ''
        if 'zagadnienie_id' in Filtry:
            where = " AND zagadnienie_id = " + str(Filtry['zagadnienie_id']).strip('L')
        else:
            for klucz, wartosc in Filtry.iteritems():
                if klucz in ('dokument_id', 'komorka_id', 'kontrahent_id'):
                    where += " AND z.{klucz} LIKE '{wartosc}%'".format(**locals())
                elif klucz in ('temat', ):
                    where += " AND UPPER({klucz}) LIKE UPPER('%{wartosc}%')".format(**locals())
                elif klucz in ('tagi', ):
                    for tag in wartosc.split(','):
                        tag = tag.strip()
                        where += " AND UPPER('{tag}') = ANY(regexp_split_to_array(UPPER(tagi), E'\\\\s*,\\\\s*')) ".format(**locals())
                elif klucz in ('stan', ):
                    where += " AND {klucz} = '{wartosc}'".format(**locals())
                elif klucz in ('operator_id', ):
                    try:
                        wartosc = int(wartosc)
                        where += " AND z.operator_id={wartosc}".format(**locals())
                    except:
                        where += " AND ope.login = '{wartosc}'".format(**locals())
                elif klucz == 'niezamkniete' and wartosc:
                    where += " AND NOT zamkniete"
                elif klucz == 'niezaksiegowane' and wartosc:
                    where += " AND NOT zaksiegowane"
                elif klucz == 'wlasne' and wartosc:
                    where += " AND z.operator_id = rsf_getuserid()"
                elif klucz == 'wprowadzenie_czas_od' and wartosc not in Puste:  #AB 05.10.22r
                    if not rsglobal.SprawdzCzyToData(wartosc):
                        stat = "Podano bdn dat od"
                    where += " AND z.wprowadzenie_czas::DATE >= '{wartosc}'".format(**locals())  #AB 05.10.22r
                elif klucz == 'wprowadzenie_czas_do' and wartosc not in Puste:  #AB 05.10.22r
                    if not rsglobal.SprawdzCzyToData(wartosc):
                        stat = "Podano bdn dat do"
                    where += " AND z.wprowadzenie_czas::DATE <= '{wartosc}'".format(**locals())  #AB 05.10.22r
        if stat == 'OK':
            #WS 2021-12-06 - pobieranie wszystkiego dla anakondy - LEFT JOIN zamiast JOIN na uprawnienia_szczegoly oraz dodatkowy warunek w WHERE
            test_op = (self.operator_id == STANDARD_OPERATOR_ID)
            sql = """SELECT {Kolumny} 
                     FROM rs_wf_zagadnienia z
                     LEFT JOIN rs_kontrahenci k USING (firma_id, kontrahent_id)
                     LEFT JOIN rs_wf_rodzaje_dokumentow dok ON (z.firma_id, dokument_id) = (dok.firma_id, dok.kod)
                     LEFT JOIN rs_wf_stany_dokumentow sta ON (z.firma_id, stan) = (sta.firma_id, sta.kod)
                     LEFT JOIN rs_operatorzy ope ON (z.operator_id) = (ope.operator_id)
                     LEFT JOIN rs_wf_uprawnienia_szczegoly sz ON (z.firma_id, rsf_getuserid(), z.dokument_id, z.komorka_id) = 
                                                                 (sz.firma_id, sz.operator_id, sz.dokument_id, sz.komorka_id) 
                                                                 AND (z.stan IS NULL OR sz.stany LIKE  '%'''||TRIM(z.stan)||'''%' )
                     WHERE z.firma_id='{firma_id}' {where}
                           AND ({test_op} OR (sz.firma_id IS NOT NULL AND sz.operator_id IS NOT NULL AND sz.dokument_id IS NOT NULL AND sz.komorka_id IS NOT NULL))
                     ORDER BY modyfikacja_czas DESC
            ;""".format(**locals())  #AB 17.11.21r dodane joiny na rodzaje i stany dokumentow
            rsglobal.ZapiszDoLogow(sql)
            stat, dane, dump, dump = Execute(self, sql, 3)
            if stat != 'OK':
                stat = 'Bd pobrania listy zagadnie: ' + stat

#-------------------------------------------------------------------------------
#---   POBRANIE DOSTEPNYCH STANOW DO PRZEPISANIA(15)  #AB 17.11.21r
#-------------------------------------------------------------------------------

if stat == 'OK' and Fun in (15, ):
    if len(Zagadnienia) == 0:
        stat = "Brak zagadnie"
    elif len(Zagadnienia) >= 1:
        sql = """SELECT dokument_id, stan, komorka_id, operator_id 
                 FROM rs_wf_zagadnienia 
                 WHERE firma_id = '{firma_id}' AND zagadnienie_id = {Zagadnienia[0]};
                 ;""".format(**locals())
        stat, zag, dump, dump = Execute(self, sql, 3)
        if stat != 'OK':
            stat = 'Bd pobrania danych zagadnienia: ' + stat
        else:
            dokument_id, stan, komorka_id, operator_id = zag[0]
            if len(Zagadnienia) == 1:
                f_dokument_id = " AND dokument_id = '{}'".format(dokument_id.strip())
                f_komorka_id  = " AND komorka_id = '{}'".format(komorka_id.strip())
                if self.operator_id != STANDARD_OPERATOR_ID:  #WS 2021-12-06
                    f_operator_id  = " AND operator_id = rsf_getuserid()"
                else:
                    f_operator_id = " "
                    # f_operator_id = " AND operator_id = {}".format(operator_id)
            else:
                f_dokument_id, f_komorka_id, f_operator_id = '', '', ''
            stary_stan  = stan.strip()
            #WS 2021-12-06
            if stary_stan == 'None':
                stary_stan = ''
    dane = []
    if stat == 'OK':
        #WS 2021-12-06 - uwzgldnienie pustego stanu
        SQL = """SELECT DISTINCT kod, opis
                 FROM rs_wf_stany_dokumentow s
                 JOIN rs_wf_uprawnienia_szczegoly u ON (s.firma_id = u.firma_id  
                 AND (CASE WHEN '{stary_stan}'='' 
                           THEN u.stany ~ (''''||TRIM(s.kod)||'''') 
                           ELSE u.przejscia LIKE '%[\\'{stary_stan}\\', \\''||TRIM(s.kod)||'\\']%' 
                                OR kod ='{stary_stan}'
                           END))
                 WHERE s.firma_id = '{firma_id}'{f_dokument_id}{f_komorka_id}{f_operator_id} AND (wprowadzanie OR modyfikacja OR zamykanie OR otwieranie OR ksiegowanie)
                 ;""".format(**locals())
        rsglobal.ZapiszDoLogow(SQL)
        stat, dane, dump, dump = Execute(self, SQL, 3)
        if stat != 'OK':
            stat = "Bd sprawdzania uprawnie: "+stat
        else:
            if len(dane) == 0:
                stat = 'Brak operatorw do ktrych mona przepisa zagadnienie.'

#-------------------------------------------------------------------------------
#---   POBRANIE DOSTEPNYCH OPERATOROW (25)  #AB 17.11.21r
#-------------------------------------------------------------------------------

if stat == 'OK' and Fun in (25, ):
    if len(Zagadnienia) == 0:
        stat = "Brak zagadnie"
    elif len(Zagadnienia) >= 1:
        sql = """SELECT dokument_id, stan, komorka_id, operator_id 
                 FROM rs_wf_zagadnienia 
                 WHERE firma_id = '{firma_id}' AND zagadnienie_id = {Zagadnienia[0]};
                 ;""".format(**locals())
        stat, zag, dump, dump = Execute(self, sql, 3)
        if stat != 'OK':
            stat = 'Bd pobrania danych zagadnienia: ' + stat
        else:
            dokument_id, stan, komorka_id, operator_id = zag[0]
            if len(Zagadnienia) == 1:
                f_dokument_id = " AND dokument_id = '{}'".format(dokument_id.strip())
                f_komorka_id  = " AND komorka_id = '{}'".format(komorka_id.strip())
                stary_stan  = stan.strip()
            else:
                f_dokument_id, f_komorka_id, f_operator_id = '', '', ''
                stary_stan  = stan.strip()
    dane = []
    if stat == 'OK':
        SQL = """SELECT DISTINCT operator_id, imie, nazwisko
                 FROM rs_operatorzy o
                 JOIN rs_wf_uprawnienia_szczegoly u USING (operator_id)
                 WHERE u.firma_id = '{firma_id}' AND u.przejscia ~ ('.*\\'{nowy_stan}\\',.*') 
                 {f_dokument_id}{f_komorka_id} AND (wprowadzanie OR modyfikacja OR zamykanie OR otwieranie OR ksiegowanie)
                 ;""".format(**locals())  #AB 17.06.22r zmienione na sprawdzanie przej zamiast stanu
        stat, dane, dump, dump = Execute(self, SQL, 3)
        if stat != 'OK':
            stat = "Bd sprawdzania uprawnie: "+stat

# ---------------------------------------------------------------------------------------------------------------------
#                                             35 - Pobranie pliku
# ---------------------------------------------------------------------------------------------------------------------

if stat == 'OK' and Fun in (35, ):

    stat, poprzednie = pom.PoprzednieDaneZagadnienia(self, firma_id, zagadnienie_id)

    if stat == 'OK':
        stat = pom.SprawdzUprawnienia(self, firma_id, self.operator_id, zagadnienie_id, 'cokolwiek',
                                      poprzednie['stan'], poprzednie['komorka_id'], poprzednie['dokument_id'],
                                      STANDARD_OPERATOR_ID=STANDARD_OPERATOR_ID)
    if stat in ('OK', 'Operator posiada uprawnienia tylko do przegldania.'):

        sql = """SELECT nr_pliku, nazwa AS opis
                 FROM rs_wf_zagadnienia_pliki z 
                 JOIN rs_pliki p ON p.firma_id=z.firma_id  AND p.plik_id=z.nr_pliku 
                             AND p.tabela ='rs_wf_zagadnienia_pliki'
                 WHERE z.firma_id='{firma_id}' AND zagadnienie_id={zagadnienie_id}
                 ORDER BY nr_pliku 
                 LIMIT 1
                 ;""".format(**locals())
        rsglobal.ZapiszDoLogow(sql)
        stat, pliki, dump, dump = Execute(self, sql, 3)
        if stat != 'OK':
            stat = "Bd sprawdzania pobierania danych pliku: "+stat
        elif len(pliki) < 1:
            stat = "Brak plikw doczonych do zagadnienia"
        else:
            nr_pliku, nazwa = pliki[0]
        stat, Tresc = fs_pliki_global.PobierzPlikZSerwera(self.KatalogPlikow, nr_pliku)
        dane = [Tresc, nazwa]


#-------------------------------------------------------------------------------
#---   POBRANIE DOSTEPNYCH STANOW DLA OPERATORA(45)
#-------------------------------------------------------------------------------

if stat == 'OK' and Fun in (45, ):
    dane = []
    if stat == 'OK':
        #WS 2021-12-06 - pobieranie wszystkiego dla anakondy - LEFT JOIN zamiast JOIN na uprawnienia_szczegoly oraz dodatkowy warunek w WHERE
        test_op = (self.operator_id == STANDARD_OPERATOR_ID)
        SQL = """SELECT DISTINCT kod, opis
                 FROM rs_wf_stany_dokumentow s
                 LEFT JOIN rs_wf_uprawnienia_szczegoly u ON (s.firma_id = u.firma_id AND u.stany LIKE '%\\''||TRIM(s.kod)||'\\'%')
                 WHERE s.firma_id = '{firma_id}' AND rsf_getuserid() IN (u.operator_id, 0) 
                 AND ({test_op} OR (wprowadzanie OR modyfikacja OR zamykanie OR otwieranie OR ksiegowanie))
                 ;""".format(**locals())
        stat, dane, dump, dump = Execute(self, SQL, 3)
        if stat != 'OK':
            stat = "Bd sprawdzania uprawnie: "+stat

