# -*- coding: iso8859_2 -*-

# -----------------------------------------------------------------------------
# Name:        fs_wf_zagadnienia_pom.py
# Purpose:     Funkcje pomocnicze dla Zagadnie
# Author:      Aleksander Bargie
# Created:     2021/12/02
# RCS-ID:      $Id: fs_wf_zagadnienia_pom.py $
# Copyright:   (c) 2021 RecordSystem
# Licence:     GNU GPL
# -----------------------------------------------------------------------------

# 2021-12-06 Wojciech Skowron   (12759) [WF] Pozwoli Anakondzie wywietla/edytowa wszystkie zagadnienia
# 2022-07-03 Wojciech Skowron   (13112) [WF] Obsuga kwoty akceptacji
# 2023-08-02 Wojciech Skowron   (13487) [WF] UtworzZagadnienie i AktualizujZagadnienie
# 2023-09-22 Wojciech Skowron   (13549) [WF] Stan, temat, opis w UtworzZagadnienie
# 2023-12-20 Wojciech Skowron   (13650) dodatkowe_wartosci w UtworzZagadnienie
# 2024-01-03 Wojciech Skowron   (13650) przesunicie nextval niej w UtworzZagadnienie

import rsglobal

"""Funkcje:
AktualizujaOstatniaAktywnosc(self, firma_id, zagadnienie_id) - aktualizuje pole ostatnia aktywnosc danego zagadnienia
CzyMozePrzyjac(self, firma_id, operator_id, dokument_id, komorka_id, stan=None) - sprawdza czy mona przypisa zagadnienie do operatora
SprawdzUprawnienia(self, firma_id, operator_id, zagadnienie_id, operacja, poprzedni_stan, poprzednia_komorka,
                       poprzedni_dokument, nowy_stan=None) - doc uniewersalna funkcja testujca uprawnienia, zmienna operacja okrela do czego
PoprzednieDaneZagadnienia(self, firma_id, zagadnienie_id) - zwraca sownik{'pole_w_bazie': wartosc} z danymi zagadnienia o danym id z bazy
UtworzZagadnienie(self, firma_id, tabela, pola_klucz, wartosci_klucz, pola_z_zagadnienia, przypnij_do, stan='', temat='', opis='', pola_dodatkowe_odpowiedniki={}, pola_dodatkowe_historia={}, dodatkowe_wartosci={})
                 - tworzy zagadnienie przepisujc pola wymienione w pola_z_zagadnienia z podanej tabeli z wiersza o podanym kluczu,
                 jeli pole ma inn nazw po stronie rs_wf_zagadnienia, mona jego odpowiednik przekaza w pola_dodatkowe_odpowiedniki, np. {'kontrahent_nazwa': 'nadawca_nazwa'},
                 wartoci do wpisania w zagadnienie ktrych nie ma w podanej tabeli mona przekaza w dodatkowe_wartosci (sownik pole: wartosc)
AktualizujZagadnienie(self, firma_id, tabela, pola_klucz, wartosci_klucz, pola_z_zagadnienia, prefix='', pola_dodatkowe_odpowiedniki={}, pola_dodatkowe_historia={})
"""

Puste = ('', '    -  -  ', None, 'None', 'NULL')

historia_pola = {}  # przy zapisie do historii zagadnienia w zmiana_co zostanie wpisana warto z tego sownika, dotyczy funkcji UtworzZagadnienie i AktualizujZagadnienie
historia_pola['dokument_id'] = "Typ dokumentu"
historia_pola['komorka_id'] = "Komrka org."
historia_pola['nip'] = "NIP"
historia_pola['kontrahent_id'] = "Kontrahent"
historia_pola['kontrahent_nazwa'] = "Nazwa kontr."
historia_pola['kontrahent_adres'] = "Adres kontr."
historia_pola['temat'] = "Temat"
historia_pola['opis'] = "Opis"
historia_pola['kwota_brutto'] = "Kwota brutto"
historia_pola['data_dokumentu'] = "Data dokumentu"
historia_pola['symbol_dokumentu'] = "Symbol dokumentu"
pola_zagadnienia = ['dokument_id', 'komorka_id', 'nip',
                    'kontrahent_id', 'kontrahent_nazwa', 'kontrahent_adres',
                    'temat', 'opis',
                    'kwota_brutto', 'data_dokumentu', 'symbol_dokumentu']


def AktualizujaOstatniaAktywnosc(self, firma_id, zagadnienie_id):
    sql = """UPDATE rs_wf_zagadnienia SET modyfikacja_czas=NOW(), modyfikacja_kto=rsf_getuserid()
             WHERE firma_id='{firma_id}' AND zagadnienie_id={zagadnienie_id}""".format(**locals())
    rsglobal.ZapiszDoLogow(sql)
    stat, dump, dump, dump = self.PolSQL.Execute(sql, 1, CzyDodawacBlad=True)
    if stat != 'OK':
        stat = 'Bd aktualizacji czasu modyfikacji zagadnienia: ' + stat
    return stat


def CzyMozePrzyjac(self, firma_id, operator_id, dokument_id, komorka_id, stan=None, STANDARD_OPERATOR_ID='0'):
    #WS 2021-12-06
    if str(operator_id) == STANDARD_OPERATOR_ID:
        return 'OK'

    global Puste
    moze = False
    firma_id, dokument_id, komorka_id = firma_id.strip(), dokument_id.strip(), komorka_id.strip()
    if stan:
        stan = stan.strip()
    sql = """SELECT stany, wprowadzanie OR modyfikacja OR zamykanie OR otwieranie OR ksiegowanie
             FROM rs_wf_uprawnienia_szczegoly
             WHERE firma_id='{firma_id}' AND operator_id={operator_id} AND dokument_id = '{dokument_id}'
               AND komorka_id = '{komorka_id}'
             ;""".format(**locals())
    rsglobal.ZapiszDoLogow(sql)
    stat, test, dump, dump = self.PolSQL.Execute(sql, 3, CzyDodawacBlad=True)
    if stat != 'OK':
        stat = 'Bd sprawdzania uprawnie operatora do przypisania mu zagadnienia: ' + stat
    if test is not None and len(test) == 1 and test[0][1]:  # sa jakies uprawnienia
        import ast
        stany = ast.literal_eval(test[0][0]) if test[0][0] not in Puste else []
        if stan:
            if stan in stany:
                moze = True
        else:
            # if stany not in Puste and len(stany) > 0:
            moze = True
    if not moze:
        stat = "Operator {} nie moe obsugiwa dokumentw typu {} w komrce {}{}."
        stat = stat.format(operator_id, dokument_id.strip(), komorka_id.strip(), (" dla stanu " + stan) if stan else '')
    return stat


#WS 2022-07-03 - kwota_brutto
def SprawdzUprawnienia(self, firma_id, operator_id, zagadnienie_id, operacja, poprzedni_stan, poprzednia_komorka,
                       poprzedni_dokument, nowy_stan=None, kwota_brutto=None, STANDARD_OPERATOR_ID='0'):
    """operacja to wprowadzanie, modyfikacja, zamykanie, otwieranie, ksiegowanie"""
    global Puste
    if isinstance(firma_id, basestring):
        firma_id = firma_id.strip()
    if isinstance(operator_id, basestring):
        operator_id = operator_id.strip()
    if isinstance(zagadnienie_id, basestring):
        zagadnienie_id = zagadnienie_id.strip()
    if isinstance(operacja, basestring):
        operacja = operacja.strip()
    if isinstance(poprzedni_stan, basestring):
        poprzedni_stan = poprzedni_stan.strip()
    if isinstance(poprzednia_komorka, basestring):
        poprzednia_komorka = poprzednia_komorka.strip()
    if isinstance(poprzedni_dokument, basestring):
        poprzedni_dokument = poprzedni_dokument.strip()
    if isinstance(nowy_stan, basestring):
        nowy_stan = nowy_stan.strip()
        if nowy_stan in Puste:
            return "Nie mona usun stanu."
    if self.operator_id == STANDARD_OPERATOR_ID:
        return 'OK'

    moze = False
    if operacja == 'cokolwiek':
        operacja = 'wprowadzanie OR modyfikacja OR zamykanie OR otwieranie OR ksiegowanie'
    sql = """SELECT stany, {operacja}, przejscia
             FROM rs_wf_uprawnienia_szczegoly
             WHERE firma_id='{firma_id}' AND operator_id={operator_id} AND dokument_id = '{poprzedni_dokument}'
               AND komorka_id = '{poprzednia_komorka}'
             ;""".format(**locals())
    rsglobal.ZapiszDoLogow(sql)
    stat, test, dump, dump = self.PolSQL.Execute(sql, 3, CzyDodawacBlad=True)
    if stat != 'OK':
        stat = 'Bd sprawdzania uprawnie operatora do przypisania mu zagadnienia: ' + stat
    if (stat =='OK' and len(test) == 1 and test[0][1]):  # sa jakies uprawnienia
        import ast
        stany = ast.literal_eval(test[0][0]) if test[0][0] not in Puste else []
        rsglobal.ZapiszDoLogow("Pobrano stany {} ".format(stany))
        rsglobal.ZapiszDoLogow("repr {} ".format(repr(stany)))
        rsglobal.ZapiszDoLogow("type {} ".format(type(stany)))
        if poprzedni_stan not in Puste:
            rsglobal.ZapiszDoLogow("poprzedni_stan {} ".format(poprzedni_stan))
            rsglobal.ZapiszDoLogow("poprzedni_stan {} ".format(repr(poprzedni_stan)))
            rsglobal.ZapiszDoLogow("poprzedni_stan in stany {} ".format(poprzedni_stan in stany))
            if poprzedni_stan in stany:
                przejscia = ast.literal_eval(test[0][2])
                for p in przejscia:
                    if poprzedni_stan == p[0]:
                        moze = True
                if not moze:
                    return 'Operator posiada uprawnienia tylko do przegldania.'
        else:
            moze = True

    rsglobal.ZapiszDoLogow("moze {} ".format(moze))
    rsglobal.ZapiszDoLogow("nowy_stan {} ".format(nowy_stan))
    if stat == 'OK' and not moze:
        if operacja == 'wprowadzanie':
            operacja = ' wprowadzania'
        elif operacja == 'modyfikacja':
            operacja = ' modyfikacji'
        elif operacja == 'zamykanie':
            operacja = ' zamykania'
        elif operacja == 'otwieranie':
            operacja = ' otwierania'
        elif operacja == 'ksiegowanie':
            operacja = ' ksigowania'
        else:
            operacja = ''
        stat = "Brak uprawnie do{} dokumentw typu {} w komrce {}{}."
        stat = stat.format(operacja, poprzedni_dokument.strip(), poprzednia_komorka.strip(), (
                " dla stanu " + poprzedni_stan) if poprzedni_stan not in Puste else '')
    elif stat == 'OK' and moze and nowy_stan:
        poprzednie = {}
        rsglobal.ZapiszDoLogow("poprzedni_stan.strip() != nowy_stan.strip() {} ".format(poprzedni_stan.strip() != nowy_stan.strip()))
        if poprzedni_stan not in Puste and poprzedni_stan.strip() != nowy_stan.strip():
            zmiana = [poprzedni_stan, nowy_stan]
            import ast
            przejscia = ast.literal_eval(test[0][2])
            rsglobal.ZapiszDoLogow("przejscia {} ".format(przejscia))
            rsglobal.ZapiszDoLogow("zmiana {} ".format(zmiana))
            rsglobal.ZapiszDoLogow("zmiana not in przejscia {} ".format(zmiana not in przejscia))
            if zmiana not in przejscia:
                stat = "Brak uprawnie do przejcia ze stanu {} do stanu {} dla dokumentu {} i komrki {}"
                stat = stat.format(poprzedni_stan, nowy_stan, poprzedni_dokument.strip(), poprzednia_komorka.strip())

            if stat == 'OK':
                #WS 2022-07-03
                sql = """SELECT akceptacja_kwoty
                         FROM rs_wf_stany_dokumentow
                         WHERE firma_id='{firma_id}' AND kod='{nowy_stan}'
                         ;""".format(**locals())
                rsglobal.ZapiszDoLogow(sql)
                stat, test, dump, dump = self.PolSQL.Execute(sql, 3, CzyDodawacBlad=True)
                if stat != 'OK':
                    stat = 'Bd pobrania danych stanu: '+stat
                elif len(test) == 0:
                    stat = 'Wprowadzony stan nie istnieje'
                elif test[0][0]:
                    sql = """SELECT CASE WHEN op.akceptacja_kwota < 0 OR zastepuje.akceptacja_kwota < 0 THEN -1 ELSE GREATEST(op.akceptacja_kwota, zastepuje.akceptacja_kwota) END
                             FROM rs_operatorzy op
                             LEFT OUTER JOIN rs_operatorzy zastepuje ON (op.operator_id=zastepuje.zastepca_id)
                             WHERE op.operator_id={operator_id};""".format(**locals())
                    rsglobal.ZapiszDoLogow(sql)
                    stat, test, dump, dump = self.PolSQL.Execute(sql, 3, CzyDodawacBlad=True)
                    if stat != 'OK':
                        stat = 'Bd pobrania danych operatora: '+stat
                    else:
                        rsglobal.ZapiszDoLogow("kwota_brutto {}".format(kwota_brutto))
                        rsglobal.ZapiszDoLogow("akceptacja_kwota {}".format(test[0][0]))
                        if test[0][0] == 0:
                            if kwota_brutto > 0:
                                stat = 'Brak uprawnie do akceptacji do zapaty'
                        elif test[0][0] > 0:
                            if kwota_brutto > test[0][0]:
                                stat = 'Brak uprawnie do akceptacji do zapaty. Kwota wiksza od {}'.format(int(test[0][0]))
    return stat


def PoprzednieDaneZagadnienia(self, firma_id, zagadnienie_id):
    SQLPOPRZEDNIE = "SELECT * FROM rs_wf_zagadnienia  WHERE firma_id='{firma_id}' AND zagadnienie_id={zagadnienie_id};".format(**locals())
    rsglobal.ZapiszDoLogow(SQLPOPRZEDNIE)
    stat, poprzednie_dane, naglowki_poprzednie, dump = self.PolSQL.Execute(SQLPOPRZEDNIE, 3, CzyDodawacBlad=True)
    poprzednie = {}
    if naglowki_poprzednie:
        for i in xrange(len(naglowki_poprzednie)):
            poprzednie[naglowki_poprzednie[i][0]] = poprzednie_dane[0][i]
    else:
        stat = "Bd pobrania aktualnych danych zagadnienia"

    if stat != 'OK':
        stat = "Bd pobrania aktualnych danych zagadnienia: {stat}".format(**locals())
    return stat, poprzednie


#WS 2023-08-02
def UtworzZagadnienie(self, firma_id, tabela, pola_klucz, wartosci_klucz, pola_z_zagadnienia, przypnij_do, stan='', temat='', opis='',
                      pola_dodatkowe_odpowiedniki={}, pola_dodatkowe_historia={}, dodatkowe_wartosci={}):
    #WS 2023-12-20 - dodatkowe_wartosci
    import rssql
    global Puste
    global pola_zagadnienia
    global historia_pola
    stat = 'BLAD'
    zagadnienie_id = None

    pola_do_przepisania = []
    wartosci_do_przepisania = []
    pola_zagadnienia_odpowiedniki = {}
    for pole in pola_zagadnienia:
        pola_zagadnienia_odpowiedniki[pole] = pole
    pola_zagadnienia_odpowiedniki.update(pola_dodatkowe_odpowiedniki)

    where_str = '=%s AND '.join(pola_klucz)+'=%s'
    sql_dane_zag = """SELECT *
                       FROM {tabela}
                       WHERE {where_str};""".format(**locals())
    args = wartosci_klucz
    rsglobal.ZapiszDoLogow('Tworzenie zagadnienia pobranie danych: '+self.PolSQL.Mogrify(sql_dane_zag, args))
    stat, dane_zag, sql_dane_zag_wyn1, dump = self.PolSQL.Execute(sql_dane_zag, 3, args)
    if stat != 'OK':
        stat = "Podczas pobierania danych z {tabela} wystpi bd: {stat}".format(**locals())
    elif len(dane_zag) < 1:
        stat = u"Nie odnaleziono danych z {tabela}"
    else:
        tr = rssql.RSTableRowset2(dane_zag, sql_dane_zag_wyn1)[0]
        for pole in [dane_pola[0] for dane_pola in sql_dane_zag_wyn1]:
            # pole_zag - nazwa pola po stronie rs_wf_zagadnienia, pole_odp - odpowiednik w tabela
            for pole_zag, pole_odp in pola_zagadnienia_odpowiedniki.iteritems():
                if pole == pole_odp and (pole_odp in pola_z_zagadnienia or pole_zag in pola_dodatkowe_odpowiedniki) and tr[pole] not in Puste:
                    pola_do_przepisania.append(pole_zag)
                    wartosci_do_przepisania.append(tr[pole])

        #WS 2023-12-20
        for pole, wart in dodatkowe_wartosci.iteritems():
            pola_do_przepisania.append(pole)
            wartosci_do_przepisania.append(wart)

    if stat == 'OK':
        dokument_id = wartosci_do_przepisania[pola_do_przepisania.index('dokument_id')].strip()
        komorka_id = wartosci_do_przepisania[pola_do_przepisania.index('komorka_id')].strip()

        if dokument_id in Puste or komorka_id in Puste:
            stat = 'Do utworzenia zagadnienia konieczne jest podanie komrki zatrudniajcej oraz typu dokumentu'
        else:
            stat = CzyMozePrzyjac(self, self.firma_id, przypnij_do, dokument_id, komorka_id, stan)

    #WS 2024-01-03 - przesunicie nextval niej
    if stat == 'OK':
        SEL_ZAG = "SELECT nextval('rs_wf_zagadnienia_zagadnienie_id_seq');"
        rsglobal.ZapiszDoLogow(SEL_ZAG)
        stat, dane_zagadnienie_id, dump, dump = self.PolSQL.Execute(SEL_ZAG, 3)
        if stat != 'OK':
            stat = 'Bd pobrania numeru nowego zagadnienia: '+stat
        else:
            zagadnienie_id = dane_zagadnienie_id[0][0]

    if stat == 'OK':
        pola_str = ','.join(pola_do_przepisania)
        values_str = '%s, '*len(pola_do_przepisania)
        INS_ZAG = """INSERT INTO rs_wf_zagadnienia 
                                 (firma_id, zagadnienie_id, {pola_str},
                                  stan, temat, opis,
                                  wprowadzenie_czas, wprowadzenie_kto, modyfikacja_czas, modyfikacja_kto,
                                  zamkniete, zaksiegowane, operator_id) 
                                 VALUES 
                                 (%s, %s, {values_str}
                                  %s, %s, %s,
                                  NOW(), rsf_getuserid(), NOW(), rsf_getuserid(), 
                                  False, False, %s);""".format(**locals())
        args = [self.firma_id, zagadnienie_id]
        args.extend(wartosci_do_przepisania)
        args.extend([stan, temat, opis])
        args.append(przypnij_do)

        INS_HISTORIA = ""
        wzor_hist = """INSERT INTO rs_wf_zagadnienia_historia (firma_id, zagadnienie_id, zmiana_czas, zmiana_kto, zmiana_co, zmiana_wartosc) 
                       VALUES('{}', {}, NOW() + INTERVAL '{} microseconds', rsf_getuserid(), '{}', '{}');
                    """.format(firma_id, '{}', '{}', '{}', '{}')

        historia_pola.update(pola_dodatkowe_historia)
        zmiana_czas = 0
        for pole, wart in zip(pola_do_przepisania, wartosci_do_przepisania):
            if pole in historia_pola and wart not in Puste:
                INS_HISTORIA += wzor_hist.format(zagadnienie_id, zmiana_czas, "{}".format(historia_pola[pole]),
                                                                              "=> {}".format(str(wart).strip("' ")[:47]))
                zmiana_czas += 1
        INS_HISTORIA += wzor_hist.format(zagadnienie_id, zmiana_czas, "Przypisano do ",
                                                                      "=> "+str(przypnij_do).strip("' "))
        zmiana_czas += 1
        if stan != '':
            INS_HISTORIA += wzor_hist.format(zagadnienie_id, zmiana_czas, "Stan ",
                                                                           "=> " + str(stan).strip("' "))
            zmiana_czas += 1
        if temat != '':
            INS_HISTORIA += wzor_hist.format(zagadnienie_id, zmiana_czas, "Temat ",
                                                                           "=> " + str(temat).strip("' ")[:47])
            zmiana_czas += 1
        if opis != '':
            INS_HISTORIA += wzor_hist.format(zagadnienie_id, zmiana_czas, "Opis ",
                                                                           "=> " + str(opis).strip("' ")[:47])

        stat, dump, dump, dump = self.PolSQL.Execute(INS_ZAG+INS_HISTORIA, 1, args)
        rsglobal.ZapiszDoLogow('Tworzenie zagadnienia: '+self.PolSQL.Mogrify(INS_ZAG, args))
        rsglobal.ZapiszDoLogow('Akt historii zagadnienia: '+INS_HISTORIA)
    if stat != 'OK':
        stat = 'Bd tworzenia zagadnienia: '+stat
    return stat, zagadnienie_id


#WS 2023-08-02
def AktualizujZagadnienie(self, firma_id, tabela, pola_klucz, wartosci_klucz, pola_z_zagadnienia, prefix='', pola_dodatkowe_odpowiedniki={}, pola_dodatkowe_historia={}):
    where_str = '=%s AND '.join(pola_klucz)+'=%s'
    sql_zag_pole   = """(SELECT {}
                         FROM {}
                         WHERE {})"""
    SQL = """SELECT zamkniete, zaksiegowane, zagadnienie_id
             FROM rs_wf_zagadnienia
             WHERE zagadnienie_id = {};
          ;""".format(sql_zag_pole.format('zagadnienie_id', tabela, where_str))
    args_klucz_zag = wartosci_klucz
    args           = args_klucz_zag
    rsglobal.ZapiszDoLogow('Akt zagadnienia pobranie danych: '+self.PolSQL.Mogrify(SQL, args))
    stat, test_zag, dump, dump = self.PolSQL.Execute(SQL, 3, args, CzyDodawacBlad=True)
    if stat != 'OK':
        stat = "Bd pobierania danych powizanego zagadnienia: " + stat
    elif len(test_zag) > 0 and test_zag[0][2] != 0:
        zagadnienie_id = test_zag[0][2]
        zamkniete      = test_zag[0][0]
        zaksiegowane   = test_zag[0][1]
        if zaksiegowane or zamkniete:
            stat = "Nie mona modyfikowa danych przepisanych na zagadnienie ktre zostao ju {}".format('zaksigowane' if zaksiegowane else 'zamknite')
        else:

            if stat == 'OK':
                stat, poprzednie = PoprzednieDaneZagadnienia(self, firma_id, zagadnienie_id)
            if stat == 'OK':
                global pola_zagadnienia
                args = []
                pola_zagadnienia_odpowiedniki = {}
                for pole in pola_zagadnienia:
                    pola_zagadnienia_odpowiedniki[pole] = pole
                pola_zagadnienia_odpowiedniki.update(pola_dodatkowe_odpowiedniki)
                historia_pola.update(pola_dodatkowe_historia)

                SQL = """UPDATE rs_wf_zagadnienia
                         SET"""
                # pole_zag - nazwa pola po stronie rs_wf_zagadnienia, pole_odp - odpowiednik w tabela
                for pole_zag, pole_odp in pola_zagadnienia_odpowiedniki.iteritems():
                    if pole_odp in pola_z_zagadnienia or pole_zag in pola_dodatkowe_odpowiedniki:
                        SQL += " {}={},".format(pole_zag, sql_zag_pole.format(pole_odp, tabela, where_str))
                        args += args_klucz_zag
                SQL = SQL.strip(',')
                SQL += """ WHERE zagadnienie_id = %s ;"""
                args.append(zagadnienie_id)
                rsglobal.ZapiszDoLogow('Akt zagadnienia: '+self.PolSQL.Mogrify(SQL, args))
                stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1, args, CzyDodawacBlad=True)
                if stat != 'OK':
                    stat = "Bd aktualizacji danych powizanego zagadnienia: " + stat
            if stat == 'OK':
                stat, nowe = PoprzednieDaneZagadnienia(self, firma_id, zagadnienie_id)

            if stat == 'OK':
                global historia_pola
                INSHISTORIA = ""
                zmiany = 0
                argshistoria = []
                wzor_hist = """INSERT INTO rs_wf_zagadnienia_historia (firma_id, zagadnienie_id, zmiana_czas, zmiana_kto, zmiana_co, zmiana_wartosc) 
                            VALUES('{}', {}, NOW() + INTERVAL '{} microseconds', rsf_getuserid(), %s, %s);
                            """.format(firma_id, zagadnienie_id, '{}')
                rsglobal.ZapiszDoLogow(nowe)
                rsglobal.ZapiszDoLogow(poprzednie)
                for pole in pola_z_zagadnienia:
                    n, p = nowe[pole], poprzednie[pole]
                    if n != p:
                        if pole == 'dokument_id':
                            SELRODZ = """SELECT (SELECT TRIM(kod)||' - '||TRIM(opis) FROM rs_wf_rodzaje_dokumentow WHERE (firma_id, kod) = ('{}','{}')) AS poprzedni, 
                                                (SELECT TRIM(kod)||' - '||TRIM(opis) FROM rs_wf_rodzaje_dokumentow WHERE (firma_id, kod) = ('{}','{}')) AS nowy
                                          ;""".format(firma_id, p, firma_id, n)
                            rsglobal.ZapiszDoLogow(SELRODZ)
                            stat, rodzaje, dump, dump = self.PolSQL.Execute(SELRODZ, 3, CzyDodawacBlad=True)
                            rsglobal.ZapiszDoLogow("pobrano rodzaje: {}".format(rodzaje))
                            if stat != 'OK':
                                stat = "Bd pobierania danych rodzajw dokumentw: {stat}".format(**locals())
                            else:
                                poprzedni, nowy = rodzaje[0]
                                if poprzedni in Puste:
                                    poprzedni = ''
                                if nowy in Puste:
                                    nowy = ''
                                if len(poprzedni) + len(nowy) > 46:
                                    zapas_poprzedni = max(23 - len(nowy), 0)
                                    zapas_nowy      = max(23 - len(poprzedni), 0)
                                    poprzedni       = poprzedni[:23+zapas_poprzedni]
                                    nowy            = nowy[:23+zapas_nowy]
                                INSHISTORIA += wzor_hist.format(zmiany)
                                argshistoria.append("{}Typ dokumentu".format(prefix))
                                argshistoria.append("{} => {}".format(poprzedni, nowy).strip())
                        elif pole == 'komorka_id':
                            SELKOM = """SELECT COALESCE((SELECT TRIM(kod)||' - '||TRIM(opis) FROM rs_komorki_zatrudniajace WHERE (firma_id, kod) = ('{}','{}')), '') AS poprzedni, 
                                               COALESCE((SELECT TRIM(kod)||' - '||TRIM(opis) FROM rs_komorki_zatrudniajace WHERE (firma_id, kod) = ('{}','{}')), '') AS nowy
                                        ;""".format(firma_id, p, firma_id,  n)
                            rsglobal.ZapiszDoLogow(SELKOM)
                            stat, komorki, dump, dump = self.PolSQL.Execute(SELKOM, 3, CzyDodawacBlad=True)
                            rsglobal.ZapiszDoLogow("pobrano komrki: {}".format(komorki))
                            if stat != 'OK':
                                stat = "Bd pobierania danych komrek zatrudniajcych: {stat}".format(**locals())
                            else:
                                poprzedni, nowy = komorki[0]
                                if poprzedni in Puste:
                                    poprzedni = ''
                                if nowy in Puste:
                                    nowy = ''
                                if len(poprzedni) + len(nowy) > 46:
                                    zapas_poprzedni = max(23 - len(nowy), 0)
                                    zapas_nowy      = max(23 - len(poprzedni), 0)
                                    poprzedni       = poprzedni[:23+zapas_poprzedni]
                                    nowy            = nowy[:23+zapas_nowy]
                                INSHISTORIA += wzor_hist.format(zmiany)
                                argshistoria.append("{}Komrka org.".format(prefix))
                                argshistoria.append("{} => {}".format(poprzedni, nowy).strip())
                        elif pole == 'kwota_brutto':
                            poprzedni, nowy = p, n
                            if poprzedni in Puste:
                                poprzedni = ''
                            if nowy in Puste:
                                nowy = ''
                            INSHISTORIA += wzor_hist.format(zmiany)
                            argshistoria.append("{}Kwota brutto".format(prefix))
                            argshistoria.append("{} => {}".format(poprzedni, nowy).strip())
                        elif pole == 'kontrahent_id':
                            SELKON = """SELECT COALESCE((SELECT TRIM(kontrahent_id)||' - '||TRIM(nazwa) FROM rs_kontrahenci WHERE (firma_id, kontrahent_id) = ('{}','{}')), '') AS poprzedni, 
                                               COALESCE((SELECT TRIM(kontrahent_id)||' - '||TRIM(nazwa) FROM rs_kontrahenci WHERE (firma_id, kontrahent_id) = ('{}','{}')), '') AS nowy
                                        ;""".format(firma_id, p, firma_id,  n)
                            rsglobal.ZapiszDoLogow(SELKON)
                            stat, kontrahenci, dump, dump = self.PolSQL.Execute(SELKON, 3, CzyDodawacBlad=True)
                            rsglobal.ZapiszDoLogow("pobrano kontrahentw: {}".format(kontrahenci))
                            if stat != 'OK':
                                stat = "Bd pobierania danych kontrahentw: {stat}".format(**locals())
                            else:
                                poprzedni, nowy = kontrahenci[0]
                                if poprzedni in Puste:
                                    poprzedni = ''
                                if nowy in Puste:
                                    nowy = ''
                                if len(poprzedni) + len(nowy) > 46:
                                    zapas_poprzedni = max(23 - len(nowy), 0)
                                    zapas_nowy      = max(23 - len(poprzedni), 0)
                                    poprzedni       = poprzedni[:23+zapas_poprzedni]
                                    nowy            = nowy[:23+zapas_nowy]
                                INSHISTORIA += wzor_hist.format(zmiany)
                                argshistoria.append("{}Kontrahent".format(prefix))
                                argshistoria.append("{} => {}".format(poprzedni, nowy).strip())
                        else:
                            poprzedni, nowy = str(p).strip(), str(n).strip()
                            if poprzedni in Puste:
                                poprzedni = ''
                            if nowy in Puste:
                                nowy = ''
                            if len(poprzedni) + len(nowy) > 46:
                                zapas_poprzedni = max(23 - len(nowy), 0)
                                zapas_nowy      = max(23 - len(poprzedni), 0)
                                poprzedni       = poprzedni[:23+zapas_poprzedni]
                                nowy            = nowy[:23+zapas_nowy]
                            INSHISTORIA += wzor_hist.format(zmiany)
                            argshistoria.append("{}{}".format(prefix, historia_pola[pole]))
                            argshistoria.append("{} => {}".format(poprzedni, nowy).strip())
                        zmiany += 1
                    if stat != 'OK':
                        INSHISTORIA = None
                        break
                if INSHISTORIA:
                    argshistoria = [str(x) if isinstance(x, unicode) else x for x in argshistoria]
                    rsglobal.ZapiszDoLogow('Akt historii zagadnienia: '+self.PolSQL.Mogrify(INSHISTORIA, argshistoria))
                    stat, dump, dump, dump = self.PolSQL.Execute(INSHISTORIA, 1, argshistoria, CzyDodawacBlad=True)
                    if stat != 'OK':
                        stat = "Bd aktualizacji historii zagadnienia: {stat}".format(**locals())

            if stat == 'OK':
                stat = AktualizujaOstatniaAktywnosc(self, firma_id, zagadnienie_id)

    return stat
