# -*- coding: iso8859_2 -*-
#-----------------------------------------------------------------------------
# Name:        fs_wydruki_certyfikat_artykulu.py
# Purpose:     Generowanie PDF z certyfikatem artykuu dla pozycji d.m.
# Author:      Wojciech Skowron
# Created:     2021/06/30
# RCS-ID:      $Id: fs_wydruki_certyfikat_artykulu.py $
# Copyright:   (c) 2021 Record System
# Licence:     GNU GPL
#-----------------------------------------------------------------------------
# 2022-07-13 Aleksander Bargie (12989) Komunikat gdy tworzenie raportu pdf nie powiedzie si

import binascii
import rsglobal
from funkcje import fs_pliki_global

stat = 'OK'
dane = []
wyn1 = []
wyn2 = []

if stat == 'OK' and not 'Funkcja' in klucze:
    stat = 'Brak podanej funkcji w kluczach'

if stat == "OK":
    Funkcja = klucze['Funkcja']
    SlPar = {'Magazyn': [1], 'Dokument': [1], 'Data': [1], 'Numer': [1], 'NrPoz': [1], 'NrPPoz': [1], 'Logo': [1]}

    Lista = rsglobal.SprawdzParametryWKluczach(SlPar, Funkcja, klucze)
    if Lista != '':
        stat = 'Uwaga : wywoanie funkcji fs_wydruki_certyfikat_artykulu z numerem ' +str(Funkcja)+' bez parametrw: '+Lista

if stat == 'OK':
    if Funkcja == 1:
        Magazyn = rsglobal.Dekoduj(klucze['Magazyn'])
        Dokument = rsglobal.Dekoduj(klucze['Dokument'])
        Data = rsglobal.Dekoduj(klucze['Data'])
        Numer = rsglobal.Dekoduj(klucze['Numer'])
        NrPoz = rsglobal.Dekoduj(klucze['NrPoz'])
        NrPPoz = rsglobal.Dekoduj(klucze['NrPPoz'])
        Logo = binascii.unhexlify(klucze['Logo'])

class CertyfikatArtykuluPDF:
    def __init__(self, self_serwera, magazyn_id, dokument_id, data, numer, nr_poz, nr_ppoz, nazwa_pliku='', logo=''):
        import rs_print_util
        import rsglobal
        rsglobal.ZapiszDoLogow(', '.join("%s: %s" % item for item in vars(self_serwera).items()))
        rs_print_util.PrzygotujCzcionki()
        self.magazyn_id = magazyn_id
        self.dokument_id = dokument_id
        self.data = data
        self.numer = numer
        self.nr_poz = nr_poz
        self.nr_ppoz = nr_ppoz
        self.nazwa_pliku = nazwa_pliku
        self.logo = logo
        self.firma_id = self_serwera.firma_id
        self.PolSQL = self_serwera.PolSQL
        self.KatalogPlikow = self_serwera.KatalogPlikow

    def PobierzDane(self):
        import rsglobal

        stat = "OK"

        # Pobranie pliku certyfikatu
        SQL = """SELECT a.plik_certyfikatu, a.artykul_id, a.nazwa FROM rs_dokumenty_magazynowe_pozycje dmp
                 LEFT OUTER JOIN rs_artykuly a USING (firma_id, artykul_id)
                 WHERE firma_id='{self.firma_id}' AND magazyn_id='{self.magazyn_id}' AND dokument_id='{self.dokument_id}' 
                       AND data='{self.data}' AND numer={self.numer} AND nr_poz={self.nr_poz} AND nr_ppoz={self.nr_ppoz};
              """.format(**locals())
        rsglobal.ZapiszDoLogow(SQL)
        stat, daneP, naglP, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
        if stat != 'OK':
            stat = "Bd pobrania danych artykuu: " + stat

        if stat == 'OK':
            # Dane oddziau
            SQL = """SELECT o.*
                     FROM rs_dokumenty_magazynowe_pozycje dmp
                     LEFT OUTER JOIN rs_magazyny m USING (firma_id, magazyn_id)
                     LEFT OUTER JOIN rs_oddzialy o USING (firma_id, oddzial_id)
                     WHERE firma_id='{self.firma_id}' AND magazyn_id='{self.magazyn_id}' AND dokument_id='{self.dokument_id}' 
                           AND data='{self.data}' AND numer={self.numer} AND nr_poz={self.nr_poz} AND nr_ppoz={self.nr_ppoz};
                  """.format(**locals())
            rsglobal.ZapiszDoLogow(SQL)
            stat, daneO, naglO, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
            if stat != 'OK':
                stat = "Bd pobrania danych oddziau: " + stat

        if stat == 'OK' and daneP[0][0] not in ('Porto1',):
            stat = 'Brak definicji pliku certyfikatu dla artykuu '+daneP[0][1].strip()+' - '+daneP[0][2].strip()+' lub definicja nieprawidowa.'
            return stat, None

        if stat == 'OK' and daneP[0][0] == 'Porto1':
            # Dane z kart przyjcia
            SQL = """SELECT COALESCE(p1.wart_s, 'BRAK'), COALESCE(p2.wart_s, 'BRAK'), COALESCE(p3.wart_s, 'BRAK'), 
                            COALESCE(p4.wart_s, 'BRAK'), COALESCE(p5.wart_s, 'BRAK'), COALESCE(p6.wart_s, 'BRAK')
                     FROM rs_dokumenty_magazynowe_pozycje_karty_przyjecia p
                     LEFT OUTER JOIN rs_dokumenty_magazynowe_pozycje_karty_przyjecia p1 ON (p.firma_id=p1.firma_id AND p.magazyn_id=p1.magazyn_id AND p.dokument_id=p1.dokument_id AND p.data=p1.data AND p.numer=p1.numer AND p.nr_poz=p1.nr_poz AND p.nr_ppoz=p1.nr_ppoz AND p1.kod='01')
                     LEFT OUTER JOIN rs_dokumenty_magazynowe_pozycje_karty_przyjecia p2 ON (p.firma_id=p2.firma_id AND p.magazyn_id=p2.magazyn_id AND p.dokument_id=p2.dokument_id AND p.data=p2.data AND p.numer=p2.numer AND p.nr_poz=p2.nr_poz AND p.nr_ppoz=p2.nr_ppoz AND p2.kod='02')
                     LEFT OUTER JOIN rs_dokumenty_magazynowe_pozycje_karty_przyjecia p3 ON (p.firma_id=p3.firma_id AND p.magazyn_id=p3.magazyn_id AND p.dokument_id=p3.dokument_id AND p.data=p3.data AND p.numer=p3.numer AND p.nr_poz=p3.nr_poz AND p.nr_ppoz=p3.nr_ppoz AND p3.kod='03')
                     LEFT OUTER JOIN rs_dokumenty_magazynowe_pozycje_karty_przyjecia p4 ON (p.firma_id=p4.firma_id AND p.magazyn_id=p4.magazyn_id AND p.dokument_id=p4.dokument_id AND p.data=p4.data AND p.numer=p4.numer AND p.nr_poz=p4.nr_poz AND p.nr_ppoz=p4.nr_ppoz AND p4.kod='04')
                     LEFT OUTER JOIN rs_dokumenty_magazynowe_pozycje_karty_przyjecia p5 ON (p.firma_id=p5.firma_id AND p.magazyn_id=p5.magazyn_id AND p.dokument_id=p5.dokument_id AND p.data=p5.data AND p.numer=p5.numer AND p.nr_poz=p5.nr_poz AND p.nr_ppoz=p5.nr_ppoz AND p5.kod='05')
                     LEFT OUTER JOIN rs_dokumenty_magazynowe_pozycje_karty_przyjecia p6 ON (p.firma_id=p6.firma_id AND p.magazyn_id=p6.magazyn_id AND p.dokument_id=p6.dokument_id AND p.data=p6.data AND p.numer=p6.numer AND p.nr_poz=p6.nr_poz AND p.nr_ppoz=p6.nr_ppoz AND p6.kod='06')
                     WHERE p.firma_id='{self.firma_id}' AND p.magazyn_id='{self.magazyn_id}' AND p.dokument_id='{self.dokument_id}' 
                           AND p.data='{self.data}' AND p.numer={self.numer} AND p.nr_poz={self.nr_poz} AND p.nr_ppoz={self.nr_ppoz};
                  """.format(**locals())
            rsglobal.ZapiszDoLogow(SQL)
            stat, daneC, naglC, dump = self.PolSQL.Execute(SQL, 3, CzyDodawacBlad=True)
            if stat != 'OK':
                stat = "Bd pobrania danych certyfikatu: " + stat

        if stat == 'OK':
            return 'OK', [daneP, naglP, daneO, naglO, daneC, naglC]
        else:
            return stat, None

    def UtworzNagl(self, tresc):
        import rs_print_util

        T = rs_print_util.Table(tresc, [17*rs_print_util.cm], [1*rs_print_util.cm])
        T.setStyle(rs_print_util.TableStyle([('TEXTCOLOR', (0, 0), (0, 0), rs_print_util.colors.black),
                                             ('FONT', (0, 0), (0, 0), 'DejaVu-Bold', 15),
                                             ('ALIGN', (0, 0), (0, 0), 'CENTER'),
                                             ('VALIGN', (0, 0), (-1, -1), 'MIDDLE')]))

        return T

    def UtworzWiersz(self, tresc, wys):
        import rs_print_util

        T = rs_print_util.Table(tresc, [17*rs_print_util.cm], [wys*rs_print_util.cm])
        T.setStyle(rs_print_util.TableStyle([('TEXTCOLOR', (0, 0), (0, 0), rs_print_util.colors.black),
                                             ('FONT', (0, 0), (0, 0), 'DejaVu', 9),
                                             ('BOX', (0, 0), (0, 0), 0.25, rs_print_util.colors.black),
                                             ('ALIGN', (0, 0), (0, 0), 'LEFT'),
                                             ('VALIGN', (0, 0), (-1, -1), 'MIDDLE')]))

        return T

    def UtworzTabeleParametry(self, tresc):
        import rs_print_util

        T = rs_print_util.Table(tresc, [4*rs_print_util.cm, 2*rs_print_util.cm, 4*rs_print_util.cm, 3.5*rs_print_util.cm, 3.5*rs_print_util.cm], [1*rs_print_util.cm]*8)
        T.setStyle(rs_print_util.TableStyle([('TEXTCOLOR', (0, 0), (-1, -1), rs_print_util.colors.black),
                                             ('FONT', (0, 0), (-1, -1), 'DejaVu', 9),
                                             ('BOX', (0, 0), (-1, -1), 0.25, rs_print_util.colors.black),
                                             ('INNERGRID', (0, 0), (-1, -1), 0.25, rs_print_util.colors.black),
                                             ('ALIGN', (0, 0), (0, 0), 'CENTER'),
                                             ('ALIGN', (0, 1), (0, -1), 'LEFT'),
                                             ('ALIGN', (1, 0), (-1, -1), 'CENTER'),
                                             ('SPAN', (0, 0), (0, 1)),
                                             ('SPAN', (1, 0), (1, 1)),
                                             ('SPAN', (2, 0), (2, 1)),
                                             ('SPAN', (3, 0), (4, 0)),
                                             ('VALIGN', (0, 0), (-1, -1), 'MIDDLE')]))

        return T

    def UtworzTabelePodpisy(self, tresc):
        import rs_print_util

        T = rs_print_util.Table(tresc, [8.5*rs_print_util.cm, 8.5*rs_print_util.cm])
        T.setStyle(rs_print_util.TableStyle([('TEXTCOLOR', (0, 0), (-1, -1), rs_print_util.colors.black),
                                             ('FONT', (0, 0), (-1, -1), 'DejaVu', 9),
                                             ('BOX', (0, 0), (-1, -1), 0.25, rs_print_util.colors.black),
                                             ('INNERGRID', (0, 0), (-1, -1), 0.25, rs_print_util.colors.black),
                                             ('ALIGN', (0, 0), (-1, -1), 'LEFT'),
                                             ('VALIGN', (0, 0), (-1, -1), 'TOP')]))

        return T

    def UtworzLogo(self, img):
        import rs_print_util
        import rsglobal

        from reportlab.platypus.flowables import Image
        from StringIO import StringIO

        if img is not None and img not in ('None', ''):
            try:
                img = Image(StringIO(img), 17*rs_print_util.cm, 2*rs_print_util.cm, kind='proportional')
                img.hAlign = 'CENTER'
            except Exception, e:
                rsglobal.ZapiszDoLogow(u'Nie udao si wczyta logo: '+str(e))

        TImg = rs_print_util.Table([[img]])
        TImg.setStyle(rs_print_util.TableStyle([('TEXTCOLOR', (0, 0), (0, 0), rs_print_util.colors.black),
                                                ('ALIGN', (0, 0), (0, 0), 'CENTER')]))

        return TImg

    def GenerujPDF(self):
        import rsglobal
        import rssql
        import rs_print_util
        import tempfile
        import traceback

        from reportlab.lib.styles import ParagraphStyle
        from reportlab.lib.enums import TA_LEFT, TA_CENTER
        from reportlab.platypus import Paragraph

        TekstSredni = ParagraphStyle(name='TekstSredni', fontName='DejaVu', fontSize=9, alignment=TA_LEFT)
        TekstSredniSrodek = ParagraphStyle(name='TekstSredniSrodek', fontName='DejaVu', fontSize=9, alignment=TA_CENTER)

        stat, Dane = self.PobierzDane()
        if stat != 'OK':
            return stat, ''

        # Plik certyfikatu
        self.DaneA = rssql.RSTableRowset2(Dane[0], Dane[1])[0]
        # Dane oddziau
        self.DaneO = rssql.RSTableRowset2(Dane[2], Dane[3])[0]

        try:
            Tresc = []

            if self.DaneA['plik_certyfikatu'] == 'Porto1':
                # Dane certyfikatu
                self.DaneC = rssql.RSTableRowset2(Dane[4], Dane[5])[0]

                Nagl = [[rs_print_util.Paragraf(u'wiadectwo jakoci paliw staych nr: ...', 'NaglDuzy')]]

                TLogo = self.UtworzLogo(self.logo)
                TNagl = self.UtworzNagl(Nagl)

                DanePrzedsiebiorcy = [[Paragraph(u'<b>Dane przedsibiorcy</b>: '+self.DaneO["nazwa"]+' '+self.DaneO["adr_ulica"]+' '+self.DaneO["adr_nry"]+', '+self.DaneO["adr_kod"]+' '+self.DaneO['adr_miej'], TekstSredni)]]
                NIP = [[Paragraph(u'<b>NIP</b> przedsibiorcy wystawiajcego wiadectwo jakoci: <b>'+str(self.DaneO["nip"])+'</b>', TekstSredni)]]
                REGON = [[Paragraph(u'<b>REGON</b> przedsibiorcy wystawiajcego wiadectwo jakoci: <b>'+str(self.DaneO["regon"])+'</b>', TekstSredni)]]
                Rodzaj = [[Paragraph(u'<b>Rodzaj paliwa staego</b>: '+self.DaneA["nazwa"], TekstSredni)]]
                System = [[Paragraph(u'<b>System certyfikacji ktry stanowi podstaw do tego aby uzna, e dany rodzaj paliwa staego spenia wszystkie wymagania jakociowe</b>: Laboratorium SGS Polska ul. Cieszyska 52A, 43-200 Pszczyna. Sprawozdanie z bada nr: ...', TekstSredni)]]
                Wskazanie = [[Paragraph(u'Wskazanie wartoci parametrw paliwa staego oraz informacja o wymaganiach jakociowych dla paliwa staego okrelona zgodnie z przepisami wydanymi na podstawie art. 3a ust. 2 ustawy z dnia 25 sierpnia 2006 r. o systemie monitorowania i kontrolowania jakoci paliw.', TekstSredni)]]

                TDanePrzedsiebiorcy = self.UtworzWiersz(DanePrzedsiebiorcy, 1)
                TNIP = self.UtworzWiersz(NIP, 1)
                TREGON = self.UtworzWiersz(REGON, 1)
                TRodzaj = self.UtworzWiersz(Rodzaj, 1)
                TSystem = self.UtworzWiersz(System, 1.5)
                TWskazanie = self.UtworzWiersz(Wskazanie, 1.5)

                Parametry = [[u'Parametr', u'Jednostka', Paragraph(u'Warto wskazana przez przedsibiorc', TekstSredniSrodek), u'Warto dopuszczalna'],
                             [u'', u'', u'', u'Minimalna', u'Maksymalna'],
                             [u'Zawarto popiou', u'%', self.DaneC[0]+u'%', u'-', u'12%'],
                             [u'Warto opaowa', u'MJ/kg', self.DaneC[1]+u' MJ/kg', u'24 MJ/kg', u'-'],
                             [u'Wymiar ziarna', u'mm', self.DaneC[2]+u' mm', u'3 mm', u'31,50 mm'],
                             [u'Zawarto siarki', u'%', self.DaneC[3]+u'%', u'-', u'1,20%'],
                             [u'Spiekalno RI', u'-', self.DaneC[4], u'-', u'25'],
                             [Paragraph(u'Zawarto wilgoci cakowitej', TekstSredni), u'%', self.DaneC[5]+u'%', u'-', u'20%']]

                TParametry = self.UtworzTabeleParametry(Parametry)

                Oswiadczenie = [[Paragraph(u'Owiadczam, e paliwo stae, dla ktrego jest wystawiane to wiadectwo, spenia wymagania jakociowe okrelone w przepisach wydanych na podstawie art. 3a ust. 2 ustawy z dnia 25 sierpnia 2006 r. o systemie monitorowania i kontrolowania jakoci paliw.', TekstSredniSrodek)]]
                Podpisy = [[u'Data i miejsce wystawienia wiadectwa\n\n\nCHRZANW ...................\n\n\n', Paragraph(u'Podpis przedsibiorcy wystawiajcego wiadectwo albo osoby upowanionej do jego reprezentowania', TekstSredni)]]

                TOswiadczenie = self.UtworzWiersz(Oswiadczenie, 1.5)
                TPodpisy = self.UtworzTabelePodpisy(Podpisy)

                Tresc.extend([TLogo, TNagl, TDanePrzedsiebiorcy, TNIP, TREGON, TRodzaj, TSystem, TWskazanie,
                              TParametry, TOswiadczenie, TPodpisy])

                if self.nazwa_pliku == '':
                    self.nazwa_pliku = tempfile.mktemp('.pdf')
                #AB 13.07.22r dodana zwrotka z ZrobPdf
                stat = rs_print_util.ZrobPdf(self.firma_id, self.nazwa_pliku, '', Tresc, ozdobniki=131,
                                             opcje={'topMargin': 0.2*rs_print_util.cm, 'bottomMargin': 1.2*rs_print_util.cm})
                if stat != 'OK':
                    return "Wystpi problem podczas generowania pliku w formacie PDF", ''
            else:
                self.nazwa_pliku = ''

        except Exception, e:
            rsglobal.ZapiszDoLogow(str(traceback.format_exc()))
            return "Bd generowania certyfikatu: "+str(e), ''
        else:
            return 'OK', self.nazwa_pliku


if stat == 'OK' and Funkcja == 1:
    PDF = CertyfikatArtykuluPDF(self, Magazyn, Dokument, Data, Numer, NrPoz, NrPPoz, '', Logo)
    stat, nazwa_pliku = PDF.GenerujPDF()

    tresc = ''
    if nazwa_pliku != '':
        tresc = fs_pliki_global.PodajTrescPliku(nazwa_pliku)
        if tresc is None:
            stat = 'Bd dostpu do wygenerowanego pliku.'
            tresc = ''

    dane = [tresc]
