# -*- coding: iso8859_2 -*-
#-----------------------------------------------------------------------------
# Name:        fs_wydruki_swiadectwo_pracy.py
# Purpose:     Generowanie wydruku wiadectwa pracy (12761)
# Author:      Wojciech Skowron
# Created:     2021/12/16
# RCS-ID:      $Id: fs_wydruki_swiadectwo_pracy.py $
# Copyright:   (c) 2021 Record System
# Licence:     GNU GPL
#-----------------------------------------------------------------------------

# 2022-02-22 Wojciech Skowron   (12844) Zmiana w zapytaniu 6.1 - urlop wypoczynkowy
# 2022-03-15 Wojciech Skowron   (12909) Poprawki
# 2022-03-18 Wojciech Skowron   (12909) Poprawki w 6.1 oraz 6.3
# 2022-04-14 Wojciech Skowron   (12969) Wydruk w .docx
# 2022-07-13 Aleksander Bargie (12989) Komunikat gdy tworzenie raportu pdf nie powiedzie si
# 2022-07-25 Wojciech Skowron   (13108) Poprawki
# 2022-07-28 Wojciech Skowron   (13114) Poprawka w groupowaniu w 6.7
# 2022-09-02 Wojciech Skowron   (13140) Usunicie przyczyny z pkt 4
# 2023-03-09 Wojciech Skowron   (13384) Nie uwzgldnia przerw w angaach w p1
# 2023-06-01 Wojciech Skowron   (13452) Dodatkowe punkty w wiadectwie pracy - 6.1 SW, 6.3 UO, 6.10 praca zdalna
# 2023-06-06 Wojciech Skowron   (13457) Zmiana w sposobie obliczania dni dla urlopu wypoczynkowego (6.2)
# 2024-08-01 Wojciech Skowron   (13659) Zmiana w 5., usunity fragment "art. 36<sup>1</sup>  1 Kodeksu pracy"

import fractions

from funkcje.fs_pliki_office import RSDOCX
from funkcje import fs_pliki_global
import rsglobal
import rssql

stat = 'OK'
dane = []

if not 'Funkcja' in klucze:
    stat = 'Brak podanej funkcji w kluczach'

if stat == 'OK':
    Funkcja = klucze['Funkcja']

    SlPar = {'Pracownik': [1], 'Zatrudnienie': [1], 'Wypowiedzenie': [1], 'CzasPracy': [1], 'DataWojskoOd': [1],
             'DataWojskoDo': [1], 'InfoDodatkowe': [1], 'ZajKomornicze': [1], 'Format': [1], }
    Lista = rsglobal.SprawdzParametryWKluczach(SlPar, Funkcja, klucze)
    if Lista != '':
        stat = u'Uwaga : wywoanie funkcji fs_wydruki_swiadectwo_pracy z numerem '+str(Funkcja)+u' bez parametrw: '+Lista

Upr = (((self.oddzial_id, '052000') in self.uprawnienia) or (self.operator_id == STANDARD_OPERATOR_ID))
if not Upr:
    stat = 'BR_UPR'

if stat == 'OK':
    if Funkcja == 1:
        Pracownik = rsglobal.Dekoduj(klucze['Pracownik'])
        Zatrudnienie = rsglobal.Dekoduj(klucze['Zatrudnienie'])
        Wypowiedzenie = rsglobal.Dekoduj(klucze['Wypowiedzenie'])
        CzasPracy = rsglobal.Dekoduj(klucze['CzasPracy'])
        DataWojskoOd = rsglobal.Dekoduj(klucze['DataWojskoOd'])
        DataWojskoDo = rsglobal.Dekoduj(klucze['DataWojskoDo'])
        InfoDodatkowe = rsglobal.Dekoduj(klucze['InfoDodatkowe'])
        ZajKomornicze = rsglobal.Dekoduj(klucze['ZajKomornicze'])
        Format = rsglobal.Dekoduj(klucze['Format'])  #WS 2022-04-14


def wykonaj_zapytanie(self, zapytanie, zmienne, tresc_bledu):
    dane_rowset = None

    zapytanie = zapytanie.format(**locals())
    stat, dane_zapytanie, wyn1, dump = self.PolSQL.Execute(zapytanie, 3, zmienne)
    if stat != 'OK':
        stat = tresc_bledu
    else:
        dane_rowset = rssql.RSTableRowset2(dane_zapytanie, wyn1)

    return stat, dane_rowset


if stat == 'OK' and Funkcja == 1:
    zapytanie = """SELECT nazwa, nip, regon, adr_kod, adr_miej, adr_ulica, adr_nry 
                   FROM rs_oddzialy 
                   WHERE firma_id=%s AND centrala;"""
    zmienne = [self.firma_id]
    rsglobal.ZapiszDoLogow('Pobranie danych oddziau: '+self.PolSQL.Mogrify(zapytanie, zmienne))
    stat, dane_oddzial = wykonaj_zapytanie(self, zapytanie, zmienne, u'Bd pobrania danych centrali')
    if len(dane_oddzial) == 0:
        stat = u"Brak oddziau oznaczonego jako centrala "

    if stat == 'OK':
        #WS 2022-07-25 - pobranie zwolnienie_data
        zapytanie = """SELECT MIN(od_kiedy) AS od_kiedy, MAX(COALESCE(do_kiedy, '3000-01-01')) AS do_kiedy, COALESCE(MAX(zwolnienie_data),MAX(do_kiedy)) AS zwolnienie_data
                        FROM (SELECT od_kiedy, do_kiedy, zwolnienie_data
                              FROM rs_kp_umowy_o_prace uop
                              JOIN rs_kp_zatrudnienie z ON (z.firma_id=uop.firma_id AND z.pracownik_id=uop.pracownik_id AND z.numer_id=uop.zatrudnienie_id)
                              WHERE uop.firma_id=%s AND uop.pracownik_id=%s AND uop.zatrudnienie_id=%s) uop;"""
        zmienne = [self.firma_id, Pracownik, Zatrudnienie]
        rsglobal.ZapiszDoLogow('Pobranie okresu zatrudnienia: '+self.PolSQL.Mogrify(zapytanie, zmienne))
        stat, dane_okres_zatrudnienia = wykonaj_zapytanie(self, zapytanie, zmienne, u'Bd pobrania okresu zatrudnienia')

    if stat == 'OK':
        ZatrudnienieOd = dane_okres_zatrudnienia[0][0]
        ZatrudnienieDo = dane_okres_zatrudnienia[0][1]
        ZwolenienieData = dane_okres_zatrudnienia[0][2]

        if ZatrudnienieDo in ('None', None):
            stat = u'Brak daty zakoczenia umowy dla wskazanego zatrudnienia'

    if stat == 'OK':
        zapytanie = """SELECT imie, nazwisko, data_urodzenia 
                       FROM rs_kp_dane_osobowe 
                       WHERE firma_id=%s AND pracownik_id=%s;"""
        zmienne = [self.firma_id, Pracownik]
        rsglobal.ZapiszDoLogow('SP 1. dane osobowe: '+self.PolSQL.Mogrify(zapytanie, zmienne))
        stat, dane_1_pracownik = wykonaj_zapytanie(self, zapytanie, zmienne, u'Bd pobrania danych osobowych pracownika')

    if stat == 'OK':
        zapytanie = """SELECT m.opis
                       FROM rs_kp_angaze a
                       JOIN rs_miejsca_pracy m ON a.firma_id = m.firma_id AND a.miejsce_id = m.kod
                       WHERE a.firma_id=%s AND a.pracownik_id=%s AND a.zatrudnienie_id=%s
                       ORDER BY od_kiedy DESC LIMIT 1;"""
        zmienne = [self.firma_id, Pracownik, Zatrudnienie]
        rsglobal.ZapiszDoLogow('SP 1. dane pracodawcy: '+self.PolSQL.Mogrify(zapytanie, zmienne))
        stat, dane_1_pracodawca = wykonaj_zapytanie(self, zapytanie, zmienne, u'Bd pobrania danych pracodawcy')

    if stat == 'OK':
        #WS 2023-03-09 - bez uwzgldniania przerw w angaach - zmiana w podzapytaniu an
        zapytanie = """SELECT etat, MIN(od_kiedy) AS od_kiedy, MAX(do_kiedy) AS do_kiedy
                       FROM (
                         SELECT etat, od_kiedy, do_kiedy, SUM(startflag) OVER (PARTITION BY etat ORDER BY etat, od_kiedy) AS grp
                         FROM(
                           SELECT etat, od_kiedy, do_kiedy, 
                                  (CASE WHEN LAG(do_kiedy) OVER (PARTITION BY etat ORDER BY etat, od_kiedy) IN (od_kiedy-INTERVAL '1 day', od_kiedy)
                                   THEN 0 ELSE 1 END) AS startflag 
                           FROM
                           (
                             SELECT etat, 
                             (CASE WHEN liczba_roznych_etatow > 1 THEN angaz_od_kiedy ELSE umowa_od_kiedy END) AS od_kiedy,
                             (CASE WHEN liczba_roznych_etatow > 1 THEN angaz_do_kiedy ELSE umowa_do_kiedy END) AS do_kiedy
                             FROM (
                             SELECT COUNT(an.etat) OVER (PARTITION BY u.umowa_id) AS liczba_roznych_etatow,
                                     an.etat, MIN(u.od_kiedy) AS umowa_od_kiedy, MAX(u.do_kiedy) AS umowa_do_kiedy, 
                                     MIN(an.od_kiedy) AS angaz_od_kiedy, MAX(an.do_kiedy) AS angaz_do_kiedy
                             FROM rs_kp_zatrudnienie z
                             JOIN rs_kp_umowy_o_prace u ON(z.firma_id=u.firma_id AND z.pracownik_id=u.pracownik_id AND z.numer_id=u.zatrudnienie_id)
                             JOIN (SELECT MIN(od_kiedy) AS od_kiedy, MAX(do_kiedy) AS do_kiedy, firma_id, pracownik_id, zatrudnienie_id, umowa_id, etat
                                   FROM rs_kp_angaze
                                   GROUP BY firma_id, pracownik_id, zatrudnienie_id, umowa_id, etat
                                   ) an
                                  ON(z.firma_id=an.firma_id AND z.pracownik_id=an.pracownik_id AND z.numer_id=an.zatrudnienie_id AND u.umowa_id=an.umowa_id)
                             WHERE z.firma_id=%s AND z.pracownik_id=%s AND z.numer_id=%s
                             GROUP BY u.umowa_id, an.etat
                             ORDER BY angaz_od_kiedy) zap) zap) zap ORDER BY od_kiedy) zap GROUP BY etat ORDER BY od_kiedy;"""
        #WS 2022-03-15
        # zapytanie = """SELECT etat, MIN(od_kiedy) AS od_kiedy, MAX(do_kiedy) AS do_kiedy
        #                FROM (
        #                  SELECT etat, od_kiedy, do_kiedy, SUM(startflag) OVER (PARTITION BY etat ORDER BY etat, od_kiedy) AS grp
        #                  FROM(
        #                    SELECT etat, od_kiedy, do_kiedy,
        #                           (CASE WHEN LAG(do_kiedy) OVER (PARTITION BY etat ORDER BY etat, od_kiedy) IN (od_kiedy-INTERVAL '1 day', od_kiedy)
        #                            THEN 0 ELSE 1 END) AS startflag
        #                    FROM
        #                    (
        #                      SELECT etat,
        #                      (CASE WHEN liczba_roznych_etatow > 1 THEN angaz_od_kiedy ELSE umowa_od_kiedy END) AS od_kiedy,
        #                      (CASE WHEN liczba_roznych_etatow > 1 THEN angaz_do_kiedy ELSE umowa_do_kiedy END) AS do_kiedy
        #                      FROM (
        #                      SELECT COUNT(an.etat) OVER (PARTITION BY u.umowa_id) AS liczba_roznych_etatow,
        #                              an.etat, MIN(u.od_kiedy) AS umowa_od_kiedy, MAX(u.do_kiedy) AS umowa_do_kiedy,
        #                              MIN(an.od_kiedy) AS angaz_od_kiedy, MAX(an.do_kiedy) AS angaz_do_kiedy
        #                      FROM rs_kp_zatrudnienie z
        #                      JOIN rs_kp_umowy_o_prace u ON(z.firma_id=u.firma_id AND z.pracownik_id=u.pracownik_id AND z.numer_id=u.zatrudnienie_id)
        #                      JOIN (SELECT a.*, SUM(startflag) OVER (PARTITION BY firma_id, pracownik_id, zatrudnienie_id, etat ORDER BY firma_id, pracownik_id, zatrudnienie_id, od_kiedy) AS grp
        #                            FROM
        #                            (SELECT a.*,
        #                                    (CASE WHEN LAG(do_kiedy) OVER (PARTITION BY firma_id, pracownik_id, zatrudnienie_id, etat ORDER BY firma_id, pracownik_id, zatrudnienie_id, od_kiedy) IN (od_kiedy-INTERVAL '1 day', od_kiedy)
        #                                     THEN 0 ELSE 1 END) AS startflag
        #                             FROM rs_kp_angaze a
        #                            ) a
        #                           ) an
        #                           ON(z.firma_id=an.firma_id AND z.pracownik_id=an.pracownik_id AND z.numer_id=an.zatrudnienie_id AND u.umowa_id=an.umowa_id)
        #                      WHERE z.firma_id=%s AND z.pracownik_id=%s AND z.numer_id=%s
        #                      GROUP BY u.umowa_id, an.etat, an.grp
        #                      ORDER BY angaz_od_kiedy) zap) zap) zap ORDER BY od_kiedy) zap GROUP BY etat, grp ORDER BY od_kiedy;"""
        zmienne = [self.firma_id, Pracownik, Zatrudnienie]
        rsglobal.ZapiszDoLogow('SP 1. dane zatrudnienia: '+self.PolSQL.Mogrify(zapytanie, zmienne))
        stat, dane_1_zatrudnienie = wykonaj_zapytanie(self, zapytanie, zmienne, u'Bd pobrania danych zatrudnienia')

    if stat == 'OK':
        #WS 2022-03-15
        zapytanie = """SELECT s.opis, MIN(an.od_kiedy) AS od_kiedy, MAX(an.do_kiedy) AS do_kiedy
                       FROM (SELECT a.*, SUM(startflag) OVER (PARTITION BY firma_id, pracownik_id, zatrudnienie_id ORDER BY od_kiedy) AS grp
                              FROM 
                              (SELECT a.*, 
                                      (CASE WHEN LAG(stanowisko_id) OVER (PARTITION BY firma_id, pracownik_id, zatrudnienie_id ORDER BY od_kiedy)=(stanowisko_id)
                                       THEN 0 ELSE 1 END) AS startflag 
                               FROM rs_kp_angaze a
                              ) a
                             ) an
                       LEFT OUTER JOIN rs_stanowiska_pracy s ON (an.firma_id=s.firma_id AND an.stanowisko_id=s.kod)
                       WHERE an.firma_id=%s AND an.pracownik_id=%s AND an.zatrudnienie_id=%s
                       GROUP BY an.pracownik_id, an.stanowisko_id, s.opis, an.grp
                       ORDER BY od_kiedy;;"""
        zmienne = [self.firma_id, Pracownik, Zatrudnienie]
        rsglobal.ZapiszDoLogow('SP 3. dane angay: '+self.PolSQL.Mogrify(zapytanie, zmienne))
        stat, dane_3 = wykonaj_zapytanie(self, zapytanie, zmienne, u'Bd pobrania danych angay')

    if stat == 'OK':
        zapytanie = """SELECT przyczyna_wyrejestrowania_zus, p.opis AS podstawa_rozw, r.opis AS przyczyna_rozw
                       FROM rs_kp_zatrudnienie z
                       LEFT OUTER JOIN rs_kp_zus_kody_podstawy_rozwiazania_stos_pracy p ON z.podst_prawna_rozw_stos_pracy = p.kod
                       LEFT OUTER JOIN rs_kp_zus_kody_rozwiazania_stosunku_pracy r ON z.kod_rozw_stos_pracy = r.kod
                       WHERE z.firma_id=%s AND z.pracownik_id=%s AND z.numer_id=%s;"""
        zmienne = [self.firma_id, Pracownik, Zatrudnienie]
        rsglobal.ZapiszDoLogow('SP 4. przyczyna rozwiazania: '+self.PolSQL.Mogrify(zapytanie, zmienne))
        stat, dane_4 = wykonaj_zapytanie(self, zapytanie, zmienne, u'Bd pobrania danych rozwizania/wyganicia stosunku pracy')

    if stat == 'OK':
        #WS 2023-06-01
        zapytanie = """SELECT COALESCE(SUM(a.dni_robocze), 0) AS suma_dni
                       FROM rs_kp_dane_osobowe_absencje a
                       WHERE a.firma_id=%s AND a.pracownik_id=%s AND a.typ_absencji_id IN ('SW') AND 
                             EXTRACT(YEAR FROM a.od_kiedy)=EXTRACT(YEAR FROM %s::DATE) AND 
                             EXTRACT(YEAR FROM a.do_kiedy)=EXTRACT(YEAR FROM %s::DATE) AND
                             a.od_kiedy>=%s AND a.do_kiedy<=%s
                       ;"""
        zmienne = [self.firma_id, Pracownik, ZwolenienieData, ZwolenienieData, ZatrudnienieOd, ZatrudnienieDo]
        rsglobal.ZapiszDoLogow('SP 6.1. sila wyzsza: '+self.PolSQL.Mogrify(zapytanie, zmienne))
        stat, dane_6_1 = wykonaj_zapytanie(self, zapytanie, zmienne, u'Bd pobrania danych o urlopie z powodu siy wyszej')

    if stat == 'OK':
        #WS 2022-02-22
        #WS 2022-03-18 - do dni dodana warto skadnika 171 z list pac z roku rozw. stos. pracy (ten skladnik tylko raz moe mie wart != 0)
        #WS 2023-06-06 - zmiana w sposobie liczenia dni (podstawa_urlopu/12*liczba_miesicy_przepracowanych)
        zapytanie = """SELECT CEIL(COALESCE(limit_dni, 0)/12*((EXTRACT(MONTHS FROM AGE(%s::DATE, GREATEST(EXTRACT(YEAR FROM %s::DATE)||'-01-01',%s)::DATE))+(CASE WHEN EXTRACT(DAYS FROM AGE(%s::DATE, GREATEST(EXTRACT(YEAR FROM %s::DATE)||'-01-01',%s)::DATE))>0 THEN 1 ELSE 0 END)))) AS dni
                       FROM rs_kp_dane_osobowe_limity_urlopow l
                       WHERE l.firma_id=%s AND l.pracownik_id=%s AND l.urlop_id='1'
                       GROUP BY l.limit_dni, l.od_dnia
                       ORDER BY l.od_dnia DESC LIMIT 1;"""
        # zapytanie = """SELECT GREATEST(COALESCE(a.suma_dni_robocze, 0)+COALESCE(MAX(lpos.wartosc), 0)-l.limit_dni_zalegle, 0) AS dni_abs,
        #                       COALESCE(limit_dni, 0) AS limit_dni
        #                FROM rs_kp_dane_osobowe_limity_urlopow l
        #                LEFT OUTER JOIN rs_kp_listy_plac lp ON (lp.firma_id=l.firma_id AND EXTRACT(YEAR FROM lp.data_wyplaty)=EXTRACT(YEAR FROM %s::DATE) AND lp.data_wyplaty BETWEEN %s AND %s)
        #                LEFT OUTER JOIN rs_kp_listy_plac_osoby_skladniki lpos ON (lpos.firma_id=lp.firma_id AND lpos.numer_listy_id=lp.numer_listy_id AND lpos.pracownik_id=l.pracownik_id AND lpos.zatrudnienie_id=%s AND lpos.skladnik_id='171')
        #                LEFT OUTER JOIN (SELECT firma_id, pracownik_id, typ_absencji_id, SUM(dni_robocze) AS suma_dni_robocze
        #                                 FROM rs_kp_dane_osobowe_absencje
        #                                 WHERE EXTRACT(YEAR FROM do_kiedy)=EXTRACT(YEAR FROM %s::DATE) AND
        #                                       od_kiedy>=%s AND do_kiedy<=%s
        #                                 GROUP BY firma_id, pracownik_id, typ_absencji_id) a ON (l.firma_id=a.firma_id AND l.pracownik_id=a.pracownik_id AND l.urlop_id=a.typ_absencji_id)
        #                WHERE l.firma_id=%s AND l.pracownik_id=%s AND l.urlop_id='1'
        #                GROUP BY a.suma_dni_robocze, l.limit_dni_zalegle, l.limit_dni, l.od_dnia
        #                ORDER BY l.od_dnia DESC LIMIT 1;"""
        # zmienne = [ZwolenienieData, ZatrudnienieOd, ZatrudnienieDo, Zatrudnienie, ZwolenienieData, ZatrudnienieOd, ZatrudnienieDo, self.firma_id, Pracownik]
        zmienne = [ZatrudnienieDo, ZatrudnienieDo, ZatrudnienieOd, ZatrudnienieDo, ZatrudnienieDo, ZatrudnienieOd, self.firma_id, Pracownik]
        rsglobal.ZapiszDoLogow('SP 6.2. urlop wypoczynkowy: '+self.PolSQL.Mogrify(zapytanie, zmienne))
        stat, dane_6_2_wypoczynkowy = wykonaj_zapytanie(self, zapytanie, zmienne, u'Bd pobrania danych o urlopie wypoczynkowym')

    if stat == 'OK':
        zapytanie = """SELECT COALESCE(SUM(dni_robocze), 0) AS dni_rok, COALESCE(MAX(limit_dni), 0) AS limit_dni
                       FROM rs_kp_dane_osobowe_absencje a
                       LEFT OUTER JOIN rs_kp_dane_osobowe_limity_urlopow l ON (l.firma_id=a.firma_id AND l.pracownik_id=a.pracownik_id AND l.urlop_id=a.typ_absencji_id)
                       WHERE a.firma_id=%s AND a.pracownik_id=%s AND a.na_zadanie=TRUE AND a.typ_absencji_id='1' AND
                             EXTRACT(YEAR FROM a.do_kiedy)=EXTRACT(YEAR FROM %s::DATE) AND
                             a.od_kiedy>=%s AND a.do_kiedy<=%s
                       GROUP BY l.od_dnia ORDER BY l.od_dnia DESC LIMIT 1
                       ;"""
        zmienne = [self.firma_id, Pracownik, ZwolenienieData, ZatrudnienieOd, ZatrudnienieDo]
        rsglobal.ZapiszDoLogow('SP 6.2. urlop na zadanie: '+self.PolSQL.Mogrify(zapytanie, zmienne))
        stat, dane_6_2_na_zadanie = wykonaj_zapytanie(self, zapytanie, zmienne, u'Bd pobrania danych o urlopie na danie')

    if stat == 'OK':
        #WS 2023-06-01
        zapytanie = """SELECT COALESCE(SUM(a.dni_robocze), 0) AS suma_dni
                       FROM rs_kp_dane_osobowe_absencje a
                       WHERE a.firma_id=%s AND a.pracownik_id=%s AND a.typ_absencji_id IN ('UO') AND 
                             EXTRACT(YEAR FROM a.od_kiedy)=EXTRACT(YEAR FROM %s::DATE) AND 
                             EXTRACT(YEAR FROM a.do_kiedy)=EXTRACT(YEAR FROM %s::DATE) AND
                             a.od_kiedy>=%s AND a.do_kiedy<=%s
                       ;"""
        zmienne = [self.firma_id, Pracownik, ZwolenienieData, ZwolenienieData, ZatrudnienieOd, ZatrudnienieDo]
        rsglobal.ZapiszDoLogow('SP 6.3. urlop opiekunczy: '+self.PolSQL.Mogrify(zapytanie, zmienne))
        stat, dane_6_3 = wykonaj_zapytanie(self, zapytanie, zmienne, u'Bd pobrania danych o urlopie opiekuczym')

    if stat == 'OK':
        zapytanie = """SELECT a.od_kiedy, a.do_kiedy
                       FROM rsv_kp_dane_osobowe_absencje_scalone a
                       WHERE a.firma_id=%s AND a.pracownik_id=%s AND a.typ_absencji_id='B' AND 
                             a.od_kiedy>=%s AND a.do_kiedy<=%s
                       ORDER BY a.od_kiedy
                       ;"""
        zmienne = [self.firma_id, Pracownik, ZatrudnienieOd, ZatrudnienieDo]
        rsglobal.ZapiszDoLogow('SP 6.4. urlop bezpatny: '+self.PolSQL.Mogrify(zapytanie, zmienne))
        stat, dane_6_4 = wykonaj_zapytanie(self, zapytanie, zmienne, u'Bd pobrania danych o urlopie bezpatnym')

    if stat == 'OK':
        #WS 2022-03-18
        zapytanie = """SELECT SUM(a.dni_kalendarzowe) AS suma_dni, COUNT(a.od_kiedy) AS liczba_czesci
                       FROM rs_kp_dane_osobowe os
                       JOIN rsv_kp_dane_osobowe_absencje_scalone a
                            ON (a.firma_id=os.firma_id AND a.pracownik_id=os.pracownik_id AND a.typ_absencji_id='O' AND
                                a.od_kiedy>=%s AND a.do_kiedy<=%s)
                       WHERE os.firma_id=%s AND os.pracownik_id=%s AND os.plec='M'
                       ;"""
        # zapytanie = """SELECT SUM(a.dni_kalendarzowe) AS suma_dni, COUNT(a.od_kiedy) AS liczba_czesci
        #                FROM rs_kp_dane_osobowe os
        #                JOIN rs_kp_rodzina r
        #                     ON (r.firma_id=os.firma_id AND r.pracownik_id=os.pracownik_id AND r.pokrewienstwo_id='11')
        #                JOIN rsv_kp_dane_osobowe_absencje_scalone a
        #                     ON (a.firma_id=r.firma_id AND a.pracownik_id=r.pracownik_id AND a.typ_absencji_id='O' AND
        #                         a.od_kiedy>=r.data_urodzenia AND a.do_kiedy<=r.data_urodzenia+INTERVAL '24 months' AND
        #                         a.od_kiedy>=%s AND a.do_kiedy<=%s)
        #                WHERE os.firma_id=%s AND os.pracownik_id=%s AND os.plec='M'
        #                GROUP BY r.rodzina_id
        #                ;"""
        zmienne = [ZatrudnienieOd, ZatrudnienieDo, self.firma_id, Pracownik]
        rsglobal.ZapiszDoLogow('SP 6.5. urlop ojcowski: '+self.PolSQL.Mogrify(zapytanie, zmienne))
        stat, dane_6_5 = wykonaj_zapytanie(self, zapytanie, zmienne, u'Bd pobrania danych o urlopie ojcowskim')

    if stat == 'OK':
        zapytanie = """SELECT SUM(a.dni_kalendarzowe) AS suma_dni, COUNT(a.od_kiedy) AS liczba_czesci
                       FROM rsv_kp_dane_osobowe_absencje_scalone a
                       WHERE a.firma_id=%s AND a.pracownik_id=%s AND 
                             a.typ_absencji_id='R' AND a.od_kiedy>=%s AND a.do_kiedy<=%s
                       ;"""
        # zapytanie = """SELECT r.imie, r.nazwisko, SUM(a.dni_kalendarzowe) AS dni_kalendarzowe,
        #                       COUNT(a.od_kiedy) AS liczba_czesci,
        #                       COUNT(r2.data_urodzenia) AS liczba_tych_samych_dat_urodzenia
        #                FROM rsv_kp_dane_osobowe_absencje_scalone a
        #                JOIN rs_kp_rodzina r ON (r.firma_id=a.firma_id AND r.pracownik_id=a.pracownik_id AND r.rodzina_id=a.rodzina_id)
        #                LEFT OUTER JOIN rs_kp_rodzina r2
        #                                ON (r2.firma_id=r.firma_id AND r2.pracownik_id=r.pracownik_id AND
        #                                    r2.data_urodzenia=r.data_urodzenia AND r2.rodzina_id!=r.rodzina_id)
        #                WHERE a.firma_id=%s AND a.pracownik_id=%s AND
        #                      a.typ_absencji_id='R' AND a.od_kiedy>=%s AND a.do_kiedy<=%s
        #                GROUP BY a.pracownik_id, r.imie, r.nazwisko, r.rodzina_id
        #                ;"""
        zmienne = [self.firma_id, Pracownik, ZatrudnienieOd, ZatrudnienieDo]
        rsglobal.ZapiszDoLogow('SP 6.6. urlop rodzicielski: '+self.PolSQL.Mogrify(zapytanie, zmienne))
        stat, dane_6_6 = wykonaj_zapytanie(self, zapytanie, zmienne, u'Bd pobrania danych o urlopie rodzicielskim')

    if stat == 'OK':
        zapytanie = """SELECT a.od_kiedy, a.do_kiedy, SUM(a.dni_kalendarzowe) OVER (PARTITION BY a.pracownik_id) AS suma_dni,
                              COUNT(a.od_kiedy) OVER (PARTITION BY a.pracownik_id) AS liczba_czesci
                       FROM rsv_kp_dane_osobowe_absencje_scalone a
                       WHERE a.firma_id=%s AND a.pracownik_id=%s AND a.typ_absencji_id='3' AND 
                             a.od_kiedy>=%s AND a.do_kiedy<=%s
                       ORDER BY a.od_kiedy
                       ;"""
        zmienne = [self.firma_id, Pracownik, ZatrudnienieOd, ZatrudnienieDo]
        rsglobal.ZapiszDoLogow('SP 6.7. urlop wychowawczy: '+self.PolSQL.Mogrify(zapytanie, zmienne))
        stat, dane_6_7 = wykonaj_zapytanie(self, zapytanie, zmienne, u'Bd pobrania danych o urlopie wychowawczym')

    if stat == 'OK':
        #WS 2022-07-25
        zapytanie = """SELECT SUM(a.dni_robocze) AS suma_dni
                       FROM rsv_kp_dane_osobowe_absencje_scalone a
                       WHERE a.firma_id=%s AND a.pracownik_id=%s AND a.typ_absencji_id='4' AND
                             EXTRACT(YEAR FROM a.do_kiedy)=EXTRACT(YEAR FROM %s::DATE) AND
                             a.od_kiedy>=%s AND a.do_kiedy<=%s
                       GROUP BY a.pracownik_id
                       ;"""
        # zapytanie = """SELECT SUM(a.dni_kalendarzowe) AS suma_dni
        #                FROM rs_kp_rodzina r
        #                JOIN rsv_kp_dane_osobowe_absencje_scalone a
        #                     ON (a.firma_id=r.firma_id AND a.pracownik_id=r.pracownik_id AND
        #                         a.typ_absencji_id='4' AND a.od_kiedy>=r.data_urodzenia AND
        #                         a.do_kiedy<r.data_urodzenia+INTERVAL '14 years' AND
        #                         EXTRACT(YEAR FROM a.do_kiedy)=EXTRACT(YEAR FROM %s::DATE)) AND
        #                         a.od_kiedy>=%s AND a.do_kiedy<=%s
        #                WHERE a.firma_id=%s AND a.pracownik_id=%s AND r.pokrewienstwo_id='11'
        #                GROUP BY a.pracownik_id, a.od_kiedy, a.do_kiedy, a.dni_kalendarzowe, r.rodzina_id
        #                ;"""
        zmienne = [self.firma_id, Pracownik, ZwolenienieData, ZatrudnienieOd, ZatrudnienieDo]
        rsglobal.ZapiszDoLogow('SP 6.9. zwolnienie od pracy art. 188: '+self.PolSQL.Mogrify(zapytanie, zmienne))
        stat, dane_6_9 = wykonaj_zapytanie(self, zapytanie, zmienne, u'Bd pobrania danych o zwolnieniu od pracy art. 188')

    if stat == 'OK':
         #WS 2022-03-15 - pobranie okresw
         zapytanie = """SELECT SUM(a.dni_kalendarzowe) OVER (PARTITION BY pracownik_id) AS suma_dni, od_kiedy, do_kiedy
                        FROM rsv_kp_dane_osobowe_absencje_scalone a
                        WHERE a.firma_id=%s AND a.pracownik_id=%s AND a.typ_absencji_id IN ('6A','6S','6W') AND 
                              EXTRACT(YEAR FROM a.od_kiedy)=EXTRACT(YEAR FROM %s::DATE) AND 
                              EXTRACT(YEAR FROM a.do_kiedy)=EXTRACT(YEAR FROM %s::DATE) AND
                              a.od_kiedy>=%s AND a.do_kiedy<=%s
                        GROUP BY pracownik_id, dni_kalendarzowe, od_kiedy, do_kiedy
                        ;"""
         zmienne = [self.firma_id, Pracownik, ZwolenienieData, ZwolenienieData, ZatrudnienieOd, ZatrudnienieDo]
         rsglobal.ZapiszDoLogow('SP 6.11. niezdolnosc do pracy: '+self.PolSQL.Mogrify(zapytanie, zmienne))
         stat, dane_6_11 = wykonaj_zapytanie(self, zapytanie, zmienne, u'Bd pobrania danych o okresie niezdolnoci do pracy')

    if stat == 'OK':
         zapytanie = """SELECT a.od_kiedy, a.do_kiedy
                        FROM rsv_kp_dane_osobowe_absencje_scalone a
                        WHERE a.firma_id=%s AND a.pracownik_id=%s AND 
                              a.typ_absencji_id IN ('6A','6S','6W') AND 
                              a.od_kiedy>='2003-01-01' AND a.do_kiedy<='2003-12-31'
                        ORDER BY a.od_kiedy
                        ;"""
         zmienne = [self.firma_id, Pracownik]
         rsglobal.ZapiszDoLogow('SP 6.12. dni 2003: '+self.PolSQL.Mogrify(zapytanie, zmienne))
         stat, dane_6_12 = wykonaj_zapytanie(self, zapytanie, zmienne, u'Bd pobrania danych')

    if stat == 'OK':
         #WS 2022-03-15 - pobranie nazwy typu absencji
         #WS 2022-07-25 - dodany typ 8
         zapytanie = """SELECT a.od_kiedy, a.do_kiedy, r.nazwa
                        FROM rsv_kp_dane_osobowe_absencje_scalone a
                        LEFT OUTER JOIN rs_kp_absencje_rodzaje r USING (firma_id, typ_absencji_id)
                        WHERE a.firma_id=%s AND a.pracownik_id=%s AND 
                              a.typ_absencji_id IN ('6A','6B','6S','6T','6W','6Z','7','8','9','W') AND 
                              a.od_kiedy>=%s AND a.do_kiedy<=%s
                        ORDER BY a.od_kiedy
                        ;"""
         zmienne = [self.firma_id, Pracownik, ZatrudnienieOd, ZatrudnienieDo]
         rsglobal.ZapiszDoLogow('SP 6.16. okresy nieskladkowe: '+self.PolSQL.Mogrify(zapytanie, zmienne))
         stat, dane_6_16 = wykonaj_zapytanie(self, zapytanie, zmienne, u'Bd pobrania danych o okresach nieskadkowych')

if stat == 'OK' and Funkcja == 1:
    class SwiadectwoPDF:
        LEFT = 0
        CENTER = 1
        RIGHT = 2
        JUSTIFY = 4

        def __init__(self):
            import rs_print_util

            rs_print_util.PrzygotujCzcionki()

        def Paragraf(self, tekst, styl, wyrownanie=-1):
            import rs_print_util

            if wyrownanie > -1:
                styl.alignment = wyrownanie
            return rs_print_util.Paragraph(tekst, styl)

        def DodajStylParagrafu(self, nazwa, font='DejaVu', rozmiar=9, odstep_przed=0, odstep_za=0, wyrownanie=JUSTIFY):
            import rs_print_util

            return rs_print_util.ParagraphStyle(name=nazwa, fontName=font, fontSize=rozmiar, spaceBefore=odstep_przed, spaceAfter=odstep_za, alignment=wyrownanie)

        def ZrobDok(self, firma_id, nazwa_pliku, Tresc):
            import rs_print_util
            #AB 13.07.22r dodana zwrotka z ZrobPdf
            return rs_print_util.ZrobPdf(firma_id, nazwa_pliku, '', Tresc, ozdobniki=135,
                                         opcje={'topMargin': rs_print_util.cm, 'bottomMargin': 1.6 * rs_print_util.cm,
                                                'leftMargin': 1.1 * rs_print_util.cm, 'rightMargin': 1.1 * rs_print_util.cm})

    if Format == 'docx':
        Swiadectwo = RSDOCX()
    else:
        Swiadectwo = SwiadectwoPDF()

    Tresc = []

    TekstSredniLewoBezOdstepow = Swiadectwo.DodajStylParagrafu(nazwa='TekstSredniLewoBezOdstepow', wyrownanie=Swiadectwo.LEFT)
    TekstSredniPrawo = Swiadectwo.DodajStylParagrafu(nazwa='TekstSredniPrawo', odstep_za=12, wyrownanie=Swiadectwo.RIGHT)
    TekstSredni = Swiadectwo.DodajStylParagrafu(nazwa='TekstSredni', odstep_przed=12, odstep_za=12)
    TekstNagl = Swiadectwo.DodajStylParagrafu(nazwa='TekstNagl', rozmiar=15, odstep_przed=30, odstep_za=40, wyrownanie=Swiadectwo.CENTER)

    #WS 2022-03-15
    MiejscowoscData = Swiadectwo.Paragraf(dane_oddzial[0]['adr_miej']+', '+ZwolenienieData, TekstSredniPrawo)
    # MiejscowoscData = Swiadectwo.Paragraf(dane_oddzial[0]['adr_miej']+', '+rsglobal.PodajDzis(), TekstSredniPrawo)
    Nazwa = Swiadectwo.Paragraf(dane_oddzial[0]['nazwa'], TekstSredniLewoBezOdstepow)
    Adres = Swiadectwo.Paragraf(dane_oddzial[0]['adr_ulica']+' '+dane_oddzial[0]['adr_nry'], TekstSredniLewoBezOdstepow)
    MiastoKod = Swiadectwo.Paragraf(dane_oddzial[0]['adr_kod']+' '+dane_oddzial[0]['adr_miej'], TekstSredniLewoBezOdstepow)
    REGON = Swiadectwo.Paragraf(dane_oddzial[0]['regon'], TekstSredniLewoBezOdstepow)
    NIP = Swiadectwo.Paragraf(dane_oddzial[0]['nip'], TekstSredniLewoBezOdstepow)
    Nagl = Swiadectwo.Paragraf(u'<b>WIADECTWO PRACY</b>', TekstNagl)
    Tresc.extend([MiejscowoscData, Nazwa, Adres, MiastoKod, REGON, NIP, Nagl])

    # 1.
    ImieNazwisko = Swiadectwo.Paragraf(u'1. Stwierdza si, e <b>'+dane_1_pracownik[0]['imie']+' '+dane_1_pracownik[0]['nazwisko']+'</b>', TekstSredni)
    Urodzony = Swiadectwo.Paragraf(u'urodzony/a <b>'+dane_1_pracownik[0]['data_urodzenia']+'</b>', TekstSredni)
    #WS 2022-03-15
    Zatrudniony = Swiadectwo.Paragraf(u'by/a zatrudniony/a w <b>'+', '.join([dane_oddzial[0]['nazwa'], dane_oddzial[0]['adr_kod']+' '+dane_oddzial[0]['adr_miej'], dane_oddzial[0]['adr_ulica']+' '+dane_oddzial[0]['adr_nry']])+'</b>', TekstSredni)
    # Zatrudniony = Swiadectwo.Paragraf(u'by/a zatrudniony/a w <b>'+dane_1_pracodawca[0]['opis']+'</b>', TekstSredni)
    Tresc.extend([ImieNazwisko, Urodzony, Zatrudniony])

    OkresyWymiary = []
    for wiersz in dane_1_zatrudnienie:
        if wiersz['etat'] == 1:
            EtatTekst = u'penego etatu'
        else:
            etat_ulamek = fractions.Fraction.from_float(wiersz['etat']).limit_denominator()
            etat_licznik = etat_ulamek.numerator
            etat_mianownik = etat_ulamek.denominator
            EtatTekst = str(int(etat_licznik))+'/'+str(int(etat_mianownik))+u' etatu'
        #WS 2022-03-15 - w okresie od -> od dnia
        OkresyWymiary.append(Swiadectwo.Paragraf(u'&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;od dnia <b>'+wiersz['od_kiedy']+u'</b> do <b>'+wiersz['do_kiedy']+u'</b> w wymiarze <b>'+EtatTekst+'</b>', TekstSredni))
    Tresc.extend(OkresyWymiary)

    # 2.
    Tekst2 = Swiadectwo.Paragraf(u'2. W okresie zatrudnienia pracownik wykonywa prac tymczasow - nie dotyczy', TekstSredni)
    Tresc.append(Tekst2)

    # 3.
    Tekst3 = Swiadectwo.Paragraf(u'3. W okresie zatrudnienia pracownik wykonywa prac:', TekstSredni)
    Tresc.append(Tekst3)

    StanowiskaOkresy = []
    for wiersz in dane_3:
        #WS 2022-03-15 - w okresie od -> od dnia
        StanowiskaOkresy.append(Swiadectwo.Paragraf(u'&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;od dnia <b>'+wiersz['od_kiedy']+u'</b> do <b>'+wiersz['do_kiedy']+u'</b> na stanowisku <b>'+wiersz['opis']+'</b> ', TekstSredni))
    Tresc.extend(StanowiskaOkresy)

    # 4.
    Wynik = u'<b>wyganicia</b>' if dane_4[0]['przyczyna_wyrejestrowania_zus'] == '500' else u'<b>rozwizania</b>'
    # Przyczyna = ''
    Podstawa = ''
    # if dane_4[0]['przyczyna_rozw'] != 'None':
    #     Przyczyna = ' <b>'+dane_4[0]['przyczyna_rozw']+'</b> '
    if dane_4[0]['podstawa_rozw'] != 'None':
        Podstawa = ' - <b>'+dane_4[0]['podstawa_rozw']+'</b> '
    Tekst4 = Swiadectwo.Paragraf(u'4. Stosunek pracy usta w wyniku '+Wynik+u' umowy o prac'+Podstawa, TekstSredni)  #WS 2022-09-02 - usunita Przyczyna
    Tresc.append(Tekst4)

    # 5.
    #WS 2024-08-01 - usunity fragment "art. 36<sup>1</sup>  1 Kodeksu pracy"
    if len(Wypowiedzenie.strip()) == 0:
        Wypowiedzenie = 'nie dotyczy'
    else:
        Wypowiedzenie = '<b>'+Wypowiedzenie+'</b>'
    Tekst5 = Swiadectwo.Paragraf(u'5. Zosta zastosowany skrcony okres wypowiedzenia umowy o prac - '+Wypowiedzenie, TekstSredni)
    Tresc.append(Tekst5)

    # 6.
    Tekst6 = Swiadectwo.Paragraf(u'6. W okresie zatrudnienia pracownik: ', TekstSredni)
    
    # 6.1  #WS 2023-06-01
    if len(dane_6_1) > 0 and dane_6_1[0]['suma_dni'] > 0:
        Tekst61 = Swiadectwo.Paragraf(u'1) wykorzysta zwolnienie od pracy przewidziane w art. 148<sup>1</sup> Kodeksu Pracy w liczbie <b>'+str(int(dane_6_1[0]['suma_dni']))+u' dni</b>', TekstSredni)
    else:
        Tekst61 = Swiadectwo.Paragraf(u'1) wykorzysta zwolnienie od pracy przewidziane w art. 148<sup>1</sup> Kodeksu Pracy - nie dotyczy', TekstSredni)

    # 6.2
    WypoczynkowyWymiar = '0'
    NaZadanieWymiar = '0'
    if len(dane_6_2_wypoczynkowy) > 0:
        WypoczynkowyWymiar = str(int(dane_6_2_wypoczynkowy[0]['dni']))+' dni'
    if len(dane_6_2_na_zadanie) > 0:
        NaZadanieWymiar = str(int(dane_6_2_na_zadanie[0]['dni_rok']))
    Tekst62 = Swiadectwo.Paragraf(u'2) wykorzysta urlop wypoczynkowy w wymiarze <b>'+WypoczynkowyWymiar+u'</b> w tym <b>'+NaZadanieWymiar+u'</b> na podstawie art. 167<sup>2</sup> Kodeksu pracy', TekstSredni)

    # 6.3  #WS 2023-06-01
    OpiekunczyWymiar = '&nbsp;&nbsp;-&nbsp;&nbsp;'
    if len(dane_6_3) > 0 and dane_6_3[0]['suma_dni'] > 0:
        OpiekunczyWymiar = '<b>'+str(int(dane_6_3[0]['suma_dni']))+' dni</b>'
    Tekst63 = Swiadectwo.Paragraf(u'3) wykorzysta urlop opiekuczy w wymiarze '+OpiekunczyWymiar, TekstSredni)

    # 6.4
    BezplatnyOd = 'None'
    BezplatnyDo = 'None'
    if len(dane_6_4) > 0:
        BezplatnyOd = '<b>'+dane_6_4[0]['od_kiedy']+'</b>'
        BezplatnyDo = '<b>'+dane_6_4[0]['do_kiedy']+'</b>'
    if BezplatnyOd != 'None' and BezplatnyDo != 'None':
        Tekst64 = Swiadectwo.Paragraf(u'4) korzysta z urlopu bezpatnego w okresie od '+BezplatnyOd+' do '+BezplatnyDo+u' na podstawie art. 174 Kodeksu pracy', TekstSredni)
    else:
        Tekst64 = Swiadectwo.Paragraf(u'4) korzysta z urlopu bezpatnego - nie dotyczy', TekstSredni)

    # 6.5
    OjcowskiWymiar = '&nbsp;&nbsp;-&nbsp;&nbsp;'
    OjcowskiCzesci = '&nbsp;&nbsp;-&nbsp;&nbsp;'
    TekstCzesci = u'czciach'
    if len(dane_6_5) > 0 and dane_6_5[0]['suma_dni'] > 0:
        # if dane_6_5[0]['suma_dni'] < 14:  #WS 2022-03-18 - wyczenie limitu 14 dni
        OjcowskiWymiar = '<b>'+str(int(dane_6_5[0]['suma_dni']))+' dni</b>'
        OjcowskiCzesci = '<b>'+str(int(dane_6_5[0]['liczba_czesci']))+'</b>'
        if dane_6_5[0]['liczba_czesci'] == 1:
            TekstCzesci = u'czci'
    Tekst65 = Swiadectwo.Paragraf(u'5) wykorzysta urlop ojcowski w wymiarze '+OjcowskiWymiar+' w '+OjcowskiCzesci+' '+TekstCzesci, TekstSredni)

    # 6.6
    RodzicielskiWymiar = '&nbsp;&nbsp;-&nbsp;&nbsp;'
    RodzicielskiCzesci = '&nbsp;&nbsp;-&nbsp;&nbsp;'
    TekstCzesci = u'czciach'
    if len(dane_6_6) > 0 and dane_6_6[0]['suma_dni'] > 0:
        #WS 2022-07-25
        RodzicielskiWymiar = '<b>'+str(int(round(float(dane_6_6[0]['suma_dni'])/float(7), 0)))+' tygodni</b>'
        # RodzicielskiWymiar = '<b>'+str(int(dane_6_6[0]['suma_dni']))+' dni</b>'
        RodzicielskiCzesci = '<b>'+str(int(dane_6_6[0]['liczba_czesci']))+'</b>'
        if dane_6_6[0]['liczba_czesci'] == 1:
            TekstCzesci = u'czci'
    Tekst66 = Swiadectwo.Paragraf(u'6) wykorzysta urlop rodzicielski udzielony na podstawie art. 182<sup>1a</sup> Kodeksu pracy w wymiarze '+RodzicielskiWymiar+' w '+RodzicielskiCzesci+' '+TekstCzesci+u' w tym na podstawie art. 182<sup>1c</sup>  3 Kodeksu pracy w&nbsp;&nbsp;-&nbsp;&nbsp;czciach', TekstSredni)

    # 6.7
    WychowawczyWymiar = '&nbsp;&nbsp;-&nbsp;&nbsp;'
    WychowawczyCzesci = '&nbsp;&nbsp;-&nbsp;&nbsp;'
    WychowawczyOkresy = '&nbsp;&nbsp;-&nbsp;&nbsp;'
    TekstOkresy = 'okresie'
    TekstCzesci = u'czciach'
    if len(dane_6_7) > 0 and dane_6_7[0]['suma_dni'] > 0:
        WychowawczyWymiar = '<b>'+str(int(dane_6_7[0]['suma_dni']))+' dni</b>'
        WychowawczyCzesci = '<b>'+str(int(dane_6_7[0]['liczba_czesci']))+'</b>'
        WychowawczyOkresy = ''
        for okres in dane_6_7:
            WychowawczyOkresy += ' od <b>'+okres['od_kiedy']+'</b> do <b>'+okres['do_kiedy']+'</b>, '
        WychowawczyOkresy = WychowawczyOkresy.strip(', ')
        if len(dane_6_7) > 1:
            TekstOkresy = 'okresach'
        if dane_6_7[0]['liczba_czesci'] == 1:
            TekstCzesci = u'czci'
    #WS 2022-07-25
    Tekst67 = Swiadectwo.Paragraf(u'7) wykorzysta urlop wychowawczy udzielony na podstawie art. 186 Kodeksu pracy w '+TekstOkresy+' '+WychowawczyOkresy+' w '+WychowawczyCzesci+' '+TekstCzesci, TekstSredni)
    # Tekst67 = Swiadectwo.Paragraf(u'5) wykorzysta urlop wychowawczy udzielony na podstawie art. 186 Kodeksu pracy w wymiarze '+WychowawczyWymiar+' w '+TekstOkresy+' '+WychowawczyOkresy+' w '+WychowawczyCzesci+' '+TekstCzesci, TekstSredni)

    # 6.8
    if CzasPracy:
        OchronaTekst = 'tak'
    else:
        OchronaTekst = 'nie'
    Tekst68 = Swiadectwo.Paragraf(u'8) korzysta z ochrony stosunku pracy, o ktrej mowa w art. 186<sup>8</sup>  1 pkt 2 Kodeksu pracy: <b>'+OchronaTekst+'</b>', TekstSredni)

    # 6.9
    if len(dane_6_9) > 0 and dane_6_9[0]['suma_dni'] > 0:
        Tekst69 = Swiadectwo.Paragraf(u'9) wykorzysta zwolnienie od pracy przewidziane w art. 188 Kodeksu pracy w liczbie <b>'+str(int(dane_6_7[0]['suma_dni']))+' dni</b>', TekstSredni)
    else:
        Tekst69 = Swiadectwo.Paragraf(u'9) wykorzysta zwolnienie od pracy przewidziane w art. 188 Kodeksu pracy - nie dotyczy', TekstSredni)

    # 6.10  #WS 2023-06-01
    Tekst610 = Swiadectwo.Paragraf(u'10) wykonywa prac zdaln przewidzian w art. 67<sup>33</sup> Kodeksu pracy w liczbie 0 dni', TekstSredni)

    # 6.11
    #WS 2022-03-15 - wypisane okresy
    NiezdolnyDni = '&nbsp;&nbsp;-&nbsp;&nbsp;'
    NiezdolnyOkresy = []
    if len(dane_6_11) > 0 and dane_6_11[0]['suma_dni'] > 0:
        NiezdolnyDni = '<b>'+str(int(dane_6_11[0]['suma_dni']))+' dni</b>: '
        for okres in dane_6_11:
            NiezdolnyOkresy.append(u'&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;od dnia <b>'+okres['od_kiedy']+u'</b> do <b>'+okres['do_kiedy']+u'</b>')
    Tekst611 = Swiadectwo.Paragraf(u'11) by niezdolny do pracy przez okres '+NiezdolnyDni, TekstSredni)
    Tekst611Okresy = []
    for okres in NiezdolnyOkresy:
        Tekst611Okresy.append(Swiadectwo.Paragraf(okres, TekstSredni))

    # 6.12
    Dni2003Tekst = 'okresie'
    Dni2003Okresy = ''
    if len(dane_6_12) > 0:
        if len(dane_6_12) > 1:
            Dni2003Tekst = 'okresach'
        for okres in dane_6_12:
            Dni2003Okresy += ' od <b>'+okres['od_kiedy']+'</b> do <b>'+okres['do_kiedy']+'</b>, '
        Dni2003Okresy = Dni2003Okresy.strip(', ')
        Tekst612 = Swiadectwo.Paragraf(u'12) nie zachowa prawa do wynagrodzenia w '+Dni2003Tekst+' '+Dni2003Okresy, TekstSredni)
    else:
        Tekst612 = Swiadectwo.Paragraf(u'12) nie zachowa prawa do wynagrodzenia - nie dotyczy', TekstSredni)

    # 6.13
    if rsglobal.SprawdzCzyToData(DataWojskoOd) and rsglobal.SprawdzCzyToData(DataWojskoDo):
        Tekst613 = Swiadectwo.Paragraf(u'13) odby sub wojskow w okresie od <b>'+DataWojskoOd+'</b> do <b>'+DataWojskoDo+'</b>', TekstSredni)
    else:
        Tekst613 = Swiadectwo.Paragraf(u'13) odby sub wojskow - nie dotyczy', TekstSredni)

    # 6.14
    Tekst614 = Swiadectwo.Paragraf(u'14) wykonywa prac w szczeglnych warunkach lub w szczeglnym charakterze - nie dotyczy', TekstSredni)

    # 6.15
    if len(InfoDodatkowe) > 0:
        DodatkoweTekst = '<b>'+InfoDodatkowe+'</b>'
    else:
        DodatkoweTekst = '- nie dotyczy'
    Tekst615 = Swiadectwo.Paragraf(u'15) wykorzysta dodatkowy urlop albo inne uprawnienia lub wiadczenia przewidziane przepisami prawa pracy '+DodatkoweTekst, TekstSredni)

    # 6.16
    #WS 2022-03-15 - wypisane okresy i typ absencji
    NieskladkoweOkresy = []
    Puste = '-'
    for okres in dane_6_16:
        Puste = ''
        NieskladkoweOkresy.append(u'&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;od dnia <b>'+okres['od_kiedy']+u'</b> do <b>'+okres['do_kiedy']+u'</b> - <b>'+okres['nazwa']+'</b>')
    Tekst616 = Swiadectwo.Paragraf(u'16) okresy nieskadkowe, przypadajce w okresie zatrudnienia wskazanym w ust. 1, uwzgldniane przy ustalaniu prawa do emerytury lub renty: '+Puste, TekstSredni)
    Tekst616Okresy = []
    for okres in NieskladkoweOkresy:
        Tekst616Okresy.append(Swiadectwo.Paragraf(okres, TekstSredni))

    Tresc.extend([Tekst6, Tekst61, Tekst62, Tekst63, Tekst64, Tekst65, Tekst66, Tekst67, Tekst68, Tekst69, Tekst610, Tekst611])
    Tresc.extend(Tekst611Okresy)
    Tresc.extend([Tekst612, Tekst613, Tekst614, Tekst615, Tekst616])
    Tresc.extend(Tekst616Okresy)

    # 7
    KomorniczeTekst = '&nbsp;&nbsp;-&nbsp;&nbsp;'
    if len(ZajKomornicze) > 0:
        KomorniczeTekst = ZajKomornicze
    Tekst7 = Swiadectwo.Paragraf(u'7. Informacja o zajciu wynagrodzenia: '+KomorniczeTekst, TekstSredni)
    Tresc.append(Tekst7)

    TekstPodpisSredniPrawo = Swiadectwo.DodajStylParagrafu(nazwa='TekstPodpisSredniPrawo', odstep_przed=60, wyrownanie=Swiadectwo.RIGHT)
    TekstPodpisMalyPrawo = Swiadectwo.DodajStylParagrafu(nazwa='TekstPodpisMalyPrawo', rozmiar=6, odstep_za=40, wyrownanie=Swiadectwo.RIGHT)

    # 8
    Tekst8 = Swiadectwo.Paragraf(u'8. Informacje uzupeniajce: ', TekstSredni)
    Tekst81 = Swiadectwo.Paragraf(u'................................................................................................................................................................................', TekstSredni)
    Tekst82 = Swiadectwo.Paragraf(u'................................................................................................................................................................................', TekstSredni)
    Tresc.extend([Tekst8, Tekst81, Tekst82])

    # podpis
    Podpis = Swiadectwo.Paragraf(u'.......................................................................', TekstPodpisSredniPrawo)
    PodpisOpis = Swiadectwo.Paragraf(u'(piecz i podpis pracodawcy lub osoby dziaajcej w jego imieniu)', TekstPodpisMalyPrawo)
    Tresc.extend([Podpis, PodpisOpis])

    TekstMalySrodek = Swiadectwo.DodajStylParagrafu(nazwa='TekstMalySrodek', rozmiar=7, odstep_przed=12, odstep_za=12, wyrownanie=Swiadectwo.CENTER)
    TekstMaly = Swiadectwo.DodajStylParagrafu(nazwa='TekstMaly', rozmiar=7, odstep_przed=12, odstep_za=12)

    # pouczenie
    PouczenieNagl = Swiadectwo.Paragraf(u'POUCZENIE', TekstMalySrodek)
    Pouczenie = Swiadectwo.Paragraf(u'Pracownik moe w cigu 14 dni od otrzymania wiadectwa pracy wystpi z wnioskiem do pracodawcy o sprostowanie wiadectwa pracy. W razie nieuwzgldnienia wniosku pracownikowi przysuguje, w cigu 14 dni od zawiadomienia o odmowie sprostowania wiadectwa pracy, prawo wystpienia z daniem jego sprostowania do sdu pracy. W przypadku niezawiadomienia przez pracodawc o odmowie sprostowania wiadectwa pracy, danie sprostowania wiadectwa pracy wnosi si do sdu pracy.', TekstMaly)
    PodstPrawna = Swiadectwo.Paragraf(u'(podstawa prawna - art. 97  2<sup>1</sup> Kodeksu pracy)', TekstMalySrodek)
    Tresc.extend([PouczenieNagl, Pouczenie, PodstPrawna])

    nazwa_pliku = 'swiadectwo_pracy_'+Pracownik+'.'+Format
    if Format == 'docx':
        Swiadectwo.ZrobDok(self.firma_id, nazwa_pliku, 1.8, 1.3, 1.6, 1.6)
    else:
        #AB 13.07.22r dodana zwrotka z ZrobDok
        stat = Swiadectwo.ZrobDok(self.firma_id, nazwa_pliku, Tresc)
        if stat != 'OK':
            stat = "Wystpi problem podczas generowania pliku w formacie PDF"
    if stat =='OK':
        tresc = fs_pliki_global.PodajTrescPliku(nazwa_pliku)
        dane = [tresc, nazwa_pliku]
