# -*- coding: iso8859_2 -*-
#-------------------------------------------------------------------------------
# Name:        fs_zamowienia_odbiorcy_import_z_netto.py
# Purpose:     import z pliku xlsx(netto)
# Author:      Aleksander Bargie
# Created:     2022/01/19
# RCS-ID:      $Id: fs_zamowienia_odbiorcy_import_z_netto.py
# Copyright:   (c) 2022 Record System
# Licence:     GNU GPL
#-------------------------------------------------------------------------------
import binascii
from datetime import datetime

import openpyxl
import tempfile
import rsglobal
import rssql

dane = []
wyn1 = []
wyn2 = []
stat = 'OK'

if 'Funkcja' not in klucze:
    stat = 'Brak podanej funkcji w kluczach'

if stat == 'OK':
    Fun = klucze['Funkcja']

    SlPar = {'TrescPliku': [3, 4], 'Typ': [3, 4]}
    Lista = rsglobal.SprawdzParametryWKluczach(SlPar, Fun, klucze)
    if Lista != '':
        import os
        stat = 'Uwaga : wywoanie funkcji '+os.path.basename(__file__)+' z numerem '+str(Fun)+' bez parametrw: '+Lista

if stat == 'OK':
    TrescPliku = binascii.unhexlify(klucze['TrescPliku'])
    Typ        = klucze['Typ']
    if Fun == 3:

        try:
            SciezkaPlikuRoboczego = tempfile.mktemp(prefix='netto', suffix='.xlsx')
            with open(SciezkaPlikuRoboczego, "wb") as plik:
                plik.write(TrescPliku)
            wb = openpyxl.load_workbook(SciezkaPlikuRoboczego)
            import os
            os.remove(SciezkaPlikuRoboczego)
        except Exception, e:
            stat = "Bd zapisu pliku tymczasowego."
            import traceback
            rsglobal.ZapiszDoLogow(traceback.format_exc())
        pyxlver = [int(x) for x in openpyxl.__version__.split('.')]
        data_zam = ('DATA:', (1, 0))
        data_dost = ('DATA DOSTAWY:', (1, 0))
        sklep = ('SKLEP:', (1, 0))
        tabela = 'Nr SAP'
        rzut1 = 'I'
        rzut2 = 'II'
        koniec_tab = u'Zamwienie sporzdzi:'.encode('utf-8') if pyxlver[0] > 2 or (
                    pyxlver[0] == 2 and pyxlver[1] >= 1) else u'Zamwienie sporzdzi:'
        puste = ('', '    -  -  ', None, 'None', 'NULL')
        z = False


class Zamowienie(object):
    def __init__(self, Pol, firma_id, typ_zamowienia, loguj=None, uwzgledniac_rz2=True):
        """Klasa pomocnicza do obslugi pojedynczego zamowienia
          """
        self.uwzgledniac_rz2  = uwzgledniac_rz2
        self.oddzial_id       = ''
        self.rzut1            = {}
        self.rzut2            = {}
        self.klucz            = {}
        self.kontrahent       = None
        self.platnik          = None
        self.data             = None
        self.Pol              = Pol
        self.firma_id         = firma_id
        self._loguj           = loguj
        self._obce_art        = {}
        self._pozycje_zam_rz1 = {}
        self._pozycje_zam_rz2 = {}
        self.zaimportowane    = 0
        self.pominiete        = set()
        self.nie_dodawac1     = set()
        self.nie_dodawac2     = set()
        self.typ_zamowienia   = typ_zamowienia
        self.loguj("Tworz zamwienie typ {}".format(typ_zamowienia))

    def __str__(self):
        out = u'''Zamwienie {}
                  kontrahent {}
                  patnik {}
                  data {}
                  firma {}
                  typ {}
                  rzut1 {}
                  rzut2 {}
        '''.format(self.klucz,
                   self.kontrahent,
                   self.platnik,
                   self.data,
                   self.firma_id,
                   self.typ_zamowienia,
                   self.rzut1,
                   self.rzut2)
        return out
    @property
    def _zamowienie(self):
        out = "Symbol obcy {}".format(self.symbol_obcy)
        if self.kontrahent:
            out += ", kontrahent {}".format(self.kontrahent)

        if self.klucz:
            out += ", zamwienie {}/{}/{}".format(self.klucz['wyznacznik_id'], self.klucz['data'], int(self.klucz['numer']))
        return out + ': '

    def ustaw_kontrahenta(self, kontrahent):
        self.loguj("Ustawiam kontrahenta {}".format(kontrahent))
        self.symbol_obcy = str(kontrahent)
        args = [self.firma_id, str(self.symbol_obcy), str(self.symbol_obcy), str(self.symbol_obcy)]
        SQL = """SELECT kontrahent_id, platnik_id, symbol_obcy
                 FROM rs_kontrahenci
                 WHERE firma_id = %s AND (symbol_obcy = '.' || %s OR symbol_obcy = %s) AND aktywny
                 ORDER BY symbol_obcy = '.' || %s  DESC 
                 ;"""

        self.loguj(self.Pol.Mogrify(SQL, args))
        stat, kontr, dump, dump = self.Pol.Execute(SQL, 3, args, CzyDodawacBlad=True)
        if stat != 'OK':
            return stat
        if len(kontr) <1:
            return '{}nie odnaleziono kontrahenta'.format(self._zamowienie)
        elif len(kontr) > 1 and not kontr[0][2].startswith('.'):
            return '{}znaleziono wielu kontrahentw'.format(self._zamowienie)
        else:
            self.kontrahent = str(kontr[0][0])
            self.platnik    = str(kontr[0][1])
            self.loguj("Uzupeniam patnika {}, kontrahenta {}.".format(repr(self.platnik), repr(self.kontrahent)))
            return stat

    def uzupelnij_obce_artykuly(self):
        self._obce_art = {}
        SQL = """SELECT artykul_id, obcy_id 
                 FROM rs_artykuly_obce o 
                 WHERE firma_id=%s AND kontrahent_id=%s AND obcy_id IS NOT NULL AND obcy_id != ''
                 ;"""
        args = [self.firma_id, self.platnik]
        self.loguj(self.Pol.Mogrify(SQL, args))
        stat, art, wyn1, dump = self.Pol.Execute(SQL, 3, args, CzyDodawacBlad=True)
        if stat != 'OK':
            return stat
        tr_art = rssql.RSTableRowset(art, wyn1)
        for i in xrange(len(art)):
            self._obce_art[tr_art.GetVal(i, 'obcy_id')] = tr_art.GetVal(i, 'artykul_id')
        if self.kontrahent != self.platnik:
            args = [self.firma_id, str(self.kontrahent)]
            self.loguj(self.Pol.Mogrify(SQL, args))
            stat, art, wyn1, dump = self.Pol.Execute(SQL, 3, args, CzyDodawacBlad=True)
            if stat != 'OK':
                return "{}problem podczas pobierania artykuw obcych: {}".format(self._zamowienie, stat)
            tr_art = rssql.RSTableRowset(art, wyn1)
            for i in xrange(len(art)):
                self._obce_art[tr_art.GetVal(i, 'obcy_id')] = tr_art.GetVal(i, 'artykul_id')
        return stat

    def loguj(self, tekst, poziom=5):
        if self._loguj is not None:
            self._loguj(tekst, poziom)

    def ustaw_date(self, data):
        self.loguj("Ustawiam dat {}".format(data))
        self.data = str(data)
        self._dzien = data.weekday()+2
        if self._dzien == 8:
            self._dzien = 1
        self.pole_dzien = 'dz_0'+str(self._dzien)
        self.loguj("Dzie {}".format(self.pole_dzien))

    def dodaj_na_rzut(self, art, ilosc, rzut=1):
        if ilosc in (None, 'None', '0', '0.0', 0, 0.0):
            self.loguj("{} rzut {} w iloci 0, pomijamy".format(art, rzut))
            return 'OK'
        if isinstance(ilosc, basestring):
            ilosc = ilosc.replace(",", "")
        self.loguj("Dodaj {} w iloci {} na rzut {}".format(art, ilosc, rzut))
        if '.'+art in self._obce_art:
            art = '.' + art
        elif art not in self._obce_art:
            stat = '{}artykuu {} nie odnaleziono w artykuach obcych'
            stat = stat.format(self._zamowienie, art)
            self.loguj(stat)
            self.loguj(self._obce_art)
            return stat
        if rzut == 1:
            self.rzut1[self._obce_art[art]] = int(ilosc)
        elif rzut == 2:
            self.rzut2[self._obce_art[art]] = int(ilosc)
        return 'OK'

    def znajdz_zamowienie(self):
        self.loguj("Szukam zamwienia")
        SQL = """SELECT wyznacznik_id, data, numer, oddzial_id
                 FROM rs_zamowienia_odbiorcy
                 WHERE firma_id=%s AND kontrahent_id=%s AND typ_zamowienia = %s AND stan ='W' AND czy_stale
                       AND dostawy_od_dnia <= %s AND (dostawy_do_dnia IS NULL or dostawy_do_dnia >= %s)
                 ;"""
        args = [self.firma_id, str(self.kontrahent), str(self.typ_zamowienia), self.data, self.data]
        self.loguj(self.Pol.Mogrify(SQL, args))
        stat, zam, wyn1, dump = self.Pol.Execute(SQL, 3, args, CzyDodawacBlad=True)
        if stat != 'OK':
            return stat
        if len(zam) < 1:
            return '{}nie odnaleziono zamwienia'.format(self._zamowienie)
        elif len(zam) > 1:
            return '{}znaleziono wiele zamwie'.format(self._zamowienie)
        else:
            tr_zam = rssql.RSTableRowset(zam, wyn1)
            self.klucz['wyznacznik_id'] = tr_zam.GetVal(0, 'wyznacznik_id')
            self.klucz['data']          = tr_zam.GetVal(0, 'data')
            self.klucz['numer']         = tr_zam.GetVal(0, 'numer')
            self.oddzial_id             = tr_zam.GetVal(0, 'oddzial_id')
        return stat

    def pobierz_pozycje(self):
        bledy = set()
        self.loguj("Pobieram pozycje")
        SQL = """SELECT td.rzut, zo.wyznacznik_id,zo.data,zo.numer,zo.oddzial_id, artykul_id, tryb_dostawy, data_od
                 FROM rs_zamowienia_odbiorcy zo
                   INNER JOIN rs_zamowienia_odbiorcy_pozycje_stale zop USING (firma_id,wyznacznik_id,data,numer)
                   INNER JOIN rs_tryby_dostaw td ON (td.firma_id=zop.firma_id AND td.kod=zop.tryb_dostawy)
                 WHERE zo.firma_id = %s AND zo.wyznacznik_id = %s AND zo.data = %s AND zo.numer = %s
                 AND data_od <= %s AND (data_do IS NULL or data_do >= %s)
                 ;"""
        args = [self.firma_id, self.klucz['wyznacznik_id'], self.klucz['data'], str(self.klucz['numer']), self.data,
                self.data]
        self.loguj(self.Pol.Mogrify(SQL, args))
        stat, poz, wyn1, dump = self.Pol.Execute(SQL, 3, args, CzyDodawacBlad=True)
        if stat != 'OK':
            return "{}bd pobrania pozycji zamwienia".format(self._zamowienie), []
        elif len(poz) == 0:
            return "{}bd pobrania pozycji zamwienia, nie udao si odnale pozycji staych".format(self._zamowienie), []
        else:
            self.liczba_pozycji_w_anakondzie = len(poz)
            tr_zam          = rssql.RSTableRowset(poz, wyn1)
            oddzial_id      = self.oddzial_id
            self.loguj("Przed pobraniem pozycji: {}".format(str(self)))
            for i in xrange(len(poz)):
                rzut         = tr_zam.GetVal(i, 'rzut')
                data_od      = tr_zam.GetVal(i, 'data_od')
                artykul_id   = tr_zam.GetVal(i, 'artykul_id')
                tryb_dostawy = tr_zam.GetVal(i, 'tryb_dostawy')
                if rzut == '1':
                    if artykul_id in self._pozycje_zam_rz1:
                        self.loguj("Art {} juz byl w self._pozycje_zam_rz1 ({})".format(artykul_id, self._pozycje_zam_rz1))
                        if artykul_id not in self.nie_dodawac1:
                            self.nie_dodawac1.add(artykul_id)
                            bledy.add("{}rzut 1 dla artykuu {} znaleziono wicej ni 1 pozycj, iloci nie zostay zaimportowane".format(self._zamowienie, artykul_id.strip()))
                    else:
                        self.loguj("Dodaje art {} do self._pozycje_zam_rz1".format(artykul_id))
                        self._pozycje_zam_rz1[artykul_id] = {'oddzial_id': oddzial_id, 'tryb_dostawy': tryb_dostawy,
                                                             'data_od': data_od}
                elif rzut == '2':
                    if artykul_id in self._pozycje_zam_rz2:
                        self.loguj(self._pozycje_zam_rz2)
                        if artykul_id not in self.nie_dodawac2:
                            self.nie_dodawac2.add(artykul_id)
                            bledy.add("{}rzut 2 dla artykuu {} znaleziono wicej ni 1 pozycj, iloci nie zostay zaimportowane".format(self._zamowienie, artykul_id.strip()))
                    else:
                        self._pozycje_zam_rz2[artykul_id] = {'oddzial_id': oddzial_id, 'tryb_dostawy': tryb_dostawy,
                                                             'data_od': data_od}
                for art in self.nie_dodawac1:
                    self._pozycje_zam_rz1.pop(art, None)
                for art in self.nie_dodawac2:
                    self._pozycje_zam_rz2.pop(art, None)
            self.tryby_rz1 = set()
            self.tryby_rz2 = set()
            for x in self._pozycje_zam_rz1.values():
                self.tryby_rz1.add(x['tryb_dostawy'])
            for x in self._pozycje_zam_rz2.values():
                self.tryby_rz2.add(x['tryb_dostawy'])
            if stat == 'OK' and len(self.tryby_rz1) > 1:
                bledy = set(["{}dla rzutu 1 znaleziono wicej ni 1 tryb dostawy, iloci z tego rzutu nie zostay pobrane".format(self._zamowienie)])
                self._pozycje_zam_rz1 = {}
            if stat == 'OK' and len(self.tryby_rz2) > 1:
                bledy = set(["{}dla rzutu 2 znaleziono wicej ni 1 tryb dostawy, iloci z tego rzutu nie zostay pobrane.".format(self._zamowienie)])
                self._pozycje_zam_rz2 = {}

            return stat, bledy

    def zapisz_ilosci_w_zamowieniach(self):
        sql_wzor = """UPDATE rs_zamowienia_odbiorcy_pozycje_stale
                      SET {}=%s
                      WHERE firma_id=%s AND wyznacznik_id=%s AND data=%s AND numer=%s AND tryb_dostawy=%s 
                        AND artykul_id=%s AND data_od=%s;
                      """.format(self.pole_dzien)
        zapytanie = ''
        argumenty = []
        #zerowanie danego dnia na zamwieniu na start
        zapytanie += """UPDATE rs_zamowienia_odbiorcy_pozycje_stale
                      SET {}=0
                      WHERE firma_id=%s AND wyznacznik_id=%s AND data=%s AND numer=%s AND data_od<=%s AND (data_do IS NULL or data_do >= %s);
                      """.format(self.pole_dzien)
        argumenty += [self.firma_id, self.klucz['wyznacznik_id'], self.klucz['data'], self.klucz['numer'], self.data, self.data]
        bledy = set()
        for art in self._pozycje_zam_rz1:
            if art in self.rzut1:
                wart = self._pozycje_zam_rz1[art]
                zapytanie += sql_wzor
                ilosc = self.rzut1[art]
                self.zaimportowane += 1
                argumenty += [ilosc, self.firma_id, self.klucz['wyznacznik_id'], self.klucz['data'], self.klucz['numer'],
                              wart['tryb_dostawy'], art, wart['data_od']]
        for art in self._pozycje_zam_rz2:
            if art in self.rzut2:
                wart = self._pozycje_zam_rz2[art]
                zapytanie += sql_wzor
                ilosc = self.rzut2[art]
                self.zaimportowane += 1
                argumenty += [ilosc, self.firma_id, self.klucz['wyznacznik_id'], self.klucz['data'], self.klucz['numer'],
                              wart['tryb_dostawy'], art, wart['data_od']]

        if len(self._pozycje_zam_rz1):
            for art in self.rzut1:
                if art not in self._pozycje_zam_rz1 and self.rzut1[art] != 0:
                    self.loguj("Pomijam {} bo {} {}".format(art, self.rzut1[art] != 0, self.rzut1[art]))
                    self.pominiete.add(art)
                    if art not in self.nie_dodawac1:  #mamy ju wczeniej dodany bd
                        bledy.add("{}artyku {} rzut 1 nie wystpuje na pozycjach zamwienia, nie zosta zaimportowany".format(self._zamowienie, art.strip()))
        elif len(self.tryby_rz1) <= 1:
            bledy.add("{} dla rzutu 1 nie znaleziono adnej pozycji dla ktrej mona zaimportowa iloci".format(self._zamowienie))

        if len(self._pozycje_zam_rz2):
            for art in self.rzut2:
                if art not in self._pozycje_zam_rz2 and self.rzut2[art] != 0:
                    self.loguj("Pomijam {} bo {} {}".format(art, self.rzut2[art] != 0, self.rzut2[art]))
                    self.pominiete.add(art)
                    if art not in self.nie_dodawac2:#mamy ju wczeniej dodany bd
                        bledy.add("{}artyku {} rzut 2 nie wystpuje na pozycjach zamwienia, nie zosta zaimportowany".format(self._zamowienie, art.strip()))
        elif self.uwzgledniac_rz2 and len(self.tryby_rz2) <= 1:
            bledy.add("{} dla rzutu 2 nie znaleziono adnej pozycji dla ktrej mona zaimportowa iloci".format(self._zamowienie))

        if zapytanie:
            self.loguj(self.Pol.Mogrify(zapytanie, argumenty))
            stat, dump, dump, dump = self.Pol.Execute(zapytanie, 1, argumenty, CzyDodawacBlad=True)
            if stat != 'OK':
                return "{}bd aktualizacji pozycji zamwienia".format(self._zamowienie), []
        else:
            self.loguj('Nic nie bdzie zapisane')
        bledy.add("{} pozycje z zaimportowanymi ilociami: {}, pozycje z wyzerowanymi ilociami: {}, pominite pozycje z zamwienia: {}"
                     .format(self._zamowienie, self.zaimportowane, self.liczba_pozycji_w_anakondzie - self.zaimportowane, len(self.pominiete)))

        return 'OK', bledy


if stat == 'OK' and Fun == 3:
    rsglobal.ZapiszDoLogow("Odczytuj plik tymczasowy {}".format(SciezkaPlikuRoboczego))
    if pyxlver[0] > 2 or (pyxlver[0] == 2 and pyxlver[1] >= 1):
        sheet = wb.active
    else:
        sheet = wb.get_active_sheet()
    # wiersze numerowane od 1, zgodnie z tym co wida w Excelu, kolumny od 0(0-A, 2-B itd.)
    Zamowienia = []
    tmp_zam = None
    tmp_data_zam = None
    tmp_data_dost = None
    tmp_sklep = None
    kol_art = None
    kol_rz1 = None
    kol_rz2 = None
    bledy_oddzialowe = dict()
    if pyxlver[0] > 2 or (pyxlver[0] == 2 and pyxlver[1] >= 1):
        iterator_wiersze = sheet.iter_rows()
    else:
        iterator_wiersze = iter(sheet.rows)
    for row in iterator_wiersze:
        wiersz = row[0].row if pyxlver[0] > 2 or (pyxlver[0] == 2 and pyxlver[1] >= 1) else sheet.rows.index(row)
        # w kadym wierszu moe by tylko 1 artyku, czyszczenie zmiennych za kadym razem przed obrbk
        tmp_art = None
        tmp_rz1 = None
        tmp_rz2 = None
        for cell in row:
            kol = cell.col_idx if pyxlver[0] > 2 or (pyxlver[0] == 2 and pyxlver[1] >= 1) else row.index(cell)
            if tmp_zam is None:  #nie utworzylimy zamwienia -  zbieramy dane z nad tabeli z artykuami
                if tmp_data_dost is None and cell.value == data_dost[0]:  # pobranie daty dostawy
                    rsglobal.ZapiszDoLogow("tmp_data_dost is None and cell.value == data_dost[0]:")
                    row_id = wiersz+data_dost[1][0]
                    col = kol+data_dost[1][1]
                    rsglobal.ZapiszDoLogow("uzupelniam {} {} = {}".format(row_id, col, sheet.cell(row=row_id, column=col).value))
                    tmp_data_dost = sheet.cell(row=row_id, column=col).value
                elif tmp_sklep is None and cell.value == sklep[0]:  # pobranie obcego symbolu sklepu
                    rsglobal.ZapiszDoLogow("tmp_sklep is None and cell.value == sklep[0]:")
                    row_id = wiersz+sklep[1][0]
                    col = kol+sklep[1][1]
                    rsglobal.ZapiszDoLogow("uzupelniam {} {} = {}".format(row_id, col, sheet.cell(row=row_id, column=col).value))
                    tmp_sklep = sheet.cell(row=row_id, column=col).value
                elif tmp_data_zam is None and cell.value == data_zam[0]:  # pobranie daty zamwienia
                    rsglobal.ZapiszDoLogow("tmp_data_zam is None and cell.value == data_zam[0]:")
                    row_id = wiersz+data_zam[1][0]
                    col = kol+data_zam[1][1]
                    rsglobal.ZapiszDoLogow("uzupelniam {} {} = {}".format(row_id, col, sheet.cell(row=row_id, column=col).value))
                    tmp_data_zam = sheet.cell(row=row_id, column=col).value
                elif str(cell.value).strip() == tabela:  # pocztek tabeli z zamwionymi ilociami
                    rsglobal.ZapiszDoLogow("cell.value == tabela")
                    # jeli nie pobrano wszystkich informacji z zamwienia a jestemy ju w tabeli z pozycjami, to znaczy e co nie wyszo
                    if tmp_data_dost is None or tmp_data_zam is None or tmp_sklep is None:
                        rsglobal.ZapiszDoLogow("tmp_data_dost is None or tmp_data_zam is None or tmp_sklep is None:")
                        stat == 'Prba wczytania zamwienia bez penych danych {}|{}, {}, {}, {}'.format(wiersz, kol, tmp_data_dost, tmp_data_zam, tmp_sklep)
                    else:
                        rsglobal.ZapiszDoLogow("tworzenie zamowienia")
                        kol_art = kol  # od razu uzupeniamy numer kolumny z symbolem artykuu
                        tmp_zam = Zamowienie(self.PolSQL, self.firma_id, Typ, rsglobal.ZapiszDoLogow)
                        rsglobal.ZapiszDoLogow(str(tmp_zam))
                        tmp_zam.ustaw_date(tmp_data_dost)
                        if stat == 'OK':
                            stat = tmp_zam.ustaw_kontrahenta(tmp_sklep)
                        if stat == 'OK':
                            stat = tmp_zam.znajdz_zamowienie()
                        if stat == 'OK':
                            stat = tmp_zam.uzupelnij_obce_artykuly()
                        if stat != 'OK':
                            if '**' not in bledy_oddzialowe:
                                bledy_oddzialowe['**'] = set()
                            bledy_oddzialowe['**'].add("Nie zaimportowano zamwienia sklepu {}: {}".format(tmp_sklep, stat))
                            tmp_zam = None
                            tmp_sklep = None
                            tmp_data_dost = None
                            tmp_data_zam = None
                            kol_art = None
                            kol_rz1 = None
                            kol_rz2 = None
                            stat = 'OK'
                            continue
            else:  # mamy zamwienie, uzupeniamy z niego iloci
                # zebralimy ju info w ktrej kolumnie jest symbol art i ilo dla rzutu I i II
                rsglobal.ZapiszDoLogow("{} {} = {}  | {}".format(wiersz, kol, repr(cell.value), cell.value == koniec_tab))
                if not (kol_art is None or kol_rz1 is None or kol_rz2 is None):
                    if cell.value == koniec_tab:  # tabela artykuw do importu si skoczya, czycimy zmienne z
                                                  # informacjami o zamwieniu aby zacz zbieranie iformacji o nowym
                        rsglobal.ZapiszDoLogow("Konczenie zamowienia")
                        Zamowienia.append(tmp_zam)
                        tmp_zam = None
                        tmp_sklep = None
                        tmp_data_dost = None
                        tmp_data_zam = None
                        kol_art = None
                        kol_rz1 = None
                        kol_rz2 = None
                    elif kol == kol_art and cell.value not in puste:  # pobranie obcego symbolu artykuu
                        tmp_art = str(cell.value)
                    elif kol == kol_rz1 and cell.value not in puste and tmp_art not in puste:  # jeli jest artyku i ilo dla rzutu 1, dodanie do danych do importu
                        tmp_rz1 = cell.value
                        stat = tmp_zam.dodaj_na_rzut(tmp_art, tmp_rz1)
                        if stat != 'OK':
                            if tmp_zam.oddzial_id not in bledy_oddzialowe:
                                bledy_oddzialowe[tmp_zam.oddzial_id] = set()
                            bledy_oddzialowe[tmp_zam.oddzial_id].add(stat)
                            stat = 'OK'
                    elif kol == kol_rz2 and cell.value not in puste and tmp_art not in puste:  # to samo w rzucie 2
                        tmp_rz2 = cell.value
                        stat = tmp_zam.dodaj_na_rzut(tmp_art, tmp_rz2, 2)
                        if stat != 'OK':
                            if tmp_zam.oddzial_id not in bledy_oddzialowe:
                                bledy_oddzialowe[tmp_zam.oddzial_id] = set()
                            bledy_oddzialowe[tmp_zam.oddzial_id].add(stat)
                            stat = 'OK'
                    else:
                        continue
                elif cell.value == rzut1:  # pobranie numeru kolumny dla rzutu 1
                    kol_rz1 = kol
                elif cell.value == rzut2:  # pobranie numeru kolumny dla rzutu 2
                    kol_rz2 = kol

            if stat != 'OK':
                rsglobal.ZapiszDoLogow(stat)
                break
        if stat != 'OK':
            rsglobal.ZapiszDoLogow(stat)
            break

if stat == 'OK' and Fun == 4:
    Zamowienia       = []
    tmp_zam          = None
    tmp_data_zam     = None
    tmp_data_dost    = None
    tmp_sklep        = None
    bledy_oddzialowe = dict()
    nowy_wiersz      = "\r\n" if "\r\n" in TrescPliku else "\n"
    #if not TrescPliku[0].isalnum():  # w pliku na pierwszym znaku zaczely pojawiac sie smieci
    #    TrescPliku = TrescPliku[1:]
    while not TrescPliku[0].isalnum():  # w pliku na pierwszym znaku zaczely pojawiac sie smieci
        TrescPliku = TrescPliku[1:]
    tmp              = TrescPliku.split(nowy_wiersz)
    naglowki         = tmp[0].split(";")
    wiersze          = tmp[1:]
    kol_art          = naglowki.index("article")
    kol_rz1          = naglowki.index("Qty")
    kol_sklep        = naglowki.index("site")
    kol_data_zam     = naglowki.index("order date")
    kol_data_dost    = naglowki.index("delivery date")
    rsglobal.ZapiszDoLogow("kol_art = " + str(kol_art))
    rsglobal.ZapiszDoLogow("kol_rz1 = " + str(kol_rz1))
    rsglobal.ZapiszDoLogow("kol_sklep = " + str(kol_sklep))
    rsglobal.ZapiszDoLogow("kol_data_zam = " + str(kol_data_zam))
    rsglobal.ZapiszDoLogow("kol_data_dost = " + str(kol_data_dost))
    for w in wiersze:
        rsglobal.ZapiszDoLogow("wiers = " + str(w))
        wiersz = w.split(";")
        if len(wiersz) < 5:
            if tmp_zam:
                Zamowienia.append(tmp_zam)
            break
        tmp_art       = wiersz[kol_art]
        tmp_rz1       = wiersz[kol_rz1]
        tmp_data_dost = datetime.strptime(wiersz[kol_data_dost], "%d.%m.%Y")
        tmp_data_zam  = datetime.strptime(wiersz[kol_data_zam], "%d.%m.%Y")
        tmp_sklep     = wiersz[kol_sklep]
        if tmp_zam and tmp_zam.symbol_obcy != tmp_sklep:
            Zamowienia.append(tmp_zam)
        if tmp_zam is None or tmp_zam.symbol_obcy != tmp_sklep:  #  pierwsze zamowienie albo nowy sklep
            tmp_zam = Zamowienie(self.PolSQL, self.firma_id, Typ, rsglobal.ZapiszDoLogow, uwzgledniac_rz2=False)
            rsglobal.ZapiszDoLogow(str(tmp_zam))
            tmp_zam.ustaw_date(tmp_data_dost)
            if stat == 'OK':
                stat = tmp_zam.ustaw_kontrahenta(tmp_sklep)
            if stat == 'OK':
                stat = tmp_zam.znajdz_zamowienie()
            if stat == 'OK':
                stat = tmp_zam.uzupelnij_obce_artykuly()
            if stat != 'OK':
                if '**' not in bledy_oddzialowe:
                    bledy_oddzialowe['**'] = set()
                bledy_oddzialowe['**'].add("Nie zaimportowano zamwienia sklepu {}: {}".format(tmp_sklep, stat))
                tmp_zam = None
                tmp_sklep = None
                tmp_data_dost = None
                tmp_data_zam = None
                stat = 'OK'
                continue
        stat = tmp_zam.dodaj_na_rzut(tmp_art, tmp_rz1)
        if stat != 'OK':
            if tmp_zam.oddzial_id not in bledy_oddzialowe:
                bledy_oddzialowe[tmp_zam.oddzial_id] = set()
            bledy_oddzialowe[tmp_zam.oddzial_id].add(stat)
            stat = 'OK'

if stat == 'OK' and Fun in (3, 4):

    for z in Zamowienia:
        rsglobal.ZapiszDoLogow(str(z))
        stat, bledy = z.pobierz_pozycje()
        if z.oddzial_id not in bledy_oddzialowe:
            bledy_oddzialowe[z.oddzial_id] = set()
        if stat != 'OK':
            bledy_oddzialowe[z.oddzial_id].add(stat)
            stat = 'OK'
            continue
        bledy_oddzialowe[z.oddzial_id].update(bledy)

        stat, bledy = z.zapisz_ilosci_w_zamowieniach()
        if stat != 'OK':
            bledy_oddzialowe[z.oddzial_id].add(stat)
            stat = 'OK'
            continue
        bledy_oddzialowe[z.oddzial_id].update(bledy)

    if len(Zamowienia) == 0:
        bledy_oddzialowe['**'] = set(['W podanym pliku nie odnaleziono nic do zaimportowania. Prosimy sprawdzi jego zawarto.'])

    dane = []
    for bl in bledy_oddzialowe.values():
        dane += sorted(bl)
    rsglobal.ZapiszDoLogow(bledy_oddzialowe)
    if bledy_oddzialowe:
        # pobranie adresw na ktre ma to pj dla konkretnego oddziau
        SQL="SELECT DISTINCT email, oddzial_id FROM rs_powiadomienia_pozycje WHERE firma_id='"+self.firma_id+"'"
        SQL+=" AND powiadomienie='S0001';"
        stat,DPoczta,pw1,dump = self.PolSQL.Execute(SQL,3)
        if stat != 'OK':
            stat="(1) Bd pobierania listy kont pocztowych : "+stat
        if stat == 'OK':
            maile_oddzialy = {}
            do_wysylki     = {}
            tr_poczta      = rssql.RSTableRowset(DPoczta, pw1)
            for i in xrange(len(DPoczta)):
                mail = tr_poczta.GetVal(i, 'email')
                oddz = tr_poczta.GetVal(i, 'oddzial_id')
                if mail not in maile_oddzialy:
                    maile_oddzialy[mail] = set()
                maile_oddzialy[mail].add(oddz)
            rsglobal.ZapiszDoLogow(maile_oddzialy)
            for oddzial in bledy_oddzialowe:
                for mail in maile_oddzialy:
                    if oddzial in maile_oddzialy[mail] or '**' in maile_oddzialy[mail] or oddzial == '**':
                        if mail not in do_wysylki:
                            do_wysylki[mail] = set()
                        do_wysylki[mail].update(bledy_oddzialowe[oddzial])
            if z:
                Temat = "Lista spraw zwizana z importem zamwie z Netto z dnia "+z.data
            else:
                Temat = "Lista spraw zwizana z importem zamwie z Netto"

            for mail in do_wysylki:
                Tresc = '\r\n'.join(sorted(do_wysylki[mail]))
                st = rsglobal.RSWyslijEmail(mail, Temat,Tresc,[],0,self.PolSQL)
                if st != 'OK':
                    dane.append("Bd wysyania maila do {}: {}".format(mail, st))
