# -*- coding: iso8859_2 -*-
#-------------------------------------------------------------------------------
# Name:        fs_import_faktur_do_paragonow_z_pc_market.py
# Purpose:     Import faktur do paragonw fiskalnych z PC Market do dowodw ksigowych
# Author:      Wojciech Skowron
# Created:     2020/08/27
# RCS-ID:      $Id: fs_import_faktur_do_paragonow_z_pc_market.py $
# Copyright:   (c) 2020 Record System
# Licence:     GNU GPL
#-------------------------------------------------------------------------------

# 2023-06-05 - (12105) Wysyka maila na adres z powiadomie


def KonczJesliBladIWyslijEmail(self, stat, rozlacz=False):
    from funkcje import fs_hd_faktury_do_paragonow_import_pom

    if stat != 'OK':
        if rozlacz:
            self.RSPCMarketSQL.Rozlacz()

        stat_email = fs_hd_faktury_do_paragonow_import_pom.WyslijEmailZBledamiPrzedImportem(self, stat, FunkcjaZapisuDoLogow=ZapiszLog)
        if stat_email != 'OK':
            exit('EMAIL_NIEWYSLANY | BLAD')
        else:
            exit('BLAD')


# ------------------------------------------------------------------------------------------------------

from funkcje import fs_hd_faktury_do_paragonow_import_pom

from datetime import datetime, timedelta
import rsglobal
import rsmssql
import traceback

stat = 'OK'

# Argumenty dni_od i dni_do podawane s jako cakowite liczby ujemne,
# np. wywoanie z argumentami "dni_od":-2,"dni_do":-1 uruchomi import wszystkich paragonw z przedwczoraj i wczoraj
SprawdzArgumenty(['dni_od', 'dni_do'])

self = rsglobal.StworzAtrapeSelfa(firma_id, PolSQL)

# ------------------------------------------------------------------------------------------------------
# Wyniki importu zostan przesane e-mailem, tre maila zapisywana na bieco do self.WynikEmail
# ------------------------------------------------------------------------------------------------------

#WS 2023-06-05
SEL = "SELECT DISTINCT email FROM rs_powiadomienia_pozycje WHERE firma_id='{firma_id}' AND powiadomienie = 'PC001';"
SEL = SEL.format(firma_id=self.firma_id)
stat, Mail, dump, dump = PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
if stat != 'OK':
    stat = 'Bd pobrania adresw email z tabeli rs_powiadomienia_pozycje: ' + stat
    KonczJesliBlad(stat)

if len(Mail) == 0:
    KonczJesliBlad("Nie odnaleziono adresw email dla powiadomienia PC001")
self.AdresatEmail = Mail[0][0]

self.WynikEmail = []
self.WynikEmail.append('Data wykonania funkcji: {}   \r\n'.format(datetime.now().strftime("%Y-%m-%d %H:%M")))

# ------------------------------------------------------------------------------------------------------
# Pobranie wszystkich sklepw ze zdefiniowanym id dla PC Market
# ------------------------------------------------------------------------------------------------------

SEL = """SELECT sklep_id, pcmarket FROM rs_hd_sklepy
         WHERE firma_id='{firma_id}' AND pcmarket != 0
         ORDER BY sklep_id ASC;"""
SEL = SEL.format(firma_id=self.firma_id)

stat, Sklepy, dump, dump = self.PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
KonczJesliBladIWyslijEmail(self, stat)

if len(Sklepy) == 0:
    KonczJesliBladIWyslijEmail(self, "Brak placwek handlowych ze zdefiniowanym id dla PC Market")

# ------------------------------------------------------------------------------------------------------
# # Utworzenie listy zawierajcej wszystkie daty z zakresu podanego w argumentach
# ------------------------------------------------------------------------------------------------------

od = min(dni_od, dni_do)
do = max(dni_od, dni_do)

Daty = []
for liczbaDni in range(od, do+1):
    Daty.append((datetime.today() + timedelta(days=liczbaDni)).strftime("%Y-%m-%d"))

param = 'Zakres importu: od {} do {}'.format(Daty[0], Daty[-1]) + \
        ', placwki: '+', '.join([sklep[0].strip() for sklep in Sklepy])+'   \r\n\r\n'
ZapiszLog(param)
self.WynikEmail.append(param)

# ------------------------------------------------------------------------------------------------------
# Poczenie z baz PC Market
# ------------------------------------------------------------------------------------------------------

try:
    self.RSPCMarketSQL = rsmssql.RSPCMarketSQL()
    stat = self.RSPCMarketSQL.Polacz()
except:
    stat = traceback.format_exc()

KonczJesliBladIWyslijEmail(self, stat)

# ------------------------------------------------------------------------------------------------------
# Znalezienie odpowiednikw kontrahentw/patnikw z faktur PC Marketu w rs_kontrahenci
# ------------------------------------------------------------------------------------------------------

try:
    stat, DokKontr, Kontrahenci, importowac = fs_hd_faktury_do_paragonow_import_pom.ZnajdzKontrahentow(self, Daty, Sklepy, FunkcjaZapisuDoLogow=ZapiszLog)
except:
    stat = traceback.format_exc()

KonczJesliBladIWyslijEmail(self, stat, rozlacz=True)

if not importowac:
    # Nie udao si znale wszystkich kontrahentw - list brakujcych wysyamy e-mailem
    fs_hd_faktury_do_paragonow_import_pom.WyslijEmailZListaBrakujacychKontrahentow(self, DokKontr, Kontrahenci, FunkcjaZapisuDoLogow=ZapiszLog)
    stat = 'BLAD'
else:
    # ------------------------------------------------------------------------------------------------------
    # Sprawdzenie, czy istniej definicje potrzebne do dodania dowodw ksigowych
    # ------------------------------------------------------------------------------------------------------

    try:
        stat = fs_hd_faktury_do_paragonow_import_pom.PrzygotujImport(self, Daty, Sklepy, FunkcjaZapisuDoLogow=ZapiszLog)
    except:
        stat = traceback.format_exc()

    KonczJesliBladIWyslijEmail(self, stat, rozlacz=True)

    # ------------------------------------------------------------------------------------------------------
    # Wykonanie importu dla kadego sklepu i dnia, niepowodzenie koczy import
    # ------------------------------------------------------------------------------------------------------

    kontynuuj = True

    for sklep in Sklepy:
        for data in Daty:
            try:
                stat = fs_hd_faktury_do_paragonow_import_pom.ImportujFakturyDoParagonow(self, data, sklep, self.WynikEmail, DokKontr, FunkcjaZapisuDoLogow=ZapiszLog)
                if stat != 'OK':
                    self.PolSQL.Rollback()
                    kontynuuj = False
                    break
            except:
                stat = 'Import przerwany:   \r\n ' + traceback.format_exc()
                self.WynikEmail.append(stat)
                ZapiszLog(stat)
                self.PolSQL.Rollback()
                kontynuuj = False
                break

        if kontynuuj:
            self.PolSQL.CommitCzysty()
        else:
            break

    # Wysanie e-maila z wynikami importu
    stat_email = fs_hd_faktury_do_paragonow_import_pom.WyslijEmailZWynikiemImportu(self, Powodzenie=kontynuuj, FunkcjaZapisuDoLogow=ZapiszLog)
    if stat != 'OK':
        stat = 'BLAD'
    if stat_email != 'OK':
        stat = 'EMAIL_NIEWYSLANY | ' + stat

self.RSPCMarketSQL.Rozlacz()
