# -*- coding: iso8859_2 -*-

#-------------------------------------------------------------------------------
# Name:        fs_kursy_walut_pobierz.py
# Purpose:     pobieranie kursw walut z Internetu
# Author:      Jan Kaliszewski
# Created:     2009/10/24
# RCS-ID:      $Id: funkcje_harmonogramu/fs_kursy_walut_pobierz.py $
# Copyright:   (c) 2009 Record System
# Licence:     GNU GPL
#-------------------------------------------------------------------------------
# Jan Kaliszewski, 2010-02-09 - dodanie obsugi daty z pliku + zmiana znaczenia
#                               podanej pustej (mantis: #0003165)

#-------------------------------------------------------------------------------
# Funkcja harmonogramu: fs_kursy_walut_pobierz.py
# -- pobiera przez Internet kursy walut, z danego banku, dla podanych tabel.
#
# Argumenty obowizkowe:
# * bank_id -- napis,
# * tabele -- napis zawierajcy id-y tabel
#             rozdzielone spacjami i/lub przecinkami,
#
# Argument opcjonalny (pominicie go jest rwnoznacze z podaniem pustego):
# * data -- napis bdcy dat w formacie RRRR-MM-DD lub symbolem takim, e:
#           -- jeeli zaczyna si od litery "d" lub "D" (jak "dzi"), uywana
#              bdzie bieca data,
#           -- jeeli zaczyna si od litery "p" lub "P" (jak "plik"), uywana
#              bdzie data z pobieranego pliku (musi by w nim okrelona),
#           -- jeeli jest pustym napisem (bd zosta pominity),
#              uywana bdzie:
#              -- data z pobieranego pliku, jeli dla danego banku zdefiniowane
#                 jest miejsce w pliku (tag XML lub kolumna[+wiersz] CSV), gdzie
#                 podawana jest data,
#              -- data bieca, jeli dla danego banku nie jest zdefiniowane
#                 miejsce w pliku, gdzie podawana jest data.
#-------------------------------------------------------------------------------


import datetime
import itertools
import rs_anakonda_daty

KODY_BLEDOW_Z_KOMUNIKATEM_W_DANYCH = (
        'BL IDTAB', 'BL DATFOR', 'BL DATROB', 'BL DATPOZ', 'BL DATPLI', 'BL POBDAT')

#-------------------------------------------------------------------------------
# Sprawdzenie argumentw

obowiazkowe = ['bank_id', 'tabele']

# czy podano obowizkowe:
SprawdzArgumenty(obowiazkowe)

# czy wszystkie s napisami:
zle_arg = {}
for nazwa_arg, wartosc_arg in argumenty.items():
    if not isinstance(wartosc_arg, basestring):
        zle_arg[nazwa_arg] = type(wartosc_arg)
if zle_arg:
    KonczJesliBlad('Nieprawidowe typy argumentw: %s.'
                   % ', '.join('"%s" (%s)' % (nazwa, typ)
                               for nazwa, typ in zle_arg.iteritems()))

# czy obowizkowe nie s puste:
puste_obowiazkowe = [nazwa_arg for nazwa_arg in obowiazkowe
                     if not argumenty[nazwa_arg]]
if puste_obowiazkowe:
    KonczJesliBlad('Puste argumenty: %s.'
                   % ', '.join('"%s"' % nazwa for nazwa in puste_obowiazkowe))

#-------------------------------------------------------------------------------
# Odczyt i dostosowanie argumentw

bank_id = rsglobal.Dekoduj(argumenty['bank_id'].strip())

listy_tabel = (li.split()
               for li in rsglobal.Dekoduj(argumenty['tabele'].split(',')))
tabele = filter(None, (tabela_id.strip()
                       for tabela_id in list(itertools.chain(*listy_tabel))))

data = rsglobal.Dekoduj(argumenty.get('data', '')).strip().lower()
if not data:
    data = str(datetime.date.today())  # dzisiejsza data (bdzie uyta, jeeli
    opis_daty = 'z pliku lub bieca (%s)' % data       # w pliku nie ma daty)
    czy_data_z_pliku = 'opcjonalnie'
elif data.startswith('p'):
    data = ''
    opis_daty = 'z pliku'
    czy_data_z_pliku = 'koniecznie'
elif data.startswith('d'):
    data = str(datetime.date.today())  # dzisiejsza (bieca) data
    opis_daty = 'bieca (%s)' % data
    czy_data_z_pliku = ''  # (nie)
else:
    # data taka, jak podano
    opis_daty = data
    czy_data_z_pliku = ''  # (nie)

Rok = int(data[0:4])
Mies = int(data[5:7])
Dzien = int(data[8:10])
if rs_anakonda_daty.CzyRoboczy(datetime.date(Rok, Mies, Dzien)): #MK 2021-03-15
    #-------------------------------------------------------------------------------
    # Waciwe pobieranie kursw

    opis_pobierania = ('kursw walut w banku %s wg tabel: %s -- data: %s'
                       % (bank_id, ', '.join(tabele), opis_daty))
    ZapiszLog('Rozpoczcie pobierania %s.' % opis_pobierania)


    bledy = []
    for tabela_id in tabele:
        stat, dane, none, none = WykonajFunkcje(
                'fs_kursy_walut_pobierz',
                [bank_id, tabela_id, data, czy_data_z_pliku, 5],
                [], [],
        )
        if stat != 'OK':
            if stat in KODY_BLEDOW_Z_KOMUNIKATEM_W_DANYCH:
                stat = dane[0]
            stat = rsglobal.Dekoduj(stat)
            bledy.append('Bd przy pobieraniu tabeli %s: "%s"' % (tabela_id, stat))

    if bledy:
        komunikat = ('Podczas pobierania %s - wystpiy problemy:\n%s'
                     % (opis_pobierania, '\n'.join(bledy)))
        KonczJesliBlad(komunikat)
else: #MK 2021-03-15
    stat = 'OK'
