# -*- coding: iso8859_2 -*-

#-------------------------------------------------------------------------------
# Name:        rs_anakonda_daty.py
# Purpose:     przetwarzanie dat
# Author:      przekopiowane z ac25/util/RSDaty.py, poprawki: Jan Kaliszewski
# Created:     2009/10/24
# RCS-ID:      $Id: rs_anakonda_daty $
# Copyright:   (c) 2009 Record System
# Licence:     GNU GPL
#-------------------------------------------------------------------------------
# 2018-07-16 Aleksander Bargie (11287) W funkcji liczcej dni i godziny nominalne w harmonogramie uwzgldni 12.11.2018
# 2020-09-14 Wojciech Skowron - uwzgldenienie 2020-08-14 w iloscDniPracujacych, CzyRoboczy, CzySwieto
# 2020-12-28 Wojciech Skowron - uwzgldenienie 2020-12-24 w iloscDniPracujacych, CzyRoboczy, CzySwieto (12234)
# 2021-01-20 Aleksander Bargie (12234) Dni nominalne w kartach pracy - uwzglednia wieta przypadajce w sobot
# 2023-08-09 Wojciech Skowron - uwzgldnienie 2023-08-14 w iloscDniPracujacych i CzyRoboczy dla Hali Wola (13512)
# 2023-08-28 Jan Raburski       (13512) - przejcie w dniach roboczych na kalendarz dodanie funkcji ..WgKalendarza
# 2023-09-05 Jan Raburski       (13533) wyrzucenie dodatkowych dni
# 2024-01-11 Wojciech Skowron   (13547) - w CzyRoboczyWgKalendarza sprawdzenie czy dzie roboczy
#                                   wg sobt, niedziel i wit (CzyRoboczy) jeli brak danych w kalendarzu na dany dzie
# 2024-02-05 Aleksander Bargie (13687) [KP] Zle liczone dni nominalne w karcie pracy

import datetime
import calendar
import rsglobal


STALE_SWIETA = (
        # (miesiac, dzien)
        (1, 1),
        (1, 6),
        (5, 1),
        (5, 3),
        (8, 15),
        (11, 1),
        (11, 11),
        (12, 25),
        (12, 26),
)

#-------------------------------------------------------------------------------

def dataWielkanocy(rok):
    u'Podaj dat pierwszego dnia Wielkanocy dla podanego roku.'
    a = rok % 19
    b = int(rok / 100)
    c = rok % 100
    d = int(b / 4)
    e = b % 4
    f = int((b + 8) / 25)
    g = int((b - f + 1) / 3)
    h = (19*a + b - d - g + 15) % 30
    i = int(c / 4)
    k = c % 4
    l = (32 + 2*e + 2*i - h - k) % 7
    m = int((a + 11*h + 22*l) / 451)
    p = (h + l - 7*m + 114) % 31
    dzien = p + 1
    miesiac = int((h + l - 7*m + 114) / 31)
    return datetime.date(rok, miesiac, dzien)

#-------------------------------------------------------------------------------

def drugiDzienWielkanocy(rok):
    u'Podaj dat drugiego dnia Wielkanocy dla podanego roku.'
    Wielkanoc = dataWielkanocy(rok)
    return Wielkanoc + datetime.timedelta(1)

#-------------------------------------------------------------------------------

def BozeCialo(rok):
    u'Podaj dat Boego Ciaa dla podanego roku.'
    Wielkanoc = dataWielkanocy(rok)
    return Wielkanoc + datetime.timedelta(60)

#-------------------------------------------------------------------------------

def LiczbaDniPracujacych(data_od, data_do):
    u'Podaj liczb dni roboczych od daty do daty.'
    dni = (data_do - data_od).days
    dni_pracujacych = dni + 1 - 2 * ((dni + data_od.weekday() + 2) / 7)
    # poprawka bdu z ac25/util/RSDaty.py:
    if data_od.weekday() == 6:
        dni_pracujacych += 1
    if data_do.weekday() == 5:
        dni_pracujacych += 1
    for rok in xrange(data_od.year, data_do.year + 1):
        for miesiac, dzien in STALE_SWIETA:
            swieto = datetime.date(rok, miesiac, dzien)
            #AB 20.01.21r
            # if (data_od <= swieto <= data_do) and swieto.weekday() not in (5, 6):
            if (data_od <= swieto <= data_do) and swieto.weekday() != 6 and swieto.weekday() != 5:
                dni_pracujacych -= 1
        for swieto_ruchome in (drugiDzienWielkanocy(rok), BozeCialo(rok)):
            if data_od <= swieto_ruchome <= data_do:
                dni_pracujacych -= 1

        # JR 2023-09-05 (13533) wyrzucenie dodatkowych dni
        ##AB 15.11.18r
        #if data_od <= datetime.date(2018, 11, 12) <= data_do:
        #    dni_pracujacych -= 1
        ##WS 2020-09-14
        #if data_od <= datetime.date(2020, 8, 14) <= data_do:
        #    dni_pracujacych -= 1
        ##WS 2023-08-09
        #if data_od <= datetime.date(2023, 8, 14) <= data_do:
        #    dni_pracujacych -= 1
        ##AB 20.01.21r
        ## #WS 2020-12-28
        # if data_od <= datetime.date(2020, 12, 24) <= data_do:
        #     dni_pracujacych -= 1
    return dni_pracujacych


# -------------------------------------------------------------------------------

def LiczbaDniPracujacychWgKalendarza(self, pracownik_id, data_od, data_do):
    u'Podaj liczb dni roboczych od daty do daty.'

    # dane pobierane sa z kalendarza przypisanego do pracownika w angazu
    # ktory obowiazuje w podanym okresie (moze byc kilka) i zatrudnienie ma czy_absencje=true
    # jesli pracownik nie ma kalendarza, to wg sobot, niedziel i swiat

    sdata_od = str(data_od)
    sdata_do = str(data_do)
    SQL = "SELECT a.kalendarz_id, a.od_kiedy, COALESCE(a.do_kiedy,'"+sdata_do+"'::DATE)  FROM rs_kp_angaze a"
    SQL += " INNER JOIN rs_kp_zatrudnienie z ON a.firma_id=z.firma_id AND a.pracownik_id = z.pracownik_id"
    SQL += " AND a.zatrudnienie_id=z.numer_id"
    SQL += " WHERE a.firma_id='"+self.firma_id+"' AND kalendarz_id IS NOT NULL"
    SQL += " AND a.pracownik_id='"+pracownik_id+"' AND z.czy_absencje AND z.stan_pracownika_id='A'"
    SQL += " AND a.od_kiedy<='"+str(data_do)+"' AND (a.do_kiedy IS NULL OR a.do_kiedy >= '" + sdata_od + "')"  #AB 2024-02-05
    SQL += " ORDER BY a.od_kiedy ;"
    stat, okresy, dump, dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return 0
    if len(okresy) == 0:
        return LiczbaDniPracujacych(data_od, data_do)
    rsglobal.ZapiszDoLogow(("Okresy:"+str(okresy)))

    dni = 0
    dni_pracujacych = 0
    for w in okresy:
        SQL = "SELECT COUNT(*), SUM(CASE WHEN status_dnia='R' THEN 1 ELSE 0 END)"
        SQL += " FROM rs_kp_kalendarze_dni  WHERE firma_id='"+self.firma_id+"' AND kalendarz_id='"+w[0]+"'"
        SQL += " AND dzien BETWEEN '"+max(sdata_od,str(w[1]))+"'::DATE AND '"+min(sdata_do,str(w[2]))+"'::DATE"
        stat, ldni, dump, dump = self.PolSQL.Execute(SQL,3)
        if stat != 'OK':
            return 0
        dni += ldni[0][0]
        dni_pracujacych += ldni[0][1]

    if dni != (data_do - data_od).days + 1:  # nie ma wszystkich dni w kalendarzach
        rsglobal.ZapiszDoLogow("Policzone dni:"+str(dni))
        return LiczbaDniPracujacych(data_od, data_do)

    return dni_pracujacych


#-------------------------------------------------------------------------------

def LiczbaDniNominalnychDlaAngazuWMiesiacu(self, pracownik_id, zatrudnienie_id, umowa_id, angaz_id, data):
    u'Podaje liczb dni roboczych wg kalendarza z angazu dla miesiaca z daty.'

    # dane pobierane sa z kalendarza przypisanego do pracownika w podanym angazu
    # dla miesiaca z podanej daty niezaleznie od stanu pracownika
    # jesli pracownik nie ma kalendarza lub kalendarz nie jest wypelniony wraca z zerem

    SQL = "SELECT a.kalendarz_id  FROM rs_kp_angaze a"
    SQL += " WHERE a.firma_id='" + self.firma_id + "' AND kalendarz_id IS NOT NULL"
    SQL += " AND a.pracownik_id='" + pracownik_id + "' AND a.zatrudnienie_id=" + str(zatrudnienie_id)
    SQL += " AND a.umowa_id=" + str(umowa_id) + " AND a.angaz_id=" + str(angaz_id)
    stat, kalendarze, dump, dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return 0
    if len(kalendarze) == 0:
        return 0
    #rsglobal.ZapiszDoLogow(("Okresy:"+str(okresy)))

    sdataod = str(data)[0:8] + '01'
    sdatado = str(data)[0:8] + str(rsglobal.PodajOstatniDzien(str(data)) + 100)[1:]
    SQL = "SELECT SUM(CASE WHEN status_dnia='R' THEN 1 ELSE 0 END)"
    SQL += " FROM rs_kp_kalendarze_dni  WHERE firma_id='"+self.firma_id+"' AND kalendarz_id='"+kalendarze[0][0]+"'"
    SQL += " AND dzien BETWEEN '"+sdataod+"'::DATE AND '"+sdatado+"'::DATE"
    stat, ldni, dump, dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return 0
    return ldni[0][0]

#-------------------------------------------------------------------------------

def iloscDniPracujacychOdPoczatkuRoku(data):
    u'Podaj liczb dni roboczych od pocztku roku do podanej daty.'
    return LiczbaDniPracujacych(datetime.date(data.year, 1, 1), data)

#-------------------------------------------------------------------------------
       
def CzyRoboczy(data):
    u'Czy podany dzie jest dniem roboczym?'
    
    rok = data.year
    swieta = [datetime.date(rok, miesiac, dzien)
              for miesiac, dzien in STALE_SWIETA]
    #AB 15.11.18r
    swieta.append(datetime.date(2018, 11, 12))
    #WS 2020-09-14
    swieta.append(datetime.date(2020, 8, 14))
    #WS 2023-08-09
    swieta.append(datetime.date(2023, 8, 14))
    #AB 20.01.21r
    # #WS 2020-12-28
    # swieta.append(datetime.date(2020, 12, 24))
    swieta.extend([drugiDzienWielkanocy(rok), BozeCialo(rok)])
    if (data.weekday() in (5, 6)) or data in swieta:
        return False
    else:
        return True

# -------------------------------------------------------------------------------

def CzyRoboczyWgKalendarza(self, pracownik_id, data):
    u'Czy podany dzie jest dniem roboczym wg kalendarza?'

    # dane pobierane sa z kalendarza przypisanego do pracownika w angazu
    # ktory obowiazuje w podanym okresie i zatrudnienie ma czy_absencje=true
    # jesli pracownik nie ma kalendarza, to wg sobot, niedziel i swiat

    sdata = str(data)
    SQL = "SELECT a.kalendarz_id FROM rs_kp_angaze a"
    SQL += " INNER JOIN rs_kp_zatrudnienie z ON a.firma_id=z.firma_id AND a.pracownik_id = z.pracownik_id"
    SQL += " AND a.zatrudnienie_id=z.numer_id"
    SQL += " WHERE a.firma_id='"+self.firma_id+"' AND kalendarz_id IS NOT NULL"
    SQL += " AND a.pracownik_id='"+pracownik_id+"' AND z.czy_absencje AND z.stan_pracownika_id='A'"
    SQL += " AND a.od_kiedy<='"+sdata+"'::DATE"
    SQL += " ORDER BY a.od_kiedy DESC LIMIT 1;"
    stat, okresy, dump, dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return False
    if len(okresy) == 0:
        return CzyRoboczy(data)

    #WS 2024-01-11 - jeli brak danych w kalendarzu na dany dzie to sprawdzamy po staremu CzyRoboczy
    # w zwizku z tym zapytanie przerobione - jeli brak danych zwraca nic, wczeniej zwracao 0
    SQL = "SELECT status_dnia='R'"
    SQL += " FROM rs_kp_kalendarze_dni  WHERE firma_id='"+self.firma_id+"' AND kalendarz_id='"+okresy[0][0]+"'"
    SQL += " AND dzien='"+sdata+"'::DATE;"
    # SQL = "SELECT COUNT(*)"
    # SQL += " FROM rs_kp_kalendarze_dni  WHERE firma_id='"+self.firma_id+"' AND kalendarz_id='"+okresy[0][0]+"'"
    # SQL += " AND dzien='"+sdata+"'::DATE AND status_dnia='R'
    # stat, ldni, dump, dump = self.PolSQL.Execute(SQL,3)
    stat, status, dump, dump = self.PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return False
    if len(status) == 0:  #WS 2024-01-11
        return CzyRoboczy(data)

    #WS 2024-01-11
    return status[0][0]
    # return ldni[0][0] == 1

#-------------------------------------------------------------------------------

def ZamienDate(data_str):
    u'Zamie napis na obiekt typu datetime.date.'
    
    return datetime.date(int(data_str[0:4]),
                         int(data_str[5:7]),
                         int(data_str[8:]))

#-------------------------------------------------------------------------------

def DodajKilkaDni(data_str, dni):
    u'Daj dat przesunit o podan liczb dni wzgldem podanej jako napis daty.'
    
    data = ZamienDate(data_str)
    nowa_data = data + datetime.timedelta(dni)
    return nowa_data.isoformat()

#-------------------------------------------------------------------------------

def CzySwieto(data_str):
    u'Czy podany dzie jest dniem witecznym lub niedziel?'

    data = ZamienDate(data_str)
    rok = data.year
    swieta = [datetime.date(rok, miesiac, dzien)
              for miesiac, dzien in STALE_SWIETA]
    #AB 15.11.18r
    swieta.append(datetime.date(2018, 11, 12))
    #WS 2020-09-14
    swieta.append(datetime.date(2020, 8, 14))
    #AB 20.01.21r
    # #WS 2020-12-28
    # swieta.append(datetime.date(2020, 12, 24))
    swieta.extend([drugiDzienWielkanocy(rok), BozeCialo(rok)])
    if (data.weekday()==6) or data in swieta:
        return True
    else:
        return False

# -------------------------------------------------------------------------------

#AB 30.11.18r
def OkresPomiedzy(dataod, datado=str(datetime.datetime.now().date())):
    if dataod == '' or dataod > datado:
        return '0000-00-00'
    else:
        try:
            dataod   = datetime.datetime.strptime(dataod, '%Y-%m-%d')
            datado   = datetime.datetime.strptime(datado, '%Y-%m-%d')
            tmpdata  = datado
            if tmpdata.day < dataod.day:
                dni = tmpdata.day - dataod.day + calendar.monthrange(dataod.year, dataod.month)[1]
                dataod -= datetime.timedelta(days=tmpdata.day)
                tmpdata -= datetime.timedelta(days=tmpdata.day)
            else:
                dni = tmpdata.day - dataod.day
            mendmc   = tmpdata.month - dataod.month
            miesiecy = mendmc + (0 if mendmc >= 0 else 12)
            lat      = tmpdata.year - dataod.year - (0 if mendmc >= 0 else 1)
            lat      = str(lat+10000)[1:]
            miesiecy = str(miesiecy+100)[1:]
            dni      = str(dni+100)[1:]

            return '{lat}-{miesiecy}-{dni}'.format(lat=lat, miesiecy=miesiecy, dni=dni)
        except ValueError, error:
            print("ValueError w rs_anakonda_daty: " + str(error))
            return '0000-00-00'

# -------------------------------------------------------------------------------

def PodzielNaOkresy(self, Pracownik, DataOd, DataDo,sql=True):
    # funkcja dzeli podany zakres dat na okresy uwzgledniajac zmiany angazy
    # i przelomy miesiecy. 
    # UWAGA: w liscie okresow moga pojawic sie okresy puste = []
    # co nalezy uwzglednic w ewentualnej analizie
    Okresy=[[]]
    if DataOd>DataDo:
        return 'OK',Okresy
    if DataOd==DataDo:
        return 'OK',[[DataOd,DataDo]]
    if sql:
        SQL ="""SELECT do_kiedy FROM rs_kp_angaze
            WHERE firma_id = '{firma}' AND pracownik_id = '{pracownik}' 
               AND (do_kiedy BETWEEN '{od}' AND '{do}')
            ORDER BY do_kiedy ;""".format(firma=self.firma_id, pracownik=Pracownik,od=DataOd, do=DataDo)
        stat, dane, dump, dump = self.PolSQL.Execute(SQL,3)
        #rsglobal.ZapiszDoLogow('JR: ' + str(SQL), 5)
        if stat != 'OK':
            stat = 'Bd pobierania dat z angazy: '+stat
            return stat,Okresy
    else:
        dane=[]
        stat='OK'
            
    if len(dane) > 0:
        # dzielimy na okresy z angazu kady okres dzielimy wg miesiecy
        dod=DataOd
        for w in dane:
            (stat,okresy)=PodzielNaOkresy(self, Pracownik,dod,w[0],False)
            if stat!='OK':
                return stat,Okresy
            Okresy.extend(okresy)
            dod=DodajKilkaDni(w[0],1)
        (stat,okresy)=PodzielNaOkresy(self, Pracownik,dod,DataDo,False)
        if stat!='OK':
            return stat,Okresy
        Okresy.extend(okresy)
    else:
        # dzielimy wg miesiecy
        while DataOd[0:7] != DataDo[0:7]:
            dost = DataOd[0:8]+str(rsglobal.PodajOstatniDzien(DataOd)+100)[1:]
            Okresy.append([DataOd, dost])
            DataOd = DodajKilkaDni(dost,1)
        Okresy.append([DataOd,DataDo])
    return stat,Okresy

