
# -*- coding: iso8859_2 -*-

#-----------------------------------------------------------------------------
# Name:        rs_anakonda_server.py
# Purpose:     serwer poredniczacy pomiadzy PostgreSQL a klientam RSAnakonda
#
# Authors:      Michal Raburski, Aleksandra Kaminska
#
# Created:     2004/03/06
# RCS-ID:      $Id: rs_anakonda_server.py $
# Copyright:   (c) 2004 Record System
# Licence:     GNU GPL
#-----------------------------------------------------------------------------
# Jan Kaliszewski, 2009-07-09:
# * nowe elementy konfiguracji dot. raportw z serwera zapas. (mantis: #0002191)
# Jan Kaliszewski, 2009-08-19:
# * RELOAD serwera (mantis: #0002109)
# Jan Kaliszewski, 2009-10-26:
# * popr. bdu krytycznego przy zerwanym pocz. serwisowym (mantis: #0002728)
# Jan Kaliszewski, 2010-04-25:
# * drobiazgi dot. serwera PAN + dot. bezpieczestwa
# Aleksander Bargie 2018-11-14 Wczytywanie tabel po restarcie serwera (11276)
# Aleksander Bargie 2020-02-11 Reklamacje RODO - przypominajki w harmonogramie (11748)
# Aleksander Bargie 2020-05-28 serwer Anakondy - najpierw wczytanie bazy, potem harmonogram  (11992)
# Aleksander Bargie 2021-02-18 Obsuy bd dotyczcy nieprawidowego hasla uzytkownika harmonogram w rs-and.conf (12344)
# Aleksander Bargie  	2022-10-24 bad reloadu serwera (12823 )
# Aleksander Bargie  	2023-03-27 bad reloadu serwera (13377)

import cPickle
import fcntl
import os
import os.path
import resource
import select
import socket
import sys
import tempfile
import time
import traceback

from const import *  # configuration as well as opertions' constants
import rsglobal      # rsglobal.ZapiszDoLogow(), rsglobal.PodajZKonf()
import rssql

PLIK_KONFIGURACJI = 'rs-and.conf'

#-------------------------------------------------------------------------------
#- funkcja uzupeniajca tabel z poczeniami informacj o awarii
#-------------------------------------------------------------------------------

def UzupelnijLogiPolaczenAwaria():
    # stworzenie poczenia
    try:
        stat = 'OK'
        PolSQL = rssql.RSSQL()
        #AB 18.02.21
        # if PolSQL.PolaczZSQL(harm_dbuser, harm_dbpass, hostSQL, portSQL, None, Baza=bazaSQL):
        stat = PolSQL.PolaczZSQL(harm_dbuser, harm_dbpass, hostSQL, portSQL, None, Baza=bazaSQL)

        if stat:
            blad = "Wystapily problemy podczas laczenia z baza jako harmonogram. "
            if stat == 7:
                blad += "Podano bledne haslo dla uzytkownika {} obslugujacego harmonogram zadan".format(harm_dbuser)
            elif stat == 8:
                blad += "Bledna baza {}".format(bazaSQL)
            elif stat == 1:
                blad += "Napotkano nieznany typ bledu."
            else:
                blad = "Wystapil problem o numerze {} podczas uzupelniania logow polaczen.".format(stat)
            # print blad  #AB 24.10.22r
            rsglobal.ZapiszDoLogow(blad)
            sys.exit(0)
            # return False
    
        # wykonanie zapisu do rs_polaczenia
        #AB 27.03.23r
        SQL= "UPDATE rs_polaczenia SET koniec = NOW(),typ='A' WHERE koniec IS NULL;" \
             "UPDATE rs_harmonogram_zadan_log SET stat='podczas wykonania zrestartowano serwer', " \
             "          wypisane='podczas wykonania zrestartowano serwer' " \
             "WHERE stat = '';"
        stat,dump,dump,dump = PolSQL.Execute(SQL,1,None,True,True)
        if stat != 'OK':
            rsglobal.ZapiszDoLogow("Bd zapytania uzupeniajcego poczenia :"+str(stat), 5)    
            
        # zamknicie poczenia
        PolSQL.RozlaczZSQL()
        
        return stat == 'OK'
    except Exception,e:
        rsglobal.ZapiszDoLogow("Bd uzupenienia pocze :"+str(e), 5)
        if PolSQL:
            PolSQL.RozlaczZSQL()
        return False
#AB 13.11.18r
#-------------------------------------------------------------------------------
#- funkcja odpalajca zapytania przed uruchomieniem nasuchiwania
#-------------------------------------------------------------------------------


def OdpalZapytania():
    try:
        PolSQL = rssql.RSSQL()
        #AB 18.02.21
        # if PolSQL.PolaczZSQL(harm_dbuser, harm_dbpass, hostSQL, portSQL, None, Baza=bazaSQL):fs_faktury
        stat = PolSQL.PolaczZSQL(harm_dbuser, harm_dbpass, hostSQL, portSQL, None, Baza=bazaSQL)

        if stat:
            blad = "Wystapily problemy podczas laczenia z baza jako harmonogram. "
            if stat == 7:
                blad += "Podano bledne haslo dla uzytkownika {} obslugujacego harmonogram zadan".format(harm_dbuser)
            elif stat == 8:
                blad += "Bledna baza {}".format(bazaSQL)
            elif stat == 1:
                blad += "Napotkano nieznany typ bledu."
            else:
                blad += "Wystapil problem o numerze {} podczas uruchamiania zapytan wczytujacych baze do pamieci.".format(stat)
            # print blad  #AB 24.10.22r
            rsglobal.ZapiszDoLogow(blad)
            sys.exit(0)
            # return False

        plik      = open('wczytaj.sql', 'r')
        zapytania = plik.readlines()
        plik.close()
        OK = True
        for zapytanie in zapytania:
            if zapytanie.strip() != '':
                start  = time.time()
                stat, dump, dump, dump = PolSQL.Execute(zapytanie, 1, None, True, True)
                if stat != 'OK':
                    OK = False
                rsglobal.ZapiszDoLogow("Zapytanie: " + zapytanie + "czas wykonania: " + str(time.time() - start) + " stat: " + stat, 1)
        PolSQL.RozlaczZSQL()
        return OK

    except Exception, e:
        rsglobal.ZapiszDoLogow("Bd w adowaniu danych do pamici:"+str(e), 5)
        if PolSQL:
            PolSQL.RozlaczZSQL()
        return False

    
#---------------------------------------------------------------------------
# Kilka rzeczy dotyczcych RELOAD-u serwera -- niezbdnych ju tutaj!
#---------------------------------------------------------------------------

# Funkcja dot. blokady zakadanej przy reloadzie:

def zdejmijBlokade(deskryptor_pliku_blokady):
    u'Odblokuj moliwo kilowania podprocesw'

    fcntl.flock(deskryptor_pliku_blokady, fcntl.LOCK_UN)  # odblokuj
    os.close(deskryptor_pliku_blokady)


# Funkcja finalizujca komunikacj z rs_anakonda_service.py po RELOAD-zie
# i ustawiajca globaln zmienn reloadServera na False:

def koniecReloadu(czy_pozytywny, komunikat, bez_delowania=False):
    u'Zakoczenie procedury reloadu serwera'
    
    global reloadSerwera
    
    readPipe, writePipe = reloadSerwera

    if czy_pozytywny:
        os.write(writePipe, SERVERRELOAD_OK)   # te wartoci nie maj znaczenia,
    else:                                      # ale lepiej niech bd dla
        os.write(writePipe, SERVERRELOAD_NOK)  # spjnoci z reszt kodu

    # komunikat (tekst) do wywietlenia przez rs_anakonda_service.py
    os.write(writePipe, rsglobal.encodeLength(komunikat))
    os.write(writePipe, komunikat)

    os.close(readPipe)
    os.close(writePipe)
    
    if not bez_delowania:
        # to samo, co dla innych czynnoci serwisowych robi ManageChildRequest()
        try:
            global childrenReadPipePidDict
            global childrenPidWritePipeDict
            pid = childrenReadPipePidDict[readPipe]
            del childrenReadPipePidDict[readPipe]
            del childrenPidWritePipeDict[pid]
        except KeyError, exc:
            print('Nie udalo sie usunac ze slownika procesu/deskryptora. %s'
                  % exc)
        
    reloadSerwera = False
    
#---------------------------------------------------------------------------

# Stwierdzenie czy mamy do czynienia z reloadem i jeeli tak
# -- pobranie z argumentw skryptu potrzebnych informacji:

argumenty_skryptu = sys.argv[1:]

reloadSerwera = False
for arg in argumenty_skryptu:
    if arg.startswith('--reload='):
        # Skoro podano argument "--reload=X,Y", uruchomienie skryptu serwera
        # jest skutkiem RELOAD-u. Zmienna globalna reloadSerwera staje si
        # krotk numerw podanych jako X,Y w argumencie -- bdcych deskryptorami
        # pipe'w do komunikacji (odczyt i zapis) z rs_anakonda_service.py,
        # by w mg prawidowo wywietli komunikat o zakoczeniu reloadu:
        reloadSerwera = arg[len('--reload='):].split(',')  # dwie wartoci
        reloadSerwera = tuple(map(int, reloadSerwera))     # str -> int
        break

if reloadSerwera:
    for arg in argumenty_skryptu:
        if arg.startswith('--blokada='):
            # Plik blokady (wykorzystujcy mechanizm udostpniany przez
            # fcntl.flock) uyty jest (tutaj oraz w moduach klient/serwis)
            # do zapewnienia, e podczas zasadniczej czci reloadu
            # *nie bd killowane* potomne procesy (w kadym razie, nie wtedy
            # gdy wszystko idzie prawidowo):
            deskryptor_pliku_blokady = int(arg[len('--blokada='):])
            break
    else:
        raise RuntimeError('To sie nie powinno zdarzyc:'
                           ' brak informacji o deskryptorze pliku blokady!!!')

    for arg in argumenty_skryptu:
        if arg.startswith('--stan='):
            # Plik stanu zawiera zamarynowane (tzn. zapisane przy pomocy
            # cPickle) obiekty, ktrych odtworzenie jest konieczne dla
            # odtworzenia stanu serwera (pocze etc. -- patrz dalej:
            # staa ZACHOWAC_PRZY_RELOADZIE):
            sciezka_pliku_stanu = arg[len('--stan='):]
            break
    else:
        zdejmijBlokade(deskryptor_pliku_blokady)
        raise RuntimeError('To sie nie powinno zdarzyc:'
                           ' brak informacji o sciezce do pliku stanu!!!')

#---------------------------------------------------------------------------

# Dalsza cz importw -- objta dla bezpieczestwa klauzul try...except...
# dzieki czemu w razie wywaki przy RELOAD-zie zostanie zdjta blokada,
# za rs_anakonda_service.py wypisze stosowny komunikat (wraz z tracebackiem):

try:
    import rs_anakonda_server_se
    import rs_anakonda_server_cl
    import rs_anakonda_server_hz
    import rs_anakonda_server_komser
    import rs_anakonda_server_pan
    import rs_process_killer
    from rs_anakonda_socket	import *	# dedykowana obsluga socketow TCP
    from OpenSSL import rand, SSL

    from signal import *    # SIG*, signal
    from mx import DateTime

    childrenReadPipePidDict = {}
    childrenPidWritePipeDict = {}
    ssl_jest_dostepne = True
    STANDARD_OPERATOR_ID=int(STANDARD_OPERATOR_ID)

    SCIEZKA_TEGO_SKRYPTU = os.path.abspath(sys.argv[0])
    (KAT_SERWERA
    ) = (rs_anakonda_server_pan.kat_serwera
        ) = rs_anakonda_server_cl.kat_serwera = os.path.dirname(SCIEZKA_TEGO_SKRYPTU)
    SCIEZKA_PLIKU_BLOKADY = os.path.abspath(NAZWA_BLOKADY_RELOADU)
    ZACHOWAC_PRZY_RELOADZIE = (
        'deskryptor_cl',  # deskryptor gwnego socketu klienckiego
        'deskryptor_se',  # deskryptor gwnego socketu serwisowego
        'clientprocPidList',
        'serviceprocPidList',
        'childrenPidAdrDict',
        'childrenReadPipePidDict',
        'childrenPidWritePipeDict',
        'clientPort',     # port pocze klienckich
        'servicePort',    # port pocze serwisowych
    )
except:
    try:  #AB 24.10.22r
        print traceback.format_exc()
    except:
        pass
    if reloadSerwera:
        zdejmijBlokade(deskryptor_pliku_blokady)
        koniecReloadu(False,
                      'BLAD KRYTYCZNY przy probie RELOAD-u, w czasie'
                      ' ponownego uruchamiania serwera -- przy importach'
                      ' i inicjowaniu stalych! %s'
                      % traceback.format_exc(),
                      bez_delowania=True)
        try:  #AB 24.10.22r
            print('BLAD KRYTYCZNY RELOAD-u serwera, moze byc'
                  ' konieczne reczne skillowanie podprocesow itp.')
        except:
            pass

    raise


########################################################################
#                                       #
#            Obsluga sygnalow                        #
#                                       #
########################################################################

##########################################################
#   rejestracja handlerow dla glownego procesu serwera   #
##########################################################

def RegisterHandlers():
    signal(SIGUSR1, ChildAbnormallyExitedHandler)
    signal(SIGCHLD, ChildExitedHandler)
    for sig in terminatingSigs:
        signal(sig, TerminationHandler)


def DeregisterHandlers():
    signal(SIGUSR1, SIG_DFL)
    signal(SIGCHLD, SIG_DFL)
    for sig in terminatingSigs:
        signal(sig, SIG_DFL)
    

##################################################
###    signal handler for SIGUSR1
##################################################


def ChildAbnormallyExitedHandler(signum, stackFrame):
    rsglobal.ZapiszDoLogow("Watek obslugujacy polaczenie klienckie zakonczyl \
sie nagle", 5)
    
    
##################################################
###    signal handler for SIGCHLD
##################################################

def ChildExitedHandler(signum, stackFrame):
  """ Funkcja wolana jest, gdy proces przechwyci sygnal SIGCHLD

  Sygnal SIGCHLD pojawia sie gdy (dowolne) dziecko procesu sie konczy.
  Domyslnie w Linuksie sygnal ten jest ignorowany.

  Python nie wydaje sie udostepniac sposobu na dolaczenie dodatkowych danych 
  do sygnalu (wtedy warto by uzyc SIGCHLD), wiec po przechwyceniu sygnalu 
  wywolywana jest funkcja sprawdzajaca ktory z procesow sie zakonczyl"""

  #print "Dziecko wyszlo"
  
  global clientprocPidList
  global serviceprocPidList

  if (signum != SIGCHLD):
    # XXX sanity check, bedzie mozna usunac
    rsglobal.ZapiszDoLogow("Handler przechwycil nieprawidlowy sygnal - niedobrze...", 1)
    return 

  try:
    pid, returnStatus = os.wait()
  except Exception, e:
    rsglobal.ZapiszDoLogow( "Dziecka nie ma: "+str(e)+"!!!!!!!!!!!!!!!!!!!!!!!!",1)
    return
  rsglobal.ZapiszDoLogow("Proces serwera o numerze pid "+str(pid)+" zakonczyl dzialanie ze statusem "+str(returnStatus), 5)
  
  if pid in clientprocPidList:
    clientprocPidList.remove(pid)
  elif pid in serviceprocPidList:
    serviceprocPidList.remove(pid)
    del childrenPidAdrDict[pid]
  elif pid == rs_anakonda_server_komser.zarzadcaTransportu.pidPotomka:
    rs_anakonda_server_komser.zarzadcaTransportu.sygnalSIGCHLD(pid, returnStatus)

  return
  
  

##################################################
###    signal handler for SIGTERM (and so on)
##################################################


def TerminationHandler(signum, stackFrame):
  """ handler do oblugi sygnalu SIGTERM - konczacego wykonanie programu """

  rsglobal.ZapiszDoLogow("TerminationHandler, signal: "+str(signum),3)
  rsglobal.ZapiszDoLogow("Przechwycilem sygnal SIGTERM, zwalniam zasoby systemowe i koncze dzialanie", 5)
  doPerformCleanup(SIGALRM)   
  sys.exit() # wywoluje funkcje ParentsFreeResources()


##################################################
###
##################################################

def ManageChildRequest(readPipe):
  """ Funkcja realizujaca pojedyncza ture komunikacja z potomkiem 

  A konkretnie z polaczeniem serisowym. Tu zamykamy tez pipe'y, bo inaczej to bardzo szkodzi..."""  
  global childrenReadPipePidDict
  global childrenPidWritePipeDict
  global clientprocPidList
  global serviceprocPidList
  global childrenPidAdrDict
  global reloadSerwera


  #    cel w jakim komunikuje sie potomek (identyfikator 'polecenia')
  try:
    option = os.read(readPipe, OPTION_LEN)
  except socket.error, exc:
    rsglobal.ZapiszDoLogow(
            'Bd odczytu ze strumienia sucego do komunikacji z procesem'
            ' obsugujcym poaczenia serwisowe ("%r"). Proces prawdopodobnie'
            ' zakoczy dziaanie.' % exc,
            2
    )
    option = 'BLAD ODCZYTU ZE STRUMIENIA'
  #    pid procesu potomnego
  pid = childrenReadPipePidDict[readPipe]
  #print "@"+str(pid)
  #    jesli do potomka mozna pisac, to chcemy znac odpowiedni deskryptor
  writePipe = childrenPidWritePipeDict[pid]
  #print "@@"+str(writePipe)

  #print "OPTION: "+option

  # Wykonanie odpowiedniej akcji w zaleznosci od polecenia
  #
  #    option == FORCE - bezwzgledne wylaczenie serwera
  if option == FORCE:
      #	proba zakonczenia wszystkich dzieci
    try:
      forceCleanup()
      # sukces  
      rsglobal.ZapiszDoLogow('Wszystko zatrzymane',1)
      signal(SIGCHLD, SIG_IGN) 
      list = []
      list.extend(clientprocPidList)
      list.extend(serviceprocPidList)      
      for pid in list:
        try:
          os.waitpid(pid, 0)
          #print "Child "+str(pid)+" exited"
        except Exception, excep:
          #print excep
          rsglobal.ZapiszDoLogow('Bd zatrzymania dziecka '+str(excep),1)
      #TODO os.write(writePipe, FORCE_OK)
    except Exception, e:
      # problem
      os.write(writePipe, FORCE_NOK)
      #print e
      rsglobal.ZapiszDoLogow('Bd czyszczenia '+str(e),1)
      comment="Wylaczenie serwera niezupelnie udane"
      # zakomunikowanie wyniku  
      i=(comment)
      os.write(writePipe, rsglobal.encodeLength(i))
      #print "iiii"
      os.write(writePipe,comment)

    # w obu przypadkach konczymy dzialanie procesu serwera
    Exit() 
    os._exit(0) # to nie wywola Exit()
  #    option == STOP - proba wylaczenie serwera
  elif option == STOP:
      #     Serwer mozna "miekko" zamknac jedynie, jesli jedynym
      #     obslugiwanym polaczeniem jest polaczenie serwisowe
      #        W takim przypadku polegamy na procesach klienckich / serwisowych
      #        ze nie postanowia nawiazywac polaczen w czasie zamykania serwera
      #        - w przeciwnym przypadku moga dostac bledy zwiazane z shutdownem
      #        polaczen TCP
      if len(clientprocPidList) == 0: 
        signal(SIGCHLD, SIG_IGN)
        if len(serviceprocPidList) > 1:
          rsglobal.ZapiszDoLogow('Niewlasciwa sytuacja - wiele polaczen serwisowych :'+str(serviceprocPidList)+' . Mimo to zamykam serwer', 1)
        else:
          rsglobal.ZapiszDoLogow('Jest ok - dziala tylko jeden proces serwisowy o pid'+str(serviceprocPidList[0]), 3)
        performStopCleanup()
        #print serviceprocPidList
        
        # XXX pozostajac pod wplywem bardzo silnej sugestii, ze polaczenie 
    # XXX serwisowe jest tylko jedno na raz
        try:
          for pid in serviceprocPidList:
            os.waitpid(pid, 0)
        except Exception, exc:
          rsglobal.ZapiszDoLogow('waitpid(%s) failed: %s' % (pid, exc), 1)
    # zwolnienie zasobow - nie trzeba zabijac potomkow - i wyjscie
    # sys.exit() podnosi wyjatek SysExit, ktory sprawia, ze wywolywane
    # jest koncowe 'sprzatanie', w tym Exit()
        os.close(readPipe)
        os.close(writePipe) 
        Exit()
        os._exit(0) 
      else:
      # przekazanie potomkowi, ze operacja sie nie powiodla
          os.write(writePipe, STOP_NOK)
      # status niepowodzenia
          comment = 'Nie mona zatrzyma serwera, bo s do niego poczenia.'
          os.write(writePipe,rsglobal.encodeLength(comment))
          os.write(writePipe, comment)
      
          
  #    option == LIST - przeslanie aktualnej listy polaczen z serwerem
  elif option == LIST: 
      os.write(writePipe, LIST_OK)
      list = []
      list.extend(clientprocPidList)
      list.extend(serviceprocPidList)
      #lista zawierajca pidy i adresy
      lista2=[]
      for pid in list:
          if pid in childrenPidAdrDict:
              lista2.append((pid,childrenPidAdrDict[pid][0]))
          else:
              lista2.append((pid,'???'))
              
      if len(lista2) > 0:
          comment = str(lista2)
      else:
          comment ='Brak pocze do serwera.'
      os.write(writePipe, rsglobal.encodeLength(comment))
      os.write(writePipe, comment)
  elif option == HZRELOAD:
      os.write(writePipe, RELOAD_OK)
      comment = rs_anakonda_server_hz.zarzadcaHarmonogramu.Reload()
      os.write(writePipe, rsglobal.encodeLength(comment))
      os.write(writePipe, comment)
  elif option == KOMSERRELOAD:
      os.write(writePipe, RELOAD_OK)
      komserConfig = getKomSerConf()
      comment = rs_anakonda_server_komser.zarzadcaTransportu.Reload(komserConfig)
      os.write(writePipe, rsglobal.encodeLength(comment))
      os.write(writePipe, comment)
  elif option == SERVERRELOAD:  # RELOAD caego serwera
      reloadSerwera = readPipe, writePipe
      return  # komunikat i zamknicie pipe'w jest w koniecReloadu()
  elif option == 'BL':
      rsglobal.ZapiszDoLogow(
            'Proces obsugujcy poczenie serwisowe poinformowa, e nastpi'
            ' bd poczenia z klientem serwisowym, wic nie wykonujemy'
            ' adnego polecenia seriwsowego.',
            2
      )
  elif option == 'BLAD ODCZYTU ZE STRUMIENIA':
      pass  # by bd odczytu ze strumienia, patrz wyej
  else:
      rsglobal.ZapiszDoLogow('Bledne polecenie: '+option,1)
      os.write(writePipe,NO_SUCH_OPTION)
      comment = 'Bledne polecenie'
      os.write(writePipe, rsglobal.encodeLength(comment))
      os.write(writePipe, comment)

  os.close(readPipe)
  os.close(writePipe)
  del childrenReadPipePidDict[readPipe]
  del childrenPidWritePipeDict[pid]
  #print 'hohoho'
  return

##################################################
###    funckja sprawdzajaca Pickle
##################################################

# [chyba nic tego nie uzywa, zwlaszcza, ze nawet
#  pickle nie jest zaimportowane]

def Sprawdzpickle(Dane):
    Udane=pickle.loads(pickle.dumps(Dane))
    if Udane != Dane:
        rsglobal.ZapiszDoLogow('Bd Pickle',2)
    
##################################################
###
##################################################

def getConfiguration():
  """ Pobiera konfiguracje korzystajac z rsglobal.PodajZKonf() 
  
  Wynikiem dzialania funkcji jest krotka (port polaczenia klienckiego, port
  polaczenia serwisowego, adres serwera SQL, port serwera SQL, plik z logami,
  poziom logowania, typ logowania, program php, baza,
  uzytkownik bazy harmonogramu, jego haslo, host bazy raportow, port bazy raportow,
  czy zbierac statystyki wykonania funkcji, raportow, katalog podstawowy gdzie zapisywane s pliki) """

  params = ('port','hostpgsql','portpgsql','logfile','loglevel','logtype',
            'phpprogram', 'baza', 'harmonogramuser', 'harmonogrampass',
            'domyslnynadawca', 'smtphost', 'hostpgsql2','portpgsql2',
            'stat_fun', 'stat_rap','katalogplikow','katalogedeklaracji','enterprise',
            'soapuser', 'soappass','soaphost', 'soapport', 'debug')
  list = rsglobal.PodajZKonf(PLIK_KONFIGURACJI, params)
  returnList = []

  returnList.append(int(list[0][0:list[0].find(',')]))
  returnList.append(int(list[0][list[0].find(',')+1:len(list[0])]))
  returnList.append(list[1])
  returnList.append(int(list[2]))
  returnList.append(list[3])
  returnList.append(int(list[4]))
  returnList.append(list[5])
  returnList.append(list[6])    # returnList[7]
  if list[7]=='':
      list[7]='anakonda'
  returnList.append(list[7])
  returnList.append(list[8])
  returnList.append(list[9])    # returnList[10]
  returnList.append(list[10])
  returnList.append(list[11])
  returnList.append(list[12])
  returnList.append(list[13])   # returnList[14]
  returnList.append(list[14].lower() in ("true", "yes", "on", "tak"))
  returnList.append(list[15].lower() in ("true", "yes", "on", "tak"))
  returnList.append(list[16])   # returnList[17]
  returnList.append(list[17])   # returnList[18]
  returnList.append(list[18].lower() in ("true", "yes", "on", "tak"))   # returnList[19]
  returnList.append(list[19])
  returnList.append(list[20]) 
  returnList.append(list[21]) 
  returnList.append(list[22]) 
  returnList.append(list[23].lower() in ("true", "yes", "on", "tak"))   # returnList[24]
  return returnList


def getKomSerConf():

  """ Pobiera konfiguracje dla transportu miedzyserwerowego

  zwraca slownik - pusty jesli nie ustawiono transportu
  """

  params = ("komserhost", "komserport", "komserklienci", "komserkatalog", "komserwer", "komsernazwa", "komserlog", "komserssl")
  vals = rsglobal.PodajZKonf(PLIK_KONFIGURACJI, params)
  opcje = dict(zip(params, vals))
  if not opcje["komserlog"]:
      del opcje["komserlog"]
  if opcje["komserwer"] and opcje["komserwer"].lower() in ('yes', 'true', 'tak', '1'):
      opcje["komserwer"] = True
  else:
      opcje["komserwer"] = False
  if opcje["komserkatalog"] and opcje["komsernazwa"] and not (
        opcje["komserhost"] or opcje["komserport"] or opcje["komserklienci"]):
      kat = os.path.join(opcje["komserkatalog"], opcje["komsernazwa"], rs_anakonda_server_komser.Magazyn.IN)
      if not os.path.exists(kat):
          os.makedirs(kat)
  opcje["komserssl"] = opcje["komserssl"].lower() in ('yes', 'true', 'tak', '1')
  if SSLContext:
      komserConfig['SSLContext'] = SSLContext
  if (opcje["komserhost"] or opcje["komserport"] or opcje["komserklienci"]) and not (
        opcje["komserport"] and opcje["komserklienci"] and opcje["komsernazwa"]):
    for i in range(1, 5):
        if not vals[i]:
            rsglobal.ZapiszDoLogow("komser: brak wartosci " + params[i], 1)
    return {}
  else:
    if opcje["komserklienci"]:
        params = ["komser-" + x.strip() for x in opcje["komserklienci"].split(",")]
        cvals = rsglobal.PodajZKonf(PLIK_KONFIGURACJI, params)
        for i in range(len(params)):
            client = params[i]
            if not cvals[i]:
                rsglobal.ZapiszDoLogow("komser: brak %s w pliku konfiguracji" % client, 1)
                return {}
            opcje[client] = cvals[i]
    return opcje

##################################################
###    Uruchamianie usug towarzyszcych
##################################################

def uruchomHarmonogram(Enterprise = False):
    u'Uruchom harmonogram zadan'

    #JR 11590 2019-08-16 dodanie 'czydebug' do wywolania harmonogramu
    params = ("debug", "port", "katalogplikow", "katalogedeklaracji", "baza")
    vals = rsglobal.PodajZKonf(PLIK_KONFIGURACJI, params)
    opcje = dict(zip(params, vals))
    if opcje["debug"]:
        czydebug = opcje["debug"].lower() in ('on', 'yes', 'true', 'tak', '1')
    else:
        czydebug = False
    try:
        if harm_dbuser:
            if hostSQL2 and portSQL2:
                rs_anakonda_server_hz.zarzadcaHarmonogramu.Uruchom(
                    (harm_dbuser, harm_dbpass, hostSQL, portSQL, bazaSQL),              #AB 11.02.20r
                    (harm_dbuser, harm_dbpass, hostSQL2, portSQL2, bazaSQL), czydebug, konfiguracja=opcje)
            else:
                rs_anakonda_server_hz.zarzadcaHarmonogramu.Uruchom(                         #AB 11.02.20r
                    (harm_dbuser, harm_dbpass, hostSQL, portSQL, bazaSQL), None, czydebug, konfiguracja=opcje)
            rs_anakonda_server_hz.zarzadcaHarmonogramu.Enterprise = Enterprise
    except Exception, e:
        msg = ("Nie udalo si uruchomic harmonogramu zadan: %s %s"
               % (e.__class__.__name__, e))
        # print "\n", msg  #AB 24.10.22r
        rsglobal.ZapiszDoLogow(msg, 1)
        sys.exit(3)
        
#-------------------------------------------------------------------------------

def uruchomKomser(opoznienie=None,Enterprise = False):
    u'Uruchom transport midzyserwerowy'
    
    try:
        if komserConfig:
            initSSL()
            if SSLContext:
                komserConfig['SSLContext'] = SSLContext
            if opoznienie is not None:
                rs_anakonda_server_komser.zarzadcaTransportu.Uruchom(
                        komserConfig,
                        czekaj=opoznienie,
                )
            else:
                rs_anakonda_server_komser.zarzadcaTransportu.Uruchom(
                        komserConfig,
                )
            rs_anakonda_server_komser.zarzadcaTransportu.Enterprise = Enterprise
    except Exception, e:
        msg = ("Blad w czasie uruchamiania tranportu miedzyserwerowego: %s %s"
               % (e.__class__.__name__, e))
        # print "\n", msg  #AB 24.10.22r
        rsglobal.ZapiszDoLogow(msg, 1)
        sys.exit(3)

#-------------------------------------------------------------------------------

def uruchomPan():
    opts = ("hostpgsql", "portpgsql", "harmonogramuser", "harmonogrampass",
            "panport", "baza", "panfirmaid", "panminkod","enterprise")
    keys = ("hostpgsql", "portpgsql", "panuser", "panpass",
            "port", "baza", "panfirmaid", "panminkod","enterprise")
    vals = rsglobal.PodajZKonf(PLIK_KONFIGURACJI, opts)
    konfiguracja = dict(zip(keys, vals))

    if konfiguracja["port"] and konfiguracja["panfirmaid"]:
        if not konfiguracja["baza"]:
            konfiguracja["baza"] = "anakonda"

        if not konfiguracja["panuser"] or not konfiguracja["panpass"]:
            msg = ("Bd konfiguracji PAN, nie ustawiono 'harmonogramuser'"
                   " i/lub 'harmonogrampass'. Serwer PAN nieuruchomiony.")
            rsglobal.ZapiszDoLogow(msg, 1)
            # print msg  #AB 24.10.22r
            return False
            
        try:
            konfiguracja["port"] = int(konfiguracja["port"])
            konfiguracja["portpgsql"] = int(konfiguracja["portpgsql"])
        except ValueError, exc:
            msg = (u"Nieprawidowy numer portu w konfiguracji pan"
                   u" - %r. Serwer PAN nieuruchomiony." % exc)
            rsglobal.ZapiszDoLogow(msg, 1)
            # print msg  #AB 24.10.22r
            return False

        try:
            if konfiguracja["panminkod"]:
                konfiguracja["panminkod"] = int(konfiguracja["panminkod"])
            else:
                konfiguracja["panminkod"] = DOMYSLNA_MIN_DLUGOSC_KODU_DOSTEPU_PAN
        except ValueError, exc:
            msg = ("Nieprawidowa minimalna dugo kodu dostpu"
                   " w konfiguracji pan - %r. Serwer PAN nieuruchomiony." % exc)
            rsglobal.ZapiszDoLogow(msg, 1)
            # print msg  #AB 24.10.22r
            return False

        return rs_anakonda_server_pan.serwer.Uruchom(konfiguracja)
        
    else:
        if konfiguracja["port"]:
            msg = ("Bd konfiguracji PAN, nieustawiona opcja"
                   " 'panfirmaid'. Serwer PAN nieuruchomiony.")
            rsglobal.ZapiszDoLogow(msg, 1)
            # print msg  #AB 24.10.22r
        elif konfiguracja["panfirmaid"]:
            msg = ("Bd konfiguracji PAN, nieustawiona opcja"
                   " 'panport'. Serwer PAN nieuruchomiony.")
            rsglobal.ZapiszDoLogow(msg, 1)
            # print msg  #AB 24.10.22r
        else:
            pass  # ok, po prostu nie uywamy PAN
        return False

#-------------------------------------------------------------------------------

def uruchomSOAP():
    try:
        if soap_dbuser:
            from enterprise import rs_anakonda_enterprise_b2b_server
            # to na potrzeby reloadu serwera soap
            reload(rs_anakonda_enterprise_b2b_server)
            rs_anakonda_enterprise_b2b_server.SerwerSOAP.UruchomSerwerSOAP(
                    (soap_dbuser, soap_dbpass, hostSQL, portSQL, bazaSQL),soap_host,soap_port)
    except Exception, e:
        msg = ("Nie udalo si uruchomi serwera SOAP: %s %s"
               % (e.__class__.__name__, e))
        # print "\n", msg  #AB 24.10.22r
        rsglobal.ZapiszDoLogow(msg, 1)
        sys.exit(3)

##################################################
###    Zatrzymywanie usug towarzyszcych
##################################################

def zatrzymajHarmonogram():
    u'Zatrzymaj harmonogram zada'
    
    try:
        rs_anakonda_server_hz.zarzadcaHarmonogramu.Zatrzymaj()
    except Exception, e:
        rsglobal.ZapiszDoLogow('Bd przy zatrzymywaniu harmonogramu zada '
                               + str(e), 1)

def zatrzymajKomser():
    u'Zatrzymaj transport midzyserwerowy'

    try:
        rs_anakonda_server_komser.zarzadcaTransportu.Zatrzymaj()
    except Exception, e:
        rsglobal.ZapiszDoLogow('Bd przy zatrzymywaniu transportu '
                               + str(e), 1)

def zatrzymajPAN():
    u'Zatrzymaj serwer PAN'

    try:
        rs_anakonda_server_pan.serwer.Zatrzymaj(force=True)
    except Exception, e:
        rsglobal.ZapiszDoLogow('Bd przy zatrzymywaniu PANu ' + str(e), 1)


def zatrzymajSOAP():
    u'Zatrzymaj serwer SOAP'
    
    try:
        from enterprise import rs_anakonda_enterprise_b2b_server
        rs_anakonda_enterprise_b2b_server.SerwerSOAP.ZatrzymajSerwerSOAP()
    except Exception, e:
        rsglobal.ZapiszDoLogow('Bd przy zatrzymywaniu serwera SOAP '
                               + str(e), 1)

#-------------------------------------------------------------------------------

##################################################
###
##################################################

#TODO byc moze to nalezy pododawac
def performStopCleanup():
  #print "Normal closing"
  return doPerformCleanup(SIGUSR1)

def doPerformCleanup(sign):
  """ 'Prywatna' funkcja, pomocnicza dla performCleanup() i forceCleanup() """
  global serviceprocPidList
  global clientprocPidList

  successful = 1;
  ex = None
  list = []
  
  list.extend(serviceprocPidList)
  list.extend(clientprocPidList)
  rsglobal.ZapiszDoLogow('lista zamykanych Pidow:'+str(list),3)
  
  wypisz = lambda tekst: rsglobal.ZapiszDoLogow(tekst, 5)
  wypisz_blad = lambda tekst: rsglobal.ZapiszDoLogow(tekst, 1)
  for pid in list:
    rsglobal.ZapiszDoLogow('Probuje wylaczyc %s sygnalem %s' % (pid, sign), 1)
   
    try:
      rsglobal.ZapiszDoLogow("os.kill("+str(pid)+","+str(sign)+")",3)
      os.kill(pid,sign)
      #rs_process_killer.kill(pid, sygnaly=(sign, SIGKILL), opoznienie=0,
      #                       odstep=rsglobal.ODSTEP_SPR_KILLOWANIA,
      #                       wypisz=wypisz, wypisz_blad=wypisz_blad,
      #                       bez_blokady=True)
    except os.error, e:
      rsglobal.ZapiszDoLogow("Nieaktualna lista pidow", 1)
      # ESRCH - nie ma juz procesu o tym pidzie
      if (e != errno.ESRCH):
        ex = e
    
  if ex != None:
    raise ex
   

def forceCleanup():
  #print "forcing closing"
  return doPerformCleanup(SIGUSR2)

def initSSL():
    global ssl_jest_dostepne, SSLContext, _probowano_inicjalizowac_ssl
    if _probowano_inicjalizowac_ssl:
        return
    if ssl_jest_dostepne and not SSLContext:
        try:
            rand.load_file(os.path.join(SSL_DIR, 'randpool.dat'))
            SSLContext = SSL.Context(SSL.SSLv23_METHOD)
            SSLContext.set_options(SSL.OP_NO_SSLv2)
            filepath = os.path.join(SSL_DIR, "anakonda-key.pem")
            assert os.path.exists(filepath), "Brak pliku %s" % filepath
            SSLContext.use_privatekey_file(filepath)
            filepath = os.path.join(SSL_DIR, "anakonda-cert.pem")
            assert os.path.exists(filepath), "Brak pliku %s" % filepath
            SSLContext.use_certificate_file(filepath)
        except Exception, exc:
            rsglobal.ZapiszDoLogow("Inizjalizacja SSL nie powioda si, "
                                   "SSL bdzie wyczony: %s"
                                   % traceback.format_exc(), 1)
            ssl_jest_dostepne = False
            SSLContext = None
        _probowano_inicjalizowac_ssl = True

_probowano_inicjalizowac_ssl = False

########################################################################
#                                                                      #
#                    Zwalnianie zasobow systemowych                    #
#                                                                      #
########################################################################

########################################################################
###           Zwolnienie zasobow systemowych glownego procesu             #
########################################################################

def Exit():
   """ Funkcja zwalniajaca zasoby systemowe """
   global clSocket
   global servSocket
   global childrenReadPipePidDict
   global childrenPidWritePipeDict

   rsglobal.ZapiszDoLogow('Prba zamknicia wszystkiego - EXIT',1)
   #print "Exit()"
   # zamkniecie, jesli jeszcze trzeba, glownego polaczenia do portu klienckiego
   if clSocket != None:
     try: 
       clSocket.shutdown(2)
     except Exception, e:
       rsglobal.ZapiszDoLogow('Bd zamkniecia glownego polaczenia do portu klienta '+str(e),1)

   zatrzymajHarmonogram()
   zatrzymajKomser()
   zatrzymajPAN()
   zatrzymajSOAP()

   # zamkniecie, jesli jeszcze trzeba, glownego polaczenia do portu klienckiego
   if servSocket != None:
     try:   
       servSocket.shutdown(2)
     except Exception, e:
       rsglobal.ZapiszDoLogow('Bd zamkniecia glownego polaczenia do portu klienta 2 '+str(e),1)
#   global connections
#   for sock in connections:
#     print "Trying to close socket"
#     try:
#       sock.shutdown(2)
#     except Exeption, e:
#       print e


   # zamkniecie pipe'ow do potomkow procesu
   for entry in childrenReadPipePidDict.items():
     try:
       pipe, pid = entry
       os.close(pipe)
     except Exception, e:
       rsglobal.ZapiszDoLogow('Bd zamkniecia pipe do potomka procesu '+str(e),1)

   for entry in childrenPidWritePipeDict.items():
     try:
       pid, pipe = entry
       os.close(pipe)
     except Exception, e:
       rsglobal.ZapiszDoLogow('Bd zamkniecia pipe do potomka procesu 2 '+str(e),1)

   if ssl_jest_dostepne and SSLContext:
       try:
           rand.write_file(os.path.join(SSL_DIR, 'randpool.dat'))
       except Exception, e:
           rsglobal.ZapiszDoLogow('Bd rand.write_file() - '+str(e),1)

   rsglobal.ZapiszDoLogow('Gniazda i pipe\'y zamkniete',1)


########################################################################
#                                                                      #
#                            Reload serwera                            #
#                                                                      #
########################################################################

def przygotujPlikBlokady():
    u'Utwrz plik blokady, jeeli jeszcze nie istnieje'
    
    #if not os.access(SCIEZKA_PLIKU_BLOKADY, os.F_OK):
    #    plik_blokady_reloadu = open(SCIEZKA_PLIKU_BLOKADY, 'wb')
    #    plik_blokady_reloadu.close()


def zalozBlokade():
    u'Zablokuj moliwo kilowania podprocesw, czekaj jeeli kilowanie trwa.'
    
    rsglobal.ZapiszDoLogow('Zakadanie blokady', 5)
    
    deskryptor_pliku_blokady = os.open(SCIEZKA_PLIKU_BLOKADY,
                                       os.O_RDWR      # <- otwrz do odczytu i zapisu
                                       | os.O_CREAT,  # <- utwrz, jeli nie istnieje
                                       0744)
    fcntl.fcntl(deskryptor_pliku_blokady, fcntl.F_SETFD, 0)  # close-on-exec = 0
    fcntl.flock(deskryptor_pliku_blokady, fcntl.LOCK_EX)     # zablokuj/czekaj
    
    time.sleep(3)
    return deskryptor_pliku_blokady


def zamarynujStan(nazwy_obiektow_stanu):
    u'Dane do odtworzenia po reloadzie zapisz do pliku i zwr ciek'

    rsglobal.ZapiszDoLogow('Marynowanie stanu', 5)

    stan = dict((nazwa, obiekt)
                for nazwa, obiekt in globals().iteritems()
                if nazwa in nazwy_obiektow_stanu)
                
    (deskryptor_pliku_stanu, sciezka_pliku_stanu
    ) = tempfile.mkstemp(suffix=PRZYROSTEK_NAZWY_PLIKU_STANU, dir=KAT_SERWERA)
    plik_stanu = os.fdopen(deskryptor_pliku_stanu, 'wb')
    try:
        cPickle.dump(stan, plik_stanu, 2)  # 2 - nowszy protok pickle (od Py2.3)
    finally:
        plik_stanu.close()
    
    return sciezka_pliku_stanu


def odmarynujStanSprzedReloadu(sciezka_pliku_stanu):
    u'Odczytaj z pliku obiekty potrzebne do odtworzenia stanu serwera'
    
    if not sciezka_pliku_stanu.endswith(PRZYROSTEK_NAZWY_PLIKU_STANU):
        raise IOError('Nazwa pliku stanu bez prawidowego przyrostka')
    if os.path.dirname(sciezka_pliku_stanu) != KAT_SERWERA:
        raise IOError('Pliku stanu musi znajdowa si w katalogu serwera')
    plik_stanu = open(sciezka_pliku_stanu, 'rb')
    stan = cPickle.load(plik_stanu)
    plik_stanu.close()
    os.remove(sciezka_pliku_stanu)
            
    return stan
    

########################################################################
#                                                                      #
#                       Waciwe ciao serwera                         #
#                                                                      #
########################################################################

ostrzezenia = []

try:
    try:
        
        # 0. Sprawdzenie czy to nie reload, jeeli tak -- pobranie stanu sprzed,
        #    jeeli nie -- demonizacja wieo uruchomionego serwera.

        if not reloadSerwera:
            # to nie jest reload, tylko pierwotny start
            try:  #AB 24.10.22r
                print 'Inicjalizacja...'
            except:
                pass
            # demonizacja
            if os.fork() != 0:
                os._exit(0)
            os.setsid()
            if os.fork() != 0:
                os._exit(0)
            os.umask(0)
        else:
            try:  #AB 24.10.22r
                print 'Inicjalizacja po RELOAD-zie...'
            except:
                pass
            stan_sprzed_reloadu = odmarynujStanSprzedReloadu(sciezka_pliku_stanu)

        # 1. Czynnosci wstepne i inicjalizacja struktur danych

        sys.exitfunc = Exit
        mainPid = os.getpid()
        rsglobal.ZapiszDoLogow('MainPid '+str(mainPid),1)
        program='python'
        SSL_DIR='ssl'
        
        SSLContext = None
        if not reloadSerwera:
            # przygotowanie pliku blokady dla reloadu serwera
            przygotujPlikBlokady()

            # w czesci catch sockety sa zamykane. Wprawdzie przy zamykaniu
            # przechwytywane sa wyjatki, niemniej jednak zgodne z regulami 
            # sztuki jest zazanczenie, ze te obiekty jeszcze nie istnieja
            
            # clSocket - glowne polaczenie klienckie, pod ten adres lacza sie
            # nowe procesy klienckie ktore chca byc obsluzone
            clSocket = None
            
            # servSocket - glowne polaczenie serwisowe, pod ten adres lacza sie
            # nowe procesy serwisowe ktore chca byc obsluzone
            servSocket = None
               
            # pid glownego procesu serwera
            # lista pidow potomkow procesu. 
            clientprocPidList = []
            serviceprocPidList = []
            # slownik w ktorym kluczami sa pidy potomkow, a wartosciami
            # adresy zwiazane z polaczeniami ktore obsluguja
            childrenPidAdrDict = {}
            # slownik w ktorym kluczami sa numery lacz nienazwanych do
            # czytania, a wartosciami numery pid'ow potomkow, z ktorymi
            # mozna sie przez nie komunikowac
            # UWAGA: sens ma tylko komunikacja z procesami obslugujacymi 
            # polaczenia serwisowe
            #childrenReadPipePidDict = {}
            # slownik w ktorym kluczami sa numery pid'ow potmkow procesu, a
            # wartosciami - numery lacz nienazwanych do pisania, ktorymi 
            # mozna sie z tymi procesami komunikowac.
            # UWAGA: Pisac mozna tylko do procesow serwera obslugujacych
            # poalczenia serwisowe
            #childrenPidWritePipeDict = {}
            #connections = [clSocket, servSocket]

        else:
            # Przywrcenie stanu sprzed RELOAD-u:
            
            clSocket = socket.fromfd(stan_sprzed_reloadu['deskryptor_cl'],
                                     socket.AF_INET, socket.SOCK_STREAM)
            clSocket = AnakondaSocket(socket_obj=clSocket)
            
            servSocket = socket.fromfd(stan_sprzed_reloadu['deskryptor_se'],
                                     socket.AF_INET, socket.SOCK_STREAM)
            servSocket = AnakondaSocket(socket_obj=servSocket)
            
            clientprocPidList = stan_sprzed_reloadu['clientprocPidList']
            serviceprocPidList = stan_sprzed_reloadu['serviceprocPidList']
            childrenPidAdrDict = stan_sprzed_reloadu['childrenPidAdrDict']
            childrenReadPipePidDict = stan_sprzed_reloadu['childrenReadPipePidDict']
            childrenPidWritePipeDict = stan_sprzed_reloadu['childrenPidWritePipeDict']

            sprzed_clientPort = stan_sprzed_reloadu['clientPort']
            sprzed_servicePort = stan_sprzed_reloadu['servicePort']

            del stan_sprzed_reloadu
        
        
        # 2. Pobranie konfiguracji
        configs=getConfiguration()
        clientPort, servicePort, hostSQL, portSQL = configs[0:4]
        if reloadSerwera and clientPort != sprzed_clientPort:
            # nie honorujemy zmiany w konfiguracji dot. portu
            # (bo nie ruszamy istniejcego poczenia clSocket)
            ostrz = ('Po RELOAD-zie uzywany jest stary port'
                     ' polaczen klienckich (zmiana w konfiguracji'
                     ' nie ma poki co zastosowania)')
            try:  #AB 24.10.22r
                print ostrz
            except:
                pass
            ostrzezenia.append(ostrz)
            clientPort = sprzed_clientPort
            
        if reloadSerwera and servicePort != sprzed_servicePort:
            # nie honorujemy zmiany w konfiguracji dot. portu
            # (bo nie ruszamy istniejcego poczenia servSocket)
            ostrz = ('Po RELOAD-zie uzywany jest stary port'
                     ' polaczen serwisowych (zmiana w konfiguracji'
                     ' nie ma poki co zastosowania)')
            try:  #AB 24.10.22r
                print ostrz
            except:
                pass
            ostrzezenia.append(ostrz)
            servicePort = sprzed_servicePort
            
        rsglobal.LogFile, rsglobal.LogLevel, rsglobal.LogType = configs[4:7]
        phpProgram, bazaSQL = configs[7:9]
        if configs[11]:
            rsglobal.DomyslnyNadawca = configs[11]
        if configs[12]:
            rsglobal.SerwerSMTP = configs[12]
        hostSQL2, portSQL2 = configs[13:15]
        rs_anakonda_server_cl.stat_fun = configs[15]
        rs_anakonda_server_cl.stat_rap = configs[16]
        rs_anakonda_server_cl.KatalogPlikow = configs[17]
        rs_anakonda_server_cl.Enterprise = configs[19]
        
        harm_dbuser, harm_dbpass = configs[9], configs[10]
        soap_dbuser, soap_dbpass = configs[20], configs[21]
        soap_host, soap_port = configs[22], configs[23]
        komserConfig = getKomSerConf()
        
        rsglobal.ZapiszDoLogow('Odczytano konfiguracj...',1)

        # 3. Zarejestrowanie obslugi sygnalow 
        #       python w przeciwienstwie do standardowego system calla nie 
        #       wymaga rejestracji handlera po przechwyceniu kazdego kolejnego 
        #       sygnalu
        RegisterHandlers()
        
    finally:
        if reloadSerwera:
            # po zarejestrowaniu obsugi sygnaw moemy ju dopuci
            # killowanie procesw potomnych
            time.sleep(3)
            zdejmijBlokade(deskryptor_pliku_blokady)
            time.sleep(3)

    #AB 28.05.20r
    # 4. Ladowanie danych do pamieci
    if not reloadSerwera:
        Wyn = OdpalZapytania()

    # 5. Uruchomienie harmonogramu zada
    uruchomHarmonogram(rs_anakonda_server_cl.Enterprise)
    
    # uzupenienie pustych logw pocze jako awarii
    if not reloadSerwera:
        Wyn = UzupelnijLogiPolaczenAwaria()

    # 6. Uruchomienie transportu - wymiany danych midzy serwerami
    if not reloadSerwera:
        uruchomKomser(Enterprise = rs_anakonda_server_cl.Enterprise)
    else:
        uruchomKomser(OPOZNIENIE_RESTARTU_KOMSERA, rs_anakonda_server_cl.Enterprise)

    # 7. Uruchomienie PAN - Produkcja ANakonda, terminale
    if not reloadSerwera:
        uruchomPan()

    # 8. Inicjalizacja potrzebnych obiektow - sockety
    # (host == '' to z punktu widzenia pythona localhost)
    try:
        if not reloadSerwera:
            host = ''
            # stworzenie socketow TCP, zbindowanie z konkretnymi portami 
            # i ustawienie nasluchu na nich
            clSocket = AnakondaSocket(socket.AF_INET, socket.SOCK_STREAM)
            servSocket = AnakondaSocket(socket.AF_INET, socket.SOCK_STREAM)

            # inicjalizacja zmiennych potrzebnych do obslugi dedykowanych socketow
            # - obiektow AnakondaSocket
            clSocket.bind((host, clientPort))
            servSocket.bind((host, servicePort))

            # polaczenie dla procesow klienckich
            clSocket.listen(5)
            # polaczenie dla procesow serwisowych
            servSocket.listen(5)
            #print "Po inicjalizacji obiektow"

        initSSL()

    except Exception, e:
        rsglobal.ZapiszDoLogow('Wystpi bd uruchamiania serwera : %s'
                               % traceback.format_exc(), 1)
        # print 'Wystpi bd uruchamiania serwera :'  #AB 24.10.22r
        # print e
        if not reloadSerwera:
            zatrzymajHarmonogram()
            zatrzymajKomser()
            zatrzymajPAN()
            zatrzymajSOAP()
            os._exit(0)
        else:
            sys.exit('BLAD KRYTYCZNY przy uruchamianiu serwera (w ramach'
                     ' RELOAD-u): %s' % traceback.format_exc())
       
    #9. Uruchomienie serwera SOAP - tylko jeli wczona funkcja enterprise
    if rs_anakonda_server_cl.Enterprise:
        uruchomSOAP()

    # 10. Glowna petla programu
    #        W nieskonczonej petli proces: nasluchuje na zdarzenia polegajace
    #        na mozliwosci czytania z glownego polaczenia klienckiego,
    #        glownego polaczenia serwisowego lub z lacza nienazwanego od
    #        ktoregos potomka, a nastepnie wywoluje dla wszystkich zdarzen
    #        odpowiednie funkcje obslugi (tworzy nowy proces lub wykonuje
    #        czynnosci administracyjne.

    #Pobranie pierwszego konfiga i przekazywanie go dalej
    lconf=getConfiguration()
    
except:
    if reloadSerwera:
        koniecReloadu(False,
                      'BLAD KRYTYCZNY przy probie RELOAD-u, w czasie'
                      ' ponownego uruchamiania serwera -- ju w "glownym'
                      ' ciele serwera" (inicjalizacja przed glowna petla). %s'
                      % traceback.format_exc(),
                      bez_delowania=True)
        print('BLAD KRYTYCZNY RELOAD-u serwera, moze byc'
              ' konieczne reczne skillowanie podprocesow itp.')
    raise
    
else:
    if reloadSerwera:
        komunikat = 'RELOAD zakonczony pomyslnie.'
        if ostrzezenia:
            komunikat += '\nOstrzezenia:\n* ' + '\n* '.join(ostrzezenia)
        koniecReloadu(True, komunikat)
        del komunikat, 
    del ostrzezenia


while not reloadSerwera:
    
    # 8a. Select
    #        Oczekiwanie na nowe polaczenie klienckie, nowe polaczenie
    #        serwisowe, lub na dane od potomkow
      
    # zdarzenia do czytania
    readEvents = [clSocket.fileno(), servSocket.fileno()]
    readEvents.extend(childrenReadPipePidDict.keys())
      
    # zdarzenia do pisania - nie czekamy na takie zdarzenia
    writeEvents = []
    # bledy - nie czekamy na takie zdarzenia
    errorEvents = []

    # oczekiwanie blokujace; zwracana jest trojka list gotowych eventow
    # (deskryptory do odczytu, deskryptory do zapisu, deskryptory z bledami)
    try:
        readyEvents = select.select(readEvents, writeEvents, errorEvents)
    except Exception, e:
        try:
          if e[0] == 4:
            rsglobal.ZapiszDoLogow("kontynuacja",3)
            continue
          else:
            rsglobal.ZapiszDoLogow("Exception...: "+str(e),3)
            raise e
        except Exception, exc:
          raise exc

      # z zalozenia wiemy, ze gotowe beda tylko eventy do czytania
      # - poniewaz lista wynikowa jest "podzbiorem" wejsciowych


    # 8b. Obsluga gotowych zdarzen
    for event in readyEvents[0]:
       
        # nowy proces kliencki do obslugi
        if event == clSocket.fileno():
          # dedykowane polaczenie dla procesu klienckiego
          connection, addr = clSocket.accept()
          #print "len(addr): "+str(len(addr))
          # pipe'y do komunikacji procesu obslugujacego z glownym procesem
          # serwera
          # TODO jesli dziecko robi dostep do bazy, to niepotrzebne readPipe, writePipe = os.pipe()
          # stworzenie procesu obslugujace polaczenie
          pid = os.fork()

          # wywolanie odpowiednich akcji dla ojca i syna
          if pid:
            # ojciec
            clientprocPidList.append(pid)
            childrenPidAdrDict[pid]=addr
          else: 
            # syn
            name='client-manager'
            file='rs_anakonda_server_cl.py'
            con= str(connection.fileno())
            #os.execlp(program,name,file, con, str(mainPid), str(addr[0]), str(addr[1]))
            rs_anakonda_server_cl.Wykonaj(int(con), mainPid, (str(addr[0]), addr[1]),lconf, SSLContext)

        # nowy proces serwisowy do obslugi
        elif event == servSocket.fileno():
          # dedykowane polaczenie dla procesu serwisowego
          connection, addr = servSocket.accept()
          #connections.append(connection)
          # pipe'y do komunikacji dwustronnej procesu obslugujacego z glownym
          # procesem serwera
          readPipe1, writePipe1 = os.pipe()
          readPipe2, writePipe2 = os.pipe()
          # stworzenie procesu obslugujace polaczenie
          pid = os.fork()
          # wywolanie odpowiednich akcji dla ojca i syna
          if pid:
            # ojciec
            os.close(writePipe1)
            os.close(readPipe2)
            serviceprocPidList.append(pid)
            childrenPidAdrDict[pid] = addr
            childrenReadPipePidDict[readPipe1] = pid
            childrenPidWritePipeDict[pid] = writePipe2
          else: 
            # syn
            os.close(readPipe1)
            os.close(writePipe2)
            name='service-manager'
            file='rs_anakonda_server_se.py'
            rdPipe=str(readPipe2)
            wrPipe=str(writePipe1)
            con=str(connection.fileno())
            #os.execlp(program,name,file,con,str(mainPid),rdPipe,wrPipe)
            rs_anakonda_server_se.Wykonaj(int(con),mainPid,int(rdPipe),int(wrPipe))
        
        # proces potomny chce sie skomunikowac z glownym procesem serwera
        else:
          # sanity check
          if event not in childrenReadPipePidDict:
            rsglobal.ZapiszDoLogow("This shouldn't have happened (3)",1)
            os._exit(0)
            break;
          else:
            try:
              ManageChildRequest(event)
            except Exception, e:
              #print "!!! "+str(e)+" !!!"
              rsglobal.ZapiszDoLogow("This shouldn't have happened (4) :"+str(e),1)
              os._exit(0)

    if reloadSerwera:
        try:
            # RELOAD serwera...
            
            rsglobal.ZapiszDoLogow('Rozpoczto RELOAD caego serwera', 3)

            # zatrzymaj komser i harmonogram
            zatrzymajHarmonogram()
            zatrzymajKomser()
            #zatrzymajPAN() - FIXME reload nie obsugiwany
            
            # zatrzymaj SOAP
            if rs_anakonda_server_cl.Enterprise:
                zatrzymajSOAP()
                
            # za blokad killowania podprocesw klienckich
            deskryptor_pliku_blokady = zalozBlokade()

            # obiekty tworzce stan do zachowania
            deskryptor_cl = clSocket.fileno()
            deskryptor_se = servSocket.fileno()

            # deksryptory pipe'w do komunikacji z rs_anakonda_service.py
            se_read, se_write = reloadSerwera

            # zachowaj stan (zamarynuj go do pliku)
            sciezka_pliku_stanu = zamarynujStan(ZACHOWAC_PRZY_RELOADZIE)
            
        except Exception:
            try:
                try:
                    rsglobal.ZapiszDoLogow('Bd RELOAD-u caego serwera,'
                                           ' prbuj wrci do ptli gwnej. %s'
                                           % traceback.format_exc(), 1)
                                        #AB 11.02.20r
                    uruchomHarmonogram(rs_anakonda_server_cl.Enterprise)
                    if rs_anakonda_server_cl.Enterprise:
                        uruchomSOAP()
                    uruchomKomser(OPOZNIENIE_RESTARTU_KOMSERA)
                    komunikat = ('Wystapil blad RELOAD-u przechwycony przed'
                                 ' zamknieciem (starego) serwera. Serwer'
                                 ' powinien nadal (po staremu) dzialac.')
                except:
                    rsglobal.ZapiszDoLogow('Nie udao si powrci do ptli'
                                           ' gwnej serwera, BD KRYTYCZNY: %s'
                                           % traceback.format_exc(), 1)
                    komunikat = ('Wystapil blad RELOAD-u przechwycony przed'
                                 ' zamknieciem (starego) serwera. Podjeto probe'
                                 ' powrotu do petli glownej, co niestety sie'
                                 ' nie udalo. BLAD KRYTYCZNY! %s'
                                 % traceback.format_exc())
                    print('BLAD KRYTYCZNY RELOAD-u serwera, moze byc'
                          ' konieczne reczne skillowanie podprocesow itp.')
                    raise
            finally:
                try:
                    # koniecznie zdejmij blokad
                    zdejmijBlokade(deskryptor_pliku_blokady)
                except NameError:
                    pass
                    
                koniecReloadu(False, komunikat)
                
else:
    # cd. RELOAD-u serwera
    # -- std ju nie ma odwrotu
    
    try:
        # wyrejestruj obsug sygnaw
        DeregisterHandlers()

        # dochodzimy do waciwego RELOAD-u...
        komenda_interpretera = rsglobal.PodajZKonf(PLIK_KONFIGURACJI,
                                                   ['pythonprogram'])[0]
        if not komenda_interpretera:
            komenda_interpretera = DOMYSLNA_KOMENDA_ITERPRETERA
        argumenty = (komenda_interpretera,
                     SCIEZKA_TEGO_SKRYPTU,
                     '--reload=%s,%s' % (se_read, se_write),
                     '--blokada=%s' % deskryptor_pliku_blokady,
                     '--stan=%s' % sciezka_pliku_stanu)
        rsglobal.ZapiszDoLogow('Uruchamianie serwera od nowa: '
                               + ' '.join(argumenty), 5)
        # KLUCZOWY MOMENT:
        # zastp aktualnie wykonywany program programem wczytywanym od nowa
        # (z zachowaniem tosamoci procesu i -- co kluczowe -- majc zapisany
        # stan, ktry nowowczytany program bdzie mg odtworzy):
        os.execvp(komenda_interpretera, argumenty)
        
    except:
        zdejmijBlokade(deskryptor_pliku_blokady)  # koniecznie zdejmij blokad
        traceback_info = traceback.format_exc()
        rsglobal.ZapiszDoLogow('Nienaprawialny bd RELOAD-u caego serwera'
                               ' juz po jego wyczeniu a przed ponownym'
                               ' uruchomieniem: %s' % traceback_info, 1)
        koniecReloadu(False, 'BLAD KRYTYCZNY przy probie RELOAD-u, juz'
                             ' po wylaczeniu serwera a przed jego ponownym'
                             ' uruchomieniem! %s' % traceback_info)
        print('BLAD KRYTYCZNY RELOAD-u serwera, moze byc'
              ' konieczne reczne skillowanie podprocesow itp.')
        raise
