
# -*- coding: iso8859_2 -*-

#-----------------------------------------------------------------------------
# Name:        rs_anakonda_server_cl.py
# Purpose:     proces serwera poredniczacego pomidzy PostgreSQL a klientem
#              RSAnakonda do obslugi polaczenia klienckiego
#
# Authors:      Michal Raburski, Aleksandra Kaminska
# Some changes: Jan Raburski
# Created:     2004/03/06
# RCS-ID:      $Id: rs_anakonda_server_cl.py $
# Copyright:   (c) 2004 Record System
# Licence:     GNU GPL
#-----------------------------------------------------------------------------
# Dariusz Boduch, 2009-02-04:
# * popr. literwki w komunikacie (mantis: 860)
# Jan Kaliszewski, 2009-06-25:
# * usuwanie procesow zwizanych z przerwanym poczeniem (mantis: 1904)
# Jan Kaliszewski, 2009-07-09:
# * PolSQL2 - raporty z serwera zapas. (mantis: 2191)
# Jan Kaliszewski, 2009-08-19:
# * zmiany dot. RELOAD-u serwera (mantis: 2109)
# Jan Kaliszewski, 2011-05-05:
# * zmiany dot. nowej implementacji aktualizacji klienta
# Aleksander Bargie, 2019-10-25:
# * Wywietlanie harmonogramu dla pracownika  (11714)
# Aleksander Bargie, 2019-11-14:
# * Krytyczny bd poczenia po rozczeniu z serwerem (11738)
# Aleksander Bargie, 2021-01-18:
# * [AD] Blokada logowania operatora z niepoprawnym kontrahentem. (12269)
# Aleksander Bargie, 2021-01-19:
# * Doda wywietlanie symbolu absencji w Harmonogramach wywietlanych na terminalu. (12273)
# Aleksander Bargie, 2021-01-19:
# * Wizualizacja karty przyjcia w przyjciach artykuu. Dodanie wpisu w sowniku dodatkowe_uprawniania (12555)
# Aleksander Bargie, 2021-07-15:
# * Wywietlanie harmonogramu na panelu - bra pod uwag grafik rzeczywisty jeli istnieje  (12573)
# Wojciech Skowron, 2021-11-03:
# * Poprawka obsugi plikw w funkcji StoperStop (12174)
# Aleksander Bargie, 2022-02-23:
# * Zmiany w aplikacji do podgldu harmonogramu po zmianach w Anakondzie  (12902)
#  Aleksander Bargie, 2022-04-06:
# * Zmiany w harmonogramie (12941)
#  Aleksander Bargie, 2022-06-22:
# * Harmonogram dla pracownikw (13069)
#  Aleksander Bargie, 2022-07-26:
# * Terminal z godzinami pracownika - wywietla godziny nadliczbowe w formacie 24 godzinnym nie 12 (13109)

import collections
import socket
import os, os.path
import sys
import rsglobal		# rsglobal.ZapiszDoLogow(), rsglobal.PodajZKonf()
import pickle
import binascii
import traceback
import rssql
from rs_anakonda_socket	import *	# dedykowana obsluga socketow TCP
from const import *	# configuration as well as operations' constants
from signal import *	# SIG*, signal
import time
import datetime
import string
import rs_anakonda_server_fn   # obsluga funkcji na serwerze
import rs_anakonda_server_ra   # obsluga raportow na serwerze
from rs_anakonda_server_update import *  # obsluga autoupdate'u na serwerze (star metod)
from rs_anakonda_server_update_new import wykonajAktualizacje  # obsluga autoupdate'u na serwerze
import rs_anakonda_socket_ssl
import rs_process_killer

connection=None
PolSQL = None
PolSQL2 = None
idwejscia = None
zamykanie_serwera = False   # ustawiane gdy dostanie SIGUSR2 lub SIGALRM, wysyane przez gwny proces serwera
trwa_wylaczanie = False     # ustawiane po otrzymaniu sygnalu do zamkniecia

zerwano_polaczenie = False  # ustawiane przed samokillowaniem
                            # z powodu utraty poczenia z klientem
kat_serwera  = None  # ustawiane w rs_anakonda_server
Enterprise = False   # informacja czy uytkownik posiada wersj  Enterprise i czy moe korzysta z odpowiednich funkcji
# kolejki do komunikacji midzy kodem uruchamiajcym potomne procesy
# a akcj po zerwaniu poczenia:
komunikacja = (collections.deque(), collections.deque())  # (potomkowie, zatrzymaj_po)

# wczanie zbierania statystyk wykonania funkcji i raportw
stat_fun = False
stat_rap = False
# uprawnienia potrzebne: [uprawnienia_dajace_dostep, ]
# np. '012010': ['092200'] oznacza, e do fsa wymagajacego 012010 mozna sie dostac posiadajac numer uprawnien 092200
# to obsluguje tylko i wylacznie odpalanie fsow, nie rozszerza self.uprawnienia wykorzystywanego w funkcjach
dodatkowe_uprawnienia = {'012010': ['092200'],
                         '012000': ['092200'],
                         '032020': ['013050']  #AB 30.06.21r
                         }

########################################################################
#             Funkcje pomocnicze    #
########################################################################

def PodajListeDzialow(Polaczenie,Operator):
    if Operator == STANDARD_OPERATOR_ID:
        return 'Wszystkie'
    try:
        select="SELECT lista_dzialow FROM rs_operatorzy WHERE operator_id = "+str(Operator)+";"
        stat,d,dump,dump =Polaczenie.Execute(select,2)
        rsglobal.ZapiszDoLogow(d,5)
        if stat=='OK':
            ListaDzialow=''
            for dz in d:
                ListaDzialow+=dz[0]+","
        return ListaDzialow.strip(',')
    except Exception,e:
        return ''
        rsglobal.ZapiszDoLogow("BLEDY W LICIE DZIAW!!! "+str(e), 1)

#-------------------------------------------------------------------------------

def PodajOperatorow(Polaczenie,Operator):
    if Operator == STANDARD_OPERATOR_ID:
        return ()
    Operatorzy=[Operator]
    try:
        d=[]
        select="SELECT operator_id from rs_operatorzy WHERE zastepca_id="+str(Operator)+";"
        st,d,dump,dump =Polaczenie.Execute(select,2)
        rsglobal.ZapiszDoLogow(d,5)
        if st=='OK':
            for op in d:
                Operatorzy.append(op[0])
        return Operatorzy
    except Exception,e:
        return ()
        rsglobal.ZapiszDoLogow("BLEDY W UPRAWNIENIACH I ZASTEPSTWACH!!! "+str(e), 1)

#-------------------------------------------------------------------------------

def PodajHandlowca(Polaczenie,Operator):
    if Operator == (STANDARD_OPERATOR_ID):
        return None
    try:
        rsglobal.ZapiszDoLogow('PodajHandlowiec',5)
        select="SELECT handlowiec_id FROM rs_operatorzy WHERE operator_id="+str(Operator)+";"
        st,d,dump,dump =Polaczenie.Execute(select,2)
        rsglobal.ZapiszDoLogow(d,5)
        if st=='OK':
            if len(d)>0:
                if d[0][0]<>'None':
                    return rsglobal.Dekoduj(d[0][0])
                else:
                    return None
        return None
    except Exception,e:
        rsglobal.ZapiszDoLogow("BLEDY W Handlowcach w operatorach!!! "+str(e), 1)

#-------------------------------------------------------------------------------

def PodajUprawnienia(Polaczenie,Operatorzy,Firma):
    if int(Operatorzy[1]) == int(STANDARD_OPERATOR_ID):
        return []
    Upr= []
    try:
        d=[]
        rsglobal.ZapiszDoLogow("ROZPOCZYNAM UPRAWNIENIA", 5)
        # Operatorzy - lista - operator, ktry sie zalogowal plus wszystkie zastepstwa ktore ma

        stat, dane, dump, dump = Polaczenie.Execute("SELECT grupa_id FROM rs_grupy WHERE firma_id='"+Firma+"' AND grupa_id != '**';",2)
        WszystkieGrupy=[]
        for i in dane:
            WszystkieGrupy.append(i[0])

        stat, dane, dump, dump = Polaczenie.Execute("SELECT oddzial_id FROM rs_oddzialy WHERE firma_id='"+Firma+"' AND oddzial_id != '**';",2)
        WszystkieOdd=[]
        for  i in dane:
            WszystkieOdd.append(i[0])
            Upr.append((i[0],'000000'))
        stat, dane, dump, dump = Polaczenie.Execute("SELECT DISTINCT oddzial_id,grupa_id FROM rs_grupa_operator_oddzial WHERE operator_id in "+Operatorzy+" AND firma_id='"+Firma+"';",2)
        GRO=[]
        for gro in dane:
            oddzial = gro[0]
            grupa = gro[1]
            if grupa == '**' and oddzial== '**':
                for gr in WszystkieGrupy:
                    for odd in WszystkieOdd:
                        if not (odd,gr) in GRO:
                            GRO.append((odd,gr))
            elif grupa =='**':
                for gr in WszystkieGrupy:
                    if not (oddzial,gr) in GRO:
                            GRO.append((oddzial,gr))
            elif oddzial == '**':
                for odd in WszystkieOdd:
                    if not (odd,grupa) in GRO:
                        GRO.append((odd,grupa))
            else:
                if not (oddzial,grupa) in GRO:
                    GRO.append((oddzial,grupa))

        GRO.sort()
        oddz=GRO[0][0]
        listagr=[]
        for og in GRO:
            if oddz!=og[0]:
                listagr=str(listagr).replace("[","(").replace("]",")")
                stat, Upraw, dump, dump = Polaczenie.Execute("SELECT DISTINCT menu_id FROM rs_grupa_menu WHERE grupa_id in %s AND firma_id='%s';" % (listagr,Firma),2)
                #rsglobal.ZapiszDoLogow("UPR GR "+gr+' '+str(Upraw), 5)
                for PunktMenu in Upraw:
                    klucz = (oddz,PunktMenu[0])
                    Upr.append(klucz)
                oddz=og[0]
                listagr=[]
            gr=og[1]
            if str(type(gr))=="<type 'unicode'>":
                gr=gr.encode('iso-8859-2')
            listagr.append(gr)
        listagr=str(listagr).replace("[","(").replace("]",")")
        stat, Upraw, dump, dump = Polaczenie.Execute("SELECT DISTINCT menu_id FROM rs_grupa_menu WHERE grupa_id in %s AND firma_id='%s';" % (listagr,Firma),2)
        for PunktMenu in Upraw:
            klucz = (oddz,PunktMenu[0])
            Upr.append(klucz)

        rsglobal.ZapiszDoLogow(str(Operatorzy),5)
        rsglobal.ZapiszDoLogow('Uprawnienia',5)
        rsglobal.ZapiszDoLogow(str(Upr),5)

        rsglobal.ZapiszDoLogow("KONIEC UPRAWNIEN ", 5)
        return Upr
    except Exception,e:
        rsglobal.ZapiszDoLogow("BLEDY W UPRAWNIENIACH!!! \n"+traceback.format_exc(), 1)

#-------------------------------------------------------------------------------

########################################################################
#    Funkcje pomocnicze dot. obsugi sygnaw i zamykania procesw     #
########################################################################

def _zamknij_polaczenie():
    global connection
    if PolSQL != None:
        rsglobal.ZapiszDoLogow('Lista parametrw:'+str(PolSQL.SelectForUpdate),3)
        for x in ['DokMag','Fakt','Kor','DowDost','ZamOdb','ParagFisk','DowKsieg','ZamDost','RapWyc','Dostawy']:
            try:
                del PolSQL.SelectForUpdate[x]
                rsglobal.ZapiszDoLogow(u'Usunicie blokady: '+str(x),3)
                PolSQL.Rollback()
                rsglobal.ZapiszDoLogow(u'I ROLLBACK po blokadzie: '+str(x),3)

            except:
                pass

    if connection:
        try:
            connection.shutdown(1)
            rsglobal.ZapiszDoLogow("Zamkniete polaczenie klienckie", 3)
        except socket.error, exc:
            # 107 - "Transport endpoint is not connected" - ju rozczone
            if exc.args[0] != 107:
                raise
        connection = None
    else:
        rsglobal.ZapiszDoLogow("Brak poczenia - nie ma czego zatrzymywa", 3)

#-------------------------------------------------------------------------------

def _skilluj_potomne_procesy():

    global komunikacja
    potomkowie, zatrzymaj_po = komunikacja  # <- obiekty collections.deque()
    zatrzymaj_po.append(True)  # <- wskazwka dla kodu odpalajcego proces potomny,
                               #    by po zakoczeniu procesu stan (przysn).
    wypisz = lambda tekst: rsglobal.ZapiszDoLogow(tekst, 5)
    wypisz_blad = lambda tekst: rsglobal.ZapiszDoLogow(tekst, 1)
    for pid in tuple(potomkowie):
        rs_process_killer.kill(pid, sygnaly=(SIGUSR1, SIGTERM, SIGKILL),
                              opoznienie=0, odstep=rsglobal.ODSTEP_SPR_KILLOWANIA,
                              wypisz=wypisz, wypisz_blad=wypisz_blad)

#-------------------------------------------------------------------------------

def _przygotuj_akcje_na_zerwanie_polaczenia():

    pid = os.getpid()

    def akcja_w_razie_zerwania():
        global zerwano_polaczenie

        # dajemy czas normalnym akcjom na zaatwienie sprawy po swojemu:
        time.sleep(5)

        zerwano_polaczenie = True

        # najpierw wyczamy potomne procesy:
        _skilluj_potomne_procesy()

        _zamknij_polaczenie()  # <- niekoniecznie potrzebne, ale raczej nie zawadzi

        # potem wyczamy sam proces (zwizany z konkretnym poczeniem),
        # zlecamy to z odrbnemu procesowi-kilerowi:
        skrypt = 'rs_process_killer.py'
        sciezka = os.path.join(os.getcwd(), skrypt)
        opcje = [skrypt,
                 '--pid=%s' % pid,
                 '--signals=%s,%s,%s' % (SIGUSR1, SIGTERM, SIGKILL),
                 '--predelay=3',
                 # odstp (w sekundach) midzy killowaniem
                 # a sprawdzaniem czy si udao:
                 '--interwal=%s' % rsglobal.ODSTEP_SPR_KILLOWANIA,
                ]
        if rsglobal.LogLevel >= 5:
            opcje.append('--logfile=%s-kiler' % rsglobal.LogFile)
        else:
            opcje.append('--quiet')

        os.spawnv(os.P_NOWAIT, sciezka, opcje)

    return akcja_w_razie_zerwania

#-------------------------------------------------------------------------------

########################################################################
#								       #
#			Obsluga sygnalow     	   #
#								       #
########################################################################

##############################################
# 	rejestracja handlerow sygnalow	     #
##############################################

def RegisterHandlers():
    # przechwycenie sygnalu SIGUSR1 - poprawne STOP serwera
    #                                 lub samokillowanie z powodu utraty
    #                                 poczenia z klientem
    signal(SIGUSR1, TerminationHandler)

    # przechwycenie sygnalu SIGUSR2 - poprawne FORCE serwera
    signal(SIGUSR2, TerminationHandler)

    # przechwycenie sygnalu SIGALRM - niezaplanowane zabicie serwera
    signal(SIGALRM, TerminationHandler)

    # obsluga innych sygnalow ktore zazwyczaj powoduja zakonczenie procesu
    for sig in terminatingSigs:
        signal(sig, AbnormalTerminationHandler)

    return

##############################################
#   handlery sygnalow                        #
##############################################

def TerminationHandler(signum, stackFrame):
    """ handler do obslugi sygnalow powodujacych nieprzewidziany koniec """

    global connection, zamykanie_serwera, trwa_wylaczanie, zerwano_polaczenie

    #print "TerminationHandler", trwa_wylaczanie
    if signum == SIGUSR2 or signum == SIGALRM:  # wyczenie serwera przez FORCE lub SIGTERM
        zamykanie_serwera = True

    if trwa_wylaczanie:
        rsglobal.ZapiszDoLogow("Proces obslugujacy polaczenie klienckie ignoruje sygnal %s - trwa zamykanie" % signum, 1)
        return
    else:
        trwa_wylaczanie = True
        rsglobal.ZapiszDoLogow("Proces obslugujacy polaczenie klienckie przechwycil sygnal %s, konczy sie" % signum, 3)

    if zerwano_polaczenie:
        rsglobal.ZapiszDoLogow("w sygna by wysany w zwizku z utrat poczenia z klientem,"
                               " wic wywoujemy Exit(), a nastpnie os._exit(os.EX_NOINPUT)", 3)
        Exit()                   # <- rcznie wywoujemy podstawowe sprztanie
        os._exit(os.EX_NOINPUT)  # <- i wychodzimy bez gadania
    else:
        _skilluj_potomne_procesy()
        # XXX aby watek kliencki mogl sie zorientowac ze cos nie tak XXX
        try:
            connection.shutdown(2)
        except (socket.error, SSL.Error):
            pass
        #rsglobal.ZapiszDoLogow('sys.exit(9)')
        sys.exit(9)


def AbnormalTerminationHandler(signum, stackFrame):
    global connection, trwa_wylaczanie
    #global mainPid

    #print "AbnormalTerminationHandler", trwa_wylaczanie
    if zerwano_polaczenie:
        trwa_wylaczanie = True
        rsglobal.ZapiszDoLogow("Proces obslugujacy polaczenie klienckie przechwycil sygnal %s, konczy sie" % signum, 3)
        rsglobal.ZapiszDoLogow("w sygna by wysany w zwizku z utrat poczenia z klientem,"
                               " wic wywoujemy Exit(), a nastpnie os._exit(os.EX_NOINPUT)", 3)
        Exit()                   # <- rcznie wywoujemy podstawowe sprztanie
        os._exit(os.EX_NOINPUT)  # <- i wychodzimy bez gadania
    elif trwa_wylaczanie:
        if hasattr(connection, 'czy_sprawdzac'):
            connection.czy_sprawdzac = False
        if signum == SIGTERM:
            rsglobal.ZapiszDoLogow("Proces obslugujacy polaczenie klienckie przechwycil SIGTERM w czasie zamykania, koncze natychmiast", 1)
            os._exit(10)
        else:
            rsglobal.ZapiszDoLogow("Proces obslugujacy polaczenie klienckie ignoruje sygnal %s - zamykanie trwa" % signum, 1)
            return
    else:
        trwa_wylaczanie = True
        rsglobal.ZapiszDoLogow("Proces obslugujacy polaczenie klienckie przechwycil sygnal"+str(signum)+", konczy sie", 3)
        _skilluj_potomne_procesy()

    #os.kill(mainPid, SIGUSR1)
    try:
        connection.shutdown(2)
    except (socket.error, SSL.Error):
        pass
    sys.exit(9)


########################################################################
###     Zwolnienie zasobow systemowych                                 #
########################################################################

# funkcja wykonywana przy sys.exit()
def Exit():
    global connection, PolSQL, PolSQL2, idwejscia, zamykanie_serwera
    #rsglobal.ZapiszDoLogow('Exit()')
    try:
        if hasattr(connection, 'czy_sprawdzac'):
            connection.czy_sprawdzac = False

        if PolSQL and hasattr(PolSQL, "Polaczenie") and PolSQL.Polaczenie:
            rsglobal.ZapiszDoLogow("Awaryjne zamykanie polaczenia z baz danych",3)
            PolSQL.Rollback()
            rsglobal.ZapiszDoLogow("odpalamy zapytanie: ",3)
            rsglobal.ZapiszDoLogow(SQL_INCORRECT_END_CONN % (idwejscia),3)
            if idwejscia:
                if zamykanie_serwera:
                    PolSQL.Execute(SQL_FORCE_END_CONN % (idwejscia),1)
                else:
                    PolSQL.Execute(SQL_INCORRECT_END_CONN % (idwejscia),1)
                PolSQL.Commit()
            PolSQL.RozlaczZSQL()

        if PolSQL2 and hasattr(PolSQL2, "Polaczenie") and PolSQL2.Polaczenie:
            rsglobal.ZapiszDoLogow("Awaryjne zamykanie raportowego polaczenia z baz danych",3)
            PolSQL2.Rollback()
            PolSQL2.RozlaczZSQL()

        _zamknij_polaczenie()

    except Exception, e:
        rsglobal.ZapiszDoLogow("rs_anakonda_server_cl.Exit, wyjatek: "+str(e), 1)
    return


# zwraca dane stopera czasu uywane pniej do zapisania statystyk
def StoperStart(typ, info):
    return (typ, time.time(), info)

# zapisuje statystyki
_byl_blad_zapisu_statystyk = 0
def StoperStop(daneStopera, info_stopu=(), pomin_krotsze_niz=None):
    global _byl_blad_zapisu_statystyk
    try:
        wpis = (daneStopera, info_stopu)
        typ = daneStopera[0]
        czas = time.time() - daneStopera[1]
        if pomin_krotsze_niz is not None and czas < pomin_krotsze_niz:
            #WS 2021-11-03
            with open("statystyki_%s_licznik" % typ, "a") as p:
                p.write(".")
            # open("statystyki_%s_licznik" % typ, "a").write(".")
        else:
            info = daneStopera[2] + info_stopu
            start = datetime.datetime.fromtimestamp(daneStopera[1]).strftime("%Y-%m-%d %T")
            stop = datetime.datetime.now().strftime("%Y-%m-%d %T")
            infos = ";".join(map(str, info))
            wpis = "%s;%s;%s;%s;%.3f\n" % (rsglobal.Oper, infos, start, stop, czas)
            #WS 2021-11-03
            with open("statystyki_%s.csv" % typ, "a") as p:
                p.write(wpis)
            # open("statystyki_%s.csv" % typ, "a").write(wpis)
    except Exception, exc:
        if time.time() - _byl_blad_zapisu_statystyk > 600:
            rsglobal.ZapiszDoLogow(u"Nie mona zapisa statystyk: %s (%s)" % (exc, wpis), 1)
            _byl_blad_zapisu_statystyk = time.time()
    return


########################################################################
#								       #
#			Wlasciwe cialo  procesu			       #
#								       #
########################################################################

def Wykonaj(con,mainPid,addr,LConf=None, SSLContext=None):
    global connection, PolSQL, PolSQL2, idwejscia
    idwejscia = None
    rsglobal.ZapiszDoLogow('idwejscia na None 1')

    # pobranie polaczenia serwisowego
    connection = socket.fromfd(con, socket.AF_INET, socket.SOCK_STREAM)
    connection = AnakondaSocket(socket_obj=connection)

    # pobranie pidu procesu glownego
    #mainPid = int(sys.argv[2])

    #addr=(sys.argv[3], int(sys.argv[4]))

    # funkcja wywolywana m.in. przy sys.exit()
    sys.exitfunc = Exit
    # konfiguracja
    fileName = "rs-and.conf"
    params = ('hostpgsql','portpgsql','logfile','loglevel','logtype','phpprogram','baza',
              'hostpgsql2','portpgsql2','katalogplikow','katalogedeklaracji')
    if LConf==None:
        rsglobal.ZapiszDoLogow('LConf:', 5)
        lista = rsglobal.PodajZKonf(fileName, params)
        hostSQL2=lista[7]
        portSQL2=lista[8]
        KatalogPlikow=lista[9]
        KatalogDeklaracji=lista[10]
    else:
        rsglobal.ZapiszDoLogow('Nie LConf:', 5)
        rsglobal.ZapiszDoLogow('LConf:'.format(repr(LConf)), 5)
        lista=LConf[2:]
        hostSQL2=lista[11]
        portSQL2=lista[12]
        KatalogPlikow=lista[15]
        KatalogDeklaracji=lista[16]
        dbuser = lista[7]
        dbpass = lista[8]

    # dodanie nazwy bazy do katalogu plikw
    if KatalogPlikow in (None,''):
        KatalogPlikow=os.getcwd()+'/pliki'
    if lista[6]=='':
        KatalogPlikow+='/anakonda'
    else:
        KatalogPlikow+='/'+lista[6]


    rsglobal.ZapiszDoLogow('Lista parametrw:'+str(lista),5)
    hostSQL=lista[0]
    portSQL=int(lista[1])
    if portSQL2:
        portSQL2 = int(portSQL2)
    rsglobal.LogFile = lista[2]
    rsglobal.LogLevel = int(lista[3])
    rsglobal.LogType = lista[4]
    phpProgram=lista[5]
    if phpProgram=='':
        phpProgram='php'
    Baza=lista[6]
    if Baza=='':
        Baza='anakonda'

    # synchronizacja poprzez sygnaly
    RegisterHandlers()

    rsglobal.ZapiszDoLogow('Rozpoczecie obslugi polaczenia klienckiego '+str(addr),1)

    dane=connection.recv()

    #jesli przyszla aktualizacja to sie robi a potem czeka
    if dane == "AKTUALIZACJA":
        try:
            rsglobal.ZapiszDoLogow(u'Rozpoczcie aktualizacji '+str(addr),3)

            updChecker = RSUpdateChecker(connection)
            updChecker.StartUpdate()
            rsglobal.ZapiszDoLogow('Koniec aktualizacji '+str(addr),3)
        except Exception,e:
            rsglobal.ZapiszDoLogow(u'Problemy z aktualizacj - '+str(e),1)

        try:
            dane=connection.recv()
        except socket.error:
            rsglobal.ZapiszDoLogow('Utracono polaczenie z '+str(addr),3)
            sys.exit()

    if dane == "SSL":
        rsglobal.ZapiszDoLogow('Klient proponuje przejsc na SSL', 2)
        if SSLContext:
            connection.send("OK")
            connection_ssl = rs_anakonda_socket_ssl.AnakondaSocketSSL(connection, SSLContext)
            if connection_ssl.accept():
                connection = connection_ssl
                #rsglobal.ZapiszDoLogow("SSL powinno dzialac, od teraz uzywam SSL", 3)
            else:
                rsglobal.ZapiszDoLogow("Nie udao si nawiza poczenia ssl", 1)
                sys.exit(1)
        else:
            connection.send("NO")
            dane = connection.recv()

        if isinstance(connection, rs_anakonda_socket_ssl.AnakondaSocketSSL):
            dane = connection.recv()
            if dane == "PING":
                connection.send("PING")
                rsglobal.ZapiszDoLogow("Stan polaczenia: %s" % connection.socket.state_string(), 3)
                dane = connection.recv()
            else:
                rsglobal.ZapiszDoLogow("Stan polaczenia SSL: %s" % connection.socket.state_string(), 3)

    if dane == "NOWA_AKTUALIZACJA":
        try:
            rsglobal.ZapiszDoLogow(u'Rozpoczecie aktualizacji %s nowa metoda' % (addr,), 3)
            wykonajAktualizacje(connection, kat_serwera)
            rsglobal.ZapiszDoLogow(u'Koniec aktualizacji %s nowa metoda' % (addr,), 3)
        except Exception:
            rsglobal.ZapiszDoLogow(u'Problemy z aktualizacja nowa metoda - %s'
                                   % traceback.format_exc(), 1)
            sys.exit(1)
        dane=connection.recv()

    rsglobal.ZapiszDoLogow("Otrzymalem: %r" % dane, 4)

    #AB 14.11.19r dodana obsuga 'Koniec'
    if dane != 'KONIEC':
        dane=rsglobal.rseval(dane)
        rsglobal.ZapiszDoLogow("Eval na : %r" % dane, 4)

    PolSQL = rssql.RSSQL()
    lprob = 0

    if len(dane) == 3:
        if PolSQL.PolaczZSQL(dbuser, dbpass, hostSQL, portSQL, None, Baza=Baza) == 0:
            firma = dane[0]
            pakiety = dane[1]
            uid = dane[2]
            sql = """SELECT imie || ' ' || nazwisko AS pracownik 
            FROM rs_kp_dane_osobowe 
            WHERE firma_id = '{}' AND uid_karty = '{}';""".format(firma, uid)
            stat, Prac, dump, dump = PolSQL.Execute(sql, 3)
            if stat != 'OK':
                rsglobal.WyslijPakietami(connection, pakiety, stat)
                sys.exit(1)
            elif not Prac:
                rsglobal.WyslijPakietami(connection, pakiety, 'PUSTO')
                sys.exit(1)
            #AB 15.07.21r uwzgldznienie grafiku rzeczywistego
            #AB 14.12.21r dodanie nieobecnoci fakt i daty nieobecnosc_za
            #AB 26.07.22r wywietla godziny nadliczbowe w formacie 24 godzinnym nie 12
            sql = """SELECT CASE WHEN fakt_nieobecnosc THEN fakt_nieobecnosc_id  ||CASE WHEN nieobecnosc_za IS NOT NULL THEN (' za ' || nieobecnosc_za::TEXT) ELSE '' END
                                 WHEN NOT fakt_nieobecnosc AND fakt_czas_do IS NOT NULL THEN '' 
                                 ELSE COALESCE(nieobecnosc_id, '')||CASE WHEN nieobecnosc_za IS NOT NULL THEN (' za ' || nieobecnosc_za::TEXT) ELSE '' END END                          
                         ||'$;'|| 
                        CASE WHEN fakt_czas_do IS NULL THEN 
                            CASE WHEN plan_czas_od != plan_czas_do
                            THEN to_char(plan_czas_od, 'YYYY-MM-DD HH24:MI') || ' - ' || to_char(plan_czas_do, 'HH24:MI')
                            ELSE to_char(plan_czas_od, 'YYYY-MM-DD ') || '-' END
                        ELSE
                            CASE WHEN  fakt_czas_od IS NULL THEN 
                                to_char(plan_czas_od, 'YYYY-MM-DD ')
                            ELSE
                                CASE WHEN fakt_czas_od != fakt_czas_do
                                THEN to_char(fakt_czas_od, 'YYYY-MM-DD HH24:MI') || ' - ' || to_char(fakt_czas_do, 'HH24:MI')
                                ELSE to_char(fakt_czas_od, 'YYYY-MM-DD ') || '-' END 
                            END
                        END||'$;'||COALESCE(to_char(godz_nadliczbowe_od, 'HH24:MI'), '')||'-'||COALESCE(to_char(godz_nadliczbowe_do, 'HH24:MI'), '')
                        AS zmiana
FROM rs_kp_grafiki_pozycje AS rkgp 
WHERE firma_id = '{}' 
    AND pracownik_id = (SELECT pracownik_id FROM rs_kp_dane_osobowe AS rkdo WHERE firma_id = rkgp.firma_id AND uid_karty = '{}')
    AND plan_czas_od::DATE >= (to_char(NOW(), 'YYYY-01-01')::DATE)
    AND plan_czas_od::DATE < (CASE WHEN EXTRACT('DAY' FROM NOW()) >= 20 THEN (to_char(NOW() + INTERVAL '2 MONTH', 'YYYY-MM-01')::DATE) ELSE (to_char(NOW() + INTERVAL '1 MONTH', 'YYYY-MM-01')::DATE) END)
ORDER BY plan_czas_od
;""".format(firma, uid)  #AB 22.06.22r
            stat, Harm, dump, dump = PolSQL.Execute(sql, 3)
            if stat != 'OK':
                rsglobal.WyslijPakietami(connection, pakiety, stat)
                sys.exit(1)
            else:
                rsglobal.ZapiszDoLogow("Harm: {}".format(Harm), 5)
                rsglobal.WyslijPakietami(connection, pakiety, [Prac[0][0], Harm] or 'PUSTO{}'.format(Prac[0][0]))
                sys.exit(1)
            # connection.send(repr(Harm or 'PUSTO'))
        else:
            rsglobal.ZapiszDoLogow(u'Problemy z polaczeniem z baza jako harmonogram - %s' % traceback.format_exc(), 1)
            sys.exit(1)

    else:

        #pobranie danych o polaczeniu
        #Jesli klient zerwie polaczenie w tym mommencie to serwer rzuci wyjatkiem o niemoznosci nasluchiwania. Wyjatek socket.error
        try:
            #Sprawdzenie czy nie przyszo polecenie 'KONIEC' (wyczenie programu bez logowania)
            if dane=='KONIEC':
                rsglobal.ZapiszDoLogow('Zaczynam obsluge KONIEC - bez zapisu w tabeli pocze',5)
                connection.send(RECEIVED)
                rsglobal.ZapiszDoLogow('Koniec obslugi KONIEC',5)
                time.sleep(3)
                rsglobal.ZapiszDoLogow('Zamykanie polaczenia z klientem z '+str(addr),3)
                sys.exit()
        except Exception,e:
            rsglobal.ZapiszDoLogow('Problemy z socketem - '+str(e),1)
            sys.exit()
        else:
            Operator = str(dane[0])
            Haslo = str(dane[1])
            WielkPakietu = int(dane[2])
            System = dane[3]
            if (not rsglobal.CzyPrawidlowyLogin(Operator)) or (rsglobal.SprawdzPolskieZnaki(Operator)) or (not Haslo.isalnum()) or (rsglobal.SprawdzPolskieZnaki(Haslo)):
                connection.send('-4')
                rsglobal.ZapiszDoLogow('Zamykanie polaczenia, z powodu bdnego loginu/hasa (polskie znaki)',1)
                connection.close()
                rsglobal.ZapiszDoLogow('Polaczenie zamkniete',1)
                sys.exit()

        #proba polaczenia z baza danych
        Wynik = PolSQL.PolaczZSQL(Operator, Haslo,hostSQL,portSQL,System,Baza)
        if  Wynik > 0:
            connection.send(str(-1*Wynik))
            rsglobal.ZapiszDoLogow('Zamykanie polaczenia, z powodu niepolaczenia z baza SQL',1)
            # nie wymagamy natychmiastowego zwolnienia polaczenia
            connection.close()
            rsglobal.ZapiszDoLogow('Polaczenie zamkniete',1)
            # danie znac glownemu procesowi serwera, ze nastepuje zakonczenie
            # procesu
            #os.kill(mainPid, SIGUSR1)
            rsglobal.ZapiszDoLogow('Zabity proces',1)
            sys.exit()
            #rsglobal.ZapiszDoLogow('Exit(1)',1)

        if Operator == STANDARD_OPERATOR:
            OpId = STANDARD_OPERATOR_ID
        else:
            stat,OpId,dump,dump_ =PolSQL.Execute(SQL_GET_USER, 4)
        rsglobal.ZapiszDoLogow('Poczony z baz danych jako %s' % Operator,3)
        #jesli jest user w bazie, ale nie w tabeli rs_operatorzy
        if OpId==-10:
            connection.send('-10')
            rsglobal.ZapiszDoLogow('Brak operatora '+Operator+' w tabeli rs_operatorzy mimo, e udao si poczenie z baz danych',1)
            sys.exit()

        if OpId==-11:
            connection.send('-11')
            rsglobal.ZapiszDoLogow('Operator '+Operator+' jest nieaktywny, nie udao si poczenie z baz danych',1)
            sys.exit()

        #rsglobal.ZapiszDoLogow("!!! Dalej",1)
        stat, dane, dump, dump = PolSQL.Execute("SELECT czy_wielokrotne, imie || ' ' || nazwisko FROM rs_operatorzy WHERE operator_id=%s;" % (str(OpId)),2)
        imienazwisko = dane[0][1]
        if not dane[0][0]:
            stat, dane, dump, dump = PolSQL.Execute("SELECT op_id FROM rs_polaczenia WHERE op_id=%s AND koniec IS NULL;" % (str(OpId)),2)
            if len(dane) > 0:
                OpId = -2  #nie mona wielokrotnych pocze
        # sprawdzenie czy aktualnie nie ma rozgoszenia zamykajcego (10 min)
        stat, dane, dump, dump = PolSQL.Execute("SELECT priorytet FROM rs_wiadomosci WHERE priorytet = 'Z' AND age(now(),czas_zapisu) < time \'0:10\';",2)
        if len(dane) > 0:
            rsglobal.ZapiszDoLogow('Nie mona si poczy bo serwer czeka na wyczenie',1)
            OpId = -3  #nie mozna sie polaczyc, bo serwer czeka na wylaczenie

        LOperatorow=[int(OpId)]
        if int(OpId)>0:
            rsglobal.ZapiszDoLogow(u'SPRAWDZANIE ZASTPSTWA',5)
            stat, dane, dump, dump = PolSQL.Execute("SELECT zastepca_id FROM rs_operatorzy WHERE operator_id = "+str(OpId)+" AND (zastepca_id!=0 AND zastepca_id is not NULL);",2)
            if stat<>'OK':
                rsglobal.ZapiszDoLogow(u'Bdy pobierania zastpcy',1)
            elif len(dane) > 0:
                rsglobal.ZapiszDoLogow(u'Nie mona si poczy bo operator ma zastpstwo',1)
                OpId = -5  #nie mona si�polaczyc, bo serwer ma zastepstwo
            else:
                #mamy operatora. Pobieramy numery operatorw dla ktrych jest on zastpc
                stat, dane, dump, dump = PolSQL.Execute("SELECT operator_id FROM rs_operatorzy WHERE zastepca_id = "+str(OpId)+" and zastepca_id>0;",2)
                if stat<>'OK':
                    rsglobal.ZapiszDoLogow('Bdy pobierania zastpstw',1)
                else:
                    for op in dane:
                        LOperatorow.append(op[0])

        Operatorzy='('+str(LOperatorow).strip('[]')+')'
        rsglobal.ZapiszDoLogow('Operatorzy='+Operatorzy,5)

        # Tworzymy ewentualne osobne poczenie do raportw:
        if hostSQL2 or portSQL2:  # ('or' - eby wyapa bd podania tylko jednego z nich)
            PolSQL2 = rssql.RSSQL()
            if PolSQL2.PolaczZSQL(Operator, Haslo, hostSQL2,
                                  portSQL2, System, Baza) > 0:
                connection.send(BLPOLSQL)
                rsglobal.ZapiszDoLogow('Zamykanie polaczenia, z powodu niepolaczenia'
                                       ' z baza SQL - dot. dodatkowego osobnego'
                                       ' poczenia SQL dla raportw', 1)
                connection.close()
                rsglobal.ZapiszDoLogow('Polaczenie zamkniete',1)
                rsglobal.ZapiszDoLogow('Zabity proces',1)
                sys.exit()
        else:

            PolSQL2 = None

        rsglobal.Oper=str(OpId)
        connection.send(str(OpId))

        _ostatnie_polecenie = "*INIT*"
        #	wstawienie rekordu do bazy polaczen
        stat,pobrane_id,dump,dump_ = PolSQL.Execute(SQL_MAX_CONN, 3)
        if stat != 'OK':
            idwejscia = None
            rsglobal.ZapiszDoLogow("Blad pobrania nowego id polaczenia: "+ stat)
        else:
            rsglobal.ZapiszDoLogow('idwejscia na {}'.format(int(pobrane_id[0][0])))
            idwejscia = int(pobrane_id[0][0])

        stat,dump,dump,dump_ = PolSQL.Execute(SQL_INSERT_CONN % (idwejscia, OpId,addr[0]), 1)

        if stat != 'OK':
            rsglobal.ZapiszDoLogow("Blad dodania nowego polaczenia: "+ stat)
        try:
            # 	zachowanie zmian i ustawienie pustej firmy
            PolSQL.Commit()
            firma = ''
            oddzial=''
            okres=''
            Upr=[]

            # inicjacja klasy obslugujacej funkcje
            funkcje = rs_anakonda_server_fn.RSFunkcje(OpId, PolSQL, addr, Operatorzy, PolSQL2, Enterprise)
            funkcje.SetKatalogPlikow(KatalogPlikow)
            funkcje.SetKatalogEdeklaracji(KatalogDeklaracji)

            # inicjacja klasy obslugujacej raporty
            dod_serwer_rap = PolSQL2 is not None
            agata = rs_anakonda_server_ra.RSAgata(OpId,PolSQL,addr,imienazwisko,phpProgram,Baza,dod_serwer_rap)

            transakcja = False
            # 	obsluga zadan w nieskonczonej petli

            # killowanie w razie zerwania poczenia, gdy proces jest zajty czym innym:

            connection.wlacz_sprawdzanie_polaczenia(
                  _przygotuj_akcje_na_zerwanie_polaczenia()
            )

            while 1:
                # pobranie kolejnego polaczenia
                try:
                    rsglobal.ZapiszDoLogow('Czekam na dane',5)
                    data = rsglobal.OdbierzPakietami(connection,WielkPakietu)
                    #data = connection.recv()
                    rsglobal.ZapiszDoLogow('Przyszlo '+str(data),5)
                except Exception:
                    rsglobal.ZapiszDoLogow('Zaczynam obsluge BLAD SOCKETU',5)
                    PolSQL.Rollback()
                    PolSQL.Execute(SQL_INCORRECT_END_CONN % (idwejscia),1)
                    idwejscia = None
                    rsglobal.ZapiszDoLogow('idwejscia na None 3')
                    PolSQL.Commit()
                    if PolSQL2:
                        PolSQL2.Rollback()
                    rsglobal.ZapiszDoLogow('Koniec obslugi BLAD SOCKETU',5)
                    time.sleep(3)
                    break
                rsglobal.ZapiszDoLogow('POLECENIE:'+data,5)
                _ostatnie_polecenie = data


                if data[0:9] == "FUNCTION2":
                    ### prby z potomkiem typu agata
                    ##skrypt = 'petla.bash'
                    ##sciezka = os.path.join(os.getcwd(), '../..', skrypt)
                    ##pid = os.spawnlp(os.P_NOWAIT, sciezka, skrypt)
                    ##if komunikacja:
                    ##    potomkowie, zatrzymaj_po = komunikacja  # 2 obiekty collections.deque()
                    ##    potomkowie.append(pid)
                    ##    os.waitpid(pid, 0)
                    ##    potomkowie.remove(pid)
                    ##    if zatrzymaj_po:
                    ##        # (jeli niepusta, id spa)
                    ##        while True: time.sleep(10)
                    ##else:
                    ##    os.waitpid(pid, 0)
                    # Funkcja - nazwa funkcji, serwer sprawdza, czy operator ma uprawnienia do wykonania funkcji
                    #           i czy parametry sa poprawne
                    # ListaKluczy - lista wartosci pol kluczowych, ktorych dotyczy zapytanie, zalezna od funkcji
                    # ListaPol - lista nazw pol (atrybutow), ktorych wartosci sa w LiscieWartosci, stosowane tylko
                    #               w wypadku funkcji wykonujacych INSERT i UPDATE
                    # ListaWartosci - lista wartosci. w przypadku, gdy ListaPol!=None kolejnosc musi odpowiadac kolejnosci pol.
                    #               w pozostalych przypadkach zawiera liste dodatkowych wartosci wymaganych przez funkcje
                    # test uprawnien, zwrot typu
                    rsglobal.ZapiszDoLogow('Zaczynam obsluge FUNCTION2',5)
                    Funkcja = ''
                    Klucz = []
                    Pola = []
                    Dane = []
                    klucz_w_liczniku = None

                    Reszta = rsglobal.rseval(data[9:])

                    if isinstance(Reszta, dict):
                        # obsluga parametrow przekazanych jako slownik
                        if 'funkcja_fs' in Reszta:
                            Funkcja = Reszta['funkcja_fs']

                        if 'klucze' in Reszta:
                            Klucz = Reszta['klucze']
                            if 'Funkcja' in Klucz:
                                klucz_w_liczniku = Klucz['Funkcja']

                        if 'pola' in Reszta:
                            Pola = Reszta['pola']

                        if 'wartosci' in Reszta:
                            Dane = Reszta['wartosci']

                    else:
                        # dotychczasowa obsluga parametrow
                        if len(Reszta) > 0:
                            Funkcja = Reszta[0]

                        if len(Reszta) > 1:
                            Klucz = Reszta[1]
                            if len(Klucz) > 0:
                                klucz_w_liczniku = Klucz[-1]

                        if len(Reszta) > 2:
                            Pola = Reszta[2]

                        if len(Reszta) > 3:
                            Dane = Reszta[3]

                    if Funkcja == '':
                        # nie podana funkcja
                        rsglobal.WyslijPakietami(connection,WielkPakietu,['X',[],[],[]])
                        continue

                    if stat_fun:
                        stoper = StoperStart("fun", (Funkcja, klucz_w_liczniku))

                    rsglobal.ZapiszDoLogow('Sprawdzenie czy funkcja '+Funkcja+' jest na liscie',5)
                    sql = "SELECT typ,menu from rs_funkcje WHERE funkcja='"+Funkcja+"';"
                    stat,dane,dump,dump = PolSQL.Execute(sql,2)

                    if len(dane) == 0:
                        #funkcji nie ma na liscie
                        rsglobal.ZapiszDoLogow('BRAK FUNKCJI '+Funkcja,5)
                        rsglobal.WyslijPakietami(connection,WielkPakietu,[u'Brak definicji funkcji '+Funkcja,[],[],[]])
                        continue

                    rsglobal.ZapiszDoLogow('Sprawdzenie typu funkcji',5)
                    typ = dane[0][0]
                    if not typ in ['T','I','L','D','N']:
                        rsglobal.WyslijPakietami(connection,WielkPakietu,['Niepoprawny typ funkcji '+Funkcja+' w definicji',[],[],[]])
                        continue

                    rsglobal.ZapiszDoLogow('Typ '+typ,5)
                    menuid = dane[0][1]
                    #Sprawdzenie uprawnien dla funkcji
                    if OpId != STANDARD_OPERATOR_ID:
                        if (not (oddzial,menuid) in Upr) and (not ('**',menuid) in Upr) and menuid<>'000000':
                            if menuid in dodatkowe_uprawnienia:
                                brak_upr = True
                                for dodatkowe_id in dodatkowe_uprawnienia[menuid]:
                                    if ((oddzial, dodatkowe_id) in Upr) or (('**', dodatkowe_id) in Upr):
                                        brak_upr = False
                                if brak_upr:
                                    #Brak uprawnien
                                    rsglobal.WyslijPakietami(connection,WielkPakietu,['Brak uprawnie do wykonania funkcji '+Funkcja,[],[],[]])
                                    continue
                            else:
                                #Brak uprawnien
                                rsglobal.WyslijPakietami(connection,WielkPakietu,['Brak uprawnie do wykonania funkcji '+Funkcja,[],[],[]])
                                continue

                    stat,dane,wyn1,wyn2 = funkcje.WykonajFunkcje(Funkcja,Klucz,Pola,Dane)

                    if stat_fun:
                        StoperStop(stoper, pomin_krotsze_niz=1)

                    rsglobal.ZapiszDoLogow('Dane '+str(dane),5)
                    rsglobal.ZapiszDoLogow('Stat '+str(stat),5)

                    rsglobal.WyslijPakietami(connection,WielkPakietu,[stat,dane,wyn1,wyn2])

                    rsglobal.ZapiszDoLogow('Koniec FUNCTION2',5)


                elif data[0:6] == "RAPORT":
                    # Raport - nazwa raportu, serwer sprawdza, czy operator ma uprawnienia do wykonania raportu
                    # ListaKluczy - lista wartosci pol kluczowych, ktorych dotyczy raport, zalezna od raportu
                    # Typ - typ raportu ('pdf','html','sxw','txt','csv','xml') - jesli brak domyslnie 'txt'
                    # test uprawnien, zwrot typu
                    rsglobal.ZapiszDoLogow('Zaczynam obsluge RAPORT',5)
                    Raport = ''
                    Klucz = []

                    Reszta = rsglobal.rseval(data[6:])
                    if len(Reszta) > 0:
                        Raport = Reszta[0]
                    if len(Reszta) > 1:
                        Klucz = Reszta[1]
                    if len(Reszta) > 2:
                        Typ = Reszta[2]
                    else:
                        Typ='txt'

                    if Raport == '':
                        # nie podana nazwa raportu
                        rsglobal.WyslijPakietami(connection,WielkPakietu,['Niepoprawna nazwa raportu',[]])
                        continue

                    if stat_rap:
                        stoper = StoperStart("rap", (Raport, ))

    ##                rsglobal.ZapiszDoLogow('Sprawdzenie czy raport '+Raport+' jest na liscie',5)
    ##                ???? stat,dane,dump,dump = PolSQL.Execute("SELECT typ,menu from rs_funkcje WHERE funkcja='"+Funkcja+"';",2)
    ##                if len(dane) == 0:
    ##                    #funkcji nie ma na liscie
    ##                    rsglobal.WyslijPakietami(connection,WielkPakietu,[u'Brak definicji raportu',[],[],[]])
    ##                    continue

                    rsglobal.ZapiszDoLogow('Sprawdzenie typu raportu',5)
                    if not Typ in ['pdf','html','sxw','txt','csv','xml','xls']:
                        rsglobal.WyslijPakietami(connection,WielkPakietu,['Niepoprawny typ raportu :'+str(Typ),[]])
                        continue

    ##                rsglobal.ZapiszDoLogow('Typ '+typ,5)
    ##                menuid = dane[0][1]
    ##                rsglobal.ZapiszDoLogow('Menu '+menuid,5)
    ##                #Sprawdzenie uprawnien dla funkcji
    ##                if OpId != STANDARD_OPERATOR_ID:
    ##                    if (not (oddzial,menuid) in Upr) and (not ('**',menuid) in Upr):
    ##                        #Brak uprawnien
    ##                        rsglobal.WyslijPakietami(connection,WielkPakietu,[u'Brak uprawnie�do wykonania funkcji',[],[],[]])
    ##                        continue

                    rsglobal.ZapiszDoLogow('Zaczynam raport waciwy '+str((Raport, Klucz, Typ,komunikacja)),5)
                    stat,dane = agata.WykonajRaport(Raport, Klucz, Typ, komunikacja=komunikacja)
                    rsglobal.ZapiszDoLogow('Koniec raportu waciwego',5)
                    #open('aaaa.pdf', 'wb').write(dane)

                    if stat_rap:
                        StoperStop(stoper)

                    rsglobal.ZapiszDoLogow('Stat '+str(stat),5)
                    if dane == None:
                        dane=''
                    rsglobal.WyslijPakietami(connection,WielkPakietu,[stat,binascii.hexlify(dane)])

                    rsglobal.ZapiszDoLogow('Koniec RAPORT',5)


                elif data[0:8] == FUNCTION:
                    #Zapytanie ma postac : FUNCTION nazwa_funkcji,[parametr1,parametr2,...,parametrN]

                    # test uprawnien, zwrot typu
                    rsglobal.ZapiszDoLogow('Zaczynam obsluge FUNCTION',5)
                    Funkcja = ''
                    Klucz = []
                    Pola = []
                    Dane = []

                    Reszta = rsglobal.rseval(data[8:])
                    if len(Reszta) > 0:
                        Funkcja = Reszta[0]
                    if len(Reszta) > 1:
                        Klucz = Reszta[1]
                    if len(Reszta) > 2:
                        Pola = Reszta[2]
                    if len(Reszta) > 3:
                        Dane = Reszta[3]

                    if Funkcja == '':
                        # nie podana funkcja
                        rsglobal.WyslijPakietami(connection,WielkPakietu,['X'])
                        continue

                    stat,dane,dump,dump = PolSQL.Execute("SELECT typ,menu from rs_funkcje WHERE funkcja='"+Funkcja+"';",2)
                    if len(dane) == 0:
                        #funkcji nie ma na liscie
                        rsglobal.WyslijPakietami(connection,WielkPakietu,['Y'])
                        continue

                    typ = dane[0][0]
                    if not typ in ['T','I','L','D','N']:
                        rsglobal.WyslijPakietami(connection,WielkPakietu,['T'])

                    rsglobal.ZapiszDoLogow('Typ '+typ,5)
                    menuid = dane[0][1]
                    rsglobal.ZapiszDoLogow('Menu '+menuid,5)
                    #Sprawdzenie uprawnien dla funkcji
                    if OpId != STANDARD_OPERATOR_ID:
                        if not (oddzial,menuid) in Upr:
                            #Brak uprawnien
                            rsglobal.WyslijPakietami(connection,WielkPakietu,['U'])
                            continue
                    KluczG = [firma,oddzial,okres,OpId]
                    Tekst = 'SELECT '+Funkcja+'('
                    Tekst = Tekst+"'"+string.replace(PolSQL.Dekoduj(str(KluczG)),"\'","\\\'")+"',"
                    Tekst = Tekst+"'"+string.replace(PolSQL.Dekoduj(str(Klucz)),"\'","\\\'")+"',"
                    Tekst = Tekst+"'"+string.replace(PolSQL.Dekoduj(str(Pola)),"\'","\\\'")+"',"
                    Tekst = Tekst+"'"+string.replace(PolSQL.Dekoduj(str(Dane)),"\'","\\\'")+"');"

                    rsglobal.ZapiszDoLogow('TEKST '+Tekst,5)

                    stat,dane,dump,dump = PolSQL.Execute(Tekst,2)
                    rsglobal.ZapiszDoLogow('Dane '+str(dane),5)
                    rsglobal.ZapiszDoLogow('Stat '+str(stat),5)

                    if typ == 'T': #zwyky tekst
                        rsglobal.WyslijPakietami(connection,WielkPakietu,[dane,stat])
                        #rsglobal.WyslijPakietami(connection,WielkPakietu,[PolSQL.Koduj(str(dane)),stat])
                    elif typ == 'I': #liczba cakowita
                        pass
                    elif typ == 'L': #logiczne
                        pass
                    elif typ == 'D': #Dane(tablica)
                        #rsglobal.WyslijPakietami(connection,WielkPakietu,[PolSQL.Koduj(str(dane)),stat])
                        rsglobal.WyslijPakietami(connection,WielkPakietu,[dane,stat])
                    elif typ == 'N': #Dane i naglowki
                        pass

                    rsglobal.ZapiszDoLogow('Koniec FUNCTION',5)


                elif data == SELECT:
                    rsglobal.WyslijPakietami(connection,WielkPakietu,[['Nielegalne uycie funkcji SELECT']])


                elif data == SELECTN:
                    # dla lepszej sychronizacji
                    rsglobal.WyslijPakietami(connection,WielkPakietu,[['Nielegalne uycie funkcji SELECTN']])

                elif data == SELECTT:  #select zwracajcy text
                    rsglobal.WyslijPakietami(connection,WielkPakietu,[['Nielegalne uycie funkcji SELECTT']])

                elif data == SELECTI:  #select zwracajcy liczb�typu integer
                    rsglobal.WyslijPakietami(connection,WielkPakietu,[['Nielegalne uycie funkcji SELECTI']])

                elif data == DELETE:
                    rsglobal.WyslijPakietami(connection,WielkPakietu,[['Nielegalne uycie funkcji DELETE']])

                elif data == MENUUPR:
                    rsglobal.WyslijPakietami(connection,WielkPakietu,[['Nielegalne uycie funkcji MENUUPR']])

                # obsluga transakcji
                elif data in (BEGIN,ROLLBACK,COMMIT):
                    connection.send(RECEIVED)
                    rsglobal.ZapiszDoLogow('Zaczynam dzialanie '+data,5)
                    #if firma==None:
                    #    rsglobal.WyslijPakietami(connection,WielkPakietu,"Brak wyboru firmy.")

                    if (data == ROLLBACK) and transakcja:
                        if PolSQL.bylo_cos_robione:
                            PolSQL.Rollback()
                            PolSQL.bylo_cos_robione = False
                        if PolSQL2 and PolSQL2.bylo_cos_robione:
                            PolSQL2.Rollback()
                            PolSQL2.bylo_cos_robione = False
                        transakcja = False
                    elif (data == COMMIT) and transakcja:
                        if PolSQL.bylo_cos_robione:
                            PolSQL.Commit()
                            PolSQL.bylo_cos_robione = False
                        if PolSQL2 and PolSQL2.bylo_cos_robione:
                            PolSQL2.Commit()
                            PolSQL2.bylo_cos_robione = False
                        transakcja = False
                    elif data == BEGIN:
                        transakcja = True
                    else:
                        rsglobal.ZapiszDoLogow(u"To nie powinno mie miejsca", 1)

                # operacje na tabeli user
                elif data in (UPDATE, INSERT, CREATEU, DROPU, ALTERU):
                    rsglobal.WyslijPakietami(connection,WielkPakietu,[['Nielegalne uycie funkcji '+data]])

                elif data == LISTAFIRM:
                    rsglobal.ZapiszDoLogow('Zaczynam obsluge LISTAFIRM',5)
                    if OpId == STANDARD_OPERATOR_ID:
                        stat, dane, dump, dump = PolSQL.Execute("SELECT firma_id FROM rs_firmy WHERE firma_id != '**'",2)
                    else:
                        rsglobal.ZapiszDoLogow('Operator :'+str(OpId),5)
                        rsglobal.ZapiszDoLogow('Operator i zastepcy :'+str(Operatorzy),5)
                        sql_firmy = """SELECT DISTINCT(firma_id) 
                                       FROM rs_grupa_operator_oddzial g
                                       JOIN rs_operatorzy o USING (operator_id)
                                       WHERE operator_id IN """+str(Operatorzy)+\
                                       " AND (kontrahent_id IS NULL  OR EXISTS(SELECT 1  FROM rs_kontrahenci k " \
                                       "    WHERE k.firma_id=g.firma_id AND k.kontrahent_id=o.kontrahent_id ));"
                        stat, dane, dump, dump = PolSQL.Execute(sql_firmy, 2)
                        rsglobal.ZapiszDoLogow('sql firmy :'+str(sql_firmy), 5)
                    rsglobal.ZapiszDoLogow('Lista firm :'+str(dane),5)
                    ListaFirm = []
                    if len(dane) > 0:
                        for f in dane:
                            ListaFirm.append(f[0])

                    firma=''
                    Upr=[]
                    if OpId != STANDARD_OPERATOR_ID:
                        if (len(ListaFirm) == 1):
                            firma = ListaFirm[0].encode('iso-8859-2')
                            #Upr = PodajUprawnienia(PolSQL,OpId,firma)
                            Upr = PodajUprawnienia(PolSQL,Operatorzy,firma)
                    rsglobal.WyslijPakietami(connection,WielkPakietu,str([ListaFirm,Upr]))
                    if OpId != STANDARD_OPERATOR_ID:
                        if (len(ListaFirm) == 1):
                            stat, dump, dump_, dump__ = PolSQL.Execute("UPDATE rs_polaczenia SET firma_id = '%s' WHERE oid=%s" % (firma,idwejscia),1)
                            funkcje.SetFirma(firma)
                            agata.SetFirma(firma)
                            ListaDzialow=PodajListeDzialow(PolSQL,OpId)
                            funkcje.SetListaDzialow(ListaDzialow)
                            agata.SetListaDzialow(ListaDzialow)
                        elif len(ListaFirm) == 0:
                            firma = ''
                            rsglobal.ZapiszDoLogow('Brak uprawnie do jakiejkolwiek firmy',2)

                    rsglobal.ZapiszDoLogow('Koniec obslugi LISTAFIRM',5)

                #-------------------------------
                #   FIRMA
                #-------------------------------
                elif data[0:6] == FIRMA:
                    rsglobal.ZapiszDoLogow('Zaczynam obsluge FIRMA',5)
                    firma = data[6:].encode('iso-8859-2')

                    if OpId == STANDARD_OPERATOR_ID:
                        stat, dane, dump, dump = PolSQL.Execute("SELECT firma_id FROM rs_firmy WHERE firma_id='%s';" % (firma),2)
                        CzyOK = (len(dane)==1)
                    else:
                        CzyOK = (firma in ListaFirm)

                    Upr = PodajUprawnienia(PolSQL,Operatorzy,firma)
                    blad_kontr = None  #AB 18.01.21r
                    #Upr = PodajUprawnienia(PolSQL,OpId,firma)
                    if CzyOK and OpId not in (0, '0'): #AB 18.01.21r
                        rsglobal.ZapiszDoLogow("SELECT kontrahent_id FROM rs_operatorzy WHERE operator_id='%s';" % (OpId), 5)
                        stat, kontr, dump, dump = PolSQL.Execute("SELECT kontrahent_id FROM rs_operatorzy WHERE operator_id='%s';" % (OpId), 5)
                        rsglobal.ZapiszDoLogow(kontr, 5)
                        if stat == 'OK' and kontr and kontr not in ('None', '', 'NULL'):
                            rsglobal.ZapiszDoLogow("SELECT kontrahent_id FROM rs_kontrahenci WHERE firma_id='%s' AND kontrahent_id='%s';" % (firma, kontr), 5)
                            stat, test_kontr, dump, dump = PolSQL.Execute("SELECT kontrahent_id FROM rs_kontrahenci WHERE firma_id='%s' AND kontrahent_id='%s';" % (firma, kontr), 2)
                            if stat == 'OK' and len(test_kontr) == 0:
                                rsglobal.ZapiszDoLogow('Kontrahenta  '+kontr+' nie ma w firmie ' + firma, 1)
                                blad_kontr = 'Kontrahenta  '+kontr+' nie ma w firmie ' + firma

                    rsglobal.ZapiszDoLogow('UPRAW '+str(CzyOK)+' '+str(Upr)[:1000])
                    if blad_kontr:  #AB 18.01.21r
                        rsglobal.ZapiszDoLogow(blad_kontr, 2)
                        rsglobal.WyslijPakietami(connection, WielkPakietu, blad_kontr)
                    elif CzyOK:
                        stat, dump, dump, dump = PolSQL.Execute("UPDATE rs_polaczenia SET firma_id = '%s' WHERE oid=%s" % (firma,idwejscia),1)
                        if not transakcja:
                            PolSQL.Commit()
                        rsglobal.ZapiszDoLogow(blad_kontr, 'Wysylam OK'+str(Upr)[:1000])
                        rsglobal.WyslijPakietami(connection,WielkPakietu,'OK'+str(Upr))
                        #connection.send() #Dobra firma
                        funkcje.SetFirma(firma)
                        agata.SetFirma(firma)
                        ListaDzialow=PodajListeDzialow(PolSQL,OpId)
                        funkcje.SetListaDzialow(ListaDzialow)
                        agata.SetListaDzialow(ListaDzialow)
                    else:
                        rsglobal.ZapiszDoLogow('Firmy '+firma+' nie ma w bazie danych',2)
                        rsglobal.WyslijPakietami(connection,WielkPakietu,'BF')
                        #connection.send('BF') #Zla firma

                    rsglobal.ZapiszDoLogow('Koniec obslugi FIRMA',5)


                #-------------------------------
                #   LISTA ODDZIAW
                #-------------------------------
                elif data == LISTAODDZ:
                    rsglobal.ZapiszDoLogow('Zaczynam obsluge LISTAODDZ',5)

                    ListaOddzialow = []
                    if OpId == STANDARD_OPERATOR_ID:
                        stat, oddzialy, dump, dump = PolSQL.Execute("SELECT oddzial_id FROM rs_oddzialy WHERE oddzial_id != '**' AND firma_id = '"+firma+"';",2)
                    else:
                        #Operatorzy=str(PodajOperatorow(PolSQL,OpId)).replace("[","(").replace("]",")")
                        stat, oddzialy, dump, dump = PolSQL.Execute("SELECT DISTINCT oddzial_id FROM rs_grupa_operator_oddzial WHERE operator_id in "+Operatorzy+" AND firma_id='"+firma+"';",2)
                        for odd in oddzialy:
                            rsglobal.ZapiszDoLogow('2 '+odd[0],5)
                            if odd[0] == '**':
                                stat, oddzialy, dump, dump = PolSQL.Execute("SELECT oddzial_id FROM rs_oddzialy WHERE oddzial_id != '**' AND firma_id = '"+firma+"';",2)
                                continue
                    for odd in oddzialy:
                        ListaOddzialow.append(odd[0])

                    rsglobal.WyslijPakietami(connection,WielkPakietu,PolSQL.Dekoduj(str(ListaOddzialow)))
                    if OpId != STANDARD_OPERATOR_ID:
                        if (len(ListaOddzialow) == 1):
                            oddzial = ListaOddzialow[0].encode('iso-8859-2')
                            #Upr = PodajUprawnienia(PolSQL,OpId,firma)
                            #Operatorzy=PodajOperatorow(PolSQL,OpId)
                            rsglobal.ZapiszDoLogow('PodajOperatorow',5)
                            rsglobal.ZapiszDoLogow(Operatorzy,5)
                            handlowiec=PodajHandlowca(PolSQL,OpId)
                            rsglobal.ZapiszDoLogow(handlowiec,5)
                            funkcje.SetOddzial(oddzial)
                            funkcje.SetUprawnienia(Upr)
                            funkcje.SetHandlowiec(handlowiec)
                            #funkcje.SetOperatorzy(Operatorzy)
                            agata.SetOddzial(oddzial)
                            agata.SetUprawnienia(Upr)
                            agata.SetHandlowiec(handlowiec)
                        elif len(ListaOddzialow) == 0:
                            oddzial = ''
                            rsglobal.ZapiszDoLogow('Brak uprawniedo jakiegokolwiek oddziau',2)

                    rsglobal.ZapiszDoLogow('Koniec obslugi LISTAODDZ',5)

                elif data[0:8] == ODDZIAL:
                    rsglobal.ZapiszDoLogow('Zaczynam obsluge ODDZIAL',5)
                    oddzial = data[8:].encode('iso-8859-2')
                    #Upr = PodajUprawnienia(PolSQL,OpId,firma)
                    #Operatorzy = PodajOperatorow(PolSQL,OpId)
                    handlowiec = PodajHandlowca(PolSQL,OpId)
                    rsglobal.ZapiszDoLogow(handlowiec,5)
                    rsglobal.ZapiszDoLogow('PodajOperatorow',5)
                    rsglobal.ZapiszDoLogow(Operatorzy,5)

                    if OpId == STANDARD_OPERATOR_ID:
                        stat, dane, dump_, dump__ = PolSQL.Execute("SELECT oddzial_id FROM rs_oddzialy WHERE firma_id='%s' AND oddzial_id='%s';" % (firma,oddzial),2)
                        CzyOK = (len(dane)==1)
                    else:
                        CzyOK = (oddzial in ListaOddzialow)

                    if CzyOK:
                        connection.send('OK') #Dobry oddzial
                        funkcje.SetOddzial(oddzial)
                        funkcje.SetUprawnienia(Upr)
                        funkcje.SetOperatorzy(Operatorzy)
                        funkcje.SetHandlowiec(handlowiec)
                        rsglobal.ZapiszDoLogow(funkcje.handlowiec_id,5)
                        agata.SetOddzial(oddzial)
                        agata.SetUprawnienia(Upr)
                        agata.SetHandlowiec(handlowiec)
                    else:
                        rsglobal.ZapiszDoLogow('Oddzialu '+oddzial+' nie ma w bazie danych',2)
                        connection.send('ZO') #Zly oddzial

                    rsglobal.ZapiszDoLogow('Koniec obslugi ODDZIAL',5)


                elif data[0:5] == OKRES:
                    rsglobal.ZapiszDoLogow('Zaczynam obsluge OKRES',5)
                    okres = data[6:].encode('iso-8859-2')
                    SQL="SELECT okres_id FROM rs_definicje_okresow WHERE firma_id='%s' AND okres_id='%s';" % (firma,okres)
                    stat, dane, dump_, dump__ = PolSQL.Execute(SQL,2)

                    CzyOK = (len(dane)==1)

                    if CzyOK:
                        connection.send('OK') #Dobry oddzial
                        funkcje.SetOkres(okres)
                        agata.SetOkres(okres)
                    else:
                        rsglobal.ZapiszDoLogow('Okresu '+okres+' nie ma w bazie danych',2)
                        connection.send('ZO') #Zly okres

                    rsglobal.ZapiszDoLogow('Koniec obslugi OKRES',5)

                elif data[0:9] == ZAPISZBIN:
                    #jako wejscie jest nazwa tabeli, lista list z kluczami, lista pol i ich typow(klucze + pole zalacznika), lista zalacznikow
                    rsglobal.ZapiszDoLogow('Zaczynam obsluge ZAPISZBIN',5)

                    Tekst = data[9:]
                    param = rsglobal.rseval(Tekst)
                    polecenie = "INSERT INTO "+param[0]+" VALUES ("
                    p = rsglobal.rseval(param[2])
                    for pole in p:
                        polecenie = polecenie+"%("+pole+")s, "
                    polecenie = polecenie[0:len(polecenie)-2]+")"
                    w = rsglobal.rseval(param[1])
                    wartosci = []
                    for i in range(len(w)):
                        if str(type(w[i]))=="<type 'unicode'>":
                            wartosci.append([p[i],w[i].encode('iso-8859-2')])
                        else:
                            wartosci.append([p[i],w[i]])
                    rsglobal.ZapiszDoLogow('wartosci '+str(dict(wartosci)),5)
                    plik = binascii.unhexlify(rsglobal.OdbierzPakietami(connection,WielkPakietu))
                    #wartosci.append([p[len(p)-1],buffer(plik)])
                    wartosci.append([p[len(p)-1],PolSQL.Binary(plik)])
                    stat,dane,dump,dump = PolSQL.Execute(polecenie,1,dict(wartosci))
                    rsglobal.WyslijPakietami(connection,WielkPakietu,stat)
                    rsglobal.ZapiszDoLogow('Stat '+stat,5)

                elif data[0:10] == WCZYTAJBIN:
                    #jako wejscie jest nazwa tabeli, lista list z kluczami, pole zawierajace bin
                    rsglobal.ZapiszDoLogow('Zaczynam obsluge WCZYTAJBIN',5)
                    param = rsglobal.rseval(data[11:])
                    rsglobal.ZapiszDoLogow('param '+str(param),5)
                    p = rsglobal.rseval(param[2])
                    where = ""
                    for pole in p:
                        where = where+pole+"=" +"%("+pole+")s AND "
                    where = where[0:len(where)-5]+";"
                    w = rsglobal.rseval(param[1])
                    wartosci = []
                    for i in range(len(w)):
                        if str(type(w[i]))=="<type 'unicode'>":
                            wartosci.append([p[i],w[i].encode('iso-8859-2')])
                        else:
                            wartosci.append([p[i],w[i]])
                    polecenie = "SELECT "+param[3]+" FROM "+param[0]+" WHERE "+where
                    rsglobal.ZapiszDoLogow('polecenie '+polecenie,5)
                    rsglobal.ZapiszDoLogow('wartosci '+str(dict(wartosci)),5)
                    stat,dane,dump,dump = PolSQL.Execute(polecenie,7,dict(wartosci))
                    rsglobal.WyslijPakietami(connection,WielkPakietu,stat)
                    rsglobal.ZapiszDoLogow('Stat '+stat,5)
                    if stat=='OK':
                        rsglobal.WyslijPakietami(connection,WielkPakietu,binascii.hexlify(dane))
                    rsglobal.ZapiszDoLogow('Koniec WCZYTAJBIN',5)

                elif data == STOP:
                    rsglobal.ZapiszDoLogow('Awaryjne zamykanie polaczenia',1)# z '+str(addr),1)
                    PolSQL.Execute(SQL_SUDDEN_END_CONN % (idwejscia),1)
                    idwejscia = None
                    rsglobal.ZapiszDoLogow('idwejscia na None 4')
                    PolSQL.Commit()
                    PolSQL.RozlaczZSQL()
                    if PolSQL2:
                        PolSQL2.Commit()
                        PolSQL2.RozlaczZSQL()
                    break

                elif data == CONN_END:
                    rsglobal.ZapiszDoLogow('Zaczynam obsluge KONIEC',4)
                    PolSQL.Execute(SQL_END_CONN % (idwejscia),1)
                    idwejscia = None
                    rsglobal.ZapiszDoLogow('idwejscia na None 5')
                    PolSQL.Commit()
                    if PolSQL2:
                        PolSQL2.Commit()
                    connection.send(RECEIVED)
                    rsglobal.ZapiszDoLogow('Koniec obslugi KONIEC',5)
                    time.sleep(3)
                    break
                else:
                    rsglobal.ZapiszDoLogow('Nowe polecenie : '+str(data),5)

                if not transakcja:
                    if PolSQL.bylo_cos_robione:
                        PolSQL.Commit()
                        PolSQL.bylo_cos_robione = False
                    if PolSQL2 and PolSQL2.bylo_cos_robione:
                        PolSQL2.Commit()
                        PolSQL2.bylo_cos_robione = False

            rsglobal.ZapiszDoLogow('Zamykanie polaczenia z klientem z '+str(addr),3)
            PolSQL.RozlaczZSQL()
            if PolSQL2:
                PolSQL2.RozlaczZSQL()
            sys.exit()
            #rsglobal.ZapiszDoLogow('ok',5)
        except Exception,e:
            if isinstance(e, SystemExit):
                # wywoano exit, jeli po bdzie to zapisano ju komunikat, to nie krytyczny bd
                raise
            else:
                rsglobal.ZapiszDoLogow('Krytyczny blad (%r): %s' % (_ostatnie_polecenie[:10], e), 1)
                # w obsudze zakoczenia jest ju zamykanie pocze z klientem i z baz danych
                sys.exit()
