# -*- coding: ISO8859-2 -*-
#-------------------------------------------------------------------------------
# Name:        rs_anakonda_server_hz.py
# Purpose:     modu obsugi harmonogramu zada wykonywanych przez serwer
#
# Authors:     Piotr Szydeko
# Created:     2008/05/14
# Copyright:   (c) 2005 Record System
# Licence:     GNU GPL
#-------------------------------------------------------------------------------

# 2012-03-26 Mariusz Rajfura    rozbicie zapisu do rs_harmonogram_log na start i koniec (5573)
# 2020-02-11 Aleksander Bargie Reklamacje RODO - przypominajki w harmonogramie (11748)
# 2020-02-21 Aleksander Bargie Bd w harmonogamie  (11872)
# 2020-03-09 Aleksander Bargie Bd w harmonogamie  (11897)
# 2021-03-01 Aleksander Bargie Harmonogram - przyczyny nie adowania nowych funkcji.   (12353)
# 2021-06-09 Aleksander Bargie (12522) Dostawy - zmiana sposobu sprawdzania wersji postgresq na dziaajc z wersjami
#                                                 10 i pniej
# 2022-10-04 Wojciech Skowron   (12174) Poprawka obsugi pliku w funkcjach _wykonajFunkcje i ZaladujFunkcje
# 2022-11-02 Wojciech Skowron   (13185) Harmonogram - gdy funkcja chodzi, nie uruchamia jej jeszcze raz, none -> dump

import os
import sys
import rsglobal		# rsglobal.ZapiszDoLogow(), rsglobal.PodajZKonf()
import rssql
from const import *	# configuration as well as operations' constants
import time
import datetime
import string
import signal
import cStringIO
import traceback
import copy
import rs_anakonda_server_fn
import rs_anakonda_server_cl

katalogFunkcji = "./funkcje_harmonogramu/"
katalogfunkcji = "./funkcje_harmonogramu/"
logPrefix = u"HZ"
LogFile = "harmonogram.log"
opoznienieKolejnychWykonan = 0.3
opoznieniePrzyStarcie = 1.5

(RSHZ_FIRMA_ID,
 RSHZ_HARMZAD_ID,
 RSHZ_NAZWA,
 RSHZ_ILE_ODSTEPOW,
 RSHZ_ODSTEP,
 RSHZ_ARGUMENTY,
 RSHZ_OPIS,
 RSHZ_ZACZNIJ,
 RSHZ_CZAS,
 RSHZ_ZALEGLE,
 RSHZ_AKTYWNE,
 RSHZ_DATA,
 RSHZ_OPERATOR_DODAL,
 RSHZ_STOP_PO_OK,
 RSHZ_POWIADOM_GDY_BLAD,
) = range(15)

RSHZ_POLA = "firma_id, harmzad_id, nazwa, ile_odstepow, odstep, replace(argumenty, '\\\\\\\\', '\\\\'), opis, zacznij, " + \
    "text(czas) AS czas, zalegle, aktywne, data, dodal_op_id, stop_po_ok, powiadom_gdy_blad"
powtorzenieNaCzas = {}

EINTR = 4   # os.errno.EINTR
ECHILD = 10 # os.errno.ECHILD
ESRCH = 3   # os.errno.ESRCH

#-------------------------------------------------------------------------------

class Zarzadca:
    # obiekt tej klasy steruje dziaaniem harmonogramu, okresowo zbiera dane
    # o zadaniach do uruchomieniach i zleca ich wykonanie
    # najwaniejsze metody to Uruchom() - tam jest fork i Zatrzymaj()
    # gwna ptl znajdziesz w metodzie main()

    def __init__(self):

        self.FunkcjeHarmonogramu = {}
        # PID potomka obsugujcego funkcje -> (firma_id, harmzad_id) zadania
        self.potomkowie = {}

        self._oldHupHandler = signal.SIG_DFL
        self._oldAlarmHandler = signal.SIG_DFL
        self._oldTermHandler = signal.SIG_DFL
        # czas kiedy ma nastapic nastepny cykl obslugi czekajacych zadan
        self._childProcess = None
        # czy w tej chwili moe uruchamia zadania, dodatkowo gdy False skoczy ptle
        self.obslugujeZadania = False
        self.PolSQL = None
        # (firma_id, harmzad_id)->pid zadan ktorych trwa obsluga zalegnych wystapien
        self.zablokowaneZadania = {}
        self.dbspec = None
        self.dbspec2 = None
        self.SQLTEST="SELECT * FROM pg_stat_activity where current_query<>'<IDLE>' and usename='harmonogram' ORDER BY query_start"
        self.SQLTESTIDLE="SELECT procpid,* FROM pg_stat_activity WHERE current_query='<IDLE> in transaction' AND usename='harmonogram' ORDER BY query_start"
        self.SQLTEST9="SELECT * FROM pg_stat_activity where state<>'idle' and usename='harmonogram' ORDER BY query_start"
        self.SQLTESTIDLE9="SELECT pid,* FROM pg_stat_activity WHERE state='idle in transaction' AND usename='harmonogram' ORDER BY query_start"

#-------------------------------------------------------------------------------

    def ZaladujFunkcje(self, katalog = katalogFunkcji):
        # wczytuje z katalogu @katalog wczystkie pliki fs_*.py
        #  kompilujc od razu ich kod, bdy zapisywane do logw
        #  pliki ktre si nie skompiluj s pomijane, te funkcje nie bd wykonywane

        localPrefix = "%s: ZaladujFunkcje():" % logPrefix
        if not os.path.isdir(katalog):
            rsglobal.ZapiszDoLogow(u"%s %r nie jest katalogiem" % (localPrefix, katalog), 1)
            return False
        self.FunkcjeHarmonogramu = {}
        path, dirs, files = os.walk(katalog).next()
        for filename in files:
            if filename.startswith("fs_") and filename.endswith(".py"):
                filepath = os.path.join(katalog, filename)
                nazwa = filename[:-3]
                rsglobal.ZapiszDoLogow(u"%s funkcja %r z pliku: %s" % (localPrefix, nazwa, filepath), 6)
                try:
                    #WS 2022-10-04
                    with open(filepath) as file:
                        tresc = file.read()
                    # tresc = open(filepath).read()
                except:
                    rsglobal.ZapiszDoLogow(u"%s nie mozna wczytac pliku %r" % (localPrefix, filepath), 1)
                    continue
                try:
                    code = compile(tresc, filepath, "exec")
                except Exception, exc:
                    rsglobal.ZapiszDoLogow(u"%s nie mozna skompilowac pliku %s: %s" % (localPrefix, filename, exc), 1)
                    continue
                self.FunkcjeHarmonogramu[nazwa] = code

        return bool(self.FunkcjeHarmonogramu)

#-------------------------------------------------------------------------------

    def _OnSIGALRM(self, sig, frame):
        pass

#-------------------------------------------------------------------------------

    def _OnSIGHUP(self, sig, frame):
        # gdy proces dostanie SIGHUP wczytuje ponownie wszystkie pliki z funkcjami
        #  i tabele rs_powtarzalno

        global powtorzenieNaCzas

        if self._childProcess:
            os.kill(self._childProcess, signal.SIGHUP)
        else:
            rsglobal.ZapiszDoLogow(u"%s: %d dostalem SIGHUP, wczytuje funkcje" % (logPrefix, os.getpid()), 2)
            self.ZaladujFunkcje()
            powt = powtorzenieNaCzas
            try:
                self._wczytajPowtarzalnosc()
            except Exception, exc:
                rsglobal.ZapiszDoLogow(u"%s: nowe powtarzalnosci odrzucone: %s" % (logPrefix, exc), 1)
                powtorzenieNaCzas = powt

#-------------------------------------------------------------------------------

    def _OnSIGTERM(self, sig, frame):

        if self.potomkowie:
            rsglobal.ZapiszDoLogow(u"%s: %d dostalem SIGTERM, przekazuje do %r" %
                (logPrefix, os.getpid(), self.potomkowie), 3)
            #for pid in self.potomkowie.keys():
            #    os.kill(pid, signal.SIGTERM)
            #    del self.potomkowie[pid]
        else:
            rsglobal.ZapiszDoLogow(u"%s: %d dostalem SIGTERM, koncze" % (logPrefix, os.getpid()), 2)
            #self._exit(EINTR)
        self._exit(EINTR)
        self.obslugujeZadania = False

#-------------------------------------------------------------------------------

    def runda(self, sekund=1):

        signal.alarm(sekund)
        try:
            pid, status = os.wait()
            if pid in self.potomkowie:
                del self.potomkowie[pid]
            if pid in self.zablokowaneZadania.values():
                for k, v in self.zablokowaneZadania.iteritems():
                    if v == pid:
                        del self.zablokowaneZadania[k]
                        break
        except OSError, exc:
            errno = exc.errno
            if errno == ECHILD:
                signal.alarm(0)
                time.sleep(sekund/2)
            elif errno == EINTR:
                minelo = signal.alarm(0)
            else:
                raise

#-------------------------------------------------------------------------------
                                                        #AB 11.02.20r
    def Uruchom(self, dbspec, dbspec2=None, debug=False, konfiguracja={}):
        # odcza proces potomny i uruchamia gwn ptl
        import os

        self.dbspec = dbspec
        self.dbspec2 = dbspec2
        self.debug = debug    #JR 11590 2019-08-16
        self.konfiguracja = konfiguracja #AB 11.02.20r
        childPID = os.fork()
        if childPID:
            self._childProcess = childPID

            return childPID
        else:
            try:
                signal.signal(signal.SIGUSR1, signal.SIG_DFL)
                signal.signal(signal.SIGCHLD, signal.SIG_DFL)
                if LogFile:
                    rsglobal.LogFile = LogFile
                self._oldHupHandler = signal.signal(signal.SIGHUP, lambda x, y: self._OnSIGHUP(x, y))
                self._oldAlarmHandler = signal.signal(signal.SIGALRM, lambda x, y: self._OnSIGALRM(x, y))
                self._oldTermHandler = signal.signal(signal.SIGTERM, lambda x, y: self._OnSIGTERM(x, y))
                if (not self.FunkcjeHarmonogramu) or self.debug:     #JR 11590 2019-08-16
                    self.ZaladujFunkcje()
                self.potomkowie = {}
                self._childProcess = None
                self.obslugujeZadania = True
            except Exception, exc:
                rsglobal.ZapiszDoLogow(u"%s: Blad w Zarzadca.Uruchom(): %s" % (logPrefix, exc), 2)
                self._exit(1)
                
            rsglobal.ZapiszDoLogow(u"%s: uruchomiony, PID=%d" % (logPrefix, os.getpid()), 1)

            while True:
                try:
                    self.main()
                except SystemExit, exc:
                    self._exit(exc.code)
                except Exception, exc:
                    # wszystkie bdy, nie tylko dotyczce postgresa
                    rsglobal.ZapiszDoLogow(u"%s: Blad w Zarzadca.main(): %s" % (logPrefix, exc), 2)
                    # nie poddawaj sie, odczekaj i spowrotem do pracy
                    if self.obslugujeZadania:
                        time.sleep(30)
                    # jeli nadal nie zlecono przerwania to robimy nastpn rundk
                    if self.obslugujeZadania:
                        rsglobal.ZapiszDoLogow(u"%s: lecimy dalej" % (logPrefix,), 2)
                        continue
                    # w przeciwnym wypadku, skoro ju not self.obslugujeZadania, to kocz
                    self._exit(1)
                # gdy nie wystpiy adne bdy - po prostu zakocz
                self._exit(0)

#-------------------------------------------------------------------------------

    def Zatrzymaj(self):

        if self._childProcess:
            os.kill(self._childProcess, signal.SIGTERM)

#-------------------------------------------------------------------------------

    def Reload(self):
        rsglobal.ZapiszDoLogow(u"ROBIE RELOAD" , 1)
        if self._childProcess:
            try:
                os.kill(self._childProcess, signal.SIGHUP)
            except OSError, exc:
                if exc.errno == ESRCH:
                    self.Uruchom(self.dbspec, self.dbspec2, self.debug)    #JR 11590 2019-08-16
                    return "%s: Zarzadca.Reload() - harmonogram nie dziala. Uruchamiam." % (logPrefix, )
                else:
                    return str(exc)
            except Exception, exc:
                return str(exc)
            else:
                
                return "Reload - sygnal przekazany"
        else:
            return "Reload - harmonogram jeszcze nie zostal uruchomiony"

#-------------------------------------------------------------------------------

    def main(self):
        # gwna cz pracy harmonogramu, w ptli niej sprawdza czy
        # ma zadania do wykonania

        time.sleep(opoznieniePrzyStarcie)
        self.PolSQL = SQLConnect(self.dbspec)
        if not self.PolSQL:
            rsglobal.ZapiszDoLogow(u"%s: brak polaczenia z baza - koncze" % (logPrefix, ), 1)
            self._exit(6)
        try:
            self._wczytajPowtarzalnosc()
        except Exception, exc:
            rsglobal.ZapiszDoLogow(u"%s: blad przy wczytywaniu powtarzalnosci: %s" % (logPrefix, exc), 1)
            self._exit(8)
            
        # jak czsto zaglda do bazy
        czasCyklu = datetime.timedelta(seconds=37)
        restartPoPrzekroczeniu = datetime.timedelta(seconds=120)

        while self.obslugujeZadania:

            if self.debug:  # JR 11590 2019-08-16
                self.ZaladujFunkcje()

            if self.PolSQL:
                self.WykonajZadaniaZalegle()
                # kiedy ma znowu zajrze do bazy
                nastepnyCykl = datetime.datetime.now()
                # test na idle
                #self.ZrobTestIdle()
                while self.obslugujeZadania:
                    if self.debug:  # JR 11590 2019-08-16
                        self.ZaladujFunkcje()
                    self.runda(2)
                    teraz = datetime.datetime.now()
                    # jeli czas bdzie cofnity o wicej ni czasCyklu to harmonogram ma wznowi prac od razu
                    if nastepnyCykl - teraz > czasCyklu:
                        nastepnyCykl = teraz
                    if teraz > nastepnyCykl:
                        # prba poczenia z baz danych w przypadku zerwania (raz na 10 minut)
                        try:
                            if not self.PolSQL:
                                self.PolSQL = SQLConnect(self.dbspec)
                        except Exception,e:
                            time.sleep(600)
                            rsglobal.ZapiszDoLogow(u"Bd odtworzenia poczenia : "+str(e), 1)

                        if teraz > nastepnyCykl + restartPoPrzekroczeniu:
                            # od ostatniego cyklu minelo zbyt duzo czasu, zacznij od poczatku
                            break
                        self.WykonajZadaniaZalegle()
                        nastepnyCykl += czasCyklu
                        try:
                            self.PolSQL.Commit()
                        except:
                            rsglobal.ZapiszDoLogow('Blad commita',1)
                            pass
    
                            
                        #self.ZrobTestIdle()

        self.PolSQL.RozlaczZSQL()

#-------------------------------------------------------------------------------

    def Wykonaj(self, zadanie):

        if not self.obslugujeZadania:
            return False
        if zadanie.id in self.zablokowaneZadania:
            return False
        else:
            pid = zadanie.Wykonaj(self)
            self.potomkowie[pid] = zadanie.id
            return True

#-------------------------------------------------------------------------------

    def _exit(self, status):

        if self.PolSQL:
            self.PolSQL.RozlaczZSQL()
        if status:
            rsglobal.ZapiszDoLogow(u"%s: %d koncze ze statusem %s" % (logPrefix, os.getpid(), status), 1)
        os._exit(status)

#-------------------------------------------------------------------------------

    def ZrobTestIdle(self):
        # test na zawieszenie procesu harmonogramu 
        import rsglobal
        rsglobal.ZapiszDoLogow(u"ROBIE TEST IDLE ", 1)
        try:
            # pole procpid na pid zmienio si od wersji 9.2 postgresa
            #AB 09.06.21r
            stat,dane,dump,dump=self.PolSQL.Execute("SHOW server_version_num;",3)
            if stat == 'OK':
                wer = dane[0][0]
                if wer < 90200:
                    stat, dane, dump, dump = self.PolSQL.Execute(self.SQLTESTIDLE,3)    
                else:
                    stat, dane, dump, dump = self.PolSQL.Execute(self.SQLTESTIDLE9,3)    
            
            if stat != "OK":
                rsglobal.ZapiszDoLogow(u"BLAD ZAPYTANIA TESTU IDLE "+stat, 1)
                if wer < 9:
                    rsglobal.ZapiszDoLogow(u"ZAPYTANIE : "+self.SQLTESTIDLE, 1)
                else:
                    rsglobal.ZapiszDoLogow(u"ZAPYTANIE : "+self.SQLTESTIDLE9, 1)
            elif len(dane)>0:
                rsglobal.ZapiszDoLogow(u"S IDLE : "+str(dane), 1)
        except Exception,e:
            rsglobal.ZapiszDoLogow(u"BLAD TESTU IDLE : "+str(e), 1)

#-------------------------------------------------------------------------------
        
    def WykonajZadaniaZalegle(self):
        # wyszukuje zadania ktre maj termin wykonania w przeszoci i traktuje je
        # tak jak to opisane w polu 'zalegle'

        def timeStep(start, step):
            # jeli to tupla to znaczy (ile_odstpw, wyraenieSQL) - odpytaj baze
            # wydajno moe budzi zastrzeenia, ale ten mechanizm bdzie uywany na prawd rzadko
            if isinstance(step, tuple):
                #print ("stepquery: SELECT '%%s'::TIMESTAMP + INTERVAL '%s'" % step) % formatTime(start)
                status, dane, dump, dump = self.PolSQL.Execute(
                    "SELECT text(%%s::TIMESTAMP + %d * INTERVAL '%s')" % step, 2, (formatTime(start), ))
                if status != "OK":
                    rsglobal.ZapiszDoLogow(u"%s %s %s" % (localPrefix, "timeStep() blad: ", status), 1)
                    zadanie.deaktywuj(self.PolSQL, rollback=True)
                    self._exit(7)
                return readTime(dane[0][0])
            return start + step

        localPrefix = "%s: WykonajZadaniaZalegle()" % logPrefix
        rsglobal.ZapiszDoLogow(localPrefix, 4)
        self.PolSQL.Commit()
        query = ("SELECT " + RSHZ_POLA + " FROM rs_harmonogram_zadan"
            " WHERE czas < NOW() AND aktywne AND ile_odstepow > 0")
        status, dane, dump, dump = self.PolSQL.Execute(query, 2)
        teraz = datetime.datetime.now()
        if status == "OK":
            for row in dane:            #AB 11.02.20r
                zadanie = Zadanie(row, konfiguracja=self.konfiguracja)
                zalegle = rsglobal.Dekoduj(row[RSHZ_ZALEGLE])
                czas = readTime(row[RSHZ_CZAS])
                rsglobal.ZapiszDoLogow(u"%s %r" % (localPrefix, zadanie.id), 5)

                if row[RSHZ_ODSTEP] in powtorzenieNaCzas:
                    step = powtorzenieNaCzas[row[RSHZ_ODSTEP]]
                    # czy mam dugo odstpu jako timedelta eby atwo liczy?
                    if step[1]:
                        step = step[1] * row[RSHZ_ILE_ODSTEPOW]
                    else:
                        # skoro nie, to przekazuj liczb i wyraenie SQLowe
                        step = (row[RSHZ_ILE_ODSTEPOW], step[0]) 
                else:
                    zadanie.deaktywuj(self.PolSQL)
                    continue
                if not row[RSHZ_ILE_ODSTEPOW]:
                    if zalegle == ZALEGLE_POMIN:
                        zadanie.deaktywuj(self.PolSQL)
                        continue
                elif zalegle == ZALEGLE_POMIN:
                    while czas < teraz:
                        czas = timeStep(czas, step)
                elif zalegle == ZALEGLE_JEDNO:
                    next = timeStep(czas, step)
                    while next < teraz:
                        czas = next
                        next = timeStep(czas, step)
                elif zalegle == ZALEGLE_WSZYSTKIE:
                    pass
                else:
                    rsglobal.ZapiszDoLogow(u"%s: nieprawidlowe zalegle: %r" % (localPrefix, zalegle), 1)

                if zadanie.czas != readTime(zadanie.row[RSHZ_CZAS]):
                    self.PolSQL.Execute("UPDATE rs_harmonogram_zadan SET czas=%s"
                        " WHERE firma_id=%s AND harmzad_id=%s", 1,
                        (formatTime(czas), zadanie.id[0], zadanie.id[1]))
                    self.PolSQL.Commit()
                if czas > teraz:
                    continue

                #WS 2022-11-02 - sprawdzenie czy poprzednie wykonanie jeszcze trwa
                SEL = """SELECT 1 FROM rs_harmonogram_zadan_log WHERE firma_id=%s AND harmzad_id=%s AND stat='';"""
                stat, dane, dump, dump = self.PolSQL.Execute(SEL, 2, (zadanie.id[0], zadanie.id[1], ))
                if stat != 'OK':
                    rsglobal.ZapiszDoLogow(u"%s: bd sprawdzania logow" % (localPrefix), 1)
                elif len(dane) > 0:
                    kom = 'Poprzednie wykonanie zadania nie zakoczyo si'
                    query = ("INSERT INTO rs_harmonogram_zadan_log"
                        " (firma_id,harmzad_id, moment, uruchomione, stat)"
                        " VALUES ('" + zadanie.id[0] + "', " + str(zadanie.id[1]) + ", '" + formatTime(zadanie.czas) + "', '" + formatTime(datetime.datetime.now()) + "', '" + kom + "') ")
                    self.PolSQL.Execute(str(query), 1)

                    rsglobal.ZapiszDoLogow(kom,1)
                    rsglobal.ZapiszDoLogow(str(query),1)

                    self.PolSQL.Commit()
                    zadanie.PolSQL = self.PolSQL
                    zadanie._wyznaczNastepneWykonanie()
                    continue

                zadanie.czas = czas
                pid = zadanie.Wykonaj(self, powtarzaj=bool(row[RSHZ_ILE_ODSTEPOW]))
                self.potomkowie[pid] = zadanie.id
                self.zablokowaneZadania[zadanie.id] = pid
                time.sleep(opoznienieKolejnychWykonan)
            self.PolSQL.Commit()

#-------------------------------------------------------------------------------

    def WykonajZadaniaNaTeraz(self):

        self.PolSQL.Commit()
        query = ("SELECT " + RSHZ_POLA + " FROM rs_harmonogram_zadan"
        " WHERE czas BETWEEN NOW() - INTERVAL '3 minutes' AND NOW() AND aktywne")
        status, dane, dump, dump = self.PolSQL.Execute(query , 2)
        if status == "OK":
            for row in dane:                #AB 11.02.20r
                zadanie = Zadanie(row, konfiguracja=self.konfiguracja)
                zadanie.czas = readTime(row[RSHZ_CZAS])
                self.Wykonaj(zadanie)

#-------------------------------------------------------------------------------

    def _wczytajPowtarzalnosc(self):
        # wczytaj dane z tabeli rs_powtarzalnosc, s potrzebne do oblicze

        global powtorzenieNaCzas
        powtorzenieNaCzas = {}
        localPrefix = "%s: _wczytajPowtarzalnosc()" % logPrefix

        query = "SELECT kod, interwal FROM rs_powtarzalnosc"
        stat, dane, dump, dump = self.PolSQL.Execute(query, 2)
        if stat != "OK":
            rsglobal.ZapiszDoLogow(u"%s: nie mozna wczytac rs_powtarzalnosc, %r" % (localPrefix, stat), 1)
            self._exit(8)
        for kod, interwal in dane:
            nowy = None
            if interwal.startswith("sql:"):
                nowy = (str(interwal.split(":", 1)[1]), None)
            elif interwal.endswith("m"):
                ile = int(interwal[:-1])
                nowy = ("%d MINUTES" % ile, datetime.timedelta(seconds=60))
            elif interwal.endswith("d"):
                ile = int(interwal[:-1])
                nowy = ("%d DAYS" % ile, datetime.timedelta(days=1))
            else:
                rsglobal.ZapiszDoLogow(u"%s: nie potrawie zinterpretowac odstepu czasu %r" % (localPrefix, interwal), 1)
                self._exit(8)
            powtorzenieNaCzas[kod] = nowy

        return powtorzenieNaCzas


zarzadcaHarmonogramu = Zarzadca()


#-------------------------------------------------------------------------------
#-- KLASA ZADANIE
#-------------------------------------------------------------------------------

class Zadanie:
    # klasa reprezentujca jedno zadanie harmonogramu, zajmuje si gwnie uruchomieniem odpowiedniej funkcji
                            #AB 11.02.20r
    def __init__(self, row, konfiguracja={}):

        self.row = row
        self.id = (rsglobal.Dekoduj(self.row[RSHZ_FIRMA_ID]), self.row[RSHZ_HARMZAD_ID])
        self.czas = datetime.datetime.now()
        self.PolSQL = None
        self._rsfunkcje = None
        self._env_out = None
        self._env_argumenty = {}
        self.osobne_pol_rap = None
        self.konfiguracja = konfiguracja #AB 11.02.20r

#-------------------------------------------------------------------------------

    def load(self, firma_id=None, harmzad_id=None, PolSQL=None):
        # wczytanie danych funkcji do obiektu

        if firma_id is None or harmzad_id is None:
            firma_id, harmzad_id = self.id
        if PolSQL is None:
            PolSQL = self.PolSQL
        query = ("SELECT " + RSHZ_POLA + " FROM rs_harmonogram_zadan"
            " WHERE firma_id='" + firma_id + "' AND harmzad_id=" + str(harmzad_id))
        status, dane, dump, dump = PolSQL.Execute(query, 2)
        if not dane:
            rsglobal.ZapiszDoLogow(u"%s: Zadanie.load(): nie mozna wczytac zadania (%r, %r)" %
                (logPrefix, firma_id, harmzad_id), 1)
            self.row = None
            return None
        else:
            self.row = dane[0]
            self.id = (rsglobal.Dekoduj(self.row[RSHZ_FIRMA_ID]), self.row[RSHZ_HARMZAD_ID])
            return self

#-------------------------------------------------------------------------------
        
    def _utworzArgumenty(self):
        # przygotowanie zmiennych lokalnych dla uruchamianej funkcji z argumentw i kilku domylnych wartoci
        try:
            args = {}
            args["stat"] = "OK"
            argumenty = eval("{%s}" % rsglobal.Dekoduj(self.row[RSHZ_ARGUMENTY]))
            args.update(argumenty)
            args["argumenty"] = argumenty
            args["firma_id"] = rsglobal.Dekoduj(self.row[RSHZ_FIRMA_ID])
            args["czas"] = formatTime(self.czas)
            args["fs_name"] = rsglobal.Dekoduj(self.row[RSHZ_NAZWA])
            args["fs_time"] = self.czas
            args["operator_id"] = self.row[RSHZ_OPERATOR_DODAL]
            args["ListaBledow"] = list()

            #AB 21.02.20r
            if 'katalogplikow' in self.konfiguracja and self.konfiguracja["katalogplikow"]:
                args['KatalogPlikow'] = self.konfiguracja["katalogplikow"]
                if args['KatalogPlikow'] in (None, ''):
                    args['KatalogPlikow'] = os.getcwd() + '/pliki'
            else:
                args['KatalogPlikow'] = os.getcwd() + '/pliki'
            #AB 21.02.20r
            if 'baza' not in self.konfiguracja or self.konfiguracja['baza'] == '':
                args['KatalogPlikow'] += '/anakonda'
            else:
                args['KatalogPlikow'] += '/' + self.konfiguracja['baza']

            #AB 09.03.20r
            if 'katalogedeklaracji' in self.konfiguracja and self.konfiguracja["katalogedeklaracji"]:
                args['KatalogDeklaracji'] = self.konfiguracja["katalogedeklaracji"]
            else:
                args['KatalogDeklaracji'] = args['KatalogPlikow'] + "/edeklaracje"

            for k in args:
                if isinstance(k, int):
                    v = args[k]
                    del args[k]
                    args["arg%d" % k] = v
        except Exception, exc:
            import traceback
            rsglobal.ZapiszDoLogow(traceback.format_exc())
            #AB 09.03.20r zakomentowane bo to tylko rzuca blad brak zmiennej localPrefix i odwraca nas od prawdziwego
            # zrodla problemu
            # rsglobal.ZapiszDoLogow(u"%s problem z argumentami zadania (%r, %r): %s" %
            #     (localPrefix, firma_id, harmzad_id, exc), 1)
        self._env_argumenty = args
        return args

#-------------------------------------------------------------------------------

    def Wykonaj(self, zarzadca, powtarzaj=False):
        # fork i wykonanie funkcji ktrej dane zawiera ten obiekt

        try:
            return self._wykonaj(zarzadca, powtarzaj)
        except Exception, exc:
            localPrefix = "%s: Zadanie.Wykonaj():" % logPrefix
            ZapiszWiadomosc(self.PolSQL, self, traceback.format_exc(), "Bd wewntrzny w harmonogramie: zadanie %r" % (self.id,),
                commit=True, adresat_id=STANDARD_OPERATOR_ID)
            rsglobal.ZapiszDoLogow(u"%s %s, BLAD: %s  %r" %
                (localPrefix, self.id, exc, traceback.extract_tb(sys.exc_info()[2])[-1]), 1)
            # bylby problem gdyby nie doszlo do forka, ale to tylko 4 linijki
            self._exit(1)

#-------------------------------------------------------------------------------

    def _wykonaj(self, zarzadca, powtarzaj=False):

        localPrefix = "%s: Zadanie.Wykonaj():" % logPrefix
        rsglobal.ZapiszDoLogow(u"%s funkcja %s, zadanie %r" %
            (localPrefix, self.row[RSHZ_NAZWA], self.id), 5)
        katalogFunkcji = zarzadca.FunkcjeHarmonogramu
        childPID = os.fork()
        if childPID:
            return childPID
        else:
            self.PolSQL = SQLConnect(zarzadca.dbspec)
            ##zu
            if not self.PolSQL:
                rsglobal.ZapiszDoLogow(u"%s brak polaczenia z baza danych, zadanie %r zaniechane"
                                       % (localPrefix, self.id), 1)
                self._exit(6)
            #self.PolSQL.Commit()

            if zarzadca.dbspec2:
                self.PolSQL2 = SQLConnect(zarzadca.dbspec2)
                if self.PolSQL2:
                    self.osobne_pol_rap = True
                else:
                    rsglobal.ZapiszDoLogow(u"%s brak dodatkowego (do raportow) polaczenia"
                                           u" z baza danych, zadanie %r zaniechane"
                                           % (localPrefix, self.id), 1)
                    self._exit(6)
            else:
                self.PolSQL2 = self.PolSQL
                self.osobne_pol_rap = False

            signal.signal(signal.SIGTERM, lambda x, y: self._OnSIGTERM(x, y))
            licznik = 0
            while True:
                self._wyznaczNastepneWykonanie()
                status = self._wykonajFunkcje(katalogFunkcji)
                self.load()
                licznik += 1
                if status and self.row[RSHZ_STOP_PO_OK]:
                    self.deaktywuj()
                    break
                if powtarzaj:
                    self.czas = readTime(self.row[RSHZ_CZAS])
                    if self.czas > datetime.datetime.now():
                        break
                    time.sleep(opoznienieKolejnychWykonan)
                else:
                    break
            if powtarzaj:
                rsglobal.ZapiszDoLogow(u"%s zakonczona wielu wywolan(%d) zadania %r" %
                    (localPrefix, licznik, self.id), 4)
            self._exit(0)

#-------------------------------------------------------------------------------

    def _wykonajFunkcje(self, katalogFunkcji):
    
        localPrefix = "%s: _wykonajFunkcje():" % logPrefix
        firma_id, harmzad_id = self.id
        start = datetime.datetime.now()
        if not self.row or not self.row[RSHZ_AKTYWNE]:
            return False
        # wywoana ma by funkcja o nazwie jak niej, sprawd od razu czy taka istnieje
        nazwa = self.row[RSHZ_NAZWA]
        if nazwa not in katalogFunkcji:
            query = ("INSERT INTO rs_harmonogram_zadan_log (firma_id, harmzad_id, moment, uruchomione, trwalo, stat, wypisane)"
                " VALUES ('" + firma_id + "', " + str(harmzad_id) + ", '" + formatTime(self.czas) + "', '" + formatTime(start) +
                "', -1, 'BLAD', 'Funkcja " + nazwa + " jest niedostepna')")
            status, dump, dump, dump = self.PolSQL.Execute(str(query), 1)
            self.PolSQL.Commit()
            if status != "OK":
                rsglobal.ZapiszDoLogow(u"%s nie znaleziono funkcji %r dla zadania (%r, %r)" %
                    (localPrefix, nazwa, firma_id, harmzad_id), 1)
            self.deaktywuj()
            return False

        # przygotowanie kodu funkcji i rodowiska dla niej: zmienne globalne i lokalne (args)
        # zarzadcaHarmonogramu.ZaladujJednaFunkcje(nazwa)
        #AB 01.03.21r ODCZYTUJEMY FUNKCJE PROSTO Z DYSKU, ZAWSZE AKTUALNA
        sciezka = os.path.join(katalogfunkcji, nazwa + '.py')
        try:
            #WS 2022-10-04
            with open(sciezka) as plik:
                tresc = plik.read()
            # tresc = open(sciezka).read()
        except:
            rsglobal.ZapiszDoLogow(u"%s nie mozna wczytac pliku %r" % (localPrefix, sciezka), 1)

        try:
            code = compile(tresc, sciezka, "exec")
        except Exception, exc:
            rsglobal.ZapiszDoLogow(u"%s nie mozna skompilowac pliku %s: %s" % (localPrefix, sciezka, exc), 1)
                    # code = katalogFunkcji[nazwa]
        ##zu
        globs = {
            "rsglobal": rsglobal,
            "PolSQL": self.PolSQL,
            "PolSQL2": self.PolSQL2,
            "exit": sys.exit,
            "WykonajFunkcje": self._env_wykonajRSFunkcje,
            "KonczJesliBlad": self._env_konczJesliBlad,
            "PokazJesliBlad": self._env_pokazJesliBlad,
            "SprawdzArgumenty": self._env_sprawdzArgumenty,
            "ZapiszLog": self._env_zapiszLog,
        }
        args = self._utworzArgumenty()
        if args is None:
            return False
        argsNaStarcie = copy.deepcopy(args)
        rsglobal.ZapiszDoLogow(u"Argumenty "+str(argsNaStarcie),1) 

        # ju zapiszemy dat pierwszego uruchomienia, w sumie nie jest do niczego potrzebna
        if not self.row[RSHZ_ZACZNIJ] or self.row[RSHZ_ZACZNIJ] == "None":
            query = "UPDATE rs_harmonogram_zadan SET zacznij=NOW() WHERE harmzad_id=" + str(harmzad_id)
            stat, dump, dump, dump = self.PolSQL.Execute(query, 1)
            self.PolSQL.Commit()
            # stat ignorowane

        rsglobal.ZapiszDoLogow("%s wywolanie funkcji %s dla czasu %s" % (localPrefix, nazwa, self.czas), 1)
        # przejmowanie wyjcia standardowego (print)
        oldstdout, oldstderr = sys.stdout, sys.stderr
        self._env_out = cStringIO.StringIO()
        sys.stdout, sys.stderr = self._env_out, self._env_out
        bylWyjatek = False

        #-------------------------------------------------------------------------------
        # zapis w logach startu wykonywania funkcji, stat ignorowany
        query = ("INSERT INTO rs_harmonogram_zadan_log"
            " (firma_id,harmzad_id, moment, uruchomione)"
            " VALUES ('" + firma_id + "', " + str(harmzad_id) + ", '" + formatTime(self.czas) + "', '" + formatTime(start) + "') ")
        self.PolSQL.Execute(str(query), 1)

        rsglobal.ZapiszDoLogow('===START===',1)
        rsglobal.ZapiszDoLogow(str(query),1)

        # commit pozwoli zobaczyc innym operatorom rozpoczete zadanie
        self.PolSQL.Commit()
        #-------------------------------------------------------------------------------

        try:
            # uruchomienie kodu wybranej funkcji
            exec code in globs, args
        except SystemExit, exc:
            if exc.code is None:
                args["stat"] = ""
            else:
                args["stat"] = exc.code
        except Exception, exc:
            args["stat"] = "BLAD"
            traceback.print_exc()
            bylWyjatek = True

        sys.stdout, sys.stderr = oldstdout, oldstderr
        if args["ListaBledow"]:
            self._env_out.write("\n\n" + u"\n".join(args["ListaBledow"]))
        if self._env_out.getvalue():
            # dane bd zapisywane do bazy, musz by potraktowane specjalnie (eh...)
            self._env_out = self._env_out.getvalue()
        else:
            self._env_out = ""
        
        #rsglobal.ZapiszDoLogow('PSQL bylo rob '+str(self.PolSQL.bylo_cos_robione),1)
        #rsglobal.ZapiszDoLogow('PSQL2 bylo rob '+str(self.PolSQL2.bylo_cos_robione),1)
        #rsglobal.ZapiszDoLogow('Osobne pol rap '+str(self.osobne_pol_rap),1)
        #rsglobal.ZapiszDoLogow('Stat '+str(args["stat"]),1)
        if args["stat"] != "OK":
            #self.PolSQL.Rollback()
            if self.PolSQL.bylo_cos_robione:
                self.PolSQL.Rollback()            
                self.PolSQL.bylo_cos_robione = False

            if self.osobne_pol_rap and self.PolSQL2.bylo_cos_robione:
                self.PolSQL2.Rollback()            
                self.PolSQL2.bylo_cos_robione = False
            # wyslanie wiadomosci z powiadomieniami
            # poza tym jeszcze wiadomoci z informacj, do usera gdy sobie yczy, do admina gdy by wyjtek
            
            mesg = u"Funkcja " + str(nazwa) + " uruchomiona " + formatTime(start) + u" zakoczya si bdem.\n\n" + self._env_out
            temat=u"[harmonogram] Bd wykonania zadania " + str(harmzad_id)
            if self.row[RSHZ_POWIADOM_GDY_BLAD]:
                ZapiszWiadomosc(self.PolSQL, self, mesg, temat , commit=False)
            if bylWyjatek:
                mesg = u"Zapis z wykonania " + nazwa + " o " + formatTime(start) + u": \nwartoci zmiennych:\n"
                for k, v in argsNaStarcie.items():
                    if k not in ("stat", "ListaBledow"):
                        mesg += (u"   %r: %r,\n" % (k, v))
                mesg += u"\n" + self._env_out
                temat=u"Bd wewntrzny w funkcji harmonogramu " + str(harmzad_id) + " - " + nazwa
                ZapiszWiadomosc(self.PolSQL, self, mesg, temat, commit=False, adresat_id=0)

            # pobranie adresw 
            SQL="SELECT DISTINCT email FROM rs_powiadomienia_pozycje WHERE firma_id='"+firma_id+"'"
            SQL+=" AND powiadomienie='HA001';"
            stat,DPoczta,dump,dump=self.PolSQL.Execute(str(SQL),3)
            if stat != 'OK':    
                stat="Bd pobierania listy kont pocztowych dla powiadomiania HA001: "+stat
            #KonczJesliBlad(stat)
            # wysanie maili
            rsglobal.ZapiszDoLogow('Tresc '+str(mesg),1)        
            for mail in DPoczta:
                rsglobal.ZapiszDoLogow('Mail do '+mail[0], 1)
                stat = rsglobal.RSWyslijEmail(mail[0], temat, mesg, [], 0, self.PolSQL)
                if stat != 'OK':
                    rsglobal.ZapiszDoLogow(stat,1)
                else:
                    rsglobal.ZapiszDoLogow(u'Wysano maile do osb z powiadomienia HA001', 1)
            
        if self.PolSQL.bylo_cos_robione:
            self.PolSQL.Commit()            
            self.PolSQL.bylo_cos_robione = False

        if self.osobne_pol_rap and self.PolSQL2.bylo_cos_robione:
            self.PolSQL2.Commit()            
            self.PolSQL2.bylo_cos_robione = False

        #-------------------------------------------------------------------------------

        # aktualizacja zapisu wykonania zadania, stat ignorowany
        query = ("UPDATE rs_harmonogram_zadan_log SET"
            " trwalo="+str((datetime.datetime.now() - start).seconds)+","
            " stat='"+args["stat"][:63].replace("'",'"')+"',"
            " wypisane="+rsglobal.ObrobTekst("STAT :"+args["stat"].replace("'",'"')+"\n"+self._env_out.replace("\\", "\\\\").replace("'", "''"))+" "
            " WHERE firma_id='"+firma_id+"' AND harmzad_id="+str(harmzad_id)+" "
            " AND moment='"+formatTime(self.czas)+"'"
            " AND uruchomione='"+formatTime(start)+"';")
        
        self.PolSQL.Execute(str(query), 1)

        rsglobal.ZapiszDoLogow('===AKTUALIZACJA===',1)
        rsglobal.ZapiszDoLogow(str(query),1)
        
        # wprowadzone zmiany od razu dostepne dla innych operatorow
        self.PolSQL.Commit()

        return args["stat"] == "OK"

#-------------------------------------------------------------------------------

    def _wyznaczNastepneWykonanie(self):

        update = []
        odstep = self.row[RSHZ_ODSTEP]
        ileOdst = self.row[RSHZ_ILE_ODSTEPOW]
        if not ileOdst:
            update.append("aktywne = FALSE")
        elif odstep in powtorzenieNaCzas:
            update.append("czas = czas + %d * INTERVAL '%s'" % (ileOdst, powtorzenieNaCzas[odstep][0]))
        else:
            rsglobal.ZapiszDoLogow(u"%s: Nieprawidlowy kod odstepu: %r dla zadania %r" %
                (logPrefix, odstep, self.id), 1)
            self.deaktywuj()
            self._exit(7)
        if update:
            query = ("UPDATE rs_harmonogram_zadan SET " + ", ".join(update) +
                " WHERE firma_id='" + self.id[0] + "' AND harmzad_id=" + str(self.id[1]))
            status, dump, dump, dump = self.PolSQL.Execute(query, 1)
            if status != "OK":
                rsglobal.ZapiszDoLogow(u"%s: proba ustawienia %s dla zadania %r nie powiodla sie." %
                    (logPrefix, ", ".join(update), self.id), 1)
                self.deaktywuj(rollback=True)
                self._exit(7)
        self.PolSQL.Commit()

#-------------------------------------------------------------------------------

    def _OnSIGTERM(self, sig, frame):

        rsglobal.ZapiszDoLogow(u"%s: %d dostalem SIGTERM w obsludze zadania %r %s" %
            (logPrefix, os.getpid(), self.id, self.row[RSHZ_NAZWA]), 2)
        self._exit(EINTR)

#-------------------------------------------------------------------------------

    def _exit(self, status):

        if self.PolSQL:
            #self.PolSQL.Rollback()
            self.PolSQL.RozlaczZSQL()
        if self.osobne_pol_rap:
            self.PolSQL2.RozlaczZSQL()
        os._exit(status)

#-------------------------------------------------------------------------------

    def _ustawRSFunkcje(self, dane):

        operator_id, firma_id, oddzial_id, okres_id = dane[:4]
        if int(operator_id) == int(STANDARD_OPERATOR_ID):
            operator_id = STANDARD_OPERATOR_ID
        else:
            operator_id = int(operator_id)
        # zdalne zlecenie ma by zachowywane jeli nie ustawiane na nowo
        if self._rsfunkcje:
            zdalne_zlecenie = self._rsfunkcje.zdalne_zlecenie
        else:
            zdalne_zlecenie = None
        # zapewnienie obiektu klasy RSFunkcje nawet przy zmienionych danych
        # cz danych mona zaktualizowa, zmiana innych wymanaga nowego obiektu
        if not self._rsfunkcje or self._rsfunkcje.operator_id != operator_id or \
                self._rsfunkcje.firma_id != firma_id:

            rsglobal.ZapiszDoLogow("%s: robi nowe RSFunkcje(%r, %r)" % (logPrefix, operator_id, firma_id), 5)
            # prba zestawienia rodowiska tak jak to jest robione w rs_anakonda_server_cl
            if operator_id == STANDARD_OPERATOR_ID:
                Operatorzy = "()"
                Uprawnienia = []
            else:
                LOperatorow = rs_anakonda_server_cl.PodajOperatorow(self.PolSQL, operator_id)
                Operatorzy = "(" + str(LOperatorow).strip("[]") + ")"
                Uprawnienia = rs_anakonda_server_cl.PodajUprawnienia(self.PolSQL, Operatorzy, firma_id)

            if self.osobne_pol_rap:
                self._rsfunkcje = rs_anakonda_server_fn.RSFunkcje(
                      operator_id, self.PolSQL, "LOCAL", Operatorzy,
                      PolSQL2=self.PolSQL2
                )
            else:
                self._rsfunkcje = rs_anakonda_server_fn.RSFunkcje(
                      operator_id, self.PolSQL, "LOCAL", Operatorzy
                )

            self._rsfunkcje.SetFirma(firma_id)
            if oddzial_id is not None:
                self._rsfunkcje.SetOddzial(oddzial_id)

            self._rsfunkcje.SetUprawnienia(Uprawnienia)
            self._rsfunkcje.SetHandlowiec(rs_anakonda_server_cl.PodajHandlowca(self.PolSQL, operator_id))
            if okres_id is not None:
               self._rsfunkcje.SetOkres(okres_id)
        else:
            rsglobal.ZapiszDoLogow("%s: uzyje RSFunkcje(%r, %r) do %r, %r" %(logPrefix, self._rsfunkcje.operator_id, self._rsfunkcje.firma_id, operator_id, firma_id), 5)
            # tylko drobna aktualizacja
            if self._rsfunkcje.oddzial_id != oddzial_id:
                self._rsfunkcje.SetOddzial(oddzial_id)
            if self._rsfunkcje.okres_id != okres_id:
                self._rsfunkcje.SetOkres(okres_id)

        if len(dane) > 4:
            self._rsfunkcje.zdalne_zlecenie = dane[4:]
        else:
            self._rsfunkcje.zdalne_zlecenie = zdalne_zlecenie

#-------------------------------------------------------------------------------

    def _env_wykonajRSFunkcje(self, Funkcja, Klucz, Pola, Dane, daneSrodowiska=None):

        if daneSrodowiska is None:
            operator_id = self.row[RSHZ_OPERATOR_DODAL]
            firma_id, harmzad_id = self.id

            # czyli w funkcji bdzie mona zmieni oddzia midzy wykonaniami
            if "oddzial_id" in self._env_argumenty:
                oddzial_id = self._env_argumenty["oddzial_id"]
            else:
                oddzial_id = None

            if "okres_id" in self._env_argumenty:
                okres_id = self._env_argumenty["okres_id"]
            else:
                okres_id = None

            daneSrodowiska = [operator_id, firma_id, oddzial_id, okres_id]
        rsglobal.ZapiszDoLogow("wykonam dla danych srodowiska: %r" % daneSrodowiska, 5)

        self._ustawRSFunkcje(daneSrodowiska)

        return self._rsfunkcje.WykonajFunkcje(Funkcja, Klucz, Pola, Dane)

#-------------------------------------------------------------------------------

    def _env_pokazJesliBlad(self, stat, komunikat=None):

        if stat != "OK" and self._env_out:
            if komunikat is None:
                komunikat = stat
            self._env_out.write(str(komunikat))
            self._env_out.write("\n")

#-------------------------------------------------------------------------------

    def _env_konczJesliBlad(self, stat, komunikat=None):

        self._env_pokazJesliBlad(stat, komunikat)
        if stat != "OK":
            sys.exit("BLAD")

#-------------------------------------------------------------------------------

    # przerywa wykonanie jesli wsrod argumentow brakuje ktoregos z wymienionych
    def _env_sprawdzArgumenty(self, wymanageArgumenty):

        brakuje = []
        for nazwa in wymanageArgumenty:
            if isinstance(nazwa, (list, tuple)):
                nazwa = nazwa[0]
            if nazwa not in self._env_argumenty:
                brakuje.append(nazwa)
        if brakuje:
            self._env_konczJesliBlad("BLAD", "\n Brakuje argumentw: " + ", ".join(brakuje))

#-------------------------------------------------------------------------------

    # zapisuje tekst do logow wykonania funkcji, zamiast print ...
    def _env_zapiszLog(self, tekst):

        # do uzywania tylko z funkcji, kiedy mamy juz srodowisko
        if not self._env_out:
            return

        if not isinstance(tekst, (unicode, str)):
            tekst = repr(tekst)

        self._env_out.write(tekst)
        self._env_out.write("\n")

#-------------------------------------------------------------------------------

    def deaktywuj(self, polsql=None, rollback=False):

        #if rollback:
        #    self.PolSQL.Rollback()
        if polsql is None:
            polsql = self.PolSQL
        if not polsql:
            return
        query = ("UPDATE rs_harmonogram_zadan SET aktywne=FALSE "
            " WHERE firma_id='" + self.id[0] + "' AND harmzad_id=" + str(self.id[1]))
        status, dump, dump, dump = polsql.Execute(query, 1)
        polsql.Commit()


#-------------------------------------------------------------------------------
#-- FUNKCJE POMOCNICZE
#-------------------------------------------------------------------------------

def formatTime(czas=None):
    # datetime.datetime -> "YYYY-MM-DD hh:mm:ss"

    if czas is None:
        czas = datetime.datetime.now()
    return czas.strftime("%F %T")

#-------------------------------------------------------------------------------

def readTime(s):
    # "YYYY-MM-DD hh:mm:ss" -> datetime.datetime

    assert len(s)>18 and (s[4], s[7], s[10], s[13], s[16]) == ('-', '-', ' ', ':', ':'), \
        "rs_anakonda_server_hz.readTime(): Nieprawidlowy format czasu %r" % s
    if s:
        ymd = s[:10].split("-")
        hms = s[11:19].split(":")
        return datetime.datetime(*map(int, ymd + hms))

#-------------------------------------------------------------------------------

def SQLConnect(dbspec):
    # tworzy nowe poczenie z baz dla procesu potomnego

    PolSQL = rssql.RSSQL()
    dbuser, dbpass, dbhost, dbport, dbase = dbspec
    if PolSQL.PolaczZSQL(dbuser, dbpass, dbhost, dbport, None, Baza=dbase):
        return False
    else:
        return PolSQL

#-------------------------------------------------------------------------------

def ZapiszWiadomosc(PolSQL, zadanie, tresc, temat=None, commit=True, adresat_id=None):

    if temat is None:
        temat = "Zadanie harmonogramu nr %d" % zadanie.id[1]
    if adresat_id is None:
        adresat_id = zadanie.row[RSHZ_OPERATOR_DODAL]
    
    query = "SELECT nextval('rs_wiadomosci_id')"
    stat, wiadomosc_id, dump, dump = PolSQL.Execute(query, 4)
    #print stat, query, wiadomosc_id
    if stat != "OK":
        return
    temat = temat.replace("\\", "\\\\").replace("'", "''")
    tresc = tresc.replace("\\", "\\\\").replace("'", "''")
    query = ("INSERT INTO rs_wiadomosci (firma_id, wiadomosc_id, operator_id, naglowek, tresc, priorytet, usunieta)"
        " VALUES ('" + zadanie.id[0] + "', " + str(wiadomosc_id) + ", " + str(adresat_id) + ", " +
        rsglobal.ObrobTekst(temat) + ", " + rsglobal.ObrobTekst(tresc) + ", 'N', FALSE)")
    stat, dump, dump, dump = PolSQL.Execute(str(query), 1)
    #print stat, query
    if stat != "OK":
        return
    query = ("INSERT INTO rs_wiadomosci_operatorzy (wiadomosc_id,folder_id,operator_id,usunieta,nowa)"
        " VALUES (" + str(wiadomosc_id) + ", 0, " + str(adresat_id) + ",FALSE,TRUE)")
    stat, dump, dump, dump = PolSQL.Execute(str(query), 1)
    #print stat, query

    if commit:
        PolSQL.Commit()
    else:
        rsglobal.ZapiszDoLogow(u"WIADOMO BEZ COMMITA", 2)
