# coding: ISO8859-2

# Jan Kaliszewski, 2009-08-19: zmiany dot. RELOAD-u serwera (mantis: 2109)

"""
  Mechanizm komunikacji miedzyserwerowej
"""

import os
import sys
import time
import fcntl
import errno
import socket
import signal
import cPickle
import asyncore
import binascii
import traceback

from OpenSSL import SSL

import rsglobal
#from opcje import opcje


WERSJA_PROTOKOLU = "0.10"
DEBUG = False

KONIEC_BLOKU = "\27"
ZNACZNIK_PRZERWANIA = "\0\0== PRZERWIJ TRANSMISJE ==\0\0" + KONIEC_BLOKU

MAG_CZEKAJ_NA_STARCIE = 1.4
MAG_ODSTEP_NOWE = 2
MAG_CZEKAJ_NA_BRAKUJACY = 10
TRANSPORT_TURA = 0.4
OPOZNIENIE_STARTU = 2
PONOWNE_POLACZENIE = 30
ODSTEP_PINGOW = 17

K_HELLO = 1
K_WIADOMOSC = 2
K_POTWIADOM = 3
K_KONIEC = 4
K_POWTORZ = 5
K_BLAD = 6
K_SKOKNUMERACJI = 7
K_PING = 8
K_DEBUGTEXT = 78

ZNANE_KODY = (K_HELLO, K_WIADOMOSC, K_POTWIADOM, K_KONIEC, K_BLAD,
    K_SKOKNUMERACJI, K_PING, K_DEBUGTEXT)


class ZarzadcaTransportu:

    def __init__(self):

        self.pidPotomka = None
        self.uruchomiony = False
        self.transport = None
        self.magazyn = None
        self.petla = True
        self.opcje = None

    def Uruchom(self, opcje, czekaj=OPOZNIENIE_STARTU):

        self.opcje = opcje
        for opt in ("komserhost", "komserport", "komserkatalog", "komsernazwa"):
            opcje[opt]
        if not os.path.isdir(opcje["komserkatalog"]):
            rsglobal.ZapiszDoLogow(u"komser: brak katalogu %r" % opcje["komserkatalog"], 1)
            return False
        if not self.opcje["komserhost"] or not self.opcje["komserport"]:
            return False
        pid = os.fork()
        self.uruchomiony = True
        if pid:
            self.pidPotomka = pid
        else:
            if "komserlog" in opcje:
                rsglobal.LogFile = opcje["komserlog"]
            self.petla = True
            self.pidPotomka = None
            #os.close(0)
            #os.close(1)
            #sys.stdout = open("komser-%s.log" % opcje["komsernazwa"], "a")
            signal.signal(signal.SIGTERM, self._onSIGTERM)
            time.sleep(czekaj)
            rsglobal.ZapiszDoLogow("komser uruchomiony pid=%s" % os.getpid(), 1)
            self.main(opcje)
            os.close(2)
            sys.stdout.close()
            self._exit(0)

    def Zatrzymaj(self):

        self.petla = False
        rsglobal.ZapiszDoLogow("komser: zlecono zatrzymanie", 4)
        if not self.pidPotomka:
            return False
        pidPotomka = self.pidPotomka
        self.pidPotomka = None  # wane, by to zrobi przed os.kill()
        os.kill(pidPotomka, signal.SIGTERM)
        self.uruchomiony = False

    def _onSIGTERM(self, sig, frame):
        # konczy prace daemona

        #print "%s %r._onSIGTERM(%r, %r)" % (os.getpid(), self, sig, frame)
        self.petla = False

    def _exit(self, status=None):
        # zwalnia zasoby i przerywa proces, tylko w daemonie
        
        rsglobal.ZapiszDoLogow("komser: _exit(%r)  %s, %s" % (status, self.uruchomiony, self.pidPotomka), 5)
        #if not self.uruchomiony or self.pidPotomka:
        #    return
        os._exit(status)

    def main(self, opcje):

        if not self.petla:
            return

        self.transport = TCPTransport(self)

        if self.transport.czySerwer():
            #print "ustawiam magazyn centralny", opcje["komserkatalog"], opcje["komsernazwa"], self.transport.znaniKlienci.keys()
            magazyn = MagazynCentralny()
            magazyn.Ustaw(opcje["komserkatalog"], opcje["komsernazwa"], self.transport.znaniKlienci.keys())
            self.magazyn = self.transport.magazyn = magazyn
        else:
            #print "ustawiam magazyn kliencki", opcje["komserkatalog"], opcje["komsernazwa"], self.transport.znaniKlienci.keys()
            magazyn = Magazyn()
            magazyn.Ustaw(opcje["komserkatalog"], opcje["komsernazwa"], self.transport.znaniKlienci.keys())
            self.magazyn = magazyn
            rsglobal.ZapiszDoLogow("komser: ustawiem magazyn do komunikacji z %s" % self.transport.znaniKlienci.keys(), 4)

        self.transport.Uruchom()

        try:
            while self.petla:
                czas = time.time()
                self.magazyn.tura(czas)
                self.transport.tura(czas)
                # TODO: wywal to
                #sys.stdout.flush()
            self.transport.Zatrzymaj()
        except Exception, exc:
            print "*****   ", exc
            raise
        except KeyboardInterrupt:
            self.transport.Zatrzymaj()
            print "   INTERRUPTED"

    def sygnalSIGCHLD(self, pid, status):

        self.pidPotomka = None
        if "komserhost" in self.opcje:
            self.Uruchom(self.opcje, czekaj=30)

    def Reload(self, opcje):

        if not opcje and self.pidPotomka:
            self.Zatrzymaj()
            return "komser zatrzymany zgodnie z now konfiguracj"
        elif not self.pidPotomka:
            self.Uruchom(opcje)
            return "Uruchamiam komser"
        else:
            self.Zatrzymaj()
            self.Uruchom(opcje, czekaj=rsglobal.OPOZNIENIE_RESTARTU_KOMSERA)
            return "Zatrzymaem komser i za 3 minuty zostanie uruchomiony ponownie"


# Podkatalog magazynu
class Podkatalog:

    ARCHIWUM = "stare"
    
    # @magazyn - magazyn do ktorego ten Podkatalog nalezy
    # @podkatalog - sciezka tego podkatalogu
    # @wlasciciel - nazwa klienta ktoremu jest przypisany
    # @czytaj - bedzie uzywany do wczytywania
    # @pisz - tutaj bedzie zapisywal
    def __init__(self, magazyn, subdir, wlasciciel, czytaj=False, pisz=False):

        self.magazyn = magazyn
        self.podkatalog = subdir
        self.katalog = os.path.join(self.magazyn.katalog, self.podkatalog)
        self.wlasciciel = wlasciciel
        self.czytaj = czytaj
        self.pisz = pisz
        self.status = None

        if not os.path.exists(self.katalog):
            os.makedirs(self.katalog)
        if self.czytaj and not os.path.exists(os.path.join(self.katalog, self.ARCHIWUM)):
            os.makedirs(os.path.join(self.katalog, self.ARCHIWUM))

    def link(self, juzNaDysku, spec):

        #print "s", juzNaDysku, "d", spec
        src = os.path.join(self.magazyn.katalog, juzNaDysku)
        if not os.path.dirname(spec):
            dst = os.path.join(self.katalog, spec)
        else:
            dst = os.path.join(self.magazyn.katalog, spec)
        #print "%s -> %s" % (src, dst)
        try:
            os.link(src, dst)
        except OSError, exc:
            if exc.errno == errno.EEXIST:
                rsglobal.ZapiszDoLogow("komser: link dst exists: %s -> %s" % (src, dst), 3)
            else:
                rsglobal.ZapiszDoLogow("komser: link %s -> %s: %s" % (src, dst, exc), 2)
            return

    def zapisz(self, wiadomosc, juzNaDysku=None):

        spec = os.path.join(self.podkatalog, wiadomosc.id)

        if juzNaDysku:
            self.link(juzNaDysku, spec)
            return juzNaDysku
        else:
            #print "**", os.path.join(self.katalog, wiadomosc.id)
            try:
                f = open(os.path.join(self.katalog, wiadomosc.id), "w")
                f.write(wiadomosc.tresc)
                f.close()
            except Exception, exc:
                rsglobal.ZapiszDoLogow("komser: Podkatalog %s zapisz %s@%s: %s" % (self.podkatalog, wiadomosc.id, wiadomosc.zrodlo, exc), 1)
                return None
            return spec

    def usun(self, id_):

        sciezka = os.path.join(self.katalog, id_)
        if os.path.exists(sciezka):
            os.remove(sciezka)

    def doArchiwum(self, id_):

        try:
            os.rename(os.path.join(self.katalog, id_), os.path.join(self.katalog, self.ARCHIWUM, id_))
        except OSError, exc:
            rsglobal.ZapiszDoLogow("komser: rename(%r, %r): %s" %
                (os.path.join(self.katalog, id_), os.path.join(self.katalog, self.ARCHIWUM, id_), exc), 1)
            return False
        return True

    # lista nazw plikow w tym podkatalogu
    # @wszystkie - czy ma pokazac takze ukryte, tymczasowe i nieprawidlowe
    def pliki(self, wszystkie=False):
        
        pliki = os.walk(self.katalog).next()[2]
        if not wszystkie:
            pliki = filter(lambda x: x[0] != "_", pliki)
            # te o nieprawidlowych nazwach tez pomin
            pliki = filter(lambda x: "_" in x, pliki)
        return pliki

    def __repr__(self):

        return "<%s %s %r>" % (self.__class__.__name__, self.wlasciciel, self.podkatalog)

    def id2spec(self, id_):

        return os.path.join(self.podkatalog, id_)

    def wczytaj(self, idLubSpec):
        
        if os.sep in idLubSpec:
            filepath = os.path.join(self.magazyn.katalog, idLubSpec)
            id_ = os.path.basename(idLubSpec)
        else:
            filepath = os.path.join(self.katalog, idLubSpec)
            id_ = idLubSpec

        wiad = Wiadomosc()
        wiad.id = id_
        if os.path.exists(filepath):
            wiad.tresc = open(filepath).read()
            wiad.crc = wiad.obliczCrc()
            return wiad

# spec - okresla zachowana wiadomosc w ramach magazynu, w tym id
class Magazyn:

    IN = "out"
    OUT = "in"
    TEMP = "__TEMP_"

    def __init__(self, parent=None):

        self.parent = parent
        self.centralny = False
        self.katalog = None
        self.klienci = []
        self.obserwuj = []
        self.zapisuj = {}

        self.kanal = None
        self.nazwaLokalna = None
        self.statusy = {}
        self.specPrefiksy = {}
        self.czekajdo = 0

    def _ustawKatalog(self, katalog, nazwa):

        assert os.path.isdir(katalog), katalog

        self.nazwaLokalna = nazwa
        self.katalog = katalog

    def Ustaw(self, katalog, nazwa, klienci):

        self.centralny = False
        self._ustawKatalog(katalog, nazwa)
        self.obserwuj = []

        self.zapisuj = {}
        for k in klienci:
            if k == nazwa:
                continue
            self.zapisuj[k] = Podkatalog(self, os.path.join(k, self.OUT), k, pisz=True)
            self.obserwuj.append(
                Podkatalog(self, os.path.join(k, self.IN), k, czytaj=True)
            )

        self._zrobStatus()

    def _zrobStatus(self):

        self.statusy = {}
        self.specPrefiksy = {}
        for podkatalog in self.obserwuj + self.zapisuj.values():
            self.statusy[podkatalog] = StatusWymiany()
            self.specPrefiksy[podkatalog.podkatalog] = podkatalog

    # za pomoca tej funkcji magazyn jest regularnie odpytywany o nowe
    # wiadomosci pojawily sie w katalogach i maja byc wyslane
    def nowe(self, czas):

        if not self.czekajdo:
            self.czekajdo = czas + MAG_CZEKAJ_NA_STARCIE
            return []
        elif self.czekajdo > czas:
            return []
        self.czekajdo = czas + MAG_ODSTEP_NOWE

        spece = []
        for podkatalog in self.obserwuj:
            pliki = self._nowe(czas, podkatalog)
            #print podkatalog.podkatalog, pliki
            spece += pliki
        #if spece:
        #    print u"do wyslania", spece
        return spece

    def _nowe(self, czas, podkatalog):

        status = self.statusy[podkatalog]
        files = podkatalog.pliki()
        #print repr(podkatalog), files
        # pomin te ktore byly juz podane
        files = filter(lambda x: x not in status.widziane, files)
        if not files:
            return []
        files.sort(self.cmp)

        if status.kursWidziane is None or self.czyNastepnik(status.kursWidziane, files[0]):
            files = self._kolejne(status.kursWidziane, files)
            status.kursWidziane = files[-1]
            if files:
                status.czekajNaBrakujacy = None
        elif status.czekajNaBrakujacy:
            if status.czekajNaBrakujacy < czas:
                self.kanal.nieciaglosc(status.kursWidziane, files[0])
                status.czekajNaBrakujacy = None
                files = self._kolejne(None, files)
                status.kursWidziane = files[-1]
            else:
                return []
        else:
            status.czekajNaBrakujacy = czas + MAG_CZEKAJ_NA_BRAKUJACY
            return []

        status.widziano(files)
        return [podkatalog.id2spec(x) for x in files]

    def dajPoSpec(self, wiadSpec):

        dir = os.path.dirname(wiadSpec)
        if dir not in self.specPrefiksy:
            return None
        wiadomosc = self.specPrefiksy[dir].wczytaj(wiadSpec)
        wiadomosc.zrodlo = self.specPrefiksy[dir].wlasciciel
        return wiadomosc

    # usuwa wiadomosc ktora zostala juz wyslana, uznana za przetworzona
    def Usun(self, zrodlo, id_):
        pk = self._podkatalogKlienta(self.obserwuj, zrodlo)
        if pk:
            pk.usun(id_)

    # zamiast usuwac lepiej bedzie przeniesc do historii
    def doArchiwum(self, zrodlo, id_):
        pk = self._podkatalogKlienta(self.obserwuj, zrodlo)
        if pk:
            pk.doArchiwum(id_)

    def spec2id(self, spec):

        return os.path.basename(spec)

    def _id2nr(self, id_):

        assert id_ and "_" in id_, "'_' in %r" % id_
        # FIXME: sypnelo sie na pliku z podwjnym rozszerzeniem
        return int(os.path.splitext(id_)[0].split("_")[-1])

    def czyNastepnik(self, id1, id2):

        return self._id2nr(id2) == self._id2nr(id1) + 1

    def cmp(self, id1, id2):

        return cmp(self._id2nr(id1), self._id2nr(id2))

    def _podkatalogKlienta(self, lista, klient):

        for pk in lista:
            if pk.wlasciciel == klient:
                return pk
        return None

    # zapisuje podana wiadomosc przychodzaca
    def Zachowaj(self, wiadomosc):

        pk = self._podkatalogKlienta(self.zapisuj.values(), wiadomosc.zrodlo)
        if pk:
            res = pk.zapisz(wiadomosc)
            return bool(res)
        else:
            rsglobal.ZapiszDoLogow("komser: Magazyn.Zachowaj() - %s - brak PK" % wiadomosc.zrodlo, 1)
            return False

    # z listy @ids zwraca kolejne idy
    def _kolejne(self, start, ids):

        if start is None:
            start = ids[0]
            s = 1
        else:
            s = 0

        for i in range(s, len(ids)):
            if not self.czyNastepnik(start, ids[i]):
                return ids[:i]
            start = ids[i]
        return ids

    def tura(self, czas):

        pass

# magazyn glowny na serwerze
# przepisuje z katalogu wejsciowego serwera do przekazania innym klientom
class MagazynCentralny(Magazyn):

    def __init__(self, parent=None):

        Magazyn.__init__(self)
        self.zapisuj = []
        self.obserwuj = None

    def tura(self, czas):

        return

        for p in self.obserwuj.pliki():
            id_ = os.path.basename(p)
            for podkatalog in self.zapisuj:
                #print >> sys.stderr, podkatalog.katalog, podkatalog.podkatalog, p, id_, self.obserwuj.id2spec(p)
                podkatalog.link(self.obserwuj.id2spec(p), p)
            self.obserwuj.usun(id_)

    def Ustaw(self, katalog, nazwa, klienci):

        self._ustawKatalog(katalog, nazwa)

        self.centralny = True
        self.klienci = klienci
        #self.obserwuj = Podkatalog(self, os.path.join(nazwa, self.IN), nazwa, czytaj=True)
        self.zapisuj = []
        #for k in self.klienci:
        #    if k == self.nazwaLokalna:
        #        continue
        #    self.zapisuj.append(
        #        Podkatalog(self, os.path.join(k, self.IN, nazwa), k, pisz=True)
        #    )

    def magazynKlienta(self, klient):

        assert self.katalog
        mag = MagazynCentralnyDlaKlienta(self)
        mag.Ustaw(self.katalog, klient, self.klienci)
        return mag


class MagazynCentralnyDlaKlienta(Magazyn):

    def __init__(self, parent):

        Magazyn.__init__(self, parent)

    def Ustaw(self, katalog, nazwa, klienci):

        self.centralny = False
        self._ustawKatalog(katalog, nazwa)

        #if self.parent.nazwaLokalna not in klienci:
        #    klienci = klienci + [self.parent.nazwaLokalna]

        # zapisuj to lista katalogow gdzie maja byc zapisane kopie wiadomosci
        self.zapisuj = {}
        self.zapisuj[self.parent.nazwaLokalna] = \
            Podkatalog(self, os.path.join(nazwa, self.OUT), self.parent.nazwaLokalna, pisz=True)
        self.obserwuj = [
            Podkatalog(self, os.path.join(nazwa, self.IN), self.parent.nazwaLokalna, czytaj=True)
        ]
            
        #for k in klienci:
        #    if k == nazwa:
        #        continue

        #    # obslugiwany-klient/IN/jakis-peer
        #    self.obserwuj.append(
        #        Podkatalog(self, os.path.join(nazwa, self.IN, k), k, czytaj=True)
        #    )

        #    if k != self.parent.nazwaLokalna:
        #        # inny-klient/IN/obslugiwany-klient
        #        self.zapisuj[k] = \
        #            Podkatalog(self, os.path.join(k, self.IN, nazwa), k, pisz=True)

        self._zrobStatus()

    def tura(self, czas):

        pass

    def Zachowaj(self, wiadomosc):

        res = self.zapisuj[self.parent.nazwaLokalna].zapisz(wiadomosc)
        return bool(res)

        #juzZapisane = None
        #   przekazywanie od razu zostao wylaczone, tylko na zadanie
        #for podkatalog in self.zapisuj.values():
        #    juzZapisane = podkatalog.zapisz(wiadomosc, juzZapisane)


# informacja o tym co zostalo wyslane, co dostalem itp
# HISTORIA - ilosc idow trzymanych w historii
class StatusWymiany:

    # wyslane - idy wiadomosci wyslanych, oczekujacych na potwierdzenie
    # potwierdzone - ostatnie potwierdzone
    # kursorWyslane - max(id) w sekwencji potwierdzonych
    # zapisane - ostatnie zapisane
    # kursZapisane - max(id) w sekwencji zapisanych
    def __init__(self):

        self.widziane = {}
        self.kursWidziane = None
        self.wyslane = {}
        self.kursWyslane = None
        self.potwierdzone = []
        self.kursZapisane = None
        self.zapisane = []

        self.czekajNaBrakujacy = 0

    def _doHistorii(self, kolejka, id_):

        kolejka.insert(0, id_)
        if len(kolejka) > self.HISTORIA:
            kolejka.pop()

    # informacja o tym ze tresc o danym id_ zostala wyslana
    def wyslano(self, id_):

        self.wyslane[id_] = True

    # oznaczenie ze wzkazany id odpowiada tresci ktorej odbior potwierdzono
    # zwracalby liste idow az do kursorOdebrane
    def potwierdzono(self, id_):

        if id_ in self.wyslane:
            del self.wyslane[id_]
        self._doHistorii(self.potwierdzono, id_)
        # aktualizowac kursorWyslane

    # oznaczenie ze wzkazany id odpowiada tresci zapisanej
    # zwracalby liste idow az do kursorZapisane
    def zapisano(self, id_):

        self._doHistorii(self.zapisane, id_)
        # aktualizowac kursorZapisane

    # na przypadek nieciaglosci wysylanych
    def skokWysylane(self, ostatniByl, nastepnyBedzie):
        pass

    # wskazanie nieciaglosci wsrod odbieranych
    def skokOdbierane(self, ostatniByl, nastepnyBedzie):
        pass

    def widziano(self, pliki):

        self.widziane.update(dict.fromkeys(pliki, True))

    # zapisuje przechowywane dane do strumienia
    def Zachowaj(self, stream):

        cPickle.dump(self.__dict__, stream)

    # odczytuje dane zapisane przez Zachowaj
    def Odtworz(self, stream):

        dane = cPickle.load(stream)
        self.__dict__.update(dane)


def int2nl(v):

    return binascii.unhexlify("%08x" % v)

def nl2int(s):

    #assert len(s) == 4
    return int(binascii.hexlify(s), 16)


# przyjmowanie nowych polaczen
class TCPSluchacz(asyncore.dispatcher):

    def __init__(self, parent, host, port):

        asyncore.dispatcher.__init__(self)
        self.parent = parent
        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        while self.parent.zarzadca.petla:
            try:
                self.bind((host, port))
            except socket.error, exc:
                # ignorujemy error(98, 'Address already in use') jesli ustawiono reconnect
                if PONOWNE_POLACZENIE and exc.args[0] == errno.EADDRINUSE:
                    time.sleep(PONOWNE_POLACZENIE)
                    continue
                raise
            break
        self.listen(3)

    def handle_accept(self):

        try:
            sock, addr = self.accept()
            backsock = sock
            if self.parent.opcje['komserssl'] and self.parent.opcje['SSLContext']:
                sock = SSL.Connection(self.parent.opcje["SSLContext"], sock)
                sock.set_accept_state()
            elif self.parent.opcje['komserssl'] and not self.parent.opcje['SSLContext']:
                # by moe Context nie zosta przekazany jak naley, albo brakuje plikw w katalogu ssl/
                rsglobal.ZapiszDoLogow("komser: ustawiono komserssl a SSL nie jest dostepny." % exc, 1)
                sock.close()
                return

            try:
                self.parent.nowePolaczenie(sock, addr)
            except SSL.Error:
                # SSL jest wczone, ale wystpi bd, dajmy szans klientowi - chocia to si zazwyczaj nie uda
                backsock.settimeout(0.5)
                tresc = chr(K_HELLO) + cPickle.dumps(("HELLO", WERSJA_PROTOKOLU, self.nazwaLokalna, int(time.time()), 'SSL'), 1)
                backsock.send(int2nl(len(tresc) + 1) + tresc + KONIEC_BLOKU)
        except socket.error, exc:
            rsglobal.ZapiszDoLogow("komser: accept error: %s" % exc, 1)


class TCPKanal(asyncore.dispatcher):

    def __init__(self, parent, sock, addr):

        asyncore.dispatcher.__init__(self, sock)
        self.parent = parent
        self.magazyn = None
        self.inbuf = ""
        self.outbuf = ""
        self.inpaczki = []
        self.outpaczki = []
        self.nazwaZdalna = None
        self.onPaczka = None
        # bledy: kiedy ostatni w serii, ile w serii
        self.bledy = (0, 0)
        self.zestawiony = False
        self.pingnij = None

    def handle_close(self):

        rsglobal.ZapiszDoLogow("komser: rozlaczenie %s" % self.addr, 4)
        self.close(czysto=False)
        self.OnClose()

    def writable(self):

        return asyncore.dispatcher.writable(self) and len(self.outbuf) > 0

    def handle_write(self):

        sent = self.send(self.outbuf)
        self.outbuf = self.outbuf[sent:]

    def handle_read(self):

        dane = self.recv(1024)
        if dane == ZNACZNIK_PRZERWANIA:
            self.inbuf = ""
        else:
            self.inbuf += dane
            #print "inbuf: %r" % self.inbuf
            while self.wypakuj():
                pass

    def zapakuj(self, kod, dane=None):

        tresc = chr(kod) + cPickle.dumps(dane, 1)
        self.outbuf += int2nl(len(tresc) + 1) + tresc + KONIEC_BLOKU

    def wypakuj(self):

        if len(self.inbuf) < 5:
            return False
        ile = nl2int(self.inbuf[:4])
        kod = ord(self.inbuf[4])
        # czy nie jest zbyt duza zeby w to uwierzyc
        if ile > 2 * 1024 * 1024 or kod not in ZNANE_KODY:
            self.inbuf = ""
            return False
        # znacznik dlugosci(4) + kod typu(1) + wiadomosc(min 1)
        if len(self.inbuf) < ile + 4:
            return False
        if self.inbuf[ile+3] != KONIEC_BLOKU:
            self.inbuf = ""
            # mozna jeszcze wyslac informacje o statusie do rozmowcy
            return False
        tresc = self.inbuf[5:ile+3]
        self.inbuf = self.inbuf[ile+4:]
        try:
            paczka = cPickle.loads(tresc)
        except Exception, exc:
            rsglobal.ZapiszDoLogow("komser: Blad parsowania otrzymanej tresci: %s" % exc, 1)
            return True
        self.inpaczki.append((kod, paczka))
        return True
        #print kod, repr(paczka)[:80], len(self.inpaczki)

    def tura(self, czas):

        if self.zestawiony:
            for spec in self.magazyn.nowe(czas):
                self.wyslij(K_WIADOMOSC, self.magazyn.dajPoSpec(spec))

            if self.pingnij and self.pingnij < czas:
                if int(self.pingnij) % 5 == 0:
                    self.wyslij(K_PING, True)
                else:
                    self.wyslij(K_PING, None)
                self.pingnij = czas + ODSTEP_PINGOW

        while self.inpaczki:
            kod, dane = self.inpaczki.pop(0)
            self.przezujPaczke(kod, dane)

        while self.outpaczki:
            self.zapakuj(*self.outpaczki.pop(0))

    def wyslij(self, kod, dane):

        if self.onPaczka:
            self.onPaczka(self, False, kod, dane)

        self.outpaczki.append((kod, dane))

    def przezujPaczke(self, kod, dane):

        if self.onPaczka:
            self.onPaczka(self, True, kod, dane)

        if kod == K_HELLO:
            if dane[1] != WERSJA_PROTOKOLU:
                self.wyslij(K_KONIEC, "wersja protokolu: %r != %r" % (dane[1], WERSJA_PROTOKOLU))
                return
            if self.nazwaZdalna != dane[2]:
                rsglobal.ZapiszDoLogow("komser: %s przedstawia si jako %s" % (self.addr, dane[2]))
                pass
            self.zestawiony = True

        elif not self.zestawiony:
            rsglobal.ZapiszDoLogow("komser: polaczenie nie zestawione a mam %s" % kod, 5)

        elif kod == K_PING:
            if dane is True:
                self.wyslij(K_PING, False)

        elif kod == K_WIADOMOSC:
            if dane.sprawdzCrc():
                if self.magazyn.Zachowaj(dane):
                    self.wyslij(K_POTWIADOM, (dane.zrodlo, dane.id))
            else:
                self.wyslij(K_POWTORZ, (dane.zrodlo, dane.id))

        elif kod == K_POTWIADOM:
            zrodlo, id_ = dane
            self.magazyn.doArchiwum(zrodlo, id_)

        elif kod == K_KONIEC:
            rsglobal.ZapiszDoLogow("komser: peer zlecil rozlaczenie %s" % dane, 4)

        elif kod == K_BLAD:
            rsglobal.ZapiszDoLogow("komser: rozmowca zglosil blad %s" % dane, 1)

        elif kod == K_SKOKNUMERACJI:
            rsglobal.ZapiszDoLogow("komser: %s zglosil skok numeracji %s" % (self.nazwaZdalna, dane), 1)

        else:
            rsglobal.ZapiszDoLogow("komser: nieznany kod wiadomosci %s, %r, %r" % (self.nazwaZdalna, kod, dane), 1)

    def handle_expt(self):

        rsglobal.ZapiszDoLogow("komser error:  handle_expt ", 4)
        asyncore.dispatcher.handle_expt(self)

    def handle_error(self):

        czas = time.time()
        if self.bledy[0] + 30 < czas:
            self.bledy = (czas, 0)
        else:
            self.bledy = (czas, self.bledy[1] + 1)
            if self.bledy[1] > 100:
                if isinstance(self.socket, SSL.ConnectionType):
                    rsglobal.ZapiszDoLogow("komser error: %s, %s" % (self, self.socket.state_string()), 1)
                    if 'read client hello A' in self.socket.state_string():
                        rsglobal.ZapiszDoLogow("Nie zestawiono polaczenia SSL. Czy klient %s uzywa SSL?" % self.nazwaZdalna, 1)
                else:
                    rsglobal.ZapiszDoLogow("komser error: %s" % self, 1)
                self.close(czysto=False)
                self.OnClose(blad=True)

    def nieciaglosc(self, ostatni, nowy):

        self.wyslij(K_SKOKNUMERACJI, (self.parent.nazwaLokalna, ostatni, nowy))

    def close(self, czysto=True):

        if czysto and self.connected:
            if self.outbuf:
                self.outbuf = ""
                self.send(ZNACZNIK_PRZERWANIA)
                #print "ZNACZNIK PRZERWANIA"
            self.zapakuj(K_KONIEC, "close()")
            self.send(self.outbuf)
            #print "wyslalem:", repr(self.outbuf)
            self.outbuf = ""
        asyncore.dispatcher.close(self)

    def OnClose(self, blad=False):

        self.parent.zamknijPolaczenie(self, zlecone=False)
        self.outbuf = ""
        self.wyslij = lambda s, k, d: None

        while self.wypakuj():
            pass
        self.inbuf = ""

        while self.inpaczki:
            kod, dane = self.inpaczki.pop(0)
            self.przezujPaczke(kod, dane)

        self.outbuf = ""


# przesylanie wiadomosci przez siec, tcp
class TCPTransport:

    def __init__(self, zarzadca):

        self.serwer = None
        self.opcje = zarzadca.opcje
        self.polaczenia = []
        self.zarzadca = zarzadca
        self.znaniKlienci = {}
        self.nazwaLokalna = None
        self.znaniKlienci = dict.fromkeys(self.opcje["komserklienci"].split(","), {"ip":None})
        self.czekajIPolacz = None

        for k in self.opcje:
            if not k.startswith("komser-"):
                continue
            nazwa = k[7:]
            v = self.opcje[k]
            ip, = v.split(",")[:1]
            self.znaniKlienci[nazwa] = {"ip": ip}

    def czySerwer(self):

        opcje = self.opcje or self.zarzadca.opcje
        return self.serwer or ("komserwer" in opcje and opcje["komserwer"] and opcje["komserwer"] not in ("0", ))

    def Uruchom(self):

        self.nazwaLokalna = self.opcje["komsernazwa"]

        if self.czySerwer():
            self.startSerwera(self.opcje)
        else:
            self.startKlienta(self.opcje)

    # zatrzymuje serwer wymiany wiadomosci lub rozlacza klienta
    def Zatrzymaj(self):

        if self.serwer:
            self.serwer.close()
            self.serwer = None
        else:
            for kanal in self.polaczenia[:]:
                self.zamknijPolaczenie(kanal)

    # uruchamia centralny serwer
    # @opcje - slownik z parametrami
    def startSerwera(self, opcje):

        self.opcje = opcje
        rsglobal.ZapiszDoLogow("komser: klienci %r" % self.znaniKlienci, 3)
        host = self.opcje["komserhost"]
        port = int(self.opcje["komserport"])

        self.serwer = TCPSluchacz(self, host, port)

    def startKlienta(self, opcje):

        rsglobal.ZapiszDoLogow("komser: startKlienta()", 2)
        self.opcje = opcje
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        addr = (self.opcje["komserhost"], int(self.opcje["komserport"]))
        odstep = PONOWNE_POLACZENIE
        pierwszaProba = True
        ssl = None
        while self.zarzadca.petla:
            try:
                if opcje['komserssl']:
                    ssl = SSL.Connection(SSL.Context(SSL.SSLv3_METHOD), sock)
                    ssl.connect(addr)
                    ssl.do_handshake()
                else:
                    sock.connect(addr)
            except socket.error, exc:
                rsglobal.ZapiszDoLogow("komser: prba poczenia - %s" % exc, 3)
                if not PONOWNE_POLACZENIE:
                    rsglobal.ZapiszDoLogow("komser: nie mona si poczy a wyczono ponawianie, kocz" % exc, 1)
                    return
                # error(110, 'Connection timed out')
                # error(111, 'Connection refused')
                if PONOWNE_POLACZENIE and isinstance(exc.args, tuple) and exc.args[0] in (110, 111):
                    if odstep < PONOWNE_POLACZENIE * 10 and not pierwszaProba:
                        odstep += PONOWNE_POLACZENIE
                else:
                    if odstep < 1800 and not pierwszaProba:
                        odstep *= 2
                time.sleep(odstep)
                pierwszaProba = False
                continue
            except Exception, exc:
                rsglobal.ZapiszDoLogow("komser: startKlienta() - bd przy prbie poczenia\n%s" % traceback.format_exc(), 1)
            break
        if opcje['komserssl']:
            sock = ssl
        polaczenie = self.nowePolaczenie(sock, addr, wychodzace=True)
        polaczenie.pingnij = time.time() + ODSTEP_PINGOW
        if opcje['komserssl']:
            polaczenie.wyslij(K_PING, True)
        return polaczenie

    def nowePolaczenie(self, sock, addr, wychodzace=False):

        klient = self._znajdzKlienta(addr[0])
        if wychodzace:
            mag = self.zarzadca.magazyn
        else:
            if not klient:
                # nieznany klient
                rsglobal.ZapiszDoLogow("komser: odrzucone polaczenie przychodzace od nieznanego klienta %s" % (addr,), 2)
                tresc = chr(K_BLAD) + cPickle.dumps("  unknown client  %s    \n" % addr[0], 1)
                outbuf = int2nl(len(tresc) + 1) + tresc + KONIEC_BLOKU
                tresc = chr(K_KONIEC) + cPickle.dumps(K_BLAD, 1)
                outbuf += int2nl(len(tresc) + 1) + tresc + KONIEC_BLOKU
                sock.send(outbuf)
                sock.close()
                return
            #print >> sys.stderr, self.zarzadca, self.zarzadca.magazyn
            mag = self.zarzadca.magazyn.magazynKlienta(klient)
            mag.nazwaLokalna = self.nazwaLokalna

        rsglobal.ZapiszDoLogow("komser: nowe polaczenie %s %r, %s" % (wychodzace and "wychodzace" or "przychodzace", addr, klient), 2)
        conn = TCPKanal(self, sock, addr)
        conn.nazwaZdalna = klient
        if "onPaczka" in self.opcje and callable(self.opcje["onPaczka"]):
            conn.onPaczka = self.opcje["onPaczka"]
        #conn.onPaczka = lambda conn, wych, typ, dane: rsglobal.ZapiszDoLogow('%s %s %s %r' % (conn.nazwaZdalna, wych and '>' or '<', typ, dane))
        conn.magazyn = mag
        mag.kanal = conn
        self.polaczenia.append(conn)
        conn.wyslij(K_HELLO, ("HELLO", WERSJA_PROTOKOLU, self.nazwaLokalna, int(time.time())))
        return conn

    # @zlecone - dostaje polecenie rozlaczenia, nie lacz ponownie
    def zamknijPolaczenie(self, kanal, zlecone=True):

        rsglobal.ZapiszDoLogow("komser: TCPTransport.zamknijPolaczenie()", 5)
        if kanal.connected:
            kanal.close(zlecone)

        if not zlecone and PONOWNE_POLACZENIE and not self.czySerwer():
            # wyczyszczenie statusu w chwili rozlaczenia, niech nastepne polaczenie zaczyna od zera
            for key in kanal.magazyn.statusy:
                kanal.magazyn.statusy[key] = StatusWymiany()

            self.czekajIPolacz = time.time() + PONOWNE_POLACZENIE

        self.polaczenia.remove(kanal)
        kanal.magazyn.kanal = None
        kanal.magazyn = None

    def _znajdzKlienta(self, ip):

        for k, v in self.znaniKlienci.items():
            if v["ip"] == ip:
                return k

    def tura(self, czas):

        koniec = czas + TRANSPORT_TURA

        if self.czekajIPolacz and self.czekajIPolacz < czas:
            rsglobal.ZapiszDoLogow(" przyszedl czas sie polaczyc", 4)
            self.czekajIPolacz = None
            self.startKlienta(self.opcje)

        for kanal in self.polaczenia:
            kanal.tura(czas)

        if not asyncore.socket_map:
            time.sleep(koniec-czas)
            return

        # w pozostaym czasie zajmij si poczeniami, niszy poziom
        while czas < koniec - 0.02:
            asyncore.poll(timeout=koniec-czas)
            czas = time.time()


class Wiadomosc:

    def __init__(self):

        self.zrodlo = None
        self.id = None
        self.czas = None
        self.crc = None
        self.tresc = None

    # crc wyliczone z self.id i self.tresc
    def obliczCrc(self):

        crc = binascii.crc32(str(self.id))
        crc = binascii.crc32(self.tresc, crc)
        return crc

    def sprawdzCrc(self):

        return self.crc == self.obliczCrc()

    def __str__(self):

        return "<%s instance: %s@%s (%r)>" % (self.__class__.__name__, self.id, self.zrodlo, isinstance(self.tresc, (str, unicode)) and self.tresc[:30] + '...' or repr(self.tresc))


######==========================         parsowanie SQLa

class ParserSQL:

	def znak(self, t, i):
		return t[i:i+1]

	def nast(self, t, i):
		return t[i+1:i+2]

	# znaki akceptowane jako czesc nazwy
	def czyLit(self, z):
		return 'a' <= z <= 'z' or 'A' <= z <= 'Z' or z == '_'

	def czyNum(self, z):
		return '0' <= z <= '9'

	def czyLitNum(self, z):
		return self.czyLit(z) or '0' <= z <= '9'

	def czyBialy(self, z):
		return z in (' ', '\t', '\n')

	# zwraca pozycje pierwszego nie białego
	def pominBiale(self, t, i):

		while self.czyBialy(self.znak(t, i)):
			i += 1
		return i

	def dajSlowo(self, t, i):

		slowo = ""
		z = self.znak(t, i)
		if self.czyLit(z):
			slowo += z; i += 1; z = self.znak(t, i)
			while self.czyLitNum(z) or z == '.':
				slowo += z; i += 1; z = self.znak(t, i)
		return slowo, i

	def dajLiczbe(self, t, i, naukowa=True):

		liczba = ""
		z = self.znak(t, i)
		if z == '-' or z == '+':
			liczba += z; i += 1; z = self.znak(t, i)
		if self.czyNum(z):
			while self.czyNum(z):
				liczba += z; i += 1; z = self.znak(t, i)
			if z == '.':
				liczba += z; i += 1; z = self.znak(t, i)
				while self.czyNum(z):
					liczba += z; i += 1; z = self.znak(t, i)
			if naukowa and (z == 'e' or z == 'E'):
				exp, j = self.dajLiczbe(t, i+1, False)
				if exp and (exp[0] not in ('+', '-') or len(exp) > 1):
					liczba += z + exp
					i = j
		return liczba, i

	def dajTekst(self, t, i):

		z = self.znak(t, i)
		wynik = ""
		if z == "'":
			wynik = "'"
			while True:
				i += 1
				z = self.znak(t, i)
				wynik += z;
				if z == "'":
					if self.znak(t, i+1) == "'":
						wynik += "'"
						i += 1
					else:
						break
				elif z == "\\":
					i += 1
					z = self.znak(t, i)
					wynik += z
		return wynik, i+1

	def dajNawias(self, t, i):

		z = self.znak(t, i)
		nawias = ""
		if z == '(':
			nawias += z
			i += 1
			while True:
				z = self.znak(t, i)
				if self.czyBialy(z):
					i = self.pominBiale(t, i)
					z = self.znak(t, i)
				if z == ')' or not z:
					nawias += z
					i += 1
					break
				wyr, i = self.dajWyrazenie(t, i)
				nawias += wyr
				bylBialy = False
				if self.czyBialy(self.znak(t, i)):
					i = self.pominBiale(t, i)
					bylBialy = True
				if self.znak(t, i) == ",":
					nawias += ", "
					i += 1
				elif bylBialy:
					nawias += " "
		return nawias, i

	def dajProstyOperand(self, t, i):

		i = self.pominBiale(t, i)
		z = self.znak(t, i)
		if self.czyLit(z):
			slowo, i = self.dajSlowo(t, i)
			if self.znak(t, i) == '(':
				nawias, i = self.dajNawias(t, i)
				return slowo + nawias, i
			else:
				return slowo, i
		elif z == "'":
			return self.dajTekst(t, i)
		elif self.czyNum(z) or z == '-' or z == '+':
			return self.dajLiczbe(t, i)
		elif z == "(":
			return self.dajNawias(t, i)

	def dajOperand(self, t, i):

		wyr, i = self.dajProstyOperand(t, i)
		if self.znak(t, i) == ':' and self.znak(t, i+1) == ":":
			typ, i = self.dajSlowo(t, i+2)
			return wyr + "::" + typ, i
		else:
			return wyr, i

	def czySepSlow(self, t, i):
		z = self.znak(t, i)
		return self.czyBialy(z) or z == "'"

	def dajWyrazenie(self, t, i):

		#print " dajWyrazenie(%r)" % t[i:]
		wyrazenie, i = self.dajOperand(t, i)

		# jak dostaniesz CASE to czytaj az trafisz na END
		if wyrazenie.upper() in ("CASE", ):
			#print " parseCASE: %r" % t[i:]
			while True:
				starei = i
				wyr, i = self.dajWyrazenie(t, i)
				if wyr[:3].upper() == "END":
					wyrazenie += " END"
					i = self.pominBiale(t, starei) + 3
					break
				else:
					#print "       + %r" % wyr
					wyrazenie += " " + wyr
		elif wyrazenie.upper() in ("WHEN", "THEN", "ELSE"):
			wyr2, i = self.dajWyrazenie(t, i)
			return wyrazenie.upper() + " " + wyr2, i
		# sprawdz czy jakis operator nie wystepuje, bo moze to nie koniec
		j = self.pominBiale(t, i)
		o1 = self.znak(t, j)
		o2 = o1 + self.znak(t, j+1)
		o3 = o2 + self.znak(t, j+2)
		o4 = o3 + self.znak(t, j+3)
		o7 = o4 + self.znak(t, j+4) + self.znak(t, j+5) + self.znak(t, j+6)
		if o2 in ("||", "==", "!=", "<>", "<=", ">=") or (self.czySepSlow(t, j+2) and o2.upper() in ("OR", "IN", "IS", "AS")):
			skladnik2, i = self.dajWyrazenie(t, j+2)
			wyrazenie += " " + o2.upper() + " " + skladnik2
		elif o1 in ('+', '-', '*', '/', '<', '>', '='):
			skladnik2, i = self.dajWyrazenie(t, j+1)
			wyrazenie += " " + o1 + " " + skladnik2
		elif o3.upper() in ("AND", "NOT") and self.czySepSlow(t, j+3):
			skladnik2, i = self.dajWyrazenie(t, j+3)
			wyrazenie += " " + o3.upper() + " " + skladnik2
		elif o4.upper() in ("LIKE", ) and self.czySepSlow(t, j+4):
			skladnik2, i = self.dajWyrazenie(t, j+4)
			wyrazenie += " " + o4.upper() + " " + skladnik2
		elif o7.upper() in ("BETWEEN", ) and czySepSlow(t, j+7):
			skladnik2, i = self.dajWyrazenie(t, j+7)
			wyrazenie += " " + o7.upper() + " " + skladnik2
		#print " << %r" % wyrazenie
		return wyrazenie, i

	# lista wyrazen oddzielonych przecinkami, niestety nie ma nawiasu zamykajacego
	def dajListe(self, t, i):

		lista = ""
		gdzie = []
		while True:
			i = self.pominBiale(t, i)
			od = len(lista)
			wyr, i = self.dajWyrazenie(t, i)
			lista += wyr
			gdzie.append((od, len(lista)))
			i = self.pominBiale(t, i)
			if self.znak(t, i) != ",":
				break
			else:
				lista += ", "
				i += 1
		return lista, i, gdzie

######==========================

zarzadcaTransportu = ZarzadcaTransportu()

def dajnr():
    nr = int(open("plik-numeru").read()) + 1
    open("plik-numeru", "w").write(str(nr))

def Zapisz(pnazwa, tresc, adresaci=None, sciezkaPliku=None):

    rsglobal.ZapiszDoLogow("rs_anakonda_server_komser.py: %s, %s" % (zarzadcaTransportu, zarzadcaTransportu.opcje), 4)
    if not zarzadcaTransportu or not zarzadcaTransportu.opcje:
        return
    opcje = zarzadcaTransportu.opcje
    if not opcje["komserkatalog"] or not opcje["komsernazwa"]:
        return

    if adresaci is None:
        # wszyscy poza sob
        adresaci = [x.strip() for x in opcje["komserklienci"].split(",")]
        if opcje["komsernazwa"] in adresaci:
            adresaci.remove(opcje["komsernazwa"])
    elif isinstance(adresaci, (str, unicode)):
        if adresaci.startswith("-"):
            # pomin wskazanych
            zopcji = opcje["komserklienci"].split(",")
            if opcje["komsernazwa"] in zopcji:
                zopcji.remove(opcje["komsernazwa"])
            for adresat in adresaci[1:].split(","):
                if adresat in zopcji:
                    zopcji.remove(adresat)
            adresaci = zopcji
        else:
            adresaci = adresaci.split(",")
    #print "ADRESACI: ", adresaci
    #rsglobal.ZapiszDoLogow("rs_anakonda_server_komser.py: ADRESACI %r" % (adresaci,), 4)

    for adresat in adresaci:

      try:
        katalog = os.path.join(opcje["komserkatalog"], adresat, Magazyn.IN)
        nazwa_pliku_numeru = os.path.join(katalog, "_OSTATNI_NUMER")
        plik_numeru = open(nazwa_pliku_numeru, "a+")
        fcntl.lockf(plik_numeru.fileno(), fcntl.LOCK_EX)
        numer = int(plik_numeru.readline().strip() or '0') + 1
        plik_numeru.seek(0)
        plik_numeru.truncate()
        print >> plik_numeru, numer
        fcntl.lockf(plik_numeru.fileno(), fcntl.LOCK_UN)

        if "." not in pnazwa:
            nazwa = "%s_%d.txt" % (pnazwa, numer)
        else:
            base, ext = os.path.splitext(pnazwa)
            nazwa = "%s_%d%s" % (base, numer, ext)

        if sciezkaPliku is None:
            sciezkaPliku = os.path.join(katalog, nazwa)
            open(sciezkaPliku, "w").write(tresc)
        else:
            os.link(sciezkaPliku, os.path.join(katalog, nazwa))
            #print "LINK", sciezkaPliku, os.path.join(katalog, nazwa)
      except Exception, exc:
          rsglobal.ZapiszDoLogow("komser.Zapisz: %s %s" % (adresat, exc), 1)


if __name__ == "__main__":

    print "\n\n  plik\tkomser.py - biblioteka\n\n"
    zarz = ZarzadcaTransportu()
    trans = TCPTransport(zarz)
    trans.startSerwer(opcje)
    zarz.Uruchom(opcje)

    nping = 0
    while asyncore.socket_map:
        asyncore.poll2(timeout=0.5)
        if nping < time.time() and trans.polaczenia:
            trans.polaczenia[0].wyslij(3, nping)
            nping = time.time() + 1
            #print trans.polaczenia, trans.polaczenia and len(trans.polaczenia[0].inpaczki)
        if trans.polaczenia and trans.polaczenia[0].inpaczki:
            p = trans.polaczenia[0].inpaczki.pop(0)
            print "<", p
            trans.polaczenia[0].wyslij(4, "przyjalem: " + repr(p)[:15] + "...")
