#!/usr/bin/env python
# -*- coding: ISO8859-2 -*-
#-----------------------------------------------------------------------------
# Name:        rs_anakonda_server_pan.py
# Purpose:     modu obsugi pocze PAN
#
# Authors:     Piotr Szydeko
# Created:     2009/07/15
# Copyright:   (c) 2009 Record System
# Licence:     GNU GPL
#-----------------------------------------------------------------------------

# Jan Kaliszewski, 2010-04-25/30:
# * implementacja aktualizacji klienta + drobne poprawki
# 2013-11-07 Mariusz Rajfura (7278) usuniecie przycinania pinu do 6 znakow


import anydbm
import cPickle
import datetime
import errno
import fcntl
import filecmp
import itertools
import operator
import os
import os.path
import sys
import time
import select
import shelve
import shutil
import signal
import socket
import traceback

import rsglobal		# .ZapiszDoLogow(), .PodajZKonf(), .RSDrzewoPlikow
import rssql
import const
import rs_anakonda_socket
import rs_anakonda_server_cl
import rs_anakonda_server_fn


kat_serwera = u''  # bdzie ustawiony z rs_anakonda_server.py

podkat_funkcji = u"funkcje"

# nazwy katalogw i plikw zwizanych z aktualizacjami klienta PAN
podkat_pomocniczy = u'.aktualizacje_pan'
podkat_cache_aktualizacji = u'cache_aktualizacji'
podkat_zrodel_klienta = u'podstawa_aktualizacji'
podkat_poprz_zrodel_klienta = u'podstawa_porownania'

nazwa_blokady_kat_akt = u'.' + podkat_zrodel_klienta + u'--blokada.flock'
nazwa_blokady_kat_poprz = u'.' + podkat_poprz_zrodel_klienta + u'--blokada.flock'
nazwa_pliku_info_o_wersjach = u'info_o_wersjach'
# musi by identyczna z PAN-klienck RSAktualizacje.NAZWA_PLIKU_WERSJI:
nazwa_pliku_wersji = u'nr_i_czas_ost_aktualizacji'
nazwa_pliku_logowania = u'pan.log'

CYFRY = frozenset('0123456789')

EINTR = errno.EINTR
ECHILD = errno.ECHILD
ESRCH = errno.ESRCH
EAGAIN = errno.EAGAIN


class Connection(object):
    def __init__(self, fd_read, fd_write):
        self._in = fd_read
        self._out = fd_write

    def recv(self):
        return eval(os.read(self._in, 1024))

    def send(self, data):
        data = repr(data)
        assert len(data) <= 1024
        return os.write(self._out, data)

    def close(self):
        os.close(self._in)
        os.close(self._out)

    def fileno(self):
        return self._in

    def poll(self, timeout):
        rfd, none, none = select.select([self._in], [], [], timeout)
        return bool(rfd)


def Pipe():
    read_end1, write_end1 = os.pipe()
    read_end2, write_end2 = os.pipe()
    return Connection(read_end1, write_end2), Connection(read_end2, write_end1)


class RSFunkcjePAN(rs_anakonda_server_fn.RSFunkcje):

    KATALOG_FUNKCJI = os.path.join(kat_serwera, podkat_funkcji)
    
    def __init__(self, procesKlienta, operator_id, firma_id, operatorzy, uprawnienia, oddzial_id):
        rs_anakonda_server_fn.RSFunkcje.__init__(self, operator_id,
                                                 procesKlienta.PolSQL,
                                                 procesKlienta.addr[0],
                                                 operatorzy)
        
        self.SetFirma(firma_id)
        self.SetOddzial(oddzial_id)
        self.SetUprawnienia(uprawnienia)

    def ImportujFunkcje(self):
        raise NotImplementedError
        
    def WykonajFunkcje(self, funkcja, klucze=None, pola=None, wartosci=None,RobicCommit=True):
        if klucze is None:
            klucze = []
        if pola is None:
            pola = []
        if wartosci is None:
            wartosci = []
        
        #zablokowalem, aby wywolywac inne funkcje - mariusz rajfura
        #if not funkcja.startswith("fs_pan_"):
        #    return u"ER:Nieprawidowa nazwa funkcji: %r" % funkcja


        #=============================<wydziwiam cos.mariusz>===================

        rsglobal.ZapiszDoLogow('Panel:Sprawdzenie czy funkcja '+funkcja+' jest na liscie',5)
        sql = "SELECT typ,menu FROM rs_funkcje WHERE funkcja='"+funkcja+"';"
        stat,dane,dump,dump = self.PolSQL.Execute(sql,2)
        if len(dane) == 0:
            #funkcji nie ma na liscie
            stat = 'Panel:BRAK FUNKCJI '+funkcja
            rsglobal.ZapiszDoLogow(stat)
            return stat, [], [], []
            
        menuid = dane[0][1]
        #Sprawdzenie uprawnien dla funkcji
        if self.operator_id <> const.STANDARD_OPERATOR_ID:
            if (not (self.oddzial_id,menuid) in self.uprawnienia) and (not ('**',menuid) in self.uprawnienia) and menuid<>'000000':
                #Brak uprawnien
                stat = 'Brak uprawnie do wykonania funkcji '+funkcja
                return stat, [], [], []

        #=============================</wydziwiam cos.mariusz>==================

            
        stat, dane, wyn1, wyn2 = rs_anakonda_server_fn.RSFunkcje.WykonajFunkcje(self, funkcja,
                                                                                klucze, pola, wartosci,RobicCommit)
        
        return stat, dane, wyn1, wyn2
        

class RSPANSerwer(object):
    # pozostajacy w parencie obiekt pozwalajacy na proste sterowanie serwerem

    def __init__(self):

        self.serwer = None
        # polaczenie z rodzicem/potomka
        self.kanal = None
        self.dzialaj = False
        self.opcje = {}
        self.potomne = []
        
        kat_pom = os.path.join(kat_serwera, podkat_pomocniczy)
        try:
            os.mkdir(kat_pom, 0700)
        except OSError, exc:
            if exc.errno != errno.EEXIST:
                raise
        else:
            rsglobal.ZapiszDoLogow(u"Nie znaleziono katalogu pomocniczego dla aktualizacji "
                                   u"klienta PAN: %r -- teraz zosta utworzony" % kat_pom, 3)
                                   
        kat_poprz = os.path.join(kat_serwera, podkat_pomocniczy, podkat_poprz_zrodel_klienta)
        try:
            os.mkdir(kat_poprz, 0700)
        except OSError, exc:
            if exc.errno != errno.EEXIST:
                raise
        else:
            rsglobal.ZapiszDoLogow(u"Nie znaleziono katalogu dla poprzednich wersji "
                                   u"klienta PAN: %r -- teraz zosta utworzony" % kat_poprz, 3)
                                   
        kat_cache = os.path.join(kat_serwera, podkat_pomocniczy, podkat_cache_aktualizacji)
        try:
            os.mkdir(kat_cache, 0700)
        except OSError, exc:
            if exc.errno != errno.EEXIST:
                raise
        else:
            rsglobal.ZapiszDoLogow(u"Nie znaleziono katalogu dla cache aktualizacji "
                                   u"klienta PAN: %r -- teraz zosta utworzony" % kat_cache, 3)

        sciezka_info_o_wersjach = os.path.join(kat_serwera, podkat_pomocniczy,
                                               nazwa_pliku_info_o_wersjach)
        try:
            rsglobal.RSSlownikWPliku(sciezka_info_o_wersjach, 'r').close()
        except anydbm.error:
            rsglobal.ZapiszDoLogow(u"Nie znaleziono pliku-sownika z info o wersjach "
                                   u"klienta PAN: %r -- zostanie teraz utworzony..."
                                   % sciezka_info_o_wersjach, 3)
            rsglobal.RSSlownikWPliku(sciezka_info_o_wersjach, 'c').close()


    def Uruchom(self, opcje):

        self.opcje = opcje
        if not self.dziala():
            self.kanal, kanal_potomka = Pipe()
            pid = os.fork()
            if pid == 0:
                try:
                    self.main(kanal_potomka)
                except Exception, exc:
                    try:
                        rsglobal.ZapiszDoLogow("RSPANSerwer.Uruchom: %s" % traceback.format_exc() + str(type(exc)))
                    finally:
                        os._exit(1)
                os._exit(0)
            else:
                self.serwer = pid
                if self.kanal.poll(4):
                    odp = self.kanal.recv()
                    if odp != u"OK":
                        rsglobal.ZapiszDoLogow(u"RSPANSerwer.Uruchom: %s" % odp)
                    return odp == u"OK"
                else:
                    # co si nie udao
                    return False
        else:
            return True

    def main(self, kanal):
        # potomek tutaj wykonuje swoj prac: oczekuje na poczenia i spawnuje robotnikw

        if nazwa_pliku_logowania:
            rsglobal.LogFile = os.path.join(kat_serwera, nazwa_pliku_logowania)
        rsglobal.ZapiszDoLogow(u"RSPANSerwer (%d) uruchomiony" % os.getpid(), 5)
        signal.signal(signal.SIGINT, signal.SIG_IGN)
        signal.signal(signal.SIGCHLD, self.onSigChld)
        self.dzialaj = True
        self.kanal = kanal

        try:
            poll = select.poll()
            poll.register(self.kanal, select.POLLIN)
            nasluch = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            nasluch.bind(("", self.opcje["port"]))
            nasluch.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            nasluch.setblocking(0)
            nasluch.listen(2)
            poll.register(nasluch, select.POLLIN)
        except socket.error, exc:
            self.kanal.send(str(exc))
            self.dzialaj = False
        else:
            self.kanal.send(u"OK")

        while self.dzialaj:
            try:
                fdevs = poll.poll()
            except select.error, exc:
                if exc.args[0] != EINTR:  # Interrupted system call
                    raise
            for fd, ev in fdevs:
                if ev != select.POLLIN:
                    pass
                elif fd == self.kanal.fileno():
                    #self.kanal.poll()
                    cmd = self.kanal.recv()

                    if cmd == u"QUIT":
                        #print >> sys.stderr, "QUIT"
                        nasluch.close()
                        if self.potomne:
                            self.kanal.send(False)
                        else:
                            self.dzialaj = False
                            self.kanal.send(True)
                        #print >> sys.stderr, "--"
                    elif cmd == u"TERM":
                        for child in self.potomne:
                            try:
                                os.kill(child, signal.SIGTERM)
                            except OSError, exc:
                                if exc.errno != ESRCH:  # No such process
                                    rsglobal.ZapiszDoLogow(u"RSPANSerwer TERM: os.kill(%d) %s" % (child, exc))
                        self.dzialaj = False
                        self.kanal.send(u"ACK")
                        nasluch.close()
                        
                elif fd == nasluch.fileno():
                    #print >> sys.stderr, "accept", fd, ev
                    try:
                        conn_addr = nasluch.accept()
                    except socket.error, exc:
                        if exc.args[0] != EAGAIN:   # Resource temporarily unavailable
                            raise
                    else:
                        RSPANObslugaKlienta(self, conn_addr)
                    #print >> sys.stderr, "+", self.potomne

        self.kanal.close()
        rsglobal.ZapiszDoLogow(u"RSPANSerwer (%d) zakoczony" % os.getpid(), 5)

    def dziala(self):

        return self.serwer is not None

    def Zatrzymaj(self, force=False):

        if self.dziala():
            zatrzymany = False
            try:
                # FIXME: nie killowa, a jeli chcesz go ubi to inaczej
                os.kill(self.serwer, signal.SIGINT)
            except OSError, exc:
                if exc.errno == ESRCH:  # No such process
                    zatrzymany = True

            if not zatrzymany:
                if force:
                    self.kanal.send(u"TERM")
                    self.kanal.recv()
                    zatrzymany = True
                else:
                    self.kanal.send(u"QUIT")
                    zatrzymany = self.kanal.recv()

            if zatrzymany:
                self.kanal.close()
                self.serwer = None
            return zatrzymany
        else:
            return True

    def onSigChld(self, sig, frame):
        pid, status = os.wait()
        if pid in self.potomne:
            self.potomne.remove(pid)
        else:
            #rsglobal.ZapiszDoLogow(u"RSPANSerwer.onSigChld: %d nie jest potomkiem" % pid, 1)
            self.potomne.append(-pid)
        #print >> sys.stderr, "-%d %r" % (pid, self.potomne)


serwer = RSPANSerwer()


class RSPANObslugaKlienta(object):

    licznik = 1

    def __init__(self, serwer, conn_addr):
        # opniacze przeciw atakom DOS:
        time.sleep(const.OPOZNIENIE_POCZATKU_OBSLUGI_KLIENTA_PAN)
        self._licznik_bledow_logowania = 0
        self._pomin_sprawdzenie_odstepu_akt = False
        self._czas_ost_zadania_dot_akt = 0
        self._czas_ost_przygotowania_akt = 0
        
        self.name = u"RSPANKlient-%d" % self.licznik
        self.__class__.licznik += 1
        self.conn = None
        self.addr = None
        self.serwer = serwer
        self.rsock = None
        self.dzialaj = True
        self.PolSQLHar = None
        self.PolSQL = None
        
        self._rsfunkcje = None
        if conn_addr:
            self.conn, self.addr = conn_addr
            pid = self.start()
            if -pid in self.serwer.potomne:
                self.serwer.potomne.remove(-pid)
            else:
                self.serwer.potomne.append(pid)

    def start(self):
        assert self.conn and self.addr
        pid = os.fork()
        if pid == 0:
            try:
                self.run()
            except Exception:
                try:
                    rsglobal.ZapiszDoLogow(u"%s zakoczony: %s" % (self.name, traceback.format_exc()), 1)
                finally:
                    os._exit(1)
            os._exit(0)
        else:
            return pid

    def run(self):
        sys.stdin.close()
        rsglobal.ZapiszDoLogow(u"%s (%d) uruchomiony, poczenie %s:%d" % (self.name, os.getpid(), self.addr[0], self.addr[1]), 5)
        self.rsock = rs_anakonda_socket.AnakondaSocket(socket_obj=self.conn)
        while self.dzialaj:
            try:
                data = self.rsock.recv()
                if not data: break
                #try:
                #    data = data.decode("utf8")
                #except UnicodeDecodeError:
                #    self.rsock.send("ER:UnicodeDecodeError")
                #    continue
                cmd = data[:16].split(" ", 1)[0]
                rsglobal.ZapiszDoLogow(u"Otrzymano polecenie %r" % cmd, 5)
                
                #print >> sys.stderr, "<", cmd
                if cmd in self._cmds:
                    try:
                        wynik = self._cmds[cmd](self, data)
                    except Exception, exc:
                        rsglobal.ZapiszDoLogow(u"%s w reakcji na %s" % (traceback.format_exc(), data[:60]), 1)
                        wynik = u"ER: BLAD w %s.%s" % (self.__class__.__name__, self._cmds[cmd].__name__)
                    if self.dzialaj:
                        #rsglobal.ZapiszDoLogow(u"PAN: Wykonano polecenie -- wynik: %r" % (wynik,), 5)
                        if wynik and isinstance(wynik, tuple) and wynik[0] == '[CALL AFTER]':
                            self.rsock.send(wynik[1])  # wysyamy wynik
                            try:
                                metoda, args, kwargs = wynik[2:]
                                metoda(*args, **kwargs)  # wywoujemy metod zamwion do wykonania
                            except Exception:
                                rsglobal.ZapiszDoLogow(u"%s w reakcji na %s, po wysaniu"
                                                       u" odpowiedzi, przy obsudze CALL AFTER"
                                                       % (traceback.format_exc(), data[:60]), 1)
                        else:
                            #print >> sys.stderr, ">", wynik
                            self.rsock.send(wynik)
                else:
                    self.rsock.send(u"ER:PAN nieznane polecenie: %s" % cmd)
                    #print >> sys.stderr, ">", u"ER:PAN nieznane polecenie: %s" % cmd
            except socket.error, exc:
                rsglobal.ZapiszDoLogow(u"Bd poczenia: %s" % traceback.format_exc(), 5)
                break
        rsglobal.ZapiszDoLogow(u"%s (%d) zakoczony" % (self.name, os.getpid()), 5)

    def SQLConnectHar(self):
        import rsglobal
        PolSQL = rssql.RSSQL()
        dbuser = self.serwer.opcje["panuser"]
        dbpass = self.serwer.opcje["panpass"]
        dbhost = self.serwer.opcje["hostpgsql"]
        dbport = self.serwer.opcje["portpgsql"]
        dbase = self.serwer.opcje["baza"]
        rsglobal.ZapiszDoLogow('SQLConnectHar',1)
        if PolSQL.PolaczZSQL(dbuser, dbpass, dbhost, dbport, None, Baza=dbase):
            return False
        else:
            self.PolSQLHar = PolSQL
            return True

    def SQLConnect(self, operator, haslo):
        import rsglobal
        PolSQL = rssql.RSSQL()
        dbuser = operator
        dbpass = haslo
        dbhost = self.serwer.opcje["hostpgsql"]
        dbport = self.serwer.opcje["portpgsql"]
        dbase = self.serwer.opcje["baza"]
        rsglobal.ZapiszDoLogow('SQLConnect',1)
        if PolSQL.PolaczZSQL(dbuser, dbpass, dbhost, dbport, None, Baza=dbase):
            return False
        else:
            self.PolSQL = PolSQL
            return True

    def _cmd_koniec(self, data):
        self.rsock.send(const.RECEIVED)
        self.rsock.close()
        self.dzialaj = False


    def _cmd_wyloguj(self, data):
        if self.PolSQL <> None:
            self.PolSQL.RozlaczZSQL()
            self.PolSQL = None

            self._rsfunkcje.operator_id = None
            self._rsfunkcje.firma_id = ''
            self._rsfunkcje.oddzial_id = ''
            self._rsfunkcje.uprawnienia = {}

        return 'wylogowano'



    def _cmd_uprawnienia(self, data):
        return str((self._rsfunkcje.operator_id,
                    self._rsfunkcje.firma_id,
                    self._rsfunkcje.oddzial_id,
                    self._rsfunkcje.uprawnienia))


    #def _cmd_time(self, data):
    #    return datetime.datetime.now().strftime("%F %T")

    def _cmd_wykonaj(self, dane):
        if not self.PolSQL or not self._rsfunkcje:
            return u"ER:niegotowy"
        
        dane = rsglobal.rseval(dane[len(const.FUNCTION)+1:])
        if not isinstance(dane, (list, tuple)) or len(dane) != 4:
            return u"ER:argumenty"
        if not isinstance(dane[0], (str, unicode)):
            return u"ER:argumenty nazwa funkcji"
        res = self._rsfunkcje.WykonajFunkcje(*dane)
        return str(res)

    def _cmd_auth(self, data):
        try:
            if len(data) > 255:
                return u"ER:przekroczony rozmiar"
            data = data[len(u"AUTH")+1:]
            #parts = data.splitlines()
            parts = data.split("#")
            if len(parts) != 2:
                return u"ER:argumenty"
            
            if self.PolSQLHar == None:
                if not self.SQLConnectHar():
                    return u"ER:blad polaczenia z baza danych"

            kod = parts[1]
            if not CYFRY.issuperset(kod):
                return u"ER:niedozwolone znaki w kodzie"
            if not (kod and len(kod) >= self.serwer.opcje["panminkod"]):
                return u"ER:za krtki kod"
            
            sql = "SELECT operator_id, login, nazwisko, imie FROM rs_operatorzy WHERE kod_dostepu='"+kod+"';"
            stat, dane, none, none = self.PolSQLHar.Execute(sql,2)
            
            if stat != "OK":
                return u"ER:%s" % stat
            elif not dane:
                return u"ER:bdny kod"
            elif len(dane) != 1:
                return u"ER:konflikt kodw"
            rsglobal.ZapiszDoLogow(u"%s zalogowany %r %s" % (self.name, dane, self.addr), 2)
            operator_id, operator, nazwisko, imie = dane[0]
            
            #logowanie operatora do bazy
            haslo = str(operator_id).zfill(4)+str(kod)
            if not self.SQLConnect(operator, haslo):
                return u"ER:blad polaczenia z baza danych"
            
            operatorzy = rs_anakonda_server_cl.PodajOperatorow(self.PolSQL,operator_id)
            operatorzy = str(operatorzy).replace('[','(').replace(']',')')
            firma_id = self.serwer.opcje["panfirmaid"].strip()
            
            #pobieramy oddzial dla operatora panelu
            sql ="SELECT generator_oddzial_id FROM rs_operatorzy WHERE operator_id="+str(operator_id)+";"
            stat, dane, none, none = self.PolSQL.Execute(sql,2)
            if stat <> 'OK':
                return u"ER:%s" % stat
            
            oddzial_id = dane[0][0]
            
            #pobieramy uprawnienia
            uprawnienia = rs_anakonda_server_cl.PodajUprawnienia(self.PolSQL, operatorzy, firma_id)
            if uprawnienia == None:
                uprawnienia = []
            
            #ustawiamy dane, ktore powinny byc zainicjowane po logowaniu
            self._rsfunkcje = RSFunkcjePAN(self, operator_id, firma_id, operatorzy, uprawnienia, oddzial_id)

            self._licznik_bledow_logowania = -1
            return u"OK:%s;%s;%s;%s" % (operator, operator_id, nazwisko, imie)
        finally:
            try:
                if self.PolSQLHar:
                    self.PolSQLHar.Commit()
            except:
                pass

                    
            self._licznik_bledow_logowania += 1
            if self._licznik_bledow_logowania > 0:
                time.sleep(self._licznik_bledow_logowania)


    # (uywa metod zdefiniowanych dalej)
    def _cmd_prepare_update(self, data):
        '''Przygotuj aktualizacj klienta, ale jej nie rb'''
        if self._nalezyty_odstep_akt():
            dane_wersji_u_klienta = data[(len(const.PREPARE_UPDATE) + 1):]
            # od razu przelij odpowiedz, a dopiero potem (eby nie zajmowa czasu
            # klientowi) przystp do przygotowania aktualizacji i zapisania jej na dysku
            return ('[CALL AFTER]', 'OK.', self._przygotuj_lub_odczytaj_aktualizacje,
                    (dane_wersji_u_klienta,), dict(zapisz=True, to_jest_przygotowanie=True))
        else:
            return "ER:zbyt may odstp od poprzedniego dania!"


    # (uywa metod zdefiniowanych dalej)
    def _cmd_update(self, data):
        '''Dokonaj aktualizacji klienta'''
        if self._nalezyty_odstep_akt():
            dane_wersji_u_klienta = data[(len(const.UPDATE) + 1):]
            # zrb aktualizacj;
            # jeeli dla podanych wersji bya ju przygotowana, odczytaj gotow z cache
            (wersja_u_klienta, najnowsza_wersja, przygotowana_aktualizacja
            ) = self._przygotuj_lub_odczytaj_aktualizacje(dane_wersji_u_klienta, zapisz=False)
            # od razu przelij aktualizacj, a dopiero potem (eby nie zajmowa czasu
            # klientowi bardziej ni to konieczne) zapisz j na dysku do pniejszych uy
            return ('[CALL AFTER]', przygotowana_aktualizacja, self._zapisz_aktualizacje,
                    (wersja_u_klienta, najnowsza_wersja, przygotowana_aktualizacja), {})
        else:
            return "ER:zbyt may odstp od poprzedniego dania!"
        
        
    _cmds = {
        const.AUTH: _cmd_auth,
        const.CONN_END: _cmd_koniec,
        const.FUNCTION: _cmd_wykonaj,
        const.PREPARE_UPDATE: _cmd_prepare_update,
        const.UPDATE: _cmd_update,
        "UPRAWNIENIA": _cmd_uprawnienia,
        "LOGOUT": _cmd_wyloguj,
    }


    #-------------------------------------------------------------------
    # Metody zwizane z aktualizacj klienta

    def _nalezyty_odstep_akt(self, min_odstep=(const.SPODZIEWANY_ODSTEP_PRZED_AKT_PAN // 2)):
        # Przeciwdziaanie skutkom atakw DOS:
        # Czy upyn naleyty czas od ostatniego dania aktualizacji?
        # + Jeli tak, ustaw czas dania ost. aktualizacji na czas biecy
        
        if self._pomin_sprawdzenie_odstepu_akt:
            # to, by moliwa bya pojedyncza awaryjna ponowna prba aktualizacji
            self._pomin_sprawdzenie_odstepu_akt = False
            return True
        else:
            teraz = time.time()
            odp = (teraz >= self._czas_ost_zadania_dot_akt + min_odstep)
            if odp:
                self._czas_ost_zadania_dot_akt = teraz
                self._pomin_sprawdzenie_odstepu_akt = True
                return True
            else:
                return False


    def _przygotuj_lub_odczytaj_aktualizacje(self, dane_wersji_u_klienta, zapisz,
                                             to_jest_przygotowanie=False):
        
        sciezka_info_o_wersjach = os.path.join(kat_serwera, podkat_pomocniczy,
                                               nazwa_pliku_info_o_wersjach)
        kat_akt = os.path.join(kat_serwera, podkat_pomocniczy, podkat_zrodel_klienta)

        # okrel numer i znacznik czasu wersji u klienta (wyjciowej)
        wersja_u_klienta_str, czas_wersji_u_klienta = dane_wersji_u_klienta.split('@')
        wersja_u_klienta = int(wersja_u_klienta_str)

        # blokada zapewniajca brak konfliktu ze skryptem adujcym kod rdowy
        sciezka_blokady_akt = os.path.join(kat_serwera, podkat_pomocniczy,
                                           nazwa_blokady_kat_akt)
        try:
            blokada_kat_akt = open(sciezka_blokady_akt, 'r')
        except EnvironmentError, exc:
            if exc.errno == errno.ENOENT:
                raise RuntimeError(u'Brak zaadowanych jakichkolwiek aktualizacji!')
                #open(sciezka_blokady_akt, 'w').close()
                #blokada_kat_akt = open(sciezka_blokady_akt, 'r')
            else:
                raise
        try:
            fcntl.flock(blokada_kat_akt, fcntl.LOCK_SH)
            
            # okrel numer i znacznik najnowszej wersji (docelowej)
            (najnowsza_wersja, czas_najnowszej_wersji
            ) = self._daj_najnowsza_wersje_i_jej_czas(sciezka_info_o_wersjach, kat_akt)
            
            if (wersja_u_klienta == najnowsza_wersja
                  and czas_wersji_u_klienta == czas_najnowszej_wersji):
                # klient ma najnowsz wersj -- "pusta" aktualizacja
                przygotowana_aktualizacja = "OK."
                
            else:
                # info_o_wersjach -- trzymany w pliku sownik o nastpujcej zawartoci:
                #   'najnowsza_wersja_i_jej_czas': (<nr najnowszej wersji>, <jej znacznik czasu>)
                # plus jaka liczba elementw:
                #   '<nr wersji>': (<znacznik czasu>, <lista usunitych>, <lista nowych/dodanych>)
                (info_o_wersjach
                ) = rsglobal.RSSlownikWPliku(sciezka_info_o_wersjach, 'r')
                try:
                    info_o_wersji_klienta = info_o_wersjach.get(str(wersja_u_klienta))
                    if not (info_o_wersji_klienta
                            and info_o_wersji_klienta[0] == czas_wersji_u_klienta):
                        # podana przez klienta wersja nie istnieje
                        # lub nie zgadza si znacznik czasu istniejcej wersji
                        # => aktualizacja caociowa (przesane bd wszystkie pliki)
                        wersja_u_klienta = 0

                    (przygotowana_aktualizacja
                    ) = self._odczytaj_aktualizacje(wersja_u_klienta, najnowsza_wersja)
                    if przygotowana_aktualizacja is not None:
                        # dla tej wersji wyjciowej i docelowej
                        # jest ju zcacheowana gotowa aktualizacja
                        zapisz = False
                    else:
                        # by dla podanej wersji wyjciowej i docelowej okreli,
                        # ktre pliki/katalogi (i w jakiej kolejnoci) maj by
                        # usunite lub skopiowane/utworzone -- "sumujemy"
                        # zapamitane (w ww. plikowym sowniku info_o_wersjach)
                        # listy usunitych oraz zmienionych/nowych cieek
                        do_usuniecia = rsglobal.RSDrzewoPlikow()
                        do_skopiowania = rsglobal.RSDrzewoPlikow()
                        for wersja in xrange(wersja_u_klienta + 1, najnowsza_wersja + 1):
                            (czas_wersji, usuniete, zmienione_i_nowe
                            ) = info_o_wersjach[str(wersja)]
                            for usu in usuniete:
                                #if usu not in do_skopiowania:
                                if not do_skopiowania.czy_jest(usu):
                                    do_usuniecia[usu] = True
                            do_skopiowania.kasuj_sciezki(usuniete)
                            do_skopiowania.ustaw_sciezki(zmienione_i_nowe, True)
                        assert czas_wersji == czas_najnowszej_wersji
                finally:
                    info_o_wersjach.close()

                if przygotowana_aktualizacja is None:

                    # lista cieek do plikw/katalogw do usunicia
                    usu_sciezki = list(do_usuniecia.sciezki_nap())

                    # lista krotek:
                    # (<cieka do pliku do umieszczenia lub katalogu do utworzenia>,
                    #  <zawarto pliku lub None [w przypadku katalogu]>)
                    kop_sciezki_i_zawartosc = []
                    for sciezka in do_skopiowania.sciezki_nap():
                        kop_sciezki_i_zawartosc.extend(
                                self._gen_sciezki_zawartosc(kat_akt, sciezka)
                        )
                    # dodajemy ciek i waciw zawarto pliku wersji
                    kop_sciezki_i_zawartosc.append((nazwa_pliku_wersji,
                                                   '%d@%s' % (najnowsza_wersja,
                                                              czas_najnowszej_wersji)))
                                                              
                    # przygotowan aktualizacj marynujemy do stringa
                    (przygotowana_aktualizacja
                    ) = ''.join(("OK:", cPickle.dumps((usu_sciezki,
                                                       kop_sciezki_i_zawartosc,
                                                       wersja_u_klienta,
                                                       najnowsza_wersja),
                                                      protocol=2)))
        finally:
            blokada_kat_akt.close()

        if zapisz:
            self._zapisz_aktualizacje(wersja_u_klienta, najnowsza_wersja, przygotowana_aktualizacja)

        if to_jest_przygotowanie:
            self._czas_ost_przygotowania_akt = time.time()

        return (wersja_u_klienta, najnowsza_wersja, przygotowana_aktualizacja)


    def _gen_sciezki_zawartosc(self, abs_kat, sciezka):
        '''Generuj krotki (<scieka pliku|katalogu>, <zawarto|None>)'''
        abs_sciezka = os.path.join(abs_kat, sciezka)
        if os.path.isdir(abs_sciezka):
            yield (sciezka, None)  # (None => to katalog)
            for nazwa in os.listdir(abs_sciezka):
                if nazwa not in const.IGNOROWANE_NAZWY_PRZY_AKTUALIZACJI_PAN:
                    # rekurencja (podkatalogi/pliki)
                    for zawartosc in self._gen_sciezki_zawartosc(
                                     abs_kat, os.path.join(sciezka, nazwa)):
                        yield zawartosc
        else:
            plik = open(abs_sciezka, 'rb')
            try:
                zawartosc = sciezka, plik.read()
            finally:
                plik.close()
            yield zawartosc


    def _daj_najnowsza_wersje_i_jej_czas(self, sciezka_info_o_wersjach, kat_akt):

        '''Porwnaj, zapamitaj i zwr odpowiednie informacje o wersjach...

        Porwnaj najnowsz wersj z jej kopi dokonan poprzednio. W razie
        rnic -- zapisz na dysku te rnice (list cieek usunitych i list
        cieek dodanych/nowych), wraz ze znacznikiem czasu; utwrz kopi
        najnowszej wersji (zastpujc ni kopi dokonan poprzednio).

        Zwr numer i znacznik czasu najnowszej wersji.
        
        '''

        #---------------------------------------------------------------
        # optymalizacja + pewnego rodzaju przeciwdziaanie skutkom atakw DOS:
        # jeeli od:
        #
        # * ostatniego utworzenia nowej wersji (aktualizacji),
        #   globalnie, tzn. niezalenie od poczenia klienckiego...
        #
        # lub
        #
        # * nieodrzuconego dania *przygotowania* aktualizacji,
        #   dla tego konkretnego poczenia klienckiego...
        #
        # ...upyno nie wicej ni okrelona liczba sekund --
        # wwczas nie bdzie adnego porwnywania, tylko zostan
        # uyte informacje o ostatniej aktualizacji
        info_o_wersjach = rsglobal.RSSlownikWPliku(sciezka_info_o_wersjach, 'r')
        try:
            (najnowsza_wersja, czas_najnowszej_wersji
            ) = info_o_wersjach.get('najnowsza_wersja_i_jej_czas', (0, '0'))
        finally:
            info_o_wersjach.close()
            
        odstep = (const.SPODZIEWANY_ODSTEP_PRZED_AKT_PAN * 3) // 2
        teraz = time.time()
        if (teraz <= float(czas_najnowszej_wersji) + odstep
              or teraz <= self._czas_ost_przygotowania_akt + odstep):
            rsglobal.ZapiszDoLogow('Aktualizacja: dane najnowszej wersji'
                                   ' (z zapamitanych): %s@%s' %
                                   (najnowsza_wersja, czas_najnowszej_wersji),
                                   5)
            # -> zwracamy bez porwnania
            return najnowsza_wersja, czas_najnowszej_wersji

        #---------------------------------------------------------------
        # -> bdzie robione porwnanie...
        kat_poprz = os.path.join(kat_serwera, podkat_pomocniczy, podkat_poprz_zrodel_klienta)
        sciezka_blokady_poprz = os.path.join(kat_serwera, podkat_pomocniczy,
                                             nazwa_blokady_kat_poprz)
        try:
            blokada_kat_poprz = open(sciezka_blokady_poprz, 'r')
        except EnvironmentError, exc:
            if exc.errno == errno.ENOENT:
                open(sciezka_blokady_poprz, 'w').close()
                blokada_kat_poprz = open(sciezka_blokady_poprz, 'r')
            else:
                raise
        try:
            ignore = list(const.IGNOROWANE_NAZWY_PRZY_AKTUALIZACJI_PAN)
            ignore.append(rsglobal.RSPlik.PODKAT_BLOKAD)
            fcntl.flock(blokada_kat_poprz, fcntl.LOCK_SH)
            # porwnujemy najnowsz wersj z dokonan poprzednio kopi
            porownanie = filecmp.dircmp(kat_poprz, kat_akt, ignore=ignore)
        finally:
            blokada_kat_poprz.close()
        # na podstawie tego porwnania tworzymy listy cieek
        # do plikw/katalogw: usunitych oraz zmienionych/nowych
        usuniete, zmienione_i_nowe = self._ktore_usu_zm_nowe(porownanie, [], [])
        
        # info_o_wersjach -- trzymany w pliku sownik o nastpujcej zawartoci:
        #   'najnowsza_wersja_i_jej_czas': (<nr najnowszej wersji>, <jej znacznik czasu>)
        # plus jaka liczba elementw:
        #   '<nr wersji>': (<znacznik czasu>, <lista usunitych>, <lista nowych/dodanych>)
        info_o_wersjach = rsglobal.RSSlownikWPliku(sciezka_info_o_wersjach, 'c')
        jest_nowa_wersja = False
        try:
            (najnowsza_wersja, czas_najnowszej_wersji
            ) = info_o_wersjach.get('najnowsza_wersja_i_jej_czas', (0, '0'))
            
            if usuniete or zmienione_i_nowe:
                jest_nowa_wersja = True
                najnowsza_wersja += 1
                czas_najnowszej_wersji = repr(time.time())
                
                info_o_wersjach[str(najnowsza_wersja)
                ] = (czas_najnowszej_wersji, usuniete, zmienione_i_nowe)
                
                info_o_wersjach['najnowsza_wersja_i_jej_czas'
                ] = najnowsza_wersja, czas_najnowszej_wersji
        finally:
            info_o_wersjach.close()

        if jest_nowa_wersja:
            # plik wersji -- po stronie serwera suy tylko do
            # debugowania (mona do niego zajrze i zobaczy,
            # jaka wersja jest zapisana jako najnowsza)
            plik_wersji = rsglobal.RSPlik(os.path.join(kat_serwera, podkat_pomocniczy,
                                                       nazwa_pliku_wersji), 'w')
            try:
                plik_wersji.write('%d@%s' % (najnowsza_wersja, czas_najnowszej_wersji))
            finally:
                plik_wersji.close()
                
            blokada_kat_poprz = open(sciezka_blokady_poprz, 'w')
            try:
                fcntl.flock(blokada_kat_poprz, fcntl.LOCK_EX)
                # usuwamy poprzedni kopi i robimy kopi najnowszej wersji
                shutil.rmtree(kat_poprz)
                shutil.copytree(kat_akt, kat_poprz)
            finally:
                blokada_kat_poprz.close()
            
        rsglobal.ZapiszDoLogow('Aktualizacja: dane najnowszej wersji'
                               ' (po wieym porwnaniu): %s@%s' %
                               (najnowsza_wersja, czas_najnowszej_wersji),
                               5)
        return najnowsza_wersja, czas_najnowszej_wersji


    def _ktore_usu_zm_nowe(self, porownanie, usuniete, zmienione_i_nowe, nadkat=()):
        
        '''Uzupenij i zwr podane listy cieek: usuniete, zmienione_i_nowe.

        Argumenty:
        * porownanie -- instancja filecmp.dircmp (z biblioteki standardowej);
        * usuniete -- lista do wypenienia (pierwotnie pusta), zawierajca
                      cieki dostpu (w formie krotek nazw) do
                      plikw/katalogw, ktre maj zosta usunite;
        * zmienione_i_nowe -- lista do wypenienia (pierwotnie pusta),
                              zawierajca cieki dostpu (w formie krotek
                              nazw) do plikw/katalogw, ktre maj zosta
                              skopiowane z nowszej wersji;
        * nadkat -- cieka (w formie krotki), ktra bdzie doklejona na
                    pocztku kadej cieki (argument uywany w kolejnych
                    rekurencyjnych wywoaniach metody).

        '''
        
        if porownanie.funny_files:
            rsglobal.ZapiszDoLogow(u"Przy aktualizacji PAN napotkano pliki,"
                                   u" ktrych nie da si porwna: %s (bd pominite)"
                                   % ', '.join(sorted(repr(os.path.join(kat, nazwa))
                                                      for nazwa in porownanie.funny_files)),
                                   3)
        usuniete.extend(nadkat + (p,) for p in porownanie.left_only)
        usuniete.extend(nadkat + (p,) for p in porownanie.common_funny)
        zmienione_i_nowe.extend(nadkat + (p,) for p in porownanie.right_only)
        zmienione_i_nowe.extend(nadkat + (p,) for p in porownanie.common_funny)
        zmienione_i_nowe.extend(nadkat + (p,) for p in porownanie.diff_files)
        for podkat, por in porownanie.subdirs.iteritems():
            # rekurencja:
            self._ktore_usu_zm_nowe(por, usuniete, zmienione_i_nowe, nadkat + (podkat,))
        return usuniete, zmienione_i_nowe


    def _odczytaj_aktualizacje(self, wersja_u_klienta, najnowsza_wersja):
        '''Sprbuj odczyta dan aktualizacj z pliku cache'''
        kat_cache = os.path.join(kat_serwera, podkat_pomocniczy,
                                 podkat_cache_aktualizacji)
        (nazwa_pliku_cache   # '<nr wersji docelowej>z<nr wersji wyjciowej>'
        ) = os.path.join(kat_cache, '%dz%d' % (najnowsza_wersja, wersja_u_klienta))
        try:
            plik_cache = rsglobal.RSPlik(nazwa_pliku_cache, 'r')
        except EnvironmentError, exc:
            if exc.errno == errno.ENOENT:  # (plik nie istnieje)
                przygotowana_aktualizacja = None   # nie jest zcacheowana
            else:
                raise
        else:
            try:
                przygotowana_aktualizacja = plik_cache.read()
            finally:
                plik_cache.close()
        return przygotowana_aktualizacja


    def _zapisz_aktualizacje(self, wersja_u_klienta, najnowsza_wersja,
                             przygotowana_aktualizacja):
        '''Utwrz dla podanej aktualizacji plik cache j zawierajcy'''
        kat_cache = os.path.join(kat_serwera, podkat_pomocniczy,
                                 podkat_cache_aktualizacji)
        self._usun_stare_i_nadmiarowe(kat_cache, najnowsza_wersja)
        (nazwa_pliku_cache   # '<nr wersji docelowej>z<nr wersji wyjciowej>'
        ) = os.path.join(kat_cache, '%dz%d' % (najnowsza_wersja, wersja_u_klienta))
        plik_cache = rsglobal.RSPlik(nazwa_pliku_cache, 'w')
        try:
            plik_cache.write(przygotowana_aktualizacja)
        finally:
            plik_cache.close()


    def _usun_stare_i_nadmiarowe(self, kat_cache, najnowsza_wersja):
        
        '''Usu stare i nadmiarowe pliki cache aktualizacji'''
        
        ls = (nazwa for nazwa in os.listdir(kat_cache)
              if nazwa != rsglobal.RSPlik.PODKAT_BLOKAD)
              
        # lista nazw plikw cache uporzdkowana od najnowszych do najstarszych
        # wg wersji docelowej, a w razie rwnoci -- wg wersji wyjciowej:
        nazwy_plikow = sorted(ls, key=self._daj_wersje, reverse=True)
        
        # wszystko, co ma wersj docelow starsz ni najnowsza -- do skasownia:
        for i, nazwa in enumerate(nazwy_plikow):
            if self._daj_wersje(nazwa)[0] != najnowsza_wersja:
                break
        else:
            i = len(nazwy_plikow)
        do_skasowania = nazwy_plikow[i:]
        
        # pliki nadmiarowe (ponad okrelon liczb) -- rownie do skasowania:
        for i in xrange(i - 1, const.MAX_LICZBA_ZCACHEOWANYCH_AKTUALIZACJI_PAN - 1, -1):
            do_skasowania.append(nazwy_plikow[i])
            
        # kasujemy:
        for nazwa in do_skasowania:
            try:
                sciezka = os.path.join(kat_cache, nazwa)
                os.remove(sciezka)
            except EnvironmentError:
                rsglobal.ZapiszDoLogow(u"Nie udao si skasowa niepotrzebnego pliku "
                                       u"cache aktualizacji klienta PAN: %r" % sciezka,
                                       3)

    @staticmethod
    def _daj_wersje(nazwa):
        '''Z nazwy pliku cache aktualizacji wycignij nr-y wersji'''
        try:
            wersja_docelowa, wersja_wyjsciowa = map(int, nazwa.split('z'))
        except (TypeError, ValueError):
            return 0, 0
        else:
            return wersja_docelowa, wersja_wyjsciowa


if __name__ == "__main__":
    konfiguracja = {u"port": 5488, u"hostpgsql": u"127.0.0.1", u"portpgsql": 5432, u"panuser": u"anakonda", u"panpass": u"anakonda", u"baza": u"anakonda", u'panfirmaid': '01', u'panminkod': 1}
    try:
        while True:
            time.sleep(15)
    except KeyboardInterrupt:
        pass
    if not serwer.Zatrzymaj():
        try:
            while True:
                time.sleep(3)
                if serwer.Zatrzymaj():
                    break
        except KeyboardInterrupt:
            serwer.Zatrzymaj(force=True)
