# -*- coding: iso8859_2 -*-
# 2012-10-24 Mariusz Rajfura (3913) aktualizacja: przekazanie czasow modyfikacji dodanych, zmienionych plikow

import os
import os.path
import cPickle
import re
import const
import rsglobal
import binascii
import string

def GetPath( ver = None ):
	if ver == 'BINARY':
	   return "acw"
	else:
	   return "ac"

#-------------------------------------------------------------------------------

def IsTxtFile(filename, TxtExt=(".py", ".txt", ".php", ".conf", ".html")):
	return os.path.splitext(filename)[1] in TxtExt

#-------------------------------------------------------------------------------

class RSFileLister:
    def __init__( self,path,Start=True ):
        self.__path = path
        self.__fileSet = {}
        if Start:
            self.Process()

    def Process(self):
        if os.path.exists( self.__path ):
            os.path.walk( self.__path, self.ProcessFilesInDir, "" )
            self.__fileSet = self.TrimDirs( '^%s(\/|$)' % self.__path, self.__fileSet )

    def TrimDirs( self, trim,Container ):
        for oldDir in Container.keys():
            newDir = re.sub( trim, '', oldDir )
            Container[newDir] = Container[oldDir]
            if newDir != oldDir:
                del Container[oldDir]
        return Container

    def ProcessFilesInDir(self, CurrPath,dirname,names ):
        CurrPath = CurrPath + dirname
        CurrPathL = os.path.normpath(CurrPath)
        self.__fileSet[CurrPath] = {}
        for filename in names:

            #if filename == 'RSAnakonda.exe':
            #    continue

            if os.path.splitext(filename)[1].lower() in (".dll", ".pyd", ".conf", ".bat", ".log"):
            #if os.path.splitext(filename)[1].lower() in (".dll", ".pyd", ".exe", ".conf", ".bat", ".log"):
            #if os.path.splitext(filename)[1].lower() in (".exe", ".conf", ".bat", ".log"):
            #if os.path.splitext(filename)[1].lower() in (".conf", ".bat", ".log"):
                continue
            
            if filename[0] != '.' and not os.path.isdir( os.path.join(CurrPathL,filename) ):
                self.__fileSet[CurrPath][filename] = os.path.getmtime(os.path.join(CurrPathL,filename))

    def GetFileSet(self):
        return self.__fileSet

    def GetDirs(self):
        return self.__fileSet.keys()

    def GetFilesInDir(self,dir):
      return self.__fileSet[dir].keys()

#-------------------------------------------------------------------------------

class ClientFileDict(dict):

    """Sownik o kluczach napisowych, nie biorcy pod uwag wielkoci
    ich liter, ale starajcy si zachowa j niezmienion.

    Prba dodania sownika jako wartoci oznacza dodanie zainicjowanej
    nim instancji niniejszej klasy.

    """
    
    def __init__(self, a_dict=None):
        dict.__init__(self)
        self._lowerkeys2dictkeys = {}
        if a_dict is None:
            a_dict = {}
        for key, value in a_dict.iteritems():
            self[key] = value

    def __repr__(self):
        return 'ClientFileDict(%s)' % dict.__repr__(self)

    def __getitem__(self, key):
        try:
            dictkey = self._lowerkeys2dictkeys[key.lower()]
        except KeyError:
            raise KeyError(key)
        else:
            return dict.__getitem__(self, dictkey)

    def __setitem__(self, key, value):
        dictkey = self._lowerkeys2dictkeys.get(key.lower(), key)
        if isinstance(value, dict):
            value = ClientFileDict(value)
        dict.__setitem__(self, dictkey, value)
        self._lowerkeys2dictkeys[key.lower()] = dictkey

    def __delitem__(self, key):
        dictkey = self._lowerkeys2dictkeys[key.lower()]
        dict.__delitem__(self, dictkey)
        del self._lowerkeys2dictkeys[key.lower()]

    def __contains__(self, key):
        return key.lower() in self._lowerkeys2dictkeys
        
    has_key = __contains__

    copy = fromkeys = get = pop = popitem = setdefault = update = None

    def as_ordinary_dict(self):
        ordinary = {}
        for key, value in self.iteritems():
            if isinstance(value, ClientFileDict):
                value = value.as_ordinary_dict()
            ordinary[key] = value
        return ordinary

#-------------------------------------------------------------------------------

class RSFileCompare:
  def __init__(self, ClientFileSet, SerwerFileSet):
    self.__CFL = ClientFileDict(ClientFileSet)
    self.__SFL = SerwerFileSet
    self.__NewFiles = {}
    self.__UpdateFiles = {}
    self.__NewFilesTimestamps = {}
    self.__UpdateFilesTimestamps = {}
    
    
    self.extToRem = ['.py','.png','.gif','.jpg','.html']  # lista rozszerzen plikow ktore mozna usuwac u klienta
    self.extDontUpdate = [".pyd", ".dll", ".log"]  # lista rozszerzen plikow ktore nie beda aktualizowane
    #self.extDontUpdate = []

  def FindDiff(self):
    for dir in self.__SFL.keys():
      if dir in self.__CFL:
        self.CmpFilesInDir(dir)
      else:
        self.__NewFiles[dir] = []
        for file in self.__SFL[dir].keys():
          self.__NewFiles[dir].append(file)

          # czas nowego pliku
          mtime = self.__SFL[dir][file]
          self.__NewFilesTimestamps[(dir,file)] = mtime
    self.FilterRemoveFiles()

  def CmpFilesInDir(self,dir):
    for sfile,mtime in self.__SFL[dir].items():
      if sfile not in self.__CFL[dir]:
        if dir not in self.__NewFiles:
          self.__NewFiles[dir] = []
        self.__NewFiles[dir].append(sfile)
        
        # czas nowego pliku
        self.__NewFilesTimestamps[(dir,sfile)] = mtime

      else:
          
        ctime = self.__CFL[dir][sfile]
        
        # pliki zmienione za wyjatkiem tych o rozszerzeniach z self.extDontUpdate
        if (str(int(float(ctime))) < str(int(float(mtime)))) and (os.path.splitext(sfile)[1].lower() not in self.extDontUpdate):
          if dir not in self.__UpdateFiles:
            self.__UpdateFiles[dir] = []
          self.__UpdateFiles[dir].append(sfile)
          
          # czas zmodyfikowanego pliku
          self.__UpdateFilesTimestamps[(dir,sfile)] = mtime
        del self.__CFL[dir][sfile]

  def FilterRemoveFiles(self):
    if len(self.__SFL) == 0:
        self.__CFL = ClientFileDict()
    for dir in self.__CFL.keys():
        for file in self.__CFL[dir].keys():
            ext = os.path.splitext( file )[1].lower()
            if ext not in self.extToRem:
                del self.__CFL[dir][file]
        if len(self.__CFL[dir]) == 0:
            del self.__CFL[dir]

  def GetUpdateFiles(self):
    return self.__UpdateFiles

  def GetNewFiles(self):
    return self.__NewFiles

  def GetRemoveFiles(self):
    return self.__CFL.as_ordinary_dict()

  def GetUpdateFilesTimestamps(self):
    return self.__UpdateFilesTimestamps

  def GetNewFilesTimestamps(self):
    return self.__NewFilesTimestamps


#-------------------------------------------------------------------------------

class RSSerwerUpdateConn:
   def __init__(self,conn):
    self.Connection = conn
    self.ComputePacketSize()
    self.ClientVer = "SOURCE"

   def ComputePacketSize(self):
	self.PacketSize = ""
	for i in range(const.UPD_TCP_LEN_LEN):
		self.PacketSize += '9'
	self.PacketSize = int(self.PacketSize)

   def ReceiveFileSet(self):
      fileSet = self.RecvPackets()
      return fileSet

   def SendFile(self,dir,filename):
    dir = os.path.join( GetPath( self.ClientVer ),dir )

    if IsTxtFile(filename):
       file = open( os.path.join(dir,filename),"r" )
       fileData = file.read()
       self.SendPackets( fileData )
    else:
       file = open( os.path.join(dir,filename),"rb" )
       fileData = file.read()
       self.SendPackets( binascii.hexlify( fileData ) )
    file.close()

   def SendDiffs(self,diffs):
    self.Connection.send( "DIFFS" )
    self.SendPackets( diffs )

   def Listen(self):
    status = 0
    while True:
       try:
          data = self.Connection.recv()
          rsglobal.ZapiszDoLogow('DATA '+data, 4)
          if data[0:4] == "FILE":
             rsglobal.ZapiszDoLogow('D1',1)
             dir,filename = rsglobal.rseval( data[4:] )
             self.SendFile( dir,filename )
             status = 1
          elif data[0:3] == "END":
             rsglobal.ZapiszDoLogow('D2', 4)
             break
       except Exception, exc:
             rsglobal.ZapiszDoLogow( "Blad transmisji danych podczas aktualizacji %s" % exc,1 )
             status = 0
             return status
         
    return status

   def RecvPackets(self):
      IloscPakietow = int(self.Connection.recv())
      lastLen = int( self.Connection.recv() )
      data=''
      i = 1
      while i < IloscPakietow:
        i = i+1
        data = data+self.Connection.recv()
      data = data+self.Connection.recv()
      data = cPickle.loads(data)
      return data

   def SendPackets(self,inputdata):
      try:
        wielk = self.PacketSize
        data = cPickle.dumps(inputdata)
        IlePak,reszta = divmod(len(data),wielk)
        if reszta > 0:
            IlePak = IlePak + 1
        self.Connection.send(str(IlePak))
        if (reszta > 0):
            lastLen = reszta
        else:
            lastLen = wielk
        self.Connection.send(str(lastLen))
        bufor =""
        for i in range(IlePak):
            if i < IlePak-1:
                bufor = bufor+data[(i*wielk):((i+1)*(wielk))]
                self.Connection.send(data[(i*wielk):((i+1)*(wielk))])
            else:
                bufor = bufor+data[(i*wielk):len(data)]
                self.Connection.send(data[(i*wielk):len(data)])
        return 1
      except Exception,e:
        return 0

   def ReceiveClientVersion(self):
	  dane =  self.Connection.recv()
	  if dane[0:3] == "VER":
	  	self.ClientVer = dane[4:]

   def GetClientVer(self):
	   return self.ClientVer

#-------------------------------------------------------------------------------

class RSUpdateChecker:
  def __init__(self,UpConn,version=1):
    self.Connection = RSSerwerUpdateConn(UpConn)
    self.Connection.ReceiveClientVersion()

  def StartUpdate(self):
    UpdateDone = False
    ClientFileSet = self.Connection.ReceiveFileSet()
    FileEngine = RSFileLister( GetPath( self.Connection.GetClientVer() ) )
    CmpEngine = RSFileCompare( ClientFileSet ,FileEngine.GetFileSet() )
    CmpEngine.FindDiff()
    
    # dodane przekazanie Timestamps
    self.Connection.SendDiffs(( CmpEngine.GetNewFiles(),
                                CmpEngine.GetUpdateFiles(),
                                CmpEngine.GetRemoveFiles(),
                                CmpEngine.GetNewFilesTimestamps(),
                                CmpEngine.GetUpdateFilesTimestamps()
                                ))
                                
    UpdateDone = self.Connection.Listen()
    return UpdateDone
