# -*- coding: iso8859_2 -*-

# Author: Jan Kaliszewski
# Copyright: (c) 2010-2011 Record System
# Licence: GNU GPL

#-----------------------------------------------------------------------
#
# Opis procedury aktualizacji i uruchamiania klienta
# -- patrz: ac25/admin/RSUpdateNewStart.py
#
#-----------------------------------------------------------------------
#
# DO ADMINW:
#
# Wysoce niezalecane jest grzebanie w podkatalogu serwera z kodem klienta
# ('ac' lub 'acw') w czasie, gdy klient dokonuje aktualizacji. Jeeli
# wic chcemy co zmienia w tyme podkatalogu na dziaajcym serwerze
# produkcyjnym, wszelkie komendy powinnimy wywoywa zabezpieczajc je
# blokad flock zakadan na plik o nazwie "BLOKADA" -- w nastpujcy
# sposb:
#
#    flock -e BLOKADA -c '<tu wpisa komend>'
#
# UWAGA:
# * klienci, ktrzy w czasie wykonywania komendy podejm prb
#   aktualizacji, bd musieli poczeka do koca wykonania teje
#   komendy (nie powinna wic trwa duej ni kilka sekund).
# * to zabezpieczenie nie dziaa pod NFS (sieciowym systemem plikw).
#
#-----------------------------------------------------------------------
#
# Pliki pobieranie podczas aktualizacji mog by kompresowane lub nie.
# Zaley to od wartoci opcji "kompresja" w konfiguracji serwera
# (patrz opis w as2.4/rs-and.conf.sample).
#
# Wysyanie/odbieranie plikw z opcjonaln kompresj/dekompresj
# zaimplementowane jest w:
# * as2.4/rs_anakonda_socket.py
# * ac25/conn/RSAnakondaSocket.py
#
#-----------------------------------------------------------------------

import fcntl
import os
import os.path as osp
import re
from operator import itemgetter

KAT_BIN = 'acw'
KAT_SRC = 'ac'

BLOKADA = 'BLOKADA'

W_OGOLE_POMIJAC = ('.svn-base',)  # rozszerzenia nazw plikw, ktre maj by pomijane
TYLKO_NOWE = ('.log',)  # rozszerzenia nazw plikw, ktre maj by pobierane tylko raz
#TYLKO_NOWE = '.pyd', '.dll', '.log'  # stare (nie jare) ustawienie
MAX_DLUG_ODEBR_SCIEZEK = 10000 * 100
MAX_DLUG_ODEBR = 16



def genSciezkiCzasy(korzen_bezwzgl, sciezka=''):
    """Generuj krotki (<wzgldna scieka pliku>, <czas jego modyfikacji>)."""
    sciezka_bezwzgl = osp.join(korzen_bezwzgl, sciezka)
    if osp.isdir(sciezka_bezwzgl):
        if not osp.islink(sciezka_bezwzgl):
            for nazwa in os.listdir(sciezka_bezwzgl):
                if nazwa != BLOKADA:
                    for sciezka_czas in genSciezkiCzasy(korzen_bezwzgl, osp.join(sciezka, nazwa)):  # rekurencja
                        yield sciezka_czas
    else:
        if osp.splitext(sciezka)[1] not in W_OGOLE_POMIJAC:
            yield (sciezka.replace('\\', '/'), osp.getmtime(sciezka_bezwzgl))


def wyslijPlik(polaczenie, sciezka_bezwzgl):
    aktualny_czas_serw = osp.getmtime(sciezka_bezwzgl)
    polaczenie.send(str(aktualny_czas_serw))
    polaczenie.wyslij_plik(sciezka_bezwzgl)


def wykonajAktualizacje(polaczenie, kat_serwera):
    bin = (polaczenie.recv(max_dlugosc=3) == 'BIN')
    if bin:
        korzen_bezwzgl = osp.join(kat_serwera, KAT_BIN)
    else:
        korzen_bezwzgl = osp.join(kat_serwera, KAT_SRC)
    blokada = open(osp.join(korzen_bezwzgl, BLOKADA), 'w')
    fcntl.flock(blokada.fileno(), fcntl.LOCK_SH)
    try:
        nowe = []
        zmieniane = []
        sciezki_czasy_klienta = dict(sciezka_czas.lower().split('|')
                                     for sciezka_czas in polaczenie.recv(max_dlugosc=MAX_DLUG_ODEBR_SCIEZEK
                                                                        ).split('\\'))
        for sciezka, czas_serw in genSciezkiCzasy(korzen_bezwzgl):
            czas_klienta_s = sciezki_czasy_klienta.get(sciezka.lower())
            if czas_klienta_s is None:
                nowe.append(sciezka)
            elif float(czas_klienta_s) < czas_serw and osp.splitext(sciezka)[1] not in TYLKO_NOWE:
                zmieniane.append(sciezka)
        nowe.sort()
        zmieniane.sort()
        liczba_nowych = len(nowe)
        liczba_zmienianych = len(zmieniane)
        liczba_usuwanych = 0
        polaczenie.send('|'.join(map(str, (liczba_nowych, liczba_zmienianych, liczba_usuwanych))))
        etapy_sciezki = (nowe, zmieniane)
        sciezki_dla_biezacego_etapu = None
        oczekiwany_nr_pliku = None
        otrzymane_nry_etapow = set()
        while True:
            odebrane = polaczenie.recv(max_dlugosc=MAX_DLUG_ODEBR)
            if odebrane == 'END':
                break
            else:
                znacznik, informacja_liczbowa = odebrane.split('\\')
                if znacznik == 'STAGE':
                    etap = int(informacja_liczbowa)
                    if etap in otrzymane_nry_etapow:
                        raise RuntimeError('Przesano numer etapu %d, ktry ju raz by przesany' % etap)
                    else:
                        otrzymane_nry_etapow.add(etap)
                    sciezki_dla_biezacego_etapu = etapy_sciezki[etap]
                    polaczenie.send('\\'.join(sciezki_dla_biezacego_etapu))
                    oczekiwany_nr_pliku = 0
                elif znacznik == 'FILE':
                    nr_pliku = int(informacja_liczbowa)
                    if nr_pliku != oczekiwany_nr_pliku:
                        raise RuntimeError('Przesano numer pliku %d, a oczekiwano %d'
                                           % (nr_pliku, oczekiwany_nr_pliku))
                    sciezka_bezwzgl = osp.join(korzen_bezwzgl, sciezki_dla_biezacego_etapu[nr_pliku])
                    wyslijPlik(polaczenie, sciezka_bezwzgl)
                    oczekiwany_nr_pliku += 1
                else:
                    raise RuntimeError('Przesano nielegalny znacznik: %r' % znacznik)
    finally:
        blokada.close()
