#!/usr/bin/python
# -*- coding: iso8859_2 -*-

#-------------------------------------------------------------------------------
# Name:        rs_pobieranie_kursow_walut.py
# Purpose:     Pobieranie (import) kursw walut udostpnianych w Internecie
# Author:      Jan Kaliszewski
# Created:     2009/10/24
# RCS-ID:      $Id: rs_pobieranie_kursow_walut.py $
# Copyright:   (c) 2009 Record System
# Licence:     GNU GPL
#-------------------------------------------------------------------------------
# 2009-11-10,    Jan Kaliszewski: dodanie sprawdzania pobieranej wartoci pod
#                                 wzgldem bezpieczestwa (mantis: #0002771)
# 2010-02-10,    Jan Kaliszewski: dodanie "guchej" obsugi ciasteczek (cookies)
#                                 (mantis: #0003165) oraz moliwoci oznaczania
#                                 pl jako niewykorzystanych rwnie dla CSV;
#                                 sprawdzanie wzorca url pod ktem dozwolonych
#                                 protokow (mantis: #0003144); dodanie
#                                 weryfikacji/pobierania daty z pliku + zmiany
#                                 w wywoywaniu funkcji harmonogramu + inne zmiany
#                                 (mantis: #0003165)
# 2020-10-20, Aleksander Bargie: (12123) Problem z funkcj fs_kursy_walut_pobierz.
# 2021-06-02, Aleksander Bargie: (12511) Problem z pobieraniem kursw walut
# 2022-01-12, Aleksander Bargie: (12810) Pobieranie kursw walut - rozwiza na dobre problem z witem w sobot
# 2024-01-24, Aleksander Bargie: (13672) rs_anakonda_daty - le liczy liczb dni pracujcych

#-------------------------------------------------------------------------------
# UWAGA!
# W czci programu dotyczcej kursw walut naley rozrni 2 rne terminy:
#
# * numer tabeli (TCNrTabeli, NrTabeli, nr_tabeli...) -- oznaczenie pojedynczej
#   tabeli kursw, zwykle z konkretnego dnia (wpisane rcznie lub
#   pobrane/wygenerowane przy pobieraniu kursw z Internetu);
#   np. "205/A/NBP/2009";
#
# * tabela (TCTabelaId, tabela_id oraz TCTabelaNazwa, tabela_nazwa...) -- oglne
#   oznaczenie tabeli kursw dla konkretnego banku, obejmujcej dany rodzaj
#   notowa (dla wszystkich dat); dla banku NBP id tabeli to np. "A",
#   za pena nazwa to np.: "Tabela A kursw rednich walut obcych".
#-------------------------------------------------------------------------------


import Cookie
import datetime
import re
import socket
import string
import traceback
import urllib2
import xml.sax, xml.sax.handler

import rs_anakonda_daty
import const


#AB 02.06.21r
def dzien_roboczy_roku_poprawka(data):
    """korekty manualne:
       -NBP nie uwzglednia 2020-08-14 jako dnia wolnego, a rs_anakonda_daty.iloscDniPracujacychOdPoczatkuRoku tak
       -2021-05-01 odejmuje dzien roboczy za swieto wypadajace w sobote, NBP nie
       -od 2022 fix na odejmowanie dni roboczych za swieta juz na zawsze
       """
    dni = rs_anakonda_daty.iloscDniPracujacychOdPoczatkuRoku(data)
    #AB 2024-01-24
    # if data >= datetime.date(2022, 1, 1):  #AB 12.01.22r
    #     rok = data.year
    #     for miesiac, dzien in rs_anakonda_daty.STALE_SWIETA:
    #         swieto = datetime.date(rok, miesiac, dzien)
    #         if (datetime.date(rok, 1, 1) <= swieto <= data) and swieto.weekday() == 5:
    #             dni += 1
    if datetime.date(2020, 8, 14) <= data <= datetime.date(2020, 12, 31):
        dni += 1
    elif datetime.date(2021, 5, 1) <= data <= datetime.date(2021, 12, 31):
        dni += 1
    return dni

if __name__ != '__main__':
    from rsglobal import Dekoduj, ZapiszDoLogow
else:
    # dla skryptu testowego (patrz: koniec pliku)
    def Dekoduj(tekst):
        if isinstance(tekst, unicode):
            return tekst.encode('iso8859_2')
        else:
            return tekst
    def ZapiszDoLogow(tekst, dump):
        print Dekoduj(tekst)


# by uatwi przejcie do nowej wersji bez przeadowywania, potem mona usun:
if not hasattr(const, 'DOZWOLONE_PROTO_POBIERANIA_KURSOW'):
    const.DOZWOLONE_PROTO_POBIERANIA_KURSOW = 'http', 'https', 'ftp'


#-------------------------------------------------------------------------------
# Dedykowana klasa bdw

class BladImportuKursowWalut(Exception):
    u'Bd przy importowaniu (pobieraniu/parsowaniu) tabeli kursw walut'

    # stat, ktre maj by przekazane w postaci kodw:
    BL_IDTAB = 'BL IDTAB'    # nieprawidowy/nieistniejcy identyfikator tabeli
    BL_DATFOR = 'BL DATFOR'  # nieprawidowa data: zy format
    BL_DATROB = 'BL DATROB'  # nieprawidowa data: nie jest dniem roboczym
    BL_DATPOZ = 'BL DATPOZ'  # nieprawidowa data: pniejsza ni dzisiejsza
    BL_DATPLI = 'BL DATPLI'  # data niezgodna z dat w pobranym pliku
    BL_POBDAT = 'BL POBDAT'  # bd pobierania: brak danych w tabeli dla daty

    def __init__(self, tekst_stat, kod_stat=None, exc_debug=None, log=None):
        self.args = (tekst_stat, kod_stat, exc_debug, log)
        if kod_stat is None:
            self.stat = Dekoduj(tekst_stat)
            self.dane = []
        else:
            self.stat = Dekoduj(kod_stat)
            self.dane = [Dekoduj(tekst_stat)]
        
        if log is None:
            log = 5
        ZapiszDoLogow('Bd przy imporcie kursw walut: %s' % tekst_stat, log)
        if exc_debug is not None:
            exc_debug = Dekoduj(exc_debug)
            ZapiszDoLogow('Dodatkowa informacja o bdzie przy imporcie kursw'
                          ' walut: %s' % exc_debug, 5)


Blad = BladImportuKursowWalut


#-------------------------------------------------------------------------------
# Klasa definiujca zestaw danych skadajcych si na kady wiersz tabeli kursw
# i zapewniajca wygodne operowanie ju zaimportowanymi danymi

class RekordTabeliKursow(object):
    u'Klasa rekordu w tabeli kursw.'
    
    pola = (
            'waluta',
            'ilosc_waluty', 
            'kurs_sredni',
            'kurs_zakup',
            'kurs_sprzedaz',
    )

    pola_na_typy = dict(
            waluta = str,
            ilosc_waluty = int,
            kurs_sredni = float,
            kurs_zakup = float,
            kurs_sprzedaz = float,
    )
            
    domyslne = dict(
            kurs_sredni = 0.0,
            kurs_zakup = 0.0,
            kurs_sprzedaz = 0.0,
    )
    
            
    def __init__(self, zamieniaj_przecinek):
        u'Inicjuj jako pusty.'
        
        self._zawartosc = dict()
        self._zamieniaj_przecinek = zamieniaj_przecinek


    def __setitem__(self, pole, wartosc):
        u'Ustaw warto metod sownikow: rekord[pole] = warto.'

        pole = Dekoduj(pole)
        if pole not in self.__class__.pola_na_typy:
            raise KeyError('Nieprawidowa nazwa pola: "%s".' % pole)
        self._zawartosc[pole] = self._konwertuj(pole, wartosc)


    def __getitem__(self, pole):
        u'Pobierz warto metod sownikow: rekord[pole].'
        
        return self._zawartosc[Dekoduj(pole)]


    def __getattr__(self, pole):
        u'Pobierz warto jak atrybut: rekord.pole.'
        
        return self._zawartosc[Dekoduj(pole)]


    def __iter__(self):
        u'Iteruj po wartociach pl, w kolejnoci jak w atrybucie `pola`.'
        
        for pole in self.__class__.pola:
            yield self._zawartosc[pole]


    def __str__(self):
        u'Czytelna reprezentacja napisowa'
        
        if self.jest_kompletny():
            pola = self.__class__.pola
            return ', '.join('%s: %s' % (pola[i], wartosc)
                             for i, wartosc in enumerate(self))
        else:
            return 'Rekord niekompletny: %s' % str(self._zawartosc)


    def _konwertuj(self, pole, wartosc):
        u'Zamie napis na warto o typie waciwym dla danego pola.'
        
        typ = self.__class__.pola_na_typy[pole]
        if typ is int:
            return int(wartosc)
        elif typ is float:
            if self._zamieniaj_przecinek:
                wartosc = wartosc.replace(',', '.')
            return float(wartosc)
        else:
            return self.bezpiecznaWart(wartosc.strip(), pole)


    def jest_kompletny(self):
        u'Czy rekord ma ustawione wszystkie pola i czy kursy s niezerowe?'
        
        zawartosc = self._zawartosc
        return (len(zawartosc) >= len(self.__class__.pola)
                and ((zawartosc['kurs_zakup'] and zawartosc['kurs_sprzedaz'])
                     or zawartosc['kurs_sredni']))


    def dopelnij(self):
        u'Wypenij niewypenione pola wartociami domylnymi.'
        
        zawartosc = self._zawartosc
        for pole, wartosc in self.__class__.domyslne.iteritems():
            if pole not in zawartosc:
                zawartosc[pole] = wartosc


    def daj_zawartosc(self):
        u'Daj zawarto jako sownik, z wartociami sformatowanymi jako napisy.'

        return dict((pole, self.formatujWart(wartosc))
                    for pole, wartosc in self._zawartosc.iteritems())


    DOZWOLONE_ZNAKI = frozenset(string.ascii_letters + '0123456789.-_:/ ')

    @classmethod
    def bezpiecznaWart(cls, wartosc, pole):
        u'Przeciwko SQL injection: zwr warto, gdy bezpieczna; inaczej bd.'

        wartosc = Dekoduj(wartosc)
        niedozwolone = set(str(wartosc)) - cls.DOZWOLONE_ZNAKI
        if niedozwolone:
            raise Blad('Pole %s ("%s") zawiera niedozwolone znaki: %s.'
                       % (pole, wartosc,
                          ', '.join("%r" % znak for znak in niedozwolone)))
        else:
            return wartosc


    @classmethod
    def formatujWart(cls, wartosc):
        if isinstance(wartosc, float):
            return "'%.4f'" % (wartosc,)
        else:
            return "'%s'" % (wartosc,)


    @classmethod
    def formatujPoleWart(cls, pole, wartosc):
        if isinstance(wartosc, float):
            return "%s='%.4f'" % (pole, wartosc)
        else:
            return "%s='%s'" % (pole, wartosc)


Rekord = RekordTabeliKursow


#-------------------------------------------------------------------------------
# Klasa abstrakcyjna definiujca podstaw mechanizmw pobierania i parsowania

class ImporterKursow(object):
    u'Klasa podstawowa dla klas-importerow tabel kursw walut.'

    MAX_DLUGOSC_CACHE_WYNIKOW = 5
    
    # dopuszczalne parametry (patrz argument konstuktora `parametry`),
    # i ich domylne wartoci wraz z typami -- typ podanej tu wartoci
    # bdzie uyty do przerobienia pobranej z bazy wartoci (majcej
    # posta napisu) na waciw warto
    PARAMETRY = dict(
            zamieniaj_przecinek = True,
            # (uwaga dot. wartoci typu bool: w bazie musz mie posta '1'/'0')
    )

    ID_DNI = [
            'poniedzialek', 'wtorek', 'sroda',
            'czwartek', 'piatek', 'sobota', 'niedziela',
    ]

    def __init__(
            self,

            bank_id,       # np. 'NBP', 'BRE', 'PKO BP' itp.
            
            wzorzec_url,   # w rodzaju 'http://...%(rok)s%(miesiac)s%(dzien)s'
                           # (patrz metoda `_przygotuj()`, a take
                           # przykadowy skrypt na samym dole)
                          
            tabele,        # sownik: identyfikatory tabel
                           #          na krotki: (nazwa, alt_wzorzec_url)
            
            # upakowane w napisach:
            
            struktura_pliku,  # rozdzielone przecinkami wartoci odpowiadajce
                              # 5 polom rekordu (zob. klasa `Rekord`), a
                              # w przypadku XML rwnie oznaczeniu caego
                              # rekordu oraz opcjonalnie numeru tabeli i daty kursw:
                              # * dla XML -- 6 do 8 xml-owych tagw
                              # * dla CSV -- 5 numerw kolumn
                              #   (numery kolumn liczone od 0)
                          
            parametry='',  # dodatkowe parametry dla importera/parsera
                           # w formie 'klucz:warto,klucz:warto,...'
    ):
        try:
            self.bank_id = bank_id.strip()
            self.wzorzec_url = self._sprawdz_wzorzec_url(wzorzec_url.strip())
            self.tabele = dict((tabela_id.strip(), (nazwa.strip(), alt_wz.strip()))
                               for tabela_id, (nazwa, alt_wz) in tabele.iteritems())
            self._ustaw_parametry(parametry.strip())
            self._ustaw_strukture(struktura_pliku.strip())
            self._cache_wynikow = {}
        except Blad:
            raise
        except Exception, exc:
            raise Blad('Bd krytyczny przy inicjowaniu importera kursw walut!'
                       ' ("%r").' % repr(exc),
                       exc_debug=traceback.format_exc(), log=1)


    def __str__(self):
        u'Czytelna reprezentacja napisowa.'
        return 'Importer kursw z %s' % self.bank_id


    #---------------------------------------------------------------------------
    # Metody abstrakcyjne
    # [musz by zaimplementowane w podklasach]

    def parsuj(self, napis, data):
        u'Parsuj podany napis -- zwr list obiektw Rekord().'
        raise NotImplementedError

    def _ustaw_strukture(self, struktura_pliku_napis):
        u'Ustaw odpowiedni(e) atrybut(y) z informacj o strukturze pliku.'
        raise NotImplementedError

    def _czy_data_ma_byc_w_pliku(self):
        u'Czy dla tego banku data jest zawarta w pliku?'
        raise NotImplementedError


    #---------------------------------------------------------------------------
    # Metody "prywatne" (lub "chronione" -- do ewentualnego uytku w podklasach)
    # [ewentualnie mog, ale nie musz by zaimplementowane w podklasach]

    def _ustaw_parametry(self, parametry_napis):
        u'Ustaw atrybut `parametry` z dodatkowymi parametrami dla parsera.'

        # mamy jeden napis
        if parametry_napis:
            
            # zamiana na list par napisw ('nazwa', 'warto')
            try:
                par_napisy = [
                    (nazwa.strip(), wartosc.strip())
                    for nazwa, wartosc in (
                        par.split(':') for par in parametry_napis.split(','))]
            except ValueError:
                raise Blad('Nie udao si sparsowa argumentu `parametry`: "%s".'
                           % parametry_napis)

            # zamiana na list par ('nazwa', warto odpowiedniego typu)
            try:
                parametry = [
                    (nazwa, type(self.PARAMETRY[nazwa])(wartosc))
                    for nazwa, wartosc in par_napisy]
            except (KeyError, ValueError, TypeError):
                raise Blad(
                      'Nie udao si sparsowa argumentu `parametry`: "%s".'
                      % parametry_napis,
                      log=3,
                      exc_debug='Nieprawidowa warto lub nazwa parametru dla'
                            ' importera kursw z %s.\n'
                            'Oto podane parametry wraz typami, na ktre miayby'
                            ' by rzutowane (NoneType oznacza, e sama nazwa'
                            ' parametru jest nieprawidowa): %s.\n'
                            'Oto dopuszczalne nazwy parametrw: %s.'
                            % (self.bank_id,
                               ", ".join('%r: %r [=> %s]'
                                         % (nazwa, wartosc,
                                            type(self.PARAMETRY.get(nazwa))
                                            .__name__)
                                         for nazwa, wartosc in par_napisy),
                               ", ".join(map(repr, self.PARAMETRY.iterkeys()))))

            # ustawienie atrybutu `parametry`, z uwzgl. domylnych wartoci
            self.parametry = self.PARAMETRY.copy()
            self.parametry.update(parametry)
            
        else:
            self.parametry = self.PARAMETRY.copy()


    def _sprawdz_wzorzec_url(self, wzorzec_url):
        u'Sprawd pocztek wzorca URL (musi okrela dozwolony protok).'

        proto = wzorzec_url.split(':')[0]  # np. 'http'
        if proto not in const.DOZWOLONE_PROTO_POBIERANIA_KURSOW:
            raise Blad('Bd we wzorcu URL: niedozwolony'
                       ' protok pobierania: "%s".' % proto)
        return wzorzec_url


    def _przygotuj(self, tabela_id, podana_data, czy_data_z_pliku):
        u'Sprawd argumenty, nastpnie zwr sownik do podstawienia w adresie.'

        # sprawdzamy czy prawidowy identyfikator tabeli
        if tabela_id not in self.tabele:
            raise Blad('Dla tego banku nie zdefiniowano tabeli "%s" (dostpne: %s).'
                       % (tabela_id, ', '.join('"%s"' % t
                                               for t in sorted(self.tabele))),
                       Blad.BL_IDTAB)
        
        dane_dla_url = dict(
                tabela_id = tabela_id,
                tabela_id_upper = tabela_id.upper(),
                tabela_id_lower = tabela_id.lower(),
        )
        
        alt_wzorzec_url = self.tabele[tabela_id][1].strip()
        if alt_wzorzec_url and alt_wzorzec_url != 'None':
            wzorzec_url = self._sprawdz_wzorzec_url(alt_wzorzec_url)
        else:
            wzorzec_url = self.wzorzec_url  # ju by sprawdzony w __init__()
        
        if czy_data_z_pliku not in ('', 'koniecznie', 'opcjonalnie'):
            raise ValueError('Argument `czy_data_z_pliku` musi by pusty'
                             ' albo mie warto "koniecznie" lub "opcjonalnie".')
        elif czy_data_z_pliku and self._czy_data_ma_byc_w_pliku():
            data = 'z pliku'
        elif czy_data_z_pliku == 'koniecznie':
            raise Blad('Dla tego banku pliki nie zawieraj daty.')
        else:
            # data nie z pliku; przygotowujemy dat i sprawdzamy czy prawidowa
            if isinstance(podana_data, datetime.date):
                data = podana_data
            else:
                try:
                    data = datetime.date(*map(int, podana_data.split('-')))
                except (ValueError, TypeError):
                    raise Blad('Podana data jest nieprawidowa.', Blad.BL_DATFOR)
            if data > datetime.date.today():
                raise Blad('Podano dat pniejsz ni dzisiejsza.', Blad.BL_DATPOZ)
            if not rs_anakonda_daty.CzyRoboczy(data):
                raise Blad('Podana data nie jest dniem roboczym.', Blad.BL_DATROB)

            # ktry to w roku poniedziaek lub wtorek, lub roda, lub...
            dni_tyg_w_roku = {}
            for i, id_dnia in enumerate(self.ID_DNI):
                if data.weekday() == i:
                    pocz_roku = datetime.date(data.year, 1, 1)
                    od_pocz_roku = (data - pocz_roku).days
                    ktory_w_roku = (od_pocz_roku - pocz_roku.weekday()) / 7 + 1
                    dni_tyg_w_roku['%s_w_roku' % id_dnia] = ktory_w_roku
                else:
                    dni_tyg_w_roku['%s_w_roku' % id_dnia] = 0
            dane_dla_url.update(dni_tyg_w_roku)

            # pozostae pola wynikajce z daty
            dane_dla_url.update(dict(
                    rok = data.year,
                    rok2 = data.year % 100,
                    miesiac = data.month,
                    dzien = data.day,
                    dzien_roku = int(data.strftime('%j')),
                    dzien_roboczy_roku = (
                        dzien_roboczy_roku_poprawka(data) #AB 20.10.20r
                    ),
                    data = str(data)
            ))
            
        # zwracamy wzorzec do generowania URL (adresu internetowego),
        # sownik z polami do podstawienia w generowanym URL oraz dat
        return wzorzec_url, dane_dla_url, data


    def _utworz_url(self, wzorzec_url, dane_dla_url, data):
        try:
            return wzorzec_url % dane_dla_url
        except KeyError, exc:
            if (data == 'z pliku'
                and exc.args[0] in ['rok', 'rok2', 'miesiac', 'dzien',
                                    'dzien_roku', 'dzien_roboczy_roku', 'data']
                                   + ['%s_w_roku' % id_dnia for id_dnia in self.ID_DNI]):
                raise Blad('Bd przy tworzeniu URL-a ze wzorca: data miaa by'
                           ' wzita z pobieranego pliku, wic wzorzec URL nie moe'
                           ' zawiera pl opartych na dacie, takich jak "%s".' % exc.args[0],
                           exc_debug='Bdny wzorzec: %r.\nTraceback bdu: %s'
                           % (wzorzec_url, traceback.format_exc()),
                           log=3)
            else:
                raise Blad('Bd przy tworzeniu URL-a ze wzorca: zawiera on'
                           ' nieprawidowe pole: "%s".' % exc.args[0],
                           exc_debug='Bdny wzorzec: %r.\nTraceback bdu: %s'
                           % (wzorzec_url, traceback.format_exc()),
                           log=3)


    def _pobierz_spod_url(self, url, tabela_id, data):
        u'Pobierz dane spod podanego adresu (URL-a).'

        timeout_dla_reszty = socket.getdefaulttimeout()
        timeout_dla_importera = const.TIMEOUT_POBIERANIA_KURSOW_WALUT
        socket.setdefaulttimeout(timeout_dla_importera)
        try:
            for proba in range(3):
                try:
                    #zrodlo = urllib2.urlopen(url)
                    # "gucha" obsuga cookies (bo niektre serwisy wymagaj):
                    zrodlo = urllib2.build_opener(urllib2.HTTPCookieProcessor()).open(url)
                    try:
                        return zrodlo.read()
                    finally:
                        zrodlo.close()
                except (urllib2.HTTPError, socket.timeout), exc:
                    if isinstance(exc, socket.timeout) or exc.code == 408:
                        if proba < 2:
                            continue
                        else:
                            raise Blad('Nie udao si pobra kursw walut'
                                       ' - przekroczenie czasu oczekiwania'
                                       ' (problem z sieci lub po stronie'
                                       ' serwera, z ktrego pobierane s kursy.')
                    elif exc.code in (403, 404, 410):
                        if data == datetime.date.today():
                            raise Blad('Pod adresem "%s" nie odnaleziono tabeli "%s" dla '
                                       ' dzisiejszej daty (%s). By moe bank'
                                       ' jeszcze nie udostpni kursw do'
                                       ' pobrania.' % (url, tabela_id, data),
                                       Blad.BL_POBDAT)
                        elif data == 'z pliku':
                            raise Blad('Pod adresem "%s" nie odnaleziono tabeli kursw.' % url)
                        else:
                            raise Blad('Pod adresem "%s" nie odnaleziono tabeli "%s" dla daty %s.'
                                       % (url, tabela_id, data), Blad.BL_POBDAT)
                    elif exc.code in (500, 503):
                        raise Blad('Problem po stronie serwera, z ktrego pobierane'
                                   ' s kursy (kod bdu HTTP: %s).' % exc.code)
                    else:
                        raise Blad('Problem z pobraniem kursw walut przez sie'
                                   ' (kod bdu HTTP: %s).' % exc.code,
                                   exc_debug=traceback.format_exc(), log=3)
                except Cookie.CookieError:
                    raise Blad('Problem z pobraniem kursw walut przez sie'
                               ' dotyczcy obsugi ciasteczek (cookies).',
                               exc_debug=traceback.format_exc())
                except urllib2.URLError:
                    raise Blad('Problem z pobraniem kursw walut przez sie.',
                               exc_debug=traceback.format_exc())
                break
        finally:
            socket.setdefaulttimeout(timeout_dla_reszty)


    def _uzupelnij_nr_tabeli(self, nr_tabeli, tabela_id, data):
        u'Utwrz, gdy nie istnieje; uzupenij o id banku, jeli go nie zawiera'

        if not nr_tabeli:
            dzien_roboczy_roku = dzien_roboczy_roku_poprawka(data) #AB 20.10.20r
            nr_tabeli = '%03d/%s/%s/%d' % (dzien_roboczy_roku,
                                           tabela_id,
                                           self.bank_id,
                                           data.year)
        elif self.bank_id not in nr_tabeli:
            nr_tabeli = '%s /%s/' % (nr_tabeli, self.bank_id)

        nr_tabeli = RekordTabeliKursow.bezpiecznaWart(nr_tabeli, pole="nr_tabeli")
                
        if len(nr_tabeli) > 50:
            raise Blad('Za dugi numer tabeli: "%s".' % nr_tabeli)
            
        return nr_tabeli


    def _daj_wynik_z_cache(self, tabela_id, data):
        u"Sprbuj pobra gotowy wynik z cache'a."

        return self._cache_wynikow.get((str(tabela_id), str(data)))


    def _zcacheuj_wynik(self, tabela_id, data, wynik):
        u"Zapisz gotowy wynik do cache'a."
        
        if len(self._cache_wynikow) > self.MAX_DLUGOSC_CACHE_WYNIKOW:
            self._cache_wynikow.popitem()
        self._cache_wynikow[(str(tabela_id), str(data))] = wynik


    #---------------------------------------------------------------------------
    # Metody publiczne
    # [ewentualnie mog, ale nie musz by zaimplementowane w podklasach]

    DATA_RE = re.compile(
            r'\b(?:'                    # rok-miesic-dzie:
                r'(?P<rmd_rok>'
                    r'(?P<rok_ts>[12]\d)?'  # [RR]  (rok -- cyfry tysicy i setek)
                    r'\d{2}'                # RR    (rok -- cyfry dziesitek i jednoci)
                r')'
                r'[-/.]'                    # (separator czyli '-', '/' lub '.')
                r'(?P<rmd_miesiac>[01]\d)'  # MM
                r'[-/.]'                    # (separator)
                r'(?P<rmd_dzien>[0-3]\d)'   # DD
            r'|'                        # lub dzie-miesic-rok:
                r'(?P<dmr_dzien>[0-3]?\d)'  # [D]D
                r'[-/.]'                    # (separator)
                r'(?P<dmr_miesiac>[01]\d)'  # MM
                r'[-/.]'                    # (separator)
                r'(?P<dmr_rok>[12]\d{3})'   # RRRR
            r')\b'
    )

    @classmethod
    def wyciagnij_date(cls, napis):
        u'''Znajd i wycignij z podanego napisu dat zapisan w formacie
        [RR]RR<separator>MM<separator>DD lub [D]D<separator>MM<separator>RRRR
        (gdzie R, M, D to odpowiednio cyfra roku, miesica, dnia,
        a <separator> to znak minusa lub ukonika, lub kropki.'''

        d = cls.DATA_RE.search(napis).groupdict()
        if d['rmd_rok'] is not None:
            if d['rok_ts'] is None:  # s tylko cyfry dziesitek i jednoci roku
                d['rmd_rok'] = '20' + d['rmd_rok']  # => powikszamy rok o 2000
            pola_daty = d['rmd_rok'], d['rmd_miesiac'], d['rmd_dzien']
        else:
            pola_daty = d['dmr_rok'], d['dmr_miesiac'], d['dmr_dzien']
        return datetime.date(*map(int, pola_daty))


    def zaimportuj(self, tabela_id, podana_data, czy_data_z_pliku=''):
        u'Zaimportuj dane z serwera bankowego.'

        try:
            tabela_id = tabela_id.strip()
            
            if not czy_data_z_pliku:
                # by moe gotowy wynik jest ju zcache-owany
                gotowy_wynik = self._daj_wynik_z_cache(tabela_id, podana_data)
                if gotowy_wynik:
                    return gotowy_wynik
            
            # sprawdzamy argumenty, przygotowujemy wzorzec URL,
            # sownik z polami do podstawienia w generowanym URL
            # i dat (data to instancja datetime.date lub napis: 'z_pliku')
            wzorzec_url, dane_dla_url, data = self._przygotuj(tabela_id, podana_data, czy_data_z_pliku)

            # generujemy adres sieciowy (URL)
            url = self._utworz_url(wzorzec_url, dane_dla_url, data)

            # pobieramy dane z sieci
            napis = self._pobierz_spod_url(url, tabela_id, data)
            
            # parsujemy dane i zwracamy:
            # * kursy_walut -- sownik kodw walut na obiekty Rekord() zawierajce poszczeglne notowania,
            # * nr_tabeli -- None lub napis zawierajcy pobrany numer tabeli,
            # * data -- ta sama, ktr podano jako argument lub pobrana z pliku, jeli podano 'z_pliku'
            try:
                kursy_walut, nr_tabeli, data = self.parsuj(napis, data)
            except Blad:
                raise
            except Exception, exc:
                raise Blad('Bd parsowania pobranych danych ("%r").' % exc,
                           exc_debug=traceback.format_exc(),
                           log=1)

            # sprawdzamy, czy wszystkie rekordy s kompletne
            for rekord in kursy_walut.itervalues():
                if not rekord.jest_kompletny():
                    raise Blad('Pobrana tabela kursw jest niekompletna.',
                               exc_debug='\n'.join(
                                     map(str, kursy_walut.itervalues())
                               ),
                               log=3)

            # uzupeniamy nr tabeli o identyfikator banku etc.,
            # tworzymy, jeli go brak
            nr_tabeli = self._uzupelnij_nr_tabeli(nr_tabeli, tabela_id, data)

            wynik = kursy_walut, nr_tabeli, str(data)
            if not czy_data_z_pliku:
                # wynik zapamitujemy do ewentualnego pniejszego uycia
                self._zcacheuj_wynik(tabela_id, data, wynik)
            
            return wynik
                
        except Blad:
            raise
        except Exception, exc:
            raise Blad('Bd krytyczny przy pobieraniu/parsowaniu kursw walut'
                       ' ("%r").' % exc,
                       exc_debug=traceback.format_exc(),
                       log=1)


#-----------------------------------------------------------------------------
# Klasy implementujce konkretne sposoby pobierania/parsowania kursw walut

class ImporterKursowCSV(ImporterKursow):
    u'Podklasa bdca podstaw przy importowaniu tabel kursw w postaci CSV.'

    # dopuszczalne `parametry` (patrz `ImporterKursow._ustaw_parametry()`)
    # i ich domylne wartoci
    PARAMETRY = ImporterKursow.PARAMETRY.copy()
    PARAMETRY.update(
            separator = ';',
            zacznij_od_wiersza = 0,  # waciwe dane zaczynaj si od niego
            wiersz_nru_tabeli = -1,  # nr tabeli jest w tym wierszu... (-1 oznacza brak)
            kolumna_nru_tabeli = 0,  # ...i w tej kolumnie
            wiersz_daty = -1,  # data jest w tym wierszu... (-1 oznacza brak)
            kolumna_daty = 0,  # ...i w tej kolumnie
    )
    

    @staticmethod
    def _nr_kolumny(oznaczenie_kolumny):
        u'Zamie oznaczenie kolumny na liczb cakowit'
        
        if ''.join(set(oznaczenie_kolumny.strip())) == '-':
            # skadajce si z minusw oznaczenie pominicia -- zwracamy (-1)
            return -1
        else:
            # oznaczenie istniejcej kolumny -- zwracamy jej (nieujemny) indeks
            return int(oznaczenie_kolumny)
            

    def _ustaw_strukture(self, struktura_pliku_napis):
        u'''Ustaw atrybut `pola_na_kolumny`

        -- sownik odwzorowujcy nazwy pl rekordu (patrz zawarto atrybutu
        `pola` klasy `Rekord`) na numery kolumn w pliku CSV (liczone od 0).
        '''

        try:
            pola_na_kolumny = dict(
                (Rekord.pola[i], self._nr_kolumny(kolumna))
                for i, kolumna in enumerate(struktura_pliku_napis.split(',')[:5])
            )
            if len(pola_na_kolumny) != 5:
                raise ValueError
        except ValueError:
            raise Blad('Nie udao si sparsowa argumentu `struktura_pliku`: %r.'
                       % struktura_pliku_napis)
        else:
            self.pola_na_kolumny = pola_na_kolumny


    def _czy_data_ma_byc_w_pliku(self):
        u'Czy dla tego banku data jest zawarta w pliku?'

        return self.parametry['wiersz_daty'] > -1


    def parsuj(self, napis, data):
        u'Parsuj podany CSV -- zwr list obiektw Rekord()'
        
        kursy_walut = {}
        nr_tabeli = None

        zacznij_od_wiersza = self.parametry['zacznij_od_wiersza']
        separator = self.parametry['separator']
        zamieniaj_przecinek = self.parametry['zamieniaj_przecinek']
        wiersz_nru_tabeli = self.parametry['wiersz_nru_tabeli']
        kolumna_nru_tabeli = self.parametry['kolumna_nru_tabeli']
        wiersz_daty = self.parametry['wiersz_daty']
        kolumna_daty = self.parametry['kolumna_daty']

        # parsuj waciwe dane
        for nr_wiersza, wiersz in enumerate(napis.splitlines()):
            if wiersz.strip():
                pola_wiersza = wiersz.split(separator)
                if nr_wiersza >= zacznij_od_wiersza:
                    rekord = Rekord(zamieniaj_przecinek)
                    for pole in Rekord.pola:
                        nr_kol = self.pola_na_kolumny[pole]
                        if nr_kol >= 0:
                            rekord[pole] = pola_wiersza[nr_kol]
                    rekord.dopelnij()
                    kursy_walut[rekord['waluta']] = rekord
                if wiersz_nru_tabeli == nr_wiersza:
                    nr_tabeli = pola_wiersza[kolumna_nru_tabeli].strip()
                if wiersz_daty == nr_wiersza:
                    data_z_pliku = self.wyciagnij_date(pola_wiersza[kolumna_daty])
                    if data == 'z pliku':
                        data = data_z_pliku
                    elif data != data_z_pliku:
                        raise Blad('Podana data (%s) jest rna od daty w pobranym pliku (%s).'
                                   % (data, data_z_pliku),
                                   Blad.BL_DATPLI)
        if data == 'z pliku':
            raise Blad('Brak informacji o dacie w pobranym pliku.')

        return kursy_walut, nr_tabeli, data


class ImporterKursowXML(ImporterKursow):
    u'Podklasa bdca podstaw przy importowaniu tabel kursw XML.'

    # dopuszczalne `parametry` (patrz `ImporterKursow._ustaw_parametry()`)
    # i ich domylne wartoci
    PARAMETRY = ImporterKursow.PARAMETRY.copy()
    PARAMETRY.update(
            tag_nru_tabeli = '',  # nr tabeli jest podawany w tym tagu (pusty oznacza brak)
            tag_daty = '',        # data jest podawana w tym tagu (pusty oznacza brak)
    )

    class ContentHandler(xml.sax.handler.ContentHandler):
        u'Waciwa implementacja sposobu w jaki interpretowane s tagi XML'

        def __init__(self, importer):
            self.importer = importer
            self.kursy_walut = {}
            self.nr_tabeli = None
            self.data_z_pliku = None
            self._teraz_dane = False
            
        def startElement(self, name, attrs):
            nazwa = self.importer.tagi.get(name)
            if nazwa is not None:
                if nazwa == 'CALY_REKORD':
                    self._rekord = Rekord(
                            self.importer.parametry['zamieniaj_przecinek'])
                elif nazwa in Rekord.pola_na_typy or nazwa in ('NR_TABELI', 'DATA_KURSOW'):
                    self._teraz_dane = True
                    self._biezace_dane = []
            
        def characters(self, content):
            if self._teraz_dane:
                self._biezace_dane.append(content)
            
        def endElement(self, name):
            nazwa = self.importer.tagi.get(name)
            if nazwa is not None:
                if nazwa == 'CALY_REKORD':
                    self._rekord.dopelnij()
                    waluta = self._rekord['waluta']
                    self.kursy_walut[waluta] = self._rekord
                elif nazwa in Rekord.pola_na_typy or nazwa in ('NR_TABELI', 'DATA_KURSOW'):
                    zawartosc = ''.join(self._biezace_dane)
                    if nazwa == 'NR_TABELI':
                        self.nr_tabeli = zawartosc.strip()
                    elif nazwa == 'DATA_KURSOW':
                        self.data_z_pliku = zawartosc
                    else:
                        self._rekord[nazwa] = zawartosc
                    self._teraz_dane = False


    @staticmethod
    def _pominieto_zakup_i_sprzedaz(lista_tagow):
        u'''Czy tagi obejmujce kursy kupna i sprzeday zastpione s
        takimi samymi oznaczeniami pominicia? (opartymi na znaku "-")'''

        zakup, sprzedaz = lista_tagow[3:5]
        return (zakup == sprzedaz
                and ''.join(set(zakup.strip())) == '-')

    
    def _ustaw_strukture(self, struktura_pliku_napis):
        u'''Ustaw atrybut `tagi`

        -- sownik odwzorowujcy tagi w danym (dla danego banku) XML-u na:
        * 5 kolejnych nazw pl rekordu (patrz atrybut `pola` klasy `Rekord`),
        * 'CALY_REKORD' (dotyczy tagu obejmujcego rekord tabeli),
        * 'NR_TABELI' (opcjonalnie -- dotyczy tagu obejmujcego
                       oznaczenie numeru tabeli),
        * 'DATA_KURSOW' (opcjonalnie -- dotyczy tagu obejmujcego
                         oznaczenie daty kursw).

        Tagi pl rekordu i tag obejmujcy cay rekord tabeli s ustalane na
        podstawie zawartoci argumentu `struktura_pliku`.
        
        Nr tabeli i data kursw s ustalane na podstawie zawartoci argumentu
        `parametry` (dziaa rwnie przestarzae rozwizanie: ustalanie numeru
        tabeli na podstawie dodatkowego 7. pola argumentu `struktura_pliku`).

        W przypadku:
        * kursu redniego, albo
        * kursu zakupu i/lub kursu sprzeday
        -- zamiast tagu wystpowa moe oznaczenie pominicia danego tagu,
        skadajce si z jednego lub wicej znakw "-".
        '''

        lista_tagow = struktura_pliku_napis.split(',')
        try:
            # tagi pl rekordu
            tagi = dict(
                (tag.strip(), Rekord.pola[i])
                for i, tag in enumerate(lista_tagow[:5])
            )

            if len(tagi) < 4 or (len(tagi) < 5
                                 and not self._pominieto_zakup_i_sprzedaz(lista_tagow)):
                raise Blad('Nie udao si sparsowa argumentu `struktura_pliku`: %r.\n'
                           'Zbyt maa liczba pl lub zdublowana nazwa tagu.'
                           % struktura_pliku_napis)

            # tag caego rekordu
            tag_calego_rekordu = lista_tagow[5].strip()
            nazwa = tagi.setdefault(tag_calego_rekordu, 'CALY_REKORD')
            if nazwa != 'CALY_REKORD':
                raise Blad('Nie udao si sparsowa argumentu `struktura_pliku`: %r.\n'
                           'Zdublowana nazwa tagu: %s' % (struktura_pliku_napis, tag_calego_rekordu))

            # tag numeru tabeli (+sprawdzenie liczby pl argumentu `struktura_pliku`)
            dlug_listy_tagow = len(lista_tagow)
            if dlug_listy_tagow == 6:
                tag_nru_tabeli = self.parametry['tag_nru_tabeli']
            elif dlug_listy_tagow == 7:
                # (zaszo historyczna: okrelanie tagu nru tabeli
                # w argumencie `struktura_pliku` zamiast w `parametry`)
                if self.parametry['tag_nru_tabeli']:
                    raise Blad('Nie udao si sparsowa argumentu `struktura_pliku`: %r.\n7. pole'
                               ' okrelajce nazw tagu z numerem tabeli jest niedozwolone,'
                               ' gdy argument `parametry` zawiera ju parametr "tag_nru_tabeli".'
                               % struktura_pliku_napis)
                tag_nru_tabeli = lista_tagow[6].strip()
            else:
                raise ValueError('Za liczba elementw: %s' % dlug_listy_tagow)
            if tag_nru_tabeli:
                nazwa = tagi.setdefault(tag_nru_tabeli, 'NR_TABELI')
                if nazwa != 'NR_TABELI':
                    raise Blad('Zdublowana nazwa tagu: %s' % tag_nru_tabeli)

            # tag daty kursw
            tag_daty = self.parametry['tag_daty']
            if tag_daty:
                nazwa = tagi.setdefault(tag_daty, 'DATA_KURSOW')
                if nazwa != 'DATA_KURSOW':
                    raise Blad('Zdublowana nazwa tagu: %s' % tag_daty)
            
        except (ValueError, IndexError):
            raise Blad('Nie udao si sparsowa argumentu `struktura_pliku`: %r.'
                       % struktura_pliku_napis, exc_debug=traceback.format_exc())
        else:
            self.tagi = tagi


    def _czy_data_ma_byc_w_pliku(self):
        u'Czy dla tego banku data jest zawarta w pliku?'

        return bool(self.parametry['tag_daty'])


    def parsuj(self, napis, data):
        u'Parsuj podany XML -- zwr list obiektw Rekord().'
        
        handler = self.ContentHandler(self)
        xml.sax.parseString(napis, handler)
        
        if handler.data_z_pliku is not None:
            data_z_pliku = self.wyciagnij_date(handler.data_z_pliku)
            if data == 'z pliku':
                data = data_z_pliku
            elif data != data_z_pliku:
                raise Blad('Podana data (%s) jest rna od daty w pobranym pliku (%s).'
                           % (data, data_z_pliku),
                           Blad.BL_DATPLI)
        elif data == 'z pliku':
            raise Blad('Brak informacji o dacie w pobranym pliku.')
            
        return handler.kursy_walut, handler.nr_tabeli, data


#-----------------------------------------------------------------------------
# Odwzorowanie wartoci `format pliku` na klasy importerw (staa):

KLASY_IMPORTEROW_KURSOW = {
        'CSV': ImporterKursowCSV,
        'XML': ImporterKursowXML,
}

#-----------------------------------------------------------------------------
# Zewntrzny interfejs

# Odwzorowanie wartoci `bank_id` na utworzone importery (zmienna):
_importery_kursow = {}


def dajImporterKursow(bank_id):
    u'Zwr importer kursw (dla danego banku) jeeli istnieje, lub zwr None.'

    bank_id = Dekoduj(bank_id).strip()
    return _importery_kursow.get(bank_id)
    

def dajUtworzImporterKursow(format_pliku, *argumenty, **nazwane_argumenty):
    u'Zwr podany importer kursw; utwrz, jeeli nie istnieje.'

    format_pliku = Dekoduj(format_pliku).strip()
    try:
        bank_id = nazwane_argumenty['bank_id']
    except KeyError:
        bank_id = argumenty[0]
    bank_id = Dekoduj(bank_id).strip()
    
    importer = dajImporterKursow(bank_id)
    if not importer:
        try:
            klasa_importera = KLASY_IMPORTEROW_KURSOW[format_pliku]
        except KeyError:
            raise Blad('Nieobsugiwany format pliku: %r' % format_pliku)
        importer = klasa_importera(*argumenty, **nazwane_argumenty)  # tworzymy
        _importery_kursow[bank_id] = importer  # zapamitujemy na przyszo
        
    return importer
        

def usunImporterKursow(bank_id):
    u'Usu importer kursw (dla danego banku) jeeli istnieje.'

    try:
        del _importery_kursow[Dekoduj(bank_id).strip()]
    except KeyError:
        pass

    
##############################################################################
# Przykad uycia klas importerw:

if __name__ == '__main__':

    def podajDate():
        u'Czysto pomocnicza funkcja na potrzeby przykadowego skryptu'

        # domylna -- ostatni roboczy dzie przed dniem dzisiejszym
        jeden_dzien = datetime.timedelta(1)
        domyslna = datetime.date.today() - jeden_dzien
        while not rs_anakonda_daty.CzyRoboczy(domyslna):
            domyslna -= jeden_dzien
            
        while True:
            wej = raw_input('Podaj dat w formacie RRRR-MM-DD [domylna: %s]: '
                            % domyslna)
            if not wej:
                data = domyslna
            else:
                try:
                    data = datetime.date(*map(int, wej.split('-')))
                except (ValueError, TypeError):
                    print '\nTo nie jest prawidowa data.',
                    continue
            return data


    # Przykady "klockw", z ktrych moe si skada wzorzec URL-a
    # (argument `wzorzec_url` konstruktora importera)
    
    '%(tabela_id)s'             # identyfikator tabeli
    '%(tabela_id_lower)s'       # identyfikator tabeli maymi literami
    '%(tabela_id_upper)s'       # identyfikator tabeli duymi literami
    # [ponisze *nie* w sytuacji, w ktrej jedyna informacja o dacie znajduje si
    # w pobieranym pliku (to zastrzeenie dotyczy wywoa z harmonogramu)]
    '%(rok)d'                   # rok
    '%(rok2)02d'                # ostatnie 2 cyfry roku
    '%(miesiac)d'               # miesic - 1 lub 2 cyfry
    '%(miesiac)02d'             # miesic - 2 cyfry (uzupen. '0')
    '%(dzien)d'                 # dzie - 1 lub 2 cyfry
    '%(dzien)02d'               # dzie - 2 cyfry (uzupen. '0')
    '%(dzien_roku)d'            # dzie roku - 1 do 3 cyfr
    '%(dzien_roku)03d'          # dzie roku - 3 cyfry (uzupen. '0')
    '%(dzien_roboczy_roku)d'    # nr dnia roboczego w roku - 1 do 3 cyfr
    '%(dzien_roboczy_roku)03d'  # nr dnia rob. w roku - 3 cyfry (uzupen. '0')
    '%(data)s'                  # data w formacie 'RRRR-MM-DD'
    '%(sroda_w_roku)d'          # ktra to roda w roku - 1 do 3 cyfr
    '%(sroda_w_roku)03d'        # ktra to roda w roku - 3 cyfry (uzupen. '0')
    # (+ tak samo dla pozostaych dni tygodnia)
    

    # Argumenty dla konstruktora obiektu importera
    # -- takie, jakby byy pobierane z bazy:
    
    argumenty_nbp = dict(
        bank_id = 'NBP',
        wzorzec_url = ('http://www.nbp.pl/kursy/xml/%(tabela_id_lower)s'
                       '%(dzien_roboczy_roku)03dz%(rok2)02d%(miesiac)02d%(dzien)02d.xml'),
        tabele = {
            'A': (u'Tabela A kursw rednich walut obcych', ''),
            'B': (u'Tabela B kursw rednich walut obcych',
                  'http://www.nbp.pl/kursy/xml/%(tabela_id_lower)s'
                  '%(sroda_w_roku)03dz%(rok2)02d%(miesiac)02d%(dzien)02d.xml'),
            'C': (u'Tabela kursw kupna i sprzeday walut obcych', ''),
        },
        struktura_pliku = ('kod_waluty, przelicznik, kurs_sredni,'
                           'kurs_kupna, kurs_sprzedazy, pozycja'),
        parametry = 'tag_nru_tabeli: numer_tabeli'
    )


    # Waciwy skrypt przykadowy:

    testy = (
        # format pliku  argumenty      tabela
        ('XML',         argumenty_nbp, 'A'),
        ('XML',         argumenty_nbp, 'B'),  # (tabela B tylko dla dat rodowych!)
        ('XML',         argumenty_nbp, 'C'),
    )

    print ('\nZaimportujemy tabele kursw dla podanej daty. '
           + ', '.join('Bank: %s, tabela: %s' % (argumenty['bank_id'], tabela_id)
                       for klasa_importera, argumenty, tabela_id in testy))
           
    data = podajDate()
    for format_pliku, argumenty, tabela_id in testy:
        print '\nBank: %s' % argumenty['bank_id']
        print 'Tabela: %s (%s)' % (tabela_id, argumenty['tabele'][tabela_id][0])
        try:
            importer = dajUtworzImporterKursow(format_pliku, **argumenty)
            kursy, nr_tabeli, _ = importer.zaimportuj(tabela_id, data)
        except Blad, exc:
            pass
        else:
            print 'Nr tabeli: %s' % nr_tabeli
            print '\n'.join(map(str, kursy.itervalues()))
