#!/usr/bin/python
# -*- coding: iso8859_2 -*-

#-----------------------------------------------------------------------------
# Name:        rs_print_util.py
# Purpose:     Klasa i zestaw funkcji do tworzenia wydrukw w tym plikw PDF
#
# Author:      Micha Raburski
#
# Created:     2012/01/24
# RCS-ID:      $Id: rs_print_util.py $
# Copyright:   (c) 2012 Record System
# Licence:     GNU GPL
#-----------------------------------------------------------------------------
#2022-06-09 Aleksander Bargie (12989) - Komunikat gdy tworzenie raportu pdf nie powiedzie si

import os
import sys
import tempfile
import string
import time
import rsglobal

from reportlab.pdfgen import canvas
from reportlab.lib.units import inch, cm, mm
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.platypus import SimpleDocTemplate, Paragraph, Spacer, Table, TableStyle, Image, PageBreak
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
from reportlab.rl_config import defaultPageSize
from reportlab.lib import colors, utils
from reportlab.lib.enums import TA_LEFT,TA_RIGHT, TA_CENTER
try:
    from reportlab.graphics.barcode import code39
except:
    pass
from reportlab import Version as ReportLab_Version
from reportlab.graphics.shapes import *
from reportlab.lib.pagesizes import A4, landscape, portrait
import datetime

PAGE_WIDTH, PAGE_HEIGHT = A4
styles = getSampleStyleSheet()
czcionkiPrzygotowane = False

# deklaracja uywanych styli
StyleRL=getSampleStyleSheet()
StyleRL.add(ParagraphStyle(name='NaglDuzy' , fontName='DejaVu-Bold' , fontSize=15, alignment=TA_CENTER))
StyleRL.add(ParagraphStyle(name='Nagl' , fontName='DejaVu-Bold' , fontSize=10, alignment=TA_CENTER))
StyleRL.add(ParagraphStyle(name='Tekst' , fontName='DejaVu' , fontSize=7, alignment=TA_LEFT))
StyleRL.add(ParagraphStyle(name='TekstCenter' , fontName='DejaVu' , fontSize=7, alignment=TA_CENTER))
StyleRL.add(ParagraphStyle(name='TekstDuzy' , fontName='DejaVu' , fontSize=12, alignment=TA_LEFT))
StyleRL.add(ParagraphStyle(name='Liczba' , fontName='DejaVu' , fontSize=7, alignment=TA_RIGHT))
normalnyStyl = StyleRL["Normal"]

Firma=''

#-------------------------------------------------------------------------------

def PFloat(wart,dl,dpp,obetnij=False):
    #--- formatuje wartosc float wg parametrow
    #--- dla obetnij=True zastepuje zera po kropce przez spacje
    wart=rsglobal.RSRound(wart,dpp)
    minus=(wart<0)
    ile=0
    if minus:
        wart=wart*-1
        minus='-'
    else:
        minus=''
    
    if dpp==0:
        tekst = '%'+str(dl)+'d'
        w = string.rjust(tekst % (wart),dl)
    else:
        tekst = '%'+str(dl)+'.'+str(dpp)+'f'
        w = string.rjust(tekst % (wart),dl+dpp)
        if obetnij:
            i = len(w)
            ile=0
            for j in range(i):
                if w[i-j-1]=='.':
                    w = w[0:i-j-1]+' '+w[i-j:i]
                    ile=ile+1
                    break
                if w[i-j-1]<>'0':
                    break
                if w[i-j-1]=='0':
                    w = w[0:i-j-1]+' '+w[i-j:i]
                    ile=ile+1
                    
    w=w.replace(".",",")
    dlug=w.find(",")
    if dlug==-1:
        buf=string.strip(w)
        if obetnij:
            tyl='                       '[0:ile]
        else:
            tyl=''
            
    else:
        buf=string.strip(w[0:dlug])
        if obetnij:
            tyl=string.strip(w[dlug:])+'                       '[0:ile]
        else:
            tyl=string.strip(w[dlug:])
            
    #jest mniej niz 3 znaki - nic nie zmieniamy
    if len(buf)<=3:
        if not obetnij:
            return string.rjust(minus+w.strip(),dl+dpp)
        else:
            return string.rjust(minus+w,dl+dpp)
    
    #wiecej niz 3 znaki - zmieniamy
    pocz=''
    while buf <> '':
        pocz=buf[-3:]+pocz
        buf=buf[0:-3]
        if len(buf)>0:
            pocz="."+pocz
        
    if obetnij:
        wyn=string.rjust((minus+pocz.strip()+tyl),dl+dpp)
    else:
        wyn=string.rjust((minus+pocz+tyl).strip(),dl+dpp)
    return wyn


#-------------------------------------------------------------------------------

def PrzygotujCzcionki():
    global czcionkiPrzygotowane
    if czcionkiPrzygotowane:
        return
    
    for nazwa, sciezka in (
                # DejaVuSerif
                ('DejaVu',                  'fonts/DejaVuSerif.ttf'),
                ('DejaVu-Bold',             'fonts/DejaVuSerif-Bold.ttf'),
                ('DejaVu-Italic',           'fonts/DejaVuSerif-Oblique.ttf'),
                ('DejaVu-BoItalic',         'fonts/DejaVuSerif-BoldOblique.ttf'),
                # QuasiCourier
                ('Qc',                      'fonts/qcrr.ttf'),
                ('Qc-Bold',                 'fonts/qcrb.ttf'),
                ('Qc-Italic',               'fonts/qcrri.ttf'),
                ('Qc-BoItalic',             'fonts/qcrbi.ttf'),
                # DejaVuSansMono
                ('DejaVuSansMono',          'fonts/DejaVuSansMono.ttf'),
                ('DejaVuSansMono-Bold',     'fonts/DejaVuSansMono-Bold.ttf'),
                ('DejaVuSansMono-Italic',   'fonts/DejaVuSansMono-Oblique.ttf'),
                ('DejaVuSansMono-BoItalic', 'fonts/DejaVuSansMono-BoldOblique.ttf'),
                # LiberationMono
                ('LiberationMono',          'fonts/LiberationMono-Regular.ttf'),
                ('LiberationMono-Bold',     'fonts/LiberationMono-Bold.ttf'),
                ('LiberationMono-Italic',   'fonts/LiberationMono-Italic.ttf'),
                ('LiberationMono-BoItalic', 'fonts/LiberationMono-BoldItalic.ttf'),
            ):
        pdfmetrics.registerFont(TTFont(nazwa, sciezka))
    
    if ReportLab_Version >= '2.2':
        pdfmetrics.registerFontFamily('DejaVu',normal='DejaVu',bold='DejaVu-Bold',italic='DejaVu-Italic',boldItalic='DejaVu-BoItalic')
    else:
        from reportlab.lib.fonts import addMapping
        addMapping('DejaVu', 0, 0, 'DejaVu') #normal
        addMapping('DejaVu', 0, 1, 'DejaVu-Italic') #italic
        addMapping('DejaVu', 1, 0, 'DejaVu-Bold') #bold
        addMapping('DejaVu', 1, 1, 'DejaVu-BoItalic') #italic and bold

    czcionkiPrzygotowane = True

#-------------------------------------------------------------------------------

def ZrobPdf(FirmaR,nazwaPliku, docID, tresc, rysujNaPierwszej=None, zostawNaPierszej=0, rysujNaKolejnych=None, ozdobniki=0xFF, opcje={},Landscape = False):
    global Firma
    """
    zapisze do pliku o podanej nazwie pdf zbudowanego z podanej tresci
    @nazwaPliku - cieka do tworzonego pliku
    @docID - tekst identyfikujcy dokument w systemie
    @tresc - lista elementw reportlaba do wstawienia w dokument
    @rysujNaPierwszej - funkcja, moe co dorysowa na pierwszej stronie
    @zostawNaPierszej - dodatkowy odstp do wstawienia na pocztku
    @rysujNaKolejnych - funkcja, jeli podana bdzie dorysowywa na innych
    @ozdobniki - czy w dokumencie umieszcza typowe elementy, dodaj:
         1 - czy wrzuca co na pierwszej stronie
         2 - czy wrzuca co na nastpne strony
         4 - czy wpisa numery stron
         8 - czy umieszcza kod kreskowy
        16 - czy wpisywa id dokumentu tekstem
        32 - czy wpisywa dat i godzin
        64 - czy wpisywac dat przy stopce
       128 - czy wpisywac sama stopke
     """
    Firma=FirmaR
    if Landscape:
        opcje['pagesize']=landscape(A4)
        PAGE_WIDTH, PAGE_HEIGHT = landscape(A4)
    else:
        PAGE_WIDTH, PAGE_HEIGHT = A4

    def ozdobStrone(canvas, doc):
        # numerowanie stron jest w NumberedCanvas.save
        canvas.saveState()
        if ozdobniki & 8:
            # kod kreskowy
            pageID = "AD%s" % docID
            barcode=code39.Standard39(pageID, barWidth=0.423*mm, barHeight=8*mm, ratio=0.5)
            barcode.drawOn(canvas, 0.5 * inch, 0.6 * inch)
        foot = "Dokument wygenerowany przez system Anakonda, firma: %s" % Firma

        if rsglobal.SprawdzPolskieZnaki(Firma):
            canvas.setFont("DejaVu", 8)
        else:
            canvas.setFont("Helvetica", 8)

        if ozdobniki & 16:
            # id dokumentu
            foot += ": %s" % docID
            canvas.drawCentredString(PAGE_WIDTH / 2, 0.50 * inch, foot)
        if ozdobniki & 32:
            # data i godzina
            czas = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            canvas.drawRightString(PAGE_WIDTH - 0.5 * inch, PAGE_HEIGHT - 0.55 * inch, czas)
        if ozdobniki & 64 and not ozdobniki & 16:
            data = datetime.datetime.now().strftime("%Y-%m-%d")
            foot += " %s" % data
            canvas.drawCentredString(PAGE_WIDTH / 2, 0.50 * inch, foot)
        if ozdobniki & 128:
            # stopka dokumentu
            foot = "Dokument wygenerowany przez system Anakonda."
            canvas.drawCentredString(PAGE_WIDTH / 2, 0.50 * inch, foot)
        canvas.restoreState()

    def ozdobPierwszaStrone(canvas, doc):
        canvas.setAuthor("Program Anakonda")
        canvas.setTitle("Dokument %s" % docID)
        if rysujNaPierwszej:
            canvas.saveState()
            rysujNaPierwszej(canvas, doc)
            canvas.restoreState()
        if not ozdobniki & 1:
            return
        ozdobStrone(canvas, doc)

    def ozdobKolejneStrony(canvas, doc):
        if rysujNaKolejnych:
            canvas.saveState()
            rysujNaKolejnych(canvas, doc)
            canvas.restoreState()
        if not ozdobniki & 2:
            return
        ozdobStrone(canvas, doc)
        canvas.saveState()

    PrzygotujCzcionki()
    doc = SimpleDocTemplate(nazwaPliku, **opcje)

    if zostawNaPierszej:
        tresc = [Spacer(1, zostawNaPierszej)] + tresc

    if ozdobniki & 4:
        canvasmaker = NumberedCanvas
    else:
        canvasmaker = canvas.Canvas
    #if Landscape:
    #    canvasmaker.setPageSize(landscape(A4))
    try:
        doc.build(tresc, onFirstPage=ozdobPierwszaStrone, onLaterPages=ozdobKolejneStrony, canvasmaker=canvasmaker)
    except:
        #AB 13.07.22r dodany return OK/Blad
        return "BLAD"
    return 'OK'

#--#############################################################################
#-- KLASA NumberedCanvas
#--#############################################################################

#sposb na dodawanie numeru i iloci stron oraz loga w nagwku, http://stackoverflow.com/questions/1087495/reportlab-page-x-of-y-numberedcanvas-and-images
class NumberedCanvas(canvas.Canvas):
    def __init__(self, *args, **kwargs):
        canvas.Canvas.__init__(self, *args, **kwargs)
        self._saved_page_states = []

    def showPage(self):
        self._saved_page_states.append(dict(self.__dict__))
        self._startPage()

    def save(self):
        """add page info to each page (page x of y)"""
        num_pages = len(self._saved_page_states)
        for state in self._saved_page_states:
            self.__dict__.update(state)
            self.draw_page_number(num_pages)
            canvas.Canvas.showPage(self)
        canvas.Canvas.save(self)

    def draw_page_number(self, page_count):
        self.setFont("Helvetica", 8)
        self.drawRightString(PAGE_WIDTH - 0.1 * inch, 0.1 * inch,
            "Strona %d / %d" % (self._pageNumber, page_count))

#-------------------------------------------------------------------------------

def Paragraf(Tekst,styl='Tekst'):
    return Paragraph(Tekst,StyleRL[styl])
#-------------------------------------------------------------------------------

def Pagebreak():
    return PageBreak()

#-------------------------------------------------------------------------------

# funkcja pomocnicza przekazywana dalej jako parametr do tworzenia PDFa
# na razie nie zaburzam struktury, zwraca tekst "dnia 2011-12-12"
def ZrobNaglowekReportlab(canvas, doc):
    canvas.setFont("DejaVu", 8)
