#!/usr/bin/python
# -*- coding: iso8859_2 -*-

#-----------------------------------------------------------------------------
# Name:        rs_process_killer.py
# Purpose:     skrypt/modu z funkcj do killowania procesw
#
# Authors:     Jan Kaliszewski
# Created:     2009/06/22
# RCS-ID:      $Id: rs_process_killer.py $
# Copyright:   (c) 2009 Record System
# Licence:     GNU GPL
#-----------------------------------------------------------------------------

import fcntl
import os
import os.path
import signal
import time
import traceback

import const


SCIEZKA_PLIKU_BLOKADY = os.path.abspath(const.NAZWA_BLOKADY_RELOADU)


def zalozBlokade():
    u'Za niewyczn blokad reloadu; poczekaj jeli reload ju trwa.'

    deskryptor_pliku_blokady = os.open(SCIEZKA_PLIKU_BLOKADY,
                                       os.O_RDONLY    # <- otwrz do odczytu
                                       | os.O_CREAT,  # <- utwrz, jeli nie istnieje
                                       0744)
    fcntl.flock(deskryptor_pliku_blokady, fcntl.LOCK_SH)  # zablokuj/czekaj
    return deskryptor_pliku_blokady


def zdejmijBlokade(deskryptor_pliku_blokady):
    u'Zdejmij blokad reloadu'

    fcntl.flock(deskryptor_pliku_blokady, fcntl.LOCK_UN)  # odblokuj
    os.close(deskryptor_pliku_blokady)


def kill(pid, sygnaly, opoznienie, odstep,
         wypisz, wypisz_blad=None, bez_blokady=False):
    u'Killuj proces kolejnymi sygnaami do skutku lub uycia ich wszystkich.'

    if wypisz_blad is None:
        wypisz_blad = wypisz
        
    if bez_blokady:
        wypisz('* rs_process_killer.kill(): zaczynam,'
               ' bez blokady reloadu')
    else:
        wypisz('* rs_process_killer.kill(): zaczynam,'
               ' zakadam blokad reloadu')
        deskryptor_pliku_blokady = zalozBlokade()
        
    try:
        time.sleep(opoznienie)
        for sygnal in sygnaly:
            wypisz('* rs_process_killer.kill(): bdziemy'
                   ' killowac proces %s sygnaem %s'
                   % (pid, sygnal))

            # killujemy:
            try:
                os.kill(pid, sygnal)
            except OSError, exc:
                wypisz_blad('* rs_process_killer.kill(): bd'
                       ' OSError przy prbie killowania'
                       ' procesu %s sygnaem %s ("%s");'
                       ' moliwe, e po prostu proces sam'
                       ' przedtem zdy si zakoczy'
                       % (pid, sygnal, exc))
            time.sleep(odstep)

            # sprawdzamy, czy rzeczywicie si udao:
            try:
                os.getsid(pid)
            except OSError, exc:
                if exc.errno == 3:  # "No such process"
                    wypisz('* rs_process_killer.kill():'
                           ' OK, proces %s nie istnieje'
                           % pid)
                    return True
                else:
                    wypisz_blad('* rs_process_killer.kill():'
                           ' nieoczekiwany bd przy prbie killowania'
                           ' procesu %s sygnaem %s: %s'
                           % (pid, sygnal, traceback.format_exc()))
                    raise

        else:
            wypisz_blad('* rs_process_killer.kill(): nie udao'
                    ' si skillowac procesu %s'
                    % pid)
            return False
    finally:
        if bez_blokady:
            wypisz('* rs_process_killer.kill(): skoczyem')
        else:
            zdejmijBlokade(deskryptor_pliku_blokady)
            wypisz('* rs_process_killer.kill(): skoczyem,'
                   ' blokada reloadu zdjta')


if __name__ == '__main__':

    import optparse
    import sys

    parser = optparse.OptionParser(usage='usage: %prog options | --help')
    parser.add_option('-p', '--pid', dest='pid')
    parser.add_option('-s', '--signals', dest='signals', default='10,15,9')
    parser.add_option('-d', '--predelay', dest='predelay', default='3')
    parser.add_option('-i', '--interwal', dest='interwal', default='3')
    parser.add_option('-l', '--logfile', dest='logfile')
    parser.add_option('-q', '--quiet', action='store_true', dest='quiet', default=False)
    opcje, argumenty = parser.parse_args()

    if opcje.pid:
        pid = int(opcje.pid)
    else:
        raise parser.error('pid?')

    if opcje.signals:
        sygnaly = [int(sig)
                   for sig in opcje.signals.split(',')]
    else:
        raise parser.error('signals?')

    if opcje.predelay:
        opoznienie = float(opcje.predelay)
    else:
        raise parser.error('predelay?')

    if opcje.interwal:
        odstep = float(opcje.interwal)
    else:
        raise parser.error('interwal?')

    if opcje.logfile and opcje.logfile != '-':
        plik = open(opcje.logfile, 'a')
    else:
        plik = sys.stdout

    try:
        if opcje.quiet:
            def wypisz(tekst):
                pass
        else:
            def wypisz(tekst):
                print >>plik, (
                      '%s [%s] %s' % (time.strftime('%Y.%m.%d %H:%M.%S'),
                                      os.getpid(),
                                      tekst)
                )

        kill(pid, sygnaly, opoznienie, odstep, wypisz)
    finally:
        if plik is not sys.stdout:
            plik.close()

