# -*- coding: iso8859_2 -*-
#-----------------------------------------------------------------------------
# Name:        rsglobal.py
# Purpose:     funkcje wspolne dla rs_anakonda_server i rs_anakonda_service
#
# Author:      Michal Raburski
#
# Created:     2004/03/06
# RCS-ID:      $Id: rsglobal.py $
# Copyright:   (c) 2002 Record System
# Licence:     GNU GPL
#-----------------------------------------------------------------------------
# Jan Kaliszewski - 2009-05-28:
# * napisane od nowa: WyslijPakietami(), OdbierzPakietami()
# Jan Kaliszewski - 2009-06-17/23/25:
# * ZapiszDoLogow() info o id procesu, poprawka collections.deque dla Py2.4
# Jan Kaliszewski - 2010-04-25:
# * klasy pomocnicze: RSDrzewoPlikow, RSPlik, RSSlownikWPliku
# Mariusz Rajfura - 2011-08-22:
# * funkcja PodajPola() - zwraca liste pol z podanej tabeli
# Mariusz Rajfura - 2013-05-09 (6860)
# - dodanie funkcji CzyPrawidlowyLogin(Login)
# - dodanie funkcji CzyPrawidloweHaslo(Haslo)
# Aleksander Bargie - 2018-12-04 (11305) - konwersja hasa w wysyaniu maili na string, bo w poczeniu SSL wyrzucao bdy
# Aleksander Bargie - 2019-04-10 (11463) - Dla operatora 0(anakonda) zawsze zapisywa do logw
# Aleksander Bargie - 2019-06-12 (11505) - Dodany timeout do wysyania maili
# Aleksander Bargie - 2019-11-21 (11744) - Metoda sprawdzajca numer konta kontrahenta na biaej licie
# Aleksander Bargie - 2019-11-22 (11754) - Metoda sprawdzajca poprawnosc numeru konta
# Wojciech Skowron   - 2019-11-28 (11751) - SprawdzCzyToGodzina
# Jan Raburski       - 2020-01-06 (11826) - ZmienNaNULL
# Aleksander Bargie - 2020-12-30 (12235) - Pin na Dowodach Dostaw(GenerujPIN)
# Aleksander Bargie - 2021-08-12 (12596) - obsuy bd serwera
# Wojciech Skowron   - 2021-10-20 (12174) - poprawiona obsuga pliku w RSWyslijEmail
# Wojciech Skowron   - 2022-05-05 (12987) - zamiana gmtime() na localtime() w PodajDzis()
# Aleksander Bargie - 2022-06-15 (13059) - Bd przy dodawaniu kontrahenta
# Aleksander Bargie - 2022-07-19 (13100) - Reklamacje klientw - prawdopodobnie po wysaniu odpowiedzi ostatecznej  okno si psuje
# Wojciech Skowron   - 2022-08-05 (13103) - Obsuga bdu gdy adres e-mail nieprawidowy w RSWyslijEmail, przeniesienie SprawdzAdresEmail z fs_powiadomienia
# Aleksander Bargie - 2022-08-26 (13134) - Bd sprawdzania certyfikatu przy sprawdzaniu biaej listy przy tworzeniu pliku pli
# Aleksander Bargie - 2023-01-02 (13287) - Wysyanie smsw o blokadzie dostaw
# Jan Raburski       - 2023-01-15 (12893) - Eliminacja ' i \ w rseval
# Aleksander Bargie - 2023-03-30 (13376) - Wysyka SMS za porednictwem modemu UART
# Wojciech Skowron   - 2023-11-29 (13532) - TransformujZXSL - transformacja xml za pomoc xsl

import collections
import time

import const
from const import *
import cPickle
from datetime import timedelta, date
import datetime
import email
from email.Header import Header
from email.MIMEBase import MIMEBase
from email.MIMEMultipart import MIMEMultipart
from email.MIMEText import MIMEText
from socket import error as SocketError  #AB 12.08.21r

import errno
import fcntl
import hashlib
import itertools
from lxml import etree
import os
import os.path
import re
try:
    import requests
except:
    pass
import shelve
import smtplib
import socket
import string
import sys
import textwrap
from time import *
import traceback
import types

LogLevel = 5
LogType = 'P'
LogFile = 'rs-and.log'
Oper = ''
DomyslnyNadawca = None
SerwerSMTP = 'localhost'
ListaStawekZn=['np','zw','oo','0E','0P']

#-------------------------------------------------------------------------------

# w Py2.4 collections.deque nie ma metody remove(), dodajemy j:

wersja_py, podwersja_py = sys.version_info[0:2]
if wersja_py <= 2 and podwersja_py <= 4:
    class _deque(collections.deque):
        def remove(self, value):
            for i, wartosc in enumerate(self):
                if wartosc == value:
                    del self[i]
                    return
            else:
                raise ValueError('deque.remove(x): x not in deque')
    collections.deque = _deque

#-------------------------------------------------------------------------------

# funkcja pomocnicza, ktra sprawdza czy w kluczach dla odpowiedniego numeru funkcji s
# odpowiednie parametry. Jeli nie to zwraca ich list

def SprawdzParametryWKluczach(SlPar,Fun,klucze):
    # struktury i przykadowe wywoanie w fs_kontrahenci_notatki
    Param=''
    for kl in SlPar:
        if Fun in SlPar[kl]:
            if not kl in klucze:
                Param+=kl+','

    return Param.strip(',')

#-------------------------------------------------------------------------------

def WyswietlBladZeSlownika(stat,SBlady):
    sep_pos = stat.find("$")
    if sep_pos < 0:
        sep_pos = len(stat)
    eCode = stat[:sep_pos]
    if eCode in SBlady:
        eArg = stat[sep_pos+1:]
        eMsg = SBlady[eCode][0]
        return eMsg.replace("$", eArg)

    return stat
    
#-------------------------------------------------------------------------------
    
def RSRound(liczba,dokladnosc=0,poprawka=1e-8):
    if liczba <0:
        wynik=round(liczba-poprawka,dokladnosc)
    else:
        wynik=round(liczba+poprawka,dokladnosc)
        
    # to suy do zmiany -0 na 0
    if wynik==0:
        wynik=0
    return wynik

#-------------------------------------------------------------------------------

def UsunKropkaZero(Liczba):
    wyn=str(Liczba)
    if wyn.find('.')>=0:
        ogon=wyn[wyn.find('.')+1:]
        for z in ogon :
            if z != '0':
                return wyn
            
        wyn=wyn[:wyn.find('.')]
    
    return wyn

#-------------------------------------------------------------------------------

def ObrobTekst(Tekst):
    if Tekst in ('',None,'None','NULL'):
        return 'NULL'
    else:
        return "'"+Dekoduj(Tekst)+"'"

#-------------------------------------------------------------------------------

#zwraca napis "TRUE" albo "FALSE" w zaleznosci od podanej wartosci
def ObrobBool(wart):
    if wart:
        return 'TRUE'
    else:
        return 'FALSE'
    
#-------------------------------------------------------------------------------
#AB 15.06.22r
def Dekoduj(tekst):
    # if str(type(tekst))=="<type 'unicode'>":
    #     nowy_tekst = ''
    #     for znak in tekst:
    #         try:
    #             nowy_tekst += znak.encode('iso8859_2')
    #         except:
    #             pass
    #     return nowy_tekst
    # else:
    #     return tekst
    if str(type(tekst)) == "<type 'unicode'>":
        tekst=tekst.replace(u"\u2019", u"`").replace(u'\u201e', u'"').replace(u'\u2013', u'-').replace(u'\u2010', u'-'). \
            replace(u'\u2011', u'-').replace(u'\u2012', u'-').replace(u'\u2014', u'-').replace(u'\u2015', u'-') \
            .replace(u'\u2212', u'-')  #AB 19.07.22
        return tekst.encode('iso8859_2', 'replace')
    else:
        return tekst

#-------------------------------------------------------------------------------

#AB 05.06.19r
def DekodujCP852(tekst):
    # return tekst.encode('cp852')
    nowy_tekst = ''
    for znak in tekst:
        try:
            nowy_tekst += znak.encode('cp852')
        except:
            pass
    return nowy_tekst

#-------------------------------------------------------------------------------

#AB 05.06.19r
def DekodujRS(tekst):
    # return Dekoduj(tekst.decode('cp852'))
    nowy_tekst = ''
    for znak in tekst:
        try:
            nowy_tekst += znak.decode('cp852')
        except:
            pass
    return Dekoduj(nowy_tekst)

#-------------------------------------------------------------------------------

def ZmienKodowanie(tekst, strona_kodowa):
    tekst=tekst.strip().replace('\'','`').replace('"','').replace('\\','/')
    if strona_kodowa.upper()<>'ISO8859-2' and strona_kodowa<>'' and strona_kodowa<>'None':
        tekst=tekst.encode('iso8859-2').decode(strona_kodowa)
    return tekst

#-------------------------------------------------------------------------------

def RSTextWrap(Tekst='',Szer=75):
    LWynik = []
    LSplits = Tekst.split('\n')
    for s in LSplits:
        if s == '':
            LWynik.append('')
        else:
            sw = textwrap.wrap(s,Szer)
            LWynik.extend(sw)
        
    return LWynik

#-------------------------------------------------------------------------------

#Zamienia \' na '
def UsSl(tekst):
    wynik=tekst.replace("\\\\\'","'")
    wynik=wynik.replace("\\\'","'")
    return wynik
    
#---------------------------------------
#--- funkcja pobierajca dane konfiguracyjne z pliku

def PodajZKonf(plik,klucze):
    i = 0
    powrot = []
    dict = {}
    try:
        konf=open(plik)
        linie = konf.readlines()
        for linia in linie:
            gdzie = string.find(linia, "=")
            if gdzie == -1:
                continue
            podzial = string.split(linia, "=", 2)  
            if len(podzial) > 2:
                continue
            klucz = podzial[0].strip()
            wartosc = podzial[1].strip()
            dict[klucz] = wartosc

        for klucz in klucze:
            powrot.append('')
            if dict.has_key(klucz):
                powrot[i] = dict[klucz]
            else:
                powrot[i]=''
            i = i+1
            
        konf.close()
    except Exception, e:
      ZapiszDoLogow('Blad otwarcia pliku konfiguracyjnego: '+str(e)+'. Plik: '+str(plik),1)
      sys.exit(0)
    return powrot

#---------------------------------------
#--- pliklog - plik logw,tekst -komunikat, loglevel - aktualny poziom logowania, inflevel - poziom informacji,
# writelevel - 'T' terminal,'P' do pliku, suffix - moliwo wybrania wasnej kocwki nazwy pliku


def ZapiszDoLogow(tekst, inflevel=5, suffix=None):
    global LogLevel, LogFile,LogType, Oper, _blokada_logowania
    try:                        #AB 10.04.19r
        if inflevel < LogLevel+1 or Oper == '0':
            if LogType == 'T':
                print tekst
            elif LogType == 'P':
                if suffix is not None:
                    plik = open(LogFile+suffix, 'a')
                elif Oper != "":
                    plik = open(LogFile+'-'+Oper,'a')
                else:
                    plik = open(LogFile,'a')
                try:  #AB 15.06.22r  zapis unicode, z encode backslashreplace
                    out = u""
                    out += unicode(strftime('%Y.%m.%d %H:%M.%S'))
                    out += u' [%s] ' % os.getpid()
                    if Oper != "":
                        out += unicode(Oper) + u' '
                    out += unicode(tekst)+ u'\n'
                    plik.write(out.encode('iso8859-2', 'backslashreplace'))
                except:
                    traceback.print_exc()
                finally:
                    plik.close()
        return 1
    except Exception,e:
        traceback.print_exc()
        print 'Bd zapisu do logw :',e,' tekstu ',tekst
        return 0 

#-------------------------------------------------------------------------------

def ZapiszDoPliku(tekst,Plik): 
    global Oper
    otwarty = False
    try:
        plik = open(Plik,'a')
        otwarty = True
        if Oper != "":
            plik.write(strftime('%Y.%m.%d %H:%M.%S')+' '+Oper+' '+str(tekst)+'\n')
        else:
            plik.write(strftime('%Y.%m.%d %H:%M.%S')+' '+str(tekst)+'\n')
        plik.close()
        otwarty = False
        return 1
    except Exception,e:
        if otwarty :
            plik.close()
        print 'Bd zapisu do pliku :',e,' do pliku',Plik,'tekstu',tekst 
        return 0 

#---------------------------------------

# Pomocnicza funkcja dla WyslijPakietami() i OdbierzPakietami()
# do skrtowej prezentacji danych w logach (by nie marnowa
# zasobw i nie zapycha logw)

def zrobSkrot(dane):
    obcieto_dane = False
    try:
        # by moe dane s sekwencj -- jeeli tak,
        # zajmijmy si od razu tylko ich maym kawakiem:
        if len(dane) > 5:
            dane = dane[:5]
            obcieto_dane = True
    except Exception:
        # (jednak nie s sekwencj lub nie poddaj si wykrojeniom)
        pass

    # zamieniamy na napis, nie duszy ni 50 znakw:
    napis = str(dane)
    if len(napis) > 50:
        napis = napis[:50]
        obcieto_dane = True

    if obcieto_dane:
        return napis + ' [... etc., obcite na potrzeby logw]'
    else:
        return napis
    
#---------------------------------------

# Nazwa tej funkcji jest troch zaszoci historyczn.
# Obecnie jedyny sens dzielenia wikszych porcji danych
# na "pakiety" wie si z potrzeb wywietlania paska
# postpu w aplikacji klienta.

def WyslijPakietami(conn, wielkosc_pakietu, dane):
    "Wysyamy potencjalnie du ilo danych."

    liczba_pakietow = '[jeszcze nie ustalono]'
    liczba_wyslanych = 0

    try:
        ZapiszDoLogow('Bdziemy wysya dane "pakietami". Najpiew'
                      ' je marynujemy (do postaci napisu)...', 5)
        co_wyslac = cPickle.dumps(dane)
        ZapiszDoLogow('...zamarynowane. Teraz ustalimy i wylemy'
                      ' informacje o dugoci danych', 5)

        skrot_danych = zrobSkrot(dane)

        # ustalamy informacje o iloci danych:
        calkowita_dlugosc = len(co_wyslac)
        liczba_rownych_pakietow, reszta = divmod(calkowita_dlugosc, wielkosc_pakietu)
        if reszta > 0:
            liczba_pakietow = liczba_rownych_pakietow + 1
            wielkosc_ostatniego = reszta
        else:
            liczba_pakietow = liczba_rownych_pakietow
            wielkosc_ostatniego = wielkosc_pakietu

        # ...i wysyamy te informacje:
        conn.send(str(liczba_pakietow))
        conn.send(str(wielkosc_ostatniego))

        # wybieramy nowy lub stary sposb przesyania, w zalenoci
        # od ustawionej (w konfiguracji klienta) wielkoci pakietu:
        if wielkosc_pakietu > LIMIT_WIELKOSCI_STAREGO_PAKIETU:
            # nowy sposb -- dane id jednym cigiem:
            ZapiszDoLogow('Rozpoczynamy "wysyk pakietami" nowym sposobem'
                          ' (liczba pakietw: %s, wielko pakietu: %s,'
                          ' wielko ostatniego: %s'
                          ' => cakowita dugo danych: %s) -- dane: %s'
                          % (liczba_pakietow, wielkosc_pakietu,
                             wielkosc_ostatniego, calkowita_dlugosc,
                             skrot_danych), 4)
            # wysyamy od razu cao:
            conn.wyslij_raz(co_wyslac)
            liczba_wyslanych = liczba_pakietow
        else:
            # stary (nieoptymalny) sposb -- z uyciem .send() dla kadego pakietu:
            ZapiszDoLogow('Rozpoczynamy "wysyk pakietami" starym sposobem'
                          ' (liczba pakietw: %s, wielko pakietu: %s,'
                          ' wielko ostatniego: %s'
                          ' => cakowita dugo danych: %s) -- dane: %s'
                          % (liczba_pakietow, wielkosc_pakietu,
                             wielkosc_ostatniego, calkowita_dlugosc,
                             skrot_danych), 4)
            ile_razem_poszlo = 0
            for liczba_wyslanych in xrange(liczba_pakietow):
                indeks = liczba_wyslanych * wielkosc_pakietu
                # (w przypadku ostatniego mniejszego pakietu -- wykrojenie
                # przekraczajce dugo napisu nie spowoduje bdu)
                pakiet = co_wyslac[indeks : indeks + wielkosc_pakietu]
                # wysyamy:
                conn.send(pakiet)
                ile_razem_poszlo += len(pakiet)
            else:
                liczba_wyslanych = liczba_pakietow

            # (sprawdzenie w celach debuggingowych)
            assert ile_razem_poszlo == calkowita_dlugosc, ('Wyglda,'
                  ' jakby ilo wysanych danych (%s) bya rna od'
                  ' waciwej iloci danych, ktre byy wysyane (%s)'
                  % (ile_razem_poszlo, calkowita_dlugosc))

    except Exception:
        ZapiszDoLogow('BD wysyania pakietami (wysano %s z %s pakietw): %s'
                      % (liczba_wyslanych, liczba_pakietow,
                         traceback.format_exc()), 1)
        return 1  # (<- tak byo, nie zmieniam - JK) [odwrotnie ni w kliencie]

    else:
        ZapiszDoLogow('Koniec wysyania... Podsumowanie: wysano'
                      ' wszystkie %s pakietw' % liczba_wyslanych, 4)
        return 0  # (<- tak byo, nie zmieniam - JK) [odwrotnie ni w kliencie]

#---------------------------------------

# Nazwa tej funkcji jest troch zaszoci historyczn.
# Obecnie jedyny sens dzielenia wikszych porcji danych
# na "pakiety" wie si z potrzeb wywietlania paska
# postpu w aplikacji klienta.

def OdbierzPakietami(conn, wielkosc_pakietu):
    "Odbieramy potencjalnie du ilo danych."

    liczba_pakietow = '[jeszcze nie pobrano informacji]'
    liczba_odebranych = 0

    try:
        ZapiszDoLogow('Bdziemy odbiera dane pakietami... Najpierw'
                      ' pobierzemy informacje o dugoci danych', 5)

        # odbieramy infromacje o iloci danych:
        liczba_pakietow = int(conn.recv())
        wielkosc_ostatniego = int(conn.recv())

        calkowita_dlugosc = (wielkosc_pakietu * (liczba_pakietow - 1)
                             + wielkosc_ostatniego)

        # wybieramy nowy lub stary sposb przesyania, w zalenoci
        # od ustawionej (w konfiguracji klienta) wielkoci pakietu:
        if wielkosc_pakietu > LIMIT_WIELKOSCI_STAREGO_PAKIETU:
            # nowy sposb -- dane id jednym cigiem:
            ZapiszDoLogow('Rozpoczynamy "odbir pakietami" nowym sposobem'
                          ' (liczba pakietw: %s, wielko pakietu: %s,'
                          ' wielko ostatniego: %s'
                          ' => cakowita dugo danych: %s)'
                          % (liczba_pakietow, wielkosc_pakietu,
                             wielkosc_ostatniego, calkowita_dlugosc), 4)
            # odbieramy od razu cao:
            odebrane = conn.odbierz_raz(calkowita_dlugosc)
            liczba_odebranych = liczba_pakietow
        else:
            # stary (nieoptymalny) sposb -- z uyciem .recv() dla kadego pakietu:
            ZapiszDoLogow('Rozpoczynamy "odbir pakietami" starym sposobem'
                          ' (liczba pakietw: %s, wielko pakietu: %s,'
                          ' wielko ostatniego: %s'
                          ' => cakowita dugo danych: %s)'
                          % (liczba_pakietow, wielkosc_pakietu,
                             wielkosc_ostatniego, calkowita_dlugosc), 4)
            odebrane = []
            for liczba_odebranych in xrange(liczba_pakietow):
                # odbieramy
                odebrane.append(conn.recv())
            else:
                liczba_odebranych = liczba_pakietow

            # przerabiamy list na napis:
            ZapiszDoLogow('Koniec przesyania przez sie kolejnymi .recv(),'
                          ' teraz zamienimy list na napis', 5)
            odebrane = ''.join(odebrane)

            # (sprawdzenie w celach debuggingowych)
            assert len(odebrane) == calkowita_dlugosc, ('Realna ilo'
                  ' odebranych danych (%s) jest rna od iloci (%s)'
                  ' wynikajcej z zadeklarowanej liczby i wielkoci pakietw'
                  % (len(odebrane), calkowita_dlugosc))

        ZapiszDoLogow('By dokoczy dziea, odmarynujemy dane z napisu.', 5)
        dane = cPickle.loads(odebrane)
        skrot_danych = zrobSkrot(dane)
    except SocketError, exc:  #AB 12.08.21r
        # skopiowane z oglnego Exception, interesuje nas tylko specjalna obsuga ECONNRESET
        if exc.errno != errno.ECONNRESET:
            ZapiszDoLogow('BD odbierania pakietami (po odebraniu'
                          ' %s z %s pakietw): %s' % (liczba_odebranych,
                                                      liczba_pakietow, traceback.format_exc()), 1)

            # tak byo, nie zmieniam (z wyj. treci komunikatu) - JK:
            try:
                conn.shutdown(2)
            except socket.error, exc:
                ZapiszDoLogow('BD przy prbie zamknicia poczenia: '
                              + traceback.format_exc(), 1)
        else:
            #  ECONNRESET odpowiada za zerwanie poczenia np. poprzez zabicie procesu Anakondy, dostajemy sygna
            #  o rozczeniu, ale nie jest to normalne zakoczenie poczenia. Nie ma potrzeby zamykania gniazda.
            ZapiszDoLogow('Przymusowe wyjcie', 1)
            sys.exit(1)
    except Exception:
        ZapiszDoLogow('BD odbierania pakietami (po odebraniu'
                      ' %s z %s pakietw): %s' % (liczba_odebranych,
                      liczba_pakietow, traceback.format_exc()), 1)

        # tak byo, nie zmieniam (z wyj. treci komunikatu) - JK:
        try:
            conn.shutdown(2)
        except socket.error, exc:
            ZapiszDoLogow('BD przy prbie zamknicia poczenia: '
                          + traceback.format_exc(), 1)
        ZapiszDoLogow('Przymusowe wyjcie', 1)
        sys.exit(1)

    else:
        # jeeli wszystko poszo dobrze:
        ZapiszDoLogow('Podsumowanie: odebrano wszystkie %s'
                      ' pakietw, zawierajacych dane: %s'
                      % (liczba_odebranych, skrot_danych), 4)
        return dane

#---------------------------------------
       
def encodeLength(string):
  return encodeLengthAdvanced(string, TCP_LEN_LEN)

#---------------------------------------

def encodeLengthAdvanced(string, targetLen):
  realLen = len(string)
  lengthStr = str(realLen)
  length = len(lengthStr)
  
  if (length > targetLen):
    ZapiszDoLogow("Zbyt dlugi komentarz - nalezy zwiekszyc wartosc stalej TCP_LEN_LEN",1)
    return
 
  strLen = "0"*(targetLen - length)+lengthStr
  return strLen

#-------------------------------------------------------------------------------

def PorownajGodziny(value1,value2): # Pawe Wjcikowski
    #  funkcja porwnuje 2 wartoci godzinowe
    temp1=str(value1)   #zamieniamy na stringi zeby mozna bylo obciac znaki
    temp2=str(value2)
    
    godz1=temp1[:2] # przypisujemy godziny do zmiennej
    min1=temp1[3:5] # przypisujemy minuty do zmiennej

    godz2=temp2[:2] #j.w
    min2=temp2[3:5] #
    
    if (int(godz1)>int(godz2)):
        return False            # godzina rozpoczecia jest wieksza od godz zakonczenia
    elif (int(godz1)==int(godz2)):
        if (int(min1)==int(min2)):
            return False        # godziny s takie same
        elif (int(min1)>int(min2)):
            return False        # godzina taka sama ale minuty pozniejsze
    return True    # wszystko OK / godz 1 jest mniejsza od godz 2/               
       
#-------------------------------------------------------------------------------

def SprCzyToGodzina(value): # Pawe Wjcikowski
    #  funkcja sprawdza czy podany cig znakw jest ma poprawny format godziny
    temp=str(value).strip()     # uzupelnia o 2 puste pola      
    if len(temp)<>5: 
        return False                # bdna dugo 
    elif temp[2]<>':':
        return 4    
    else:
        if not (temp[:2].isdigit() or temp[3:5].isdigit()):
            return False            # skada sie nie tylko z cyfr
        elif (int(temp[:2])>23) or (int(temp[3:5])>59):
            return False            # niepoprawny format  
    return True    # wszystko OK                     

#-------------------------------------------------------------------------------

def PodajOstatniDzien(data): # P.W - funkcja podaje ostatni dzien miesiaca majac date ( wczesniej spr. date )
    #spr. ostatni dzien       
    data=data.strip()
    rok = int(data[0:4])
    mies = int(data[5:7])
    ilosc_dni = 0
    Przestepny = (rok % 4 == 0) and ((rok % 100 != 0) or (rok % 400 == 0))
    if Przestepny:
        Dni=[31,29,31,30,31,30,31,31,30,31,30,31]
    else:
        Dni=[31,28,31,30,31,30,31,31,30,31,30,31]
    ilosc_dni = Dni[mies-1]  
    return ilosc_dni

#-------------------------------------------------------------------------------

def SprawdzCzyToData(data):
    try:
        data=data.strip()
        Rok = int(data[0:4])
        Mc = int(data[5:7])
        Dzien = int(data[8:10])

        if Rok < 1900:
            return False
        if Rok > 2500:
            return False
        #Czy prawidowy miesic
        if (Mc < 1) or (Mc > 12):
            return False
        
        #Czy rok przest\u0119pny
        Przestepny = (Rok % 4 == 0) and ((Rok % 100 != 0) or (Rok % 400 == 0))

        if Przestepny:
            Dni=[31,29,31,30,31,30,31,31,30,31,30,32]
        else:
            Dni=[31,28,31,30,31,30,31,31,30,31,30,32]

        OK=((Dzien >= 1) and (Dzien <= Dni[Mc-1]))
        return OK
    
    except Exception,e:
        
        return False

#-------------------------------------------------------------------------------

#WS 2019-11-27
def SprawdzCzyToGodzina(godzina):
    try:
        Godzina = int(godzina[0:2])
        Minuta = int(godzina[3:5])
        Sekunda = 0
        if len(godzina) > 5:
            Sekunda = int(godzina[6:8])

        if int(Godzina) < 0 or int(Godzina) > 23:
            return False
        if int(Minuta) < 0 or int(Minuta) > 59:
            return False
        if int(Sekunda) < 0 or int(Sekunda) > 59:
            return False
    except Exception:
        return False
    else:
        return True

#-------------------------------------------------------------------------------

def PrzesunDateODni(Data, IleDni):

    if not SprawdzCzyToData(Data):
        return '    -  -  '

    Rok = int(Data[0:4])
    Mies = int(Data[5:7])
    Dzien = int(Data[8:10])

    DataP = date(Rok, Mies, Dzien) + timedelta(days=IleDni)
    return str(DataP.year)+'-'+str(DataP.month).zfill(2)+'-'+str(DataP.day).zfill(2)

#---------------------------------------
    # na podstawie daty podaje jaki to dzien tygodnia
def PodajJakiToDzienTygodnia(data):
    data=data.strip()
    R = int(data[0:4])
    M = int(data[5:7])
    D = int(data[8:10])   
        
    if not SprawdzCzyToData(data):
        return False
    
    Data = date(R, M, D)  # podajemy rok, miesiac, dzien
     
    DniTygodnia = ['Poniedziaek', 'Wtorek', 'roda', 'Czwartek', 'Pitek', 'Sobota', 'Niedziela']
    
    return DniTygodnia[date.weekday(Data)]

#-----------------------------------------

# zwraca liczb dni uwzgldniaj obie daty w zakresie, ich kolejno jest niewana
def IleDniWZakresieDat(d1,d2):
    if not SprawdzCzyToData(d1):
        return -1
    if not SprawdzCzyToData(d2):
        return -1

    delta = date(int(d1[:4]),int(d1[5:7]),int(d1[8:])) - date(int(d2[:4]),int(d2[5:7]),int(d2[8:]))
    IleDni = delta.days
    if IleDni < 0:
        IleDni = IleDni * -1

    return IleDni+1

#-----------------------------------------

def WartoscSkladnikaWZakresieDat(DataOd,DataDo,LWart):
    # DataOd - pocztkowa data analizy
    # DataDo - kocowa data analizy
    # LWart = [[data od,warto,co],[...]] - lista wartoci z datami startowymi
    stat = 'OK'
    wart = 0
    LWart.sort() # daty na licie bd po kolei
    # jeli DataOd mniejsza od najmniejszej daty to si nie da zrobi
    if DataOd < LWart[0][0]:
        return "Dostpne wartoci i ich zakresy ("+str(LWart)+") nie pokrywaj pocztku okresu do analizy ("+DataOd+") ",0

    i = -1 # indeks na licie wartoci

    # pobieramy pierwszy indeks dla ktrego bdziemy liczy
    for poz in LWart:
        if poz[0] <= DataOd:
            i+=1

    DataTmp = DataOd
    Ostatni = PodajOstatniDzien(DataTmp) # wtedy zmieniamy miesic (rwnoczenie mamy liczb dni w miesicu do podzielenia wartoci)
    while DataTmp <= DataDo:
        # liczymy "warto dnia"
        WartTmp = LWart[i][1] / Ostatni

        # dodajemy do wartoci skadnika
        wart += WartTmp

        # dodajemy dzie do DatyTmp
        if int(DataTmp[8:]) == Ostatni: # zmiana miesica
            if int(DataTmp[5:7]) == 12: # zmiana roku
                DataTmp = str(int(DataTmp[:4])+1)+'01-01'
            else:
                Mc = '0'+str(int(DataTmp[5:7])+1)
                DataTmp = DataTmp[:5]+Mc[-2:]+'-01'
            # pobieramy now liczb dni w miesicu
            Ostatni = PodajOstatniDzien(DataTmp)
        else: # tylko dzie si zmienia
            Dz = '0'+str(int(DataTmp[8:])+1)
            DataTmp = DataTmp[:8]+Dz[-2:]

        # sprawdzamy czy nie zmieni si indeks z LWart
        if LWart[i] != LWart[-1]: # jeli ju nie jestemy na ostatnim
            if LWart[i+1][0] <= DataTmp:
                i+=1

    return stat,RSRound(wart,2)

#-----------------------------------------

def ZmienStringNaSlownik(tekst):
    Slownik = []
    tekst=tekst[1:len(tekst)-1]
    i = 0
    while tekst != '':
        tupla = tekst[2:tekst.find(')')]
        j = 0
        SlownikP =[]
        while tupla != '':
            SlownikP.append(tupla[0:tupla.find('"')])
            if tupla.find(',') != -1:
                tupla = tupla[tupla.find(',')+2:len(tupla)]
            else:
                tupla = ''    
            j= j+1
        Slownik.append(SlownikP)
        tekst = tekst[tekst.find(')')+2:len(tekst)]
        i=i+1

    return Slownik

#-----------------------------------------

def ZmienStringNaSlownik2(tekst):  #['el1', 'el2']
    Slownik = []
    print tekst
    tekst=tekst[2:len(tekst)-1]
    while tekst != '':
        print tekst
        Slownik.append(tekst[0:tekst.find("'")])
        if tekst.find(',') != -1:
            tekst = tekst[tekst.find(',')+2:len(tekst)]
        else:
            tekst = ''    
        
    return Slownik

#-----------------------------------------

def ZmienStringNaTuple2El(tekst):
    #tupla w postaci ('element1', 'element2')
    el1 = ''
    el2 = ''
    tekst=tekst[1:len(tekst)-1] #obranie z nawiasw kwadratowych
    el1 =tekst[0:tekst.find(',')-1]
    el1=el1[1:len(tekst)-1] #obranie z nawiasw okrgych
    el2 =tekst[tekst.find(',')+3:len(tekst)-1]

    return (el1,el2)

#-----------------------------------------

def ZmienStringNaTuple3El(tekst):
    #tupla w postaci ('element1', 'element2', 'element3')
    el1 = ''
    el2 = ''
    el3 = ''
    tekst=tekst[1:len(tekst)-1] #obranie z nawiasw
    el1 =tekst[1:tekst.find(',')-1]
    tekst = tekst[tekst.find(',')+2:len(tekst)]
    el2=tekst[1:tekst.find(',')-1]
    tekst = tekst[tekst.find(',')+2:len(tekst)]
    el3=tekst[1:len(tekst)-1]

    return (el1,el2,el3)

#-----------------------------------------

def PodajDzis():
    #WS 2022-05-05
    Dzis = localtime()
    # Dzis = gmtime()
    Rok = str(Dzis[0])
    Mc = str(Dzis[1])
    if Dzis[1] < 10:
        Mc = '0'+Mc
    Dzien = str(Dzis[2])
    if Dzis[2] < 10:
        Dzien = '0'+Dzien
        
    return Rok+'-'+Mc+'-'+Dzien

#-------------------------------------------------------------------------------

def PodajNrDniaTygodnia(Data,Tryb=1):
    # wraca z dniem tygodnia
    # w kilku trybach:
    # 0 - wwczas 0 to poniedziaek a 6 to niedziela
    # 1 - wwczas 1 to poniedziaek a 7 to niedziela (domylny)
    # 2 - wwczas 0 to niedziela a 6 to sobota
    # 3 - wwczas 1 to niedziela a 7 to sobota
    t1 = strptime(Data,"%Y-%m-%d")
    t2 = mktime(t1)
    #Tryb = 0
    Dzien=localtime(t2)[6]
    #przesunicie z poniedziaku na niedziel
    if Tryb in [2,3]:
        Dzien=(Dzien+1) % 7
    #przesunicie o 1 dzie
    if Tryb in [1,3]:
        Dzien=Dzien+1
        
    return Dzien

#-------------------------------------------------------------------------------


def SprawdzPolskieZnaki(tekst):
    #sprawdza czy s polskie znaki
    return ('' in tekst) or ('' in tekst) or ('' in tekst) or ('' in tekst) or ('' in tekst) or ('' in tekst) or ('' in tekst) or ('' in tekst) or ('' in tekst) or ('' in tekst) or ('' in tekst) or ('' in tekst) or ('' in tekst) or ('' in tekst) or ('' in tekst) or ('' in tekst) or ('' in tekst) or ('' in tekst)

#-------------------------------------------------------------------------------
    
def SprRegon(value):
    #funkcja sprawdza poprawno - regon    
    if len(value)<>9 and len(value)<>14:
        return 'BL_REG_DLUG'
    else:
        if not value[0:8].isdigit() or len(value)>10 and (not value[10:].isdigit()):
            return 'BL_REG_DIGIT'
        
        if len(value)>9 and value[9:10]<>'-':
            return 'BL_REG_MIN'
        
        suma=0
        li=[8,9,2,3,4,5,6,7]
        for i,j in enumerate(li):
            suma+= int(value[i])*j
        if (suma%11%10) != int(value[8]):
            return 'BL_REG_CRC'
    return 'OK'

#-------------------------------------------------------------------------------

def SprNip(value, Kraj='PL'):
    #funkcja sprawdza poprawno - nip 
    if Kraj=='PL': # sprawdzany tylko dla polski
        if len(value)<>10:
            return 'BL_NIP_DLUG'
        else:    
            if not value.isdigit():
                return 'BL_NIP_DIGIT'
            suma=0
            li=[6,5,7,2,3,4,5,6,7]
            for i,j in enumerate(li):
                suma+= int(value[i])*j
            if (suma%11) != int(value[9]):
                return 'BL_NIP_CRC'
    return 'OK'

#-------------------------------------------------------------------------------

#AB 22.11.19r
def SprIBAN(value):
    #funkcja sprawdza poprawno - IBAN
    if not value.isdigit():
        return 'BL_IBAN_DIGIT'
    if len(value) != 26:
        return 'BL_IBAN_DLUG'
    # https://pl.wikipedia.org/wiki/Mi%C4%99dzynarodowy_numer_rachunku_bankowego
    # 2521 dla Polski wklepane na stae
    i = int(''.join(j for j in value[2:]) + '2521' + ''.join(j for j in value[:2]))
    if i % 97 != 1:
        return 'BL_IBAN_CRC'
    return 'OK'

#-------------------------------------------------------------------------------

def SprPesel(value):
    #funkcja sprawdza poprawno - pesel 
    if len(value)<>11:
        return 'BL_PES_DLUG'
    else:    
        if not value.isdigit():
            return 'BL_PES_DIGIT'
        suma=0
        li=[1,3,7,9,1,3,7,9,1,3]
        for i,j in enumerate(li):
            suma+= int(value[i])*j
        if ((10-(suma%10))%10) != int(value[10]):
            return 'BL_PES_CRC'
    return 'OK'

#-------------------------------------------------------------------------------

def SprPeselZData(pesel, date):
    """funkcja sprawdza poprawno peselu z dat urodzenia"""
    if pesel != None and date != None:
        pesel_year = pesel[0:2]
        pesel_month = pesel[2:4]
        pesel_day = pesel[4:6]
        date_year = date[2:4]
        date_month = date[5:7]
        date_day = date[8:]

        is_correct_year = date_year == pesel_year
        is_correct_day = date_day == pesel_day
        is_correct_month = False
        for century in [0, 20, 60, 80]:
            if int(date_month) + century == int(pesel_month):
                is_correct_month = True
        return is_correct_year == is_correct_month == is_correct_day == True
    return True

#-------------------------------------------------------------------------------

def SprNumerKonta(value):
    #  funkcja sprawdza poprawnoc wprowadzonego numeru konta bankowego ( dla polski )
    #  algorytm :
    #  do numeru konta dopisujemy z prawej strony cig 2521, ktry odpowiada kodowi literowemu PL (P - 25, L - 21), 
    #  liczb kontroln (pierwsze dwie cyfry numeru NRB) przenosimy na koniec (z lewej strony na praw), 
    #  dla tak uzyskanego cigu znakw obliczamy iloczyny czstkowe - mnoc kolejne cyfry numeru z kolejnymi wagami, przy czym numery czytamy od prawej do lewej, 
    #  sumujemy uzyskane iloczyny czstkowe i dzielimy wynik modulo 97, 
    #  Jeeli wynik jest rwny 1 to numer jest poprawny. #
    temp=str(value)
    if len(temp)<>26:
        return 'BL_DL'                # bdna dugo 
    else:
        if not temp.isdigit():
            return 'BL_NUMERIC'            # skada sie nie tylko z cyfr
        temp+='2521'
        numer=temp[2:] + temp[0:2] 
        
        suma=0
        wagi=[1,10,3,30,9,90,27,76,81,34,49,5,50,15,53,45,62,38,89,17,73,51,25,56,75,71,31,19,93,57]
        j=1
         
        for i in wagi:
            suma+= int(numer[-j])*i       
            j=j+1
        if (suma%97)==1:
            return 'OK'                # poprawny
        else:
            return 'BL_KONTO'                # bdny

#-------------------------------------------------------------------------------

def SprawdzAdresIp(ipadr):
    #funkcja sprawdza poprawnosc adresu IP
    #poprawny zakres od : 0.0.0.0  do 255.255.255.255
    ips = ipadr.split('.')
    if len(ips) != 4:
        return False
    for i in ips:
        if len(i) == 0 or len(i) > 3:
            return False
        for j in i:
            if not j.isdigit():
                return False
        if int(i) > 255:
            return False
        
    return True

#-------------------------------------------------------------------------------

def UtworzFiltr(lista_pol,operator2='AND',operator3='upper',operator='LIKE',puste_pola=['','    -  -  ']): 
    # do rsglobal na serwerze w takiej postaci mozna by wrzucic
    # na tyle ogolna ze mozna wykorzystac w innych miejscach
    #nazwa - atrybur relacji
    #pole - warto porwnywana
    #operator - operator porownania, domylnie 'LIKE'
    #operator2 - operator logiczny, domyslnie 'AND'
    #operator3 - funkcje sql np. upper, lower itp, domyslnie upper
    #lista_pol:
    # lista krotek formatu (nazwa,wartosc,{atrybuty})
    # gdzie atrybuty to lista dodatkowych wartosci tj. {'op1':operator2,'op2':operator,'op3':operator3}, nieobowiazkowa
    dodatki = {'LIKE':'%'}
    dodatek = dodatki.get(operator,'')
    filtr = ''
    czypierwszy=True
    for i in lista_pol:
        #print i
        operator2n,operator3n,operatorn,dodatekn = operator2,operator3,operator,dodatek #kopia domyslnych wartosci
        if i[1] not in puste_pola:
            if len(i) == 3: #wydaje sie ze mamy dodatkowe atrybuty dla tego pola
                operatorn=i[2].get('op2',operator)
                operator2n=i[2].get('op1',operator2)
                operator3n=i[2].get('op3',operator3)
                dodatekn = dodatki.get(operatorn,'')
            if czypierwszys:
                filtr+=" %s(%s) %s '%s%s'"%(operator3n,str(i[0]),operatorn,str(i[1]).strip().upper(),dodatekn)
                czypierwszy=False
            else:
                filtr+=" %s %s(%s) %s '%s%s'" %(operator2n,operator3n,str(i[0]),operatorn,str(i[1]).strip().upper(),dodatekn)
    return filtr

#-------------------------------------------------------------------------------


class RSTableRow(tuple):
  # class for each row
  
  def __getattr__(self,i):
    return tuple.__getitem__(self,self.__Field2Index__(i))

  def __call__(self,i):
    return tuple.__getitem__(self,self.__Field2Index__(i))

  def __getitem__(self,i):
#    try:
#      rsmess.RSFakt(i)
#      if str(type(tuple.__getitem__(self,self.__Field2Index__(i)))) == "<type 'libpq.PgBoole>":
      if str(type(tuple.__getitem__(self,self.__Field2Index__(i)))) == "bool":
        j = str(tuple.__getitem__(self,self.__Field2Index__(i)))
        if j == 't':
            j = True
        else:
            j = False
        return j
    
      #if type(i) != types.SliceType:
      return tuple.__getitem__(self,self.__Field2Index__(i))
      #else:
      #  if i.start is not None and i.stop is not None:
      #    return self[self.__Field2Index__(i.start):self.__Field2Index__(i.stop)]
      #  elif i.start is None:
      #    return self[:self.__Field2Index__(i.stop)]
      #  elif i.stop is None:
      #    return self[self.__Field2Index__(i.start):]
      #  else:
      #    return self[:]
#    except StandardError, inst:
#      rsmess.RSBlad('BD',inst)
  def __Field2Index__(self,i):
    return i

#---------------------------------------------------------------------------


class RSList(list):
  
  def __init__(self,rowset):
    
    for r in rowset:
      #self.append(r)  
      list.append(self,r)

#---------------------------------------

    def append(self,new):
    #fR = RSTableRow(new)
    #fR.__Field2Index__ = self.__fieldToIndex
    #list.append(self,fR)
        return

#---------------------------------------------------------------------------

reLiczba = re.compile("[+-]?\d*(\.\d+)?([eE][+-]?\d+)?|\d+[lL]")
reKoniecCudz = re.compile(r"[^\\](\\\\)*'")
reKoniecCudzD = re.compile(r'[^\\](\\\\)*"')
biale = " \n\r\t"
separatory = biale + ",)}]:"
reEscapeSek = re.compile(r"""\\([nrt'"\\]|o[0-7]{3,3}|x[0-9a-fA-F]{2,2})""")
reEscapeSekU = re.compile(r"""\\([nrt'"\\]|u[0-9a-fA-F]{4,4}|U[0-9a-fA-F]{8,8}|o[0-7]{3,3}|x[0-9a-fA-F]{2,2})""")

class Bufor:
    def __init__(self, tekst, pozycja=0):
        self.t = tekst
        self.pozycja = pozycja
    def znajdz(self, znak):
        return self.t.find(znak, self.pozycja)
    def znajdzRE(self, re_):
        return re.finditer(re_, self.t[self.pozycja:])
    def skocz(self, pozycja):
        self.pozycja = pozycja
    def dalej(self, zmiana=1):
        self.pozycja += zmiana
    def znak(self, rel=0):
        return self.t[self.pozycja + rel:self.pozycja + rel + 1]
    def pominBiale(self):
        while self.znak() and self.znak() in biale:
            self.pozycja += 1
    def fragment(self, ile, od=0):
        # jeli od<0 to pilnujemy eby nie wyj na minus co moe zmieni pozycje cur w res
        sp = self.pozycja
        return self.t[max(0, sp+od):sp+ile]
    def koniec(self):
        return self.pozycja >= len(self.t)

def _wczytaj_liczbe(bufor):
    mi = bufor.znajdzRE(reLiczba)
    try:
        m = mi.next()
    except StopIteration:
        raise Exception, "proba wczytania liczby a takiej nie widze"
    liczba = m.group()
    bufor.dalej(m.end())
    if ("." in liczba) or ("e" in liczba):
        return float(liczba)
    else:
        return int(liczba)

def _escapeSek(match):
    k = match.group()
    if k[1] in "nrt\\\'\"":
        return {"n": "\n", "r": "\r", "t": "\t", "\\": "\\", '"': '"', "'": "'"}[k[1]]
    elif k[1] == "x":
        return chr(int(k[2:], 16))
    elif k[1] == "o":
        return chr(int(k[2:], 8))

def _escapeSekU(match):
    k = match.group()
    if k[1] in "nrt\\\'\"":
        return {"n": "\n", "r": "\r", "t": "\t", "\\": "\\", '"': '"', "'": "'"}[k[1]]
    elif k[1] in "xuU":
        return unichr(int(k[2:], 16))
    elif k[1] == "o":
        return unichr(int(k[2:], 8))

def _wczytaj_str(bufor):
    uni = bufor.znak() in 'uU'
    if uni:
        bufor.dalej()

    cudzyslow = bufor.znak()
    if cudzyslow == "'":
        reZamykajacy = reKoniecCudz
    elif cudzyslow == '"':
        reZamykajacy = reKoniecCudzD

    # znajdz zamykajacy cudzyslow, poprzedzony slashem - nie, dwoma - ok itd.
    mi = bufor.znajdzRE(reZamykajacy)
    try:
        koniec = mi.next().end()
    except StopIteration:
        raise Exception, "niezamkniety cudzyslow"

    tekst = bufor.fragment(koniec-1, od=1)
    bufor.dalej(koniec)
    if uni:
        return unicode(re.sub(reEscapeSekU, _escapeSekU, tekst))
    else:
        return str(re.sub(reEscapeSek, _escapeSek, tekst))

def _evalparser(bufor):
    # rozpoznaj znak
    # [, (, liczba, ciag znakow
    # wczytaj ten element i wrzuc na stos
    # ), ] - znajdz odpowiednie i stworz pojemnik
    # przecinek?
    # None
    NAWIAS_OO = "NAWIAS("
    NAWIAS_KO = "NAWIAS["
    NAWIAS_WO = "NAWIAS{"
    PRZECINEK = "PRZECINEK"
    DWUKROPEK = "DWUKROPEK"

    # zbiera ze stosu elementy oddzielone przecinkami az do terminatora
    def zdejmijListe(terminator):
        bylPrzecinek = False
        v = stos.pop()
        if v is PRZECINEK:
            v = stos.pop()
            bylPrzecinek = True
        lista = []
        while v != terminator:
            lista.insert(0, v)
            v = stos.pop()
            assert v is terminator or v is PRZECINEK, "spodziewalem sie przecinka: %r" % v
            if v is PRZECINEK:
                v = stos.pop()
                bylPrzecinek = True
        return lista, bylPrzecinek

    # zbiera elementy ze stosu i ewentualnie robi z nich tuple
    def zdejmijSerie():
        lista, bylPrzecinek = zdejmijListe(NAWIAS_OO)
        if not bylPrzecinek and len(lista) == 1:
            return lista[0]
        else:
            return tuple(lista)

    def zbudujSlownik():
        fragment = []
        # czesc stosu z danymi slownika
        while stos and (stos[-1] is not NAWIAS_WO):
            fragment.insert(0, stos.pop())
        stos.pop()
        items = []
        #if fragment[-1] is PRZECINEK:
        #    fragment.pop()
        while fragment:
            assert len(fragment) >= 3
            k = fragment.pop(0)
            assert fragment.pop(0) is DWUKROPEK
            v = fragment.pop(0)
            items.append((k, v))
            if fragment:
                assert fragment[0] is PRZECINEK
                fragment.pop(0)

        return dict(items)

    stos = [NAWIAS_OO]
    while not bufor.koniec():
        bufor.pominBiale()
        c = bufor.znak()
        if not c:
            break
        elif c in "([{,":
            stos.append({"(": NAWIAS_OO, "[": NAWIAS_KO, "{": NAWIAS_WO, ",": PRZECINEK}[c])
            bufor.dalej()
        elif c in "0123456789.+-":
            stos.append(_wczytaj_liczbe(bufor))
        elif c in "'\"" or (c in "uU" and bufor.znak(1) in "'\""):
            stos.append(_wczytaj_str(bufor))
        elif c in "N" and bufor.fragment(4) == "None" and bufor.znak(4) in separatory:
            stos.append(None)
            bufor.dalej(4)
        elif c == "]":
            stos.append(zdejmijListe(NAWIAS_KO)[0])
            bufor.dalej()
        elif c == ")":
            stos.append(zdejmijSerie())
            bufor.dalej()
        elif c == ":":
            stos.append(DWUKROPEK)
            bufor.dalej()
        elif c == "}":
            stos.append(zbudujSlownik())
            bufor.dalej()
        elif c == "T" and bufor.fragment(4) == "True" and bufor.znak(4) in separatory:
            stos.append(True)
            bufor.dalej(4)
        elif c == "F" and bufor.fragment(5) == "False" and bufor.znak(5) in separatory:
            stos.append(False)
            bufor.dalej(5)
        else:
            raise Exception, "nie rozpoznany token: %r w %s" % (c, bufor.fragment(15, -3))

    return zdejmijSerie()

#-------------------------------------------------------------------------------

def rseval(tekst):
    try:
        return _evalparser(Bufor(tekst))
    except Exception, exc:
        ZapiszDoLogow("rseval blad: %s" % exc, 1)
        return "BLAD"

#-------------------------------------------------------------------------------

def SprawdzAdresEmail(adres):
    return re.match(r"^[A-Za-z0-9\.\+_-]+@[A-Za-z0-9\._-]+\.[a-zA-Z]*$", adres)
    
#-------------------------------------------------------------------------------

# wysya email o podanym temacie i treci do podanego adresata
# zalaczniki - lista cieek do plikw do zaczenia
def RSWyslijEmail(adresat, temat, tresc, zalaczniki=[], operator_id=-1, polsql=None, nadawca=None, cc=None, bcc=None):
    #JR 2023-04-19 (13408) jesli adresat zawiera wiecej niz jeden ares to puszczamy na pierwszy a reszte cc
    czywiecej = adresat.strip().replace(';',',').find(',')
    if czywiecej > -1:
        return RSWyslijEmail(adresat[0:czywiecej], temat, tresc, zalaczniki, operator_id, polsql, nadawca, adresat[czywiecej+1:],bcc)

    suffix = '.maile'
    try:
        ZapiszDoLogow('!!! #### Zaczynam funkcj wysyania mailem',1)
        ZapiszDoLogow('PARAMETRY '+str((adresat, temat, tresc, zalaczniki, operator_id, polsql, nadawca, cc, bcc)), 1)
        mail_log = "PARAMETRY:"
        mail_log += " \tadresat = " + repr(adresat)
        mail_log += " \ttemat = " + repr(temat)
        # if LogLevel >= 5:
        #     mail_log += " \ttresc = " + repr(tresc)
        mail_log += " \tzalaczniki: " + str(len(zalaczniki))
        mail_log += " \toperator_id = " + repr(operator_id)
        mail_log += " \tnadawca = " + repr(nadawca)
        mail_log += " \tcc = " + repr(cc)
        mail_log += " \tbcc = " + repr(bcc)

        def buildTextPart(text, enc="ISO-8859-2"):
            # w nowszych wersjach moduu email mona byoby:
            #mimeTresc = MIMEText(text.encode("utf8"), "plain", "UTF-8")
            mimeTresc = MIMEText("plain")
            if isinstance(text, unicode):
                mimeTresc = MIMEText(text.encode("utf8"), "plain", "UTF-8")
            else:
                mimeTresc = MIMEText(text, "plain", enc)

            return mimeTresc

        if zalaczniki:
            msg = MIMEMultipart()
            # jeli wystpuj zaczniki do tre wiadomoci wczana tak jak zacznik
            msg.attach(buildTextPart(tresc))
        else:
            msg = buildTextPart(tresc)
        ZapiszDoLogow('Bdzie zebranie danych operatora',1)
        # zebranie danych z operatora
        if operator_id != -1:
            if polsql is None:
                blad = "Podano operator_id=%r a polsql=%r, przekaz PolSQL" % (operator_id, polsql)
                mail_log = "BLAD: "+blad+" \t"+mail_log
                ZapiszDoLogow(mail_log, inflevel=1, suffix=suffix)
                return blad
            query = ("SELECT login, email_uzytkownik, email_haslo, email_serwer, email_port, email_szyfrowanie" +
                " FROM rs_operatorzy WHERE operator_id=%s")
            stat, dane, none, none = polsql.Execute(query, 2, (operator_id,))
            if stat != "OK":
                blad = "Bd SQL op"
                mail_log = "BLAD: "+blad+" \t"+mail_log
                ZapiszDoLogow(mail_log, inflevel=1, suffix=suffix)
                return blad
            if not dane:
                blad = "Operator nie istnieje"
                mail_log = "BLAD: "+blad+" \t"+mail_log
                ZapiszDoLogow(mail_log, inflevel=1, suffix=suffix)
                return blad
            operator, s_user, s_pass, s_server, s_port, s_sec = dane[0]
            if s_user in ('', 'None') or s_server in ('', 'None'):
                blad = "Operator %s nie ma ustawionego loginu lub serwera STMP" % (operator, )
                mail_log = "BLAD: "+blad+" \t"+mail_log
                ZapiszDoLogow(mail_log, inflevel=1, suffix=suffix)
                return blad
            if s_sec == "None":
                s_sec = ""
            s_sec = s_sec.upper()
            if s_sec not in ("", "SSL", "TLS"):
                blad = "Operator %s ma ustawiony nieznany typ szyfrowania %r" % (operator, s_sec)
                mail_log = "BLAD: "+blad+" \t"+mail_log
                ZapiszDoLogow(mail_log, inflevel=1, suffix=suffix)
                return blad
            if s_port == "None" or s_port == "":
                s_port = {"": smtplib.SMTP_PORT, "SSL": 465, "TLS": 587}[s_sec]
            else:
                try:
                    s_port = int(s_port)
                except ValueError:
                    blad = "Operator %s ma ustawiony nieprawidowy numer portu %r" % (operator, s_port)
                    mail_log = "BLAD: "+blad+" \t"+mail_log
                    ZapiszDoLogow(mail_log, inflevel=1, suffix=suffix)
                    return blad
            # sprbujmy ustali nadawc jeli nie podany
            if nadawca is None:
                if '@' in s_user:
                    nadawca = s_user
                else:
                    nadawca = "%s@%s" % (s_user, s_server)

            mail_log += " \toperator = " + repr(operator)
            mail_log += " \ts_user = " + repr(s_user)
            mail_log += " \ts_pass = " + repr(s_pass)
            mail_log += " \ts_server = " + repr(s_server)
            mail_log += " \ts_port = " + repr(s_port)
            mail_log += " \ts_sec = " + repr(s_sec)

        if nadawca is None:
            nadawca = DomyslnyNadawca
            if nadawca is None:
                blad = "Nie podano nadawcy i nie ustawiono domylnego."
                ZapiszDoLogow("Nie mona wysa emaila. " + blad, 1)
                mail_log = "BLAD: "+blad+" \t"+mail_log
                ZapiszDoLogow(mail_log, inflevel=1, suffix=suffix)
                return blad
        ZapiszDoLogow('Nadawca '+str(nadawca),1)
        temat = unicode(temat)
        ZapiszDoLogow('Temat '+temat,1)
        #WS 2019-05-07
        for k, v in (("From", nadawca), ("To", adresat), ("Subject", temat), ("cc", cc)):
        # for k, v in (("From", nadawca), ("To", adresat), ("Subject", temat), ("cc", cc), ("bcc", bcc)):
            if v:
                msg[k] = v

        msg["Date"] = email.Utils.formatdate(localtime=True)
        msg["User-Agent"] = "Anakonda"

        mail_log += " \tmsg[\"Date\"] = " + repr(msg["Date"])
        mail_log += " \tmsg[\"From\"] = " + repr(msg["From"])
        mail_log += " \tmsg[\"To\"] = " + repr(msg["To"])
        mail_log += " \tmsg[\"Subject\"] = " + repr(msg["Subject"])
        mail_log += " \tmsg[\"cc\"] = " + repr(msg["cc"])
        # dodanie zacznikw
        for plik in zalaczniki:
            if not os.path.exists(plik):
                blad = "Plik %s nie znaleziony." % plik
                mail_log = "BLAD: "+blad+" \t"+mail_log
                ZapiszDoLogow(mail_log, inflevel=1, suffix=suffix)
                return blad
            rest, ext = os.path.splitext(plik)
            try:
                #WS 2021-10-20
                with open(plik, "rb") as p:
                    zawartosc = p.read()
                # zawartosc = open(plik, "rb").read()
            except Exception, exc:
                blad = "Nie mona otworzy pliku %s (%s)" % (plik, exc)
                ZapiszDoLogow("RSWyslijEmail: " + blad)
                mail_log = "BLAD: "+blad+" \t"+mail_log
                ZapiszDoLogow(mail_log, inflevel=1, suffix=suffix)
                return blad

            try:
                # pliki tekstowe jako zaczniki tekstowe, pdf z odpowienim typem
                if ext.lower() in (".txt", ".log"):
                    att = buildTextPart(zawartosc)
                else:
                    if ext.lower() == ".pdf":
                        att = MIMEBase("application", "pdf")
                    else:
                        att = MIMEBase("application", "octet-stream")
                    att.set_payload(zawartosc)
                    email.Encoders.encode_base64(att)
                att.add_header('content-disposition', 'attachment', filename=os.path.basename(plik))
                msg.attach(att)
            except Exception, exc:
                blad = "Nie mona zbudowa zacznika %s (%s)" % (plik, exc)
                ZapiszDoLogow("RSWyslijEmail: " + blad)
                mail_log = "BLAD: "+blad+" \t"+mail_log
                ZapiszDoLogow(mail_log, inflevel=1, suffix=suffix)
                return blad

        # wysanie wiadomoci serwerowi SMTP
        ZapiszDoLogow('Wysyanie wiadomoci serwerowi SMTP',1)
        ZapiszDoLogow('Wiadomo '+str(msg),1)
        try:
            if operator_id == -1:
                #AB 12.06.19r
                server = smtplib.SMTP(timeout=60)
                server.connect(SerwerSMTP)
            else:
                if s_sec == "SSL":
                    try:
                        smtplib.SMTP_SSL
                    except AttributeError:
                        blad = "STMP SSL nie obsugiwane"
                        mail_log = "BLAD: "+blad+" \t"+mail_log
                        ZapiszDoLogow(mail_log, inflevel=1, suffix=suffix)
                        return blad
                    server = smtplib.SMTP_SSL()
                else:
                #AB 12.06.19r
                    server = smtplib.SMTP(timeout=60)
                server.connect(s_server, s_port)
                if s_sec == "TLS":
                    server.ehlo()
                    server.starttls()
                    server.ehlo()
                if s_user:
                    #AB 04.12.18r
                    server.login(s_user, str(s_pass))
            ZapiszDoLogow('Samo wysyanie ostateczne w nastpnym wierszu',1)
            #WS 2019-05-07
            if ',' in adresat:
                adresaci = [x.strip() for x in adresat.strip()]
            else:
                adresaci = [adresat]
            if cc is not None and cc != '':
                adresaci.extend(cc.split(","))
            if bcc is not None and cc != '':
                adresaci.extend(bcc.split(","))
                ZapiszDoLogow("Bcc: " + str(bcc))

            mail_log += " \tadresaci = " + repr(adresaci)
            server.sendmail(nadawca, adresaci, msg.as_string())
            # server.sendmail(nadawca, adresat, msg.as_string())
            ZapiszDoLogow('WYSANE',1)
            server.close()
        except Exception, exc:
            #WS 2022-08-05
            if isinstance(exc, smtplib.SMTPRecipientsRefused):
                adresy = []

                for adres in adresaci:
                    if not SprawdzAdresEmail(adres):
                        adresy.append(adres)

                for arg in exc.args:
                    for adres, blad in arg.iteritems():
                        if len(blad) > 1 and (blad[0] == 504 and blad[1].startswith('5.5.2') or blad[0] == 501 and blad[1].startswith('5.1.3')):
                            adresy.append(adres)

                adresy = list(set(adresy))
                blad = ', '.join(adresy)

                if len(adresy) == 0:
                    pass
                else:
                    if len(adresy) == 1:
                        blad = 'Nieprawidowy adres e-mail: '+blad
                    else:
                        blad = 'Nieprawidowe adresy e-mail: '+blad
                    mail_log = "BLAD: "+blad+" \t"+mail_log
                    ZapiszDoLogow(mail_log, inflevel=1, suffix=suffix)
                    return blad

            blad = "Nie mona wysa wiadomoci: {}".format(traceback.format_exc())
            mail_log = "BLAD: "+blad+" \t"+mail_log
            ZapiszDoLogow(mail_log, inflevel=1, suffix=suffix)
            return blad

        mail_log = "SUKCES \t" + mail_log
        ZapiszDoLogow(mail_log, inflevel=1, suffix=suffix)
        return "OK"
    except Exception, e:
        ZapiszDoLogow(mail_log, inflevel=1, suffix=suffix)
        raise e

#-----------------------------------------------------------------------------

class RSDrzewoPlikow(object):
    
    '''Drzewiasty quasi-sownik odwzorowujcy scieki na dowolne wartoci.

    Terminologia:
    
    * drzewo, poddrzewo, naddrzewo -- instancja klasy RSDrzewoPlikow;
    * cieka, podcieka, nadcieka -- sekwencja zawierajca nazwy, z ktrych
      skada si cieka dostpu do pliku/katalogu (uwaga: nie s obsugiwane
      bezwzgldne cieki dostpu, tzn. zaczynajce si od '/'); sekwencja ta
      peni rol podobn do sownikowego klucza;
    * nazwa -- lokalna nazwa pliku/katalogu (instancja str lub unicode), bdca
      elementem cieki;
    * przodek -- naddrzewo/nadcieka;
    * potomek -- poddrzewo/podcieka.

    cieki zwracane s jako listy; wewntrznie przetwarzane -- jako listy oraz
    instancje collections.deque; przyjmowane z zewntrz -- jako:
    
    * sekwencje (lub jakiekolwiek inne obiekty iterowalne) zawierajce nazwy,
    * napisy bdce waciwymi ciekami dostpu,
    * puste sekwencje (odnoszce si do caego drzewa) bd wszechobejmujce
      wykrojenia [:] (oznaczajce to samo, co puste sekwencje).

    '''


    # Stae i klasy pomocnicze

    BRAK = object()
    
    class KoniecSciezki(Exception):
        '''OK: wszystkie nazwy skadajce si na ciek zostay uyte'''

    class Znaleziono(Exception):
        '''OK: znaleziono poddrzewo z ustawion wartoci [patrz znajdz()]'''


    # Inicjalizacja

    def __init__(self, dawca=None):
        
        '''Zainicjuj drzewo -- puste lub zabierajc zawarto drzewu-dawcy'''

        if dawca is None:
            self._nazwy_na_poddrzewa = {}
            # miejsce na dowolny obiekt, np. informacj o n-rze wersji pliku:
            self._wartosc = self.BRAK
        else:
            nazwy_na_poddrzewa = dawca._nazwy_na_poddrzewa
            wartosc = dawca._wartosc
            
            dawca._nazwy_na_poddrzewa = {}
            dawca._wartosc = self.BRAK
            
            self._nazwy_na_poddrzewa = nazwy_na_poddrzewa
            self._wartosc = wartosc
            
        self._wzorzec_repr = '<%s %s: %%s %%s>' % (self.__class__.__name__,
                                                   hex(id(self)))


    # Metody prywatne

    def _przygotuj_sciezke(self, sciezka):
        if isinstance(sciezka, slice) and sciezka == slice(None, None, None):
            # podano wszechobejmujce wykrojenie
            sciezka = []
        elif isinstance(sciezka, basestring):
            # podano napis -- ciek dostpu do pliku
            sciezka = sciezka.split(os.sep)
            if sciezka and not sciezka[0]:
                sciezka[0] = os.sep
        try:
            return collections.deque(sciezka)
        except (TypeError, AttributeError):
            raise TypeError("Argument 'sciezka' powinien by "
                            "sekwencj (np. list) napisw, napisem "
                            "lub wszechobejmujcym wykrojeniem [:]")

    @staticmethod
    def _bierz_poddrzewo(drzewo, nazwa):
        return drzewo._nazwy_na_poddrzewa[nazwa]

    @classmethod
    def _bierz_poddrzewo_dodajac(cls, drzewo, nazwa):
        if not isinstance(nazwa, basestring):
            raise TypeError("Nieprawidowy typ nazwy: %r. Nazwy, z ktrych "
                            "skada si cieka musz by napisami"
                            % type(nazwa))
        if nazwa in ('', os.curdir, os.pardir) or (os.sep in nazwa):
            raise ValueError("Nieprawidowa nazwa: %r. Nazwy, z ktrych "
                             "skada si cieka nie mog by puste, "
                             "rwne %r lub %r, ani zawiera %r."
                             % (nazwa, os.curdir, os.pardir, os.sep))
        return drzewo._nazwy_na_poddrzewa.setdefault(nazwa, cls())

    def _siegnij_glebiej(self, sciezka, poprz_nazwa, naddrzewo,
                         metoda_brania_poddrzewa):
        if sciezka:
            nazwa = sciezka.popleft()
            poddrzewo = metoda_brania_poddrzewa(self, nazwa)
            # rekursja:
            poddrzewo._siegnij_glebiej(sciezka, nazwa, self,
                                       metoda_brania_poddrzewa)
        else:
            raise self.KoniecSciezki(naddrzewo, poprz_nazwa, self)

    def _szukaj_glebiej(self, sciezka, nadsciezka):
        if self._wartosc is not self.BRAK:
            raise self.Znaleziono(nadsciezka)
        elif sciezka:
            nazwa = sciezka.popleft()
            nadsciezka.append(nazwa)
            poddrzewo = self._bierz_poddrzewo(self, nazwa)
            # rekursja:
            poddrzewo._szukaj_glebiej(sciezka, nadsciezka)
        else:
            raise AttributeError


    # Publiczne wasnoci, operacje, metody

    @property
    def wartosc(self):
        "Warto przypisana temu drzewu (dowolny obiekt, np. nr wersji pliku)"
        if self._wartosc is self.BRAK:
            raise AttributeError("Drzewo %s jest drzewem przejciowym "
                                 "(nie ma ustawionego atrybutu 'wartosc')")
        return self._wartosc
        

    def __repr__(self):
        "Zwr reprezentacj napisow pokazujc aktualn zawarto"
        if self._wartosc is self.BRAK:
            wartosc = '-'
        else:
            wartosc = repr(self._wartosc)
        return self._wzorzec_repr % (wartosc, list(self.sciezki_nap()))


    # sciezka in drzewo
    def __contains__(self, sciezka):
        
        '''Zwr True, jeli poddrzewo do ktrego prowadzi podana cieka
        istnieje i ma ustawiony atrybut 'wartosc'; jeli nie -- False.

        Argument 'sciezka' to:
        * albo: cieka (sekwencja nazw),
        * albo: napis bdcy ciek dostpu,
        * albo: pusta sekwencja lub wszechobejmujce wykrojenie [:] -- wwczas
          zwracana jest warto True, jeli obiekt caego drzewa (self) ma
          ustawiony atrybut 'wartosc'; w przeciwnym razie -- False.
        
        '''
        
        sciezka = self._przygotuj_sciezke(sciezka)
        try:
            self._siegnij_glebiej(sciezka, None, None, self._bierz_poddrzewo)
        except self.KoniecSciezki, exc:
            _, _, ost_drzewo = exc.args
            return (ost_drzewo._wartosc is not self.BRAK)
        except KeyError:
            return False
        else:
            assert False, "Powinien zosta osignity koniec cieki!"


    def znajdz(self, sciezka):

        '''Dla kolejnych nadcieek podanej cieki (poczynajc od korzenia
        czyli pustej listy -- a do samej podanej cieki) dokonuj sprawdze
        takich jak dla __contains__ -- tak dugo, a wynik sprawdzenia bdzie
        pozytywny (dane drzewo bdzie mie ustawiony atrubyt 'wartosc'):
        wwczas zwr dan (nad)ciek.

        W razie natknicia si na brak poddrzewa -- rzu KeyError.
        W razie dojcia do penej cieki bez znalezienia poddrzewa, ktre
        miaoby ustawiony atrybut 'wartosc' -- rzu AttributeError.
        
        Argument 'sciezka' to:
        * albo: cieka (sekwencja nazw),
        * albo: napis bdcy ciek dostpu,
        * albo: pusta sekwencja lub wszechobejmujce wykrojenie [:] -- wwczas
          zwracana jest pusta lista, jeli obiekt caego drzewa (self) ma
          ustawiony atrybut 'wartosc'; w przeciwnym razie rzucany jest wyjtek
          AttributeError.
        
        '''

        sciezka = self._przygotuj_sciezke(sciezka)
        try:
            self._szukaj_glebiej(sciezka, nadsciezka=[])
        except self.Znaleziono, exc:
            return exc.args[0]


    def czy_jest(self, sciezka):

        '''Jak znajdz(), ale zwracaj True w razie znalezienia, a False
        jeeli znajdz() zgosio KeyError lub AttributeError.'''

        try:
            self.znajdz(sciezka)
        except (KeyError, AttributeError):
            return False
        else:
            return True


    # drzewo[sciezka]
    def __getitem__(self, sciezka):
        
        '''Zwr atrybut 'wartosc' poddrzewa, do ktrego prowadzi cieka;
        w razie jej braku rzu wyjtek KeyError; jeeli nie ma ona ustawionego
        atrybutu 'wartosc' -- rzu AttributeError.

        Argument 'sciezka' to:
        * albo: cieka (jako sekwencja nazw),
        * albo: napis bdcy ciek dostpu,
        * albo: pusta sekwencja lub wszechobejmujce wykrojenie [:] -- wwczas
          zwracany jest atrybut 'wartosc' obiektu tego drzewa (self._wartosc).
        
        '''
        
        sciezka = self._przygotuj_sciezke(sciezka)
        try:
            self._siegnij_glebiej(sciezka, None, None, self._bierz_poddrzewo)
        except self.KoniecSciezki, exc:
            _, _, ost_drzewo = exc.args
            return ost_drzewo.wartosc
        else:
            assert False, "Powinien zosta osignity koniec cieki!"


    def ustaw_i_zwroc(self, sciezka, wartosc):

        '''Utwrz i zwr nowy obiekt drzewa przenisszy do niego atrybut
        'wartosc' i wszystkie ewentualne poddrzewa tego poddrzewa, do ktrego
        prowadzi podana cieka -- dla niego za ustaw atrybut 'wartosc' na
        podany. Jeeli nie istaniao, dodaj je wraz z jego brakujcymi
        przodkami (atrybut 'wartosc' dla tak tworzonych przodkw nie jest
        ustawiany).
        
        Argument 'sciezka' to:
        * albo: cieka (sekwencja nazw),
        * albo: napis bdcy ciek dostpu,
        * albo: pusta sekwencja lub wszechobejmujce wykrojenie [:] -- wwczas
          drzewem, dla ktrego ustawiana jest nowa warto, jest to drzewo
          (self).
        
        '''

        sciezka = self._przygotuj_sciezke(sciezka)
        try:
            self._siegnij_glebiej(sciezka, None, None,
                                  self._bierz_poddrzewo_dodajac)
        except self.KoniecSciezki, exc:
            _, _, ost_drzewo = exc.args
            zwracane_drzewo = self.__class__(dawca=ost_drzewo)
            ost_drzewo._wartosc = wartosc
            return zwracane_drzewo
        else:
            assert False, "Powinien zosta osignity koniec cieki!"
        

    # drzewo[sciezka] = ...
    def __setitem__(self, sciezka, wartosc):
        '''To samo, co metoda ustaw_i_zwroc() -- ale zawsze zwracaj None'''
        self.ustaw_i_zwroc(sciezka, wartosc)


    # del drzewo[sciezka]
    def __delitem__(self, sciezka):
        
        '''Usu podan ciek wraz z jej podciekami (odetnij poddrzewo,
        do ktrego prowadzi); w razie jej braku rzu wyjtek KeyError; jeeli
        nie ma ona ustawionego atrybutu 'wartosc' usu j normalnie, ale potem
        rzu wyjtek AttributeError.

        Argument 'sciezka' to:
        * albo: cieka (sekwencja nazw),
        * albo: napis bdcy ciek dostpu,
        * albo: pusta sekwencja lub wszechobejmujce wykrojenie [:] -- wwczas
          odcinani s wszyscy bezporedni potomkowie.
        
        '''
        
        sciezka = self._przygotuj_sciezke(sciezka)
        if not sciezka:
            self._nazwy_na_poddrzewa.clear()
            self.wartosc
        else:
            try:
                self._siegnij_glebiej(sciezka, None, None,
                                      self._bierz_poddrzewo)
            except self.KoniecSciezki, exc:
                przedost_drzewo, nazwa, _ = exc.args
                del przedost_drzewo._nazwy_na_poddrzewa[nazwa]
                przedost_drzewo.wartosc
            else:
                assert False, "Powinien zosta osignity koniec cieki!"


    def ustaw_sciezki(self, sciezki, wartosc):
        
        '''Dla kadego z poddrzew, do ktrych prowadz podane cieki,
        sprawd czy_jest() i jeeli nie, ustaw podan warto (w razie
        potrzeby dodaj drzewa/cieki -- w uyciu jest ustaw_i_zwroc()).

        W przypadku poddrzewa majcego potomkw, po ustawieniu dla niego
        atrybutu 'wartosc', skasuj tyche potomkw i rzu KeyError z nowym
        obiektem drzewa -- zawierajcym tyche potomkw i dotychczasow
        (star) warto -- jako argumentem.
        
        '''
         
        for sciezka in sciezki:
            if not self.czy_jest(sciezka):
                zwrot = self.ustaw_i_zwroc(sciezka, wartosc)
                if zwrot._nazwy_na_poddrzewa:
                    raise KeyError(zwrot)


    def kasuj_sciezki(self, sciezki):
        "Masowe __delitem__(), ignoruj wyjtki: KeyError i AttributeError"
        for sciezka in sciezki:
            try:
                del self[sciezka]
            except (KeyError, AttributeError):
                pass


    # for sciezka in drzewo.sciezki...
    def sciezki(self, z_przejsciowymi=False, dzieci_przodem=False,
                _nadsciezka=[]):  # <- tu [] jako domylny jest OK 
        
        '''Iteruj po ciekach (cieki zwracane w formie list).

        Argumenty:
        * z_przejsciowymi (bool) -- jeeli True, iteruj po wszystkich ciekach;
          jeeli False (domylnie), tylko po ciekach, ktrych drzewa maj
          ustawiony atrybut 'wartosc';
        * dzieci_przodem (bool) -- jeeli True, zagwarantowana jest kolejno
          dzieci-przed-rodzicami; jeeli False (domylnie), zagwarantowana jest
          kolejno rodzice-przed-dziemi.

        '''
        
        sciezki_i_poddrzewa = ((_nadsciezka + [nazwa], poddrzewo)
                               for nazwa, poddrzewo
                               in self._nazwy_na_poddrzewa.items())
        if dzieci_przodem:
            for sciezka, poddrzewo in sciezki_i_poddrzewa:
                for podsciezka in poddrzewo.sciezki(z_przejsciowymi,  # rekurencja
                                                    dzieci_przodem, sciezka):
                    yield podsciezka
                if z_przejsciowymi or poddrzewo._wartosc is not self.BRAK:
                    yield sciezka
        else:
            for sciezka, poddrzewo in sciezki_i_poddrzewa:
                if z_przejsciowymi or poddrzewo._wartosc is not self.BRAK:
                    yield sciezka
                for podsciezka in poddrzewo.sciezki(z_przejsciowymi,  # rekurencja
                                                    dzieci_przodem, sciezka):
                    yield podsciezka
        

    # for sciezka in drzewo...
    def __iter__(self):
        "To samo, co sciezki() z domylnymi argumentami"
        return self.sciezki()


    # for s in drzewo.sciezki_nap...
    def sciezki_nap(self, z_przejsciowymi=False, dzieci_przodem=False):
        "Iteruj po napisach -- ciekach dostpu; argumenty jak dla sciezki()"
        return (os.path.join(*sciezka)
                for sciezka in self.sciezki(z_przejsciowymi, dzieci_przodem))


    # for wartosc, sciezka in drzewo.wartosci_i_sciezki...
    def wartosci_i_sciezki(self, z_przejsciowymi=False, dzieci_przodem=False,
                           _nadsciezka=[]):  # <- tu [] jako domylny jest OK 
        '''Iteruj po parach (<warto lub None>, <cieka w formie listy>).
        Argumenty -- argumenty jak dla sciezki().'''
        sciezki_i_poddrzewa = ((_nadsciezka + [nazwa], poddrzewo)
                               for nazwa, poddrzewo
                               in self._nazwy_na_poddrzewa.items())
        if dzieci_przodem:
            for sciezka, poddrzewo in sciezki_i_poddrzewa:
                for para in poddrzewo.wartosci_i_sciezki(z_przejsciowymi,  # rekurencja
                                                         dzieci_przodem, sciezka):
                    yield para
                wartosc = poddrzewo._wartosc
                if z_przejsciowymi or wartosc is not self.BRAK:
                    yield wartosc, sciezka
        else:
            for sciezka, poddrzewo in sciezki_i_poddrzewa:
                wartosc = poddrzewo._wartosc
                if z_przejsciowymi or wartosc is not self.BRAK:
                    yield wartosc, sciezka
                for para in poddrzewo.wartosci_i_sciezki(z_przejsciowymi,  # rekurencja
                                                         dzieci_przodem, sciezka):
                    yield para


    # for wartosc, s in drzewo.wartosci_i_sciezki_nap...
    def wartosci_i_sciezki_nap(self, z_przejsciowymi=False, dzieci_przodem=False):
        '''Iteruj po parach (<warto lub None>, <cieka dostpu (napis)>).
        Argumenty -- argumenty jak dla sciezki().'''
        return ((wartosc, os.path.join(*sciezka))
                for wartosc, sciezka in self.wartosci_i_sciezki(z_przejsciowymi,
                                                                dzieci_przodem))

#-----------------------------------------------------------------------------

class RSPlik(object):

    "Plik bezpieczny dla wielu wtkw/procesw (blokowany przez flock)"

    PODKAT_BLOKAD = '.RSPlik-flock'
    PRZYROSTEK_PLIKU_BLOKADY = '.flock'

    DOZWOLONE_TRYBY = frozenset(['r', 'rU', 'rb', 'w', 'wb', 'a', 'ab'])
    DOMYSLNY_TRYB = 'r'
    TRYB_PLIKU_BLOKADY = dict(r='r', w='w', a='w')
    FLOCK_OP = dict(r=fcntl.LOCK_SH, w=fcntl.LOCK_EX)


    def __init__(self, sciezka, tryb=None):
        "Tryb: 'r', 'w', 'a', 'rU' (3 pierwsze mog mie dodane 'b')"
        if tryb is None:
            tryb = self.DOMYSLNY_TRYB
        if tryb not in self.DOZWOLONE_TRYBY:
            raise TypeError('%s wymaga jednego z trybw: %s'
                            % (self.__class__.__name__,
                               ', '.join(map(repr, sorted(self.DOZWOLONE_TRYBY)))))
        self.plik_blokady = self._zaloz_blokade(sciezka, tryb)
        try:
            self.plik_glowny = self._otworz_plik(sciezka, tryb)
        except:
            exc = sys.exc_info()[1]
            try:
                self.close()
            except Exception:
                pass
            raise exc

    @classmethod
    def _zaloz_blokade(cls, sciezka, tryb):
        kat, nazwa = os.path.split(sciezka)
        kat_blokad = os.path.join(kat, cls.PODKAT_BLOKAD)
        try:
            os.mkdir(kat_blokad, 0700)
        except OSError, exc:
            if exc.errno != errno.EEXIST:  # (EEXIST == ju istnieje)
                raise
        (sciezka_pliku_blokady
        ) = os.path.join(kat_blokad, nazwa + cls.PRZYROSTEK_PLIKU_BLOKADY)
        tryb_pliku_blokady = cls.TRYB_PLIKU_BLOKADY[tryb.rstrip('bU')]
        try:
            plik_blokady = open(sciezka_pliku_blokady, tryb_pliku_blokady)
        except EnvironmentError, exc:
            if exc.errno == errno.ENOENT:   # (ENOENT == nie istnieje)
                open(sciezka_pliku_blokady, 'w').close()
                plik_blokady = open(sciezka_pliku_blokady, tryb_pliku_blokady)
            else:
                raise
        fcntl.flock(plik_blokady, cls.FLOCK_OP[tryb_pliku_blokady])
        return plik_blokady

    @staticmethod
    def _otworz_plik(sciezka, tryb):
        return open(sciezka, tryb)

    def __getattr__(self, name):
        return getattr(self.plik_glowny, name)

    def __iter__(self):
        return iter(self.plik_glowny)

    def close(self):
        try:
            self.plik_glowny.close()
        finally:
            self.plik_blokady.close()        

#-----------------------------------------------------------------------------

class RSSlownikWPliku(RSPlik):

    "Sownik w pliku (shelve) bezpieczny dla wielu (blokowany przez flock)"

    DOZWOLONE_TRYBY = frozenset(['r', 'w', 'c', 'n'])
    DOMYSLNY_TRYB = 'c'
    TRYB_PLIKU_BLOKADY = dict(r='r', w='w', c='w', n='w')

    @staticmethod
    def _otworz_plik(sciezka, tryb):
        "Tryb: 'r', 'w', 'c', 'n' -- vide: argument `flag' dla shelve.open()"
        return shelve.open(sciezka, flag=tryb, protocol=2)

    def __contains__(self, key):
        return key in self.plik_glowny

    def __len__(self):
        return len(self.plik_glowny)

    def __getitem__(self, key):
        return self.plik_glowny[key]

    def __setitem__(self, key, value):
        self.plik_glowny[key] = value

    def __delitem__(self, key):
        del self.plik_glowny[key]

#-------------------------------------------------------------------------------

def PolaTabeli(polSQL=None, Tabela='', BezPolParam=[]):
    # zwraca liste pol z podanej tabeli, bez pol podanych w liscie BezPol[]
    ListaPol = []
    stat = 'OK'
    
    if polSQL == None:
        stat = 'Brak poczenia z baz.'
        return stat, ListaPol

    if Tabela.strip() == '':
        stat = 'Nie podano nazwy tabeli.'
        return stat, ListaPol
    
    BezPol = ['z_czas','z_typ','z_op_id']
    if len(BezPolParam) > 0:
        BezPol.extend(BezPolParam)
    
    sql = "SELECT atr.attname FROM pg_attribute atr"
    sql+= " INNER JOIN pg_class cl ON (cl.relname='"+Dekoduj(Tabela)+"')"
    sql+= " WHERE atr.attrelid=cl.oid AND atr.attstattarget=-1"
    sql+= " AND NOT atr.attname IN "+str(BezPol).replace('[','(').replace(']',')')+";"
    
    stat,d,dump,dump = polSQL.Execute(sql,2)
    if stat != 'OK':
        stat = "Bd pobrania listy pl z tabeli '"+Tabela+"'"
    else:
        for i in xrange(len(d)):
            ListaPol.append(Dekoduj(d[i][0]))
    
    return stat, ListaPol

#-------------------------------------------------------------------------------

def PrzenumerujPozycje(PolSQL,Tabela,Klucz,NrPoz,NrPPoz=-1,PolePoz='nr_poz',PolePPoz='nr_ppoz'):
    # wywoywane tylko wewntrznie wic zakadam poprawno danych
    if PolSQL == None:
        return 'Brak poczenia z baz.'
    ZapiszDoLogow('prz 1',1)
    # sprawdzenie nieistnienia podanej pozycji
    SQL="SELECT "+PolePoz+" FROM "+Tabela+" WHERE "+Klucz+" AND "+PolePoz+"="+str(NrPoz)
    if NrPPoz>-1:
        SQL+=" AND "+PolePPoz+"="+str(NrPPoz)
    ZapiszDoLogow('prz 2',1)
    stat,Dane,dump,dump = PolSQL.Execute(SQL,3)

    if stat != 'OK':
        return "Bd pobrania teoretycznie usunitego pola z tabeli '"+Tabela+"' : "+stat
    ZapiszDoLogow('prz 3',1)
    if len(Dane)>0:
        return ' Nie mona przenumerowa pozycji, bo pozycja pocztkowa istnieje'
    # jeli podpozycja<>-1 to pobieramy list podpozycji do przenumerwania. 
    # Jeli puste i podpozycja bazowa rwna 0 to znaczy, e to bya ostatnia
    # podpozycja i przenumerowa trzeba pozycje. A jeli s to przenumerowujemy podpozycje
    if NrPPoz<>-1:
        SQL="SELECT "+PolePPoz+" FROM "+Tabela+" WHERE "+Klucz+" AND "+PolePoz+"="+str(NrPoz)
        SQL+=" AND "+PolePPoz+">"+str(NrPPoz)+" ORDER BY "+PolePPoz+";"
        stat,DanePPoz,dump,dump = PolSQL.Execute(SQL,3)
        if stat != 'OK':
            return "Bd pobrania podpozycji do zmiany z tabeli '"+Tabela+"' : "+stat

        # przenumerowujemy podpozycje
        if len(DanePPoz)>0:
            for poz in DanePPoz:
                SQL="UPDATE "+Tabela+" SET "+PolePPoz+"="+str(NrPPoz)+" WHERE "+Klucz+" AND "+PolePoz+"="+str(NrPoz)
                SQL+=" AND "+PolePPoz+"="+str(poz[0])+";"
                stat,dump,dump,dump = PolSQL.Execute(SQL,1)
                if stat != 'OK':
                    return "Bd przenumerowania podpozycji z tabeli '"+Tabela+"' : "+stat
                NrPPoz+=1
            return 'OK'

        # to oznacza, e bya to niezerowa podpozycja ale bya ostatnia i dlatego dane byy puste.
        # w przeciwnym przypadku nie ma podpozycji. Zatem po prostu pobieramy dane do przenumerowania pozycji
        if NrPPoz>0:
            return 'OK'
    
        # pobranie danych do przenumerowania pozycji
    ZapiszDoLogow('prz 4',1)
    SQL="SELECT "+PolePoz
    if NrPPoz<>-1:
        SQL+=","+PolePPoz
    SQL+=" FROM "+Tabela+" WHERE "+Klucz+" AND "+PolePoz+">"+str(NrPoz)
    SQL+=" ORDER BY "+PolePoz+";"
    stat,DanePoz,dump,dump = PolSQL.Execute(SQL,3)
    if stat != 'OK':
        return "Bd pobrania pozycji do zmiany z tabeli '"+Tabela+"' : "+stat
    ZapiszDoLogow('prz 5',1)
    if len(DanePoz)>0:
        NrPocz=0
        ZapiszDoLogow('SQL '+SQL,1)
        for poz in DanePoz:
            # zwikszamy numer pozycji tylko jak zwiksza si numer pozycji w kopiowanych danych
            if DanePoz[0][0]<>NrPocz:
                SQL="UPDATE "+Tabela+" SET "+PolePoz+"="+str(NrPoz)
                SQL+=" WHERE "+Klucz+" AND "+PolePoz+"="+str(poz[0])
                ZapiszDoLogow('SQL '+SQL,1)
                stat,dump,dump,dump = PolSQL.Execute(SQL,1)
                if stat != 'OK':
                    return "Bd przenumerowania pozycji z tabeli '"+Tabela+"' : "+stat
                NrPocz=DanePoz[0][0]
                NrPoz+=1

    ZapiszDoLogow('prz 6',1)
    return 'OK'

#-------------------------------------------------------------------------------

# Funkcja sprawdza, czy podany login jest prawidlowy.
# Te same zasady dotycza hasla, wiec mozna sprawdzac tez haslo

def CzyPrawidlowyLogin(Login):
    if len(Login) == 0:
        return False

    if not Login.isalnum():
        return False
    
    for s in Login:
        if s.isalpha():
            if not s.islower():
                return False

    return True

#-------------------------------------------------------------------------------

def CzyPrawidloweHaslo(Haslo):
    if len(Haslo) == 0:
        return False

    if not Haslo.isalnum():
        return False
    
    return True
    
#-------------------------------------------------------------------------------

def StworzAtrapeSelfa(firma,PolSQL,okres='',oddzial=''):
    global ListaStawekZn
    
    class Atrapa:pass
    Atr=Atrapa()
    Atr.firma_id = firma
    Atr.PolSQL = PolSQL
    Atr.okres_id = okres
    Atr.oddzial_id = oddzial
    Atr.ListaStawekZn=ListaStawekZn
    return Atr

#-------------------------------------------------------------------------------

def WyznaczFiltrNaCeche(CechaOd,CechaDo,Tabela):
    Filtr = ''
    if CechaOd != '':
        if CechaDo == '' and ('_' in CechaOd or '%' in CechaOd):
            if not '%' in CechaOd:
                CechaOd=CechaOd+'%'
            Filtr+=" AND UPPER("+Tabela+".cecha1_wartosc) LIKE UPPER('"+CechaOd+"')"
        else:
            Filtr+=" AND "+Tabela+".cecha1_wartosc >= '"+CechaOd+"'"
                
    if CechaDo != '':
        if CechaOd == '' and ('_' in CechaDo or '%' in CechaDo):
            if not '%' in CechaDo:
                CechaDo=CechaDo+'%'
            Filtr+=" AND UPPER("+Tabela+".cecha1_wartosc) LIKE UPPER('"+CechaDo+"')"
        else:
            Filtr+=" AND ("+Tabela+".cecha1_wartosc <= '"+CechaDo+"' OR "+Tabela+".cecha1_wartosc IS NULL)"

    return Filtr

#-------------------------------------------------------------------------------

def PierwszyDzienMiesiaca(data, jakimiesiac=0):
    try:
        Rok = int(data[0:4])
        Mc = int(data[5:7])+jakimiesiac
        Dzien = int(data[8:10])
        while Mc<1:
            Mc+=12
            Rok-=1
        while Mc>12:
            Mc-=12
            Rok+=1
        return str(Rok)+'-'+str(Mc+100)[1:3]+'-01'
    except:
        return data

#-------------------------------------------------------------------------------

def OstatniDzienMiesiaca(data, jakimiesiac=0):
    try:
        Rok = int(data[0:4])
        Mc = int(data[5:7])+jakimiesiac
        Dzien = int(data[8:10])
        while Mc<1:
            Mc+=12
            Rok-=1
        while Mc>12:
            Mc-=12
            Rok+=1
        Przestepny = (Rok % 4 == 0) and ((Rok % 100 != 0) or (Rok % 400 == 0))
        if Przestepny:
            Dni=[31,29,31,30,31,30,31,31,30,31,30,31]
        else:
            Dni=[31,28,31,30,31,30,31,31,30,31,30,31]
        return str(Rok)+'-'+str(Mc+100)[1:3]+'-'+str(Dni[Mc-1]+100)[1:3]
    except:
        return data

# -------------------------------------------------------------------------------


def SprawdzNaBialejLiscie(konto, nip=None, regon=None, data=str(datetime.datetime.now())[:10], liczba_prob=1,
                          szukac_po_ibanie=False, verbose=False, verify=True, self_fun=None):  #AB 26.08.22r
    """Funkcja sprawdzajca zgodno numeru konta z bia list z uyciem api pod adresem wl-api.mf.gov.pl.
    Numer konta musi by przypisany do firmy, ktr identyfikuje si po NIPie lub REGONie.
    Jeli metoda check nie poskutkuje, wykonywane jest wyszukanie po numerze IBAN poniewa aktualnie tylko w taki sposb
    da si w tym API zweryfikowa rachunki wirtualne.

    :type  konto:            str
    :param konto:            (obowizkowe) numer IBAN, ignorujemy znaki nie bdce cyframi, funkcja sprawdza tylko
                             dugo(26 cyfr), dla bdnej sumy kontrolnej API zwrci bd
    :type  nip:              str
    :param nip:              (obowizkowy jeli nie podano REGONu) ignorujemy znaki nie bdce cyframi, funkcja sprawdza
                             tylko dugo(10), dla bdnej sumy kontrolnej API zwrci bd
    :type  regon:            str
    :param regon:            (obowizkowy jeli nie podano NIPu ignorujemy znaki nie bdce cyframi, funkcja sprawdza
                             tylko dugo(10), dla bdnej sumy kontrolnej API zwrci bd
    :type  data:             str/datetime.date
    :param data:             (opcjonalne) na kiedy sprawdzona ma by biaa lista, moliwo podania zmiennej typu
                             datetime.date lub tekstu w formacie 'YYYY-MM-DD', jeli nic nie podano sprawdzenie zostanie
                             wykonane na dzi
    :type  liczba_prob:      int
    :param liczba_prob:      ile razy probowa, dla kodu "503 - Usuga niedostpna" Anakonda bdzie prbowa ponownie
                             tyle razy, ile wpisano w tym argumencie(domylnie 1 prba czyli bez powtrek
    :type  szukac_po_ibanie: bool
    :param verbose:          czy od razu korzysta z wyszukiwania wg. numeru IBAN(dla rekursji gdy serwis ma problemy,
                             ale wiadomo ju e pary nie udao si znale)
    :type  verbose:          bool
    :param verbose:          wybr midzy prostym wynikiem i zoonym
    :type  verify:           bool
    :param verify:           czy sprawdza certyfikat SSL API, umoliwia pominicie kontroli
    :type  self_fun:         object
    :param self_fun:         self potrzebnyh do wysania maila o problemie z API

    Zwraca:

    dla verbose = False(domylnie):
                  -1 dla jakiegokolwiek bdu,

                  0 dla nie znalezionej pary,

                  string zawierajcy requestId jeli API odnalazo par


    dla verbose = True tupl w 3 formatach:
                  bd:           (None, tre bdu)

                  nie znaleziono: (False, requestId)

                  znaleziono:     (True, requestId)

"""


    api_url = 'https://wl-api.mf.gov.pl/'

    if nip:
        nip = re.sub('[^0-9]', '', nip)
        if len(nip) != 10:
            komunikat = "NIP nie ma 10 cyfr"
            if verbose:
                return None, komunikat
            else:
                ZapiszDoLogow(komunikat)
                return -1
        query = 'api/check/nip/{nip}/bank-account/{konto}'

    elif regon:
        regon = re.sub('[^0-9]', '', regon)
        if len(regon) not in (9, 14):
            komunikat = "REGON nie ma 9 ani 14 cyfr"
            if verbose:
                return None, komunikat
            else:
                ZapiszDoLogow(komunikat)
                return -1
        query = 'api/check/regon/{regon}/bank-account/{konto}'

    else:
        komunikat = "Nie podano NIP ani REGON"
        if verbose:
            return None, komunikat
        else:
            ZapiszDoLogow(komunikat)
            return -1

    if isinstance(data, datetime.date):
        data = str(data)[:10]

    konto = re.sub('[^0-9]', '', konto)

    if len(konto) != 26:
        komunikat = "Rachunek nie ma 26 cyfr"
        if verbose:
            return None, komunikat
        else:
            ZapiszDoLogow(komunikat)
            return -1

    try:
        datetime.datetime.strptime(data, '%Y-%m-%d')
    except ValueError:
        komunikat = "Niepoprawny format daty, oczekiwano RRRR-MM-DD."
        if verbose:
            return None, komunikat
        else:
            ZapiszDoLogow(komunikat)
            return -1
    if not szukac_po_ibanie:
        query = api_url + query + '?date={data}'
        query = query.format(**locals())

        ZapiszDoLogow(query)
        if not verify and self_fun:  #AB 26.08.22r
            komunikat = "Sprawdzenie biaej listy ignorujc certyfikat SSL"
            tresc = """{}
                Firma {}
                Operator {} 
                Czas {}""".format(komunikat, self_fun.firma_id, self_fun.operator_id, str(datetime.datetime.now())[:-7])
            RSWyslijEmail(const.MAIL_SERWISOWY, komunikat, tresc, [], 0, self_fun.PolSQL)

        try:  #AB 26.08.22r
            response = requests.get(query, verify=verify)
        except requests.adapters.SSLError:  #AB 26.08.22r
            komunikat = "Problem weryfikacji certyfikatu SSL dla witryny {}".format(api_url)
            if self_fun.PolSQL:
                tresc = """{}
                Firma {}
                Operator {} 
                Czas {}""".format(komunikat, self_fun.firma_id, self_fun.operator_id, str(datetime.datetime.now())[:-7])
                RSWyslijEmail(const.MAIL_SERWISOWY, komunikat, tresc, [], 0, self_fun.PolSQL)
            if verbose:
                return 'SSL', komunikat
            else:
                return -2
        if response.status_code != 200:
            if liczba_prob > 1 and response.status_code == 503:
                ZapiszDoLogow("Powtarzanie dla kodu 503")
                return SprawdzNaBialejLiscie(konto, nip, regon, data, liczba_prob - 1, szukac_po_ibanie, verbose, verify)  #AB 26.08.22r

            tresc = ''
            try:
                tresc = response.json()['message']
            except:
                pass
            komunikat = "Otrzymano kod odpowiedzi: {}{}".format(response.status_code, (" - " + tresc) if tresc else "")
            komunikat_bledu = "Problem weryfikacji biaej listy: {}".format(komunikat)  #AB 26.08.22r
            if self_fun.PolSQL:  #AB 26.08.22r
                tresc = """{}
                Firma {}
                Operator {} 
                Czas {}""".format(komunikat_bledu, self_fun.firma_id, self_fun.operator_id, str(datetime.datetime.now())[:-7])
                RSWyslijEmail(const.MAIL_SERWISOWY, "Podczas sprawdzania biaej listy wystapi bd", tresc, [], 0, self_fun.PolSQL)
            if verbose:
                return None, komunikat
            else:
                ZapiszDoLogow(komunikat)
                return -1
        wynik = response.json()['result']
        if wynik['accountAssigned'] == 'TAK':
            if verbose:
                return True, wynik['requestId']
            else:
                return wynik['requestId']
        elif wynik['accountAssigned'] == 'NIE':
            return SprawdzNaBialejLiscie(konto, nip, regon, data, liczba_prob, True, verbose, verify)  #AB 26.08.22r
        else:
            if verbose:
                return False, wynik['requestId']
            else:
                return -1


    else: #AB 18.12.19r
        query2 = "api/search/bank-account/{konto}"
        query2 = api_url + query2 + '?date={data}'
        query2 = query2.format(**locals())

        ZapiszDoLogow(query2)
        try:  #AB 26.08.22r
            response = requests.get(query2, verify=verify)
        except requests.adapters.SSLError:
            if verbose:
                return 'SSL', "Problem weryfikacji certyfikatu SSL dla witryny {}".format(api_url)
            else:
                return -2
        if response.status_code != 200:
            if liczba_prob > 1 and response.status_code == 503:
                ZapiszDoLogow("Powtarzanie dla kodu 503")
                return SprawdzNaBialejLiscie(konto, nip, regon, data, liczba_prob - 1, True, verbose, verify)  #AB 26.08.22r
            tresc = ''
            try:
                tresc = response.json()['message']
            except:
                pass
            komunikat = "Otrzymano kod odpowiedzi: {}{}".format(response.status_code, (" - " + tresc) if tresc else "")
            komunikat_bledu = "Problem weryfikacji biaej listy: {}".format(komunikat)  #AB 26.08.22r
            if self_fun.PolSQL:  #AB 26.08.22r
                tresc = """{}
                Firma {}
                Operator {} 
                Czas {}""".format(komunikat_bledu, self_fun.firma_id, self_fun.operator_id, str(datetime.datetime.now())[:-7])
                RSWyslijEmail(const.MAIL_SERWISOWY, "Podczas sprawdzania biaej listy wystapi bd", tresc, [], 0, self_fun.PolSQL)
            if verbose:
                return None, komunikat
            else:
                ZapiszDoLogow(komunikat)
                return -1
        wynik2 = response.json()['result']
        requestId = wynik2['requestId']
        subjects  = wynik2['subjects']
        ZapiszDoLogow("Znaleziono po numerze rachunku: {}".format(subjects))
        for subject in subjects:
            if subject['hasVirtualAccounts'] and subject['nip'] == nip if nip else regon == subject['regon']:
                if verbose:
                    return True, requestId
                else:
                    return requestId
        if verbose:
            return False, wynik2['requestId']
        else:
            return 0

# -------------------------------------------------------------------------------

def ZmienNaNULL(wartosc):
    # dla potrzeb INSERT lub UPDATE w PostgreSQL zamienia podan wartosc
    # na string lub w przypadku "pustych" wartosci na napis NULL

    if str(wartosc).strip() in ['', None, '<None>', 'None', '    -  -  ', '-  -']:
        return "NULL"
    return "'" + str(wartosc) + "'"

# -------------------------------------------------------------------------------

#AB 30.12.20r
def GenerujPIN(tekst, dlugosc=5):
    """Metoda generuje pin o okrelonej dugoci w oparciu o podany tekst"""
    return string.zfill(str(int(int(hashlib.sha1(tekst.encode("utf-8")).hexdigest(), 16) % (10 ** dlugosc))), dlugosc)

# -------------------------------------------------------------------------------


def UsunPolskieZnaki(w):
    """
    :param w: tekst z ktrego maj by usunite poslkie znaki
    :type  w: basestring
    :return: tekst po usunieciu polskich znakow
    """
    w = w.replace('', 'a')
    w = w.replace('', 'c')
    w = w.replace('', 'e')
    w = w.replace('', 'l')
    w = w.replace('', 'n')
    w = w.replace('', 'o')
    w = w.replace('', 's')
    w = w.replace('', 'z')
    w = w.replace('', 'z')
    w = w.replace('', 'A')
    w = w.replace('', 'C')
    w = w.replace('', 'E')
    w = w.replace('', 'L')
    w = w.replace('', 'N')
    w = w.replace('', 'O')
    w = w.replace('', 'S')
    w = w.replace('', 'Z')
    w = w.replace('', 'Z')
    return w

# -------------------------------------------------------------------------------


def check_pid(pid):
    """ sprawdza czy istnieje pid. """
    try:
        os.kill(pid, 0)  # 0 nic nie robi, nie zabija, error oznacza brak procesu
    except OSError:
        return False
    else:
        return True

# -------------------------------------------------------------------------------


def WyslijSms(odbiorca, tresc, firma_id, polsql, funkcja, kontrahent_id=None):
    """
    :param odbiorca:  numer telefonu odbiorcy
    :type  odbiorca:  basestring
    :param    tresc:  tre wiadomoci SMS, limit 800 znakw
    :type     tresc:  basestring
    :param firma_id:  symbol firmy
    :type  firma_id:  basestring
    :param   polsql:  zmienna z poczeniem SQL do bazy
    :type    polsql:  rssql.RSSQL
    :param  funkcja:  funkcja wysyajca sms, kod_funkcji.co_filename w normalnych funkcjach, fs_name w harmonogramie
    :type   funkcja:  basestring
    :return:
    """

    #JR 2023-04-19 (13408) jesli ista numerow to wysyla po kolei
    listanumerow = odbiorca.strip().replace(';',',').split(',')
    if len(listanumerow) > 1:
        for nr in listanumerow:
            stat = WyslijSms(nr, tresc, firma_id, polsql, funkcja, kontrahent_id)
            if stat != 'OK':
                return stat
        return 'OK'

    plikKonf    = "rs-and.conf"
    param       = ('bramka_login', 'port_sms')
    pochodzenie = funkcja
    login, port_sms = PodajZKonf(plikKonf, param)
    if login == '' and port_sms == '':
        return "Wysyka sms nie zostaa skonfigurowana, skontaktuj si z administratorem."
    odbiorca  = odbiorca.replace(' ', '').replace('-', '').replace('(', '').replace(')', '').replace('+', '')
    if len(odbiorca) == 9:
        odbiorca = '48'+odbiorca
    if len(odbiorca) != 11:
        ZapiszDoLogow(odbiorca)
        return "Niepoprawny numer telefonu odbiorcy."

    SQL = """INSERT INTO rs_sms
             (firma_id, odbiorca, kontrahent_id, tresc, pochodzenie, wysylajacy)
             VALUES
             (%s,       %s,       %s,            %s,    %s,          rsf_getuserid())"""

    args = [firma_id, odbiorca, kontrahent_id, tresc, pochodzenie]
    args = [str(x) if isinstance(x, basestring) else x for x in args]

    ZapiszDoLogow(polsql.Mogrify(SQL, args))
    stat, dump, dump, dump = polsql.Execute(SQL, 1, args, CzyDodawacBlad=True)
    if stat != 'OK':
        stat = 'Bd zapisu wiadomoci do wysania: ' + stat
    return stat


# -------------------------------------------------------------------------------


def _WyslijSmsy(PolSQL, wiadomosci):
    global LogLevel, LogFile, LogType, Oper
    powtarzac = 20
    suffix    = '.sms'
    plik_blok = os.path.abspath(".blokada_modem_sms")
    modem     = None
    stat      = 'OK'
    plikKonf  = "rs-and.conf"
    param     = ('bramka_login', 'bramka_haslo', 'port_sms', 'czestotliwosc_modemu', 'usuwac_polskie_znaki', 'sim_pin')
    bledy     = []
    login, haslo, port_sms, czestotliwosc_modemu, usuwac_polskie_znaki, pin = PodajZKonf(plikKonf, param)

    usuwac_polskie_znaki = usuwac_polskie_znaki.lower() in ('yes', 'true', 'tak', '1', '')  # domyslnie usuwac
    if login == '' and port_sms == '':
        bledy.append('Nie skonfigurowano metody wysyki wiadomoci sms')
        sms_stat = 'Nie skonfigurowano metody wysyki wiadomoci sms'
        metoda = ''
        for wiadomosc in wiadomosci:
            sms_id, odbiorca, tresc = wiadomosc
            args = [sms_stat, metoda, sms_id]
            sql = """UPDATE rs_sms
                     SET stat=%s, proby=proby+1, wysylka=clock_timestamp(), metoda=%s
                     WHERE sms_id=%s;"""
            stat, dump, dump, dump = PolSQL.Execute(sql, 1, args, CzyDodawacBlad=True)
            if stat != 'OK':
                stat = 'Aktualizacji statusu wysyki: ' + stat
                bledy.append("{}\t{}\t{}\t{}".format(sms_id, sms_stat, metoda, stat))
            else:
                PolSQL.Commit()
        return bledy

    for wiadomosc in wiadomosci:
        sms_id, odbiorca, tresc = wiadomosc
        metoda = 'modem' if port_sms else 'bramka'

        if usuwac_polskie_znaki:
            tresc = UsunPolskieZnaki(tresc)

        if port_sms:
            try:
                odbiorca = '+'+odbiorca

                if modem is None:

                    czestotliwosc_modemu = int(czestotliwosc_modemu) if czestotliwosc_modemu else 9600

                    from sms.gsmmodem import GsmModem
                    from sms.gsmmodem.exceptions import TimeoutException, PinRequiredError, IncorrectPinError, CmsError

                    p = powtarzac
                    while p:
                        if os.path.isfile(plik_blok):
                            with open(plik_blok) as f:
                                pid = f.readlines()[0]
                                f.close()
                                if not check_pid(int(pid)):
                                    os.remove(plik_blok)
                                    break
                            stat = 'Modem jest zajety'
                            sleep(30)
                            p -= 1
                        else:
                            stat = 'OK'
                            break
                    if stat == 'OK':
                        with open(plik_blok, "w") as f:
                            f.write(str(os.getpid()))
                            f.close()

                        try:
                            modem = GsmModem(port_sms, czestotliwosc_modemu)

                            p = powtarzac
                            while p:
                                try:
                                    p -= 1
                                    modem.connect(pin)
                                    stat = 'OK'
                                    break
                                except PinRequiredError:
                                    stat = 'Blad kodu pin 1, skontaktuj sie z administratorem'
                                    modem = None
                                except IncorrectPinError:
                                    stat = 'Blad kodu pin 2, skontaktuj sie z administratorem'
                                    modem = None
                                except (TimeoutException, IOError) as e:
                                    if modem.alive:
                                        modem.close()
                                    stat = "Brak odpowiedzi modemu. Skontaktuj si z administratorem."
                                    modem = None
                                    sleep(5)
                                except CmsError:
                                    p = 0
                                    stat = 'OK'
                                    sleep(12)
                            try:
                                modem.waitForNetworkCoverage(5)
                            except TimeoutException:
                                stat = 'Modu sms nie jest w stanie poczy si z sieci. Skontaktuj si z administartorem.'
                                modem.close()
                                modem = None
                        except Exception, e:
                            stat = "Problem nawizywania poczenia z modemem: {}".format(e)
                            modem = None
                if modem is not None:
                    try:
                        sms = modem.sendSms(odbiorca, tresc)
                    except TimeoutException:
                        stat = 'Nie udao si wysa wiadomoci, przekroczono czas oczekiwania'
            except Exception, e:
                stat = "Wyjatek: " + str(e)
            if stat != 'OK':
                bledy.append("{}\t{}\t{}".format(sms_id, stat, metoda))
            args = [stat, metoda, sms_id]
            sql = """UPDATE rs_sms
                     SET stat=%s, proby=proby+1, wysylka=clock_timestamp(), metoda=%s
                     WHERE sms_id=%s;"""
            stat, dump, dump, dump = PolSQL.Execute(sql, 1, args, CzyDodawacBlad=True)
            if stat != 'OK':
                stat = 'Aktualizacji statusu wysyki: ' + stat
                bledy.append("{}\t{}\t{}".format(sms_id, stat, metoda))
            else:
                PolSQL.Commit()
        else:
            import requests, urllib
            stat = 'OK'

            war_pl_znaki = ''
            if not usuwac_polskie_znaki and UsunPolskieZnaki(tresc) != tresc:
                war_pl_znaki = "&encoding=UTF-8&unicode=1"
            login = urllib.quote(login.encode('utf8'))
            haslo = urllib.quote(haslo.encode('utf8'))
            tresc = urllib.quote(tresc.encode('utf8'))
            url = "https://api.gsmservice.pl/v5/send.php?login={login}{war_pl_znaki}&pass={haslo}&recipient={odbiorca}&message=" \
                  "{tresc}&msg_type=3".format(**locals())
            resp = requests.get(url)
            if resp.status_code != 200:
                resp = requests.get(url)
            if resp.status_code != 200:
                stat = "Bd wysyania wiadomoci, otrzymano kod odpowiedzi: "+str(resp.status_code)
            odpowiedz = resp.text.split('|')
            if odpowiedz[0] != 'OK':
                #WS 2022-09-22 - obsuga wybranych bdw
                kod_bledu = odpowiedz[1]
                tresc_bledu = ''
                slownik_bledow = {
                    '110': "nie wprowadzono treci wiadomoci",
                    '111': "wprowadzona tre wiadomoci zawiera niedozwolone znaki",
                    '112': "nieobsugiwany format kodowania treci wiadomoci (encoding)",
                    '116': "numer odbiorcy wiadomoci jest nieprawidowy",
                    '120': "numer, pod ktry prbowano wysa wiadomo jest obecnie nieobsugiwany",
                }
                stat = "Bd wysyania wiadomoci, kod bdu: "+str(kod_bledu)+' - '+\
                       (slownik_bledow[str(kod_bledu)] if str(kod_bledu) in slownik_bledow
                                                       else 'skontaktuj si z administratorem')

            args = [stat, metoda, sms_id]
            if stat != 'OK':
                bledy.append("{}\t{}\t{}".format(sms_id, stat, metoda))
            sql = """UPDATE rs_sms
                     SET stat=%s, proby=proby+1, wysylka=clock_timestamp(), metoda=%s
                     WHERE sms_id=%s;"""
            stat, dump, dump, dump = PolSQL.Execute(sql, 1, args, CzyDodawacBlad=True)
            if stat != 'OK':
                stat = 'Aktualizacji statusu wysyki: ' + stat
            else:
                PolSQL.Commit()

    if modem is not None and modem.alive:
        modem.close()
        os.remove(plik_blok)

    return bledy

# -------------------------------------------------------------------------------

#WS 2023-11-29
def TransformujZXSL(xml_tresc='', xsl_sciezka=''):
    parser = etree.XMLParser(no_network=False)
    xml = etree.XML(xml_tresc, parser)
    xsl = etree.parse(xsl_sciezka, parser)
    transform = etree.XSLT(xsl)
    return transform(xml)
