# -*- coding: iso8859_2 -*-
# -------------------------------------------------------------------------------
# Name:        rsmssql.py
# Purpose:     obsuga pocze z Microsoft SQL Server dla serwera rs_anakonda_server
#
# Author:      Aleksander Bargie
#
# Created:     2018/09/21
# RCS-ID:      $Id: rsmssql.py $
# Copyright:   (c) 2018
# Licence:     GNU GPL
# -------------------------------------------------------------------------------

# 2020-03-11 Wojciech Skowron - (11796) klasa RSPCMarketSQL suca do czenia z PC Market
# 2022-02-23 Aleksander Bargie (12901) Jeli zapytanie wywalio si, ZapisDoLogow w Execute

import pymssql
import rsglobal


class RSMSSQL:

    # ---------------------------------------
    # --- funkcja czca si z baz SQL
    # ---------------------------------------
    def PolaczZSQL(self, Operator, Haslo, Host, Port, Baza='anakonda'):
        try:
            self.PolaczenieMS = pymssql.connect(Host, Operator, Haslo, Baza, port=Port)
            return 'OK'
        except Exception, e:
            #rsglobal.ZapiszDoLogow('BLAD POLACZENIA Z SERWEREM MSSQL.: '+str(e), 1)
            print e
            return str(e)

    # -----------------------------------
    # --- funkcja rozczajca baz SQL
    # -----------------------------------

    def RozlaczZSQL(self):
        if self.PolaczenieMS is not None:
            self.PolaczenieMS.close()
            del self.PolaczenieMS
            self.PolaczenieMS = None


    def Execute(self, polecenie):
        desc  = []
        values = []
        stat  = 'OK'
        cursor = self.PolaczenieMS.cursor()
        try:
            cursor.execute(polecenie)
        except Exception, e:
            #AB 28.02.22r
            rsglobal.ZapiszDoLogow("Bd wykonania zapytania", 1)
            rsglobal.ZapiszDoLogow(polecenie, 1)
            stat = str(e)
            return stat, values, desc
        for row in cursor.fetchall():
            if desc == []:
                for i in xrange(len(row)):
                    desc.append(cursor.description[i][0])
            vals = []
            for i in xrange(len(desc)):
                k = str(type(row[i]))
                # str
                if k in ("<type 'unicode'>"):
                    #WS 2019-08-13
                    vals.append(unicode(row[i]))
                    # vals.append(str(row[i]))
                # int
                elif k in ("<type 'int'>"):
                    vals.append(int(row[i]))
                # bool
                elif k in ("bool","16"):
                    vals.append('bool')
                # num
                elif k in ("<class 'decimal.Decimal'>"):
                    vals.append(float(row[i]))
                # date
                elif k in ('<type \'datetime.datetime\'>'):
                    d = str(row[i])
                    if d[11:22] == "00:00:00.00":
                        d = d[0:10]
                    vals.append(d)
                # binary
                elif k in ("bin","17"):
                    vals.append('bin')
                # time
                elif k in ("time","timetz","1083","1184"):
                    vals.append('time')
                elif k in ("<type 'NoneType'>"):
                    vals.append(None)
                # unknown
                else:
                    # rsglobal.ZapiszDoLogow(u'Nieobsugiwany typ danych: '+k+' pole:'+desc[i])
                    vals.append(k)
                    self.wystapilBlad = True

            values.append(vals)
        return stat, values, desc


# -----------------------------------
# --- polczenie z PC Market
# -----------------------------------

class RSPCMarketSQL(RSMSSQL):
    def Polacz(self):
        plikKonf = "rs-and.conf"
        parametry = ('pcmoperator', 'pcmhaslo', 'pcmhost', 'pcmport', 'pcmbaza')
        lista = rsglobal.PodajZKonf(plikKonf, parametry)

        Logowanie = {}

        for param, wart in zip(parametry, lista):
            if wart == '':
                #WS 2020-07-14
                return "Brak parametru '" + param + "' w pliku konfiguracyjnym serwera."
                # return "Brak parametru '" + param + "' w pliku konfiguracyjnym serwera.", None
            else:
                Logowanie[param] = wart

        stat = self.PolaczZSQL(Logowanie['pcmoperator'], Logowanie['pcmhaslo'], Logowanie['pcmhost'],
                               Logowanie['pcmport'], Baza=Logowanie['pcmbaza'])

        if stat == 'OK':
            rsglobal.ZapiszDoLogow('Pomylne poczono z serwerem PC Market')
            return stat
        else:
            stat = 'Dane logowania niepoprawne lub wystpi bd poczenia z serwerem PC Market:\n' + stat
            rsglobal.ZapiszDoLogow(stat)
            return stat

    def Rozlacz(self):
        self.RozlaczZSQL()
