# -*- coding: iso8859_2 -*-

#-------------------------------------------------------------------------------
# Name:        rssql.py
# Purpose:     obsuga pocze z PostgreSQL dla serwera rs_anakonda_server
#
# Author:      Jan Raburski
#
# Created:     2004/03/06
# RCS-ID:      $Id: rssql.py $
# Copyright:   (c) 2002
# Licence:     GNU GPL
#-------------------------------------------------------------------------------
# popr. dla raportw z serwera zapas. (mantis: 0002191) - Jan Kaliszewski, 2009-07-09
# 2010.12.03 MR - dodanie typw do RSTableRowset oraz czyszczene kodu ze zbdnych zakomentowanyh sekcji (4122)
# 2013-01-02 Mariusz Rajfura (6385) uzycie isinstance w RSTableRowset
# 2013-03-19 Mariusz Rajfura (6633) nowy typy danych: 1186,interval,701 (double precision)
# 2018-01-09 Aleksander Bargie (10885) nowe typy danych: timetz, timestamptz
# 2020-04-30 Wojciech Skowron (11940) RSTableRowset2
# 2021-06-09 Aleksander Bargie (12522) Dostawy - zmiana sposobu sprawdzania wersji postgresq na dziaajc z wersjami
#                                                 10 i pniej
# 2021-01-03 Wojciech Skowron   (12761) Metoda Mogrify() w RSSQL
# 2022-02-07 Wojciech Skowron   (12849) CzyDodawacBlad=True w Execute()
# 2022-02-23 Aleksander Bargie (12901) Jeli zapytanie wywalio si, ZapisDoLogow w Execute
# 2022-06-15 Aleksander Bargie (13059) Bd przy dodawaniu kontrahenta
# 2022-12-06 Aleksander Bargie (13266) Obsuga typu current_time w rssql
# 2023-07-11 Aleksander Bargie (13481) RSSQL "Dane nie moga zawierac ciagu unicode" -> automatycznie rzutowa unicode na str

import rsglobal
import os
import signal
import types
import string
import subprocess

try:
    import psycopg
except:
    import psycopg2 as psycopg

#-------------------------------------------------------------------------------
plZnaki = '󳡯ӣ'
podmianki = {}
for znak in plZnaki:
    podmianki[repr(znak).strip("'")] = znak

class RSSQL:
    def __init__(self):
        self.bylo_cos_robione = False
        self.SelectForUpdate = {}       # sownik z danymi, ktre maj by blokowane podczas rnych funkcji
        self.kodowanie = None
        self.CzyCommitRollback = True   # jeli ustawiona na False, to nie robi si commit i rollback

    #---------------------------------------
    #--- funkcja czca si z baz SQL
    #---------------------------------------

    def PolaczZSQL(self, Operator, Haslo, Host, Port, System, Baza='anakonda'):
        try:
            connectstring="user=%s password=%s host=%s port=%s dbname=%s"%(str(Operator),str(Haslo),str(Host),str(Port),str(Baza))
            self.Polaczenie = psycopg.connect(connectstring)
            self.Polaczenie.set_client_encoding('iso8859_2')
            self.Polaczenie.set_isolation_level(1)
            stat, kodowanie, dump, dump = self.Execute("SHOW SERVER_ENCODING;", 5)
            if stat != 'OK':
                raise Exception("Problem podczas weryfikacji kodowania serwera: "+ stat)
            elif kodowanie != 'LATIN2':
                self.kodowanie = kodowanie
            else:
                self.kodowanie = None

            return 0
        except StandardError, inst:
            rsglobal.ZapiszDoLogow('BLAD POLACZENIA Z SERWEREM SQL.: '+str(inst),1)
            # jeli jest nazwa uytkownika to znaczy, e bd hasa
            if '"'+Operator+'"' in str(inst):
                return 7
            # jeli jest host to znaczy, e bd znajdowania bazy
            if '"'+str(Host)+'"' in str(inst):
                return 8
            # inne
            else:
                return 1

    #-----------------------------------
    #--- funkcja rozczajca baz SQL
    #-----------------------------------

    def RozlaczZSQL(self):
        if self.Polaczenie != None:
            self.Polaczenie.close()
            del self.Polaczenie
            self.Polaczenie = None
            self.SelectForUpdate = {}

#-------------------------------------------------------------------------------

    def WykonajSelectForUpdate(self):
        stat='OK'
        # prba zaoenia blokad
        for kl in self.SelectForUpdate:
            if stat=='OK':
                try:
                    self.kursSFU=self.Polaczenie.cursor()
                    self.kursSFU.execute(self.SelectForUpdate[kl])
                except Exception, inst:
                    self.Polaczenie.rollback()
                    stat="Nie mona zablokowa wybranego dokumentu typu "+self.STypyBlokad[kl]+' '+str(inst)

        # jeli si nie udao to cigamy swoje dotychczasowe blokady (moliwe e wiszce z innego procesu)
        if stat<>'OK':
            try:
                stat='OK'
                # pole procpid na pid zmienio si od wersji 9.2 postgresa
                #AB 09.06.21r
                SQL = "SHOW server_version_num;"
                stat,dane,dump,dump=self.Execute(SQL,3)
                if stat == 'OK':
                    if dane[0][0] < 90200:
                        PolePid = 'procpid'
                        WarQuery = "current_query='<IDLE> in transaction'"
                    else:
                        PolePid = 'pid'
                        WarQuery = "state='idle in transaction'"

                # pobranie procesw, ktre maj tego usera co ja i blokuj co
                SQL="SELECT "+PolePid+" FROM pg_stat_activity WHERE usename=getpgusername()"
                SQL+=" AND "+WarQuery+" AND NOT "+PolePid+"="+str(os.getpid())
                stat,dane,dump,dump=self.Execute(SQL,3)

                rsglobal.ZapiszDoLogow('STat '+stat,1)
                if stat=='OK':
                    if len(dane)>0:
                        # ubicie znalezionych procesw
                        for poz in dane:
                            subprocess.call(['kill', str(poz[0])])

            except Exception,e:
                stat='Bdy z ubijaniem procesw '+str(e)

        if stat=='OK':
            # po wyczyszczeniu ponowna prba
            for kl in self.SelectForUpdate:
                if stat=='OK':
                    try:
                        self.kursSFU=self.Polaczenie.cursor()
                        self.kursSFU.execute(self.SelectForUpdate[kl])
                    except Exception, inst:
                        self.Polaczenie.rollback()
                        stat="Nie mona zablokowa wybranego dokumentu typu "+self.STypyBlokad[kl]+' '+str(inst)

        return stat

#-------------------------------------------------------------------------------

    def Commit(self):
        if self.CzyCommitRollback:  #JR 2023-11-11 (13603)
            self.Polaczenie.commit()
        return self.WykonajSelectForUpdate()

#-------------------------------------------------------------------------------

    def CommitCzysty(self):
        self.SelectForUpdate = {}
        if self.CzyCommitRollback:  #JR 2023-11-11 (13603)
            self.Polaczenie.commit()

#-------------------------------------------------------------------------------

    def Rollback(self):
        if self.CzyCommitRollback:  #JR 2023-11-11 (13603)
            self.Polaczenie.rollback()
        return self.WykonajSelectForUpdate()

#-------------------------------------------------------------------------------

    def przygotuj_polecenie(self, polecenie):
        if self.kodowanie is not None:
            polecenie = polecenie.decode('iso8859-2')
            for x in podmianki:
                polecenie = polecenie.replace(x, podmianki[x])
        return polecenie

#-------------------------------------------------------------------------------

    #WS 2022-02-07 - CzyDodawacBlad domylnie na True
    def Execute(self,polecenie,Funkcja,dane=None,CzyCommit = False,CzyDodawacBlad=True): #Funkcja - 1 - wraca {OK,BLAD}; 2 - wraca z fetchall,3 - wraca z fetchall() i z descr()
        self.bylo_cos_robione = True
        # rsglobal.ZapiszDoLogow('rssql POLECENIE: ' + str(polecenie), 5)
        # rsglobal.ZapiszDoLogow('rssql funkcja: ' + str(Funkcja), 5)
        # rsglobal.ZapiszDoLogow('rssql dane: ' + str(dane), 5)
        polecenie = self.przygotuj_polecenie(polecenie)
        rowset = None
        try:
            self.kurs=self.Polaczenie.cursor()
            stat = 'OK'
            wyn1 = None
            wyn2 = None
            wyn3 = None
            # unicode w danych w psycopg moe powodowa bdy, nawet SIGSEGV przy polskich znakach, moe zaley od wersji
            if dane:
                assert isinstance(dane, (list, tuple, dict)), "Dane musza byc sekwencja lub slownikiem, jest %s" % type(dane)
                #AB 2023-07-11
                dane = [str(x) if isinstance(x, unicode) else x for x in dane]
                # unicodeWDanych = filter(lambda x: isinstance(x, unicode), dane)
                # assert not unicodeWDanych, "Dane nie moga zawierac ciagu unicode: %r" % unicodeWDanych
            polecenie = polecenie.encode('iso8859-2', 'replace')  #AB 15.06.22r
            if Funkcja == 1:
                # wykonanie polecenia
                if dane==None:
                    self.kurs.execute(polecenie)
                else:
                    self.kurs.execute(polecenie,dane)
                #rsglobal.ZapiszDoLogow('rssql POLECENIE: ' + str(polecenie), 5)
                #rsglobal.ZapiszDoLogow('rssql STATUS: ' + str(self.kurs.statusmessage), 5)
                wyn1 = self.kurs.lastrowid & 0xffffffff
                wyn2 = str(self.kurs.statusmessage).strip()
                if CzyCommit:
                    stat=self.Commit()
            elif Funkcja == 2:
                # pobranie danych 'fetchall z polecenia SELECT
                if dane==None:
                    self.kurs.execute(polecenie)
                else:
                    self.kurs.execute(polecenie,dane)
                rowset = RSTableRowset(self.kurs.fetchall(),self.kurs.description)
                wyn1, dump = rowset.getlist()
            elif Funkcja == 3:
                # pobranie wszystkich danych oraz opisu kursora z polecenia SELECT
                if dane==None:
                    self.kurs.execute(polecenie)
                else:
                    self.kurs.execute(polecenie,dane)
                rowset = RSTableRowset(self.kurs.fetchall(),self.kurs.description)
                wyn1,wyn2 = rowset.getlist()
            elif Funkcja == 4:
                # pobranie wartoci typu INT z polecenia SELECT
                self.kurs.execute(polecenie)
                rowset = RSTableRowset(self.kurs.fetchmany(2),self.kurs.description)
                liczba, dump = rowset.getlist()
                wyn1 = int(liczba[0][0])
            elif Funkcja == 5:
                # pobranie wartoci typu TEXT z polecenia SELECT
                self.kurs.execute(polecenie)
                rowset = RSTableRowset(self.kurs.fetchmany(2),self.kurs.description)
                tekst, dump = rowset.getlist()
                if tekst == None or tekst == []:
                    wyn1 = ''
                else:
                    wyn1 = tekst[0][0]
            elif Funkcja == 6:
                # pobranie opisu kodu z listy
                self.kurs.execute(polecenie)
                rowset = RSTableRowset(self.kurs.fetchmany(2),self.kurs.description)
                tekst, dump = rowset.getlist()
                if tekst == None or tekst == []:
                    wyn1 = ''
                else:
                    wyn1 = tekst[0][0]
            elif Funkcja == 7:
                # pobranie wartosci binarnej
                if dane==None:
                    self.kurs.execute(polecenie)
                else:
                    self.kurs.execute(polecenie,dane)
                tekst=self.kurs.fetchall()
                if tekst == None or tekst == []:
                    wyn1 = None
                else:
                    wyn1 = tekst[0][0]
        except Exception, inst:
            import traceback
            rsglobal.ZapiszDoLogow(traceback.format_exc(), 1)
            if 'deadlock' in str(inst).lower():
                stat=' Wystpi bd zakleszczenia danych. Naley ponownie uruchomi funkcj.'
            else:
                stat = 'BLAD'
                if CzyDodawacBlad:
                    stat+=' '+str(inst)
            if dane is not None:  #AB 28.02.22r
                try:  # czasem powodem bdu moze by bedne uzupenienie danych, dlatego try
                    polecenie = self.Mogrify(polecenie, dane)
                except Exception, e:  #AB 15.06.22r str na unicode
                    rsglobal.ZapiszDoLogow(u'Blad obslugi danych do zapytania: '+unicode(e), 1)
                    rsglobal.ZapiszDoLogow(u'dane: '+unicode(dane), 1)

            rsglobal.ZapiszDoLogow(u'Zapytanie : '+unicode(polecenie), 1)
            rsglobal.ZapiszDoLogow(u'Blad : '+unicode(inst), 1)
            self.Rollback();

        if rowset is not None and rowset.wystapilBlad:
            rsglobal.ZapiszDoLogow("RSTableRowset: zapytanie bylo\n%s  |%r" % (polecenie, dane), 1)
            
        self.kurs.close()
        return stat,wyn1,wyn2,wyn3

#-------------------------------------------------------------------------------

    def Dekoduj(self,polecenie):
        nowe = string.replace(polecenie,",u'",",'")
        nowe = string.replace(nowe,"(u'","('")
        nowe = string.replace(nowe,"[u'","['")
        nowe = string.replace(nowe,", u'",",'")
        return nowe

#-------------------------------------------------------------------------------

    def Koduj(self,polecenie):
        nowe = string.replace(polecenie,",'",",u'")
        nowe = string.replace(nowe,"('","(u'")
        nowe = string.replace(nowe,"['","[u'")
        nowe = string.replace(nowe,", '",",u'")
        return nowe

#-------------------------------------------------------------------------------

    def Binary(self,plik):
        return psycopg.Binary(plik)

#-------------------------------------------------------------------------------

    #WS 2021-01-03
    def Mogrify(self, polecenie, parametry):
        with self.Polaczenie.cursor() as kurs:
            return kurs.mogrify(polecenie, parametry)

#---------------------------------------------------------------------------

class RSFieldNameError(Exception):
  def __init__(self, value):
    self.value = value
  def __str__(self):
    return `"""Field name '%s' does not exist""" % self.value`

#---------------------------------------------------------------------------

class RSTableRow(tuple):
  # class for each row
  
  def __getattr__(self,i):
    return tuple.__getitem__(self,self.__Field2Index__(i))

  def __call__(self,i):
    return tuple.__getitem__(self,self.__Field2Index__(i))

  def __getitem__(self,i):
    if str(type(tuple.__getitem__(self,self.__Field2Index__(i)))) == "<type 'libpq.PgBoole>":
        j = str(tuple.__getitem__(self,self.__Field2Index__(i)))
        if j == 't':
            j = True
        else:
            j = False
        return j

    return tuple.__getitem__(self,self.__Field2Index__(i))


  def __Field2Index__(self,i):
    return i

#---------------------------------------------------------------------------


class RSTableRowset(list):
  # list to hold the rows
  
  def __init__(self,rowset,description):
    # save the description as is
    self.description = RSTableRow(description)
    self.description.__Field2Index__ = self.__fieldToIndex
    
    # Create the list and dict of fields
    self.fields = []
    self.__fieldDict = {}
    for f in range(len(description)):
      if isinstance(description[f],(list,tuple)):
        self.__fieldDict[description[f][0].lower()] = f
        self.fields.append( description[f][0].lower())
      else:
        self.__fieldDict[description[f].lower()] = f
        self.fields.append( description[f].lower())
    self.wystapilBlad = False
    
    self.descr = []
    for f in range(len(description)):
        w=[]
        w.append(description[f][0])
        k = str(description[f][1])

        #AB
        if k in ("str","char","varchar","timestamp","timestamptz","name","text","bpchar","1042","1043","25","19","705",
                 "1186","interval","869"):
            w.append( 'str')
        elif k in ("int","int2","int4","integer","int8","21","23","20","26"):
            w.append( 'int')
        elif k in ("bool","16"):
            w.append ('bool')
        elif k in ("numeric","1700","701"):
            w.append( 'numeric')
        elif k in ("date","1082","1114"):
            w.append('date')
        elif k in ("bin","17"):
            w.append('bin')
        #AB 2022-12-06
        elif k in ("time","timetz","current_time","1083","1184","1266"):
            w.append('time')
        else:
            rsglobal.ZapiszDoLogow(u'Nieobsugiwany typ w init: '+k+' pole:'+description[f][0],1)
            w.append(k)
            self.wystapilBlad = True
        w.append(description[f][2])
        w.append(description[f][3])
        w.append(description[f][4])
        w.append(description[f][5])
        w.append(description[f][6])
        list.append(self.descr,w)

    # Add all the rows
    self.dane = []
    for r in rowset:
      self.append(r)

#-------------------------------------------------------------------------------

  def append(self,new):
    # Create a new record
    wiersz=[]
    l = len(self.descr)
    for j in range(l):
        if self.descr[j][1]=='str':
            wiersz.append(unicode(str(new[j]),'iso8859_2'))
        elif self.descr[j][1]=='int':
            d = str(new[j])
            if d == 'None':
                d = '0'
            wiersz.append(int(d))
        elif self.descr[j][1]=='bool':
            wiersz.append(bool(new[j]))
        elif self.descr[j][1]=='bin':
            wiersz.append(new[j])
        elif self.descr[j][1]=='numeric':
            d = str(new[j])
            if d == 'None':
                d = '0'
            wiersz.append(float(d))
        elif new[j]==None:
            wiersz.append('None')
        elif self.descr[j][1]=='date':
            d = str(new[j])
            if d[11:22]=="00:00:00.00":
                d = d[0:10]
            wiersz.append(d)
        elif self.descr[j][1]=='time':
            d = str(new[j])
            d = d[0:5]
            wiersz.append(d)
        else:
            wiersz.append('None')
    list.append(self.dane,wiersz)
    return

#-------------------------------------------------------------------------------

  def getlist(self):
    return self.dane, self.descr

#-------------------------------------------------------------------------------

  def delete(self,i):
    list.remove(self,i)
    return

#-------------------------------------------------------------------------------

  # Look up the field and return the index  
  def __fieldToIndex(self,field):
        try:
            return int(field)
        except:
            pass
        try:
            return self.__fieldDict[field.lower()]
        except:
            raise RSFieldNameError, field

#-------------------------------------------------------------------------------

  def GetFieldNo(self,field):
      return self.__fieldToIndex(field)

#-------------------------------------------------------------------------------

  def GetDescription(self,field):
      return self.descr[self.__fieldToIndex(field)]
  
#-------------------------------------------------------------------------------

  def GetFieldName(self,field):
      return self.descr[self.__fieldToIndex(field)][0]
      
#-------------------------------------------------------------------------------

  def GetFieldType(self,field):
      return self.descr[self.__fieldToIndex(field)][1]

#-------------------------------------------------------------------------------

  def GetFieldDisplaySize(self,field):
      return self.descr[self.__fieldToIndex(field)][2]

#-------------------------------------------------------------------------------

  def GetFieldInternalSize(self,field):
      return self.descr[self.__fieldToIndex(field)][3]

#-------------------------------------------------------------------------------

  def GetFieldPrecision(self,field):
      return self.descr[self.__fieldToIndex(field)][4]

#-------------------------------------------------------------------------------

  def GetFieldScale(self,field):
      return self.descr[self.__fieldToIndex(field)][5]

#-------------------------------------------------------------------------------

  def GetFieldNullOk(self,field):
      return self.descr[self.__fieldToIndex(field)][6]

#-------------------------------------------------------------------------------

  def GetVal(self,row,field):
      wyn= self.dane[row][self.GetFieldNo(field)]
      if str(type(wyn)) =="<type 'unicode'>":
            wyn=wyn.encode('iso-8859-2')
      return wyn

#---------------------------------------------------------------------------


#WS 2020-04-30
# RSTableRowset2 pozwala tak jak RSTableRowset po stronie klienta odwoywa si do pl uywajc nazwy kolumny (11940)
class RSTableRowset2(RSTableRowset):
    def append(self, new):
        # Nowy rekord
        TR = RSTableRow(new)
        # Przekazanie do rekordu funkcji, ktra odnajdzie indeks kolumny po nazwie
        TR.__Field2Index__ = self._RSTableRowset__fieldToIndex
        # Dodanie rekordu do self
        list.append(self, TR)

        super(RSTableRowset2, self).append(new)
