#! /usr/bin/env python
# -*- coding: iso8859_2 -*-
#
# Skrypt adujcy z podanego katalogu kod klienta PAN majcy by rdem aktualizacji
#
# Autor: Jan Kaliszewski
# Data: 2010-05-18
# Licencja: GNU GPL
# Copyright: (c) 2010 Record System


#
# Importy

import anydbm
import errno
import fcntl
import logging
import os
import os.path
import random
import shelve
import shutil
import sys
import time


#
# Ustawienia

nazwa_pliku_log = u'pan--ladowanie_podstawy_aktualizacji.log'
poziom_logowania = logging.DEBUG
#poziom_logowania = logging.INFO
liczba_prob = 3
interwal = 2

# nazwy katalogw i plikw zwizanych z aktualizacjami klienta PAN
# (musz by identyczne z analogicznymi w rs_anakonda_server_pan.py):
podkat_pomocniczy = u'.aktualizacje_pan'
podkat_cache_aktualizacji = u'cache_aktualizacji'
podkat_zrodel_klienta = u'podstawa_aktualizacji'
podkat_poprz_zrodel_klienta = u'podstawa_porownania'
nazwa_blokady_kat_akt = u'.' + podkat_zrodel_klienta + u'--blokada.flock'
nazwa_pliku_info_o_wersjach = u'info_o_wersjach'


#
# Przygotowanie

if len(sys.argv) != 2:
    print >>sys.stderr, u'Sposb uycia: %s KATALOG_RDOWY' % sys.argv[0]
    sys.exit(2)

kat_zrodlowy = os.path.abspath(sys.argv[1])
kat_serwera = os.path.dirname(os.path.abspath(sys.argv[0]))
kat_pom = os.path.join(kat_serwera, podkat_pomocniczy)
kat_tymczas = os.path.join(kat_pom, '.tymczas-%d' % random.randint(1, sys.maxint))
kat_akt = os.path.join(kat_pom, podkat_zrodel_klienta)
sciezka_blokady_akt = os.path.join(kat_pom, nazwa_blokady_kat_akt)
sciezka_pliku_log = os.path.join(kat_serwera, nazwa_pliku_log)

logging.basicConfig(filename=sciezka_pliku_log, level=poziom_logowania,
                    format=(sys.argv[0] + ': %(asctime)s %(levelname)-8s %(message)s'),
                    datefmt='%Y-%m-%d %H:%M:%S')


#
# Utworzenie brakujcych katalogw

try:
    os.mkdir(kat_pom, 0700)
except OSError, exc:
    if exc.errno != errno.EEXIST:
        logging.exception('Nie udao si utworzy staego katalogu pomocniczego'
                          ' dla aktualizacji klienta PAN: %s -- bo:', kat_pom)
        print >>sys.stderr, (u'Nie udao si: brak moliwoci utworzenia staego'
                             u' katalogu pomocniczego %s -- szczegy w %s'
                             % (kat_pom, sciezka_pliku_log))
        sys.exit(1)
else:
    logging.warning('Nie znaleziono staego katalogu pomocniczego dla aktualizacji'
                    ' klienta PAN: %s -- teraz zosta utworzony', kat_pom)

kat_poprz = os.path.join(kat_pom, podkat_poprz_zrodel_klienta)
try:
    os.mkdir(kat_poprz, 0700)
except OSError, exc:
    if exc.errno != errno.EEXIST:
        raise
else:
    logging.warning('Nie znaleziono katalogu dla poprzednich wersji '
                    ' klienta PAN: %s -- teraz zosta utworzony', kat_poprz)
                           
kat_cache = os.path.join(kat_pom, podkat_cache_aktualizacji)
try:
    os.mkdir(kat_cache, 0700)
except OSError, exc:
    if exc.errno != errno.EEXIST:
        raise
else:
    logging.warning('Nie znaleziono katalogu dla cache aktualizacji'
                    ' klienta PAN: %s -- teraz zosta utworzony', kat_cache)

sciezka_info_o_wersjach = os.path.join(kat_pom, nazwa_pliku_info_o_wersjach)
try:
    shelve.open(sciezka_info_o_wersjach, 'r', protocol=2).close()
except anydbm.error:
    logging.warning('Nie znaleziono pliku-sownika z info o wersjach'
                    ' klienta PAN: %s -- zostanie teraz utworzony...',
                    sciezka_info_o_wersjach)
    shelve.open(sciezka_info_o_wersjach, 'c', protocol=2).close()


#
# Waciwy skrypt

logging.info('Zaczynamy waciwe dziaanie...')
try:
    blokada_kat_akt = open(sciezka_blokady_akt, 'w')
    try:
        for proba in xrange(liczba_prob):
            try:
                logging.debug('Zakadamy blokad...')
                # zakadamy blokad wyczn (LOCK_EX) -- w sposb nieblokujcy (LOCK_NB)
                fcntl.flock(blokada_kat_akt, fcntl.LOCK_EX | fcntl.LOCK_NB)
                logging.debug('=> zaoona')
            except IOError:
                if proba < liczba_prob - 1:
                    logging.debug('Nie udao si zaoy blokady, sprbujemy ponownie...')
                    print >>sys.stderr, u'Nie udao si zaoy blokady, sprbujemy ponownie...'
                    time.sleep(interwal)
                    continue
                else:
                    logging.exception('Nie udao si zaoy blokady %s, bo:',
                                      sciezka_blokady_akt)
                    print >>sys.stderr, (u'Nie udao si: blokada %s zajta,'
                                         u' sprbuj pniej.' % sciezka_blokady_akt)
                    sys.exit(1)
            else:
                try:
                    logging.debug('Kopiujemy z katalogu rdowego %s do katalogu'
                                  ' tymczasowego %s...', kat_zrodlowy, kat_tymczas)
                    shutil.copytree(kat_zrodlowy, kat_tymczas)
                    logging.debug('=> skopiowane')
                    try:
                        logging.debug('Kasujemy docelowy katalog aktualizacji %s...', kat_akt)
                        shutil.rmtree(kat_akt)
                    except EnvironmentError, exc:
                        if exc.errno == errno.ENOENT:  # (katalog nie istnieje)
                            logging.warning('Nie byo katalogu docelowego %s', kat_akt)
                        else:
                            raise
                    else:
                        logging.debug('=> skasowany')
                    logging.debug('Zamieniamy katalog tymczasowy %s na docelowy %s...',
                                  kat_tymczas, kat_akt)
                    os.rename(kat_tymczas, kat_akt)
                    logging.debug('=> zamieniony')
                except:
                    logging.exception('Nie udao si zaadowa aktualizacji z %s, bo:', kat_zrodlowy)
                    print >>sys.stderr, (u'Bd podczas adowania podstawy'
                                         u' aktualizacji -- szczegy w %s' % sciezka_pliku_log)
                    print >>sys.stderr, (u'UWAGA! Podstawa aktualizacji klientw'
                                         u' PAN moga zosta uszkodzona -- naley doprowadzi rzecz'
                                         u' do porzdku czyli do poprawnego zaadowania podstawy'
                                         u' aktualizacji.')
                    if os.path.isdir(kat_tymczas):
                        logging.debug('Pozosta katalog tymczasowy, kasujemy go...')
                        shutil.rmtree(kat_tymczas)
                        logging.debug('=> skasowany')
                    sys.exit(1)
                else:
                    logging.info('Skrypt wykona zadanie -- zaadowa podstaw'
                                 ' aktualizacji z katalogu %s', kat_zrodlowy)
                    break
    finally:
        logging.debug('Zdejmujemy blokad...')
        blokada_kat_akt.close()
        logging.debug('=> zdjta')
finally:
    logging.info('Koczymy')
