# -*- coding: ISO8859-2 -*-
#-------------------------------------------------------------------------------
# Name:             fs_dokumenty_magazynowe.py
# Purpose:          operacje na dokumencie magazynowym
# Author:           Piotr Witkowski
# Created:          2016/01/28
# Copyright:        (c) 2016 RecordSystem
# Licence:          GNU GPL
#-------------------------------------------------------------------------------
#2009-08-31 Mariusz Rajfura (2431) (przejscie na fun nry:5,6,7,40,41)
#2010-07-09 Dariusz Boduch  (3609) fun nr 43: zmiana filtrowania dla przesylek (przy multiwyborze - tylko dokumenty rozchodowe i bez przypisanej przesylki
#2010-08-02 Dariusz Boduch  (3767) pobranie danych przesyki, do ktrej jest przypisany dokument mag. 
#2011-02-02 Mariusz Rajfura (4223) nowe pole 'produkt_id'
#2011-02-18 Mariusz Rajfura (4229) modyf.ilosci.prod.w zam.odb.na pdst.produkcji G->Z,Z->G 
#2011-02-21 Micha Raburski (4285) poprawka w porwnywaniu iloci na fakturze przypisanej i na dokumencie
#2011-02-21 Micha Raburski (4230) zmiany - dokument typu 'C'. Dodana funkcja 50
#2011-04-09 Mariusz Rajfura (4472) kontrola zaleglosci tylko dla dokumentu typu 'W'
#2011-04-11 Michal Raburski, Mariusz Rajfura (4458) - wirtualne artykuly-przyjecia
#2011-04-13 Mariusz Rajfura (4489) popr.pobrania lokacji 
#2011-05-05 Mariusz Rajfura (4567) sprawdzanie zmiennej 'lokacja_prz' przy zamykaniu z dok.rozch.W->G 
#2011-11-07 Mariusz Rajfura (5181) prawidlowa kolejnosc sprawdzania stat 
#2011-11-09 Mariusz Rajfura (5184) blokada zamykania dok.typu C bez pozycji 
#2011-11-13 Micha Raburski (6251) blokada otwierania dok. przych jeli sprzeczna z rezerwacj 
#2013-02-08 Mariusz Rajfura (6513) pobranie stanu dokumentu (Funkcja==68)
#2013-02-21 Mariusz Rajfura (6536) klarowny komunikat bledu przy probie usuwania dok.mag powiazanego z planem produkcji
#2013-07-13 Micha Raburski (7031) dynamiczny przelicznik w dokumentach magazynowych
#2013-07-31 Mariusz Rajfura (7075,7114) tworz.korekty dok.mag.; kontrole;wizualizacja korekty na gridzie
#2013-09-03 Mariusz Rajfura (7122) porzadki w przesylkach: kontrola listy dok.(Funkcja==69)
#2014-02-04 Micha Raburski (7650) dodanie filtra 'zmienione'
#2014-07-21 Micha Raburski (8144) filtr na miejsce dostawy
#2014-09-30 Micha Raburski (8394) uwzgldnianie filtra na limity przydatnoci
#2016-01-28 Piotr Witkowski (9424) Dokumenty magazynowe nowy grid
#2016-07-20 Piotr Witkowski (9886) dodanie komunikatu o zmianie artykulu - nowa funkcja 72
#2016-08-23 Piotr Witkowski (9985) Receptury - kontrola aktualnoci
#2016-10-29 Micha Raburski (10035) konsolidacja artykuw w dokumentach typu M
#2017-08-22 Piotr Witkowski (10679)  Sprawdzi czy wpisanie daty_do w projektach blokuje mozliwosc wprowadzania
#2017-10-31 Rafa Woniak (10734) zmiana zapisu do logw z 1 na 5
#2018-12-17 Aleksander Bargie (11333) Otwieranie dokumentu magazynowego PR
#2020-03-10 Jan Raburski (11876)  Zamykanie i otwieranie d.m. typu 'W' z magazynw detalicznych
#2020-04-01 Aleksander Bargie (11924)  BLAD podczas zamykania dokumentu magazynowego.
#2020-05-19 Wojciech Skowron (11959) Zamykanie dok. mag. z artykuami z pust kategori
#2021-10-14 Aleksander Bargie (12668)  Wywietl dokumenty magazynowe, niezamknite, przekroczono limit gbokoci stosu
#2023-01-12 Jan Raburski (13286) Zmiana w zamykaniu d.m. przychodowych detalicznych
#2023-03-02 Wojciech Skowron (13363) Blokada otwierania dokumentw typu 'I'
#2023-06-15 Aleksander Bargie (13465)  Zamykajc DM w mag. det. na dat przed otwartym Spisem z Natury, jeli wydawany jest zamiennik, nie aktualizuje jego iloci
#2023-06-28 Aleksander Bargie (13471) Zamykanie dokumentu w magazynie handlowym trwa ponad 30 minut gdy obecny jest w nim niezamknity spis z natury
#2023-08-25 Jan Raburski (13513) nie wpiswanie orginalu jesli taki sam
#2023-08-25 Aleksander Bargie (13513) Problem z korygowaniem PW/RW jeeli magazyn RW jest detaliczny
#2023-09-06 Aleksander Bargie (13535) Bd zamykania dokumentu magazynowego ZD w SPC
#2023-10-19 Aleksander Bargie (13586) [MG] Kolumna 'Il.w jm2' na gridzie dokumentw magazynowych
#2023-11-21 Jan Raburski (13602) Dodatkowa kontrola na typ artykulu przy zamykaniu

#funkcje:
# 5 - POBRANIE DANYCH DO GRIDA
# 6 - usuwanie dokumentu utworzonego z paragonu,  POBRANIE DANYCH DO GRIDA
# 7 - powizanie dokumentw, POBRANIE DANYCH DO GRIDA
# 8 - W --> G, POBRANIE DANYCH DO GRIDA
# 9 - ZAMYKANIE DOKUMENTU G --> Z,POBRANIE DANYCH DO GRIDA
# 10 - OTWIERANIE DOKUMENTU DO EDYCJI,POBRANIE DANYCH DO GRIDA
# 11-  OTWARCIE DOKUMENTU Z --> G,POBRANIE DANYCH DO GRIDA
# 15 - SPRAWDZENIE CZY BD ZAMYKANE SPISY Z NATURY
# 16 - usuwanie dokumentu utworzonego z paragonu
# 18 - PRZYGOTOWANIE DOKUMENTU DO ZAMKNIECIA W --> G bez pobierania danych
# 19 - ZAMYKANIE DOKUMENTU G --> Z bez pobierania danych
# 20 - G->W bez pobierania danych
# 21 - OTWARCIE DOKUMENTU Z --> G bez pobierania danych
# 26 - usuwanie ze spisu z natury
# 28 - W --> G (rozbijanie/scalanie pozycji dokumentu w stanie G. Pomija cz kontroli i zmian)
# 29 - ZAMYKANIE DOKUMENTU G --> Z (ze spisu z natury (nie zaznacza dokumenty_po w spisie))
# 30 - OTWIERANIE DOKUMENTU DO EDYCJI (tylko scalanie pozycji
# 31 - OTWARCIE DOKUMENTU Z --> G ze spisu z natury
# 40 - POBRANIE DANYCH FAKTURY Z DOKUMENTU MAGAZYNOWEGO
# 41 - ACZENIE DOKUMENTU Z KOREKT, POBRANIE DANYCH DO GRIDA,
# 42 - POBRANIE LISTY NIEZGODNOSCI W ILOSCIACH ARTYKULOW PRZY ZAMYKANIU (bez wczytania grida)
# 50 - sprawdzenie typu dokumentu
# 66 - Sprawdzenie limitu
# 67 - Utworzenie przesylki z dokumentu, POBRANIE DANYCH DO GRIDA
# 68 - POBRANIE STANU DOKUMENTU
# 69 - KONTROLA DOKUMENTOW DO UTWORZENIA WYSYLKI
# 71 - odwiazanie dokumentw,  POBRANIE DANYCH DO GRIDA
# 72 - SPRAWDZENIE CECHY NA POZYCJACH Z ARTYKULAMI

#########################################
# Na potrzeby obslugi magazynu 'detaliczny' oprogramowane osobne uproszczone funkcje
# WYDAWANIA (tylko dokumenty 'W') z magazynu (bez cech, lokacji, terminw itd.)
# za to umoliwiajce wydanie czego nie ma w magazynie
# Aby nie zmieniac nic w kliencie dla wybranych funkcji zamykania / otwierania
# zmieniany jest numer funkcji poprzez dodania 100, poniej numery
#########################################

# 108 - W --> G, POBRANIE DANYCH DO GRIDA (zamyka od razu do 'Z' i wpisuje ceny na hd_paragony)
# 109 - ZAMYKANIE DOKUMENTU G --> Z,POBRANIE DANYCH DO GRIDA (nic nie robi, nie powinno sie wydarzy)
# 110 - OTWIERANIE DOKUMENTU DO EDYCJI,POBRANIE DANYCH DO GRIDA (nic nie robi, nie powinno sie wydarzy)
# 111 - OTWARCIE DOKUMENTU Z --> G,POBRANIE DANYCH DO GRIDA (otwiera od razu do 'W' i usuwa ceny z hd_paragonow)
# 118 - PRZYGOTOWANIE DOKUMENTU DO ZAMKNIECIA W --> G bez pobierania danych (zamyka od razu do 'Z')
# 119 - ZAMYKANIE DOKUMENTU G --> Z bez pobierania danych (nie robi nic)
# 120 - G->W bez pobierania danych (nie robi nic)
# 121 - OTWARCIE DOKUMENTU Z --> G bez pobierania danych (otwiera od razu do 'W')
# 129 - ZAMYKANIE DOKUMENTU G --> Z (ze spisu z natury (nie zaznacza dokumenty_po w spisie) nie robi nic)
# 131 - OTWARCIE DOKUMENTU Z --> G ze spisu z natury (otwiera od razu do 'W')
# UWAGA: Aktualnie obslugiwane tylko 108, 111, 118,121, 131

#-#######################################
#- UWAGA! PO ZMIANIE PARAMETRW TRZEBA ZMIENI PLIKI:
#    RSDokumentyMagazynowe, RSTworzeniePW, RSTworzenieMM, RSParagonyFiskalne, 
#    fs_dokumenty_magazynowe_klucz,fs_spisy_z_natury_pom,fs_dokumenty_magazynowe_pom,
#    fs_spis_z_natury, fs_komser_dokumenty_magazynowe
#    trzeba te zmieni warto zmiennej DlugoscFiltra
#-#######################################

import math

import rsglobal
from funkcje import fs_dokumenty_magazynowe_lokacje_pom
from funkcje import fs_dokumenty_magazynowe_pom
from funkcje import fs_dokumenty_magazynowe_produkcja_pom
from funkcje import fs_dokumenty_magazynowe_rezerwacje_pom
from funkcje import fs_kolejkowanie_pom
from funkcje import fs_kontrahenci_kuke_pom
from funkcje import fs_spis_z_natury_pom
from funkcje import fs_zaleglosci_sprawdzenie


#-##############################################################################
# FUNKCJA ZDEJMUJCA POWIZANIE Z DOKUMENTW
#-##############################################################################

def ZdejmijPowiazanieZDokumentow(self,Dokum,Mag,Data,Numer, firma_id):
    stat='OK'
    SQL="SELECT dokument_zwiaz FROM rs_dokumenty_magazynowe WHERE firma_id='"+firma_id+"'"
    SQL+=" AND dokument_id='"+Dokum+"' AND magazyn_id='"+Mag+"' AND data='"+Data+"' AND numer="+str(Numer)
    SQL+=" AND COALESCE(dokument_zwiaz,'') != '';"
    stat,DaneDok,dump,dump=self.PolSQL.Execute(SQL, 3)
    if stat<>'OK':
        return stat
    
    if len(DaneDok)==0:
        return 'OK'

    DokZwiaz=rsglobal.Dekoduj(DaneDok[0][0])
    DokPierw=Mag+'/'+Dokum+'/'+Data+'/'+str(Numer).split('.')[0]

    # robimy update :
    # 1. Na dokumentach, ktre maj wpisany ten sam dokument zwizany co pierwotny
    # 2. Na dokumentach, ktre maj wpisany dokument zwizany bdcy pierwotnym
    # 3. Na dokumencie pierwotnym
    # 4. Na dokumencie zwizanym z pierwotnym
    UPD ="UPDATE rs_dokumenty_magazynowe SET dokument_zwiaz=NULL"
    UPD+=" WHERE firma_id='"+firma_id+"' AND (dokument_zwiaz IN ('"+DokZwiaz+"','"+DokPierw+"')"
    UPD+=" OR magazyn_id||'/'||dokument_id||'/'||data||numer::TEXT IN ('"+DokZwiaz+"','"+DokPierw+"'))"
    UPD+=";"
    
    stat,dump,dump,dump=self.PolSQL.Execute(UPD, 1)
    return stat



#-##############################################################################
#-##################   CZʦ GWNA OKNA   #####################################
#-##############################################################################
#zostawione na potrzeby komsera i starych funkcji ewentualnie. Powinno zosta znalezione i wyplenione.

stat='OK'
firma_id = self.firma_id
if not 'Funkcja' in klucze:
    stat='Brak podanej funkcji w kluczach'


OperatorWMS=0  # zaslepka dla synchronizacji

if stat=='OK':    
    Funkcja = klucze['Funkcja']    
    wszystkie = [5,6,7,8,9,10,11,15,16,18,19,20,21,26,29,30,31,40,41,42,50,66,67,68,69,71,72]
    filtry = [5,6,7,8,9,10,11,41,67,71]
    SlPar={'Mag':wszystkie,'Typ':wszystkie,'Data':wszystkie,'Numer':wszystkie,
           'StanF':filtry,'MagF':filtry,'DokF':filtry,'DataOdF':filtry,
           'DataDoF':filtry,'NrF':filtry,'KontrF':filtry,'MiejDostF':filtry,
           'FaktF':filtry,'ProjektF':filtry,'KluczeF':filtry,'NiezamknieteF':filtry,
           'ZmienioneF':filtry,'NiewyslaneTowF':filtry,'CzyPrzesylki':filtry,
           'Wybrane':[69],'Mag2':[7],'Typ2':[7],'Data2':[7],'Numer2':[7],
           'KWyzn':[41],'KData':[41],'KNr':[41],
           'typdok':[42],'parametry':[42],'dok_d':[42],'data_d':[42],'numer_d':[42],'mag_d':[42,]}

    Lista=rsglobal.SprawdzParametryWKluczach(SlPar,Funkcja,klucze)
    if Lista<>'':
        stat='Uwaga : wywoanie funkcji fs_dokumenty_magazynowe z numerem ' +str(Funkcja)+' bez parametrw: '+Lista

    if stat=='OK':
        if Funkcja in wszystkie:             
            Mag = str(klucze['Mag'])
            Dok = str(klucze['Typ'])
            Data  = str(klucze['Data'])
            Numer = str(klucze['Numer']) 
        if Funkcja in filtry:                                
            StanF = str(klucze['StanF'])
            MagF = str(klucze['MagF'])
            DokF = str(klucze['DokF'])
            DataOdF = str(klucze['DataOdF'])
            DataDoF = str(klucze['DataDoF'])
            NrF = str(klucze['NrF'])
            KontrF = str(klucze['KontrF'])
            MiejDostF = str(klucze['MiejDostF'])
            FaktF = str(klucze['FaktF'])
            ProjektF = str(klucze['ProjektF'])
            KluczeF = klucze['KluczeF']
            NiezamknieteF = klucze['NiezamknieteF']
            ZmienioneF = klucze['ZmienioneF']
            NiewyslaneTowF = klucze['NiewyslaneTowF']
            CzyPrzesylki = klucze['CzyPrzesylki']
        if Funkcja in [69]:             
            Wybrane = str(klucze['Wybrane'])
        if Funkcja in [7]:             
            Mag2 = str(klucze['Mag2'])
            Dok2 = str(klucze['Typ2'])
            Data2 = str(klucze['Data2'])
            Numer2 = str(klucze['Numer2'])
        if Funkcja in [41]:             
            KWyzn = str(klucze['KWyzn'])
            KData = str(klucze['KData'])
            KNr = str(klucze['KNr'])
        if Funkcja in [42]:             
            typdok = str(klucze['typdok'])
            parametry = str(klucze['parametry'])
            dok_d = str(klucze['dok_d'])
            data_d = str(klucze['data_d'])
            numer_d = str(klucze['numer_d'])
            mag_d = str(klucze['mag_d'])
dane = []
wyn1 = []
wyn2 = []

#-------------------------------------------------------------------------------
# SPRAWDZENIE CZY NA DOKUMENT NIE ZAPLATALA SIE USLUGA LUB INNE
# dla 8 i 18 (z W na G/Z)
#-------------------------------------------------------------------------------

if stat == 'OK' and Funkcja in (8,18,):
    SQL = "SELECT p.artykul_id"
    SQL += " FROM rs_dokumenty_magazynowe_pozycje p JOIN rs_artykuly a ON p.firma_id = a.firma_id AND a.artykul_id=p.artykul_id "
    SQL += " WHERE p.firma_id='"+firma_id+"' AND dokument_id='"+Dok+"' AND magazyn_id='"+Mag+"' AND data='"+Data+"' AND numer="+str(Numer)
    SQL += " AND a.typ_artykulu IN ('31', '99')"
    stat, dane, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat != 'OK':
        stat = u'Bd sprawdzania typw artykuw' + stat
    elif len(dane) > 0:
        stat = 'Nieprawidowy typ artykuu(w) : '
        for w in dane:
            stat += w[0].strip() + ', '
        stat = stat[:-2] + '.'


#-------------------------------------------------------------------------------
# SPRAWDZENIE CZY WYDANIE Z MAGAZYNU DETALICZNEGO- (8,9,10,11,18,19,20,21,29,31)
# dalej obslugiwane funkcje 108,111,118,121,131 tylko
# Dodatkowo ustawienie: CzyMagazynDetaliczny dla roznych wyjatkow
#-------------------------------------------------------------------------------

CzyMagazynDetaliczny = False
if stat == 'OK' and Funkcja in (8,9,10,11,18,19,20,21,29,31,):
    SQL = "SELECT detaliczny FROM rs_magazyny "
    SQL += "WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"'"
    stat, dane, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat == 'OK':
        if len(dane) == 0:
            stat = 'Brak definicji magazynu'
        else:
            CzyMagazynDetaliczny = dane[0][0]
            if CzyMagazynDetaliczny:  # sprawdzamy czy dokument jestrozchodowy
                SQL = "SELECT typ_dokumentu FROM rs_typy_dokumentow_m  "
                SQL += "WHERE firma_id='" + firma_id + "' AND dokument_id='" + Dok + "'"
                stat, dane, dump, dump = self.PolSQL.Execute(SQL, 3)
                if stat == 'OK':
                    if len(dane) == 0:
                        stat = 'Brak definicji typu dokumentu'
                    else:
                        if dane[0][0] == 'W':  # zwiekszamy numer funkcji do osobnej obslugi
                            Funkcja += 100

#-------------------------------------------------------------------------------
# SPRAWDZENIE CECHY NA POZYCJACH Z ARTYKULAMI- (72)
#-------------------------------------------------------------------------------

if stat == 'OK' and Funkcja in (72,):
    SQL="SELECT p.artykul_id,COALESCE(a.cecha1_opis,'') = COALESCE(p.cecha1_opis,'') "
    SQL+="FROM rs_dokumenty_magazynowe_pozycje p LEFT OUTER JOIN rs_artykuly a ON p.firma_id = a.firma_id AND a.artykul_id=p.artykul_id "
    SQL+="WHERE p.firma_id='"+firma_id+"' AND dokument_id='"+Dok+"' AND magazyn_id='"+Mag+"' AND data='"+Data+"' AND numer="+str(Numer)
    stat, dane, dump, dump = self.PolSQL.Execute(SQL, 3)

#-------------------------------------------------------------------------------
# DELETE - (6,16 - usuwanie dokumentu utworzonego z paragonu, 26 - usuwanie ze spisu z natury)
#-------------------------------------------------------------------------------

if stat == 'OK' and Funkcja in (6,16,26):
    #sql do aktualizacji rs_lokacje
    #w wyniku analizy rs_dokumenty_magazynowe_lokacje.lokacja_dokad
    UPDLOK = ''

    #sprawdzenie uprawnien do magazynu i typu dokumentu
    sql="SELECT rsf_sprawdz_upr_do_dok('"+firma_id+"','"+Mag+"','"+Dok+"',5);"
    stat, TestUpr, dump, dump = self.PolSQL.Execute(sql, 3)
    if stat=='OK':
        if len(TestUpr)==0:
            stat='Bd sprawdzania uprawnie do magazynu i typw dokumentw.'
        elif not TestUpr[0][0]:
            stat= "Brak uprawnie do usuwania dokumentu."
        
    if stat == 'OK':
        #sprawdzenie w przypadku opcji autonumeracji, czy to ostatni z danego dnia
        if self.automat_numery:
            sql ="SELECT rsf_sprawdz_czy_mozna_usunac_dokument('"+firma_id+"',"
            sql+=" '[\\\""+Mag+"\\\",\\\""+Dok+"\\\",\\\""+Data+"\\\","+Numer+"]','M');"
            stat,dane,dump,dump = self.PolSQL.Execute(sql,3)
            if stat != 'OK':
                stat = 'Bd pobrania informacji o autonumeracji : '+stat
            else:
                if dane[0][0] != 'OK':
                    stat = dane[0][0]
                    
    if stat == 'OK' and Funkcja==6:
        #sprawdzenie w przypadku opcji Wymuszac WZ na paragonie, czy jest podpiety jakis paragon
        sql = "SELECT wymuszac_wz_na_parag FROM rs_firmy WHERE firma_id = '"+firma_id+"';"
        stat,dane,dump,dump = self.PolSQL.Execute(sql,2)
        if stat != 'OK':
            stat = 'Bd pobrania informacji o wymuszaniu paragonw na dok.WZ : '+stat
        else:
            if dane[0][0]:
                sql ="SELECT oddzial_id||'/'||kasa_id||'/'||data||'/'||numer FROM rs_paragony_fiskalne"
                sql+=" WHERE firma_id = '"+firma_id+"' AND magazyn_id = '"+Mag+"' AND dokument_id = '"+Dok+"'"
                sql+=" AND data_dm = '"+Data+"' AND numer_dm = "+Numer+";"
                stat,dane,dump,dump = self.PolSQL.Execute(sql,3)
                if stat != 'OK':
                    stat = 'Bd pobrania o informacji o paragonie : '+stat
                else:
                    if len(dane) > 0:
                        stat = 'Dokument jest pobrany na paragon '+dane[0][0]+' i nie mona go usun'
                        
    if stat == 'OK':
        #sprawdzenie czy dokument nie jest utworzony z dostawy
        sql ="SELECT data||'/'||numer FROM rs_dostawy"
        sql+=" WHERE firma_id = '"+firma_id+"' AND d_magazyn_id = '"+Mag+"' AND d_dokument_id = '"+Dok+"'"
        sql+=" AND d_data = '"+Data+"' AND d_numer = "+Numer+";"
        stat,dane,dump,dump = self.PolSQL.Execute(sql,3)
        if stat != 'OK':
            stat = 'Bd pobrania o informacji o dostawie : '+stat
        else:
            if len(dane) > 0:
                stat = 'Dokument zosta utworzony z dostawy '+dane[0][0]+' i nie mona go usun'
    
    if stat == 'OK':
        #sprawdzenie czy dokument nie jest utworzony z dowodu dostawy jako WZ
        sql ="SELECT firma_id FROM rs_dowody_dostaw"
        sql+=" WHERE firma_id = '"+firma_id+"' AND wz_magazyn_id = '"+Mag+"' AND wz_dokument_id = '"+Dok+"'"
        sql+=" AND wz_data = '"+Data+"' AND wz_numer = "+Numer+";"
        stat,dane,dump,dump = self.PolSQL.Execute(sql,3)
        if stat != 'OK':
            stat = 'Bd pobrania o informacji dokumencie WZ z dowodw dostaw : '+stat
        else:
            if len(dane) > 0:
                stat = 'Dokument zosta utworzony z dowodw dostaw i nie mona go usun'
    
    if stat == 'OK':
        #sprawdzenie czy dokument nie jest dokumentem PW (lub korekt PW), ktry jest wpisany na RW (lub korekcie RW)
        sql ="SELECT firma_id FROM rs_dokumenty_magazynowe"
        sql+=" WHERE firma_id = '"+firma_id+"' AND pw_magazyn_id = '"+Mag+"' AND pw_dokument_id = '"+Dok+"'"
        sql+=" AND pw_data = '"+Data+"' AND pw_numer = "+Numer+";"
        stat,dane,dump,dump = self.PolSQL.Execute(sql,3)
        if stat != 'OK':
            stat = 'Bd pobrania o informacji dokumencie RW z dokumentw magazynowych : '+stat
        else:
            if len(dane) > 0:
                stat = 'Z dokumentu zostao utworzone RW lub korekta RW i nie mona go usun'
    
    if stat == 'OK':
        #sprawdzenie czy dokument nie jest utworzony z dowodu dostawy jako ZW
        sql ="SELECT firma_id FROM rs_dowody_dostaw"
        sql+=" WHERE firma_id = '"+firma_id+"' AND zw_magazyn_id = '"+Mag+"' AND zw_dokument_id = '"+Dok+"'"
        sql+=" AND zw_data = '"+Data+"' AND zw_numer = "+Numer+";"
        stat,dane,dump,dump = self.PolSQL.Execute(sql,3)
        if stat != 'OK':
            stat = 'Bd pobrania o informacji dokumencie ZW z dowodw dostaw : '+stat
        else:
            if len(dane) > 0:
                stat = 'Dokument zosta utworzony z dowodw dostaw i nie mona go usun'
                
    if stat == 'OK':
        #blokada przed DELETE
        sql ="SELECT stan,magazyn_dokad FROM rs_dokumenty_magazynowe"
        sql+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"'"
        sql+=" AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+Numer+" FOR UPDATE NOWAIT;"
        stat,Test,dump,dump = self.PolSQL.Execute(sql, 3, CzyDodawacBlad=True)
        if 'nie mona naoy blokady' in stat:
            stat = 'Dokument jest zablokowany przez innego uytkownika'
        elif stat != 'OK':
            stat = 'Bd zakadnania blokacy na dokument magazynowy: '+stat
    
    if stat=='OK':
        if len(Test)==0:
            stat='Brak dokumentu o podanym kluczu.'
        elif Test[0][0] != 'W':
            stat='Uwaga, nie mona usun dokumentu w stanie '+Test[0][0]
            
    MagazynMaLokacje = False
    MaGridaLokacji = False
    MagL = Mag.strip()
    
    if stat=='OK':
        if Dok == 'MM':
            #jesli dok MM, to lokacja dokad jest w polu 'magazyn_dokad'
            MagL = rsglobal.Dekoduj(Test[0][1]).strip()

        #sprawdzimy, czy magazyn_dokad posiada lokacje, bo jesli nie,
        #to wogole nie zajmujemy sie wycofywaniem ilosci z lokacji
        sql ="SELECT czy_lokacje FROM rs_magazyny"
        sql+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+MagL+"';"
        stat,d,dump,dump = self.PolSQL.Execute(sql,3)
        if stat != 'OK':
            stat = 'Bd pobrania danych o lokacjach w magazynie: '+stat
            dane.append(stat)
        elif len(d) != 1:
            stat = 'Brak magazynu DOKD'
            dane.sppend(stat)
        elif d[0][0]:
            #magazyn DOKAD nie obsluguje lokacji, a to oznacza, ze nie aktualizujemy rs_lokacje
            MagazynMaLokacje = True
        
    if stat == 'OK' and MagazynMaLokacje:
        #sprawdzam czy pozycja posiada grida z lokacjami
        #jesli tak, to sprawdzam czy mozna wycofac stany z lokacji_dokad
    
        sql = "SELECT lokacja_skad,lokacja_dokad,ilosc FROM rs_dokumenty_magazynowe_lokacje"
        sql += " WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"' AND dokument_id='"+Dok+"'"
        sql += " AND data='"+Data+"' AND numer="+str(Numer)+" FOR UPDATE NOWAIT;"
        stat,daneLOK,dump,dump = self.PolSQL.Execute(sql,3,CzyDodawacBlad=True)
        if 'nie mona naoy blokady' in stat:
            stat = 'Lokacje s zablokowane przez innego uytkownika'
            dane.append(stat)
        elif stat != 'OK':
            stat = 'Bd pobrania danych lokacji doczonych do dokumentu: '+stat
            dane.append(stat)
        elif len(daneLOK) > 0:
            MaGridaLokacji = True
    
    if stat == 'OK' and MaGridaLokacji:
        for i in xrange(len(daneLOK)):
            LokacjaDokad = rsglobal.Dekoduj(daneLOK[i][1]).strip()
            Ilosc = daneLOK[i][2]
            if LokacjaDokad != 'None':
                sql ="SELECT rsf_podaj_dane_lokacji(firma_id,lokacja_id,'zajete')::NUMERIC,"
                sql+=" magazyn_id,CEIL("+str(Ilosc)+"),do_wydania FROM rs_lokacje"
                sql+=" WHERE firma_id='"+firma_id+"' AND lokacja_id='"+LokacjaDokad+"' FOR UPDATE NOWAIT;"

                stat,d,dump,dump = self.PolSQL.Execute(sql,3,CzyDodawacBlad=True)

                if 'nie mona naoy blokady' in stat:
                    stat = 'Lokacje s zablokowane przez innego uytkownika'
                    dane.append(stat)
                elif stat != 'OK':
                    stat = 'Bd pobrania danych lokacji '+stat
                    dane.append(stat)
                
                elif len(d) != 1:
                    stat = 'Brak lokacji '+LokacjaDokad
                    dane.append(stat)
                
                elif rsglobal.Dekoduj(d[0][1]).strip() == 'None':
                    #lokacja dotyczy magazynu, wiec sprawdzamy tylko ilosc
                    if d[0][2] > d[0][0]:
                        stat = 'Brak wystarczajcej iloci w lokacji '+LokacjaDokad
                        dane.append(stat)
                    
                elif rsglobal.Dekoduj(d[0][1]).strip() == MagL:
                    #lokacja dotyczy magazynu, wiec sprawdzamy tylko ilosc
                    if d[0][2] > d[0][0]:
                        stat = 'Brak wystarczajcej iloci w lokacji '+LokacjaDokad
                        dane.append(stat)
            
                else:
                    stat = 'Brak lokacji '+LokacjaDokad+' z magazynu '+Mag
                    dane.append(stat)

    #czy byly jakies bledy 'po drodze'
    if stat == 'OK':
        if len(dane) > 0:
            stat == dane[0]
                    
    # dodatkowe rozliczenie pola do_wydania w lokacjach dla dokumentu rozchodowego
    if stat=='OK':
        stat=fs_dokumenty_magazynowe_lokacje_pom.ZmienIlosciDoWydaniaWLokacjachNaCalymDokumencie(self,Mag,Dok,Data,Numer,'-')
    
    # usunicie rezerwacji
    if stat == 'OK':
        stat=fs_dokumenty_magazynowe_rezerwacje_pom.ZmienRezerwacjeNaArtPrzyjDokumentC(self,[Mag,Dok,Data,Numer],[],'-')
    
    # zdjcie ewentualnego powizania z innymi dokumentami
    if stat == 'OK':
        stat=ZdejmijPowiazanieZDokumentow(self,Dok,Mag,Data,Numer, firma_id)

    # ewentualne czyszczenie wirtualnych artykuw przyjcia
    if stat=='OK':
        # wymuszone usuniecie artykulow-przyjecia, przed usuniciem dokumentu
        stat=fs_dokumenty_magazynowe_lokacje_pom.OdswiezWirtualnePozPrzyj(self,Mag,Dok,Data,Numer,0,True)
        
    if stat == 'OK':
        #tworzymy SQL-a do DELETE
        #Wyczyszczenie przypisanego dokumentu do paragonu
        sql="UPDATE rs_paragony_fiskalne SET magazyn_id=NULL,dokument_id=NULL,data_dm=NULL,numer_dm=0"
        sql+=" WHERE firma_id = '"+firma_id+"' AND magazyn_id = '"+Mag+"' AND dokument_id = '"+Dok+"'"
        sql+=" AND data_dm = '"+Data+"' AND numer_dm = "+Numer+"; "
    
        #Wyczyszczenie informacji o WZ z dowodw dostaw
        if Dok == 'WZ':
            sql+="UPDATE rs_dowody_dostaw SET wz_magazyn_id = NULL, wz_dokument_id = NULL, wz_data = NULL, wz_numer = NULL"
            sql+=" WHERE firma_id = '"+firma_id+"' AND wz_magazyn_id = '"+Mag+"' AND wz_dokument_id = 'WZ'"
            sql+=" AND wz_data = '"+Data+"' AND wz_numer = "+Numer+"; "
        
        #Wyczyszczenie informacji o WZ z dowodw dostaw
        if Dok == 'ZW':
            sql+="UPDATE rs_dowody_dostaw SET zw_magazyn_id = NULL, zw_dokument_id = NULL, zw_data = NULL, zw_numer = NULL"
            sql+=" WHERE firma_id = '"+firma_id+"' AND zw_magazyn_id = '"+Mag+"' AND zw_dokument_id = 'ZW'"
            sql+=" AND zw_data = '"+Data+"' AND zw_numer = "+Numer+"; "
        
        sql+="DELETE FROM rs_dokumenty_magazynowe"
        sql+=" WHERE firma_id = '"+firma_id+"' AND magazyn_id = '"+Mag+"' AND dokument_id = '"+Dok+"'"
        sql+=" AND data = '"+Data+"' AND numer = "+Numer+"; "
          
        #Wyczyszczenie przypisanego spisu z natury
        if Dok=='RW':
            Gdzie=Numer.find(".")
            if Gdzie>0:
                Numer= Numer[:Gdzie]  
                DokRW=Mag.rstrip()+'/RW/'+Data+'/'+Numer
                sql+="UPDATE rs_inwentaryzacja SET dokument_rw = NULL"
                sql+=" WHERE firma_id = '"+firma_id+"' AND dokument_rw = '"+DokRW+"'; "
        
        # wywolanie funkcji z podaniem opisu ewentualnego bledu do analizy
        stat,d,dump,blad = self.PolSQL.Execute(sql,1,CzyDodawacBlad=True)
        if stat != 'OK':
            if 'rs_produkcja_plan' in str(stat).lower():
                stat = 'Nie mona usuwa dokumentw utworzonych na panelu w planie produkcji'
            else:
                stat = 'Bd usuwania dokumentu : '+stat


#-------------------------------------------------------------------------------
# SPRAWDZENIE TYPU DOKUMENTU - 50
#-------------------------------------------------------------------------------

if Funkcja == 50 and stat == 'OK':
    SQL="SELECT typ_dokumentu,COALESCE(korekta,FALSE)"
    SQL+=" FROM rs_typy_dokumentow_m WHERE firma_id='"+firma_id+"'"
    SQL+=" AND dokument_id='"+Dok+"';"
    stat,dane,dump,dump = self.PolSQL.Execute(SQL,3)


#-------------------------------------------------------------------------------
# POWIAZANIE DOKUMENTOW - 7
#-------------------------------------------------------------------------------

if Funkcja == 7 and stat == 'OK':
    if stat=='OK':
        Klucz1=Mag+'/'+Dok+'/'+Data+'/'+Numer.split('.')[0]
        Klucz2=Mag2+'/'+Dok2+'/'+Data2+'/'+Numer2.split('.')[0]
    
        sql ="UPDATE rs_dokumenty_magazynowe SET dokument_zwiaz = '"+Klucz2+"'"
        sql+=" WHERE firma_id= '"+firma_id+"' AND magazyn_id = '"+Mag+"' AND dokument_id = '"+Dok+"'"
        sql+=" AND data = '"+Data+"' AND numer = "+Numer+";"
        sql+=" "
        sql+="UPDATE rs_dokumenty_magazynowe SET dokument_zwiaz = '"+Klucz1+"'"
        sql+=" WHERE firma_id = '"+firma_id+"' AND magazyn_id = '"+Mag2+"' AND dokument_id='"+Dok2+"'"
        sql+=" AND data = '"+Data2+"' AND numer = "+Numer2+";"
        
        stat,d,dump,dump = self.PolSQL.Execute(sql,1)
        if stat != 'OK':
            stat = 'Bd operacji powizania dokumentw : '+stat
        

#-------------------------------------------------------------------------------
# ODWIAZANIE DOKUMENTOW - 71
#-------------------------------------------------------------------------------

if Funkcja == 71 and stat == 'OK':
    stat=ZdejmijPowiazanieZDokumentow(self,Dok,Mag,Data,Numer,firma_id)


#-------------------------------------------------------------------------------
# POBRANIE DANYCH FAKTURY Z DOKUMENTU MAGAZYNOWEGO - 40
#-------------------------------------------------------------------------------

if Funkcja == 40 and stat == 'OK':
    sql ="SELECT f_wyznacznik_id, f_data, f_numer FROM rs_dokumenty_magazynowe"
    sql+=" WHERE firma_id = '"+firma_id+"' AND magazyn_id = '"+Mag+"'"
    sql+=" AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer='"+Numer+"';"
    
    stat,dane,dump,dump = self.PolSQL.Execute(sql,3)
    if stat != 'OK':
        stat = 'Bd pobrania danych faktury z dokumentu magazynowego : '+stat


#-------------------------------------------------------------------------------
# POBRANIE STANU DOKUMENTU - 68
#-------------------------------------------------------------------------------

if Funkcja == 68 and stat == 'OK':
    sql ="SELECT stan FROM rs_dokumenty_magazynowe"
    sql+=" WHERE firma_id = '"+firma_id+"' AND magazyn_id = '"+Mag+"'"
    sql+=" AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer='"+Numer+"';"
    
    stat,dane,dump,dump = self.PolSQL.Execute(sql,3)
    if stat != 'OK':
        stat = 'Bd pobrania stanu z dokumentu magazynowego : '+stat

    
#-------------------------------------------------------------------------------
# POWIAZANIE DOKUMENTU MAGAZYNOWEGO Z KOREKTA - 41
#-------------------------------------------------------------------------------

if Funkcja == 41 and stat == 'OK':
    sql ="UPDATE rs_dokumenty_magazynowe"
    sql+=" SET k_wyznacznik_id = '"+KWyzn+"', k_data = '"+KData+"', k_numer= "+KNr
    sql+=" WHERE firma_id = '"+firma_id+"' AND magazyn_id = '"+Mag+"'"
    sql+=" AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer='"+Numer+"';"
    
    stat,d,dump,dump = self.PolSQL.Execute(sql,1)
    if stat != 'OK':
        stat = 'Bd zapisu danych korekty w dokumencie magazynowym : '+stat


#-------------------------------------------------------------------------------
# KONTROLA DOKUMENTOW DO UTWORZENIA WYSYLKI - 69
#-------------------------------------------------------------------------------

if Funkcja == 69 and stat == 'OK':
    for dok in KluczeF:
        NrDok=dok[0]+'/'+dok[1]+'/'+dok[2]+'/'+rsglobal.UsunKropkaZero(dok[3])

        SQL ="SELECT dm.stan,tdm.typ_dokumentu"
        SQL+=" FROM rs_dokumenty_magazynowe dm"
        SQL+=" INNER JOIN rs_typy_dokumentow_m tdm USING (firma_id,dokument_id)"
        SQL+=" WHERE dm.firma_id='"+firma_id+"' AND dm.magazyn_id='"+dok[0]+"'"
        SQL+=" AND dokument_id='"+dok[1]+"' AND data='"+dok[2]+"' AND numer="+str(dok[3])+";"
        
        stat,DaneDok,dump,dump=self.PolSQL.Execute(rsglobal.Dekoduj(SQL),3)
        if stat<>'OK':
            stat="Bd kontroli dokumentu "+NrDok+" : "+stat
        elif len(DaneDok)==0:
            stat="Brak dokumentu "+NrDok
        elif DaneDok[0][0]=='W':
            stat='Dokument '+NrDok+' jest w stanie W i nie mona go pobra do przesyki'
        elif not DaneDok[0][1] in ['W','M']:
            stat='Dokument '+NrDok+' jest typu '+DaneDok[0][1]+' i nie mona go pobra do przesyki'
        
        if stat != 'OK':
            break


#-------------------------------------------------------------------------------
# Sprawdzenie limitu (66)
#-------------------------------------------------------------------------------

if Funkcja == 66 and stat == 'OK':
    sql ="SELECT dm.kontrahent_id, dm.kwota, tdm.typ_dokumentu"
    sql+=" FROM rs_dokumenty_magazynowe dm"
    sql+=" INNER JOIN rs_typy_dokumentow_m tdm USING (firma_id,dokument_id)"
    sql+=" WHERE dm.firma_id='"+firma_id+"' AND dm.magazyn_id='"+Mag+"'"
    sql+=" AND dm.dokument_id='"+Dok+"' AND dm.data='"+Data+"' AND dm.numer="+str(Numer)+";"

    stat,dn,dump,dump = self.PolSQL.Execute(sql,3)
    if stat != 'OK':
        stat = 'Bd sprawdzenia danych dokumentu: '+stat
        dane.append(stat)
    elif len(dn)==0:
        stat = 'Brak dokumentu o podanym kluczu'
        dane.append(stat)

    if stat == 'OK' and rsglobal.Dekoduj(dn[0][2]) == 'W' and not rsglobal.Dekoduj(dn[0][0]).strip() in ('','None'):
        #jesli to dokument typu 'W' to sprawdzenie kontrahenta jesli na dokumencie jest kontrahent
        sql ="SELECT kontrahent_id FROM rs_kontrahenci"
        sql+=" WHERE firma_id = '"+firma_id+"'"
        sql+=" AND kontrahent_id = '"+str(dn[0][0])+"' AND aktywny;"
        
        stat,k,dump,dump = self.PolSQL.Execute(sql,3)
        if stat != 'OK':
            stat = 'Bd sprawdzenia danych kontrahenta'
            dane.append(stat)
        elif len(k) != 1:
            stat = 'Bdny nr kontrahenta'
            dane.append(stat)
            
    if stat == 'OK' and rsglobal.Dekoduj(dn[0][2]) == 'W' and not rsglobal.Dekoduj(dn[0][0]).strip() in ('','None'):
        #jesli to dokument typu 'W' i jest kontrahent, to porownanie stanu zaleglosci z limitem
        stat = fs_zaleglosci_sprawdzenie.SprawdzLimitKontrahenta(self,dn[0][0],dn[0][1],Data)
        if stat == 'LIMIT':
            #podmieniam stat, bo osiagniecie limitu nie blokuje operacji
            stat = 'OK'
            dane.append(["Kontrahent '"+str(dn[0][0]).strip()+"' wykorzysta limit.",'LIMIT',dn[0][0]])


#-------------------------------------------------------------------------------
# Utworzenie przesylki z dokumentu (67)
#-------------------------------------------------------------------------------

if Funkcja == 67 and stat == 'OK':
    if not ( ((self.oddzial_id,'231013') in self.uprawnienia) and  \
             ((self.oddzial_id,'231023') in self.uprawnienia) \
             or (self.operator_id == STANDARD_OPERATOR_ID)):
        stat = 'Brak uprawnie do utworzenia przesyki'
        dane.append(stat)
    
    if stat == 'OK':
        sql ="SELECT tm.typ_dokumentu, m.stan, m.p_numer FROM rs_dokumenty_magazynowe m"
        sql+=" INNER JOIN rs_typy_dokumentow_m tm USING (firma_id, dokument_id)"
        sql+=" WHERE m.firma_id='"+firma_id+"' AND m.magazyn_id='"+Mag+"'"
        sql+=" AND m.dokument_id='"+Dok+"' AND m.data='"+Data+"' AND m.numer="+str(Numer)
        sql+=" FOR UPDATE NOWAIT;"
    
        stat,dn,dump,dump = self.PolSQL.Execute(sql,3,CzyDodawacBlad=True)
        if 'nie mona naoy blokady' in stat:
            stat = 'Dokument jest zablokowany przez innego uytkownika'
            dane.append(stat)
        elif stat != 'OK':
            stat = 'Bd sprawdzenia danych dokumentu: '+stat
        elif len(dn) == 0:
            stat = "Brak dokumentu o podanym kluczu"
            dane.append(stat)
        elif not dn[0][0] in ('W','M'):
            stat = "Przesyk mona utworzy tylko dla\ndokumentw rodzaju 'W' i 'M' (rozchd)"
            dane.append(stat)
        elif dn[0][1] == 'W':
            stat = "Dokument w stanie 'W'.\nUtworzenie przesyki niemoliwe."
            dane.append(stat)
        elif dn[0][2] > 0:
            stat = "Dokument ju posiada dane przesyki."
            dane.append(stat)

    if stat == 'OK':
        sql = "SELECT MAX(numer) FROM rs_przesylki"
        sql+= " WHERE firma_id='"+firma_id+"' and data=NOW()::DATE;"
    
        stat,p,dump,dump = self.PolSQL.Execute(sql,3)
        if stat != 'OK':
            stat = 'Bd pobrania numeru przesyki: '+stat
            dane.append(stat)
    
    if stat == 'OK':
        sql = "INSERT INTO rs_przesylki (firma_id,data,numer) VALUES "
        sql+= " ('"+firma_id+"',NOW()::DATE,"+str(p[0][0]+1)+"); "
        sql+= "UPDATE rs_dokumenty_magazynowe"
        sql+=" SET p_numer="+str(p[0][0]+1)+", p_data=NOW()::DATE"
        sql+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"'"
        sql+=" AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+str(Numer)+";"
        
        stat,d,dump,dump = self.PolSQL.Execute(sql,1)
        if stat != 'OK':
            stat = 'Bd utworzenia przesyki: '+stat
            dane.append(blad)


#@##########################################
# TESTY OGLNE DLA OTWIERANIA I ZAMYKANIA NA KADYM POZIOMIE
#@##########################################

# 8 W->G, 9 G->Z, 10 G->W, 11 Z->G, 18 W->G bez pobierania danych, 19 G->Z bez pobierania danych
# 20 G->W bez pobierania danych, 21 Z->G bez pobierania danych, 29 G -> Z ze spisu z natury (nie zaznacza dokumenty_po w spisie)
# 31 Z->G ze spisu z natury
# 108,111,118,121,131 zamykanie i otwieranie rozchodowych detalicznych

if Funkcja in [8,9,10,11,18,19,20,21,28,29,30,31,108,111,118,121,131] and stat=='OK':
    #rsglobal.ZapiszDoLogow('1 FUNKCJA:'+str(Funkcja),1)
    #sprawdzenie uprawnie
    TestUpr='-1'
    if Funkcja in [8,18,28,108,118]:
        TestUpr='11'
        TekstUpr='przygotowania'
    elif Funkcja in [9,19,29]:
        TestUpr='2'
        TekstUpr='zamykania'
    elif Funkcja in [10,20,30]:
        TestUpr='12'
        TekstUpr='otwarcia do edycji'
    elif Funkcja in [11,21,31,111,121,131]:
        # dodatkowe sprawdzenie. Jeli dokument z paragonu, ktry jest otwarty to pozwalamy otwiera nawet 
        # w przypadku braku uprawnie
        SQL="SELECT stan FROM rs_paragony_fiskalne WHERE firma_id='"+firma_id+"' AND "
        SQL+=" dokument_id='"+Dok+"' AND magazyn_id='"+Mag+"' AND data_dm='"+Data+"' AND numer_dm="+Numer
        stat, TestPar, dump, dump = self.PolSQL.Execute(SQL, 3)
        if stat<>'OK':
            stat='Bd sprawdzania stanu paragonu: '+stat
        if stat=='OK':
            if len(TestPar)>0:
                if TestPar[0][0]=='W':
                    TestUpr='0'
        if TestUpr<>'0':
            TestUpr='3'
        TekstUpr='otwierania'

    if stat=='OK':
        if TestUpr<>'0':
            SQL="SELECT rsf_sprawdz_upr_do_dok('"+firma_id+"','"+Mag+"','"+Dok+"',"+TestUpr+");"
            stat, TestUpr, dump, dump = self.PolSQL.Execute(SQL, 3)
            if stat=='OK':
                if len(TestUpr)==0:
                    stat='Bd sprawdzania uprawnie'
                elif not TestUpr[0][0]:
                    stat= "Brak uprawnie do "+TekstUpr+" dokumentu : "+Dok+'/'+Mag+'/'+Data+'/'+Numer
    
    # otwieranie Z->G
    #sprawdzenie istnienia korekt do otwieranego dokumentu
    if stat=='OK' and Funkcja in [11,21,31,111,121,131]:
        sql ="SELECT magazyn_id,dokument_id,data,numer"
        sql+=" FROM rs_dokumenty_magazynowe"
        sql+=" WHERE firma_id='"+firma_id+"' AND kor_dm_magazyn_id='"+Mag+"'"
        sql+=" AND kor_dm_dokument_id='"+Dok+"' AND kor_dm_data='"+Data+"'"
        sql+=" AND kor_dm_numer="+rsglobal.UsunKropkaZero(Numer)+";"
        stat,Korekty,dump,dump = self.PolSQL.Execute(sql,3)
        if stat != 'OK':
            stat = 'Bd sprawdzenia korekt do otwieranego dokumentu: '+stat
        elif len(Korekty) > 0:
            stat = 'Otwarcie dokumentu niemoliwe poniewa istniej przypisane korekty'

    #sprawdzanie poprawnosci zakresu dat
    if stat=='OK':
        SQL="SELECT rsf_sprawdz_date_z_zakresem('"+firma_id+Mag+"','"+Data+"',2);"
        stat, TestZakr, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat=='OK':
        if len(TestZakr)==0:
            stat='Bd sprawdzania zakresu dat'
        elif not TestZakr[0][0]:
            stat="Data dokumentu spoza zakresu"

    #opis dokumentu do logowania
    if stat=='OK':
        sql="SELECT rsf_podaj_numer_dokumentu_na_wydruk('"+firma_id+"',"
        sql+="'[\\\'"+Mag+"\\\',\\\'"+Dok+"\\\',\\\'"+Data+"\\\',\\\'"+Numer+"\\\']','M');"
        stat,DokumentDoLogowania,dump,dump = self.PolSQL.Execute(sql,3)
        if stat != 'OK':
            stat = 'Bd pobrania numeru dokumentu do wydruku: '+stat
        else:
            DokumentDoLogowania = rsglobal.Dekoduj(DokumentDoLogowania[0][0])

    #pobranie stanu dokumentu z blokad dokumentu
    if stat=='OK':
        Stan='X'
        #rsglobal.ZapiszDoLogow('2 ', 1)
        SQL="SELECT stan,COALESCE(magazyn_skad,''),COALESCE(magazyn_dokad,''),COALESCE(opis,''),COALESCE(kor_dm_magazyn_id,'') "
        SQL+=" FROM rs_dokumenty_magazynowe WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"'"
        SQL+=" AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+Numer+" FOR UPDATE NOWAIT;"
        stat, Test, dump, dump = self.PolSQL.Execute(SQL, 3,CzyDodawacBlad=True)
        
        if stat<>'OK':
            stat = 'Dokument jest zablokowany przez innego uytkownika i nie mona zmieni jego stanu'
            
    if stat=='OK':
        if len(Test)==0:
            stat='Brak dokumentu o podanym kluczu'
        else:
            Stan=rsglobal.Dekoduj(Test[0][0])
            TestMagSkad=rsglobal.Dekoduj(Test[0][1])
            TestMagDokad=rsglobal.Dekoduj(Test[0][2])
            TestOpis=rsglobal.Dekoduj(Test[0][3])
            TestKorekta=rsglobal.Dekoduj(Test[0][4])

    #sprawdzenie czy zamknite spisy z natury nie stoj na przeszkodzie zamykania/otwierania dokumentu
    if stat=='OK':
        if TestMagSkad<>'':
            SQL="SELECT COALESCE(MAX(data)::TEXT,'') FROM rs_inwentaryzacja WHERE firma_id='"+firma_id+"' "
            SQL+=" AND magazyn_id='"+TestMagSkad+"' AND stan='Z';"
            stat, Test, dump, dump = self.PolSQL.Execute(SQL, 3)
            if stat=='OK':
                if Test[0][0]<>'':
                    if Funkcja in [8,9,18,19,28]:
                        if Data<Test[0][0]:
                            stat="Poniewa istniej pniejsze od dokumentu zamknite spisy z natury "
                            stat+="w magazynie "+TestMagSkad+" to nie mona dokona "+TekstUpr+" dokumentu"
                    elif Funkcja in [11,21]:
                        if Data<=Test[0][0]:
                            stat="Poniewa istniej pniejsze od dokumentu zamknite spisy z natury "
                            stat+="w magazynie "+TestMagSkad+" to nie mona dokona "+TekstUpr+" dokumentu"
                    # to wyjtek dla spisw z natury
                    elif Funkcja in [31]:
                        if Data<Test[0][0]:
                            stat="Poniewa istniej pniejsze od dokumentu zamknite spisy z natury "
                            stat+="w magazynie "+TestMagSkad+" to nie mona dokona "+TekstUpr+" dokumentu"
                    elif Funkcja in [10,20,30]:
                        if Data<Test[0][0]:
                            #mozna otworzyc G->W dok.z daty spisu skoro mozna je zamknac
                            stat="Poniewa istniej pniejsze od dokumentu zamknite spisy z natury "
                            stat+="w magazynie "+TestMagSkad+" to nie mona dokona "+TekstUpr+" dokumentu"
        
        if stat=='OK' and TestMagDokad<>'':
            SQL="SELECT COALESCE(MAX(data)::TEXT,'') FROM rs_inwentaryzacja WHERE firma_id='"+firma_id+"' "
            SQL+=" AND magazyn_id='"+TestMagDokad+"' AND stan='Z';"
            stat, Test, dump, dump = self.PolSQL.Execute(SQL, 3)
            if stat=='OK':
                if Test[0][0]<>'':
                    if Funkcja in [8,9,18,19,28,29]:
                        if Data<Test[0][0]:
                            stat="Poniewa istniej pniejsze od dokumentu zamknite spisy z natury "
                            stat+="w magazynie "+TestMagDokad+" to nie mona dokona "+TekstUpr+" dokumentu"
                    elif Funkcja in [11,21]:
                        if Data<=Test[0][0]:
                            stat="Poniewa istniej pniejsze od dokumentu zamknite spisy z natury "
                            stat+="w magazynie "+TestMagDokad+" to nie mona dokona "+TekstUpr+" dokumentu"
                    elif Funkcja in [31]:
                        if Data<Test[0][0]:
                            stat="Poniewa istniej pniejsze od dokumentu zamknite spisy z natury "
                            stat+="w magazynie "+TestMagDokad+" to nie mona dokona "+TekstUpr+" dokumentu"
                    elif Funkcja in [10,20,30]:
                        if Data<Test[0][0]:
                            #mozna otworzyc G->W dok.z daty spisu skoro mozna je zamknac
                            stat="Poniewa istniej pniejsze od dokumentu zamknite spisy z natury "
                            stat+="w magazynie "+TestMagDokad+" to nie mona dokona "+TekstUpr+" dokumentu"
               

    #sprawdzenie czy otwarte spisy z natury nie stoj na przeszkodzie zamykania/otwierania dokumentu
    if stat=='OK':
        if TestMagSkad<>'' or TestMagDokad<>'':
            #rsglobal.ZapiszDoLogow('3' , 1)
            # pobieramy list kategorii artykuw wystpujcych na dokumencie
            SQL="SELECT DISTINCT a.kateg_artykulu FROM rs_dokumenty_magazynowe_pozycje dp "
            SQL+=" INNER JOIN rs_artykuly a USING (firma_id,artykul_id) "
            SQL+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"'"
            SQL+=" AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+Numer+";"
            stat, DKateg, dump, dump = self.PolSQL.Execute(SQL, 3)    
    if stat=='OK':
        if TestMagSkad<>'':
            # pobieramy list otwartych spisw z natury
            SQL="SELECT data,COALESCE(kateg_artykulu,'') FROM rs_inwentaryzacja WHERE firma_id='"+firma_id+"' "
            SQL+=" AND magazyn_id='"+TestMagSkad+"' AND stan='W' ORDER BY data DESC;"
            stat, DSpisy, dump, dump = self.PolSQL.Execute(SQL, 3)
            
            # w ptli sprawdzamy czy kategoria wystpuje na jakim spisie z natury. Jeli tak to albo jest wczeniejsza (ok)
            # albo pniejsza (blokada)
            if stat=='OK':
                for poz in DKateg:
                    if stat=='OK':
                        Znalezione=False

                        for spis in DSpisy:
                            if not Znalezione:
                                # jeli kategoria pasuje
                                if spis[1] == '' or  poz[0].strip().find(spis[1].strip()) == 0:
                                    # jeli data spisu mniejsza ni dokumentu jest bd (lepiej nie bdzie, bo order by jest datami)
                                    if spis[0]<Data:
                                        stat="Poniewa istniej wczeniejsze od dokumentu otwarte spisy z natury "
                                        stat+="w magazynie "+TestMagSkad+" dla kategorii '"+spis[1]+"' to nie mona dokona "+TekstUpr+" dokumentu"
                                    else:
                                        Znalezione=True

        if stat=='OK' and TestMagDokad<>'':
            # pobieramy list otwartych spisw z natury
            SQL="SELECT data,COALESCE(kateg_artykulu,'') FROM rs_inwentaryzacja WHERE firma_id='"+firma_id+"' "
            SQL+=" AND magazyn_id='"+TestMagDokad+"' AND stan='W' ORDER BY data DESC;"
            stat, DSpisy, dump, dump = self.PolSQL.Execute(SQL, 3)
            
            # w ptli sprawdzamy czy kategoria wystpuje na jakim spisie z natury. Jeli tak to albo jest wczeniejsza (ok)
            # albo pniejsza (blokada)
            if stat=='OK':
                for poz in DKateg:
                    if stat=='OK':
                        Znalezione=False

                        for spis in DSpisy:
                            if not Znalezione:
                                # jeli kategoria pasuje
                                if spis[1] == '' or  poz[0].strip().find(spis[1].strip()) == 0:
                                    # jeli data spisu mniejsza ni dokumentu jest bd (lepiej nie bdzie, bo order by jest datami)
                                    if spis[0]<Data:
                                        stat="Poniewa istniej wczeniejsze od dokumentu otwarte spisy z natury "
                                        stat+="w magazynie "+TestMagDokad+" dla kategorii '"+spis[1]+"' to nie mona dokona "+TekstUpr+" dokumentu"
                                    else:
                                        Znalezione=True


    #blokada artykuw znajdujcych si na pozycjach dokumentu
    if stat=='OK':
        #rsglobal.ZapiszDoLogow('4', 1)
        SQL="SELECT artykul_id,aktywny FROM rs_artykuly WHERE firma_id='"+firma_id+"' "
        SQL+=" AND artykul_id IN (SELECT DISTINCT artykul_id FROM rs_dokumenty_magazynowe_pozycje "
        SQL+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"'"
        SQL+=" AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+Numer+")"
        SQL+=" FOR UPDATE NOWAIT;"
        stat, Test, dump, dump = self.PolSQL.Execute(SQL, 3,CzyDodawacBlad=True)
        if stat<>'OK':
            stat='Dostp do tabeli bazy danych jest zablokowany. Sprbuj ponownie za jaki czas. Bd :'+stat
    
    # sprawdzenie czy nie jest robione z W na G i z G na Z dla artykuw nieaktywnych    
    if stat=='OK':
        ArtNieakt=''
        for art in Test:
            if not art[1]:
                ArtNieakt+=rsglobal.Dekoduj(art[0])+','
        if ArtNieakt<>'':
            stat="Nie mona wykona operacji na dokumencie, poniewa na pozycjach s nieaktywne artykuy : "+ArtNieakt.strip(',')
    

    #pobranie typu dokumentu    
    if stat=='OK':
        Typ='X'
        SQL="SELECT typ_dokumentu,czy_urzadzenia FROM rs_typy_dokumentow_m WHERE firma_id='"+firma_id+"'"
        SQL+=" AND dokument_id='"+Dok+"';"
        stat, Test, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat=='OK':
        if len(Test)==0:
            stat='Brak typu dokumentu o podanym kluczu'
        else:
            Typ=rsglobal.Dekoduj(Test[0][0])
            CzyUrzadzenia=Test[0][1]

    #sprawdzenie czy nie ma w pozycjach takiego artykuu ktry nie jest wany w recepturach. Jeli nie to blokujemy przygotowanie.
    if stat == 'OK' and Stan == 'W':
        #sprawdzenie jakie artykuly s w pozycjach pasujcych do receptur
        select = "SELECT DISTINCT p.artykul_id"
        select += " FROM rs_dokumenty_magazynowe_pozycje p "
        select += " JOIN rs_typy_dokumentow_m t ON p.firma_id=t.firma_id and p.dokument_id = t.dokument_id "
        select += " JOIN rs_receptury r ON p.firma_id = r.firma_id and p.artykul_id = r.artykul_id "
        select += " JOIN rs_magazyny m ON p.firma_id = m.firma_id AND m.magazyn_id=p.magazyn_id "
        select += " WHERE p.firma_id='" + firma_id + "' AND (r.oddzial_id = m.oddzial_id or r.oddzial_id ='**')"
        select += " AND p.magazyn_id='" + Mag + "' AND p.dokument_id='" + Dok + "'"
        select += " AND p.data='" + Data + "' AND p.numer=" + Numer + " AND t.typ_dokumentu='P' AND r.stan = 'Z' "
        select += " AND p.dokument_id in (SELECT dokument_id FROM rs_dokumenty_magazyny WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"')"
        select += " ORDER BY 1;"
        stat, Test1, dump, dump = self.PolSQL.Execute(select, 3)

    if stat == 'OK' and Stan == 'W':
        #porwnanie tych artykuw z wanoci
        select = "SELECT DISTINCT p.artykul_id"
        select += " FROM rs_dokumenty_magazynowe_pozycje p "
        select += " JOIN rs_typy_dokumentow_m t ON p.firma_id=t.firma_id and p.dokument_id = t.dokument_id "
        select += " JOIN rs_receptury r ON p.firma_id = r.firma_id and p.artykul_id = r.artykul_id "
        select += " JOIN rs_magazyny m ON p.firma_id = m.firma_id AND m.magazyn_id=p.magazyn_id "
        select += " WHERE p.firma_id='" + firma_id + "' AND (r.oddzial_id = m.oddzial_id or r.oddzial_id ='**') "
        select += " AND p.magazyn_id='" + Mag + "' AND p.dokument_id='" + Dok + "'"
        select += " AND p.data='" + Data + "' AND p.numer=" + Numer + " AND t.typ_dokumentu='P' AND r.stan = 'Z'"
        select += " AND r.data_od = (SELECT max(rrr.data_od) FROM rs_receptury rrr WHERE r.artykul_id = rrr.artykul_id AND rrr.data_od <= '"+Data+"'"
        select += " AND rrr.stan = 'Z' AND (rrr.oddzial_id = m.oddzial_id or rrr.oddzial_id ='**') AND rrr.firma_id='"+firma_id+ "' )"
        select += " AND '" + Data + "' BETWEEN r.data_od and COALESCE(r.data_do,(Case WHEN r.data_do is Null THEN '" + Data + "'     ELSE r.data_do    END)) "
        select += " AND p.dokument_id in (SELECT dokument_id FROM rs_dokumenty_magazyny WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"')"
        select += " ORDER BY 1;"
        stat, Wazn, dump, dump = self.PolSQL.Execute(select, 3)
    if stat == 'OK' and Stan == 'W':
        art= []
        nazwa=[]
        bledne = []
        if len(Test1) != len(Wazn):
            test2 = [x.strip() for row in Test1 for x in row]
            wazn2 = [row[0].strip() for row in Wazn]
            sql = "SELECT artykul_id || '-' || nazwa FROM rs_artykuly WHERE firma_id='"
            sql+= firma_id+"' AND artykul_id in ("
            for x in test2:
                if x not in wazn2:
                    bledne.append(x)
            for x,y in enumerate(bledne):
                sql += "'"+str(y)+"'"

                if x+1 < len(bledne):
                    sql+=","
            sql+=");"
            stat, kom, dump, dump = self.PolSQL.Execute(sql, 3)
            if stat=='OK' and len(kom)>0:
                stat = u'Brak wanej receptury dla artykuu: '
                for x in range(len(kom)):
                    stat += str(kom[x][0])
                    if len(kom)-1 == x:
                        stat += u'.'
                    else:
                        stat += u', '

    # pobranie informacji o magazynach dokumentu - czy ma uwzgldnia ceny.
    if stat=='OK':
        SQL="SELECT COALESCE(dm.magazyn_skad,''),m1.czy_ceny,m1.czy_lokacje,COALESCE(dm.magazyn_dokad,''),"
        SQL+=" m2.czy_ceny,m2.czy_lokacje FROM rs_dokumenty_magazynowe dm "
        SQL+=" LEFT OUTER JOIN rs_magazyny m1 ON (dm.firma_id=m1.firma_id AND dm.magazyn_skad=m1.magazyn_id) "
        SQL+=" LEFT OUTER JOIN rs_magazyny m2 ON (dm.firma_id=m2.firma_id AND dm.magazyn_dokad=m2.magazyn_id) "
        SQL+=" WHERE dm.firma_id='"+firma_id+"' AND dm.magazyn_id='"+Mag+"' AND dm.dokument_id='"+Dok+"'"
        SQL+=" AND dm.data='"+Data+"' AND dm.numer="+Numer+";"

        stat, DCzyCeny, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat=='OK':
        if len(DCzyCeny)==0:
            stat='Brak typu dokumentu o podanym kluczu'
        else:
            CzyCenySkad=DCzyCeny[0][1]
            CzyLokacjeSkad=DCzyCeny[0][2]
            CzyCenyDokad=DCzyCeny[0][4]
            CzyLokacjeDokad=DCzyCeny[0][5]
            if DCzyCeny[0][0]<>'' and DCzyCeny[0][3]<>'':
                if CzyCenySkad != CzyCenyDokad:
                    stat='Magazyny skd i dokd maj inny sposb pamitania cen'

    # sprawdzenie czy uzupenione s wymagane karty przyjcia
    if stat=='OK' and Funkcja in [8,18,28]:
        SQL ="SELECT DISTINCT dmp.artykul_id||'-'||a.nazwa FROM rs_karty_przyjecia kp \n"
        SQL+="JOIN rs_artykuly_karty_przyjecia akp ON akp.firma_id=kp.firma_id AND akp.kod=kp.kod \n"
        SQL+="JOIN rs_magazyny_dokumenty_parametry mdp ON kp.firma_id=mdp.firma_id AND mdp.karta_przyjecia=True \n"
        SQL+="JOIN rs_dokumenty_magazynowe_pozycje dmp ON dmp.firma_id=kp.firma_id AND dmp.artykul_id=akp.artykul_id "
        SQL+="AND mdp.magazyn_id=dmp.magazyn_id AND mdp.dokument_id=dmp.dokument_id \n"
        SQL+="JOIN rs_artykuly a ON a.artykul_id=dmp.artykul_id and a.firma_id=dmp.firma_id \n"
        SQL+=" WHERE dmp.firma_id='"+firma_id+"' AND dmp.magazyn_id='"+Mag+"' AND dmp.dokument_id='"+Dok+"'"
        SQL+=" AND dmp.data='"+Data+"' AND dmp.numer="+Numer+" \n"
        SQL+="AND NOT EXISTS (SELECT dmpkp.kod FROM rs_dokumenty_magazynowe_pozycje_karty_przyjecia dmpkp \n"
        SQL+="WHERE dmpkp.firma_id=dmp.firma_id and dmpkp.magazyn_id=dmp.magazyn_id and dmpkp.dokument_id=dmp.dokument_id \n"
        SQL+="and dmp.numer=dmpkp.numer AND dmpkp.data=dmp.data and dmp.nr_poz=dmpkp.nr_poz and dmp.nr_ppoz=dmpkp.nr_ppoz);"
        stat, spr, dump, dump = self.PolSQL.Execute(SQL, 3)
        if stat<>'OK':
            stat = u"Bd sprawdzenia kart przyjcia w pozycjach dokumentw magazynowych"
        elif len(spr)>1:
            stat='Dla artykuw: '
            for x in spr:
                stat+=str(x[0])+', '
            stat=stat[:-2]+' nie s uzupenione wszystkie potwierdzenia kart przyjcia w pozycjach dokumentu magazynowego'
        elif len(spr)==1:
            stat='Dla artykuu: '+str(spr[0][0])+' nie s uzupenione wszystkie potwierdzenia kart przyjcia w pozycjach dokumentu magazynowego'


#@###########################################
# PRZYGOTOWANIE DOKUMENTU DO ZAMKNIECIA   W --> G  (8,18,28)
#@##########################################
# 28 to rozbijanie/scalanie pozycji dokumentu w stanie G. Pomija cz kontroli i zmian

#----------------------------------------
# obsuga zmiany cech dokumentu (TYP C)
#----------------------------------------
if Funkcja in [8,18,28] and stat=='OK' and Typ=='C':
    # pobranie danych z dokumentu
    SQL="SELECT nr_poz, nr_ppoz, dokument_przych, nr_serii, termin_przyd,cecha1_wartosc,cecha1_opis "
    SQL+=" FROM rs_dokumenty_magazynowe_pozycje WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"'"
    SQL+=" AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+Numer+";"
    stat, DPoz, dump, dump = self.PolSQL.Execute(SQL, 3)
    # ptla po pozycjach
    SQL=""
    if stat=='OK':
        if len(DPoz)==0:
            stat='Brak pozycji na dokumencie magazynowym.'
        
    if stat=='OK':
        # pobranie danych z artykuw przyjcia
        for poz in DPoz:
            if stat=='OK':
                Poz=str(poz[0])
                PPoz=str(poz[1])
                DokPrzych = rsglobal.Dekoduj(poz[2])
                DokAP = DokPrzych[6:8]
                DataAP = DokPrzych[8:18]
                NumerAP = DokPrzych[18:22]
                PozAP = DokPrzych[22:26]
                PPozAP = DokPrzych[26:30]  
                NrSerii = rsglobal.Dekoduj(poz[3])
                TerminPrzyd = rsglobal.Dekoduj(poz[4])
                if TerminPrzyd in ['','None',None]:
                    TerminPrzyd = 'NULL'
                else:
                    TerminPrzyd="'"+TerminPrzyd+"'"
                
                CechaWart = rsglobal.Dekoduj(poz[5])
                CechaOpis = rsglobal.Dekoduj(poz[6])
                if CechaWart in ['','None',None]:
                    CechaWart = 'NULL'
                else:
                    stat=fs_dokumenty_magazynowe_pom.SprawdzPoprawnoscCechy(self,CechaOpis,CechaWart)
                    CechaWart="'"+CechaWart+"'"

            if stat=='OK':
                if DataAP.strip()=='' or DokAP.strip()=='' or NumerAP.strip()=='' or PozAP.strip()=='' or PPozAP.strip()=='':
                    stat='Na pozycji '+Poz+'/'+PPoz+' nie podano lub jest bdny artyku przyjcia na ktrym jest robiona zmiana cechy'
                    
            if stat=='OK':
                # wstawienie obrotw ze starych artykuw przyjecia
                SQL1= "INSERT INTO rs_obroty_magazynowe "
                SQL1+="(firma_id,magazyn_id,dokument_id,data,numer,"
                SQL1+=" typ,typ_znak,nr_poz,nr_ppoz,"
                SQL1+="kontrahent_id,nr_zlecenia,konto_kosztow,nr_faktury,"
                SQL1+="artykul_id,ilosc1,ilosc2,cena1,cena2,cena_pierw1,cena_pierw2,"
                SQL1+="wartosc,nr_serii,termin_przyd,waluta,cena_wal1,cena_wal2,"
                SQL1+="cena_pierw_wal1,cena_pierw_wal2,wartosc_wal,"
                SQL1+="dokument_przych,kierowca,wejscie_data,wejscie_magazyn,"
                SQL1+="dostawca_id,czy_nocny,cecha1_opis,cecha1_wartosc,produkt_id)"
                SQL1+=" SELECT '"+firma_id+"','"+Mag+"','"+Dok+"','"+Data+"',"+Numer
                SQL1+=",'C',"
                    #miejce na typ_znak
                SQL2=str(Poz)+","+str(PPoz)+","
                SQL2+="dm.kontrahent_id,dm.nr_zlecenia,dm.konto_kosztow,dm.nr_faktury,"
                SQL2+="ap.artykul_id,ap.ilosc1,ap.ilosc2,ap.cena1,ap.cena2,ap.cena_pierw1,ap.cena_pierw2,"
                SQL2+="ROUND(ap.cena1*ap.ilosc1,2),ap.nr_serii,ap.termin_przyd,ap.waluta,ap.cena_wal1,ap.cena_wal2,"
                SQL2+="ap.cena_pierw_wal1,ap.cena_pierw_wal2,ROUND((ap.cena_wal1*ap.ilosc1)+0.0001,2),"
                SQL2+="'"+DokPrzych+"',dm.kierowca,ap.wejscie_data,ap.wejscie_magazyn,"
                SQL2+="dm.dostawca_id,dm.czy_nocny,ap.cecha1_opis,ap.cecha1_wartosc,NULL"                
                SQL2+=" FROM rs_artykuly_przyjecia ap "
                SQL2+=" INNER JOIN rs_dokumenty_magazynowe dm USING (firma_id,magazyn_id,dokument_id,data,numer) "
                SQL2+=" WHERE ap.firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"' AND data='"+DataAP+"'"
                SQL2+=" AND dokument_id='"+DokAP+"' AND numer="+NumerAP+" AND nr_poz="+PozAP+" AND nr_ppoz="+PPozAP+";"
                
                SQL+=SQL1+"-1,"+SQL2
                
                # update na rs_artykuly_przyjecia
                SQL+=" UPDATE rs_artykuly_przyjecia SET nr_serii='"+NrSerii+"',termin_przyd="+TerminPrzyd+","
                SQL+="cecha1_wartosc="+CechaWart+" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"'"
                SQL+=" AND data='"+DataAP+"' AND dokument_id='"+DokAP+"' AND numer="+NumerAP
                SQL+=" AND nr_poz="+PozAP+" AND nr_ppoz="+PPozAP+";"
        
                # wstawienie obrotw z nowych artykuw przyjcia
                SQL+=SQL1+"1,"+SQL2
                
                # zmiana cechy na pozycjach spisu z natury
                SQL+="UPDATE rs_inwentaryzacja_pozycje_przyjecia SET nr_serii='"+NrSerii+"',termin_przyd="+TerminPrzyd+","
                SQL+="cecha1_wartosc="+CechaWart+" "
                SQL+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"' AND ap_data='"+DataAP+"' "
                SQL+=" AND ap_dokument_id='"+DokAP+"' AND ap_numer="+NumerAP+" AND ap_nr_poz="+PozAP+" AND ap_nr_ppoz="+PPozAP
                SQL+=" AND (SELECT stan FROM rs_inwentaryzacja WHERE firma_id='"+firma_id+"'"
                SQL+=" AND magazyn_id='"+Mag+"' AND data=rs_inwentaryzacja_pozycje_przyjecia.data "
                SQL+=" AND kateg_artykulu=rs_inwentaryzacja_pozycje_przyjecia.kateg_artykulu)='W';"

        if stat=='OK':
            # zamknicie dokumentu na Z
            SQL+="UPDATE rs_dokumenty_magazynowe SET stan='Z' WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"'"
            SQL+=" AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+Numer+";"
            stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1)
        
    # jeli si udao to zdejmujemy rezerwacje
    if stat=='OK':
        stat=fs_dokumenty_magazynowe_rezerwacje_pom.ZmienRezerwacjeNaArtPrzyjDokumentC(self,[Mag,Dok,Data,Numer],[],'-')


#----------------------------------------
# obsuga pozostaych typw dokumentw
#----------------------------------------

elif Funkcja in [8,18,28]  and stat=='OK':
    # ----------------------------------------
    # sprawdzenie projektu    # sprawdzenie projektu PW 2017-08-22
    # ----------------------------------------
    if stat == 'OK':
        sql = "SELECT projekt_id FROM rs_dokumenty_magazynowe WHERE firma_id='" + firma_id + "' "
        sql += "AND magazyn_id='" + Mag + "' AND dokument_id = '" + Dok + "' AND data='" + Data + "' AND numer='" + str(
            Numer) + "';"
        stat, dd, dump, dump = self.PolSQL.Execute(sql, 3)
        if stat != 'OK':
            stat = u'Bdy przy pobraniu projektu z dokumentu magazynowego:' + stat
        elif stat == 'OK' and len(dd) > 0 and dd[0][0]:
            sql = "SELECT data_do>='" + Data + "' FROM rs_projekty WHERE firma_id='" + firma_id + "' "
            sql += "AND projekt_id='" + dd[0][0] + "' and data_do is not null;"
            stat, d, dump, dump = self.PolSQL.Execute(sql, 3)
            if stat != 'OK':
                stat = 'Bdy przy sprawdzaniu daty projektu :' + stat
            elif len(d) == 1 and d[0][0] == False:
                stat = "Data dokumentu magazynowego jest pniejsza ni 'data do' projektu."

    if Funkcja != 28:
        if Stan != "W":
            stat = "Dokument jest w stanie '"+Stan+"' - nie mona przygotowa dokumentu do zamknicia"
            rsglobal.ZapiszDoLogow("(DM) OPERACJA W->G: "+DokumentDoLogowania+" by w stanie '"+Stan+"'",5)

    # test czy jeli sprawdzana jest korekta innego dokumentu to dokument ma wpisany opis
    if stat=='OK':
        if TestKorekta <>'':
            if TestOpis == '':
                stat='Na korekcie dokumentu magazynowego musi si znale opis powodu wystawienia korekty'
        
    if stat=='OK':
        PolaNaglowka=''
        DEL=''
        kwota = 0
        kwota_wal = 0
        kwotap = 0
        kwotap_wal = 0
        #Sownik z informacj jaki artyku zosta zamieniony na jaki zamiennik i w jakiej iloci.
        SArtZam={}
    
        if Funkcja != 28:
            #sprawdzenie obecnoci odpowiednich danych na nagowku dokumentu
            SQL="SELECT rsf_sprawdz_naglowek_dok_mag('"+firma_id+"','"+Mag+"','"+Dok+"','"+Data+"',"+Numer+");"
            stat, Test, dump, dump = self.PolSQL.Execute(SQL, 3)

            if stat == "OK":
                if Test[0][0]<>'OK':
                    stat=Test[0][0]

    #pobranie danych nagwka
    TypDokumentu='X'
    if stat == "OK":
        SQL ="SELECT dm.lifo,dm.kontrahent_id,dm.dostawca_id,dm.magazyn_dokad,td.typ_dokumentu,"
        SQL+=" dm.zo_wyznacznik_id,dm.zo_data,dm.zo_numer,dm.czy_pobrane_z_zamowien,dm.operator_wms"
        SQL+=" FROM rs_dokumenty_magazynowe dm INNER JOIN rs_typy_dokumentow_m td USING (firma_id,dokument_id) "
        SQL+=" WHERE firma_id='"+firma_id+"'"
        SQL+=" AND magazyn_id='"+Mag+"' AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+Numer+";"
        stat, DaneN, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat == "OK":
        if len(DaneN)==0:
            stat="Brak podanego dokumentu magazynowego"
        elif len(DaneN)>1:
            stat="Podany dokument wystpuje w wicej ni raz w bazie danych!"
        else:
            TypDokumentu=DaneN[0][4]
        
    #sprawdzenie czy kontrahent i dostawca s aktywni w chwili zamykania dokumentu    
    if Funkcja != 28:
        if stat == "OK":
            if DaneN[0][1] not in ['',None,'None']:
                SQL="SELECT aktywny FROM rs_kontrahenci WHERE firma_id='"+firma_id+"' AND kontrahent_id='"+rsglobal.Dekoduj(DaneN[0][1])+"';"
                stat, DaneTest, dump, dump = self.PolSQL.Execute(SQL, 3)
                if stat =="OK":
                    if len(DaneTest)==0:
                        stat='Brak kontrahenta, ktry jest podany w nagwku dokumentu: '+DaneN[0][1]
                    elif not DaneTest[0][0]:
                        stat='Kontrahent z nagwka dokumentu jest nieaktywny'
                        
        if stat == "OK":
            if DaneN[0][2] not in ['',None,'None']:
                SQL="SELECT aktywny FROM rs_kontrahenci WHERE firma_id='"+firma_id+"' AND kontrahent_id='"+rsglobal.Dekoduj(DaneN[0][2])+"';"
                stat, DaneTest, dump, dump = self.PolSQL.Execute(SQL, 3)
                if stat =="OK":
                    if len(DaneTest)==0:
                        stat='Brak dostawcy, ktry jest podany w nagwku dokumentu: '+DaneN[0][1]
                    elif not DaneTest[0][0]:
                        stat='Dostawca z nagwka dokumentu jest nieaktywny'
                    
                    
    #pobranie pozycji z potencjalnymi danymi lokacji. Jak ich nie ma to dane lokacji s puste
    if stat == "OK":
        #kolumny 0 - 6
        SQLPoz="SELECT dp.nr_poz,dp.nr_ppoz,dp.artykul_id,dp.ilosc1,dp.ilosc2,dp.cena1,dp.cena2,"
        # kolumny 7 - 12
        SQLPoz+=" dp.cena_wal1,dp.cena_wal2,COALESCE(dp.nr_serii,''),dp.termin_przyd,dp.opis,dp.czy_rozbicie,"
        #kolumny 13 - 17
        SQLPoz+=" dp.dokument_przych,dp.czy_cena,dp.czy_seria,dp.czy_termin,dp.czy_dodatkowy_nr_serii,"
        # kolumny 18 - 22
        SQLPoz+=" dp.lokacja_skad,dp.lokacja_dokad,dl.lokacja_skad,dl.lokacja_dokad,dl.ilosc,"
        # kolumny 23 - 24
        SQLPoz+=" dp.cecha1_opis,COALESCE(dp.cecha1_wartosc,''),"
        #kolumny 25 - 29
        SQLPoz+=" dp.zo_wyznacznik_id,dp.zo_data,dp.zo_numer,dp.zo_nr_poz,dp.zo_nr_ppoz,"
        #kolumny 30 - 31
        SQLPoz+=" dp.produkt_id, dp.dokument_przych,"
        # kolumna 32 - dostpne dni przydatnoci
        SQLPoz+=" CASE WHEN dp.termin_przyd IS NULL THEN 10000 ELSE termin_przyd - '"+Data+"'::DATE END,"
        # kolumny 33 - 36
        SQLPoz+=" COALESCE(dp.opakowanie_id,''), COALESCE(dp.paleta_id,''),dp.zo_nr_zlec,dp.typ_zlec,"
        # kolumna 37
        SQLPoz+=" COALESCE(a.tylko_w_opakowaniach,FALSE)"
        
        SQLPoz+=" FROM rs_dokumenty_magazynowe_pozycje dp LEFT OUTER JOIN rs_dokumenty_magazynowe_lokacje dl "
        SQLPoz+=" USING (firma_id,magazyn_id,dokument_id,data,numer,nr_poz,nr_ppoz)"
        SQLPoz+=" LEFT OUTER JOIN rs_artykuly a ON (a.firma_id=dp.firma_id AND a.artykul_id=dp.artykul_id)"
        SQLPoz+=" WHERE dp.firma_id='"+firma_id+"' AND dp.magazyn_id='"+Mag+"'"
        SQLPoz+=" AND dp.dokument_id='"+Dok+"' AND dp.data='"+Data+"' AND dp.numer="+Numer
        SQLPoz+=" ORDER BY dp.nr_poz,dp.nr_ppoz,dl.lp;"
        stat, DaneP, dump, dump = self.PolSQL.Execute(SQLPoz, 3)
            
    if stat == "OK":
        if len(DaneP)==0:
            stat="Brak pozycji na dokumencie magazynowym"            


    #sprawdzenie czy, w razie koniecznoci, dane lokacji s poprawnie uzupenione
    if stat == "OK":
        STest={}
        for poz in DaneP:
            if CzyLokacjeSkad:
                if (not poz[18] in (None,'None','')) and (not poz[20] in (None,'None','')):
                    stat="W pozycji "+str(poz[0])+"/"+str(poz[1])+" podano rwnoczenie magazyn lokacji skd na pozycji i w podatabeli lokacji"
                    break
                
                # ta blokada zdjta (stat zostaje OK). Jeli mamy dokument wydania to moe by tak, 
                # e lokacje nie s podane, ale s potem pobierane z dokumentw przychodowych.
                # Zablokowane zostay tylko dokumenty typu L
                if (poz[18] in (None,'None','')) and (poz[20] in (None,'None','')):
                    if TypDokumentu=='L':
                        stat="W pozycji "+str(poz[0])+"/"+str(poz[1])+" nie podano lokacji skd, a s one wymagane przez magazyn"
                        break
            
            if CzyLokacjeDokad:
                if (not poz[19] in (None,'None','')) and (not poz[21] in (None,'None','')):
                    stat="W pozycji "+str(poz[0])+"/"+str(poz[1])+" podano rwnoczenie magazyn lokacji dokd na pozycji i w podatabeli lokacji"
                    break

                if (poz[19] in (None,'None','')) and (poz[21] in (None,'None','')):
                    stat="W pozycji "+str(poz[0])+"/"+str(poz[1])+" nie podano lokacji dokd, a s one wymagane przez magazyn"
                    break
                
            
            if CzyLokacjeSkad or CzyLokacjeDokad:
                # jeli puste lokacje na pozycji to sumujemy
                if (poz[18] in (None,'None','')) and (poz[19] in (None,'None','')) and not ((poz[20] in (None,'None','')) and (poz[21] in (None,'None',''))):
                    if poz[1]>0:
                        stat="W pozycji "+str(kl[0]).strip('.0')+"/"+str(kl[1]).strip('.0')+" podana jest tabela z lokacjami"
                        break
                    kl=(poz[0],poz[1])
                
                    if kl in STest:
                        STest[kl][1]+=poz[22]
                    else:
                        if self.lokacje_jm=='1':
                            STest[kl]=[poz[3],poz[22]]
                        else:
                            STest[kl]=[poz[4],poz[22]]
                
        for kl in STest:
            if stat=='OK':
                if rsglobal.RSRound(STest[kl][0],4)<>rsglobal.RSRound(STest[kl][1],4):
                    stat="W pozycji "+str(kl[0]).strip('.0')+" nierwna ilo na pozycji : "+str(STest[kl][0])+" oraz w lokacjach : "+str(STest[kl][1])

    if stat == "OK":
        #----------------------------------
        # DOKUMENT WYDANIA (z wyjatkiem detalu)
        #----------------------------------
        
        if Typ in ['W','M','L','D']:
            # sownik wykorzystanych rezerwacji
            SlowWykRez = {}
            # sownik limitw przydatnoci
            SLimityPrzyd = {}
            
            # wyznaczenie kolejnoci sprawdzania  danych na podstawie wpisw w bazie
            if stat=='OK':
                DokPom=[Mag,Dok,Data,Numer]
                (stat,OrderZDanych) = fs_kolejkowanie_pom.PodajOrderByDokumentuLubTekstu(self.PolSQL,firma_id,DokPom)
                if stat == 'OK':
                    if OrderZDanych == '':
                        stat='Brak okrelonego kolejkowania w artykuach przyjcia'
                        
            # zdjcie iloci do pobrania z lokacji
            if stat=='OK':
                stat=fs_dokumenty_magazynowe_lokacje_pom.ZmienIlosciDoWydaniaWLokacjachNaCalymDokumencie(self,Mag,Dok,Data,Numer,'-')   

            if stat=='OK':
                # sprawdzenie czy w magazynie jest blokada terminu patnoci z karencj
                CzyBlokadaTerminu=False
                if Typ=='W':
                    SQL="SELECT NOT czy_przeterminowane FROM rs_magazyny WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"';"
                    stat, DaneMag, dump, dump = self.PolSQL.Execute(SQL, 3)
                    if stat=='OK':
                        if len(DaneMag)>0:
                            CzyBlokadaTerminu=DaneMag[0][0]
                        
                # Ptla po pozycjach dokumentu magazynowego. Pobranie danych.
                Wynik = []
                TestowyNrPoz=0
                ppoz=0
                OperatorWMS = DaneN[0][9]

                for poz in DaneP:
                    if stat=='OK':
                        if TestowyNrPoz!=poz[0]:
                            TestowyNrPoz=poz[0]
                            ppoz=0
                    
                    if stat == "OK":
                        art=rsglobal.Dekoduj(poz[2]).strip()
                        Artykuly="'"+art+"'"
                        #sprawdzenie czy bdziemy uywa zamiennikw te (przy okazji pobieramy przelicznik)
                        SQL="SELECT wydawac_zamiennik,COALESCE(zamiennik_id,artykul_id),przelicznik,dni_karencji," # 0-3
                        SQL+="dynamiczny_przelicznik,wydanie_proporcjonalne,jm_kalk::INT" # 4-6
                        SQL+=" FROM rs_artykuly WHERE firma_id='"+firma_id+"' AND artykul_id='"+art+"';"
                        stat, TestZam, dump, dump = self.PolSQL.Execute(SQL, 3)

                    if stat=='OK':
                        WydawacZam=TestZam[0][0]
                        Zamiennik=rsglobal.Dekoduj(TestZam[0][1]).strip()
                        Przelicznik=TestZam[0][2]
                        DniKarencji=rsglobal.UsunKropkaZero(TestZam[0][3])
                        DynamPrzel=TestZam[0][4]
                        if TestZam[0][5]:
                            WydanieProporcjonalne = TestZam[0][6]
                        else:
                            WydanieProporcjonalne = 0
                            
                        if DniKarencji==None:
                            DniKarencji=0
                        
                        if WydawacZam:
                            SQL="SELECT artykul_id FROM rs_artykuly WHERE firma_id='"+firma_id+"'"
                            SQL+=" AND COALESCE(zamiennik_id,artykul_id)='"+Zamiennik+"'"
                            stat, DaneZam, dump, dump = self.PolSQL.Execute(SQL, 3)
                            if stat=='OK':
                                Artykuly=''
                                for artyk in DaneZam:
                                    Artykuly+="'"+rsglobal.Dekoduj(artyk[0]).strip()+"',"
                                Artykuly=Artykuly.strip(',')
                        
                    if stat=='OK':
                        Przyczyny=['- fizyczny brak artykuu w magazynie']
                        Filtr =" FROM rs_artykuly_przyjecia WHERE firma_id='"+firma_id+"' "
                        Filtr+=" AND magazyn_id = '"+Mag+"' AND data <= '"+Data+"' AND artykul_id IN ("+Artykuly+") "
                        Filtr+=" AND (ilosc1-rezerwacja1>0 OR ilosc2-rezerwacja2>0) AND NOT wirtualne"
                        cena1=poz[5]
                        cena2=poz[6]
                        dmnrpoz = poz[0]
                        dmnrppoz = poz[1]
                        ilosc1=rsglobal.RSRound(poz[3],6)
                        rsglobal.ZapiszDoLogow('ilosc1 na poczatku '+str(ilosc1),5)
                        if ilosc1==0 and Typ != 'D':
                            stat='UWAGA! Zerowa ilo w pozycji '+rsglobal.UsunKropkaZero(dmnrpoz)+'/'+rsglobal.UsunKropkaZero(dmnrppoz)
                            
                    if stat=='OK':
                        ilosc2=rsglobal.RSRound(poz[4],6)
                        NrSerii = rsglobal.Dekoduj(poz[9])
                        Termin = rsglobal.Dekoduj(poz[10])
                        Cecha1Opis = rsglobal.Dekoduj(poz[23])
                        Cecha1Wart = rsglobal.Dekoduj(poz[24]).strip()
                        if Cecha1Wart<>'':
                            stat=fs_dokumenty_magazynowe_pom.SprawdzPoprawnoscCechy(self,Cecha1Opis,Cecha1Wart)
                            if stat!= 'OK':
                                stat+='. Dotyczy artykuu '+art
                    
                    if stat=='OK':
                        ZOWyznacznik = rsglobal.Dekoduj(poz[25]).strip()
                        ZOData = rsglobal.Dekoduj(poz[26])
                        ZONumer = rsglobal.Dekoduj(poz[27])
                        ZONrPoz = poz[28]
                        ZONrPPoz = poz[29]
                        ZONrZlec = poz[35]
                        ZOTypZlec = poz[36]
                        Produkt = rsglobal.Dekoduj(poz[30]).strip()
                        DokAPF=''
                        DokPrzychF = rsglobal.Dekoduj(poz[31]).strip()
                        if not DokPrzychF in ('None',None,''):
                            DokAPF = DokPrzychF[6:8]
                            DataAPF = DokPrzychF[8:18]
                            NumerAPF = DokPrzychF[18:22]
                            PozAPF = DokPrzychF[22:26]
                            PPozAPF = DokPrzychF[26:30]  
                        
                        if ZOWyznacznik in ['None',None,'']:
                            ZOWyznacznik='NULL'
                        else:
                            ZOWyznacznik = "'"+ZOWyznacznik+"'"
                            
                        if ZOData in ['    -  -  ',None,'None']:
                            ZOData='NULL'
                        else:
                            ZOData="'"+ZOData+"'"
                        
                        if ZONrZlec in ['    -  -  ',None,'None']:
                            ZONrZlec='NULL'
                        else:
                            ZONrZlec="'"+ZONrZlec+"'"
                            
                        if ZOTypZlec in ['    -  -  ',None,'None']:
                            ZOTypZlec='NULL'
                        else:
                            ZOTypZlec="'"+ZOTypZlec+"'"
                        
                        if Termin in ['    -  -  ',None,'None']:
                            Termin=''
                        
                        OpisWp = rsglobal.Dekoduj(poz[11]).strip()
                        if OpisWp in ['None',None]:
                            OpsWp=""
                        czycena = cena1>0
                        if czycena:
                            czy_cena="TRUE"
                        else:
                            czy_cena="FALSE"
                
                        czyseria = (NrSerii != '')
                        if czyseria:
                            czy_seria="TRUE"
                        else:
                            czy_seria="FALSE"
                
                        czytermin = (Termin <>'')
                        if czytermin:
                            czy_termin="TRUE"
                        else:
                            czy_termin="FALSE"
                
                        czyrozbicie = poz[12]
                        if czyrozbicie:
                            czy_rozbicie="TRUE"
                        else:
                            czy_rozbicie="FALSE"
                            
                        if Cecha1Wart in ('','None',None):
                            czy_cecha1="FALSE"
                        else:
                            czy_cecha1="TRUE"

                        if cena1 > 0:
                            Przyczyny.append('- niezgodno cen')
                            Filtr+=" AND cena1 = "+str(cena1)
                        elif cena2 > 0:
                            Filtr+=" AND cena2 = "+str(cena2)
                            
                        if NrSerii not in ['','None']:
                            Przyczyny.append('- niezgodno numerw serii')
                            Filtr+=" AND trim(trailing from nr_serii) = trim(trailing from '"+NrSerii+"')"
                
                        if Termin!= '':
                            Przyczyny.append('- niezgodno terminu przydatnoci')
                            Filtr+=" AND termin_przyd='"+Termin+"'"
                        
                        if not Cecha1Wart in ('','None',None):
                            Przyczyny.append('- niezgodno cech')
                            Filtr+=" AND TRIM(cecha1_wartosc)='"+Cecha1Wart.strip()+"'"
                            
                        # uwzgldnienie dni karencji w filtrze
                        if DniKarencji>0 and CzyBlokadaTerminu:
                            Przyczyny.append('- przekroczony termin karencji')
                            Filtr+=" AND (termin_przyd-INTEGER '"+str(DniKarencji)+"'>=current_date OR termin_przyd IS NULL)"
                            
                        # dokadnie wybrany dokument przyjcia
                        czy_dok_przych="FALSE"
                        if not DokAPF in ('None',None,'','  '):
                            Przyczyny.append('- brak artykuu na konkretnym wybranym dokumencie')
                            czy_dok_przych="TRUE"
                            Filtr+=" AND data='"+DataAPF+"' AND dokument_id='"+DokAPF+"' AND numer="+NumerAPF
                            Filtr+=" AND nr_poz="+PozAPF+" AND nr_ppoz="+PPozAPF+" "
                
                        # jeli w magazynie s uwzgldniane lokalizacje i s podane na dokumencie to 
                        # dorzucamy je do filtra - najpierw sprawdzana jest lokacja z tabelki a potem z pozycji
                        LokacjaSkad="NULL"
                        
                        CzyLokacje="FALSE"
                        if CzyLokacjeSkad:
                            LokacjaSkad=poz[20]
                            if LokacjaSkad in ('',None,'None'):
                                LokacjaSkad=poz[18]
                            else:
                                # jest z tabeli - bierzemy stamtd ilosc
                                ilosc1=poz[22]
                                if self.lokacje_jm=='2':
                                    ilosc1=rsglobal.RSRound(ilosc1/Przelicznik,4)
                                CzyLokacje="TRUE"
                            
                            if LokacjaSkad in ('',None,'None'):
                                LokacjaSkad="NULL"
                            else:
                                LokacjaSkad="'"+rsglobal.Dekoduj(LokacjaSkad)+"'"
                                Filtr=Filtr+" AND lokacja_id="+LokacjaSkad
                                
                        # uwzgldniamy te lokacj dokd - na potrzeby MM
                        LokacjaDokad="NULL"
                        if CzyLokacjeDokad:        
                            LokacjaDokad=poz[21]
                            if LokacjaDokad in ('',None,'None'):
                                LokacjaDokad=poz[19]
                            
                            if LokacjaDokad in ('',None,'None'):
                                LokacjaDokad="NULL"
                            else:
                                LokacjaDokad="'"+rsglobal.Dekoduj(LokacjaDokad)+"'"
                        
                        if Produkt=='None':
                            Produkt = 'NULL'
                        else:
                            Produkt = "'"+Produkt+"'"
                        
                        czy_opak = 'FALSE'
                        czy_paleta = 'FALSE'
                        Opak = poz[33]
                        Paleta = poz[34]
                        TylkoWOpakowaniach = poz[37]

                        if Opak != '':
                            Przyczyny.append('- brak artykuu o podanym opakowaniu')
                            czy_opak="TRUE"
                            Filtr+=" AND opakowanie_id = '"+Opak+"'"
                            
                        if Paleta != '':
                            Przyczyny.append('- brak artykuu o podanej palecie')
                            czy_paleta="TRUE"
                            Filtr+=" AND paleta_id = '"+Paleta+"'"
                        
                        # sprawdzenie czy dla podanego kontrahenta nie obowizuj tylko niektre opakowania
                        if DaneN[0][1] not in ['',None,'None'] and TylkoWOpakowaniach:
                            SQL="SELECT DISTINCT opakowanie_id FROM rs_artykuly_opakowania_zbiorcze WHERE "
                            SQL+="firma_id='"+firma_id+"' AND artykul_id='"+art+"' AND kontrahent_id='"+rsglobal.Dekoduj(DaneN[0][1])+"'"
                            stat, DTestOp, dump, dump = self.PolSQL.Execute(SQL, 3)
                            if stat != 'OK':
                                stat = 'Bd pobierania danych o opakowaniach kontrahenta: '+stat
                            elif len(DTestOp) > 0:
                                Przyczyny.append('- brak artykuu w opakowaniach przypisanych do kontrahenta')
                                Filtr+=" AND opakowanie_id IN ("
                                for opakt in DTestOp:
                                    Filtr+="'"+rsglobal.Dekoduj(opakt[0])+"',"
                                Filtr = Filtr.strip(',')+') '

                    if stat == "OK":
                        # kolumny 0-9
                        SQL="SELECT data,dokument_id,numer,nr_poz,nr_ppoz,ilosc1,ilosc2,cena1,cena2,cena_wal1,"
                        # kolumny 10-15
                        SQL+="cena_wal2,cena_pierw1,cena_pierw2,cena_pierw_wal1,cena_pierw_wal2,rezerwacja1,"
                        # kolumny 16-22
                        SQL+="nr_serii,termin_przyd,opis,wejscie_data,wejscie_magazyn, dostawca_id,artykul_id,"
                        # kolumny 23-26
                        SQL+="cecha1_opis,cecha1_wartosc,lokacja_id,rezerwacja2,"
                        # kolumna 27 - dostpne dni przydatnoci
                        SQL+=" CASE WHEN termin_przyd IS NULL THEN 10000 ELSE termin_przyd - '"+Data+"'::DATE END,"
                        # kolumny 28 - 30
                        SQL+="COALESCE(opakowanie_id,''),COALESCE(paleta_id,''),COALESCE(wsp_ilosci_opakowanie,0),"
                        # kolumny 31 - 32
                        SQL+="COALESCE(wsp_ilosci_paleta),COALESCE(wsp_ilosci_lokacja,0) "
                        SQL+=Filtr
                        SQL+=" ORDER BY CASE WHEN (artykul_id='"+art+"') THEN 1 ELSE 2 END,"
                        SQL+=OrderZDanych+",data,dokument_id,numer,nr_poz,nr_ppoz FOR UPDATE NOWAIT;"
                        rsglobal.ZapiszDoLogow('!!! SQLAP '+SQL,5)
                        stat, DaneAP, dump, dump = self.PolSQL.Execute(SQL, 3,CzyDodawacBlad=True)
                        rsglobal.ZapiszDoLogow('!!! DaneAP '+str(DaneAP),5)
                        if stat != 'OK':
                            stat = 'Bd pobrania danych z artykuw przyje (potencjalne zakleszczenie danych, sprbuj jeszce raz): '+stat
                            dane.append(stat)
                        
                    # sprawdzenie czy wszystkie pozycje maj wypenion dat wejcia
                    if stat == "OK":
                        LPustychDat=[]
                        for pozap in DaneAP:
                            if pozap[19] in ['','    -  -  ',None,"None"]:
                                LPustychDat.append(rsglobal.rseval(str(pozap)))
                        #i ewentualne wysanie poczty do admina
                        if len(LPustychDat)>0:
                            SQLNrW="SELECT nextval('rs_wiadomosci_id');"    
                            stat,DNrW,dump,dump = self.PolSQL.Execute(SQLNrW,3)
                            if stat<>'OK':
                                stat = 'Bdy pobierania id wiadomoci :'+stat
                                dane.append(stat)
                            else:
                                NumerWiad=DNrW[0][0]
                                Temat='Puste daty wejcia w dokumentach'
                                Tresc=" Znaleziono na nastpujcych dokumentach :\n"
                                for poz in LPustychDat:
                                    Tresc+='mag: '+Mag+" dok: "+poz[1]+" data : "+poz[0]+" numer : "+str(poz[2])+" nr poz : "+str(poz[3])+" nr ppoz : "+str(poz[4])+"\n"
                                TypMaila='N'
                                SQLInsW="INSERT INTO rs_wiadomosci (firma_id,wiadomosc_id,operator_id,naglowek,tresc,priorytet,usunieta)"
                                SQLInsW+=" VALUES ('"+firma_id+"',"+str(NumerWiad)+",0,'"+str(Temat)+"'"
                                SQLInsW+=",'"+str(Tresc)+"','"+str(TypMaila)+"',FALSE);"
                                SQLInsW+="INSERT INTO rs_wiadomosci_operatorzy (wiadomosc_id,folder_id,operator_id,usunieta,nowa) "
                                SQLInsW+="VALUES ("+str(NumerWiad)+",0,0,FALSE,TRUE);"   
                                stat,dump,dump,dump = self.PolSQL.Execute(SQLInsW,1)   
                                if stat<>'OK':
                                    stat = 'Bdy dodawania wiadomoci :'+stat
                                    dane.append(stat)
                    
                    # sprawdzenie czy limity przydatnoci s przestrzegane. Jeli nie to rcznie usuwamy takie wiersze
                    # Nie mona da tego do zapytania, bo trzeba uwzgldni zamienniki. Dziaa tylko dla WZ
                    if stat == "OK" and Dok == 'WZ' and DaneN[0][1] != '':
                        DodacPrzyczynyLimity = True
                        DAPTmp = []
                        for pozap in DaneAP:
                            if stat == 'OK':
                                # jeli nie ma limitu dla artykuu to go pobieramy
                                if not pozap[22] in SLimityPrzyd:
                                    stat,Limity = fs_dokumenty_magazynowe_pom.PodajLimityTerminuPrzydatnosci(self,pozap[22],DaneN[0][1],'W')
                                    if stat == 'OK':
                                        SLimityPrzyd[pozap[22]] = Limity
                                        
                            if stat == 'OK':
                                if SLimityPrzyd[pozap[22]][0] != 0:
                                # jeli dni limitu s wiksze ni dni dostpne
                                    if SLimityPrzyd[pozap[22]][0] > pozap[27]:
                                        if DodacPrzyczynyLimity:
                                            Przyczyny.append('- przekroczono dopuszczalne limity terminw przydatnoci')
                                            DodacPrzyczynyLimity = False
                                    else:
                                        # limit w normie
                                        DAPTmp.append(rsglobal.rseval(str(pozap)))
                                else:
                                    # brak limitu
                                    DAPTmp.append(rsglobal.rseval(str(pozap)))
                        
                        if stat == 'OK':            
                            DaneAP = rsglobal.rseval(str(DAPTmp))

                    if stat == 'OK':            
                        if len(DaneAP) == 0:
                            stat = 'Brak pozycji w artykuach-przyjeciach o podanych parametrach w pozycji '+str(poz[0])+"/"+str(poz[1])+" indeksy: "+str(Artykuly)
                            stat+="\n Moliwe przyczyny to:"
                            for p in Przyczyny:
                                stat+="\n"+p
                                        
                            dane.append(stat)

                    # petla po artykulach przyjecia, ktore pasuja do wzorca. Skladamy liste zapytan, ktore wykonane 
                    # wygeneruja poprawne zdjecie ilosci z magazynow dla pojedynczej pozycji
                    if stat == "OK":
                        # lista artykuw z przelicznikami i ilociami do zdjcia,
                        # z ktrej bdziemy korzysta pniej i uzupenia w miar potrzeby
                        # LArt[symbol artykulu] = [przelicznik,ilosc1,ilosc2]
                        LArt={}
                        # Sownik z limitami, ktre mona pobra w oparciu o rs_artykuly_magazyny oraz rs_rezerwacje
                        SLimity={}
                        UPD=''
                        lokacja_prz = DaneAP[0][25]
                        
                        #prba znalezienia rezerwacji na ten artyku w rezerwacjach. Pobieramy przed ptl, by nie 
                        # pobiera niezapisanych rezerwacji dwa razy dla rnych podpozycji
                        # pobieramy te ewentualne rezerwacje dla zamiennikw tego artykuu
                        if Typ=='W':                            
                            KontrRez=rsglobal.Dekoduj(DaneN[0][1])
                            ArtRez=art
                            MagRez=Mag
    
                            SQLRez ="SELECT rezerwacja_id,rezerwacja-zrealizowano,artykul_id,rezerwacja-zrealizowano"
                            SQLRez+=" FROM rs_artykuly_rezerwacje WHERE "
                            SQLRez+=" firma_id='"+firma_id+"' AND artykul_id IN ("+Artykuly+")"
                            SQLRez+=" AND kontrahent_id='"+KontrRez+"' AND magazyn_id='"+MagRez+"'"
                            SQLRez+=" AND rezerwacja-zrealizowano>0 AND not zamknieta"
                            SQLRez+=" ORDER BY czas_rezerwacji;"
                            stat, DaneRez, dump, dump = self.PolSQL.Execute(SQLRez, 3)
                            
                            # uzupenienie SLimity
                            if stat=='OK':
                                for rezerw in DaneRez:
                                    SLimity[rsglobal.Dekoduj(rezerw[2]).strip()]=[rezerw[1],0]
                                    
                    if stat=='OK':
                        # pobranie danych do limitw z artykuly_magazyny
                        SQLArtMag="SELECT artykul_id,ilosc1-rezerwacja1,ilosc2-rezerwacja2 FROM rs_artykuly_magazyny "
                        SQLArtMag+=" WHERE firma_id='"+firma_id+"' AND artykul_id IN ("+Artykuly+")"
                        SQLArtMag+=" AND magazyn_id='"+Mag+"';"
                        stat, DaneArtMag, dump, dump = self.PolSQL.Execute(SQLArtMag, 3)
                        # uzupenienie SLimity
                        if stat=='OK':
                            for artmag in DaneArtMag:
                                if rsglobal.Dekoduj(artmag[0]).strip() in SLimity:
                                    SLimity[rsglobal.Dekoduj(artmag[0]).strip()][0]+=artmag[1]
                                    SLimity[rsglobal.Dekoduj(artmag[0]).strip()][1]+=artmag[2]
                                else:
                                    SLimity[rsglobal.Dekoduj(artmag[0]).strip()]=[artmag[1],artmag[2]]
                            
                    if stat == "OK":   
                        # to pozwala wej w ptl AP, ale od razu jest zastpowane przez prawdziw warto zmiennej
                        TestDynamPrzel=True
                        dokumentAP=''
                        for pozAP in DaneAP:
                            if stat =='OK':
                                artp=rsglobal.Dekoduj(pozAP[22]).strip()

                                #jeli nie ma artykuu na licie to pobieramy dla niego przelicznik i informacje o dynamicznym przeliczniku
                                # i informacje o cakowitoci jednostek miar
                                if not artp in LArt:
                                    SQL="SELECT a.przelicznik,a.dynamiczny_przelicznik,j1.calkowita,j2.calkowita "
                                    SQL+=" FROM rs_artykuly a "
                                    SQL+=" LEFT OUTER JOIN rs_jednostki_miar j1 ON (j1.firma_id=a.firma_id AND j1.kod=a.jm1)"
                                    SQL+=" LEFT OUTER JOIN rs_jednostki_miar j2 ON (j2.firma_id=a.firma_id AND j2.kod=a.jm2)"
                                    SQL+=" WHERE a.firma_id='"+firma_id+"' AND a.artykul_id='"+artp+"';"
                                    stat, DanePrzel, dump, dump = self.PolSQL.Execute(SQL, 3)
                                    if stat=='OK':
                                        LArt[artp]=[DanePrzel[0][0],0,0,DanePrzel[0][1],DanePrzel[0][2],DanePrzel[0][3]]
                                        przel=LArt[artp][0]
                                        DynamPrzel=LArt[artp][3]
                                        CalkJm1=LArt[artp][4]
                                        CalkJm2=LArt[artp][5]
                                        TestDynamPrzel=DynamPrzel=LArt[artp][3]

                            if stat == "OK":
                                if artp in SLimity and ((ilosc1>0 and SLimity[artp][0]>0) or (TestDynamPrzel and ilosc2>0 and SLimity[artp][1]>0)):
                                    przel=LArt[artp][0]
                                    DynamPrzel=LArt[artp][3]
                                    CalkJm1=LArt[artp][4]
                                    CalkJm2=LArt[artp][5]
                                    TestDynamPrzel=DynamPrzel=LArt[artp][3]
                                    dataAP = rsglobal.Dekoduj(pozAP[0])
                                    dokumentAP = rsglobal.Dekoduj(pozAP[1])
                                    numerAP = pozAP[2]
                                    nrpozAP = pozAP[3]
                                    nrppozAP = pozAP[4]
                                    c1 = pozAP[7]
                                    c2 = pozAP[8]
                                    cw1 = pozAP[9]
                                    cw2 = pozAP[10]
                                    cp1 = pozAP[11]
                                    cp2 = pozAP[12]
                                    cpw1 = pozAP[13]
                                    cpw2 = pozAP[14]
                                    il1 = rsglobal.RSRound(pozAP[5],6)
                                    il2 = rsglobal.RSRound(pozAP[6],6)
                                    r1 = pozAP[15]
                                    r2 = pozAP[26]
                                    if pozAP[28] == '':
                                        OpakAP = 'NULL'
                                    else:
                                        OpakAP = "'"+rsglobal.Dekoduj(pozAP[28])+"'"

                                    if pozAP[29] == '':
                                        PaletaAP = 'NULL'
                                    else:
                                        PaletaAP = "'"+rsglobal.Dekoduj(pozAP[29])+"'"

                                    WspOpakAP = pozAP[30]
                                    WspPaletaAP = pozAP[31]
                                    WspLokacjaAP = pozAP[32]
                                    
                                    cecha1_opisAP = rsglobal.Dekoduj(pozAP[23])
                                    cecha1_wartAP = rsglobal.Dekoduj(pozAP[24])

                                    if cecha1_opisAP not in ("",None,'None'):
                                        cecha1_opisAP="'"+cecha1_opisAP+"'"
                                    else:
                                        cecha1_opisAP = "''"
                                        
                                    if cecha1_wartAP not in ("",None,'None'):
                                        cecha1_wartAP="'"+cecha1_wartAP+"'"
                                    else:
                                        cecha1_wartAP = "''"

                                    if OpisWp not in ("",None,'None'):
                                        opis="'"+OpisWp+"'"
                                    else:
                                        opis = "''"
            
                                    wejscie_data=pozAP[19]
                                    wejscie_magazyn=pozAP[20]
                                    dostawca_id=pozAP[21]
                                    if wejscie_data in [None,"None"]:
                                        wejscie_data="NULL"
                                    else:
                                        wejscie_data="'"+rsglobal.Dekoduj(wejscie_data)+"'"
                                        
                                    if wejscie_magazyn in [None,"None"]:
                                        wejscie_magazyn="NULL"
                                    else:
                                        wejscie_magazyn="'"+rsglobal.Dekoduj(wejscie_magazyn)+"'"
                                        
                                    if dostawca_id in [None,"None"]:
                                        dostawca_id="NULL"
                                    else:
                                        dostawca_id="'"+rsglobal.Dekoduj(dostawca_id)+"'"
    
                                    seria = pozAP[16]
                                    if seria in (None,"None"):
                                        seria = "NULL"
                                    else:
                                        seria = "'"+rsglobal.Dekoduj(seria)+"'"
                                    
                                    termin = pozAP[17]
                                    if termin in (None,"None"):
                                        termin = "NULL"
                                    else:
                                        termin = "'"+rsglobal.Dekoduj(termin)+"'"
                                    
                                    if pozAP[25].strip("'").strip()<>lokacja_prz.strip("'").strip():
                                        CzyLokacje='TRUE'
                                    
                                    lokacja_prz = pozAP[25]
                                    if lokacja_prz in (None,"None"):
                                        Lokacja_Prz = "NULL"
                                    else:
                                        Lokacja_Prz = "'"+rsglobal.Dekoduj(lokacja_prz)+"'"
                        
                                    nr = "0000"+str(numerAP)
                                    nr = nr[len(nr)-6:len(nr)-2]
                        
                                    poztmp = "0000"+str(pozAP[3])
                                    poztmp = poztmp[len(poztmp)-6:len(poztmp)-2]
            
                                    ppoztmp = "0000"+str(pozAP[4])
                                    ppoztmp = ppoztmp[len(ppoztmp)-6:len(ppoztmp)-2]
                                    magtmp=Mag+'    '
                                    dok_prz = firma_id+magtmp[:4]+dokumentAP+str(dataAP)+nr+poztmp+ppoztmp
            
                                    if DynamPrzel and WydanieProporcjonalne != 0:
                                        # wydanie proporcjonalne
                                        if WydanieProporcjonalne == 1:
                                            # ze zdefiniowan pierwsz jednostk miary
                                            if ilosc1 > il1-r1:
                                                ilo = il1-r1
                                            else:
                                                ilo = ilosc1
                                            ilo2=rsglobal.RSRound(ilo*(il2/il1),6)
                                        else:
                                            # ze zdefiniowan drug jednostk miary
                                            if ilosc2 > il2-r2:
                                                ilo2 = il2-r2
                                            else:
                                                ilo2 = ilosc2
                                            ilo=rsglobal.RSRound(ilo2*(il1/il2),6)
                                    else:
                                        # wydawanie normalne nieproporcjonalne
                                        if ilosc1 > il1-r1:
                                            ilo = il1-r1
                                        else:
                                            ilo = ilosc1

                                        # jeli jest dynamiczny przelicznik to wydajemy jednostki niezalenie    
                                        if DynamPrzel:
                                            if ilosc2 > il2-r2:
                                                ilo2 = il2-r2
                                            else:
                                                ilo2 = ilosc2
                                        else:
                                            # wydanie wzgldem pierwszej zawsze wiodcej jednostki miary
                                            ilo2=rsglobal.RSRound(ilo*przel,6)
                                        
                                    # jeli (w dynamicznym przeliczniku to moe wystpi)
                                    # iloci si nie pokrywaj i mona wyda 0 to przechodzimy do nastpnej pozycji AP
                                    if ilo==0 and ilo2==0:
                                        continue
                                    
                                    ppoz = ppoz+1
                                    
                                    # to jest kontrola by nie przekroczy dostpnej iloci z rs_artykuly_magazyny
                                    ilo=min(ilo,SLimity[artp][0])
                                    ilo2=min(ilo2,SLimity[artp][1])

                                    kwota=rsglobal.RSRound(kwota+rsglobal.RSRound(ilo*c1,2),2)
                                    kwota_wal=rsglobal.RSRound(kwota_wal+rsglobal.RSRound(ilo*cw1,2),2)
                        
                                    if cp1!=0:
                                        kwotap=rsglobal.RSRound(kwotap+rsglobal.RSRound(ilo*cp1,2),2)
                                        kwotap_wal=rsglobal.RSRound(kwotap_wal+rsglobal.RSRound(ilo*cpw1,2),2)
                                    else:
                                        kwotap=rsglobal.RSRound(kwotap+rsglobal.RSRound(ilo*c1,2),2)
                                        kwotap_wal=rsglobal.RSRound(kwotap_wal+rsglobal.RSRound(ilo*cw1,2),2)
            
                                    # zmiana danych w rs_artykuly
                                    LArt[artp][1]+=ilo
                                    if DynamPrzel:
                                        LArt[artp][2]+=rsglobal.RSRound(ilo2,6)
                                    else:
                                        LArt[artp][2]+=rsglobal.RSRound(ilo*przel,6)
                                    
                                    #jeli zamiennik, to informacj o tym umieszczamy w sowniku
                                    oryginal = 'NULL'  #AB 2023-08-25
                                    if artp != art:
                                        if (art,artp) not in SArtZam:
                                            SArtZam[(art,artp)]=0
                                        SArtZam[(art,artp)]+=ilo
                                        oryginal = "'"+art+"'"  #AB 2023-08-25
                            
                                    UPD+="UPDATE rs_artykuly_przyjecia SET rezerwacja1=rezerwacja1+"+str(ilo)+","
                                    if DynamPrzel:
                                        UPD+="rezerwacja2=rezerwacja2+"+str(ilo2)
                                    else:
                                        UPD+="rezerwacja2=ROUND((rezerwacja1+"+str(ilo)+")*"+str(przel)+",4)"
                                    UPD+=" WHERE firma_id='"+firma_id+"' AND dokument_id='"+dokumentAP+"' AND magazyn_id='"+Mag+"' "
                                    UPD+=" AND data='"+dataAP+"' AND numer="+str(numerAP)+" AND nr_poz="+str(nrpozAP)+" AND nr_ppoz="+str(nrppozAP)+";"

                                    if not czyrozbicie and (OperatorWMS == 0 or dmnrppoz == 0):
                                        if DynamPrzel:
                                            Ilosc2Tmp=ilo2
                                        else:
                                            Ilosc2Tmp=rsglobal.RSRound(ilo*przel,6)
                                            
                                        # ilo lokacyjna
                                        if self.lokacje_jm == '1':
                                            IloscLok = ilo
                                        else:
                                            IloscLok = Ilosc2Tmp
                                        if OpakAP != 'NULL' and PaletaAP != 'NULL':
                                            IloscLok = rsglobal.RSRound(IloscLok/WspOpakAP/WspPaletaAP/WspLokacjaAP,4)
                                            
                                        UPD+="INSERT INTO rs_dokumenty_magazynowe_pozycje (firma_id,magazyn_id,dokument_id,data,numer,nr_poz,"
                                        UPD+="nr_ppoz,artykul_id,ilosc1,ilosc2,cena1,cena2,cena_wal1,cena_wal2,cena_pierw1,cena_pierw2,"
                                        UPD+="cena_pierw_wal1,cena_pierw_wal2,nr_serii,termin_przyd,dokument_przych,czy_cena,czy_seria,"
                                        UPD+="czy_termin,opis,wejscie_data,wejscie_magazyn,dostawca_id,oryginal,"
                                        UPD+="lokacja_skad,lokacja_dokad,czy_lokacje,cecha1_opis,cecha1_wartosc,czy_cecha1,"
                                        UPD+="zo_wyznacznik_id,zo_data,zo_numer,zo_nr_poz,zo_nr_ppoz,zo_nr_zlec,typ_zlec,produkt_id,czy_dok_przych,"
                                        UPD+="opakowanie_id,paleta_id,wsp_ilosci_opakowanie,wsp_ilosci_paleta,wsp_ilosci_lokacja,"
                                        UPD+="ilosc_lokacja,czy_opakowanie,czy_paleta)"
                                        UPD+=" VALUES ('"+firma_id+"','"+Mag+"','"+Dok+"','"+Data+"',"+Numer+","+str(dmnrpoz)+","+str(ppoz)+","
                                        UPD+="'"+artp+"',"+str(ilo)+","+str(Ilosc2Tmp)+","+str(c1)+","+str(c2)+","+str(cw1)+","+str(cw2)+","+str(cp1)+","+str(cp2)+","
                                        UPD+=str(cpw1)+","+str(cpw2)+","+seria+","+termin+",'"+dok_prz+"',"+czy_cena+","+czy_seria+","
                                                                                                                        #AB 2023-08-25
                                        UPD+=czy_termin+","+opis+","+wejscie_data+","+wejscie_magazyn+","+dostawca_id+","+oryginal+","
                                        UPD+=Lokacja_Prz+","+LokacjaDokad+","+CzyLokacje+","+cecha1_opisAP+","+cecha1_wartAP+","+czy_cecha1+","
                                        UPD+=ZOWyznacznik+","+ZOData+","+str(ZONumer)+","+str(ZONrPoz)+","+str(ZONrPPoz)+","+ZONrZlec+","+ZOTypZlec+","+Produkt+","+czy_dok_przych+","
                                        UPD+=OpakAP+","+PaletaAP+","+str(WspOpakAP)+","+str(WspPaletaAP)+","+str(WspLokacjaAP)+","
                                        UPD+=str(IloscLok)+","+czy_opak+','+czy_paleta+");"
                                    else:
                                        if ilo<ilosc1:
                                            #rsglobal.ZapiszDoLogow
                                            stat= "Pozycja z rozbicia nr "+str(dmnrpoz)+"/"+str(ppoz)+" nie moe zosta wydana w caoci."
                                        if DynamPrzel:
                                            if ilo2<ilosc2:
                                                stat= "Pozycja z rozbicia nr "+str(dmnrpoz)+"/"+str(ppoz)+" nie moe zosta wydana w caoci."

                                        UPD+="UPDATE rs_dokumenty_magazynowe_pozycje SET cena1="+str(c1)+",cena2="+str(c2)+",cena_wal1="+str(cw1)+","
                                        UPD+=" cena_wal2="+str(cw2)+",cena_pierw1="+str(cp1)+",cena_pierw2="+str(cp2)+","
                                        UPD+=" cena_pierw_wal1="+str(cpw1)+",cena_pierw_wal2="+str(cpw2)+",nr_serii="+seria+",termin_przyd="+termin+","
                                        UPD+=" dokument_przych='"+dok_prz+"',czy_cena="+czy_cena+",czy_seria="+czy_seria+",czy_termin="+czy_termin+","
                                        UPD+=" opis="+opis+",wejscie_data="+wejscie_data+",wejscie_magazyn="+wejscie_magazyn+", dostawca_id="+dostawca_id+","
                                                                            #AB 2023-08-25
                                        UPD+=" artykul_id='"+artp+"',oryginal="+oryginal+",cecha1_opis="+cecha1_opisAP+",cecha1_wartosc="+cecha1_wartAP+","
                                        UPD+=" czy_dok_przych="+czy_dok_przych+", czy_cecha1="+czy_cecha1+","
                                        UPD+=" opakowanie_id="+OpakAP+", paleta_id="+PaletaAP+",wsp_ilosci_opakowanie="+str(WspOpakAP)+","
                                        UPD+=" wsp_ilosci_paleta="+str(WspPaletaAP)+",wsp_ilosci_lokacja="+str(WspLokacjaAP)+","
                                        if OpakAP == 'NULL' or PaletaAP == 'NULL':   #AB 01.04.20r dopisany przecinek
                                            UPD+=" ilosc_lokacja = ilosc"+self.lokacje_jm+","
                                        else:
                                            UPD+=" ilosc_lokacja = ROUND(ilosc"+self.lokacje_jm+"/"+str(WspOpakAP)+"/"+str(WspPaletaAP)+"/"+str(WspLokacjaAP)+",4),"
                                        UPD+="czy_opakowanie = "+czy_opak+", czy_paleta = "+czy_paleta
                                        UPD+=" WHERE firma_id='"+firma_id+"' AND dokument_id='"+Dok+"' "
                                        UPD+=" AND magazyn_id='"+Mag+"' AND data='"+Data+"' AND numer="+Numer+" AND nr_poz="+str(dmnrpoz)+" AND nr_ppoz="+str(dmnrppoz)+";"
                                    
                                    ilosc1 = rsglobal.RSRound(ilosc1-ilo,6)
                                    ilosc2 = rsglobal.RSRound(ilosc2-ilo2,6)    
                                    
                                    SLimity[artp][0]=rsglobal.RSRound(SLimity[artp][0]-ilo,6)
                                    SLimity[artp][1]=rsglobal.RSRound(SLimity[artp][1]-ilo2,6)
                                    #i ewentualne poczenie w par 
                                    # i zdjcie iloci z rezerwacji
                                    
                                    if stat=='OK':
                                        if Typ=='W':
                                            IleRez=ilo
                                            IleRez2=ilo2
                                            if not czyrozbicie:
                                                PPozRez=str(ppoz)
                                            else:
                                                PPozRez=str(dmnrppoz)
                                            
                                            for rez in DaneRez:
                                                IlePozRez=min(IleRez,rez[1])
                                                IlePozRez2Int=0 # zalepka
                                                if DynamPrzel:
                                                    IlePozRez2Int=min(IleRez2,rez[3])
                                                    IlePozRez2=str(min(IleRez2,rez[3]))
                                                else:
                                                    IlePozRez2="ROUND((rezerwacja1- "+str(IlePozRez)+")*"+str(przel)+",4)"
                                                
                                                if (IlePozRez>0 or (DynamPrzel and IlePozRez2Int>0)) and rsglobal.Dekoduj(rezerw[2]).strip()==artp.strip():
                                                    #jeli zamykamy ca rezerwacj
                                                    ZamknietaRez=''
                                                    if IlePozRez==rez[1]:
                                                        ZamknietaRez=",zamknieta=TRUE"
                                                        
                                                    # zapisujemy na dole, ale od razu cigamy odpowiednie iloci z rezerwacji z danych
                                                    
                                                    # zapis do rezerwacji z ewentualnym zamkniciem
                                                    UPD+="UPDATE rs_artykuly_rezerwacje SET zrealizowano=zrealizowano+"+str(IlePozRez)
                                                    UPD+=ZamknietaRez+" WHERE firma_id='"+firma_id+"' AND rezerwacja_id="+str(rez[0])+";"
                                                    
                                                    #dopisanie informacji o wykorzystanej ilosci z rezerwacji,
                                                    #ktora bedzie wyswietlona operatorowi w ukladzie:
                                                    #artykul - ilosc z rezerwacji
                                                    
                                                    if SlowWykRez.has_key(rez[2]):
                                                        SlowWykRez[rez[2]] = SlowWykRez[rez[2]]+IlePozRez
                                                    else:
                                                        SlowWykRez[rez[2]] = IlePozRez
                                                    
                                                    # zdjcie z rezerwacji ktra zostaa wprowadzona z tytuu rezerwowania. Zaraz wrci w postaci 
                                                    #  zmiany z W na G
                                                    UPD+="UPDATE rs_artykuly_magazyny SET rezerwacja1=rezerwacja1- "+str(IlePozRez)
                                                    UPD+=", rezerwacja2="+IlePozRez2
                                                    UPD+=" WHERE firma_id='"+firma_id+"' AND artykul_id='"+artp+"' AND magazyn_id='"+Mag+"';"
                                                    UPD+="UPDATE rs_artykuly SET rezerwacja1=rezerwacja1- "+str(IlePozRez)
                                                    UPD+=", rezerwacja2="+IlePozRez2
                                                    UPD+=" WHERE firma_id='"+firma_id+"' AND artykul_id='"+artp+"';"
                                                    #zapis do tabeli czcej rezerwacj i pozycj dokumentu magazynowego
                                                    UPD+="INSERT INTO rs_dokumenty_magazynowe_rezerwacje (firma_id,magazyn_id,"
                                                    UPD+="dokument_id,data,numer, nr_poz, nr_ppoz, rezerwacja_id,ilosc1) VALUES "
                                                    UPD+="('"+firma_id+"','"+Mag+"','"+Dok+"','"+Data+"',"+Numer+","
                                                    UPD+=str(dmnrpoz)+","+PPozRez+","+str(rez[0])+","+str(IlePozRez)+");"
                                                    
                                                    IleRez=IleRez-IlePozRez
                                                    rez[1]=rez[1]-IlePozRez
                                                    # to rozliczamy tylko przy dynamicznym przeliczniku
                                                    if DynamPrzel:
                                                        IleRez2=IleRez2-IlePozRez2Int
                                                        rez[3]=rez[3]-IlePozRez2Int
                                                
                        # skoczyy si potencjalne artykuly przyjecia, a ciagle nie wydalismy wszystkiego co trzeba
                        if stat=='OK':
                            rsglobal.ZapiszDoLogow('!!! Dodatkowe ilosciowanie '+str(ilosc1)+' '+str(ilosc2),5)
                            if ilosc1> 0 or (ilosc2>0 and DynamPrzel):
                                # jeli to dynamiczny przelicznik, ale wydalimy tylko jedn jednostk miar,
                                #i ta niewydana nie jest cakowita to do ostatniej pozycji dokumentu magazynowego
                                # dodajemy ilo ktrej brakuje w tej drugiej jednostce
                                if DynamPrzel and ((ilosc1> 0 and ilosc2==0 and not CalkJm1) or (ilosc1==0 and ilosc2>0 and not CalkJm2)) and dokumentAP != '':
                                    UPDTmp="UPDATE rs_dokumenty_magazynowe_pozycje SET ilosc1=ilosc1+"+str(ilosc1)+",ilosc2=ilosc2+"+str(ilosc2)
                                    UPDTmp+=" WHERE firma_id='"+firma_id+"' AND dokument_id='"+Dok+"' "
                                    UPDTmp+=" AND magazyn_id='"+Mag+"' AND data='"+Data+"' AND numer="+Numer+" AND nr_poz="+str(dmnrpoz)+" AND nr_ppoz="+str(dmnrppoz)+";"
                                    
                                    UPD+="UPDATE rs_dokumenty_magazynowe_pozycje SET ilosc1=ilosc1+"+str(ilosc1)+",ilosc2=ilosc2+"+str(ilosc2)
                                    UPD+=" WHERE firma_id='"+firma_id+"' AND dokument_id='"+Dok+"' "
                                    UPD+=" AND magazyn_id='"+Mag+"' AND data='"+Data+"' AND numer="+Numer+" AND nr_poz="+str(dmnrpoz)+" AND nr_ppoz="+str(ppoz)+";"
                                    
                                    # dodajemy to te do rezerwacji artykuu na ostatniej pozycji artykuw przyj
                                    UPD+="UPDATE rs_artykuly_przyjecia SET rezerwacja1=rezerwacja1+"+str(ilosc1)+",rezerwacja2=rezerwacja2+"+str(ilosc2)
                                    UPD+=" WHERE firma_id='"+firma_id+"' AND dokument_id='"+dokumentAP+"' AND magazyn_id='"+Mag+"' "
                                    UPD+=" AND data='"+dataAP+"' AND numer="+str(numerAP)+" AND nr_poz="+str(nrpozAP)+" AND nr_ppoz="+str(nrppozAP)+";"
                                    
                                    # dodajemy to te do rezerwacji artykuw w rs_artykuly i rs_artykuly_magazyny
                                    UPD+="UPDATE rs_artykuly SET rezerwacja1=rezerwacja1+"+str(ilosc1)+",rezerwacja2=rezerwacja2+"+str(ilosc2)
                                    UPD+=" WHERE firma_id='"+firma_id+"' AND artykul_id='"+art+"';"
                                    UPD+="UPDATE rs_artykuly_magazyny SET rezerwacja1=rezerwacja1+"+str(ilosc1)+",rezerwacja2=rezerwacja2+"+str(ilosc2)
                                    UPD+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"' AND artykul_id='"+art+"';"
                                
                                # jeli nie udao si wyda ktrej z jednostek, 
                                # a powyszy warunek (ten z poprzedniego ifa) nie zadziaa
                                else:
                                    stat="Za maa ilo artykuu "+art+" (poz "+rsglobal.UsunKropkaZero(dmnrpoz)+"/"+rsglobal.UsunKropkaZero(dmnrppoz)+") do wydania o podanych parametrach. "
                                    stat+="\n Moliwe przyczyny to:"
                                    for p in Przyczyny:
                                        stat+="\n"+p
                                    self.PolSQL.Rollback()
                                
                        # Wydalimy wszystko co trzeba. Powinno da si wyda. Dodatkowo dorzucamy to co trzeba zmieni na rs_artykuly i artykuly_magazyny
                        if stat =='OK':
                            # jeli si dodao i na grze zmieniy si lokacje wydania to przy otwieraniu chcemy zrobi tabelk. W tym celu
                            # nawet jeli wczeniej nie byo lokacji to bdzie po otwarciu - zmieniamy wszystkim pozycjom czy_lokacje na True
                            if CzyLokacje=='TRUE':
                                UPD+="UPDATE rs_dokumenty_magazynowe_pozycje SET czy_lokacje=TRUE "
                                UPD+=" WHERE firma_id='"+firma_id+"' AND dokument_id='"+Dok+"' "
                                UPD+=" AND magazyn_id='"+Mag+"' AND data='"+Data+"' AND numer="+Numer
                                UPD+=" AND nr_poz="+str(dmnrpoz)+";"
                                
                            for artyk in LArt:
                                przel="(SELECT przelicznik FROM rs_artykuly WHERE firma_id='"+firma_id+"' AND artykul_id='"+artyk+"')"
                                il1=LArt[artyk][1]
                                il2=LArt[artyk][2]
                                UPD+="UPDATE rs_artykuly SET rezerwacja1=rezerwacja1+"+str(il1)+","
                                if DynamPrzel:
                                    UPD+=" rezerwacja2=rezerwacja2+"+str(il2)
                                else:
                                    UPD+=" rezerwacja2=ROUND((rezerwacja1+"+str(il1)+")*przelicznik,4)"
                                UPD+=" WHERE firma_id='"+firma_id+"' AND artykul_id='"+artyk+"';"
                            
                                UPD+="UPDATE rs_artykuly_magazyny SET rezerwacja1=rezerwacja1+"+str(il1)+","
                                if DynamPrzel:
                                    UPD+=" rezerwacja2=rezerwacja2+"+str(il2)
                                else:
                                    UPD+=" rezerwacja2=ROUND((rezerwacja1+"+str(il1)+")*"+przel+",4)"
                                UPD+=" WHERE firma_id='"+firma_id+"' AND artykul_id='"+artyk+"' AND magazyn_id='"+Mag+"';"

                            #zaatwienie caej pozycji
                            if UPD != '':
                                stat, dump, dump, dump = self.PolSQL.Execute(UPD, 1)
                            else:
                                stat='UWAGA - update wydajcy w pozycji '+rsglobal.UsunKropkaZero(dmnrpoz)+'/'+rsglobal.UsunKropkaZero(dmnrppoz)+' jest pusty. Poinformuj serwis'

            #cay dokument wydania i wszystkie jego pozycje sa ok. Mozemy zamknac spraw.
            if stat=='OK':
                # przepisujemy palety z rs_dokumenty_magazynowe_pozycje_palety z pozycji 0 do tych z numerami
                DEL = "INSERT INTO rs_dokumenty_magazynowe_pozycje_palety "
                DEL+=" SELECT DISTINCT dp.firma_id,dp.magazyn_id,dp.dokument_id,dp.data,dp.numer,dp.nr_poz,dp.nr_ppoz,dpp.nr_palety_id"
                DEL+=" FROM rs_dokumenty_magazynowe_pozycje dp INNER JOIN rs_dokumenty_magazynowe_pozycje_palety dpp "
                DEL+=" ON (dpp.firma_id=dp.firma_id AND dpp.magazyn_id=dp.magazyn_id AND dpp.dokument_id=dp.dokument_id "
                DEL+=" AND dpp.data=dp.data AND dpp.numer=dp.numer AND dpp.nr_poz= dp.nr_poz AND dpp.nr_ppoz=0)"
                DEL+=" WHERE dp.firma_id='"+firma_id+"'"
                DEL+=" AND dp.dokument_id='"+Dok+"' AND dp.magazyn_id='"+Mag+"' AND dp.data='"+Data+"' AND dp.numer="+str(Numer)+" AND dp.nr_ppoz>0;"
                #usuniecie pozycji z ppoz 0 (mozna je usunac co wynika z algorytmu)
                DEL+="DELETE FROM rs_dokumenty_magazynowe_pozycje WHERE firma_id='"+firma_id+"'"
                DEL+=" AND dokument_id='"+Dok+"' AND magazyn_id='"+Mag+"' AND data='"+Data+"' AND numer="+str(Numer)+" AND nr_ppoz=0;"
                stat, Test, dump, dump = self.PolSQL.Execute(DEL, 1)
            
            # woenie iloci do pobrania z lokacji
            if stat=='OK':
                stat=fs_dokumenty_magazynowe_lokacje_pom.ZmienIlosciDoWydaniaWLokacjachNaCalymDokumencie(self,Mag,Dok,Data,Numer,'+')   
            
            if stat=="OK":
                #ustawienie pl w nagwku, ktre si zmieni
                PolaNaglowka= ",kwota="+str(kwota)+", kwota_wal="+str(kwota_wal)+", kwota_pierw="+str(kwotap)+", kwota_pierw_wal="+str(kwotap_wal)
                
                # sprawdzenie limitu KUKE
                SQL ="SELECT COALESCE(kontrahent_id,'') FROM rs_dokumenty_magazynowe WHERE firma_id='"+firma_id+"'"
                SQL+=" AND dokument_id='"+Dok+"' AND magazyn_id='"+Mag+"' AND data='"+Data+"' AND numer="+str(Numer)
                stat,DKuke,dump,dump=self.PolSQL.Execute(SQL,3)
                if stat<>'OK':
                    stat = 'Bd pobierania danych o kontrahencie '+stat
                if stat =='OK':
                    if DKuke[0][0] != '':
                        stat,Przekroczenie=fs_kontrahenci_kuke_pom.SprawdzLimitKuke(self,DKuke[0][0],rsglobal.PodajDzis(),kwota)
                        if stat == 'OK':
                            if Przekroczenie > 0:
                                stat='Przekroczono kwote ubezpieczenia Kuke o '+str(Przekroczenie)
                
            if stat=="OK":    
                #uwzgldnienie ewentualnych zamiennikw, ktre zostay wydane zamiast oryginalnych artykuw
                KluczeZam=SArtZam.keys()
                KluczeZam.sort()
                
                #lista zamiennikow
                Zamienniki = []
                for kluczz in KluczeZam:
                    tekst="Wydano artyku "+kluczz[1]+" zamiast artykuu "+kluczz[0]+" w iloci:"+str(SArtZam[kluczz])
                    #tekst=rsglobal.Dekoduj(tekst)
                    Zamienniki.append(tekst)
                if len(Zamienniki) > 0:
                    wyn2.append(('ZAMIENNIKI',Zamienniki))
                
                #lista wykorzystanych rezerwacji
                Rezerwacje = []
                rezerwacje = SlowWykRez.keys()
                rezerwacje.sort()
                for a in rezerwacje:
                    Rezerwacje.append("Artyku "+str(a)+": pobrano z rezerwacji: "+str(SlowWykRez[a]))
                if len(Rezerwacje) > 0:
                    wyn2.append(('REZERWACJE',Rezerwacje))
                

            if stat=='OK':
                #ewentualne cignicie iloci wydanych za zamwienie.
                if DaneN[0][8]:
                    WyznZ=rsglobal.Dekoduj(DaneN[0][5])
                    DataZ=rsglobal.Dekoduj(DaneN[0][6])
                    NrZ=str(DaneN[0][7])
                    
                    #pobranie pozycji dokumentu po wszystkich operacjach
                    sql ="SELECT COALESCE(oryginal,artykul_id),ilosc1,ilosc2,"
                    sql+=" zo_wyznacznik_id,zo_data,zo_numer,zo_nr_poz,zo_nr_ppoz,produkt_id"
                    sql+=" FROM rs_dokumenty_magazynowe_pozycje"
                    sql+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"'"
                    sql+=" AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+str(Numer)
                    sql+=" AND zo_wyznacznik_id IS NOT NULL AND zo_data IS NOT NULL"
                    sql+=" AND zo_numer>0 AND zo_nr_zlec IS NULL" # odrzucamy pozycje ze zlece
                    sql+=" ORDER BY nr_poz,nr_ppoz;"

                    stat,DanePP,dump,dump = self.PolSQL.Execute(sql, 3)
                    if stat != 'OK':
                        stat = 'Bd pobrania pozycji dokumentu: '+stat

                    #czy pozycje wynikaly z pobrania  konkretnych zamowien (multiselect)
                    ZamMultiSelect = False
                    if stat == 'OK':
                        if len(DanePP) > 0:
                            ZamMultiSelect = True
                    
                    if stat == 'OK' and ZamMultiSelect:
                        #obsluga zamowien pobranych przez multiselect
                        UPD = ''
                        
                        #pobranie pozycji z zamowien przypisanych do pozycji
                        sql ="SELECT zop.artykul_id,"
                        sql+=" zop.ilosc1-zop.anulowano1-zop.dostarczono1,"
                        sql+=" zop.ilosc2-zop.anulowano2-zop.dostarczono2,"
                        sql+=" zop.wyznacznik_id,zop.data,zop.numer,zop.nr_poz,zop.nr_ppoz,"
                        sql+=" a.przelicznik, a.dynamiczny_przelicznik"
                        sql+=" FROM rs_zamowienia_odbiorcy_pozycje zop"
                        sql+=" INNER JOIN rs_artykuly a USING (firma_id,artykul_id)"
                        sql+=" INNER JOIN rs_zamowienia_odbiorcy zo USING (firma_id,wyznacznik_id,data,numer)"
                        sql+=" WHERE zop.firma_id='"+firma_id+"'"
                        sql+=" AND zo.kontrahent_id='"+rsglobal.Dekoduj(DaneN[0][1])+"'"
                        sql+=" AND zo.stan='G' AND zop.ilosc1-zop.anulowano1-zop.dostarczono1>0"
                        sql+=" AND zop.ilosc2-zop.anulowano2-zop.dostarczono2>0 AND ("
                        for i in xrange(len(DanePP)):
                            if i > 0:
                                sql+=" OR "
                            sql+="(zop.wyznacznik_id='"+rsglobal.Dekoduj(DanePP[i][3])+"'"
                            sql+=" AND zop.data='"+DanePP[i][4]+"' AND zop.numer="+str(DanePP[i][5])
                            sql+=" AND zop.nr_poz="+str(DanePP[i][6])+" AND zop.nr_ppoz="+str(DanePP[i][7])+")"
                        
                        sql+=") FOR UPDATE NOWAIT;"
                        
                        stat,ZPoz,dump,dump = self.PolSQL.Execute(sql,3,CzyDodawacBlad=True)
                        if stat != 'OK':
                            stat = 'Bd pobrania pozycji z wybranych zamwie (potencjalne zakleszczenie danych, sprbuj jeszce raz): '+stat
                        elif len(ZPoz) == 0:
                            stat = 'Brak wybranych pozycji zamwie'
                    
                        if stat == 'OK':
                            #slownik z pozycjami zamowien klucz=(art,wyzn,data,numer,poz,ppoz)
                            SlowZPoz = {}
                            for i in xrange(len(ZPoz)):
                                poz = (ZPoz[i][0],ZPoz[i][3],ZPoz[i][4],ZPoz[i][5],ZPoz[i][6],ZPoz[i][7])
                                if not poz in SlowZPoz:
                                    ilosci = [ZPoz[i][1],ZPoz[i][2],ZPoz[i][8],ZPoz[i][9]]
                                    SlowZPoz[poz] = ilosci
                            
                            #aktualizacja pozycji zamowien
                            for i in xrange(len(DanePP)):
                                art = (DanePP[i][0],DanePP[i][3],DanePP[i][4],DanePP[i][5],DanePP[i][6],DanePP[i][7])
                                if art in SlowZPoz:
                                    DynamPrzel=SlowZPoz[art][3]
                                    
                                    il1 = min(DanePP[i][1],SlowZPoz[art][0])
                                    if DynamPrzel:
                                        il1 = min(DanePP[i][2],SlowZPoz[art][1])
                                    else:
                                        il2 = min(rsglobal.RSRound(il1*SlowZPoz[art][2],4),SlowZPoz[art][1])
                                
                                    if il1 > 0 or il2 > 0:
                                        UPD+="UPDATE rs_zamowienia_odbiorcy_pozycje SET"
                                        UPD+=" dostarczono1=dostarczono1+"+str(il1)+","
                                        UPD+=" dostarczono2=dostarczono2+"+str(il2)
                                        UPD+=" WHERE firma_id='"+firma_id+"'"
                                        UPD+=" AND wyznacznik_id='"+rsglobal.Dekoduj(art[1])+"'"
                                        UPD+=" AND data='"+rsglobal.Dekoduj(art[2])+"'"
                                        UPD+=" AND numer="+str(art[3])
                                        UPD+=" AND nr_poz="+str(art[4])+" AND nr_ppoz="+str(art[5])+"; "
                                        
                                        SlowZPoz[art][0] -= il1
                                        SlowZPoz[art][1] -= il2

                        if stat == 'OK' and UPD != '':
                            stat, dump, dump, dump = self.PolSQL.Execute(UPD,1)
                            if stat != 'OK':
                                stat = 'Bd aktualizacji pozycji zamwien: '+stat

                        if stat == 'OK' and UPD != '':            
                            #ewentualne zamkniecie zamowien, ktore zostaly zrealizowane
                            sql ="SELECT zop.wyznacznik_id,zop.data,zop.numer,"
                            sql+=" SUM(CASE WHEN zop.ilosc1-zop.anulowano1-zop.dostarczono1>0 THEN 1 ELSE 0 END) "
                            sql+=" FROM rs_zamowienia_odbiorcy_pozycje zop"
                            sql+=" INNER JOIN rs_zamowienia_odbiorcy zo USING (firma_id,wyznacznik_id,data,numer)"
                            sql+=" WHERE zop.firma_id='"+firma_id+"' AND zo.stan='G' AND ("

                            for i in xrange(len(DanePP)):
                                if i > 0:
                                    sql+=" OR "
                                sql+="(zop.wyznacznik_id='"+rsglobal.Dekoduj(DanePP[i][3])+"'"
                                sql+=" AND zop.data='"+DanePP[i][4]+"' AND zop.numer="+str(DanePP[i][5])+")"
                                
                            sql+=") GROUP BY zop.wyznacznik_id,zop.data,zop.numer;"

                            stat, DaneZamN, dump, dump = self.PolSQL.Execute(sql, 3)
                            if stat != 'OK':
                                stat = 'Bd pobrania danych z wybranych zamwie: '+stat

                            if stat == 'OK':
                                UPD = ''
                                for poz in DaneZamN:
                                    if poz[3]==0:
                                        UPD+="UPDATE rs_zamowienia_odbiorcy SET stan='Z'"
                                        UPD+=" WHERE firma_id='"+firma_id+"' "
                                        UPD+=" AND wyznacznik_id='"+rsglobal.Dekoduj(poz[0])+"'"
                                        UPD+=" AND data='"+rsglobal.Dekoduj(poz[1])+"'"
                                        UPD+=" AND numer="+str(poz[2])+";"
                                 
                                if UPD != '':
                                    stat,d,dump,dump = self.PolSQL.Execute(UPD,1)   
                                    if stat != 'OK':
                                        stat = 'Bd zamykania zrealizowanych zamwie: '+stat
           

        #----------------------------------
        # DOKUMENT PRZYJCIA
        #----------------------------------
        
        # ponowne pobranie pozycji MM ze wzgldu na ceny lub lokalizacje
        if stat == "OK" and Typ in ['M','L']:
            stat, DaneP, dump, dump = self.PolSQL.Execute(SQLPoz, 3)
        
        if stat=='OK' and Typ in ['P','M','L']:
            ListaKat = []
            ListaPozLok=[]
            
            #sprawdzenie kategorii artykulow z kategoriami w magazynie
            if stat=='OK' and Typ=='M':
                #magazyn dokad pobieramy z naglowka
                MagDokad = rsglobal.Dekoduj(DaneN[0][3])
            else:
                MagDokad = Mag
            
            SQL ="SELECT kategoria FROM rs_magazyny_kategorie"
            SQL+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+MagDokad+"';"
            stat, DaneKat, dump, dump = self.PolSQL.Execute(SQL, 3)
            if stat=='OK':
                for kat in DaneKat:
                    ListaKat.append(kat[0])
                
            # do sensownego zliczania kwoty potrzebujemy, by kada pozycja bya zliczona raz. 
            LPozLiczonych=[]
            SLimityPrzyd = {}
            
            for poz in DaneP:
                if stat=='OK':
                    ilosc1=poz[3]
                    cena1=poz[5]
                    cenaw1=poz[7]
                    if not (poz[0],poz[1]) in LPozLiczonych:
                        kwota=rsglobal.RSRound(kwota+rsglobal.RSRound(ilosc1*cena1,2),2)
                        kwota_wal=rsglobal.RSRound(kwota_wal+rsglobal.RSRound(ilosc1*cenaw1,2),2)
                        LPozLiczonych.append((poz[0],poz[1]))
                        
                    if cena1==0 and CzyCenyDokad:
                        stat ="Zerowa cena artykuu "+str(poz[2]).strip()+" w pozycji "+str(poz[0]).strip()+"/"+ str(poz[1]).strip()
           
                if stat=='OK':
                    if ListaKat<>[]:
                        SQL="SELECT kateg_artykulu FROM rs_artykuly WHERE firma_id='"+firma_id+"' "
                        SQL+=" AND artykul_id='"+rsglobal.Dekoduj(poz[2])+"';"
                        stat, Test, dump, dump = self.PolSQL.Execute(SQL, 3)
                        if stat=='OK':
                            if len(Test)==0:
                                stat= "Brak artykuu "+(poz[2]).strip()
                            elif not Test[0][0] in ListaKat:
                                stat= "Artyku "+poz[2].strip()+" nie moe by przyjty do magazynu "+str(MagDokad)+" - nieuprawniona kategoria : "+Test[0][0]
                if stat=='OK':
                    Cecha1Opis = rsglobal.Dekoduj(poz[23])
                    Cecha1Wart = rsglobal.Dekoduj(poz[24]).strip()
                    stat=fs_dokumenty_magazynowe_pom.SprawdzPoprawnoscCechy(self,Cecha1Opis,Cecha1Wart)
                    if stat!= 'OK':
                        stat+='. Dotyczy artykuu '+poz[2].strip()

                # sprawdzenie poprawnoci limitw terminw przydatnoci
                if stat == 'OK' and Dok == 'PZ' and DaneN[0][1] != '':
                    if not poz[2] in SLimityPrzyd:
                        stat,Limity = fs_dokumenty_magazynowe_pom.PodajLimityTerminuPrzydatnosci(self,poz[2],DaneN[0][1],'P')
                        if stat == 'OK':
                            SLimityPrzyd[poz[2]] = Limity
                    
                    if stat == 'OK' and SLimityPrzyd[poz[2]][0] != 0:
                        # jeli dni limitu s wiksze ni dni dostpne
                        if SLimityPrzyd[poz[2]][0] > poz[32]:
                            stat= "Artyku "+poz[2].strip()+" nie moe by przyjty -  minimalny limit dni przydatnoci: "+str(rsglobal.UsunKropkaZero(SLimityPrzyd[poz[2]][0])) + ' dni, w pozycji ('+str(poz[0])+'/'+str(poz[1])+') przekroczono o '+str(-1*poz[32])+' dni'
                    
                # jeli magazyn wymaga zapisu dokd i jest wypeniona tabelka z lokacjami
                # to s one po kolei uwzgldniane. Na koniec usuwana jest pozycja podstawowa.
                if stat=='OK':
                    if CzyLokacjeDokad:
                        ZaokrJm="CASE WHEN (SELECT calkowita FROM rs_jednostki_miar WHERE firma_id='"+firma_id+"'"
                        ZaokrJm+=" AND kod=a.jm"+str(3-int(self.lokacje_jm))+") THEN 0 ELSE 4 END"
                        if self.lokacje_jm=='1':
                            Ilosc1='dl.ilosc'
                            Ilosc2="ROUND(dl.ilosc*a.przelicznik,"+ZaokrJm+")"
                        else:
                            Ilosc1="ROUND(dl.ilosc/a.przelicznik,"+ZaokrJm+")"
                            Ilosc2='dl.ilosc'
                                
                        
                        #podpozycja 0 i tabelka wypeniona
                        if poz[1]==0 and not poz[21] in ('','None',None):
                            if not poz[0] in ListaPozLok:
                                ListaPozLok.append(poz[0])
                                # wstawienie podpozycji i usunicie pozycji zero
                                SQLP="INSERT INTO rs_dokumenty_magazynowe_pozycje (firma_id,magazyn_id,dokument_id,data,"
                                SQLP+="numer,nr_poz,nr_ppoz,artykul_id,ilosc1,ilosc2,"
                                SQLP+="cena1,cena2,nr_serii,termin_przyd,cena_wal1,cena_wal2,opis,"
                                SQLP+="czy_dodatkowy_nr_serii,dostawca_id,oryginal,lokacja_skad,lokacja_dokad,czy_lokacje,"
                                SQLP+="cecha1_opis,cecha1_wartosc,produkt_id,"
                                SQLP+="opakowanie_id,paleta_id,wsp_ilosci_opakowanie,wsp_ilosci_paleta) "
                                SQLP+=" SELECT dp.firma_id,dp.magazyn_id,dp.dokument_id,dp.data,"
                                SQLP+="dp.numer,dp.nr_poz,dl.lp,dp.artykul_id,"+Ilosc1+","+Ilosc2+","
                                SQLP+="dp.cena1,dp.cena2,dp.nr_serii,dp.termin_przyd,dp.cena_wal1,dp.cena_wal2,dp.opis,"
                                SQLP+="dp.czy_dodatkowy_nr_serii,dp.dostawca_id,dp.oryginal,dl.lokacja_skad,dl.lokacja_dokad,TRUE,"
                                SQLP+=" dp.cecha1_opis,dp.cecha1_wartosc,dp.produkt_id,"
                                SQLP+=" dp.opakowanie_id,dp.paleta_id,dp.wsp_ilosci_opakowanie,dp.wsp_ilosci_paleta"
                                SQLP+=" FROM rs_dokumenty_magazynowe_pozycje dp INNER JOIN "
                                SQLP+=" rs_dokumenty_magazynowe_lokacje dl USING (firma_id, magazyn_id, dokument_id,"
                                SQLP+=" data, numer, nr_poz, nr_ppoz) INNER JOIN rs_artykuly a USING (firma_id,artykul_id) "
                                SQLP+=" WHERE dp.firma_id='"+firma_id+"' and dp.magazyn_id='"+Mag+"'"
                                SQLP+=" AND dp.dokument_id='"+Dok+"' AND dp.data='"+Data+"' AND dp.numer="+str(Numer)
                                SQLP+=" AND dp.nr_poz="+str(poz[0])+" AND dp.nr_ppoz=0;"
                                # przepisujemy palety z rs_dokumenty_magazynowe_pozycje_palety z pozycji 0 do tych z numerami
                                SQLP+= "INSERT INTO rs_dokumenty_magazynowe_pozycje_palety "
                                SQLP+=" SELECT DISTINCT dp.firma_id,dp.magazyn_id,dp.dokument_id,dp.data,dp.numer,dp.nr_poz,dp.nr_ppoz,dpp.nr_palety_id"
                                SQLP+=" FROM rs_dokumenty_magazynowe_pozycje dp INNER JOIN rs_dokumenty_magazynowe_pozycje_palety dpp "
                                SQLP+=" ON (dpp.firma_id=dp.firma_id AND dpp.magazyn_id=dp.magazyn_id AND dpp.dokument_id=dp.dokument_id "
                                SQLP+=" AND dpp.data=dp.data AND dpp.numer=dp.numer AND dpp.nr_poz= dp.nr_poz AND dpp.nr_ppoz=0)"
                                SQLP+=" WHERE dp.firma_id='"+firma_id+"'"
                                SQLP+=" AND dp.dokument_id='"+Dok+"' AND dp.magazyn_id='"+Mag+"' AND dp.data='"+Data+"' AND dp.numer="+str(Numer)+" AND nr_poz="+str(poz[0])+" AND dp.nr_ppoz>0;"
                
                                SQLP+=" DELETE FROM rs_dokumenty_magazynowe_pozycje WHERE firma_id='"+firma_id+"'"
                                SQLP+=" and magazyn_id='"+Mag+"' AND dokument_id='"+Dok+"' AND data='"+Data+"'"
                                SQLP+="  AND numer="+str(Numer)+" AND nr_poz="+str(poz[0])+" AND nr_ppoz=0;"
                                stat, dump, dump, dump = self.PolSQL.Execute(SQLP, 1)
                            
            if stat == 'OK':    
                # sprawdzenie czy w pozycjach z podanym opakowaniem i palet jest wypeniona iloc paletowa i opakowa
                SQL = "SELECT nr_poz,nr_ppoz FROM rs_dokumenty_magazynowe_pozycje "
                SQL+= " WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"'"
                SQL+= " AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+str(Numer)
                SQL+= " AND NOT opakowanie_id IS NULL AND NOT paleta_id IS NULL "
                SQL+= " AND (wsp_ilosci_opakowanie = 0 OR wsp_ilosci_paleta = 0)"
                stat, DTest, dump, dump = self.PolSQL.Execute(SQL, 3)
                if stat!= 'OK':
                    stat = 'Bdy sprawdzania poprawnoci opakowa/palet :'+stat
            
            if stat == 'OK':
                if len(DTest) > 0:
                    stat = "Nie mona zamkn dokumentu, bo dla niektrych pozycji"
                    stat+="  nie ma zdefiniowanych pozycji w opakowaniach zbiorczych: \n "
                    for p in DTest:
                        stat+=str(p[0])+"/"+str(p[1])+','
                    stat = stat.strip(',')
            
            if stat == 'OK':
                stat = fs_dokumenty_magazynowe_lokacje_pom.OdswiezWirtualnePozPrzyj(self,Mag,Dok,Data,Numer)
                    
            if stat == 'OK':
                if Typ == 'P':
                    PolaNaglowka = ",kwota="+str(kwota)+", kwota_wal="+str(kwota_wal)

    #----------------------------------
    # POWRT DO OGLNEGO ALGORYTMU
    #----------------------------------
    
    if stat=='OK' and Typ <>'D':
        if kwota==0 and CzyCenyDokad:
            stat="Zerowa kwota dokumentu"        
        elif kwota_wal==0 and CzyCenyDokad:
            stat="Zerowa kwota walutowa dokumentu"        

    if stat == 'OK':
        stat = fs_dokumenty_magazynowe_pom.ZmienIlosciNaZleceniachDlaDokumentu(self,Mag,Dok,Data,Numer,'+')  
        
    if stat=='OK' and Funkcja != 28:
        #rsglobal.ZapiszDoLogow('1 '+str(Funkcja),5)
        SQL="UPDATE rs_dokumenty_magazynowe SET stan='G' "+PolaNaglowka+" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"'"
        SQL+=" AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+Numer+";"
        
        stat, Test, dump, dump = self.PolSQL.Execute(SQL, 1)


#@##########################################
# ZAMYKANIE DOKUMENTU     G --> Z (9,19,29)
#@##########################################

if Funkcja in [9,19,29] and stat=='OK' and Typ<>'C':
    if Stan != "G":
        stat = "Dokument jest w stanie '"+Stan+"' - nie mona zamkn dokumentu"

    if stat=='OK':
    #test zalenoci dokumentu od faktury
        SQL="SELECT rsf_sprawdz_powiazanie_na_dokumencie('"+firma_id+"','"+Mag+"','"+Dok+"','"+Data+"',"+Numer+");"
        stat, TestFakt, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat=='OK':
        stat=TestFakt[0][0]

    #pobranie danych nagwka i pozycji.
    if stat == "OK":
        # kolumny 0-6
        #rsglobal.ZapiszDoLogow('6 ',1)
        SQL="SELECT waluta,kontrahent_id,COALESCE(dostawca_id,kontrahent_id),nr_zlecenia,konto_kosztow,nr_faktury,kierowca,"
        # kolumny 7-13
        SQL+="czy_nocny,zo_wyznacznik_id,zo_data,zo_numer,czy_pobrane_z_zamowien,magazyn_dokad,numer_dokad FROM rs_dokumenty_magazynowe WHERE firma_id='"+firma_id+"'"
        SQL+=" AND magazyn_id='"+Mag+"' AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+Numer+";"
        stat, DaneN, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat == "OK":
        if len(DaneN)==0:
            stat="Brak podanego dokumentu magazynowego"
        elif len(DaneN)>1:
            stat="Podany dokument wystpuje w wicej ni raz w bazie danych!"
        else:
            magprzyj = rsglobal.Dekoduj(DaneN[0][12])
            numprzyj = str(DaneN[0][13])
            if numprzyj in ('None','0','0.0'):
                numprzyj = Numer
            
    if stat == "OK":
        # kolumny 0 - 10
        #rsglobal.ZapiszDoLogow('123 ',7)
        SQL="SELECT nr_poz,nr_ppoz,artykul_id,ilosc1,ilosc2,cena1,cena2,cena_wal1,cena_wal2,cena_pierw1,cena_pierw2,"
        # kolumny 11 - 16
        SQL+="cena_pierw_wal1,cena_pierw_wal2,COALESCE(nr_serii,''),termin_przyd,dokument_przych,wejscie_data,"
        # kolumny 17 - 23
        SQL+="wejscie_magazyn,dostawca_id,opis,COALESCE(oryginal,artykul_id),czy_lokacje,lokacja_skad,lokacja_dokad,"
        # kolumny 24 - 28
        SQL+="cecha1_opis,cecha1_wartosc,produkt_id,COALESCE(opakowanie_id,''),COALESCE(paleta_id,''),"
        # kolumny 29 - 32
        SQL+="COALESCE(wsp_ilosci_opakowanie,0),COALESCE(wsp_ilosci_paleta,0),COALESCE(wsp_ilosci_lokacja,0),COALESCE(ilosc_lokacja,0)"
        SQL+=" FROM rs_dokumenty_magazynowe_pozycje WHERE firma_id='"+firma_id+"'"
        SQL+=" AND magazyn_id='"+Mag+"' AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+Numer+";"
        rsglobal.ZapiszDoLogow('123SQL '+SQL)
        stat, DaneP, dump, dump = self.PolSQL.Execute(SQL, 3)

    if stat == "OK":
        if len(DaneP)==0:
            stat="Brak pozycji na dokumencie magazynowym"            

    #----------------------------------
    # DOKUMENT WYDANIA (z wyjatkiem detalu)
    #----------------------------------

    if stat=='OK' and Typ in ["W","M","L","D"]:
        # dane do obrotow magazynowych
        waluta=rsglobal.Dekoduj(DaneN[0][0])
        kontrahent=DaneN[0][1]
        if kontrahent in [None,'None']:
            kontrahent="NULL"
        else:
            kontrahent="'"+rsglobal.Dekoduj(kontrahent)+"'"

        region="NULL"
        if kontrahent<>"NULL" and CzyUrzadzenia:
            SQL="SELECT region FROM rs_kontrahenci WHERE firma_id='"+firma_id+"' AND kontrahent_id="+kontrahent+";"
            stat, DaneReg, dump, dump = self.PolSQL.Execute(SQL, 3)
            if stat=='OK':
                if rsglobal.Dekoduj(DaneReg[0][0]) != 'None':
                    region="'"+rsglobal.Dekoduj(DaneReg[0][0])+"'"
                
        if stat=='OK':
            zlecenie=rsglobal.Dekoduj(DaneN[0][3])
            if zlecenie=='None':
                zlecenie="NULL"
            else:
                zlecenie="'"+zlecenie+"'"

            kontok=rsglobal.Dekoduj(DaneN[0][4])
            if kontok=='None':
                kontok="NULL"
            else:
                kontok="'"+kontok+"'"
                
            faktura=rsglobal.Dekoduj(DaneN[0][5])
            if faktura=='None':
                faktura="NULL"
            else:
                faktura="'"+faktura.replace("'","\\\'")+"'"

            kierowca= rsglobal.Dekoduj(DaneN[0][6])
            if kierowca=='None':
                kierowca="NULL"
            else:
                kierowca="'"+kierowca+"'"
        
        for poz in DaneP:

            rsglobal.ZapiszDoLogow('POZPOZPOZ            '+str(poz))
            if stat=='OK':
                dokprzych = rsglobal.Dekoduj(poz[15])
                typAP = dokprzych[6:8]
                dataAP = dokprzych[8:18]
                numerAP = dokprzych[18:22]
                pozAP = dokprzych[22:26]
                ppozAP = dokprzych[26:30]  
                art = rsglobal.Dekoduj(poz[2]).strip()
                przel="(SELECT przelicznik FROM rs_artykuly WHERE firma_id='"+firma_id+"' AND artykul_id='"+art+"')"
                ilosc1 = poz[3]
                ilosc2 = poz[4]
                nr_poz=poz[0]
                nr_ppoz=poz[1]
                seria=poz[13]
                if seria in ['None',None]:
                    seria=""
                else:
                    seria=rsglobal.Dekoduj(seria.rstrip())
                termin=poz[14]
                if termin in ['None',None]:
                    termin="NULL"
                else:
                    termin="'"+rsglobal.Dekoduj(termin)+"'"
                    
                cecha1opis=rsglobal.Dekoduj(poz[24])
                cecha1wart=rsglobal.Dekoduj(poz[25])
                produkt=rsglobal.Dekoduj(poz[26]).strip()
                if produkt in ['','None',None]:
                    produkt='NULL'
                else:
                    produkt="'"+produkt+"'"
                
                cena1=poz[5]
                cena2=poz[6]
                cenaw1=poz[7]
                cenaw2=poz[8]
                cenap1=poz[9]
                cenap2=poz[10]
                cenapw1=poz[11]
                cenapw2=poz[12]

                wejscie_data=poz[16]
                if wejscie_data in ['None',None]:
                    wejscie_data="NULL"
                else:
                    wejscie_data="'"+rsglobal.Dekoduj(wejscie_data)+"'"
                wejscie_magazyn=poz[17]
                if wejscie_magazyn in ['None',None]:
                    wejscie_magazyn="NULL"
                else:
                    wejscie_magazyn="'"+rsglobal.Dekoduj(wejscie_magazyn)+"'"

                dostawca= rsglobal.Dekoduj(poz[18])
                if dostawca in ['None',None]:
                    dostawca=rsglobal.Dekoduj(DaneN[0][2])
    
                if dostawca in ['None',None]:
                    dostawca="NULL"
                else:
                    dostawca="'"+dostawca+"'"

                czynocny=DaneN[0][7]
                if czynocny:
                    czynocny="TRUE"
                else:
                    czynocny="FALSE"
                    
                LokSkad = rsglobal.Dekoduj(poz[22])                            
                if LokSkad in ['None',None]:
                    LokSkad="NULL"
                else:
                    LokSkad="'"+LokSkad+"'"                            

                # opakowanie/paleta
                if poz[27] == '':
                    Opak = 'NULL'
                else:
                    Opak = "'"+rsglobal.Dekoduj(poz[27])+"'"

                if poz[28] == '':
                    Paleta = 'NULL'
                else:
                    Paleta = "'"+rsglobal.Dekoduj(poz[28])+"'"

                WspOpak = poz[29]
                WspPaleta = poz[30]
                WspLokacja = poz[31]
                IloscLok = poz[32]
                
                # pobranie informacji czy przelicznik jest dynamiczny
                SQL="SELECT dynamiczny_przelicznik,sledzenie_dostaw FROM rs_artykuly "
                SQL+=" WHERE firma_id='"+firma_id+"' AND artykul_id='"+art+"';"
                stat, DArt, dump, dump = self.PolSQL.Execute(SQL, 3)
                if stat<>"OK":
                    stat=' Bdy pobierania informacji o dynamicznym przeliczniku :'+stat
            
            if stat=='OK':
                DynamPrzel = DArt[0][0]                
                SledzDost  = DArt[0][1]                
                ZmieniacIlosci = not DynamPrzel

                SQL=''

                # najpierw pobieramy aktualn ilo z artykuw przyjcia. Jeli jest rna od iloci pozycji, ale 
                # jedna z iloci bdzie wynosi 0 po zamkniciu dokumentu to najpierw zmieniamy pozycj dokumentu,
                # a potem normalnie zamykamy. 
                
                ZmianaDoWydania=0
                ZmianaDoWydania1=0
                ZmianaDoWydania2=0

                # pobranie iloci z artykuw przyjcia
                #rsglobal.ZapiszDoLogow('8 ',1)
                SQLDaneAP="SELECT ilosc1,ilosc2 FROM rs_artykuly_przyjecia "
                SQLDaneAP+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"' AND data='"+dataAP+"' AND dokument_id='"+typAP+"'"
                SQLDaneAP+=" AND numer='"+numerAP+"' AND nr_poz='"+pozAP+"' AND nr_ppoz='"+ppozAP+"';"
                stat, DaneLokAP, dump, dump = self.PolSQL.Execute(SQLDaneAP, 3)
               
            if stat=='OK':
                IloscAPTest1=DaneLokAP[0][0]
                IloscAPTest2=DaneLokAP[0][1]
                RedukcjaDyn1=0
                RedukcjaDyn2=0
                
                if ZmieniacIlosci:
                    if (IloscAPTest1==ilosc1 and IloscAPTest2!=ilosc2) or (IloscAPTest2==ilosc2 and IloscAPTest1!=ilosc1):
                        ZmianaDoWydania1=ilosc1-IloscAPTest1
                        ZmianaDoWydania2=ilosc2-IloscAPTest2
                        if CzyLokacjeSkad:
                            if self.lokacje_jm=='1':
                                ZmianaDoWydania=ilosc1-IloscAPTest1
                            else:
                                ZmianaDoWydania=ilosc2-IloscAPTest2
                        # jeli podane opakowanie/paleta to przeliczamy t ilo o wspczynniki
                        if Opak != 'NULL' and Paleta != 'NULL':
                            ZmianaDoWydania = rsglobal.RSRound(ZmianaDoWydania/WspOpak/WspPaleta/WspLokacja,4)
                            
                        ilosc1=IloscAPTest1
                        ilosc2=IloscAPTest2
                        
                        SQL+="UPDATE rs_dokumenty_magazynowe_pozycje SET ilosc1="+str(ilosc1)+", ilosc2="+str(ilosc2)
                        SQL+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"' AND dokument_id='"+Dok+"'"
                        SQL+=" AND data='"+Data+"' AND numer="+Numer+" AND nr_poz="+str(nr_poz)
                        SQL+=" AND nr_ppoz="+str(nr_ppoz)+";"
                else:
                    # dla dynamicznego przelicznika wyznaczamy ewentualnie zredukowane iloci w przypadku zamykania z nadwyk
                    if ilosc1 > IloscAPTest1:
                        RedukcjaDyn1= ilosc1 - IloscAPTest1
                    if ilosc2 > IloscAPTest2:
                        RedukcjaDyn2=ilosc2 - IloscAPTest2
                
                        
            if stat == 'OK':
                # zmiana na lokacjach
                if CzyLokacjeSkad:
                    if self.lokacje_jm=='1':
                        IloscLok=ilosc1
                        PoleIlosci=0
                    else:
                        IloscLok=ilosc2
                        PoleIlosci=1
                    
                    SQL+="UPDATE rs_lokacje SET do_wydania=do_wydania- "+str(ZmianaDoWydania)
                    SQL+=" WHERE firma_id='"+firma_id+"' AND lokacja_id="+LokSkad+";"
                
                if DynamPrzel:
                    Rezerw2Art = 'rezerwacja2- '+str(ilosc2)
                    Rezerw2 = 'rezerwacja2- '+str(ilosc2)
                    Rezerw2AP = 'rezerwacja2- '+str(ilosc2)
                else:
                    Rezerw2Art = "ROUND((rezerwacja1- "+str(ilosc1)+")*przelicznik,4)"
                    Rezerw2 = "ROUND((rezerwacja1- "+str(ilosc1)+")*"+przel+",4)"
                    Rezerw2AP = "ROUND((rezerwacja1- "+str(ilosc1+ZmianaDoWydania1)+")*"+przel+",4)"
                    
                #zapis do artykuw i artykuw przyj
                SQL+="UPDATE rs_artykuly SET ilosc1=ilosc1- "+str(ilosc1-RedukcjaDyn1)+",ilosc2=ilosc2- "+str(ilosc2-RedukcjaDyn2)+","
                SQL+="rezerwacja1=rezerwacja1- "+str(ilosc1)+",rezerwacja2="+Rezerw2Art+","
                SQL+="ost_rozchod=CASE WHEN '"+Data+"'<=ost_rozchod THEN ost_rozchod ELSE '"+Data+"' END "
                SQL+=" WHERE firma_id='"+firma_id+"' AND artykul_id='"+art+"';"

                SQL+="UPDATE rs_artykuly_magazyny SET ilosc1=ilosc1- "+str(ilosc1-RedukcjaDyn1)+",ilosc2=ilosc2- "+str(ilosc2-RedukcjaDyn2)+","
                SQL+="rezerwacja1=rezerwacja1- "+str(ilosc1)+",rezerwacja2="+Rezerw2+","
                SQL+="ost_rozchod=CASE WHEN '"+Data+"'<=ost_rozchod THEN ost_rozchod ELSE '"+Data+"' END "
                SQL+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"' AND artykul_id='"+art+"';"
                # Update na zwyke iloci

                #AB 29.06.21r
                # Przepisanie kart przyjec z rs_artykuly_przyjecia_karty_przyjecia
                SQLKartyPrzyjec = """SELECT firma_id, magazyn_id, data, dokument_id, numer, nr_poz, nr_ppoz, kod,
                                            COALESCE(wart_b::TEXT, 'None'), COALESCE(wart_d::TEXT, 'None'), 
                                            COALESCE(wart_s::TEXT, 'None'), COALESCE(wart_n::TEXT, 'None'), operator 
                                     FROM rs_artykuly_przyjecia_karty_przyjecia 
                                     WHERE (firma_id, magazyn_id, "data", dokument_id, numer, nr_poz, nr_ppoz) = ('{}', '{}', '{}', '{}', {}, {}, {})
                                     ;""".format(firma_id, Mag, dataAP, typAP, numerAP, pozAP, ppozAP)
                rsglobal.ZapiszDoLogow(' '*200 + SQLKartyPrzyjec)
                stat, KartyPrzyjec, dump, dump = self.PolSQL.Execute(SQLKartyPrzyjec, 3, CzyDodawacBlad=True)
                if stat != 'OK':
                    stat = 'Bd pobierania kart przyj dla pozycji dokumentu magazynowego'
                elif len(KartyPrzyjec) >= 1:
                    for nr_wiersza_przyjec in xrange(len(KartyPrzyjec)):
                        w = KartyPrzyjec[nr_wiersza_przyjec]
                        INSKartyPrzyjec = """ INSERT INTO rs_dokumenty_magazynowe_pozycje_karty_przyjecia 
                                             (firma_id, magazyn_id, dokument_id, "data", numer, nr_poz, nr_ppoz, kod, wart_b, wart_d, wart_s, wart_n, operator) 
                                       VALUES('{}',      '{}',          '{}',     '{}',    {},    {},     {},   '{}',   {},     {},     {},     {},     {});
                                        ;""".format(w[0], w[1],           Dok,     Data,  Numer, nr_poz, nr_ppoz, w[7], rsglobal.ZmienNaNULL(w[8]),
                                                    rsglobal.ZmienNaNULL(w[9]), rsglobal.ZmienNaNULL(w[10]), rsglobal.ZmienNaNULL(w[11]), w[12])
                        rsglobal.ZapiszDoLogow(INSKartyPrzyjec)
                        SQL += INSKartyPrzyjec


                #rsglobal.ZapiszDoLogow('11 ', 1)
                SQL+="UPDATE rs_artykuly_przyjecia SET ilosc1=ilosc1- "+str(ilosc1-RedukcjaDyn1)+",ilosc2=ilosc2- "+str(ilosc2-RedukcjaDyn2)+","
                SQL+="rezerwacja1=rezerwacja1- "+str(ilosc1+ZmianaDoWydania1)+",rezerwacja2="+Rezerw2AP
                SQL+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"' AND data='"+dataAP+"' AND dokument_id='"+typAP+"'"
                SQL+=" AND numer='"+numerAP+"' AND nr_poz='"+pozAP+"' AND nr_ppoz='"+ppozAP+"';"
                # Update na ilo lokacyjn
                #rsglobal.ZapiszDoLogow('12 ',1)
                SQL+="UPDATE rs_artykuly_przyjecia SET ilosc_lokacja = CASE WHEN opakowanie_id IS NULL OR paleta_id IS NULL THEN "
                SQL+=" ilosc"+self.lokacje_jm+" ELSE ROUND(ilosc"+self.lokacje_jm+"/wsp_ilosci_opakowanie/wsp_ilosci_paleta/wsp_ilosci_lokacja,4) END"
                SQL+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"' AND data='"+dataAP+"' AND dokument_id='"+typAP+"'"
                SQL+=" AND numer='"+numerAP+"' AND nr_poz='"+pozAP+"' AND nr_ppoz='"+ppozAP+"';"
                
                # wyzerowanie obu ilosci, jesli jedna z nich jest 0
                if ZmieniacIlosci:
                    SQL+="UPDATE rs_artykuly SET ilosc1=0,ilosc2=0"
                    SQL+=" WHERE firma_id='"+firma_id+"' AND artykul_id='"+art+"' AND (ilosc1=0 OR ilosc2=0);"
                    #rsglobal.ZapiszDoLogow('13 ', 1)
                    SQL+="UPDATE rs_artykuly_przyjecia SET ilosc1=0,ilosc2=0,ilosc_lokacja = 0"
                    SQL+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"' AND data='"+dataAP+"'"
                    SQL+=" AND dokument_id='"+typAP+"' AND numer='"+numerAP+"' AND nr_poz='"+pozAP+"'"
                    SQL+=" AND nr_ppoz='"+ppozAP+"' AND (ilosc1=0 OR ilosc2=0);"
                    
                    SQL+="UPDATE rs_artykuly_magazyny SET ilosc1=0,ilosc2=0  WHERE firma_id='"+firma_id+"'"
                    SQL+=" AND magazyn_id='"+Mag+"' AND artykul_id='"+art+"' AND (ilosc1=0 OR ilosc2=0);"
                                        
                
                # zapis do obrotow magazynowych
                if self.lokacje_jm =='1':
                    IloscLokObr = ilosc1
                else:
                    IloscLokObr = ilosc2
                if Opak != 'NULL' and Paleta != 'NULL':
                    IloscLokObr = rsglobal.RSRound(IloscLokObr/WspOpak/WspPaleta/WspLokacja,4)
                SQL+= "INSERT INTO rs_obroty_magazynowe (firma_id,magazyn_id,dokument_id,data,numer,typ,typ_znak,nr_poz,nr_ppoz,"
                SQL+="kontrahent_id,nr_zlecenia,konto_kosztow,nr_faktury,artykul_id,ilosc1,ilosc2,cena1,cena2,cena_pierw1,cena_pierw2,"
                SQL+="wartosc,nr_serii,termin_przyd,waluta,cena_wal1,cena_wal2,cena_pierw_wal1,cena_pierw_wal2,wartosc_wal,"
                SQL+="dokument_przych,kierowca,wejscie_data,wejscie_magazyn,dostawca_id,czy_nocny,cecha1_opis,cecha1_wartosc,produkt_id,"
                SQL+="opakowanie_id,paleta_id,wsp_ilosci_opakowanie,wsp_ilosci_paleta,wsp_ilosci_lokacja,ilosc_lokacja) VALUES "
                SQL+="('"+firma_id+"','"+Mag+"','"+Dok+"','"+Data+"',"+Numer+",'W',-1,"+str(nr_poz)+","+str(nr_ppoz)+","
                SQL+=kontrahent+","+zlecenie+","+kontok+","+faktura+","+"'"+art+"',"+str(ilosc1)+","+str(ilosc2)+","+str(cena1)+","+str(cena2)+","+str(cenap1)+","+str(cenap2)+","
                SQL+=str(rsglobal.RSRound(ilosc1*cena1,2))+",'"+seria+"',"+termin+",'"+waluta+"',"+str(cenaw1)+","+str(cenaw2)+","+str(cenapw1)+","+str(cenapw2)+","+str(rsglobal.RSRound(ilosc1*cenaw1,2))+","
                SQL+="'"+dokprzych+"',"+kierowca+","+wejscie_data+","+wejscie_magazyn+","+dostawca+","+czynocny+",'"+cecha1opis+"','"+cecha1wart+"',"+produkt+","
                SQL+=Opak+","+Paleta+","+str(WspOpak)+","+str(WspPaleta)+","+str(WspLokacja)+","+str(IloscLokObr)+");"

                #tworzenie urzdze
                if CzyUrzadzenia:
                    if seria != "":
                        if kontrahent<>"NULL":
                            # sprawdzenie czy mona doda urzdzenie o tym numerze 
                            STEST="SELECT nr_serii FROM rs_serwis_urzadzenia WHERE firma_id='"+firma_id+"' "
                            STEST+="AND artykul_id='"+art+"' AND nr_serii='"+seria+"' AND kontrahent_id="+kontrahent+";"
                            stat, DTestSerwis, dump, dump = self.PolSQL.Execute(STEST, 3)  
                            if len(DTestSerwis)>0:
                                stat="Nie mona doda urzdzenia "+art+" o numerze "+seria+" dla kontrahenta "+kontrahent+" do serwisu."
                            # samo dodanie urzdzenia
                            if stat=='OK':
                                SQL+="INSERT INTO rs_serwis_urzadzenia (firma_id,artykul_id,nr_serii,kontrahent_id,sprzedaz_data,region,magazyn_id,"
                                SQL+="dokument_id,data_dok,numer_dok)"
                                SQL+="VALUES ('"+firma_id+"','"+art+"','"+seria+"',"+kontrahent+",'"+Data+"',"+region+",'"+Mag+"','"+Dok+"','"+Data+"',"+Numer+");"

                #waciwy zapis danych z pozycji
            if stat=='OK':
                stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1)
               
            if stat=='OK':
                #zmiana ilosci na spisach z natury                  
                if Typ in ['W','M','D']:
                    # na artykuach przyjecia
                    # sprawdzamy czy istnieje spis, ktry warto zmienia
                    stat,LDataKat=fs_spis_z_natury_pom.PodajDatyIKategorieSpisow(self,Mag,Data,art)
                    if stat=='OK':
                        # dla kadego pasujcego spisu
                        for spis in LDataKat:
                            if stat=='OK':
                                DataSpisu = spis[0]
                                KategSpisu = spis[1]
                                # odwieamy przyjcia spisu dla naszego artykuu
                                stat = fs_spis_z_natury_pom.OdswiezPozycjeArtykulowPrzyjecWSpisieZNatury(self, DataSpisu, Mag, KategSpisu, art)
                                
                                # przeliczamy iloci stwierdzone na pozycjach spisu
                                if stat=='OK':
                                    stat,dump = fs_spis_z_natury_pom.WstawIlosciStwierdzoneDoPozycjiArtykulowPrzyjeciaNaSpisieZNatury(self, DataSpisu, Mag, KategSpisu, art)
                            
        # zdjcie iloci do pobrania z lokacji
        if stat=='OK':
            stat=fs_dokumenty_magazynowe_lokacje_pom.ZmienIlosciDoWydaniaWLokacjachNaCalymDokumencie(self,Mag,Dok,Data,Numer,'-')   

        
    #----------------------------------
    # DOKUMENT PRZYJECIA
    #----------------------------------
    
    if stat=='OK' and Typ in ["P","M","L"]:
        #zapis dostawcy na pozycjach dokumentu przychodowego
        if Typ=='P':
            SQL="SELECT rsf_zapisz_dostawce_na_pozycje('"+firma_id+"','"+Mag+"','"+Dok+"','"+Data+"',"+Numer+");"
            stat, DaneDost, dump, dump = self.PolSQL.Execute(SQL, 3)
            if stat=="OK":
                stat=TestFakt[0][0]
                
        #zapis waciwy dokumentu przychodowego
        if stat=="OK":
        
            waluta=rsglobal.Dekoduj(DaneN[0][0])
            
            zlecenie=rsglobal.Dekoduj(DaneN[0][3])
            if zlecenie=='None':
                zlecenie="NULL"
            else:
                zlecenie="'"+zlecenie+"'"

            kontok=rsglobal.Dekoduj(DaneN[0][4])
            if kontok=='None':
                kontok="NULL"
            else:
                kontok="'"+kontok+"'"

            faktura=rsglobal.Dekoduj(DaneN[0][5])
            if faktura=='None':
                faktura="NULL"
            else:
                faktura="'"+faktura.replace("'","\\\'")+"'"
        
            for poz in DaneP:
                if stat=='OK':
                    nr_poz=str(poz[0])
                    nr_ppoz=str(poz[1])
                    seria=poz[13]
                    if seria in ['None',None]:
                        seria=""
                    else:
                        seria=rsglobal.Dekoduj(seria.rstrip())
                    termin=poz[14]
                    if termin in ['None',None]:
                        termin="NULL"
                    else:
                        termin="'"+rsglobal.Dekoduj(termin)+"'"

                    artykul = rsglobal.Dekoduj(poz[2]).strip()
                    ilosc1 = poz[3]
                    ilosc2 = poz[4]
                    cena1=poz[5]
                    cena2=poz[6]
                    cenaw1=poz[7]
                    cenaw2=poz[8]
                    cenap1=poz[9]
                    cenap2=poz[10]
                    cenapw1=poz[11]
                    cenapw2=poz[12]    
                    
                    if Typ == "P" :
                        wejscie_data="'"+Data+"'"
                        wejscie_magazyn="'"+Mag+"'"
                    else:
                        wejscie_data=poz[16]
                        if wejscie_data in ['None',None]:
                            wejscie_data="NULL"
                        else:
                            wejscie_data="'"+rsglobal.Dekoduj(wejscie_data)+"'"
                        wejscie_magazyn=poz[17]
                        if wejscie_magazyn in ['None',None]:
                            wejscie_magazyn="NULL"
                        else:
                            wejscie_magazyn="'"+rsglobal.Dekoduj(wejscie_magazyn)+"'"
                            
                    opis= rsglobal.Dekoduj(poz[19])
                    if opis in ['None',None]:
                        opis="NULL"
                    else:
                        opis="'"+opis+"'"

                    dokumentprzych = rsglobal.Dekoduj(poz[15])                            
                    if dokumentprzych in ['None',None]:
                        dokumentprzych="NULL"
                    else:
                        dokumentprzych="'"+dokumentprzych+"'"
                        
                    LokDokad = rsglobal.Dekoduj(poz[23])                            
                    if LokDokad in ['None',None]:
                        LokDokad="NULL"
                    else:
                        LokDokad="'"+LokDokad+"'"                        

                    Cecha1Opis = rsglobal.Dekoduj(poz[24])                            
                    if Cecha1Opis in ['None',None]:
                        Cecha1Opis=""

                    Cecha1Wart = rsglobal.Dekoduj(poz[25])                            
                    if Cecha1Wart in ['None',None]:
                        Cecha1Wart=""

                    Produkt = rsglobal.Dekoduj(poz[26])
                    if Produkt in ['None',None]:
                        Produkt="NULL"
                    else:
                        Produkt="'"+Produkt+"'"
                        
                    kierowca = rsglobal.Dekoduj(DaneN[0][6])                            
                    if kierowca in ['None',None]:
                        kierowca="NULL"
                    else:
                        kierowca="'"+kierowca+"'"
                    
                    dostawca=rsglobal.Dekoduj(DaneN[0][2])
    
                    if dostawca in ['None',None]:
                        dostawca="NULL"
                    else:
                        dostawca="'"+dostawca+"'"

                    kontrahent=DaneN[0][1]
                    if kontrahent in [None,'None']:
                        kontrahent="NULL"
                    else:
                        kontrahent="'"+rsglobal.Dekoduj(kontrahent)+"'"

                    czynocny=DaneN[0][7]
                    if czynocny:
                        czynocny="TRUE"
                    else:
                        czynocny="FALSE"
                        
                    if poz[27] == '':
                        Opak = 'NULL'
                    else:
                        Opak = "'"+rsglobal.Dekoduj(poz[27])+"'"
                        
                    if poz[28] == '':
                        Paleta = 'NULL'
                    else:
                        Paleta = "'"+rsglobal.Dekoduj(poz[28])+"'"

                    WspOpak = str(poz[29])
                    WspPaleta = str(poz[30])
                    WspLokacja = str(poz[31])
                    IloscLokacja = str(poz[32])

                    # zmiana pozycji przyjcia z wirtualnych na widoczne
                    #rsglobal.ZapiszDoLogow('14 ', 1)
                    SQL="UPDATE rs_artykuly_przyjecia SET wirtualne=FALSE WHERE firma_id='"+firma_id+"'" 
                    SQL+=" AND magazyn_id='"+magprzyj+"' AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+numprzyj+";"
                    # wstawienie obrotw magazynowych
                    SQL+=" INSERT INTO rs_obroty_magazynowe (firma_id,magazyn_id,dokument_id,data,numer,typ,typ_znak,nr_poz,nr_ppoz,"
                    SQL+=" kontrahent_id,nr_zlecenia,konto_kosztow,nr_faktury,artykul_id,ilosc1,ilosc2,"
                    SQL+=" cena1,cena2,cena_pierw1,cena_pierw2,wartosc,nr_serii,termin_przyd,waluta,"
                    SQL+=" cena_wal1,cena_wal2,cena_pierw_wal1,cena_pierw_wal2,wartosc_wal,dokument_przych,"
                    SQL+=" kierowca,wejscie_data,wejscie_magazyn,dostawca_id,czy_nocny,cecha1_opis,cecha1_wartosc,produkt_id,"
                    SQL+=" opakowanie_id,paleta_id,wsp_ilosci_opakowanie,wsp_ilosci_paleta,wsp_ilosci_lokacja,ilosc_lokacja) VALUES"
                    SQL+= "('"+firma_id+"','"+magprzyj+"','"+Dok+"','"+Data+"',"+numprzyj+",'P',1,"+nr_poz+","+nr_ppoz+","
                    SQL+= kontrahent+","+zlecenie+","+kontok+","+faktura+",'"+artykul+"',"+str(ilosc1)+","+str(ilosc2)+","
                    SQL+= str(cena1)+","+str(cena2)+","+str(cenap1)+","+str(cenap2)+","+str(rsglobal.RSRound(ilosc1*cena1,2))+",'"+seria+"',"+termin+",'"+waluta+"',"
                    SQL+= str(cenaw1)+","+str(cenaw2)+","+str(cenapw1)+","+str(cenapw2)+","+str(rsglobal.RSRound(ilosc1*cenaw1,2))+","+dokumentprzych+","
                    SQL+= kierowca+","+wejscie_data+","+wejscie_magazyn+","+dostawca+","+czynocny+",'"+Cecha1Opis+"','"+Cecha1Wart+"',"+Produkt+","
                    SQL+= Opak+","+Paleta+","+WspOpak+","+WspPaleta+","+WspLokacja+","+IloscLokacja+");"
                    
                    stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1)
                    
                if stat=="OK":
                    #zapis do magazynw
                    SQL="SELECT rsf_zapisz_do_magazynow('"+firma_id+"','"+magprzyj+"','"+artykul+"','"+Data+"',"+str(ilosc1)+","+str(ilosc2)+","+str(cena1)+","+str(cena2)+");"
                    stat, DaneZap, dump, dump = self.PolSQL.Execute(SQL, 3)
                if stat=="OK":
                    if DaneZap[0][0] != 'OK':
                        stat="Bd zapisu do magazynw: "+DaneZap[0][0]



                if stat=="OK":       
                    if Typ in ['P','M']:
                        # na artykuach przyjecia
                        # sprawdzamy czy istnieje spis, ktry warto zmienia
                        stat,LDataKat=fs_spis_z_natury_pom.PodajDatyIKategorieSpisow(self,magprzyj,Data,artykul)
                        if stat=='OK':
                            # dla kadego pasujcego spisu
                            for spis in LDataKat:
                                if stat=='OK':
                                    DataSpisu = spis[0]
                                    KategSpisu = spis[1]
                                    # odwieamy przyjcia spisu dla naszego artykuu
                                    stat = fs_spis_z_natury_pom.OdswiezPozycjeArtykulowPrzyjecWSpisieZNatury(self, DataSpisu, magprzyj, KategSpisu, artykul)
                                    # przeliczamy iloci stwierdzone na pozycjach spisu
                                    if stat=='OK':
                                        stat,dump = fs_spis_z_natury_pom.WstawIlosciStwierdzoneDoPozycjiArtykulowPrzyjeciaNaSpisieZNatury(self, DataSpisu, magprzyj, KategSpisu, artykul)

            # Cz algorytmu czca kosze (i inne artykuy 'konsolidowane') w grupy dla dokumentw typu M
            if stat =='OK' and Typ == 'M':
                sttmp = 'OK'
                # sprawdzamy, czy w magazynie s lokacje. Jeli tak, to tego algorytmu nie wykonujemy
                SQL = "SELECT czy_lokacje FROM rs_magazyny WHERE firma_id='"+firma_id+"' AND magazyn_id='"+magprzyj+"';"
                stat, DMag, dump, dump = self.PolSQL.Execute(SQL, 3)
                if stat == 'OK':
                    if DMag[0][0]:
                        sttmp = 'LOK'

                if stat =='OK' and sttmp == 'OK':
                    # sprawdzamy pozycje, na ktrych wystpuj artykuy z opcj 'konsolidowany' (czymy unikalne dane z artykuy przyjcia i obroty magazynowe)
                    SQL = "SELECT SUM(ap.ilosc1),SUM(ap.ilosc2),ap.nr_poz, COALESCE(ap.dostawca_id,''), COALESCE(ap.nr_faktury,''), ap.nr_serii, COALESCE(ap.termin_przyd::TEXT,''), ap.artykul_id,ap.cena1,ap.cena2,ap.cena_pierw1,ap.cena_pierw2," \
                        " ap.waluta, ap.cena_wal1, ap.cena_wal2, ap.cena_pierw_wal1, ap.cena_pierw_wal2, ap.opis,ap.cecha1_opis,ap.cecha1_wartosc,COALESCE(ap.opakowanie_id,''),COALESCE(ap.paleta_id,''),ap.wsp_ilosci_opakowanie,ap.wsp_ilosci_paleta," \
                        " COALESCE(ob.kontrahent_id,''), COALESCE(ob.nr_zlecenia,''), COALESCE(ob.konto_kosztow,''), COALESCE(ob.kierowca,''),ob.czy_nocny,COALESCE(ob.produkt_id,'')" \
                        " FROM rs_artykuly_przyjecia ap INNER JOIN rs_artykuly a USING (firma_id,artykul_id)" \
                        " INNER JOIN rs_obroty_magazynowe ob ON (ob.firma_id=ap.firma_id AND ob.magazyn_id=ap.magazyn_id AND ob.dokument_id=ap.dokument_id AND ob.data=ap.data AND ob.numer=ap.numer " \
                        " AND ob.typ='P' AND ob.typ_znak=1 AND ob.nr_poz=ap.nr_poz AND ob.nr_ppoz=ap.nr_ppoz)" \
                        " WHERE ap.firma_id='"+firma_id+"' AND ap.magazyn_id= '"+magprzyj+"' AND ap.dokument_id = '"+Dok+"' AND ap.data = '"+Data+"' AND ap.numer = "+numprzyj+" AND a.konsolidowany " \
                        " GROUP BY ap.nr_poz, COALESCE(ap.dostawca_id,''), ap.nr_faktury, ap.nr_serii, COALESCE(ap.termin_przyd::TEXT,''), ap.artykul_id,ap.cena1,ap.cena2,ap.cena_pierw1,ap.cena_pierw2," \
                        " ap.waluta, ap.cena_wal1, ap.cena_wal2, ap.cena_pierw_wal1, ap.cena_pierw_wal2, ap.opis,ap.cecha1_opis,ap.cecha1_wartosc,COALESCE(ap.opakowanie_id,''),COALESCE(ap.paleta_id,''),ap.wsp_ilosci_opakowanie,ap.wsp_ilosci_paleta," \
                        " COALESCE(ob.kontrahent_id,''), COALESCE(ob.nr_zlecenia,''), COALESCE(ob.konto_kosztow,''), COALESCE(ob.kierowca,''),ob.czy_nocny,COALESCE(ob.produkt_id,'')" \
                        " ORDER BY nr_poz"

                    stat, DKonsol, dump, dump = self.PolSQL.Execute(SQL, 3)

                    # mamy wszystkie pozycje do podstawienia. Usuwamy je z artykuw przyj i obrotw (za pomoc filtra na konsolidowany)
                    if stat == 'OK' and len(DKonsol) > 0:
                        SQL = "DELETE FROM rs_artykuly_przyjecia WHERE firma_id='"+firma_id+"' AND magazyn_id= '"+magprzyj+"' AND dokument_id = '"+Dok+"' AND data = '"+Data+"' AND numer = "+numprzyj+" " \
                                "AND (SELECT konsolidowany FROM rs_artykuly WHERE firma_id='"+firma_id+"' AND artykul_id = rs_artykuly_przyjecia.artykul_id); "
                        stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1)
                        if stat == 'OK':
                            SQL = "DELETE FROM rs_obroty_magazynowe WHERE firma_id='"+firma_id+"' AND magazyn_id= '"+magprzyj+"' AND dokument_id = '"+Dok+"' AND data = '"+Data+"' AND numer = "+numprzyj+" AND typ = 'P'" \
                                "AND (SELECT konsolidowany FROM rs_artykuly WHERE firma_id='"+firma_id+"' AND artykul_id = rs_obroty_magazynowe.artykul_id);"
                            stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1)

                        # pobieramy zajte numery pozycji i podpozycji
                        if stat == 'OK':
                            SQL="SELECT nr_poz,nr_ppoz FROM rs_artykuly_przyjecia WHERE firma_id='"+firma_id+"' AND magazyn_id= '"+magprzyj+"'" \
                                " AND dokument_id = '"+Dok+"' AND data = '"+Data+"' AND numer = "+numprzyj+" ORDER BY nr_poz,nr_ppoz"
                            stat, DNryPoz, dump, dump = self.PolSQL.Execute(SQL, 3)
                            if stat =='OK':
                                SNryPoz = {}
                                for npoz in DNryPoz:
                                    if not npoz[0] in SNryPoz:
                                        SNryPoz[npoz[0]] = [npoz[1]]
                                    else:
                                        SNryPoz[npoz[0]].append(npoz[1])

                        # wstawiamy kolejne pozycje/podpozycje do artykuw przyj i obrotw
                        if stat == 'OK':
                            TNrPoz = 0
                            TNrPPoz = 1
                            for pozkon in DKonsol:
                                if stat == 'OK':
                                    # zerujemy podpozycje dla nowej pozycji
                                    if TNrPoz != pozkon[2]:
                                        TNrPoz = pozkon[2]
                                        TNrPPoz = 1

                                    # omijamy istniejce podpozycje
                                    if TNrPoz in SNryPoz:
                                        while TNrPPoz in SNryPoz[TNrPoz]:
                                            TNrPPoz += 1

                                    Dostawca ="'"+pozkon[3]+"'"
                                    if Dostawca == "''":
                                        Dostawca = 'NULL'

                                    NrFaktury ="'"+pozkon[4].strip()+"'"
                                    if NrFaktury == "''":
                                        NrFaktury = 'NULL'

                                    Termin ="'"+pozkon[6].strip()+"'"
                                    if Termin in ["''",'-  -']:
                                        Termin = 'NULL'

                                    Opakow ="'"+pozkon[20].strip()+"'"
                                    if Opakow == "''":
                                        Opakow = 'NULL'

                                    PaletaKon ="'"+pozkon[21].strip()+"'"
                                    if PaletaKon == "''":
                                        PaletaKon = 'NULL'

                                    # mamy woln podpozycj to wstawiamy wiersz do artykuw przyj i obrotw

                                    # wstawienie do artykuw przyj
                                    SQL = "INSERT INTO rs_artykuly_przyjecia  (firma_id,magazyn_id,data,dokument_id,numer,nr_poz,nr_ppoz," \
                                          "dostawca_id,nr_faktury,nr_serii,termin_przyd,artykul_id,ilosc_prz1,ilosc_prz2,cena1,cena2,cena_pierw1,cena_pierw2,waluta," \
                                          "cena_wal1,cena_wal2,cena_pierw_wal1,cena_pierw_wal2,ilosc1,ilosc2,rezerwacja1,rezerwacja2,wejscie_data,wejscie_magazyn," \
                                          "opis,cecha1_opis,cecha1_wartosc,wirtualne,opakowanie_id,paleta_id,wsp_ilosci_opakowanie,wsp_ilosci_paleta) VALUES " \
                                          " ('"+firma_id+"','"+magprzyj+"','"+Data+"','"+Dok+"',"+numprzyj+","+str(TNrPoz)+","+str(TNrPPoz)+"," \
                                          ""+Dostawca+","+NrFaktury+",'"+pozkon[5]+"',"+Termin+",'"+pozkon[7]+"',"+str(pozkon[0])+","+str(pozkon[1])+","+str(pozkon[8])+","+str(pozkon[9])+","+str(pozkon[10])+","+str(pozkon[11])+",'"+pozkon[12]+"'," \
                                          ""+str(pozkon[13])+","+str(pozkon[14])+","+str(pozkon[15])+","+str(pozkon[16])+","+str(pozkon[0])+","+str(pozkon[1])+",0,0,NOW(),'"+magprzyj+"'," \
                                          "'"+pozkon[17]+"','"+pozkon[18]+"','"+pozkon[19]+"',FALSE,"+Opakow+","+PaletaKon+","+str(pozkon[22])+","+str(pozkon[23])+");"


                                    rsglobal.ZapiszDoLogow('INSERT  rs_artykuly_przyjecia' + SQL, 5)

                                    # wstawienie obrotu
                                    ObKontr = "'"+pozkon[24]+"'"
                                    if ObKontr == "''":
                                        ObKontr = 'NULL'

                                    ObZlec = "'"+pozkon[25]+"'"
                                    if ObZlec == "''":
                                        ObZlec = 'NULL'

                                    ObKontoK = "'"+pozkon[26]+"'"
                                    if ObKontoK == "''":
                                        ObKontoK = 'NULL'

                                    ObKierowca = "'"+pozkon[27]+"'"
                                    if ObKierowca == "''":
                                        ObKierowca = 'NULL'

                                    if pozkon[28]:
                                        ObCzyNocny = 'TRUE'
                                    else:
                                        ObCzyNocny = 'FALSE'

                                    ObProdukt = "'"+pozkon[29]+"'"
                                    if ObProdukt == "''":
                                        ObProdukt = 'NULL'

                                    SQL+=" INSERT INTO rs_obroty_magazynowe (firma_id,magazyn_id,dokument_id,data,numer,typ,typ_znak,nr_poz,nr_ppoz," \
                                        " kontrahent_id,nr_zlecenia,konto_kosztow,nr_faktury,artykul_id,ilosc1,ilosc2," \
                                        " cena1,cena2,cena_pierw1,cena_pierw2,wartosc,nr_serii,termin_przyd,waluta," \
                                        " cena_wal1,cena_wal2,cena_pierw_wal1,cena_pierw_wal2,wartosc_wal,dokument_przych," \
                                        " kierowca,wejscie_data,wejscie_magazyn,dostawca_id,czy_nocny,cecha1_opis,cecha1_wartosc,produkt_id," \
                                        " opakowanie_id,paleta_id,wsp_ilosci_opakowanie,wsp_ilosci_paleta,wsp_ilosci_lokacja,ilosc_lokacja) VALUES" \
                                        "('"+firma_id+"','"+magprzyj+"','"+Dok+"','"+Data+"',"+numprzyj+",'P',1,"+str(TNrPoz)+","+str(TNrPPoz)+"," \
                                        ""+ ObKontr+","+ObZlec+","+ObKontoK+",'"+pozkon[27]+"','"+pozkon[7]+"',"+str(pozkon[0])+","+str(pozkon[1])+"," \
                                        ""+str(pozkon[8])+","+str(pozkon[9])+","+str(pozkon[10])+","+str(pozkon[11])+","+str(rsglobal.RSRound(pozkon[0]*pozkon[8],2))+",'"+pozkon[5]+"',"+Termin+",'"+pozkon[12]+"'," \
                                        ""+str(pozkon[13])+","+str(pozkon[14])+","+str(pozkon[15])+","+str(pozkon[16])+","+str(rsglobal.RSRound(pozkon[0]*pozkon[8],2))+",''," \
                                        ""+ObKierowca+",NOW(),'"+magprzyj+"',"+Dostawca+","+ObCzyNocny+",'"+pozkon[17]+"','"+pozkon[18]+"',"+ObProdukt+"," \
                                        ""+Opakow+","+PaletaKon+","+str(pozkon[22])+","+str(pozkon[23])+",0,0);"

                                    # wstawienie pozycji
                                    stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1)

                                    if stat == 'OK':
                                        TNrPPoz += 1

        if stat=="OK":
            if Typ=="P":
                #rsglobal.ZapiszDoLogow('15 ',1)
                SQL="UPDATE rs_dokumenty_magazynowe_pozycje SET wejscie_magazyn='"+Mag+"', wejscie_data='"+Data+"' "
                SQL+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"' AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+Numer+";" 
                stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1)

    #----------------------------------
    # POWRT DO OGLNEGO ALGORYTMU
    #----------------------------------
    
    if stat=='OK':
        #JR 12549 (przepisywanie kart przyjecia)
        SQL = "INSERT INTO rs_artykuly_przyjecia_karty_przyjecia "
        SQL += " (firma_id,magazyn_id,data, dokument_id, numer,nr_poz,nr_ppoz,"
        SQL += " kod, wart_b, wart_d, wart_n, wart_s,operator) "
        SQL += " (SELECT firma_id,magazyn_id,data, dokument_id, numer,nr_poz,nr_ppoz,"
        SQL += " kod, wart_b, wart_d, wart_n, wart_s,operator FROM rs_dokumenty_magazynowe_pozycje_karty_przyjecia"
        SQL += " WHERE firma_id='" + firma_id + "' AND data='" + Data + "' AND magazyn_id='" + magprzyj
        SQL += "' AND dokument_id='" + Dok + "' AND numer=" + numprzyj + ");"
        stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1)

    if stat=='OK':
        #rsglobal.ZapiszDoLogow('16 ',1)
        SQL="UPDATE rs_dokumenty_magazynowe SET stan='Z' WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"'"
        SQL+=" AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+Numer+";"
        stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1)
        #JR 2023-01-21 (13286) - otwarcie i zamkniecie rozchodowych w detalicznym
        if stat == 'OK':
            if Typ == 'P':
                if CzyMagazynDetaliczny:
                    stat = fs_dokumenty_magazynowe_pom.HDPrzeliczWydania(self, Mag, Dok, Data, Numer)

        
    # dla dokumentw PW i ich korekt uruchamiamy funkcj tworzc RW
    if stat=='OK':
        if Typ in ("P","W"):
            stat=fs_dokumenty_magazynowe_pom.UtworzRWzPW(self,Mag,Dok,Data,Numer)
            rsglobal.ZapiszDoLogow('porwzpw \n')
            rsglobal.ZapiszDoLogow('Funkcja \n'+str(Funkcja))

    if stat=='OK':
        # zablokowanie inwentaryzacji pniejszych ni dokument ( nie robimy dla PW i RW ze spisw)
        if Funkcja!=29:
            SQL="UPDATE rs_inwentaryzacja SET czy_dokumenty_po=TRUE WHERE firma_id='"+firma_id+"'"
            SQL+=" AND magazyn_id IN ('"+TestMagSkad+"','"+TestMagDokad+"') AND stan='Z' AND data>='"+Data+"' AND NOT COALESCE(czy_dokumenty_po,FALSE);"
            stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1)    
        
    if stat=='OK':
        # dodaj ilosci z produkcji na zamowienia odbiorcow
        stat=fs_dokumenty_magazynowe_produkcja_pom.ZmienIlosciWyprodukoweNaZamowieniachOdbiorcow(self,[Mag,Dok,Data,Numer],'+')

    if stat=='OK':
        # pobranie magazynu i numeru przeciwnego na dokumenty typu M
        if Typ=='M':
            #rsglobal.ZapiszDoLogow('17 ',1)
            SQL="UPDATE rs_obroty_magazynowe SET "
            SQL+="magazyn_przeciwny= rsf_podaj_magazyn_przeciwstawny(firma_id,magazyn_id,dokument_id,data,numer,typ_znak),"
            SQL+="numer_przeciwny= rsf_podaj_numer_przeciwstawny(firma_id,magazyn_id,dokument_id,data,numer,typ_znak)"
            SQL+=" WHERE firma_id='"+firma_id+"' AND dokument_id='"+Dok+"' AND data='"+Data+"' AND magazyn_przeciwny IS NULL;"
        
            stat, Test, dump, dump = self.PolSQL.Execute(SQL, 1)

#@##########################################
# ZAMYKANIE DOKUMENTU WYDANIA DETALICZNEGO   W --> Z (108, 118)
#@##########################################

if Funkcja in (108,118,) and stat == 'OK':
    rsglobal.ZapiszDoLogow('Funkcjaa \n'+str(Funkcja))
    # wszystkie wczesniejsze dokumenty magazynowe musza byc pozamykane.
    SQL = "SELECT count(*) FROM rs_dokumenty_magazynowe dm"
    SQL += " INNER JOIN rs_typy_dokumentow_m td USING (firma_id,dokument_id) "
    SQL += " WHERE firma_id='" + firma_id + "' AND magazyn_id='" + Mag + "'"
    SQL += " AND stan NOT IN ('Z','K') AND (data<'" + Data + "'"
    SQL += " OR (data='" + Data + "' AND dokument_id<>'" + Dok + "' AND td.typ_dokumentu<>'W'));"
    rsglobal.ZapiszDoLogow(SQL)
    stat, iled, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat == 'OK':
        if iled[0][0] > 0:
            stat = 'Nie zamknitych jest '+ str(iled[0][0]) + ' wczeniejszych dokumentw magazynowych.'

    # wszystkie wczesniejsze dostawy musza byc pozamykane.
    if stat == 'OK':
        SQL = "SELECT count(*) FROM rs_dostawy WHERE firma_id='" + firma_id + "'"
        SQL += " AND (d_magazyn_id='" + Mag + "' OR d_magazyn_id IS NULL) "
        SQL += " AND stan NOT IN ('Z','K') AND data<='" + Data + "'"
        rsglobal.ZapiszDoLogow(SQL)
        stat, iled, dump, dump = self.PolSQL.Execute(SQL, 3)
        if stat == 'OK':
            if iled[0][0] > 0:
                stat = 'Nie zamknitych jest ' + str(iled[0][0]) + ' wczeniejszych dokumentw dostaw.'

    if stat == 'OK':
        # sprawdzenie obecnoci odpowiednich danych na nagowku dokumentu
        SQL = "SELECT rsf_sprawdz_naglowek_dok_mag('" + firma_id + "','" + Mag + "','" + Dok + "','" + Data + "'," + Numer + ");"
        rsglobal.ZapiszDoLogow(SQL)
        stat, Test, dump, dump = self.PolSQL.Execute(SQL, 3)
        if stat == "OK":
            if Test[0][0] != 'OK':
                stat = Test[0][0]

    # wycena pozycji
    if stat == 'OK':
        SQL = "SELECT nr_poz,artykul_id,ilosc1,COALESCE(cena1,0), nag.kontrahent_id, produkt_id,"  #AB 2023-08-25
        SQL += " COALESCE(dokument_przych,'')"  #JR 2023-09-22
        SQL += " FROM rs_dokumenty_magazynowe_pozycje INNER JOIN rs_dokumenty_magazynowe nag USING(firma_id,magazyn_id,dokument_id,data,numer) "
        SQL += " WHERE firma_id='" + firma_id + "'"
        SQL += " AND magazyn_id='" + Mag + "' AND dokument_id='" + Dok + "' AND data='" + Data + "' AND numer=" + Numer + ";"
        rsglobal.ZapiszDoLogow('123SQL '+SQL)
        stat, DaneP, dump, dump = self.PolSQL.Execute(SQL, 3)

    if stat == "OK":
        if len(DaneP) == 0:
            stat = "Brak pozycji na dokumencie magazynowym"
        else:
            Wartosc = 0
            for wiersz in DaneP:
                wiersz[4] = rsglobal.ZmienNaNULL(wiersz[4])  # jesli brak kontrahenta
                if wiersz[4] != 'NULL':
                    wiersz[4] = rsglobal.Dekoduj(wiersz[4])
                #AB 2023-09-06
                wiersz[5] = rsglobal.ZmienNaNULL(wiersz[5])  # jesli brak produktu
                if wiersz[5] != 'NULL':
                    wiersz[5] = rsglobal.Dekoduj(wiersz[5])
                stat, Wydania = fs_dokumenty_magazynowe_pom.HDWydajArtykul(self, wiersz[1], Mag, Data, wiersz[2], wiersz[3], wiersz[6])
                # Wydania = lista list:
                #  Artykul (moze byc zamiennik), Ilosc1, Ilosc2, Cena1, Cena2, Dokument przychodowy, Nr komunikatu - 0 poprawnie
                #    1 - Wydano poprawnie zamiennik, 2 - wydano po ostatniej cenie w magazynie
                #    3 - wydano po ostatniej cenie w firmie
                # robi updaty na artykuly przyjecia, artykuly magazyny i artykuly
                if stat != 'OK':
                    break
                nrppoz = 0
                if len(Wydania) == 1:
                    nrppoz = -1
                # usuniecie poycji i wstawienie pozycji w petli, zapis do obrotow
                UPDINSP = "DELETE FROM rs_dokumenty_magazynowe_pozycje WHERE firma_id='" + firma_id + "'"
                UPDINSP += " AND magazyn_id='" + Mag + "' AND dokument_id='" + Dok + "' AND data='" + Data + "'"
                UPDINSP += " AND numer=" + Numer + " AND nr_poz=" + str(wiersz[0]) + ";"
                # art_do_aktualizacji = set([rsglobal.Dekoduj(wiersz[1])])
                for wyd in Wydania:
                    Wartosc += rsglobal.RSRound(wyd[1] * wyd[3], 2)
                    nrppoz += 1
                    if wyd[0].strip() == wiersz[1].strip():  #JR 2023-08-25 (13513) nie wpisywanie orginalu jesli taki sam
                        orginal = 'NULL'
                    else:
                        orginal = "'" + rsglobal.Dekoduj(wiersz[1]) + "'"

                    UPDINSP += "INSERT INTO rs_dokumenty_magazynowe_pozycje (firma_id,magazyn_id,dokument_id,data,"
                    UPDINSP += "numer,nr_poz,nr_ppoz,artykul_id,"
                    UPDINSP += "ilosc1,ilosc2,cena1,cena2,oryginal,dokument_przych,cena_wal1,cena_wal2, produkt_id)"  #AB 2023-08-25
                    UPDINSP += " VALUES ('" + firma_id + "','" + Mag + "','" + Dok + "','" + Data + "',"
                    UPDINSP += Numer + "," + str(wiersz[0]) + "," + str(nrppoz) + ",'" + rsglobal.Dekoduj(wyd[0]) + "',"
                    UPDINSP += str(wyd[1]) + "," + str(wyd[2]) + "," + str(wyd[3]) + "," + str(wyd[4]) + ","
                    UPDINSP += orginal + ",'" + rsglobal.Dekoduj(wyd[5]) + "',"+str(wyd[3]) + "," + str(wyd[4])+"," + wiersz[5]+");\n"  #AB 2023-08-25
                    UPDINSP += "INSERT INTO rs_obroty_magazynowe (firma_id,magazyn_id,dokument_id,data,numer,typ,typ_znak,nr_poz,nr_ppoz,"
                    UPDINSP += "waluta,cena_wal1,cena_wal2,wartosc_wal,"
                    UPDINSP += "artykul_id,ilosc1,ilosc2,cena1,cena2,wartosc,dokument_przych,kontrahent_id) VALUES"
                    UPDINSP += " ('"+firma_id+"','"+Mag+"','"+Dok+"','"+Data+"',"+Numer+",'W',-1,"+str(wiersz[0])+","+str(nrppoz)+","
                    UPDINSP += "'PLN',"+str(wyd[3]) + ","+str(wyd[4]) + "," + str(rsglobal.RSRound(wyd[1]*wyd[3], 2))+","
                    UPDINSP += "'" + rsglobal.Dekoduj(wyd[0]) + "'," + str(wyd[1]) + ","+str(wyd[2]) + ","+str(wyd[3]) + ","+str(wyd[4]) + ","
                    UPDINSP += str(rsglobal.RSRound(wyd[1]*wyd[3], 2))+",'" + wyd[5] + "'," + wiersz[4] + ");\n"
                    # art_do_aktualizacji.add(rsglobal.Dekoduj(wyd[0]))
                stat, dump, dump, dump = self.PolSQL.Execute(UPDINSP, 1)
                rsglobal.ZapiszDoLogow("UPDINSP: \n"+UPDINSP)
                if stat != 'OK':
                    break
                #AB 28.06.23r
                # # zmiana ilosci na spisach z natury
                # if stat == "OK":
                #     # sprawdzamy czy istnieje spis, ktry warto zmienia
                #     for art in art_do_aktualizacji:  #AB 15.06.23r
                #         if stat == 'OK':
                #             stat, LDataKat = fs_spis_z_natury_pom.PodajDatyIKategorieSpisow(self, Mag, Data, art)
                #             if stat == 'OK':
                #                 # dla kadego pasujcego spisu
                #                 for spis in LDataKat:
                #                     if stat == 'OK':
                #                         DataSpisu = spis[0]
                #                         KategSpisu = spis[1]
                #                         # odwieamy przyjcia spisu dla naszego artykuu
                #                         stat = fs_spis_z_natury_pom.OdswiezPozycjeArtykulowPrzyjecWSpisieZNatury(self, DataSpisu, Mag, KategSpisu, art)
                #                         # przeliczamy iloci stwierdzone na pozycjach spisu
                #                         if stat == 'OK':
                #                             stat, dump = fs_spis_z_natury_pom.WstawIlosciStwierdzoneDoPozycjiArtykulowPrzyjeciaNaSpisieZNatury(
                #                                     self, DataSpisu, Mag, KategSpisu, art)



    #AB 28.06.23r
    #zmiana iloci na spisach z natury
    if stat == 'OK':
        stat = fs_spis_z_natury_pom.ZaktualizujSpisyPoZmianachNaDokumencieMagazynowym(self, Mag, Data, Dok, Numer)


    if stat == 'OK':
        # rsglobal.ZapiszDoLogow('16 ',1)
        Wartosc = rsglobal.RSRound(Wartosc, 2)
        SQL = "UPDATE rs_dokumenty_magazynowe SET stan='Z', kwota= " + str(Wartosc) + ", kwota_wal= " + str(Wartosc)
        SQL += " WHERE firma_id='" + firma_id + "' AND magazyn_id='" + Mag + "'"
        SQL += " AND dokument_id='" + Dok + "' AND data='" + Data + "' AND numer=" + Numer + ";"

        stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1)

#@##########################################
# OTWIERANIE DOKUMENTU DO EDYCJI   G --> W (10)
#@##########################################
# 30 - tylko scalanie pozycji

#WS 2023-03-02
if Funkcja in (10, 20, 30) and stat=='OK' and Typ not in ('C', 'I'):
# if Funkcja in (10, 20, 30) and stat=='OK' and Typ<>'C':
    if Funkcja != 30:
        if Stan != "G":
            stat = "Dokument jest w stanie '"+Stan+"' - nie mona przygotowa dokumentu do edycji"

        #sprawdzenie czy nie ma przypisanej faktury
        # W TestFakt 'przy okazji' pytam o kontrahenta - prosze nie usuwac - wykorzystane w zamowieniach - Mariusz
                
        if stat=='OK':
            SQL ="SELECT (f_data IS NULL) AND (k_data IS NULL),kontrahent_id,operator_wms"
            SQL+=" FROM rs_dokumenty_magazynowe "
            SQL+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"' AND dokument_id='"+Dok+"' "
            SQL+=" AND data='"+Data+"' AND numer="+str(Numer)+";"
            stat, TestFakt, dump, dump = self.PolSQL.Execute(SQL, 3)
            
        if stat=='OK':
            if len(TestFakt)==0:
                stat="Brak dokumentu o podanych parametrach"
            elif not TestFakt[0][0]:
                stat="Nie mona otworzy dokumentu - dokument ma przypisan faktur"
            else:
                OperatorWMS = TestFakt[0][2]

    if stat=='OK':
        #czy dokument nie jest przypisany do przesylki
        SQL="SELECT p_data, p_numer FROM rs_dokumenty_magazynowe WHERE firma_id='"+firma_id+"'"
        SQL+=" AND magazyn_id='"+Mag+"' AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+Numer+";"
        stat, DanePR, dump, dump = self.PolSQL.Execute(SQL, 3)
        if stat != "OK":
            stat='Bd sprawdzenia dokumentu magazynowego:'+stat
        else:
            if len(DanePR)==0:
                stat="Brak podanego dokumentu magazynowego"
            elif DanePR[0][0] not in (None,'','None'):
                stat='Dokument powizany z przesyk nr '+str(DanePR[0][1])+' z dnia '+str(DanePR[0][0])+'.'


    if stat=='OK':
        #----------------------------------
        # DOKUMENT PRZYJCIA
        #----------------------------------
        
        #Czycimy kwot nagowka oraz uwzgldniamy kwestie zwizane z lokacjami
        if Typ in ['M','P','L']:
            PolaNaglowka=",kwota=0, kwota_wal=0"
            # LOKACJE
            # przegldamy list pozycji, jeli stwierdzamy, e jest to podpozycja z lokacji to :
            # 1. W ptli tworzymy pozycje nr 0.
            # 2. W oparciu o podpozycje z czy_lokacja=True tworzymy tabelk. Od razu dla wszystkich pozycji
            # 3. Grupowo usuwamy podpozycje z czy_lokacja=True
            # 4. Potem tworzymy ponownie zsumowane artykuy przyjcia
            
            # wiemy, e w tym magazynie obsugiwane s lokacje
            if CzyLokacjeDokad and Typ=='P':
                # pobranie danych pozycji, ktre bd zmieniane 
                pola="dp.nr_poz,dp.artykul_id,dp.cena1,dp.cena2,dp.nr_serii,"   # 2 -6
                pola+="dp.termin_przyd,dp.cena_wal1,dp.cena_wal2,dp.opis,"      # 7 - 10
                pola+="dp.czy_dodatkowy_nr_serii,dp.dostawca_id,dp.oryginal,"   # 11 - 13
                pola+="dp.cecha1_opis,dp.cecha1_wartosc,dp.produkt_id, "        # 14 - 16
                pola+="COALESCE(dp.opakowanie_id,''),COALESCE(dp.paleta_id,''),COALESCE(dp.wsp_ilosci_opakowanie,0)," # 17 - 19
                pola+="COALESCE(dp.wsp_ilosci_paleta,0),COALESCE(dp.wsp_ilosci_lokacja,0),COALESCE(dp.ilosc_lokacja,0)" # 20 - 22
                
                SQLDP=" SELECT SUM(dp.ilosc1), ROUND(SUM(dp.ilosc1)*a.przelicznik,4),"+pola
                SQLDP+=" FROM rs_dokumenty_magazynowe_pozycje dp INNER JOIN rs_artykuly a USING (firma_id,artykul_id) "
                SQLDP+=" WHERE dp.firma_id='"+firma_id+"' and dp.magazyn_id='"+Mag+"'"
                SQLDP+=" AND dp.dokument_id='"+Dok+"' AND dp.data='"+Data+"' AND dp.numer="+str(Numer)
                SQLDP+=" AND dp.nr_ppoz>0 AND czy_lokacje AND NOT a.dynamiczny_przelicznik GROUP BY "+pola+",a.przelicznik;"
                
                stat, DaneLok, dump, dump = self.PolSQL.Execute(SQLDP, 3)

                if stat=='OK':
                    # jeli jest co do zrobienia to robimy
                    if len(DaneLok)>0:
                        SQLLok=''
                        #1. Utworzenie w ptli pozycji 0
                        for poz in DaneLok:
                            TerminPrzyd=rsglobal.Dekoduj(poz[7])
                            if TerminPrzyd in (None,'None'):
                                TerminPrzyd='NULL'
                            else:
                                TerminPrzyd="'"+TerminPrzyd+"'"
                            
                            if poz[11]:
                                CzyDodNrSerii='TRUE'
                            else:
                                CzyDodNrSerii='FALSE'
                            
                            Oryg=rsglobal.Dekoduj(poz[13])
                            if Oryg in (None,'None'):
                                Oryg='NULL'
                            else:
                                Oryg="'"+Oryg+"'"
                            
                            Cecha1Opis=rsglobal.Dekoduj(poz[14])
                            if Cecha1Opis in (None,'None'):
                                Cecha1Opis=''
                                
                            Cecha1Wart=rsglobal.Dekoduj(poz[15])
                            if Cecha1Wart in (None,'None'):
                                Cecha1Wart=''

                            Produkt=rsglobal.Dekoduj(poz[16])
                            if Produkt in [None,'None']:
                                Produkt='NULL'
                            else:
                                Produkt="'"+Produkt+"'"
                                
                            if poz[17] == '':
                                Opak = 'NULL'
                            else:
                                Opak = "'"+rsglobal.Dekoduj(poz[17])+"'"
                                
                            if poz[18] == '':
                                Paleta = 'NULL'
                            else:
                                Paleta = "'"+rsglobal.Dekoduj(poz[18])+"'"
                                
                            WspIlosciOpak = poz[19]
                            WspIlosciPaleta = poz[20]
                            WspIlosciLokacja = poz[21]
                            if self.lokacje_jm =='1':
                                IloscLok = poz[0]
                            else:
                                IloscLok = poz[1]
                            if poz[17] <>'' and poz[18] != '':
                                IloscLok = rsglobal.Dekoduj(IloscLok / WspIlosciOpak / WspIlosciPaleta / WspIlosciLokacja,4)
                                
                            SQLLok+="INSERT INTO rs_dokumenty_magazynowe_pozycje (firma_id,magazyn_id,dokument_id,data,"
                            SQLLok+="numer,nr_poz,nr_ppoz,artykul_id,"
                            SQLLok+="ilosc1,ilosc2,cena1,cena2,nr_serii,termin_przyd,cena_wal1,cena_wal2,opis,"
                            SQLLok+="czy_dodatkowy_nr_serii,dostawca_id,oryginal,cecha1_opis,cecha1_wartosc,produkt_id,"
                            SQLLok+="opakowanie_id,paleta_id,wsp_ilosci_opakowanie,wsp_ilosci_paleta,wsp_ilosci_lokacja,ilosc_lokacja) "
                            SQLLok+=" VALUES ('"+firma_id+"','"+Mag+"','"+Dok+"','"+Data+"',"
                            SQLLok+=str(Numer)+","+str(poz[2])+",0,'"+rsglobal.Dekoduj(poz[3])+"',"
                            SQLLok+=str(poz[0])+","+str(poz[1])+","+str(poz[4])+","+str(poz[5])+","
                            SQLLok+="'"+rsglobal.Dekoduj(poz[6])+"',"+TerminPrzyd+","
                            SQLLok+=str(poz[8])+","+str(poz[9])+",'"+rsglobal.Dekoduj(poz[10])+"',"
                            SQLLok+=CzyDodNrSerii+",NULL,"+Oryg+",'"+Cecha1Opis+"','"+Cecha1Wart+"',"+Produkt+","
                            SQLLok+=Opak+","+Paleta+","+str(WspIlosciOpak)+","+str(WspIlosciPaleta)+","+str(WspIlosciLokacja)+","+str(IloscLok)+");"
                
                        #2. Utworzenie tabelki z lokacjami
                        SQLLok+="INSERT INTO rs_dokumenty_magazynowe_lokacje (firma_id,magazyn_id,dokument_id,"
                        SQLLok+="data,numer,nr_poz,nr_ppoz,lp,lokacja_skad,lokacja_dokad,ilosc)"
                        SQLLok+="SELECT firma_id,magazyn_id,dokument_id,data,numer,nr_poz,0,nr_ppoz,"
                        SQLLok+="lokacja_skad,lokacja_dokad,ilosc"+self.lokacje_jm+" FROM rs_dokumenty_magazynowe_pozycje "
                        SQLLok+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"' AND dokument_id='"+Dok+"'"
                        SQLLok+=" AND data='"+Data+"' AND numer="+str(Numer)+" AND czy_lokacje;"
                        
                        # przepisujemy palety z rs_dokumenty_magazynowe_pozycje_palety z pozycji 0 do tych z numerami
                        SQLLok+= "INSERT INTO rs_dokumenty_magazynowe_pozycje_palety "
                        SQLLok+=" SELECT DISTINCT dp.firma_id,dp.magazyn_id,dp.dokument_id,dp.data,dp.numer,dp.nr_poz,0,dpp.nr_palety_id"
                        SQLLok+=" FROM rs_dokumenty_magazynowe_pozycje dp INNER JOIN rs_dokumenty_magazynowe_pozycje_palety dpp "
                        SQLLok+=" ON (dpp.firma_id=dp.firma_id AND dpp.magazyn_id=dp.magazyn_id AND dpp.dokument_id=dp.dokument_id "
                        SQLLok+=" AND dpp.data=dp.data AND dpp.numer=dp.numer AND dpp.nr_poz= dp.nr_poz AND dpp.nr_ppoz=0)"
                        SQLLok+=" WHERE dp.firma_id='"+firma_id+"'"
                        SQLLok+=" AND dp.dokument_id='"+Dok+"' AND dp.magazyn_id='"+Mag+"' AND dp.data='"+Data+"' AND dp.numer="+str(Numer)+" AND czy_lokacje;"
                        
                        #3. Usunicie podpozycji z pozycji dokumentu magazynowego z czy_lokacje=True
                        SQLLok+="DELETE FROM rs_dokumenty_magazynowe_pozycje "
                        SQLLok+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"' AND dokument_id='"+Dok+"'"
                        SQLLok+=" AND data='"+Data+"' AND numer="+str(Numer)+" AND nr_ppoz>0 AND czy_lokacje;"
                        stat, dump, dump, dump = self.PolSQL.Execute(SQLLok, 1)
                        
                        #4. Wstawienie zsumowanych wirtualnych pozycji przyjcia
                        stat=fs_dokumenty_magazynowe_lokacje_pom.OdswiezWirtualnePozPrzyj(self,Mag,Dok,Data,Numer)


        #----------------------------------
        # DOKUMENT WYDANIA
        #----------------------------------
        if Typ in ['M','W','L','D']:
            if stat == "OK":
                # kolumny 0 - 5
                SQL="SELECT dp.nr_poz,dp.nr_ppoz,dp.artykul_id,COALESCE(dp.oryginal,dp.artykul_id),dp.ilosc1,dp.ilosc2,"
                # kolumny 6 - 12
                SQL+="dp.cena1,dp.cena2,dp.cena_wal1,dp.cena_wal2,COALESCE(dp.nr_serii,''),dp.termin_przyd,dp.opis,"
                # kolumny 13 - 17
                SQL+="dp.czy_rozbicie,dp.dokument_przych,dp.czy_cena,dp.czy_seria,dp.czy_termin,"
                # kolumny 18 - 21
                SQL+="dp.czy_dodatkowy_nr_serii,dp.lokacja_skad,dp.lokacja_dokad,dp.czy_lokacje,"
                # kolumny 22 - 24
                SQL+="a.cecha1_opis,dp.cecha1_wartosc,dp.czy_cecha1,"
                # kolumny 25 - 29
                SQL+=" dp.zo_wyznacznik_id,dp.zo_data,dp.zo_numer,dp.zo_nr_poz,dp.zo_nr_ppoz,"
                # kolumny 30 - 33
                SQL+="dp.produkt_id, dp.czy_dok_przych,a.wydanie_proporcjonalne,a.jm_kalk::INT,"
                # kolumny 34 - 38
                SQL+="COALESCE(opakowanie_id,''),COALESCE(paleta_id,''),COALESCE(wsp_ilosci_opakowanie,0),COALESCE(wsp_ilosci_paleta,0),COALESCE(wsp_ilosci_lokacja,0),"
                # kolumny 39 - 42
                SQL+="COALESCE(czy_opakowanie,FALSE),COALESCE(czy_paleta,FALSE),dp.zo_nr_zlec,dp.typ_zlec"
                
                SQL+=" FROM rs_dokumenty_magazynowe_pozycje dp INNER JOIN rs_artykuly a USING (firma_id,artykul_id) "
                SQL+=" WHERE dp.firma_id='"+firma_id+"'"
                SQL+=" AND dp.magazyn_id='"+Mag+"' AND dp.dokument_id='"+Dok+"' AND dp.data='"+Data+"' AND dp.numer="+Numer
                SQL+=" ORDER BY nr_poz,nr_ppoz;"
                stat, DaneP, dump, dump = self.PolSQL.Execute(SQL, 3)
            if stat == "OK":
                if len(DaneP)==0:
                    stat="Brak pozycji na dokumencie magazynowym"            

            # zdjcie iloci do pobrania z lokacji
            if stat=='OK':
                stat=fs_dokumenty_magazynowe_lokacje_pom.ZmienIlosciDoWydaniaWLokacjachNaCalymDokumencie(self,Mag,Dok,Data,Numer,'-')   

            if stat == "OK":
                UPD=''
                LParyLok=[] # [(nr poz, lokacja skad, lokacja dokad),...]
                for poz in DaneP:
                    if stat == "OK":
                        dokprzych = rsglobal.Dekoduj(poz[14])
                        DPfirma_id = dokprzych[0:2]
                        DPmagazyn_id = dokprzych[2:6]
                        DPtyp = dokprzych[6:8]
                        DPdata = dokprzych[8:18]
                        DPnumer = dokprzych[18:22]
                        DPpoz = dokprzych[22:26]
                        DPppoz = dokprzych[26:30]
                        art = rsglobal.Dekoduj(poz[2]).strip()
                        przel="(SELECT przelicznik FROM rs_artykuly WHERE firma_id='"+firma_id+"' AND artykul_id='"+art+"')"
                        oryg = rsglobal.Dekoduj(poz[3]).strip()
                        ilosc1 = poz[4]
                        ilosc2 = poz[5]
                        cena1 = poz[6]
                        cena2 = poz[7]
                        cenaw1 = poz[8]
                        cenaw2 = poz[9]
                        seria = rsglobal.Dekoduj(poz[10])
                        termin = rsglobal.Dekoduj(poz[11])
                        czycena = poz[15]
                        czyseria = poz[16]
                        czytermin = poz[17]
                        czyrozbicie = poz[13]
                        czydodnrserii = poz[18]
                        czy_dok_przych = poz[31]
                        if poz[32]:
                            WydanieProporcjonalne = poz[33]
                        else:
                            WydanieProporcjonalne = 0
                        
                        ZOWyznacznik = rsglobal.Dekoduj(poz[25]).strip()
                        ZOData = rsglobal.Dekoduj(poz[26])
                        ZONumer = rsglobal.Dekoduj(poz[27])
                        ZONrPoz = poz[28]
                        ZONrPPoz = poz[29]
                        ZONrZlec = poz[41]
                        ZOTypZlec = poz[42]
                        Produkt = rsglobal.Dekoduj(poz[30]).strip()
                        if Produkt in ['None',None]:
                            Produkt='NULL'
                        else:
                            Produkt="'"+Produkt+"'"
                        
                        if ZOWyznacznik in ['None',None,'']:
                            ZOWyznacznik='NULL'
                        else:
                            ZOWyznacznik = "'"+ZOWyznacznik+"'"
                            
                        if ZOData in ['    -  -  ',None,'None']:
                            ZOData='NULL'
                        else:
                            ZOData="'"+ZOData+"'"
                            
                        if ZONrZlec in ['    -  -  ',None,'None']:
                            ZONrZlec='NULL'
                        else:
                            ZONrZlec="'"+ZONrZlec+"'"
                            
                        if ZOTypZlec in ['    -  -  ',None,'None']:
                            ZOTypZlec='NULL'
                        else:
                            ZOTypZlec="'"+ZOTypZlec+"'"
                        
                        if rsglobal.Dekoduj(poz[19]) in ('','None',None):
                            lokacjaskad ='NULL'
                        else:
                            lokacjaskad ="'"+rsglobal.Dekoduj(poz[19])+"'"
                        if rsglobal.Dekoduj(poz[20]) in ('','None',None):
                            lokacjadokad ='NULL'
                        else:
                            lokacjadokad ="'"+rsglobal.Dekoduj(poz[20])+"'"
                            
                        czylokacje = poz[21]

                        if poz[24]:
                            Cecha1Opis = rsglobal.Dekoduj(poz[22])
                        else:
                            Cecha1Opis = ''

                        if poz[24]:
                            Cecha1Wart = rsglobal.Dekoduj(poz[23])
                        else:
                            Cecha1Wart = ''
                        
                        OpisWp= rsglobal.Dekoduj(poz[12])
                        if OpisWp not in ("",None):
                            opis="'"+OpisWp+"'"
                        else:
                            opis = "NULL"
    
                        if poz[34] == '' or not poz[39]:
                            Opak = 'NULL'
                        else:
                            Opak = "'"+rsglobal.Dekoduj(poz[34])+"'"
                                
                        if poz[35] == '' or not poz[40]:
                            Paleta = 'NULL'
                        else:
                            Paleta = "'"+rsglobal.Dekoduj(poz[35])+"'"
                            
                        WspOpak = 0 
                        WspPaleta = 0
                        WspLokacja = 0
                        if Opak != 'NULL' and Paleta != 'NULL':
                            WspOpak = poz[36] 
                            WspPaleta = poz[37]
                            WspLokacja = poz[38]
                        
                        if not czycena:
                            cena1 = 0
                            cena2 = 0
                            cenaw1 = 0
                            cenaw2 = 0
                
                        if (not czyseria) or czydodnrserii:
                            seria = ""
                
                        if not czytermin:
                            termin = "NULL"
                        else:
                            termin="'"+termin+"'"
                        
                        DokPrzychPoz=''
                        if czy_dok_przych:
                            DokPrzychPoz=dokprzych
    
                        # jeli czy_lokacje to bdziemy robi tabelk, a lokacje na pozycji bd puste.
                        # w przeciwnym przypadku przepisujemy je z pierwszej pozycji (bo ta idzie do inserta)
                        LokacjaSkad="NULL"
                        LokacjaDokad="NULL"
                        if not czylokacje:
                            if CzyLokacjeSkad:
                                LokacjaSkad=lokacjaskad
    
                        if not czylokacje:
                            if CzyLokacjeDokad:
                                LokacjaDokad=lokacjadokad
    
                        # pobranie informacji czy przelicznik jest dynamiczny
                        SQL="SELECT dynamiczny_przelicznik,sledzenie_dostaw FROM rs_artykuly "
                        SQL+=" WHERE firma_id='"+firma_id+"' AND artykul_id='"+art+"';"
                        stat, DArt, dump, dump = self.PolSQL.Execute(SQL, 3)
                        if stat<>"OK":
                            stat=' Bdy pobierania informacji o dynamicznym przeliczniku :'+stat
                
                    if stat=='OK':
                        DynamPrzel = DArt[0][0]                
                        if DynamPrzel:
                            Rezerw2Art = 'rezerwacja2- '+str(ilosc2)
                            Rezerw2 = 'rezerwacja2- '+str(ilosc2)
                        else:
                            Rezerw2Art = "ROUND((rezerwacja1- "+str(ilosc1)+")*przelicznik,4)"
                            Rezerw2 = "ROUND((rezerwacja1- "+str(ilosc1)+")*"+przel+",4)"
            
                        if Funkcja != 30:
                            #zmiana danych w tabelach z ilociami/rezerwacjami artykuw
                            # nie dotyczy otwierania/scalania dokumentu z komsera
                            UPD+="UPDATE rs_artykuly SET rezerwacja1=rezerwacja1- "+str(ilosc1)+","
                            UPD+="rezerwacja2="+Rezerw2Art+" WHERE firma_id='"+firma_id+"'"
                            UPD+=" AND artykul_id='"+art+"';"
                            
                            UPD+="UPDATE rs_artykuly_magazyny SET rezerwacja1=rezerwacja1- "+str(ilosc1)+","
                            UPD+="rezerwacja2="+Rezerw2+" WHERE firma_id='"+firma_id+"'"
                            UPD+=" AND magazyn_id='"+Mag+"' AND artykul_id='"+art+"';"
        
                            UPD+="UPDATE rs_artykuly_przyjecia SET rezerwacja1=rezerwacja1- "+str(ilosc1)+","
                            UPD+="rezerwacja2="+Rezerw2+" WHERE firma_id='"+firma_id+"'"
                            UPD+=" AND magazyn_id='"+Mag+"' AND data='"+DPdata+"' AND dokument_id='"+DPtyp+"'"
                            UPD+=" AND numer="+DPnumer+" AND nr_poz="+DPpoz+" AND nr_ppoz="+DPppoz+";"
                            
                        #sumowanie iloci w pozycjach
                        nrpoz = poz[0]
                        nrppoz = poz[1]
                        
                        # zmienne pomocnicze do wstawienia -1 w pozycje proporcjonalne po otworzeniu dokumentu
                        Il1Tmp=ilosc1
                        Il2Tmp=ilosc2
                        
                        if WydanieProporcjonalne==1:
                            Il2Tmp = -1
                        elif WydanieProporcjonalne==2:
                            Il1Tmp = -1
                            
                        if not czyrozbicie and OperatorWMS == 0:
                            if nrppoz==1:
                                UPD+="INSERT INTO rs_dokumenty_magazynowe_pozycje "
                                UPD+="(firma_id,magazyn_id,dokument_id,data,numer,nr_poz,nr_ppoz,artykul_id,"
                                UPD+="ilosc1,ilosc2,cena1,cena2,cena_wal1,cena_wal2,cena_pierw1,cena_pierw2,"
                                UPD+="cena_pierw_wal1,cena_pierw_wal2,nr_serii,termin_przyd,dokument_przych,"
                                UPD+="opis,czy_dodatkowy_nr_serii,lokacja_skad,lokacja_dokad,cecha1_opis,cecha1_wartosc,"
                                UPD+="zo_wyznacznik_id,zo_data,zo_numer,zo_nr_poz,zo_nr_ppoz,zo_nr_zlec,typ_zlec,produkt_id,dostawca_id,"
                                UPD+="opakowanie_id,paleta_id,wsp_ilosci_opakowanie,wsp_ilosci_paleta,wsp_ilosci_lokacja)"
                                UPD+=" VALUES ('"+firma_id+"','"+Mag+"','"+Dok+"','"+Data+"',"+Numer+","+str(nrpoz)+",0,'"+oryg+"',"
                                UPD+=str(Il1Tmp)+","+str(Il2Tmp)+","+str(cena1)+","+str(cena2)+","+str(cenaw1)+","+str(cenaw2)+",0,0,"
                                UPD+="0,0,'"+seria+"',"+termin+",'"+DokPrzychPoz+"',"+opis+",FALSE,"+LokacjaSkad+","+LokacjaDokad+",'"+Cecha1Opis+"','"+Cecha1Wart+"',"
                                UPD+=ZOWyznacznik+","+ZOData+","+str(ZONumer)+","+str(ZONrPoz)+","+str(ZONrPPoz)+","+str(ZONrZlec)+","+str(ZOTypZlec)+","+Produkt+",NULL,"
                                UPD+=Opak+","+Paleta+","+str(WspOpak)+","+str(WspPaleta)+","+str(WspLokacja)+");"
                            else:
                                if WydanieProporcjonalne==1:
                                    Il2Tmp = 0
                                elif WydanieProporcjonalne==2:
                                    Il1Tmp = 0
                                    
                                UPD+="UPDATE rs_dokumenty_magazynowe_pozycje SET ilosc1 = ilosc1+"+str(Il1Tmp)+","
                                UPD+="ilosc2 = ilosc2+"+str(Il2Tmp)+",opis="+opis+",dostawca_id=NULL WHERE firma_id='"+firma_id+"'"
                                UPD+=" AND magazyn_id='"+Mag+"' AND data='"+Data+"' AND dokument_id='"+Dok+"'"
                                UPD+=" AND numer="+Numer+" AND nr_poz="+str(nrpoz)+" AND nr_ppoz=0;"
    
                            # dodatkowo kad podpozycj uzgldniamy w tabelce. Albo dodajemy now albo dodajemy ilo 
                            # do istniejcego wiersza. informacje o aktualnym stanie tabelki trzymamy w zmiennej LParyLok
                            if czylokacje:
                                klucz=(nrpoz,lokacjaskad,lokacjadokad)
                                if klucz in LParyLok:
                                    if lokacjaskad=='NULL':
                                        FiltrSkad=" AND lokacja_skad IS NULL "
                                    else:
                                        FiltrSkad=" AND lokacja_skad = "+lokacjaskad
                                        
                                    if lokacjadokad=='NULL':
                                        FiltrDokad=" AND lokacja_dokad IS NULL "
                                    else:
                                        FiltrDokad=" AND lokacja_dokad = "+lokacjadokad
                                        
                                    #--- update istniejcego wiersza
                                    if self.lokacje_jm=='1':
                                        IloscLokUpd=ilosc1
                                    else:
                                        IloscLokUpd=ilosc2
                                        
                                    UPD+="UPDATE rs_dokumenty_magazynowe_lokacje SET ilosc=ilosc+"+str(IloscLokUpd)
                                    UPD+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"' "
                                    UPD+=" AND dokument_id='"+Dok+"' AND data='"+Data+"' "
                                    UPD+=" AND numer="+Numer+" AND nr_poz="+str(nrpoz)+" AND nr_ppoz=0 "
                                    UPD+=FiltrSkad+FiltrDokad+";"
                                else:
                                    #--- wstawienie nowego wiersza
                                    # zliczenie lp w tabelce
                                    Lp=1
                                    for k in LParyLok:
                                        if k[0]==nrpoz:
                                            Lp=Lp+1
                                    
                                    if self.lokacje_jm=='1':
                                        IloscLok=ilosc1
                                    else:
                                        IloscLok=ilosc2
                                    UPD+="INSERT INTO rs_dokumenty_magazynowe_lokacje (firma_id,magazyn_id,dokument_id,"
                                    UPD+="data,numer,nr_poz,nr_ppoz,lp,lokacja_skad,lokacja_dokad,ilosc)"
                                    UPD+="VALUES('"+firma_id+"','"+Mag+"','"+Dok+"',"
                                    UPD+="'"+Data+"',"+Numer+","+str(nrpoz)+",0,"+str(Lp)+","+lokacjaskad+","+lokacjadokad+","+str(IloscLok)+");"
                                    
                                    LParyLok.append(klucz)
    
                        else:
                            UPD+="UPDATE rs_dokumenty_magazynowe_pozycje SET cena1="+str(cena1)+",cena2="+str(cena2)+","
                            UPD+="cena_wal1="+str(cenaw1)+",cena_wal2="+str(cenaw2)+",cena_pierw1=0,cena_pierw2=0,"
                            UPD+="cena_pierw_wal1=0,cena_pierw_wal2=0,nr_serii='"+seria+"',termin_przyd="+termin+","
                            UPD+="opis="+opis+",czy_dodatkowy_nr_serii=FALSE, cecha1_opis='"+Cecha1Opis+"',"
                            UPD+="cecha1_wartosc='"+Cecha1Wart+"',dostawca_id=NULL,"
                            UPD+="opakowanie_id="+Opak+",paleta_id="+Paleta+",wsp_ilosci_opakowanie="+str(WspOpak)+","
                            UPD+="wsp_ilosci_paleta="+str(WspPaleta)+",wsp_ilosci_lokacja="+str(WspLokacja)
                            UPD+=" WHERE firma_id='"+firma_id+"'"
                            UPD+=" AND magazyn_id='"+Mag+"' AND data='"+Data+"' AND dokument_id='"+Dok+"'"
                            UPD+=" AND numer="+Numer+" AND nr_poz="+str(nrpoz)+" AND nr_ppoz="+str(nrppoz)+";"
                
                #ZMIANY COFANIA Z rs_dok_mag_rezerwacje PRZED ICH USUNICIEM
                # (wane by zrobi to PRZED usuwaniem pozycji bo inaczej usuwane s rs_dokumenty_magazynowe_rezerwacje)
                
                # pobieramy sumy danych, ktre bd zdejmowane, a potem robimy update w ptli
                SEL="SELECT dr.rezerwacja_id,dp.artykul_id,dr.ilosc1,a.przelicznik,a.dynamiczny_przelicznik"
                SEL+=" FROM rs_dokumenty_magazynowe_rezerwacje dr "
                SEL+=" INNER JOIN rs_dokumenty_magazynowe_pozycje dp "
                SEL+=" USING (firma_id,dokument_id,magazyn_id,data,numer,nr_poz,nr_ppoz) "
                SEL+=" INNER JOIN rs_artykuly a USING (firma_id,artykul_id) "
                SEL+=" WHERE dr.firma_id='"+firma_id+"' AND dr.dokument_id='"+Dok+"'"
                SEL+=" AND dr.magazyn_id='"+Mag+"' AND dr.data='"+Data+"' AND dr.numer="+Numer+"; "
                stat, DRez, dump, dump = self.PolSQL.Execute(SEL, 3)
            if stat == "OK":
                for poz in DRez:
                    if poz[4]:
                        #Rezerwacja2="rezerwacja2+"+str(poz[5]) # TODO
                        Rezerwacja2="ROUND((rezerwacja1+"+str(poz[2])+")*"+str(poz[3])+",4)"
                    else:
                        Rezerwacja2="ROUND((rezerwacja1+"+str(poz[2])+")*"+str(poz[3])+",4)"
                    
                    # jeli typ w to cigamy ewentualne rezerwacje z powrotem 
                    UPD+="UPDATE rs_artykuly_rezerwacje SET zamknieta=FALSE,zrealizowano=zrealizowano- "+str(poz[2])
                    UPD+=" WHERE firma_id='"+firma_id+"' AND rezerwacja_id="+str(poz[0])+";"
                    
                    #wrzucamy z powrotem rezerwacj do rs_artykuly
                    UPD+="UPDATE rs_artykuly SET rezerwacja1=rezerwacja1+"+str(poz[2])+","
                    UPD+=" rezerwacja2="+Rezerwacja2
                    UPD+="WHERE firma_id='"+firma_id+"' AND artykul_id='"+rsglobal.Dekoduj(poz[1])+"';"

                    #wrzucamy z powrotem rezerwacj do rs_artykuly_magazyny
                    UPD+="UPDATE rs_artykuly_magazyny SET rezerwacja1=rezerwacja1+"+str(poz[2])+","
                    UPD+=" rezerwacja2="+Rezerwacja2
                    UPD+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"'"
                    UPD+=" AND artykul_id='"+rsglobal.Dekoduj(poz[1])+"';"
                    
                # i usuwamy pozycje z rs_dokumenty_magazynowe_rezerwacje
                UPD+="DELETE FROM rs_dokumenty_magazynowe_rezerwacje WHERE firma_id='"+firma_id+"' AND dokument_id='"+Dok+"'"
                UPD+=" AND magazyn_id='"+Mag+"' AND data='"+Data+"' AND numer="+Numer+";"

                if OperatorWMS == 0:
                    # przepisujemy palety z rs_dokumenty_magazynowe_pozycje_palety z pozycji 0 do tych z numerami
                    UPD+= "INSERT INTO rs_dokumenty_magazynowe_pozycje_palety "
                    UPD+=" SELECT DISTINCT dp.firma_id,dp.magazyn_id,dp.dokument_id,dp.data,dp.numer,dp.nr_poz,0,dpp.nr_palety_id"
                    UPD+=" FROM rs_dokumenty_magazynowe_pozycje dp INNER JOIN rs_dokumenty_magazynowe_pozycje_palety dpp "
                    UPD+=" ON (dpp.firma_id=dp.firma_id AND dpp.magazyn_id=dp.magazyn_id AND dpp.dokument_id=dp.dokument_id "
                    UPD+=" AND dpp.data=dp.data AND dpp.numer=dp.numer AND dpp.nr_poz= dp.nr_poz AND dpp.nr_ppoz>0)"
                    UPD+=" WHERE dp.firma_id='"+firma_id+"'"
                    UPD+=" AND dp.dokument_id='"+Dok+"' AND dp.magazyn_id='"+Mag+"' AND dp.data='"+Data+"'"
                    UPD+=" AND dp.numer="+str(Numer)+" AND dp.nr_ppoz>0 AND NOT czy_rozbicie;"
                
                    # usunicie niepotrzebnych pozycji z pozycji dokumentow magazynowych
                    UPD+="DELETE FROM rs_dokumenty_magazynowe_pozycje WHERE firma_id='"+firma_id+"'"
                    UPD+=" AND dokument_id='"+Dok+"' AND magazyn_id='"+Mag+"' AND data='"+Data+"'"
                    UPD+=" AND numer="+Numer+" AND nr_ppoz>0 AND not czy_rozbicie;"

                stat, dump, dump, dump = self.PolSQL.Execute(UPD, 1)
            
            PolaNaglowka=",kwota=0, kwota_wal=0, kwota_pierw=0, kwota_pierw_wal=0"
    
            # ustawienie iloci lokacyjnej dla caego dokumentu
            if stat=='OK':
                stat=fs_dokumenty_magazynowe_lokacje_pom.UstawWspIIloscLokacyjnaNaDM(self,Mag,Dok,Data,Numer,0,0,True)
            
            # woenie iloci do pobrania z lokacji
            if stat=='OK':
                stat=fs_dokumenty_magazynowe_lokacje_pom.ZmienIlosciDoWydaniaWLokacjachNaCalymDokumencie(self,Mag,Dok,Data,Numer,'+')   

            # uwzgldnienie sklejenia na MM w wirtualnych artykulach przyjecia
            if stat=='OK' and Typ=='M':
                stat=fs_dokumenty_magazynowe_lokacje_pom.OdswiezWirtualnePozPrzyj(self,Mag,Dok,Data,Numer)

            #obsluga zamowien
            #czy pozycje wynikaly z pobrania  konkretnych zamowien (multiselect)
            ZamMultiSelect = False
            if stat == 'OK':
                for i in xrange(len(DaneP)):
                    if rsglobal.Dekoduj(DaneP[i][25]) != 'None' and \
                       rsglobal.Dekoduj(DaneP[i][26]) != 'None' and \
                       DaneP[i][27] > 0 and DaneP[i][41] in ['',None,'None']:
                        ZamMultiSelect = True
                        break

            if stat == 'OK' and ZamMultiSelect:
                #obsluga zamowien pobranych przez multiselect
                UPD = ''
                
                #pobranie pozycji z zamowien przypisanych do pozycji
                sql ="SELECT zop.artykul_id,"
                sql+=" zop.dostarczono1,"
                sql+=" zop.dostarczono2,"
                sql+=" zop.wyznacznik_id,zop.data,zop.numer,zop.nr_poz,zop.nr_ppoz,"
                sql+=" a.przelicznik, a.dynamiczny_przelicznik"
                sql+=" FROM rs_zamowienia_odbiorcy_pozycje zop"
                sql+=" INNER JOIN rs_artykuly a USING (firma_id,artykul_id)"
                sql+=" INNER JOIN rs_zamowienia_odbiorcy zo USING (firma_id,wyznacznik_id,data,numer)"
                sql+=" WHERE zop.firma_id='"+firma_id+"'"
                sql+=" AND zo.kontrahent_id='"+rsglobal.Dekoduj(TestFakt[0][1])+"'"
                sql+=" AND zo.stan IN ('G','Z') AND zop.dostarczono1>0"
                sql+=" AND zop.dostarczono2>0 AND ("
                
                licz = 0
                for i in xrange(len(DaneP)):
                    if rsglobal.Dekoduj(DaneP[i][25]) != 'None' and \
                       rsglobal.Dekoduj(DaneP[i][26]) != 'None' and \
                        DaneP[i][27] > 0 and DaneP[i][41] in ['',None,'None']:

                        licz += 1
                        if licz > 1:
                            sql+=" OR "
                            
                        sql+="(zop.wyznacznik_id='"+rsglobal.Dekoduj(DaneP[i][25])+"'"
                        sql+=" AND zop.data='"+DaneP[i][26]+"' AND zop.numer="+str(DaneP[i][27])
                        sql+=" AND zop.nr_poz="+str(DaneP[i][28])+" AND zop.nr_ppoz="+str(DaneP[i][29])+")"
                
                sql+=") FOR UPDATE NOWAIT;"
                
                stat,ZPoz,dump,dump = self.PolSQL.Execute(sql,3,CzyDodawacBlad=True)
                if stat != 'OK':
                    stat = 'Bd pobrania pozycji z wybranych zamwie (potencjalne zakleszczenie danych, sprbuj jeszce raz): '+stat
                elif len(ZPoz) == 0:
                    stat = 'Brak wybranych pozycji zamwie'
            
                if stat == 'OK':
                    #slownik z pozycjami zamowien klucz=(art,wyzn,data,numer,poz,ppoz)
                    SlowZPoz = {}
                    for i in xrange(len(ZPoz)):
                        poz = (ZPoz[i][0],ZPoz[i][3],ZPoz[i][4],ZPoz[i][5],ZPoz[i][6],ZPoz[i][7])
                        if not poz in SlowZPoz:
                            ilosci = [ZPoz[i][1],ZPoz[i][2],ZPoz[i][8],ZPoz[i][9]]
                            SlowZPoz[poz] = ilosci
                    
                    #aktualizacja pozycji zamowien
                    for i in xrange(len(DaneP)):
                        if rsglobal.Dekoduj(DaneP[i][25]) != 'None' and \
                           rsglobal.Dekoduj(DaneP[i][26]) != 'None' and \
                            DaneP[i][27] > 0:
                        
                            art = (DaneP[i][3],DaneP[i][25],DaneP[i][26],DaneP[i][27],DaneP[i][28],DaneP[i][29])
                            if art in SlowZPoz:
                                il1 = min(DaneP[i][4],SlowZPoz[art][0])
                                if SlowZPoz[art][3]:
                                    # dynamiczny przelicznik
                                    il2 = min(DaneP[i][5],SlowZPoz[art][1])
                                else:
                                    il2 = min(rsglobal.RSRound(il1*SlowZPoz[art][2],4),SlowZPoz[art][1])
                            
                                if il1 > 0:
                                    UPD+="UPDATE rs_zamowienia_odbiorcy_pozycje SET"
                                    UPD+=" dostarczono1=dostarczono1- "+str(il1)+","
                                    UPD+=" dostarczono2=dostarczono2- "+str(il2)
                                    UPD+=" WHERE firma_id='"+firma_id+"'"
                                    UPD+=" AND wyznacznik_id='"+rsglobal.Dekoduj(art[1])+"'"
                                    UPD+=" AND data='"+rsglobal.Dekoduj(art[2])+"'"
                                    UPD+=" AND numer="+str(art[3])
                                    UPD+=" AND nr_poz="+str(art[4])+" AND nr_ppoz="+str(art[5])+"; "
                                    
                                    SlowZPoz[art][0] -= il1
                                    SlowZPoz[art][1] -= il2

                if stat == 'OK' and UPD != '':
                    stat, dump, dump, dump = self.PolSQL.Execute(UPD,1)
                    if stat != 'OK':
                        stat = 'Bd aktualizacji pozycji zamwien: '+stat

                if stat == 'OK' and UPD != '':            
                    #otworzenie zamowien do stanu G, ktore zostaly wycofane
                    sql ="SELECT zop.wyznacznik_id,zop.data,zop.numer,"
                    sql+=" SUM(CASE WHEN zop.ilosc1-zop.anulowano1-zop.dostarczono1>0 THEN 1 ELSE 0 END) "
                    sql+=" FROM rs_zamowienia_odbiorcy_pozycje zop"
                    sql+=" INNER JOIN rs_zamowienia_odbiorcy zo USING (firma_id,wyznacznik_id,data,numer)"
                    sql+=" WHERE zop.firma_id='"+firma_id+"' AND zo.stan IN ('G','Z') AND ("

                    licz = 0
                    for i in xrange(len(DaneP)):
                        if rsglobal.Dekoduj(DaneP[i][25]) != 'None' and \
                           rsglobal.Dekoduj(DaneP[i][26]) != 'None' and \
                            DaneP[i][27] > 0:
                                
                            licz += 1
                            if licz > 1:
                                sql+=" OR "
                            sql+="(zop.wyznacznik_id='"+rsglobal.Dekoduj(DaneP[i][25])+"'"
                            sql+=" AND zop.data='"+DaneP[i][26]+"' AND zop.numer="+str(DaneP[i][27])+")"
                        
                    sql+=") GROUP BY zop.wyznacznik_id,zop.data,zop.numer;"

                    stat, DaneZamN, dump, dump = self.PolSQL.Execute(sql, 3)
                    if stat != 'OK':
                        stat = 'Bd pobrania danych z wybranych zamwie: '+stat

                    if stat == 'OK':
                        UPD = ''
                        for poz in DaneZamN:
                            if poz[3]>0:
                                UPD+="UPDATE rs_zamowienia_odbiorcy SET stan='G'"
                                UPD+=" WHERE firma_id='"+firma_id+"' "
                                UPD+=" AND wyznacznik_id='"+rsglobal.Dekoduj(poz[0])+"'"
                                UPD+=" AND data='"+rsglobal.Dekoduj(poz[1])+"'"
                                UPD+=" AND numer="+str(poz[2])+";"
                         
                        if UPD != '':
                            stat,d,dump,dump = self.PolSQL.Execute(UPD,1)
                            if stat != 'OK':
                                stat = 'Bd zamykania zrealizowanych zamwie: '+stat


            if stat=='OK' and not ZamMultiSelect:
                #ewentualne wycofanie ilosci dostarczonej z zamowienia,
                #jesli dokument byl powiazany z zamowieniem
                #stara procedura
                
                SQL ="SELECT zo_wyznacznik_id,zo_data,zo_numer,"
                SQL+=" czy_pobrane_z_zamowien, kontrahent_id"
                SQL+=" FROM rs_dokumenty_magazynowe"
                SQL+=" WHERE firma_id='"+firma_id+"'"
                SQL+=" AND magazyn_id='"+Mag+"' AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+Numer+";"

                stat, DaneN, dump, dump = self.PolSQL.Execute(SQL, 3)
                if stat == "OK":
                    if len(DaneN)==0:
                        stat="Brak podanego dokumentu magazynowego"
                    elif len(DaneN)>1:
                        stat="Podany dokument wystpuje w wicej ni raz w bazie danych!"
                    
                if stat == 'OK' and DaneN[0][3]:
                    WyznZ=rsglobal.Dekoduj(DaneN[0][0])
                    DataZ=rsglobal.Dekoduj(DaneN[0][1])
                    NrZ=str(DaneN[0][2])
                    
                    SlowDok={}
                    for poz in DaneP:
                        oryg = rsglobal.Dekoduj(poz[3]).strip()
                        if oryg in SlowDok:
                            SlowDok[oryg]=[SlowDok[oryg][0]+ poz[4],SlowDok[oryg][1]+poz[5]]
                        else:
                            SlowDok[oryg]=[poz[4],poz[5]]
                    
                    #Pobranie pozycji z zamowien odbiorcw
                    SQL="SELECT dpz.wyznacznik_id,dpz.data,dpz.numer, dpz.nr_poz, dpz.artykul_id,"
                    SQL+=" dpz.dostarczono1, dpz.dostarczono2,a.przelicznik"
                    SQL+=" FROM rs_zamowienia_odbiorcy_pozycje dpz INNER JOIN rs_zamowienia_odbiorcy zo USING (firma_id,wyznacznik_id,data,numer) "
                    SQL+=" INNER JOIN rs_artykuly a USING (firma_id,artykul_id) "
                    SQL+="WHERE dpz.firma_id='"+firma_id+"' AND dpz.dostarczono1>0 AND dpz.dostarczono2>0 "
                    if not WyznZ in ["",None,"None"]:
                        SQL+="AND dpz.wyznacznik_id='"+WyznZ+"' AND dpz.data='"+DataZ+"' AND dpz.numer="+NrZ+" "
                    else:
                        SQL+="AND zo.stan IN ('G','Z') AND zo.kontrahent_id='"+rsglobal.Dekoduj(DaneN[0][4])+"' "
            
                    SQL+=" ORDER BY dpz.artykul_id,dpz.wyznacznik_id,dpz.data,dpz.numer,dpz.nr_poz;"    
                        
                    stat, DaneZam, dump, dump = self.PolSQL.Execute(SQL, 3)

                    INSERT = ''
                   
                    for poz in DaneZam:
                        ArtS=rsglobal.Dekoduj(poz[4].strip())
                        Przel=poz[7]
                        if ArtS in SlowDok:
                            ileD1=SlowDok[ArtS][0]
                            ileD2=SlowDok[ArtS][1]
                            if ileD1>0:
                                ile1=min(ileD1, poz[5])
                                ile2=min(rsglobal.RSRound(ileD1*Przel,4), poz[6])
                                
                                if ile1>0:
                                    INSERT+="UPDATE rs_zamowienia_odbiorcy_pozycje SET dostarczono1=dostarczono1- "+str(ile1)+" ,dostarczono2=dostarczono2- "+str(ile2)+" "
                                    INSERT+="WHERE firma_id='"+firma_id+"' AND wyznacznik_id='"+rsglobal.Dekoduj(poz[0])+"' AND data='"+rsglobal.Dekoduj(poz[1])+"' "
                                    INSERT+="AND numer="+str(poz[2])+" AND nr_poz="+str(poz[3])+"; "
                                    
                                    SlowDok[ArtS][0]=ileD1-ile1
                                    SlowDok[ArtS][1]=ileD2-ile2
                            
                    if INSERT != "":
                        stat,dump,dump,dump = self.PolSQL.Execute(INSERT, 1)
                        
                        if stat == 'OK':
                            #zmiana stanu zamowienia na G
                            Select="SELECT dpz.wyznacznik_id,dpz.data,dpz.numer,"
                            Select+=" SUM(CASE WHEN dpz.ilosc1- dpz.anulowano1- dpz.dostarczono1 > 0 THEN 1 ELSE 0 END) "
                            Select+=" FROM rs_zamowienia_odbiorcy_pozycje dpz INNER JOIN rs_zamowienia_odbiorcy zo USING (firma_id,wyznacznik_id,data,numer)"
                            Select+=" WHERE dpz.firma_id='"+firma_id+"' "
                            if not WyznZ in ["",None,"None"]:
                                Select+=" AND dpz.wyznacznik_id='"+WyznZ+"' AND dpz.data='"+DataZ+"' AND dpz.numer="+NrZ+" "
                            else:
                                Select+=" AND zo.stan='Z' AND zo.kontrahent_id='"+rsglobal.Dekoduj(DaneN[0][4])+"' "
                            Select+=" GROUP BY dpz.wyznacznik_id,dpz.data,dpz.numer;"
                    
                            stat, DaneZamN, dump, dump = self.PolSQL.Execute(Select, 3) 
                
                            UPD=""
                            for poz in DaneZamN:
                                if poz[3]>0:
                                    UPD+="UPDATE rs_zamowienia_odbiorcy SET stan='G' WHERE firma_id='"+firma_id+"' "
                                    UPD+=" AND wyznacznik_id='"+rsglobal.Dekoduj(poz[0])+"' AND data='"+rsglobal.Dekoduj(poz[1])+"' AND numer="+str(poz[2])+";"
                            
                            if UPD != "":
                                stat, dump, dump, dump = self.PolSQL.Execute(UPD, 1) 

    
                
    #----------------------------------
    # POWRT DO OGLNEGO ALGORYTMU
    #----------------------------------
    if stat == 'OK':
        stat = fs_dokumenty_magazynowe_pom.ZmienIlosciNaZleceniachDlaDokumentu(self,Mag,Dok,Data,Numer,'-')

    if stat=='OK' and Funkcja != 30 and Typ != 'K':
        #ostateczna zmiana stanu z G na W
        SQL="UPDATE rs_dokumenty_magazynowe SET stan='W'"+PolaNaglowka+" WHERE firma_id='"+firma_id+"' "
        SQL+="AND magazyn_id='"+Mag+"' AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+Numer+";"

        stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1)


#@##########################################
# OTWARCIE DOKUMENTU   Z --> G  (11,21,31 i 111,121,131 detal wydanie z Z --> W = spis z natury detal)
#@##########################################

if Funkcja in (11,21,31,111,121,131) and stat=='OK' and Typ in ('C', 'I'):  #WS 2023-03-02
    #WS 2023-03-02
    stat='Nie mona otwiera dokumentw typu '+Typ
    # stat='Nie mona otwiera dokumentw typu C'

#WS 2023-03-02
if Funkcja in (11,21,31) and stat=='OK' and Typ not in ('C', 'I', 'K'):
# if Funkcja in (11,21,31) and stat=='OK' and Typ<>'C' and Typ<>'K':
    if Stan != "Z":
        stat = "Dokument jest w stanie '"+Stan+"' - nie mona otworzy dokumentu"

    if stat=='OK':
        #pobranie danych nagwka i pozycji.
        #kolumny 0-1
        SQL="SELECT magazyn_dokad,numer_dokad,zmieniony FROM rs_dokumenty_magazynowe WHERE firma_id='"+firma_id+"'"
        SQL+=" AND magazyn_id='"+Mag+"' AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+Numer+";"
        stat, DaneN, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat == "OK":
        if len(DaneN)==0:
            stat="Brak podanego dokumentu magazynowego"
        elif len(DaneN)>1:
            stat="Podany dokument wystpuje wicej ni raz w bazie danych!"
        elif DaneN[0][2]:
            stat="Dokument by zmieniany automatycznie i nie mona go otworzy"
        else:
            magprzyj = rsglobal.Dekoduj(DaneN[0][0])
            numprzyj = str(DaneN[0][1])
            if numprzyj in ['None','0.0']:
                numprzyj = Numer

    if stat == "OK":
        #kolumny 0 - 8
        SQL="SELECT nr_poz,nr_ppoz,artykul_id,ilosc1,ilosc2,cena1,cena2,dokument_przych,nr_serii,"
        # kolumny 9-11
        SQL+="lokacja_skad,lokacja_dokad,cecha1_wartosc"
        SQL+=" FROM rs_dokumenty_magazynowe_pozycje WHERE firma_id='"+firma_id+"'"
        SQL+=" AND magazyn_id='"+Mag+"' AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+Numer+";"
        stat, DaneP, dump, dump = self.PolSQL.Execute(SQL, 3)
            
    if stat == "OK":
        if len(DaneP)==0:
            stat="Brak pozycji na dokumencie magazynowym"            
        
        # dokumentw przeceny nie mona otwiera
    if stat == "OK":
        #AB 17.12.18r
        if Typ == 'R' and Dok == 'PR':
            stat = 'Dokumentu przeceny nie mona otwiera'
        
        #----------------------------------
        # DOKUMENT PRZYJCIA
        #----------------------------------
    if stat == "OK":
        if Typ in ["P","M","L"]:
            SQL="SELECT rsf_sprawdz_otworz_dok_mag('"+firma_id+"','"+Mag+"','"+Dok+"','"+Data+"',"+Numer+");"
            stat, Test, dump, dump = self.PolSQL.Execute(SQL, 3)
            if stat=='OK':
                stat= Test[0][0]
        
            # sprawdzenie czy po otworzeniu dokumentu nie zostanie zaburzona rezerwacja w magazynie i nie zmieni si na ujemn.
            if stat=='OK':
                SQL = "SELECT detaliczny FROM rs_magazyny "
                SQL += "WHERE firma_id='" + firma_id + "' AND magazyn_id='" + magprzyj + "'"
                stat, Test, dump, dump = self.PolSQL.Execute(SQL, 3)
                if stat == 'OK':
                    if len(Test) == 0:
                        stat = 'Brak definicji magazynu'
                    else:
                        CzyMagazynDetaliczny = Test[0][0]
                for poz in DaneP:
                    if stat=='OK':
                        #artykul_id,ilosc1 - 2,3
                    
                        # pobieramy aktualne rezerwacje i stany w magazynie i sprawdzamy czy po odjciu iloci 
                        # nie zostanie nam mniej ni 0.
                        # WYJATEK: magazyn detaliczny
                        SQL="SELECT ilosc1,rezerwacja1 FROM rs_artykuly_magazyny WHERE firma_id='"+firma_id+"'"
                        SQL+=" AND artykul_id='"+rsglobal.Dekoduj(poz[2])+"' AND magazyn_id='"+magprzyj+"';"
                        stat, Test, dump, dump = self.PolSQL.Execute(SQL, 3)
                        if stat=='OK':
                            if (Test[0][0]-poz[3]<Test[0][1]) and not CzyMagazynDetaliczny:
                                stat="Nie mona otworzy dokumentu bo wwczas rezerwacja dla artykuu "+poz[2].strip()
                                stat+=" bdzie wiksza ni stan artykuu w magazynie"
            
            # zmiana pozycji przyjcia z widocznych na wirtualne
            if stat=='OK':
                SQL+="UPDATE rs_artykuly_przyjecia SET wirtualne=TRUE WHERE firma_id='"+firma_id+"'" 
                SQL+=" AND magazyn_id='"+magprzyj+"' AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+numprzyj+";"                    
                stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1)
            
            for poz in DaneP:
                if stat=='OK':
                    nr_poz=str(poz[0])
                    nr_ppoz=str(poz[1])
                    artykul=rsglobal.Dekoduj(poz[2]).strip()
                    ilosc1=poz[3]
                    ilosc2=poz[4]
                    cena1=poz[5]
                    cena2=poz[6]
                    
                if stat=='OK':
                    SQL="SELECT rsf_zapisz_do_magazynow('"+firma_id+"','"+magprzyj+"','"+artykul+"','"+Data+"',"+str(-ilosc1)+","+str(-ilosc2)+","+str(cena1)+","+str(cena2)+");" 
                    stat, Test, dump, dump = self.PolSQL.Execute(SQL, 3)
                if stat=='OK':
                    if Test[0][0] != "OK":
                        stat= "Nie mona otworzy dokumentu"

                if stat=="OK":       
                    if Typ in ['P','M']:
                        # na artykuach przyjecia
                        # sprawdzamy czy istnieje spis, ktry warto zmienia
                        stat,LDataKat=fs_spis_z_natury_pom.PodajDatyIKategorieSpisow(self,magprzyj,Data,artykul)
                        if stat=='OK':
                            # dla kadego pasujcego spisu
                            for spis in LDataKat:
                                if stat=='OK':
                                    DataSpisu = spis[0]
                                    KategSpisu = spis[1]
                                    # odwieamy przyjcia spisu dla naszego artykuu
                                    stat = fs_spis_z_natury_pom.OdswiezPozycjeArtykulowPrzyjecWSpisieZNatury(self, DataSpisu, magprzyj, KategSpisu, artykul)
                                    
                                    # przeliczamy iloci stwierdzone na pozycjach spisu
                                    if stat=='OK':
                                        stat,dump = fs_spis_z_natury_pom.WstawIlosciStwierdzoneDoPozycjiArtykulowPrzyjeciaNaSpisieZNatury(self, DataSpisu, magprzyj, KategSpisu, artykul)
                                                        
            if stat=='OK':
                SQL="DELETE FROM rs_obroty_magazynowe WHERE firma_id='"+firma_id+"' AND magazyn_id='"+magprzyj+"' AND dokument_id='"+Dok+"'"
                SQL+=" AND data='"+Data+"' AND numer="+numprzyj+";"        
                stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1)

            # jeli na MM s pozycje konsolidacyjne to po zmianie stanu z Z na G naley odwiey pozycje przyj
            if stat == 'OK' and Typ == 'M':
                SQL = "SELECT dmp.artykul_id FROM rs_dokumenty_magazynowe_pozycje dmp INNER JOIN rs_artykuly a USING (firma_id, artykul_id) " \
                      " WHERE dmp.firma_id='"+firma_id+"' AND dmp.magazyn_id='"+Mag+"' AND dmp.dokument_id='"+Dok+"' AND dmp.data='"+Data+"' AND dmp.numer ="+Numer+" AND a.konsolidowany"
                stat, Test, dump, dump = self.PolSQL.Execute(SQL, 3)
                if stat =='OK':
                    if len(Test) > 0:
                        stat = fs_dokumenty_magazynowe_lokacje_pom.OdswiezWirtualnePozPrzyj(self,Mag,Dok,Data,Numer)

        #----------------------------------
        # DOKUMENT WYDANIA
        #----------------------------------
        if stat=='OK' and Typ in ["W","M","L","D"]:
            #pobranie kontrahenta z nagwka dokumentu
            SQL="SELECT kontrahent_id FROM rs_dokumenty_magazynowe WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"' "
            SQL+=" AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+Numer+" AND kontrahent_id IS NOT NULL;"
            stat, TestKontr, dump, dump = self.PolSQL.Execute(SQL, 3)
            
            # woenie iloci do pobrania z lokacji
            if stat=='OK':
                stat=fs_dokumenty_magazynowe_lokacje_pom.ZmienIlosciDoWydaniaWLokacjachNaCalymDokumencie(self,Mag,Dok,Data,Numer,'+',False)   
                
            if stat=='OK':
                #jest kontrahent - warto cos robic
                kontr=""
                if len(TestKontr)>0:
                    kontr=rsglobal.Dekoduj(TestKontr[0][0])
        
                    #sprawdzenie czy nie ma zgosze do urzdze z dokumentu
                    SQL="SELECT dmp.nr_poz,dmp.nr_ppoz,dmp.artykul_id,sz.zgloszenie_id FROM rs_dokumenty_magazynowe_pozycje dmp "
                    SQL+="INNER JOIN rs_dokumenty_magazynowe dm USING (firma_id,magazyn_id,dokument_id,data,numer)"
                    SQL+=" INNER JOIN rs_serwis_zgloszenia sz ON dmp.firma_id=sz.firma_id AND dmp.artykul_id=sz.artykul_id "
                    SQL+=" AND dmp.nr_serii=sz.nr_serii AND dm.kontrahent_id=sz.kontrahent_id "
                    SQL+=" WHERE dmp.firma_id='"+firma_id+"' AND dmp.magazyn_id='"+Mag+"' AND dmp.dokument_id='"+Dok+"' "
                    SQL+=" AND dmp.data='"+Data+"' AND dmp.numer="+Numer+";"
                    stat, TestUrz, dump, dump = self.PolSQL.Execute(SQL, 3)
                    if stat=='OK':
                        if len(TestUrz)>0:
                            stat= "Nie mona otworzy dokumentu, bo na urzdzenie "+TestUrz[0][2].strip()+" z pozycji "+str(TestUrz[0][0])+"/"+str(TestUrz[0][1])+" z dokumentu zostay ju wystawione zgoszenia serwisowe"
                
                LZamOtw=[]
                
            for poz in DaneP:
                if stat=='OK':
                    dokprzych = rsglobal.Dekoduj(poz[7])
                    magazyn_idAP = dokprzych[2:6]
                    typAP = dokprzych[6:8]
                    dataAP = dokprzych[8:18]
                    numerAP = dokprzych[18:22]
                    pozAP = dokprzych[22:26]
                    ppozAP = dokprzych[26:30]
                    art = rsglobal.Dekoduj(poz[2])
                    przel="(SELECT przelicznik FROM rs_artykuly WHERE firma_id='"+firma_id+"' AND artykul_id='"+art+"')"
                    ilosc1 = poz[3]
                    ilosc2 = poz[4]
                    if self.lokacje_jm=='1':
                        IloscLok=ilosc1
                    else:
                        IloscLok=ilosc2
                    nr_serii = rsglobal.Dekoduj(poz[8])
                    LokSkad = rsglobal.Dekoduj(poz[9])   
                    if LokSkad in ['None',None]:
                        LokSkad="NULL"
                    else:
                        LokSkad="'"+LokSkad+"'"                            
                  
                    # pobranie informacji czy przelicznik jest dynamiczny
                    SQL="SELECT dynamiczny_przelicznik,sledzenie_dostaw FROM rs_artykuly "
                    SQL+=" WHERE firma_id='"+firma_id+"' AND artykul_id='"+art+"';"
                    stat, DArt, dump, dump = self.PolSQL.Execute(SQL, 3)
                    if stat<>"OK":
                        stat=' Bdy pobierania informacji o dynamicznym przeliczniku :'+stat
                
                if stat=='OK':
                    DynamPrzel = DArt[0][0]                
                    SledzDost  = DArt[0][1]                
                    
                    #czyszczenie urzadzen
                    if kontr not in ["None","",None]:
                        if nr_serii not in ["None","",None]:
                            SQL="DELETE FROM rs_serwis_urzadzenia WHERE firma_id='"+firma_id+"' AND artykul_id='"+art+"' "
                            SQL+="AND nr_serii='"+nr_serii+"' AND kontrahent_id='"+kontr+"';"
                            stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1)
                    
                if stat=='OK':
                    if not (typAP in ("  ","None",None)):
                        SQL="SELECT * FROM rs_artykuly_magazyny WHERE firma_id='"+firma_id+"' AND artykul_id='"+art+"'"
                        SQL+=" AND magazyn_id='"+Mag+"';"
                        stat, DaneAM, dump, dump = self.PolSQL.Execute(SQL, 3)
                        if stat=='OK':
                            if len(DaneAM)==0:
                                SQL="INSERT INTO rs_artykuly_magazyny (firma_id,magazyn_id,artykul_id,ilosc1,ilosc2,ost_cena1,ost_cena2,"
                                SQL+="ost_przychod) VALUES ('"+firma_id+"','"+Mag+"','"+art+"',0,0,0,0,NULL);"
                                stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1)
                    
                # zmiana na lokacjach
                if stat=='OK':
                    SQL=''
                    if CzyLokacjeSkad:
                        # pobranie iloci z artykuw przyjcia
                        SQLDaneAP="SELECT ilosc"+self.lokacje_jm+" FROM rs_artykuly_przyjecia "
                        SQLDaneAP+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"' AND data='"+dataAP+"' AND dokument_id='"+typAP+"'"
                        SQLDaneAP+=" AND numer='"+numerAP+"' AND nr_poz='"+pozAP+"' AND nr_ppoz='"+ppozAP+"';"
                        stat, DaneLokAP, dump, dump = self.PolSQL.Execute(SQLDaneAP, 3)
                        if stat=='OK':
                            if self.lokacje_jm=='1':
                                IloscLok=ilosc1
                            else:
                                IloscLok=ilosc2
                
                # sprawdzenie czy mona przywrci ilo na artykuy przyjcia w zwizku z potencjaln zmian cech
                if stat=='OK':
                    SQLTestZm="SELECT COALESCE(cecha1_wartosc,'') FROM rs_artykuly_przyjecia "
                    SQLTestZm+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"' AND data='"+dataAP+"'"
                    SQLTestZm+=" AND dokument_id='"+typAP+"' AND numer='"+numerAP+"' AND nr_poz='"+pozAP+"' AND nr_ppoz='"+ppozAP+"';"
                    stat, DTestZm, dump, dump = self.PolSQL.Execute(SQLTestZm, 3)
                    if stat=='OK':
                        if len(DTestZm)==0:
                            stat='Brak pozycji '+dokprzych+' w artykuach przyjciach'
                        elif DTestZm[0][0].strip()<>poz[11].strip():
                            stat='Zmieniono ju cech na pozycji '+dokprzych+' w artykuach przyjciach - otwarcie dokumentu niemoliwe'
                            
                # sprawdzenie czy dla dynamicznego przelicznik da si woy wystarczajco duo na pozycj przyjcia
                # pobranie iloci z artykuw przyjcia
                RedukcjaDyn1=0
                RedukcjaDyn2=0
                
                if stat=='OK' and DynamPrzel:
                    SQLDaneAP="SELECT ilosc_prz1-ilosc1,ilosc_prz2-ilosc2 FROM rs_artykuly_przyjecia "
                    SQLDaneAP+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"' AND data='"+dataAP+"' AND dokument_id='"+typAP+"'"
                    SQLDaneAP+=" AND numer='"+numerAP+"' AND nr_poz='"+pozAP+"' AND nr_ppoz='"+ppozAP+"';"
                    stat, DaneLokAP, dump, dump = self.PolSQL.Execute(SQLDaneAP, 3)
                   
                    if stat=='OK':
                        IloscAPTest1=DaneLokAP[0][0]
                        IloscAPTest2=DaneLokAP[0][1]
                        if ilosc1 > IloscAPTest1:
                            RedukcjaDyn1= ilosc1- IloscAPTest1
                        if ilosc2 > IloscAPTest2:
                            RedukcjaDyn2= ilosc2- IloscAPTest2
                
                # zmiany iloci
                if stat=='OK':
                    if DynamPrzel:
                        Rezerw2Art = 'rezerwacja2+'+str(ilosc2)
                        Rezerw2 = 'rezerwacja2+'+str(ilosc2)
                    else:
                        Rezerw2Art = "ROUND((rezerwacja1+"+str(ilosc1)+")*przelicznik,4)"
                        Rezerw2 = "ROUND((rezerwacja1+"+str(ilosc1)+")*"+przel+",4)"
                        
                        
                    SQL+="UPDATE rs_artykuly SET ilosc1=ilosc1+"+str(ilosc1-RedukcjaDyn1)+",ilosc2=ilosc2+"+str(ilosc2-RedukcjaDyn2)+","
                    SQL+="rezerwacja1=rezerwacja1+"+str(ilosc1)+",rezerwacja2="+Rezerw2Art
                    SQL+=" WHERE firma_id='"+firma_id+"' AND artykul_id='"+art+"';"
                    
                    SQL+="UPDATE rs_artykuly_magazyny SET ilosc1=ilosc1+"+str(ilosc1-RedukcjaDyn1)+",ilosc2=ilosc2+"+str(ilosc2-RedukcjaDyn2)+","
                    SQL+="rezerwacja1=rezerwacja1+"+str(ilosc1)+",rezerwacja2="+Rezerw2
                    SQL+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"' AND artykul_id='"+art+"';"

                    SIlosc1="CASE WHEN ilosc2+"+str(ilosc2-RedukcjaDyn2)+" = ilosc_prz2 THEN ilosc_prz1 ELSE ilosc1+"+str(ilosc1-RedukcjaDyn1)+" END "
                    SIlosc2="CASE WHEN ilosc1+"+str(ilosc1-RedukcjaDyn1)+" = ilosc_prz1 THEN ilosc_prz2 ELSE ilosc2+"+str(ilosc2-RedukcjaDyn2)+" END "
                    
                    SQL+="UPDATE rs_artykuly_przyjecia SET ilosc1="+SIlosc1+",ilosc2="+SIlosc2+","
                    SQL+="rezerwacja1=rezerwacja1+"+str(ilosc1)+",rezerwacja2="+Rezerw2
                    SQL+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"' AND data='"+dataAP+"'"
                    SQL+=" AND dokument_id='"+typAP+"' AND numer='"+numerAP+"' AND nr_poz='"+pozAP+"' AND nr_ppoz='"+ppozAP+"';"

                    # Update na ilo lokacyjn
                    SQL+="UPDATE rs_artykuly_przyjecia SET ilosc_lokacja = CASE WHEN opakowanie_id IS NULL OR paleta_id IS NULL THEN "
                    SQL+=" ilosc"+self.lokacje_jm+" ELSE ROUND(ilosc"+self.lokacje_jm+"/wsp_ilosci_opakowanie/wsp_ilosci_paleta/wsp_ilosci_lokacja,4) END"
                    SQL+=" WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"' AND data='"+dataAP+"' AND dokument_id='"+typAP+"'"
                    SQL+=" AND numer='"+numerAP+"' AND nr_poz='"+pozAP+"' AND nr_ppoz='"+ppozAP+"';"
                    stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1)
                
                #zmiana ilosci na spisach z natury                
                if stat=="OK":       
                    if Typ in ['W','M']:
                        # na artykuach przyjecia
                        # sprawdzamy czy istnieje spis, ktry warto zmienia
                        stat,LDataKat=fs_spis_z_natury_pom.PodajDatyIKategorieSpisow(self,Mag,Data,art)
                        if stat=='OK':
                            # dla kadego pasujcego spisu
                            for spis in LDataKat:
                                if stat=='OK':
                                    DataSpisu = spis[0]
                                    KategSpisu = spis[1]
                                    # odwieamy przyjcia spisu dla naszego artykuu
                                    stat = fs_spis_z_natury_pom.OdswiezPozycjeArtykulowPrzyjecWSpisieZNatury(self, DataSpisu, Mag, KategSpisu, art)
                                    
                                    # przeliczamy iloci stwierdzone na pozycjach spisu
                                    if stat=='OK':
                                        stat,dump = fs_spis_z_natury_pom.WstawIlosciStwierdzoneDoPozycjiArtykulowPrzyjeciaNaSpisieZNatury(self, DataSpisu, Mag, KategSpisu, art)
                                                        
            if stat=='OK':
                SQL="DELETE FROM rs_obroty_magazynowe WHERE firma_id='"+firma_id+"' AND magazyn_id='"+Mag+"'"
                SQL+=" AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+Numer+";"
                stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1)
            #AB 29.06.21r
            # usuniecie kart przyjec z pozycji dokumentu magazynowego
            if stat == 'OK':
                SQL = """DELETE FROM rs_dokumenty_magazynowe_pozycje_karty_przyjecia 
                       WHERE (firma_id, magazyn_id, dokument_id, data, numer) = 
                             (  '{}',      '{}',        '{}',    '{}',   {})
                 ;""".format(firma_id,     Mag,         Dok,     Data, Numer)
                rsglobal.ZapiszDoLogow(SQL)
                stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1)
        
    #----------------------------------
    # POWRT DO OGLNEGO ALGORYTMU
    #----------------------------------
    if stat=='OK':
        #JR 12549 (usuwanie kart przyjecia)
        SQL = "DELETE FROM rs_artykuly_przyjecia_karty_przyjecia "
        SQL += " WHERE firma_id='" + firma_id + "' AND data='" + Data + "' AND magazyn_id='" + magprzyj
        SQL += "' AND dokument_id='" + Dok + "' AND numer=" + numprzyj + ";"
        stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1)

    if stat=='OK':
        #ostateczna zmiana stanu z Z na G
        SQL="UPDATE rs_dokumenty_magazynowe SET stan='G' WHERE firma_id='"+firma_id+"' "
        SQL+="AND magazyn_id='"+Mag+"' AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+Numer+";"
        
        stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1)

    if stat=='OK':
        # odejmuje ilosci z produkcji na zamowieniach odbiorcow
        stat=fs_dokumenty_magazynowe_produkcja_pom.ZmienIlosciWyprodukoweNaZamowieniachOdbiorcow(self,[Mag,Dok,Data,Numer],'-')
    
    if stat=='OK':
        if CzyLokacjeSkad:
            # sprawdzenie, czy po otwarciu dokumentu na lokacjach wszystko jest OK
            # wczeniej nie wiemy, czy nie sklej si artykuy przyjcia z innymi i si nie uda mimo dwch pozycji
            stat,dump,dump,LLokacje=fs_dokumenty_magazynowe_lokacje_pom.PodajLokacjeDokumentuLubPozycji(self,Mag,Dok,Data,Numer,NrPoz=0)
            if stat=='OK':
                stat,dane=fs_dokumenty_magazynowe_lokacje_pom.SprawdzListeLokacji(self,LLokacje,Typ)
                
#@##########################################
# -- 111,121,131 OTWARCIE DOKUMENTU ROZCHODOWEGO W MAGAZYNIE DETALICZNYM Z 'Z' NA 'W'
#@##########################################

if stat=='OK' and Funkcja in [111,121,131]:
    if Stan != "Z":
        stat = "Dokument jest w stanie '" + Stan + "' - nie mona otworzy dokumentu"
    # tmp_art = set()

    if stat == 'OK':
        # pobranie pozycji do cofniecia zmian w rs_artykuly_przyjecia
        SQL = "SELECT artykul_id,ilosc1,ilosc2,COALESCE(dokument_przych,'')"
        SQL += " FROM rs_dokumenty_magazynowe_pozycje WHERE firma_id='" + firma_id + "'"
        SQL += " AND magazyn_id='" + Mag + "' AND dokument_id='" + Dok + "' AND data='" + Data + "' AND numer=" + Numer+";"
        #SQL += " AND dokument_przych != '' AND dokument_przych IS NOT NULL;"
        stat, DaneP, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat == 'OK':
        UPD = ""
        for w in DaneP:
            # tmp_art.add(w[0])
            if w[3].strip() != '':     # cofniecie w artykuly_przyjecia
                UPD += "UPDATE rs_artykuly_przyjecia SET ilosc1=ilosc1 + " + str(w[1]) + ", ilosc2=ilosc2 + " + str(str(w[2]))
                UPD += " WHERE firma_id='" + w[3][0:2] + "' AND magazyn_id='" + w[3][2:6] + "'"
                UPD += " AND artykul_id='" + w[0] + "' AND data='" + w[3][8:18] + "' AND dokument_id='" + w[3][6:8] + "'"
                UPD += " AND numer=" + w[3][18:22] + " AND nr_poz=" + w[3][22:26] + " AND nr_ppoz=" + w[3][26:] + ";"
            UPD += "UPDATE rs_artykuly_magazyny SET ilosc1=ilosc1 + " + str(w[1]) + ", ilosc2=ilosc2 + " + str(str(w[2]))
            UPD += " WHERE firma_id='" + firma_id + "' AND magazyn_id='" + Mag + "'"
            UPD += " AND artykul_id='" + w[0] + "';"
            UPD += "UPDATE rs_artykuly SET ilosc1=ilosc1 + " + str(w[1]) + ", ilosc2=ilosc2 + " + str(w[2])
            UPD += " WHERE firma_id='" + firma_id + "' AND artykul_id='" + w[0] + "';"

        rsglobal.ZapiszDoLogow("UPD JR:" + UPD)
        if UPD != "":  # zwiekszenie ilosci w przyjeciach, magazynach i artykulach
            stat, dump, dump, dump = self.PolSQL.Execute(UPD, 1)

    if stat == 'OK':
        # pobranie pozycji do scalenia
        SQL = "SELECT nr_poz,COALESCE(oryginal, artykul_id) as artykul_id ,sum(ilosc1),sum(ilosc2)"  #AB 2023-08-25
        SQL += " FROM rs_dokumenty_magazynowe_pozycje WHERE firma_id='" + firma_id + "'"
        SQL += " AND magazyn_id='" + Mag + "' AND dokument_id='" + Dok + "' AND data='" + Data + "' AND numer=" + Numer
        SQL += " GROUP BY nr_poz, COALESCE(oryginal, artykul_id);"  #AB 2023-08-25
        rsglobal.ZapiszDoLogow("SELECT JR:" + SQL)
        stat, DaneP, dump, dump = self.PolSQL.Execute(SQL, 3)
    if stat == 'OK':
        UPDINSP = "DELETE FROM rs_dokumenty_magazynowe_pozycje WHERE firma_id='" + firma_id + "'"
        UPDINSP += " AND magazyn_id='" + Mag + "' AND dokument_id='" + Dok + "' AND data='" + Data + "'"
        UPDINSP += " AND numer=" + Numer + ";"
        for w in DaneP:
            UPDINSP += "INSERT INTO rs_dokumenty_magazynowe_pozycje (firma_id,magazyn_id,dokument_id,data,"
            UPDINSP += "numer,nr_poz,nr_ppoz,artykul_id,ilosc1,ilosc2)"
            UPDINSP += " VALUES ('" + firma_id + "','" + Mag + "','" + Dok + "','" + Data + "',"
            UPDINSP += Numer + "," + str(w[0]) + ",0,'" + rsglobal.Dekoduj(w[1]) + "',"
            UPDINSP += " " + str(w[2]) + "," + str(w[3]) + ");"
        stat, dump, dump, dump = self.PolSQL.Execute(UPDINSP, 1)

    #AB 28.06.23r
    # # zmiana ilosci na spisach z natury
    # if stat == "OK":
    #     # sprawdzamy czy istnieje spis, ktry warto zmienia
    #     for w in DaneP:
    #         tmp_art.add(w[1])
    #         for art in tmp_art:  #AB 15.06.23r
    #             if stat =='OK':
    #                 stat, LDataKat = fs_spis_z_natury_pom.PodajDatyIKategorieSpisow(self, Mag, Data, art)
    #                 if stat == 'OK':
    #                     # dla kadego pasujcego spisu
    #                     for spis in LDataKat:
    #                         if stat == 'OK':
    #                             DataSpisu = spis[0]
    #                             KategSpisu = spis[1]
    #                             # odwieamy przyjcia spisu dla naszego artykuu
    #                             stat = fs_spis_z_natury_pom.OdswiezPozycjeArtykulowPrzyjecWSpisieZNatury(self, DataSpisu, Mag,
    #                                                                                                      KategSpisu, art)
    #                             # przeliczamy iloci stwierdzone na pozycjach spisu
    #                             if stat == 'OK':
    #                                 stat, dump = fs_spis_z_natury_pom.WstawIlosciStwierdzoneDoPozycjiArtykulowPrzyjeciaNaSpisieZNatury(
    #                                     self, DataSpisu, Mag, KategSpisu, art)

    #AB 28.06.23r
    #zmiana iloci na spisach z natury
    if stat == 'OK':
        stat = fs_spis_z_natury_pom.ZaktualizujSpisyPoZmianachNaDokumencieMagazynowym(self, Mag, Data, Dok, Numer)

    if stat == 'OK':
        # ostateczna zmiana stanu z Z na W
        SQL = "UPDATE rs_dokumenty_magazynowe SET stan='W' WHERE firma_id='" + firma_id + "' "
        SQL += "AND magazyn_id='" + Mag + "' AND dokument_id='" + Dok + "' AND data='" + Data + "' AND numer=" + Numer + ";"
        stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1)
    if stat == 'OK':
        SQL = "DELETE FROM rs_obroty_magazynowe WHERE firma_id='" + firma_id + "' AND magazyn_id='" + Mag + "'"
        SQL += " AND dokument_id='" + Dok + "' AND data='" + Data + "' AND numer=" + Numer + ";"
        stat, dump, dump, dump = self.PolSQL.Execute(SQL, 1)

    Funkcja -= 100   # powrot do starego numeru

#@##########################################
# -- SPRAWDZENIE CZY BD ZAMYKANE SPISY Z NATURY  15 -- #
#@##########################################

if stat=='OK' and Funkcja in [15]:
    query = "SELECT rsf_sprawdz_pozycje_dokumentu_ze_spisem_z_natury('"+firma_id+"','"+Mag+"','"+Dok+"','"+Data+"',"+str(Numer)+")"
    stat, dane, none, none = self.PolSQL.Execute(query, 2, CzyDodawacBlad=True)
    if stat != "OK":
        #WS 2020-05-19 - dopisany stat
        stat = u'Krytyczny bd sprawdzania dokumentu ze spisami z natury: '+stat

#-------------------------------------------------------------------------------
# ACZENIE DOKUMENTU Z KOREKT
#-------------------------------------------------------------------------------

if Funkcja == 41 and stat == 'OK':    
    sql ="SELECT artykul,ilosc_w_korekcie,ilosc_w_dokumentach"
    sql+=" FROM (SELECT p.artykul_id as artykul,"
    sql+=" CASE"
    sql+=" WHEN (SELECT SUM(ilosc1_p-ilosc1) FROM rs_korekty_pozycje WHERE firma_id='"+firma_id+"'"
    sql+=" AND wyznacznik_id='"+KWyzn+"' AND data='"+KData+"' AND numer="+KNr+" AND artykul_id=p.artykul_id) is NULL"
    sql+=" THEN 0"
    sql+=" ELSE (SELECT SUM(ilosc1) FROM rs_korekty_pozycje"
    sql+=" WHERE firma_id='"+firma_id+"'"
    sql+=" AND wyznacznik_id='"+KWyzn+"' AND data='"+KData+"' AND numer="+KNr+" AND artykul_id=p.artykul_id)"
    sql+=" END"
    sql+=" as ilosc_w_korekcie,SUM(p.ilosc1) as ilosc_w_dokumentach"
    sql+=" FROM rs_dokumenty_magazynowe_pozycje p"
    sql+=" INNER JOIN rs_dokumenty_magazynowe d ON (p.firma_id=d.firma_id AND p.magazyn_id=d.magazyn_id AND p.dokument_id=d.dokument_id AND p.data=d.data AND p.numer=d.numer)"
    sql+=" WHERE p.firma_id='"+firma_id+"' AND d.k_wyznacznik_id='"+KWyzn+"' AND d.k_data='"+KData+"' AND d.k_numer="+KNr+" GROUP BY p.artykul_id) AS tab"
    sql+=" WHERE ilosc_w_korekcie!=ilosc_w_dokumentach ORDER BY ilosc_w_korekcie-ilosc_w_dokumentach;"
    
    stat,d,dump,dump = self.PolSQL.Execute(sql, 3)
    if stat != 'OK':
        stat = 'Bd pobrania informacji o ilociach artykuw : '+stat
    else:
        for i in d:
            if stat=='OK':
                #sprawdzenie czy typ artykuu nie jest 31 lub 99
                sqlpom="SELECT typ_artykulu FROM rs_artykuly WHERE firma_id='"+firma_id+"'"
                sqlpom+=" AND artykul_id='"+rsglobal.Dekoduj(i[0].strip())+"'"
                stat,DTest,dump,dump = self.PolSQL.Execute(sqlpom, 3)
                if stat=='OK':
                    if not DTest[0][0] in ['31','99']:
                        wyn2.append('Niezgodno w iloci artykuu '+i[0].strip()+' - w korekcie '+KWyzn+'/'+KData+'/'+KNr+' : '+str(i[1])+\
                            ', w pozycjach dokumentw magazynowych zwizanych z t korekt : '+str(i[2]))
    
    if stat=='OK':
        sql ="SELECT artykul_id,SUM(ilosc1_p-ilosc1),0 FROM rs_korekty_pozycje k"
        sql+=" WHERE firma_id='"+firma_id+"' AND wyznacznik_id='"+KWyzn+"' AND data='"+KData+"' AND numer="+KNr
        sql+=" AND (SELECT SUM(p.ilosc1) as ilosc_w_dokumentach FROM rs_dokumenty_magazynowe_pozycje p"
        sql+=" INNER JOIN rs_dokumenty_magazynowe d ON (p.firma_id=d.firma_id AND p.magazyn_id=d.magazyn_id"
        sql+=" AND p.dokument_id=d.dokument_id AND p.data=d.data AND p.numer=d.numer)"
        sql+=" WHERE p.firma_id='"+firma_id+"' AND d.k_wyznacznik_id='"+KWyzn+"' AND d.k_data='"+KData+"' AND d.k_numer="+KNr
        sql+=" AND p.artykul_id=k.artykul_id) is NULL GROUP BY k.artykul_id" 
        
        stat,d,dump,dump = self.PolSQL.Execute(sql, 3)
        if stat != 'OK':
            stat = 'Bd pobrania informacji o ilociach artykuw w korekcie : '+stat
        else:
            for i in d:
                if stat=='OK':
                    #sprawdzenie czy typ artykuu nie jest 31 lub 99
                    sqlpom="SELECT typ_artykulu FROM rs_artykuly WHERE firma_id='"+firma_id+"'"
                    sqlpom+=" AND artykul_id='"+rsglobal.Dekoduj(i[0].strip())+"'"
                    stat,DTest,dump,dump = self.PolSQL.Execute(sqlpom, 3)
                    if stat=='OK':
                        if not DTest[0][0] in ['31','99']:
                            wyn2.append('Niezgodno w iloci artykuu '+i[0].strip()+' - w korekcie '+KWyzn+'/'+KData+'/'+KNr+\
                                ' : '+str(i[1])+', w pozycjach dokumentw magazynowych zwizanych z t korekt : '+str(i[2]))

# -------------------------------------------------------------------------------
# POWROT DO STARYCH NUMEROW FUNKCJI JESLI MAGAZYN DETALICZNY I DOKUMENT ROZCHODOWY
# -------------------------------------------------------------------------------

if Funkcja > 100:
    Funkcja -= 100

#-------------------------------------------------------------------------------
# POBRANIE DANYCH DO GRIDA
#-------------------------------------------------------------------------------

if Funkcja in (5,6,7,8,9,10,11,41,67,71) and stat == 'OK':


#-------------------------------------------------------------------------------
# stworzenie FILTRA do pobrania grida
#-------------------------------------------------------------------------------

    FILTR = "(m.firma_id='"+firma_id+"' "
        
    if StanF != '':
        FILTR +=" AND m.stan='"+string.upper(StanF)+"'"
    if MagF != '':
        FILTR+=" AND (m.magazyn_skad LIKE '"+MagF+"%'"
        FILTR+=" OR m.magazyn_dokad LIKE '"+MagF+"%')"
    if DokF != '':
        FILTR+=" AND m.dokument_id LIKE '"+DokF+"%'"
    if DataOdF != '    -  -  ':
        if rsglobal.SprawdzCzyToData(DataOdF):
            FILTR+=" AND (m.data>='"+DataOdF+"') "
    if DataDoF != '    -  -  ':
        if rsglobal.SprawdzCzyToData(DataDoF):
            FILTR+=" AND (m.data<='"+DataDoF+"') "
    if NrF != '0':
        FILTR+=" AND m.numer = "+NrF
    if KontrF != '':
        FILTR+=" AND m.kontrahent_id LIKE '"+KontrF+"%'"
    if MiejDostF != '':
        FILTR+=" AND m.miejsce_dostawy LIKE '"+MiejDostF+"%'"
    if FaktF != '':
        FILTR+=" AND m.nr_faktury LIKE '"+FaktF+"%'"
    if ProjektF != '':
        FILTR+=" AND m.projekt_id='"+ProjektF+"'"
    if NiezamknieteF:
        FILTR+=" AND m.stan in ('W','G')"
    if ZmienioneF:
        FILTR+=" AND m.zmieniony"
        
    FILTR+=')'


#-------------------------------------------------------------------------------
# dopisanie do FILTRA uprawnien do magazynow i typow dokumentow
# dzieki temu na gridzie nie pokaze sie to, do czego uzytkownik nie ma uprawnien,
# a tym samym nie wywola operacji na dokumencie, ktorego nie zobaczy
#-------------------------------------------------------------------------------

    if stat=='OK' and self.operator_id != STANDARD_OPERATOR_ID:
        if NiezamknieteF:
            #jesli wybrano opcje Niezamkniete
            #pobieramy do filtra wszystkie dozwolone magazyny i
            #dozwolone dokumenty w magazynach

            sql ="SELECT DISTINCT magazyn_id,dokument_id"
            sql+=" FROM rs_dokumenty_m_upr"
            sql+=" WHERE firma_id='"+firma_id+"'"
            sql+=" AND operator_id IN "+self.operatorzy
            sql+=" AND (wprowadzanie OR edycja OR usuwanie OR przegladanie OR wnag OR zamykanie OR otwieranie OR gnaw);"
            stat,dmag,dump,dump = self.PolSQL.Execute(sql,3)       
            if stat != 'OK':
                stat = 'Bd pobrania listy dozwolonych magazynw: '+stat
            elif len(dmag) == 0:
                stat = 'Brak dozwolonych magazynw'
            else:
                #AB 14.10.21r
                FILTR += """AND (m.magazyn_id, m.dokument_id) IN (
                            SELECT DISTINCT magazyn_id,dokument_id
                            FROM rs_dokumenty_m_upr
                            WHERE firma_id='{}'
                                AND operator_id IN {}
                                AND (wprowadzanie OR edycja OR usuwanie OR przegladanie OR wnag OR zamykanie OR otwieranie OR gnaw))
                        """.format(firma_id, self.operatorzy)
                
        if stat=='OK': # dodane 31.08.2010r. -PW. nadpisywao staty uzyskiwane powyzej ( nie dziaao filtrowanie ) 
            
            sql = "SELECT magazyn_id FROM rs_magazyny WHERE firma_id='"+firma_id+"' AND magazyn_id='"+MagF+"';"
            stat,mag,dump,dump=self.PolSQL.Execute(sql, 3)
            dl=len(mag)

        if stat == 'OK':
            sql = "SELECT typ_dokumentu FROM rs_typy_dokumentow_m WHERE firma_id='"+firma_id+"' AND dokument_id='"+DokF+"';"
            stat,dok,dump,dump=self.PolSQL.Execute(sql, 3)
            dl=dl+len(dok)
            
            if stat == 'OK' and dl>0:
                if dl==2:
                    stat,upr,dump,dump=self.PolSQL.Execute("SELECT rsf_sprawdz_upr_do_dok('"+firma_id+"','"+MagF+"','"+DokF+"',6);",3)        
                    if stat=='OK':
                        if len(upr)==0:
                            #brak danych
                            stat="Brak uprawnie do dokumentw w podanym magazynie."
                        elif not upr[0][0]:
                            #nie ma uprawnie
                            stat="Brak uprawnie do dokumentw w podanym magazynie."
                        else:
                            #wszystko ok
                            dane=[""]
                            
                # podany jeden magazyn
                elif len(mag)==1:
                    SQL="SELECT DISTINCT dokument_id FROM rs_dokumenty_m_upr WHERE firma_id='"+firma_id+"' AND dokument_id LIKE '"+DokF+"%' "
                    SQL+=" AND magazyn_id='"+MagF+"' AND operator_id in "+self.operatorzy
                    SQL+=" AND (wprowadzanie OR edycja OR usuwanie OR przegladanie OR wnag OR zamykanie OR otwieranie OR gnaw);"
                    stat,upr,dump,dump=self.PolSQL.Execute(SQL,3)
                    if stat<>'OK':
                        stat='Bd sprawdzenia dokumentu magazynowego dla jednego magazynu'
                    elif len(upr)==0:
                        stat="Brak uprawnie do operacji na podanym magazynie."
                    else:
                        Lu=''
                        for u in upr:
                            Lu=Lu+"'"+rsglobal.Dekoduj(u[0])+"',"
                        FILTR += " AND m.dokument_id IN ("+Lu[0:-1]+")"
                        
                # podany jeden dokument
                elif len(dok)==1:   
                    SQL="SELECT DISTINCT magazyn_id FROM rs_dokumenty_m_upr WHERE firma_id='"+firma_id+"' AND dokument_id= '"+DokF+"' "
                    SQL+=" AND magazyn_id LIKE '"+MagF+"%' AND operator_id in "+self.operatorzy
                    SQL+=" AND (wprowadzanie OR edycja OR usuwanie OR przegladanie OR wnag OR zamykanie OR otwieranie OR gnaw);"
                    stat,upr,dump,dump=self.PolSQL.Execute(SQL,3)
                    if stat<>'OK':
                        stat='Bd sprawdzenia dokumentu dla jednego magazynu'
                    elif len(upr)==0:
                        stat="Brak uprawnie do operacji na podanym typie dokumentu."
                    else:
                        Lu=''
                        for u in upr:
                            Lu=Lu+"'"+rsglobal.Dekoduj(u[0])+"',"
                        FILTR += " AND (m.magazyn_skad IN ("+Lu[0:-1]+") OR m.magazyn_dokad IN ("+Lu[0:-1]+"))"


    #pobranie klucza wynikajacego z numerow nowych kluczy
    if KluczeF != []:
        FILTR +=" OR ((m.firma_id='"+firma_id+"') AND ("
        Pierwszy = True
        for klucz in KluczeF:
            if not Pierwszy:
                FILTR+=" OR "
            else:
                Pierwszy = False
            FILTR+=" (m.magazyn_id='"+str(klucz[0])+"' AND m.dokument_id='"+str(klucz[1])+"' AND m.data='"+str(klucz[2])+"' AND m.numer="+str(klucz[3])+")"
        FILTR+='))'


#-------------------------------------------------------------------------------
# POBRANIE DANYCH DO GRIDA GWNE ZADANIE
#-------------------------------------------------------------------------------
    if stat=='OK':
        DokumentZwiaz="COALESCE(m.dokument_zwiaz,m.pw_magazyn_id||'/'||m.pw_dokument_id||'/'||m.pw_data||'/'||m.pw_numer,'')"
        SQL="SELECT m.stan,m.magazyn_id,m.dokument_id,m.data,m.numer,m.magazyn_skad,m.magazyn_dokad,m.numer_dokad,"
        SQL+=" m.kontrahent_id,m.wersja,m.kwota,CASE WHEN m.f_numer>0 AND (m.f_wyznacznik_id IS NOT NULL OR m.f_data IS NOT NULL) "
        SQL+=" THEN 'F - '||rsf_podaj_numer_dokumentu_na_wydruk(m.firma_id,'[\"'||m.f_wyznacznik_id||'\",\"'||m.f_data||'\",\"'||m.f_numer||'\"]', 'F') "
        SQL+=" WHEN m.k_numer>0 AND (m.k_wyznacznik_id IS NOT NULL OR m.k_data IS NOT NULL) THEN 'K - '||rsf_podaj_numer_dokumentu_na_wydruk(m.firma_id,'[\"'||k_wyznacznik_id||'\",\"'||k_data||'\",\"'||k_numer||'\"]', 'K') ELSE '' END,"
        SQL+=" m.nr_zlecenia,m.nr_faktury,m.waluta,m.kwota_wal,"+DokumentZwiaz+",m.czy_pobrane_z_zamowien,"
        SQL+=" CASE WHEN m.zo_numer>0 AND m.zo_wyznacznik_id IS NOT NULL AND m.zo_data IS NOT NULL"
        SQL+=" THEN rsf_podaj_numer_dokumentu_na_wydruk(m.firma_id,'[\"'||m.zo_wyznacznik_id||'\",\"'||m.zo_data||'\",\"'||m.zo_numer||'\"]', 'F') ELSE '' END, "
        SQL+=" COALESCE(p_data::text,''), p_numer,"
        # dane korygowanego dokumentu
        SQL+=" CASE WHEN m.kor_dm_magazyn_id IS NULL THEN ''"
        SQL+="      ELSE m.kor_dm_magazyn_id||'/'||m.kor_dm_dokument_id||'/'||m.kor_dm_data||'/'||m.kor_dm_numer END, "
        SQL+="      (SELECT COALESCE(SUM(ilosc2)) FROM rs_dokumenty_magazynowe_pozycje poz "  #AB 2023-10-19
        SQL+="      WHERE (poz.firma_id, poz.magazyn_id, poz.dokument_id, poz.data, poz.numer) = "
        SQL+="            (m.firma_id, m.magazyn_id, m.dokument_id, m.data, m.numer)) as il_w_jm2"

        if CzyPrzesylki:
            SQL+=" FROM rs_dokumenty_magazynowe m "
            SQL+=" LEFT OUTER JOIN rs_typy_dokumentow_m tm using(firma_id, dokument_id)"
            SQL+=" WHERE tm.typ_dokumentu IN ('W','M') AND m.p_numer is NULL "
            SQL+=" AND stan in ('Z','K','G') AND "+FILTR
            SQL+="ORDER BY m.magazyn_id,m.dokument_id,m.data,m.numer;"
        elif NiewyslaneTowF:
            SQL+=" FROM rs_dokumenty_magazynowe m LEFT OUTER JOIN rs_przesylki p ON(m.firma_id=p.firma_id AND m.p_data=p.data AND m.p_numer=p.numer) "
            SQL+=" WHERE  (p.stan<>'Z' OR m.p_numer IS NULL) AND "+FILTR+" ORDER BY m.magazyn_id,m.dokument_id,m.data,m.numer;"
        else:
            SQL+=" FROM rs_dokumenty_magazynowe m WHERE "+FILTR+" ORDER BY m.magazyn_id,m.dokument_id,m.data,m.numer;"
        stat,dane,wyn1,tr = self.PolSQL.Execute(SQL, 3)
        if stat != 'OK':
            stat = 'Bd pobrania listy dokumentw magazynowych : '+stat


#-------------------------------------------------------------------------------
# POBRANIE LISTY NIEZGODNOSCI W ILOSCIACH ARTYKULOW PRZY ZAMYKANIU - 42
# nie mozna polaczyc tego z wczytaniem grida, gdyz Funkcja==8,9 wykorzystuje wyn2[]
#-------------------------------------------------------------------------------

if Funkcja == 42 and stat == 'OK':

    if typdok == 'K':
        #pobranie klucza korekty
        klucz = "SELECT k_wyznacznik_id,k_data,k_numer FROM rs_dokumenty_magazynowe "
        klucz+= " WHERE firma_id='"+firma_id+"' AND magazyn_id='"+mag_d+"' AND dokument_id='"+dok_d+"' "
        klucz+= " AND data='"+data_d+"' AND numer="+numer_d+""
        stat,korekta,dump,dump = self.PolSQL.Execute(klucz, 3)
        if stat != 'OK':
            stat = 'Bd pobrania informacji o korekcie : '+stat
        elif len(korekta)>0:
            wyznacznik = korekta[0][0]
            data = korekta[0][1]
            numer = str(korekta[0][2])
        if stat=='OK':
            sql ="SELECT artykul,ilosc_w_korekcie,ilosc_w_dokumentach"
            sql+=" FROM (select p.artykul_id as artykul,"
            sql+=" CASE WHEN (SELECT SUM(ilosc1) FROM rs_korekty_pozycje WHERE firma_id='"+firma_id+"'"
            sql+=" AND wyznacznik_id='"+wyznacznik+"' AND data='"+data+"' AND numer="+numer
            sql+=" AND artykul_id=p.artykul_id) IS NULL THEN 0"
            sql+=" ELSE (SELECT SUM(ilosc1) FROM rs_korekty_pozycje WHERE firma_id='"+firma_id+"'"
            sql+=" AND wyznacznik_id='"+wyznacznik+"' AND data='"+data+"' AND numer="+numer
            sql+=" AND artykul_id=p.artykul_id) END"
            sql+=" AS ilosc_w_korekcie,SUM(p.ilosc1) AS ilosc_w_dokumentach"
            sql+=" FROM rs_dokumenty_magazynowe_pozycje p"
            sql+=" INNER JOIN rs_dokumenty_magazynowe d ON (p.firma_id=d.firma_id AND p.magazyn_id=d.magazyn_id"
            sql+=" AND p.dokument_id=d.dokument_id AND p.data=d.data AND p.numer=d.numer)"
            sql+=" WHERE p.firma_id='"+firma_id+"' AND d.k_wyznacznik_id='"+wyznacznik+"' AND d.k_data='"+data+"'"
            sql+=" AND d.k_numer="+numer+" GROUP BY p.artykul_id) AS tab"
            sql+=" WHERE ilosc_w_korekcie!=ilosc_w_dokumentach ORDER BY ilosc_w_korekcie-ilosc_w_dokumentach;"

            stat,dane,dump,dump = self.PolSQL.Execute(sql, 3)
            if stat != 'OK':
                stat = 'Bd pobrania informacji o ilociach artykuw : '+stat
            else:
                for i in dane:
                    if stat=='OK':
                        #sprawdzenie czy typ artykuu nie jest 31 lub 99
                        sqlpom="SELECT typ_artykulu FROM rs_artykuly WHERE firma_id='"+firma_id+"'"
                        sqlpom+=" AND artykul_id='"+rsglobal.Dekoduj(i[0].strip())+"'"
                        stat,DTest,dump,dump = self.PolSQL.Execute(sqlpom, 3)
                        if stat=='OK':
                            if not DTest[0][0] in ['31','99']:
                                wyn2.append('niezgodno w iloci artykuu '+i[0].strip()+' - w korekcie '+wyznacznik+'/'+data+'/'+numer+\
                                    ' : '+str(i[1])+', w pozycjach dokumentw magazynowych zwizanych z t korekt : '+str(i[2]))

            if stat=='OK':
                sql ="SELECT artykul_id,sum(ilosc1),0 FROM rs_korekty_pozycje k WHERE firma_id='"+firma_id+"'"
                sql+=" AND wyznacznik_id='"+wyznacznik+"' AND data='"+data+"' AND numer="+numer+" AND"
                sql+=" (SELECT SUM(p.ilosc1) AS ilosc_w_dokumentach FROM rs_dokumenty_magazynowe_pozycje p"
                sql+=" INNER JOIN rs_dokumenty_magazynowe d ON"
                sql+=" (p.firma_id=d.firma_id AND p.magazyn_id=d.magazyn_id AND p.dokument_id=d.dokument_id AND p.data=d.data"
                sql+="  AND p.numer=d.numer) WHERE p.firma_id='"+firma_id+"' AND d.k_wyznacznik_id='"+wyznacznik+"'"
                sql+=" AND d.k_data='"+data+"' AND d.k_numer="+numer+" AND p.artykul_id=k.artykul_id) IS NULL"
                sql+=" GROUP BY k.artykul_id;"

                stat,dane,dump,dump = self.PolSQL.Execute(sql, 3)
                if stat != 'OK':
                    stat = 'Bd pobrania informacji o ilociach artykuw w korekcie: '+stat
                else:
                    for i in dane:
                        if stat=='OK':
                            #sprawdzenie czy typ artykuu nie jest 31 lub 99
                            sqlpom="SELECT typ_artykulu FROM rs_artykuly WHERE firma_id='"+firma_id+"'"
                            sqlpom+=" AND artykul_id='"+rsglobal.Dekoduj(i[0].strip())+"'"
                            stat,DTest,dump,dump = self.PolSQL.Execute(sqlpom, 3)
                            if stat=='OK':
                                if not DTest[0][0] in ['31','99']:
                                    wyn2.append('niezgodno w iloci artykuu '+i[0].strip()+' - w korekcie '+\
                                        wyznacznik+'/'+data+'/'+numer+' : '+str(i[1])+\
                                        ', w pozycjach dokumentw magazynowych zwizanych z t korekt : '+str(i[2]))

    elif typdok == 'F':
        #pobranie klucza faktury
        klucz = "SELECT f_wyznacznik_id,f_data,f_numer FROM rs_dokumenty_magazynowe "
        klucz+= " WHERE firma_id='"+firma_id+"' AND magazyn_id='"+mag_d+"' AND dokument_id='"+dok_d+"' "
        klucz+= " AND data='"+data_d+"' AND numer="+numer_d+""
        stat,faktura,dump,dump = self.PolSQL.Execute(klucz, 3)
        if stat != 'OK':
            stat = 'Bd pobrania informacji o fakturze : '+stat
        elif len(faktura)>0:
            wyznacznik = faktura[0][0]
            data = faktura[0][1]
            numer = str(faktura[0][2])


        if stat=='OK':
            sql ="SELECT artykul,ilosc_w_fakturze,ilosc_w_dokumentach"
            sql+=" FROM (SELECT p.artykul_id AS artykul,"
            sql+=" CASE WHEN (SELECT SUM(ilosc1) FROM rs_faktury_pozycje WHERE firma_id='"+firma_id+"'"
            sql+="  AND wyznacznik_id='"+wyznacznik+"' AND data='"+data+"' AND numer="+numer
            sql+="  AND artykul_id=p.artykul_id) IS NULL THEN 0"
            sql+=" ELSE (SELECT SUM(ilosc1) FROM rs_faktury_pozycje WHERE firma_id='"+firma_id+"'"
            sql+="  AND wyznacznik_id='"+wyznacznik+"' AND data='"+data+"' AND numer="+numer
            sql+="  AND artykul_id=p.artykul_id)"
            sql+=" END AS ilosc_w_fakturze,SUM(p.ilosc1) AS ilosc_w_dokumentach"
            sql+=" FROM rs_dokumenty_magazynowe_pozycje p"
            sql+=" INNER JOIN rs_dokumenty_magazynowe d ON"
            sql+=" (p.firma_id=d.firma_id AND p.magazyn_id=d.magazyn_id AND p.dokument_id=d.dokument_id"
            sql+=" AND p.data=d.data AND p.numer=d.numer) WHERE p.firma_id='"+firma_id+"'"
            sql+=" AND d.f_wyznacznik_id='"+wyznacznik+"' AND d.f_data='"+data+"' AND d.f_numer="+numer
            sql+=" GROUP BY p.artykul_id) AS tab WHERE ilosc_w_fakturze!=ilosc_w_dokumentach"
            sql+=" ORDER BY ilosc_w_fakturze-ilosc_w_dokumentach;"

            stat,dane,dump,dump = self.PolSQL.Execute(sql, 3)
            if stat != 'OK':
                stat = 'Bd pobrania informacji o ilociach artykuw : '+stat
            else:
                for i in dane:
                    if stat=='OK':
                        #sprawdzenie czy typ artykuu nie jest 31 lub 99
                        sqlpom="SELECT typ_artykulu FROM rs_artykuly WHERE firma_id='"+firma_id+"'"
                        sqlpom+=" AND artykul_id='"+rsglobal.Dekoduj(i[0].strip())+"'"
                        stat,DTest,dump,dump = self.PolSQL.Execute(sqlpom, 3)
                        if stat=='OK':
                            if not DTest[0][0] in ['31','99']:
                                wyn2.append('niezgodno w iloci artykuu '+i[0].strip()+' - w fakturze '+\
                                wyznacznik+'/'+data+'/'+numer+' : '+str(i[1])+\
                                ', w pozycjach dokumentw magazynowych zwizanych z t faktur : '+str(i[2]))
                if stat=='OK':
                    sql ="SELECT artykul_id,SUM(ilosc1),0 FROM rs_faktury_pozycje k"
                    sql+=" WHERE firma_id='"+firma_id+"' AND wyznacznik_id='"+wyznacznik+"' AND data='"+data+"'"
                    sql+=" AND numer="+numer+" AND"
                    sql+=" (SELECT SUM(p.ilosc1) AS ilosc_w_dokumentach FROM rs_dokumenty_magazynowe_pozycje p"
                    sql+="  INNER JOIN rs_dokumenty_magazynowe d ON (p.firma_id=d.firma_id AND p.magazyn_id=d.magazyn_id"
                    sql+="  AND p.dokument_id=d.dokument_id AND p.data=d.data AND p.numer=d.numer)"
                    sql+="  WHERE p.firma_id='"+firma_id+"' AND d.f_wyznacznik_id='"+wyznacznik+"' AND d.f_data='"+data+"'"
                    sql+="  AND d.f_numer="+numer+" AND p.artykul_id=k.artykul_id) IS NULL GROUP BY k.artykul_id;"

                    stat,dane,dump,dump = self.PolSQL.Execute(sql, 3)
                    if stat != 'OK':
                        stat = 'Bd pobrania informacji o ilociach artykuw na fakturze : '+stat
                    else:
                        for i in dane:
                            if stat=='OK':
                                #sprawdzenie czy typ artykuu nie jest 31 lub 99
                                sqlpom="SELECT typ_artykulu FROM rs_artykuly WHERE firma_id='"+firma_id+"'"
                                sqlpom+=" AND artykul_id='"+rsglobal.Dekoduj(i[0].strip())+"'"
                                stat,DTest,dump,dump = self.PolSQL.Execute(sqlpom, 3)
                                if stat=='OK':
                                    if not DTest[0][0] in ['31','99']:
                                        wyn2.append('niezgodno w iloci artykuu '+i[0].strip()+' - w fakturze '+wyznacznik+'/'+data+'/'+numer+\
                                        ' : '+str(i[1])+', w pozycjach dokumentw magazynowych zwizanych z t faktur : '+str(i[2]))

#-------------------------------------------------------------------------------


#@##########################################
# KOMSER
#@##########################################

def dolozDodatkoweDane(dodatkoweDane, nazwa, row, desc):
    nowe = []
    for d, info in zip(row, desc):
        # tu uwzgldniamy przypadek gdy numer_dokad powinien by NULL a jest 0
        if info[0]=='numer_dokad' and d==0:
            d = None
        if d == "None":
            d = None
        if info[0] not in ("z_czas", "z_typ", "z_op_id"):
            nowe.append((info[0], d))
    dodatkoweDane.append((nazwa.upper(), nowe))

# komser: no to jesli mamy cos do przeslania to zapisujemy
if stat == "OK" and Funkcja in (12, 9, 19, 11, 21) and firma_id == '01':  #WS 2020-12-16 - firma_id
    if Funkcja==9:
        Funkcja=19
    # komser: sprawdzamy czy dane maj by przesyane dalej
    stat, jest, none, none = self.PolSQL.Execute("SELECT COUNT(*) FROM rs_komser_funkcje WHERE nazwa='%s'" % funkcja, 4)
    if stat != "OK":
        stat = "BL rs_komser_funkcje"
    elif jest and Funkcja in [11]:
        stat, odp, none, none = self.WykonajFunkcje("fs_komser_zapisz", ["funkcja"],
            ["fs_komser_dokumenty_magazynowe", klucze, pola, wartosci])
    elif jest:
        komserDane = []
        #rsglobal.ZapiszDoLogow('10', 1)
        SQL="SELECT * FROM rs_dokumenty_magazynowe WHERE firma_id='"+firma_id+"'"
        SQL+=" AND magazyn_id='"+Mag+"' AND dokument_id='"+Dok+"' AND data='"+Data+"' AND numer="+str(Numer)
        stat, dokument, desc, dump = self.PolSQL.Execute(SQL, 3)
        if stat == "OK":
            stat, pozycje, pdesc, dump = self.PolSQL.Execute("SELECT * FROM rs_dokumenty_magazynowe_pozycje"
                " WHERE firma_id=%s AND magazyn_id=%s AND dokument_id=%s AND data=%s AND numer=%s", 3,
                (firma_id, Mag, Dok, Data, Numer))
        if stat == "OK" and dokument:
            dolozDodatkoweDane(komserDane, "DOKMAGAZ", dokument[0], desc)
            for pozycja in pozycje:
                dolozDodatkoweDane(komserDane, "DMAGPOZ", pozycja, pdesc)
            stat, odp, none, none = self.WykonajFunkcje("fs_komser_zapisz", ["funkcja", "dodatkowe-dane:", komserDane],
                ["fs_komser_dokumenty_magazynowe", klucze, pola, wartosci])